# pylint: disable=line-too-long, invalid-name, missing-function-docstring, missing-module-docstring, superfluous-parens
# pylint: disable=wrong-import-position, unused-import, unused-wildcard-import, wildcard-import, wrong-import-order, missing-class-docstring
from __future__ import annotations
from typing import List, Optional
from datetime import date
from datetime import time
from datetime import datetime
from decimal import Decimal
from pydantic import Field
from rosetta.runtime.utils import *

__all__ = ['PortfolioReturnTerms']

from cdm.product.template.ReturnTerms import ReturnTerms

class PortfolioReturnTerms(ReturnTerms):
  """
  Specifies an individual type of return of a Performance Payout, when such individual return is part of an aggregation of multiple similar returns, at Performance Payout level.
  """
  payerReceiver: PayerReceiver = Field(..., description="Canonical representation of the payer and receiver parties applicable to each individual return leg.")
  """
  Canonical representation of the payer and receiver parties applicable to each individual return leg.
  """
  underlier: Product = Field(..., description="Defines the product that is the subject of a tradable product definition, an underlying product definition, a physical exercise, a position, or other purposes.")
  """
  Defines the product that is the subject of a tradable product definition, an underlying product definition, a physical exercise, a position, or other purposes.
  """
  quantity: Optional[AttributeWithAddress[NonNegativeQuantitySchedule] | NonNegativeQuantitySchedule] = Field(None, description="Notional quantity of the underlier that applies to each individual return leg.")
  """
  Notional quantity of the underlier that applies to each individual return leg.
  """
  initialValuationPrice: Optional[AttributeWithAddress[PriceSchedule] | PriceSchedule] = Field(None, description="Specifies the initial valuation price(s) of the underlier. This price can be expressed either as an actual amount/currency, as a determination method, or by reference to another value specified in the swap document.")
  """
  Specifies the initial valuation price(s) of the underlier. This price can be expressed either as an actual amount/currency, as a determination method, or by reference to another value specified in the swap document.
  """
  interimValuationPrice: Optional[AttributeWithAddress[PriceSchedule] | PriceSchedule] = Field(None, description="Specifies the initial valuation price(s) of the underlier. This price can be expressed either as an actual amount/currency, as a determination method, or by reference to another value specified in the swap document.")
  """
  Specifies the initial valuation price(s) of the underlier. This price can be expressed either as an actual amount/currency, as a determination method, or by reference to another value specified in the swap document.
  """
  finalValuationPrice: Optional[AttributeWithAddress[PriceSchedule] | PriceSchedule] = Field(None, description="2018 ISDA CDM Equity Confirmation for Security Equity Swap: Final Price | Specifies the final valuation price of the underlier. This price can be expressed either as an actual amount/currency, as a determination method, or by reference to another value specified in the swap document.")
  """
  2018 ISDA CDM Equity Confirmation for Security Equity Swap: Final Price | Specifies the final valuation price of the underlier. This price can be expressed either as an actual amount/currency, as a determination method, or by reference to another value specified in the swap document.
  """

from cdm.base.staticdata.party.PayerReceiver import PayerReceiver
from cdm.product.template.Product import Product
from cdm.base.math.NonNegativeQuantitySchedule import NonNegativeQuantitySchedule
from cdm.observable.asset.PriceSchedule import PriceSchedule

PortfolioReturnTerms.update_forward_refs()
