package drr.enrichment.eic.functions;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

public class GetAcceptedEicCodesImpl extends GetAcceptedEicCodes {

    private static final Logger LOGGER = LoggerFactory.getLogger(GetAcceptedEicCodesImpl.class);

    // Downloaded from https://www.acer.europa.eu/remit-documents/remit-reporting-guidance > List of accepted EICs (10/10/2023)
    private static final String ACCEPTED_EIC_CODES_CSV = "regulatory-reporting/lookup/List-of-Accepted-EICs-2023-10-10.csv";

    public static GetAcceptedEicCodes getInstanceWithPreloadedData() {
        List<String> acceptedEicCodes = List.of("10YAT-APG------L",
                "10YBE----------2",
                "10YCA-BULGARIA-R",
                "10YCY-1001A0003J",
                "10YCZ-CEPS-----N",
                "10YDE-ENBW-----N",
                "10YDE-EON------1",
                "10YDE-RWENET---I",
                "10YDE-VE-------2",
                "11YRBAHNSTROM--P",
                "10YDK-1--------W",
                "10YDK-2--------M",
                "10Y1001A1001A39I",
                "10YGR-HTSO-----Y",
                "10YES-REE------0",
                "10YFI-1--------U",
                "10YFR-RTE------C",
                "10YHR-HEP------M",
                "10YHU-MAVIR----U",
                "10YIE-1001A00010",
                "10Y1001A1001A66F",
                "10Y1001A1001A67D",
                "10Y1001A1001A68B",
                "10Y1001A1001A70O",
                "10Y1001A1001A71M",
                "10Y1001A1001A73I",
                "10Y1001A1001A74G",
                "10Y1001A1001A75E",
                "10Y1001A1001A788",
                "10Y1001A1001A80L",
                "10Y1001A1001A81J",
                "10Y1001A1001A877",
                "10Y1001A1001A885",
                "10Y1001A1001A893",
                "10Y1001C--000611",
                "10Y1001C--00096J",
                "10YLT-1001A0008Q",
                "10YLU-CEGEDEL-NQ",
                "10YLV-1001A00074",
                "10YNL----------L",
                "10YPL-AREA-----S",
                "10YPT-REN------W",
                "10YRO-TEL------P",
                "10Y1001A1001A44P",
                "10Y1001A1001A45N",
                "10Y1001A1001A46L",
                "10Y1001A1001A47J",
                "10YSI-ELES-----O",
                "10YSK-SEPS-----K",
                "10Y1001A1001A016",
                "10Y1001A1001A63L",
                "10Y1001A1001A59C",
                "10YIT-GRTN-----B",
                "10Y1001A1001A91G",
                "10YSE-1--------K",
                "21Y---A001A004-2",
                "21Y000000000009E",
                "21Y000000000024I",
                "21Y000000000023K",
                "21Y000000000026E",
                "21Y---A001A001-B",
                "21Y000000000003Q",
                "21Y000000000106G",
                "21Y0000000001359",
                "21Y000000000131H",
                "21Y---A002A005-T",
                "21Y000000000078W",
                "21Y000000000001U",
                "21Y0000000001278",
                "21Y000000000105I",
                "39YSINBP000000NE",
                "21Y---A001A009-O",
                "21Y---A001A010-A",
                "21Y000000000079U",
                "21Y000000000101Q",
                "21YNL----TTF---1",
                "21Y000000000087V",
                "21Y000000000085Z",
                "21Y000000000086X",
                "16ZVTP---------N",
                "60Y000000000001I",
                "21Z0000000004693",
                "21Y000000000103M",
                "21Z000000000248J",
                "58Z--VTPN-NAT--1",
                "37Z005053MH0000D",
                "66Y-00000000001F",
                "37Y005053MH0000R",
                "21Y000000000123G",
                "21Y---A001A012-4",
                "21Y---A001A013-1",
                "21Z000000000172Q",
                "21Z000000000173O",
                "21Z000000000216W",
                "21Z000000000217U",
                "21Z000000000001G",
                "21Z000000000002E",
                "21Z0000000001240",
                "21Z000000000125Z",
                "21Z000000000161V",
                "21Z000000000256K",
                "21Z000000000257I",
                "21Z000000000258G",
                "21Z000000000003C",
                "21Z000000000004A",
                "21Z0000000000058",
                "21Z000000000175K",
                "21Z0000000000082",
                "21Z0000000000090",
                "21Z000000000102A",
                "21Z0000000001046",
                "21Z0000000002115",
                "21Z000000000245P",
                "21Z000000000246N",
                "21Z0000000000155",
                "21Z0000000000163",
                "21Z000000000147P",
                "21Z000000000174M",
                "21Z000000000010F",
                "21Z000000000011D",
                "21Z0000000001038",
                "21Z000000000359A",
                "21Z0000000000139",
                "21Z0000000000147",
                "21Z0000000000171",
                "21Z000000000019Y",
                "21Z0000000001054",
                "21Z0000000001062",
                "58Z-IP-00034-STZ",
                "21Z000000000137S",
                "21Z0000000002798",
                "58Z-000000007-KZ",
                "58Z-00000015-S2M",
                "21Z000000000157M",
                "58Z-VTP-ICGB-BGN",
                "21Z000000000156O",
                "21Z0000000000066",
                "21Z0000000001321",
                "21Z000000000136U",
                "21Y---A001A016-T",
                "21Z000000000206Z",
                "21Z000000000066P",
                "21Z000000000171S",
                "21Z000000000242V",
                "21Z000000000452K",
                "27ZG-LAA-CZ-AT-F",
                "21Z000000000261R",
                "21Z0000000000228",
                "21Z0000000000236",
                "21Z0000000000244",
                "21Z000000000091Q",
                "21Z000000000162T",
                "27ZG-BAER-CZ-DED",
                "27ZG-UVAL-CZ-PLZ",
                "21Y000000000069X",
                "21Z000000000237O",
                "21Z000000000241X",
                "21Z000000000255M",
                "21Z000000000278A",
                "21Z000000000281L",
                "21Z000000000370M",
                "37Z000000007357L",
                "21Z000000000372I",
                "21Z000000000271O",
                "21Z000000000038U",
                "21Z000000000252S",
                "21Z000000000259E",
                "21Z000000000260T",
                "21Z0000000001216",
                "21Z0000000001232",
                "21Z000000000253Q",
                "21Z000000000254O",
                "21Z0000000004839",
                "21Z000000000092O",
                "21Z000000000039S",
                "21Z0000000001208",
                "21Z0000000000406",
                "21Z000000000090S",
                "21Z000000000251U",
                "21Z0000000000252",
                "21Z000000000027Z",
                "21Z000000000238M",
                "21Z0000000000260",
                "21Z000000000144V",
                "21Z0000000002131",
                "21Z0000000004952",
                "21Z000000000152W",
                "21Z000000000504R",
                "21Z000000000453I",
                "21Z0000000000422",
                "21Z000000000472E",
                "21Z000000000473C",
                "21Z0000000004758",
                "21Z000000000233W",
                "18Z000000000GVLQ",
                "21Z000000000063V",
                "21Z000000000064T",
                "21Z000000000065R",
                "21Z000000000129R",
                "21Z0000000001402",
                "21Z0000000002123",
                "21Z000000000285D",
                "21Z0000000001410",
                "21Z000000000062X",
                "21Z000000000282J",
                "21Z0000000004847",
                "21Y---A001A007-U",
                "21Y---A001A008-R",
                "21Z000000000028X",
                "21Z0000000000309",
                "21Z0000000000317",
                "21Z0000000000325",
                "21Z0000000000333",
                "21Z000000000165N",
                "21Z000000000166L",
                "21Z0000000003980",
                "21Z000000000448B",
                "21Z000000000455E",
                "21Z000000000148N",
                "21Y---A001A006-X",
                "21Z000000000037W",
                "21Z0000000003786",
                "21Y---A001A014-Z",
                "21Z000000000036Y",
                "21Z0000000003200",
                "21Z0000000000341",
                "21Z000000000491A",
                "21Z000000000249H",
                "21Z000000000236Q",
                "21Z000000000154S",
                "21Z000000000270Q",
                "21Z000000000283H",
                "47Z000000000045M",
                "21Z0000000000430",
                "21Z0000000002107",
                "21Z000000000276E",
                "21Z000000000277C",
                "21Z000000000474A",
                "21Z000000000274I",
                "21Z000000000044Z",
                "21Z000000000461J",
                "21Z000000000462H",
                "21Z000000000463F",
                "21Z000000000464D",
                "21Z000000000465B",
                "21Z0000000004669",
                "21Z0000000004677",
                "21Z0000000001305",
                "21Z0000000001070",
                "21Z000000000153U",
                "21Z000000000149L",
                "21Z000000000045X",
                "21Z000000000087H",
                "21Z0000000001119",
                "21Z0000000001127",
                "21Z0000000001135",
                "21Z0000000001143",
                "21Z0000000001151",
                "21Z000000000119U",
                "21Z000000000240Z",
                "21Z000000000243T",
                "52Z0000000001429",
                "52Z0000000000805",
                "37Z000000008198A",
                "37Z000000008197C",
                "52Z0000000001437",
                "21Z000000000067N",
                "21Z000000000068L",
                "21Z000000000069J",
                "21Z000000000070Y",
                "21Z000000000101C",
                "21Z000000000169F",
                "21Z000000000071W",
                "21Z000000000072U",
                "21Z000000000073S",
                "21Z000000000074Q",
                "21Z000000000075O",
                "21Z000000000076M",
                "21Z000000000077K",
                "21Z000000000078I",
                "21Z000000000079G",
                "21Z000000000108Z",
                "21Z000000000109X",
                "21Z000000000110B",
                "21Z000000000117Y",
                "21Z000000000118W",
                "21Z000000000135W",
                "21Z000000000170U",
                "21Z0000000002042",
                "21Z000000000250W",
                "21Z000000000344N",
                "21Z000000000046V",
                "21Z000000000048R",
                "21Z000000000049P",
                "21Z0000000000503",
                "21Z0000000000511",
                "21Z000000000053Y",
                "21Z0000000004855",
                "21Z000000000047T",
                "21Z000000000054W",
                "21Z000000000055U",
                "21Z000000000145T",
                "21Z000000000146R",
                "21Z000000000167J",
                "21Z000000000168H",
                "21Z000000000399Z",
                "21Z000000000239K",
                "21Z000000000456C",
                "21Z000000000056S",
                "21Z000000000057Q",
                "21Z000000000089D",
                "21Z180960164W65R",
                "21Z000000000266H",
                "21Z000000000058O",
                "21Z6035937275964",
                "21Z000000000509H",
                "21Z000000000304Z",
                "21Z000000000305X",
                "21Z000000000306V",
                "21Z000000000151Y",
                "21Z000000000159I",
                "21Z000000000160X",
                "21Z000000000356G",
                "21Z000000000373G",
                "21Z000000000059M",
                "21Z000000000155Q",
                "21Z000000000128T",
                "21Z000000000451M",
                "21Z0000000004596",
                "21Z000000000460L",
                "21Z0000000004685",
                "21Y---A001A023-Y",
                "21Z0000000000600",
                "21Z000000000163R",
                "21Z000000000164P",
                "21Z000000000265J",
                "21Z000000000061Z",
                "21Z000000000358C",
                "21Z000000000357E",
                "21Z000000000080V",
                "21Z000000000139O",
                "21Z000000000143X",
                "21Z000000000158K",
                "21Z000000000085L",
                "21Z0000000001899",
                "21Z000000000384B",
                "21Z0000000003859",
                "21Z0000000003867",
                "21Z0000000003875",
                "21Z000000000083P",
                "21Z0000000000074",
                "21Z000000000081T",
                "21Z000000000082R",
                "59ZPIOMBINO11112",
                "58Z-000000012-KD",
                "21Z000000000291I",
                "21Z000000000381H",
                "21Z000000000383D",
                "21Z000000000382F",
                "21Z000000000309P",
                "27ZG-HORY-CZ-PLS",
                "63Z756767365899Y",
                "21Z000000000512S",
                "21W000000000078N",
                "21W000000000057V",
                "21W000000000081Y",
                "21W000000000079L",
                "21W0000000001245",
                "21W000000000031C",
                "21W000000000074V",
                "21W000000000102F",
                "21W000000000075T",
                "21W000000000076R",
                "21W000000000111E",
                "21W000000000110G",
                "21W0000000001091",
                "21W0000000001180",
                "21W000000000120D",
                "21W0000000001148",
                "21W0000000001164",
                "21W0000000001083",
                "21W0000000001075",
                "21W0000000001059",
                "21W000000000104B",
                "21W000000000103D",
                "21W0000000001067",
                "21W000000000005D",
                "21W000000000097J",
                "21W000000000100J",
                "21W0000000000168",
                "21W0000000000249",
                "21W000000000023B",
                "21W0000000000281",
                "21W000000000013E",
                "21W0000000000095",
                "21W000000000021F",
                "21W000000000011I",
                "21W000000000010K",
                "21W000000000030E",
                "21W0000000000613",
                "21W0000000000605",
                "21W000000000090X",
                "21W000000000066U",
                "21W000000000065W",
                "21W000000000003H",
                "21W0000000000524",
                "21W0000000000532",
                "21W000000000085Q",
                "21W000000000083U",
                "21W000000000059R",
                "21W000000000055Z",
                "21W000000000056X",
                "21W000000000091V",
                "21W000000000064Y",
                "21W000000000092T",
                "21W0000000000192",
                "21W0000000000508",
                "21W000000000004F",
                "21W000000000058T",
                "21W000000000067S",
                "21W000000000049U",
                "21W0000000000176",
                "21W0000000000273",
                "21W0000000000540",
                "21W000000000048W",
                "21W000000000089I",
                "21W0000000000621",
                "21W000000000093R",
                "21W0000000000184",
                "21W0000000000516",
                "21W0000000000265",
                "21W000000000020H",
                "21W0000000000427",
                "37W000000000002O",
                "21W000000000121B",
                "21W000000000129W",
                "37W000000000106C",
                "37W000000000107A",
                "21W000000000040B",
                "21W000000000039X",
                "21W0000000000362",
                "21W000000000038Z",
                "21W0000000000346",
                "21W0000000000370",
                "21W0000000000338",
                "21W0000000000354",
                "21W000000000119Z",
                "21W000000000032A",
                "18W000000000GVMT",
                "44W-00000000000X",
                "44W-00000000001V",
                "44W-00000000002T",
                "44W-00000000004P",
                "44W-00000000005N",
                "44W-T-HY-0000005",
                "44W-00000000003R",
                "44W-T-YT-000020M",
                "63W197197128864M",
                "21W0000000000451",
                "63W631527814486R",
                "63W943693783886F",
                "21W000000000084S",
                "21W0000000000710",
                "21W0000000000702",
                "21W000000000072Z",
                "21W000000000073X",
                "21W000000000069O",
                "21W000000000068Q",
                "21W000000000077P",
                "31W-0000-G-000-Z",
                "21W000000000086O",
                "21W000000000087M",
                "59W-IGSTORAGE-0Q",
                "21W0000000000443",
                "21W000000000082W",
                "59W0000000000011",
                "21W000000000095N",
                "21W0000000001253",
                "21W000000000113A",
                "21W0000000001229",
                "21W000000000012G",
                "21W0000000000079",
                "21W000000000002J",
                "21W0000000000087",
                "21W000000000001L",
                "21W000000000015A",
                "21W000000000014C",
                "21W000000000006B",
                "52W000000000001W",
                "21W000000000096L",
                "16WAS01--------D",
                "16WTGNL01------O",
                "21W0000000001237",
                "21W0000000001156",
                "21W0000000000435",
                "21W000000000088K",
                "21W000000000047Y",
                "59WFSRUGOLARTUNH",
                "21W000000000130A",
                "21W000000000128Y",
                "37W000000000001Q",
                "45W000000000112V",
                "63W835858561887S");
        return new GetAcceptedEicCodesImpl(acceptedEicCodes);
    }

    private final List<String> acceptedEicCodesCsv;

    public GetAcceptedEicCodesImpl(List<String> acceptedEicCodesCsv) {
        this.acceptedEicCodesCsv = acceptedEicCodesCsv;
    }

    @Override
    protected List<String> doEvaluate() {
        return acceptedEicCodesCsv;
    }
}
