package drr.projection.iso20022.esma.emir.refit.margin.functions;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.esma.emir.refit.margin.CollateralReport;
import iso20022.auth108.esma.ActiveOrHistoricCurrencyAnd5DecimalAmount;
import iso20022.auth108.esma.PostedMarginOrCollateral6__1;
import iso20022.auth108.esma.PostedMarginOrCollateral6__1.PostedMarginOrCollateral6__1Builder;
import java.math.BigDecimal;
import java.util.Optional;
import javax.inject.Inject;


@ImplementedBy(Create_PostedMarginReport.Create_PostedMarginReportDefault.class)
public abstract class Create_PostedMarginReport implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param collateralReport 
	* @return postMarginData 
	*/
	public PostedMarginOrCollateral6__1 evaluate(CollateralReport collateralReport) {
		PostedMarginOrCollateral6__1.PostedMarginOrCollateral6__1Builder postMarginDataBuilder = doEvaluate(collateralReport);
		
		final PostedMarginOrCollateral6__1 postMarginData;
		if (postMarginDataBuilder == null) {
			postMarginData = null;
		} else {
			postMarginData = postMarginDataBuilder.build();
			objectValidator.validate(PostedMarginOrCollateral6__1.class, postMarginData);
		}
		
		return postMarginData;
	}

	protected abstract PostedMarginOrCollateral6__1.PostedMarginOrCollateral6__1Builder doEvaluate(CollateralReport collateralReport);

	public static class Create_PostedMarginReportDefault extends Create_PostedMarginReport {
		@Override
		protected PostedMarginOrCollateral6__1.PostedMarginOrCollateral6__1Builder doEvaluate(CollateralReport collateralReport) {
			PostedMarginOrCollateral6__1.PostedMarginOrCollateral6__1Builder postMarginData = PostedMarginOrCollateral6__1.builder();
			return assignOutput(postMarginData, collateralReport);
		}
		
		protected PostedMarginOrCollateral6__1.PostedMarginOrCollateral6__1Builder assignOutput(PostedMarginOrCollateral6__1.PostedMarginOrCollateral6__1Builder postMarginData, CollateralReport collateralReport) {
			postMarginData = toBuilder(PostedMarginOrCollateral6__1.builder()
				.setInitlMrgnPstdPreHrcut(ActiveOrHistoricCurrencyAnd5DecimalAmount.builder()
					.setValue(MapperS.of(collateralReport).<BigDecimal>map("getInitialMarginPostedByTheCounterparty1PreHaircut", _collateralReport -> _collateralReport.getInitialMarginPostedByTheCounterparty1PreHaircut()).get())
					.setCcy(MapperS.of(collateralReport).<ISOCurrencyCodeEnum>map("getInitialMarginPostedByCounterparty1Currency", _collateralReport -> _collateralReport.getInitialMarginPostedByCounterparty1Currency()).map("to-string", ISOCurrencyCodeEnum::toDisplayString).get())
					.build()
				)
				.setInitlMrgnPstdPstHrcut(ActiveOrHistoricCurrencyAnd5DecimalAmount.builder()
					.setValue(MapperS.of(collateralReport).<BigDecimal>map("getInitialMarginPostedByTheCounterparty1PostHaircut", _collateralReport -> _collateralReport.getInitialMarginPostedByTheCounterparty1PostHaircut()).get())
					.setCcy(MapperS.of(collateralReport).<ISOCurrencyCodeEnum>map("getInitialMarginPostedByCounterparty1Currency", _collateralReport -> _collateralReport.getInitialMarginPostedByCounterparty1Currency()).map("to-string", ISOCurrencyCodeEnum::toDisplayString).get())
					.build()
				)
				.setVartnMrgnPstdPreHrcut(ActiveOrHistoricCurrencyAnd5DecimalAmount.builder()
					.setValue(MapperS.of(collateralReport).<BigDecimal>map("getVariationMarginPostedByTheCounterparty1PreHaircut", _collateralReport -> _collateralReport.getVariationMarginPostedByTheCounterparty1PreHaircut()).get())
					.setCcy(MapperS.of(collateralReport).<ISOCurrencyCodeEnum>map("getVariationMarginPostedByCounterparty1Currency", _collateralReport -> _collateralReport.getVariationMarginPostedByCounterparty1Currency()).map("to-string", ISOCurrencyCodeEnum::toDisplayString).get())
					.build()
				)
				.setVartnMrgnPstdPstHrcut(ActiveOrHistoricCurrencyAnd5DecimalAmount.builder()
					.setValue(MapperS.of(collateralReport).<BigDecimal>map("getVariationMarginPostedByTheCounterparty1PostHaircut", _collateralReport -> _collateralReport.getVariationMarginPostedByTheCounterparty1PostHaircut()).get())
					.setCcy(MapperS.of(collateralReport).<ISOCurrencyCodeEnum>map("getVariationMarginPostedByCounterparty1Currency", _collateralReport -> _collateralReport.getVariationMarginPostedByCounterparty1Currency()).map("to-string", ISOCurrencyCodeEnum::toDisplayString).get())
					.build()
				)
				.setXcssCollPstd(ActiveOrHistoricCurrencyAnd5DecimalAmount.builder()
					.setValue(MapperS.of(collateralReport).<BigDecimal>map("getExcessCollateralPostedByTheCounterparty1", _collateralReport -> _collateralReport.getExcessCollateralPostedByTheCounterparty1()).get())
					.setCcy(MapperS.of(collateralReport).<String>map("getCurrencyOfTheExcessCollateralPosted", _collateralReport -> _collateralReport.getCurrencyOfTheExcessCollateralPosted()).map("to-string", Object::toString).get())
					.build()
				)
				.build()
			);
			
			return Optional.ofNullable(postMarginData)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
