package drr.projection.iso20022.esma.emir.refit.trade.functions;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRTransactionReport;
import drr.regulation.esma.emir.refit.trade.PackageTransactionPrice;
import drr.regulation.esma.emir.refit.trade.PackageTransactionSpread;
import drr.standards.iso.PriceNotationEnum;
import iso20022.auth030.esma.ActiveOrHistoricCurrencyAnd13DecimalAmount__1;
import iso20022.auth030.esma.AmountAndDirection106__1;
import iso20022.auth030.esma.AmountAndDirection106__4;
import iso20022.auth030.esma.Package4__1;
import iso20022.auth030.esma.Package4__1.Package4__1Builder;
import iso20022.auth030.esma.SecuritiesTransactionPrice17Choice__2;
import iso20022.auth030.esma.SecuritiesTransactionPrice20Choice__1;
import java.math.BigDecimal;
import java.util.Optional;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(GetPackg.GetPackgDefault.class)
public abstract class GetPackg implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param drrReport 
	* @return packg 
	*/
	public Package4__1 evaluate(ESMAEMIRTransactionReport drrReport) {
		Package4__1.Package4__1Builder packgBuilder = doEvaluate(drrReport);
		
		final Package4__1 packg;
		if (packgBuilder == null) {
			packg = null;
		} else {
			packg = packgBuilder.build();
			objectValidator.validate(Package4__1.class, packg);
		}
		
		return packg;
	}

	protected abstract Package4__1.Package4__1Builder doEvaluate(ESMAEMIRTransactionReport drrReport);

	public static class GetPackgDefault extends GetPackg {
		@Override
		protected Package4__1.Package4__1Builder doEvaluate(ESMAEMIRTransactionReport drrReport) {
			Package4__1.Package4__1Builder packg = Package4__1.builder();
			return assignOutput(packg, drrReport);
		}
		
		protected Package4__1.Package4__1Builder assignOutput(Package4__1.Package4__1Builder packg, ESMAEMIRTransactionReport drrReport) {
			packg = toBuilder(Package4__1.builder()
				.setCmplxTradId(MapperS.of(drrReport).<String>map("getPackageIdentifier", eSMAEMIRTransactionReport -> eSMAEMIRTransactionReport.getPackageIdentifier()).get())
				.setPric(MapperS.of(drrReport).<PackageTransactionPrice>map("getPackageTransactionPrice", eSMAEMIRTransactionReport -> eSMAEMIRTransactionReport.getPackageTransactionPrice())
					.mapSingleToItem(item -> {
						AmountAndDirection106__4 ifThenElseResult0 = null;
						if (areEqual(item.<PriceNotationEnum>map("getPackageTransactionPriceNotation", packageTransactionPrice -> packageTransactionPrice.getPackageTransactionPriceNotation()), MapperS.of(PriceNotationEnum.MONETARY), CardinalityOperator.All).getOrDefault(false)) {
							ifThenElseResult0 = AmountAndDirection106__4.builder()
								.setAmt(ActiveOrHistoricCurrencyAnd13DecimalAmount__1.builder()
									.setValue(item.<BigDecimal>map("getPackageTransactionPriceMonetary", packageTransactionPrice -> packageTransactionPrice.getPackageTransactionPriceMonetary()).get())
									.setCcy(MapperS.of(drrReport).<PackageTransactionPrice>map("getPackageTransactionPrice", eSMAEMIRTransactionReport -> eSMAEMIRTransactionReport.getPackageTransactionPrice()).<ISOCurrencyCodeEnum>map("getPackageTransactionPriceCurrency", packageTransactionPrice -> packageTransactionPrice.getPackageTransactionPriceCurrency()).map("to-string", ISOCurrencyCodeEnum::toDisplayString).get())
									.build()
								)
								.build()
							;
						}
						BigDecimal ifThenElseResult1 = null;
						if (areEqual(item.<PriceNotationEnum>map("getPackageTransactionPriceNotation", packageTransactionPrice -> packageTransactionPrice.getPackageTransactionPriceNotation()), MapperS.of(PriceNotationEnum.PERCENTAGE), CardinalityOperator.All).getOrDefault(false)) {
							ifThenElseResult1 = item.<BigDecimal>map("getPackageTransactionPricePercentage", packageTransactionPrice -> packageTransactionPrice.getPackageTransactionPricePercentage()).get();
						}
						return MapperS.of(SecuritiesTransactionPrice17Choice__2.builder()
							.setMntryVal(ifThenElseResult0)
							.setPctg(ifThenElseResult1)
							.build()
						);
					}).get())
				.setSprd(MapperS.of(drrReport).<PackageTransactionSpread>map("getPackageTransactionSpread", eSMAEMIRTransactionReport -> eSMAEMIRTransactionReport.getPackageTransactionSpread())
					.mapSingleToItem(item -> {
						AmountAndDirection106__1 ifThenElseResult0 = null;
						if (areEqual(item.<PriceNotationEnum>map("getPackageTransactionSpreadNotation", packageTransactionSpread -> packageTransactionSpread.getPackageTransactionSpreadNotation()), MapperS.of(PriceNotationEnum.MONETARY), CardinalityOperator.All).getOrDefault(false)) {
							ifThenElseResult0 = AmountAndDirection106__1.builder()
								.setAmt(ActiveOrHistoricCurrencyAnd13DecimalAmount__1.builder()
									.setValue(item.<BigDecimal>map("getPackageTransactionSpreadMonetary", packageTransactionSpread -> packageTransactionSpread.getPackageTransactionSpreadMonetary()).get())
									.setCcy(MapperS.of(drrReport).<PackageTransactionSpread>map("getPackageTransactionSpread", eSMAEMIRTransactionReport -> eSMAEMIRTransactionReport.getPackageTransactionSpread()).<ISOCurrencyCodeEnum>map("getPackageTransactionSpreadCurrency", packageTransactionSpread -> packageTransactionSpread.getPackageTransactionSpreadCurrency()).map("to-string", ISOCurrencyCodeEnum::toDisplayString).get())
									.build()
								)
								.build()
							;
						}
						BigDecimal ifThenElseResult1 = null;
						if (areEqual(item.<PriceNotationEnum>map("getPackageTransactionSpreadNotation", packageTransactionSpread -> packageTransactionSpread.getPackageTransactionSpreadNotation()), MapperS.of(PriceNotationEnum.PERCENTAGE), CardinalityOperator.All).getOrDefault(false)) {
							ifThenElseResult1 = item.<BigDecimal>map("getPackageTransactionSpreadPercentage", packageTransactionSpread -> packageTransactionSpread.getPackageTransactionSpreadPercentage()).get();
						}
						Integer ifThenElseResult2 = null;
						if (areEqual(item.<PriceNotationEnum>map("getPackageTransactionSpreadNotation", packageTransactionSpread -> packageTransactionSpread.getPackageTransactionSpreadNotation()), MapperS.of(PriceNotationEnum.BASIS), CardinalityOperator.All).getOrDefault(false)) {
							final BigDecimal bigDecimal = item.<BigDecimal>map("getPackageTransactionSpreadBasis", packageTransactionSpread -> packageTransactionSpread.getPackageTransactionSpreadBasis()).get();
							ifThenElseResult2 = bigDecimal == null ? null : bigDecimal.intValueExact();
						}
						return MapperS.of(SecuritiesTransactionPrice20Choice__1.builder()
							.setMntryVal(ifThenElseResult0)
							.setPctg(ifThenElseResult1)
							.setBsisPtSprd(ifThenElseResult2)
							.build()
						);
					}).get())
				.build()
			);
			
			return Optional.ofNullable(packg)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
