package drr.projection.iso20022.esma.emir.refit.trade.projections;

import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import iso20022.auth030.esma.Schedule10__1;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class Schedule10__1Project_EsmaEmirTradeReportToIso20022TypeTabulator implements Tabulator<Schedule10__1> {
	private final Field qtyField;
	private final Field uadjstdFctvDtField;
	private final Field uadjstdEndDtField;
	
	public Schedule10__1Project_EsmaEmirTradeReportToIso20022TypeTabulator() {
		this.qtyField = new FieldImpl(
			"qty",
			false,
			Optional.empty(),
			Optional.empty(),
			Arrays.asList()
		);
		this.uadjstdFctvDtField = new FieldImpl(
			"uadjstdFctvDt",
			false,
			Optional.empty(),
			Optional.empty(),
			Arrays.asList()
		);
		this.uadjstdEndDtField = new FieldImpl(
			"uadjstdEndDt",
			false,
			Optional.empty(),
			Optional.empty(),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(qtyField, uadjstdFctvDtField, uadjstdEndDtField);
	}
	
	@Override
	public List<FieldValue> tabulate(Schedule10__1 input) {
		Optional<BigDecimal> qty = Optional.ofNullable(input.getQty());
		Optional<Date> uadjstdFctvDt = Optional.ofNullable(input.getUadjstdFctvDt());
		Optional<Date> uadjstdEndDt = Optional.ofNullable(input.getUadjstdEndDt());
		return Arrays.asList(
			new FieldValueImpl(qtyField, qty),
			new FieldValueImpl(uadjstdFctvDtField, uadjstdFctvDt),
			new FieldValueImpl(uadjstdEndDtField, uadjstdEndDt)
		);
	}
}
