package drr.projection.iso20022.jfsa.rewrite.trade.functions;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.records.Date;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;
import drr.regulation.jfsa.rewrite.trade.OtherPaymentReport;
import iso20022.auth030.jfsa.ActiveOrHistoricCurrencyAnd5DecimalAmount;
import iso20022.auth030.jfsa.AmountAndDirection106__2;
import iso20022.auth030.jfsa.OrganisationIdentification15Choice__2;
import iso20022.auth030.jfsa.OtherPayment5__1;
import iso20022.auth030.jfsa.OtherPayment5__1.OtherPayment5__1Builder;
import iso20022.auth030.jfsa.PartyIdentification236Choice__1;
import iso20022.auth030.jfsa.PaymentType4Code;
import iso20022.auth030.jfsa.PaymentType5Choice__1;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.inject.Inject;


@ImplementedBy(GetOthrPmt.GetOthrPmtDefault.class)
public abstract class GetOthrPmt implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param drrReport 
	* @return othrPmt 
	*/
	public List<? extends OtherPayment5__1> evaluate(JFSATransactionReport drrReport) {
		List<OtherPayment5__1.OtherPayment5__1Builder> othrPmtBuilder = doEvaluate(drrReport);
		
		final List<? extends OtherPayment5__1> othrPmt;
		if (othrPmtBuilder == null) {
			othrPmt = null;
		} else {
			othrPmt = othrPmtBuilder.stream().map(OtherPayment5__1::build).collect(Collectors.toList());
			objectValidator.validate(OtherPayment5__1.class, othrPmt);
		}
		
		return othrPmt;
	}

	protected abstract List<OtherPayment5__1.OtherPayment5__1Builder> doEvaluate(JFSATransactionReport drrReport);

	public static class GetOthrPmtDefault extends GetOthrPmt {
		@Override
		protected List<OtherPayment5__1.OtherPayment5__1Builder> doEvaluate(JFSATransactionReport drrReport) {
			List<OtherPayment5__1.OtherPayment5__1Builder> othrPmt = new ArrayList<>();
			return assignOutput(othrPmt, drrReport);
		}
		
		protected List<OtherPayment5__1.OtherPayment5__1Builder> assignOutput(List<OtherPayment5__1.OtherPayment5__1Builder> othrPmt, JFSATransactionReport drrReport) {
			othrPmt.addAll(toBuilder(MapperS.of(drrReport).<OtherPaymentReport>mapC("getOtherPayment", jFSATransactionReport -> jFSATransactionReport.getOtherPayment())
				.mapItem(item -> MapperS.of(OtherPayment5__1.builder()
					.setPmtTp(PaymentType5Choice__1.builder()
						.setTp(item.<String>map("getPaymentType", otherPaymentReport -> otherPaymentReport.getPaymentType()).checkedMap("to-enum", PaymentType4Code::fromDisplayName, IllegalArgumentException.class).get())
						.build()
					)
					.setPmtAmt(AmountAndDirection106__2.builder()
						.setAmt(ActiveOrHistoricCurrencyAnd5DecimalAmount.builder()
							.setValue(item.<BigDecimal>map("getAmount", otherPaymentReport -> otherPaymentReport.getAmount()).get())
							.setCcy(item.<ISOCurrencyCodeEnum>map("getCurrency", otherPaymentReport -> otherPaymentReport.getCurrency()).map("to-string", ISOCurrencyCodeEnum::toDisplayString).get())
							.build()
						)
						.build()
					)
					.setPmtDt(item.<Date>map("getDate", otherPaymentReport -> otherPaymentReport.getDate()).get())
					.setPmtPyer(PartyIdentification236Choice__1.builder()
						.setLgl(OrganisationIdentification15Choice__2.builder()
							.setLei(item.<String>map("getPayer", otherPaymentReport -> otherPaymentReport.getPayer()).get())
							.build()
						)
						.build()
					)
					.setPmtRcvr(PartyIdentification236Choice__1.builder()
						.setLgl(OrganisationIdentification15Choice__2.builder()
							.setLei(item.<String>map("getReceiver", otherPaymentReport -> otherPaymentReport.getReceiver()).get())
							.build()
						)
						.build()
					)
					.build()
				)).getMulti()));
			
			return Optional.ofNullable(othrPmt)
				.map(o -> o.stream().map(i -> i.prune()).collect(Collectors.toList()))
				.orElse(null);
		}
	}
}
