package drr.projection.iso20022.jfsa.rewrite.trade.functions;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;
import drr.regulation.jfsa.rewrite.trade.PriceAndNotation;
import drr.standards.iso.PriceNotationEnum;
import iso20022.auth030.jfsa.ActiveOrHistoricCurrencyAnd13DecimalAmount;
import iso20022.auth030.jfsa.AmountAndDirection106__1;
import iso20022.auth030.jfsa.SecuritiesTransactionPrice17Choice__1;
import iso20022.auth030.jfsa.SecuritiesTransactionPrice17Choice__1.SecuritiesTransactionPrice17Choice__1Builder;
import java.math.BigDecimal;
import java.util.Optional;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(GetPric.GetPricDefault.class)
public abstract class GetPric implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param drrReport 
	* @return pric 
	*/
	public SecuritiesTransactionPrice17Choice__1 evaluate(JFSATransactionReport drrReport) {
		SecuritiesTransactionPrice17Choice__1.SecuritiesTransactionPrice17Choice__1Builder pricBuilder = doEvaluate(drrReport);
		
		final SecuritiesTransactionPrice17Choice__1 pric;
		if (pricBuilder == null) {
			pric = null;
		} else {
			pric = pricBuilder.build();
			objectValidator.validate(SecuritiesTransactionPrice17Choice__1.class, pric);
		}
		
		return pric;
	}

	protected abstract SecuritiesTransactionPrice17Choice__1.SecuritiesTransactionPrice17Choice__1Builder doEvaluate(JFSATransactionReport drrReport);

	public static class GetPricDefault extends GetPric {
		@Override
		protected SecuritiesTransactionPrice17Choice__1.SecuritiesTransactionPrice17Choice__1Builder doEvaluate(JFSATransactionReport drrReport) {
			SecuritiesTransactionPrice17Choice__1.SecuritiesTransactionPrice17Choice__1Builder pric = SecuritiesTransactionPrice17Choice__1.builder();
			return assignOutput(pric, drrReport);
		}
		
		protected SecuritiesTransactionPrice17Choice__1.SecuritiesTransactionPrice17Choice__1Builder assignOutput(SecuritiesTransactionPrice17Choice__1.SecuritiesTransactionPrice17Choice__1Builder pric, JFSATransactionReport drrReport) {
			pric = toBuilder(MapperS.of(drrReport).<PriceAndNotation>map("getPrice", jFSATransactionReport -> jFSATransactionReport.getPrice())
				.mapSingleToItem(item -> {
					AmountAndDirection106__1 ifThenElseResult0 = null;
					if (areEqual(item.<PriceNotationEnum>map("getPriceNotation", priceAndNotation -> priceAndNotation.getPriceNotation()), MapperS.of(PriceNotationEnum.MONETARY), CardinalityOperator.All).getOrDefault(false)) {
						ifThenElseResult0 = AmountAndDirection106__1.builder()
							.setAmt(ActiveOrHistoricCurrencyAnd13DecimalAmount.builder()
								.setValue(item.<BigDecimal>map("getPriceMonetary", priceAndNotation -> priceAndNotation.getPriceMonetary()).get())
								.setCcy(MapperS.of(drrReport).<PriceAndNotation>map("getPrice", jFSATransactionReport -> jFSATransactionReport.getPrice()).<ISOCurrencyCodeEnum>map("getPriceCurrency", priceAndNotation -> priceAndNotation.getPriceCurrency()).map("to-string", ISOCurrencyCodeEnum::toDisplayString).get())
								.build()
							)
							.build()
						;
					}
					BigDecimal ifThenElseResult1 = null;
					if (areEqual(item.<PriceNotationEnum>map("getPriceNotation", priceAndNotation -> priceAndNotation.getPriceNotation()), MapperS.of(PriceNotationEnum.PERCENTAGE), CardinalityOperator.All).getOrDefault(false)) {
						ifThenElseResult1 = item.<BigDecimal>map("getPricePercentage", priceAndNotation -> priceAndNotation.getPricePercentage()).get();
					}
					return MapperS.of(SecuritiesTransactionPrice17Choice__1.builder()
						.setMntryVal(ifThenElseResult0)
						.setDcml(ifThenElseResult1)
						.build()
					);
				}).get());
			
			return Optional.ofNullable(pric)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
