package drr.projection.iso20022.jfsa.rewrite.trade.functions;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;
import drr.regulation.jfsa.rewrite.trade.StrikePriceAndNotation;
import drr.standards.iso.PriceNotationEnum;
import iso20022.auth030.jfsa.ActiveOrHistoricCurrencyAnd13DecimalAmount;
import iso20022.auth030.jfsa.AmountAndDirection106__1;
import iso20022.auth030.jfsa.SecuritiesTransactionPrice17Choice__1;
import iso20022.auth030.jfsa.SecuritiesTransactionPrice17Choice__1.SecuritiesTransactionPrice17Choice__1Builder;
import java.math.BigDecimal;
import java.util.Optional;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(GetStrkPric.GetStrkPricDefault.class)
public abstract class GetStrkPric implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param drrReport 
	* @return strkPric 
	*/
	public SecuritiesTransactionPrice17Choice__1 evaluate(JFSATransactionReport drrReport) {
		SecuritiesTransactionPrice17Choice__1.SecuritiesTransactionPrice17Choice__1Builder strkPricBuilder = doEvaluate(drrReport);
		
		final SecuritiesTransactionPrice17Choice__1 strkPric;
		if (strkPricBuilder == null) {
			strkPric = null;
		} else {
			strkPric = strkPricBuilder.build();
			objectValidator.validate(SecuritiesTransactionPrice17Choice__1.class, strkPric);
		}
		
		return strkPric;
	}

	protected abstract SecuritiesTransactionPrice17Choice__1.SecuritiesTransactionPrice17Choice__1Builder doEvaluate(JFSATransactionReport drrReport);

	public static class GetStrkPricDefault extends GetStrkPric {
		@Override
		protected SecuritiesTransactionPrice17Choice__1.SecuritiesTransactionPrice17Choice__1Builder doEvaluate(JFSATransactionReport drrReport) {
			SecuritiesTransactionPrice17Choice__1.SecuritiesTransactionPrice17Choice__1Builder strkPric = SecuritiesTransactionPrice17Choice__1.builder();
			return assignOutput(strkPric, drrReport);
		}
		
		protected SecuritiesTransactionPrice17Choice__1.SecuritiesTransactionPrice17Choice__1Builder assignOutput(SecuritiesTransactionPrice17Choice__1.SecuritiesTransactionPrice17Choice__1Builder strkPric, JFSATransactionReport drrReport) {
			strkPric = toBuilder(MapperS.of(drrReport).<StrikePriceAndNotation>map("getStrikePrice", jFSATransactionReport -> jFSATransactionReport.getStrikePrice())
				.mapSingleToItem(item -> {
					AmountAndDirection106__1 ifThenElseResult1 = null;
					if (areEqual(item.<PriceNotationEnum>map("getStrikePriceNotation", strikePriceAndNotation -> strikePriceAndNotation.getStrikePriceNotation()), MapperS.of(PriceNotationEnum.MONETARY), CardinalityOperator.All).getOrDefault(false)) {
						String ifThenElseResult0 = null;
						if (areEqual(MapperS.of(drrReport).<String>map("getContractType", jFSATransactionReport -> jFSATransactionReport.getContractType()), MapperS.of("OPTN"), CardinalityOperator.All).or(areEqual(MapperS.of(drrReport).<String>map("getContractType", jFSATransactionReport -> jFSATransactionReport.getContractType()), MapperS.of("SWPT"), CardinalityOperator.All)).and(areEqual(areEqual(MapperS.of(drrReport).<String>map("getAssetClass", jFSATransactionReport -> jFSATransactionReport.getAssetClass()), MapperS.of("CURR"), CardinalityOperator.All).or(areEqual(MapperS.of(drrReport).<String>map("getAssetClass", jFSATransactionReport -> jFSATransactionReport.getAssetClass()), MapperS.of("INTR"), CardinalityOperator.All)), MapperS.of(false), CardinalityOperator.All)).getOrDefault(false)) {
							ifThenElseResult0 = MapperS.of(drrReport).<StrikePriceAndNotation>map("getStrikePrice", jFSATransactionReport -> jFSATransactionReport.getStrikePrice()).<String>map("getStrikePriceCurrency", strikePriceAndNotation -> strikePriceAndNotation.getStrikePriceCurrency()).map("to-string", Object::toString).get();
						}
						ifThenElseResult1 = AmountAndDirection106__1.builder()
							.setAmt(ActiveOrHistoricCurrencyAnd13DecimalAmount.builder()
								.setValue(item.<BigDecimal>map("getStrikePriceMonetary", strikePriceAndNotation -> strikePriceAndNotation.getStrikePriceMonetary()).get())
								.setCcy(ifThenElseResult0)
								.build()
							)
							.build()
						;
					}
					BigDecimal ifThenElseResult2 = null;
					if (areEqual(item.<PriceNotationEnum>map("getStrikePriceNotation", strikePriceAndNotation -> strikePriceAndNotation.getStrikePriceNotation()), MapperS.of(PriceNotationEnum.DECIMAL), CardinalityOperator.All).getOrDefault(false)) {
						ifThenElseResult2 = item.<BigDecimal>map("getStrikePriceDecimal", strikePriceAndNotation -> strikePriceAndNotation.getStrikePriceDecimal()).get();
					}
					return MapperS.of(SecuritiesTransactionPrice17Choice__1.builder()
						.setMntryVal(ifThenElseResult1)
						.setDcml(ifThenElseResult2)
						.build()
					);
				}).get());
			
			return Optional.ofNullable(strkPric)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
