package drr.projection.iso20022.jfsa.rewrite.trade.functions;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.records.Date;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;
import drr.regulation.jfsa.rewrite.trade.PriceAndNotation;
import drr.regulation.jfsa.rewrite.trade.PriceScheduleReport;
import iso20022.auth030.jfsa.ActiveOrHistoricCurrencyAnd13DecimalAmount;
import iso20022.auth030.jfsa.AmountAndDirection106__1;
import iso20022.auth030.jfsa.PriceData2__1;
import iso20022.auth030.jfsa.PriceData2__1.PriceData2__1Builder;
import iso20022.auth030.jfsa.Schedule1__1;
import iso20022.auth030.jfsa.SecuritiesTransactionPrice17Choice__1;
import java.math.BigDecimal;
import java.util.Optional;
import javax.inject.Inject;


@ImplementedBy(GetTxPric1.GetTxPric1Default.class)
public abstract class GetTxPric1 implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected Create_UnitOfMeasure8Choice__1 create_UnitOfMeasure8Choice__1;
	@Inject protected GetPric getPric;

	/**
	* @param drrReport 
	* @return txPric 
	*/
	public PriceData2__1 evaluate(JFSATransactionReport drrReport) {
		PriceData2__1.PriceData2__1Builder txPricBuilder = doEvaluate(drrReport);
		
		final PriceData2__1 txPric;
		if (txPricBuilder == null) {
			txPric = null;
		} else {
			txPric = txPricBuilder.build();
			objectValidator.validate(PriceData2__1.class, txPric);
		}
		
		return txPric;
	}

	protected abstract PriceData2__1.PriceData2__1Builder doEvaluate(JFSATransactionReport drrReport);

	public static class GetTxPric1Default extends GetTxPric1 {
		@Override
		protected PriceData2__1.PriceData2__1Builder doEvaluate(JFSATransactionReport drrReport) {
			PriceData2__1.PriceData2__1Builder txPric = PriceData2__1.builder();
			return assignOutput(txPric, drrReport);
		}
		
		protected PriceData2__1.PriceData2__1Builder assignOutput(PriceData2__1.PriceData2__1Builder txPric, JFSATransactionReport drrReport) {
			txPric = toBuilder(PriceData2__1.builder()
				.setPric(getPric.evaluate(drrReport))
				.setSchdlPrd(MapperS.of(drrReport).<PriceScheduleReport>mapC("getPriceSchedule", jFSATransactionReport -> jFSATransactionReport.getPriceSchedule())
					.mapItem(item -> MapperS.of(Schedule1__1.builder()
						.setUadjstdFctvDt(item.<Date>map("getEffectiveDate", priceScheduleReport -> priceScheduleReport.getEffectiveDate()).get())
						.setUadjstdEndDt(item.<Date>map("getEndDate", priceScheduleReport -> priceScheduleReport.getEndDate()).get())
						.setPric(SecuritiesTransactionPrice17Choice__1.builder()
							.setMntryVal(AmountAndDirection106__1.builder()
								.setAmt(ActiveOrHistoricCurrencyAnd13DecimalAmount.builder()
									.setValue(item.<BigDecimal>map("getPriceMonetary", priceScheduleReport -> priceScheduleReport.getPriceMonetary()).get())
									.setCcy(MapperS.of(drrReport).<PriceAndNotation>map("getPrice", jFSATransactionReport -> jFSATransactionReport.getPrice()).<ISOCurrencyCodeEnum>map("getPriceCurrency", priceAndNotation -> priceAndNotation.getPriceCurrency()).map("to-string", ISOCurrencyCodeEnum::toDisplayString).get())
									.build()
								)
								.build()
							)
							.setDcml(item.<BigDecimal>map("getPricePercentage", priceScheduleReport -> priceScheduleReport.getPricePercentage()).get())
							.build()
						)
						.build()
					)).getMulti())
				.setUnitOfMeasr(create_UnitOfMeasure8Choice__1.evaluate(MapperS.of(drrReport).<String>map("getPriceUnitOfMeasure", jFSATransactionReport -> jFSATransactionReport.getPriceUnitOfMeasure()).get()))
				.build()
			);
			
			return Optional.ofNullable(txPric)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
