package drr.projection.iso20022.jfsa.rewrite.trade.functions;

import cdm.base.staticdata.asset.common.ProductIdTypeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;
import iso20022.auth030.jfsa.BasketConstituents3__1;
import iso20022.auth030.jfsa.CustomBasket4__1;
import iso20022.auth030.jfsa.GenericIdentification184;
import iso20022.auth030.jfsa.IndexIdentification1__1;
import iso20022.auth030.jfsa.InstrumentIdentification6Choice__1;
import iso20022.auth030.jfsa.SecurityIdentification41Choice__1;
import iso20022.auth030.jfsa.SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder;
import java.util.Collections;
import java.util.Optional;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(GetUndrlygInstrm.GetUndrlygInstrmDefault.class)
public abstract class GetUndrlygInstrm implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param drrReport 
	* @return undrlygInstrm 
	*/
	public SecurityIdentification41Choice__1 evaluate(JFSATransactionReport drrReport) {
		SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder undrlygInstrmBuilder = doEvaluate(drrReport);
		
		final SecurityIdentification41Choice__1 undrlygInstrm;
		if (undrlygInstrmBuilder == null) {
			undrlygInstrm = null;
		} else {
			undrlygInstrm = undrlygInstrmBuilder.build();
			objectValidator.validate(SecurityIdentification41Choice__1.class, undrlygInstrm);
		}
		
		return undrlygInstrm;
	}

	protected abstract SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder doEvaluate(JFSATransactionReport drrReport);

	public static class GetUndrlygInstrmDefault extends GetUndrlygInstrm {
		@Override
		protected SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder doEvaluate(JFSATransactionReport drrReport) {
			SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder undrlygInstrm = SecurityIdentification41Choice__1.builder();
			return assignOutput(undrlygInstrm, drrReport);
		}
		
		protected SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder assignOutput(SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder undrlygInstrm, JFSATransactionReport drrReport) {
			String ifThenElseResult0 = null;
			if (areEqual(MapperS.of(drrReport).<String>map("getUnderlyingIdentificationType", jFSATransactionReport -> jFSATransactionReport.getUnderlyingIdentificationType()), MapperS.of("I"), CardinalityOperator.All).getOrDefault(false)) {
				ifThenElseResult0 = MapperS.of(drrReport).<String>map("getUnderlyingIdentification", jFSATransactionReport -> jFSATransactionReport.getUnderlyingIdentification()).get();
			}
			String ifThenElseResult1 = null;
			if (areEqual(MapperS.of(drrReport).<String>map("getUnderlyingIdentificationType", jFSATransactionReport -> jFSATransactionReport.getUnderlyingIdentificationType()), MapperS.of("X"), CardinalityOperator.All).getOrDefault(false)) {
				ifThenElseResult1 = MapperS.of(drrReport).<String>map("getUnderlyingIndexIndicator", jFSATransactionReport -> jFSATransactionReport.getUnderlyingIndexIndicator()).map("to-string", Object::toString).get();
			}
			String ifThenElseResult2 = null;
			if (areEqual(MapperS.of(drrReport).<String>map("getUnderlyingIdentificationType", jFSATransactionReport -> jFSATransactionReport.getUnderlyingIdentificationType()), MapperS.of("X"), CardinalityOperator.All).getOrDefault(false)) {
				ifThenElseResult2 = MapperS.of(drrReport).<String>map("getNameOfTheUnderlyingIndex", jFSATransactionReport -> jFSATransactionReport.getNameOfTheUnderlyingIndex()).get();
			}
			String ifThenElseResult3 = null;
			if (areEqual(MapperS.of(drrReport).<ProductIdTypeEnum>mapC("getBasketConstituentIdentifierSource", jFSATransactionReport -> jFSATransactionReport.getBasketConstituentIdentifierSource()), MapperS.of(ProductIdTypeEnum.ISIN), CardinalityOperator.All).getOrDefault(false)) {
				ifThenElseResult3 = MapperS.of(drrReport).<String>mapC("getIdentifierOfBasketConstituents", jFSATransactionReport -> jFSATransactionReport.getIdentifierOfBasketConstituents()).get();
			}
			String ifThenElseResult4 = null;
			if (notEqual(MapperS.of(drrReport).<ProductIdTypeEnum>mapC("getBasketConstituentIdentifierSource", jFSATransactionReport -> jFSATransactionReport.getBasketConstituentIdentifierSource()), MapperS.of(ProductIdTypeEnum.ISIN), CardinalityOperator.All).getOrDefault(false)) {
				ifThenElseResult4 = MapperS.of(drrReport).<String>mapC("getIdentifierOfBasketConstituents", jFSATransactionReport -> jFSATransactionReport.getIdentifierOfBasketConstituents()).get();
			}
			final BasketConstituents3__1 basketConstituents3__1 = BasketConstituents3__1.builder()
				.setInstrmId(InstrumentIdentification6Choice__1.builder()
					.setIsin(ifThenElseResult3)
					.setOthrId(GenericIdentification184.builder()
						.setId(ifThenElseResult4)
						.setSrc(MapperS.of(MapperS.of(drrReport).<ProductIdTypeEnum>mapC("getBasketConstituentIdentifierSource", jFSATransactionReport -> jFSATransactionReport.getBasketConstituentIdentifierSource()).get()).map("to-string", ProductIdTypeEnum::toDisplayString).get())
						.build()
					)
					.build()
				)
				.build()
			;
			undrlygInstrm = toBuilder(SecurityIdentification41Choice__1.builder()
				.setIsin(ifThenElseResult0)
				.setIndx(IndexIdentification1__1.builder()
					.setIndx(ifThenElseResult1)
					.setNm(ifThenElseResult2)
					.build()
				)
				.setBskt(CustomBasket4__1.builder()
					.setId(MapperS.of(drrReport).<String>map("getCustomBasketCode", jFSATransactionReport -> jFSATransactionReport.getCustomBasketCode()).get())
					.setCnsttnts((basketConstituents3__1 == null ? Collections.<BasketConstituents3__1>emptyList() : Collections.singletonList(basketConstituents3__1)))
					.build()
				)
				.build()
			);
			
			return Optional.ofNullable(undrlygInstrm)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
