package drr.regulation.asic.rewrite.margin;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import drr.regulation.asic.rewrite.margin.ASICMarginReport;
import drr.regulation.asic.rewrite.margin.ASICMarginReport.ASICMarginReportBuilder;
import drr.regulation.asic.rewrite.margin.ASICMarginReport.ASICMarginReportBuilderImpl;
import drr.regulation.asic.rewrite.margin.ASICMarginReport.ASICMarginReportImpl;
import drr.regulation.asic.rewrite.margin.meta.ASICMarginReportMeta;
import drr.regulation.common.MarginActionEnum;
import iso20022.auth108.esma.CollateralisationType3Code__1;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="ASICMarginReport", builder=ASICMarginReport.ASICMarginReportBuilderImpl.class, version="4.2.1")
public interface ASICMarginReport extends RosettaModelObject {

	ASICMarginReportMeta metaData = new ASICMarginReportMeta();

	/*********************** Getter Methods  ***********************/
	String getUniqueTransactionIdentifier();
	String getReportingEntity();
	String getCounterparty1();
	String getSmallScaleBuySideEntityIndicator();
	String getCounterparty2();
	ZonedDateTime getCollateralTimestamp();
	CollateralisationType3Code__1 getCollateralisationCategory();
	Boolean getCollateralPortfolioIndicator();
	String getPortfolioContainingNonReportedComponentIndicator();
	String getInitialMarginCollateralPortfolioCode();
	String getCollateralPortfolioCodeVM();
	BigDecimal getInitialMarginPostedByTheReportingEntityPreHaircut();
	BigDecimal getInitialMarginPostedByTheReportingEntityPostHaircut();
	ISOCurrencyCodeEnum getCurrencyofInitialMarginPosted();
	BigDecimal getInitialMarginCollectedByTheReportingEntityPreHaircut();
	BigDecimal getInitialMarginCollectedByTheReportingEntityPostHaircut();
	ISOCurrencyCodeEnum getCurrencyofInitialMarginCollected();
	BigDecimal getVariationMarginPostedByTheReportingEntityPreHaircut();
	ISOCurrencyCodeEnum getCurrencyOfVariationMarginPosted();
	BigDecimal getVariationMarginCollectedByTheReportingEntityPreHaircut();
	ISOCurrencyCodeEnum getCurrencyOfVariationMarginCollected();
	MarginActionEnum getActionType();
	ZonedDateTime getReportingTimestamp();
	String getReportSubmittingEntityID();

	/*********************** Build Methods  ***********************/
	ASICMarginReport build();
	
	ASICMarginReport.ASICMarginReportBuilder toBuilder();
	
	static ASICMarginReport.ASICMarginReportBuilder builder() {
		return new ASICMarginReport.ASICMarginReportBuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends ASICMarginReport> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends ASICMarginReport> getType() {
		return ASICMarginReport.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("uniqueTransactionIdentifier"), String.class, getUniqueTransactionIdentifier(), this);
		processor.processBasic(path.newSubPath("reportingEntity"), String.class, getReportingEntity(), this);
		processor.processBasic(path.newSubPath("counterparty1"), String.class, getCounterparty1(), this);
		processor.processBasic(path.newSubPath("smallScaleBuySideEntityIndicator"), String.class, getSmallScaleBuySideEntityIndicator(), this);
		processor.processBasic(path.newSubPath("counterparty2"), String.class, getCounterparty2(), this);
		processor.processBasic(path.newSubPath("collateralTimestamp"), ZonedDateTime.class, getCollateralTimestamp(), this);
		processor.processBasic(path.newSubPath("collateralisationCategory"), CollateralisationType3Code__1.class, getCollateralisationCategory(), this);
		processor.processBasic(path.newSubPath("collateralPortfolioIndicator"), Boolean.class, getCollateralPortfolioIndicator(), this);
		processor.processBasic(path.newSubPath("portfolioContainingNonReportedComponentIndicator"), String.class, getPortfolioContainingNonReportedComponentIndicator(), this);
		processor.processBasic(path.newSubPath("initialMarginCollateralPortfolioCode"), String.class, getInitialMarginCollateralPortfolioCode(), this);
		processor.processBasic(path.newSubPath("collateralPortfolioCodeVM"), String.class, getCollateralPortfolioCodeVM(), this);
		processor.processBasic(path.newSubPath("initialMarginPostedByTheReportingEntityPreHaircut"), BigDecimal.class, getInitialMarginPostedByTheReportingEntityPreHaircut(), this);
		processor.processBasic(path.newSubPath("initialMarginPostedByTheReportingEntityPostHaircut"), BigDecimal.class, getInitialMarginPostedByTheReportingEntityPostHaircut(), this);
		processor.processBasic(path.newSubPath("currencyofInitialMarginPosted"), ISOCurrencyCodeEnum.class, getCurrencyofInitialMarginPosted(), this);
		processor.processBasic(path.newSubPath("initialMarginCollectedByTheReportingEntityPreHaircut"), BigDecimal.class, getInitialMarginCollectedByTheReportingEntityPreHaircut(), this);
		processor.processBasic(path.newSubPath("initialMarginCollectedByTheReportingEntityPostHaircut"), BigDecimal.class, getInitialMarginCollectedByTheReportingEntityPostHaircut(), this);
		processor.processBasic(path.newSubPath("currencyofInitialMarginCollected"), ISOCurrencyCodeEnum.class, getCurrencyofInitialMarginCollected(), this);
		processor.processBasic(path.newSubPath("variationMarginPostedByTheReportingEntityPreHaircut"), BigDecimal.class, getVariationMarginPostedByTheReportingEntityPreHaircut(), this);
		processor.processBasic(path.newSubPath("currencyOfVariationMarginPosted"), ISOCurrencyCodeEnum.class, getCurrencyOfVariationMarginPosted(), this);
		processor.processBasic(path.newSubPath("variationMarginCollectedByTheReportingEntityPreHaircut"), BigDecimal.class, getVariationMarginCollectedByTheReportingEntityPreHaircut(), this);
		processor.processBasic(path.newSubPath("currencyOfVariationMarginCollected"), ISOCurrencyCodeEnum.class, getCurrencyOfVariationMarginCollected(), this);
		processor.processBasic(path.newSubPath("actionType"), MarginActionEnum.class, getActionType(), this);
		processor.processBasic(path.newSubPath("reportingTimestamp"), ZonedDateTime.class, getReportingTimestamp(), this);
		processor.processBasic(path.newSubPath("reportSubmittingEntityID"), String.class, getReportSubmittingEntityID(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface ASICMarginReportBuilder extends ASICMarginReport, RosettaModelObjectBuilder {
		ASICMarginReport.ASICMarginReportBuilder setUniqueTransactionIdentifier(String uniqueTransactionIdentifier);
		ASICMarginReport.ASICMarginReportBuilder setReportingEntity(String reportingEntity);
		ASICMarginReport.ASICMarginReportBuilder setCounterparty1(String counterparty1);
		ASICMarginReport.ASICMarginReportBuilder setSmallScaleBuySideEntityIndicator(String smallScaleBuySideEntityIndicator);
		ASICMarginReport.ASICMarginReportBuilder setCounterparty2(String counterparty2);
		ASICMarginReport.ASICMarginReportBuilder setCollateralTimestamp(ZonedDateTime collateralTimestamp);
		ASICMarginReport.ASICMarginReportBuilder setCollateralisationCategory(CollateralisationType3Code__1 collateralisationCategory);
		ASICMarginReport.ASICMarginReportBuilder setCollateralPortfolioIndicator(Boolean collateralPortfolioIndicator);
		ASICMarginReport.ASICMarginReportBuilder setPortfolioContainingNonReportedComponentIndicator(String portfolioContainingNonReportedComponentIndicator);
		ASICMarginReport.ASICMarginReportBuilder setInitialMarginCollateralPortfolioCode(String initialMarginCollateralPortfolioCode);
		ASICMarginReport.ASICMarginReportBuilder setCollateralPortfolioCodeVM(String collateralPortfolioCodeVM);
		ASICMarginReport.ASICMarginReportBuilder setInitialMarginPostedByTheReportingEntityPreHaircut(BigDecimal initialMarginPostedByTheReportingEntityPreHaircut);
		ASICMarginReport.ASICMarginReportBuilder setInitialMarginPostedByTheReportingEntityPostHaircut(BigDecimal initialMarginPostedByTheReportingEntityPostHaircut);
		ASICMarginReport.ASICMarginReportBuilder setCurrencyofInitialMarginPosted(ISOCurrencyCodeEnum currencyofInitialMarginPosted);
		ASICMarginReport.ASICMarginReportBuilder setInitialMarginCollectedByTheReportingEntityPreHaircut(BigDecimal initialMarginCollectedByTheReportingEntityPreHaircut);
		ASICMarginReport.ASICMarginReportBuilder setInitialMarginCollectedByTheReportingEntityPostHaircut(BigDecimal initialMarginCollectedByTheReportingEntityPostHaircut);
		ASICMarginReport.ASICMarginReportBuilder setCurrencyofInitialMarginCollected(ISOCurrencyCodeEnum currencyofInitialMarginCollected);
		ASICMarginReport.ASICMarginReportBuilder setVariationMarginPostedByTheReportingEntityPreHaircut(BigDecimal variationMarginPostedByTheReportingEntityPreHaircut);
		ASICMarginReport.ASICMarginReportBuilder setCurrencyOfVariationMarginPosted(ISOCurrencyCodeEnum currencyOfVariationMarginPosted);
		ASICMarginReport.ASICMarginReportBuilder setVariationMarginCollectedByTheReportingEntityPreHaircut(BigDecimal variationMarginCollectedByTheReportingEntityPreHaircut);
		ASICMarginReport.ASICMarginReportBuilder setCurrencyOfVariationMarginCollected(ISOCurrencyCodeEnum currencyOfVariationMarginCollected);
		ASICMarginReport.ASICMarginReportBuilder setActionType(MarginActionEnum actionType);
		ASICMarginReport.ASICMarginReportBuilder setReportingTimestamp(ZonedDateTime reportingTimestamp);
		ASICMarginReport.ASICMarginReportBuilder setReportSubmittingEntityID(String reportSubmittingEntityID);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("uniqueTransactionIdentifier"), String.class, getUniqueTransactionIdentifier(), this);
			processor.processBasic(path.newSubPath("reportingEntity"), String.class, getReportingEntity(), this);
			processor.processBasic(path.newSubPath("counterparty1"), String.class, getCounterparty1(), this);
			processor.processBasic(path.newSubPath("smallScaleBuySideEntityIndicator"), String.class, getSmallScaleBuySideEntityIndicator(), this);
			processor.processBasic(path.newSubPath("counterparty2"), String.class, getCounterparty2(), this);
			processor.processBasic(path.newSubPath("collateralTimestamp"), ZonedDateTime.class, getCollateralTimestamp(), this);
			processor.processBasic(path.newSubPath("collateralisationCategory"), CollateralisationType3Code__1.class, getCollateralisationCategory(), this);
			processor.processBasic(path.newSubPath("collateralPortfolioIndicator"), Boolean.class, getCollateralPortfolioIndicator(), this);
			processor.processBasic(path.newSubPath("portfolioContainingNonReportedComponentIndicator"), String.class, getPortfolioContainingNonReportedComponentIndicator(), this);
			processor.processBasic(path.newSubPath("initialMarginCollateralPortfolioCode"), String.class, getInitialMarginCollateralPortfolioCode(), this);
			processor.processBasic(path.newSubPath("collateralPortfolioCodeVM"), String.class, getCollateralPortfolioCodeVM(), this);
			processor.processBasic(path.newSubPath("initialMarginPostedByTheReportingEntityPreHaircut"), BigDecimal.class, getInitialMarginPostedByTheReportingEntityPreHaircut(), this);
			processor.processBasic(path.newSubPath("initialMarginPostedByTheReportingEntityPostHaircut"), BigDecimal.class, getInitialMarginPostedByTheReportingEntityPostHaircut(), this);
			processor.processBasic(path.newSubPath("currencyofInitialMarginPosted"), ISOCurrencyCodeEnum.class, getCurrencyofInitialMarginPosted(), this);
			processor.processBasic(path.newSubPath("initialMarginCollectedByTheReportingEntityPreHaircut"), BigDecimal.class, getInitialMarginCollectedByTheReportingEntityPreHaircut(), this);
			processor.processBasic(path.newSubPath("initialMarginCollectedByTheReportingEntityPostHaircut"), BigDecimal.class, getInitialMarginCollectedByTheReportingEntityPostHaircut(), this);
			processor.processBasic(path.newSubPath("currencyofInitialMarginCollected"), ISOCurrencyCodeEnum.class, getCurrencyofInitialMarginCollected(), this);
			processor.processBasic(path.newSubPath("variationMarginPostedByTheReportingEntityPreHaircut"), BigDecimal.class, getVariationMarginPostedByTheReportingEntityPreHaircut(), this);
			processor.processBasic(path.newSubPath("currencyOfVariationMarginPosted"), ISOCurrencyCodeEnum.class, getCurrencyOfVariationMarginPosted(), this);
			processor.processBasic(path.newSubPath("variationMarginCollectedByTheReportingEntityPreHaircut"), BigDecimal.class, getVariationMarginCollectedByTheReportingEntityPreHaircut(), this);
			processor.processBasic(path.newSubPath("currencyOfVariationMarginCollected"), ISOCurrencyCodeEnum.class, getCurrencyOfVariationMarginCollected(), this);
			processor.processBasic(path.newSubPath("actionType"), MarginActionEnum.class, getActionType(), this);
			processor.processBasic(path.newSubPath("reportingTimestamp"), ZonedDateTime.class, getReportingTimestamp(), this);
			processor.processBasic(path.newSubPath("reportSubmittingEntityID"), String.class, getReportSubmittingEntityID(), this);
		}
		

		ASICMarginReport.ASICMarginReportBuilder prune();
	}

	/*********************** Immutable Implementation of ASICMarginReport  ***********************/
	class ASICMarginReportImpl implements ASICMarginReport {
		private final String uniqueTransactionIdentifier;
		private final String reportingEntity;
		private final String counterparty1;
		private final String smallScaleBuySideEntityIndicator;
		private final String counterparty2;
		private final ZonedDateTime collateralTimestamp;
		private final CollateralisationType3Code__1 collateralisationCategory;
		private final Boolean collateralPortfolioIndicator;
		private final String portfolioContainingNonReportedComponentIndicator;
		private final String initialMarginCollateralPortfolioCode;
		private final String collateralPortfolioCodeVM;
		private final BigDecimal initialMarginPostedByTheReportingEntityPreHaircut;
		private final BigDecimal initialMarginPostedByTheReportingEntityPostHaircut;
		private final ISOCurrencyCodeEnum currencyofInitialMarginPosted;
		private final BigDecimal initialMarginCollectedByTheReportingEntityPreHaircut;
		private final BigDecimal initialMarginCollectedByTheReportingEntityPostHaircut;
		private final ISOCurrencyCodeEnum currencyofInitialMarginCollected;
		private final BigDecimal variationMarginPostedByTheReportingEntityPreHaircut;
		private final ISOCurrencyCodeEnum currencyOfVariationMarginPosted;
		private final BigDecimal variationMarginCollectedByTheReportingEntityPreHaircut;
		private final ISOCurrencyCodeEnum currencyOfVariationMarginCollected;
		private final MarginActionEnum actionType;
		private final ZonedDateTime reportingTimestamp;
		private final String reportSubmittingEntityID;
		
		protected ASICMarginReportImpl(ASICMarginReport.ASICMarginReportBuilder builder) {
			this.uniqueTransactionIdentifier = builder.getUniqueTransactionIdentifier();
			this.reportingEntity = builder.getReportingEntity();
			this.counterparty1 = builder.getCounterparty1();
			this.smallScaleBuySideEntityIndicator = builder.getSmallScaleBuySideEntityIndicator();
			this.counterparty2 = builder.getCounterparty2();
			this.collateralTimestamp = builder.getCollateralTimestamp();
			this.collateralisationCategory = builder.getCollateralisationCategory();
			this.collateralPortfolioIndicator = builder.getCollateralPortfolioIndicator();
			this.portfolioContainingNonReportedComponentIndicator = builder.getPortfolioContainingNonReportedComponentIndicator();
			this.initialMarginCollateralPortfolioCode = builder.getInitialMarginCollateralPortfolioCode();
			this.collateralPortfolioCodeVM = builder.getCollateralPortfolioCodeVM();
			this.initialMarginPostedByTheReportingEntityPreHaircut = builder.getInitialMarginPostedByTheReportingEntityPreHaircut();
			this.initialMarginPostedByTheReportingEntityPostHaircut = builder.getInitialMarginPostedByTheReportingEntityPostHaircut();
			this.currencyofInitialMarginPosted = builder.getCurrencyofInitialMarginPosted();
			this.initialMarginCollectedByTheReportingEntityPreHaircut = builder.getInitialMarginCollectedByTheReportingEntityPreHaircut();
			this.initialMarginCollectedByTheReportingEntityPostHaircut = builder.getInitialMarginCollectedByTheReportingEntityPostHaircut();
			this.currencyofInitialMarginCollected = builder.getCurrencyofInitialMarginCollected();
			this.variationMarginPostedByTheReportingEntityPreHaircut = builder.getVariationMarginPostedByTheReportingEntityPreHaircut();
			this.currencyOfVariationMarginPosted = builder.getCurrencyOfVariationMarginPosted();
			this.variationMarginCollectedByTheReportingEntityPreHaircut = builder.getVariationMarginCollectedByTheReportingEntityPreHaircut();
			this.currencyOfVariationMarginCollected = builder.getCurrencyOfVariationMarginCollected();
			this.actionType = builder.getActionType();
			this.reportingTimestamp = builder.getReportingTimestamp();
			this.reportSubmittingEntityID = builder.getReportSubmittingEntityID();
		}
		
		@Override
		@RosettaAttribute("uniqueTransactionIdentifier")
		public String getUniqueTransactionIdentifier() {
			return uniqueTransactionIdentifier;
		}
		
		@Override
		@RosettaAttribute("reportingEntity")
		public String getReportingEntity() {
			return reportingEntity;
		}
		
		@Override
		@RosettaAttribute("counterparty1")
		public String getCounterparty1() {
			return counterparty1;
		}
		
		@Override
		@RosettaAttribute("smallScaleBuySideEntityIndicator")
		public String getSmallScaleBuySideEntityIndicator() {
			return smallScaleBuySideEntityIndicator;
		}
		
		@Override
		@RosettaAttribute("counterparty2")
		public String getCounterparty2() {
			return counterparty2;
		}
		
		@Override
		@RosettaAttribute("collateralTimestamp")
		public ZonedDateTime getCollateralTimestamp() {
			return collateralTimestamp;
		}
		
		@Override
		@RosettaAttribute("collateralisationCategory")
		public CollateralisationType3Code__1 getCollateralisationCategory() {
			return collateralisationCategory;
		}
		
		@Override
		@RosettaAttribute("collateralPortfolioIndicator")
		public Boolean getCollateralPortfolioIndicator() {
			return collateralPortfolioIndicator;
		}
		
		@Override
		@RosettaAttribute("portfolioContainingNonReportedComponentIndicator")
		public String getPortfolioContainingNonReportedComponentIndicator() {
			return portfolioContainingNonReportedComponentIndicator;
		}
		
		@Override
		@RosettaAttribute("initialMarginCollateralPortfolioCode")
		public String getInitialMarginCollateralPortfolioCode() {
			return initialMarginCollateralPortfolioCode;
		}
		
		@Override
		@RosettaAttribute("collateralPortfolioCodeVM")
		public String getCollateralPortfolioCodeVM() {
			return collateralPortfolioCodeVM;
		}
		
		@Override
		@RosettaAttribute("initialMarginPostedByTheReportingEntityPreHaircut")
		public BigDecimal getInitialMarginPostedByTheReportingEntityPreHaircut() {
			return initialMarginPostedByTheReportingEntityPreHaircut;
		}
		
		@Override
		@RosettaAttribute("initialMarginPostedByTheReportingEntityPostHaircut")
		public BigDecimal getInitialMarginPostedByTheReportingEntityPostHaircut() {
			return initialMarginPostedByTheReportingEntityPostHaircut;
		}
		
		@Override
		@RosettaAttribute("currencyofInitialMarginPosted")
		public ISOCurrencyCodeEnum getCurrencyofInitialMarginPosted() {
			return currencyofInitialMarginPosted;
		}
		
		@Override
		@RosettaAttribute("initialMarginCollectedByTheReportingEntityPreHaircut")
		public BigDecimal getInitialMarginCollectedByTheReportingEntityPreHaircut() {
			return initialMarginCollectedByTheReportingEntityPreHaircut;
		}
		
		@Override
		@RosettaAttribute("initialMarginCollectedByTheReportingEntityPostHaircut")
		public BigDecimal getInitialMarginCollectedByTheReportingEntityPostHaircut() {
			return initialMarginCollectedByTheReportingEntityPostHaircut;
		}
		
		@Override
		@RosettaAttribute("currencyofInitialMarginCollected")
		public ISOCurrencyCodeEnum getCurrencyofInitialMarginCollected() {
			return currencyofInitialMarginCollected;
		}
		
		@Override
		@RosettaAttribute("variationMarginPostedByTheReportingEntityPreHaircut")
		public BigDecimal getVariationMarginPostedByTheReportingEntityPreHaircut() {
			return variationMarginPostedByTheReportingEntityPreHaircut;
		}
		
		@Override
		@RosettaAttribute("currencyOfVariationMarginPosted")
		public ISOCurrencyCodeEnum getCurrencyOfVariationMarginPosted() {
			return currencyOfVariationMarginPosted;
		}
		
		@Override
		@RosettaAttribute("variationMarginCollectedByTheReportingEntityPreHaircut")
		public BigDecimal getVariationMarginCollectedByTheReportingEntityPreHaircut() {
			return variationMarginCollectedByTheReportingEntityPreHaircut;
		}
		
		@Override
		@RosettaAttribute("currencyOfVariationMarginCollected")
		public ISOCurrencyCodeEnum getCurrencyOfVariationMarginCollected() {
			return currencyOfVariationMarginCollected;
		}
		
		@Override
		@RosettaAttribute("actionType")
		public MarginActionEnum getActionType() {
			return actionType;
		}
		
		@Override
		@RosettaAttribute("reportingTimestamp")
		public ZonedDateTime getReportingTimestamp() {
			return reportingTimestamp;
		}
		
		@Override
		@RosettaAttribute("reportSubmittingEntityID")
		public String getReportSubmittingEntityID() {
			return reportSubmittingEntityID;
		}
		
		@Override
		public ASICMarginReport build() {
			return this;
		}
		
		@Override
		public ASICMarginReport.ASICMarginReportBuilder toBuilder() {
			ASICMarginReport.ASICMarginReportBuilder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(ASICMarginReport.ASICMarginReportBuilder builder) {
			ofNullable(getUniqueTransactionIdentifier()).ifPresent(builder::setUniqueTransactionIdentifier);
			ofNullable(getReportingEntity()).ifPresent(builder::setReportingEntity);
			ofNullable(getCounterparty1()).ifPresent(builder::setCounterparty1);
			ofNullable(getSmallScaleBuySideEntityIndicator()).ifPresent(builder::setSmallScaleBuySideEntityIndicator);
			ofNullable(getCounterparty2()).ifPresent(builder::setCounterparty2);
			ofNullable(getCollateralTimestamp()).ifPresent(builder::setCollateralTimestamp);
			ofNullable(getCollateralisationCategory()).ifPresent(builder::setCollateralisationCategory);
			ofNullable(getCollateralPortfolioIndicator()).ifPresent(builder::setCollateralPortfolioIndicator);
			ofNullable(getPortfolioContainingNonReportedComponentIndicator()).ifPresent(builder::setPortfolioContainingNonReportedComponentIndicator);
			ofNullable(getInitialMarginCollateralPortfolioCode()).ifPresent(builder::setInitialMarginCollateralPortfolioCode);
			ofNullable(getCollateralPortfolioCodeVM()).ifPresent(builder::setCollateralPortfolioCodeVM);
			ofNullable(getInitialMarginPostedByTheReportingEntityPreHaircut()).ifPresent(builder::setInitialMarginPostedByTheReportingEntityPreHaircut);
			ofNullable(getInitialMarginPostedByTheReportingEntityPostHaircut()).ifPresent(builder::setInitialMarginPostedByTheReportingEntityPostHaircut);
			ofNullable(getCurrencyofInitialMarginPosted()).ifPresent(builder::setCurrencyofInitialMarginPosted);
			ofNullable(getInitialMarginCollectedByTheReportingEntityPreHaircut()).ifPresent(builder::setInitialMarginCollectedByTheReportingEntityPreHaircut);
			ofNullable(getInitialMarginCollectedByTheReportingEntityPostHaircut()).ifPresent(builder::setInitialMarginCollectedByTheReportingEntityPostHaircut);
			ofNullable(getCurrencyofInitialMarginCollected()).ifPresent(builder::setCurrencyofInitialMarginCollected);
			ofNullable(getVariationMarginPostedByTheReportingEntityPreHaircut()).ifPresent(builder::setVariationMarginPostedByTheReportingEntityPreHaircut);
			ofNullable(getCurrencyOfVariationMarginPosted()).ifPresent(builder::setCurrencyOfVariationMarginPosted);
			ofNullable(getVariationMarginCollectedByTheReportingEntityPreHaircut()).ifPresent(builder::setVariationMarginCollectedByTheReportingEntityPreHaircut);
			ofNullable(getCurrencyOfVariationMarginCollected()).ifPresent(builder::setCurrencyOfVariationMarginCollected);
			ofNullable(getActionType()).ifPresent(builder::setActionType);
			ofNullable(getReportingTimestamp()).ifPresent(builder::setReportingTimestamp);
			ofNullable(getReportSubmittingEntityID()).ifPresent(builder::setReportSubmittingEntityID);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			ASICMarginReport _that = getType().cast(o);
		
			if (!Objects.equals(uniqueTransactionIdentifier, _that.getUniqueTransactionIdentifier())) return false;
			if (!Objects.equals(reportingEntity, _that.getReportingEntity())) return false;
			if (!Objects.equals(counterparty1, _that.getCounterparty1())) return false;
			if (!Objects.equals(smallScaleBuySideEntityIndicator, _that.getSmallScaleBuySideEntityIndicator())) return false;
			if (!Objects.equals(counterparty2, _that.getCounterparty2())) return false;
			if (!Objects.equals(collateralTimestamp, _that.getCollateralTimestamp())) return false;
			if (!Objects.equals(collateralisationCategory, _that.getCollateralisationCategory())) return false;
			if (!Objects.equals(collateralPortfolioIndicator, _that.getCollateralPortfolioIndicator())) return false;
			if (!Objects.equals(portfolioContainingNonReportedComponentIndicator, _that.getPortfolioContainingNonReportedComponentIndicator())) return false;
			if (!Objects.equals(initialMarginCollateralPortfolioCode, _that.getInitialMarginCollateralPortfolioCode())) return false;
			if (!Objects.equals(collateralPortfolioCodeVM, _that.getCollateralPortfolioCodeVM())) return false;
			if (!Objects.equals(initialMarginPostedByTheReportingEntityPreHaircut, _that.getInitialMarginPostedByTheReportingEntityPreHaircut())) return false;
			if (!Objects.equals(initialMarginPostedByTheReportingEntityPostHaircut, _that.getInitialMarginPostedByTheReportingEntityPostHaircut())) return false;
			if (!Objects.equals(currencyofInitialMarginPosted, _that.getCurrencyofInitialMarginPosted())) return false;
			if (!Objects.equals(initialMarginCollectedByTheReportingEntityPreHaircut, _that.getInitialMarginCollectedByTheReportingEntityPreHaircut())) return false;
			if (!Objects.equals(initialMarginCollectedByTheReportingEntityPostHaircut, _that.getInitialMarginCollectedByTheReportingEntityPostHaircut())) return false;
			if (!Objects.equals(currencyofInitialMarginCollected, _that.getCurrencyofInitialMarginCollected())) return false;
			if (!Objects.equals(variationMarginPostedByTheReportingEntityPreHaircut, _that.getVariationMarginPostedByTheReportingEntityPreHaircut())) return false;
			if (!Objects.equals(currencyOfVariationMarginPosted, _that.getCurrencyOfVariationMarginPosted())) return false;
			if (!Objects.equals(variationMarginCollectedByTheReportingEntityPreHaircut, _that.getVariationMarginCollectedByTheReportingEntityPreHaircut())) return false;
			if (!Objects.equals(currencyOfVariationMarginCollected, _that.getCurrencyOfVariationMarginCollected())) return false;
			if (!Objects.equals(actionType, _that.getActionType())) return false;
			if (!Objects.equals(reportingTimestamp, _that.getReportingTimestamp())) return false;
			if (!Objects.equals(reportSubmittingEntityID, _that.getReportSubmittingEntityID())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (uniqueTransactionIdentifier != null ? uniqueTransactionIdentifier.hashCode() : 0);
			_result = 31 * _result + (reportingEntity != null ? reportingEntity.hashCode() : 0);
			_result = 31 * _result + (counterparty1 != null ? counterparty1.hashCode() : 0);
			_result = 31 * _result + (smallScaleBuySideEntityIndicator != null ? smallScaleBuySideEntityIndicator.hashCode() : 0);
			_result = 31 * _result + (counterparty2 != null ? counterparty2.hashCode() : 0);
			_result = 31 * _result + (collateralTimestamp != null ? collateralTimestamp.hashCode() : 0);
			_result = 31 * _result + (collateralisationCategory != null ? collateralisationCategory.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (collateralPortfolioIndicator != null ? collateralPortfolioIndicator.hashCode() : 0);
			_result = 31 * _result + (portfolioContainingNonReportedComponentIndicator != null ? portfolioContainingNonReportedComponentIndicator.hashCode() : 0);
			_result = 31 * _result + (initialMarginCollateralPortfolioCode != null ? initialMarginCollateralPortfolioCode.hashCode() : 0);
			_result = 31 * _result + (collateralPortfolioCodeVM != null ? collateralPortfolioCodeVM.hashCode() : 0);
			_result = 31 * _result + (initialMarginPostedByTheReportingEntityPreHaircut != null ? initialMarginPostedByTheReportingEntityPreHaircut.hashCode() : 0);
			_result = 31 * _result + (initialMarginPostedByTheReportingEntityPostHaircut != null ? initialMarginPostedByTheReportingEntityPostHaircut.hashCode() : 0);
			_result = 31 * _result + (currencyofInitialMarginPosted != null ? currencyofInitialMarginPosted.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (initialMarginCollectedByTheReportingEntityPreHaircut != null ? initialMarginCollectedByTheReportingEntityPreHaircut.hashCode() : 0);
			_result = 31 * _result + (initialMarginCollectedByTheReportingEntityPostHaircut != null ? initialMarginCollectedByTheReportingEntityPostHaircut.hashCode() : 0);
			_result = 31 * _result + (currencyofInitialMarginCollected != null ? currencyofInitialMarginCollected.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (variationMarginPostedByTheReportingEntityPreHaircut != null ? variationMarginPostedByTheReportingEntityPreHaircut.hashCode() : 0);
			_result = 31 * _result + (currencyOfVariationMarginPosted != null ? currencyOfVariationMarginPosted.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (variationMarginCollectedByTheReportingEntityPreHaircut != null ? variationMarginCollectedByTheReportingEntityPreHaircut.hashCode() : 0);
			_result = 31 * _result + (currencyOfVariationMarginCollected != null ? currencyOfVariationMarginCollected.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (actionType != null ? actionType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (reportingTimestamp != null ? reportingTimestamp.hashCode() : 0);
			_result = 31 * _result + (reportSubmittingEntityID != null ? reportSubmittingEntityID.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ASICMarginReport {" +
				"uniqueTransactionIdentifier=" + this.uniqueTransactionIdentifier + ", " +
				"reportingEntity=" + this.reportingEntity + ", " +
				"counterparty1=" + this.counterparty1 + ", " +
				"smallScaleBuySideEntityIndicator=" + this.smallScaleBuySideEntityIndicator + ", " +
				"counterparty2=" + this.counterparty2 + ", " +
				"collateralTimestamp=" + this.collateralTimestamp + ", " +
				"collateralisationCategory=" + this.collateralisationCategory + ", " +
				"collateralPortfolioIndicator=" + this.collateralPortfolioIndicator + ", " +
				"portfolioContainingNonReportedComponentIndicator=" + this.portfolioContainingNonReportedComponentIndicator + ", " +
				"initialMarginCollateralPortfolioCode=" + this.initialMarginCollateralPortfolioCode + ", " +
				"collateralPortfolioCodeVM=" + this.collateralPortfolioCodeVM + ", " +
				"initialMarginPostedByTheReportingEntityPreHaircut=" + this.initialMarginPostedByTheReportingEntityPreHaircut + ", " +
				"initialMarginPostedByTheReportingEntityPostHaircut=" + this.initialMarginPostedByTheReportingEntityPostHaircut + ", " +
				"currencyofInitialMarginPosted=" + this.currencyofInitialMarginPosted + ", " +
				"initialMarginCollectedByTheReportingEntityPreHaircut=" + this.initialMarginCollectedByTheReportingEntityPreHaircut + ", " +
				"initialMarginCollectedByTheReportingEntityPostHaircut=" + this.initialMarginCollectedByTheReportingEntityPostHaircut + ", " +
				"currencyofInitialMarginCollected=" + this.currencyofInitialMarginCollected + ", " +
				"variationMarginPostedByTheReportingEntityPreHaircut=" + this.variationMarginPostedByTheReportingEntityPreHaircut + ", " +
				"currencyOfVariationMarginPosted=" + this.currencyOfVariationMarginPosted + ", " +
				"variationMarginCollectedByTheReportingEntityPreHaircut=" + this.variationMarginCollectedByTheReportingEntityPreHaircut + ", " +
				"currencyOfVariationMarginCollected=" + this.currencyOfVariationMarginCollected + ", " +
				"actionType=" + this.actionType + ", " +
				"reportingTimestamp=" + this.reportingTimestamp + ", " +
				"reportSubmittingEntityID=" + this.reportSubmittingEntityID +
			'}';
		}
	}

	/*********************** Builder Implementation of ASICMarginReport  ***********************/
	class ASICMarginReportBuilderImpl implements ASICMarginReport.ASICMarginReportBuilder {
	
		protected String uniqueTransactionIdentifier;
		protected String reportingEntity;
		protected String counterparty1;
		protected String smallScaleBuySideEntityIndicator;
		protected String counterparty2;
		protected ZonedDateTime collateralTimestamp;
		protected CollateralisationType3Code__1 collateralisationCategory;
		protected Boolean collateralPortfolioIndicator;
		protected String portfolioContainingNonReportedComponentIndicator;
		protected String initialMarginCollateralPortfolioCode;
		protected String collateralPortfolioCodeVM;
		protected BigDecimal initialMarginPostedByTheReportingEntityPreHaircut;
		protected BigDecimal initialMarginPostedByTheReportingEntityPostHaircut;
		protected ISOCurrencyCodeEnum currencyofInitialMarginPosted;
		protected BigDecimal initialMarginCollectedByTheReportingEntityPreHaircut;
		protected BigDecimal initialMarginCollectedByTheReportingEntityPostHaircut;
		protected ISOCurrencyCodeEnum currencyofInitialMarginCollected;
		protected BigDecimal variationMarginPostedByTheReportingEntityPreHaircut;
		protected ISOCurrencyCodeEnum currencyOfVariationMarginPosted;
		protected BigDecimal variationMarginCollectedByTheReportingEntityPreHaircut;
		protected ISOCurrencyCodeEnum currencyOfVariationMarginCollected;
		protected MarginActionEnum actionType;
		protected ZonedDateTime reportingTimestamp;
		protected String reportSubmittingEntityID;
	
		public ASICMarginReportBuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("uniqueTransactionIdentifier")
		public String getUniqueTransactionIdentifier() {
			return uniqueTransactionIdentifier;
		}
		
		@Override
		@RosettaAttribute("reportingEntity")
		public String getReportingEntity() {
			return reportingEntity;
		}
		
		@Override
		@RosettaAttribute("counterparty1")
		public String getCounterparty1() {
			return counterparty1;
		}
		
		@Override
		@RosettaAttribute("smallScaleBuySideEntityIndicator")
		public String getSmallScaleBuySideEntityIndicator() {
			return smallScaleBuySideEntityIndicator;
		}
		
		@Override
		@RosettaAttribute("counterparty2")
		public String getCounterparty2() {
			return counterparty2;
		}
		
		@Override
		@RosettaAttribute("collateralTimestamp")
		public ZonedDateTime getCollateralTimestamp() {
			return collateralTimestamp;
		}
		
		@Override
		@RosettaAttribute("collateralisationCategory")
		public CollateralisationType3Code__1 getCollateralisationCategory() {
			return collateralisationCategory;
		}
		
		@Override
		@RosettaAttribute("collateralPortfolioIndicator")
		public Boolean getCollateralPortfolioIndicator() {
			return collateralPortfolioIndicator;
		}
		
		@Override
		@RosettaAttribute("portfolioContainingNonReportedComponentIndicator")
		public String getPortfolioContainingNonReportedComponentIndicator() {
			return portfolioContainingNonReportedComponentIndicator;
		}
		
		@Override
		@RosettaAttribute("initialMarginCollateralPortfolioCode")
		public String getInitialMarginCollateralPortfolioCode() {
			return initialMarginCollateralPortfolioCode;
		}
		
		@Override
		@RosettaAttribute("collateralPortfolioCodeVM")
		public String getCollateralPortfolioCodeVM() {
			return collateralPortfolioCodeVM;
		}
		
		@Override
		@RosettaAttribute("initialMarginPostedByTheReportingEntityPreHaircut")
		public BigDecimal getInitialMarginPostedByTheReportingEntityPreHaircut() {
			return initialMarginPostedByTheReportingEntityPreHaircut;
		}
		
		@Override
		@RosettaAttribute("initialMarginPostedByTheReportingEntityPostHaircut")
		public BigDecimal getInitialMarginPostedByTheReportingEntityPostHaircut() {
			return initialMarginPostedByTheReportingEntityPostHaircut;
		}
		
		@Override
		@RosettaAttribute("currencyofInitialMarginPosted")
		public ISOCurrencyCodeEnum getCurrencyofInitialMarginPosted() {
			return currencyofInitialMarginPosted;
		}
		
		@Override
		@RosettaAttribute("initialMarginCollectedByTheReportingEntityPreHaircut")
		public BigDecimal getInitialMarginCollectedByTheReportingEntityPreHaircut() {
			return initialMarginCollectedByTheReportingEntityPreHaircut;
		}
		
		@Override
		@RosettaAttribute("initialMarginCollectedByTheReportingEntityPostHaircut")
		public BigDecimal getInitialMarginCollectedByTheReportingEntityPostHaircut() {
			return initialMarginCollectedByTheReportingEntityPostHaircut;
		}
		
		@Override
		@RosettaAttribute("currencyofInitialMarginCollected")
		public ISOCurrencyCodeEnum getCurrencyofInitialMarginCollected() {
			return currencyofInitialMarginCollected;
		}
		
		@Override
		@RosettaAttribute("variationMarginPostedByTheReportingEntityPreHaircut")
		public BigDecimal getVariationMarginPostedByTheReportingEntityPreHaircut() {
			return variationMarginPostedByTheReportingEntityPreHaircut;
		}
		
		@Override
		@RosettaAttribute("currencyOfVariationMarginPosted")
		public ISOCurrencyCodeEnum getCurrencyOfVariationMarginPosted() {
			return currencyOfVariationMarginPosted;
		}
		
		@Override
		@RosettaAttribute("variationMarginCollectedByTheReportingEntityPreHaircut")
		public BigDecimal getVariationMarginCollectedByTheReportingEntityPreHaircut() {
			return variationMarginCollectedByTheReportingEntityPreHaircut;
		}
		
		@Override
		@RosettaAttribute("currencyOfVariationMarginCollected")
		public ISOCurrencyCodeEnum getCurrencyOfVariationMarginCollected() {
			return currencyOfVariationMarginCollected;
		}
		
		@Override
		@RosettaAttribute("actionType")
		public MarginActionEnum getActionType() {
			return actionType;
		}
		
		@Override
		@RosettaAttribute("reportingTimestamp")
		public ZonedDateTime getReportingTimestamp() {
			return reportingTimestamp;
		}
		
		@Override
		@RosettaAttribute("reportSubmittingEntityID")
		public String getReportSubmittingEntityID() {
			return reportSubmittingEntityID;
		}
		
	
		@Override
		@RosettaAttribute("uniqueTransactionIdentifier")
		public ASICMarginReport.ASICMarginReportBuilder setUniqueTransactionIdentifier(String uniqueTransactionIdentifier) {
			this.uniqueTransactionIdentifier = uniqueTransactionIdentifier==null?null:uniqueTransactionIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("reportingEntity")
		public ASICMarginReport.ASICMarginReportBuilder setReportingEntity(String reportingEntity) {
			this.reportingEntity = reportingEntity==null?null:reportingEntity;
			return this;
		}
		@Override
		@RosettaAttribute("counterparty1")
		public ASICMarginReport.ASICMarginReportBuilder setCounterparty1(String counterparty1) {
			this.counterparty1 = counterparty1==null?null:counterparty1;
			return this;
		}
		@Override
		@RosettaAttribute("smallScaleBuySideEntityIndicator")
		public ASICMarginReport.ASICMarginReportBuilder setSmallScaleBuySideEntityIndicator(String smallScaleBuySideEntityIndicator) {
			this.smallScaleBuySideEntityIndicator = smallScaleBuySideEntityIndicator==null?null:smallScaleBuySideEntityIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("counterparty2")
		public ASICMarginReport.ASICMarginReportBuilder setCounterparty2(String counterparty2) {
			this.counterparty2 = counterparty2==null?null:counterparty2;
			return this;
		}
		@Override
		@RosettaAttribute("collateralTimestamp")
		public ASICMarginReport.ASICMarginReportBuilder setCollateralTimestamp(ZonedDateTime collateralTimestamp) {
			this.collateralTimestamp = collateralTimestamp==null?null:collateralTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("collateralisationCategory")
		public ASICMarginReport.ASICMarginReportBuilder setCollateralisationCategory(CollateralisationType3Code__1 collateralisationCategory) {
			this.collateralisationCategory = collateralisationCategory==null?null:collateralisationCategory;
			return this;
		}
		@Override
		@RosettaAttribute("collateralPortfolioIndicator")
		public ASICMarginReport.ASICMarginReportBuilder setCollateralPortfolioIndicator(Boolean collateralPortfolioIndicator) {
			this.collateralPortfolioIndicator = collateralPortfolioIndicator==null?null:collateralPortfolioIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("portfolioContainingNonReportedComponentIndicator")
		public ASICMarginReport.ASICMarginReportBuilder setPortfolioContainingNonReportedComponentIndicator(String portfolioContainingNonReportedComponentIndicator) {
			this.portfolioContainingNonReportedComponentIndicator = portfolioContainingNonReportedComponentIndicator==null?null:portfolioContainingNonReportedComponentIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("initialMarginCollateralPortfolioCode")
		public ASICMarginReport.ASICMarginReportBuilder setInitialMarginCollateralPortfolioCode(String initialMarginCollateralPortfolioCode) {
			this.initialMarginCollateralPortfolioCode = initialMarginCollateralPortfolioCode==null?null:initialMarginCollateralPortfolioCode;
			return this;
		}
		@Override
		@RosettaAttribute("collateralPortfolioCodeVM")
		public ASICMarginReport.ASICMarginReportBuilder setCollateralPortfolioCodeVM(String collateralPortfolioCodeVM) {
			this.collateralPortfolioCodeVM = collateralPortfolioCodeVM==null?null:collateralPortfolioCodeVM;
			return this;
		}
		@Override
		@RosettaAttribute("initialMarginPostedByTheReportingEntityPreHaircut")
		public ASICMarginReport.ASICMarginReportBuilder setInitialMarginPostedByTheReportingEntityPreHaircut(BigDecimal initialMarginPostedByTheReportingEntityPreHaircut) {
			this.initialMarginPostedByTheReportingEntityPreHaircut = initialMarginPostedByTheReportingEntityPreHaircut==null?null:initialMarginPostedByTheReportingEntityPreHaircut;
			return this;
		}
		@Override
		@RosettaAttribute("initialMarginPostedByTheReportingEntityPostHaircut")
		public ASICMarginReport.ASICMarginReportBuilder setInitialMarginPostedByTheReportingEntityPostHaircut(BigDecimal initialMarginPostedByTheReportingEntityPostHaircut) {
			this.initialMarginPostedByTheReportingEntityPostHaircut = initialMarginPostedByTheReportingEntityPostHaircut==null?null:initialMarginPostedByTheReportingEntityPostHaircut;
			return this;
		}
		@Override
		@RosettaAttribute("currencyofInitialMarginPosted")
		public ASICMarginReport.ASICMarginReportBuilder setCurrencyofInitialMarginPosted(ISOCurrencyCodeEnum currencyofInitialMarginPosted) {
			this.currencyofInitialMarginPosted = currencyofInitialMarginPosted==null?null:currencyofInitialMarginPosted;
			return this;
		}
		@Override
		@RosettaAttribute("initialMarginCollectedByTheReportingEntityPreHaircut")
		public ASICMarginReport.ASICMarginReportBuilder setInitialMarginCollectedByTheReportingEntityPreHaircut(BigDecimal initialMarginCollectedByTheReportingEntityPreHaircut) {
			this.initialMarginCollectedByTheReportingEntityPreHaircut = initialMarginCollectedByTheReportingEntityPreHaircut==null?null:initialMarginCollectedByTheReportingEntityPreHaircut;
			return this;
		}
		@Override
		@RosettaAttribute("initialMarginCollectedByTheReportingEntityPostHaircut")
		public ASICMarginReport.ASICMarginReportBuilder setInitialMarginCollectedByTheReportingEntityPostHaircut(BigDecimal initialMarginCollectedByTheReportingEntityPostHaircut) {
			this.initialMarginCollectedByTheReportingEntityPostHaircut = initialMarginCollectedByTheReportingEntityPostHaircut==null?null:initialMarginCollectedByTheReportingEntityPostHaircut;
			return this;
		}
		@Override
		@RosettaAttribute("currencyofInitialMarginCollected")
		public ASICMarginReport.ASICMarginReportBuilder setCurrencyofInitialMarginCollected(ISOCurrencyCodeEnum currencyofInitialMarginCollected) {
			this.currencyofInitialMarginCollected = currencyofInitialMarginCollected==null?null:currencyofInitialMarginCollected;
			return this;
		}
		@Override
		@RosettaAttribute("variationMarginPostedByTheReportingEntityPreHaircut")
		public ASICMarginReport.ASICMarginReportBuilder setVariationMarginPostedByTheReportingEntityPreHaircut(BigDecimal variationMarginPostedByTheReportingEntityPreHaircut) {
			this.variationMarginPostedByTheReportingEntityPreHaircut = variationMarginPostedByTheReportingEntityPreHaircut==null?null:variationMarginPostedByTheReportingEntityPreHaircut;
			return this;
		}
		@Override
		@RosettaAttribute("currencyOfVariationMarginPosted")
		public ASICMarginReport.ASICMarginReportBuilder setCurrencyOfVariationMarginPosted(ISOCurrencyCodeEnum currencyOfVariationMarginPosted) {
			this.currencyOfVariationMarginPosted = currencyOfVariationMarginPosted==null?null:currencyOfVariationMarginPosted;
			return this;
		}
		@Override
		@RosettaAttribute("variationMarginCollectedByTheReportingEntityPreHaircut")
		public ASICMarginReport.ASICMarginReportBuilder setVariationMarginCollectedByTheReportingEntityPreHaircut(BigDecimal variationMarginCollectedByTheReportingEntityPreHaircut) {
			this.variationMarginCollectedByTheReportingEntityPreHaircut = variationMarginCollectedByTheReportingEntityPreHaircut==null?null:variationMarginCollectedByTheReportingEntityPreHaircut;
			return this;
		}
		@Override
		@RosettaAttribute("currencyOfVariationMarginCollected")
		public ASICMarginReport.ASICMarginReportBuilder setCurrencyOfVariationMarginCollected(ISOCurrencyCodeEnum currencyOfVariationMarginCollected) {
			this.currencyOfVariationMarginCollected = currencyOfVariationMarginCollected==null?null:currencyOfVariationMarginCollected;
			return this;
		}
		@Override
		@RosettaAttribute("actionType")
		public ASICMarginReport.ASICMarginReportBuilder setActionType(MarginActionEnum actionType) {
			this.actionType = actionType==null?null:actionType;
			return this;
		}
		@Override
		@RosettaAttribute("reportingTimestamp")
		public ASICMarginReport.ASICMarginReportBuilder setReportingTimestamp(ZonedDateTime reportingTimestamp) {
			this.reportingTimestamp = reportingTimestamp==null?null:reportingTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("reportSubmittingEntityID")
		public ASICMarginReport.ASICMarginReportBuilder setReportSubmittingEntityID(String reportSubmittingEntityID) {
			this.reportSubmittingEntityID = reportSubmittingEntityID==null?null:reportSubmittingEntityID;
			return this;
		}
		
		@Override
		public ASICMarginReport build() {
			return new ASICMarginReport.ASICMarginReportImpl(this);
		}
		
		@Override
		public ASICMarginReport.ASICMarginReportBuilder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public ASICMarginReport.ASICMarginReportBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getUniqueTransactionIdentifier()!=null) return true;
			if (getReportingEntity()!=null) return true;
			if (getCounterparty1()!=null) return true;
			if (getSmallScaleBuySideEntityIndicator()!=null) return true;
			if (getCounterparty2()!=null) return true;
			if (getCollateralTimestamp()!=null) return true;
			if (getCollateralisationCategory()!=null) return true;
			if (getCollateralPortfolioIndicator()!=null) return true;
			if (getPortfolioContainingNonReportedComponentIndicator()!=null) return true;
			if (getInitialMarginCollateralPortfolioCode()!=null) return true;
			if (getCollateralPortfolioCodeVM()!=null) return true;
			if (getInitialMarginPostedByTheReportingEntityPreHaircut()!=null) return true;
			if (getInitialMarginPostedByTheReportingEntityPostHaircut()!=null) return true;
			if (getCurrencyofInitialMarginPosted()!=null) return true;
			if (getInitialMarginCollectedByTheReportingEntityPreHaircut()!=null) return true;
			if (getInitialMarginCollectedByTheReportingEntityPostHaircut()!=null) return true;
			if (getCurrencyofInitialMarginCollected()!=null) return true;
			if (getVariationMarginPostedByTheReportingEntityPreHaircut()!=null) return true;
			if (getCurrencyOfVariationMarginPosted()!=null) return true;
			if (getVariationMarginCollectedByTheReportingEntityPreHaircut()!=null) return true;
			if (getCurrencyOfVariationMarginCollected()!=null) return true;
			if (getActionType()!=null) return true;
			if (getReportingTimestamp()!=null) return true;
			if (getReportSubmittingEntityID()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public ASICMarginReport.ASICMarginReportBuilder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			ASICMarginReport.ASICMarginReportBuilder o = (ASICMarginReport.ASICMarginReportBuilder) other;
			
			
			merger.mergeBasic(getUniqueTransactionIdentifier(), o.getUniqueTransactionIdentifier(), this::setUniqueTransactionIdentifier);
			merger.mergeBasic(getReportingEntity(), o.getReportingEntity(), this::setReportingEntity);
			merger.mergeBasic(getCounterparty1(), o.getCounterparty1(), this::setCounterparty1);
			merger.mergeBasic(getSmallScaleBuySideEntityIndicator(), o.getSmallScaleBuySideEntityIndicator(), this::setSmallScaleBuySideEntityIndicator);
			merger.mergeBasic(getCounterparty2(), o.getCounterparty2(), this::setCounterparty2);
			merger.mergeBasic(getCollateralTimestamp(), o.getCollateralTimestamp(), this::setCollateralTimestamp);
			merger.mergeBasic(getCollateralisationCategory(), o.getCollateralisationCategory(), this::setCollateralisationCategory);
			merger.mergeBasic(getCollateralPortfolioIndicator(), o.getCollateralPortfolioIndicator(), this::setCollateralPortfolioIndicator);
			merger.mergeBasic(getPortfolioContainingNonReportedComponentIndicator(), o.getPortfolioContainingNonReportedComponentIndicator(), this::setPortfolioContainingNonReportedComponentIndicator);
			merger.mergeBasic(getInitialMarginCollateralPortfolioCode(), o.getInitialMarginCollateralPortfolioCode(), this::setInitialMarginCollateralPortfolioCode);
			merger.mergeBasic(getCollateralPortfolioCodeVM(), o.getCollateralPortfolioCodeVM(), this::setCollateralPortfolioCodeVM);
			merger.mergeBasic(getInitialMarginPostedByTheReportingEntityPreHaircut(), o.getInitialMarginPostedByTheReportingEntityPreHaircut(), this::setInitialMarginPostedByTheReportingEntityPreHaircut);
			merger.mergeBasic(getInitialMarginPostedByTheReportingEntityPostHaircut(), o.getInitialMarginPostedByTheReportingEntityPostHaircut(), this::setInitialMarginPostedByTheReportingEntityPostHaircut);
			merger.mergeBasic(getCurrencyofInitialMarginPosted(), o.getCurrencyofInitialMarginPosted(), this::setCurrencyofInitialMarginPosted);
			merger.mergeBasic(getInitialMarginCollectedByTheReportingEntityPreHaircut(), o.getInitialMarginCollectedByTheReportingEntityPreHaircut(), this::setInitialMarginCollectedByTheReportingEntityPreHaircut);
			merger.mergeBasic(getInitialMarginCollectedByTheReportingEntityPostHaircut(), o.getInitialMarginCollectedByTheReportingEntityPostHaircut(), this::setInitialMarginCollectedByTheReportingEntityPostHaircut);
			merger.mergeBasic(getCurrencyofInitialMarginCollected(), o.getCurrencyofInitialMarginCollected(), this::setCurrencyofInitialMarginCollected);
			merger.mergeBasic(getVariationMarginPostedByTheReportingEntityPreHaircut(), o.getVariationMarginPostedByTheReportingEntityPreHaircut(), this::setVariationMarginPostedByTheReportingEntityPreHaircut);
			merger.mergeBasic(getCurrencyOfVariationMarginPosted(), o.getCurrencyOfVariationMarginPosted(), this::setCurrencyOfVariationMarginPosted);
			merger.mergeBasic(getVariationMarginCollectedByTheReportingEntityPreHaircut(), o.getVariationMarginCollectedByTheReportingEntityPreHaircut(), this::setVariationMarginCollectedByTheReportingEntityPreHaircut);
			merger.mergeBasic(getCurrencyOfVariationMarginCollected(), o.getCurrencyOfVariationMarginCollected(), this::setCurrencyOfVariationMarginCollected);
			merger.mergeBasic(getActionType(), o.getActionType(), this::setActionType);
			merger.mergeBasic(getReportingTimestamp(), o.getReportingTimestamp(), this::setReportingTimestamp);
			merger.mergeBasic(getReportSubmittingEntityID(), o.getReportSubmittingEntityID(), this::setReportSubmittingEntityID);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			ASICMarginReport _that = getType().cast(o);
		
			if (!Objects.equals(uniqueTransactionIdentifier, _that.getUniqueTransactionIdentifier())) return false;
			if (!Objects.equals(reportingEntity, _that.getReportingEntity())) return false;
			if (!Objects.equals(counterparty1, _that.getCounterparty1())) return false;
			if (!Objects.equals(smallScaleBuySideEntityIndicator, _that.getSmallScaleBuySideEntityIndicator())) return false;
			if (!Objects.equals(counterparty2, _that.getCounterparty2())) return false;
			if (!Objects.equals(collateralTimestamp, _that.getCollateralTimestamp())) return false;
			if (!Objects.equals(collateralisationCategory, _that.getCollateralisationCategory())) return false;
			if (!Objects.equals(collateralPortfolioIndicator, _that.getCollateralPortfolioIndicator())) return false;
			if (!Objects.equals(portfolioContainingNonReportedComponentIndicator, _that.getPortfolioContainingNonReportedComponentIndicator())) return false;
			if (!Objects.equals(initialMarginCollateralPortfolioCode, _that.getInitialMarginCollateralPortfolioCode())) return false;
			if (!Objects.equals(collateralPortfolioCodeVM, _that.getCollateralPortfolioCodeVM())) return false;
			if (!Objects.equals(initialMarginPostedByTheReportingEntityPreHaircut, _that.getInitialMarginPostedByTheReportingEntityPreHaircut())) return false;
			if (!Objects.equals(initialMarginPostedByTheReportingEntityPostHaircut, _that.getInitialMarginPostedByTheReportingEntityPostHaircut())) return false;
			if (!Objects.equals(currencyofInitialMarginPosted, _that.getCurrencyofInitialMarginPosted())) return false;
			if (!Objects.equals(initialMarginCollectedByTheReportingEntityPreHaircut, _that.getInitialMarginCollectedByTheReportingEntityPreHaircut())) return false;
			if (!Objects.equals(initialMarginCollectedByTheReportingEntityPostHaircut, _that.getInitialMarginCollectedByTheReportingEntityPostHaircut())) return false;
			if (!Objects.equals(currencyofInitialMarginCollected, _that.getCurrencyofInitialMarginCollected())) return false;
			if (!Objects.equals(variationMarginPostedByTheReportingEntityPreHaircut, _that.getVariationMarginPostedByTheReportingEntityPreHaircut())) return false;
			if (!Objects.equals(currencyOfVariationMarginPosted, _that.getCurrencyOfVariationMarginPosted())) return false;
			if (!Objects.equals(variationMarginCollectedByTheReportingEntityPreHaircut, _that.getVariationMarginCollectedByTheReportingEntityPreHaircut())) return false;
			if (!Objects.equals(currencyOfVariationMarginCollected, _that.getCurrencyOfVariationMarginCollected())) return false;
			if (!Objects.equals(actionType, _that.getActionType())) return false;
			if (!Objects.equals(reportingTimestamp, _that.getReportingTimestamp())) return false;
			if (!Objects.equals(reportSubmittingEntityID, _that.getReportSubmittingEntityID())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (uniqueTransactionIdentifier != null ? uniqueTransactionIdentifier.hashCode() : 0);
			_result = 31 * _result + (reportingEntity != null ? reportingEntity.hashCode() : 0);
			_result = 31 * _result + (counterparty1 != null ? counterparty1.hashCode() : 0);
			_result = 31 * _result + (smallScaleBuySideEntityIndicator != null ? smallScaleBuySideEntityIndicator.hashCode() : 0);
			_result = 31 * _result + (counterparty2 != null ? counterparty2.hashCode() : 0);
			_result = 31 * _result + (collateralTimestamp != null ? collateralTimestamp.hashCode() : 0);
			_result = 31 * _result + (collateralisationCategory != null ? collateralisationCategory.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (collateralPortfolioIndicator != null ? collateralPortfolioIndicator.hashCode() : 0);
			_result = 31 * _result + (portfolioContainingNonReportedComponentIndicator != null ? portfolioContainingNonReportedComponentIndicator.hashCode() : 0);
			_result = 31 * _result + (initialMarginCollateralPortfolioCode != null ? initialMarginCollateralPortfolioCode.hashCode() : 0);
			_result = 31 * _result + (collateralPortfolioCodeVM != null ? collateralPortfolioCodeVM.hashCode() : 0);
			_result = 31 * _result + (initialMarginPostedByTheReportingEntityPreHaircut != null ? initialMarginPostedByTheReportingEntityPreHaircut.hashCode() : 0);
			_result = 31 * _result + (initialMarginPostedByTheReportingEntityPostHaircut != null ? initialMarginPostedByTheReportingEntityPostHaircut.hashCode() : 0);
			_result = 31 * _result + (currencyofInitialMarginPosted != null ? currencyofInitialMarginPosted.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (initialMarginCollectedByTheReportingEntityPreHaircut != null ? initialMarginCollectedByTheReportingEntityPreHaircut.hashCode() : 0);
			_result = 31 * _result + (initialMarginCollectedByTheReportingEntityPostHaircut != null ? initialMarginCollectedByTheReportingEntityPostHaircut.hashCode() : 0);
			_result = 31 * _result + (currencyofInitialMarginCollected != null ? currencyofInitialMarginCollected.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (variationMarginPostedByTheReportingEntityPreHaircut != null ? variationMarginPostedByTheReportingEntityPreHaircut.hashCode() : 0);
			_result = 31 * _result + (currencyOfVariationMarginPosted != null ? currencyOfVariationMarginPosted.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (variationMarginCollectedByTheReportingEntityPreHaircut != null ? variationMarginCollectedByTheReportingEntityPreHaircut.hashCode() : 0);
			_result = 31 * _result + (currencyOfVariationMarginCollected != null ? currencyOfVariationMarginCollected.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (actionType != null ? actionType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (reportingTimestamp != null ? reportingTimestamp.hashCode() : 0);
			_result = 31 * _result + (reportSubmittingEntityID != null ? reportSubmittingEntityID.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ASICMarginReportBuilder {" +
				"uniqueTransactionIdentifier=" + this.uniqueTransactionIdentifier + ", " +
				"reportingEntity=" + this.reportingEntity + ", " +
				"counterparty1=" + this.counterparty1 + ", " +
				"smallScaleBuySideEntityIndicator=" + this.smallScaleBuySideEntityIndicator + ", " +
				"counterparty2=" + this.counterparty2 + ", " +
				"collateralTimestamp=" + this.collateralTimestamp + ", " +
				"collateralisationCategory=" + this.collateralisationCategory + ", " +
				"collateralPortfolioIndicator=" + this.collateralPortfolioIndicator + ", " +
				"portfolioContainingNonReportedComponentIndicator=" + this.portfolioContainingNonReportedComponentIndicator + ", " +
				"initialMarginCollateralPortfolioCode=" + this.initialMarginCollateralPortfolioCode + ", " +
				"collateralPortfolioCodeVM=" + this.collateralPortfolioCodeVM + ", " +
				"initialMarginPostedByTheReportingEntityPreHaircut=" + this.initialMarginPostedByTheReportingEntityPreHaircut + ", " +
				"initialMarginPostedByTheReportingEntityPostHaircut=" + this.initialMarginPostedByTheReportingEntityPostHaircut + ", " +
				"currencyofInitialMarginPosted=" + this.currencyofInitialMarginPosted + ", " +
				"initialMarginCollectedByTheReportingEntityPreHaircut=" + this.initialMarginCollectedByTheReportingEntityPreHaircut + ", " +
				"initialMarginCollectedByTheReportingEntityPostHaircut=" + this.initialMarginCollectedByTheReportingEntityPostHaircut + ", " +
				"currencyofInitialMarginCollected=" + this.currencyofInitialMarginCollected + ", " +
				"variationMarginPostedByTheReportingEntityPreHaircut=" + this.variationMarginPostedByTheReportingEntityPreHaircut + ", " +
				"currencyOfVariationMarginPosted=" + this.currencyOfVariationMarginPosted + ", " +
				"variationMarginCollectedByTheReportingEntityPreHaircut=" + this.variationMarginCollectedByTheReportingEntityPreHaircut + ", " +
				"currencyOfVariationMarginCollected=" + this.currencyOfVariationMarginCollected + ", " +
				"actionType=" + this.actionType + ", " +
				"reportingTimestamp=" + this.reportingTimestamp + ", " +
				"reportSubmittingEntityID=" + this.reportSubmittingEntityID +
			'}';
		}
	}
}
