package drr.regulation.asic.rewrite.margin.meta;

import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyFunctionFactory;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorFactory;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import drr.regulation.asic.rewrite.margin.ASICMarginReport;
import drr.regulation.asic.rewrite.margin.validation.ASICMarginReportTypeFormatValidator;
import drr.regulation.asic.rewrite.margin.validation.ASICMarginReportValidator;
import drr.regulation.asic.rewrite.margin.validation.exists.ASICMarginReportOnlyExistsValidator;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Function;


/**
 * @version 4.2.1
 */
@RosettaMeta(model=ASICMarginReport.class)
public class ASICMarginReportMeta implements RosettaMetaData<ASICMarginReport> {

	@Override
	public List<Validator<? super ASICMarginReport>> dataRules(ValidatorFactory factory) {
		return Arrays.asList(
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR300101.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR301001.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR301002.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR301101.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR301102.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR301201.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR301301.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR301401.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR301402.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR301501.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR301601.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR301701.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR301702.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR301801.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR301901.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR302001.class),
			factory.create(drr.regulation.asic.rewrite.margin.validation.datarule.ASICMarginReportDTCCASICBR302101.class)
		);
	}
	
	@Override
	public List<Function<? super ASICMarginReport, QualifyResult>> getQualifyFunctions(QualifyFunctionFactory factory) {
		return Collections.emptyList();
	}

	@Override
	public Validator<? super ASICMarginReport> validator() {
		return new ASICMarginReportValidator();
	}

	@Override
	public Validator<? super ASICMarginReport> typeFormatValidator() {
		return new ASICMarginReportTypeFormatValidator();
	}
	
	@Override
	public ValidatorWithArg<? super ASICMarginReport, Set<String>> onlyExistsValidator() {
		return new ASICMarginReportOnlyExistsValidator();
	}
}
