package drr.regulation.asic.rewrite.margin.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaReport;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.asic.rewrite.margin.ASICMarginReport;
import drr.regulation.asic.rewrite.margin.ASICMarginReport.ASICMarginReportBuilder;
import drr.regulation.common.CollateralReportInstruction;
import java.util.Optional;
import javax.inject.Inject;


@RosettaReport(namespace="drr.regulation.asic.rewrite.margin", body="ASIC", corpusList={"Margin"})
@ImplementedBy(ASICMarginReportFunction.ASICMarginReportFunctionDefault.class)
public abstract class ASICMarginReportFunction implements ReportFunction<CollateralReportInstruction, ASICMarginReport> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ActionTypeRule actionType;
	@Inject protected CollateralPortfolioCodeIMRule collateralPortfolioCodeIM;
	@Inject protected CollateralPortfolioCodeVMRule collateralPortfolioCodeVM;
	@Inject protected CollateralPortfolioIndicatorRule collateralPortfolioIndicator;
	@Inject protected CollateralTimestampRule collateralTimestamp;
	@Inject protected CollateralisationCategoryRule collateralisationCategory;
	@Inject protected Counterparty1Rule counterparty1;
	@Inject protected Counterparty2Rule counterparty2;
	@Inject protected CurrencyOfInitialMarginCollectedRule currencyOfInitialMarginCollected;
	@Inject protected CurrencyOfInitialMarginPostedRule currencyOfInitialMarginPosted;
	@Inject protected CurrencyOfVariationMarginCollectedRule currencyOfVariationMarginCollected;
	@Inject protected CurrencyOfVariationMarginPostedRule currencyOfVariationMarginPosted;
	@Inject protected InitialMarginCollectedByTheReportingEntityPostHaircutRule initialMarginCollectedByTheReportingEntityPostHaircut;
	@Inject protected InitialMarginCollectedByTheReportingEntityPreHaircutRule initialMarginCollectedByTheReportingEntityPreHaircut;
	@Inject protected InitialMarginPostedByTheReportingEntityPostHaircutRule initialMarginPostedByTheReportingEntityPostHaircut;
	@Inject protected InitialMarginPostedByTheReportingEntityPreHaircutRule initialMarginPostedByTheReportingEntityPreHaircut;
	@Inject protected PortfolioContainingNonReportedComponentIndicatorRule portfolioContainingNonReportedComponentIndicator;
	@Inject protected ReportingEntityRule reportingEntity;
	@Inject protected ReportingSubmittingEntityRule reportingSubmittingEntity;
	@Inject protected ReportingTimestampRule reportingTimestamp;
	@Inject protected SmallScaleBuySideEntityIndicatorRule smallScaleBuySideEntityIndicator;
	@Inject protected UniqueTransactionIdentifierRule uniqueTransactionIdentifier;
	@Inject protected VariationMarginCollectedByTheReportingEntityPreHaircutRule variationMarginCollectedByTheReportingEntityPreHaircut;
	@Inject protected VariationMarginPostedByTheReportingEntityPreHaircutRule variationMarginPostedByTheReportingEntityPreHaircut;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public ASICMarginReport evaluate(CollateralReportInstruction input) {
		ASICMarginReport.ASICMarginReportBuilder outputBuilder = doEvaluate(input);
		
		final ASICMarginReport output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.build();
			objectValidator.validate(ASICMarginReport.class, output);
		}
		
		return output;
	}

	protected abstract ASICMarginReport.ASICMarginReportBuilder doEvaluate(CollateralReportInstruction input);

	public static class ASICMarginReportFunctionDefault extends ASICMarginReportFunction {
		@Override
		protected ASICMarginReport.ASICMarginReportBuilder doEvaluate(CollateralReportInstruction input) {
			ASICMarginReport.ASICMarginReportBuilder output = ASICMarginReport.builder();
			return assignOutput(output, input);
		}
		
		protected ASICMarginReport.ASICMarginReportBuilder assignOutput(ASICMarginReport.ASICMarginReportBuilder output, CollateralReportInstruction input) {
			output
				.setUniqueTransactionIdentifier(uniqueTransactionIdentifier.evaluate(input));
			
			output
				.setReportingEntity(reportingEntity.evaluate(input));
			
			output
				.setCounterparty1(counterparty1.evaluate(input));
			
			output
				.setSmallScaleBuySideEntityIndicator(smallScaleBuySideEntityIndicator.evaluate(input));
			
			output
				.setCounterparty2(counterparty2.evaluate(input));
			
			output
				.setCollateralTimestamp(collateralTimestamp.evaluate(input));
			
			output
				.setCollateralisationCategory(collateralisationCategory.evaluate(input));
			
			output
				.setCollateralPortfolioIndicator(collateralPortfolioIndicator.evaluate(input));
			
			output
				.setPortfolioContainingNonReportedComponentIndicator(portfolioContainingNonReportedComponentIndicator.evaluate(input));
			
			output
				.setInitialMarginCollateralPortfolioCode(collateralPortfolioCodeIM.evaluate(input));
			
			output
				.setCollateralPortfolioCodeVM(collateralPortfolioCodeVM.evaluate(input));
			
			output
				.setInitialMarginPostedByTheReportingEntityPreHaircut(initialMarginPostedByTheReportingEntityPreHaircut.evaluate(input));
			
			output
				.setInitialMarginPostedByTheReportingEntityPostHaircut(initialMarginPostedByTheReportingEntityPostHaircut.evaluate(input));
			
			output
				.setCurrencyofInitialMarginPosted(currencyOfInitialMarginPosted.evaluate(input));
			
			output
				.setInitialMarginCollectedByTheReportingEntityPreHaircut(initialMarginCollectedByTheReportingEntityPreHaircut.evaluate(input));
			
			output
				.setInitialMarginCollectedByTheReportingEntityPostHaircut(initialMarginCollectedByTheReportingEntityPostHaircut.evaluate(input));
			
			output
				.setCurrencyofInitialMarginCollected(currencyOfInitialMarginCollected.evaluate(input));
			
			output
				.setVariationMarginPostedByTheReportingEntityPreHaircut(variationMarginPostedByTheReportingEntityPreHaircut.evaluate(input));
			
			output
				.setCurrencyOfVariationMarginPosted(currencyOfVariationMarginPosted.evaluate(input));
			
			output
				.setVariationMarginCollectedByTheReportingEntityPreHaircut(variationMarginCollectedByTheReportingEntityPreHaircut.evaluate(input));
			
			output
				.setCurrencyOfVariationMarginCollected(currencyOfVariationMarginCollected.evaluate(input));
			
			output
				.setActionType(actionType.evaluate(input));
			
			output
				.setReportingTimestamp(reportingTimestamp.evaluate(input));
			
			output
				.setReportSubmittingEntityID(reportingSubmittingEntity.evaluate(input));
			
			return Optional.ofNullable(output)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
