package drr.regulation.asic.rewrite.margin.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.asic.rewrite.margin.ASICMarginReport;
import drr.regulation.common.MarginActionEnum;
import iso20022.auth108.esma.CollateralisationType3Code__1;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class ASICMarginReportTypeTabulator implements Tabulator<ASICMarginReport> {
	private final Field uniqueTransactionIdentifierField;
	private final Field reportingEntityField;
	private final Field counterparty1Field;
	private final Field smallScaleBuySideEntityIndicatorField;
	private final Field counterparty2Field;
	private final Field collateralTimestampField;
	private final Field collateralisationCategoryField;
	private final Field collateralPortfolioIndicatorField;
	private final Field portfolioContainingNonReportedComponentIndicatorField;
	private final Field initialMarginCollateralPortfolioCodeField;
	private final Field collateralPortfolioCodeVMField;
	private final Field initialMarginPostedByTheReportingEntityPreHaircutField;
	private final Field initialMarginPostedByTheReportingEntityPostHaircutField;
	private final Field currencyofInitialMarginPostedField;
	private final Field initialMarginCollectedByTheReportingEntityPreHaircutField;
	private final Field initialMarginCollectedByTheReportingEntityPostHaircutField;
	private final Field currencyofInitialMarginCollectedField;
	private final Field variationMarginPostedByTheReportingEntityPreHaircutField;
	private final Field currencyOfVariationMarginPostedField;
	private final Field variationMarginCollectedByTheReportingEntityPreHaircutField;
	private final Field currencyOfVariationMarginCollectedField;
	private final Field actionTypeField;
	private final Field reportingTimestampField;
	private final Field reportSubmittingEntityIDField;
	
	public ASICMarginReportTypeTabulator() {
		this.uniqueTransactionIdentifierField = new FieldImpl(
			"uniqueTransactionIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "UniqueTransactionIdentifier")),
			Optional.of("3.1 Unique transaction identifier"),
			Arrays.asList()
		);
		this.reportingEntityField = new FieldImpl(
			"reportingEntity",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "ReportingEntity")),
			Optional.of("3.2 Reporting entity"),
			Arrays.asList()
		);
		this.counterparty1Field = new FieldImpl(
			"counterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "Counterparty1")),
			Optional.of("3.3 Counterparty 1"),
			Arrays.asList()
		);
		this.smallScaleBuySideEntityIndicatorField = new FieldImpl(
			"smallScaleBuySideEntityIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "SmallScaleBuySideEntityIndicator")),
			Optional.of("3.4 Counterparty 1"),
			Arrays.asList()
		);
		this.counterparty2Field = new FieldImpl(
			"counterparty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "Counterparty2")),
			Optional.of("3.5 Counterparty 2"),
			Arrays.asList()
		);
		this.collateralTimestampField = new FieldImpl(
			"collateralTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "CollateralTimestamp")),
			Optional.of("3.6 Collateral timestamp"),
			Arrays.asList()
		);
		this.collateralisationCategoryField = new FieldImpl(
			"collateralisationCategory",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "CollateralisationCategory")),
			Optional.of("3.7 Collateralisation category"),
			Arrays.asList()
		);
		this.collateralPortfolioIndicatorField = new FieldImpl(
			"collateralPortfolioIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "CollateralPortfolioIndicator")),
			Optional.of("3.8 Collateral portfolio indicator"),
			Arrays.asList()
		);
		this.portfolioContainingNonReportedComponentIndicatorField = new FieldImpl(
			"portfolioContainingNonReportedComponentIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "PortfolioContainingNonReportedComponentIndicator")),
			Optional.of("3.9 Portfolio containing non-reported component indicator"),
			Arrays.asList()
		);
		this.initialMarginCollateralPortfolioCodeField = new FieldImpl(
			"initialMarginCollateralPortfolioCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "CollateralPortfolioCodeIM")),
			Optional.of("3.10 Collateral portfolio code (IM)"),
			Arrays.asList()
		);
		this.collateralPortfolioCodeVMField = new FieldImpl(
			"collateralPortfolioCodeVM",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "CollateralPortfolioCodeVM")),
			Optional.of("3.11 Collateral portfolio code (VM)"),
			Arrays.asList()
		);
		this.initialMarginPostedByTheReportingEntityPreHaircutField = new FieldImpl(
			"initialMarginPostedByTheReportingEntityPreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "InitialMarginPostedByTheReportingEntityPreHaircut")),
			Optional.of("3.12 Initial margin posted by the Reporting Entity (pre-haircut)"),
			Arrays.asList()
		);
		this.initialMarginPostedByTheReportingEntityPostHaircutField = new FieldImpl(
			"initialMarginPostedByTheReportingEntityPostHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "InitialMarginPostedByTheReportingEntityPostHaircut")),
			Optional.of("3.13 Initial margin posted by the Reporting Entity (post-haircut)"),
			Arrays.asList()
		);
		this.currencyofInitialMarginPostedField = new FieldImpl(
			"currencyofInitialMarginPosted",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "CurrencyOfInitialMarginPosted")),
			Optional.of("3.14 Currency of initial margin posted"),
			Arrays.asList()
		);
		this.initialMarginCollectedByTheReportingEntityPreHaircutField = new FieldImpl(
			"initialMarginCollectedByTheReportingEntityPreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "InitialMarginCollectedByTheReportingEntityPreHaircut")),
			Optional.of("3.15 Initial margin collected by the Reporting Entity (pre-haircut)"),
			Arrays.asList()
		);
		this.initialMarginCollectedByTheReportingEntityPostHaircutField = new FieldImpl(
			"initialMarginCollectedByTheReportingEntityPostHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "InitialMarginCollectedByTheReportingEntityPostHaircut")),
			Optional.of("3.16 Initial margin collected by the Reporting Entity (post-haircut)"),
			Arrays.asList()
		);
		this.currencyofInitialMarginCollectedField = new FieldImpl(
			"currencyofInitialMarginCollected",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "CurrencyOfInitialMarginCollected")),
			Optional.of("3.17 Currency of initial margin collected"),
			Arrays.asList()
		);
		this.variationMarginPostedByTheReportingEntityPreHaircutField = new FieldImpl(
			"variationMarginPostedByTheReportingEntityPreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "VariationMarginPostedByTheReportingEntityPreHaircut")),
			Optional.of("3.18 Variation margin posted by the Reporting Entity (pre-haircut)"),
			Arrays.asList()
		);
		this.currencyOfVariationMarginPostedField = new FieldImpl(
			"currencyOfVariationMarginPosted",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "CurrencyOfVariationMarginPosted")),
			Optional.of("3.19 Currency of variation margin posted."),
			Arrays.asList()
		);
		this.variationMarginCollectedByTheReportingEntityPreHaircutField = new FieldImpl(
			"variationMarginCollectedByTheReportingEntityPreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "VariationMarginCollectedByTheReportingEntityPreHaircut")),
			Optional.of("3.20 Variation margin collected by the Reporting Entity (pre-haircut)"),
			Arrays.asList()
		);
		this.currencyOfVariationMarginCollectedField = new FieldImpl(
			"currencyOfVariationMarginCollected",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "CurrencyOfVariationMarginCollected")),
			Optional.of("3.21 Currency of variation margin collected."),
			Arrays.asList()
		);
		this.actionTypeField = new FieldImpl(
			"actionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "ActionType")),
			Optional.of("3.22 Action type"),
			Arrays.asList()
		);
		this.reportingTimestampField = new FieldImpl(
			"reportingTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "ReportingTimestamp")),
			Optional.of("3.23 Reporting timestamp"),
			Arrays.asList()
		);
		this.reportSubmittingEntityIDField = new FieldImpl(
			"reportSubmittingEntityID",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "margin"), "ReportingSubmittingEntity")),
			Optional.of("3.24 Report submitting entity"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(uniqueTransactionIdentifierField, reportingEntityField, counterparty1Field, smallScaleBuySideEntityIndicatorField, counterparty2Field, collateralTimestampField, collateralisationCategoryField, collateralPortfolioIndicatorField, portfolioContainingNonReportedComponentIndicatorField, initialMarginCollateralPortfolioCodeField, collateralPortfolioCodeVMField, initialMarginPostedByTheReportingEntityPreHaircutField, initialMarginPostedByTheReportingEntityPostHaircutField, currencyofInitialMarginPostedField, initialMarginCollectedByTheReportingEntityPreHaircutField, initialMarginCollectedByTheReportingEntityPostHaircutField, currencyofInitialMarginCollectedField, variationMarginPostedByTheReportingEntityPreHaircutField, currencyOfVariationMarginPostedField, variationMarginCollectedByTheReportingEntityPreHaircutField, currencyOfVariationMarginCollectedField, actionTypeField, reportingTimestampField, reportSubmittingEntityIDField);
	}
	
	@Override
	public List<FieldValue> tabulate(ASICMarginReport input) {
		Optional<String> uniqueTransactionIdentifier = Optional.ofNullable(input.getUniqueTransactionIdentifier());
		Optional<String> reportingEntity = Optional.ofNullable(input.getReportingEntity());
		Optional<String> counterparty1 = Optional.ofNullable(input.getCounterparty1());
		Optional<String> smallScaleBuySideEntityIndicator = Optional.ofNullable(input.getSmallScaleBuySideEntityIndicator());
		Optional<String> counterparty2 = Optional.ofNullable(input.getCounterparty2());
		Optional<ZonedDateTime> collateralTimestamp = Optional.ofNullable(input.getCollateralTimestamp());
		Optional<CollateralisationType3Code__1> collateralisationCategory = Optional.ofNullable(input.getCollateralisationCategory());
		Optional<Boolean> collateralPortfolioIndicator = Optional.ofNullable(input.getCollateralPortfolioIndicator());
		Optional<String> portfolioContainingNonReportedComponentIndicator = Optional.ofNullable(input.getPortfolioContainingNonReportedComponentIndicator());
		Optional<String> initialMarginCollateralPortfolioCode = Optional.ofNullable(input.getInitialMarginCollateralPortfolioCode());
		Optional<String> collateralPortfolioCodeVM = Optional.ofNullable(input.getCollateralPortfolioCodeVM());
		Optional<BigDecimal> initialMarginPostedByTheReportingEntityPreHaircut = Optional.ofNullable(input.getInitialMarginPostedByTheReportingEntityPreHaircut());
		Optional<BigDecimal> initialMarginPostedByTheReportingEntityPostHaircut = Optional.ofNullable(input.getInitialMarginPostedByTheReportingEntityPostHaircut());
		Optional<ISOCurrencyCodeEnum> currencyofInitialMarginPosted = Optional.ofNullable(input.getCurrencyofInitialMarginPosted());
		Optional<BigDecimal> initialMarginCollectedByTheReportingEntityPreHaircut = Optional.ofNullable(input.getInitialMarginCollectedByTheReportingEntityPreHaircut());
		Optional<BigDecimal> initialMarginCollectedByTheReportingEntityPostHaircut = Optional.ofNullable(input.getInitialMarginCollectedByTheReportingEntityPostHaircut());
		Optional<ISOCurrencyCodeEnum> currencyofInitialMarginCollected = Optional.ofNullable(input.getCurrencyofInitialMarginCollected());
		Optional<BigDecimal> variationMarginPostedByTheReportingEntityPreHaircut = Optional.ofNullable(input.getVariationMarginPostedByTheReportingEntityPreHaircut());
		Optional<ISOCurrencyCodeEnum> currencyOfVariationMarginPosted = Optional.ofNullable(input.getCurrencyOfVariationMarginPosted());
		Optional<BigDecimal> variationMarginCollectedByTheReportingEntityPreHaircut = Optional.ofNullable(input.getVariationMarginCollectedByTheReportingEntityPreHaircut());
		Optional<ISOCurrencyCodeEnum> currencyOfVariationMarginCollected = Optional.ofNullable(input.getCurrencyOfVariationMarginCollected());
		Optional<MarginActionEnum> actionType = Optional.ofNullable(input.getActionType());
		Optional<ZonedDateTime> reportingTimestamp = Optional.ofNullable(input.getReportingTimestamp());
		Optional<String> reportSubmittingEntityID = Optional.ofNullable(input.getReportSubmittingEntityID());
		return Arrays.asList(
			new FieldValueImpl(uniqueTransactionIdentifierField, uniqueTransactionIdentifier),
			new FieldValueImpl(reportingEntityField, reportingEntity),
			new FieldValueImpl(counterparty1Field, counterparty1),
			new FieldValueImpl(smallScaleBuySideEntityIndicatorField, smallScaleBuySideEntityIndicator),
			new FieldValueImpl(counterparty2Field, counterparty2),
			new FieldValueImpl(collateralTimestampField, collateralTimestamp),
			new FieldValueImpl(collateralisationCategoryField, collateralisationCategory),
			new FieldValueImpl(collateralPortfolioIndicatorField, collateralPortfolioIndicator),
			new FieldValueImpl(portfolioContainingNonReportedComponentIndicatorField, portfolioContainingNonReportedComponentIndicator),
			new FieldValueImpl(initialMarginCollateralPortfolioCodeField, initialMarginCollateralPortfolioCode),
			new FieldValueImpl(collateralPortfolioCodeVMField, collateralPortfolioCodeVM),
			new FieldValueImpl(initialMarginPostedByTheReportingEntityPreHaircutField, initialMarginPostedByTheReportingEntityPreHaircut),
			new FieldValueImpl(initialMarginPostedByTheReportingEntityPostHaircutField, initialMarginPostedByTheReportingEntityPostHaircut),
			new FieldValueImpl(currencyofInitialMarginPostedField, currencyofInitialMarginPosted),
			new FieldValueImpl(initialMarginCollectedByTheReportingEntityPreHaircutField, initialMarginCollectedByTheReportingEntityPreHaircut),
			new FieldValueImpl(initialMarginCollectedByTheReportingEntityPostHaircutField, initialMarginCollectedByTheReportingEntityPostHaircut),
			new FieldValueImpl(currencyofInitialMarginCollectedField, currencyofInitialMarginCollected),
			new FieldValueImpl(variationMarginPostedByTheReportingEntityPreHaircutField, variationMarginPostedByTheReportingEntityPreHaircut),
			new FieldValueImpl(currencyOfVariationMarginPostedField, currencyOfVariationMarginPosted),
			new FieldValueImpl(variationMarginCollectedByTheReportingEntityPreHaircutField, variationMarginCollectedByTheReportingEntityPreHaircut),
			new FieldValueImpl(currencyOfVariationMarginCollectedField, currencyOfVariationMarginCollected),
			new FieldValueImpl(actionTypeField, actionType),
			new FieldValueImpl(reportingTimestampField, reportingTimestamp),
			new FieldValueImpl(reportSubmittingEntityIDField, reportSubmittingEntityID)
		);
	}
}
