package drr.regulation.asic.rewrite.margin.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.asic.rewrite.margin.ASICMarginReport;


/**
 * @version 4.2.1
 */
@RosettaDataRule("ASICMarginReportDTCC_ASIC_BR_3011_01")
@ImplementedBy(ASICMarginReportDTCCASICBR301101.Default.class)
public interface ASICMarginReportDTCCASICBR301101 extends Validator<ASICMarginReport> {
	
	String NAME = "ASICMarginReportDTCC_ASIC_BR_3011_01";
	String DEFINITION = "True";
	
	ValidationResult<ASICMarginReport> validate(RosettaPath path, ASICMarginReport aSICMarginReport);
	
	class Default implements ASICMarginReportDTCCASICBR301101 {
	
		@Override
		public ValidationResult<ASICMarginReport> validate(RosettaPath path, ASICMarginReport aSICMarginReport) {
			ComparisonResult result = executeDataRule(aSICMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ASICMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ASICMarginReport aSICMarginReport) {
			try {
				return ComparisonResult.of(MapperS.of(true));
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ASICMarginReportDTCCASICBR301101 {
	
		@Override
		public ValidationResult<ASICMarginReport> validate(RosettaPath path, ASICMarginReport aSICMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICMarginReport", path, DEFINITION);
		}
	}
}
