package drr.regulation.asic.rewrite.margin.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.asic.rewrite.margin.ASICMarginReport;
import drr.regulation.common.MarginActionEnum;
import iso20022.auth108.esma.CollateralisationType3Code__1;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ASICMarginReportDTCC_ASIC_BR_3020_01")
@ImplementedBy(ASICMarginReportDTCCASICBR302001.Default.class)
public interface ASICMarginReportDTCCASICBR302001 extends Validator<ASICMarginReport> {
	
	String NAME = "ASICMarginReportDTCC_ASIC_BR_3020_01";
	String DEFINITION = "if actionType = MarginActionEnum -> MARU then if (smallScaleBuySideEntityIndicator <> \"SSBS\" and (collateralisationCategory = CollateralisationType3Code__1 -> PRC2 or collateralisationCategory = CollateralisationType3Code__1 -> PRCL or collateralisationCategory = CollateralisationType3Code__1 -> OWC2 or collateralisationCategory = CollateralisationType3Code__1 -> OWP1 or collateralisationCategory = CollateralisationType3Code__1 -> OWP2 or collateralisationCategory = CollateralisationType3Code__1 -> FLCL)) then variationMarginCollectedByTheReportingEntityPreHaircut exists and variationMarginCollectedByTheReportingEntityPreHaircut >= 0 else if (smallScaleBuySideEntityIndicator <> \"SSBS\" and (collateralisationCategory <> CollateralisationType3Code__1 -> PRC2 or collateralisationCategory <> CollateralisationType3Code__1 -> PRCL or collateralisationCategory <> CollateralisationType3Code__1 -> OWC2 or collateralisationCategory <> CollateralisationType3Code__1 -> OWP1 or collateralisationCategory <> CollateralisationType3Code__1 -> OWP2 or collateralisationCategory <> CollateralisationType3Code__1 -> FLCL)) then variationMarginCollectedByTheReportingEntityPreHaircut is absent";
	
	ValidationResult<ASICMarginReport> validate(RosettaPath path, ASICMarginReport aSICMarginReport);
	
	class Default implements ASICMarginReportDTCCASICBR302001 {
	
		@Override
		public ValidationResult<ASICMarginReport> validate(RosettaPath path, ASICMarginReport aSICMarginReport) {
			ComparisonResult result = executeDataRule(aSICMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ASICMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ASICMarginReport aSICMarginReport) {
			try {
				if (areEqual(MapperS.of(aSICMarginReport).<MarginActionEnum>map("getActionType", _aSICMarginReport -> _aSICMarginReport.getActionType()), MapperS.of(MarginActionEnum.MARU), CardinalityOperator.All).getOrDefault(false)) {
					if (notEqual(MapperS.of(aSICMarginReport).<String>map("getSmallScaleBuySideEntityIndicator", _aSICMarginReport -> _aSICMarginReport.getSmallScaleBuySideEntityIndicator()), MapperS.of("SSBS"), CardinalityOperator.Any).and(areEqual(MapperS.of(aSICMarginReport).<CollateralisationType3Code__1>map("getCollateralisationCategory", _aSICMarginReport -> _aSICMarginReport.getCollateralisationCategory()), MapperS.of(CollateralisationType3Code__1.PRC2), CardinalityOperator.All).or(areEqual(MapperS.of(aSICMarginReport).<CollateralisationType3Code__1>map("getCollateralisationCategory", _aSICMarginReport -> _aSICMarginReport.getCollateralisationCategory()), MapperS.of(CollateralisationType3Code__1.PRCL), CardinalityOperator.All)).or(areEqual(MapperS.of(aSICMarginReport).<CollateralisationType3Code__1>map("getCollateralisationCategory", _aSICMarginReport -> _aSICMarginReport.getCollateralisationCategory()), MapperS.of(CollateralisationType3Code__1.OWC2), CardinalityOperator.All)).or(areEqual(MapperS.of(aSICMarginReport).<CollateralisationType3Code__1>map("getCollateralisationCategory", _aSICMarginReport -> _aSICMarginReport.getCollateralisationCategory()), MapperS.of(CollateralisationType3Code__1.OWP1), CardinalityOperator.All)).or(areEqual(MapperS.of(aSICMarginReport).<CollateralisationType3Code__1>map("getCollateralisationCategory", _aSICMarginReport -> _aSICMarginReport.getCollateralisationCategory()), MapperS.of(CollateralisationType3Code__1.OWP2), CardinalityOperator.All)).or(areEqual(MapperS.of(aSICMarginReport).<CollateralisationType3Code__1>map("getCollateralisationCategory", _aSICMarginReport -> _aSICMarginReport.getCollateralisationCategory()), MapperS.of(CollateralisationType3Code__1.FLCL), CardinalityOperator.All))).getOrDefault(false)) {
						return exists(MapperS.of(aSICMarginReport).<BigDecimal>map("getVariationMarginCollectedByTheReportingEntityPreHaircut", _aSICMarginReport -> _aSICMarginReport.getVariationMarginCollectedByTheReportingEntityPreHaircut())).and(greaterThanEquals(MapperS.of(aSICMarginReport).<BigDecimal>map("getVariationMarginCollectedByTheReportingEntityPreHaircut", _aSICMarginReport -> _aSICMarginReport.getVariationMarginCollectedByTheReportingEntityPreHaircut()), MapperS.of(BigDecimal.valueOf(0)), CardinalityOperator.All));
					}
					if (notEqual(MapperS.of(aSICMarginReport).<String>map("getSmallScaleBuySideEntityIndicator", _aSICMarginReport -> _aSICMarginReport.getSmallScaleBuySideEntityIndicator()), MapperS.of("SSBS"), CardinalityOperator.Any).and(notEqual(MapperS.of(aSICMarginReport).<CollateralisationType3Code__1>map("getCollateralisationCategory", _aSICMarginReport -> _aSICMarginReport.getCollateralisationCategory()), MapperS.of(CollateralisationType3Code__1.PRC2), CardinalityOperator.Any).or(notEqual(MapperS.of(aSICMarginReport).<CollateralisationType3Code__1>map("getCollateralisationCategory", _aSICMarginReport -> _aSICMarginReport.getCollateralisationCategory()), MapperS.of(CollateralisationType3Code__1.PRCL), CardinalityOperator.Any)).or(notEqual(MapperS.of(aSICMarginReport).<CollateralisationType3Code__1>map("getCollateralisationCategory", _aSICMarginReport -> _aSICMarginReport.getCollateralisationCategory()), MapperS.of(CollateralisationType3Code__1.OWC2), CardinalityOperator.Any)).or(notEqual(MapperS.of(aSICMarginReport).<CollateralisationType3Code__1>map("getCollateralisationCategory", _aSICMarginReport -> _aSICMarginReport.getCollateralisationCategory()), MapperS.of(CollateralisationType3Code__1.OWP1), CardinalityOperator.Any)).or(notEqual(MapperS.of(aSICMarginReport).<CollateralisationType3Code__1>map("getCollateralisationCategory", _aSICMarginReport -> _aSICMarginReport.getCollateralisationCategory()), MapperS.of(CollateralisationType3Code__1.OWP2), CardinalityOperator.Any)).or(notEqual(MapperS.of(aSICMarginReport).<CollateralisationType3Code__1>map("getCollateralisationCategory", _aSICMarginReport -> _aSICMarginReport.getCollateralisationCategory()), MapperS.of(CollateralisationType3Code__1.FLCL), CardinalityOperator.Any))).getOrDefault(false)) {
						return notExists(MapperS.of(aSICMarginReport).<BigDecimal>map("getVariationMarginCollectedByTheReportingEntityPreHaircut", _aSICMarginReport -> _aSICMarginReport.getVariationMarginCollectedByTheReportingEntityPreHaircut()));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ASICMarginReportDTCCASICBR302001 {
	
		@Override
		public ValidationResult<ASICMarginReport> validate(RosettaPath path, ASICMarginReport aSICMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICMarginReport", path, DEFINITION);
		}
	}
}
