package drr.regulation.asic.rewrite.margin.validation.datarule;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.asic.rewrite.margin.ASICMarginReport;
import drr.regulation.common.MarginActionEnum;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ASICMarginReportDTCC_ASIC_BR_3021_01")
@ImplementedBy(ASICMarginReportDTCCASICBR302101.Default.class)
public interface ASICMarginReportDTCCASICBR302101 extends Validator<ASICMarginReport> {
	
	String NAME = "ASICMarginReportDTCC_ASIC_BR_3021_01";
	String DEFINITION = "if actionType = MarginActionEnum -> MARU then if variationMarginCollectedByTheReportingEntityPreHaircut exists then currencyOfVariationMarginCollected exists and [ISOCurrencyCodeEnum->XAG, ISOCurrencyCodeEnum->XAU, ISOCurrencyCodeEnum->XBA, ISOCurrencyCodeEnum->XBB, ISOCurrencyCodeEnum->XBC, ISOCurrencyCodeEnum->XBD, ISOCurrencyCodeEnum->XDR, ISOCurrencyCodeEnum->XPD, ISOCurrencyCodeEnum->XPT, ISOCurrencyCodeEnum->XXX] any <> currencyOfVariationMarginCollected";
	
	ValidationResult<ASICMarginReport> validate(RosettaPath path, ASICMarginReport aSICMarginReport);
	
	class Default implements ASICMarginReportDTCCASICBR302101 {
	
		@Override
		public ValidationResult<ASICMarginReport> validate(RosettaPath path, ASICMarginReport aSICMarginReport) {
			ComparisonResult result = executeDataRule(aSICMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ASICMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ASICMarginReport aSICMarginReport) {
			try {
				if (areEqual(MapperS.of(aSICMarginReport).<MarginActionEnum>map("getActionType", _aSICMarginReport -> _aSICMarginReport.getActionType()), MapperS.of(MarginActionEnum.MARU), CardinalityOperator.All).getOrDefault(false)) {
					if (exists(MapperS.of(aSICMarginReport).<BigDecimal>map("getVariationMarginCollectedByTheReportingEntityPreHaircut", _aSICMarginReport -> _aSICMarginReport.getVariationMarginCollectedByTheReportingEntityPreHaircut())).getOrDefault(false)) {
						return exists(MapperS.of(aSICMarginReport).<ISOCurrencyCodeEnum>map("getCurrencyOfVariationMarginCollected", _aSICMarginReport -> _aSICMarginReport.getCurrencyOfVariationMarginCollected())).and(notEqual(MapperC.<ISOCurrencyCodeEnum>of(MapperS.of(ISOCurrencyCodeEnum.XAG), MapperS.of(ISOCurrencyCodeEnum.XAU), MapperS.of(ISOCurrencyCodeEnum.XBA), MapperS.of(ISOCurrencyCodeEnum.XBB), MapperS.of(ISOCurrencyCodeEnum.XBC), MapperS.of(ISOCurrencyCodeEnum.XBD), MapperS.of(ISOCurrencyCodeEnum.XDR), MapperS.of(ISOCurrencyCodeEnum.XPD), MapperS.of(ISOCurrencyCodeEnum.XPT), MapperS.of(ISOCurrencyCodeEnum.XXX)), MapperS.of(aSICMarginReport).<ISOCurrencyCodeEnum>map("getCurrencyOfVariationMarginCollected", _aSICMarginReport -> _aSICMarginReport.getCurrencyOfVariationMarginCollected()), CardinalityOperator.Any));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ASICMarginReportDTCCASICBR302101 {
	
		@Override
		public ValidationResult<ASICMarginReport> validate(RosettaPath path, ASICMarginReport aSICMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICMarginReport", path, DEFINITION);
		}
	}
}
