package drr.regulation.asic.rewrite.trade;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import drr.enrichment.common.EnrichmentData;
import drr.regulation.asic.rewrite.trade.ASICNonReportable;
import drr.regulation.asic.rewrite.trade.ASICNonReportable.ASICNonReportableBuilder;
import drr.regulation.asic.rewrite.trade.ASICNonReportable.ASICNonReportableBuilderImpl;
import drr.regulation.asic.rewrite.trade.ASICNonReportable.ASICNonReportableImpl;
import drr.regulation.asic.rewrite.trade.meta.ASICNonReportableMeta;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="ASICNonReportable", builder=ASICNonReportable.ASICNonReportableBuilderImpl.class, version="4.2.1")
public interface ASICNonReportable extends RosettaModelObject {

	ASICNonReportableMeta metaData = new ASICNonReportableMeta();

	/*********************** Getter Methods  ***********************/
	/**
	 * Populated by pre-enrichment process
	 */
	EnrichmentData getEnrichment();

	/*********************** Build Methods  ***********************/
	ASICNonReportable build();
	
	ASICNonReportable.ASICNonReportableBuilder toBuilder();
	
	static ASICNonReportable.ASICNonReportableBuilder builder() {
		return new ASICNonReportable.ASICNonReportableBuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends ASICNonReportable> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends ASICNonReportable> getType() {
		return ASICNonReportable.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processRosetta(path.newSubPath("enrichment"), processor, EnrichmentData.class, getEnrichment());
	}
	

	/*********************** Builder Interface  ***********************/
	interface ASICNonReportableBuilder extends ASICNonReportable, RosettaModelObjectBuilder {
		EnrichmentData.EnrichmentDataBuilder getOrCreateEnrichment();
		EnrichmentData.EnrichmentDataBuilder getEnrichment();
		ASICNonReportable.ASICNonReportableBuilder setEnrichment(EnrichmentData enrichment);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processRosetta(path.newSubPath("enrichment"), processor, EnrichmentData.EnrichmentDataBuilder.class, getEnrichment());
		}
		

		ASICNonReportable.ASICNonReportableBuilder prune();
	}

	/*********************** Immutable Implementation of ASICNonReportable  ***********************/
	class ASICNonReportableImpl implements ASICNonReportable {
		private final EnrichmentData enrichment;
		
		protected ASICNonReportableImpl(ASICNonReportable.ASICNonReportableBuilder builder) {
			this.enrichment = ofNullable(builder.getEnrichment()).map(f->f.build()).orElse(null);
		}
		
		@Override
		@RosettaAttribute("enrichment")
		public EnrichmentData getEnrichment() {
			return enrichment;
		}
		
		@Override
		public ASICNonReportable build() {
			return this;
		}
		
		@Override
		public ASICNonReportable.ASICNonReportableBuilder toBuilder() {
			ASICNonReportable.ASICNonReportableBuilder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(ASICNonReportable.ASICNonReportableBuilder builder) {
			ofNullable(getEnrichment()).ifPresent(builder::setEnrichment);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			ASICNonReportable _that = getType().cast(o);
		
			if (!Objects.equals(enrichment, _that.getEnrichment())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (enrichment != null ? enrichment.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ASICNonReportable {" +
				"enrichment=" + this.enrichment +
			'}';
		}
	}

	/*********************** Builder Implementation of ASICNonReportable  ***********************/
	class ASICNonReportableBuilderImpl implements ASICNonReportable.ASICNonReportableBuilder {
	
		protected EnrichmentData.EnrichmentDataBuilder enrichment;
	
		public ASICNonReportableBuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("enrichment")
		public EnrichmentData.EnrichmentDataBuilder getEnrichment() {
			return enrichment;
		}
		
		@Override
		public EnrichmentData.EnrichmentDataBuilder getOrCreateEnrichment() {
			EnrichmentData.EnrichmentDataBuilder result;
			if (enrichment!=null) {
				result = enrichment;
			}
			else {
				result = enrichment = EnrichmentData.builder();
			}
			
			return result;
		}
	
		@Override
		@RosettaAttribute("enrichment")
		public ASICNonReportable.ASICNonReportableBuilder setEnrichment(EnrichmentData enrichment) {
			this.enrichment = enrichment==null?null:enrichment.toBuilder();
			return this;
		}
		
		@Override
		public ASICNonReportable build() {
			return new ASICNonReportable.ASICNonReportableImpl(this);
		}
		
		@Override
		public ASICNonReportable.ASICNonReportableBuilder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public ASICNonReportable.ASICNonReportableBuilder prune() {
			if (enrichment!=null && !enrichment.prune().hasData()) enrichment = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEnrichment()!=null && getEnrichment().hasData()) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public ASICNonReportable.ASICNonReportableBuilder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			ASICNonReportable.ASICNonReportableBuilder o = (ASICNonReportable.ASICNonReportableBuilder) other;
			
			merger.mergeRosetta(getEnrichment(), o.getEnrichment(), this::setEnrichment);
			
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			ASICNonReportable _that = getType().cast(o);
		
			if (!Objects.equals(enrichment, _that.getEnrichment())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (enrichment != null ? enrichment.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ASICNonReportableBuilder {" +
				"enrichment=" + this.enrichment +
			'}';
		}
	}
}
