package drr.regulation.asic.rewrite.trade;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import drr.regulation.asic.rewrite.trade.BasketConstituentsReport;
import drr.regulation.asic.rewrite.trade.BasketConstituentsReport.BasketConstituentsReportBuilder;
import drr.regulation.asic.rewrite.trade.BasketConstituentsReport.BasketConstituentsReportBuilderImpl;
import drr.regulation.asic.rewrite.trade.BasketConstituentsReport.BasketConstituentsReportImpl;
import drr.regulation.asic.rewrite.trade.meta.BasketConstituentsReportMeta;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="BasketConstituentsReport", builder=BasketConstituentsReport.BasketConstituentsReportBuilderImpl.class, version="4.2.1")
public interface BasketConstituentsReport extends RosettaModelObject {

	BasketConstituentsReportMeta metaData = new BasketConstituentsReportMeta();

	/*********************** Getter Methods  ***********************/
	String getIdentifier();

	/*********************** Build Methods  ***********************/
	BasketConstituentsReport build();
	
	BasketConstituentsReport.BasketConstituentsReportBuilder toBuilder();
	
	static BasketConstituentsReport.BasketConstituentsReportBuilder builder() {
		return new BasketConstituentsReport.BasketConstituentsReportBuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends BasketConstituentsReport> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends BasketConstituentsReport> getType() {
		return BasketConstituentsReport.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("identifier"), String.class, getIdentifier(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface BasketConstituentsReportBuilder extends BasketConstituentsReport, RosettaModelObjectBuilder {
		BasketConstituentsReport.BasketConstituentsReportBuilder setIdentifier(String identifier);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("identifier"), String.class, getIdentifier(), this);
		}
		

		BasketConstituentsReport.BasketConstituentsReportBuilder prune();
	}

	/*********************** Immutable Implementation of BasketConstituentsReport  ***********************/
	class BasketConstituentsReportImpl implements BasketConstituentsReport {
		private final String identifier;
		
		protected BasketConstituentsReportImpl(BasketConstituentsReport.BasketConstituentsReportBuilder builder) {
			this.identifier = builder.getIdentifier();
		}
		
		@Override
		@RosettaAttribute("identifier")
		public String getIdentifier() {
			return identifier;
		}
		
		@Override
		public BasketConstituentsReport build() {
			return this;
		}
		
		@Override
		public BasketConstituentsReport.BasketConstituentsReportBuilder toBuilder() {
			BasketConstituentsReport.BasketConstituentsReportBuilder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(BasketConstituentsReport.BasketConstituentsReportBuilder builder) {
			ofNullable(getIdentifier()).ifPresent(builder::setIdentifier);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			BasketConstituentsReport _that = getType().cast(o);
		
			if (!Objects.equals(identifier, _that.getIdentifier())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BasketConstituentsReport {" +
				"identifier=" + this.identifier +
			'}';
		}
	}

	/*********************** Builder Implementation of BasketConstituentsReport  ***********************/
	class BasketConstituentsReportBuilderImpl implements BasketConstituentsReport.BasketConstituentsReportBuilder {
	
		protected String identifier;
	
		public BasketConstituentsReportBuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("identifier")
		public String getIdentifier() {
			return identifier;
		}
		
	
		@Override
		@RosettaAttribute("identifier")
		public BasketConstituentsReport.BasketConstituentsReportBuilder setIdentifier(String identifier) {
			this.identifier = identifier==null?null:identifier;
			return this;
		}
		
		@Override
		public BasketConstituentsReport build() {
			return new BasketConstituentsReport.BasketConstituentsReportImpl(this);
		}
		
		@Override
		public BasketConstituentsReport.BasketConstituentsReportBuilder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public BasketConstituentsReport.BasketConstituentsReportBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIdentifier()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public BasketConstituentsReport.BasketConstituentsReportBuilder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			BasketConstituentsReport.BasketConstituentsReportBuilder o = (BasketConstituentsReport.BasketConstituentsReportBuilder) other;
			
			
			merger.mergeBasic(getIdentifier(), o.getIdentifier(), this::setIdentifier);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			BasketConstituentsReport _that = getType().cast(o);
		
			if (!Objects.equals(identifier, _that.getIdentifier())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (identifier != null ? identifier.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "BasketConstituentsReportBuilder {" +
				"identifier=" + this.identifier +
			'}';
		}
	}
}
