package drr.regulation.asic.rewrite.trade;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import drr.regulation.asic.rewrite.trade.SpreadLeg1;
import drr.regulation.asic.rewrite.trade.SpreadLeg1.SpreadLeg1Builder;
import drr.regulation.asic.rewrite.trade.SpreadLeg1.SpreadLeg1BuilderImpl;
import drr.regulation.asic.rewrite.trade.SpreadLeg1.SpreadLeg1Impl;
import drr.regulation.asic.rewrite.trade.meta.SpreadLeg1Meta;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="SpreadLeg1", builder=SpreadLeg1.SpreadLeg1BuilderImpl.class, version="4.2.1")
public interface SpreadLeg1 extends RosettaModelObject {

	SpreadLeg1Meta metaData = new SpreadLeg1Meta();

	/*********************** Getter Methods  ***********************/
	BigDecimal getSpreadLeg1Monetary();
	BigDecimal getSpreadLeg1Percentage();
	BigDecimal getSpreadLeg1Basis();
	PriceNotationEnum getSpreadLeg1Notation();

	/*********************** Build Methods  ***********************/
	SpreadLeg1 build();
	
	SpreadLeg1.SpreadLeg1Builder toBuilder();
	
	static SpreadLeg1.SpreadLeg1Builder builder() {
		return new SpreadLeg1.SpreadLeg1BuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends SpreadLeg1> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends SpreadLeg1> getType() {
		return SpreadLeg1.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("spreadLeg1Monetary"), BigDecimal.class, getSpreadLeg1Monetary(), this);
		processor.processBasic(path.newSubPath("spreadLeg1Percentage"), BigDecimal.class, getSpreadLeg1Percentage(), this);
		processor.processBasic(path.newSubPath("spreadLeg1Basis"), BigDecimal.class, getSpreadLeg1Basis(), this);
		processor.processBasic(path.newSubPath("spreadLeg1Notation"), PriceNotationEnum.class, getSpreadLeg1Notation(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface SpreadLeg1Builder extends SpreadLeg1, RosettaModelObjectBuilder {
		SpreadLeg1.SpreadLeg1Builder setSpreadLeg1Monetary(BigDecimal spreadLeg1Monetary);
		SpreadLeg1.SpreadLeg1Builder setSpreadLeg1Percentage(BigDecimal spreadLeg1Percentage);
		SpreadLeg1.SpreadLeg1Builder setSpreadLeg1Basis(BigDecimal spreadLeg1Basis);
		SpreadLeg1.SpreadLeg1Builder setSpreadLeg1Notation(PriceNotationEnum spreadLeg1Notation);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("spreadLeg1Monetary"), BigDecimal.class, getSpreadLeg1Monetary(), this);
			processor.processBasic(path.newSubPath("spreadLeg1Percentage"), BigDecimal.class, getSpreadLeg1Percentage(), this);
			processor.processBasic(path.newSubPath("spreadLeg1Basis"), BigDecimal.class, getSpreadLeg1Basis(), this);
			processor.processBasic(path.newSubPath("spreadLeg1Notation"), PriceNotationEnum.class, getSpreadLeg1Notation(), this);
		}
		

		SpreadLeg1.SpreadLeg1Builder prune();
	}

	/*********************** Immutable Implementation of SpreadLeg1  ***********************/
	class SpreadLeg1Impl implements SpreadLeg1 {
		private final BigDecimal spreadLeg1Monetary;
		private final BigDecimal spreadLeg1Percentage;
		private final BigDecimal spreadLeg1Basis;
		private final PriceNotationEnum spreadLeg1Notation;
		
		protected SpreadLeg1Impl(SpreadLeg1.SpreadLeg1Builder builder) {
			this.spreadLeg1Monetary = builder.getSpreadLeg1Monetary();
			this.spreadLeg1Percentage = builder.getSpreadLeg1Percentage();
			this.spreadLeg1Basis = builder.getSpreadLeg1Basis();
			this.spreadLeg1Notation = builder.getSpreadLeg1Notation();
		}
		
		@Override
		@RosettaAttribute("spreadLeg1Monetary")
		public BigDecimal getSpreadLeg1Monetary() {
			return spreadLeg1Monetary;
		}
		
		@Override
		@RosettaAttribute("spreadLeg1Percentage")
		public BigDecimal getSpreadLeg1Percentage() {
			return spreadLeg1Percentage;
		}
		
		@Override
		@RosettaAttribute("spreadLeg1Basis")
		public BigDecimal getSpreadLeg1Basis() {
			return spreadLeg1Basis;
		}
		
		@Override
		@RosettaAttribute("spreadLeg1Notation")
		public PriceNotationEnum getSpreadLeg1Notation() {
			return spreadLeg1Notation;
		}
		
		@Override
		public SpreadLeg1 build() {
			return this;
		}
		
		@Override
		public SpreadLeg1.SpreadLeg1Builder toBuilder() {
			SpreadLeg1.SpreadLeg1Builder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(SpreadLeg1.SpreadLeg1Builder builder) {
			ofNullable(getSpreadLeg1Monetary()).ifPresent(builder::setSpreadLeg1Monetary);
			ofNullable(getSpreadLeg1Percentage()).ifPresent(builder::setSpreadLeg1Percentage);
			ofNullable(getSpreadLeg1Basis()).ifPresent(builder::setSpreadLeg1Basis);
			ofNullable(getSpreadLeg1Notation()).ifPresent(builder::setSpreadLeg1Notation);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			SpreadLeg1 _that = getType().cast(o);
		
			if (!Objects.equals(spreadLeg1Monetary, _that.getSpreadLeg1Monetary())) return false;
			if (!Objects.equals(spreadLeg1Percentage, _that.getSpreadLeg1Percentage())) return false;
			if (!Objects.equals(spreadLeg1Basis, _that.getSpreadLeg1Basis())) return false;
			if (!Objects.equals(spreadLeg1Notation, _that.getSpreadLeg1Notation())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (spreadLeg1Monetary != null ? spreadLeg1Monetary.hashCode() : 0);
			_result = 31 * _result + (spreadLeg1Percentage != null ? spreadLeg1Percentage.hashCode() : 0);
			_result = 31 * _result + (spreadLeg1Basis != null ? spreadLeg1Basis.hashCode() : 0);
			_result = 31 * _result + (spreadLeg1Notation != null ? spreadLeg1Notation.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SpreadLeg1 {" +
				"spreadLeg1Monetary=" + this.spreadLeg1Monetary + ", " +
				"spreadLeg1Percentage=" + this.spreadLeg1Percentage + ", " +
				"spreadLeg1Basis=" + this.spreadLeg1Basis + ", " +
				"spreadLeg1Notation=" + this.spreadLeg1Notation +
			'}';
		}
	}

	/*********************** Builder Implementation of SpreadLeg1  ***********************/
	class SpreadLeg1BuilderImpl implements SpreadLeg1.SpreadLeg1Builder {
	
		protected BigDecimal spreadLeg1Monetary;
		protected BigDecimal spreadLeg1Percentage;
		protected BigDecimal spreadLeg1Basis;
		protected PriceNotationEnum spreadLeg1Notation;
	
		public SpreadLeg1BuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("spreadLeg1Monetary")
		public BigDecimal getSpreadLeg1Monetary() {
			return spreadLeg1Monetary;
		}
		
		@Override
		@RosettaAttribute("spreadLeg1Percentage")
		public BigDecimal getSpreadLeg1Percentage() {
			return spreadLeg1Percentage;
		}
		
		@Override
		@RosettaAttribute("spreadLeg1Basis")
		public BigDecimal getSpreadLeg1Basis() {
			return spreadLeg1Basis;
		}
		
		@Override
		@RosettaAttribute("spreadLeg1Notation")
		public PriceNotationEnum getSpreadLeg1Notation() {
			return spreadLeg1Notation;
		}
		
	
		@Override
		@RosettaAttribute("spreadLeg1Monetary")
		public SpreadLeg1.SpreadLeg1Builder setSpreadLeg1Monetary(BigDecimal spreadLeg1Monetary) {
			this.spreadLeg1Monetary = spreadLeg1Monetary==null?null:spreadLeg1Monetary;
			return this;
		}
		@Override
		@RosettaAttribute("spreadLeg1Percentage")
		public SpreadLeg1.SpreadLeg1Builder setSpreadLeg1Percentage(BigDecimal spreadLeg1Percentage) {
			this.spreadLeg1Percentage = spreadLeg1Percentage==null?null:spreadLeg1Percentage;
			return this;
		}
		@Override
		@RosettaAttribute("spreadLeg1Basis")
		public SpreadLeg1.SpreadLeg1Builder setSpreadLeg1Basis(BigDecimal spreadLeg1Basis) {
			this.spreadLeg1Basis = spreadLeg1Basis==null?null:spreadLeg1Basis;
			return this;
		}
		@Override
		@RosettaAttribute("spreadLeg1Notation")
		public SpreadLeg1.SpreadLeg1Builder setSpreadLeg1Notation(PriceNotationEnum spreadLeg1Notation) {
			this.spreadLeg1Notation = spreadLeg1Notation==null?null:spreadLeg1Notation;
			return this;
		}
		
		@Override
		public SpreadLeg1 build() {
			return new SpreadLeg1.SpreadLeg1Impl(this);
		}
		
		@Override
		public SpreadLeg1.SpreadLeg1Builder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public SpreadLeg1.SpreadLeg1Builder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getSpreadLeg1Monetary()!=null) return true;
			if (getSpreadLeg1Percentage()!=null) return true;
			if (getSpreadLeg1Basis()!=null) return true;
			if (getSpreadLeg1Notation()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public SpreadLeg1.SpreadLeg1Builder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			SpreadLeg1.SpreadLeg1Builder o = (SpreadLeg1.SpreadLeg1Builder) other;
			
			
			merger.mergeBasic(getSpreadLeg1Monetary(), o.getSpreadLeg1Monetary(), this::setSpreadLeg1Monetary);
			merger.mergeBasic(getSpreadLeg1Percentage(), o.getSpreadLeg1Percentage(), this::setSpreadLeg1Percentage);
			merger.mergeBasic(getSpreadLeg1Basis(), o.getSpreadLeg1Basis(), this::setSpreadLeg1Basis);
			merger.mergeBasic(getSpreadLeg1Notation(), o.getSpreadLeg1Notation(), this::setSpreadLeg1Notation);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			SpreadLeg1 _that = getType().cast(o);
		
			if (!Objects.equals(spreadLeg1Monetary, _that.getSpreadLeg1Monetary())) return false;
			if (!Objects.equals(spreadLeg1Percentage, _that.getSpreadLeg1Percentage())) return false;
			if (!Objects.equals(spreadLeg1Basis, _that.getSpreadLeg1Basis())) return false;
			if (!Objects.equals(spreadLeg1Notation, _that.getSpreadLeg1Notation())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (spreadLeg1Monetary != null ? spreadLeg1Monetary.hashCode() : 0);
			_result = 31 * _result + (spreadLeg1Percentage != null ? spreadLeg1Percentage.hashCode() : 0);
			_result = 31 * _result + (spreadLeg1Basis != null ? spreadLeg1Basis.hashCode() : 0);
			_result = 31 * _result + (spreadLeg1Notation != null ? spreadLeg1Notation.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SpreadLeg1Builder {" +
				"spreadLeg1Monetary=" + this.spreadLeg1Monetary + ", " +
				"spreadLeg1Percentage=" + this.spreadLeg1Percentage + ", " +
				"spreadLeg1Basis=" + this.spreadLeg1Basis + ", " +
				"spreadLeg1Notation=" + this.spreadLeg1Notation +
			'}';
		}
	}
}
