package drr.regulation.asic.rewrite.trade.meta;

import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyFunctionFactory;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorFactory;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import drr.regulation.asic.rewrite.trade.ASICTransactionReport;
import drr.regulation.asic.rewrite.trade.validation.ASICTransactionReportTypeFormatValidator;
import drr.regulation.asic.rewrite.trade.validation.ASICTransactionReportValidator;
import drr.regulation.asic.rewrite.trade.validation.exists.ASICTransactionReportOnlyExistsValidator;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Function;


/**
 * @version 4.2.1
 */
@RosettaMeta(model=ASICTransactionReport.class)
public class ASICTransactionReportMeta implements RosettaMetaData<ASICTransactionReport> {

	@Override
	public List<Validator<? super ASICTransactionReport>> dataRules(ValidatorFactory factory) {
		return Arrays.asList(
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR100101.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR100201.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR100601.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR100701.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR100702.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR100901.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR101001.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR101101.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR101201.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR101301.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR101401.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR101501.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR101601.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR101801.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR101901.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR102001.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR102101.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR102301.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR102401.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR102501.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR102601.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR102701.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR102801.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR102901.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR103001.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR103101.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR103102.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR103201.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR103301.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR103401.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR103501.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR103601.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR103701.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR103801.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR103901.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR104001.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR104101.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR104201.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR104202.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR104301.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR104401.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR104501.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR104601.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR104701.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR104702.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR104703.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR104704.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR104801.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR104901.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR105001.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR105002.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR105101.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR105201.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR105301.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR105401.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR105501.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR105502.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR105601.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR105701.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR105801.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR105901.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR106001.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR106002.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR106101.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR106201.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR106301.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR106401.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR106501.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR106601.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR106701.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR106702.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR106801.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR106901.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR106902.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR107001.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR107002.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR107101.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR107102.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR107201.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR107301.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR107401.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR107501.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR107502.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR107503.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR107601.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR107701.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR107801.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR107901.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR108001.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR108002.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR108101.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR108201.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR108401.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR108402.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR108501.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR108701.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR108801.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR108901.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR109001.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR109101.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR109301.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR109401.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR109501.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR109601.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR109701.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR109801.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR109901.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR110001.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR110101.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR110201.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR200801.class),
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.ASICTransactionReportDTCCASICBR201001.class)
		);
	}
	
	@Override
	public List<Function<? super ASICTransactionReport, QualifyResult>> getQualifyFunctions(QualifyFunctionFactory factory) {
		return Collections.emptyList();
	}

	@Override
	public Validator<? super ASICTransactionReport> validator() {
		return new ASICTransactionReportValidator();
	}

	@Override
	public Validator<? super ASICTransactionReport> typeFormatValidator() {
		return new ASICTransactionReportTypeFormatValidator();
	}
	
	@Override
	public ValidatorWithArg<? super ASICTransactionReport, Set<String>> onlyExistsValidator() {
		return new ASICTransactionReportOnlyExistsValidator();
	}
}
