package drr.regulation.asic.rewrite.trade.meta;

import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyFunctionFactory;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorFactory;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import drr.regulation.asic.rewrite.trade.StrikePriceAndNotation;
import drr.regulation.asic.rewrite.trade.validation.StrikePriceAndNotationTypeFormatValidator;
import drr.regulation.asic.rewrite.trade.validation.StrikePriceAndNotationValidator;
import drr.regulation.asic.rewrite.trade.validation.exists.StrikePriceAndNotationOnlyExistsValidator;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Function;


/**
 * @version 4.2.1
 */
@RosettaMeta(model=StrikePriceAndNotation.class)
public class StrikePriceAndNotationMeta implements RosettaMetaData<StrikePriceAndNotation> {

	@Override
	public List<Validator<? super StrikePriceAndNotation>> dataRules(ValidatorFactory factory) {
		return Arrays.asList(
			factory.create(drr.regulation.asic.rewrite.trade.validation.datarule.StrikePriceAndNotationMonetaryOrPercentage.class)
		);
	}
	
	@Override
	public List<Function<? super StrikePriceAndNotation, QualifyResult>> getQualifyFunctions(QualifyFunctionFactory factory) {
		return Collections.emptyList();
	}

	@Override
	public Validator<? super StrikePriceAndNotation> validator() {
		return new StrikePriceAndNotationValidator();
	}

	@Override
	public Validator<? super StrikePriceAndNotation> typeFormatValidator() {
		return new StrikePriceAndNotationTypeFormatValidator();
	}
	
	@Override
	public ValidatorWithArg<? super StrikePriceAndNotation, Set<String>> onlyExistsValidator() {
		return new StrikePriceAndNotationOnlyExistsValidator();
	}
}
