package drr.regulation.asic.rewrite.trade.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaReport;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.enrichment.common.reports.EnrichmentDataRule;
import drr.regulation.asic.rewrite.trade.ASICTransactionReport;
import drr.regulation.asic.rewrite.trade.ASICTransactionReport.ASICTransactionReportBuilder;
import drr.regulation.common.TransactionReportInstruction;
import java.util.Optional;
import javax.inject.Inject;


@RosettaReport(namespace="drr.regulation.asic.rewrite.trade", body="ASIC", corpusList={"Trade"})
@ImplementedBy(ASICTradeReportFunction.ASICTradeReportFunctionDefault.class)
public abstract class ASICTradeReportFunction implements ReportFunction<TransactionReportInstruction, ASICTransactionReport> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ActionTypeRule actionType;
	@Inject protected AssetClassRule assetClass;
	@Inject protected BasketConstituentIdentifierSourceRule basketConstituentIdentifierSource;
	@Inject protected BasketConstituentsRule basketConstituents;
	@Inject protected Beneficiary1Rule beneficiary1;
	@Inject protected Beneficiary1IdentifierTypeIndicatorRule beneficiary1IdentifierTypeIndicator;
	@Inject protected BrokerRule broker;
	@Inject protected CDSIndexAttachmentPointRule cDSIndexAttachmentPoint;
	@Inject protected CDSIndexDetachmentPointRule cDSIndexDetachmentPoint;
	@Inject protected CallAmountRule callAmount;
	@Inject protected CallCurrencyRule callCurrency;
	@Inject protected CentralCounterpartyRule centralCounterparty;
	@Inject protected ClearedRule cleared;
	@Inject protected ClearingMemberRule clearingMember;
	@Inject protected ClearingTimestampRule clearingTimestamp;
	@Inject protected ContractTypeRule contractType;
	@Inject protected Counterparty1Rule counterparty1;
	@Inject protected Counterparty2Rule counterparty2;
	@Inject protected CounterpartyIdentifierTypeIndicatorRule counterpartyIdentifierTypeIndicator;
	@Inject protected CountryOfCounterparty2Rule countryOfCounterparty2;
	@Inject protected CustomBasketCodeRule customBasketCode;
	@Inject protected DayCountConventionLeg1Rule dayCountConventionLeg1;
	@Inject protected DayCountConventionLeg2Rule dayCountConventionLeg2;
	@Inject protected DeltaRule delta;
	@Inject protected Direction1Rule direction1;
	@Inject protected Direction2Leg1Rule direction2Leg1;
	@Inject protected Direction2Leg2Rule direction2Leg2;
	@Inject protected EffectiveDateRule effectiveDate;
	@Inject protected EnrichmentDataRule enrichmentData;
	@Inject protected EventIdentifierRule eventIdentifier;
	@Inject protected EventTimestampRule eventTimestamp;
	@Inject protected EventTypeRule eventType;
	@Inject protected ExchangeRateRule exchangeRate;
	@Inject protected ExchangeRateBasisRule exchangeRateBasis;
	@Inject protected ExecutionAgentOfTheCounterparty1Rule executionAgentOfTheCounterparty1;
	@Inject protected ExecutionTimestampRule executionTimestamp;
	@Inject protected ExpirationDateRule expirationDate;
	@Inject protected FixedRateLeg1Rule fixedRateLeg1;
	@Inject protected FixedRateLeg2Rule fixedRateLeg2;
	@Inject protected FloatingRateReferencePeriodLeg2Rule floatingRateReferencePeriodLeg2;
	@Inject protected FloatingRateReferencePeriodMultiplierLeg2Rule floatingRateReferencePeriodMultiplierLeg2;
	@Inject protected IdentifierOfTheFloatingRateLeg2Rule identifierOfTheFloatingRateLeg2;
	@Inject protected IndexFactorRule indexFactor;
	@Inject protected MaturityDateOfTheUnderlierRule maturityDateOfTheUnderlier;
	@Inject protected NextFloatingReferenceResetDateLeg1Rule nextFloatingReferenceResetDateLeg1;
	@Inject protected NextFloatingReferenceResetDateLeg2Rule nextFloatingReferenceResetDateLeg2;
	@Inject protected NotionalAmountLeg1Rule notionalAmountLeg1;
	@Inject protected NotionalAmountLeg2Rule notionalAmountLeg2;
	@Inject protected NotionalAmountScheduleLeg1Rule notionalAmountScheduleLeg1;
	@Inject protected NotionalAmountScheduleLeg2Rule notionalAmountScheduleLeg2;
	@Inject protected NotionalCurrencyLeg1Rule notionalCurrencyLeg1;
	@Inject protected NotionalCurrencyLeg2Rule notionalCurrencyLeg2;
	@Inject protected NotionalQuantityLeg1Rule notionalQuantityLeg1;
	@Inject protected NotionalQuantityLeg2Rule notionalQuantityLeg2;
	@Inject protected OptionPremiumAmountRule optionPremiumAmount;
	@Inject protected OptionPremiumCurrencyRule optionPremiumCurrency;
	@Inject protected OptionPremiumPaymentDateRule optionPremiumPaymentDate;
	@Inject protected OtherPaymentRule otherPayment;
	@Inject protected PackageIdentifierRule packageIdentifier;
	@Inject protected PackageTransactionPriceCurrencyRule packageTransactionPriceCurrency;
	@Inject protected PackageTransactionPriceMonetaryRule packageTransactionPriceMonetary;
	@Inject protected PackageTransactionPriceNotationRule packageTransactionPriceNotation;
	@Inject protected PackageTransactionPricePercentageRule packageTransactionPricePercentage;
	@Inject protected PackageTransactionSpreadBasisRule packageTransactionSpreadBasis;
	@Inject protected PackageTransactionSpreadCurrencyRule packageTransactionSpreadCurrency;
	@Inject protected PackageTransactionSpreadMonetaryRule packageTransactionSpreadMonetary;
	@Inject protected PackageTransactionSpreadNotationRule packageTransactionSpreadNotation;
	@Inject protected PackageTransactionSpreadPercentageRule packageTransactionSpreadPercentage;
	@Inject protected PaymentFrequencyPeriodLeg1Rule paymentFrequencyPeriodLeg1;
	@Inject protected PaymentFrequencyPeriodLeg2Rule paymentFrequencyPeriodLeg2;
	@Inject protected PaymentFrequencyPeriodMultiplierLeg1Rule paymentFrequencyPeriodMultiplierLeg1;
	@Inject protected PaymentFrequencyPeriodMultiplierLeg2Rule paymentFrequencyPeriodMultiplierLeg2;
	@Inject protected PlatformIdentifierRule platformIdentifier;
	@Inject protected PriceRule price;
	@Inject protected PriceCurrencyRule priceCurrency;
	@Inject protected PriceNotationRule priceNotation;
	@Inject protected PriceUnitOfMeasureRule priceUnitOfMeasure;
	@Inject protected PriorUTIRule priorUTI;
	@Inject protected PutAmountRule putAmount;
	@Inject protected PutCurrencyRule putCurrency;
	@Inject protected QuantityUnitOfMeasureLeg1Rule quantityUnitOfMeasureLeg1;
	@Inject protected QuantityUnitOfMeasureLeg2Rule quantityUnitOfMeasureLeg2;
	@Inject protected ReportSubmittingEntityRule reportSubmittingEntity;
	@Inject protected ReportingEntityRule reportingEntity;
	@Inject protected ReportingTimestampRule reportingTimestamp;
	@Inject protected SettlementCurrencyLeg1Rule settlementCurrencyLeg1;
	@Inject protected SettlementCurrencyLeg2Rule settlementCurrencyLeg2;
	@Inject protected SmallScaleBuySideEntityIndicatorRule smallScaleBuySideEntityIndicator;
	@Inject protected SpreadCurrencyLeg1Rule spreadCurrencyLeg1;
	@Inject protected SpreadCurrencyLeg2Rule spreadCurrencyLeg2;
	@Inject protected SpreadLeg1BasisRule spreadLeg1Basis;
	@Inject protected SpreadLeg1MonetaryRule spreadLeg1Monetary;
	@Inject protected SpreadLeg1NotationRule spreadLeg1Notation;
	@Inject protected SpreadLeg1PercentageRule spreadLeg1Percentage;
	@Inject protected SpreadLeg2BasisRule spreadLeg2Basis;
	@Inject protected SpreadLeg2MonetaryRule spreadLeg2Monetary;
	@Inject protected SpreadLeg2NotationRule spreadLeg2Notation;
	@Inject protected SpreadLeg2PercentageRule spreadLeg2Percentage;
	@Inject protected SpreadNotationLeg1Rule spreadNotationLeg1;
	@Inject protected SpreadNotationLeg2Rule spreadNotationLeg2;
	@Inject protected StrikePriceCurrencyCurrencyPairRule strikePriceCurrencyCurrencyPair;
	@Inject protected StrikePriceMonetaryRule strikePriceMonetary;
	@Inject protected StrikePriceNotationRule strikePriceNotation;
	@Inject protected StrikePricePercentageRule strikePricePercentage;
	@Inject protected TotalNotionalQuantityLeg1Rule totalNotionalQuantityLeg1;
	@Inject protected TotalNotionalQuantityLeg2Rule totalNotionalQuantityLeg2;
	@Inject protected UnderlierIDRule underlierID;
	@Inject protected UnderlierIDSourceRule underlierIDSource;
	@Inject protected UniqueProductIdentifierRule uniqueProductIdentifier;
	@Inject protected UniqueTransactionIdentifierRule uniqueTransactionIdentifier;
	@Inject protected ValuationAmountRule valuationAmount;
	@Inject protected ValuationCurrencyRule valuationCurrency;
	@Inject protected ValuationMethodRule valuationMethod;
	@Inject protected ValuationTimestampRule valuationTimestamp;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public ASICTransactionReport evaluate(TransactionReportInstruction input) {
		ASICTransactionReport.ASICTransactionReportBuilder outputBuilder = doEvaluate(input);
		
		final ASICTransactionReport output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.build();
			objectValidator.validate(ASICTransactionReport.class, output);
		}
		
		return output;
	}

	protected abstract ASICTransactionReport.ASICTransactionReportBuilder doEvaluate(TransactionReportInstruction input);

	public static class ASICTradeReportFunctionDefault extends ASICTradeReportFunction {
		@Override
		protected ASICTransactionReport.ASICTransactionReportBuilder doEvaluate(TransactionReportInstruction input) {
			ASICTransactionReport.ASICTransactionReportBuilder output = ASICTransactionReport.builder();
			return assignOutput(output, input);
		}
		
		protected ASICTransactionReport.ASICTransactionReportBuilder assignOutput(ASICTransactionReport.ASICTransactionReportBuilder output, TransactionReportInstruction input) {
			output
				.setUniqueTransactionIdentifier(uniqueTransactionIdentifier.evaluate(input));
			
			output
				.setUniqueProductIdentifier(uniqueProductIdentifier.evaluate(input));
			
			output
				.setAssetClass(assetClass.evaluate(input));
			
			output
				.setContractType(contractType.evaluate(input));
			
			output
				.setReportingEntity(reportingEntity.evaluate(input));
			
			output
				.setCounterparty1(counterparty1.evaluate(input));
			
			output
				.setCounterparty2(counterparty2.evaluate(input));
			
			output
				.setCounterpartyIdentifierTypeIndicator(counterpartyIdentifierTypeIndicator.evaluate(input));
			
			output
				.setCountryOfTheCounterParty2(countryOfCounterparty2.evaluate(input));
			
			output
				.setBeneficiary1(beneficiary1.evaluate(input));
			
			output
				.setBeneficiary1IdentifierTypeIndicator(beneficiary1IdentifierTypeIndicator.evaluate(input));
			
			output
				.setBroker(broker.evaluate(input));
			
			output
				.setExecutionAgentOfTheCounterparty1(executionAgentOfTheCounterparty1.evaluate(input));
			
			output
				.setDirection(direction1.evaluate(input));
			
			output
				.setDirection2OfLeg1(direction2Leg1.evaluate(input));
			
			output
				.setDirection2OfLeg2(direction2Leg2.evaluate(input));
			
			output
				.setEffectiveDate(effectiveDate.evaluate(input));
			
			output
				.setExpirationDate(expirationDate.evaluate(input));
			
			output
				.setExecutionTimestamp(executionTimestamp.evaluate(input));
			
			output
				.setEventTimestamp(eventTimestamp.evaluate(input));
			
			output
				.setClearingTimestamp(clearingTimestamp.evaluate(input));
			
			output
				.setCleared(cleared.evaluate(input));
			
			output
				.setCentralCounterparty(centralCounterparty.evaluate(input));
			
			output
				.setClearingMember(clearingMember.evaluate(input));
			
			output
				.setPlatformIdentifier(platformIdentifier.evaluate(input));
			
			output
				.setNotionalLeg1(notionalAmountLeg1.evaluate(input));
			
			output
				.setNotionalLeg2(notionalAmountLeg2.evaluate(input));
			
			output
				.setTotalNotionalQuantityLeg1(totalNotionalQuantityLeg1.evaluate(input));
			
			output
				.setTotalNotionalQuantityLeg2(totalNotionalQuantityLeg2.evaluate(input));
			
			output
				.setNotionalQuantityLeg1(notionalQuantityLeg1.evaluate(input));
			
			output
				.setNotionalQuantityLeg2(notionalQuantityLeg2.evaluate(input));
			
			output
				.setCallAmount(callAmount.evaluate(input));
			
			output
				.setPutAmount(putAmount.evaluate(input));
			
			output
				.setNotionalCurrencyLeg1(notionalCurrencyLeg1.evaluate(input));
			
			output
				.setNotionalCurrencyLeg2(notionalCurrencyLeg2.evaluate(input));
			
			output
				.setQuantityUnitOfMeasureLeg1(quantityUnitOfMeasureLeg1.evaluate(input));
			
			output
				.setQuantityUnitOfMeasureLeg2(quantityUnitOfMeasureLeg2.evaluate(input));
			
			output
				.setCallCurrency(callCurrency.evaluate(input));
			
			output
				.setPutCurrency(putCurrency.evaluate(input));
			
			output
				.setNotionalAmountScheduleLeg1(notionalAmountScheduleLeg1.evaluate(input));
			
			output
				.setNotionalAmountScheduleLeg2(notionalAmountScheduleLeg2.evaluate(input));
			
			output
				.setPriceNotation(priceNotation.evaluate(input));
			
			output
				.setPrice(price.evaluate(input));
			
			output
				.setPriceCurrency(priceCurrency.evaluate(input));
			
			output
				.setPriceUnitOfMeasure(priceUnitOfMeasure.evaluate(input));
			
			output
				.setFixedRateLeg1(fixedRateLeg1.evaluate(input));
			
			output
				.setFixedRateLeg2(fixedRateLeg2.evaluate(input));
			
			output
				.setSpreadNotationLeg1(spreadNotationLeg1.evaluate(input));
			
			output
				.getOrCreateSpreadLeg1()
				.setSpreadLeg1Monetary(spreadLeg1Monetary.evaluate(input));
			
			output
				.getOrCreateSpreadLeg1()
				.setSpreadLeg1Percentage(spreadLeg1Percentage.evaluate(input));
			
			output
				.getOrCreateSpreadLeg1()
				.setSpreadLeg1Basis(spreadLeg1Basis.evaluate(input));
			
			output
				.getOrCreateSpreadLeg1()
				.setSpreadLeg1Notation(spreadLeg1Notation.evaluate(input));
			
			output
				.setSpreadNotationLeg2(spreadNotationLeg2.evaluate(input));
			
			output
				.getOrCreateSpreadLeg2()
				.setSpreadLeg2Monetary(spreadLeg2Monetary.evaluate(input));
			
			output
				.getOrCreateSpreadLeg2()
				.setSpreadLeg2Percentage(spreadLeg2Percentage.evaluate(input));
			
			output
				.getOrCreateSpreadLeg2()
				.setSpreadLeg2Basis(spreadLeg2Basis.evaluate(input));
			
			output
				.getOrCreateSpreadLeg2()
				.setSpreadLeg2Notation(spreadLeg2Notation.evaluate(input));
			
			output
				.setSpreadCurrencyLeg1(spreadCurrencyLeg1.evaluate(input));
			
			output
				.setSpreadCurrencyLeg2(spreadCurrencyLeg2.evaluate(input));
			
			output
				.setStrikePriceNotation(strikePriceNotation.evaluate(input));
			
			output
				.getOrCreateStrikePrice()
				.setStrikePriceMonetary(strikePriceMonetary.evaluate(input));
			
			output
				.getOrCreateStrikePrice()
				.setStrikePricePercentage(strikePricePercentage.evaluate(input));
			
			output
				.setStrikePriceCurrency(strikePriceCurrencyCurrencyPair.evaluate(input));
			
			output
				.setOptionPremiumAmount(optionPremiumAmount.evaluate(input));
			
			output
				.setOptionPremiumCurrency(optionPremiumCurrency.evaluate(input));
			
			output
				.setOptionPremiumPaymentDate(optionPremiumPaymentDate.evaluate(input));
			
			output
				.setExchangeRate(exchangeRate.evaluate(input));
			
			output
				.setExchangeRateBasis(exchangeRateBasis.evaluate(input));
			
			output
				.setDayCountConventionLeg1(dayCountConventionLeg1.evaluate(input));
			
			output
				.setDayCountConventionLeg2(dayCountConventionLeg2.evaluate(input));
			
			output
				.setPaymentFrequencyPeriodLeg1(paymentFrequencyPeriodLeg1.evaluate(input));
			
			output
				.setPaymentFrequencyPeriodLeg2(paymentFrequencyPeriodLeg2.evaluate(input));
			
			output
				.setPaymentFrequencyPeriodMultiplierLeg1(paymentFrequencyPeriodMultiplierLeg1.evaluate(input));
			
			output
				.setPaymentFrequencyPeriodMultiplierLeg2(paymentFrequencyPeriodMultiplierLeg2.evaluate(input));
			
			output
				.setSettlementCurrencyLeg1(settlementCurrencyLeg1.evaluate(input));
			
			output
				.setSettlementCurrencyLeg2(settlementCurrencyLeg2.evaluate(input));
			
			output
				.setOtherPayment(otherPayment.evaluate(input));
			
			output
				.setCustomBasketCode(customBasketCode.evaluate(input));
			
			output
				.setBasketConstituents(basketConstituents.evaluate(input));
			
			output
				.setBasketConstituentIdentifierSource(basketConstituentIdentifierSource.evaluate(input));
			
			output
				.setUnderlyingIdentification(underlierID.evaluate(input));
			
			output
				.setUnderlierIDSource(underlierIDSource.evaluate(input));
			
			output
				.setMaturityDateOfTheUnderlying(maturityDateOfTheUnderlier.evaluate(input));
			
			output
				.setIdentifierOfTheFloatingRateLeg2(identifierOfTheFloatingRateLeg2.evaluate(input));
			
			output
				.setFloatingRateReferencePeriodOfLeg2TimePeriod(floatingRateReferencePeriodLeg2.evaluate(input));
			
			output
				.setFloatingRateReferencePeriodOfLeg2Multiplier(floatingRateReferencePeriodMultiplierLeg2.evaluate(input));
			
			output
				.setCdSIndexAttachmentPoint(cDSIndexAttachmentPoint.evaluate(input));
			
			output
				.setCdSIndexDetachmentPoint(cDSIndexDetachmentPoint.evaluate(input));
			
			output
				.setIndexFactor(indexFactor.evaluate(input));
			
			output
				.setPackageIdentifier(packageIdentifier.evaluate(input));
			
			output
				.getOrCreatePackageTransactionPrice()
				.setPackageTransactionPriceMonetary(packageTransactionPriceMonetary.evaluate(input));
			
			output
				.getOrCreatePackageTransactionPrice()
				.setPackageTransactionPricePercentage(packageTransactionPricePercentage.evaluate(input));
			
			output
				.getOrCreatePackageTransactionPrice()
				.setPackageTransactionPriceNotation(packageTransactionPriceNotation.evaluate(input));
			
			output
				.getOrCreatePackageTransactionPrice()
				.setPackageTransactionPriceCurrency(packageTransactionPriceCurrency.evaluate(input));
			
			output
				.getOrCreatePackageTransactionSpread()
				.setPackageTransactionSpreadMonetary(packageTransactionSpreadMonetary.evaluate(input));
			
			output
				.getOrCreatePackageTransactionSpread()
				.setPackageTransactionSpreadPercentage(packageTransactionSpreadPercentage.evaluate(input));
			
			output
				.getOrCreatePackageTransactionSpread()
				.setPackageTransactionSpreadBasis(packageTransactionSpreadBasis.evaluate(input));
			
			output
				.getOrCreatePackageTransactionSpread()
				.setPackageTransactionSpreadNotation(packageTransactionSpreadNotation.evaluate(input));
			
			output
				.getOrCreatePackageTransactionSpread()
				.setPackageTransactionSpreadCurrency(packageTransactionSpreadCurrency.evaluate(input));
			
			output
				.setPriorUTI(priorUTI.evaluate(input));
			
			output
				.setEventIdentifier(eventIdentifier.evaluate(input));
			
			output
				.setActionType(actionType.evaluate(input));
			
			output
				.setEventType(eventType.evaluate(input));
			
			output
				.setReportingTimestamp(reportingTimestamp.evaluate(input));
			
			output
				.setReportSubmittingEntityID(reportSubmittingEntity.evaluate(input));
			
			output
				.setValuationTimestamp(valuationTimestamp.evaluate(input));
			
			output
				.setValuationAmount(valuationAmount.evaluate(input));
			
			output
				.setValuationCurrency(valuationCurrency.evaluate(input));
			
			output
				.setValuationMethod(valuationMethod.evaluate(input));
			
			output
				.setDelta(delta.evaluate(input));
			
			output
				.setNextFloatingReferenceResetDateLeg1(nextFloatingReferenceResetDateLeg1.evaluate(input));
			
			output
				.setNextFloatingReferenceResetDateLeg2(nextFloatingReferenceResetDateLeg2.evaluate(input));
			
			output
				.setSmallScaleBuySideEntityIndicator(smallScaleBuySideEntityIndicator.evaluate(input));
			
			output
				.getOrCreateNonReportable()
				.setEnrichment(enrichmentData.evaluate(input));
			
			return Optional.ofNullable(output)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
