package drr.regulation.asic.rewrite.trade.reports;

import cdm.base.staticdata.asset.common.ISOCountryCodeEnum;
import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import cdm.base.staticdata.asset.common.ProductIdTypeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.model.lib.reports.Tabulator.MultiNestedFieldValueImpl;
import com.rosetta.model.lib.reports.Tabulator.NestedFieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.asic.rewrite.trade.ASICTransactionReport;
import iso20022.auth030.esma.InterestComputationMethod4Code;
import iso20022.auth030.esma.ValuationType1Code;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.inject.Inject;


public class ASICTransactionReportTypeTabulator implements Tabulator<ASICTransactionReport> {
	private final Field uniqueTransactionIdentifierField;
	private final Field uniqueProductIdentifierField;
	private final Field assetClassField;
	private final Field contractTypeField;
	private final Field reportingEntityField;
	private final Field counterparty1Field;
	private final Field counterparty2Field;
	private final Field counterpartyIdentifierTypeIndicatorField;
	private final Field countryOfTheCounterParty2Field;
	private final Field beneficiary1Field;
	private final Field beneficiary1IdentifierTypeIndicatorField;
	private final Field brokerField;
	private final Field executionAgentOfTheCounterparty1Field;
	private final Field directionField;
	private final Field direction2OfLeg1Field;
	private final Field direction2OfLeg2Field;
	private final Field effectiveDateField;
	private final Field expirationDateField;
	private final Field executionTimestampField;
	private final Field eventTimestampField;
	private final Field clearingTimestampField;
	private final Field clearedField;
	private final Field centralCounterpartyField;
	private final Field clearingMemberField;
	private final Field platformIdentifierField;
	private final Field notionalLeg1Field;
	private final Field notionalLeg2Field;
	private final Field totalNotionalQuantityLeg1Field;
	private final Field totalNotionalQuantityLeg2Field;
	private final Field notionalQuantityLeg1Field;
	private final Field notionalQuantityLeg2Field;
	private final Field callAmountField;
	private final Field putAmountField;
	private final Field notionalCurrencyLeg1Field;
	private final Field notionalCurrencyLeg2Field;
	private final Field quantityUnitOfMeasureLeg1Field;
	private final Field quantityUnitOfMeasureLeg2Field;
	private final Field callCurrencyField;
	private final Field putCurrencyField;
	private final Field notionalAmountScheduleLeg1Field;
	private final Field notionalAmountScheduleLeg2Field;
	private final Field priceNotationField;
	private final Field priceField;
	private final Field priceCurrencyField;
	private final Field priceUnitOfMeasureField;
	private final Field fixedRateLeg1Field;
	private final Field fixedRateLeg2Field;
	private final Field spreadNotationLeg1Field;
	private final Field spreadLeg1Field;
	private final Field spreadNotationLeg2Field;
	private final Field spreadLeg2Field;
	private final Field spreadCurrencyLeg1Field;
	private final Field spreadCurrencyLeg2Field;
	private final Field strikePriceNotationField;
	private final Field strikePriceField;
	private final Field strikePriceCurrencyField;
	private final Field optionPremiumAmountField;
	private final Field optionPremiumCurrencyField;
	private final Field optionPremiumPaymentDateField;
	private final Field exchangeRateField;
	private final Field exchangeRateBasisField;
	private final Field dayCountConventionLeg1Field;
	private final Field dayCountConventionLeg2Field;
	private final Field paymentFrequencyPeriodLeg1Field;
	private final Field paymentFrequencyPeriodLeg2Field;
	private final Field paymentFrequencyPeriodMultiplierLeg1Field;
	private final Field paymentFrequencyPeriodMultiplierLeg2Field;
	private final Field settlementCurrencyLeg1Field;
	private final Field settlementCurrencyLeg2Field;
	private final Field otherPaymentField;
	private final Field customBasketCodeField;
	private final Field basketConstituentsField;
	private final Field basketConstituentIdentifierSourceField;
	private final Field underlyingIdentificationField;
	private final Field underlierIDSourceField;
	private final Field maturityDateOfTheUnderlyingField;
	private final Field identifierOfTheFloatingRateLeg2Field;
	private final Field floatingRateReferencePeriodOfLeg2TimePeriodField;
	private final Field floatingRateReferencePeriodOfLeg2MultiplierField;
	private final Field cdSIndexAttachmentPointField;
	private final Field cdSIndexDetachmentPointField;
	private final Field indexFactorField;
	private final Field packageIdentifierField;
	private final Field packageTransactionPriceField;
	private final Field packageTransactionSpreadField;
	private final Field priorUTIField;
	private final Field eventIdentifierField;
	private final Field actionTypeField;
	private final Field eventTypeField;
	private final Field reportingTimestampField;
	private final Field reportSubmittingEntityIDField;
	private final Field valuationTimestampField;
	private final Field valuationAmountField;
	private final Field valuationCurrencyField;
	private final Field valuationMethodField;
	private final Field deltaField;
	private final Field nextFloatingReferenceResetDateLeg1Field;
	private final Field nextFloatingReferenceResetDateLeg2Field;
	private final Field smallScaleBuySideEntityIndicatorField;
	
	private final NotionalAmountScheduleLeg1ReportTypeTabulator notionalAmountScheduleLeg1ReportTypeTabulator;
	private final NotionalAmountScheduleLeg2ReportTypeTabulator notionalAmountScheduleLeg2ReportTypeTabulator;
	private final SpreadLeg1TypeTabulator spreadLeg1TypeTabulator;
	private final SpreadLeg2TypeTabulator spreadLeg2TypeTabulator;
	private final StrikePriceAndNotationTypeTabulator strikePriceAndNotationTypeTabulator;
	private final OtherPaymentReportTypeTabulator otherPaymentReportTypeTabulator;
	private final BasketConstituentsReportTypeTabulator basketConstituentsReportTypeTabulator;
	private final PackageTransactionPriceTypeTabulator packageTransactionPriceTypeTabulator;
	private final PackageTransactionSpreadTypeTabulator packageTransactionSpreadTypeTabulator;
	
	@Inject
	public ASICTransactionReportTypeTabulator(NotionalAmountScheduleLeg1ReportTypeTabulator notionalAmountScheduleLeg1ReportTypeTabulator, NotionalAmountScheduleLeg2ReportTypeTabulator notionalAmountScheduleLeg2ReportTypeTabulator, SpreadLeg1TypeTabulator spreadLeg1TypeTabulator, SpreadLeg2TypeTabulator spreadLeg2TypeTabulator, StrikePriceAndNotationTypeTabulator strikePriceAndNotationTypeTabulator, OtherPaymentReportTypeTabulator otherPaymentReportTypeTabulator, BasketConstituentsReportTypeTabulator basketConstituentsReportTypeTabulator, PackageTransactionPriceTypeTabulator packageTransactionPriceTypeTabulator, PackageTransactionSpreadTypeTabulator packageTransactionSpreadTypeTabulator) {
		this.notionalAmountScheduleLeg1ReportTypeTabulator = notionalAmountScheduleLeg1ReportTypeTabulator;
		this.notionalAmountScheduleLeg2ReportTypeTabulator = notionalAmountScheduleLeg2ReportTypeTabulator;
		this.spreadLeg1TypeTabulator = spreadLeg1TypeTabulator;
		this.spreadLeg2TypeTabulator = spreadLeg2TypeTabulator;
		this.strikePriceAndNotationTypeTabulator = strikePriceAndNotationTypeTabulator;
		this.otherPaymentReportTypeTabulator = otherPaymentReportTypeTabulator;
		this.basketConstituentsReportTypeTabulator = basketConstituentsReportTypeTabulator;
		this.packageTransactionPriceTypeTabulator = packageTransactionPriceTypeTabulator;
		this.packageTransactionSpreadTypeTabulator = packageTransactionSpreadTypeTabulator;
		this.uniqueTransactionIdentifierField = new FieldImpl(
			"uniqueTransactionIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "UniqueTransactionIdentifier")),
			Optional.of("1.1 Unique transaction identifier"),
			Arrays.asList()
		);
		this.uniqueProductIdentifierField = new FieldImpl(
			"uniqueProductIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "UniqueProductIdentifier")),
			Optional.of("1.2 Unique transaction identifier"),
			Arrays.asList()
		);
		this.assetClassField = new FieldImpl(
			"assetClass",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "AssetClass")),
			Optional.of("1.3 Asset class"),
			Arrays.asList()
		);
		this.contractTypeField = new FieldImpl(
			"contractType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ContractType")),
			Optional.of("1.4 Contract type"),
			Arrays.asList()
		);
		this.reportingEntityField = new FieldImpl(
			"reportingEntity",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ReportingEntity")),
			Optional.of("1.5 Reporting entity"),
			Arrays.asList()
		);
		this.counterparty1Field = new FieldImpl(
			"counterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "Counterparty1")),
			Optional.of("1.6 Counterparty 1"),
			Arrays.asList()
		);
		this.counterparty2Field = new FieldImpl(
			"counterparty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "Counterparty2")),
			Optional.of("1.7 Counterparty 2"),
			Arrays.asList()
		);
		this.counterpartyIdentifierTypeIndicatorField = new FieldImpl(
			"counterpartyIdentifierTypeIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "CounterpartyIdentifierTypeIndicator")),
			Optional.of("1.8 Counterparty  identifier type indicator"),
			Arrays.asList()
		);
		this.countryOfTheCounterParty2Field = new FieldImpl(
			"countryOfTheCounterParty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "CountryOfCounterparty2")),
			Optional.of("1.9 Country of Counterparty 2"),
			Arrays.asList()
		);
		this.beneficiary1Field = new FieldImpl(
			"beneficiary1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "Beneficiary1")),
			Optional.of("1.10 Beneficiary 1"),
			Arrays.asList()
		);
		this.beneficiary1IdentifierTypeIndicatorField = new FieldImpl(
			"beneficiary1IdentifierTypeIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "Beneficiary1IdentifierTypeIndicator")),
			Optional.of("1.11 Beneficiary 1 identifier type indicator"),
			Arrays.asList()
		);
		this.brokerField = new FieldImpl(
			"broker",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "Broker")),
			Optional.of("1.12 Broker"),
			Arrays.asList()
		);
		this.executionAgentOfTheCounterparty1Field = new FieldImpl(
			"executionAgentOfTheCounterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ExecutionAgentOfTheCounterparty1")),
			Optional.of("1.13 Execution agent of the Counterparty 1"),
			Arrays.asList()
		);
		this.directionField = new FieldImpl(
			"direction",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "Direction1")),
			Optional.of("1.14 Direction 1"),
			Arrays.asList()
		);
		this.direction2OfLeg1Field = new FieldImpl(
			"direction2OfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "Direction2Leg1")),
			Optional.of("1.15 Direction 2 - Leg 1"),
			Arrays.asList()
		);
		this.direction2OfLeg2Field = new FieldImpl(
			"direction2OfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "Direction2Leg2")),
			Optional.of("1.16 Direction 2 - Leg 2"),
			Arrays.asList()
		);
		this.effectiveDateField = new FieldImpl(
			"effectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "EffectiveDate")),
			Optional.of("1.17 Effective date"),
			Arrays.asList()
		);
		this.expirationDateField = new FieldImpl(
			"expirationDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ExpirationDate")),
			Optional.of("1.18 Expiration date"),
			Arrays.asList()
		);
		this.executionTimestampField = new FieldImpl(
			"executionTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ExecutionTimestamp")),
			Optional.of("1.19 Execution timestamp"),
			Arrays.asList()
		);
		this.eventTimestampField = new FieldImpl(
			"eventTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "EventTimestamp")),
			Optional.of("1.20 Event timestamp"),
			Arrays.asList()
		);
		this.clearingTimestampField = new FieldImpl(
			"clearingTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ClearingTimestamp")),
			Optional.of("1.21 Clearing timestamp"),
			Arrays.asList()
		);
		this.clearedField = new FieldImpl(
			"cleared",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "Cleared")),
			Optional.of("1.22 Cleared"),
			Arrays.asList()
		);
		this.centralCounterpartyField = new FieldImpl(
			"centralCounterparty",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "CentralCounterparty")),
			Optional.of("1.23 Central counterparty"),
			Arrays.asList()
		);
		this.clearingMemberField = new FieldImpl(
			"clearingMember",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ClearingMember")),
			Optional.of("1.24 Clearing member"),
			Arrays.asList()
		);
		this.platformIdentifierField = new FieldImpl(
			"platformIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PlatformIdentifier")),
			Optional.of("1.25 Platform identifier"),
			Arrays.asList()
		);
		this.notionalLeg1Field = new FieldImpl(
			"notionalLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "NotionalAmountLeg1")),
			Optional.of("1.26 Notional amount - Leg 1"),
			Arrays.asList()
		);
		this.notionalLeg2Field = new FieldImpl(
			"notionalLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "NotionalAmountLeg2")),
			Optional.of("1.27 Notional amount - Leg 2"),
			Arrays.asList()
		);
		this.totalNotionalQuantityLeg1Field = new FieldImpl(
			"totalNotionalQuantityLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "TotalNotionalQuantityLeg1")),
			Optional.of("1.28 Total notional quantity - Leg 1"),
			Arrays.asList()
		);
		this.totalNotionalQuantityLeg2Field = new FieldImpl(
			"totalNotionalQuantityLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "TotalNotionalQuantityLeg2")),
			Optional.of("1.29 Total notional quantity - Leg 2"),
			Arrays.asList()
		);
		this.notionalQuantityLeg1Field = new FieldImpl(
			"notionalQuantityLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "NotionalQuantityLeg1")),
			Optional.of("1.30 Notional quantity - Leg 1"),
			Arrays.asList()
		);
		this.notionalQuantityLeg2Field = new FieldImpl(
			"notionalQuantityLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "NotionalQuantityLeg2")),
			Optional.of("1.31 Notional quantity - Leg 2"),
			Arrays.asList()
		);
		this.callAmountField = new FieldImpl(
			"callAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "CallAmount")),
			Optional.of("1.32 Call amount"),
			Arrays.asList()
		);
		this.putAmountField = new FieldImpl(
			"putAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PutAmount")),
			Optional.of("1.33 Put amount"),
			Arrays.asList()
		);
		this.notionalCurrencyLeg1Field = new FieldImpl(
			"notionalCurrencyLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "NotionalCurrencyLeg1")),
			Optional.of("1.34 Notional currency - Leg 1"),
			Arrays.asList()
		);
		this.notionalCurrencyLeg2Field = new FieldImpl(
			"notionalCurrencyLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "NotionalCurrencyLeg2")),
			Optional.of("1.35 Notional currency - Leg 2"),
			Arrays.asList()
		);
		this.quantityUnitOfMeasureLeg1Field = new FieldImpl(
			"quantityUnitOfMeasureLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "QuantityUnitOfMeasureLeg1")),
			Optional.of("1.36 Quantity unit of measure - Leg 1"),
			Arrays.asList()
		);
		this.quantityUnitOfMeasureLeg2Field = new FieldImpl(
			"quantityUnitOfMeasureLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "QuantityUnitOfMeasureLeg2")),
			Optional.of("1.37 Quantity unit of measure - Leg 2"),
			Arrays.asList()
		);
		this.callCurrencyField = new FieldImpl(
			"callCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "CallCurrency")),
			Optional.of("1.38 Call currency"),
			Arrays.asList()
		);
		this.putCurrencyField = new FieldImpl(
			"putCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PutCurrency")),
			Optional.of("1.39 Put currency"),
			Arrays.asList()
		);
		this.notionalAmountScheduleLeg1Field = new FieldImpl(
			"notionalAmountScheduleLeg1",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "NotionalAmountScheduleLeg1")),
			Optional.empty(),
			notionalAmountScheduleLeg1ReportTypeTabulator.getFields()
		);
		this.notionalAmountScheduleLeg2Field = new FieldImpl(
			"notionalAmountScheduleLeg2",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "NotionalAmountScheduleLeg2")),
			Optional.empty(),
			notionalAmountScheduleLeg2ReportTypeTabulator.getFields()
		);
		this.priceNotationField = new FieldImpl(
			"priceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PriceNotation")),
			Optional.of("1.46 Price notation"),
			Arrays.asList()
		);
		this.priceField = new FieldImpl(
			"price",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "Price")),
			Optional.of("1.47 Price"),
			Arrays.asList()
		);
		this.priceCurrencyField = new FieldImpl(
			"priceCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PriceCurrency")),
			Optional.of("1.48 Price currency"),
			Arrays.asList()
		);
		this.priceUnitOfMeasureField = new FieldImpl(
			"priceUnitOfMeasure",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PriceUnitOfMeasure")),
			Optional.of("1.49 Price unit of measure"),
			Arrays.asList()
		);
		this.fixedRateLeg1Field = new FieldImpl(
			"fixedRateLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "FixedRateLeg1")),
			Optional.of("1.50 Fixed rate - Leg 1"),
			Arrays.asList()
		);
		this.fixedRateLeg2Field = new FieldImpl(
			"fixedRateLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "FixedRateLeg2")),
			Optional.of("1.51 Fixed rate - Leg 2"),
			Arrays.asList()
		);
		this.spreadNotationLeg1Field = new FieldImpl(
			"spreadNotationLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SpreadNotationLeg1")),
			Optional.of("1.52 Spread notation - Leg 1"),
			Arrays.asList()
		);
		this.spreadLeg1Field = new FieldImpl(
			"spreadLeg1",
			false,
			Optional.empty(),
			Optional.empty(),
			spreadLeg1TypeTabulator.getFields()
		);
		this.spreadNotationLeg2Field = new FieldImpl(
			"spreadNotationLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SpreadNotationLeg2")),
			Optional.of("1.54 Spread notation - Leg 2"),
			Arrays.asList()
		);
		this.spreadLeg2Field = new FieldImpl(
			"spreadLeg2",
			false,
			Optional.empty(),
			Optional.empty(),
			spreadLeg2TypeTabulator.getFields()
		);
		this.spreadCurrencyLeg1Field = new FieldImpl(
			"spreadCurrencyLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SpreadCurrencyLeg1")),
			Optional.of("1.56 Spread currency - Leg 1"),
			Arrays.asList()
		);
		this.spreadCurrencyLeg2Field = new FieldImpl(
			"spreadCurrencyLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SpreadCurrencyLeg2")),
			Optional.of("1.57 Spread currency - Leg 2"),
			Arrays.asList()
		);
		this.strikePriceNotationField = new FieldImpl(
			"strikePriceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "StrikePriceNotation")),
			Optional.of("1.58 Strike price notation"),
			Arrays.asList()
		);
		this.strikePriceField = new FieldImpl(
			"strikePrice",
			false,
			Optional.empty(),
			Optional.empty(),
			strikePriceAndNotationTypeTabulator.getFields()
		);
		this.strikePriceCurrencyField = new FieldImpl(
			"strikePriceCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "StrikePriceCurrencyCurrencyPair")),
			Optional.of("1.60 Strike price currency/currency pair"),
			Arrays.asList()
		);
		this.optionPremiumAmountField = new FieldImpl(
			"optionPremiumAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "OptionPremiumAmount")),
			Optional.of("1.61 Option premium amount"),
			Arrays.asList()
		);
		this.optionPremiumCurrencyField = new FieldImpl(
			"optionPremiumCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "OptionPremiumCurrency")),
			Optional.of("1.62 Option premium currency"),
			Arrays.asList()
		);
		this.optionPremiumPaymentDateField = new FieldImpl(
			"optionPremiumPaymentDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "OptionPremiumPaymentDate")),
			Optional.of("1.63 Option premium payment date"),
			Arrays.asList()
		);
		this.exchangeRateField = new FieldImpl(
			"exchangeRate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ExchangeRate")),
			Optional.of("1.64 Option premium payment date"),
			Arrays.asList()
		);
		this.exchangeRateBasisField = new FieldImpl(
			"exchangeRateBasis",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ExchangeRateBasis")),
			Optional.of("1.65 Exchange rate basis"),
			Arrays.asList()
		);
		this.dayCountConventionLeg1Field = new FieldImpl(
			"dayCountConventionLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "DayCountConventionLeg1")),
			Optional.of("1.66 Day count convention - Leg 1"),
			Arrays.asList()
		);
		this.dayCountConventionLeg2Field = new FieldImpl(
			"dayCountConventionLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "DayCountConventionLeg2")),
			Optional.of("1.67 Day count convention - Leg 2"),
			Arrays.asList()
		);
		this.paymentFrequencyPeriodLeg1Field = new FieldImpl(
			"paymentFrequencyPeriodLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PaymentFrequencyPeriodLeg1")),
			Optional.of("1.68 Payment frequency period - Leg 1"),
			Arrays.asList()
		);
		this.paymentFrequencyPeriodLeg2Field = new FieldImpl(
			"paymentFrequencyPeriodLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PaymentFrequencyPeriodLeg2")),
			Optional.of("1.69 Payment frequency period - Leg 2"),
			Arrays.asList()
		);
		this.paymentFrequencyPeriodMultiplierLeg1Field = new FieldImpl(
			"paymentFrequencyPeriodMultiplierLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PaymentFrequencyPeriodMultiplierLeg1")),
			Optional.of("1.70 Payment frequency period multiplier - Leg 1"),
			Arrays.asList()
		);
		this.paymentFrequencyPeriodMultiplierLeg2Field = new FieldImpl(
			"paymentFrequencyPeriodMultiplierLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PaymentFrequencyPeriodMultiplierLeg2")),
			Optional.of("1.71 Payment frequency period multiplier - Leg 2"),
			Arrays.asList()
		);
		this.settlementCurrencyLeg1Field = new FieldImpl(
			"settlementCurrencyLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SettlementCurrencyLeg1")),
			Optional.of("1.72 Settlement currency - Leg 1"),
			Arrays.asList()
		);
		this.settlementCurrencyLeg2Field = new FieldImpl(
			"settlementCurrencyLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SettlementCurrencyLeg2")),
			Optional.of("1.73 Settlement currency - Leg 2"),
			Arrays.asList()
		);
		this.otherPaymentField = new FieldImpl(
			"otherPayment",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "OtherPayment")),
			Optional.empty(),
			otherPaymentReportTypeTabulator.getFields()
		);
		this.customBasketCodeField = new FieldImpl(
			"customBasketCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "CustomBasketCode")),
			Optional.of("1.80 Custom basket code"),
			Arrays.asList()
		);
		this.basketConstituentsField = new FieldImpl(
			"basketConstituents",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "BasketConstituents")),
			Optional.empty(),
			basketConstituentsReportTypeTabulator.getFields()
		);
		this.basketConstituentIdentifierSourceField = new FieldImpl(
			"basketConstituentIdentifierSource",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "BasketConstituentIdentifierSource")),
			Optional.of("1.82 Basket constituent identifier source"),
			Arrays.asList()
		);
		this.underlyingIdentificationField = new FieldImpl(
			"underlyingIdentification",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "UnderlierID")),
			Optional.of("1.83 Underlier ID (Other)"),
			Arrays.asList()
		);
		this.underlierIDSourceField = new FieldImpl(
			"underlierIDSource",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "UnderlierIDSource")),
			Optional.of("1.84 Underlier ID (Other) source"),
			Arrays.asList()
		);
		this.maturityDateOfTheUnderlyingField = new FieldImpl(
			"maturityDateOfTheUnderlying",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "MaturityDateOfTheUnderlier")),
			Optional.of("1.85 Maturity date of the underlier"),
			Arrays.asList()
		);
		this.identifierOfTheFloatingRateLeg2Field = new FieldImpl(
			"identifierOfTheFloatingRateLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "IdentifierOfTheFloatingRateLeg2")),
			Optional.of("1.86 Identifier of the floating rate - Leg 2"),
			Arrays.asList()
		);
		this.floatingRateReferencePeriodOfLeg2TimePeriodField = new FieldImpl(
			"floatingRateReferencePeriodOfLeg2TimePeriod",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "FloatingRateReferencePeriodLeg2")),
			Optional.of("1.87 Floating rate reference period - Leg 2"),
			Arrays.asList()
		);
		this.floatingRateReferencePeriodOfLeg2MultiplierField = new FieldImpl(
			"floatingRateReferencePeriodOfLeg2Multiplier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "FloatingRateReferencePeriodMultiplierLeg2")),
			Optional.of("1.88 Floating rate reference period muliplier- Leg 2"),
			Arrays.asList()
		);
		this.cdSIndexAttachmentPointField = new FieldImpl(
			"cdSIndexAttachmentPoint",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "CDSIndexAttachmentPoint")),
			Optional.of("1.89 CDS index attachment point"),
			Arrays.asList()
		);
		this.cdSIndexDetachmentPointField = new FieldImpl(
			"cdSIndexDetachmentPoint",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "CDSIndexDetachmentPoint")),
			Optional.of("1.90 CDS index detachment point"),
			Arrays.asList()
		);
		this.indexFactorField = new FieldImpl(
			"indexFactor",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "IndexFactor")),
			Optional.of("1.91 Index factor"),
			Arrays.asList()
		);
		this.packageIdentifierField = new FieldImpl(
			"packageIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PackageIdentifier")),
			Optional.of("1.92 Package identifier"),
			Arrays.asList()
		);
		this.packageTransactionPriceField = new FieldImpl(
			"packageTransactionPrice",
			false,
			Optional.empty(),
			Optional.empty(),
			packageTransactionPriceTypeTabulator.getFields()
		);
		this.packageTransactionSpreadField = new FieldImpl(
			"packageTransactionSpread",
			false,
			Optional.empty(),
			Optional.empty(),
			packageTransactionSpreadTypeTabulator.getFields()
		);
		this.priorUTIField = new FieldImpl(
			"priorUTI",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PriorUTI")),
			Optional.of("1.99 Prior UTI"),
			Arrays.asList()
		);
		this.eventIdentifierField = new FieldImpl(
			"eventIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "EventIdentifier")),
			Optional.of("1.100 Event identifier"),
			Arrays.asList()
		);
		this.actionTypeField = new FieldImpl(
			"actionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ActionType")),
			Optional.of("1.101 Action type"),
			Arrays.asList()
		);
		this.eventTypeField = new FieldImpl(
			"eventType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "EventType")),
			Optional.of("1.102 Event type"),
			Arrays.asList()
		);
		this.reportingTimestampField = new FieldImpl(
			"reportingTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ReportingTimestamp")),
			Optional.of("1.103 Reporting timestamp"),
			Arrays.asList()
		);
		this.reportSubmittingEntityIDField = new FieldImpl(
			"reportSubmittingEntityID",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ReportSubmittingEntity")),
			Optional.of("1.104 Report submitting entity"),
			Arrays.asList()
		);
		this.valuationTimestampField = new FieldImpl(
			"valuationTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ValuationTimestamp")),
			Optional.of("2.6 Valuation Timestamp"),
			Arrays.asList()
		);
		this.valuationAmountField = new FieldImpl(
			"valuationAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ValuationAmount")),
			Optional.of("2.7 Valuation amount"),
			Arrays.asList()
		);
		this.valuationCurrencyField = new FieldImpl(
			"valuationCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ValuationCurrency")),
			Optional.of("2.8 Valuation currency"),
			Arrays.asList()
		);
		this.valuationMethodField = new FieldImpl(
			"valuationMethod",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "ValuationMethod")),
			Optional.of("2.9 Valuation method"),
			Arrays.asList()
		);
		this.deltaField = new FieldImpl(
			"delta",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "Delta")),
			Optional.of("2.10 Delta"),
			Arrays.asList()
		);
		this.nextFloatingReferenceResetDateLeg1Field = new FieldImpl(
			"nextFloatingReferenceResetDateLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "NextFloatingReferenceResetDateLeg1")),
			Optional.of("2.11 Next floating reference reset date - Leg 1"),
			Arrays.asList()
		);
		this.nextFloatingReferenceResetDateLeg2Field = new FieldImpl(
			"nextFloatingReferenceResetDateLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "NextFloatingReferenceResetDateLeg2")),
			Optional.of("2.12 Next floating reference reset date - Leg 2"),
			Arrays.asList()
		);
		this.smallScaleBuySideEntityIndicatorField = new FieldImpl(
			"smallScaleBuySideEntityIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SmallScaleBuySideEntityIndicator")),
			Optional.of("2.4 Small-scale buy-side Entity indicator"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(uniqueTransactionIdentifierField, uniqueProductIdentifierField, assetClassField, contractTypeField, reportingEntityField, counterparty1Field, counterparty2Field, counterpartyIdentifierTypeIndicatorField, countryOfTheCounterParty2Field, beneficiary1Field, beneficiary1IdentifierTypeIndicatorField, brokerField, executionAgentOfTheCounterparty1Field, directionField, direction2OfLeg1Field, direction2OfLeg2Field, effectiveDateField, expirationDateField, executionTimestampField, eventTimestampField, clearingTimestampField, clearedField, centralCounterpartyField, clearingMemberField, platformIdentifierField, notionalLeg1Field, notionalLeg2Field, totalNotionalQuantityLeg1Field, totalNotionalQuantityLeg2Field, notionalQuantityLeg1Field, notionalQuantityLeg2Field, callAmountField, putAmountField, notionalCurrencyLeg1Field, notionalCurrencyLeg2Field, quantityUnitOfMeasureLeg1Field, quantityUnitOfMeasureLeg2Field, callCurrencyField, putCurrencyField, notionalAmountScheduleLeg1Field, notionalAmountScheduleLeg2Field, priceNotationField, priceField, priceCurrencyField, priceUnitOfMeasureField, fixedRateLeg1Field, fixedRateLeg2Field, spreadNotationLeg1Field, spreadLeg1Field, spreadNotationLeg2Field, spreadLeg2Field, spreadCurrencyLeg1Field, spreadCurrencyLeg2Field, strikePriceNotationField, strikePriceField, strikePriceCurrencyField, optionPremiumAmountField, optionPremiumCurrencyField, optionPremiumPaymentDateField, exchangeRateField, exchangeRateBasisField, dayCountConventionLeg1Field, dayCountConventionLeg2Field, paymentFrequencyPeriodLeg1Field, paymentFrequencyPeriodLeg2Field, paymentFrequencyPeriodMultiplierLeg1Field, paymentFrequencyPeriodMultiplierLeg2Field, settlementCurrencyLeg1Field, settlementCurrencyLeg2Field, otherPaymentField, customBasketCodeField, basketConstituentsField, basketConstituentIdentifierSourceField, underlyingIdentificationField, underlierIDSourceField, maturityDateOfTheUnderlyingField, identifierOfTheFloatingRateLeg2Field, floatingRateReferencePeriodOfLeg2TimePeriodField, floatingRateReferencePeriodOfLeg2MultiplierField, cdSIndexAttachmentPointField, cdSIndexDetachmentPointField, indexFactorField, packageIdentifierField, packageTransactionPriceField, packageTransactionSpreadField, priorUTIField, eventIdentifierField, actionTypeField, eventTypeField, reportingTimestampField, reportSubmittingEntityIDField, valuationTimestampField, valuationAmountField, valuationCurrencyField, valuationMethodField, deltaField, nextFloatingReferenceResetDateLeg1Field, nextFloatingReferenceResetDateLeg2Field, smallScaleBuySideEntityIndicatorField);
	}
	
	@Override
	public List<FieldValue> tabulate(ASICTransactionReport input) {
		Optional<String> uniqueTransactionIdentifier = Optional.ofNullable(input.getUniqueTransactionIdentifier());
		Optional<String> uniqueProductIdentifier = Optional.ofNullable(input.getUniqueProductIdentifier());
		Optional<String> assetClass = Optional.ofNullable(input.getAssetClass());
		Optional<String> contractType = Optional.ofNullable(input.getContractType());
		Optional<String> reportingEntity = Optional.ofNullable(input.getReportingEntity());
		Optional<String> counterparty1 = Optional.ofNullable(input.getCounterparty1());
		Optional<String> counterparty2 = Optional.ofNullable(input.getCounterparty2());
		Optional<Boolean> counterpartyIdentifierTypeIndicator = Optional.ofNullable(input.getCounterpartyIdentifierTypeIndicator());
		Optional<ISOCountryCodeEnum> countryOfTheCounterParty2 = Optional.ofNullable(input.getCountryOfTheCounterParty2());
		Optional<String> beneficiary1 = Optional.ofNullable(input.getBeneficiary1());
		Optional<Boolean> beneficiary1IdentifierTypeIndicator = Optional.ofNullable(input.getBeneficiary1IdentifierTypeIndicator());
		Optional<String> broker = Optional.ofNullable(input.getBroker());
		Optional<String> executionAgentOfTheCounterparty1 = Optional.ofNullable(input.getExecutionAgentOfTheCounterparty1());
		Optional<String> direction = Optional.ofNullable(input.getDirection());
		Optional<String> direction2OfLeg1 = Optional.ofNullable(input.getDirection2OfLeg1());
		Optional<String> direction2OfLeg2 = Optional.ofNullable(input.getDirection2OfLeg2());
		Optional<Date> effectiveDate = Optional.ofNullable(input.getEffectiveDate());
		Optional<Date> expirationDate = Optional.ofNullable(input.getExpirationDate());
		Optional<ZonedDateTime> executionTimestamp = Optional.ofNullable(input.getExecutionTimestamp());
		Optional<ZonedDateTime> eventTimestamp = Optional.ofNullable(input.getEventTimestamp());
		Optional<ZonedDateTime> clearingTimestamp = Optional.ofNullable(input.getClearingTimestamp());
		Optional<String> cleared = Optional.ofNullable(input.getCleared());
		Optional<String> centralCounterparty = Optional.ofNullable(input.getCentralCounterparty());
		Optional<String> clearingMember = Optional.ofNullable(input.getClearingMember());
		Optional<String> platformIdentifier = Optional.ofNullable(input.getPlatformIdentifier());
		Optional<BigDecimal> notionalLeg1 = Optional.ofNullable(input.getNotionalLeg1());
		Optional<BigDecimal> notionalLeg2 = Optional.ofNullable(input.getNotionalLeg2());
		Optional<BigDecimal> totalNotionalQuantityLeg1 = Optional.ofNullable(input.getTotalNotionalQuantityLeg1());
		Optional<BigDecimal> totalNotionalQuantityLeg2 = Optional.ofNullable(input.getTotalNotionalQuantityLeg2());
		Optional<BigDecimal> notionalQuantityLeg1 = Optional.ofNullable(input.getNotionalQuantityLeg1());
		Optional<BigDecimal> notionalQuantityLeg2 = Optional.ofNullable(input.getNotionalQuantityLeg2());
		Optional<BigDecimal> callAmount = Optional.ofNullable(input.getCallAmount());
		Optional<BigDecimal> putAmount = Optional.ofNullable(input.getPutAmount());
		Optional<ISOCurrencyCodeEnum> notionalCurrencyLeg1 = Optional.ofNullable(input.getNotionalCurrencyLeg1());
		Optional<ISOCurrencyCodeEnum> notionalCurrencyLeg2 = Optional.ofNullable(input.getNotionalCurrencyLeg2());
		Optional<String> quantityUnitOfMeasureLeg1 = Optional.ofNullable(input.getQuantityUnitOfMeasureLeg1());
		Optional<String> quantityUnitOfMeasureLeg2 = Optional.ofNullable(input.getQuantityUnitOfMeasureLeg2());
		Optional<ISOCurrencyCodeEnum> callCurrency = Optional.ofNullable(input.getCallCurrency());
		Optional<ISOCurrencyCodeEnum> putCurrency = Optional.ofNullable(input.getPutCurrency());
		Optional<List<List<FieldValue>>> notionalAmountScheduleLeg1 = Optional.ofNullable(input.getNotionalAmountScheduleLeg1())
			.map(x -> x.stream()
				.map(_x -> notionalAmountScheduleLeg1ReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<List<List<FieldValue>>> notionalAmountScheduleLeg2 = Optional.ofNullable(input.getNotionalAmountScheduleLeg2())
			.map(x -> x.stream()
				.map(_x -> notionalAmountScheduleLeg2ReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<String> priceNotation = Optional.ofNullable(input.getPriceNotation());
		Optional<BigDecimal> price = Optional.ofNullable(input.getPrice());
		Optional<ISOCurrencyCodeEnum> priceCurrency = Optional.ofNullable(input.getPriceCurrency());
		Optional<String> priceUnitOfMeasure = Optional.ofNullable(input.getPriceUnitOfMeasure());
		Optional<BigDecimal> fixedRateLeg1 = Optional.ofNullable(input.getFixedRateLeg1());
		Optional<BigDecimal> fixedRateLeg2 = Optional.ofNullable(input.getFixedRateLeg2());
		Optional<String> spreadNotationLeg1 = Optional.ofNullable(input.getSpreadNotationLeg1());
		Optional<List<FieldValue>> spreadLeg1 = Optional.ofNullable(input.getSpreadLeg1())
			.map(x -> spreadLeg1TypeTabulator.tabulate(x));
		Optional<String> spreadNotationLeg2 = Optional.ofNullable(input.getSpreadNotationLeg2());
		Optional<List<FieldValue>> spreadLeg2 = Optional.ofNullable(input.getSpreadLeg2())
			.map(x -> spreadLeg2TypeTabulator.tabulate(x));
		Optional<ISOCurrencyCodeEnum> spreadCurrencyLeg1 = Optional.ofNullable(input.getSpreadCurrencyLeg1());
		Optional<ISOCurrencyCodeEnum> spreadCurrencyLeg2 = Optional.ofNullable(input.getSpreadCurrencyLeg2());
		Optional<String> strikePriceNotation = Optional.ofNullable(input.getStrikePriceNotation());
		Optional<List<FieldValue>> strikePrice = Optional.ofNullable(input.getStrikePrice())
			.map(x -> strikePriceAndNotationTypeTabulator.tabulate(x));
		Optional<String> strikePriceCurrency = Optional.ofNullable(input.getStrikePriceCurrency());
		Optional<BigDecimal> optionPremiumAmount = Optional.ofNullable(input.getOptionPremiumAmount());
		Optional<ISOCurrencyCodeEnum> optionPremiumCurrency = Optional.ofNullable(input.getOptionPremiumCurrency());
		Optional<Date> optionPremiumPaymentDate = Optional.ofNullable(input.getOptionPremiumPaymentDate());
		Optional<BigDecimal> exchangeRate = Optional.ofNullable(input.getExchangeRate());
		Optional<String> exchangeRateBasis = Optional.ofNullable(input.getExchangeRateBasis());
		Optional<InterestComputationMethod4Code> dayCountConventionLeg1 = Optional.ofNullable(input.getDayCountConventionLeg1());
		Optional<InterestComputationMethod4Code> dayCountConventionLeg2 = Optional.ofNullable(input.getDayCountConventionLeg2());
		Optional<String> paymentFrequencyPeriodLeg1 = Optional.ofNullable(input.getPaymentFrequencyPeriodLeg1());
		Optional<String> paymentFrequencyPeriodLeg2 = Optional.ofNullable(input.getPaymentFrequencyPeriodLeg2());
		Optional<Integer> paymentFrequencyPeriodMultiplierLeg1 = Optional.ofNullable(input.getPaymentFrequencyPeriodMultiplierLeg1());
		Optional<Integer> paymentFrequencyPeriodMultiplierLeg2 = Optional.ofNullable(input.getPaymentFrequencyPeriodMultiplierLeg2());
		Optional<ISOCurrencyCodeEnum> settlementCurrencyLeg1 = Optional.ofNullable(input.getSettlementCurrencyLeg1());
		Optional<String> settlementCurrencyLeg2 = Optional.ofNullable(input.getSettlementCurrencyLeg2());
		Optional<List<List<FieldValue>>> otherPayment = Optional.ofNullable(input.getOtherPayment())
			.map(x -> x.stream()
				.map(_x -> otherPaymentReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<String> customBasketCode = Optional.ofNullable(input.getCustomBasketCode());
		Optional<List<List<FieldValue>>> basketConstituents = Optional.ofNullable(input.getBasketConstituents())
			.map(x -> x.stream()
				.map(_x -> basketConstituentsReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<List<? extends ProductIdTypeEnum>> basketConstituentIdentifierSource = Optional.ofNullable(input.getBasketConstituentIdentifierSource());
		Optional<String> underlyingIdentification = Optional.ofNullable(input.getUnderlyingIdentification());
		Optional<String> underlierIDSource = Optional.ofNullable(input.getUnderlierIDSource());
		Optional<Date> maturityDateOfTheUnderlying = Optional.ofNullable(input.getMaturityDateOfTheUnderlying());
		Optional<String> identifierOfTheFloatingRateLeg2 = Optional.ofNullable(input.getIdentifierOfTheFloatingRateLeg2());
		Optional<String> floatingRateReferencePeriodOfLeg2TimePeriod = Optional.ofNullable(input.getFloatingRateReferencePeriodOfLeg2TimePeriod());
		Optional<Integer> floatingRateReferencePeriodOfLeg2Multiplier = Optional.ofNullable(input.getFloatingRateReferencePeriodOfLeg2Multiplier());
		Optional<BigDecimal> cdSIndexAttachmentPoint = Optional.ofNullable(input.getCdSIndexAttachmentPoint());
		Optional<BigDecimal> cdSIndexDetachmentPoint = Optional.ofNullable(input.getCdSIndexDetachmentPoint());
		Optional<BigDecimal> indexFactor = Optional.ofNullable(input.getIndexFactor());
		Optional<String> packageIdentifier = Optional.ofNullable(input.getPackageIdentifier());
		Optional<List<FieldValue>> packageTransactionPrice = Optional.ofNullable(input.getPackageTransactionPrice())
			.map(x -> packageTransactionPriceTypeTabulator.tabulate(x));
		Optional<List<FieldValue>> packageTransactionSpread = Optional.ofNullable(input.getPackageTransactionSpread())
			.map(x -> packageTransactionSpreadTypeTabulator.tabulate(x));
		Optional<String> priorUTI = Optional.ofNullable(input.getPriorUTI());
		Optional<String> eventIdentifier = Optional.ofNullable(input.getEventIdentifier());
		Optional<String> actionType = Optional.ofNullable(input.getActionType());
		Optional<String> eventType = Optional.ofNullable(input.getEventType());
		Optional<ZonedDateTime> reportingTimestamp = Optional.ofNullable(input.getReportingTimestamp());
		Optional<String> reportSubmittingEntityID = Optional.ofNullable(input.getReportSubmittingEntityID());
		Optional<ZonedDateTime> valuationTimestamp = Optional.ofNullable(input.getValuationTimestamp());
		Optional<BigDecimal> valuationAmount = Optional.ofNullable(input.getValuationAmount());
		Optional<ISOCurrencyCodeEnum> valuationCurrency = Optional.ofNullable(input.getValuationCurrency());
		Optional<ValuationType1Code> valuationMethod = Optional.ofNullable(input.getValuationMethod());
		Optional<BigDecimal> delta = Optional.ofNullable(input.getDelta());
		Optional<Date> nextFloatingReferenceResetDateLeg1 = Optional.ofNullable(input.getNextFloatingReferenceResetDateLeg1());
		Optional<Date> nextFloatingReferenceResetDateLeg2 = Optional.ofNullable(input.getNextFloatingReferenceResetDateLeg2());
		Optional<String> smallScaleBuySideEntityIndicator = Optional.ofNullable(input.getSmallScaleBuySideEntityIndicator());
		return Arrays.asList(
			new FieldValueImpl(uniqueTransactionIdentifierField, uniqueTransactionIdentifier),
			new FieldValueImpl(uniqueProductIdentifierField, uniqueProductIdentifier),
			new FieldValueImpl(assetClassField, assetClass),
			new FieldValueImpl(contractTypeField, contractType),
			new FieldValueImpl(reportingEntityField, reportingEntity),
			new FieldValueImpl(counterparty1Field, counterparty1),
			new FieldValueImpl(counterparty2Field, counterparty2),
			new FieldValueImpl(counterpartyIdentifierTypeIndicatorField, counterpartyIdentifierTypeIndicator),
			new FieldValueImpl(countryOfTheCounterParty2Field, countryOfTheCounterParty2),
			new FieldValueImpl(beneficiary1Field, beneficiary1),
			new FieldValueImpl(beneficiary1IdentifierTypeIndicatorField, beneficiary1IdentifierTypeIndicator),
			new FieldValueImpl(brokerField, broker),
			new FieldValueImpl(executionAgentOfTheCounterparty1Field, executionAgentOfTheCounterparty1),
			new FieldValueImpl(directionField, direction),
			new FieldValueImpl(direction2OfLeg1Field, direction2OfLeg1),
			new FieldValueImpl(direction2OfLeg2Field, direction2OfLeg2),
			new FieldValueImpl(effectiveDateField, effectiveDate),
			new FieldValueImpl(expirationDateField, expirationDate),
			new FieldValueImpl(executionTimestampField, executionTimestamp),
			new FieldValueImpl(eventTimestampField, eventTimestamp),
			new FieldValueImpl(clearingTimestampField, clearingTimestamp),
			new FieldValueImpl(clearedField, cleared),
			new FieldValueImpl(centralCounterpartyField, centralCounterparty),
			new FieldValueImpl(clearingMemberField, clearingMember),
			new FieldValueImpl(platformIdentifierField, platformIdentifier),
			new FieldValueImpl(notionalLeg1Field, notionalLeg1),
			new FieldValueImpl(notionalLeg2Field, notionalLeg2),
			new FieldValueImpl(totalNotionalQuantityLeg1Field, totalNotionalQuantityLeg1),
			new FieldValueImpl(totalNotionalQuantityLeg2Field, totalNotionalQuantityLeg2),
			new FieldValueImpl(notionalQuantityLeg1Field, notionalQuantityLeg1),
			new FieldValueImpl(notionalQuantityLeg2Field, notionalQuantityLeg2),
			new FieldValueImpl(callAmountField, callAmount),
			new FieldValueImpl(putAmountField, putAmount),
			new FieldValueImpl(notionalCurrencyLeg1Field, notionalCurrencyLeg1),
			new FieldValueImpl(notionalCurrencyLeg2Field, notionalCurrencyLeg2),
			new FieldValueImpl(quantityUnitOfMeasureLeg1Field, quantityUnitOfMeasureLeg1),
			new FieldValueImpl(quantityUnitOfMeasureLeg2Field, quantityUnitOfMeasureLeg2),
			new FieldValueImpl(callCurrencyField, callCurrency),
			new FieldValueImpl(putCurrencyField, putCurrency),
			new MultiNestedFieldValueImpl(notionalAmountScheduleLeg1Field, notionalAmountScheduleLeg1),
			new MultiNestedFieldValueImpl(notionalAmountScheduleLeg2Field, notionalAmountScheduleLeg2),
			new FieldValueImpl(priceNotationField, priceNotation),
			new FieldValueImpl(priceField, price),
			new FieldValueImpl(priceCurrencyField, priceCurrency),
			new FieldValueImpl(priceUnitOfMeasureField, priceUnitOfMeasure),
			new FieldValueImpl(fixedRateLeg1Field, fixedRateLeg1),
			new FieldValueImpl(fixedRateLeg2Field, fixedRateLeg2),
			new FieldValueImpl(spreadNotationLeg1Field, spreadNotationLeg1),
			new NestedFieldValueImpl(spreadLeg1Field, spreadLeg1),
			new FieldValueImpl(spreadNotationLeg2Field, spreadNotationLeg2),
			new NestedFieldValueImpl(spreadLeg2Field, spreadLeg2),
			new FieldValueImpl(spreadCurrencyLeg1Field, spreadCurrencyLeg1),
			new FieldValueImpl(spreadCurrencyLeg2Field, spreadCurrencyLeg2),
			new FieldValueImpl(strikePriceNotationField, strikePriceNotation),
			new NestedFieldValueImpl(strikePriceField, strikePrice),
			new FieldValueImpl(strikePriceCurrencyField, strikePriceCurrency),
			new FieldValueImpl(optionPremiumAmountField, optionPremiumAmount),
			new FieldValueImpl(optionPremiumCurrencyField, optionPremiumCurrency),
			new FieldValueImpl(optionPremiumPaymentDateField, optionPremiumPaymentDate),
			new FieldValueImpl(exchangeRateField, exchangeRate),
			new FieldValueImpl(exchangeRateBasisField, exchangeRateBasis),
			new FieldValueImpl(dayCountConventionLeg1Field, dayCountConventionLeg1),
			new FieldValueImpl(dayCountConventionLeg2Field, dayCountConventionLeg2),
			new FieldValueImpl(paymentFrequencyPeriodLeg1Field, paymentFrequencyPeriodLeg1),
			new FieldValueImpl(paymentFrequencyPeriodLeg2Field, paymentFrequencyPeriodLeg2),
			new FieldValueImpl(paymentFrequencyPeriodMultiplierLeg1Field, paymentFrequencyPeriodMultiplierLeg1),
			new FieldValueImpl(paymentFrequencyPeriodMultiplierLeg2Field, paymentFrequencyPeriodMultiplierLeg2),
			new FieldValueImpl(settlementCurrencyLeg1Field, settlementCurrencyLeg1),
			new FieldValueImpl(settlementCurrencyLeg2Field, settlementCurrencyLeg2),
			new MultiNestedFieldValueImpl(otherPaymentField, otherPayment),
			new FieldValueImpl(customBasketCodeField, customBasketCode),
			new MultiNestedFieldValueImpl(basketConstituentsField, basketConstituents),
			new FieldValueImpl(basketConstituentIdentifierSourceField, basketConstituentIdentifierSource),
			new FieldValueImpl(underlyingIdentificationField, underlyingIdentification),
			new FieldValueImpl(underlierIDSourceField, underlierIDSource),
			new FieldValueImpl(maturityDateOfTheUnderlyingField, maturityDateOfTheUnderlying),
			new FieldValueImpl(identifierOfTheFloatingRateLeg2Field, identifierOfTheFloatingRateLeg2),
			new FieldValueImpl(floatingRateReferencePeriodOfLeg2TimePeriodField, floatingRateReferencePeriodOfLeg2TimePeriod),
			new FieldValueImpl(floatingRateReferencePeriodOfLeg2MultiplierField, floatingRateReferencePeriodOfLeg2Multiplier),
			new FieldValueImpl(cdSIndexAttachmentPointField, cdSIndexAttachmentPoint),
			new FieldValueImpl(cdSIndexDetachmentPointField, cdSIndexDetachmentPoint),
			new FieldValueImpl(indexFactorField, indexFactor),
			new FieldValueImpl(packageIdentifierField, packageIdentifier),
			new NestedFieldValueImpl(packageTransactionPriceField, packageTransactionPrice),
			new NestedFieldValueImpl(packageTransactionSpreadField, packageTransactionSpread),
			new FieldValueImpl(priorUTIField, priorUTI),
			new FieldValueImpl(eventIdentifierField, eventIdentifier),
			new FieldValueImpl(actionTypeField, actionType),
			new FieldValueImpl(eventTypeField, eventType),
			new FieldValueImpl(reportingTimestampField, reportingTimestamp),
			new FieldValueImpl(reportSubmittingEntityIDField, reportSubmittingEntityID),
			new FieldValueImpl(valuationTimestampField, valuationTimestamp),
			new FieldValueImpl(valuationAmountField, valuationAmount),
			new FieldValueImpl(valuationCurrencyField, valuationCurrency),
			new FieldValueImpl(valuationMethodField, valuationMethod),
			new FieldValueImpl(deltaField, delta),
			new FieldValueImpl(nextFloatingReferenceResetDateLeg1Field, nextFloatingReferenceResetDateLeg1),
			new FieldValueImpl(nextFloatingReferenceResetDateLeg2Field, nextFloatingReferenceResetDateLeg2),
			new FieldValueImpl(smallScaleBuySideEntityIndicatorField, smallScaleBuySideEntityIndicator)
		);
	}
}
