package drr.regulation.asic.rewrite.trade.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.IsActionTypeCORR;
import drr.regulation.common.functions.IsActionTypeEROR;
import drr.regulation.common.functions.IsActionTypeMODI;
import drr.regulation.common.functions.IsActionTypeNEWT;
import drr.regulation.common.functions.IsActionTypeREVI;
import drr.regulation.common.functions.IsActionTypeTERM;
import drr.regulation.common.functions.IsActionTypeVALU;
import javax.inject.Inject;


@ImplementedBy(ActionTypeRule.ActionTypeRuleDefault.class)
public abstract class ActionTypeRule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected IsActionTypeCORR isActionTypeCORR;
	@Inject protected IsActionTypeEROR isActionTypeEROR;
	@Inject protected IsActionTypeMODI isActionTypeMODI;
	@Inject protected IsActionTypeNEWT isActionTypeNEWT;
	@Inject protected IsActionTypeREVI isActionTypeREVI;
	@Inject protected IsActionTypeTERM isActionTypeTERM;
	@Inject protected IsActionTypeVALU isActionTypeVALU;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class ActionTypeRuleDefault extends ActionTypeRule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			output = MapperS.of(input)
				.mapSingleToItem(item -> {
					final Boolean boolean0 = isActionTypeCORR.evaluate(item.get());
					if ((boolean0 == null ? false : boolean0)) {
						return MapperS.of("CORR");
					}
					final Boolean boolean1 = isActionTypeEROR.evaluate(item.get());
					if ((boolean1 == null ? false : boolean1)) {
						return MapperS.of("EROR");
					}
					final Boolean boolean2 = isActionTypeREVI.evaluate(item.get());
					if ((boolean2 == null ? false : boolean2)) {
						return MapperS.of("REVI");
					}
					final Boolean boolean3 = isActionTypeVALU.evaluate(item.get());
					if ((boolean3 == null ? false : boolean3)) {
						return MapperS.of("VALU");
					}
					final Boolean boolean4 = isActionTypeTERM.evaluate(item.get());
					if ((boolean4 == null ? false : boolean4)) {
						return MapperS.of("TERM");
					}
					final Boolean boolean5 = isActionTypeMODI.evaluate(item.get());
					if ((boolean5 == null ? false : boolean5)) {
						return MapperS.of("MODI");
					}
					final Boolean boolean6 = isActionTypeNEWT.evaluate(item.get());
					if ((boolean6 == null ? false : boolean6)) {
						return MapperS.of("NEWT");
					}
					return MapperS.of("ToDo");
				}).get();
			
			return output;
		}
	}
}
