package drr.regulation.asic.rewrite.trade.reports;

import cdm.base.staticdata.party.PayerReceiver;
import cdm.product.qualification.functions.Qualify_AssetClass_Credit;
import cdm.product.qualification.functions.Qualify_InterestRate_CapFloor;
import cdm.product.qualification.functions.Qualify_InterestRate_Option_Swaption;
import cdm.product.template.ContractualProduct;
import cdm.product.template.EconomicTerms;
import cdm.product.template.FixedPricePayout;
import cdm.product.template.Payout;
import cdm.product.template.PerformancePayout;
import cdm.product.template.Product;
import cdm.product.template.TradableProduct;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.CommodityLeg1;
import drr.regulation.common.functions.FXLeg1;
import drr.regulation.common.functions.FixedPriceLeg1;
import drr.regulation.common.functions.InterestRateLeg1;
import drr.regulation.common.functions.IsAllowableAction;
import drr.regulation.common.functions.IsCommodityCommodity;
import drr.regulation.common.functions.IsCommoditySwapProduct;
import drr.regulation.common.functions.IsEquity;
import drr.regulation.common.functions.IsFXForward;
import drr.regulation.common.functions.IsVarianceSwap;
import drr.regulation.common.functions.IsVolatilitySwap;
import drr.regulation.common.functions.ProductForTrade;
import drr.regulation.common.functions.TradeForEvent;
import drr.standards.iosco.cde.functions.CDEDirection2;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(Direction2Leg1Rule.Direction2Leg1RuleDefault.class)
public abstract class Direction2Leg1Rule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDEDirection2 cDEDirection2;
	@Inject protected CommodityLeg1 commodityLeg1;
	@Inject protected FXLeg1 fXLeg1;
	@Inject protected FixedPriceLeg1 fixedPriceLeg1;
	@Inject protected InterestRateLeg1 interestRateLeg1;
	@Inject protected IsAllowableAction isAllowableAction;
	@Inject protected IsCommodityCommodity isCommodityCommodity;
	@Inject protected IsCommoditySwapProduct isCommoditySwapProduct;
	@Inject protected IsEquity isEquity;
	@Inject protected IsFXForward isFXForward;
	@Inject protected IsVarianceSwap isVarianceSwap;
	@Inject protected IsVolatilitySwap isVolatilitySwap;
	@Inject protected ProductForTrade productForTrade;
	@Inject protected Qualify_AssetClass_Credit qualify_AssetClass_Credit;
	@Inject protected Qualify_InterestRate_CapFloor qualify_InterestRate_CapFloor;
	@Inject protected Qualify_InterestRate_Option_Swaption qualify_InterestRate_Option_Swaption;
	@Inject protected TradeForEvent tradeForEvent;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class Direction2Leg1RuleDefault extends Direction2Leg1Rule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult = MapperS.of(input)
				.filterSingleNullSafe(item -> isAllowableAction.evaluate(item.get()));
			output = thenResult
				.mapSingleToItem(item -> MapperS.of(cDEDirection2.evaluate(item.get(), MapperS.of(tradeForEvent.evaluate(item.get()))
					.mapSingleToItem(trade -> trade.<TradableProduct>map("getTradableProduct", _trade -> _trade.getTradableProduct()).<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct())
						.mapSingleToItem(_item -> {
							if (ComparisonResult.of(MapperS.of(isCommoditySwapProduct.evaluate(_item.get()))).and(ComparisonResult.of(MapperS.of(isCommodityCommodity.evaluate(_item.get())))).getOrDefault(false)) {
								return MapperS.of(commodityLeg1.evaluate(_item.get())).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver());
							}
							if (ComparisonResult.of(MapperS.of(isCommoditySwapProduct.evaluate(_item.get()))).and(exists(_item.<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<FixedPricePayout>mapC("getFixedPricePayout", payout -> payout.getFixedPricePayout()))).getOrDefault(false)) {
								return MapperS.of(fixedPriceLeg1.evaluate(_item.get())).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver());
							}
							if (ComparisonResult.of(MapperS.of(isEquity.evaluate(_item.get()))).and(areEqual(ComparisonResult.of(MapperS.of(isVarianceSwap.evaluate(_item.get()))).or(ComparisonResult.of(MapperS.of(isVolatilitySwap.evaluate(_item.get())))), MapperS.of(false), CardinalityOperator.All)).getOrDefault(false)) {
								return MapperS.of(_item.<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<PerformancePayout>mapC("getPerformancePayout", payout -> payout.getPerformancePayout()).get()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver());
							}
							final Boolean _boolean = isFXForward.evaluate(_item.get());
							if ((_boolean == null ? false : _boolean)) {
								return MapperS.of(fXLeg1.evaluate(productForTrade.evaluate(trade.get()))).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver());
							}
							if (areEqual(MapperS.of(qualify_InterestRate_Option_Swaption.evaluate(_item.<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get())), MapperS.of(false), CardinalityOperator.All).and(areEqual(MapperS.of(qualify_InterestRate_CapFloor.evaluate(_item.<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get())), MapperS.of(false), CardinalityOperator.All)).and(areEqual(MapperS.of(qualify_AssetClass_Credit.evaluate(_item.<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get())), MapperS.of(false), CardinalityOperator.All)).getOrDefault(false)) {
								return MapperS.of(interestRateLeg1.evaluate(_item.get())).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver());
							}
							return MapperS.<PayerReceiver>ofNull();
						})).get()))).get();
			
			return output;
		}
	}
}
