package drr.regulation.asic.rewrite.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.asic.rewrite.trade.NotionalAmountScheduleLeg1Report;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class NotionalAmountScheduleLeg1ReportTypeTabulator implements Tabulator<NotionalAmountScheduleLeg1Report> {
	private final Field amountField;
	private final Field effectiveDateField;
	
	public NotionalAmountScheduleLeg1ReportTypeTabulator() {
		this.amountField = new FieldImpl(
			"amount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "NotionalAmountScheduleAmountLeg1")),
			Optional.of("1.42 Notional amount schedule amount - Leg 1"),
			Arrays.asList()
		);
		this.effectiveDateField = new FieldImpl(
			"effectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "NotionalAmountScheduleEffectiveDateLeg1")),
			Optional.of("1.40 Notional amount schedule effective date - Leg 1"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(amountField, effectiveDateField);
	}
	
	@Override
	public List<FieldValue> tabulate(NotionalAmountScheduleLeg1Report input) {
		Optional<BigDecimal> amount = Optional.ofNullable(input.getAmount());
		Optional<Date> effectiveDate = Optional.ofNullable(input.getEffectiveDate());
		return Arrays.asList(
			new FieldValueImpl(amountField, amount),
			new FieldValueImpl(effectiveDateField, effectiveDate)
		);
	}
}
