package drr.regulation.asic.rewrite.trade.reports;

import cdm.base.datetime.Frequency;
import cdm.base.math.DatedValue;
import cdm.base.math.NonNegativeQuantitySchedule;
import cdm.base.math.metafields.FieldWithMetaNonNegativeQuantitySchedule;
import cdm.event.common.Trade;
import cdm.product.common.settlement.PriceQuantity;
import cdm.product.template.TradableProduct;
import cdm.product.template.TradeLot;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.CommodityLeg2;
import drr.regulation.common.functions.CommodityObservablePriceQuantity;
import drr.regulation.common.functions.IsCommodity;
import drr.regulation.common.functions.ProductForTrade;
import drr.regulation.common.functions.TradeForEvent;
import java.math.BigDecimal;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(NotionalQuantityLeg2Rule.NotionalQuantityLeg2RuleDefault.class)
public abstract class NotionalQuantityLeg2Rule implements ReportFunction<TransactionReportInstruction, BigDecimal> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CommodityLeg2 commodityLeg2;
	@Inject protected CommodityObservablePriceQuantity commodityObservablePriceQuantity;
	@Inject protected IsCommodity isCommodity;
	@Inject protected ProductForTrade productForTrade;
	@Inject protected TradeForEvent tradeForEvent;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public BigDecimal evaluate(TransactionReportInstruction input) {
		BigDecimal output = doEvaluate(input);
		
		return output;
	}

	protected abstract BigDecimal doEvaluate(TransactionReportInstruction input);

	public static class NotionalQuantityLeg2RuleDefault extends NotionalQuantityLeg2Rule {
		@Override
		protected BigDecimal doEvaluate(TransactionReportInstruction input) {
			BigDecimal output = null;
			return assignOutput(output, input);
		}
		
		protected BigDecimal assignOutput(BigDecimal output, TransactionReportInstruction input) {
			final MapperS<Trade> thenResult0 = MapperS.of(input)
				.mapSingleToItem(item -> MapperS.of(tradeForEvent.evaluate(item.get())));
			final MapperS<PriceQuantity> thenResult1 = thenResult0
				.mapSingleToItem(trade -> {
					final Boolean _boolean = isCommodity.evaluate(productForTrade.evaluate(trade.get()));
					if ((_boolean == null ? false : _boolean)) {
						return MapperS.of(commodityObservablePriceQuantity.evaluate(trade.<TradableProduct>map("getTradableProduct", _trade -> _trade.getTradableProduct()).<TradeLot>mapC("getTradeLot", tradableProduct -> tradableProduct.getTradeLot()).get(), commodityLeg2.evaluate(productForTrade.evaluate(trade.get()))));
					}
					return MapperS.<PriceQuantity>ofNull();
				});
			final MapperC<NonNegativeQuantitySchedule> thenResult2 = thenResult1
				.mapSingleToList(item -> item.<FieldWithMetaNonNegativeQuantitySchedule>mapC("getQuantity", priceQuantity -> priceQuantity.getQuantity()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()));
			final MapperC<NonNegativeQuantitySchedule> thenResult3 = thenResult2
				.filterItemNullSafe(item -> exists(item.<Frequency>map("getFrequency", quantitySchedule -> quantitySchedule.getFrequency())).get());
			final MapperC<BigDecimal> thenResult4 = thenResult3
				.mapItem(quantitySchedule -> {
					if (exists(quantitySchedule.<BigDecimal>map("getValue", measureBase -> measureBase.getValue())).getOrDefault(false)) {
						return quantitySchedule.<BigDecimal>map("getValue", measureBase -> measureBase.getValue());
					}
					if (exists(quantitySchedule.<DatedValue>mapC("getDatedValue", measureSchedule -> measureSchedule.getDatedValue())).and(areEqual(MapperS.of(distinct(quantitySchedule.<DatedValue>mapC("getDatedValue", measureSchedule -> measureSchedule.getDatedValue()).<BigDecimal>map("getValue", datedValue -> datedValue.getValue())).resultCount()), MapperS.of(1), CardinalityOperator.All)).getOrDefault(false)) {
						return quantitySchedule.<DatedValue>mapC("getDatedValue", measureSchedule -> measureSchedule.getDatedValue()).<BigDecimal>map("getValue", datedValue -> datedValue.getValue())
							.first();
					}
					return MapperS.<BigDecimal>ofNull();
				});
			output = MapperS.of(distinct(thenResult4).get()).get();
			
			return output;
		}
	}
}
