package drr.regulation.asic.rewrite.trade.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.asic.rewrite.trade.PackageTransactionPrice;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class PackageTransactionPriceTypeTabulator implements Tabulator<PackageTransactionPrice> {
	private final Field packageTransactionPriceMonetaryField;
	private final Field packageTransactionPricePercentageField;
	private final Field packageTransactionPriceNotationField;
	private final Field packageTransactionPriceCurrencyField;
	
	public PackageTransactionPriceTypeTabulator() {
		this.packageTransactionPriceMonetaryField = new FieldImpl(
			"packageTransactionPriceMonetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PackageTransactionPriceMonetary")),
			Optional.of("1.94 Package transaction price (Monetary)"),
			Arrays.asList()
		);
		this.packageTransactionPricePercentageField = new FieldImpl(
			"packageTransactionPricePercentage",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PackageTransactionPricePercentage")),
			Optional.of("1.94 Package transaction price (Percentage)"),
			Arrays.asList()
		);
		this.packageTransactionPriceNotationField = new FieldImpl(
			"packageTransactionPriceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PackageTransactionPriceNotation")),
			Optional.of("1.93 Package transaction price notation"),
			Arrays.asList()
		);
		this.packageTransactionPriceCurrencyField = new FieldImpl(
			"packageTransactionPriceCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PackageTransactionPriceCurrency")),
			Optional.of("1.95 Package transaction price currency"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(packageTransactionPriceMonetaryField, packageTransactionPricePercentageField, packageTransactionPriceNotationField, packageTransactionPriceCurrencyField);
	}
	
	@Override
	public List<FieldValue> tabulate(PackageTransactionPrice input) {
		Optional<BigDecimal> packageTransactionPriceMonetary = Optional.ofNullable(input.getPackageTransactionPriceMonetary());
		Optional<BigDecimal> packageTransactionPricePercentage = Optional.ofNullable(input.getPackageTransactionPricePercentage());
		Optional<String> packageTransactionPriceNotation = Optional.ofNullable(input.getPackageTransactionPriceNotation());
		Optional<ISOCurrencyCodeEnum> packageTransactionPriceCurrency = Optional.ofNullable(input.getPackageTransactionPriceCurrency());
		return Arrays.asList(
			new FieldValueImpl(packageTransactionPriceMonetaryField, packageTransactionPriceMonetary),
			new FieldValueImpl(packageTransactionPricePercentageField, packageTransactionPricePercentage),
			new FieldValueImpl(packageTransactionPriceNotationField, packageTransactionPriceNotation),
			new FieldValueImpl(packageTransactionPriceCurrencyField, packageTransactionPriceCurrency)
		);
	}
}
