package drr.regulation.asic.rewrite.trade.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.asic.rewrite.trade.PackageTransactionSpread;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class PackageTransactionSpreadTypeTabulator implements Tabulator<PackageTransactionSpread> {
	private final Field packageTransactionSpreadMonetaryField;
	private final Field packageTransactionSpreadPercentageField;
	private final Field packageTransactionSpreadBasisField;
	private final Field packageTransactionSpreadNotationField;
	private final Field packageTransactionSpreadCurrencyField;
	
	public PackageTransactionSpreadTypeTabulator() {
		this.packageTransactionSpreadMonetaryField = new FieldImpl(
			"packageTransactionSpreadMonetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PackageTransactionSpreadMonetary")),
			Optional.of("1.97 Package transaction spread (Monetary)"),
			Arrays.asList()
		);
		this.packageTransactionSpreadPercentageField = new FieldImpl(
			"packageTransactionSpreadPercentage",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PackageTransactionSpreadPercentage")),
			Optional.of("1.97 Package transaction spread (Percentage)"),
			Arrays.asList()
		);
		this.packageTransactionSpreadBasisField = new FieldImpl(
			"packageTransactionSpreadBasis",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PackageTransactionSpreadBasis")),
			Optional.of("1.97 Package transaction spread (Basis)"),
			Arrays.asList()
		);
		this.packageTransactionSpreadNotationField = new FieldImpl(
			"packageTransactionSpreadNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PackageTransactionSpreadNotation")),
			Optional.of("1.96 Package transaction spread notation"),
			Arrays.asList()
		);
		this.packageTransactionSpreadCurrencyField = new FieldImpl(
			"packageTransactionSpreadCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "PackageTransactionSpreadCurrency")),
			Optional.of("1.98 Package transaction spread currency"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(packageTransactionSpreadMonetaryField, packageTransactionSpreadPercentageField, packageTransactionSpreadBasisField, packageTransactionSpreadNotationField, packageTransactionSpreadCurrencyField);
	}
	
	@Override
	public List<FieldValue> tabulate(PackageTransactionSpread input) {
		Optional<BigDecimal> packageTransactionSpreadMonetary = Optional.ofNullable(input.getPackageTransactionSpreadMonetary());
		Optional<BigDecimal> packageTransactionSpreadPercentage = Optional.ofNullable(input.getPackageTransactionSpreadPercentage());
		Optional<BigDecimal> packageTransactionSpreadBasis = Optional.ofNullable(input.getPackageTransactionSpreadBasis());
		Optional<String> packageTransactionSpreadNotation = Optional.ofNullable(input.getPackageTransactionSpreadNotation());
		Optional<ISOCurrencyCodeEnum> packageTransactionSpreadCurrency = Optional.ofNullable(input.getPackageTransactionSpreadCurrency());
		return Arrays.asList(
			new FieldValueImpl(packageTransactionSpreadMonetaryField, packageTransactionSpreadMonetary),
			new FieldValueImpl(packageTransactionSpreadPercentageField, packageTransactionSpreadPercentage),
			new FieldValueImpl(packageTransactionSpreadBasisField, packageTransactionSpreadBasis),
			new FieldValueImpl(packageTransactionSpreadNotationField, packageTransactionSpreadNotation),
			new FieldValueImpl(packageTransactionSpreadCurrencyField, packageTransactionSpreadCurrency)
		);
	}
}
