package drr.regulation.asic.rewrite.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.asic.rewrite.trade.SpreadLeg1;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class SpreadLeg1TypeTabulator implements Tabulator<SpreadLeg1> {
	private final Field spreadLeg1MonetaryField;
	private final Field spreadLeg1PercentageField;
	private final Field spreadLeg1BasisField;
	private final Field spreadLeg1NotationField;
	
	public SpreadLeg1TypeTabulator() {
		this.spreadLeg1MonetaryField = new FieldImpl(
			"spreadLeg1Monetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SpreadLeg1Monetary")),
			Optional.of("1.53 Spread - Leg 1 (Monetary)"),
			Arrays.asList()
		);
		this.spreadLeg1PercentageField = new FieldImpl(
			"spreadLeg1Percentage",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SpreadLeg1Percentage")),
			Optional.of("1.53 Spread - Leg 1 (Percentage)"),
			Arrays.asList()
		);
		this.spreadLeg1BasisField = new FieldImpl(
			"spreadLeg1Basis",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SpreadLeg1Basis")),
			Optional.of("1.53 Spread - Leg 1 (Basis)"),
			Arrays.asList()
		);
		this.spreadLeg1NotationField = new FieldImpl(
			"spreadLeg1Notation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SpreadLeg1Notation")),
			Optional.of("1.53 Spread of Leg 1 Notation (Non Reportable)"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(spreadLeg1MonetaryField, spreadLeg1PercentageField, spreadLeg1BasisField, spreadLeg1NotationField);
	}
	
	@Override
	public List<FieldValue> tabulate(SpreadLeg1 input) {
		Optional<BigDecimal> spreadLeg1Monetary = Optional.ofNullable(input.getSpreadLeg1Monetary());
		Optional<BigDecimal> spreadLeg1Percentage = Optional.ofNullable(input.getSpreadLeg1Percentage());
		Optional<BigDecimal> spreadLeg1Basis = Optional.ofNullable(input.getSpreadLeg1Basis());
		Optional<PriceNotationEnum> spreadLeg1Notation = Optional.ofNullable(input.getSpreadLeg1Notation());
		return Arrays.asList(
			new FieldValueImpl(spreadLeg1MonetaryField, spreadLeg1Monetary),
			new FieldValueImpl(spreadLeg1PercentageField, spreadLeg1Percentage),
			new FieldValueImpl(spreadLeg1BasisField, spreadLeg1Basis),
			new FieldValueImpl(spreadLeg1NotationField, spreadLeg1Notation)
		);
	}
}
