package drr.regulation.asic.rewrite.trade.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.GetNotationEnum;
import drr.regulation.common.functions.IsAllowableAction;
import drr.standards.iosco.cde.reports.CDESpreadLeg2NotationRule;
import drr.standards.iso.PriceNotationEnum;
import javax.inject.Inject;


@ImplementedBy(SpreadLeg2NotationRule.SpreadLeg2NotationRuleDefault.class)
public abstract class SpreadLeg2NotationRule implements ReportFunction<TransactionReportInstruction, PriceNotationEnum> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDESpreadLeg2NotationRule cDESpreadLeg2Notation;
	@Inject protected GetNotationEnum getNotationEnum;
	@Inject protected IsAllowableAction isAllowableAction;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public PriceNotationEnum evaluate(TransactionReportInstruction input) {
		PriceNotationEnum output = doEvaluate(input);
		
		return output;
	}

	protected abstract PriceNotationEnum doEvaluate(TransactionReportInstruction input);

	public static class SpreadLeg2NotationRuleDefault extends SpreadLeg2NotationRule {
		@Override
		protected PriceNotationEnum doEvaluate(TransactionReportInstruction input) {
			PriceNotationEnum output = null;
			return assignOutput(output, input);
		}
		
		protected PriceNotationEnum assignOutput(PriceNotationEnum output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult = MapperS.of(input)
				.filterSingleNullSafe(item -> isAllowableAction.evaluate(item.get()));
			output = thenResult
				.mapSingleToItem(item -> MapperS.of(getNotationEnum.evaluate(cDESpreadLeg2Notation.evaluate(item.get())))).get();
			
			return output;
		}
	}
}
