package drr.regulation.asic.rewrite.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.asic.rewrite.trade.SpreadLeg2;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class SpreadLeg2TypeTabulator implements Tabulator<SpreadLeg2> {
	private final Field spreadLeg2MonetaryField;
	private final Field spreadLeg2PercentageField;
	private final Field spreadLeg2BasisField;
	private final Field spreadLeg2NotationField;
	
	public SpreadLeg2TypeTabulator() {
		this.spreadLeg2MonetaryField = new FieldImpl(
			"spreadLeg2Monetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SpreadLeg2Monetary")),
			Optional.of("1.55 Spread - Leg 1 (Monetary)"),
			Arrays.asList()
		);
		this.spreadLeg2PercentageField = new FieldImpl(
			"spreadLeg2Percentage",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SpreadLeg2Percentage")),
			Optional.of("1.55 Spread - Leg 2 (Percentage)"),
			Arrays.asList()
		);
		this.spreadLeg2BasisField = new FieldImpl(
			"spreadLeg2Basis",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SpreadLeg2Basis")),
			Optional.of("1.55 Spread - Leg 2 (Basis)"),
			Arrays.asList()
		);
		this.spreadLeg2NotationField = new FieldImpl(
			"spreadLeg2Notation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "SpreadLeg2Notation")),
			Optional.of("1.55 Spread of Leg 21 Notation (Non Reportable)"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(spreadLeg2MonetaryField, spreadLeg2PercentageField, spreadLeg2BasisField, spreadLeg2NotationField);
	}
	
	@Override
	public List<FieldValue> tabulate(SpreadLeg2 input) {
		Optional<BigDecimal> spreadLeg2Monetary = Optional.ofNullable(input.getSpreadLeg2Monetary());
		Optional<BigDecimal> spreadLeg2Percentage = Optional.ofNullable(input.getSpreadLeg2Percentage());
		Optional<BigDecimal> spreadLeg2Basis = Optional.ofNullable(input.getSpreadLeg2Basis());
		Optional<PriceNotationEnum> spreadLeg2Notation = Optional.ofNullable(input.getSpreadLeg2Notation());
		return Arrays.asList(
			new FieldValueImpl(spreadLeg2MonetaryField, spreadLeg2Monetary),
			new FieldValueImpl(spreadLeg2PercentageField, spreadLeg2Percentage),
			new FieldValueImpl(spreadLeg2BasisField, spreadLeg2Basis),
			new FieldValueImpl(spreadLeg2NotationField, spreadLeg2Notation)
		);
	}
}
