package drr.regulation.asic.rewrite.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.asic.rewrite.trade.StrikePriceAndNotation;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class StrikePriceAndNotationTypeTabulator implements Tabulator<StrikePriceAndNotation> {
	private final Field strikePriceMonetaryField;
	private final Field strikePricePercentageField;
	
	public StrikePriceAndNotationTypeTabulator() {
		this.strikePriceMonetaryField = new FieldImpl(
			"strikePriceMonetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "StrikePriceMonetary")),
			Optional.of("1.59 Strike price (Monetary)"),
			Arrays.asList()
		);
		this.strikePricePercentageField = new FieldImpl(
			"strikePricePercentage",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "asic", "rewrite", "trade"), "StrikePricePercentage")),
			Optional.of("1.59 Strike price (Percentage)"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(strikePriceMonetaryField, strikePricePercentageField);
	}
	
	@Override
	public List<FieldValue> tabulate(StrikePriceAndNotation input) {
		Optional<BigDecimal> strikePriceMonetary = Optional.ofNullable(input.getStrikePriceMonetary());
		Optional<BigDecimal> strikePricePercentage = Optional.ofNullable(input.getStrikePricePercentage());
		return Arrays.asList(
			new FieldValueImpl(strikePriceMonetaryField, strikePriceMonetary),
			new FieldValueImpl(strikePricePercentageField, strikePricePercentage)
		);
	}
}
