package drr.regulation.asic.rewrite.trade.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.asic.rewrite.trade.PackageTransactionPrice;
import java.util.List;
import java.util.regex.Pattern;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkNumber;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkString;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class PackageTransactionPriceTypeFormatValidator implements Validator<PackageTransactionPrice> {

	private List<ComparisonResult> getComparisonResults(PackageTransactionPrice o) {
		return Lists.<ComparisonResult>newArrayList(
				checkNumber("packageTransactionPriceMonetary", o.getPackageTransactionPriceMonetary(), of(18), of(13), empty(), empty()), 
				checkNumber("packageTransactionPricePercentage", o.getPackageTransactionPricePercentage(), of(11), of(10), empty(), empty()), 
				checkString("packageTransactionPriceNotation", o.getPackageTransactionPriceNotation(), 1, of(1), of(Pattern.compile("[1-4]")))
			);
	}

	@Override
	public ValidationResult<PackageTransactionPrice> validate(RosettaPath path, PackageTransactionPrice o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("PackageTransactionPrice", ValidationType.TYPE_FORMAT, "PackageTransactionPrice", path, "", error);
		}
		return success("PackageTransactionPrice", ValidationType.TYPE_FORMAT, "PackageTransactionPrice", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, PackageTransactionPrice o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("PackageTransactionPrice", ValidationType.TYPE_FORMAT, "PackageTransactionPrice", path, "", res.getError());
				}
				return success("PackageTransactionPrice", ValidationType.TYPE_FORMAT, "PackageTransactionPrice", path, "");
			})
			.collect(toList());
	}

}
