package drr.regulation.asic.rewrite.trade.validation;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.asic.rewrite.trade.PackageTransactionSpread;
import java.math.BigDecimal;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class PackageTransactionSpreadValidator implements Validator<PackageTransactionSpread> {

	private List<ComparisonResult> getComparisonResults(PackageTransactionSpread o) {
		return Lists.<ComparisonResult>newArrayList(
				checkCardinality("packageTransactionSpreadMonetary", (BigDecimal) o.getPackageTransactionSpreadMonetary() != null ? 1 : 0, 0, 1), 
				checkCardinality("packageTransactionSpreadPercentage", (BigDecimal) o.getPackageTransactionSpreadPercentage() != null ? 1 : 0, 0, 1), 
				checkCardinality("packageTransactionSpreadBasis", (BigDecimal) o.getPackageTransactionSpreadBasis() != null ? 1 : 0, 0, 1), 
				checkCardinality("packageTransactionSpreadNotation", (String) o.getPackageTransactionSpreadNotation() != null ? 1 : 0, 0, 1), 
				checkCardinality("packageTransactionSpreadCurrency", (ISOCurrencyCodeEnum) o.getPackageTransactionSpreadCurrency() != null ? 1 : 0, 0, 1)
			);
	}

	@Override
	public ValidationResult<PackageTransactionSpread> validate(RosettaPath path, PackageTransactionSpread o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("PackageTransactionSpread", ValidationType.CARDINALITY, "PackageTransactionSpread", path, "", error);
		}
		return success("PackageTransactionSpread", ValidationType.CARDINALITY, "PackageTransactionSpread", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, PackageTransactionSpread o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("PackageTransactionSpread", ValidationType.CARDINALITY, "PackageTransactionSpread", path, "", res.getError());
				}
				return success("PackageTransactionSpread", ValidationType.CARDINALITY, "PackageTransactionSpread", path, "");
			})
			.collect(toList());
	}

}
