package drr.regulation.asic.rewrite.trade.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.asic.rewrite.trade.StrikePriceAndNotation;
import java.math.BigDecimal;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class StrikePriceAndNotationValidator implements Validator<StrikePriceAndNotation> {

	private List<ComparisonResult> getComparisonResults(StrikePriceAndNotation o) {
		return Lists.<ComparisonResult>newArrayList(
				checkCardinality("strikePriceMonetary", (BigDecimal) o.getStrikePriceMonetary() != null ? 1 : 0, 0, 1), 
				checkCardinality("strikePricePercentage", (BigDecimal) o.getStrikePricePercentage() != null ? 1 : 0, 0, 1)
			);
	}

	@Override
	public ValidationResult<StrikePriceAndNotation> validate(RosettaPath path, StrikePriceAndNotation o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("StrikePriceAndNotation", ValidationType.CARDINALITY, "StrikePriceAndNotation", path, "", error);
		}
		return success("StrikePriceAndNotation", ValidationType.CARDINALITY, "StrikePriceAndNotation", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, StrikePriceAndNotation o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("StrikePriceAndNotation", ValidationType.CARDINALITY, "StrikePriceAndNotation", path, "", res.getError());
				}
				return success("StrikePriceAndNotation", ValidationType.CARDINALITY, "StrikePriceAndNotation", path, "");
			})
			.collect(toList());
	}

}
