package drr.regulation.asic.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.asic.rewrite.trade.ASICTransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ASICTransactionReportDTCC_ASIC_BR_1016_01")
@ImplementedBy(ASICTransactionReportDTCCASICBR101601.Default.class)
public interface ASICTransactionReportDTCCASICBR101601 extends Validator<ASICTransactionReport> {
	
	String NAME = "ASICTransactionReportDTCC_ASIC_BR_1016_01";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType then if direction2OfLeg1 exists then direction2OfLeg2 exists else if ((direction is absent) and (direction2OfLeg1 is absent) or (direction2OfLeg1 = direction2OfLeg2)) then direction2OfLeg2 is absent";
	
	ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport);
	
	class Default implements ASICTransactionReportDTCCASICBR101601 {
	
		@Override
		public ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport) {
			ComparisonResult result = executeDataRule(aSICTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ASICTransactionReport aSICTransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(aSICTransactionReport).<String>map("getActionType", _aSICTransactionReport -> _aSICTransactionReport.getActionType()), CardinalityOperator.Any).getOrDefault(false)) {
					if (exists(MapperS.of(aSICTransactionReport).<String>map("getDirection2OfLeg1", _aSICTransactionReport -> _aSICTransactionReport.getDirection2OfLeg1())).getOrDefault(false)) {
						return exists(MapperS.of(aSICTransactionReport).<String>map("getDirection2OfLeg2", _aSICTransactionReport -> _aSICTransactionReport.getDirection2OfLeg2()));
					}
					if (notExists(MapperS.of(aSICTransactionReport).<String>map("getDirection", _aSICTransactionReport -> _aSICTransactionReport.getDirection())).and(notExists(MapperS.of(aSICTransactionReport).<String>map("getDirection2OfLeg1", _aSICTransactionReport -> _aSICTransactionReport.getDirection2OfLeg1()))).or(areEqual(MapperS.of(aSICTransactionReport).<String>map("getDirection2OfLeg1", _aSICTransactionReport -> _aSICTransactionReport.getDirection2OfLeg1()), MapperS.of(aSICTransactionReport).<String>map("getDirection2OfLeg2", _aSICTransactionReport -> _aSICTransactionReport.getDirection2OfLeg2()), CardinalityOperator.All)).getOrDefault(false)) {
						return notExists(MapperS.of(aSICTransactionReport).<String>map("getDirection2OfLeg2", _aSICTransactionReport -> _aSICTransactionReport.getDirection2OfLeg2()));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ASICTransactionReportDTCCASICBR101601 {
	
		@Override
		public ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION);
		}
	}
}
