package drr.regulation.asic.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.asic.rewrite.trade.ASICTransactionReport;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ASICTransactionReportDTCC_ASIC_BR_1031_02")
@ImplementedBy(ASICTransactionReportDTCCASICBR103102.Default.class)
public interface ASICTransactionReportDTCCASICBR103102 extends Validator<ASICTransactionReport> {
	
	String NAME = "ASICTransactionReportDTCC_ASIC_BR_1031_02";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType then if notionalQuantityLeg2 <= totalNotionalQuantityLeg2 then True else if notionalQuantityLeg2 < 0 then False";
	
	ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport);
	
	class Default implements ASICTransactionReportDTCCASICBR103102 {
	
		@Override
		public ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport) {
			ComparisonResult result = executeDataRule(aSICTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ASICTransactionReport aSICTransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(aSICTransactionReport).<String>map("getActionType", _aSICTransactionReport -> _aSICTransactionReport.getActionType()), CardinalityOperator.Any).getOrDefault(false)) {
					if (lessThanEquals(MapperS.of(aSICTransactionReport).<BigDecimal>map("getNotionalQuantityLeg2", _aSICTransactionReport -> _aSICTransactionReport.getNotionalQuantityLeg2()), MapperS.of(aSICTransactionReport).<BigDecimal>map("getTotalNotionalQuantityLeg2", _aSICTransactionReport -> _aSICTransactionReport.getTotalNotionalQuantityLeg2()), CardinalityOperator.All).getOrDefault(false)) {
						return ComparisonResult.of(MapperS.of(true));
					}
					if (lessThan(MapperS.of(aSICTransactionReport).<BigDecimal>map("getNotionalQuantityLeg2", _aSICTransactionReport -> _aSICTransactionReport.getNotionalQuantityLeg2()), MapperS.of(BigDecimal.valueOf(0)), CardinalityOperator.All).getOrDefault(false)) {
						return ComparisonResult.of(MapperS.of(false));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ASICTransactionReportDTCCASICBR103102 {
	
		@Override
		public ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION);
		}
	}
}
