package drr.regulation.asic.rewrite.trade.validation.datarule;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.asic.rewrite.trade.ASICTransactionReport;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ASICTransactionReportDTCC_ASIC_BR_1035_01")
@ImplementedBy(ASICTransactionReportDTCCASICBR103501.Default.class)
public interface ASICTransactionReportDTCCASICBR103501 extends Validator<ASICTransactionReport> {
	
	String NAME = "ASICTransactionReportDTCC_ASIC_BR_1035_01";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType then if notionalLeg2 exists then (notionalCurrencyLeg2 exists and [ISOCurrencyCodeEnum->XAG, ISOCurrencyCodeEnum->XAU, ISOCurrencyCodeEnum->XBA, ISOCurrencyCodeEnum->XBB, ISOCurrencyCodeEnum->XBC, ISOCurrencyCodeEnum->XBD, ISOCurrencyCodeEnum->XDR, ISOCurrencyCodeEnum->XPD, ISOCurrencyCodeEnum->XPT, ISOCurrencyCodeEnum->XXX] any <> notionalCurrencyLeg2 and ((notionalCurrencyLeg2 = callCurrency) or (notionalCurrencyLeg2 = putCurrency)))";
	
	ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport);
	
	class Default implements ASICTransactionReportDTCCASICBR103501 {
	
		@Override
		public ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport) {
			ComparisonResult result = executeDataRule(aSICTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ASICTransactionReport aSICTransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(aSICTransactionReport).<String>map("getActionType", _aSICTransactionReport -> _aSICTransactionReport.getActionType()), CardinalityOperator.Any).getOrDefault(false)) {
					if (exists(MapperS.of(aSICTransactionReport).<BigDecimal>map("getNotionalLeg2", _aSICTransactionReport -> _aSICTransactionReport.getNotionalLeg2())).getOrDefault(false)) {
						return exists(MapperS.of(aSICTransactionReport).<ISOCurrencyCodeEnum>map("getNotionalCurrencyLeg2", _aSICTransactionReport -> _aSICTransactionReport.getNotionalCurrencyLeg2())).and(notEqual(MapperC.<ISOCurrencyCodeEnum>of(MapperS.of(ISOCurrencyCodeEnum.XAG), MapperS.of(ISOCurrencyCodeEnum.XAU), MapperS.of(ISOCurrencyCodeEnum.XBA), MapperS.of(ISOCurrencyCodeEnum.XBB), MapperS.of(ISOCurrencyCodeEnum.XBC), MapperS.of(ISOCurrencyCodeEnum.XBD), MapperS.of(ISOCurrencyCodeEnum.XDR), MapperS.of(ISOCurrencyCodeEnum.XPD), MapperS.of(ISOCurrencyCodeEnum.XPT), MapperS.of(ISOCurrencyCodeEnum.XXX)), MapperS.of(aSICTransactionReport).<ISOCurrencyCodeEnum>map("getNotionalCurrencyLeg2", _aSICTransactionReport -> _aSICTransactionReport.getNotionalCurrencyLeg2()), CardinalityOperator.Any)).and(areEqual(MapperS.of(aSICTransactionReport).<ISOCurrencyCodeEnum>map("getNotionalCurrencyLeg2", _aSICTransactionReport -> _aSICTransactionReport.getNotionalCurrencyLeg2()), MapperS.of(aSICTransactionReport).<ISOCurrencyCodeEnum>map("getCallCurrency", _aSICTransactionReport -> _aSICTransactionReport.getCallCurrency()), CardinalityOperator.All).or(areEqual(MapperS.of(aSICTransactionReport).<ISOCurrencyCodeEnum>map("getNotionalCurrencyLeg2", _aSICTransactionReport -> _aSICTransactionReport.getNotionalCurrencyLeg2()), MapperS.of(aSICTransactionReport).<ISOCurrencyCodeEnum>map("getPutCurrency", _aSICTransactionReport -> _aSICTransactionReport.getPutCurrency()), CardinalityOperator.All)));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ASICTransactionReportDTCCASICBR103501 {
	
		@Override
		public ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION);
		}
	}
}
