package drr.regulation.asic.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.asic.rewrite.trade.ASICTransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ASICTransactionReportDTCC_ASIC_BR_1070_02")
@ImplementedBy(ASICTransactionReportDTCCASICBR107002.Default.class)
public interface ASICTransactionReportDTCCASICBR107002 extends Validator<ASICTransactionReport> {
	
	String NAME = "ASICTransactionReportDTCC_ASIC_BR_1070_02";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType then if [\"YEAR\", \"MNTH\", \"WEEK\", \"DAIL\", \"EXPI\"] any = paymentFrequencyPeriodLeg1 then paymentFrequencyPeriodMultiplierLeg1 exists and (paymentFrequencyPeriodMultiplierLeg1 < 1000 and paymentFrequencyPeriodMultiplierLeg1 > 0) else if paymentFrequencyPeriodLeg1 = \"EXPI\" then paymentFrequencyPeriodMultiplierLeg1 = 1 else if paymentFrequencyPeriodLeg1 is absent or paymentFrequencyPeriodLeg1 = \"ADHO\" then paymentFrequencyPeriodMultiplierLeg1 is absent";
	
	ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport);
	
	class Default implements ASICTransactionReportDTCCASICBR107002 {
	
		@Override
		public ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport) {
			ComparisonResult result = executeDataRule(aSICTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ASICTransactionReport aSICTransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(aSICTransactionReport).<String>map("getActionType", _aSICTransactionReport -> _aSICTransactionReport.getActionType()), CardinalityOperator.Any).getOrDefault(false)) {
					if (areEqual(MapperC.<String>of(MapperS.of("YEAR"), MapperS.of("MNTH"), MapperS.of("WEEK"), MapperS.of("DAIL"), MapperS.of("EXPI")), MapperS.of(aSICTransactionReport).<String>map("getPaymentFrequencyPeriodLeg1", _aSICTransactionReport -> _aSICTransactionReport.getPaymentFrequencyPeriodLeg1()), CardinalityOperator.Any).getOrDefault(false)) {
						return exists(MapperS.of(aSICTransactionReport).<Integer>map("getPaymentFrequencyPeriodMultiplierLeg1", _aSICTransactionReport -> _aSICTransactionReport.getPaymentFrequencyPeriodMultiplierLeg1())).and(lessThan(MapperS.of(aSICTransactionReport).<Integer>map("getPaymentFrequencyPeriodMultiplierLeg1", _aSICTransactionReport -> _aSICTransactionReport.getPaymentFrequencyPeriodMultiplierLeg1()), MapperS.of(1000), CardinalityOperator.All).and(greaterThan(MapperS.of(aSICTransactionReport).<Integer>map("getPaymentFrequencyPeriodMultiplierLeg1", _aSICTransactionReport -> _aSICTransactionReport.getPaymentFrequencyPeriodMultiplierLeg1()), MapperS.of(0), CardinalityOperator.All)));
					}
					if (areEqual(MapperS.of(aSICTransactionReport).<String>map("getPaymentFrequencyPeriodLeg1", _aSICTransactionReport -> _aSICTransactionReport.getPaymentFrequencyPeriodLeg1()), MapperS.of("EXPI"), CardinalityOperator.All).getOrDefault(false)) {
						return areEqual(MapperS.of(aSICTransactionReport).<Integer>map("getPaymentFrequencyPeriodMultiplierLeg1", _aSICTransactionReport -> _aSICTransactionReport.getPaymentFrequencyPeriodMultiplierLeg1()), MapperS.of(1), CardinalityOperator.All);
					}
					if (notExists(MapperS.of(aSICTransactionReport).<String>map("getPaymentFrequencyPeriodLeg1", _aSICTransactionReport -> _aSICTransactionReport.getPaymentFrequencyPeriodLeg1())).or(areEqual(MapperS.of(aSICTransactionReport).<String>map("getPaymentFrequencyPeriodLeg1", _aSICTransactionReport -> _aSICTransactionReport.getPaymentFrequencyPeriodLeg1()), MapperS.of("ADHO"), CardinalityOperator.All)).getOrDefault(false)) {
						return notExists(MapperS.of(aSICTransactionReport).<Integer>map("getPaymentFrequencyPeriodMultiplierLeg1", _aSICTransactionReport -> _aSICTransactionReport.getPaymentFrequencyPeriodMultiplierLeg1()));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ASICTransactionReportDTCCASICBR107002 {
	
		@Override
		public ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION);
		}
	}
}
