package drr.regulation.asic.rewrite.trade.validation.datarule;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.asic.rewrite.trade.ASICTransactionReport;
import drr.regulation.asic.rewrite.trade.PackageTransactionSpread;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ASICTransactionReportDTCC_ASIC_BR_1098_01")
@ImplementedBy(ASICTransactionReportDTCCASICBR109801.Default.class)
public interface ASICTransactionReportDTCCASICBR109801 extends Validator<ASICTransactionReport> {
	
	String NAME = "ASICTransactionReportDTCC_ASIC_BR_1098_01";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType then if packageTransactionSpread->packageTransactionSpreadNotation = \"1\" then packageTransactionSpread->packageTransactionSpreadCurrency exists and [ISOCurrencyCodeEnum->XAG, ISOCurrencyCodeEnum->XAU, ISOCurrencyCodeEnum->XBA, ISOCurrencyCodeEnum->XBB, ISOCurrencyCodeEnum->XBC, ISOCurrencyCodeEnum->XBD, ISOCurrencyCodeEnum->XDR, ISOCurrencyCodeEnum->XPD, ISOCurrencyCodeEnum->XPT, ISOCurrencyCodeEnum->XXX] any <> packageTransactionSpread->packageTransactionSpreadCurrency else if packageTransactionSpread->packageTransactionSpreadNotation = \"3\" or packageTransactionSpread->packageTransactionSpreadNotation = \"4\" then packageTransactionSpread->packageTransactionSpreadCurrency is absent";
	
	ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport);
	
	class Default implements ASICTransactionReportDTCCASICBR109801 {
	
		@Override
		public ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport) {
			ComparisonResult result = executeDataRule(aSICTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ASICTransactionReport aSICTransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(aSICTransactionReport).<String>map("getActionType", _aSICTransactionReport -> _aSICTransactionReport.getActionType()), CardinalityOperator.Any).getOrDefault(false)) {
					if (areEqual(MapperS.of(aSICTransactionReport).<PackageTransactionSpread>map("getPackageTransactionSpread", _aSICTransactionReport -> _aSICTransactionReport.getPackageTransactionSpread()).<String>map("getPackageTransactionSpreadNotation", packageTransactionSpread -> packageTransactionSpread.getPackageTransactionSpreadNotation()), MapperS.of("1"), CardinalityOperator.All).getOrDefault(false)) {
						return exists(MapperS.of(aSICTransactionReport).<PackageTransactionSpread>map("getPackageTransactionSpread", _aSICTransactionReport -> _aSICTransactionReport.getPackageTransactionSpread()).<ISOCurrencyCodeEnum>map("getPackageTransactionSpreadCurrency", packageTransactionSpread -> packageTransactionSpread.getPackageTransactionSpreadCurrency())).and(notEqual(MapperC.<ISOCurrencyCodeEnum>of(MapperS.of(ISOCurrencyCodeEnum.XAG), MapperS.of(ISOCurrencyCodeEnum.XAU), MapperS.of(ISOCurrencyCodeEnum.XBA), MapperS.of(ISOCurrencyCodeEnum.XBB), MapperS.of(ISOCurrencyCodeEnum.XBC), MapperS.of(ISOCurrencyCodeEnum.XBD), MapperS.of(ISOCurrencyCodeEnum.XDR), MapperS.of(ISOCurrencyCodeEnum.XPD), MapperS.of(ISOCurrencyCodeEnum.XPT), MapperS.of(ISOCurrencyCodeEnum.XXX)), MapperS.of(aSICTransactionReport).<PackageTransactionSpread>map("getPackageTransactionSpread", _aSICTransactionReport -> _aSICTransactionReport.getPackageTransactionSpread()).<ISOCurrencyCodeEnum>map("getPackageTransactionSpreadCurrency", packageTransactionSpread -> packageTransactionSpread.getPackageTransactionSpreadCurrency()), CardinalityOperator.Any));
					}
					if (areEqual(MapperS.of(aSICTransactionReport).<PackageTransactionSpread>map("getPackageTransactionSpread", _aSICTransactionReport -> _aSICTransactionReport.getPackageTransactionSpread()).<String>map("getPackageTransactionSpreadNotation", packageTransactionSpread -> packageTransactionSpread.getPackageTransactionSpreadNotation()), MapperS.of("3"), CardinalityOperator.All).or(areEqual(MapperS.of(aSICTransactionReport).<PackageTransactionSpread>map("getPackageTransactionSpread", _aSICTransactionReport -> _aSICTransactionReport.getPackageTransactionSpread()).<String>map("getPackageTransactionSpreadNotation", packageTransactionSpread -> packageTransactionSpread.getPackageTransactionSpreadNotation()), MapperS.of("4"), CardinalityOperator.All)).getOrDefault(false)) {
						return notExists(MapperS.of(aSICTransactionReport).<PackageTransactionSpread>map("getPackageTransactionSpread", _aSICTransactionReport -> _aSICTransactionReport.getPackageTransactionSpread()).<ISOCurrencyCodeEnum>map("getPackageTransactionSpreadCurrency", packageTransactionSpread -> packageTransactionSpread.getPackageTransactionSpreadCurrency()));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ASICTransactionReportDTCCASICBR109801 {
	
		@Override
		public ValidationResult<ASICTransactionReport> validate(RosettaPath path, ASICTransactionReport aSICTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ASICTransactionReport", path, DEFINITION);
		}
	}
}
