package drr.regulation.asic.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.asic.rewrite.trade.SpreadLeg1;
import java.util.Arrays;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("SpreadLeg1MonetaryOrPercentageOrBasis")
@ImplementedBy(SpreadLeg1MonetaryOrPercentageOrBasis.Default.class)
public interface SpreadLeg1MonetaryOrPercentageOrBasis extends Validator<SpreadLeg1> {
	
	String NAME = "SpreadLeg1MonetaryOrPercentageOrBasis";
	String DEFINITION = "optional choice spreadLeg1Monetary, spreadLeg1Percentage, spreadLeg1Basis";
	
	ValidationResult<SpreadLeg1> validate(RosettaPath path, SpreadLeg1 spreadLeg1);
	
	class Default implements SpreadLeg1MonetaryOrPercentageOrBasis {
	
		@Override
		public ValidationResult<SpreadLeg1> validate(RosettaPath path, SpreadLeg1 spreadLeg1) {
			ComparisonResult result = executeDataRule(spreadLeg1);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "SpreadLeg1", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "SpreadLeg1", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(SpreadLeg1 spreadLeg1) {
			try {
				return choice(MapperS.of(spreadLeg1), Arrays.asList("spreadLeg1Monetary", "spreadLeg1Percentage", "spreadLeg1Basis"), ChoiceRuleValidationMethod.OPTIONAL);
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements SpreadLeg1MonetaryOrPercentageOrBasis {
	
		@Override
		public ValidationResult<SpreadLeg1> validate(RosettaPath path, SpreadLeg1 spreadLeg1) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "SpreadLeg1", path, DEFINITION);
		}
	}
}
