package drr.regulation.asic.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.asic.rewrite.trade.SpreadLeg2;
import java.util.Arrays;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("SpreadLeg2MonetaryOrPercentageOrBasis")
@ImplementedBy(SpreadLeg2MonetaryOrPercentageOrBasis.Default.class)
public interface SpreadLeg2MonetaryOrPercentageOrBasis extends Validator<SpreadLeg2> {
	
	String NAME = "SpreadLeg2MonetaryOrPercentageOrBasis";
	String DEFINITION = "optional choice spreadLeg2Monetary, spreadLeg2Percentage, spreadLeg2Basis";
	
	ValidationResult<SpreadLeg2> validate(RosettaPath path, SpreadLeg2 spreadLeg2);
	
	class Default implements SpreadLeg2MonetaryOrPercentageOrBasis {
	
		@Override
		public ValidationResult<SpreadLeg2> validate(RosettaPath path, SpreadLeg2 spreadLeg2) {
			ComparisonResult result = executeDataRule(spreadLeg2);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "SpreadLeg2", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "SpreadLeg2", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(SpreadLeg2 spreadLeg2) {
			try {
				return choice(MapperS.of(spreadLeg2), Arrays.asList("spreadLeg2Monetary", "spreadLeg2Percentage", "spreadLeg2Basis"), ChoiceRuleValidationMethod.OPTIONAL);
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements SpreadLeg2MonetaryOrPercentageOrBasis {
	
		@Override
		public ValidationResult<SpreadLeg2> validate(RosettaPath path, SpreadLeg2 spreadLeg2) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "SpreadLeg2", path, DEFINITION);
		}
	}
}
