package drr.regulation.cftc.rewrite.functions;

import cdm.base.staticdata.party.Counterparty;
import cdm.base.staticdata.party.CounterpartyRoleEnum;
import cdm.base.staticdata.party.Party;
import cdm.base.staticdata.party.Party.PartyBuilder;
import cdm.base.staticdata.party.PayerReceiver;
import cdm.base.staticdata.party.functions.ExtractCounterpartyByRole;
import cdm.base.staticdata.party.metafields.ReferenceWithMetaParty;
import cdm.product.asset.CreditDefaultPayout;
import cdm.product.qualification.functions.Qualify_InterestRate_CapFloor;
import cdm.product.qualification.functions.Qualify_InterestRate_Option_Swaption;
import cdm.product.template.ContractualProduct;
import cdm.product.template.EconomicTerms;
import cdm.product.template.FixedPricePayout;
import cdm.product.template.Payout;
import cdm.product.template.TradableProduct;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.common.ReportableEvent;
import drr.regulation.common.functions.CommodityLeg1;
import drr.regulation.common.functions.FXLeg1;
import drr.regulation.common.functions.FixedPriceLeg1;
import drr.regulation.common.functions.InterestRateLeg1;
import drr.regulation.common.functions.IsCommodityCommodity;
import drr.regulation.common.functions.IsCommoditySwapProduct;
import drr.regulation.common.functions.IsFXForward;
import drr.regulation.common.functions.ProductForEvent;
import drr.regulation.common.functions.ProductForTrade;
import drr.regulation.common.functions.TradeForEvent;
import java.util.Optional;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(ReceiverPartyLeg1.ReceiverPartyLeg1Default.class)
public abstract class ReceiverPartyLeg1 implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected CommodityLeg1 commodityLeg1;
	@Inject protected ExtractCounterpartyByRole extractCounterpartyByRole;
	@Inject protected FXLeg1 fXLeg1;
	@Inject protected FixedPriceLeg1 fixedPriceLeg1;
	@Inject protected InterestRateLeg1 interestRateLeg1;
	@Inject protected IsCommodityCommodity isCommodityCommodity;
	@Inject protected IsCommoditySwapProduct isCommoditySwapProduct;
	@Inject protected IsFXForward isFXForward;
	@Inject protected ProductForEvent productForEvent;
	@Inject protected ProductForTrade productForTrade;
	@Inject protected Qualify_InterestRate_CapFloor qualify_InterestRate_CapFloor;
	@Inject protected Qualify_InterestRate_Option_Swaption qualify_InterestRate_Option_Swaption;
	@Inject protected TradeForEvent tradeForEvent;

	/**
	* @param reportableEvent 
	* @return party 
	*/
	public Party evaluate(ReportableEvent reportableEvent) {
		Party.PartyBuilder partyBuilder = doEvaluate(reportableEvent);
		
		final Party party;
		if (partyBuilder == null) {
			party = null;
		} else {
			party = partyBuilder.build();
			objectValidator.validate(Party.class, party);
		}
		
		return party;
	}

	protected abstract Party.PartyBuilder doEvaluate(ReportableEvent reportableEvent);

	protected abstract MapperS<CounterpartyRoleEnum> counterpartyRole(ReportableEvent reportableEvent);

	public static class ReceiverPartyLeg1Default extends ReceiverPartyLeg1 {
		@Override
		protected Party.PartyBuilder doEvaluate(ReportableEvent reportableEvent) {
			Party.PartyBuilder party = Party.builder();
			return assignOutput(party, reportableEvent);
		}
		
		protected Party.PartyBuilder assignOutput(Party.PartyBuilder party, ReportableEvent reportableEvent) {
			party = toBuilder(MapperS.of(extractCounterpartyByRole.evaluate(MapperS.of(tradeForEvent.evaluate(reportableEvent)).<TradableProduct>map("getTradableProduct", trade -> trade.getTradableProduct()).<Counterparty>mapC("getCounterparty", tradableProduct -> tradableProduct.getCounterparty()).getMulti(), counterpartyRole(reportableEvent).get())).<ReferenceWithMetaParty>map("getPartyReference", counterparty -> counterparty.getPartyReference()).<Party>map("getValue", _f->_f.getValue()).get());
			
			return Optional.ofNullable(party)
				.map(o -> o.prune())
				.orElse(null);
		}
		
		@Override
		protected MapperS<CounterpartyRoleEnum> counterpartyRole(ReportableEvent reportableEvent) {
			if (ComparisonResult.of(MapperS.of(isCommoditySwapProduct.evaluate(productForTrade.evaluate(tradeForEvent.evaluate(reportableEvent))))).and(ComparisonResult.of(MapperS.of(isCommodityCommodity.evaluate(productForTrade.evaluate(tradeForEvent.evaluate(reportableEvent)))))).getOrDefault(false)) {
				return MapperS.of(commodityLeg1.evaluate(productForTrade.evaluate(tradeForEvent.evaluate(reportableEvent)))).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getReceiver", payerReceiver -> payerReceiver.getReceiver());
			}
			if (ComparisonResult.of(MapperS.of(isCommoditySwapProduct.evaluate(productForTrade.evaluate(tradeForEvent.evaluate(reportableEvent))))).and(exists(MapperS.of(productForTrade.evaluate(tradeForEvent.evaluate(reportableEvent))).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<FixedPricePayout>mapC("getFixedPricePayout", payout -> payout.getFixedPricePayout()))).getOrDefault(false)) {
				return MapperS.of(fixedPriceLeg1.evaluate(productForTrade.evaluate(tradeForEvent.evaluate(reportableEvent)))).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getReceiver", payerReceiver -> payerReceiver.getReceiver());
			}
			final Boolean _boolean = isFXForward.evaluate(productForTrade.evaluate(tradeForEvent.evaluate(reportableEvent)));
			if ((_boolean == null ? false : _boolean)) {
				return MapperS.of(fXLeg1.evaluate(productForEvent.evaluate(reportableEvent))).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getReceiver", payerReceiver -> payerReceiver.getReceiver());
			}
			if (areEqual(MapperS.of(qualify_InterestRate_Option_Swaption.evaluate(MapperS.of(productForTrade.evaluate(tradeForEvent.evaluate(reportableEvent))).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get())), MapperS.of(false), CardinalityOperator.All).and(areEqual(MapperS.of(qualify_InterestRate_CapFloor.evaluate(MapperS.of(productForTrade.evaluate(tradeForEvent.evaluate(reportableEvent))).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get())), MapperS.of(false), CardinalityOperator.All)).and(notExists(MapperS.of(productForTrade.evaluate(tradeForEvent.evaluate(reportableEvent))).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<CreditDefaultPayout>map("getCreditDefaultPayout", payout -> payout.getCreditDefaultPayout()))).getOrDefault(false)) {
				return MapperS.of(interestRateLeg1.evaluate(productForTrade.evaluate(tradeForEvent.evaluate(reportableEvent)))).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getReceiver", payerReceiver -> payerReceiver.getReceiver());
			}
			return MapperS.<CounterpartyRoleEnum>ofNull();
		}
	}
}
