package drr.regulation.cftc.rewrite.meta;

import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyFunctionFactory;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorFactory;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import drr.regulation.cftc.rewrite.CFTCPart43TransactionReport;
import drr.regulation.cftc.rewrite.validation.CFTCPart43TransactionReportTypeFormatValidator;
import drr.regulation.cftc.rewrite.validation.CFTCPart43TransactionReportValidator;
import drr.regulation.cftc.rewrite.validation.exists.CFTCPart43TransactionReportOnlyExistsValidator;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Function;


/**
 * @version 4.2.1
 */
@RosettaMeta(model=CFTCPart43TransactionReport.class)
public class CFTCPart43TransactionReportMeta implements RosettaMetaData<CFTCPart43TransactionReport> {

	@Override
	public List<Validator<? super CFTCPart43TransactionReport>> dataRules(ValidatorFactory factory) {
		return Arrays.asList(
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportCounterparty1Format.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportCounterparty2Format.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportOtherPartyPaymentPayerIdentifierFormat.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportOtherPartyPaymentReceiverIdentifierFormat.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportEventTypeCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportAmendmentIndicatorCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportEventTimestampCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportNotionalAmountCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportNotionalCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportNotionalAmountScheduleCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportCallAmountCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportCallCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportPutAmountCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportPutCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportNotionalQuantityCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportQuantityFrequencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportQuantityFrequencyMultiplierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportQuantityUnitOfMeasureCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportTotalNotionalQuantityCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportPackageTransactionPriceCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportPackageTransactionPriceCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportPackageTransactionSpreadCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportPackageTransactionSpreadCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportDayCountConventionCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportFloatingRateResetFrequencyPeriodCond.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportFloatingRateResetFrequencyPeriodMultiplierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportOtherPaymentTypeCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportPaymentFrequencyPeriodCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportPaymentFrequencyPeriodMultiplierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportExchangeRateCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportExchangeRateBasisCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportFixedRateCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportPriceCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportPriceCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportPriceNotationCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportPriceUnitOfMesureCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportSpreadCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportSpreadCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportSpreadNotationCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportStrikePriceCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportStrikePriceCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportStrikePriceNotationCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportOptionPremiumAmountCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportOptionPremiumCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportFirstExerciseDateCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportIndexFactorCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportSettlementCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportNonStandardizedTermIndicatorCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportExpirationDateCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportReportingTimestampCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportPlatformIdentifierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportPrimeBrokerageTransactionIndicatorCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportUniqueSwapIdentifierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart43TransactionReportUniqueTransactionIdentifierCondition.class)
		);
	}
	
	@Override
	public List<Function<? super CFTCPart43TransactionReport, QualifyResult>> getQualifyFunctions(QualifyFunctionFactory factory) {
		return Collections.emptyList();
	}

	@Override
	public Validator<? super CFTCPart43TransactionReport> validator() {
		return new CFTCPart43TransactionReportValidator();
	}

	@Override
	public Validator<? super CFTCPart43TransactionReport> typeFormatValidator() {
		return new CFTCPart43TransactionReportTypeFormatValidator();
	}
	
	@Override
	public ValidatorWithArg<? super CFTCPart43TransactionReport, Set<String>> onlyExistsValidator() {
		return new CFTCPart43TransactionReportOnlyExistsValidator();
	}
}
