package drr.regulation.cftc.rewrite.meta;

import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyFunctionFactory;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorFactory;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import drr.regulation.cftc.rewrite.CFTCPart45TransactionReport;
import drr.regulation.cftc.rewrite.validation.CFTCPart45TransactionReportTypeFormatValidator;
import drr.regulation.cftc.rewrite.validation.CFTCPart45TransactionReportValidator;
import drr.regulation.cftc.rewrite.validation.exists.CFTCPart45TransactionReportOnlyExistsValidator;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Function;


/**
 * @version 4.2.1
 */
@RosettaMeta(model=CFTCPart45TransactionReport.class)
public class CFTCPart45TransactionReportMeta implements RosettaMetaData<CFTCPart45TransactionReport> {

	@Override
	public List<Validator<? super CFTCPart45TransactionReport>> dataRules(ValidatorFactory factory) {
		return Arrays.asList(
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportCounterparty1.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportCounterparty2.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportBuyerIdentifier.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportSellerIdentifier.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPayerIdentifierLeg1.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPayerIdentifierLeg2.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportReceiverIdentifierLeg1.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportReceiverIdentifierLeg2.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportOtherPartyPaymentPayerIdentifierFormat.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportOtherPartyPaymentReceiverIdentifierFormat.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportCentralCounterpartyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportClearingAcountOriginCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportClearerMemberCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportClearingSwapUSICondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportClearingSwapUTICondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportOriginalSwapUSICondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportOriginalSwapUTICondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportOriginalSwapSDRidentifierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportClearingReceiptTimestampCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportClearingExceptionsAndExemptions1Condition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportClearingExceptionsAndExemptions2Condition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportCounterparty1FinancialEntityIndicatorCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportBuyerIdentifierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportSellerIdentifierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPayerIdentifierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportReceiverIdentifierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportEventTypeCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportAmendmentIndicatorCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportEventIdentifierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportEventTimestampCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportNotionalAmountCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportNotionalCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportNotionalAmountScheduleCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportCallAmountCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportCallCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPutAmountCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPutCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportNotionalQuantityCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportQuantityFrequencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportQuantityFrequencyMultiplierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportQuantityUnitOfMeasureCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportTotalNotionalQuantityCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPackageIdentifierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPackageTransactionPriceCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPackageTransactionPriceCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPackageTransactionSpreadCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPackageTransactionSpreadCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportDayCountConventionCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportFixingDateCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportFloatingRateResetFrequencyPeriodCond.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportFloatingRateResetFrequencyPeriodMultiplierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportOtherPaymentTypeCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPaymentFrequencyPeriodCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPaymentFrequencyPeriodMultiplierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportExchangeRateCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportExchangeRateBasisCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportFixedRateCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPriceCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPriceCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPriceNotationCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPriceUnitOfMesureCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportSpreadCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportSpreadCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportSpreadNotationCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportStrikePriceCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportStrikePriceCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportStrikePriceNotationCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportOptionPremiumAmountCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportOptionPremiumCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportOptionPremiumPaymentDateCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportFirstExerciseDateCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportCDSIndexAttachmentPointCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportCDSIndexDetachmentPointCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportIndexFactorCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportFinalContractualSettlementDateCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportSettlementCurrencyCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportNonStandardizedTermIndicatorCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportExpirationDateCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportReportingTimestampCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPlatformIdentifierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPrimeBrokerageTransactionIndicatorCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPriorUSICondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportPriorUTICondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportUniqueSwapIdentifierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportUniqueTransactionIdentifierCondition.class),
			factory.create(drr.regulation.cftc.rewrite.validation.datarule.CFTCPart45TransactionReportNewSDRIdentifierCondition.class)
		);
	}
	
	@Override
	public List<Function<? super CFTCPart45TransactionReport, QualifyResult>> getQualifyFunctions(QualifyFunctionFactory factory) {
		return Collections.emptyList();
	}

	@Override
	public Validator<? super CFTCPart45TransactionReport> validator() {
		return new CFTCPart45TransactionReportValidator();
	}

	@Override
	public Validator<? super CFTCPart45TransactionReport> typeFormatValidator() {
		return new CFTCPart45TransactionReportTypeFormatValidator();
	}
	
	@Override
	public ValidatorWithArg<? super CFTCPart45TransactionReport, Set<String>> onlyExistsValidator() {
		return new CFTCPart45TransactionReportOnlyExistsValidator();
	}
}
