package drr.regulation.cftc.rewrite.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.cftc.rewrite.CFTCGuidebookPart45;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class CFTCGuidebookPart45TypeTabulator implements Tabulator<CFTCGuidebookPart45> {
	private final Field optionTypeField;
	private final Field optionStyleField;
	private final Field assetClassField;
	private final Field mandatoryClearingIndicatorField;
	private final Field maturityDateOfTheUnderlierField;
	private final Field largeNotionalOffFacilitySwapElectionIndicatorField;
	private final Field sefOrDcmIndicatorField;
	private final Field sefOrDcmAnonymousExecutionIndicatorField;
	private final Field messageTypeField;
	private final Field settlementTypeField;
	private final Field sdMspIndicatorParty1Field;
	private final Field sdMspIndicatorParty2Field;
	private final Field physicalCommodityIndicatorField;
	
	public CFTCGuidebookPart45TypeTabulator() {
		this.optionTypeField = new FieldImpl(
			"optionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "tr"), "OptionType")),
			Optional.of("TR Option type"),
			Arrays.asList()
		);
		this.optionStyleField = new FieldImpl(
			"optionStyle",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "tr"), "OptionStyle")),
			Optional.of("TR Option Style"),
			Arrays.asList()
		);
		this.assetClassField = new FieldImpl(
			"assetClass",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "tr"), "AssetClass")),
			Optional.of("TR Asset Class"),
			Arrays.asList()
		);
		this.mandatoryClearingIndicatorField = new FieldImpl(
			"mandatoryClearingIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "tr"), "MandatoryClearingIndicator")),
			Optional.of("TR Mandatory Clearing Indicator"),
			Arrays.asList()
		);
		this.maturityDateOfTheUnderlierField = new FieldImpl(
			"maturityDateOfTheUnderlier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "tr"), "MaturityDateOfTheUnderlier")),
			Optional.of("TR Maturity Date of the Underlier"),
			Arrays.asList()
		);
		this.largeNotionalOffFacilitySwapElectionIndicatorField = new FieldImpl(
			"largeNotionalOffFacilitySwapElectionIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "tr"), "LargeNotionalOffFacilitySwapElectionIndicator")),
			Optional.of("TR Large notional off-facility swap election indicator"),
			Arrays.asList()
		);
		this.sefOrDcmIndicatorField = new FieldImpl(
			"sefOrDcmIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "tr"), "SefOrDcmIndicator")),
			Optional.of("TR SEF or DCM indicator"),
			Arrays.asList()
		);
		this.sefOrDcmAnonymousExecutionIndicatorField = new FieldImpl(
			"sefOrDcmAnonymousExecutionIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "tr"), "SefOrDcmAnonymousExecutionIndicator")),
			Optional.of("TR SEF or DCM Anonymous Execution Indicator"),
			Arrays.asList()
		);
		this.messageTypeField = new FieldImpl(
			"messageType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "tr"), "MessageTypeTradeState")),
			Optional.of("TR Message Type"),
			Arrays.asList()
		);
		this.settlementTypeField = new FieldImpl(
			"settlementType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "tr"), "SettlementType")),
			Optional.of("TR Settlement Type"),
			Arrays.asList()
		);
		this.sdMspIndicatorParty1Field = new FieldImpl(
			"sdMspIndicatorParty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "tr"), "SdMspIndicatorParty1")),
			Optional.of("TR SD MSP Indicator - Counterparty 1"),
			Arrays.asList()
		);
		this.sdMspIndicatorParty2Field = new FieldImpl(
			"sdMspIndicatorParty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "tr"), "SdMspIndicatorParty2")),
			Optional.of("TR SD MSP Indicator - Counterparty 2"),
			Arrays.asList()
		);
		this.physicalCommodityIndicatorField = new FieldImpl(
			"physicalCommodityIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "tr"), "PhysicalCommodityIndicator")),
			Optional.of("TR Physical commodity contract indicator"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(optionTypeField, optionStyleField, assetClassField, mandatoryClearingIndicatorField, maturityDateOfTheUnderlierField, largeNotionalOffFacilitySwapElectionIndicatorField, sefOrDcmIndicatorField, sefOrDcmAnonymousExecutionIndicatorField, messageTypeField, settlementTypeField, sdMspIndicatorParty1Field, sdMspIndicatorParty2Field, physicalCommodityIndicatorField);
	}
	
	@Override
	public List<FieldValue> tabulate(CFTCGuidebookPart45 input) {
		Optional<String> optionType = Optional.ofNullable(input.getOptionType());
		Optional<String> optionStyle = Optional.ofNullable(input.getOptionStyle());
		Optional<String> assetClass = Optional.ofNullable(input.getAssetClass());
		Optional<Boolean> mandatoryClearingIndicator = Optional.ofNullable(input.getMandatoryClearingIndicator());
		Optional<Date> maturityDateOfTheUnderlier = Optional.ofNullable(input.getMaturityDateOfTheUnderlier());
		Optional<Boolean> largeNotionalOffFacilitySwapElectionIndicator = Optional.ofNullable(input.getLargeNotionalOffFacilitySwapElectionIndicator());
		Optional<Boolean> sefOrDcmIndicator = Optional.ofNullable(input.getSefOrDcmIndicator());
		Optional<Boolean> sefOrDcmAnonymousExecutionIndicator = Optional.ofNullable(input.getSefOrDcmAnonymousExecutionIndicator());
		Optional<String> messageType = Optional.ofNullable(input.getMessageType());
		Optional<String> settlementType = Optional.ofNullable(input.getSettlementType());
		Optional<Boolean> sdMspIndicatorParty1 = Optional.ofNullable(input.getSdMspIndicatorParty1());
		Optional<Boolean> sdMspIndicatorParty2 = Optional.ofNullable(input.getSdMspIndicatorParty2());
		Optional<Boolean> physicalCommodityIndicator = Optional.ofNullable(input.getPhysicalCommodityIndicator());
		return Arrays.asList(
			new FieldValueImpl(optionTypeField, optionType),
			new FieldValueImpl(optionStyleField, optionStyle),
			new FieldValueImpl(assetClassField, assetClass),
			new FieldValueImpl(mandatoryClearingIndicatorField, mandatoryClearingIndicator),
			new FieldValueImpl(maturityDateOfTheUnderlierField, maturityDateOfTheUnderlier),
			new FieldValueImpl(largeNotionalOffFacilitySwapElectionIndicatorField, largeNotionalOffFacilitySwapElectionIndicator),
			new FieldValueImpl(sefOrDcmIndicatorField, sefOrDcmIndicator),
			new FieldValueImpl(sefOrDcmAnonymousExecutionIndicatorField, sefOrDcmAnonymousExecutionIndicator),
			new FieldValueImpl(messageTypeField, messageType),
			new FieldValueImpl(settlementTypeField, settlementType),
			new FieldValueImpl(sdMspIndicatorParty1Field, sdMspIndicatorParty1),
			new FieldValueImpl(sdMspIndicatorParty2Field, sdMspIndicatorParty2),
			new FieldValueImpl(physicalCommodityIndicatorField, physicalCommodityIndicator)
		);
	}
}
