package drr.regulation.cftc.rewrite.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.cftc.rewrite.CFTCNonReportablePart43;
import drr.regulation.common.PartyIdentifierFormatEnum;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class CFTCNonReportablePart43TypeTabulator implements Tabulator<CFTCNonReportablePart43> {
	private final Field counterparty1FormatField;
	private final Field counterparty2FormatField;
	
	public CFTCNonReportablePart43TypeTabulator() {
		this.counterparty1FormatField = new FieldImpl(
			"counterparty1Format",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "Counterparty1Format")),
			Optional.of("Counterparty1 Format (non-reportable)"),
			Arrays.asList()
		);
		this.counterparty2FormatField = new FieldImpl(
			"counterparty2Format",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "Counterparty2Format")),
			Optional.of("Counterparty2 Format (non-reportable)"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(counterparty1FormatField, counterparty2FormatField);
	}
	
	@Override
	public List<FieldValue> tabulate(CFTCNonReportablePart43 input) {
		Optional<PartyIdentifierFormatEnum> counterparty1Format = Optional.ofNullable(input.getCounterparty1Format());
		Optional<PartyIdentifierFormatEnum> counterparty2Format = Optional.ofNullable(input.getCounterparty2Format());
		return Arrays.asList(
			new FieldValueImpl(counterparty1FormatField, counterparty1Format),
			new FieldValueImpl(counterparty2FormatField, counterparty2Format)
		);
	}
}
