package drr.regulation.cftc.rewrite.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.cftc.rewrite.CFTCNonReportablePart45;
import drr.regulation.common.PartyIdentifierFormatEnum;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class CFTCNonReportablePart45TypeTabulator implements Tabulator<CFTCNonReportablePart45> {
	private final Field counterparty1FormatField;
	private final Field counterparty2FormatField;
	private final Field buyerIdentifierFormatField;
	private final Field sellerIdentifierFormatField;
	private final Field payerIdentifierLeg1FormatField;
	private final Field payerIdentifierLeg2FormatField;
	private final Field receiverIdentifierLeg1FormatField;
	private final Field receiverIdentifierLeg2FormatField;
	
	public CFTCNonReportablePart45TypeTabulator() {
		this.counterparty1FormatField = new FieldImpl(
			"counterparty1Format",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "Counterparty1Format")),
			Optional.of("Counterparty1 Format (non-reportable)"),
			Arrays.asList()
		);
		this.counterparty2FormatField = new FieldImpl(
			"counterparty2Format",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "Counterparty2Format")),
			Optional.of("Counterparty2 Format (non-reportable)"),
			Arrays.asList()
		);
		this.buyerIdentifierFormatField = new FieldImpl(
			"buyerIdentifierFormat",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "BuyerIdentifierFormat")),
			Optional.of("BuyerIdentifier Format (non-reportable)"),
			Arrays.asList()
		);
		this.sellerIdentifierFormatField = new FieldImpl(
			"sellerIdentifierFormat",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "SellerIdentifierFormat")),
			Optional.of("SellerIdentifier Format (non-reportable)"),
			Arrays.asList()
		);
		this.payerIdentifierLeg1FormatField = new FieldImpl(
			"payerIdentifierLeg1Format",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PayerLeg1IdentifierFormat")),
			Optional.of("PayerLeg1Identifier Format (non-reportable)"),
			Arrays.asList()
		);
		this.payerIdentifierLeg2FormatField = new FieldImpl(
			"payerIdentifierLeg2Format",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PayerLeg2IdentifierFormat")),
			Optional.of("PayerLeg2Identifier Format (non-reportable)"),
			Arrays.asList()
		);
		this.receiverIdentifierLeg1FormatField = new FieldImpl(
			"receiverIdentifierLeg1Format",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ReceiverLeg1IdentifierFormat")),
			Optional.of("ReceiverLeg1Identifier Format (non-reportable)"),
			Arrays.asList()
		);
		this.receiverIdentifierLeg2FormatField = new FieldImpl(
			"receiverIdentifierLeg2Format",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ReceiverLeg2IdentifierFormat")),
			Optional.of("ReceiverLeg2Identifier Format (non-reportable)"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(counterparty1FormatField, counterparty2FormatField, buyerIdentifierFormatField, sellerIdentifierFormatField, payerIdentifierLeg1FormatField, payerIdentifierLeg2FormatField, receiverIdentifierLeg1FormatField, receiverIdentifierLeg2FormatField);
	}
	
	@Override
	public List<FieldValue> tabulate(CFTCNonReportablePart45 input) {
		Optional<PartyIdentifierFormatEnum> counterparty1Format = Optional.ofNullable(input.getCounterparty1Format());
		Optional<PartyIdentifierFormatEnum> counterparty2Format = Optional.ofNullable(input.getCounterparty2Format());
		Optional<PartyIdentifierFormatEnum> buyerIdentifierFormat = Optional.ofNullable(input.getBuyerIdentifierFormat());
		Optional<PartyIdentifierFormatEnum> sellerIdentifierFormat = Optional.ofNullable(input.getSellerIdentifierFormat());
		Optional<PartyIdentifierFormatEnum> payerIdentifierLeg1Format = Optional.ofNullable(input.getPayerIdentifierLeg1Format());
		Optional<PartyIdentifierFormatEnum> payerIdentifierLeg2Format = Optional.ofNullable(input.getPayerIdentifierLeg2Format());
		Optional<PartyIdentifierFormatEnum> receiverIdentifierLeg1Format = Optional.ofNullable(input.getReceiverIdentifierLeg1Format());
		Optional<PartyIdentifierFormatEnum> receiverIdentifierLeg2Format = Optional.ofNullable(input.getReceiverIdentifierLeg2Format());
		return Arrays.asList(
			new FieldValueImpl(counterparty1FormatField, counterparty1Format),
			new FieldValueImpl(counterparty2FormatField, counterparty2Format),
			new FieldValueImpl(buyerIdentifierFormatField, buyerIdentifierFormat),
			new FieldValueImpl(sellerIdentifierFormatField, sellerIdentifierFormat),
			new FieldValueImpl(payerIdentifierLeg1FormatField, payerIdentifierLeg1Format),
			new FieldValueImpl(payerIdentifierLeg2FormatField, payerIdentifierLeg2Format),
			new FieldValueImpl(receiverIdentifierLeg1FormatField, receiverIdentifierLeg1Format),
			new FieldValueImpl(receiverIdentifierLeg2FormatField, receiverIdentifierLeg2Format)
		);
	}
}
