package drr.regulation.cftc.rewrite.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaReport;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.cftc.rewrite.CFTCPart45TransactionReport;
import drr.regulation.cftc.rewrite.CFTCPart45TransactionReport.CFTCPart45TransactionReportBuilder;
import drr.regulation.cftc.tr.reports.AssetClassRule;
import drr.regulation.cftc.tr.reports.LargeNotionalOffFacilitySwapElectionIndicatorRule;
import drr.regulation.cftc.tr.reports.MandatoryClearingIndicatorRule;
import drr.regulation.cftc.tr.reports.MaturityDateOfTheUnderlierRule;
import drr.regulation.cftc.tr.reports.MessageTypeTradeStateRule;
import drr.regulation.cftc.tr.reports.OptionStyleRule;
import drr.regulation.cftc.tr.reports.OptionTypeRule;
import drr.regulation.cftc.tr.reports.PhysicalCommodityIndicatorRule;
import drr.regulation.cftc.tr.reports.SdMspIndicatorParty1Rule;
import drr.regulation.cftc.tr.reports.SdMspIndicatorParty2Rule;
import drr.regulation.cftc.tr.reports.SefOrDcmAnonymousExecutionIndicatorRule;
import drr.regulation.cftc.tr.reports.SefOrDcmIndicatorRule;
import drr.regulation.cftc.tr.reports.SettlementTypeRule;
import drr.regulation.common.TransactionReportInstruction;
import java.util.Optional;
import javax.inject.Inject;


@RosettaReport(namespace="drr.regulation.cftc.rewrite", body="CFTC", corpusList={"Part45"})
@ImplementedBy(CFTCPart45ReportFunction.CFTCPart45ReportFunctionDefault.class)
public abstract class CFTCPart45ReportFunction implements ReportFunction<TransactionReportInstruction, CFTCPart45TransactionReport> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ActionTypeRule actionType;
	@Inject protected AllocationIndicatorRule allocationIndicator;
	@Inject protected AmendmentIndicatorRule amendmentIndicator;
	@Inject protected AssetClassRule assetClass;
	@Inject protected BlockTradeElectionIndicatorRule blockTradeElectionIndicator;
	@Inject protected BuyerIdentifierRule buyerIdentifier;
	@Inject protected BuyerIdentifierFormatRule buyerIdentifierFormat;
	@Inject protected CDSIndexAttachmentPointRule cDSIndexAttachmentPoint;
	@Inject protected CDSIndexDetachmentPointRule cDSIndexDetachmentPoint;
	@Inject protected CallAmountRule callAmount;
	@Inject protected CallCurrencyRule callCurrency;
	@Inject protected CentralCounterpartyRule centralCounterparty;
	@Inject protected ClearedRule cleared;
	@Inject protected ClearingAccountOriginRule clearingAccountOrigin;
	@Inject protected ClearingExceptionsAndExemptionsCounterparty1Rule clearingExceptionsAndExemptionsCounterparty1;
	@Inject protected ClearingExceptionsAndExemptionsCounterparty2Rule clearingExceptionsAndExemptionsCounterparty2;
	@Inject protected ClearingMemberRule clearingMember;
	@Inject protected ClearingReceiptTimestampRule clearingReceiptTimestamp;
	@Inject protected ClearingSwapUSIsRule clearingSwapUSIs;
	@Inject protected ClearingSwapUTIsRule clearingSwapUTIs;
	@Inject protected Counterparty1Rule counterparty1;
	@Inject protected Counterparty1FederalEntityIndicatorRule counterparty1FederalEntityIndicator;
	@Inject protected Counterparty1FinancialEntityIndicatorRule counterparty1FinancialEntityIndicator;
	@Inject protected Counterparty1FormatRule counterparty1Format;
	@Inject protected Counterparty2Rule counterparty2;
	@Inject protected Counterparty2FederalEntityIndicatorRule counterparty2FederalEntityIndicator;
	@Inject protected Counterparty2FinancialEntityIndicatorRule counterparty2FinancialEntityIndicator;
	@Inject protected Counterparty2FormatRule counterparty2Format;
	@Inject protected Counterparty2IdentifierSourceRule counterparty2IdentifierSource;
	@Inject protected CustomBasketIndicatorRule customBasketIndicator;
	@Inject protected EffectiveDateRule effectiveDate;
	@Inject protected EmbeddedOptionTypeRule embeddedOptionType;
	@Inject protected EventIdentifierRule eventIdentifier;
	@Inject protected EventTimestampRule eventTimestamp;
	@Inject protected EventTypeRule eventType;
	@Inject protected ExchangeRateBasisRule exchangeRateBasis;
	@Inject protected ExchangeRateLeg1Rule exchangeRateLeg1;
	@Inject protected ExecutionTimestampRule executionTimestamp;
	@Inject protected ExpirationDateRule expirationDate;
	@Inject protected FinalContractualSettlementDateRule finalContractualSettlementDate;
	@Inject protected FirstExerciseDateRule firstExerciseDate;
	@Inject protected FixedRateDayCountConventionLeg1Rule fixedRateDayCountConventionLeg1;
	@Inject protected FixedRateDayCountConventionLeg2Rule fixedRateDayCountConventionLeg2;
	@Inject protected FixedRateLeg1Rule fixedRateLeg1;
	@Inject protected FixedRateLeg2Rule fixedRateLeg2;
	@Inject protected FixedRatePaymentFrequencyPeriodLeg1Rule fixedRatePaymentFrequencyPeriodLeg1;
	@Inject protected FixedRatePaymentFrequencyPeriodLeg2Rule fixedRatePaymentFrequencyPeriodLeg2;
	@Inject protected FixedRatePaymentFrequencyPeriodMultiplierLeg1Rule fixedRatePaymentFrequencyPeriodMultiplierLeg1;
	@Inject protected FixedRatePaymentFrequencyPeriodMultiplierLeg2Rule fixedRatePaymentFrequencyPeriodMultiplierLeg2;
	@Inject protected FixingDateLeg1Rule fixingDateLeg1;
	@Inject protected FixingDateLeg2Rule fixingDateLeg2;
	@Inject protected FloatingRateDayCountConventionLeg1Rule floatingRateDayCountConventionLeg1;
	@Inject protected FloatingRateDayCountConventionLeg2Rule floatingRateDayCountConventionLeg2;
	@Inject protected FloatingRatePaymentFrequencyPeriodLeg1Rule floatingRatePaymentFrequencyPeriodLeg1;
	@Inject protected FloatingRatePaymentFrequencyPeriodLeg2Rule floatingRatePaymentFrequencyPeriodLeg2;
	@Inject protected FloatingRatePaymentFrequencyPeriodMultiplierLeg1Rule floatingRatePaymentFrequencyPeriodMultiplierLeg1;
	@Inject protected FloatingRatePaymentFrequencyPeriodMultiplierLeg2Rule floatingRatePaymentFrequencyPeriodMultiplierLeg2;
	@Inject protected FloatingRateResetFrequencyPeriodLeg1Rule floatingRateResetFrequencyPeriodLeg1;
	@Inject protected FloatingRateResetFrequencyPeriodLeg2Rule floatingRateResetFrequencyPeriodLeg2;
	@Inject protected FloatingRateResetFrequencyPeriodMultiplierLeg1Rule floatingRateResetFrequencyPeriodMultiplierLeg1;
	@Inject protected FloatingRateResetFrequencyPeriodMultiplierLeg2Rule floatingRateResetFrequencyPeriodMultiplierLeg2;
	@Inject protected IndexFactorRule indexFactor;
	@Inject protected InitialMarginCollateralPortfolioCodeRule initialMarginCollateralPortfolioCode;
	@Inject protected JurisdictionRule jurisdiction;
	@Inject protected LargeNotionalOffFacilitySwapElectionIndicatorRule largeNotionalOffFacilitySwapElectionIndicator;
	@Inject protected MandatoryClearingIndicatorRule mandatoryClearingIndicator;
	@Inject protected MaturityDateOfTheUnderlierRule maturityDateOfTheUnderlier;
	@Inject protected MessageTypeTradeStateRule messageTypeTradeState;
	@Inject protected NewSDRIdentifierRule newSDRIdentifier;
	@Inject protected NonStandardizedTermIndicatorRule nonStandardizedTermIndicator;
	@Inject protected NotionalAmountLeg1Rule notionalAmountLeg1;
	@Inject protected NotionalAmountLeg2Rule notionalAmountLeg2;
	@Inject protected NotionalAmountScheduleLeg1Rule notionalAmountScheduleLeg1;
	@Inject protected NotionalAmountScheduleLeg2Rule notionalAmountScheduleLeg2;
	@Inject protected NotionalCurrencyLeg1Rule notionalCurrencyLeg1;
	@Inject protected NotionalCurrencyLeg2Rule notionalCurrencyLeg2;
	@Inject protected NotionalQuantityLeg1Rule notionalQuantityLeg1;
	@Inject protected NotionalQuantityLeg2Rule notionalQuantityLeg2;
	@Inject protected OptionPremiumAmountRule optionPremiumAmount;
	@Inject protected OptionPremiumCurrencyRule optionPremiumCurrency;
	@Inject protected OptionPremiumPaymentDateRule optionPremiumPaymentDate;
	@Inject protected OptionStyleRule optionStyle;
	@Inject protected OptionTypeRule optionType;
	@Inject protected OriginalSwapSDRIdentifierRule originalSwapSDRIdentifier;
	@Inject protected OriginalSwapUSIRule originalSwapUSI;
	@Inject protected OriginalSwapUTIRule originalSwapUTI;
	@Inject protected OtherPaymentRule otherPayment;
	@Inject protected PackageIdentifierRule packageIdentifier;
	@Inject protected PackageIndicatorRule packageIndicator;
	@Inject protected PackageTransactionPriceRule packageTransactionPrice;
	@Inject protected PackageTransactionPriceCurrencyRule packageTransactionPriceCurrency;
	@Inject protected PackageTransactionPriceNotationRule packageTransactionPriceNotation;
	@Inject protected PackageTransactionSpreadRule packageTransactionSpread;
	@Inject protected PackageTransactionSpreadCurrencyRule packageTransactionSpreadCurrency;
	@Inject protected PackageTransactionSpreadNotationRule packageTransactionSpreadNotation;
	@Inject protected PayerIdentifierLeg1Rule payerIdentifierLeg1;
	@Inject protected PayerIdentifierLeg2Rule payerIdentifierLeg2;
	@Inject protected PayerLeg1IdentifierFormatRule payerLeg1IdentifierFormat;
	@Inject protected PayerLeg2IdentifierFormatRule payerLeg2IdentifierFormat;
	@Inject protected PhysicalCommodityIndicatorRule physicalCommodityIndicator;
	@Inject protected PlatformIdentifierRule platformIdentifier;
	@Inject protected PostPricedSwapIndicatorRule postPricedSwapIndicator;
	@Inject protected PriceRule price;
	@Inject protected PriceCurrencyRule priceCurrency;
	@Inject protected PriceNotationRule priceNotation;
	@Inject protected PriceUnitOfMeasureRule priceUnitOfMeasure;
	@Inject protected PrimeBrokerageTransactionIndicatorRule primeBrokerageTransactionIndicator;
	@Inject protected PriorUSIRule priorUSI;
	@Inject protected PriorUTIRule priorUTI;
	@Inject protected PutAmountRule putAmount;
	@Inject protected PutCurrencyRule putCurrency;
	@Inject protected QuantityFrequencyLeg1Rule quantityFrequencyLeg1;
	@Inject protected QuantityFrequencyLeg2Rule quantityFrequencyLeg2;
	@Inject protected QuantityFrequencyMultiplierLeg1Rule quantityFrequencyMultiplierLeg1;
	@Inject protected QuantityFrequencyMultiplierLeg2Rule quantityFrequencyMultiplierLeg2;
	@Inject protected QuantityUnitOfMeasureLeg1Rule quantityUnitOfMeasureLeg1;
	@Inject protected QuantityUnitOfMeasureLeg2Rule quantityUnitOfMeasureLeg2;
	@Inject protected ReceiverIdentifierLeg1Rule receiverIdentifierLeg1;
	@Inject protected ReceiverIdentifierLeg2Rule receiverIdentifierLeg2;
	@Inject protected ReceiverLeg1IdentifierFormatRule receiverLeg1IdentifierFormat;
	@Inject protected ReceiverLeg2IdentifierFormatRule receiverLeg2IdentifierFormat;
	@Inject protected ReportingTimestampRule reportingTimestamp;
	@Inject protected SdMspIndicatorParty1Rule sdMspIndicatorParty1;
	@Inject protected SdMspIndicatorParty2Rule sdMspIndicatorParty2;
	@Inject protected SefOrDcmAnonymousExecutionIndicatorRule sefOrDcmAnonymousExecutionIndicator;
	@Inject protected SefOrDcmIndicatorRule sefOrDcmIndicator;
	@Inject protected SellerIdentifierRule sellerIdentifier;
	@Inject protected SellerIdentifierFormatRule sellerIdentifierFormat;
	@Inject protected SettlementCurrencyLeg1Rule settlementCurrencyLeg1;
	@Inject protected SettlementCurrencyLeg2Rule settlementCurrencyLeg2;
	@Inject protected SettlementLocationLeg1Rule settlementLocationLeg1;
	@Inject protected SettlementLocationLeg2Rule settlementLocationLeg2;
	@Inject protected SettlementTypeRule settlementType;
	@Inject protected SpreadCurrencyLeg1Rule spreadCurrencyLeg1;
	@Inject protected SpreadCurrencyLeg2Rule spreadCurrencyLeg2;
	@Inject protected SpreadLeg1Rule spreadLeg1;
	@Inject protected SpreadLeg2Rule spreadLeg2;
	@Inject protected SpreadNotationLeg1Rule spreadNotationLeg1;
	@Inject protected SpreadNotationLeg2Rule spreadNotationLeg2;
	@Inject protected StrikePriceRule strikePrice;
	@Inject protected StrikePriceCurrencyRule strikePriceCurrency;
	@Inject protected StrikePriceNotationRule strikePriceNotation;
	@Inject protected SubmitterIdentifierRule submitterIdentifier;
	@Inject protected TotalNotionalQuantityLeg1Rule totalNotionalQuantityLeg1;
	@Inject protected TotalNotionalQuantityLeg2Rule totalNotionalQuantityLeg2;
	@Inject protected UniqueProductIdentifierRule uniqueProductIdentifier;
	@Inject protected UniqueSwapIdentifierRule uniqueSwapIdentifier;
	@Inject protected UniqueTransactionIdentifierRule uniqueTransactionIdentifier;
	@Inject protected VariationMarginCollateralPortfolioCodeRule variationMarginCollateralPortfolioCode;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public CFTCPart45TransactionReport evaluate(TransactionReportInstruction input) {
		CFTCPart45TransactionReport.CFTCPart45TransactionReportBuilder outputBuilder = doEvaluate(input);
		
		final CFTCPart45TransactionReport output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.build();
			objectValidator.validate(CFTCPart45TransactionReport.class, output);
		}
		
		return output;
	}

	protected abstract CFTCPart45TransactionReport.CFTCPart45TransactionReportBuilder doEvaluate(TransactionReportInstruction input);

	public static class CFTCPart45ReportFunctionDefault extends CFTCPart45ReportFunction {
		@Override
		protected CFTCPart45TransactionReport.CFTCPart45TransactionReportBuilder doEvaluate(TransactionReportInstruction input) {
			CFTCPart45TransactionReport.CFTCPart45TransactionReportBuilder output = CFTCPart45TransactionReport.builder();
			return assignOutput(output, input);
		}
		
		protected CFTCPart45TransactionReport.CFTCPart45TransactionReportBuilder assignOutput(CFTCPart45TransactionReport.CFTCPart45TransactionReportBuilder output, TransactionReportInstruction input) {
			output
				.setCleared(cleared.evaluate(input));
			
			output
				.setCentralCounterparty(centralCounterparty.evaluate(input));
			
			output
				.setClearingAccountOrigin(clearingAccountOrigin.evaluate(input));
			
			output
				.setClearingMember(clearingMember.evaluate(input));
			
			output
				.setClearingSwapUSIs(clearingSwapUSIs.evaluate(input));
			
			output
				.setClearingSwapUTIs(clearingSwapUTIs.evaluate(input));
			
			output
				.setOriginalSwapUSI(originalSwapUSI.evaluate(input));
			
			output
				.setOriginalSwapUTI(originalSwapUTI.evaluate(input));
			
			output
				.setOriginalSwapSDRIdentifier(originalSwapSDRIdentifier.evaluate(input));
			
			output
				.setClearingReceiptTimestamp(clearingReceiptTimestamp.evaluate(input));
			
			output
				.setClearingExceptionsAndExemptionsCounterparty1(clearingExceptionsAndExemptionsCounterparty1.evaluate(input));
			
			output
				.setClearingExceptionsAndExemptionsCounterparty2(clearingExceptionsAndExemptionsCounterparty2.evaluate(input));
			
			output
				.setCounterparty1(counterparty1.evaluate(input));
			
			output
				.setCounterparty2(counterparty2.evaluate(input));
			
			output
				.setCounterparty2IdentifierSource(counterparty2IdentifierSource.evaluate(input));
			
			output
				.setCounterparty1FinancialEntityIndicator(counterparty1FinancialEntityIndicator.evaluate(input));
			
			output
				.setCounterparty2FinancialEntityIndicator(counterparty2FinancialEntityIndicator.evaluate(input));
			
			output
				.setBuyerIdentifier(buyerIdentifier.evaluate(input));
			
			output
				.setSellerIdentifier(sellerIdentifier.evaluate(input));
			
			output
				.setPayerIdentifierLeg1(payerIdentifierLeg1.evaluate(input));
			
			output
				.setPayerIdentifierLeg2(payerIdentifierLeg2.evaluate(input));
			
			output
				.setReceiverIdentifierLeg1(receiverIdentifierLeg1.evaluate(input));
			
			output
				.setReceiverIdentifierLeg2(receiverIdentifierLeg2.evaluate(input));
			
			output
				.setSubmitterIdentifier(submitterIdentifier.evaluate(input));
			
			output
				.setCounterparty1FederalEntityIndicator(counterparty1FederalEntityIndicator.evaluate(input));
			
			output
				.setCounterparty2FederalEntityIndicator(counterparty2FederalEntityIndicator.evaluate(input));
			
			output
				.setCustomBasketIndicator(customBasketIndicator.evaluate(input));
			
			output
				.setActionType(actionType.evaluate(input));
			
			output
				.setEventType(eventType.evaluate(input));
			
			output
				.setAmendmentIndicator(amendmentIndicator.evaluate(input));
			
			output
				.setEventIdentifier(eventIdentifier.evaluate(input));
			
			output
				.setEventTimestamp(eventTimestamp.evaluate(input));
			
			output
				.setNotionalAmountLeg1(notionalAmountLeg1.evaluate(input));
			
			output
				.setNotionalAmountLeg2(notionalAmountLeg2.evaluate(input));
			
			output
				.setNotionalCurrencyLeg1(notionalCurrencyLeg1.evaluate(input));
			
			output
				.setNotionalCurrencyLeg2(notionalCurrencyLeg2.evaluate(input));
			
			output
				.setNotionalAmountScheduleLeg1(notionalAmountScheduleLeg1.evaluate(input));
			
			output
				.setNotionalAmountScheduleLeg2(notionalAmountScheduleLeg2.evaluate(input));
			
			output
				.setCallAmount(callAmount.evaluate(input));
			
			output
				.setCallCurrency(callCurrency.evaluate(input));
			
			output
				.setPutAmount(putAmount.evaluate(input));
			
			output
				.setPutCurrency(putCurrency.evaluate(input));
			
			output
				.setNotionalQuantityLeg1(notionalQuantityLeg1.evaluate(input));
			
			output
				.setNotionalQuantityLeg2(notionalQuantityLeg2.evaluate(input));
			
			output
				.setQuantityFrequencyLeg1(quantityFrequencyLeg1.evaluate(input));
			
			output
				.setQuantityFrequencyLeg2(quantityFrequencyLeg2.evaluate(input));
			
			output
				.setQuantityFrequencyMultiplierLeg1(quantityFrequencyMultiplierLeg1.evaluate(input));
			
			output
				.setQuantityFrequencyMultiplierLeg2(quantityFrequencyMultiplierLeg2.evaluate(input));
			
			output
				.setQuantityUnitOfMeasureLeg1(quantityUnitOfMeasureLeg1.evaluate(input));
			
			output
				.setQuantityUnitOfMeasureLeg2(quantityUnitOfMeasureLeg2.evaluate(input));
			
			output
				.setTotalNotionalQuantityLeg1(totalNotionalQuantityLeg1.evaluate(input));
			
			output
				.setTotalNotionalQuantityLeg2(totalNotionalQuantityLeg2.evaluate(input));
			
			output
				.setPackageIndicator(packageIndicator.evaluate(input));
			
			output
				.setPackageIdentifier(packageIdentifier.evaluate(input));
			
			output
				.setPackageTransactionPrice(packageTransactionPrice.evaluate(input));
			
			output
				.setPackageTransactionPriceCurrency(packageTransactionPriceCurrency.evaluate(input));
			
			output
				.setPackageTransactionPriceNotation(packageTransactionPriceNotation.evaluate(input));
			
			output
				.setPackageTransactionSpread(packageTransactionSpread.evaluate(input));
			
			output
				.setPackageTransactionSpreadCurrency(packageTransactionSpreadCurrency.evaluate(input));
			
			output
				.setPackageTransactionSpreadNotation(packageTransactionSpreadNotation.evaluate(input));
			
			output
				.setFixedRateDayCountConventionLeg1(fixedRateDayCountConventionLeg1.evaluate(input));
			
			output
				.setFixedRateDayCountConventionLeg2(fixedRateDayCountConventionLeg2.evaluate(input));
			
			output
				.setFloatingRateDayCountConventionLeg1(floatingRateDayCountConventionLeg1.evaluate(input));
			
			output
				.setFloatingRateDayCountConventionLeg2(floatingRateDayCountConventionLeg2.evaluate(input));
			
			output
				.setFixingDateLeg1(fixingDateLeg1.evaluate(input));
			
			output
				.setFixingDateLeg2(fixingDateLeg2.evaluate(input));
			
			output
				.setFloatingRateResetFrequencyPeriodLeg1(floatingRateResetFrequencyPeriodLeg1.evaluate(input));
			
			output
				.setFloatingRateResetFrequencyPeriodLeg2(floatingRateResetFrequencyPeriodLeg2.evaluate(input));
			
			output
				.setFloatingRateResetFrequencyPeriodMultiplierLeg1(floatingRateResetFrequencyPeriodMultiplierLeg1.evaluate(input));
			
			output
				.setFloatingRateResetFrequencyPeriodMultiplierLeg2(floatingRateResetFrequencyPeriodMultiplierLeg2.evaluate(input));
			
			output
				.setOtherPayment(otherPayment.evaluate(input));
			
			output
				.setFixedRatePaymentFrequencyPeriodLeg1(fixedRatePaymentFrequencyPeriodLeg1.evaluate(input));
			
			output
				.setFixedRatePaymentFrequencyPeriodLeg2(fixedRatePaymentFrequencyPeriodLeg2.evaluate(input));
			
			output
				.setFloatingRatePaymentFrequencyPeriodLeg1(floatingRatePaymentFrequencyPeriodLeg1.evaluate(input));
			
			output
				.setFloatingRatePaymentFrequencyPeriodLeg2(floatingRatePaymentFrequencyPeriodLeg2.evaluate(input));
			
			output
				.setFixedRatePaymentFrequencyPeriodMultiplierLeg1(fixedRatePaymentFrequencyPeriodMultiplierLeg1.evaluate(input));
			
			output
				.setFixedRatePaymentFrequencyPeriodMultiplierLeg2(fixedRatePaymentFrequencyPeriodMultiplierLeg2.evaluate(input));
			
			output
				.setFloatingRatePaymentFrequencyPeriodMultiplierLeg1(floatingRatePaymentFrequencyPeriodMultiplierLeg1.evaluate(input));
			
			output
				.setFloatingRatePaymentFrequencyPeriodMultiplierLeg2(floatingRatePaymentFrequencyPeriodMultiplierLeg2.evaluate(input));
			
			output
				.setExchangeRateLeg1(exchangeRateLeg1.evaluate(input));
			
			output
				.setExchangeRateBasis(exchangeRateBasis.evaluate(input));
			
			output
				.setFixedRateLeg1(fixedRateLeg1.evaluate(input));
			
			output
				.setFixedRateLeg2(fixedRateLeg2.evaluate(input));
			
			output
				.setPostPricedSwapIndicator(postPricedSwapIndicator.evaluate(input));
			
			output
				.setPrice(price.evaluate(input));
			
			output
				.setPriceCurrency(priceCurrency.evaluate(input));
			
			output
				.setPriceNotation(priceNotation.evaluate(input));
			
			output
				.setPriceUnitOfMeasure(priceUnitOfMeasure.evaluate(input));
			
			output
				.setSpreadLeg1(spreadLeg1.evaluate(input));
			
			output
				.setSpreadLeg2(spreadLeg2.evaluate(input));
			
			output
				.setSpreadCurrencyLeg1(spreadCurrencyLeg1.evaluate(input));
			
			output
				.setSpreadCurrencyLeg2(spreadCurrencyLeg2.evaluate(input));
			
			output
				.setSpreadNotationLeg1(spreadNotationLeg1.evaluate(input));
			
			output
				.setSpreadNotationLeg2(spreadNotationLeg2.evaluate(input));
			
			output
				.setStrikePrice(strikePrice.evaluate(input));
			
			output
				.setStrikePriceCurrency(strikePriceCurrency.evaluate(input));
			
			output
				.setStrikePriceNotation(strikePriceNotation.evaluate(input));
			
			output
				.setOptionPremiumAmount(optionPremiumAmount.evaluate(input));
			
			output
				.setOptionPremiumCurrency(optionPremiumCurrency.evaluate(input));
			
			output
				.setOptionPremiumPaymentDate(optionPremiumPaymentDate.evaluate(input));
			
			output
				.setFirstExerciseDate(firstExerciseDate.evaluate(input));
			
			output
				.setCDSIndexAttachmentPoint(cDSIndexAttachmentPoint.evaluate(input));
			
			output
				.setCDSIndexDetachmentPoint(cDSIndexDetachmentPoint.evaluate(input));
			
			output
				.setIndexFactor(indexFactor.evaluate(input));
			
			output
				.setEmbeddedOptionType(embeddedOptionType.evaluate(input));
			
			output
				.setUniqueProductIdentifier(uniqueProductIdentifier.evaluate(input));
			
			output
				.setFinalContractualSettlementDate(finalContractualSettlementDate.evaluate(input));
			
			output
				.setSettlementCurrencyLeg1(settlementCurrencyLeg1.evaluate(input));
			
			output
				.setSettlementCurrencyLeg2(settlementCurrencyLeg2.evaluate(input));
			
			output
				.setSettlementLocationLeg1(settlementLocationLeg1.evaluate(input));
			
			output
				.setSettlementLocationLeg2(settlementLocationLeg2.evaluate(input));
			
			output
				.setAllocationIndicator(allocationIndicator.evaluate(input));
			
			output
				.setNonStandardizedTermIndicator(nonStandardizedTermIndicator.evaluate(input));
			
			output
				.setBlockTradeElectionIndicator(blockTradeElectionIndicator.evaluate(input));
			
			output
				.setEffectiveDate(effectiveDate.evaluate(input));
			
			output
				.setExpirationDate(expirationDate.evaluate(input));
			
			output
				.setExecutionTimestamp(executionTimestamp.evaluate(input));
			
			output
				.setReportingTimestamp(reportingTimestamp.evaluate(input));
			
			output
				.setPlatformIdentifier(platformIdentifier.evaluate(input));
			
			output
				.setPrimeBrokerageTransactionIndicator(primeBrokerageTransactionIndicator.evaluate(input));
			
			output
				.setPriorUSI(priorUSI.evaluate(input));
			
			output
				.setPriorUTI(priorUTI.evaluate(input));
			
			output
				.setUniqueSwapIdentifier(uniqueSwapIdentifier.evaluate(input));
			
			output
				.setUniqueTransactionIdentifier(uniqueTransactionIdentifier.evaluate(input));
			
			output
				.setJurisdiction(jurisdiction.evaluate(input));
			
			output
				.setNewSDRIdentifier(newSDRIdentifier.evaluate(input));
			
			output
				.setInitialMarginCollateralPortfolioCode(initialMarginCollateralPortfolioCode.evaluate(input));
			
			output
				.setVariationMarginCollateralPortfolioCode(variationMarginCollateralPortfolioCode.evaluate(input));
			
			output
				.getOrCreateCftcGuidebook()
				.setOptionType(optionType.evaluate(input));
			
			output
				.getOrCreateCftcGuidebook()
				.setOptionStyle(optionStyle.evaluate(input));
			
			output
				.getOrCreateCftcGuidebook()
				.setAssetClass(assetClass.evaluate(input));
			
			output
				.getOrCreateCftcGuidebook()
				.setMandatoryClearingIndicator(mandatoryClearingIndicator.evaluate(input));
			
			output
				.getOrCreateCftcGuidebook()
				.setMaturityDateOfTheUnderlier(maturityDateOfTheUnderlier.evaluate(input));
			
			output
				.getOrCreateCftcGuidebook()
				.setLargeNotionalOffFacilitySwapElectionIndicator(largeNotionalOffFacilitySwapElectionIndicator.evaluate(input));
			
			output
				.getOrCreateCftcGuidebook()
				.setSefOrDcmIndicator(sefOrDcmIndicator.evaluate(input));
			
			output
				.getOrCreateCftcGuidebook()
				.setSefOrDcmAnonymousExecutionIndicator(sefOrDcmAnonymousExecutionIndicator.evaluate(input));
			
			output
				.getOrCreateCftcGuidebook()
				.setMessageType(messageTypeTradeState.evaluate(input));
			
			output
				.getOrCreateCftcGuidebook()
				.setSettlementType(settlementType.evaluate(input));
			
			output
				.getOrCreateCftcGuidebook()
				.setSdMspIndicatorParty1(sdMspIndicatorParty1.evaluate(input));
			
			output
				.getOrCreateCftcGuidebook()
				.setSdMspIndicatorParty2(sdMspIndicatorParty2.evaluate(input));
			
			output
				.getOrCreateCftcGuidebook()
				.setPhysicalCommodityIndicator(physicalCommodityIndicator.evaluate(input));
			
			output
				.getOrCreateNonReportable()
				.setCounterparty1Format(counterparty1Format.evaluate(input));
			
			output
				.getOrCreateNonReportable()
				.setCounterparty2Format(counterparty2Format.evaluate(input));
			
			output
				.getOrCreateNonReportable()
				.setBuyerIdentifierFormat(buyerIdentifierFormat.evaluate(input));
			
			output
				.getOrCreateNonReportable()
				.setSellerIdentifierFormat(sellerIdentifierFormat.evaluate(input));
			
			output
				.getOrCreateNonReportable()
				.setPayerIdentifierLeg1Format(payerLeg1IdentifierFormat.evaluate(input));
			
			output
				.getOrCreateNonReportable()
				.setPayerIdentifierLeg2Format(payerLeg2IdentifierFormat.evaluate(input));
			
			output
				.getOrCreateNonReportable()
				.setReceiverIdentifierLeg1Format(receiverLeg1IdentifierFormat.evaluate(input));
			
			output
				.getOrCreateNonReportable()
				.setReceiverIdentifierLeg2Format(receiverLeg2IdentifierFormat.evaluate(input));
			
			return Optional.ofNullable(output)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
