package drr.regulation.cftc.rewrite.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.model.lib.reports.Tabulator.MultiNestedFieldValueImpl;
import com.rosetta.model.lib.reports.Tabulator.NestedFieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.cftc.rewrite.CFTCPart45TransactionReport;
import iso20022.auth030.esma.InterestComputationMethod4Code;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.inject.Inject;


public class CFTCPart45TransactionReportTypeTabulator implements Tabulator<CFTCPart45TransactionReport> {
	private final Field clearedField;
	private final Field centralCounterpartyField;
	private final Field clearingAccountOriginField;
	private final Field clearingMemberField;
	private final Field clearingSwapUSIsField;
	private final Field clearingSwapUTIsField;
	private final Field originalSwapUSIField;
	private final Field originalSwapUTIField;
	private final Field originalSwapSDRIdentifierField;
	private final Field clearingReceiptTimestampField;
	private final Field clearingExceptionsAndExemptionsCounterparty1Field;
	private final Field clearingExceptionsAndExemptionsCounterparty2Field;
	private final Field counterparty1Field;
	private final Field counterparty2Field;
	private final Field counterparty2IdentifierSourceField;
	private final Field counterparty1FinancialEntityIndicatorField;
	private final Field counterparty2FinancialEntityIndicatorField;
	private final Field buyerIdentifierField;
	private final Field sellerIdentifierField;
	private final Field payerIdentifierLeg1Field;
	private final Field payerIdentifierLeg2Field;
	private final Field receiverIdentifierLeg1Field;
	private final Field receiverIdentifierLeg2Field;
	private final Field submitterIdentifierField;
	private final Field counterparty1FederalEntityIndicatorField;
	private final Field counterparty2FederalEntityIndicatorField;
	private final Field customBasketIndicatorField;
	private final Field actionTypeField;
	private final Field eventTypeField;
	private final Field amendmentIndicatorField;
	private final Field eventIdentifierField;
	private final Field eventTimestampField;
	private final Field notionalAmountLeg1Field;
	private final Field notionalAmountLeg2Field;
	private final Field notionalCurrencyLeg1Field;
	private final Field notionalCurrencyLeg2Field;
	private final Field notionalAmountScheduleLeg1Field;
	private final Field notionalAmountScheduleLeg2Field;
	private final Field callAmountField;
	private final Field callCurrencyField;
	private final Field putAmountField;
	private final Field putCurrencyField;
	private final Field notionalQuantityLeg1Field;
	private final Field notionalQuantityLeg2Field;
	private final Field quantityFrequencyLeg1Field;
	private final Field quantityFrequencyLeg2Field;
	private final Field quantityFrequencyMultiplierLeg1Field;
	private final Field quantityFrequencyMultiplierLeg2Field;
	private final Field quantityUnitOfMeasureLeg1Field;
	private final Field quantityUnitOfMeasureLeg2Field;
	private final Field totalNotionalQuantityLeg1Field;
	private final Field totalNotionalQuantityLeg2Field;
	private final Field packageIndicatorField;
	private final Field packageIdentifierField;
	private final Field packageTransactionPriceField;
	private final Field packageTransactionPriceCurrencyField;
	private final Field packageTransactionPriceNotationField;
	private final Field packageTransactionSpreadField;
	private final Field packageTransactionSpreadCurrencyField;
	private final Field packageTransactionSpreadNotationField;
	private final Field fixedRateDayCountConventionLeg1Field;
	private final Field fixedRateDayCountConventionLeg2Field;
	private final Field floatingRateDayCountConventionLeg1Field;
	private final Field floatingRateDayCountConventionLeg2Field;
	private final Field fixingDateLeg1Field;
	private final Field fixingDateLeg2Field;
	private final Field floatingRateResetFrequencyPeriodLeg1Field;
	private final Field floatingRateResetFrequencyPeriodLeg2Field;
	private final Field floatingRateResetFrequencyPeriodMultiplierLeg1Field;
	private final Field floatingRateResetFrequencyPeriodMultiplierLeg2Field;
	private final Field otherPaymentField;
	private final Field fixedRatePaymentFrequencyPeriodLeg1Field;
	private final Field fixedRatePaymentFrequencyPeriodLeg2Field;
	private final Field floatingRatePaymentFrequencyPeriodLeg1Field;
	private final Field floatingRatePaymentFrequencyPeriodLeg2Field;
	private final Field fixedRatePaymentFrequencyPeriodMultiplierLeg1Field;
	private final Field fixedRatePaymentFrequencyPeriodMultiplierLeg2Field;
	private final Field floatingRatePaymentFrequencyPeriodMultiplierLeg1Field;
	private final Field floatingRatePaymentFrequencyPeriodMultiplierLeg2Field;
	private final Field exchangeRateLeg1Field;
	private final Field exchangeRateBasisField;
	private final Field fixedRateLeg1Field;
	private final Field fixedRateLeg2Field;
	private final Field postPricedSwapIndicatorField;
	private final Field priceField;
	private final Field priceCurrencyField;
	private final Field priceNotationField;
	private final Field priceUnitOfMeasureField;
	private final Field spreadLeg1Field;
	private final Field spreadLeg2Field;
	private final Field spreadCurrencyLeg1Field;
	private final Field spreadCurrencyLeg2Field;
	private final Field spreadNotationLeg1Field;
	private final Field spreadNotationLeg2Field;
	private final Field strikePriceField;
	private final Field strikePriceCurrencyField;
	private final Field strikePriceNotationField;
	private final Field optionPremiumAmountField;
	private final Field optionPremiumCurrencyField;
	private final Field optionPremiumPaymentDateField;
	private final Field firstExerciseDateField;
	private final Field cDSIndexAttachmentPointField;
	private final Field cDSIndexDetachmentPointField;
	private final Field indexFactorField;
	private final Field embeddedOptionTypeField;
	private final Field uniqueProductIdentifierField;
	private final Field finalContractualSettlementDateField;
	private final Field settlementCurrencyLeg1Field;
	private final Field settlementCurrencyLeg2Field;
	private final Field settlementLocationLeg1Field;
	private final Field settlementLocationLeg2Field;
	private final Field allocationIndicatorField;
	private final Field nonStandardizedTermIndicatorField;
	private final Field blockTradeElectionIndicatorField;
	private final Field effectiveDateField;
	private final Field expirationDateField;
	private final Field executionTimestampField;
	private final Field reportingTimestampField;
	private final Field platformIdentifierField;
	private final Field primeBrokerageTransactionIndicatorField;
	private final Field priorUSIField;
	private final Field priorUTIField;
	private final Field uniqueSwapIdentifierField;
	private final Field uniqueTransactionIdentifierField;
	private final Field jurisdictionField;
	private final Field newSDRIdentifierField;
	private final Field initialMarginCollateralPortfolioCodeField;
	private final Field variationMarginCollateralPortfolioCodeField;
	private final Field cftcGuidebookField;
	private final Field nonReportableField;
	
	private final ClearingSwapUSIsReportTypeTabulator clearingSwapUSIsReportTypeTabulator;
	private final ClearingSwapUTIsReportTypeTabulator clearingSwapUTIsReportTypeTabulator;
	private final ClearingExceptionsAndExemptionsCounterparty1ReportTypeTabulator clearingExceptionsAndExemptionsCounterparty1ReportTypeTabulator;
	private final ClearingExceptionsAndExemptionsCounterparty2ReportTypeTabulator clearingExceptionsAndExemptionsCounterparty2ReportTypeTabulator;
	private final NotionalAmountScheduleLeg1ReportTypeTabulator notionalAmountScheduleLeg1ReportTypeTabulator;
	private final NotionalAmountScheduleLeg2ReportTypeTabulator notionalAmountScheduleLeg2ReportTypeTabulator;
	private final OtherPaymentReportTypeTabulator otherPaymentReportTypeTabulator;
	private final CFTCGuidebookPart45TypeTabulator cFTCGuidebookPart45TypeTabulator;
	private final CFTCNonReportablePart45TypeTabulator cFTCNonReportablePart45TypeTabulator;
	
	@Inject
	public CFTCPart45TransactionReportTypeTabulator(ClearingSwapUSIsReportTypeTabulator clearingSwapUSIsReportTypeTabulator, ClearingSwapUTIsReportTypeTabulator clearingSwapUTIsReportTypeTabulator, ClearingExceptionsAndExemptionsCounterparty1ReportTypeTabulator clearingExceptionsAndExemptionsCounterparty1ReportTypeTabulator, ClearingExceptionsAndExemptionsCounterparty2ReportTypeTabulator clearingExceptionsAndExemptionsCounterparty2ReportTypeTabulator, NotionalAmountScheduleLeg1ReportTypeTabulator notionalAmountScheduleLeg1ReportTypeTabulator, NotionalAmountScheduleLeg2ReportTypeTabulator notionalAmountScheduleLeg2ReportTypeTabulator, OtherPaymentReportTypeTabulator otherPaymentReportTypeTabulator, CFTCGuidebookPart45TypeTabulator cFTCGuidebookPart45TypeTabulator, CFTCNonReportablePart45TypeTabulator cFTCNonReportablePart45TypeTabulator) {
		this.clearingSwapUSIsReportTypeTabulator = clearingSwapUSIsReportTypeTabulator;
		this.clearingSwapUTIsReportTypeTabulator = clearingSwapUTIsReportTypeTabulator;
		this.clearingExceptionsAndExemptionsCounterparty1ReportTypeTabulator = clearingExceptionsAndExemptionsCounterparty1ReportTypeTabulator;
		this.clearingExceptionsAndExemptionsCounterparty2ReportTypeTabulator = clearingExceptionsAndExemptionsCounterparty2ReportTypeTabulator;
		this.notionalAmountScheduleLeg1ReportTypeTabulator = notionalAmountScheduleLeg1ReportTypeTabulator;
		this.notionalAmountScheduleLeg2ReportTypeTabulator = notionalAmountScheduleLeg2ReportTypeTabulator;
		this.otherPaymentReportTypeTabulator = otherPaymentReportTypeTabulator;
		this.cFTCGuidebookPart45TypeTabulator = cFTCGuidebookPart45TypeTabulator;
		this.cFTCNonReportablePart45TypeTabulator = cFTCNonReportablePart45TypeTabulator;
		this.clearedField = new FieldImpl(
			"cleared",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "Cleared")),
			Optional.of("01 Cleared"),
			Arrays.asList()
		);
		this.centralCounterpartyField = new FieldImpl(
			"centralCounterparty",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "CentralCounterparty")),
			Optional.of("02 Central counterparty"),
			Arrays.asList()
		);
		this.clearingAccountOriginField = new FieldImpl(
			"clearingAccountOrigin",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ClearingAccountOrigin")),
			Optional.of("03 Clearing account origin"),
			Arrays.asList()
		);
		this.clearingMemberField = new FieldImpl(
			"clearingMember",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ClearingMember")),
			Optional.of("04 Clearing member"),
			Arrays.asList()
		);
		this.clearingSwapUSIsField = new FieldImpl(
			"clearingSwapUSIs",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ClearingSwapUSIs")),
			Optional.empty(),
			clearingSwapUSIsReportTypeTabulator.getFields()
		);
		this.clearingSwapUTIsField = new FieldImpl(
			"clearingSwapUTIs",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ClearingSwapUTIs")),
			Optional.empty(),
			clearingSwapUTIsReportTypeTabulator.getFields()
		);
		this.originalSwapUSIField = new FieldImpl(
			"originalSwapUSI",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OriginalSwapUSI")),
			Optional.of("07 Original swap USI"),
			Arrays.asList()
		);
		this.originalSwapUTIField = new FieldImpl(
			"originalSwapUTI",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OriginalSwapUTI")),
			Optional.of("08 Original swap UTI"),
			Arrays.asList()
		);
		this.originalSwapSDRIdentifierField = new FieldImpl(
			"originalSwapSDRIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OriginalSwapSDRIdentifier")),
			Optional.of("09 Original swap SDR identifier"),
			Arrays.asList()
		);
		this.clearingReceiptTimestampField = new FieldImpl(
			"clearingReceiptTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ClearingReceiptTimestamp")),
			Optional.of("10 Clearing receipt timestamp"),
			Arrays.asList()
		);
		this.clearingExceptionsAndExemptionsCounterparty1Field = new FieldImpl(
			"clearingExceptionsAndExemptionsCounterparty1",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ClearingExceptionsAndExemptionsCounterparty1")),
			Optional.empty(),
			clearingExceptionsAndExemptionsCounterparty1ReportTypeTabulator.getFields()
		);
		this.clearingExceptionsAndExemptionsCounterparty2Field = new FieldImpl(
			"clearingExceptionsAndExemptionsCounterparty2",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ClearingExceptionsAndExemptionsCounterparty2")),
			Optional.empty(),
			clearingExceptionsAndExemptionsCounterparty2ReportTypeTabulator.getFields()
		);
		this.counterparty1Field = new FieldImpl(
			"counterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "Counterparty1")),
			Optional.of("13 Counterparty 1"),
			Arrays.asList()
		);
		this.counterparty2Field = new FieldImpl(
			"counterparty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "Counterparty2")),
			Optional.of("14 Counterparty 2"),
			Arrays.asList()
		);
		this.counterparty2IdentifierSourceField = new FieldImpl(
			"counterparty2IdentifierSource",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "Counterparty2IdentifierSource")),
			Optional.of("15 Counterparty 2 identifier source"),
			Arrays.asList()
		);
		this.counterparty1FinancialEntityIndicatorField = new FieldImpl(
			"counterparty1FinancialEntityIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "Counterparty1FinancialEntityIndicator")),
			Optional.of("16 Counterparty 1 financial entity indicator"),
			Arrays.asList()
		);
		this.counterparty2FinancialEntityIndicatorField = new FieldImpl(
			"counterparty2FinancialEntityIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "Counterparty2FinancialEntityIndicator")),
			Optional.of("17 Counterparty 2 financial entity indicator"),
			Arrays.asList()
		);
		this.buyerIdentifierField = new FieldImpl(
			"buyerIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "BuyerIdentifier")),
			Optional.of("18 Buyer identifier"),
			Arrays.asList()
		);
		this.sellerIdentifierField = new FieldImpl(
			"sellerIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "SellerIdentifier")),
			Optional.of("19 Seller identifier"),
			Arrays.asList()
		);
		this.payerIdentifierLeg1Field = new FieldImpl(
			"payerIdentifierLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PayerIdentifierLeg1")),
			Optional.of("20 Payer identifier - Leg 1"),
			Arrays.asList()
		);
		this.payerIdentifierLeg2Field = new FieldImpl(
			"payerIdentifierLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PayerIdentifierLeg2")),
			Optional.of("20 Payer identifier - Leg 2"),
			Arrays.asList()
		);
		this.receiverIdentifierLeg1Field = new FieldImpl(
			"receiverIdentifierLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ReceiverIdentifierLeg1")),
			Optional.of("21 Receiver identifier - Leg 1"),
			Arrays.asList()
		);
		this.receiverIdentifierLeg2Field = new FieldImpl(
			"receiverIdentifierLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ReceiverIdentifierLeg2")),
			Optional.of("21 Receiver identifier - Leg 2"),
			Arrays.asList()
		);
		this.submitterIdentifierField = new FieldImpl(
			"submitterIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "SubmitterIdentifier")),
			Optional.of("22 Submitter identifier"),
			Arrays.asList()
		);
		this.counterparty1FederalEntityIndicatorField = new FieldImpl(
			"counterparty1FederalEntityIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "Counterparty1FederalEntityIndicator")),
			Optional.of("23 Counterparty 1 federal entity indicator"),
			Arrays.asList()
		);
		this.counterparty2FederalEntityIndicatorField = new FieldImpl(
			"counterparty2FederalEntityIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "Counterparty2FederalEntityIndicator")),
			Optional.of("24 Counterparty 2 federal entity indicator"),
			Arrays.asList()
		);
		this.customBasketIndicatorField = new FieldImpl(
			"customBasketIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "CustomBasketIndicator")),
			Optional.of("25 Custom basket indicator"),
			Arrays.asList()
		);
		this.actionTypeField = new FieldImpl(
			"actionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ActionType")),
			Optional.of("26 Action type"),
			Arrays.asList()
		);
		this.eventTypeField = new FieldImpl(
			"eventType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "EventType")),
			Optional.of("27 Event type"),
			Arrays.asList()
		);
		this.amendmentIndicatorField = new FieldImpl(
			"amendmentIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "AmendmentIndicator")),
			Optional.of("28 Amendment indicator"),
			Arrays.asList()
		);
		this.eventIdentifierField = new FieldImpl(
			"eventIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "EventIdentifier")),
			Optional.of("29 Event identifier"),
			Arrays.asList()
		);
		this.eventTimestampField = new FieldImpl(
			"eventTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "EventTimestamp")),
			Optional.of("30 Event timestamp"),
			Arrays.asList()
		);
		this.notionalAmountLeg1Field = new FieldImpl(
			"notionalAmountLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "NotionalAmountLeg1")),
			Optional.of("31 Notional amount-Leg 1"),
			Arrays.asList()
		);
		this.notionalAmountLeg2Field = new FieldImpl(
			"notionalAmountLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "NotionalAmountLeg2")),
			Optional.of("31 Notional amount-Leg 2"),
			Arrays.asList()
		);
		this.notionalCurrencyLeg1Field = new FieldImpl(
			"notionalCurrencyLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "NotionalCurrencyLeg1")),
			Optional.of("32 Notional currency-Leg 1"),
			Arrays.asList()
		);
		this.notionalCurrencyLeg2Field = new FieldImpl(
			"notionalCurrencyLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "NotionalCurrencyLeg2")),
			Optional.of("32 Notional currency-Leg 2"),
			Arrays.asList()
		);
		this.notionalAmountScheduleLeg1Field = new FieldImpl(
			"notionalAmountScheduleLeg1",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "NotionalAmountScheduleLeg1")),
			Optional.empty(),
			notionalAmountScheduleLeg1ReportTypeTabulator.getFields()
		);
		this.notionalAmountScheduleLeg2Field = new FieldImpl(
			"notionalAmountScheduleLeg2",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "NotionalAmountScheduleLeg2")),
			Optional.empty(),
			notionalAmountScheduleLeg2ReportTypeTabulator.getFields()
		);
		this.callAmountField = new FieldImpl(
			"callAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "CallAmount")),
			Optional.of("36 Call amount"),
			Arrays.asList()
		);
		this.callCurrencyField = new FieldImpl(
			"callCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "CallCurrency")),
			Optional.of("37 Call currency"),
			Arrays.asList()
		);
		this.putAmountField = new FieldImpl(
			"putAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PutAmount")),
			Optional.of("38 Put Amount"),
			Arrays.asList()
		);
		this.putCurrencyField = new FieldImpl(
			"putCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PutCurrency")),
			Optional.of("39 Put currency"),
			Arrays.asList()
		);
		this.notionalQuantityLeg1Field = new FieldImpl(
			"notionalQuantityLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "NotionalQuantityLeg1")),
			Optional.of("40 Notional quantity-Leg 1"),
			Arrays.asList()
		);
		this.notionalQuantityLeg2Field = new FieldImpl(
			"notionalQuantityLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "NotionalQuantityLeg2")),
			Optional.of("40 Notional quantity-Leg 2"),
			Arrays.asList()
		);
		this.quantityFrequencyLeg1Field = new FieldImpl(
			"quantityFrequencyLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "QuantityFrequencyLeg1")),
			Optional.of("41 Quantity frequency-Leg 1"),
			Arrays.asList()
		);
		this.quantityFrequencyLeg2Field = new FieldImpl(
			"quantityFrequencyLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "QuantityFrequencyLeg2")),
			Optional.of("41 Quantity frequency-Leg 2"),
			Arrays.asList()
		);
		this.quantityFrequencyMultiplierLeg1Field = new FieldImpl(
			"quantityFrequencyMultiplierLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "QuantityFrequencyMultiplierLeg1")),
			Optional.of("42 Quantity frequency multiplier-Leg 1"),
			Arrays.asList()
		);
		this.quantityFrequencyMultiplierLeg2Field = new FieldImpl(
			"quantityFrequencyMultiplierLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "QuantityFrequencyMultiplierLeg2")),
			Optional.of("42 Quantity frequency multiplier-Leg 2"),
			Arrays.asList()
		);
		this.quantityUnitOfMeasureLeg1Field = new FieldImpl(
			"quantityUnitOfMeasureLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "QuantityUnitOfMeasureLeg1")),
			Optional.of("43 Quantity unit of measure-Leg 1"),
			Arrays.asList()
		);
		this.quantityUnitOfMeasureLeg2Field = new FieldImpl(
			"quantityUnitOfMeasureLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "QuantityUnitOfMeasureLeg2")),
			Optional.of("43 Quantity unit of measure-Leg 2"),
			Arrays.asList()
		);
		this.totalNotionalQuantityLeg1Field = new FieldImpl(
			"totalNotionalQuantityLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "TotalNotionalQuantityLeg1")),
			Optional.of("44 Total notional quantity-Leg 1"),
			Arrays.asList()
		);
		this.totalNotionalQuantityLeg2Field = new FieldImpl(
			"totalNotionalQuantityLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "TotalNotionalQuantityLeg2")),
			Optional.of("44 Total notional quantity-Leg 2"),
			Arrays.asList()
		);
		this.packageIndicatorField = new FieldImpl(
			"packageIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PackageIndicator")),
			Optional.of("45 Package Indicator"),
			Arrays.asList()
		);
		this.packageIdentifierField = new FieldImpl(
			"packageIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PackageIdentifier")),
			Optional.of("46 Package Identifier"),
			Arrays.asList()
		);
		this.packageTransactionPriceField = new FieldImpl(
			"packageTransactionPrice",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PackageTransactionPrice")),
			Optional.of("47 Package Transaction Price"),
			Arrays.asList()
		);
		this.packageTransactionPriceCurrencyField = new FieldImpl(
			"packageTransactionPriceCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PackageTransactionPriceCurrency")),
			Optional.of("48 Package Transaction Price Currency"),
			Arrays.asList()
		);
		this.packageTransactionPriceNotationField = new FieldImpl(
			"packageTransactionPriceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PackageTransactionPriceNotation")),
			Optional.of("49 Package Transaction Price Notation"),
			Arrays.asList()
		);
		this.packageTransactionSpreadField = new FieldImpl(
			"packageTransactionSpread",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PackageTransactionSpread")),
			Optional.of("50 Package Transaction Spread"),
			Arrays.asList()
		);
		this.packageTransactionSpreadCurrencyField = new FieldImpl(
			"packageTransactionSpreadCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PackageTransactionSpreadCurrency")),
			Optional.of("51 Package Transaction Spread Currency"),
			Arrays.asList()
		);
		this.packageTransactionSpreadNotationField = new FieldImpl(
			"packageTransactionSpreadNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PackageTransactionSpreadNotation")),
			Optional.of("52 Package Transaction Spread Notation"),
			Arrays.asList()
		);
		this.fixedRateDayCountConventionLeg1Field = new FieldImpl(
			"fixedRateDayCountConventionLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FixedRateDayCountConventionLeg1")),
			Optional.of("53 Fixed rate day count convention-Leg 1"),
			Arrays.asList()
		);
		this.fixedRateDayCountConventionLeg2Field = new FieldImpl(
			"fixedRateDayCountConventionLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FixedRateDayCountConventionLeg2")),
			Optional.of("53 Fixed rate day count convention-Leg 2"),
			Arrays.asList()
		);
		this.floatingRateDayCountConventionLeg1Field = new FieldImpl(
			"floatingRateDayCountConventionLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FloatingRateDayCountConventionLeg1")),
			Optional.of("53 Floating rate day count convention-Leg 1"),
			Arrays.asList()
		);
		this.floatingRateDayCountConventionLeg2Field = new FieldImpl(
			"floatingRateDayCountConventionLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FloatingRateDayCountConventionLeg2")),
			Optional.of("53 Floating rate day count convention-Leg 2"),
			Arrays.asList()
		);
		this.fixingDateLeg1Field = new FieldImpl(
			"fixingDateLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FixingDateLeg1")),
			Optional.of("54 Fixing date-Leg 1"),
			Arrays.asList()
		);
		this.fixingDateLeg2Field = new FieldImpl(
			"fixingDateLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FixingDateLeg2")),
			Optional.of("54 Fixing date-Leg 2"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodLeg1Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FloatingRateResetFrequencyPeriodLeg1")),
			Optional.of("55 Floating rate reset frequency period-Leg 1"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodLeg2Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FloatingRateResetFrequencyPeriodLeg2")),
			Optional.of("55 Floating rate reset frequency period-Leg 2"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodMultiplierLeg1Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodMultiplierLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FloatingRateResetFrequencyPeriodMultiplierLeg1")),
			Optional.of("56 Floating rate reset frequency period multiplier-Leg 1"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodMultiplierLeg2Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodMultiplierLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FloatingRateResetFrequencyPeriodMultiplierLeg2")),
			Optional.of("56 Floating rate reset frequency period multiplier-Leg 2"),
			Arrays.asList()
		);
		this.otherPaymentField = new FieldImpl(
			"otherPayment",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OtherPayment")),
			Optional.empty(),
			otherPaymentReportTypeTabulator.getFields()
		);
		this.fixedRatePaymentFrequencyPeriodLeg1Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FixedRatePaymentFrequencyPeriodLeg1")),
			Optional.of("63 Fixed rate payment frequency period-Leg 1"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodLeg2Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FixedRatePaymentFrequencyPeriodLeg2")),
			Optional.of("63 Fixed rate payment frequency period-Leg 2"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodLeg1Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FloatingRatePaymentFrequencyPeriodLeg1")),
			Optional.of("63 Floating rate payment frequency period-Leg 1"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodLeg2Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FloatingRatePaymentFrequencyPeriodLeg2")),
			Optional.of("63 Floating rate payment frequency period-Leg 2"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodMultiplierLeg1Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodMultiplierLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FixedRatePaymentFrequencyPeriodMultiplierLeg1")),
			Optional.of("64 Fixed rate payment frequency period multiplier-Leg 1"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodMultiplierLeg2Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodMultiplierLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FixedRatePaymentFrequencyPeriodMultiplierLeg2")),
			Optional.of("64 Fixed rate payment frequency period multiplier-Leg 2"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodMultiplierLeg1Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodMultiplierLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FloatingRatePaymentFrequencyPeriodMultiplierLeg1")),
			Optional.of("64 Floating rate payment frequency period multiplier-Leg 1"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodMultiplierLeg2Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodMultiplierLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FloatingRatePaymentFrequencyPeriodMultiplierLeg2")),
			Optional.of("64 Floating rate payment frequency period multiplier-Leg 2"),
			Arrays.asList()
		);
		this.exchangeRateLeg1Field = new FieldImpl(
			"exchangeRateLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ExchangeRateLeg1")),
			Optional.of("65 Exchange rate-Leg 1"),
			Arrays.asList()
		);
		this.exchangeRateBasisField = new FieldImpl(
			"exchangeRateBasis",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ExchangeRateBasis")),
			Optional.of("66 Exchange rate basis"),
			Arrays.asList()
		);
		this.fixedRateLeg1Field = new FieldImpl(
			"fixedRateLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FixedRateLeg1")),
			Optional.of("67 Fixed rate-Leg 1"),
			Arrays.asList()
		);
		this.fixedRateLeg2Field = new FieldImpl(
			"fixedRateLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FixedRateLeg2")),
			Optional.of("67 Fixed rate-Leg 2"),
			Arrays.asList()
		);
		this.postPricedSwapIndicatorField = new FieldImpl(
			"postPricedSwapIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PostPricedSwapIndicator")),
			Optional.of("68 Post-priced swap indicator"),
			Arrays.asList()
		);
		this.priceField = new FieldImpl(
			"price",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "Price")),
			Optional.of("69 Price"),
			Arrays.asList()
		);
		this.priceCurrencyField = new FieldImpl(
			"priceCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PriceCurrency")),
			Optional.of("70 Price currency"),
			Arrays.asList()
		);
		this.priceNotationField = new FieldImpl(
			"priceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PriceNotation")),
			Optional.of("71 Price notation"),
			Arrays.asList()
		);
		this.priceUnitOfMeasureField = new FieldImpl(
			"priceUnitOfMeasure",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PriceUnitOfMeasure")),
			Optional.of("72 Price unit of measure"),
			Arrays.asList()
		);
		this.spreadLeg1Field = new FieldImpl(
			"spreadLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "SpreadLeg1")),
			Optional.of("73 Spread-Leg 1"),
			Arrays.asList()
		);
		this.spreadLeg2Field = new FieldImpl(
			"spreadLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "SpreadLeg2")),
			Optional.of("73 Spread-Leg 2"),
			Arrays.asList()
		);
		this.spreadCurrencyLeg1Field = new FieldImpl(
			"spreadCurrencyLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "SpreadCurrencyLeg1")),
			Optional.of("74 Spread currency-Leg 1"),
			Arrays.asList()
		);
		this.spreadCurrencyLeg2Field = new FieldImpl(
			"spreadCurrencyLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "SpreadCurrencyLeg2")),
			Optional.of("74 Spread currency-Leg 2"),
			Arrays.asList()
		);
		this.spreadNotationLeg1Field = new FieldImpl(
			"spreadNotationLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "SpreadNotationLeg1")),
			Optional.of("75 Spread notation-Leg 1"),
			Arrays.asList()
		);
		this.spreadNotationLeg2Field = new FieldImpl(
			"spreadNotationLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "SpreadNotationLeg2")),
			Optional.of("75 Spread notation-Leg 2"),
			Arrays.asList()
		);
		this.strikePriceField = new FieldImpl(
			"strikePrice",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "StrikePrice")),
			Optional.of("76 Strike Price"),
			Arrays.asList()
		);
		this.strikePriceCurrencyField = new FieldImpl(
			"strikePriceCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "StrikePriceCurrency")),
			Optional.of("77 Strike Price Currency"),
			Arrays.asList()
		);
		this.strikePriceNotationField = new FieldImpl(
			"strikePriceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "StrikePriceNotation")),
			Optional.of("78 Strike Price notation"),
			Arrays.asList()
		);
		this.optionPremiumAmountField = new FieldImpl(
			"optionPremiumAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OptionPremiumAmount")),
			Optional.of("79 Option Premium Amount"),
			Arrays.asList()
		);
		this.optionPremiumCurrencyField = new FieldImpl(
			"optionPremiumCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OptionPremiumCurrency")),
			Optional.of("80 Option Premium Currency"),
			Arrays.asList()
		);
		this.optionPremiumPaymentDateField = new FieldImpl(
			"optionPremiumPaymentDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OptionPremiumPaymentDate")),
			Optional.of("81 Option Premium Payment Date"),
			Arrays.asList()
		);
		this.firstExerciseDateField = new FieldImpl(
			"firstExerciseDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FirstExerciseDate")),
			Optional.of("82 First exercise date"),
			Arrays.asList()
		);
		this.cDSIndexAttachmentPointField = new FieldImpl(
			"cDSIndexAttachmentPoint",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "CDSIndexAttachmentPoint")),
			Optional.of("83 CDS Index Attachment Point"),
			Arrays.asList()
		);
		this.cDSIndexDetachmentPointField = new FieldImpl(
			"cDSIndexDetachmentPoint",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "CDSIndexDetachmentPoint")),
			Optional.of("84 CDS Index Detachment Point"),
			Arrays.asList()
		);
		this.indexFactorField = new FieldImpl(
			"indexFactor",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "IndexFactor")),
			Optional.of("85 Index factor"),
			Arrays.asList()
		);
		this.embeddedOptionTypeField = new FieldImpl(
			"embeddedOptionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "EmbeddedOptionType")),
			Optional.of("86 Embedded Option Type"),
			Arrays.asList()
		);
		this.uniqueProductIdentifierField = new FieldImpl(
			"uniqueProductIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "UniqueProductIdentifier")),
			Optional.of("87 Unique product identifier"),
			Arrays.asList()
		);
		this.finalContractualSettlementDateField = new FieldImpl(
			"finalContractualSettlementDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "FinalContractualSettlementDate")),
			Optional.of("88 Final Contractual Settlement Date"),
			Arrays.asList()
		);
		this.settlementCurrencyLeg1Field = new FieldImpl(
			"settlementCurrencyLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "SettlementCurrencyLeg1")),
			Optional.of("89 Settlement Currency Leg 1"),
			Arrays.asList()
		);
		this.settlementCurrencyLeg2Field = new FieldImpl(
			"settlementCurrencyLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "SettlementCurrencyLeg2")),
			Optional.of("89 Settlement Currency Leg 2"),
			Arrays.asList()
		);
		this.settlementLocationLeg1Field = new FieldImpl(
			"settlementLocationLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "SettlementLocationLeg1")),
			Optional.of("90 Settlement Location- Leg 1"),
			Arrays.asList()
		);
		this.settlementLocationLeg2Field = new FieldImpl(
			"settlementLocationLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "SettlementLocationLeg2")),
			Optional.of("90 Settlement Location- Leg 2"),
			Arrays.asList()
		);
		this.allocationIndicatorField = new FieldImpl(
			"allocationIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "AllocationIndicator")),
			Optional.of("91 Allocation Indicator"),
			Arrays.asList()
		);
		this.nonStandardizedTermIndicatorField = new FieldImpl(
			"nonStandardizedTermIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "NonStandardizedTermIndicator")),
			Optional.of("92 Non-standardized Term Indicator"),
			Arrays.asList()
		);
		this.blockTradeElectionIndicatorField = new FieldImpl(
			"blockTradeElectionIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "BlockTradeElectionIndicator")),
			Optional.of("93 Block Trade Election Indicator"),
			Arrays.asList()
		);
		this.effectiveDateField = new FieldImpl(
			"effectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "EffectiveDate")),
			Optional.of("94 Effective Date"),
			Arrays.asList()
		);
		this.expirationDateField = new FieldImpl(
			"expirationDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ExpirationDate")),
			Optional.of("95 Expiration date"),
			Arrays.asList()
		);
		this.executionTimestampField = new FieldImpl(
			"executionTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ExecutionTimestamp")),
			Optional.of("96 Execution Timestamp"),
			Arrays.asList()
		);
		this.reportingTimestampField = new FieldImpl(
			"reportingTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "ReportingTimestamp")),
			Optional.of("97 Reporting timestamp"),
			Arrays.asList()
		);
		this.platformIdentifierField = new FieldImpl(
			"platformIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PlatformIdentifier")),
			Optional.of("98 Platform Identifier"),
			Arrays.asList()
		);
		this.primeBrokerageTransactionIndicatorField = new FieldImpl(
			"primeBrokerageTransactionIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PrimeBrokerageTransactionIndicator")),
			Optional.of("99 Prime brokerage transaction indicator"),
			Arrays.asList()
		);
		this.priorUSIField = new FieldImpl(
			"priorUSI",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PriorUSI")),
			Optional.of("100 Prior USI"),
			Arrays.asList()
		);
		this.priorUTIField = new FieldImpl(
			"priorUTI",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "PriorUTI")),
			Optional.of("101 Prior UTI"),
			Arrays.asList()
		);
		this.uniqueSwapIdentifierField = new FieldImpl(
			"uniqueSwapIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "UniqueSwapIdentifier")),
			Optional.of("102 Unique Swap Identifier (USI)"),
			Arrays.asList()
		);
		this.uniqueTransactionIdentifierField = new FieldImpl(
			"uniqueTransactionIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "UniqueTransactionIdentifier")),
			Optional.of("103 Unique Transaction Identifier (UTI)"),
			Arrays.asList()
		);
		this.jurisdictionField = new FieldImpl(
			"jurisdiction",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "Jurisdiction")),
			Optional.of("104 Jurisdiction"),
			Arrays.asList()
		);
		this.newSDRIdentifierField = new FieldImpl(
			"newSDRIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "NewSDRIdentifier")),
			Optional.of("105 New SDR Identifier"),
			Arrays.asList()
		);
		this.initialMarginCollateralPortfolioCodeField = new FieldImpl(
			"initialMarginCollateralPortfolioCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "InitialMarginCollateralPortfolioCode")),
			Optional.of("116 Initial Margin Collateral Portfolio Code"),
			Arrays.asList()
		);
		this.variationMarginCollateralPortfolioCodeField = new FieldImpl(
			"variationMarginCollateralPortfolioCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "VariationMarginCollateralPortfolioCode")),
			Optional.of("124 Variation Margin Collateral Portfolio Code"),
			Arrays.asList()
		);
		this.cftcGuidebookField = new FieldImpl(
			"cftcGuidebook",
			false,
			Optional.empty(),
			Optional.empty(),
			cFTCGuidebookPart45TypeTabulator.getFields()
		);
		this.nonReportableField = new FieldImpl(
			"nonReportable",
			false,
			Optional.empty(),
			Optional.empty(),
			cFTCNonReportablePart45TypeTabulator.getFields()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(clearedField, centralCounterpartyField, clearingAccountOriginField, clearingMemberField, clearingSwapUSIsField, clearingSwapUTIsField, originalSwapUSIField, originalSwapUTIField, originalSwapSDRIdentifierField, clearingReceiptTimestampField, clearingExceptionsAndExemptionsCounterparty1Field, clearingExceptionsAndExemptionsCounterparty2Field, counterparty1Field, counterparty2Field, counterparty2IdentifierSourceField, counterparty1FinancialEntityIndicatorField, counterparty2FinancialEntityIndicatorField, buyerIdentifierField, sellerIdentifierField, payerIdentifierLeg1Field, payerIdentifierLeg2Field, receiverIdentifierLeg1Field, receiverIdentifierLeg2Field, submitterIdentifierField, counterparty1FederalEntityIndicatorField, counterparty2FederalEntityIndicatorField, customBasketIndicatorField, actionTypeField, eventTypeField, amendmentIndicatorField, eventIdentifierField, eventTimestampField, notionalAmountLeg1Field, notionalAmountLeg2Field, notionalCurrencyLeg1Field, notionalCurrencyLeg2Field, notionalAmountScheduleLeg1Field, notionalAmountScheduleLeg2Field, callAmountField, callCurrencyField, putAmountField, putCurrencyField, notionalQuantityLeg1Field, notionalQuantityLeg2Field, quantityFrequencyLeg1Field, quantityFrequencyLeg2Field, quantityFrequencyMultiplierLeg1Field, quantityFrequencyMultiplierLeg2Field, quantityUnitOfMeasureLeg1Field, quantityUnitOfMeasureLeg2Field, totalNotionalQuantityLeg1Field, totalNotionalQuantityLeg2Field, packageIndicatorField, packageIdentifierField, packageTransactionPriceField, packageTransactionPriceCurrencyField, packageTransactionPriceNotationField, packageTransactionSpreadField, packageTransactionSpreadCurrencyField, packageTransactionSpreadNotationField, fixedRateDayCountConventionLeg1Field, fixedRateDayCountConventionLeg2Field, floatingRateDayCountConventionLeg1Field, floatingRateDayCountConventionLeg2Field, fixingDateLeg1Field, fixingDateLeg2Field, floatingRateResetFrequencyPeriodLeg1Field, floatingRateResetFrequencyPeriodLeg2Field, floatingRateResetFrequencyPeriodMultiplierLeg1Field, floatingRateResetFrequencyPeriodMultiplierLeg2Field, otherPaymentField, fixedRatePaymentFrequencyPeriodLeg1Field, fixedRatePaymentFrequencyPeriodLeg2Field, floatingRatePaymentFrequencyPeriodLeg1Field, floatingRatePaymentFrequencyPeriodLeg2Field, fixedRatePaymentFrequencyPeriodMultiplierLeg1Field, fixedRatePaymentFrequencyPeriodMultiplierLeg2Field, floatingRatePaymentFrequencyPeriodMultiplierLeg1Field, floatingRatePaymentFrequencyPeriodMultiplierLeg2Field, exchangeRateLeg1Field, exchangeRateBasisField, fixedRateLeg1Field, fixedRateLeg2Field, postPricedSwapIndicatorField, priceField, priceCurrencyField, priceNotationField, priceUnitOfMeasureField, spreadLeg1Field, spreadLeg2Field, spreadCurrencyLeg1Field, spreadCurrencyLeg2Field, spreadNotationLeg1Field, spreadNotationLeg2Field, strikePriceField, strikePriceCurrencyField, strikePriceNotationField, optionPremiumAmountField, optionPremiumCurrencyField, optionPremiumPaymentDateField, firstExerciseDateField, cDSIndexAttachmentPointField, cDSIndexDetachmentPointField, indexFactorField, embeddedOptionTypeField, uniqueProductIdentifierField, finalContractualSettlementDateField, settlementCurrencyLeg1Field, settlementCurrencyLeg2Field, settlementLocationLeg1Field, settlementLocationLeg2Field, allocationIndicatorField, nonStandardizedTermIndicatorField, blockTradeElectionIndicatorField, effectiveDateField, expirationDateField, executionTimestampField, reportingTimestampField, platformIdentifierField, primeBrokerageTransactionIndicatorField, priorUSIField, priorUTIField, uniqueSwapIdentifierField, uniqueTransactionIdentifierField, jurisdictionField, newSDRIdentifierField, initialMarginCollateralPortfolioCodeField, variationMarginCollateralPortfolioCodeField, cftcGuidebookField, nonReportableField);
	}
	
	@Override
	public List<FieldValue> tabulate(CFTCPart45TransactionReport input) {
		Optional<String> cleared = Optional.ofNullable(input.getCleared());
		Optional<String> centralCounterparty = Optional.ofNullable(input.getCentralCounterparty());
		Optional<String> clearingAccountOrigin = Optional.ofNullable(input.getClearingAccountOrigin());
		Optional<String> clearingMember = Optional.ofNullable(input.getClearingMember());
		Optional<List<List<FieldValue>>> clearingSwapUSIs = Optional.ofNullable(input.getClearingSwapUSIs())
			.map(x -> x.stream()
				.map(_x -> clearingSwapUSIsReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<List<List<FieldValue>>> clearingSwapUTIs = Optional.ofNullable(input.getClearingSwapUTIs())
			.map(x -> x.stream()
				.map(_x -> clearingSwapUTIsReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<String> originalSwapUSI = Optional.ofNullable(input.getOriginalSwapUSI());
		Optional<String> originalSwapUTI = Optional.ofNullable(input.getOriginalSwapUTI());
		Optional<String> originalSwapSDRIdentifier = Optional.ofNullable(input.getOriginalSwapSDRIdentifier());
		Optional<ZonedDateTime> clearingReceiptTimestamp = Optional.ofNullable(input.getClearingReceiptTimestamp());
		Optional<List<List<FieldValue>>> clearingExceptionsAndExemptionsCounterparty1 = Optional.ofNullable(input.getClearingExceptionsAndExemptionsCounterparty1())
			.map(x -> x.stream()
				.map(_x -> clearingExceptionsAndExemptionsCounterparty1ReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<List<List<FieldValue>>> clearingExceptionsAndExemptionsCounterparty2 = Optional.ofNullable(input.getClearingExceptionsAndExemptionsCounterparty2())
			.map(x -> x.stream()
				.map(_x -> clearingExceptionsAndExemptionsCounterparty2ReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<String> counterparty1 = Optional.ofNullable(input.getCounterparty1());
		Optional<String> counterparty2 = Optional.ofNullable(input.getCounterparty2());
		Optional<String> counterparty2IdentifierSource = Optional.ofNullable(input.getCounterparty2IdentifierSource());
		Optional<Boolean> counterparty1FinancialEntityIndicator = Optional.ofNullable(input.getCounterparty1FinancialEntityIndicator());
		Optional<Boolean> counterparty2FinancialEntityIndicator = Optional.ofNullable(input.getCounterparty2FinancialEntityIndicator());
		Optional<String> buyerIdentifier = Optional.ofNullable(input.getBuyerIdentifier());
		Optional<String> sellerIdentifier = Optional.ofNullable(input.getSellerIdentifier());
		Optional<String> payerIdentifierLeg1 = Optional.ofNullable(input.getPayerIdentifierLeg1());
		Optional<String> payerIdentifierLeg2 = Optional.ofNullable(input.getPayerIdentifierLeg2());
		Optional<String> receiverIdentifierLeg1 = Optional.ofNullable(input.getReceiverIdentifierLeg1());
		Optional<String> receiverIdentifierLeg2 = Optional.ofNullable(input.getReceiverIdentifierLeg2());
		Optional<String> submitterIdentifier = Optional.ofNullable(input.getSubmitterIdentifier());
		Optional<Boolean> counterparty1FederalEntityIndicator = Optional.ofNullable(input.getCounterparty1FederalEntityIndicator());
		Optional<Boolean> counterparty2FederalEntityIndicator = Optional.ofNullable(input.getCounterparty2FederalEntityIndicator());
		Optional<Boolean> customBasketIndicator = Optional.ofNullable(input.getCustomBasketIndicator());
		Optional<String> actionType = Optional.ofNullable(input.getActionType());
		Optional<String> eventType = Optional.ofNullable(input.getEventType());
		Optional<Boolean> amendmentIndicator = Optional.ofNullable(input.getAmendmentIndicator());
		Optional<String> eventIdentifier = Optional.ofNullable(input.getEventIdentifier());
		Optional<ZonedDateTime> eventTimestamp = Optional.ofNullable(input.getEventTimestamp());
		Optional<BigDecimal> notionalAmountLeg1 = Optional.ofNullable(input.getNotionalAmountLeg1());
		Optional<BigDecimal> notionalAmountLeg2 = Optional.ofNullable(input.getNotionalAmountLeg2());
		Optional<ISOCurrencyCodeEnum> notionalCurrencyLeg1 = Optional.ofNullable(input.getNotionalCurrencyLeg1());
		Optional<ISOCurrencyCodeEnum> notionalCurrencyLeg2 = Optional.ofNullable(input.getNotionalCurrencyLeg2());
		Optional<List<List<FieldValue>>> notionalAmountScheduleLeg1 = Optional.ofNullable(input.getNotionalAmountScheduleLeg1())
			.map(x -> x.stream()
				.map(_x -> notionalAmountScheduleLeg1ReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<List<List<FieldValue>>> notionalAmountScheduleLeg2 = Optional.ofNullable(input.getNotionalAmountScheduleLeg2())
			.map(x -> x.stream()
				.map(_x -> notionalAmountScheduleLeg2ReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<BigDecimal> callAmount = Optional.ofNullable(input.getCallAmount());
		Optional<ISOCurrencyCodeEnum> callCurrency = Optional.ofNullable(input.getCallCurrency());
		Optional<BigDecimal> putAmount = Optional.ofNullable(input.getPutAmount());
		Optional<ISOCurrencyCodeEnum> putCurrency = Optional.ofNullable(input.getPutCurrency());
		Optional<BigDecimal> notionalQuantityLeg1 = Optional.ofNullable(input.getNotionalQuantityLeg1());
		Optional<BigDecimal> notionalQuantityLeg2 = Optional.ofNullable(input.getNotionalQuantityLeg2());
		Optional<String> quantityFrequencyLeg1 = Optional.ofNullable(input.getQuantityFrequencyLeg1());
		Optional<String> quantityFrequencyLeg2 = Optional.ofNullable(input.getQuantityFrequencyLeg2());
		Optional<Integer> quantityFrequencyMultiplierLeg1 = Optional.ofNullable(input.getQuantityFrequencyMultiplierLeg1());
		Optional<Integer> quantityFrequencyMultiplierLeg2 = Optional.ofNullable(input.getQuantityFrequencyMultiplierLeg2());
		Optional<String> quantityUnitOfMeasureLeg1 = Optional.ofNullable(input.getQuantityUnitOfMeasureLeg1());
		Optional<String> quantityUnitOfMeasureLeg2 = Optional.ofNullable(input.getQuantityUnitOfMeasureLeg2());
		Optional<BigDecimal> totalNotionalQuantityLeg1 = Optional.ofNullable(input.getTotalNotionalQuantityLeg1());
		Optional<BigDecimal> totalNotionalQuantityLeg2 = Optional.ofNullable(input.getTotalNotionalQuantityLeg2());
		Optional<Boolean> packageIndicator = Optional.ofNullable(input.getPackageIndicator());
		Optional<String> packageIdentifier = Optional.ofNullable(input.getPackageIdentifier());
		Optional<BigDecimal> packageTransactionPrice = Optional.ofNullable(input.getPackageTransactionPrice());
		Optional<ISOCurrencyCodeEnum> packageTransactionPriceCurrency = Optional.ofNullable(input.getPackageTransactionPriceCurrency());
		Optional<String> packageTransactionPriceNotation = Optional.ofNullable(input.getPackageTransactionPriceNotation());
		Optional<BigDecimal> packageTransactionSpread = Optional.ofNullable(input.getPackageTransactionSpread());
		Optional<ISOCurrencyCodeEnum> packageTransactionSpreadCurrency = Optional.ofNullable(input.getPackageTransactionSpreadCurrency());
		Optional<String> packageTransactionSpreadNotation = Optional.ofNullable(input.getPackageTransactionSpreadNotation());
		Optional<InterestComputationMethod4Code> fixedRateDayCountConventionLeg1 = Optional.ofNullable(input.getFixedRateDayCountConventionLeg1());
		Optional<InterestComputationMethod4Code> fixedRateDayCountConventionLeg2 = Optional.ofNullable(input.getFixedRateDayCountConventionLeg2());
		Optional<InterestComputationMethod4Code> floatingRateDayCountConventionLeg1 = Optional.ofNullable(input.getFloatingRateDayCountConventionLeg1());
		Optional<InterestComputationMethod4Code> floatingRateDayCountConventionLeg2 = Optional.ofNullable(input.getFloatingRateDayCountConventionLeg2());
		Optional<Date> fixingDateLeg1 = Optional.ofNullable(input.getFixingDateLeg1());
		Optional<Date> fixingDateLeg2 = Optional.ofNullable(input.getFixingDateLeg2());
		Optional<String> floatingRateResetFrequencyPeriodLeg1 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodLeg1());
		Optional<String> floatingRateResetFrequencyPeriodLeg2 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodLeg2());
		Optional<Integer> floatingRateResetFrequencyPeriodMultiplierLeg1 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodMultiplierLeg1());
		Optional<Integer> floatingRateResetFrequencyPeriodMultiplierLeg2 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodMultiplierLeg2());
		Optional<List<List<FieldValue>>> otherPayment = Optional.ofNullable(input.getOtherPayment())
			.map(x -> x.stream()
				.map(_x -> otherPaymentReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<String> fixedRatePaymentFrequencyPeriodLeg1 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodLeg1());
		Optional<String> fixedRatePaymentFrequencyPeriodLeg2 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodLeg2());
		Optional<String> floatingRatePaymentFrequencyPeriodLeg1 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodLeg1());
		Optional<String> floatingRatePaymentFrequencyPeriodLeg2 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodLeg2());
		Optional<Integer> fixedRatePaymentFrequencyPeriodMultiplierLeg1 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodMultiplierLeg1());
		Optional<Integer> fixedRatePaymentFrequencyPeriodMultiplierLeg2 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodMultiplierLeg2());
		Optional<Integer> floatingRatePaymentFrequencyPeriodMultiplierLeg1 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodMultiplierLeg1());
		Optional<Integer> floatingRatePaymentFrequencyPeriodMultiplierLeg2 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodMultiplierLeg2());
		Optional<BigDecimal> exchangeRateLeg1 = Optional.ofNullable(input.getExchangeRateLeg1());
		Optional<String> exchangeRateBasis = Optional.ofNullable(input.getExchangeRateBasis());
		Optional<BigDecimal> fixedRateLeg1 = Optional.ofNullable(input.getFixedRateLeg1());
		Optional<BigDecimal> fixedRateLeg2 = Optional.ofNullable(input.getFixedRateLeg2());
		Optional<Boolean> postPricedSwapIndicator = Optional.ofNullable(input.getPostPricedSwapIndicator());
		Optional<BigDecimal> price = Optional.ofNullable(input.getPrice());
		Optional<ISOCurrencyCodeEnum> priceCurrency = Optional.ofNullable(input.getPriceCurrency());
		Optional<String> priceNotation = Optional.ofNullable(input.getPriceNotation());
		Optional<String> priceUnitOfMeasure = Optional.ofNullable(input.getPriceUnitOfMeasure());
		Optional<BigDecimal> spreadLeg1 = Optional.ofNullable(input.getSpreadLeg1());
		Optional<BigDecimal> spreadLeg2 = Optional.ofNullable(input.getSpreadLeg2());
		Optional<ISOCurrencyCodeEnum> spreadCurrencyLeg1 = Optional.ofNullable(input.getSpreadCurrencyLeg1());
		Optional<ISOCurrencyCodeEnum> spreadCurrencyLeg2 = Optional.ofNullable(input.getSpreadCurrencyLeg2());
		Optional<String> spreadNotationLeg1 = Optional.ofNullable(input.getSpreadNotationLeg1());
		Optional<String> spreadNotationLeg2 = Optional.ofNullable(input.getSpreadNotationLeg2());
		Optional<BigDecimal> strikePrice = Optional.ofNullable(input.getStrikePrice());
		Optional<String> strikePriceCurrency = Optional.ofNullable(input.getStrikePriceCurrency());
		Optional<String> strikePriceNotation = Optional.ofNullable(input.getStrikePriceNotation());
		Optional<BigDecimal> optionPremiumAmount = Optional.ofNullable(input.getOptionPremiumAmount());
		Optional<ISOCurrencyCodeEnum> optionPremiumCurrency = Optional.ofNullable(input.getOptionPremiumCurrency());
		Optional<Date> optionPremiumPaymentDate = Optional.ofNullable(input.getOptionPremiumPaymentDate());
		Optional<Date> firstExerciseDate = Optional.ofNullable(input.getFirstExerciseDate());
		Optional<BigDecimal> cDSIndexAttachmentPoint = Optional.ofNullable(input.getCDSIndexAttachmentPoint());
		Optional<BigDecimal> cDSIndexDetachmentPoint = Optional.ofNullable(input.getCDSIndexDetachmentPoint());
		Optional<BigDecimal> indexFactor = Optional.ofNullable(input.getIndexFactor());
		Optional<String> embeddedOptionType = Optional.ofNullable(input.getEmbeddedOptionType());
		Optional<String> uniqueProductIdentifier = Optional.ofNullable(input.getUniqueProductIdentifier());
		Optional<Date> finalContractualSettlementDate = Optional.ofNullable(input.getFinalContractualSettlementDate());
		Optional<ISOCurrencyCodeEnum> settlementCurrencyLeg1 = Optional.ofNullable(input.getSettlementCurrencyLeg1());
		Optional<ISOCurrencyCodeEnum> settlementCurrencyLeg2 = Optional.ofNullable(input.getSettlementCurrencyLeg2());
		Optional<String> settlementLocationLeg1 = Optional.ofNullable(input.getSettlementLocationLeg1());
		Optional<String> settlementLocationLeg2 = Optional.ofNullable(input.getSettlementLocationLeg2());
		Optional<String> allocationIndicator = Optional.ofNullable(input.getAllocationIndicator());
		Optional<Boolean> nonStandardizedTermIndicator = Optional.ofNullable(input.getNonStandardizedTermIndicator());
		Optional<Boolean> blockTradeElectionIndicator = Optional.ofNullable(input.getBlockTradeElectionIndicator());
		Optional<Date> effectiveDate = Optional.ofNullable(input.getEffectiveDate());
		Optional<Date> expirationDate = Optional.ofNullable(input.getExpirationDate());
		Optional<ZonedDateTime> executionTimestamp = Optional.ofNullable(input.getExecutionTimestamp());
		Optional<ZonedDateTime> reportingTimestamp = Optional.ofNullable(input.getReportingTimestamp());
		Optional<String> platformIdentifier = Optional.ofNullable(input.getPlatformIdentifier());
		Optional<Boolean> primeBrokerageTransactionIndicator = Optional.ofNullable(input.getPrimeBrokerageTransactionIndicator());
		Optional<String> priorUSI = Optional.ofNullable(input.getPriorUSI());
		Optional<String> priorUTI = Optional.ofNullable(input.getPriorUTI());
		Optional<String> uniqueSwapIdentifier = Optional.ofNullable(input.getUniqueSwapIdentifier());
		Optional<String> uniqueTransactionIdentifier = Optional.ofNullable(input.getUniqueTransactionIdentifier());
		Optional<String> jurisdiction = Optional.ofNullable(input.getJurisdiction());
		Optional<String> newSDRIdentifier = Optional.ofNullable(input.getNewSDRIdentifier());
		Optional<String> initialMarginCollateralPortfolioCode = Optional.ofNullable(input.getInitialMarginCollateralPortfolioCode());
		Optional<String> variationMarginCollateralPortfolioCode = Optional.ofNullable(input.getVariationMarginCollateralPortfolioCode());
		Optional<List<FieldValue>> cftcGuidebook = Optional.ofNullable(input.getCftcGuidebook())
			.map(x -> cFTCGuidebookPart45TypeTabulator.tabulate(x));
		Optional<List<FieldValue>> nonReportable = Optional.ofNullable(input.getNonReportable())
			.map(x -> cFTCNonReportablePart45TypeTabulator.tabulate(x));
		return Arrays.asList(
			new FieldValueImpl(clearedField, cleared),
			new FieldValueImpl(centralCounterpartyField, centralCounterparty),
			new FieldValueImpl(clearingAccountOriginField, clearingAccountOrigin),
			new FieldValueImpl(clearingMemberField, clearingMember),
			new MultiNestedFieldValueImpl(clearingSwapUSIsField, clearingSwapUSIs),
			new MultiNestedFieldValueImpl(clearingSwapUTIsField, clearingSwapUTIs),
			new FieldValueImpl(originalSwapUSIField, originalSwapUSI),
			new FieldValueImpl(originalSwapUTIField, originalSwapUTI),
			new FieldValueImpl(originalSwapSDRIdentifierField, originalSwapSDRIdentifier),
			new FieldValueImpl(clearingReceiptTimestampField, clearingReceiptTimestamp),
			new MultiNestedFieldValueImpl(clearingExceptionsAndExemptionsCounterparty1Field, clearingExceptionsAndExemptionsCounterparty1),
			new MultiNestedFieldValueImpl(clearingExceptionsAndExemptionsCounterparty2Field, clearingExceptionsAndExemptionsCounterparty2),
			new FieldValueImpl(counterparty1Field, counterparty1),
			new FieldValueImpl(counterparty2Field, counterparty2),
			new FieldValueImpl(counterparty2IdentifierSourceField, counterparty2IdentifierSource),
			new FieldValueImpl(counterparty1FinancialEntityIndicatorField, counterparty1FinancialEntityIndicator),
			new FieldValueImpl(counterparty2FinancialEntityIndicatorField, counterparty2FinancialEntityIndicator),
			new FieldValueImpl(buyerIdentifierField, buyerIdentifier),
			new FieldValueImpl(sellerIdentifierField, sellerIdentifier),
			new FieldValueImpl(payerIdentifierLeg1Field, payerIdentifierLeg1),
			new FieldValueImpl(payerIdentifierLeg2Field, payerIdentifierLeg2),
			new FieldValueImpl(receiverIdentifierLeg1Field, receiverIdentifierLeg1),
			new FieldValueImpl(receiverIdentifierLeg2Field, receiverIdentifierLeg2),
			new FieldValueImpl(submitterIdentifierField, submitterIdentifier),
			new FieldValueImpl(counterparty1FederalEntityIndicatorField, counterparty1FederalEntityIndicator),
			new FieldValueImpl(counterparty2FederalEntityIndicatorField, counterparty2FederalEntityIndicator),
			new FieldValueImpl(customBasketIndicatorField, customBasketIndicator),
			new FieldValueImpl(actionTypeField, actionType),
			new FieldValueImpl(eventTypeField, eventType),
			new FieldValueImpl(amendmentIndicatorField, amendmentIndicator),
			new FieldValueImpl(eventIdentifierField, eventIdentifier),
			new FieldValueImpl(eventTimestampField, eventTimestamp),
			new FieldValueImpl(notionalAmountLeg1Field, notionalAmountLeg1),
			new FieldValueImpl(notionalAmountLeg2Field, notionalAmountLeg2),
			new FieldValueImpl(notionalCurrencyLeg1Field, notionalCurrencyLeg1),
			new FieldValueImpl(notionalCurrencyLeg2Field, notionalCurrencyLeg2),
			new MultiNestedFieldValueImpl(notionalAmountScheduleLeg1Field, notionalAmountScheduleLeg1),
			new MultiNestedFieldValueImpl(notionalAmountScheduleLeg2Field, notionalAmountScheduleLeg2),
			new FieldValueImpl(callAmountField, callAmount),
			new FieldValueImpl(callCurrencyField, callCurrency),
			new FieldValueImpl(putAmountField, putAmount),
			new FieldValueImpl(putCurrencyField, putCurrency),
			new FieldValueImpl(notionalQuantityLeg1Field, notionalQuantityLeg1),
			new FieldValueImpl(notionalQuantityLeg2Field, notionalQuantityLeg2),
			new FieldValueImpl(quantityFrequencyLeg1Field, quantityFrequencyLeg1),
			new FieldValueImpl(quantityFrequencyLeg2Field, quantityFrequencyLeg2),
			new FieldValueImpl(quantityFrequencyMultiplierLeg1Field, quantityFrequencyMultiplierLeg1),
			new FieldValueImpl(quantityFrequencyMultiplierLeg2Field, quantityFrequencyMultiplierLeg2),
			new FieldValueImpl(quantityUnitOfMeasureLeg1Field, quantityUnitOfMeasureLeg1),
			new FieldValueImpl(quantityUnitOfMeasureLeg2Field, quantityUnitOfMeasureLeg2),
			new FieldValueImpl(totalNotionalQuantityLeg1Field, totalNotionalQuantityLeg1),
			new FieldValueImpl(totalNotionalQuantityLeg2Field, totalNotionalQuantityLeg2),
			new FieldValueImpl(packageIndicatorField, packageIndicator),
			new FieldValueImpl(packageIdentifierField, packageIdentifier),
			new FieldValueImpl(packageTransactionPriceField, packageTransactionPrice),
			new FieldValueImpl(packageTransactionPriceCurrencyField, packageTransactionPriceCurrency),
			new FieldValueImpl(packageTransactionPriceNotationField, packageTransactionPriceNotation),
			new FieldValueImpl(packageTransactionSpreadField, packageTransactionSpread),
			new FieldValueImpl(packageTransactionSpreadCurrencyField, packageTransactionSpreadCurrency),
			new FieldValueImpl(packageTransactionSpreadNotationField, packageTransactionSpreadNotation),
			new FieldValueImpl(fixedRateDayCountConventionLeg1Field, fixedRateDayCountConventionLeg1),
			new FieldValueImpl(fixedRateDayCountConventionLeg2Field, fixedRateDayCountConventionLeg2),
			new FieldValueImpl(floatingRateDayCountConventionLeg1Field, floatingRateDayCountConventionLeg1),
			new FieldValueImpl(floatingRateDayCountConventionLeg2Field, floatingRateDayCountConventionLeg2),
			new FieldValueImpl(fixingDateLeg1Field, fixingDateLeg1),
			new FieldValueImpl(fixingDateLeg2Field, fixingDateLeg2),
			new FieldValueImpl(floatingRateResetFrequencyPeriodLeg1Field, floatingRateResetFrequencyPeriodLeg1),
			new FieldValueImpl(floatingRateResetFrequencyPeriodLeg2Field, floatingRateResetFrequencyPeriodLeg2),
			new FieldValueImpl(floatingRateResetFrequencyPeriodMultiplierLeg1Field, floatingRateResetFrequencyPeriodMultiplierLeg1),
			new FieldValueImpl(floatingRateResetFrequencyPeriodMultiplierLeg2Field, floatingRateResetFrequencyPeriodMultiplierLeg2),
			new MultiNestedFieldValueImpl(otherPaymentField, otherPayment),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodLeg1Field, fixedRatePaymentFrequencyPeriodLeg1),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodLeg2Field, fixedRatePaymentFrequencyPeriodLeg2),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodLeg1Field, floatingRatePaymentFrequencyPeriodLeg1),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodLeg2Field, floatingRatePaymentFrequencyPeriodLeg2),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodMultiplierLeg1Field, fixedRatePaymentFrequencyPeriodMultiplierLeg1),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodMultiplierLeg2Field, fixedRatePaymentFrequencyPeriodMultiplierLeg2),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodMultiplierLeg1Field, floatingRatePaymentFrequencyPeriodMultiplierLeg1),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodMultiplierLeg2Field, floatingRatePaymentFrequencyPeriodMultiplierLeg2),
			new FieldValueImpl(exchangeRateLeg1Field, exchangeRateLeg1),
			new FieldValueImpl(exchangeRateBasisField, exchangeRateBasis),
			new FieldValueImpl(fixedRateLeg1Field, fixedRateLeg1),
			new FieldValueImpl(fixedRateLeg2Field, fixedRateLeg2),
			new FieldValueImpl(postPricedSwapIndicatorField, postPricedSwapIndicator),
			new FieldValueImpl(priceField, price),
			new FieldValueImpl(priceCurrencyField, priceCurrency),
			new FieldValueImpl(priceNotationField, priceNotation),
			new FieldValueImpl(priceUnitOfMeasureField, priceUnitOfMeasure),
			new FieldValueImpl(spreadLeg1Field, spreadLeg1),
			new FieldValueImpl(spreadLeg2Field, spreadLeg2),
			new FieldValueImpl(spreadCurrencyLeg1Field, spreadCurrencyLeg1),
			new FieldValueImpl(spreadCurrencyLeg2Field, spreadCurrencyLeg2),
			new FieldValueImpl(spreadNotationLeg1Field, spreadNotationLeg1),
			new FieldValueImpl(spreadNotationLeg2Field, spreadNotationLeg2),
			new FieldValueImpl(strikePriceField, strikePrice),
			new FieldValueImpl(strikePriceCurrencyField, strikePriceCurrency),
			new FieldValueImpl(strikePriceNotationField, strikePriceNotation),
			new FieldValueImpl(optionPremiumAmountField, optionPremiumAmount),
			new FieldValueImpl(optionPremiumCurrencyField, optionPremiumCurrency),
			new FieldValueImpl(optionPremiumPaymentDateField, optionPremiumPaymentDate),
			new FieldValueImpl(firstExerciseDateField, firstExerciseDate),
			new FieldValueImpl(cDSIndexAttachmentPointField, cDSIndexAttachmentPoint),
			new FieldValueImpl(cDSIndexDetachmentPointField, cDSIndexDetachmentPoint),
			new FieldValueImpl(indexFactorField, indexFactor),
			new FieldValueImpl(embeddedOptionTypeField, embeddedOptionType),
			new FieldValueImpl(uniqueProductIdentifierField, uniqueProductIdentifier),
			new FieldValueImpl(finalContractualSettlementDateField, finalContractualSettlementDate),
			new FieldValueImpl(settlementCurrencyLeg1Field, settlementCurrencyLeg1),
			new FieldValueImpl(settlementCurrencyLeg2Field, settlementCurrencyLeg2),
			new FieldValueImpl(settlementLocationLeg1Field, settlementLocationLeg1),
			new FieldValueImpl(settlementLocationLeg2Field, settlementLocationLeg2),
			new FieldValueImpl(allocationIndicatorField, allocationIndicator),
			new FieldValueImpl(nonStandardizedTermIndicatorField, nonStandardizedTermIndicator),
			new FieldValueImpl(blockTradeElectionIndicatorField, blockTradeElectionIndicator),
			new FieldValueImpl(effectiveDateField, effectiveDate),
			new FieldValueImpl(expirationDateField, expirationDate),
			new FieldValueImpl(executionTimestampField, executionTimestamp),
			new FieldValueImpl(reportingTimestampField, reportingTimestamp),
			new FieldValueImpl(platformIdentifierField, platformIdentifier),
			new FieldValueImpl(primeBrokerageTransactionIndicatorField, primeBrokerageTransactionIndicator),
			new FieldValueImpl(priorUSIField, priorUSI),
			new FieldValueImpl(priorUTIField, priorUTI),
			new FieldValueImpl(uniqueSwapIdentifierField, uniqueSwapIdentifier),
			new FieldValueImpl(uniqueTransactionIdentifierField, uniqueTransactionIdentifier),
			new FieldValueImpl(jurisdictionField, jurisdiction),
			new FieldValueImpl(newSDRIdentifierField, newSDRIdentifier),
			new FieldValueImpl(initialMarginCollateralPortfolioCodeField, initialMarginCollateralPortfolioCode),
			new FieldValueImpl(variationMarginCollateralPortfolioCodeField, variationMarginCollateralPortfolioCode),
			new NestedFieldValueImpl(cftcGuidebookField, cftcGuidebook),
			new NestedFieldValueImpl(nonReportableField, nonReportable)
		);
	}
}
