package drr.regulation.cftc.rewrite.reports;

import cdm.base.staticdata.party.Party;
import cdm.base.staticdata.party.metafields.ReferenceWithMetaParty;
import cdm.event.workflow.WorkflowStep;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperListOfLists;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.cftc.rewrite.ClearingExceptionsAndExemptionsCounterparty2Report;
import drr.regulation.cftc.rewrite.ClearingExceptionsAndExemptionsCounterparty2Report.ClearingExceptionsAndExemptionsCounterparty2ReportBuilder;
import drr.regulation.common.ClearingException;
import drr.regulation.common.ClearingExceptionReason;
import drr.regulation.common.ClearingExceptionReasonEnum;
import drr.regulation.common.PartyInformation;
import drr.regulation.common.ReportableInformation;
import drr.regulation.common.ReportingRegime;
import drr.regulation.common.ReportingSide;
import drr.regulation.common.SupervisoryBodyEnum;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.IntendedToClear;
import drr.regulation.common.functions.IsCleared;
import drr.regulation.common.metafields.FieldWithMetaSupervisoryBodyEnum;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(ClearingExceptionsAndExemptionsCounterparty2Rule.ClearingExceptionsAndExemptionsCounterparty2RuleDefault.class)
public abstract class ClearingExceptionsAndExemptionsCounterparty2Rule implements ReportFunction<TransactionReportInstruction, List<? extends ClearingExceptionsAndExemptionsCounterparty2Report>> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ClearingExceptionsAndExemptionsCounterparty2ExtractionRule clearingExceptionsAndExemptionsCounterparty2Extraction;
	@Inject protected IntendedToClear intendedToClear;
	@Inject protected IsCleared isCleared;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public List<? extends ClearingExceptionsAndExemptionsCounterparty2Report> evaluate(TransactionReportInstruction input) {
		List<ClearingExceptionsAndExemptionsCounterparty2Report.ClearingExceptionsAndExemptionsCounterparty2ReportBuilder> outputBuilder = doEvaluate(input);
		
		final List<? extends ClearingExceptionsAndExemptionsCounterparty2Report> output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.stream().map(ClearingExceptionsAndExemptionsCounterparty2Report::build).collect(Collectors.toList());
			objectValidator.validate(ClearingExceptionsAndExemptionsCounterparty2Report.class, output);
		}
		
		return output;
	}

	protected abstract List<ClearingExceptionsAndExemptionsCounterparty2Report.ClearingExceptionsAndExemptionsCounterparty2ReportBuilder> doEvaluate(TransactionReportInstruction input);

	public static class ClearingExceptionsAndExemptionsCounterparty2RuleDefault extends ClearingExceptionsAndExemptionsCounterparty2Rule {
		@Override
		protected List<ClearingExceptionsAndExemptionsCounterparty2Report.ClearingExceptionsAndExemptionsCounterparty2ReportBuilder> doEvaluate(TransactionReportInstruction input) {
			List<ClearingExceptionsAndExemptionsCounterparty2Report.ClearingExceptionsAndExemptionsCounterparty2ReportBuilder> output = new ArrayList<>();
			return assignOutput(output, input);
		}
		
		protected List<ClearingExceptionsAndExemptionsCounterparty2Report.ClearingExceptionsAndExemptionsCounterparty2ReportBuilder> assignOutput(List<ClearingExceptionsAndExemptionsCounterparty2Report.ClearingExceptionsAndExemptionsCounterparty2ReportBuilder> output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult0 = MapperS.of(input)
				.filterSingleNullSafe(item -> areEqual(MapperS.of(isCleared.evaluate(item.<WorkflowStep>map("getOriginatingWorkflowStep", reportableEvent -> reportableEvent.getOriginatingWorkflowStep()).get())), MapperS.of(false), CardinalityOperator.All).and(areEqual(MapperS.of(intendedToClear.evaluate(item.get())), MapperS.of(false), CardinalityOperator.All)).get());
			final MapperC<ClearingExceptionReason> thenResult1 = thenResult0
				.mapSingleToList(reportInstruction -> {
					final MapperC<PartyInformation> _thenResult0 = reportInstruction.<ReportableInformation>map("getReportableInformation", reportableEvent -> reportableEvent.getReportableInformation()).<PartyInformation>mapC("getPartyInformation", reportableInformation -> reportableInformation.getPartyInformation())
						.filterItemNullSafe(item -> areEqual(item.<ReferenceWithMetaParty>map("getPartyReference", partyInformation -> partyInformation.getPartyReference()).<Party>map("getValue", _f->_f.getValue()), reportInstruction.<ReportingSide>map("getReportingSide", transactionReportInstruction -> transactionReportInstruction.getReportingSide()).<ReferenceWithMetaParty>map("getReportingCounterparty", reportingSide -> reportingSide.getReportingCounterparty()).<Party>map("getValue", _f->_f.getValue()), CardinalityOperator.All).get());
					final MapperC<PartyInformation> _thenResult1 = _thenResult0
						.filterItemNullSafe(item -> areEqual(item.<ReportingRegime>mapC("getRegimeInformation", partyInformation -> partyInformation.getRegimeInformation()).<ClearingException>map("getClearingException", reportingRegime -> reportingRegime.getClearingException()).<Boolean>map("getClearingException", clearingException -> clearingException.getClearingException()), MapperS.of(true), CardinalityOperator.Any).get());
					final MapperListOfLists<ReportingRegime> thenResult2 = _thenResult1
						.mapItemToList(item -> item.<ReportingRegime>mapC("getRegimeInformation", partyInformation -> partyInformation.getRegimeInformation()));
					final MapperC<ReportingRegime> thenResult3 = thenResult2
						.flattenList();
					final MapperC<ReportingRegime> thenResult4 = thenResult3
						.filterItemNullSafe(item -> areEqual(item.<FieldWithMetaSupervisoryBodyEnum>map("getSupervisoryBody", reportingRegime -> reportingRegime.getSupervisoryBody()).<SupervisoryBodyEnum>map("getValue", _f->_f.getValue()), MapperS.of(SupervisoryBodyEnum.CFTC), CardinalityOperator.All).get());
					final MapperListOfLists<ClearingExceptionReasonEnum> thenResult5 = thenResult4
						.mapItemToList(item -> item.<ClearingException>map("getClearingException", reportingRegime -> reportingRegime.getClearingException()).<ClearingExceptionReasonEnum>mapC("getClearingExceptionReason", clearingException -> clearingException.getClearingExceptionReason()));
					final MapperC<ClearingExceptionReasonEnum> thenResult6 = thenResult5
						.flattenList();
					return thenResult6
						.mapItem(item -> MapperS.of(ClearingExceptionReason.builder()
							.setExceptionReason(item.get())
							.build()
						));
				});
			output = toBuilder(thenResult1
				.mapItem(item -> MapperS.of(ClearingExceptionsAndExemptionsCounterparty2Report.builder()
					.setValue(clearingExceptionsAndExemptionsCounterparty2Extraction.evaluate(item.get()))
					.build()
				)).getMulti());
			
			return Optional.ofNullable(output)
				.map(o -> o.stream().map(i -> i.prune()).collect(Collectors.toList()))
				.orElse(null);
		}
	}
}
