package drr.regulation.cftc.rewrite.reports;

import cdm.base.staticdata.identifier.AssignedIdentifier;
import cdm.base.staticdata.identifier.TradeIdentifierTypeEnum;
import cdm.event.common.BusinessEvent;
import cdm.event.common.Instruction;
import cdm.event.common.Trade;
import cdm.event.common.TradeIdentifier;
import cdm.event.common.TradeState;
import cdm.event.common.metafields.ReferenceWithMetaTradeState;
import cdm.event.workflow.WorkflowStep;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.cftc.rewrite.ClearingSwapUTIsReport;
import drr.regulation.cftc.rewrite.ClearingSwapUTIsReport.ClearingSwapUTIsReportBuilder;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.FindLatestAssignedIdentifier;
import drr.regulation.common.functions.IntendedToClear;
import drr.regulation.common.functions.IsActionTypeTERM;
import drr.regulation.common.functions.IsCleared;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(ClearingSwapUTIsRule.ClearingSwapUTIsRuleDefault.class)
public abstract class ClearingSwapUTIsRule implements ReportFunction<TransactionReportInstruction, List<? extends ClearingSwapUTIsReport>> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ClearingSwapUTIRule clearingSwapUTI;
	@Inject protected FindLatestAssignedIdentifier findLatestAssignedIdentifier;
	@Inject protected IntendedToClear intendedToClear;
	@Inject protected IsActionTypeTERM isActionTypeTERM;
	@Inject protected IsCleared isCleared;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public List<? extends ClearingSwapUTIsReport> evaluate(TransactionReportInstruction input) {
		List<ClearingSwapUTIsReport.ClearingSwapUTIsReportBuilder> outputBuilder = doEvaluate(input);
		
		final List<? extends ClearingSwapUTIsReport> output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.stream().map(ClearingSwapUTIsReport::build).collect(Collectors.toList());
			objectValidator.validate(ClearingSwapUTIsReport.class, output);
		}
		
		return output;
	}

	protected abstract List<ClearingSwapUTIsReport.ClearingSwapUTIsReportBuilder> doEvaluate(TransactionReportInstruction input);

	public static class ClearingSwapUTIsRuleDefault extends ClearingSwapUTIsRule {
		@Override
		protected List<ClearingSwapUTIsReport.ClearingSwapUTIsReportBuilder> doEvaluate(TransactionReportInstruction input) {
			List<ClearingSwapUTIsReport.ClearingSwapUTIsReportBuilder> output = new ArrayList<>();
			return assignOutput(output, input);
		}
		
		protected List<ClearingSwapUTIsReport.ClearingSwapUTIsReportBuilder> assignOutput(List<ClearingSwapUTIsReport.ClearingSwapUTIsReportBuilder> output, TransactionReportInstruction input) {
			final MapperS<BusinessEvent> thenResult0 = MapperS.of(input)
				.mapSingleToItem(item -> {
					if (ComparisonResult.of(MapperS.of(isCleared.evaluate(item.<WorkflowStep>map("getOriginatingWorkflowStep", reportableEvent -> reportableEvent.getOriginatingWorkflowStep()).get()))).or(ComparisonResult.of(MapperS.of(intendedToClear.evaluate(item.get()))).and(ComparisonResult.of(MapperS.of(isActionTypeTERM.evaluate(item.get()))))).getOrDefault(false)) {
						return item.<WorkflowStep>map("getOriginatingWorkflowStep", reportableEvent -> reportableEvent.getOriginatingWorkflowStep()).<BusinessEvent>map("getBusinessEvent", workflowStep -> workflowStep.getBusinessEvent());
					}
					return MapperS.<BusinessEvent>ofNull();
				});
			final MapperC<AssignedIdentifier> thenResult1 = thenResult0
				.mapSingleToList(businessEvent -> {
					final MapperC<TradeIdentifier> _thenResult0 = businessEvent.<TradeState>mapC("getAfter", _businessEvent -> _businessEvent.getAfter()).<Trade>map("getTrade", tradeState -> tradeState.getTrade()).<TradeIdentifier>mapC("getTradeIdentifier", trade -> trade.getTradeIdentifier())
						.filterItemNullSafe(item -> notEqual(businessEvent.<Instruction>mapC("getInstruction", eventInstruction -> eventInstruction.getInstruction()).<ReferenceWithMetaTradeState>map("getBefore", instruction -> instruction.getBefore()).<TradeState>map("getValue", _f->_f.getValue()).<Trade>map("getTrade", tradeState -> tradeState.getTrade()).<TradeIdentifier>mapC("getTradeIdentifier", trade -> trade.getTradeIdentifier()), item, CardinalityOperator.All).get());
					final MapperC<TradeIdentifier> _thenResult1 = _thenResult0
						.filterItemNullSafe(item -> areEqual(item.<TradeIdentifierTypeEnum>map("getIdentifierType", tradeIdentifier -> tradeIdentifier.getIdentifierType()), MapperS.of(TradeIdentifierTypeEnum.UNIQUE_TRANSACTION_IDENTIFIER), CardinalityOperator.All).get());
					return _thenResult1
						.mapItem(item -> MapperS.of(findLatestAssignedIdentifier.evaluate(item.get())));
				});
			output = toBuilder(thenResult1
				.mapItem(item -> MapperS.of(ClearingSwapUTIsReport.builder()
					.setIdentifier(clearingSwapUTI.evaluate(item.get()))
					.build()
				)).getMulti());
			
			return Optional.ofNullable(output)
				.map(o -> o.stream().map(i -> i.prune()).collect(Collectors.toList()))
				.orElse(null);
		}
	}
}
