package drr.regulation.cftc.rewrite.reports;

import cdm.base.math.DatedValue;
import cdm.base.math.NonNegativeQuantitySchedule;
import cdm.base.math.UnitType;
import cdm.base.math.metafields.ReferenceWithMetaNonNegativeQuantitySchedule;
import cdm.event.common.Trade;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import com.rosetta.model.metafields.FieldWithMetaString;
import drr.regulation.cftc.rewrite.NotionalAmountScheduleLeg1Report;
import drr.regulation.cftc.rewrite.NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.GetLeg1ResolvablePriceQuantity;
import drr.regulation.common.functions.TradeForEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(NotionalAmountScheduleLeg1Rule.NotionalAmountScheduleLeg1RuleDefault.class)
public abstract class NotionalAmountScheduleLeg1Rule implements ReportFunction<TransactionReportInstruction, List<? extends NotionalAmountScheduleLeg1Report>> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected GetLeg1ResolvablePriceQuantity getLeg1ResolvablePriceQuantity;
	@Inject protected NotionalAmountScheduleLeg1AmountRule notionalAmountScheduleLeg1Amount;
	@Inject protected NotionalAmountScheduleLeg1EffectiveDateRule notionalAmountScheduleLeg1EffectiveDate;
	@Inject protected TradeForEvent tradeForEvent;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public List<? extends NotionalAmountScheduleLeg1Report> evaluate(TransactionReportInstruction input) {
		List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> outputBuilder = doEvaluate(input);
		
		final List<? extends NotionalAmountScheduleLeg1Report> output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.stream().map(NotionalAmountScheduleLeg1Report::build).collect(Collectors.toList());
			objectValidator.validate(NotionalAmountScheduleLeg1Report.class, output);
		}
		
		return output;
	}

	protected abstract List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> doEvaluate(TransactionReportInstruction input);

	public static class NotionalAmountScheduleLeg1RuleDefault extends NotionalAmountScheduleLeg1Rule {
		@Override
		protected List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> doEvaluate(TransactionReportInstruction input) {
			List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> output = new ArrayList<>();
			return assignOutput(output, input);
		}
		
		protected List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> assignOutput(List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> output, TransactionReportInstruction input) {
			final MapperS<Trade> thenResult0 = MapperS.of(input)
				.mapSingleToItem(item -> MapperS.of(tradeForEvent.evaluate(item.get())));
			final MapperS<NonNegativeQuantitySchedule> thenResult1 = thenResult0
				.mapSingleToItem(item -> MapperS.of(getLeg1ResolvablePriceQuantity.evaluate(item.get())).<ReferenceWithMetaNonNegativeQuantitySchedule>map("getQuantitySchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getQuantitySchedule()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()));
			final MapperS<NonNegativeQuantitySchedule> thenResult2 = thenResult1
				.filterSingleNullSafe(item -> exists(item.<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue())).get());
			final MapperC<DatedValue> thenResult3 = thenResult2
				.mapSingleToList(item -> item.<DatedValue>mapC("getDatedValue", measureSchedule -> measureSchedule.getDatedValue()));
			output = toBuilder(thenResult3
				.mapItem(item -> MapperS.of(NotionalAmountScheduleLeg1Report.builder()
					.setAmount(notionalAmountScheduleLeg1Amount.evaluate(item.get()))
					.setEffectiveDate(notionalAmountScheduleLeg1EffectiveDate.evaluate(item.get()))
					.build()
				)).getMulti());
			
			return Optional.ofNullable(output)
				.map(o -> o.stream().map(i -> i.prune()).collect(Collectors.toList()))
				.orElse(null);
		}
	}
}
