package drr.regulation.cftc.rewrite.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.cftc.rewrite.OtherPaymentReport;
import drr.regulation.common.PartyIdentifierFormatEnum;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class OtherPaymentReportTypeTabulator implements Tabulator<OtherPaymentReport> {
	private final Field paymentTypeField;
	private final Field amountField;
	private final Field currencyField;
	private final Field dateField;
	private final Field payerField;
	private final Field payerFormatField;
	private final Field receiverField;
	private final Field receiverFormatField;
	
	public OtherPaymentReportTypeTabulator() {
		this.paymentTypeField = new FieldImpl(
			"paymentType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OtherPaymentType")),
			Optional.of("57/62-$ 57 Other payment type"),
			Arrays.asList()
		);
		this.amountField = new FieldImpl(
			"amount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OtherPaymentAmount")),
			Optional.of("57/62-$ 58 Other payment amount"),
			Arrays.asList()
		);
		this.currencyField = new FieldImpl(
			"currency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OtherPaymentCurrency")),
			Optional.of("57/62-$ 59 Other payment currency"),
			Arrays.asList()
		);
		this.dateField = new FieldImpl(
			"date",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OtherPaymentDate")),
			Optional.of("57/62-$ 60 Other payment date"),
			Arrays.asList()
		);
		this.payerField = new FieldImpl(
			"payer",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OtherPaymentPayer")),
			Optional.of("57/62-$ 61 Other payment payer"),
			Arrays.asList()
		);
		this.payerFormatField = new FieldImpl(
			"payerFormat",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OtherPaymentPayerFormat")),
			Optional.of("57/62-$ 61 Other payment payer format (non-reportable)"),
			Arrays.asList()
		);
		this.receiverField = new FieldImpl(
			"receiver",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OtherPaymentReceiver")),
			Optional.of("57/62-$ 62 Other payment receiver"),
			Arrays.asList()
		);
		this.receiverFormatField = new FieldImpl(
			"receiverFormat",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "cftc", "rewrite"), "OtherPaymentReceiverFormat")),
			Optional.of("57/62-$ 62 Other payment receiver format (non-reportable)"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(paymentTypeField, amountField, currencyField, dateField, payerField, payerFormatField, receiverField, receiverFormatField);
	}
	
	@Override
	public List<FieldValue> tabulate(OtherPaymentReport input) {
		Optional<String> paymentType = Optional.ofNullable(input.getPaymentType());
		Optional<BigDecimal> amount = Optional.ofNullable(input.getAmount());
		Optional<ISOCurrencyCodeEnum> currency = Optional.ofNullable(input.getCurrency());
		Optional<Date> date = Optional.ofNullable(input.getDate());
		Optional<String> payer = Optional.ofNullable(input.getPayer());
		Optional<PartyIdentifierFormatEnum> payerFormat = Optional.ofNullable(input.getPayerFormat());
		Optional<String> receiver = Optional.ofNullable(input.getReceiver());
		Optional<PartyIdentifierFormatEnum> receiverFormat = Optional.ofNullable(input.getReceiverFormat());
		return Arrays.asList(
			new FieldValueImpl(paymentTypeField, paymentType),
			new FieldValueImpl(amountField, amount),
			new FieldValueImpl(currencyField, currency),
			new FieldValueImpl(dateField, date),
			new FieldValueImpl(payerField, payer),
			new FieldValueImpl(payerFormatField, payerFormat),
			new FieldValueImpl(receiverField, receiver),
			new FieldValueImpl(receiverFormatField, receiverFormat)
		);
	}
}
