package drr.regulation.cftc.rewrite.reports;

import cdm.event.common.TradeState;
import cdm.event.common.TransferState;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.cftc.rewrite.OtherPaymentReport;
import drr.regulation.cftc.rewrite.OtherPaymentReport.OtherPaymentReportBuilder;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.TradeStateForEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.inject.Inject;


@ImplementedBy(OtherPaymentRule.OtherPaymentRuleDefault.class)
public abstract class OtherPaymentRule implements ReportFunction<TransactionReportInstruction, List<? extends OtherPaymentReport>> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected OtherPaymentAmountRule otherPaymentAmount;
	@Inject protected OtherPaymentCurrencyRule otherPaymentCurrency;
	@Inject protected OtherPaymentDateRule otherPaymentDate;
	@Inject protected OtherPaymentPayerRule otherPaymentPayer;
	@Inject protected OtherPaymentPayerFormatRule otherPaymentPayerFormat;
	@Inject protected OtherPaymentReceiverRule otherPaymentReceiver;
	@Inject protected OtherPaymentReceiverFormatRule otherPaymentReceiverFormat;
	@Inject protected OtherPaymentTypeRule otherPaymentType;
	@Inject protected TradeStateForEvent tradeStateForEvent;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public List<? extends OtherPaymentReport> evaluate(TransactionReportInstruction input) {
		List<OtherPaymentReport.OtherPaymentReportBuilder> outputBuilder = doEvaluate(input);
		
		final List<? extends OtherPaymentReport> output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.stream().map(OtherPaymentReport::build).collect(Collectors.toList());
			objectValidator.validate(OtherPaymentReport.class, output);
		}
		
		return output;
	}

	protected abstract List<OtherPaymentReport.OtherPaymentReportBuilder> doEvaluate(TransactionReportInstruction input);

	public static class OtherPaymentRuleDefault extends OtherPaymentRule {
		@Override
		protected List<OtherPaymentReport.OtherPaymentReportBuilder> doEvaluate(TransactionReportInstruction input) {
			List<OtherPaymentReport.OtherPaymentReportBuilder> output = new ArrayList<>();
			return assignOutput(output, input);
		}
		
		protected List<OtherPaymentReport.OtherPaymentReportBuilder> assignOutput(List<OtherPaymentReport.OtherPaymentReportBuilder> output, TransactionReportInstruction input) {
			final MapperS<TradeState> thenResult0 = MapperS.of(input)
				.mapSingleToItem(item -> MapperS.of(tradeStateForEvent.evaluate(item.get())));
			final MapperC<TransferState> thenResult1 = thenResult0
				.mapSingleToList(item -> item.<TransferState>mapC("getTransferHistory", tradeState -> tradeState.getTransferHistory()));
			output = toBuilder(thenResult1
				.mapItem(item -> MapperS.of(OtherPaymentReport.builder()
					.setPaymentType(otherPaymentType.evaluate(item.get()))
					.setAmount(otherPaymentAmount.evaluate(item.get()))
					.setCurrency(otherPaymentCurrency.evaluate(item.get()))
					.setDate(otherPaymentDate.evaluate(item.get()))
					.setPayer(otherPaymentPayer.evaluate(item.get()))
					.setPayerFormat(otherPaymentPayerFormat.evaluate(item.get()))
					.setReceiver(otherPaymentReceiver.evaluate(item.get()))
					.setReceiverFormat(otherPaymentReceiverFormat.evaluate(item.get()))
					.build()
				)).getMulti());
			
			return Optional.ofNullable(output)
				.map(o -> o.stream().map(i -> i.prune()).collect(Collectors.toList()))
				.orElse(null);
		}
	}
}
