package drr.regulation.cftc.rewrite.reports;

import cdm.event.common.Trade;
import cdm.product.qualification.functions.Qualify_Commodity_Option;
import cdm.product.qualification.functions.Qualify_Commodity_Swap_Basis;
import cdm.product.qualification.functions.Qualify_Commodity_Swap_FixedFloat;
import cdm.product.qualification.functions.Qualify_Commodity_Swaption;
import cdm.product.template.ContractualProduct;
import cdm.product.template.EconomicTerms;
import cdm.product.template.OptionPayout;
import cdm.product.template.Payout;
import cdm.product.template.TradableProduct;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.CommodityLeg1;
import drr.regulation.common.functions.FixedPriceLeg1;
import drr.regulation.common.functions.IsCommodity;
import drr.regulation.common.functions.IsCommodityCommodity;
import drr.regulation.common.functions.IsEquity;
import drr.regulation.common.functions.ProductForTrade;
import drr.regulation.common.functions.TradeForEvent;
import drr.regulation.common.functions.UnderlierForProduct;
import drr.standards.iosco.cde.reports.CDECommodityTotalNotionalQuantityRule;
import drr.standards.iosco.cde.reports.CDEEquityTotalNotionalQuantityRule;
import drr.standards.iosco.cde.reports.CDEFixedPriceTotalNotionalQuantityRule;
import drr.standards.iosco.cde.reports.CDEOptionTotalNotionalQuantityRule;
import java.math.BigDecimal;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(TotalNotionalQuantityLeg1Rule.TotalNotionalQuantityLeg1RuleDefault.class)
public abstract class TotalNotionalQuantityLeg1Rule implements ReportFunction<TransactionReportInstruction, BigDecimal> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDECommodityTotalNotionalQuantityRule cDECommodityTotalNotionalQuantity;
	@Inject protected CDEEquityTotalNotionalQuantityRule cDEEquityTotalNotionalQuantity;
	@Inject protected CDEFixedPriceTotalNotionalQuantityRule cDEFixedPriceTotalNotionalQuantity;
	@Inject protected CDEOptionTotalNotionalQuantityRule cDEOptionTotalNotionalQuantity;
	@Inject protected CommodityLeg1 commodityLeg1;
	@Inject protected FixedPriceLeg1 fixedPriceLeg1;
	@Inject protected IsCommodity isCommodity;
	@Inject protected IsCommodityCommodity isCommodityCommodity;
	@Inject protected IsEquity isEquity;
	@Inject protected ProductForTrade productForTrade;
	@Inject protected Qualify_Commodity_Option qualify_Commodity_Option;
	@Inject protected Qualify_Commodity_Swap_Basis qualify_Commodity_Swap_Basis;
	@Inject protected Qualify_Commodity_Swap_FixedFloat qualify_Commodity_Swap_FixedFloat;
	@Inject protected Qualify_Commodity_Swaption qualify_Commodity_Swaption;
	@Inject protected TradeForEvent tradeForEvent;
	@Inject protected UnderlierForProduct underlierForProduct;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public BigDecimal evaluate(TransactionReportInstruction input) {
		BigDecimal output = doEvaluate(input);
		
		return output;
	}

	protected abstract BigDecimal doEvaluate(TransactionReportInstruction input);

	public static class TotalNotionalQuantityLeg1RuleDefault extends TotalNotionalQuantityLeg1Rule {
		@Override
		protected BigDecimal doEvaluate(TransactionReportInstruction input) {
			BigDecimal output = null;
			return assignOutput(output, input);
		}
		
		protected BigDecimal assignOutput(BigDecimal output, TransactionReportInstruction input) {
			final MapperS<Trade> thenResult = MapperS.of(input)
				.mapSingleToItem(item -> MapperS.of(tradeForEvent.evaluate(item.get())));
			output = thenResult
				.mapSingleToItem(item -> {
					if (areEqual(MapperS.of(isEquity.evaluate(productForTrade.evaluate(item.get()))), MapperS.of(true), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of(cDEEquityTotalNotionalQuantity.evaluate(item.<TradableProduct>map("getTradableProduct", trade -> trade.getTradableProduct()).get()));
					}
					final Boolean boolean0 = isCommodityCommodity.evaluate(productForTrade.evaluate(item.get()));
					if ((boolean0 == null ? false : boolean0)) {
						return MapperS.of(cDECommodityTotalNotionalQuantity.evaluate(commodityLeg1.evaluate(productForTrade.evaluate(item.get()))));
					}
					if (areEqual(MapperS.of(isCommodity.evaluate(productForTrade.evaluate(item.get()))), MapperS.of(true), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of(cDEFixedPriceTotalNotionalQuantity.evaluate(fixedPriceLeg1.evaluate(productForTrade.evaluate(item.get()))));
					}
					if (ComparisonResult.of(MapperS.of(qualify_Commodity_Swaption.evaluate(MapperS.of(productForTrade.evaluate(item.get())).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get()))).and(ComparisonResult.of(MapperS.of(qualify_Commodity_Swap_Basis.evaluate(MapperS.of(underlierForProduct.evaluate(productForTrade.evaluate(item.get()))).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get())))).getOrDefault(false)) {
						return MapperS.of(cDECommodityTotalNotionalQuantity.evaluate(commodityLeg1.evaluate(underlierForProduct.evaluate(productForTrade.evaluate(item.get())))));
					}
					if (ComparisonResult.of(MapperS.of(qualify_Commodity_Swaption.evaluate(MapperS.of(productForTrade.evaluate(item.get())).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get()))).and(ComparisonResult.of(MapperS.of(qualify_Commodity_Swap_FixedFloat.evaluate(MapperS.of(underlierForProduct.evaluate(productForTrade.evaluate(item.get()))).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get())))).getOrDefault(false)) {
						return MapperS.of(cDEFixedPriceTotalNotionalQuantity.evaluate(fixedPriceLeg1.evaluate(underlierForProduct.evaluate(productForTrade.evaluate(item.get())))));
					}
					final Boolean boolean1 = qualify_Commodity_Option.evaluate(MapperS.of(productForTrade.evaluate(item.get())).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get());
					if ((boolean1 == null ? false : boolean1)) {
						return MapperS.of(cDEOptionTotalNotionalQuantity.evaluate(MapperS.of(productForTrade.evaluate(item.get())).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<OptionPayout>mapC("getOptionPayout", payout -> payout.getOptionPayout()).get()));
					}
					return MapperS.<BigDecimal>ofNull();
				}).get();
			
			return output;
		}
	}
}
