package drr.regulation.cftc.rewrite.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.CFTCNonReportablePart45;
import drr.regulation.common.PartyIdentifierFormatEnum;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class CFTCNonReportablePart45Validator implements Validator<CFTCNonReportablePart45> {

	private List<ComparisonResult> getComparisonResults(CFTCNonReportablePart45 o) {
		return Lists.<ComparisonResult>newArrayList(
				checkCardinality("counterparty1Format", (PartyIdentifierFormatEnum) o.getCounterparty1Format() != null ? 1 : 0, 0, 1), 
				checkCardinality("counterparty2Format", (PartyIdentifierFormatEnum) o.getCounterparty2Format() != null ? 1 : 0, 0, 1), 
				checkCardinality("buyerIdentifierFormat", (PartyIdentifierFormatEnum) o.getBuyerIdentifierFormat() != null ? 1 : 0, 0, 1), 
				checkCardinality("sellerIdentifierFormat", (PartyIdentifierFormatEnum) o.getSellerIdentifierFormat() != null ? 1 : 0, 0, 1), 
				checkCardinality("payerIdentifierLeg1Format", (PartyIdentifierFormatEnum) o.getPayerIdentifierLeg1Format() != null ? 1 : 0, 0, 1), 
				checkCardinality("payerIdentifierLeg2Format", (PartyIdentifierFormatEnum) o.getPayerIdentifierLeg2Format() != null ? 1 : 0, 0, 1), 
				checkCardinality("receiverIdentifierLeg1Format", (PartyIdentifierFormatEnum) o.getReceiverIdentifierLeg1Format() != null ? 1 : 0, 0, 1), 
				checkCardinality("receiverIdentifierLeg2Format", (PartyIdentifierFormatEnum) o.getReceiverIdentifierLeg2Format() != null ? 1 : 0, 0, 1)
			);
	}

	@Override
	public ValidationResult<CFTCNonReportablePart45> validate(RosettaPath path, CFTCNonReportablePart45 o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("CFTCNonReportablePart45", ValidationType.CARDINALITY, "CFTCNonReportablePart45", path, "", error);
		}
		return success("CFTCNonReportablePart45", ValidationType.CARDINALITY, "CFTCNonReportablePart45", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, CFTCNonReportablePart45 o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("CFTCNonReportablePart45", ValidationType.CARDINALITY, "CFTCNonReportablePart45", path, "", res.getError());
				}
				return success("CFTCNonReportablePart45", ValidationType.CARDINALITY, "CFTCNonReportablePart45", path, "");
			})
			.collect(toList());
	}

}
