package drr.regulation.cftc.rewrite.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.ClearingExceptionsAndExemptionsCounterparty1Report;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkString;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class ClearingExceptionsAndExemptionsCounterparty1ReportTypeFormatValidator implements Validator<ClearingExceptionsAndExemptionsCounterparty1Report> {

	private List<ComparisonResult> getComparisonResults(ClearingExceptionsAndExemptionsCounterparty1Report o) {
		return Lists.<ComparisonResult>newArrayList(
				checkString("value", o.getValue(), 1, of(4), empty())
			);
	}

	@Override
	public ValidationResult<ClearingExceptionsAndExemptionsCounterparty1Report> validate(RosettaPath path, ClearingExceptionsAndExemptionsCounterparty1Report o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("ClearingExceptionsAndExemptionsCounterparty1Report", ValidationType.TYPE_FORMAT, "ClearingExceptionsAndExemptionsCounterparty1Report", path, "", error);
		}
		return success("ClearingExceptionsAndExemptionsCounterparty1Report", ValidationType.TYPE_FORMAT, "ClearingExceptionsAndExemptionsCounterparty1Report", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, ClearingExceptionsAndExemptionsCounterparty1Report o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("ClearingExceptionsAndExemptionsCounterparty1Report", ValidationType.TYPE_FORMAT, "ClearingExceptionsAndExemptionsCounterparty1Report", path, "", res.getError());
				}
				return success("ClearingExceptionsAndExemptionsCounterparty1Report", ValidationType.TYPE_FORMAT, "ClearingExceptionsAndExemptionsCounterparty1Report", path, "");
			})
			.collect(toList());
	}

}
