package drr.regulation.cftc.rewrite.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.ClearingSwapUSIsReport;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class ClearingSwapUSIsReportValidator implements Validator<ClearingSwapUSIsReport> {

	private List<ComparisonResult> getComparisonResults(ClearingSwapUSIsReport o) {
		return Lists.<ComparisonResult>newArrayList(
				checkCardinality("identifier", (String) o.getIdentifier() != null ? 1 : 0, 1, 1)
			);
	}

	@Override
	public ValidationResult<ClearingSwapUSIsReport> validate(RosettaPath path, ClearingSwapUSIsReport o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("ClearingSwapUSIsReport", ValidationType.CARDINALITY, "ClearingSwapUSIsReport", path, "", error);
		}
		return success("ClearingSwapUSIsReport", ValidationType.CARDINALITY, "ClearingSwapUSIsReport", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, ClearingSwapUSIsReport o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("ClearingSwapUSIsReport", ValidationType.CARDINALITY, "ClearingSwapUSIsReport", path, "", res.getError());
				}
				return success("ClearingSwapUSIsReport", ValidationType.CARDINALITY, "ClearingSwapUSIsReport", path, "");
			})
			.collect(toList());
	}

}
