package drr.regulation.cftc.rewrite.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.ClearingSwapUTIsReport;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class ClearingSwapUTIsReportValidator implements Validator<ClearingSwapUTIsReport> {

	private List<ComparisonResult> getComparisonResults(ClearingSwapUTIsReport o) {
		return Lists.<ComparisonResult>newArrayList(
				checkCardinality("identifier", (String) o.getIdentifier() != null ? 1 : 0, 1, 1)
			);
	}

	@Override
	public ValidationResult<ClearingSwapUTIsReport> validate(RosettaPath path, ClearingSwapUTIsReport o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("ClearingSwapUTIsReport", ValidationType.CARDINALITY, "ClearingSwapUTIsReport", path, "", error);
		}
		return success("ClearingSwapUTIsReport", ValidationType.CARDINALITY, "ClearingSwapUTIsReport", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, ClearingSwapUTIsReport o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("ClearingSwapUTIsReport", ValidationType.CARDINALITY, "ClearingSwapUTIsReport", path, "", res.getError());
				}
				return success("ClearingSwapUTIsReport", ValidationType.CARDINALITY, "ClearingSwapUTIsReport", path, "");
			})
			.collect(toList());
	}

}
