package drr.regulation.cftc.rewrite.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.NotionalAmountScheduleLeg2Report;
import java.math.BigDecimal;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class NotionalAmountScheduleLeg2ReportValidator implements Validator<NotionalAmountScheduleLeg2Report> {

	private List<ComparisonResult> getComparisonResults(NotionalAmountScheduleLeg2Report o) {
		return Lists.<ComparisonResult>newArrayList(
				checkCardinality("amount", (BigDecimal) o.getAmount() != null ? 1 : 0, 1, 1), 
				checkCardinality("effectiveDate", (Date) o.getEffectiveDate() != null ? 1 : 0, 1, 1)
			);
	}

	@Override
	public ValidationResult<NotionalAmountScheduleLeg2Report> validate(RosettaPath path, NotionalAmountScheduleLeg2Report o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("NotionalAmountScheduleLeg2Report", ValidationType.CARDINALITY, "NotionalAmountScheduleLeg2Report", path, "", error);
		}
		return success("NotionalAmountScheduleLeg2Report", ValidationType.CARDINALITY, "NotionalAmountScheduleLeg2Report", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, NotionalAmountScheduleLeg2Report o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("NotionalAmountScheduleLeg2Report", ValidationType.CARDINALITY, "NotionalAmountScheduleLeg2Report", path, "", res.getError());
				}
				return success("NotionalAmountScheduleLeg2Report", ValidationType.CARDINALITY, "NotionalAmountScheduleLeg2Report", path, "");
			})
			.collect(toList());
	}

}
