package drr.regulation.cftc.rewrite.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.CFTCGuidebookPart43;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("CFTCGuidebookPart43SefOrDcmAnonymousIndicator")
@ImplementedBy(CFTCGuidebookPart43SefOrDcmAnonymousIndicator.Default.class)
public interface CFTCGuidebookPart43SefOrDcmAnonymousIndicator extends Validator<CFTCGuidebookPart43> {
	
	String NAME = "CFTCGuidebookPart43SefOrDcmAnonymousIndicator";
	String DEFINITION = "if sefOrDcmIndicator = True then sefOrDcmAnonymousExecutionIndicator exists";
	
	ValidationResult<CFTCGuidebookPart43> validate(RosettaPath path, CFTCGuidebookPart43 cFTCGuidebookPart43);
	
	class Default implements CFTCGuidebookPart43SefOrDcmAnonymousIndicator {
	
		@Override
		public ValidationResult<CFTCGuidebookPart43> validate(RosettaPath path, CFTCGuidebookPart43 cFTCGuidebookPart43) {
			ComparisonResult result = executeDataRule(cFTCGuidebookPart43);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCGuidebookPart43", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "CFTCGuidebookPart43", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(CFTCGuidebookPart43 cFTCGuidebookPart43) {
			try {
				if (areEqual(MapperS.of(cFTCGuidebookPart43).<Boolean>map("getSefOrDcmIndicator", _cFTCGuidebookPart43 -> _cFTCGuidebookPart43.getSefOrDcmIndicator()), MapperS.of(true), CardinalityOperator.All).getOrDefault(false)) {
					return exists(MapperS.of(cFTCGuidebookPart43).<Boolean>map("getSefOrDcmAnonymousExecutionIndicator", _cFTCGuidebookPart43 -> _cFTCGuidebookPart43.getSefOrDcmAnonymousExecutionIndicator()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements CFTCGuidebookPart43SefOrDcmAnonymousIndicator {
	
		@Override
		public ValidationResult<CFTCGuidebookPart43> validate(RosettaPath path, CFTCGuidebookPart43 cFTCGuidebookPart43) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCGuidebookPart43", path, DEFINITION);
		}
	}
}
