package drr.regulation.cftc.rewrite.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.CFTCGuidebookPart45;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("CFTCGuidebookPart45SefOrDcmAnonymousIndicator")
@ImplementedBy(CFTCGuidebookPart45SefOrDcmAnonymousIndicator.Default.class)
public interface CFTCGuidebookPart45SefOrDcmAnonymousIndicator extends Validator<CFTCGuidebookPart45> {
	
	String NAME = "CFTCGuidebookPart45SefOrDcmAnonymousIndicator";
	String DEFINITION = "if sefOrDcmIndicator = True then sefOrDcmAnonymousExecutionIndicator exists";
	
	ValidationResult<CFTCGuidebookPart45> validate(RosettaPath path, CFTCGuidebookPart45 cFTCGuidebookPart45);
	
	class Default implements CFTCGuidebookPart45SefOrDcmAnonymousIndicator {
	
		@Override
		public ValidationResult<CFTCGuidebookPart45> validate(RosettaPath path, CFTCGuidebookPart45 cFTCGuidebookPart45) {
			ComparisonResult result = executeDataRule(cFTCGuidebookPart45);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCGuidebookPart45", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "CFTCGuidebookPart45", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(CFTCGuidebookPart45 cFTCGuidebookPart45) {
			try {
				if (areEqual(MapperS.of(cFTCGuidebookPart45).<Boolean>map("getSefOrDcmIndicator", _cFTCGuidebookPart45 -> _cFTCGuidebookPart45.getSefOrDcmIndicator()), MapperS.of(true), CardinalityOperator.All).getOrDefault(false)) {
					return exists(MapperS.of(cFTCGuidebookPart45).<Boolean>map("getSefOrDcmAnonymousExecutionIndicator", _cFTCGuidebookPart45 -> _cFTCGuidebookPart45.getSefOrDcmAnonymousExecutionIndicator()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements CFTCGuidebookPart45SefOrDcmAnonymousIndicator {
	
		@Override
		public ValidationResult<CFTCGuidebookPart45> validate(RosettaPath path, CFTCGuidebookPart45 cFTCGuidebookPart45) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCGuidebookPart45", path, DEFINITION);
		}
	}
}
