package drr.regulation.cftc.rewrite.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.CFTCPart43TransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("CFTCPart43TransactionReportEventTypeCondition")
@ImplementedBy(CFTCPart43TransactionReportEventTypeCondition.Default.class)
public interface CFTCPart43TransactionReportEventTypeCondition extends Validator<CFTCPart43TransactionReport> {
	
	String NAME = "CFTCPart43TransactionReportEventTypeCondition";
	String DEFINITION = "if actionType = \"NEWT\" then [\"TRAD\", \"NOVA\", \"COMP\", \"EXER\", \"CORP\"] any = eventType else if actionType = \"MODI\" then [\"TRAD\", \"NOVA\", \"COMP\", \"CREV\", \"CORP\", \"UPDT\"] any = eventType else if actionType = \"TERM\" then [\"NOVA\", \"COMP\", \"ETRM\", \"CLRG\", \"EXER\", \"CLAL\", \"CREV\", \"CORP\"] any = eventType else [\"CORR\", \"EROR\", \"REVI\", \"VALU\", \"MARU\", \"PRTO\"] any = actionType and eventType is absent";
	
	ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport);
	
	class Default implements CFTCPart43TransactionReportEventTypeCondition {
	
		@Override
		public ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			ComparisonResult result = executeDataRule(cFTCPart43TransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			try {
				if (areEqual(MapperS.of(cFTCPart43TransactionReport).<String>map("getActionType", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getActionType()), MapperS.of("NEWT"), CardinalityOperator.All).getOrDefault(false)) {
					return areEqual(MapperC.<String>of(MapperS.of("TRAD"), MapperS.of("NOVA"), MapperS.of("COMP"), MapperS.of("EXER"), MapperS.of("CORP")), MapperS.of(cFTCPart43TransactionReport).<String>map("getEventType", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getEventType()), CardinalityOperator.Any);
				}
				if (areEqual(MapperS.of(cFTCPart43TransactionReport).<String>map("getActionType", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getActionType()), MapperS.of("MODI"), CardinalityOperator.All).getOrDefault(false)) {
					return areEqual(MapperC.<String>of(MapperS.of("TRAD"), MapperS.of("NOVA"), MapperS.of("COMP"), MapperS.of("CREV"), MapperS.of("CORP"), MapperS.of("UPDT")), MapperS.of(cFTCPart43TransactionReport).<String>map("getEventType", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getEventType()), CardinalityOperator.Any);
				}
				if (areEqual(MapperS.of(cFTCPart43TransactionReport).<String>map("getActionType", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getActionType()), MapperS.of("TERM"), CardinalityOperator.All).getOrDefault(false)) {
					return areEqual(MapperC.<String>of(MapperS.of("NOVA"), MapperS.of("COMP"), MapperS.of("ETRM"), MapperS.of("CLRG"), MapperS.of("EXER"), MapperS.of("CLAL"), MapperS.of("CREV"), MapperS.of("CORP")), MapperS.of(cFTCPart43TransactionReport).<String>map("getEventType", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getEventType()), CardinalityOperator.Any);
				}
				return areEqual(MapperC.<String>of(MapperS.of("CORR"), MapperS.of("EROR"), MapperS.of("REVI"), MapperS.of("VALU"), MapperS.of("MARU"), MapperS.of("PRTO")), MapperS.of(cFTCPart43TransactionReport).<String>map("getActionType", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getActionType()), CardinalityOperator.Any).and(notExists(MapperS.of(cFTCPart43TransactionReport).<String>map("getEventType", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getEventType())));
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements CFTCPart43TransactionReportEventTypeCondition {
	
		@Override
		public ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION);
		}
	}
}
