package drr.regulation.cftc.rewrite.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.CFTCPart43TransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("CFTCPart43TransactionReportExpirationDateCondition")
@ImplementedBy(CFTCPart43TransactionReportExpirationDateCondition.Default.class)
public interface CFTCPart43TransactionReportExpirationDateCondition extends Validator<CFTCPart43TransactionReport> {
	
	String NAME = "CFTCPart43TransactionReportExpirationDateCondition";
	String DEFINITION = "if expirationDate exists then expirationDate = effectiveDate or expirationDate > effectiveDate else False";
	
	ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport);
	
	class Default implements CFTCPart43TransactionReportExpirationDateCondition {
	
		@Override
		public ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			ComparisonResult result = executeDataRule(cFTCPart43TransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			try {
				if (exists(MapperS.of(cFTCPart43TransactionReport).<Date>map("getExpirationDate", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getExpirationDate())).getOrDefault(false)) {
					return areEqual(MapperS.of(cFTCPart43TransactionReport).<Date>map("getExpirationDate", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getExpirationDate()), MapperS.of(cFTCPart43TransactionReport).<Date>map("getEffectiveDate", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getEffectiveDate()), CardinalityOperator.All).or(greaterThan(MapperS.of(cFTCPart43TransactionReport).<Date>map("getExpirationDate", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getExpirationDate()), MapperS.of(cFTCPart43TransactionReport).<Date>map("getEffectiveDate", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getEffectiveDate()), CardinalityOperator.All));
				}
				return ComparisonResult.of(MapperS.of(false));
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements CFTCPart43TransactionReportExpirationDateCondition {
	
		@Override
		public ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION);
		}
	}
}
