package drr.regulation.cftc.rewrite.validation.datarule;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.CFTCPart43TransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("CFTCPart43TransactionReportPackageTransactionPriceCurrencyCondition")
@ImplementedBy(CFTCPart43TransactionReportPackageTransactionPriceCurrencyCondition.Default.class)
public interface CFTCPart43TransactionReportPackageTransactionPriceCurrencyCondition extends Validator<CFTCPart43TransactionReport> {
	
	String NAME = "CFTCPart43TransactionReportPackageTransactionPriceCurrencyCondition";
	String DEFINITION = "if packageTransactionPriceNotation = \"1\" then packageTransactionPriceCurrency exists else packageTransactionPriceCurrency is absent";
	
	ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport);
	
	class Default implements CFTCPart43TransactionReportPackageTransactionPriceCurrencyCondition {
	
		@Override
		public ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			ComparisonResult result = executeDataRule(cFTCPart43TransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			try {
				if (areEqual(MapperS.of(cFTCPart43TransactionReport).<String>map("getPackageTransactionPriceNotation", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getPackageTransactionPriceNotation()), MapperS.of("1"), CardinalityOperator.All).getOrDefault(false)) {
					return exists(MapperS.of(cFTCPart43TransactionReport).<ISOCurrencyCodeEnum>map("getPackageTransactionPriceCurrency", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getPackageTransactionPriceCurrency()));
				}
				return notExists(MapperS.of(cFTCPart43TransactionReport).<ISOCurrencyCodeEnum>map("getPackageTransactionPriceCurrency", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getPackageTransactionPriceCurrency()));
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements CFTCPart43TransactionReportPackageTransactionPriceCurrencyCondition {
	
		@Override
		public ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION);
		}
	}
}
