package drr.regulation.cftc.rewrite.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.CFTCGuidebookPart43;
import drr.regulation.cftc.rewrite.CFTCPart43TransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("CFTCPart43TransactionReportPaymentFrequencyPeriodMultiplierCondition")
@ImplementedBy(CFTCPart43TransactionReportPaymentFrequencyPeriodMultiplierCondition.Default.class)
public interface CFTCPart43TransactionReportPaymentFrequencyPeriodMultiplierCondition extends Validator<CFTCPart43TransactionReport> {
	
	String NAME = "CFTCPart43TransactionReportPaymentFrequencyPeriodMultiplierCondition";
	String DEFINITION = "if [\"Credit\", \"InterestRate\", \"Equity\", \"Commodity\"] any = cftcGuidebook -> assetClass then ((if fixedRatePaymentFrequencyPeriodLeg1 exists and fixedRatePaymentFrequencyPeriodLeg1 <> \"ADHO\" then fixedRatePaymentFrequencyPeriodMultiplierLeg1 exists else if floatingRatePaymentFrequencyPeriodLeg1 exists and floatingRatePaymentFrequencyPeriodLeg1 <> \"ADHO\" then floatingRatePaymentFrequencyPeriodMultiplierLeg1 exists else fixedRatePaymentFrequencyPeriodMultiplierLeg1 is absent and floatingRatePaymentFrequencyPeriodMultiplierLeg1 is absent) and (if fixedRatePaymentFrequencyPeriodLeg2 exists and fixedRatePaymentFrequencyPeriodLeg2 <> \"ADHO\" then fixedRatePaymentFrequencyPeriodMultiplierLeg2 exists else if floatingRatePaymentFrequencyPeriodLeg2 exists and floatingRatePaymentFrequencyPeriodLeg2 <> \"ADHO\" then floatingRatePaymentFrequencyPeriodMultiplierLeg2 exists else fixedRatePaymentFrequencyPeriodMultiplierLeg2 is absent and floatingRatePaymentFrequencyPeriodMultiplierLeg2 is absent)) else fixedRatePaymentFrequencyPeriodMultiplierLeg1 is absent and floatingRatePaymentFrequencyPeriodMultiplierLeg1 is absent and fixedRatePaymentFrequencyPeriodMultiplierLeg2 is absent and floatingRatePaymentFrequencyPeriodMultiplierLeg2 is absent";
	
	ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport);
	
	class Default implements CFTCPart43TransactionReportPaymentFrequencyPeriodMultiplierCondition {
	
		@Override
		public ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			ComparisonResult result = executeDataRule(cFTCPart43TransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("Credit"), MapperS.of("InterestRate"), MapperS.of("Equity"), MapperS.of("Commodity")), MapperS.of(cFTCPart43TransactionReport).<CFTCGuidebookPart43>map("getCftcGuidebook", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getCftcGuidebook()).<String>map("getAssetClass", cFTCGuidebookPart43 -> cFTCGuidebookPart43.getAssetClass()), CardinalityOperator.Any).getOrDefault(false)) {
					final ComparisonResult ifThenElseResult0;
					if (exists(MapperS.of(cFTCPart43TransactionReport).<String>map("getFixedRatePaymentFrequencyPeriodLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRatePaymentFrequencyPeriodLeg1())).and(notEqual(MapperS.of(cFTCPart43TransactionReport).<String>map("getFixedRatePaymentFrequencyPeriodLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRatePaymentFrequencyPeriodLeg1()), MapperS.of("ADHO"), CardinalityOperator.Any)).getOrDefault(false)) {
						ifThenElseResult0 = exists(MapperS.of(cFTCPart43TransactionReport).<Integer>map("getFixedRatePaymentFrequencyPeriodMultiplierLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRatePaymentFrequencyPeriodMultiplierLeg1()));
					} else if (exists(MapperS.of(cFTCPart43TransactionReport).<String>map("getFloatingRatePaymentFrequencyPeriodLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFloatingRatePaymentFrequencyPeriodLeg1())).and(notEqual(MapperS.of(cFTCPart43TransactionReport).<String>map("getFloatingRatePaymentFrequencyPeriodLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFloatingRatePaymentFrequencyPeriodLeg1()), MapperS.of("ADHO"), CardinalityOperator.Any)).getOrDefault(false)) {
						ifThenElseResult0 = exists(MapperS.of(cFTCPart43TransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierLeg1()));
					} else {
						ifThenElseResult0 = notExists(MapperS.of(cFTCPart43TransactionReport).<Integer>map("getFixedRatePaymentFrequencyPeriodMultiplierLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRatePaymentFrequencyPeriodMultiplierLeg1())).and(notExists(MapperS.of(cFTCPart43TransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierLeg1())));
					}
					final ComparisonResult ifThenElseResult1;
					if (exists(MapperS.of(cFTCPart43TransactionReport).<String>map("getFixedRatePaymentFrequencyPeriodLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRatePaymentFrequencyPeriodLeg2())).and(notEqual(MapperS.of(cFTCPart43TransactionReport).<String>map("getFixedRatePaymentFrequencyPeriodLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRatePaymentFrequencyPeriodLeg2()), MapperS.of("ADHO"), CardinalityOperator.Any)).getOrDefault(false)) {
						ifThenElseResult1 = exists(MapperS.of(cFTCPart43TransactionReport).<Integer>map("getFixedRatePaymentFrequencyPeriodMultiplierLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRatePaymentFrequencyPeriodMultiplierLeg2()));
					} else if (exists(MapperS.of(cFTCPart43TransactionReport).<String>map("getFloatingRatePaymentFrequencyPeriodLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFloatingRatePaymentFrequencyPeriodLeg2())).and(notEqual(MapperS.of(cFTCPart43TransactionReport).<String>map("getFloatingRatePaymentFrequencyPeriodLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFloatingRatePaymentFrequencyPeriodLeg2()), MapperS.of("ADHO"), CardinalityOperator.Any)).getOrDefault(false)) {
						ifThenElseResult1 = exists(MapperS.of(cFTCPart43TransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierLeg2()));
					} else {
						ifThenElseResult1 = notExists(MapperS.of(cFTCPart43TransactionReport).<Integer>map("getFixedRatePaymentFrequencyPeriodMultiplierLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRatePaymentFrequencyPeriodMultiplierLeg2())).and(notExists(MapperS.of(cFTCPart43TransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierLeg2())));
					}
					return ifThenElseResult0.and(ifThenElseResult1);
				}
				return notExists(MapperS.of(cFTCPart43TransactionReport).<Integer>map("getFixedRatePaymentFrequencyPeriodMultiplierLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRatePaymentFrequencyPeriodMultiplierLeg1())).and(notExists(MapperS.of(cFTCPart43TransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierLeg1()))).and(notExists(MapperS.of(cFTCPart43TransactionReport).<Integer>map("getFixedRatePaymentFrequencyPeriodMultiplierLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRatePaymentFrequencyPeriodMultiplierLeg2()))).and(notExists(MapperS.of(cFTCPart43TransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierLeg2())));
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements CFTCPart43TransactionReportPaymentFrequencyPeriodMultiplierCondition {
	
		@Override
		public ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION);
		}
	}
}
