package drr.regulation.cftc.rewrite.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.CFTCGuidebookPart43;
import drr.regulation.cftc.rewrite.CFTCPart43TransactionReport;
import drr.regulation.cftc.rewrite.OtherPaymentReport;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("CFTCPart43TransactionReportSpreadCondition")
@ImplementedBy(CFTCPart43TransactionReportSpreadCondition.Default.class)
public interface CFTCPart43TransactionReportSpreadCondition extends Validator<CFTCPart43TransactionReport> {
	
	String NAME = "CFTCPart43TransactionReportSpreadCondition";
	String DEFINITION = "if cftcGuidebook -> assetClass = \"Credit\" then (if fixedRateLeg1 is absent and otherPayment -> paymentType any <> \"UFRO\" and postPricedSwapIndicator = False and cftcGuidebook -> optionStyle is absent then spreadLeg1 exists else spreadLeg1 is absent) and (if fixedRateLeg2 is absent and otherPayment -> paymentType any <> \"UFRO\" and postPricedSwapIndicator = False and cftcGuidebook -> optionStyle is absent then spreadLeg2 exists else spreadLeg2 is absent) else if cftcGuidebook -> assetClass = \"InterestRate\" then (if fixedRateLeg1 is absent and postPricedSwapIndicator = False and cftcGuidebook -> optionStyle is absent then spreadLeg1 exists else spreadLeg1 is absent) and (if fixedRateLeg2 is absent and postPricedSwapIndicator = False and cftcGuidebook -> optionStyle is absent then spreadLeg2 exists else spreadLeg2 is absent) else if cftcGuidebook -> assetClass = \"Equity\" and (price is absent and postPricedSwapIndicator = False and cftcGuidebook -> optionStyle is absent) then spreadLeg1 exists or spreadLeg2 exists else if cftcGuidebook -> assetClass = \"Commodity\" then (if (price is absent or fixedRateLeg1 is absent) and postPricedSwapIndicator = False and cftcGuidebook -> optionStyle is absent then spreadLeg1 exists else spreadLeg1 is absent) and (if (price is absent or fixedRateLeg2 is absent) and postPricedSwapIndicator = False and cftcGuidebook -> optionStyle is absent then spreadLeg2 exists else spreadLeg2 is absent) else spreadLeg1 is absent and spreadLeg2 is absent";
	
	ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport);
	
	class Default implements CFTCPart43TransactionReportSpreadCondition {
	
		@Override
		public ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			ComparisonResult result = executeDataRule(cFTCPart43TransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			try {
				if (areEqual(MapperS.of(cFTCPart43TransactionReport).<CFTCGuidebookPart43>map("getCftcGuidebook", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getCftcGuidebook()).<String>map("getAssetClass", cFTCGuidebookPart43 -> cFTCGuidebookPart43.getAssetClass()), MapperS.of("Credit"), CardinalityOperator.All).getOrDefault(false)) {
					final ComparisonResult ifThenElseResult0;
					if (notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getFixedRateLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRateLeg1())).and(notEqual(MapperS.of(cFTCPart43TransactionReport).<OtherPaymentReport>mapC("getOtherPayment", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getOtherPayment()).<String>map("getPaymentType", otherPaymentReport -> otherPaymentReport.getPaymentType()), MapperS.of("UFRO"), CardinalityOperator.Any)).and(areEqual(MapperS.of(cFTCPart43TransactionReport).<Boolean>map("getPostPricedSwapIndicator", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getPostPricedSwapIndicator()), MapperS.of(false), CardinalityOperator.All)).and(notExists(MapperS.of(cFTCPart43TransactionReport).<CFTCGuidebookPart43>map("getCftcGuidebook", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getCftcGuidebook()).<String>map("getOptionStyle", cFTCGuidebookPart43 -> cFTCGuidebookPart43.getOptionStyle()))).getOrDefault(false)) {
						ifThenElseResult0 = exists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg1()));
					} else {
						ifThenElseResult0 = notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg1()));
					}
					final ComparisonResult ifThenElseResult1;
					if (notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getFixedRateLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRateLeg2())).and(notEqual(MapperS.of(cFTCPart43TransactionReport).<OtherPaymentReport>mapC("getOtherPayment", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getOtherPayment()).<String>map("getPaymentType", otherPaymentReport -> otherPaymentReport.getPaymentType()), MapperS.of("UFRO"), CardinalityOperator.Any)).and(areEqual(MapperS.of(cFTCPart43TransactionReport).<Boolean>map("getPostPricedSwapIndicator", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getPostPricedSwapIndicator()), MapperS.of(false), CardinalityOperator.All)).and(notExists(MapperS.of(cFTCPart43TransactionReport).<CFTCGuidebookPart43>map("getCftcGuidebook", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getCftcGuidebook()).<String>map("getOptionStyle", cFTCGuidebookPart43 -> cFTCGuidebookPart43.getOptionStyle()))).getOrDefault(false)) {
						ifThenElseResult1 = exists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg2()));
					} else {
						ifThenElseResult1 = notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg2()));
					}
					return ifThenElseResult0.and(ifThenElseResult1);
				}
				if (areEqual(MapperS.of(cFTCPart43TransactionReport).<CFTCGuidebookPart43>map("getCftcGuidebook", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getCftcGuidebook()).<String>map("getAssetClass", cFTCGuidebookPart43 -> cFTCGuidebookPart43.getAssetClass()), MapperS.of("InterestRate"), CardinalityOperator.All).getOrDefault(false)) {
					final ComparisonResult ifThenElseResult2;
					if (notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getFixedRateLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRateLeg1())).and(areEqual(MapperS.of(cFTCPart43TransactionReport).<Boolean>map("getPostPricedSwapIndicator", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getPostPricedSwapIndicator()), MapperS.of(false), CardinalityOperator.All)).and(notExists(MapperS.of(cFTCPart43TransactionReport).<CFTCGuidebookPart43>map("getCftcGuidebook", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getCftcGuidebook()).<String>map("getOptionStyle", cFTCGuidebookPart43 -> cFTCGuidebookPart43.getOptionStyle()))).getOrDefault(false)) {
						ifThenElseResult2 = exists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg1()));
					} else {
						ifThenElseResult2 = notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg1()));
					}
					final ComparisonResult ifThenElseResult3;
					if (notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getFixedRateLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRateLeg2())).and(areEqual(MapperS.of(cFTCPart43TransactionReport).<Boolean>map("getPostPricedSwapIndicator", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getPostPricedSwapIndicator()), MapperS.of(false), CardinalityOperator.All)).and(notExists(MapperS.of(cFTCPart43TransactionReport).<CFTCGuidebookPart43>map("getCftcGuidebook", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getCftcGuidebook()).<String>map("getOptionStyle", cFTCGuidebookPart43 -> cFTCGuidebookPart43.getOptionStyle()))).getOrDefault(false)) {
						ifThenElseResult3 = exists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg2()));
					} else {
						ifThenElseResult3 = notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg2()));
					}
					return ifThenElseResult2.and(ifThenElseResult3);
				}
				if (areEqual(MapperS.of(cFTCPart43TransactionReport).<CFTCGuidebookPart43>map("getCftcGuidebook", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getCftcGuidebook()).<String>map("getAssetClass", cFTCGuidebookPart43 -> cFTCGuidebookPart43.getAssetClass()), MapperS.of("Equity"), CardinalityOperator.All).and(notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getPrice", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getPrice())).and(areEqual(MapperS.of(cFTCPart43TransactionReport).<Boolean>map("getPostPricedSwapIndicator", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getPostPricedSwapIndicator()), MapperS.of(false), CardinalityOperator.All)).and(notExists(MapperS.of(cFTCPart43TransactionReport).<CFTCGuidebookPart43>map("getCftcGuidebook", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getCftcGuidebook()).<String>map("getOptionStyle", cFTCGuidebookPart43 -> cFTCGuidebookPart43.getOptionStyle())))).getOrDefault(false)) {
					return exists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg1())).or(exists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg2())));
				}
				if (areEqual(MapperS.of(cFTCPart43TransactionReport).<CFTCGuidebookPart43>map("getCftcGuidebook", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getCftcGuidebook()).<String>map("getAssetClass", cFTCGuidebookPart43 -> cFTCGuidebookPart43.getAssetClass()), MapperS.of("Commodity"), CardinalityOperator.All).getOrDefault(false)) {
					final ComparisonResult ifThenElseResult4;
					if (notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getPrice", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getPrice())).or(notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getFixedRateLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRateLeg1()))).and(areEqual(MapperS.of(cFTCPart43TransactionReport).<Boolean>map("getPostPricedSwapIndicator", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getPostPricedSwapIndicator()), MapperS.of(false), CardinalityOperator.All)).and(notExists(MapperS.of(cFTCPart43TransactionReport).<CFTCGuidebookPart43>map("getCftcGuidebook", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getCftcGuidebook()).<String>map("getOptionStyle", cFTCGuidebookPart43 -> cFTCGuidebookPart43.getOptionStyle()))).getOrDefault(false)) {
						ifThenElseResult4 = exists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg1()));
					} else {
						ifThenElseResult4 = notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg1()));
					}
					final ComparisonResult ifThenElseResult5;
					if (notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getPrice", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getPrice())).or(notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getFixedRateLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getFixedRateLeg2()))).and(areEqual(MapperS.of(cFTCPart43TransactionReport).<Boolean>map("getPostPricedSwapIndicator", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getPostPricedSwapIndicator()), MapperS.of(false), CardinalityOperator.All)).and(notExists(MapperS.of(cFTCPart43TransactionReport).<CFTCGuidebookPart43>map("getCftcGuidebook", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getCftcGuidebook()).<String>map("getOptionStyle", cFTCGuidebookPart43 -> cFTCGuidebookPart43.getOptionStyle()))).getOrDefault(false)) {
						ifThenElseResult5 = exists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg2()));
					} else {
						ifThenElseResult5 = notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg2()));
					}
					return ifThenElseResult4.and(ifThenElseResult5);
				}
				return notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg1", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg1())).and(notExists(MapperS.of(cFTCPart43TransactionReport).<BigDecimal>map("getSpreadLeg2", _cFTCPart43TransactionReport -> _cFTCPart43TransactionReport.getSpreadLeg2())));
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements CFTCPart43TransactionReportSpreadCondition {
	
		@Override
		public ValidationResult<CFTCPart43TransactionReport> validate(RosettaPath path, CFTCPart43TransactionReport cFTCPart43TransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart43TransactionReport", path, DEFINITION);
		}
	}
}
