package drr.regulation.cftc.rewrite.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.CFTCPart45TransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("CFTCPart45TransactionReportClearingAcountOriginCondition")
@ImplementedBy(CFTCPart45TransactionReportClearingAcountOriginCondition.Default.class)
public interface CFTCPart45TransactionReportClearingAcountOriginCondition extends Validator<CFTCPart45TransactionReport> {
	
	String NAME = "CFTCPart45TransactionReportClearingAcountOriginCondition";
	String DEFINITION = "if cleared = \"Y\" then clearingAccountOrigin exists else if cleared = \"I\" or cleared = \"N\" then clearingAccountOrigin is absent";
	
	ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport);
	
	class Default implements CFTCPart45TransactionReportClearingAcountOriginCondition {
	
		@Override
		public ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			ComparisonResult result = executeDataRule(cFTCPart45TransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			try {
				if (areEqual(MapperS.of(cFTCPart45TransactionReport).<String>map("getCleared", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getCleared()), MapperS.of("Y"), CardinalityOperator.All).getOrDefault(false)) {
					return exists(MapperS.of(cFTCPart45TransactionReport).<String>map("getClearingAccountOrigin", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getClearingAccountOrigin()));
				}
				if (areEqual(MapperS.of(cFTCPart45TransactionReport).<String>map("getCleared", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getCleared()), MapperS.of("I"), CardinalityOperator.All).or(areEqual(MapperS.of(cFTCPart45TransactionReport).<String>map("getCleared", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getCleared()), MapperS.of("N"), CardinalityOperator.All)).getOrDefault(false)) {
					return notExists(MapperS.of(cFTCPart45TransactionReport).<String>map("getClearingAccountOrigin", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getClearingAccountOrigin()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements CFTCPart45TransactionReportClearingAcountOriginCondition {
	
		@Override
		public ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION);
		}
	}
}
