package drr.regulation.cftc.rewrite.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.CFTCPart45TransactionReport;
import java.time.ZonedDateTime;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("CFTCPart45TransactionReportEventTimestampCondition")
@ImplementedBy(CFTCPart45TransactionReportEventTimestampCondition.Default.class)
public interface CFTCPart45TransactionReportEventTimestampCondition extends Validator<CFTCPart45TransactionReport> {
	
	String NAME = "CFTCPart45TransactionReportEventTimestampCondition";
	String DEFINITION = "eventTimestamp = executionTimestamp or eventTimestamp > executionTimestamp";
	
	ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport);
	
	class Default implements CFTCPart45TransactionReportEventTimestampCondition {
	
		@Override
		public ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			ComparisonResult result = executeDataRule(cFTCPart45TransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			try {
				return areEqual(MapperS.of(cFTCPart45TransactionReport).<ZonedDateTime>map("getEventTimestamp", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getEventTimestamp()), MapperS.of(cFTCPart45TransactionReport).<ZonedDateTime>map("getExecutionTimestamp", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getExecutionTimestamp()), CardinalityOperator.All).or(greaterThan(MapperS.of(cFTCPart45TransactionReport).<ZonedDateTime>map("getEventTimestamp", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getEventTimestamp()), MapperS.of(cFTCPart45TransactionReport).<ZonedDateTime>map("getExecutionTimestamp", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getExecutionTimestamp()), CardinalityOperator.All));
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements CFTCPart45TransactionReportEventTimestampCondition {
	
		@Override
		public ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION);
		}
	}
}
