package drr.regulation.cftc.rewrite.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.CFTCPart45TransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("CFTCPart45TransactionReportFloatingRateResetFrequencyPeriodMultiplierCondition")
@ImplementedBy(CFTCPart45TransactionReportFloatingRateResetFrequencyPeriodMultiplierCondition.Default.class)
public interface CFTCPart45TransactionReportFloatingRateResetFrequencyPeriodMultiplierCondition extends Validator<CFTCPart45TransactionReport> {
	
	String NAME = "CFTCPart45TransactionReportFloatingRateResetFrequencyPeriodMultiplierCondition";
	String DEFINITION = "(if floatingRateResetFrequencyPeriodLeg1 exists and floatingRateResetFrequencyPeriodLeg1 <> \"ADHO\" then floatingRateResetFrequencyPeriodMultiplierLeg1 exists else floatingRateResetFrequencyPeriodMultiplierLeg1 is absent) and (if floatingRateResetFrequencyPeriodLeg2 exists and floatingRateResetFrequencyPeriodLeg2 <> \"ADHO\" then floatingRateResetFrequencyPeriodMultiplierLeg2 exists else floatingRateResetFrequencyPeriodMultiplierLeg2 is absent)";
	
	ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport);
	
	class Default implements CFTCPart45TransactionReportFloatingRateResetFrequencyPeriodMultiplierCondition {
	
		@Override
		public ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			ComparisonResult result = executeDataRule(cFTCPart45TransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			try {
				final ComparisonResult ifThenElseResult0;
				if (exists(MapperS.of(cFTCPart45TransactionReport).<String>map("getFloatingRateResetFrequencyPeriodLeg1", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getFloatingRateResetFrequencyPeriodLeg1())).and(notEqual(MapperS.of(cFTCPart45TransactionReport).<String>map("getFloatingRateResetFrequencyPeriodLeg1", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getFloatingRateResetFrequencyPeriodLeg1()), MapperS.of("ADHO"), CardinalityOperator.Any)).getOrDefault(false)) {
					ifThenElseResult0 = exists(MapperS.of(cFTCPart45TransactionReport).<Integer>map("getFloatingRateResetFrequencyPeriodMultiplierLeg1", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getFloatingRateResetFrequencyPeriodMultiplierLeg1()));
				} else {
					ifThenElseResult0 = notExists(MapperS.of(cFTCPart45TransactionReport).<Integer>map("getFloatingRateResetFrequencyPeriodMultiplierLeg1", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getFloatingRateResetFrequencyPeriodMultiplierLeg1()));
				}
				final ComparisonResult ifThenElseResult1;
				if (exists(MapperS.of(cFTCPart45TransactionReport).<String>map("getFloatingRateResetFrequencyPeriodLeg2", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getFloatingRateResetFrequencyPeriodLeg2())).and(notEqual(MapperS.of(cFTCPart45TransactionReport).<String>map("getFloatingRateResetFrequencyPeriodLeg2", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getFloatingRateResetFrequencyPeriodLeg2()), MapperS.of("ADHO"), CardinalityOperator.Any)).getOrDefault(false)) {
					ifThenElseResult1 = exists(MapperS.of(cFTCPart45TransactionReport).<Integer>map("getFloatingRateResetFrequencyPeriodMultiplierLeg2", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getFloatingRateResetFrequencyPeriodMultiplierLeg2()));
				} else {
					ifThenElseResult1 = notExists(MapperS.of(cFTCPart45TransactionReport).<Integer>map("getFloatingRateResetFrequencyPeriodMultiplierLeg2", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getFloatingRateResetFrequencyPeriodMultiplierLeg2()));
				}
				return ifThenElseResult0.and(ifThenElseResult1);
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements CFTCPart45TransactionReportFloatingRateResetFrequencyPeriodMultiplierCondition {
	
		@Override
		public ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION);
		}
	}
}
