package drr.regulation.cftc.rewrite.validation.datarule;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.CFTCGuidebookPart45;
import drr.regulation.cftc.rewrite.CFTCPart45TransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("CFTCPart45TransactionReportNotionalCurrencyCondition")
@ImplementedBy(CFTCPart45TransactionReportNotionalCurrencyCondition.Default.class)
public interface CFTCPart45TransactionReportNotionalCurrencyCondition extends Validator<CFTCPart45TransactionReport> {
	
	String NAME = "CFTCPart45TransactionReportNotionalCurrencyCondition";
	String DEFINITION = "if cftcGuidebook -> optionStyle exists then ((if notionalCurrencyLeg1 exists then notionalCurrencyLeg1 any = [callCurrency, putCurrency]) and (if notionalCurrencyLeg2 exists then notionalCurrencyLeg2 any = [callCurrency, putCurrency]))";
	
	ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport);
	
	class Default implements CFTCPart45TransactionReportNotionalCurrencyCondition {
	
		@Override
		public ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			ComparisonResult result = executeDataRule(cFTCPart45TransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			try {
				if (exists(MapperS.of(cFTCPart45TransactionReport).<CFTCGuidebookPart45>map("getCftcGuidebook", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getCftcGuidebook()).<String>map("getOptionStyle", cFTCGuidebookPart45 -> cFTCGuidebookPart45.getOptionStyle())).getOrDefault(false)) {
					final ComparisonResult ifThenElseResult0;
					if (exists(MapperS.of(cFTCPart45TransactionReport).<ISOCurrencyCodeEnum>map("getNotionalCurrencyLeg1", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getNotionalCurrencyLeg1())).getOrDefault(false)) {
						ifThenElseResult0 = areEqual(MapperS.of(cFTCPart45TransactionReport).<ISOCurrencyCodeEnum>map("getNotionalCurrencyLeg1", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getNotionalCurrencyLeg1()), MapperC.<ISOCurrencyCodeEnum>of(MapperS.of(cFTCPart45TransactionReport).<ISOCurrencyCodeEnum>map("getCallCurrency", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getCallCurrency()), MapperS.of(cFTCPart45TransactionReport).<ISOCurrencyCodeEnum>map("getPutCurrency", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getPutCurrency())), CardinalityOperator.Any);
					} else {
						ifThenElseResult0 = ComparisonResult.successEmptyOperand("");
					}
					final ComparisonResult ifThenElseResult1;
					if (exists(MapperS.of(cFTCPart45TransactionReport).<ISOCurrencyCodeEnum>map("getNotionalCurrencyLeg2", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getNotionalCurrencyLeg2())).getOrDefault(false)) {
						ifThenElseResult1 = areEqual(MapperS.of(cFTCPart45TransactionReport).<ISOCurrencyCodeEnum>map("getNotionalCurrencyLeg2", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getNotionalCurrencyLeg2()), MapperC.<ISOCurrencyCodeEnum>of(MapperS.of(cFTCPart45TransactionReport).<ISOCurrencyCodeEnum>map("getCallCurrency", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getCallCurrency()), MapperS.of(cFTCPart45TransactionReport).<ISOCurrencyCodeEnum>map("getPutCurrency", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getPutCurrency())), CardinalityOperator.Any);
					} else {
						ifThenElseResult1 = ComparisonResult.successEmptyOperand("");
					}
					return ifThenElseResult0.and(ifThenElseResult1);
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements CFTCPart45TransactionReportNotionalCurrencyCondition {
	
		@Override
		public ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION);
		}
	}
}
