package drr.regulation.cftc.rewrite.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.cftc.rewrite.CFTCGuidebookPart45;
import drr.regulation.cftc.rewrite.CFTCPart45TransactionReport;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("CFTCPart45TransactionReportQuantityFrequencyCondition")
@ImplementedBy(CFTCPart45TransactionReportQuantityFrequencyCondition.Default.class)
public interface CFTCPart45TransactionReportQuantityFrequencyCondition extends Validator<CFTCPart45TransactionReport> {
	
	String NAME = "CFTCPart45TransactionReportQuantityFrequencyCondition";
	String DEFINITION = "if cftcGuidebook -> assetClass = \"Commodity\" then ((if notionalQuantityLeg1 exists then quantityFrequencyLeg1 exists) and (if notionalQuantityLeg1 is absent then quantityFrequencyLeg1 is absent) and (if notionalQuantityLeg2 exists then quantityFrequencyLeg2 exists) and (if notionalQuantityLeg2 is absent then quantityFrequencyLeg2 is absent)) else quantityFrequencyLeg1 is absent and quantityFrequencyLeg2 is absent";
	
	ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport);
	
	class Default implements CFTCPart45TransactionReportQuantityFrequencyCondition {
	
		@Override
		public ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			ComparisonResult result = executeDataRule(cFTCPart45TransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			try {
				if (areEqual(MapperS.of(cFTCPart45TransactionReport).<CFTCGuidebookPart45>map("getCftcGuidebook", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getCftcGuidebook()).<String>map("getAssetClass", cFTCGuidebookPart45 -> cFTCGuidebookPart45.getAssetClass()), MapperS.of("Commodity"), CardinalityOperator.All).getOrDefault(false)) {
					final ComparisonResult ifThenElseResult0;
					if (exists(MapperS.of(cFTCPart45TransactionReport).<BigDecimal>map("getNotionalQuantityLeg1", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getNotionalQuantityLeg1())).getOrDefault(false)) {
						ifThenElseResult0 = exists(MapperS.of(cFTCPart45TransactionReport).<String>map("getQuantityFrequencyLeg1", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getQuantityFrequencyLeg1()));
					} else {
						ifThenElseResult0 = ComparisonResult.successEmptyOperand("");
					}
					final ComparisonResult ifThenElseResult1;
					if (notExists(MapperS.of(cFTCPart45TransactionReport).<BigDecimal>map("getNotionalQuantityLeg1", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getNotionalQuantityLeg1())).getOrDefault(false)) {
						ifThenElseResult1 = notExists(MapperS.of(cFTCPart45TransactionReport).<String>map("getQuantityFrequencyLeg1", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getQuantityFrequencyLeg1()));
					} else {
						ifThenElseResult1 = ComparisonResult.successEmptyOperand("");
					}
					final ComparisonResult ifThenElseResult2;
					if (exists(MapperS.of(cFTCPart45TransactionReport).<BigDecimal>map("getNotionalQuantityLeg2", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getNotionalQuantityLeg2())).getOrDefault(false)) {
						ifThenElseResult2 = exists(MapperS.of(cFTCPart45TransactionReport).<String>map("getQuantityFrequencyLeg2", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getQuantityFrequencyLeg2()));
					} else {
						ifThenElseResult2 = ComparisonResult.successEmptyOperand("");
					}
					final ComparisonResult ifThenElseResult3;
					if (notExists(MapperS.of(cFTCPart45TransactionReport).<BigDecimal>map("getNotionalQuantityLeg2", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getNotionalQuantityLeg2())).getOrDefault(false)) {
						ifThenElseResult3 = notExists(MapperS.of(cFTCPart45TransactionReport).<String>map("getQuantityFrequencyLeg2", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getQuantityFrequencyLeg2()));
					} else {
						ifThenElseResult3 = ComparisonResult.successEmptyOperand("");
					}
					return ifThenElseResult0.and(ifThenElseResult1).and(ifThenElseResult2).and(ifThenElseResult3);
				}
				return notExists(MapperS.of(cFTCPart45TransactionReport).<String>map("getQuantityFrequencyLeg1", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getQuantityFrequencyLeg1())).and(notExists(MapperS.of(cFTCPart45TransactionReport).<String>map("getQuantityFrequencyLeg2", _cFTCPart45TransactionReport -> _cFTCPart45TransactionReport.getQuantityFrequencyLeg2())));
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements CFTCPart45TransactionReportQuantityFrequencyCondition {
	
		@Override
		public ValidationResult<CFTCPart45TransactionReport> validate(RosettaPath path, CFTCPart45TransactionReport cFTCPart45TransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CFTCPart45TransactionReport", path, DEFINITION);
		}
	}
}
