package drr.regulation.common;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import drr.regulation.common.CommodityTimeUnitEnum;
import drr.regulation.common.ESMATransactionInformation;
import drr.regulation.common.ESMATransactionInformation.ESMATransactionInformationBuilder;
import drr.regulation.common.ESMATransactionInformation.ESMATransactionInformationBuilderImpl;
import drr.regulation.common.ESMATransactionInformation.ESMATransactionInformationImpl;
import drr.regulation.common.meta.ESMATransactionInformationMeta;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="ESMATransactionInformation", builder=ESMATransactionInformation.ESMATransactionInformationBuilderImpl.class, version="4.2.1")
public interface ESMATransactionInformation extends RosettaModelObject {

	ESMATransactionInformationMeta metaData = new ESMATransactionInformationMeta();

	/*********************** Getter Methods  ***********************/
	/**
	 * Specifies the approximate duration of the contract in the context of commoditiy trading. Exact boundaries of each value have not been precisely defined by so far. This attribute is expected to be removed when values exact definitions of values emerge and a functional assessment of duration - based upon start and end dates and times - can be implemented.
	 */
	CommodityTimeUnitEnum getDuration();

	/*********************** Build Methods  ***********************/
	ESMATransactionInformation build();
	
	ESMATransactionInformation.ESMATransactionInformationBuilder toBuilder();
	
	static ESMATransactionInformation.ESMATransactionInformationBuilder builder() {
		return new ESMATransactionInformation.ESMATransactionInformationBuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends ESMATransactionInformation> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends ESMATransactionInformation> getType() {
		return ESMATransactionInformation.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("duration"), CommodityTimeUnitEnum.class, getDuration(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface ESMATransactionInformationBuilder extends ESMATransactionInformation, RosettaModelObjectBuilder {
		ESMATransactionInformation.ESMATransactionInformationBuilder setDuration(CommodityTimeUnitEnum duration);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("duration"), CommodityTimeUnitEnum.class, getDuration(), this);
		}
		

		ESMATransactionInformation.ESMATransactionInformationBuilder prune();
	}

	/*********************** Immutable Implementation of ESMATransactionInformation  ***********************/
	class ESMATransactionInformationImpl implements ESMATransactionInformation {
		private final CommodityTimeUnitEnum duration;
		
		protected ESMATransactionInformationImpl(ESMATransactionInformation.ESMATransactionInformationBuilder builder) {
			this.duration = builder.getDuration();
		}
		
		@Override
		@RosettaAttribute("duration")
		public CommodityTimeUnitEnum getDuration() {
			return duration;
		}
		
		@Override
		public ESMATransactionInformation build() {
			return this;
		}
		
		@Override
		public ESMATransactionInformation.ESMATransactionInformationBuilder toBuilder() {
			ESMATransactionInformation.ESMATransactionInformationBuilder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(ESMATransactionInformation.ESMATransactionInformationBuilder builder) {
			ofNullable(getDuration()).ifPresent(builder::setDuration);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			ESMATransactionInformation _that = getType().cast(o);
		
			if (!Objects.equals(duration, _that.getDuration())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (duration != null ? duration.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ESMATransactionInformation {" +
				"duration=" + this.duration +
			'}';
		}
	}

	/*********************** Builder Implementation of ESMATransactionInformation  ***********************/
	class ESMATransactionInformationBuilderImpl implements ESMATransactionInformation.ESMATransactionInformationBuilder {
	
		protected CommodityTimeUnitEnum duration;
	
		public ESMATransactionInformationBuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("duration")
		public CommodityTimeUnitEnum getDuration() {
			return duration;
		}
		
	
		@Override
		@RosettaAttribute("duration")
		public ESMATransactionInformation.ESMATransactionInformationBuilder setDuration(CommodityTimeUnitEnum duration) {
			this.duration = duration==null?null:duration;
			return this;
		}
		
		@Override
		public ESMATransactionInformation build() {
			return new ESMATransactionInformation.ESMATransactionInformationImpl(this);
		}
		
		@Override
		public ESMATransactionInformation.ESMATransactionInformationBuilder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public ESMATransactionInformation.ESMATransactionInformationBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDuration()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public ESMATransactionInformation.ESMATransactionInformationBuilder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			ESMATransactionInformation.ESMATransactionInformationBuilder o = (ESMATransactionInformation.ESMATransactionInformationBuilder) other;
			
			
			merger.mergeBasic(getDuration(), o.getDuration(), this::setDuration);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			ESMATransactionInformation _that = getType().cast(o);
		
			if (!Objects.equals(duration, _that.getDuration())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (duration != null ? duration.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ESMATransactionInformationBuilder {" +
				"duration=" + this.duration +
			'}';
		}
	}
}
