package drr.regulation.common;

import com.rosetta.model.lib.annotations.RosettaEnum;
import com.rosetta.model.lib.annotations.RosettaEnumValue;
import java.util.Collections;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;


/**
 * Floating rate index ISO Code List corresponding to the version published by EMIR Refit as part of its requirements.
 * @version 4.2.1
 */
@RosettaEnum("ISOFloatingRateIndexEnum")
public enum ISOFloatingRateIndexEnum {

	/**
	 * BBSW
	 */
	@RosettaEnumValue(value = "BBSW") BBSW("BBSW"),
	
	/**
	 * BUBOR
	 */
	@RosettaEnumValue(value = "BUBO") BUBO("BUBO"),
	
	/**
	 * CDOR
	 */
	@RosettaEnumValue(value = "CDOR") CDOR("CDOR"),
	
	/**
	 * CIBOR
	 */
	@RosettaEnumValue(value = "CIBO") CIBO("CIBO"),
	
	/**
	 * CORA
	 */
	@RosettaEnumValue(value = "CORA") CORA("CORA"),
	
	/**
	 * Effective Federal Funds Rate OBFR = Overnight Bank Funding Rate
	 */
	@RosettaEnumValue(value = "EFFR") EFFR("EFFR"),
	
	/**
	 * EONIA
	 */
	@RosettaEnumValue(value = "EONA") EONA("EONA"),
	
	/**
	 * EONIA SWAP
	 */
	@RosettaEnumValue(value = "EONS") EONS("EONS"),
	
	/**
	 * STR
	 */
	@RosettaEnumValue(value = "ESTR") ESTR("ESTR"),
	
	/**
	 * EURIBOR
	 */
	@RosettaEnumValue(value = "EURI") EURI("EURI"),
	
	/**
	 * HKIO
	 */
	@RosettaEnumValue(value = "HKIO") HKIO("HKIO"),
	
	/**
	 * ISDAFIX
	 */
	@RosettaEnumValue(value = "ISDA") ISDA("ISDA"),
	
	/**
	 * JIBAR
	 */
	@RosettaEnumValue(value = "JIBA") JIBA("JIBA"),
	
	/**
	 * LIBOR
	 */
	@RosettaEnumValue(value = "LIBO") LIBO("LIBO"),
	
	/**
	 * Muni AAA
	 */
	@RosettaEnumValue(value = "MAAA") MAAA("MAAA"),
	
	/**
	 * MOSPRIM
	 */
	@RosettaEnumValue(value = "MOSP") MOSP("MOSP"),
	
	/**
	 * NIBOR
	 */
	@RosettaEnumValue(value = "NIBO") NIBO("NIBO"),
	
	/**
	 * PRIBOR
	 */
	@RosettaEnumValue(value = "PRBO") PRBO("PRBO"),
	
	/**
	 * SOFR
	 */
	@RosettaEnumValue(value = "SOFR") SOFR("SOFR"),
	
	/**
	 * SONIA
	 */
	@RosettaEnumValue(value = "SONA") SONA("SONA"),
	
	/**
	 * STIBOR
	 */
	@RosettaEnumValue(value = "STBO") STBO("STBO"),
	
	/**
	 * SWAP
	 */
	@RosettaEnumValue(value = "SWAP") SWAP("SWAP"),
	
	/**
	 * TIBOR
	 */
	@RosettaEnumValue(value = "TIBO") TIBO("TIBO"),
	
	/**
	 * TELBOR
	 */
	@RosettaEnumValue(value = "TLBO") TLBO("TLBO"),
	
	/**
	 * Treasury
	 */
	@RosettaEnumValue(value = "TREA") TREA("TREA"),
	
	/**
	 * WIBOR
	 */
	@RosettaEnumValue(value = "WIBO") WIBO("WIBO")
;
	private static Map<String, ISOFloatingRateIndexEnum> values;
	static {
        Map<String, ISOFloatingRateIndexEnum> map = new ConcurrentHashMap<>();
		for (ISOFloatingRateIndexEnum instance : ISOFloatingRateIndexEnum.values()) {
			map.put(instance.toDisplayString(), instance);
		}
		values = Collections.unmodifiableMap(map);
    }

	private final String rosettaName;
	private final String displayName;

	ISOFloatingRateIndexEnum(String rosettaName) {
		this(rosettaName, null);
	}

	ISOFloatingRateIndexEnum(String rosettaName, String displayName) {
		this.rosettaName = rosettaName;
		this.displayName = displayName;
	}

	public static ISOFloatingRateIndexEnum fromDisplayName(String name) {
		ISOFloatingRateIndexEnum value = values.get(name);
		if (value == null) {
			throw new IllegalArgumentException("No enum constant with display name \"" + name + "\".");
		}
		return value;
	}

	@Override
	public String toString() {
		return toDisplayString();
	}

	public String toDisplayString() {
		return displayName != null ?  displayName : rosettaName;
	}
}
