package drr.regulation.common;

import com.rosetta.model.lib.annotations.RosettaEnum;
import com.rosetta.model.lib.annotations.RosettaEnumValue;
import java.util.Collections;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;


/**
 * Taxonomy for Non-Financial Counterparties. The categories below correspond to the main sections of NACE classification as defined in Regulation (EC) No 1893/2006 of the European Parliament and of the Council(10).
 * @version 4.2.1
 */
@RosettaEnum("NonFinancialSectorEnum")
public enum NonFinancialSectorEnum {

	/**
	 * Agriculture, forestry and fishing
	 */
	@RosettaEnumValue(value = "A") A("A"),
	
	/**
	 * Mining and quarrying
	 */
	@RosettaEnumValue(value = "B") B("B"),
	
	/**
	 * Manufacturing
	 */
	@RosettaEnumValue(value = "C") C("C"),
	
	/**
	 * Electricity, gas, steam and air conditioning supply
	 */
	@RosettaEnumValue(value = "D") D("D"),
	
	/**
	 * Water supply, sewerage, waste management and remediation activities. Note that &#39;E&#39; clashes with the DSL keyword for mathematics E notation, so the name is prefixed with character &#39;^&#39; to indicate that it is a name rather than keyword.
	 */
	@RosettaEnumValue(value = "E") E("E"),
	
	/**
	 * Construction
	 */
	@RosettaEnumValue(value = "F") F("F"),
	
	/**
	 * Wholesale and retail trade, repair of motor vehicles and motorcycles
	 */
	@RosettaEnumValue(value = "G") G("G"),
	
	/**
	 * Transportation and storage
	 */
	@RosettaEnumValue(value = "H") H("H"),
	
	/**
	 * Accommodation and food service activities
	 */
	@RosettaEnumValue(value = "I") I("I"),
	
	/**
	 * Information and communication
	 */
	@RosettaEnumValue(value = "J") J("J"),
	
	/**
	 * Financial and insurance activities
	 */
	@RosettaEnumValue(value = "K") K("K"),
	
	/**
	 * Real estate activities
	 */
	@RosettaEnumValue(value = "L") L("L"),
	
	/**
	 * Professional, scientific and technical activities
	 */
	@RosettaEnumValue(value = "M") M("M"),
	
	/**
	 * Administrative and support service activities
	 */
	@RosettaEnumValue(value = "N") N("N"),
	
	/**
	 * Public administration and defence; compulsory social security
	 */
	@RosettaEnumValue(value = "O") O("O"),
	
	/**
	 * Education
	 */
	@RosettaEnumValue(value = "P") P("P"),
	
	/**
	 * Human health and social work activities
	 */
	@RosettaEnumValue(value = "Q") Q("Q"),
	
	/**
	 * Arts, entertainment and recreation
	 */
	@RosettaEnumValue(value = "R") R("R"),
	
	/**
	 * Other service activities
	 */
	@RosettaEnumValue(value = "S") S("S"),
	
	/**
	 * Activities of households as employers; undifferentiated goods  and services  producing activities of households for own use
	 */
	@RosettaEnumValue(value = "T") T("T"),
	
	/**
	 * Activities of extraterritorial organizations and bodies
	 */
	@RosettaEnumValue(value = "U") U("U")
;
	private static Map<String, NonFinancialSectorEnum> values;
	static {
        Map<String, NonFinancialSectorEnum> map = new ConcurrentHashMap<>();
		for (NonFinancialSectorEnum instance : NonFinancialSectorEnum.values()) {
			map.put(instance.toDisplayString(), instance);
		}
		values = Collections.unmodifiableMap(map);
    }

	private final String rosettaName;
	private final String displayName;

	NonFinancialSectorEnum(String rosettaName) {
		this(rosettaName, null);
	}

	NonFinancialSectorEnum(String rosettaName, String displayName) {
		this.rosettaName = rosettaName;
		this.displayName = displayName;
	}

	public static NonFinancialSectorEnum fromDisplayName(String name) {
		NonFinancialSectorEnum value = values.get(name);
		if (value == null) {
			throw new IllegalArgumentException("No enum constant with display name \"" + name + "\".");
		}
		return value;
	}

	@Override
	public String toString() {
		return toDisplayString();
	}

	public String toDisplayString() {
		return displayName != null ?  displayName : rosettaName;
	}
}
