package drr.regulation.common;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import com.rosetta.model.lib.records.Date;
import drr.regulation.common.PriceReportablePeriod;
import drr.regulation.common.PriceReportablePeriod.PriceReportablePeriodBuilder;
import drr.regulation.common.PriceReportablePeriod.PriceReportablePeriodBuilderImpl;
import drr.regulation.common.PriceReportablePeriod.PriceReportablePeriodImpl;
import drr.regulation.common.meta.PriceReportablePeriodMeta;
import drr.regulation.esma.emir.refit.trade.PriceAndNotation;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="PriceReportablePeriod", builder=PriceReportablePeriod.PriceReportablePeriodBuilderImpl.class, version="4.2.1")
public interface PriceReportablePeriod extends RosettaModelObject {

	PriceReportablePeriodMeta metaData = new PriceReportablePeriodMeta();

	/*********************** Getter Methods  ***********************/
	PriceAndNotation getPeriodValue();
	Date getPeriodEffectiveDate();
	Date getPeriodEndDate();

	/*********************** Build Methods  ***********************/
	PriceReportablePeriod build();
	
	PriceReportablePeriod.PriceReportablePeriodBuilder toBuilder();
	
	static PriceReportablePeriod.PriceReportablePeriodBuilder builder() {
		return new PriceReportablePeriod.PriceReportablePeriodBuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends PriceReportablePeriod> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends PriceReportablePeriod> getType() {
		return PriceReportablePeriod.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processRosetta(path.newSubPath("periodValue"), processor, PriceAndNotation.class, getPeriodValue());
		processor.processBasic(path.newSubPath("periodEffectiveDate"), Date.class, getPeriodEffectiveDate(), this);
		processor.processBasic(path.newSubPath("periodEndDate"), Date.class, getPeriodEndDate(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface PriceReportablePeriodBuilder extends PriceReportablePeriod, RosettaModelObjectBuilder {
		PriceAndNotation.PriceAndNotationBuilder getOrCreatePeriodValue();
		PriceAndNotation.PriceAndNotationBuilder getPeriodValue();
		PriceReportablePeriod.PriceReportablePeriodBuilder setPeriodValue(PriceAndNotation periodValue);
		PriceReportablePeriod.PriceReportablePeriodBuilder setPeriodEffectiveDate(Date periodEffectiveDate);
		PriceReportablePeriod.PriceReportablePeriodBuilder setPeriodEndDate(Date periodEndDate);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processRosetta(path.newSubPath("periodValue"), processor, PriceAndNotation.PriceAndNotationBuilder.class, getPeriodValue());
			processor.processBasic(path.newSubPath("periodEffectiveDate"), Date.class, getPeriodEffectiveDate(), this);
			processor.processBasic(path.newSubPath("periodEndDate"), Date.class, getPeriodEndDate(), this);
		}
		

		PriceReportablePeriod.PriceReportablePeriodBuilder prune();
	}

	/*********************** Immutable Implementation of PriceReportablePeriod  ***********************/
	class PriceReportablePeriodImpl implements PriceReportablePeriod {
		private final PriceAndNotation periodValue;
		private final Date periodEffectiveDate;
		private final Date periodEndDate;
		
		protected PriceReportablePeriodImpl(PriceReportablePeriod.PriceReportablePeriodBuilder builder) {
			this.periodValue = ofNullable(builder.getPeriodValue()).map(f->f.build()).orElse(null);
			this.periodEffectiveDate = builder.getPeriodEffectiveDate();
			this.periodEndDate = builder.getPeriodEndDate();
		}
		
		@Override
		@RosettaAttribute("periodValue")
		public PriceAndNotation getPeriodValue() {
			return periodValue;
		}
		
		@Override
		@RosettaAttribute("periodEffectiveDate")
		public Date getPeriodEffectiveDate() {
			return periodEffectiveDate;
		}
		
		@Override
		@RosettaAttribute("periodEndDate")
		public Date getPeriodEndDate() {
			return periodEndDate;
		}
		
		@Override
		public PriceReportablePeriod build() {
			return this;
		}
		
		@Override
		public PriceReportablePeriod.PriceReportablePeriodBuilder toBuilder() {
			PriceReportablePeriod.PriceReportablePeriodBuilder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(PriceReportablePeriod.PriceReportablePeriodBuilder builder) {
			ofNullable(getPeriodValue()).ifPresent(builder::setPeriodValue);
			ofNullable(getPeriodEffectiveDate()).ifPresent(builder::setPeriodEffectiveDate);
			ofNullable(getPeriodEndDate()).ifPresent(builder::setPeriodEndDate);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			PriceReportablePeriod _that = getType().cast(o);
		
			if (!Objects.equals(periodValue, _that.getPeriodValue())) return false;
			if (!Objects.equals(periodEffectiveDate, _that.getPeriodEffectiveDate())) return false;
			if (!Objects.equals(periodEndDate, _that.getPeriodEndDate())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (periodValue != null ? periodValue.hashCode() : 0);
			_result = 31 * _result + (periodEffectiveDate != null ? periodEffectiveDate.hashCode() : 0);
			_result = 31 * _result + (periodEndDate != null ? periodEndDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PriceReportablePeriod {" +
				"periodValue=" + this.periodValue + ", " +
				"periodEffectiveDate=" + this.periodEffectiveDate + ", " +
				"periodEndDate=" + this.periodEndDate +
			'}';
		}
	}

	/*********************** Builder Implementation of PriceReportablePeriod  ***********************/
	class PriceReportablePeriodBuilderImpl implements PriceReportablePeriod.PriceReportablePeriodBuilder {
	
		protected PriceAndNotation.PriceAndNotationBuilder periodValue;
		protected Date periodEffectiveDate;
		protected Date periodEndDate;
	
		public PriceReportablePeriodBuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("periodValue")
		public PriceAndNotation.PriceAndNotationBuilder getPeriodValue() {
			return periodValue;
		}
		
		@Override
		public PriceAndNotation.PriceAndNotationBuilder getOrCreatePeriodValue() {
			PriceAndNotation.PriceAndNotationBuilder result;
			if (periodValue!=null) {
				result = periodValue;
			}
			else {
				result = periodValue = PriceAndNotation.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("periodEffectiveDate")
		public Date getPeriodEffectiveDate() {
			return periodEffectiveDate;
		}
		
		@Override
		@RosettaAttribute("periodEndDate")
		public Date getPeriodEndDate() {
			return periodEndDate;
		}
		
	
		@Override
		@RosettaAttribute("periodValue")
		public PriceReportablePeriod.PriceReportablePeriodBuilder setPeriodValue(PriceAndNotation periodValue) {
			this.periodValue = periodValue==null?null:periodValue.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("periodEffectiveDate")
		public PriceReportablePeriod.PriceReportablePeriodBuilder setPeriodEffectiveDate(Date periodEffectiveDate) {
			this.periodEffectiveDate = periodEffectiveDate==null?null:periodEffectiveDate;
			return this;
		}
		@Override
		@RosettaAttribute("periodEndDate")
		public PriceReportablePeriod.PriceReportablePeriodBuilder setPeriodEndDate(Date periodEndDate) {
			this.periodEndDate = periodEndDate==null?null:periodEndDate;
			return this;
		}
		
		@Override
		public PriceReportablePeriod build() {
			return new PriceReportablePeriod.PriceReportablePeriodImpl(this);
		}
		
		@Override
		public PriceReportablePeriod.PriceReportablePeriodBuilder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public PriceReportablePeriod.PriceReportablePeriodBuilder prune() {
			if (periodValue!=null && !periodValue.prune().hasData()) periodValue = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getPeriodValue()!=null && getPeriodValue().hasData()) return true;
			if (getPeriodEffectiveDate()!=null) return true;
			if (getPeriodEndDate()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public PriceReportablePeriod.PriceReportablePeriodBuilder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			PriceReportablePeriod.PriceReportablePeriodBuilder o = (PriceReportablePeriod.PriceReportablePeriodBuilder) other;
			
			merger.mergeRosetta(getPeriodValue(), o.getPeriodValue(), this::setPeriodValue);
			
			merger.mergeBasic(getPeriodEffectiveDate(), o.getPeriodEffectiveDate(), this::setPeriodEffectiveDate);
			merger.mergeBasic(getPeriodEndDate(), o.getPeriodEndDate(), this::setPeriodEndDate);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			PriceReportablePeriod _that = getType().cast(o);
		
			if (!Objects.equals(periodValue, _that.getPeriodValue())) return false;
			if (!Objects.equals(periodEffectiveDate, _that.getPeriodEffectiveDate())) return false;
			if (!Objects.equals(periodEndDate, _that.getPeriodEndDate())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (periodValue != null ? periodValue.hashCode() : 0);
			_result = 31 * _result + (periodEffectiveDate != null ? periodEffectiveDate.hashCode() : 0);
			_result = 31 * _result + (periodEndDate != null ? periodEndDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PriceReportablePeriodBuilder {" +
				"periodValue=" + this.periodValue + ", " +
				"periodEffectiveDate=" + this.periodEffectiveDate + ", " +
				"periodEndDate=" + this.periodEndDate +
			'}';
		}
	}
}
