package drr.regulation.common.functions;

import cdm.base.datetime.AdjustableDate;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.records.Date;
import java.util.Collections;
import java.util.List;
import javax.inject.Inject;


@ImplementedBy(CFTCMinAdjustableDateResolution.CFTCMinAdjustableDateResolutionDefault.class)
public abstract class CFTCMinAdjustableDateResolution implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected CFTCAdjustableDateResolution cFTCAdjustableDateResolution;

	/**
	* @param adjustableDates 
	* @return date 
	*/
	public Date evaluate(List<? extends AdjustableDate> adjustableDates) {
		Date date = doEvaluate(adjustableDates);
		
		return date;
	}

	protected abstract Date doEvaluate(List<? extends AdjustableDate> adjustableDates);

	public static class CFTCMinAdjustableDateResolutionDefault extends CFTCMinAdjustableDateResolution {
		@Override
		protected Date doEvaluate(List<? extends AdjustableDate> adjustableDates) {
			if (adjustableDates == null) {
				adjustableDates = Collections.emptyList();
			}
			Date date = null;
			return assignOutput(date, adjustableDates);
		}
		
		protected Date assignOutput(Date date, List<? extends AdjustableDate> adjustableDates) {
			final MapperC<Date> thenResult = MapperC.<AdjustableDate>of(adjustableDates)
				.mapItem(item -> MapperS.of(cFTCAdjustableDateResolution.evaluate(item.get())));
			date = thenResult
				.min().get();
			
			return date;
		}
	}
}
