package drr.regulation.common.functions;

import cdm.product.asset.CommodityPayout;
import cdm.product.asset.CommodityPayout.CommodityPayoutBuilder;
import cdm.product.template.ContractualProduct;
import cdm.product.template.EconomicTerms;
import cdm.product.template.Payout;
import cdm.product.template.Product;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import java.util.Optional;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CommodityLeg2.CommodityLeg2Default.class)
public abstract class CommodityLeg2 implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected CommodityCommodityLeg2 commodityCommodityLeg2;
	@Inject protected IsCommodity isCommodity;
	@Inject protected IsCommodityCommodity isCommodityCommodity;
	@Inject protected IsSingleOnlyCommodityPayout isSingleOnlyCommodityPayout;

	/**
	* @param product 
	* @return commodityLeg2 
	*/
	public CommodityPayout evaluate(Product product) {
		CommodityPayout.CommodityPayoutBuilder commodityLeg2Builder = doEvaluate(product);
		
		final CommodityPayout commodityLeg2;
		if (commodityLeg2Builder == null) {
			commodityLeg2 = null;
		} else {
			commodityLeg2 = commodityLeg2Builder.build();
			objectValidator.validate(CommodityPayout.class, commodityLeg2);
		}
		
		return commodityLeg2;
	}

	protected abstract CommodityPayout.CommodityPayoutBuilder doEvaluate(Product product);

	protected abstract MapperC<? extends CommodityPayout> commodityPayouts(Product product);

	public static class CommodityLeg2Default extends CommodityLeg2 {
		@Override
		protected CommodityPayout.CommodityPayoutBuilder doEvaluate(Product product) {
			CommodityPayout.CommodityPayoutBuilder commodityLeg2 = CommodityPayout.builder();
			return assignOutput(commodityLeg2, product);
		}
		
		protected CommodityPayout.CommodityPayoutBuilder assignOutput(CommodityPayout.CommodityPayoutBuilder commodityLeg2, Product product) {
			final Boolean _boolean = isCommodityCommodity.evaluate(product);
			if ((_boolean == null ? false : _boolean)) {
				commodityLeg2 = toBuilder(commodityCommodityLeg2.evaluate(commodityPayouts(product).getMulti()));
			} else if (ComparisonResult.of(MapperS.of(isCommodity.evaluate(product))).and(areEqual(MapperS.of(isSingleOnlyCommodityPayout.evaluate(product)), MapperS.of(false), CardinalityOperator.All)).getOrDefault(false)) {
				commodityLeg2 = toBuilder(commodityPayouts(product).get());
			} else {
				commodityLeg2 = null;
			}
			
			return Optional.ofNullable(commodityLeg2)
				.map(o -> o.prune())
				.orElse(null);
		}
		
		@Override
		protected MapperC<? extends CommodityPayout> commodityPayouts(Product product) {
			return MapperS.of(product).<ContractualProduct>map("getContractualProduct", _product -> _product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<CommodityPayout>mapC("getCommodityPayout", payout -> payout.getCommodityPayout());
		}
	}
}
