package drr.regulation.common.functions;

import cdm.event.common.ActionEnum;
import cdm.event.common.BusinessEvent;
import cdm.event.workflow.EventInstruction;
import cdm.event.workflow.WorkflowStep;
import cdm.event.workflow.functions.Create_AcceptedWorkflowStepFromInstruction;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.common.ReportableEvent;
import drr.regulation.common.ReportableEvent.ReportableEventBuilder;
import drr.regulation.common.ReportableInformation;
import java.util.Optional;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(Create_ReportableEventFromInstruction.Create_ReportableEventFromInstructionDefault.class)
public abstract class Create_ReportableEventFromInstruction implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected Create_AcceptedWorkflowStepFromInstruction create_AcceptedWorkflowStepFromInstruction;
	@Inject protected Create_CancelledWorkflowStepFromInstruction create_CancelledWorkflowStepFromInstruction;

	/**
	* @param originatingWorkflowStep 
	* @param reportableInformation 
	* @return reportableEvent 
	*/
	public ReportableEvent evaluate(WorkflowStep originatingWorkflowStep, ReportableInformation reportableInformation) {
		ReportableEvent.ReportableEventBuilder reportableEventBuilder = doEvaluate(originatingWorkflowStep, reportableInformation);
		
		final ReportableEvent reportableEvent;
		if (reportableEventBuilder == null) {
			reportableEvent = null;
		} else {
			reportableEvent = reportableEventBuilder.build();
			objectValidator.validate(ReportableEvent.class, reportableEvent);
		}
		
		return reportableEvent;
	}

	protected abstract ReportableEvent.ReportableEventBuilder doEvaluate(WorkflowStep originatingWorkflowStep, ReportableInformation reportableInformation);

	public static class Create_ReportableEventFromInstructionDefault extends Create_ReportableEventFromInstruction {
		@Override
		protected ReportableEvent.ReportableEventBuilder doEvaluate(WorkflowStep originatingWorkflowStep, ReportableInformation reportableInformation) {
			ReportableEvent.ReportableEventBuilder reportableEvent = ReportableEvent.builder();
			return assignOutput(reportableEvent, originatingWorkflowStep, reportableInformation);
		}
		
		protected ReportableEvent.ReportableEventBuilder assignOutput(ReportableEvent.ReportableEventBuilder reportableEvent, WorkflowStep originatingWorkflowStep, ReportableInformation reportableInformation) {
			final WorkflowStep ifThenElseResult;
			if (notExists(MapperS.of(originatingWorkflowStep).<BusinessEvent>map("getBusinessEvent", workflowStep -> workflowStep.getBusinessEvent())).and(exists(MapperS.of(originatingWorkflowStep).<EventInstruction>map("getProposedEvent", workflowStep -> workflowStep.getProposedEvent()))).and(areEqual(MapperS.of(originatingWorkflowStep).<ActionEnum>map("getAction", workflowStep -> workflowStep.getAction()), MapperS.of(ActionEnum.CANCEL), CardinalityOperator.All)).getOrDefault(false)) {
				ifThenElseResult = create_CancelledWorkflowStepFromInstruction.evaluate(originatingWorkflowStep);
			} else if (notExists(MapperS.of(originatingWorkflowStep).<BusinessEvent>map("getBusinessEvent", workflowStep -> workflowStep.getBusinessEvent())).and(exists(MapperS.of(originatingWorkflowStep).<EventInstruction>map("getProposedEvent", workflowStep -> workflowStep.getProposedEvent()))).getOrDefault(false)) {
				ifThenElseResult = create_AcceptedWorkflowStepFromInstruction.evaluate(originatingWorkflowStep);
			} else {
				ifThenElseResult = originatingWorkflowStep;
			}
			reportableEvent
				.setOriginatingWorkflowStep(ifThenElseResult);
			
			reportableEvent
				.setReportableInformation(reportableInformation);
			
			return Optional.ofNullable(reportableEvent)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
