package drr.regulation.common.functions;

import cdm.base.math.NonNegativeQuantitySchedule;
import cdm.base.math.UnitType;
import cdm.base.math.metafields.ReferenceWithMetaNonNegativeQuantitySchedule;
import cdm.product.asset.ForeignExchange;
import cdm.product.common.settlement.Cashflow;
import cdm.product.common.settlement.Cashflow.CashflowBuilder;
import cdm.product.common.settlement.ResolvablePriceQuantity;
import cdm.product.template.Product;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.metafields.FieldWithMetaString;
import java.util.Optional;
import javax.inject.Inject;


@ImplementedBy(FXLeg1.FXLeg1Default.class)
public abstract class FXLeg1 implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected IsFXForward isFXForward;
	@Inject protected IsFXOption isFXOption;
	@Inject protected UnderlierForProduct underlierForProduct;

	/**
	* @param product 
	* @return fxLeg1 
	*/
	public Cashflow evaluate(Product product) {
		Cashflow.CashflowBuilder fxLeg1Builder = doEvaluate(product);
		
		final Cashflow fxLeg1;
		if (fxLeg1Builder == null) {
			fxLeg1 = null;
		} else {
			fxLeg1 = fxLeg1Builder.build();
			objectValidator.validate(Cashflow.class, fxLeg1);
		}
		
		return fxLeg1;
	}

	protected abstract Cashflow.CashflowBuilder doEvaluate(Product product);

	public static class FXLeg1Default extends FXLeg1 {
		@Override
		protected Cashflow.CashflowBuilder doEvaluate(Product product) {
			Cashflow.CashflowBuilder fxLeg1 = Cashflow.builder();
			return assignOutput(fxLeg1, product);
		}
		
		protected Cashflow.CashflowBuilder assignOutput(Cashflow.CashflowBuilder fxLeg1, Product product) {
			if (ComparisonResult.of(MapperS.of(isFXForward.evaluate(product))).or(ComparisonResult.of(MapperS.of(isFXOption.evaluate(product)))).getOrDefault(false)) {
				fxLeg1 = toBuilder(MapperC.<Cashflow>of(MapperS.of(underlierForProduct.evaluate(product)).<ForeignExchange>map("getForeignExchange", _product -> _product.getForeignExchange()).<Cashflow>map("getExchangedCurrency1", foreignExchange -> foreignExchange.getExchangedCurrency1()), MapperS.of(underlierForProduct.evaluate(product)).<ForeignExchange>map("getForeignExchange", _product -> _product.getForeignExchange()).<Cashflow>map("getExchangedCurrency2", foreignExchange -> foreignExchange.getExchangedCurrency2()))
					.min(item -> item.<ResolvablePriceQuantity>map("getPriceQuantity", payoutBase -> payoutBase.getPriceQuantity()).<ReferenceWithMetaNonNegativeQuantitySchedule>map("getQuantitySchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getQuantitySchedule()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()).<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue())).get());
			} else {
				fxLeg1 = null;
			}
			
			return Optional.ofNullable(fxLeg1)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
