package drr.regulation.common.functions;

import cdm.base.math.NonNegativeQuantitySchedule;
import cdm.base.math.UnitType;
import cdm.base.math.metafields.ReferenceWithMetaNonNegativeQuantitySchedule;
import cdm.event.common.Trade;
import cdm.product.asset.ForeignExchange;
import cdm.product.common.settlement.Cashflow;
import cdm.product.common.settlement.Cashflow.CashflowBuilder;
import cdm.product.common.settlement.ResolvablePriceQuantity;
import cdm.product.template.Product;
import cdm.product.template.TradableProduct;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.metafields.FieldWithMetaString;
import java.util.Optional;
import javax.inject.Inject;


@ImplementedBy(FXLeg2.FXLeg2Default.class)
public abstract class FXLeg2 implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected IsFXForward isFXForward;
	@Inject protected IsFXOption isFXOption;
	@Inject protected UnderlierForProduct underlierForProduct;

	/**
	* @param trade 
	* @return fxLeg2 
	*/
	public Cashflow evaluate(Trade trade) {
		Cashflow.CashflowBuilder fxLeg2Builder = doEvaluate(trade);
		
		final Cashflow fxLeg2;
		if (fxLeg2Builder == null) {
			fxLeg2 = null;
		} else {
			fxLeg2 = fxLeg2Builder.build();
			objectValidator.validate(Cashflow.class, fxLeg2);
		}
		
		return fxLeg2;
	}

	protected abstract Cashflow.CashflowBuilder doEvaluate(Trade trade);

	protected abstract MapperS<? extends Product> product(Trade trade);

	public static class FXLeg2Default extends FXLeg2 {
		@Override
		protected Cashflow.CashflowBuilder doEvaluate(Trade trade) {
			Cashflow.CashflowBuilder fxLeg2 = Cashflow.builder();
			return assignOutput(fxLeg2, trade);
		}
		
		protected Cashflow.CashflowBuilder assignOutput(Cashflow.CashflowBuilder fxLeg2, Trade trade) {
			if (ComparisonResult.of(MapperS.of(isFXForward.evaluate(product(trade).get()))).or(ComparisonResult.of(MapperS.of(isFXOption.evaluate(product(trade).get())))).getOrDefault(false)) {
				fxLeg2 = toBuilder(MapperC.<Cashflow>of(MapperS.of(underlierForProduct.evaluate(product(trade).get())).<ForeignExchange>map("getForeignExchange", _product -> _product.getForeignExchange()).<Cashflow>map("getExchangedCurrency1", foreignExchange -> foreignExchange.getExchangedCurrency1()), MapperS.of(underlierForProduct.evaluate(product(trade).get())).<ForeignExchange>map("getForeignExchange", _product -> _product.getForeignExchange()).<Cashflow>map("getExchangedCurrency2", foreignExchange -> foreignExchange.getExchangedCurrency2()))
					.max(item -> item.<ResolvablePriceQuantity>map("getPriceQuantity", payoutBase -> payoutBase.getPriceQuantity()).<ReferenceWithMetaNonNegativeQuantitySchedule>map("getQuantitySchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getQuantitySchedule()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()).<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue())).get());
			} else {
				fxLeg2 = null;
			}
			
			return Optional.ofNullable(fxLeg2)
				.map(o -> o.prune())
				.orElse(null);
		}
		
		@Override
		protected MapperS<? extends Product> product(Trade trade) {
			return MapperS.of(trade).<TradableProduct>map("getTradableProduct", _trade -> _trade.getTradableProduct()).<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct());
		}
	}
}
