package drr.regulation.common.functions;

import cdm.product.template.FixedPricePayout;
import cdm.product.template.FixedPricePayout.FixedPricePayoutBuilder;
import cdm.product.template.Payout;
import cdm.product.template.Product;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import java.util.Optional;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(FixedPriceLeg1.FixedPriceLeg1Default.class)
public abstract class FixedPriceLeg1 implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected EconomicTermsForProduct economicTermsForProduct;
	@Inject protected IsCommodity isCommodity;

	/**
	* @param product 
	* @return fixedPrice 
	*/
	public FixedPricePayout evaluate(Product product) {
		FixedPricePayout.FixedPricePayoutBuilder fixedPriceBuilder = doEvaluate(product);
		
		final FixedPricePayout fixedPrice;
		if (fixedPriceBuilder == null) {
			fixedPrice = null;
		} else {
			fixedPrice = fixedPriceBuilder.build();
			objectValidator.validate(FixedPricePayout.class, fixedPrice);
		}
		
		return fixedPrice;
	}

	protected abstract FixedPricePayout.FixedPricePayoutBuilder doEvaluate(Product product);

	public static class FixedPriceLeg1Default extends FixedPriceLeg1 {
		@Override
		protected FixedPricePayout.FixedPricePayoutBuilder doEvaluate(Product product) {
			FixedPricePayout.FixedPricePayoutBuilder fixedPrice = FixedPricePayout.builder();
			return assignOutput(fixedPrice, product);
		}
		
		protected FixedPricePayout.FixedPricePayoutBuilder assignOutput(FixedPricePayout.FixedPricePayoutBuilder fixedPrice, Product product) {
			if (ComparisonResult.of(MapperS.of(isCommodity.evaluate(product))).and(exists(MapperS.of(economicTermsForProduct.evaluate(product)).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<FixedPricePayout>mapC("getFixedPricePayout", payout -> payout.getFixedPricePayout()))).getOrDefault(false)) {
				fixedPrice = toBuilder(MapperS.of(economicTermsForProduct.evaluate(product)).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<FixedPricePayout>mapC("getFixedPricePayout", payout -> payout.getFixedPricePayout()).get());
			} else {
				fixedPrice = null;
			}
			
			return Optional.ofNullable(fixedPrice)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
