package drr.regulation.common.functions;

import cdm.base.staticdata.asset.common.Commodity;
import cdm.base.staticdata.asset.common.Index;
import cdm.base.staticdata.asset.common.Loan;
import cdm.base.staticdata.asset.common.ProductBase;
import cdm.base.staticdata.asset.common.ProductIdentifier;
import cdm.base.staticdata.asset.common.ProductIdentifier.ProductIdentifierBuilder;
import cdm.base.staticdata.asset.common.Security;
import cdm.base.staticdata.asset.common.metafields.ReferenceWithMetaCommodity;
import cdm.base.staticdata.asset.common.metafields.ReferenceWithMetaProductIdentifier;
import cdm.event.common.Trade;
import cdm.product.asset.BasketReferenceInformation;
import cdm.product.asset.CommodityPayout;
import cdm.product.asset.CreditDefaultPayout;
import cdm.product.asset.GeneralTerms;
import cdm.product.asset.ReferenceInformation;
import cdm.product.asset.ReferenceObligation;
import cdm.product.asset.ReferencePair;
import cdm.product.asset.ReferencePool;
import cdm.product.asset.ReferencePoolItem;
import cdm.product.template.Basket;
import cdm.product.template.Payout;
import cdm.product.template.PerformancePayout;
import cdm.product.template.Product;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(GetBasketConstituentsProductIdentifier.GetBasketConstituentsProductIdentifierDefault.class)
public abstract class GetBasketConstituentsProductIdentifier implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected EconomicTermsForProduct economicTermsForProduct;
	@Inject protected IsSingleOnlyCommodityPayout isSingleOnlyCommodityPayout;
	@Inject protected ProductForTrade productForTrade;
	@Inject protected UnderlierForProduct underlierForProduct;

	/**
	* @param trade 
	* @return productIdentifier 
	*/
	public List<? extends ProductIdentifier> evaluate(Trade trade) {
		List<ProductIdentifier.ProductIdentifierBuilder> productIdentifierBuilder = doEvaluate(trade);
		
		final List<? extends ProductIdentifier> productIdentifier;
		if (productIdentifierBuilder == null) {
			productIdentifier = null;
		} else {
			productIdentifier = productIdentifierBuilder.stream().map(ProductIdentifier::build).collect(Collectors.toList());
			objectValidator.validate(ProductIdentifier.class, productIdentifier);
		}
		
		return productIdentifier;
	}

	protected abstract List<ProductIdentifier.ProductIdentifierBuilder> doEvaluate(Trade trade);

	protected abstract MapperC<? extends Product> basketConstituent(Trade trade);

	protected abstract MapperC<? extends ReferenceObligation> referenceObligation(Trade trade);

	protected abstract MapperC<? extends ProductBase> constituent(Trade trade);

	public static class GetBasketConstituentsProductIdentifierDefault extends GetBasketConstituentsProductIdentifier {
		@Override
		protected List<ProductIdentifier.ProductIdentifierBuilder> doEvaluate(Trade trade) {
			List<ProductIdentifier.ProductIdentifierBuilder> productIdentifier = new ArrayList<>();
			return assignOutput(productIdentifier, trade);
		}
		
		protected List<ProductIdentifier.ProductIdentifierBuilder> assignOutput(List<ProductIdentifier.ProductIdentifierBuilder> productIdentifier, Trade trade) {
			productIdentifier.addAll(toBuilder(constituent(trade).<ReferenceWithMetaProductIdentifier>mapC("getProductIdentifier", productBase -> productBase.getProductIdentifier()).<ProductIdentifier>map("getValue", _f->_f.getValue()).getMulti()));
			
			return Optional.ofNullable(productIdentifier)
				.map(o -> o.stream().map(i -> i.prune()).collect(Collectors.toList()))
				.orElse(null);
		}
		
		@Override
		protected MapperC<? extends Product> basketConstituent(Trade trade) {
			if (exists(MapperS.of(underlierForProduct.evaluate(productForTrade.evaluate(trade))).<Basket>map("getBasket", product -> product.getBasket()).<Product>mapC("getBasketConstituent", basket -> basket.getBasketConstituent())).getOrDefault(false)) {
				return MapperS.of(underlierForProduct.evaluate(productForTrade.evaluate(trade))).<Basket>map("getBasket", product -> product.getBasket()).<Product>mapC("getBasketConstituent", basket -> basket.getBasketConstituent());
			}
			if (exists(MapperS.of(economicTermsForProduct.evaluate(productForTrade.evaluate(trade))).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<PerformancePayout>mapC("getPerformancePayout", payout -> payout.getPerformancePayout()).<Product>map("getUnderlier", performancePayout -> performancePayout.getUnderlier()).<Basket>map("getBasket", product -> product.getBasket()).<Product>mapC("getBasketConstituent", basket -> basket.getBasketConstituent())).getOrDefault(false)) {
				return MapperS.of(economicTermsForProduct.evaluate(productForTrade.evaluate(trade))).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<PerformancePayout>mapC("getPerformancePayout", payout -> payout.getPerformancePayout()).<Product>map("getUnderlier", performancePayout -> performancePayout.getUnderlier()).<Basket>map("getBasket", product -> product.getBasket()).<Product>mapC("getBasketConstituent", basket -> basket.getBasketConstituent());
			}
			if (exists(MapperS.of(economicTermsForProduct.evaluate(productForTrade.evaluate(trade))).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<CommodityPayout>mapC("getCommodityPayout", payout -> payout.getCommodityPayout()).<Product>map("getUnderlier", commodityPayout -> commodityPayout.getUnderlier()).<Basket>map("getBasket", product -> product.getBasket()).<Product>mapC("getBasketConstituent", basket -> basket.getBasketConstituent())).and(areEqual(MapperS.of(isSingleOnlyCommodityPayout.evaluate(productForTrade.evaluate(trade))), MapperS.of(false), CardinalityOperator.All)).getOrDefault(false)) {
				return MapperS.of(economicTermsForProduct.evaluate(productForTrade.evaluate(trade))).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<CommodityPayout>mapC("getCommodityPayout", payout -> payout.getCommodityPayout()).<Product>map("getUnderlier", commodityPayout -> commodityPayout.getUnderlier()).<Basket>map("getBasket", product -> product.getBasket()).<Product>mapC("getBasketConstituent", basket -> basket.getBasketConstituent());
			}
			return MapperC.<Product>ofNull();
		}
		
		@Override
		protected MapperC<? extends ReferenceObligation> referenceObligation(Trade trade) {
			if (exists(MapperS.of(economicTermsForProduct.evaluate(productForTrade.evaluate(trade))).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<CreditDefaultPayout>map("getCreditDefaultPayout", payout -> payout.getCreditDefaultPayout()).<GeneralTerms>map("getGeneralTerms", creditDefaultPayout -> creditDefaultPayout.getGeneralTerms()).<BasketReferenceInformation>map("getBasketReferenceInformation", generalTerms -> generalTerms.getBasketReferenceInformation()).<ReferencePool>map("getReferencePool", basketReferenceInformation -> basketReferenceInformation.getReferencePool()).<ReferencePoolItem>mapC("getReferencePoolItem", referencePool -> referencePool.getReferencePoolItem()).<ReferencePair>map("getReferencePair", referencePoolItem -> referencePoolItem.getReferencePair()).<ReferenceObligation>map("getReferenceObligation", referencePair -> referencePair.getReferenceObligation())).getOrDefault(false)) {
				return MapperS.of(economicTermsForProduct.evaluate(productForTrade.evaluate(trade))).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<CreditDefaultPayout>map("getCreditDefaultPayout", payout -> payout.getCreditDefaultPayout()).<GeneralTerms>map("getGeneralTerms", creditDefaultPayout -> creditDefaultPayout.getGeneralTerms()).<BasketReferenceInformation>map("getBasketReferenceInformation", generalTerms -> generalTerms.getBasketReferenceInformation()).<ReferencePool>map("getReferencePool", basketReferenceInformation -> basketReferenceInformation.getReferencePool()).<ReferencePoolItem>mapC("getReferencePoolItem", referencePool -> referencePool.getReferencePoolItem()).<ReferencePair>map("getReferencePair", referencePoolItem -> referencePoolItem.getReferencePair()).<ReferenceObligation>map("getReferenceObligation", referencePair -> referencePair.getReferenceObligation());
			}
			if (exists(MapperS.of(economicTermsForProduct.evaluate(underlierForProduct.evaluate(productForTrade.evaluate(trade)))).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<CreditDefaultPayout>map("getCreditDefaultPayout", payout -> payout.getCreditDefaultPayout()).<GeneralTerms>map("getGeneralTerms", creditDefaultPayout -> creditDefaultPayout.getGeneralTerms()).<ReferenceInformation>map("getReferenceInformation", generalTerms -> generalTerms.getReferenceInformation()).<ReferenceObligation>mapC("getReferenceObligation", referenceInformation -> referenceInformation.getReferenceObligation())).getOrDefault(false)) {
				return MapperS.of(economicTermsForProduct.evaluate(underlierForProduct.evaluate(productForTrade.evaluate(trade)))).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<CreditDefaultPayout>map("getCreditDefaultPayout", payout -> payout.getCreditDefaultPayout()).<GeneralTerms>map("getGeneralTerms", creditDefaultPayout -> creditDefaultPayout.getGeneralTerms()).<ReferenceInformation>map("getReferenceInformation", generalTerms -> generalTerms.getReferenceInformation()).<ReferenceObligation>mapC("getReferenceObligation", referenceInformation -> referenceInformation.getReferenceObligation());
			}
			return MapperC.<ReferenceObligation>ofNull();
		}
		
		@Override
		protected MapperC<? extends ProductBase> constituent(Trade trade) {
			if (exists(basketConstituent(trade).<Security>map("getSecurity", product -> product.getSecurity())).getOrDefault(false)) {
				return basketConstituent(trade).<Security>map("getSecurity", product -> product.getSecurity());
			}
			if (exists(basketConstituent(trade).<Loan>map("getLoan", product -> product.getLoan())).getOrDefault(false)) {
				return basketConstituent(trade).<Loan>map("getLoan", product -> product.getLoan());
			}
			if (exists(basketConstituent(trade).<ReferenceWithMetaCommodity>map("getCommodity", product -> product.getCommodity()).<Commodity>map("getValue", _f->_f.getValue())).getOrDefault(false)) {
				return basketConstituent(trade).<ReferenceWithMetaCommodity>map("getCommodity", product -> product.getCommodity()).<Commodity>map("getValue", _f->_f.getValue());
			}
			if (exists(basketConstituent(trade).<Index>map("getIndex", product -> product.getIndex())).getOrDefault(false)) {
				return basketConstituent(trade).<Index>map("getIndex", product -> product.getIndex());
			}
			if (exists(referenceObligation(trade).<Security>map("getSecurity", _referenceObligation -> _referenceObligation.getSecurity())).getOrDefault(false)) {
				return referenceObligation(trade).<Security>map("getSecurity", _referenceObligation -> _referenceObligation.getSecurity());
			}
			if (exists(referenceObligation(trade).<Loan>map("getLoan", _referenceObligation -> _referenceObligation.getLoan())).getOrDefault(false)) {
				return referenceObligation(trade).<Loan>map("getLoan", _referenceObligation -> _referenceObligation.getLoan());
			}
			return MapperC.<ProductBase>ofNull();
		}
	}
}
