package drr.regulation.common.functions;

import cdm.base.staticdata.asset.common.IndexReferenceInformation;
import cdm.observable.asset.FloatingRateOption;
import cdm.product.asset.InterestRatePayout;
import cdm.product.template.Product;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.metafields.FieldWithMetaString;
import drr.regulation.common.ReportableEvent;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(GetNameOfTheFloatingRateOfLeg2.GetNameOfTheFloatingRateOfLeg2Default.class)
public abstract class GetNameOfTheFloatingRateOfLeg2 implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected InterestRateLeg2 interestRateLeg2;
	@Inject protected IsCreditDefaultSwap isCreditDefaultSwap;
	@Inject protected IsEquity isEquity;
	@Inject protected ProductForEvent productForEvent;
	@Inject protected ProductOrUnderlierProduct productOrUnderlierProduct;
	@Inject protected RateOption rateOption;

	/**
	* @param reportableEvent 
	* @return name 
	*/
	public String evaluate(ReportableEvent reportableEvent) {
		String name = doEvaluate(reportableEvent);
		
		return name;
	}

	protected abstract String doEvaluate(ReportableEvent reportableEvent);

	public static class GetNameOfTheFloatingRateOfLeg2Default extends GetNameOfTheFloatingRateOfLeg2 {
		@Override
		protected String doEvaluate(ReportableEvent reportableEvent) {
			String name = null;
			return assignOutput(name, reportableEvent);
		}
		
		protected String assignOutput(String name, ReportableEvent reportableEvent) {
			final MapperS<Product> thenResult0 = MapperS.of(productForEvent.evaluate(reportableEvent));
			final MapperS<Product> thenResult1 = thenResult0
				.filterSingleNullSafe(item -> areEqual(MapperS.of(isEquity.evaluate(item.get())), MapperS.of(false), CardinalityOperator.All).and(areEqual(MapperS.of(isCreditDefaultSwap.evaluate(item.get())), MapperS.of(false), CardinalityOperator.All)).get());
			final MapperS<Product> thenResult2 = thenResult1
				.mapSingleToItem(item -> MapperS.of(productOrUnderlierProduct.evaluate(item.get())));
			final MapperS<InterestRatePayout> thenResult3 = thenResult2
				.mapSingleToItem(item -> MapperS.of(interestRateLeg2.evaluate(item.get())));
			final MapperS<FloatingRateOption> thenResult4 = thenResult3
				.mapSingleToItem(item -> MapperS.of(rateOption.evaluate(item.get())));
			name = thenResult4
				.mapSingleToItem(item -> item.<IndexReferenceInformation>map("getIndexReferenceInformation", floatingRateOption -> floatingRateOption.getIndexReferenceInformation()).<FieldWithMetaString>map("getIndexName", indexReferenceInformation -> indexReferenceInformation.getIndexName()).<String>map("getValue", _f->_f.getValue())).get();
			
			return name;
		}
	}
}
