package drr.regulation.common.functions;

import cdm.base.math.ArithmeticOperationEnum;
import cdm.base.math.DatedValue;
import cdm.base.math.UnitType;
import cdm.observable.asset.PriceExpressionEnum;
import cdm.observable.asset.PriceSchedule;
import cdm.observable.asset.PriceTypeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.metafields.FieldWithMetaString;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(GetPriceNotation.GetPriceNotationDefault.class)
public abstract class GetPriceNotation implements RosettaFunction {

	/**
	* @param price 
	* @return notation 
	*/
	public PriceNotationEnum evaluate(PriceSchedule price) {
		PriceNotationEnum notation = doEvaluate(price);
		
		return notation;
	}

	protected abstract PriceNotationEnum doEvaluate(PriceSchedule price);

	public static class GetPriceNotationDefault extends GetPriceNotation {
		@Override
		protected PriceNotationEnum doEvaluate(PriceSchedule price) {
			PriceNotationEnum notation = null;
			return assignOutput(notation, price);
		}
		
		protected PriceNotationEnum assignOutput(PriceNotationEnum notation, PriceSchedule price) {
			if (exists(MapperS.of(price).<BigDecimal>map("getValue", measureBase -> measureBase.getValue())).or(exists(MapperS.of(price).<DatedValue>mapC("getDatedValue", measureSchedule -> measureSchedule.getDatedValue()))).getOrDefault(false)) {
				if (areEqual(MapperS.of(price).<PriceTypeEnum>map("getPriceType", priceSchedule -> priceSchedule.getPriceType()), MapperS.of(PriceTypeEnum.INTEREST_RATE), CardinalityOperator.All).and(areEqual(MapperS.of(price).<ArithmeticOperationEnum>map("getArithmeticOperator", priceSchedule -> priceSchedule.getArithmeticOperator()), MapperS.of(ArithmeticOperationEnum.ADD), CardinalityOperator.All)).getOrDefault(false)) {
					notation = PriceNotationEnum.BASIS;
				} else if (areEqual(MapperS.of(price).<PriceTypeEnum>map("getPriceType", priceSchedule -> priceSchedule.getPriceType()), MapperS.of(PriceTypeEnum.INTEREST_RATE), CardinalityOperator.All).or(areEqual(MapperS.of(price).<PriceTypeEnum>map("getPriceType", priceSchedule -> priceSchedule.getPriceType()), MapperS.of(PriceTypeEnum.VARIANCE), CardinalityOperator.All)).or(areEqual(MapperS.of(price).<PriceTypeEnum>map("getPriceType", priceSchedule -> priceSchedule.getPriceType()), MapperS.of(PriceTypeEnum.VOLATILITY), CardinalityOperator.All)).or(areEqual(MapperS.of(price).<PriceExpressionEnum>map("getPriceExpression", priceSchedule -> priceSchedule.getPriceExpression()), MapperS.of(PriceExpressionEnum.PERCENTAGE_OF_NOTIONAL), CardinalityOperator.All)).getOrDefault(false)) {
					notation = PriceNotationEnum.PERCENTAGE;
				} else if (areEqual(MapperS.of(price).<PriceTypeEnum>map("getPriceType", priceSchedule -> priceSchedule.getPriceType()), MapperS.of(PriceTypeEnum.EXCHANGE_RATE), CardinalityOperator.All).or(areEqual(MapperS.of(price).<PriceTypeEnum>map("getPriceType", priceSchedule -> priceSchedule.getPriceType()), MapperS.of(PriceTypeEnum.ASSET_PRICE), CardinalityOperator.All)).or(areEqual(MapperS.of(price).<PriceTypeEnum>map("getPriceType", priceSchedule -> priceSchedule.getPriceType()), MapperS.of(PriceTypeEnum.CASH_PRICE), CardinalityOperator.All)).or(exists(MapperS.of(price).<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue())).or(areEqual(MapperS.of(price).<PriceExpressionEnum>map("getPriceExpression", priceSchedule -> priceSchedule.getPriceExpression()), MapperS.of(PriceExpressionEnum.ABSOLUTE_TERMS), CardinalityOperator.All))).getOrDefault(false)) {
					notation = PriceNotationEnum.MONETARY;
				} else {
					notation = PriceNotationEnum.DECIMAL;
				}
			} else {
				notation = null;
			}
			
			return notation;
		}
	}
}
