package drr.regulation.common.functions;

import cdm.base.staticdata.identifier.AssignedIdentifier;
import cdm.base.staticdata.party.Party;
import cdm.base.staticdata.party.PartyRole;
import cdm.base.staticdata.party.PartyRoleEnum;
import cdm.base.staticdata.party.functions.FilterPartyRole;
import cdm.base.staticdata.party.metafields.ReferenceWithMetaParty;
import cdm.event.common.TradeIdentifier;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperListOfLists;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.metafields.FieldWithMetaString;
import drr.regulation.common.ReportableEvent;
import drr.regulation.common.ReportableInformation;
import java.util.Collections;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(GetReportTrackingNumber.GetReportTrackingNumberDefault.class)
public abstract class GetReportTrackingNumber implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected FilterPartyRole filterPartyRole;
	@Inject protected GetVenueOfExecution getVenueOfExecution;
	@Inject protected PositionForEvent positionForEvent;
	@Inject protected TradeForEvent tradeForEvent;

	/**
	* @param reportableEvent 
	* @return reportingTrackingNumber 
	*/
	public String evaluate(ReportableEvent reportableEvent) {
		String reportingTrackingNumber = doEvaluate(reportableEvent);
		
		return reportingTrackingNumber;
	}

	protected abstract String doEvaluate(ReportableEvent reportableEvent);

	protected abstract MapperS<? extends Party> role(ReportableEvent reportableEvent);

	protected abstract MapperS<String> repTrack(ReportableEvent reportableEvent);

	protected abstract MapperS<String> mic(ReportableEvent reportableEvent);

	protected abstract MapperS<String> level(ReportableEvent reportableEvent);

	public static class GetReportTrackingNumberDefault extends GetReportTrackingNumber {
		@Override
		protected String doEvaluate(ReportableEvent reportableEvent) {
			String reportingTrackingNumber = null;
			return assignOutput(reportingTrackingNumber, reportableEvent);
		}
		
		protected String assignOutput(String reportingTrackingNumber, ReportableEvent reportableEvent) {
			if (areEqual(exists(mic(reportableEvent)), MapperS.of(false), CardinalityOperator.All).or(areEqual(mic(reportableEvent), MapperS.of("XOFF"), CardinalityOperator.All).or(areEqual(mic(reportableEvent), MapperS.of("XXXX"), CardinalityOperator.All))).or(areEqual(level(reportableEvent), MapperS.of("PSTN"), CardinalityOperator.All)).getOrDefault(false)) {
				reportingTrackingNumber = null;
			} else {
				reportingTrackingNumber = repTrack(reportableEvent).get();
			}
			
			return reportingTrackingNumber;
		}
		
		@Override
		protected MapperS<? extends Party> role(ReportableEvent reportableEvent) {
			return MapperS.of(MapperC.<PartyRole>of(filterPartyRole.evaluate(MapperS.of(tradeForEvent.evaluate(reportableEvent)).<PartyRole>mapC("getPartyRole", trade -> trade.getPartyRole()).getMulti(), PartyRoleEnum.EXECUTION_FACILITY)).<ReferenceWithMetaParty>map("getPartyReference", partyRole -> partyRole.getPartyReference()).<Party>map("getValue", _f->_f.getValue()).get());
		}
		
		@Override
		protected MapperS<String> repTrack(ReportableEvent reportableEvent) {
			final MapperC<TradeIdentifier> thenResult0 = MapperS.of(tradeForEvent.evaluate(reportableEvent)).<TradeIdentifier>mapC("getTradeIdentifier", trade -> trade.getTradeIdentifier())
				.filterItemNullSafe(item -> areEqual(item.<ReferenceWithMetaParty>map("getIssuerReference", identifier -> identifier.getIssuerReference()).<Party>map("getValue", _f->_f.getValue()), role(reportableEvent), CardinalityOperator.All).get());
			final MapperListOfLists<String> thenResult1 = thenResult0
				.mapItemToList(item -> item.<AssignedIdentifier>mapC("getAssignedIdentifier", identifier -> identifier.getAssignedIdentifier()).<FieldWithMetaString>map("getIdentifier", assignedIdentifier -> assignedIdentifier.getIdentifier()).<String>map("getValue", _f->_f.getValue()));
			final MapperC<String> thenResult2 = thenResult1
				.flattenList();
			final MapperC<String> ifThenElseResult;
			if (exists(thenResult2).getOrDefault(false)) {
				ifThenElseResult = thenResult2;
			} else {
				ifThenElseResult = MapperC.of(Collections.singletonList("RTNNotProvided"));
			}
			return MapperS.of(ifThenElseResult.get());
		}
		
		@Override
		protected MapperS<String> mic(ReportableEvent reportableEvent) {
			return MapperS.of(getVenueOfExecution.evaluate(MapperS.of(reportableEvent).<ReportableInformation>map("getReportableInformation", _reportableEvent -> _reportableEvent.getReportableInformation()).get()));
		}
		
		@Override
		protected MapperS<String> level(ReportableEvent reportableEvent) {
			if (exists(MapperS.of(tradeForEvent.evaluate(reportableEvent))).getOrDefault(false)) {
				return MapperS.of("TCTN");
			}
			if (exists(MapperS.of(positionForEvent.evaluate(reportableEvent))).getOrDefault(false)) {
				return MapperS.of("PSTN");
			}
			return MapperS.<String>ofNull();
		}
	}
}
