package drr.regulation.common.functions;

import cdm.event.common.EventIntentEnum;
import cdm.event.workflow.EventInstruction;
import cdm.event.workflow.WorkflowStep;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.common.ReportableEvent;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(IntendedToClear.IntendedToClearDefault.class)
public abstract class IntendedToClear implements RosettaFunction {

	/**
	* @param reportableEvent 
	* @return isIntendedToClear 
	*/
	public Boolean evaluate(ReportableEvent reportableEvent) {
		Boolean isIntendedToClear = doEvaluate(reportableEvent);
		
		return isIntendedToClear;
	}

	protected abstract Boolean doEvaluate(ReportableEvent reportableEvent);

	public static class IntendedToClearDefault extends IntendedToClear {
		@Override
		protected Boolean doEvaluate(ReportableEvent reportableEvent) {
			Boolean isIntendedToClear = null;
			return assignOutput(isIntendedToClear, reportableEvent);
		}
		
		protected Boolean assignOutput(Boolean isIntendedToClear, ReportableEvent reportableEvent) {
			if (areEqual(MapperS.of(reportableEvent).<WorkflowStep>map("getOriginatingWorkflowStep", _reportableEvent -> _reportableEvent.getOriginatingWorkflowStep()).<EventInstruction>map("getNextEvent", workflowStep -> workflowStep.getNextEvent()).<EventIntentEnum>map("getIntent", eventInstruction -> eventInstruction.getIntent()), MapperS.of(EventIntentEnum.CLEARING), CardinalityOperator.All).getOrDefault(false)) {
				isIntendedToClear = true;
			} else {
				isIntendedToClear = false;
			}
			
			return isIntendedToClear;
		}
	}
}
