package drr.regulation.common.functions;

import cdm.observable.asset.PriceSchedule;
import cdm.observable.asset.PriceSchedule.PriceScheduleBuilder;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.common.ReportableEvent;
import drr.regulation.common.ReportableInformation;
import java.util.Optional;
import javax.inject.Inject;


@ImplementedBy(LastAvailableSpotPrice.LastAvailableSpotPriceDefault.class)
public abstract class LastAvailableSpotPrice implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param reportableEvent 
	* @return lastAvailableSpotPrice 
	*/
	public PriceSchedule evaluate(ReportableEvent reportableEvent) {
		PriceSchedule.PriceScheduleBuilder lastAvailableSpotPriceBuilder = doEvaluate(reportableEvent);
		
		final PriceSchedule lastAvailableSpotPrice;
		if (lastAvailableSpotPriceBuilder == null) {
			lastAvailableSpotPrice = null;
		} else {
			lastAvailableSpotPrice = lastAvailableSpotPriceBuilder.build();
			objectValidator.validate(PriceSchedule.class, lastAvailableSpotPrice);
		}
		
		return lastAvailableSpotPrice;
	}

	protected abstract PriceSchedule.PriceScheduleBuilder doEvaluate(ReportableEvent reportableEvent);

	public static class LastAvailableSpotPriceDefault extends LastAvailableSpotPrice {
		@Override
		protected PriceSchedule.PriceScheduleBuilder doEvaluate(ReportableEvent reportableEvent) {
			PriceSchedule.PriceScheduleBuilder lastAvailableSpotPrice = PriceSchedule.builder();
			return assignOutput(lastAvailableSpotPrice, reportableEvent);
		}
		
		protected PriceSchedule.PriceScheduleBuilder assignOutput(PriceSchedule.PriceScheduleBuilder lastAvailableSpotPrice, ReportableEvent reportableEvent) {
			lastAvailableSpotPrice = toBuilder(MapperS.of(reportableEvent).<ReportableInformation>map("getReportableInformation", _reportableEvent -> _reportableEvent.getReportableInformation()).<PriceSchedule>map("getUnderlyingAssetLastAvailableSpotPrice", reportableInformation -> reportableInformation.getUnderlyingAssetLastAvailableSpotPrice()).get());
			
			return Optional.ofNullable(lastAvailableSpotPrice)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
