package drr.regulation.common.functions;

import cdm.base.datetime.AdjustableDate;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.records.Date;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(MaxTerminationDate.MaxTerminationDateDefault.class)
public abstract class MaxTerminationDate implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected CFTCAdjustableDateResolution cFTCAdjustableDateResolution;

	/**
	* @param date1 
	* @param date2 
	* @return date 
	*/
	public Date evaluate(AdjustableDate date1, AdjustableDate date2) {
		Date date = doEvaluate(date1, date2);
		
		return date;
	}

	protected abstract Date doEvaluate(AdjustableDate date1, AdjustableDate date2);

	public static class MaxTerminationDateDefault extends MaxTerminationDate {
		@Override
		protected Date doEvaluate(AdjustableDate date1, AdjustableDate date2) {
			Date date = null;
			return assignOutput(date, date1, date2);
		}
		
		protected Date assignOutput(Date date, AdjustableDate date1, AdjustableDate date2) {
			if (greaterThan(MapperS.of(cFTCAdjustableDateResolution.evaluate(date1)), MapperS.of(cFTCAdjustableDateResolution.evaluate(date2)), CardinalityOperator.All).getOrDefault(false)) {
				date = cFTCAdjustableDateResolution.evaluate(date1);
			} else {
				date = cFTCAdjustableDateResolution.evaluate(date2);
			}
			
			return date;
		}
	}
}
