package drr.regulation.common.functions;

import cdm.base.staticdata.party.NaturalPerson;
import cdm.base.staticdata.party.NaturalPersonRole;
import cdm.base.staticdata.party.NaturalPersonRoleEnum;
import cdm.base.staticdata.party.Party;
import cdm.base.staticdata.party.PersonIdentifier;
import cdm.base.staticdata.party.metafields.FieldWithMetaNaturalPersonRoleEnum;
import cdm.base.staticdata.party.metafields.FieldWithMetaPersonIdentifier;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperListOfLists;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.metafields.FieldWithMetaString;
import drr.regulation.common.PartyIdentifierFormatEnum;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(PartyIdentifierFormat.PartyIdentifierFormatDefault.class)
public abstract class PartyIdentifierFormat implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected PartyIdentifierNaturalPersonRoles partyIdentifierNaturalPersonRoles;

	/**
	* @param party 
	* @return partyIdentifierFormat 
	*/
	public PartyIdentifierFormatEnum evaluate(Party party) {
		PartyIdentifierFormatEnum partyIdentifierFormat = doEvaluate(party);
		
		return partyIdentifierFormat;
	}

	protected abstract PartyIdentifierFormatEnum doEvaluate(Party party);

	protected abstract MapperS<String> personId(Party party);

	public static class PartyIdentifierFormatDefault extends PartyIdentifierFormat {
		@Override
		protected PartyIdentifierFormatEnum doEvaluate(Party party) {
			PartyIdentifierFormatEnum partyIdentifierFormat = null;
			return assignOutput(partyIdentifierFormat, party);
		}
		
		protected PartyIdentifierFormatEnum assignOutput(PartyIdentifierFormatEnum partyIdentifierFormat, Party party) {
			if (exists(personId(party)).getOrDefault(false)) {
				partyIdentifierFormat = PartyIdentifierFormatEnum.LEI_AND_PERSON;
			} else {
				partyIdentifierFormat = PartyIdentifierFormatEnum.LEI;
			}
			
			return partyIdentifierFormat;
		}
		
		@Override
		protected MapperS<String> personId(Party party) {
			final MapperC<NaturalPerson> thenResult0 = MapperS.of(party)
				.mapSingleToList(item -> item.<NaturalPerson>mapC("getPerson", _party -> _party.getPerson()));
			final MapperC<NaturalPerson> thenResult1 = thenResult0
				.filterItemNullSafe(item -> areEqual(item.<NaturalPersonRole>mapC("getPersonRole", naturalPerson -> naturalPerson.getPersonRole()).<FieldWithMetaNaturalPersonRoleEnum>mapC("getRole", naturalPersonRole -> naturalPersonRole.getRole()).<NaturalPersonRoleEnum>map("getValue", _f->_f.getValue())
					.mapItem(r -> contains(MapperC.<NaturalPersonRoleEnum>of(partyIdentifierNaturalPersonRoles.evaluate()), r).asMapper()), MapperS.of(true), CardinalityOperator.Any).get());
			final MapperListOfLists<String> thenResult2 = thenResult1
				.mapItemToList(item -> item.<FieldWithMetaPersonIdentifier>mapC("getPersonId", naturalPerson -> naturalPerson.getPersonId()).<PersonIdentifier>map("getValue", _f->_f.getValue()).<FieldWithMetaString>map("getIdentifier", personIdentifier -> personIdentifier.getIdentifier()).<String>map("getValue", _f->_f.getValue()));
			final MapperC<String> thenResult3 = thenResult2
				.flattenList();
			return MapperS.of(thenResult3.get());
		}
	}
}
