package drr.regulation.common.functions;

import cdm.base.staticdata.party.NaturalPerson;
import cdm.base.staticdata.party.NaturalPersonRole;
import cdm.base.staticdata.party.NaturalPersonRoleEnum;
import cdm.base.staticdata.party.Party;
import cdm.base.staticdata.party.PersonIdentifier;
import cdm.base.staticdata.party.metafields.FieldWithMetaNaturalPersonRoleEnum;
import cdm.base.staticdata.party.metafields.FieldWithMetaPersonIdentifier;
import cdm.base.staticdata.party.metafields.ReferenceWithMetaNaturalPerson;
import cdm.base.staticdata.party.metafields.ReferenceWithMetaParty;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperListOfLists;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.metafields.FieldWithMetaString;
import drr.regulation.common.PartyIdentifierFormatEnum;
import drr.regulation.common.PartyInformation;
import java.util.Collections;
import java.util.List;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(PartyInformationIdentifierFormat.PartyInformationIdentifierFormatDefault.class)
public abstract class PartyInformationIdentifierFormat implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected PartyIdentifierNaturalPersonRoles partyIdentifierNaturalPersonRoles;

	/**
	* @param party 
	* @param partyInformation 
	* @return partyIdentifierFormat 
	*/
	public PartyIdentifierFormatEnum evaluate(Party party, List<? extends PartyInformation> partyInformation) {
		PartyIdentifierFormatEnum partyIdentifierFormat = doEvaluate(party, partyInformation);
		
		return partyIdentifierFormat;
	}

	protected abstract PartyIdentifierFormatEnum doEvaluate(Party party, List<? extends PartyInformation> partyInformation);

	protected abstract MapperS<String> personId(Party party, List<? extends PartyInformation> partyInformation);

	public static class PartyInformationIdentifierFormatDefault extends PartyInformationIdentifierFormat {
		@Override
		protected PartyIdentifierFormatEnum doEvaluate(Party party, List<? extends PartyInformation> partyInformation) {
			if (partyInformation == null) {
				partyInformation = Collections.emptyList();
			}
			PartyIdentifierFormatEnum partyIdentifierFormat = null;
			return assignOutput(partyIdentifierFormat, party, partyInformation);
		}
		
		protected PartyIdentifierFormatEnum assignOutput(PartyIdentifierFormatEnum partyIdentifierFormat, Party party, List<? extends PartyInformation> partyInformation) {
			if (exists(personId(party, partyInformation)).getOrDefault(false)) {
				partyIdentifierFormat = PartyIdentifierFormatEnum.LEI_AND_PERSON;
			} else {
				partyIdentifierFormat = PartyIdentifierFormatEnum.LEI;
			}
			
			return partyIdentifierFormat;
		}
		
		@Override
		protected MapperS<String> personId(Party party, List<? extends PartyInformation> partyInformation) {
			final MapperC<PartyInformation> thenResult0 = MapperC.<PartyInformation>of(partyInformation)
				.filterItemNullSafe(item -> areEqual(item.<ReferenceWithMetaParty>map("getPartyReference", _partyInformation -> _partyInformation.getPartyReference()).<Party>map("getValue", _f->_f.getValue()), MapperS.of(party), CardinalityOperator.All).get());
			final MapperListOfLists<NaturalPersonRole> thenResult1 = thenResult0
				.mapItemToList(item -> item.<NaturalPersonRole>mapC("getRelatedPerson", _partyInformation -> _partyInformation.getRelatedPerson()));
			final MapperC<NaturalPersonRole> thenResult2 = thenResult1
				.flattenList();
			final MapperC<NaturalPersonRole> thenResult3 = thenResult2
				.filterItemNullSafe(item -> areEqual(item.<FieldWithMetaNaturalPersonRoleEnum>mapC("getRole", naturalPersonRole -> naturalPersonRole.getRole()).<NaturalPersonRoleEnum>map("getValue", _f->_f.getValue())
					.mapItem(r -> contains(MapperC.<NaturalPersonRoleEnum>of(partyIdentifierNaturalPersonRoles.evaluate()), r).asMapper()), MapperS.of(true), CardinalityOperator.Any).get());
			final MapperListOfLists<String> thenResult4 = thenResult3
				.mapItemToList(item -> item.<ReferenceWithMetaNaturalPerson>map("getPersonReference", naturalPersonRole -> naturalPersonRole.getPersonReference()).<NaturalPerson>map("getValue", _f->_f.getValue()).<FieldWithMetaPersonIdentifier>mapC("getPersonId", naturalPerson -> naturalPerson.getPersonId()).<PersonIdentifier>map("getValue", _f->_f.getValue()).<FieldWithMetaString>map("getIdentifier", personIdentifier -> personIdentifier.getIdentifier()).<String>map("getValue", _f->_f.getValue()));
			final MapperC<String> thenResult5 = thenResult4
				.flattenList();
			return MapperS.of(thenResult5.get());
		}
	}
}
