package drr.regulation.common.functions;

import cdm.base.staticdata.party.NaturalPerson;
import cdm.base.staticdata.party.NaturalPersonRole;
import cdm.base.staticdata.party.NaturalPersonRoleEnum;
import cdm.base.staticdata.party.Party;
import cdm.base.staticdata.party.PartyIdentifier;
import cdm.base.staticdata.party.PartyIdentifierTypeEnum;
import cdm.base.staticdata.party.PersonIdentifier;
import cdm.base.staticdata.party.metafields.FieldWithMetaNaturalPersonRoleEnum;
import cdm.base.staticdata.party.metafields.FieldWithMetaPersonIdentifier;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.MapperMaths;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperListOfLists;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.metafields.FieldWithMetaString;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(PartyLeiAndPersonByPartyRoles.PartyLeiAndPersonByPartyRolesDefault.class)
public abstract class PartyLeiAndPersonByPartyRoles implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected PartyIdentifierNaturalPersonRoles partyIdentifierNaturalPersonRoles;
	@Inject protected PartyLei partyLei0;

	/**
	* @param party 
	* @return partyPersonLei 
	*/
	public String evaluate(Party party) {
		String partyPersonLei = doEvaluate(party);
		
		return partyPersonLei;
	}

	protected abstract String doEvaluate(Party party);

	protected abstract MapperS<String> partyLei1(Party party);

	protected abstract MapperS<String> personId(Party party);

	public static class PartyLeiAndPersonByPartyRolesDefault extends PartyLeiAndPersonByPartyRoles {
		@Override
		protected String doEvaluate(Party party) {
			String partyPersonLei = null;
			return assignOutput(partyPersonLei, party);
		}
		
		protected String assignOutput(String partyPersonLei, Party party) {
			if (exists(personId(party)).getOrDefault(false)) {
				partyPersonLei = MapperMaths.<String, String, String>add(partyLei1(party), personId(party)).get();
			} else {
				partyPersonLei = partyLei1(party).get();
			}
			
			return partyPersonLei;
		}
		
		@Override
		protected MapperS<String> partyLei1(Party party) {
			if (areEqual(MapperS.of(party).<PartyIdentifier>mapC("getPartyId", _party -> _party.getPartyId()).<PartyIdentifierTypeEnum>map("getIdentifierType", partyIdentifier -> partyIdentifier.getIdentifierType()), MapperS.of(PartyIdentifierTypeEnum.LEI), CardinalityOperator.Any).getOrDefault(false)) {
				return MapperS.of(partyLei0.evaluate(MapperS.of(party).<PartyIdentifier>mapC("getPartyId", _party -> _party.getPartyId()).getMulti()));
			}
			return MapperS.<String>ofNull();
		}
		
		@Override
		protected MapperS<String> personId(Party party) {
			final MapperC<NaturalPerson> thenResult0 = MapperS.of(party)
				.mapSingleToList(item -> item.<NaturalPerson>mapC("getPerson", _party -> _party.getPerson()));
			final MapperC<NaturalPerson> thenResult1 = thenResult0
				.filterItemNullSafe(item -> areEqual(item.<NaturalPersonRole>mapC("getPersonRole", naturalPerson -> naturalPerson.getPersonRole()).<FieldWithMetaNaturalPersonRoleEnum>mapC("getRole", naturalPersonRole -> naturalPersonRole.getRole()).<NaturalPersonRoleEnum>map("getValue", _f->_f.getValue())
					.mapItem(r -> contains(MapperC.<NaturalPersonRoleEnum>of(partyIdentifierNaturalPersonRoles.evaluate()), r).asMapper()), MapperS.of(true), CardinalityOperator.Any).get());
			final MapperListOfLists<String> thenResult2 = thenResult1
				.mapItemToList(item -> item.<FieldWithMetaPersonIdentifier>mapC("getPersonId", naturalPerson -> naturalPerson.getPersonId()).<PersonIdentifier>map("getValue", _f->_f.getValue()).<FieldWithMetaString>map("getIdentifier", personIdentifier -> personIdentifier.getIdentifier()).<String>map("getValue", _f->_f.getValue()));
			final MapperC<String> thenResult3 = thenResult2
				.flattenList();
			return MapperS.of(thenResult3.get());
		}
	}
}
