package drr.regulation.common.functions;

import cdm.base.staticdata.party.NaturalPerson;
import cdm.base.staticdata.party.NaturalPersonRole;
import cdm.base.staticdata.party.NaturalPersonRoleEnum;
import cdm.base.staticdata.party.Party;
import cdm.base.staticdata.party.PartyIdentifier;
import cdm.base.staticdata.party.PartyIdentifierTypeEnum;
import cdm.base.staticdata.party.PersonIdentifier;
import cdm.base.staticdata.party.metafields.FieldWithMetaNaturalPersonRoleEnum;
import cdm.base.staticdata.party.metafields.FieldWithMetaPersonIdentifier;
import cdm.base.staticdata.party.metafields.ReferenceWithMetaNaturalPerson;
import cdm.base.staticdata.party.metafields.ReferenceWithMetaParty;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.MapperMaths;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperListOfLists;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.metafields.FieldWithMetaString;
import drr.regulation.common.PartyInformation;
import java.util.Collections;
import java.util.List;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(PartyLeiAndPersonByRoles.PartyLeiAndPersonByRolesDefault.class)
public abstract class PartyLeiAndPersonByRoles implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected PartyIdentifierNaturalPersonRoles partyIdentifierNaturalPersonRoles;
	@Inject protected PartyLei partyLei0;

	/**
	* @param party 
	* @param partyInformation 
	* @return partyPersonLei 
	*/
	public String evaluate(Party party, List<? extends PartyInformation> partyInformation) {
		String partyPersonLei = doEvaluate(party, partyInformation);
		
		return partyPersonLei;
	}

	protected abstract String doEvaluate(Party party, List<? extends PartyInformation> partyInformation);

	protected abstract MapperS<String> partyLei1(Party party, List<? extends PartyInformation> partyInformation);

	protected abstract MapperS<String> personId(Party party, List<? extends PartyInformation> partyInformation);

	public static class PartyLeiAndPersonByRolesDefault extends PartyLeiAndPersonByRoles {
		@Override
		protected String doEvaluate(Party party, List<? extends PartyInformation> partyInformation) {
			if (partyInformation == null) {
				partyInformation = Collections.emptyList();
			}
			String partyPersonLei = null;
			return assignOutput(partyPersonLei, party, partyInformation);
		}
		
		protected String assignOutput(String partyPersonLei, Party party, List<? extends PartyInformation> partyInformation) {
			if (exists(personId(party, partyInformation)).getOrDefault(false)) {
				partyPersonLei = MapperMaths.<String, String, String>add(partyLei1(party, partyInformation), personId(party, partyInformation)).get();
			} else {
				partyPersonLei = partyLei1(party, partyInformation).get();
			}
			
			return partyPersonLei;
		}
		
		@Override
		protected MapperS<String> partyLei1(Party party, List<? extends PartyInformation> partyInformation) {
			if (areEqual(MapperS.of(party).<PartyIdentifier>mapC("getPartyId", _party -> _party.getPartyId()).<PartyIdentifierTypeEnum>map("getIdentifierType", partyIdentifier -> partyIdentifier.getIdentifierType()), MapperS.of(PartyIdentifierTypeEnum.LEI), CardinalityOperator.Any).getOrDefault(false)) {
				return MapperS.of(partyLei0.evaluate(MapperS.of(party).<PartyIdentifier>mapC("getPartyId", _party -> _party.getPartyId()).getMulti()));
			}
			return MapperS.<String>ofNull();
		}
		
		@Override
		protected MapperS<String> personId(Party party, List<? extends PartyInformation> partyInformation) {
			final MapperC<PartyInformation> thenResult0 = MapperC.<PartyInformation>of(partyInformation)
				.filterItemNullSafe(item -> areEqual(item.<ReferenceWithMetaParty>map("getPartyReference", _partyInformation -> _partyInformation.getPartyReference()).<Party>map("getValue", _f->_f.getValue()), MapperS.of(party), CardinalityOperator.All).get());
			final MapperListOfLists<NaturalPersonRole> thenResult1 = thenResult0
				.mapItemToList(item -> item.<NaturalPersonRole>mapC("getRelatedPerson", _partyInformation -> _partyInformation.getRelatedPerson()));
			final MapperC<NaturalPersonRole> thenResult2 = thenResult1
				.flattenList();
			final MapperC<NaturalPersonRole> thenResult3 = thenResult2
				.filterItemNullSafe(item -> areEqual(item.<FieldWithMetaNaturalPersonRoleEnum>mapC("getRole", naturalPersonRole -> naturalPersonRole.getRole()).<NaturalPersonRoleEnum>map("getValue", _f->_f.getValue())
					.mapItem(r -> contains(MapperC.<NaturalPersonRoleEnum>of(partyIdentifierNaturalPersonRoles.evaluate()), r).asMapper()), MapperS.of(true), CardinalityOperator.Any).get());
			final MapperListOfLists<String> thenResult4 = thenResult3
				.mapItemToList(item -> item.<ReferenceWithMetaNaturalPerson>map("getPersonReference", naturalPersonRole -> naturalPersonRole.getPersonReference()).<NaturalPerson>map("getValue", _f->_f.getValue()).<FieldWithMetaPersonIdentifier>mapC("getPersonId", naturalPerson -> naturalPerson.getPersonId()).<PersonIdentifier>map("getValue", _f->_f.getValue()).<FieldWithMetaString>map("getIdentifier", personIdentifier -> personIdentifier.getIdentifier()).<String>map("getValue", _f->_f.getValue()));
			final MapperC<String> thenResult5 = thenResult4
				.flattenList();
			return MapperS.of(thenResult5.get());
		}
	}
}
