package drr.regulation.common.functions;

import cdm.base.datetime.CalculationPeriodFrequency;
import cdm.base.datetime.Frequency;
import cdm.base.datetime.Frequency.FrequencyBuilder;
import cdm.base.datetime.PeriodExtendedEnum;
import cdm.base.math.DatedValue;
import cdm.base.math.NonNegativeQuantitySchedule;
import cdm.base.math.metafields.FieldWithMetaNonNegativeQuantitySchedule;
import cdm.product.common.schedule.CalculationPeriodDates;
import cdm.product.common.settlement.PriceQuantity;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import java.math.BigDecimal;
import java.util.Optional;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(QuantityFrequencyOrCalculationPeriod.QuantityFrequencyOrCalculationPeriodDefault.class)
public abstract class QuantityFrequencyOrCalculationPeriod implements RosettaFunction {
	
	@Inject protected ModelObjectValidator objectValidator;

	/**
	* @param calculationPeriodDates 
	* @param priceQuantity 
	* @return frequency 
	*/
	public Frequency evaluate(CalculationPeriodDates calculationPeriodDates, PriceQuantity priceQuantity) {
		Frequency.FrequencyBuilder frequencyBuilder = doEvaluate(calculationPeriodDates, priceQuantity);
		
		final Frequency frequency;
		if (frequencyBuilder == null) {
			frequency = null;
		} else {
			frequency = frequencyBuilder.build();
			objectValidator.validate(Frequency.class, frequency);
		}
		
		return frequency;
	}

	protected abstract Frequency.FrequencyBuilder doEvaluate(CalculationPeriodDates calculationPeriodDates, PriceQuantity priceQuantity);

	protected abstract MapperC<? extends NonNegativeQuantitySchedule> quantity(CalculationPeriodDates calculationPeriodDates, PriceQuantity priceQuantity);

	public static class QuantityFrequencyOrCalculationPeriodDefault extends QuantityFrequencyOrCalculationPeriod {
		@Override
		protected Frequency.FrequencyBuilder doEvaluate(CalculationPeriodDates calculationPeriodDates, PriceQuantity priceQuantity) {
			Frequency.FrequencyBuilder frequency = Frequency.builder();
			return assignOutput(frequency, calculationPeriodDates, priceQuantity);
		}
		
		protected Frequency.FrequencyBuilder assignOutput(Frequency.FrequencyBuilder frequency, CalculationPeriodDates calculationPeriodDates, PriceQuantity priceQuantity) {
			if (notEqual(MapperS.of(quantity(calculationPeriodDates, priceQuantity).<Frequency>map("getFrequency", quantitySchedule -> quantitySchedule.getFrequency()).<PeriodExtendedEnum>map("getPeriod", _frequency -> _frequency.getPeriod()).get()), MapperS.of(PeriodExtendedEnum.C), CardinalityOperator.Any).getOrDefault(false)) {
				frequency = toBuilder(quantity(calculationPeriodDates, priceQuantity).<Frequency>map("getFrequency", quantitySchedule -> quantitySchedule.getFrequency()).get());
			} else if (areEqual(MapperS.of(quantity(calculationPeriodDates, priceQuantity).<Frequency>map("getFrequency", quantitySchedule -> quantitySchedule.getFrequency()).<PeriodExtendedEnum>map("getPeriod", _frequency -> _frequency.getPeriod()).get()), MapperS.of(PeriodExtendedEnum.C), CardinalityOperator.All).getOrDefault(false)) {
				frequency = toBuilder(MapperS.of(calculationPeriodDates).<CalculationPeriodFrequency>map("getCalculationPeriodFrequency", _calculationPeriodDates -> _calculationPeriodDates.getCalculationPeriodFrequency()).get());
			} else {
				frequency = null;
			}
			
			return Optional.ofNullable(frequency)
				.map(o -> o.prune())
				.orElse(null);
		}
		
		@Override
		protected MapperC<? extends NonNegativeQuantitySchedule> quantity(CalculationPeriodDates calculationPeriodDates, PriceQuantity priceQuantity) {
			return MapperS.of(priceQuantity).<FieldWithMetaNonNegativeQuantitySchedule>mapC("getQuantity", _priceQuantity -> _priceQuantity.getQuantity()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue())
				.filterItemNullSafe(item -> exists(item.<BigDecimal>map("getValue", measureBase -> measureBase.getValue())).or(areEqual(MapperS.of(distinct(item.<DatedValue>mapC("getDatedValue", measureSchedule -> measureSchedule.getDatedValue()).<BigDecimal>map("getValue", datedValue -> datedValue.getValue())).resultCount()), MapperS.of(1), CardinalityOperator.All)).get());
		}
	}
}
