package drr.regulation.common.meta;

import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyFunctionFactory;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorFactory;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import drr.regulation.common.ReportingRegime;
import drr.regulation.common.validation.ReportingRegimeTypeFormatValidator;
import drr.regulation.common.validation.ReportingRegimeValidator;
import drr.regulation.common.validation.exists.ReportingRegimeOnlyExistsValidator;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Function;


/**
 * @version 4.2.1
 */
@RosettaMeta(model=ReportingRegime.class)
public class ReportingRegimeMeta implements RosettaMetaData<ReportingRegime> {

	@Override
	public List<Validator<? super ReportingRegime>> dataRules(ValidatorFactory factory) {
		return Arrays.asList(
			factory.create(drr.regulation.common.validation.datarule.ReportingRegimeSupervisorInformation.class),
			factory.create(drr.regulation.common.validation.datarule.ReportingRegimeRegimeInformation.class)
		);
	}
	
	@Override
	public List<Function<? super ReportingRegime, QualifyResult>> getQualifyFunctions(QualifyFunctionFactory factory) {
		return Collections.emptyList();
	}

	@Override
	public Validator<? super ReportingRegime> validator() {
		return new ReportingRegimeValidator();
	}

	@Override
	public Validator<? super ReportingRegime> typeFormatValidator() {
		return new ReportingRegimeTypeFormatValidator();
	}
	
	@Override
	public ValidatorWithArg<? super ReportingRegime, Set<String>> onlyExistsValidator() {
		return new ReportingRegimeOnlyExistsValidator();
	}
}
