package drr.regulation.common.reports;

import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.NestedFieldValueImpl;
import drr.regulation.common.PriceReportablePeriod;
import drr.regulation.esma.emir.refit.trade.reports.PriceAndNotationTypeTabulator;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import javax.inject.Inject;


public class PriceReportablePeriodTypeTabulator implements Tabulator<PriceReportablePeriod> {
	private final Field periodValueField;
	
	private final PriceAndNotationTypeTabulator priceAndNotationTypeTabulator;
	
	@Inject
	public PriceReportablePeriodTypeTabulator(PriceAndNotationTypeTabulator priceAndNotationTypeTabulator) {
		this.priceAndNotationTypeTabulator = priceAndNotationTypeTabulator;
		this.periodValueField = new FieldImpl(
			"periodValue",
			false,
			Optional.empty(),
			Optional.empty(),
			priceAndNotationTypeTabulator.getFields()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(periodValueField);
	}
	
	@Override
	public List<FieldValue> tabulate(PriceReportablePeriod input) {
		Optional<List<FieldValue>> periodValue = Optional.ofNullable(input.getPeriodValue())
			.map(x -> priceAndNotationTypeTabulator.tabulate(x));
		return Arrays.asList(
			new NestedFieldValueImpl(periodValueField, periodValue)
		);
	}
}
