package drr.regulation.common.validation;

import cdm.base.staticdata.party.metafields.ReferenceWithMetaParty;
import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.PartyInformation;
import drr.regulation.common.ReportingRegime;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class PartyInformationValidator implements Validator<PartyInformation> {

	private List<ComparisonResult> getComparisonResults(PartyInformation o) {
		return Lists.<ComparisonResult>newArrayList(
				checkCardinality("partyReference", (ReferenceWithMetaParty) o.getPartyReference() != null ? 1 : 0, 1, 1), 
				checkCardinality("regimeInformation", (List<? extends ReportingRegime>) o.getRegimeInformation() == null ? 0 : ((List<? extends ReportingRegime>) o.getRegimeInformation()).size(), 1, 0)
			);
	}

	@Override
	public ValidationResult<PartyInformation> validate(RosettaPath path, PartyInformation o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("PartyInformation", ValidationType.CARDINALITY, "PartyInformation", path, "", error);
		}
		return success("PartyInformation", ValidationType.CARDINALITY, "PartyInformation", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, PartyInformation o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("PartyInformation", ValidationType.CARDINALITY, "PartyInformation", path, "", res.getError());
				}
				return success("PartyInformation", ValidationType.CARDINALITY, "PartyInformation", path, "");
			})
			.collect(toList());
	}

}
