package drr.regulation.common.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.CollateralDetails;
import drr.regulation.common.RegimeReportableCollateral;
import drr.regulation.common.ReportableInformation;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class RegimeReportableCollateralValidator implements Validator<RegimeReportableCollateral> {

	private List<ComparisonResult> getComparisonResults(RegimeReportableCollateral o) {
		return Lists.<ComparisonResult>newArrayList(
				checkCardinality("collateralDetails", (CollateralDetails) o.getCollateralDetails() != null ? 1 : 0, 0, 1), 
				checkCardinality("reportableInformation", (ReportableInformation) o.getReportableInformation() != null ? 1 : 0, 1, 1)
			);
	}

	@Override
	public ValidationResult<RegimeReportableCollateral> validate(RosettaPath path, RegimeReportableCollateral o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("RegimeReportableCollateral", ValidationType.CARDINALITY, "RegimeReportableCollateral", path, "", error);
		}
		return success("RegimeReportableCollateral", ValidationType.CARDINALITY, "RegimeReportableCollateral", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, RegimeReportableCollateral o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("RegimeReportableCollateral", ValidationType.CARDINALITY, "RegimeReportableCollateral", path, "", res.getError());
				}
				return success("RegimeReportableCollateral", ValidationType.CARDINALITY, "RegimeReportableCollateral", path, "");
			})
			.collect(toList());
	}

}
