package drr.regulation.common.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.ClearingException;
import drr.regulation.common.ClearingExceptionReasonEnum;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ClearingExceptionClearingException")
@ImplementedBy(ClearingExceptionClearingException.Default.class)
public interface ClearingExceptionClearingException extends Validator<ClearingException> {
	
	String NAME = "ClearingExceptionClearingException";
	String DEFINITION = "if clearingException = True then clearingExceptionReason exists else if clearingExceptionReason exists then clearingException = True";
	
	ValidationResult<ClearingException> validate(RosettaPath path, ClearingException clearingException);
	
	class Default implements ClearingExceptionClearingException {
	
		@Override
		public ValidationResult<ClearingException> validate(RosettaPath path, ClearingException clearingException) {
			ComparisonResult result = executeDataRule(clearingException);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ClearingException", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ClearingException", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ClearingException clearingException) {
			try {
				if (areEqual(MapperS.of(clearingException).<Boolean>map("getClearingException", _clearingException -> _clearingException.getClearingException()), MapperS.of(true), CardinalityOperator.All).getOrDefault(false)) {
					return exists(MapperS.of(clearingException).<ClearingExceptionReasonEnum>mapC("getClearingExceptionReason", _clearingException -> _clearingException.getClearingExceptionReason()));
				}
				if (exists(MapperS.of(clearingException).<ClearingExceptionReasonEnum>mapC("getClearingExceptionReason", _clearingException -> _clearingException.getClearingExceptionReason())).getOrDefault(false)) {
					return areEqual(MapperS.of(clearingException).<Boolean>map("getClearingException", _clearingException -> _clearingException.getClearingException()), MapperS.of(true), CardinalityOperator.All);
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ClearingExceptionClearingException {
	
		@Override
		public ValidationResult<ClearingException> validate(RosettaPath path, ClearingException clearingException) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ClearingException", path, DEFINITION);
		}
	}
}
