package drr.regulation.common.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.RegimeReportingSide;
import drr.regulation.common.ReportingSide;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("RegimeReportingSideReportingSide")
@ImplementedBy(RegimeReportingSideReportingSide.Default.class)
public interface RegimeReportingSideReportingSide extends Validator<RegimeReportingSide> {
	
	String NAME = "RegimeReportingSideReportingSide";
	String DEFINITION = "reportingSide exists or delegatedReportingSide exists";
	
	ValidationResult<RegimeReportingSide> validate(RosettaPath path, RegimeReportingSide regimeReportingSide);
	
	class Default implements RegimeReportingSideReportingSide {
	
		@Override
		public ValidationResult<RegimeReportingSide> validate(RosettaPath path, RegimeReportingSide regimeReportingSide) {
			ComparisonResult result = executeDataRule(regimeReportingSide);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "RegimeReportingSide", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "RegimeReportingSide", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(RegimeReportingSide regimeReportingSide) {
			try {
				return exists(MapperS.of(regimeReportingSide).<ReportingSide>map("getReportingSide", _regimeReportingSide -> _regimeReportingSide.getReportingSide())).or(exists(MapperS.of(regimeReportingSide).<ReportingSide>map("getDelegatedReportingSide", _regimeReportingSide -> _regimeReportingSide.getDelegatedReportingSide())));
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements RegimeReportingSideReportingSide {
	
		@Override
		public ValidationResult<RegimeReportingSide> validate(RosettaPath path, RegimeReportingSide regimeReportingSide) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "RegimeReportingSide", path, DEFINITION);
		}
	}
}
