package drr.regulation.common.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.CFTCPartyInformation;
import drr.regulation.common.ESMAPartyInformation;
import drr.regulation.common.ReportingRegime;
import drr.regulation.common.SECPartyInformation;
import drr.regulation.common.SupervisoryBodyEnum;
import drr.regulation.common.metafields.FieldWithMetaSupervisoryBodyEnum;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ReportingRegimeRegimeInformation")
@ImplementedBy(ReportingRegimeRegimeInformation.Default.class)
public interface ReportingRegimeRegimeInformation extends Validator<ReportingRegime> {
	
	String NAME = "ReportingRegimeRegimeInformation";
	String DEFINITION = "if supervisoryBody = SupervisoryBodyEnum -> CFTC then cftcPartyInformation exists else if supervisoryBody = SupervisoryBodyEnum -> SEC then secPartyInformation exists else if supervisoryBody = SupervisoryBodyEnum -> ESMA then esmaPartyInformation exists";
	
	ValidationResult<ReportingRegime> validate(RosettaPath path, ReportingRegime reportingRegime);
	
	class Default implements ReportingRegimeRegimeInformation {
	
		@Override
		public ValidationResult<ReportingRegime> validate(RosettaPath path, ReportingRegime reportingRegime) {
			ComparisonResult result = executeDataRule(reportingRegime);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ReportingRegime", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ReportingRegime", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ReportingRegime reportingRegime) {
			try {
				if (areEqual(MapperS.of(reportingRegime).<FieldWithMetaSupervisoryBodyEnum>map("getSupervisoryBody", _reportingRegime -> _reportingRegime.getSupervisoryBody()).<SupervisoryBodyEnum>map("getValue", _f->_f.getValue()), MapperS.of(SupervisoryBodyEnum.CFTC), CardinalityOperator.All).getOrDefault(false)) {
					return exists(MapperS.of(reportingRegime).<CFTCPartyInformation>map("getCftcPartyInformation", _reportingRegime -> _reportingRegime.getCftcPartyInformation()));
				}
				if (areEqual(MapperS.of(reportingRegime).<FieldWithMetaSupervisoryBodyEnum>map("getSupervisoryBody", _reportingRegime -> _reportingRegime.getSupervisoryBody()).<SupervisoryBodyEnum>map("getValue", _f->_f.getValue()), MapperS.of(SupervisoryBodyEnum.SEC), CardinalityOperator.All).getOrDefault(false)) {
					return exists(MapperS.of(reportingRegime).<SECPartyInformation>map("getSecPartyInformation", _reportingRegime -> _reportingRegime.getSecPartyInformation()));
				}
				if (areEqual(MapperS.of(reportingRegime).<FieldWithMetaSupervisoryBodyEnum>map("getSupervisoryBody", _reportingRegime -> _reportingRegime.getSupervisoryBody()).<SupervisoryBodyEnum>map("getValue", _f->_f.getValue()), MapperS.of(SupervisoryBodyEnum.ESMA), CardinalityOperator.All).getOrDefault(false)) {
					return exists(MapperS.of(reportingRegime).<ESMAPartyInformation>map("getEsmaPartyInformation", _reportingRegime -> _reportingRegime.getEsmaPartyInformation()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ReportingRegimeRegimeInformation {
	
		@Override
		public ValidationResult<ReportingRegime> validate(RosettaPath path, ReportingRegime reportingRegime) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ReportingRegime", path, DEFINITION);
		}
	}
}
