package drr.regulation.common.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.RegimeNameEnum;
import drr.regulation.common.ReportingRegime;
import drr.regulation.common.SupervisoryBodyEnum;
import drr.regulation.common.metafields.FieldWithMetaRegimeNameEnum;
import drr.regulation.common.metafields.FieldWithMetaSupervisoryBodyEnum;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ReportingRegimeSupervisorInformation")
@ImplementedBy(ReportingRegimeSupervisorInformation.Default.class)
public interface ReportingRegimeSupervisorInformation extends Validator<ReportingRegime> {
	
	String NAME = "ReportingRegimeSupervisorInformation";
	String DEFINITION = "if regimeName = RegimeNameEnum -> DoddFrankAct then supervisoryBody = SupervisoryBodyEnum -> CFTC or supervisoryBody = SupervisoryBodyEnum -> SEC";
	
	ValidationResult<ReportingRegime> validate(RosettaPath path, ReportingRegime reportingRegime);
	
	class Default implements ReportingRegimeSupervisorInformation {
	
		@Override
		public ValidationResult<ReportingRegime> validate(RosettaPath path, ReportingRegime reportingRegime) {
			ComparisonResult result = executeDataRule(reportingRegime);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ReportingRegime", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ReportingRegime", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ReportingRegime reportingRegime) {
			try {
				if (areEqual(MapperS.of(reportingRegime).<FieldWithMetaRegimeNameEnum>map("getRegimeName", _reportingRegime -> _reportingRegime.getRegimeName()).<RegimeNameEnum>map("getValue", _f->_f.getValue()), MapperS.of(RegimeNameEnum.DODD_FRANK_ACT), CardinalityOperator.All).getOrDefault(false)) {
					return areEqual(MapperS.of(reportingRegime).<FieldWithMetaSupervisoryBodyEnum>map("getSupervisoryBody", _reportingRegime -> _reportingRegime.getSupervisoryBody()).<SupervisoryBodyEnum>map("getValue", _f->_f.getValue()), MapperS.of(SupervisoryBodyEnum.CFTC), CardinalityOperator.All).or(areEqual(MapperS.of(reportingRegime).<FieldWithMetaSupervisoryBodyEnum>map("getSupervisoryBody", _reportingRegime -> _reportingRegime.getSupervisoryBody()).<SupervisoryBodyEnum>map("getValue", _f->_f.getValue()), MapperS.of(SupervisoryBodyEnum.SEC), CardinalityOperator.All));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ReportingRegimeSupervisorInformation {
	
		@Override
		public ValidationResult<ReportingRegime> validate(RosettaPath path, ReportingRegime reportingRegime) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ReportingRegime", path, DEFINITION);
		}
	}
}
