package drr.regulation.common.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.ESMATransactionInformation;
import drr.regulation.common.SupervisoryBodyEnum;
import drr.regulation.common.TransactionInformation;
import drr.regulation.common.metafields.FieldWithMetaSupervisoryBodyEnum;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("TransactionInformationRegimeInformation")
@ImplementedBy(TransactionInformationRegimeInformation.Default.class)
public interface TransactionInformationRegimeInformation extends Validator<TransactionInformation> {
	
	String NAME = "TransactionInformationRegimeInformation";
	String DEFINITION = "if supervisoryBody = SupervisoryBodyEnum -> ESMA then esmaTransactionInformation exists";
	
	ValidationResult<TransactionInformation> validate(RosettaPath path, TransactionInformation transactionInformation);
	
	class Default implements TransactionInformationRegimeInformation {
	
		@Override
		public ValidationResult<TransactionInformation> validate(RosettaPath path, TransactionInformation transactionInformation) {
			ComparisonResult result = executeDataRule(transactionInformation);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "TransactionInformation", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "TransactionInformation", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(TransactionInformation transactionInformation) {
			try {
				if (areEqual(MapperS.of(transactionInformation).<FieldWithMetaSupervisoryBodyEnum>map("getSupervisoryBody", _transactionInformation -> _transactionInformation.getSupervisoryBody()).<SupervisoryBodyEnum>map("getValue", _f->_f.getValue()), MapperS.of(SupervisoryBodyEnum.ESMA), CardinalityOperator.All).getOrDefault(false)) {
					return exists(MapperS.of(transactionInformation).<ESMATransactionInformation>map("getEsmaTransactionInformation", _transactionInformation -> _transactionInformation.getEsmaTransactionInformation()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements TransactionInformationRegimeInformation {
	
		@Override
		public ValidationResult<TransactionInformation> validate(RosettaPath path, TransactionInformation transactionInformation) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "TransactionInformation", path, DEFINITION);
		}
	}
}
