package drr.regulation.esma.emir.refit.margin.meta;

import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyFunctionFactory;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorFactory;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import drr.regulation.esma.emir.refit.margin.ESMAEMIRMarginReport;
import drr.regulation.esma.emir.refit.margin.validation.ESMAEMIRMarginReportTypeFormatValidator;
import drr.regulation.esma.emir.refit.margin.validation.ESMAEMIRMarginReportValidator;
import drr.regulation.esma.emir.refit.margin.validation.exists.ESMAEMIRMarginReportOnlyExistsValidator;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Function;


/**
 * @version 4.2.1
 */
@RosettaMeta(model=ESMAEMIRMarginReport.class)
public class ESMAEMIRMarginReportMeta implements RosettaMetaData<ESMAEMIRMarginReport> {

	@Override
	public List<Validator<? super ESMAEMIRMarginReport>> dataRules(ValidatorFactory factory) {
		return Arrays.asList(
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300101.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300102.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300103.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300104.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300201.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300202.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300203.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300301.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300302.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300303.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300401.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300402.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300403.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300404.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300501.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300502.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300601.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300602.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300603.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300604.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300605.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300606.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300701.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300702.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300703.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300704.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300801.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300802.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300901.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR300902.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301001.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301002.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301101.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301201.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301202.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301301.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301302.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301303.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301401.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301402.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301501.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301502.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301503.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301504.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301601.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301602.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301603.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301604.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301701.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301702.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301801.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301901.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR301902.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302001.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302002.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302101.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302102.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302103.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302201.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302202.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302301.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302302.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302303.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302304.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302401.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302402.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302403.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302404.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302501.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302502.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302601.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302701.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302702.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302801.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302901.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302902.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302903.class),
			factory.create(drr.regulation.esma.emir.refit.margin.validation.datarule.ESMAEMIRMarginReportEMIRVR302904.class)
		);
	}
	
	@Override
	public List<Function<? super ESMAEMIRMarginReport, QualifyResult>> getQualifyFunctions(QualifyFunctionFactory factory) {
		return Collections.emptyList();
	}

	@Override
	public Validator<? super ESMAEMIRMarginReport> validator() {
		return new ESMAEMIRMarginReportValidator();
	}

	@Override
	public Validator<? super ESMAEMIRMarginReport> typeFormatValidator() {
		return new ESMAEMIRMarginReportTypeFormatValidator();
	}
	
	@Override
	public ValidatorWithArg<? super ESMAEMIRMarginReport, Set<String>> onlyExistsValidator() {
		return new ESMAEMIRMarginReportOnlyExistsValidator();
	}
}
