package drr.regulation.esma.emir.refit.margin.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaReport;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.enrichment.common.reports.CollateralEnrichmentDataRule;
import drr.regulation.common.CollateralReportInstruction;
import drr.regulation.esma.emir.refit.margin.ESMAEMIRMarginReport;
import drr.regulation.esma.emir.refit.margin.ESMAEMIRMarginReport.ESMAEMIRMarginReportBuilder;
import java.util.Optional;
import javax.inject.Inject;


@RosettaReport(namespace="drr.regulation.esma.emir.refit.margin", body="ESMA", corpusList={"EMIR", "Margin"})
@ImplementedBy(ESMAEMIRMarginReportFunction.ESMAEMIRMarginReportFunctionDefault.class)
public abstract class ESMAEMIRMarginReportFunction implements ReportFunction<CollateralReportInstruction, ESMAEMIRMarginReport> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ActionTypeRule actionType;
	@Inject protected CollateralEnrichmentDataRule collateralEnrichmentData;
	@Inject protected CollateralPortfolioCodeRule collateralPortfolioCode;
	@Inject protected CollateralPortfolioIndicatorRule collateralPortfolioIndicator;
	@Inject protected CollateralTimestampRule collateralTimestamp;
	@Inject protected CollateralisationCategoryRule collateralisationCategory;
	@Inject protected Counterparty1Rule counterparty1;
	@Inject protected Counterparty2Rule counterparty2;
	@Inject protected Counterparty2IdentifierTypeRule counterparty2IdentifierType;
	@Inject protected CurrencyOfExcessCollateralCollectedRule currencyOfExcessCollateralCollected;
	@Inject protected CurrencyOfTheExcessCollateralPostedRule currencyOfTheExcessCollateralPosted;
	@Inject protected EntityResponsibleForReportingRule entityResponsibleForReporting;
	@Inject protected EventDateRule eventDate;
	@Inject protected ExcessCollateralCollectedByTheCounterparty1Rule excessCollateralCollectedByTheCounterparty1;
	@Inject protected ExcessCollateralPostedByTheCounterparty1Rule excessCollateralPostedByTheCounterparty1;
	@Inject protected InitialMarginCollectedByCounterparty1CurrencyRule initialMarginCollectedByCounterparty1Currency;
	@Inject protected InitialMarginCollectedByCounterparty1PostHaircutRule initialMarginCollectedByCounterparty1PostHaircut;
	@Inject protected InitialMarginCollectedByCounterparty1PreHaircutRule initialMarginCollectedByCounterparty1PreHaircut;
	@Inject protected InitialMarginPostedByCounterparty1CurrencyRule initialMarginPostedByCounterparty1Currency;
	@Inject protected InitialMarginPostedByTheCounterparty1PostHaircutRule initialMarginPostedByTheCounterparty1PostHaircut;
	@Inject protected InitialMarginPostedByTheCounterparty1PreHaircutRule initialMarginPostedByTheCounterparty1PreHaircut;
	@Inject protected MICCollateralRule mICCollateral;
	@Inject protected ReportSubmittingEntityIDRule reportSubmittingEntityID;
	@Inject protected ReportingTimestampRule reportingTimestamp;
	@Inject protected UTIRule uTI;
	@Inject protected VariationMarginCollectedByCounterparty1CurrencyRule variationMarginCollectedByCounterparty1Currency;
	@Inject protected VariationMarginCollectedByTheCounterparty1PostHaircutRule variationMarginCollectedByTheCounterparty1PostHaircut;
	@Inject protected VariationMarginCollectedByTheCounterparty1PreHaircutRule variationMarginCollectedByTheCounterparty1PreHaircut;
	@Inject protected VariationMarginPostedByCounterparty1CurrencyRule variationMarginPostedByCounterparty1Currency;
	@Inject protected VariationMarginPostedByTheCounterparty1PostHaircutRule variationMarginPostedByTheCounterparty1PostHaircut;
	@Inject protected VariationMarginPostedByTheCounterparty1PreHaircutRule variationMarginPostedByTheCounterparty1PreHaircut;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public ESMAEMIRMarginReport evaluate(CollateralReportInstruction input) {
		ESMAEMIRMarginReport.ESMAEMIRMarginReportBuilder outputBuilder = doEvaluate(input);
		
		final ESMAEMIRMarginReport output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.build();
			objectValidator.validate(ESMAEMIRMarginReport.class, output);
		}
		
		return output;
	}

	protected abstract ESMAEMIRMarginReport.ESMAEMIRMarginReportBuilder doEvaluate(CollateralReportInstruction input);

	public static class ESMAEMIRMarginReportFunctionDefault extends ESMAEMIRMarginReportFunction {
		@Override
		protected ESMAEMIRMarginReport.ESMAEMIRMarginReportBuilder doEvaluate(CollateralReportInstruction input) {
			ESMAEMIRMarginReport.ESMAEMIRMarginReportBuilder output = ESMAEMIRMarginReport.builder();
			return assignOutput(output, input);
		}
		
		protected ESMAEMIRMarginReport.ESMAEMIRMarginReportBuilder assignOutput(ESMAEMIRMarginReport.ESMAEMIRMarginReportBuilder output, CollateralReportInstruction input) {
			output
				.getOrCreatePartiesToTheDerivative()
				.setReportingTimestamp(reportingTimestamp.evaluate(input));
			
			output
				.getOrCreatePartiesToTheDerivative()
				.setReportSubmittingEntityID(reportSubmittingEntityID.evaluate(input));
			
			output
				.getOrCreatePartiesToTheDerivative()
				.setEntityResponsibleForReporting(entityResponsibleForReporting.evaluate(input));
			
			output
				.getOrCreatePartiesToTheDerivative()
				.setCounterparty1(counterparty1.evaluate(input));
			
			output
				.getOrCreatePartiesToTheDerivative()
				.setCounterparty2IdentifierType(counterparty2IdentifierType.evaluate(input));
			
			output
				.getOrCreatePartiesToTheDerivative()
				.setCounterparty2(counterparty2.evaluate(input));
			
			output
				.getOrCreatePartiesToTheDerivative()
				.getOrCreateNonReportable()
				.setEnrichment(collateralEnrichmentData.evaluate(input));
			
			output
				.getOrCreatePartiesToTheDerivative()
				.getOrCreateNonReportable()
				.setMic(mICCollateral.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setCollateralTimestamp(collateralTimestamp.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setCollateralPortfolioCode(collateralPortfolioCode.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setCollateralPortfolioIndicator(collateralPortfolioIndicator.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setUti(uTI.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setCollateralisationCategory(collateralisationCategory.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setInitialMarginPostedByTheCounterparty1PreHaircut(initialMarginPostedByTheCounterparty1PreHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setInitialMarginPostedByTheCounterparty1PostHaircut(initialMarginPostedByTheCounterparty1PostHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setInitialMarginPostedByCounterparty1Currency(initialMarginPostedByCounterparty1Currency.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setVariationMarginPostedByTheCounterparty1PreHaircut(variationMarginPostedByTheCounterparty1PreHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setVariationMarginPostedByTheCounterparty1PostHaircut(variationMarginPostedByTheCounterparty1PostHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setVariationMarginPostedByCounterparty1Currency(variationMarginPostedByCounterparty1Currency.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setInitialMarginCollectedByCounterparty1PreHaircut(initialMarginCollectedByCounterparty1PreHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setInitialMarginCollectedByCounterparty1PostHaircut(initialMarginCollectedByCounterparty1PostHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setInitialMarginCollectedByCounterparty1Currency(initialMarginCollectedByCounterparty1Currency.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setVariationMarginCollectedByTheCounterparty1PreHaircut(variationMarginCollectedByTheCounterparty1PreHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setVariationMarginCollectedByTheCounterparty1PostHaircut(variationMarginCollectedByTheCounterparty1PostHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setVariationMarginCollectedByCounterparty1Currency(variationMarginCollectedByCounterparty1Currency.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setExcessCollateralPostedByTheCounterparty1(excessCollateralPostedByTheCounterparty1.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setCurrencyOfTheExcessCollateralPosted(currencyOfTheExcessCollateralPosted.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setExcessCollateralCollectedByTheCounterparty1(excessCollateralCollectedByTheCounterparty1.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setCurrencyOfTheExcessCollateralCollected(currencyOfExcessCollateralCollected.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setActionType(actionType.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setEventDate(eventDate.evaluate(input));
			
			return Optional.ofNullable(output)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
