package drr.regulation.esma.emir.refit.margin.reports;

import cdm.base.math.RoundingDirectionEnum;
import cdm.base.math.functions.RoundToPrecision;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.CollateralReportInstruction;
import drr.standards.iosco.cde.reports.CDEExcessCollateralPostedByTheReportingCounterpartyRule;
import java.math.BigDecimal;
import javax.inject.Inject;


@ImplementedBy(ExcessCollateralPostedByTheCounterparty1Rule.ExcessCollateralPostedByTheCounterparty1RuleDefault.class)
public abstract class ExcessCollateralPostedByTheCounterparty1Rule implements ReportFunction<CollateralReportInstruction, BigDecimal> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDEExcessCollateralPostedByTheReportingCounterpartyRule cDEExcessCollateralPostedByTheReportingCounterparty;
	@Inject protected RoundToPrecision roundToPrecision;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public BigDecimal evaluate(CollateralReportInstruction input) {
		BigDecimal output = doEvaluate(input);
		
		return output;
	}

	protected abstract BigDecimal doEvaluate(CollateralReportInstruction input);

	public static class ExcessCollateralPostedByTheCounterparty1RuleDefault extends ExcessCollateralPostedByTheCounterparty1Rule {
		@Override
		protected BigDecimal doEvaluate(CollateralReportInstruction input) {
			BigDecimal output = null;
			return assignOutput(output, input);
		}
		
		protected BigDecimal assignOutput(BigDecimal output, CollateralReportInstruction input) {
			final MapperS<Integer> thenResult = MapperS.of(cDEExcessCollateralPostedByTheReportingCounterparty.evaluate(input));
			output = thenResult
				.mapSingleToItem(item -> {
					final Integer integer = item.get();
					return MapperS.of(roundToPrecision.evaluate((integer == null ? null : BigDecimal.valueOf(integer)), 5, RoundingDirectionEnum.NEAREST));
				}).get();
			
			return output;
		}
	}
}
