package drr.regulation.esma.emir.refit.margin.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.model.lib.reports.Tabulator.NestedFieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.esma.emir.refit.margin.PartiesToTheDerivative;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import javax.inject.Inject;


public class PartiesToTheDerivativeTypeTabulator implements Tabulator<PartiesToTheDerivative> {
	private final Field reportingTimestampField;
	private final Field reportSubmittingEntityIDField;
	private final Field entityResponsibleForReportingField;
	private final Field counterparty1Field;
	private final Field counterparty2IdentifierTypeField;
	private final Field counterparty2Field;
	private final Field nonReportableField;
	
	private final ESMAEMIRNonReportableCollateralDataTypeTabulator eSMAEMIRNonReportableCollateralDataTypeTabulator;
	
	@Inject
	public PartiesToTheDerivativeTypeTabulator(ESMAEMIRNonReportableCollateralDataTypeTabulator eSMAEMIRNonReportableCollateralDataTypeTabulator) {
		this.eSMAEMIRNonReportableCollateralDataTypeTabulator = eSMAEMIRNonReportableCollateralDataTypeTabulator;
		this.reportingTimestampField = new FieldImpl(
			"reportingTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "margin"), "ReportingTimestamp")),
			Optional.of("3.1 Reporting Timestamp"),
			Arrays.asList()
		);
		this.reportSubmittingEntityIDField = new FieldImpl(
			"reportSubmittingEntityID",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "margin"), "ReportSubmittingEntityID")),
			Optional.of("3.2 Report Submitting entity ID"),
			Arrays.asList()
		);
		this.entityResponsibleForReportingField = new FieldImpl(
			"entityResponsibleForReporting",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "margin"), "EntityResponsibleForReporting")),
			Optional.of("3.3 Entity responsible for reporting"),
			Arrays.asList()
		);
		this.counterparty1Field = new FieldImpl(
			"counterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "margin"), "Counterparty1")),
			Optional.of("3.4 Counterparty 1 (Reporting counterparty)"),
			Arrays.asList()
		);
		this.counterparty2IdentifierTypeField = new FieldImpl(
			"counterparty2IdentifierType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "margin"), "Counterparty2IdentifierType")),
			Optional.of("3.5 Counterparty 2 identifier type"),
			Arrays.asList()
		);
		this.counterparty2Field = new FieldImpl(
			"counterparty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "margin"), "Counterparty2")),
			Optional.of("3.6 Counterparty 2"),
			Arrays.asList()
		);
		this.nonReportableField = new FieldImpl(
			"nonReportable",
			false,
			Optional.empty(),
			Optional.empty(),
			eSMAEMIRNonReportableCollateralDataTypeTabulator.getFields()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(reportingTimestampField, reportSubmittingEntityIDField, entityResponsibleForReportingField, counterparty1Field, counterparty2IdentifierTypeField, counterparty2Field, nonReportableField);
	}
	
	@Override
	public List<FieldValue> tabulate(PartiesToTheDerivative input) {
		Optional<ZonedDateTime> reportingTimestamp = Optional.ofNullable(input.getReportingTimestamp());
		Optional<String> reportSubmittingEntityID = Optional.ofNullable(input.getReportSubmittingEntityID());
		Optional<String> entityResponsibleForReporting = Optional.ofNullable(input.getEntityResponsibleForReporting());
		Optional<String> counterparty1 = Optional.ofNullable(input.getCounterparty1());
		Optional<Boolean> counterparty2IdentifierType = Optional.ofNullable(input.getCounterparty2IdentifierType());
		Optional<String> counterparty2 = Optional.ofNullable(input.getCounterparty2());
		Optional<List<FieldValue>> nonReportable = Optional.ofNullable(input.getNonReportable())
			.map(x -> eSMAEMIRNonReportableCollateralDataTypeTabulator.tabulate(x));
		return Arrays.asList(
			new FieldValueImpl(reportingTimestampField, reportingTimestamp),
			new FieldValueImpl(reportSubmittingEntityIDField, reportSubmittingEntityID),
			new FieldValueImpl(entityResponsibleForReportingField, entityResponsibleForReporting),
			new FieldValueImpl(counterparty1Field, counterparty1),
			new FieldValueImpl(counterparty2IdentifierTypeField, counterparty2IdentifierType),
			new FieldValueImpl(counterparty2Field, counterparty2),
			new NestedFieldValueImpl(nonReportableField, nonReportable)
		);
	}
}
