package drr.regulation.esma.emir.refit.margin.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.esma.emir.refit.margin.CollateralReport;
import drr.regulation.esma.emir.refit.margin.ESMAEMIRMarginReport;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ESMAEMIRMarginReportEMIR_VR_3016_01")
@ImplementedBy(ESMAEMIRMarginReportEMIRVR301601.Default.class)
public interface ESMAEMIRMarginReportEMIRVR301601 extends Validator<ESMAEMIRMarginReport> {
	
	String NAME = "ESMAEMIRMarginReportEMIR_VR_3016_01";
	String DEFINITION = "if collateral -> variationMarginPostedByTheCounterparty1PreHaircut exists then collateral -> variationMarginPostedByTheCounterparty1PostHaircut >= 0";
	
	ValidationResult<ESMAEMIRMarginReport> validate(RosettaPath path, ESMAEMIRMarginReport eSMAEMIRMarginReport);
	
	class Default implements ESMAEMIRMarginReportEMIRVR301601 {
	
		@Override
		public ValidationResult<ESMAEMIRMarginReport> validate(RosettaPath path, ESMAEMIRMarginReport eSMAEMIRMarginReport) {
			ComparisonResult result = executeDataRule(eSMAEMIRMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ESMAEMIRMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ESMAEMIRMarginReport eSMAEMIRMarginReport) {
			try {
				if (exists(MapperS.of(eSMAEMIRMarginReport).<CollateralReport>map("getCollateral", _eSMAEMIRMarginReport -> _eSMAEMIRMarginReport.getCollateral()).<BigDecimal>map("getVariationMarginPostedByTheCounterparty1PreHaircut", collateralReport -> collateralReport.getVariationMarginPostedByTheCounterparty1PreHaircut())).getOrDefault(false)) {
					return greaterThanEquals(MapperS.of(eSMAEMIRMarginReport).<CollateralReport>map("getCollateral", _eSMAEMIRMarginReport -> _eSMAEMIRMarginReport.getCollateral()).<BigDecimal>map("getVariationMarginPostedByTheCounterparty1PostHaircut", collateralReport -> collateralReport.getVariationMarginPostedByTheCounterparty1PostHaircut()), MapperS.of(BigDecimal.valueOf(0)), CardinalityOperator.All);
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ESMAEMIRMarginReportEMIRVR301601 {
	
		@Override
		public ValidationResult<ESMAEMIRMarginReport> validate(RosettaPath path, ESMAEMIRMarginReport eSMAEMIRMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRMarginReport", path, DEFINITION);
		}
	}
}
