package drr.regulation.esma.emir.refit.margin.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.esma.emir.refit.margin.CollateralReport;
import drr.regulation.esma.emir.refit.margin.ESMAEMIRMarginReport;
import iso20022.auth108.esma.CollateralisationType3Code__1;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ESMAEMIRMarginReportEMIR_VR_3016_02")
@ImplementedBy(ESMAEMIRMarginReportEMIRVR301602.Default.class)
public interface ESMAEMIRMarginReportEMIRVR301602 extends Validator<ESMAEMIRMarginReport> {
	
	String NAME = "ESMAEMIRMarginReportEMIR_VR_3016_02";
	String DEFINITION = "if [CollateralisationType3Code__1 -> UNCL, CollateralisationType3Code__1 -> PRC2, CollateralisationType3Code__1 -> OWC2] contains collateral -> collateralisationCategory then collateral -> variationMarginPostedByTheCounterparty1PostHaircut is absent else collateral -> variationMarginPostedByTheCounterparty1PostHaircut exists";
	
	ValidationResult<ESMAEMIRMarginReport> validate(RosettaPath path, ESMAEMIRMarginReport eSMAEMIRMarginReport);
	
	class Default implements ESMAEMIRMarginReportEMIRVR301602 {
	
		@Override
		public ValidationResult<ESMAEMIRMarginReport> validate(RosettaPath path, ESMAEMIRMarginReport eSMAEMIRMarginReport) {
			ComparisonResult result = executeDataRule(eSMAEMIRMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ESMAEMIRMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ESMAEMIRMarginReport eSMAEMIRMarginReport) {
			try {
				if (contains(MapperC.<CollateralisationType3Code__1>of(MapperS.of(CollateralisationType3Code__1.UNCL), MapperS.of(CollateralisationType3Code__1.PRC2), MapperS.of(CollateralisationType3Code__1.OWC2)), MapperS.of(eSMAEMIRMarginReport).<CollateralReport>map("getCollateral", _eSMAEMIRMarginReport -> _eSMAEMIRMarginReport.getCollateral()).<CollateralisationType3Code__1>map("getCollateralisationCategory", collateralReport -> collateralReport.getCollateralisationCategory())).getOrDefault(false)) {
					return notExists(MapperS.of(eSMAEMIRMarginReport).<CollateralReport>map("getCollateral", _eSMAEMIRMarginReport -> _eSMAEMIRMarginReport.getCollateral()).<BigDecimal>map("getVariationMarginPostedByTheCounterparty1PostHaircut", collateralReport -> collateralReport.getVariationMarginPostedByTheCounterparty1PostHaircut()));
				}
				return exists(MapperS.of(eSMAEMIRMarginReport).<CollateralReport>map("getCollateral", _eSMAEMIRMarginReport -> _eSMAEMIRMarginReport.getCollateral()).<BigDecimal>map("getVariationMarginPostedByTheCounterparty1PostHaircut", collateralReport -> collateralReport.getVariationMarginPostedByTheCounterparty1PostHaircut()));
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ESMAEMIRMarginReportEMIRVR301602 {
	
		@Override
		public ValidationResult<ESMAEMIRMarginReport> validate(RosettaPath path, ESMAEMIRMarginReport eSMAEMIRMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRMarginReport", path, DEFINITION);
		}
	}
}
