package drr.regulation.esma.emir.refit.margin.validation.datarule;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.esma.emir.refit.margin.CollateralReport;
import drr.regulation.esma.emir.refit.margin.ESMAEMIRMarginReport;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ESMAEMIRMarginReportEMIR_VR_3025_01")
@ImplementedBy(ESMAEMIRMarginReportEMIRVR302501.Default.class)
public interface ESMAEMIRMarginReportEMIRVR302501 extends Validator<ESMAEMIRMarginReport> {
	
	String NAME = "ESMAEMIRMarginReportEMIR_VR_3025_01";
	String DEFINITION = "if collateral -> variationMarginCollectedByTheCounterparty1PreHaircut exists and collateral -> variationMarginCollectedByTheCounterparty1PostHaircut exists then True else collateral -> variationMarginCollectedByCounterparty1Currency is absent";
	
	ValidationResult<ESMAEMIRMarginReport> validate(RosettaPath path, ESMAEMIRMarginReport eSMAEMIRMarginReport);
	
	class Default implements ESMAEMIRMarginReportEMIRVR302501 {
	
		@Override
		public ValidationResult<ESMAEMIRMarginReport> validate(RosettaPath path, ESMAEMIRMarginReport eSMAEMIRMarginReport) {
			ComparisonResult result = executeDataRule(eSMAEMIRMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ESMAEMIRMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ESMAEMIRMarginReport eSMAEMIRMarginReport) {
			try {
				if (exists(MapperS.of(eSMAEMIRMarginReport).<CollateralReport>map("getCollateral", _eSMAEMIRMarginReport -> _eSMAEMIRMarginReport.getCollateral()).<BigDecimal>map("getVariationMarginCollectedByTheCounterparty1PreHaircut", collateralReport -> collateralReport.getVariationMarginCollectedByTheCounterparty1PreHaircut())).and(exists(MapperS.of(eSMAEMIRMarginReport).<CollateralReport>map("getCollateral", _eSMAEMIRMarginReport -> _eSMAEMIRMarginReport.getCollateral()).<BigDecimal>map("getVariationMarginCollectedByTheCounterparty1PostHaircut", collateralReport -> collateralReport.getVariationMarginCollectedByTheCounterparty1PostHaircut()))).getOrDefault(false)) {
					return ComparisonResult.of(MapperS.of(true));
				}
				return notExists(MapperS.of(eSMAEMIRMarginReport).<CollateralReport>map("getCollateral", _eSMAEMIRMarginReport -> _eSMAEMIRMarginReport.getCollateral()).<ISOCurrencyCodeEnum>map("getVariationMarginCollectedByCounterparty1Currency", collateralReport -> collateralReport.getVariationMarginCollectedByCounterparty1Currency()));
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ESMAEMIRMarginReportEMIRVR302501 {
	
		@Override
		public ValidationResult<ESMAEMIRMarginReport> validate(RosettaPath path, ESMAEMIRMarginReport eSMAEMIRMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRMarginReport", path, DEFINITION);
		}
	}
}
