package drr.regulation.esma.emir.refit.trade;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import drr.enrichment.common.EnrichmentData;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRNonReportable;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilderImpl;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRNonReportable.ESMAEMIRNonReportableImpl;
import drr.regulation.esma.emir.refit.trade.meta.ESMAEMIRNonReportableMeta;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="ESMAEMIRNonReportable", builder=ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilderImpl.class, version="4.2.1")
public interface ESMAEMIRNonReportable extends RosettaModelObject {

	ESMAEMIRNonReportableMeta metaData = new ESMAEMIRNonReportableMeta();

	/*********************** Getter Methods  ***********************/
	/**
	 * Populated by pre-enrichment process
	 */
	EnrichmentData getEnrichment();

	/*********************** Build Methods  ***********************/
	ESMAEMIRNonReportable build();
	
	ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder toBuilder();
	
	static ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder builder() {
		return new ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends ESMAEMIRNonReportable> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends ESMAEMIRNonReportable> getType() {
		return ESMAEMIRNonReportable.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processRosetta(path.newSubPath("enrichment"), processor, EnrichmentData.class, getEnrichment());
	}
	

	/*********************** Builder Interface  ***********************/
	interface ESMAEMIRNonReportableBuilder extends ESMAEMIRNonReportable, RosettaModelObjectBuilder {
		EnrichmentData.EnrichmentDataBuilder getOrCreateEnrichment();
		EnrichmentData.EnrichmentDataBuilder getEnrichment();
		ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder setEnrichment(EnrichmentData enrichment);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processRosetta(path.newSubPath("enrichment"), processor, EnrichmentData.EnrichmentDataBuilder.class, getEnrichment());
		}
		

		ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder prune();
	}

	/*********************** Immutable Implementation of ESMAEMIRNonReportable  ***********************/
	class ESMAEMIRNonReportableImpl implements ESMAEMIRNonReportable {
		private final EnrichmentData enrichment;
		
		protected ESMAEMIRNonReportableImpl(ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder builder) {
			this.enrichment = ofNullable(builder.getEnrichment()).map(f->f.build()).orElse(null);
		}
		
		@Override
		@RosettaAttribute("enrichment")
		public EnrichmentData getEnrichment() {
			return enrichment;
		}
		
		@Override
		public ESMAEMIRNonReportable build() {
			return this;
		}
		
		@Override
		public ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder toBuilder() {
			ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder builder) {
			ofNullable(getEnrichment()).ifPresent(builder::setEnrichment);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			ESMAEMIRNonReportable _that = getType().cast(o);
		
			if (!Objects.equals(enrichment, _that.getEnrichment())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (enrichment != null ? enrichment.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ESMAEMIRNonReportable {" +
				"enrichment=" + this.enrichment +
			'}';
		}
	}

	/*********************** Builder Implementation of ESMAEMIRNonReportable  ***********************/
	class ESMAEMIRNonReportableBuilderImpl implements ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder {
	
		protected EnrichmentData.EnrichmentDataBuilder enrichment;
	
		public ESMAEMIRNonReportableBuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("enrichment")
		public EnrichmentData.EnrichmentDataBuilder getEnrichment() {
			return enrichment;
		}
		
		@Override
		public EnrichmentData.EnrichmentDataBuilder getOrCreateEnrichment() {
			EnrichmentData.EnrichmentDataBuilder result;
			if (enrichment!=null) {
				result = enrichment;
			}
			else {
				result = enrichment = EnrichmentData.builder();
			}
			
			return result;
		}
	
		@Override
		@RosettaAttribute("enrichment")
		public ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder setEnrichment(EnrichmentData enrichment) {
			this.enrichment = enrichment==null?null:enrichment.toBuilder();
			return this;
		}
		
		@Override
		public ESMAEMIRNonReportable build() {
			return new ESMAEMIRNonReportable.ESMAEMIRNonReportableImpl(this);
		}
		
		@Override
		public ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder prune() {
			if (enrichment!=null && !enrichment.prune().hasData()) enrichment = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEnrichment()!=null && getEnrichment().hasData()) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder o = (ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder) other;
			
			merger.mergeRosetta(getEnrichment(), o.getEnrichment(), this::setEnrichment);
			
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			ESMAEMIRNonReportable _that = getType().cast(o);
		
			if (!Objects.equals(enrichment, _that.getEnrichment())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (enrichment != null ? enrichment.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ESMAEMIRNonReportableBuilder {" +
				"enrichment=" + this.enrichment +
			'}';
		}
	}
}
