package drr.regulation.esma.emir.refit.trade.meta;

import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyFunctionFactory;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorFactory;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRTransactionReport;
import drr.regulation.esma.emir.refit.trade.validation.ESMAEMIRTransactionReportTypeFormatValidator;
import drr.regulation.esma.emir.refit.trade.validation.ESMAEMIRTransactionReportValidator;
import drr.regulation.esma.emir.refit.trade.validation.exists.ESMAEMIRTransactionReportOnlyExistsValidator;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Function;


/**
 * @version 4.2.1
 */
@RosettaMeta(model=ESMAEMIRTransactionReport.class)
public class ESMAEMIRTransactionReportMeta implements RosettaMetaData<ESMAEMIRTransactionReport> {

	@Override
	public List<Validator<? super ESMAEMIRTransactionReport>> dataRules(ValidatorFactory factory) {
		return Arrays.asList(
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100102.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100103.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100104.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100105.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100202.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100203.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100204.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100302.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100303.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100304.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100305.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100402.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100403.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100404.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100405.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100602.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100603.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100702.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100802.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100902.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100903.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100904.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100905.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100906.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100907.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR100908.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101002.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101102.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101202.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101203.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101302.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101402.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101502.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101602.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101603.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101604.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101605.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101606.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101607.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101702.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101802.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101803.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101902.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR101903.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR102001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR102002.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200102.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200103.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200104.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200202.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200203.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200204.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200302.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200303.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200304.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200305.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200402.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200403.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200404.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200502.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200503.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200504.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200702.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200802.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200803.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200804.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR200902.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201302.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201303.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201304.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201305.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201402.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201403.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201502.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201702.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201703.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201902.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201904.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR201905.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202002.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202003.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202202.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202203.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202302.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202303.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202304.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202305.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202306.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202307.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202402.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202403.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202502.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202503.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202802.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202803.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202804.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202805.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR202901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203002.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203003.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203202.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203203.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203204.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203205.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203302.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203303.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203304.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203602.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203802.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR203901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204002.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204003.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204004.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204102.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204202.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204203.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204302.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204402.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204403.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204404.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204405.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204502.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204503.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204504.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204505.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204506.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204507.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204508.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204602.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204603.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204604.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204802.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204803.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204902.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR204903.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205102.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205202.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205302.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205303.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205304.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205402.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205403.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205602.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR205901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR206001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR206101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR206201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR206202.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR206301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR206401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR206501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR206502.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR206601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR206801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR206901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207102.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207502.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207702.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207703.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207704.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207705.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207802.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207803.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207804.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207805.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR207902.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR208001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR208101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR208201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR208301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR208302.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR208401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR208402.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR208501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR208502.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR208601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR208701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR208801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR208901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR209001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR209101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR209201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR209301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR209302.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR209303.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR209402.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR209501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR209601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR209701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR209801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR209901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR209902.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR210001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR210002.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR210101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR210102.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR210201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR210301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR210401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR210501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR210601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR210701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR210801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR210901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR210902.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR210903.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211102.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211103.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211104.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211202.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211302.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211303.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211402.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211403.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211502.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211702.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211703.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211704.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211705.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211706.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211707.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211708.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211709.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211710.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211802.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211803.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211804.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211805.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211806.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211807.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211808.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211809.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211810.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211811.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211812.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211813.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR211901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR212001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR212101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR212201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR212301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR212401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR212501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR212601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR212701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR212801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR212901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213402.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213403.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213404.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213602.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213702.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213802.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213803.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR213901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214002.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214202.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214402.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214403.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214404.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214501.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214601.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214701.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214801.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214901.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR214902.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215001.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215002.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215101.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215102.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215103.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215104.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215105.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215106.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215201.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215202.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215203.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215204.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215205.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215301.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215302.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215303.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215304.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215305.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215306.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215307.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215308.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215401.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215402.class),
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.ESMAEMIRTransactionReportEMIRVR215403.class)
		);
	}
	
	@Override
	public List<Function<? super ESMAEMIRTransactionReport, QualifyResult>> getQualifyFunctions(QualifyFunctionFactory factory) {
		return Collections.emptyList();
	}

	@Override
	public Validator<? super ESMAEMIRTransactionReport> validator() {
		return new ESMAEMIRTransactionReportValidator();
	}

	@Override
	public Validator<? super ESMAEMIRTransactionReport> typeFormatValidator() {
		return new ESMAEMIRTransactionReportTypeFormatValidator();
	}
	
	@Override
	public ValidatorWithArg<? super ESMAEMIRTransactionReport, Set<String>> onlyExistsValidator() {
		return new ESMAEMIRTransactionReportOnlyExistsValidator();
	}
}
