package drr.regulation.esma.emir.refit.trade.meta;

import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyFunctionFactory;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorFactory;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import drr.regulation.esma.emir.refit.trade.PackageTransactionPrice;
import drr.regulation.esma.emir.refit.trade.validation.PackageTransactionPriceTypeFormatValidator;
import drr.regulation.esma.emir.refit.trade.validation.PackageTransactionPriceValidator;
import drr.regulation.esma.emir.refit.trade.validation.exists.PackageTransactionPriceOnlyExistsValidator;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Function;


/**
 * @version 4.2.1
 */
@RosettaMeta(model=PackageTransactionPrice.class)
public class PackageTransactionPriceMeta implements RosettaMetaData<PackageTransactionPrice> {

	@Override
	public List<Validator<? super PackageTransactionPrice>> dataRules(ValidatorFactory factory) {
		return Arrays.asList(
			factory.create(drr.regulation.esma.emir.refit.trade.validation.datarule.PackageTransactionPriceMonetaryOrPercentageOrBasis.class)
		);
	}
	
	@Override
	public List<Function<? super PackageTransactionPrice, QualifyResult>> getQualifyFunctions(QualifyFunctionFactory factory) {
		return Collections.emptyList();
	}

	@Override
	public Validator<? super PackageTransactionPrice> validator() {
		return new PackageTransactionPriceValidator();
	}

	@Override
	public Validator<? super PackageTransactionPrice> typeFormatValidator() {
		return new PackageTransactionPriceTypeFormatValidator();
	}
	
	@Override
	public ValidatorWithArg<? super PackageTransactionPrice, Set<String>> onlyExistsValidator() {
		return new PackageTransactionPriceOnlyExistsValidator();
	}
}
