package drr.regulation.esma.emir.refit.trade.reports;

import cdm.base.staticdata.asset.common.ISOCountryCodeEnum;
import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.model.lib.reports.Tabulator.MultiNestedFieldValueImpl;
import com.rosetta.model.lib.reports.Tabulator.NestedFieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRTransactionReport;
import iso20022.auth030.esma.EnergyLoadType1Code;
import iso20022.auth030.esma.Frequency13Code__1;
import iso20022.auth030.esma.InterestComputationMethod4Code;
import iso20022.auth030.esma.ValuationType1Code;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.inject.Inject;


public class ESMAEMIRTransactionReportTypeTabulator implements Tabulator<ESMAEMIRTransactionReport> {
	private final Field reportingTimestampField;
	private final Field reportSubmittingEntityIDField;
	private final Field entityResponsibleForReportingField;
	private final Field counterparty1Field;
	private final Field natureOfCounterparty1Field;
	private final Field corporateSectorOfTheCounterparty1Field;
	private final Field clearingThresholdOfCounterparty1Field;
	private final Field counterparty2Field;
	private final Field countryOfTheCounterParty2Field;
	private final Field natureOfCounterparty2Field;
	private final Field corporateSectorOfTheCounterparty2Field;
	private final Field counterparty2IdentifierTypeField;
	private final Field clearingMemberField;
	private final Field clearingThresholdOfCounterparty2Field;
	private final Field reportingObligationOfTheCounterparty2Field;
	private final Field brokerIDField;
	private final Field directionField;
	private final Field directionOfLeg1Field;
	private final Field directionOfLeg2Field;
	private final Field directlyLinkedToCommercialActivityOrTreasuryFinancingField;
	private final Field utiField;
	private final Field utiProprietaryField;
	private final Field reportTrackingNumberField;
	private final Field priorUtiField;
	private final Field priorUtiProprietaryField;
	private final Field ptrrIdField;
	private final Field packageIdentifierField;
	private final Field subsequentPositionUtiField;
	private final Field isinField;
	private final Field upiField;
	private final Field productClassificationField;
	private final Field contractTypeField;
	private final Field assetClassField;
	private final Field isCryptoField;
	private final Field underlyingIdentificationTypeField;
	private final Field underlyingIdentificationField;
	private final Field nameOfTheUnderlyingIndexField;
	private final Field indicatorOfTheUnderlyingIndexField;
	private final Field basketConstituentsField;
	private final Field customBasketCodeField;
	private final Field settlementCurrency1Field;
	private final Field settlementCurrency2Field;
	private final Field valuationAmountField;
	private final Field valuationCurrencyField;
	private final Field valuationMethodField;
	private final Field deltaField;
	private final Field valuationTimestampField;
	private final Field confirmationTimestampField;
	private final Field confirmedField;
	private final Field clearingObligationField;
	private final Field clearedField;
	private final Field clearingTimestampField;
	private final Field centralCounterpartyField;
	private final Field masterAgreementTypeField;
	private final Field otherMasterAgreementTypeField;
	private final Field masterAgreementVersionField;
	private final Field intragroupField;
	private final Field ptrrField;
	private final Field typeOfPTRRTechniqueField;
	private final Field ptrrServiceProviderField;
	private final Field venueOfExecutionField;
	private final Field executionTimestampField;
	private final Field effectiveDateField;
	private final Field expirationDateField;
	private final Field earlyTerminationDateField;
	private final Field finalContractualSettlementDateField;
	private final Field deliveryTypeField;
	private final Field priceField;
	private final Field priceScheduleField;
	private final Field packageTransactionPriceField;
	private final Field notionalLeg1Field;
	private final Field notionalCurrency1Field;
	private final Field notionalAmountScheduleLeg1Field;
	private final Field totalNotionalQuantityLeg1Field;
	private final Field notionalQuantityScheduleLeg1Field;
	private final Field totalNotionalQuantityLeg2Field;
	private final Field notionalLeg2Field;
	private final Field notionalCurrency2Field;
	private final Field notionalAmountScheduleLeg2Field;
	private final Field notionalQuantityScheduleLeg2Field;
	private final Field otherPaymentField;
	private final Field fixedRateOfLeg1Field;
	private final Field fixedRateDayCountConventionLeg1Field;
	private final Field fixedRatePaymentFrequencyPeriodLeg1Field;
	private final Field fixedRatePaymentFrequencyPeriodMultiplierLeg1Field;
	private final Field identifierOfFloatingRateOfLeg1Field;
	private final Field indicatorOfTheFloatingRateOfLeg1Field;
	private final Field nameOfTheFloatingRateOfLeg1Field;
	private final Field floatingRateDayCountConventionOfLeg1Field;
	private final Field floatingRatePaymentFrequencyPeriodOfLeg1Field;
	private final Field floatingRatePaymentFrequencyPeriodMultiplierOfLeg1Field;
	private final Field floatingRateReferencePeriodOfLeg1TimePeriodField;
	private final Field floatingRateReferencePeriodOfLeg1MultiplierField;
	private final Field floatingRateResetFrequencyPeriodOfLeg1Field;
	private final Field floatingRateResetFrequencyPeriodMultiplierLeg1Field;
	private final Field spreadOfLeg1Field;
	private final Field fixedRateOfLeg2Field;
	private final Field fixedRateDayCountConventionLeg2Field;
	private final Field fixedRatePaymentFrequencyPeriodLeg2Field;
	private final Field fixedRatePaymentFrequencyPeriodMultiplierLeg2Field;
	private final Field identifierOfFloatingRateOfLeg2Field;
	private final Field indicatorOfTheFloatingRateOfLeg2Field;
	private final Field nameOfTheFloatingRateOfLeg2Field;
	private final Field floatingRateDayCountConventionOfLeg2Field;
	private final Field floatingRatePaymentFrequencyPeriodOfLeg2Field;
	private final Field floatingRatePaymentFrequencyPeriodMultiplierOfLeg2Field;
	private final Field floatingRateReferencePeriodOfLeg2TimePeriodField;
	private final Field floatingRateReferencePeriodOfLeg2MultiplierField;
	private final Field floatingRateResetFrequencyPeriodOfLeg2Field;
	private final Field floatingRateResetFrequencyPeriodMultiplierLeg2Field;
	private final Field spreadOfLeg2Field;
	private final Field packageTransactionSpreadField;
	private final Field exchangeRateField;
	private final Field forwardExchangeRateField;
	private final Field exchangeRateBasisField;
	private final Field baseProductField;
	private final Field subProductField;
	private final Field furtherSubProductField;
	private final Field deliveryPointField;
	private final Field interconnectionPointField;
	private final Field loadTypeField;
	private final Field deliveryReportField;
	private final Field optionTypeField;
	private final Field optionStyleField;
	private final Field strikePriceField;
	private final Field strikePriceScheduleField;
	private final Field optionPremiumAmountField;
	private final Field optionPremiumCurrencyField;
	private final Field optionPremiumPaymentDateField;
	private final Field maturityDateOfTheUnderlyingField;
	private final Field seniorityField;
	private final Field referenceEntityField;
	private final Field seriesField;
	private final Field versionField;
	private final Field indexFactorField;
	private final Field trancheField;
	private final Field cdSIndexAttachmentPointField;
	private final Field cdSIndexDetachmentPointField;
	private final Field collateralPortfolioIndicatorField;
	private final Field collateralPortfolioCodeField;
	private final Field actionTypeField;
	private final Field eventTypeField;
	private final Field eventDateField;
	private final Field levelField;
	
	private final BasketConstituentsReportTypeTabulator basketConstituentsReportTypeTabulator;
	private final PriceAndNotationTypeTabulator priceAndNotationTypeTabulator;
	private final PriceScheduleReportTypeTabulator priceScheduleReportTypeTabulator;
	private final PackageTransactionPriceTypeTabulator packageTransactionPriceTypeTabulator;
	private final NotionalAmountScheduleLeg1ReportTypeTabulator notionalAmountScheduleLeg1ReportTypeTabulator;
	private final NotionalQuantityScheduleLeg1ReportTypeTabulator notionalQuantityScheduleLeg1ReportTypeTabulator;
	private final NotionalAmountScheduleLeg2ReportTypeTabulator notionalAmountScheduleLeg2ReportTypeTabulator;
	private final NotionalQuantityScheduleLeg2ReportTypeTabulator notionalQuantityScheduleLeg2ReportTypeTabulator;
	private final OtherPaymentReportTypeTabulator otherPaymentReportTypeTabulator;
	private final SpreadOfLeg1TypeTabulator spreadOfLeg1TypeTabulator;
	private final SpreadOfLeg2TypeTabulator spreadOfLeg2TypeTabulator;
	private final PackageTransactionSpreadTypeTabulator packageTransactionSpreadTypeTabulator;
	private final DeliveryReportTypeTabulator deliveryReportTypeTabulator;
	private final StrikePriceAndNotationTypeTabulator strikePriceAndNotationTypeTabulator;
	private final StrikePriceScheduleReportTypeTabulator strikePriceScheduleReportTypeTabulator;
	
	@Inject
	public ESMAEMIRTransactionReportTypeTabulator(BasketConstituentsReportTypeTabulator basketConstituentsReportTypeTabulator, PriceAndNotationTypeTabulator priceAndNotationTypeTabulator, PriceScheduleReportTypeTabulator priceScheduleReportTypeTabulator, PackageTransactionPriceTypeTabulator packageTransactionPriceTypeTabulator, NotionalAmountScheduleLeg1ReportTypeTabulator notionalAmountScheduleLeg1ReportTypeTabulator, NotionalQuantityScheduleLeg1ReportTypeTabulator notionalQuantityScheduleLeg1ReportTypeTabulator, NotionalAmountScheduleLeg2ReportTypeTabulator notionalAmountScheduleLeg2ReportTypeTabulator, NotionalQuantityScheduleLeg2ReportTypeTabulator notionalQuantityScheduleLeg2ReportTypeTabulator, OtherPaymentReportTypeTabulator otherPaymentReportTypeTabulator, SpreadOfLeg1TypeTabulator spreadOfLeg1TypeTabulator, SpreadOfLeg2TypeTabulator spreadOfLeg2TypeTabulator, PackageTransactionSpreadTypeTabulator packageTransactionSpreadTypeTabulator, DeliveryReportTypeTabulator deliveryReportTypeTabulator, StrikePriceAndNotationTypeTabulator strikePriceAndNotationTypeTabulator, StrikePriceScheduleReportTypeTabulator strikePriceScheduleReportTypeTabulator) {
		this.basketConstituentsReportTypeTabulator = basketConstituentsReportTypeTabulator;
		this.priceAndNotationTypeTabulator = priceAndNotationTypeTabulator;
		this.priceScheduleReportTypeTabulator = priceScheduleReportTypeTabulator;
		this.packageTransactionPriceTypeTabulator = packageTransactionPriceTypeTabulator;
		this.notionalAmountScheduleLeg1ReportTypeTabulator = notionalAmountScheduleLeg1ReportTypeTabulator;
		this.notionalQuantityScheduleLeg1ReportTypeTabulator = notionalQuantityScheduleLeg1ReportTypeTabulator;
		this.notionalAmountScheduleLeg2ReportTypeTabulator = notionalAmountScheduleLeg2ReportTypeTabulator;
		this.notionalQuantityScheduleLeg2ReportTypeTabulator = notionalQuantityScheduleLeg2ReportTypeTabulator;
		this.otherPaymentReportTypeTabulator = otherPaymentReportTypeTabulator;
		this.spreadOfLeg1TypeTabulator = spreadOfLeg1TypeTabulator;
		this.spreadOfLeg2TypeTabulator = spreadOfLeg2TypeTabulator;
		this.packageTransactionSpreadTypeTabulator = packageTransactionSpreadTypeTabulator;
		this.deliveryReportTypeTabulator = deliveryReportTypeTabulator;
		this.strikePriceAndNotationTypeTabulator = strikePriceAndNotationTypeTabulator;
		this.strikePriceScheduleReportTypeTabulator = strikePriceScheduleReportTypeTabulator;
		this.reportingTimestampField = new FieldImpl(
			"reportingTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ReportingTimestamp")),
			Optional.of("1.1 Reporting Timestamp"),
			Arrays.asList()
		);
		this.reportSubmittingEntityIDField = new FieldImpl(
			"reportSubmittingEntityID",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ReportSubmittingEntityID")),
			Optional.of("1.2 Report Submitting entity ID"),
			Arrays.asList()
		);
		this.entityResponsibleForReportingField = new FieldImpl(
			"entityResponsibleForReporting",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "EntityResponsibleForReporting")),
			Optional.of("1.3 Entity responsible for reporting"),
			Arrays.asList()
		);
		this.counterparty1Field = new FieldImpl(
			"counterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Counterparty1")),
			Optional.of("1.4 Counterparty 1 (reporting counterparty)"),
			Arrays.asList()
		);
		this.natureOfCounterparty1Field = new FieldImpl(
			"natureOfCounterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "NatureOfCounterparty1")),
			Optional.of("1.5 Nature of the Counterparty 1"),
			Arrays.asList()
		);
		this.corporateSectorOfTheCounterparty1Field = new FieldImpl(
			"corporateSectorOfTheCounterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "CorporateSectorOfTheCounterparty1")),
			Optional.of("1.6 Corporate sector of the counterparty 1"),
			Arrays.asList()
		);
		this.clearingThresholdOfCounterparty1Field = new FieldImpl(
			"clearingThresholdOfCounterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ClearingThresholdOfCounterparty1")),
			Optional.of("1.7 Clearing Threshold of Counterparty 1"),
			Arrays.asList()
		);
		this.counterparty2Field = new FieldImpl(
			"counterparty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Counterparty2")),
			Optional.of("1.9 Counterparty 2"),
			Arrays.asList()
		);
		this.countryOfTheCounterParty2Field = new FieldImpl(
			"countryOfTheCounterParty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "CountryOfTheCounterParty2")),
			Optional.of("1.10 Country of the counterparty 2"),
			Arrays.asList()
		);
		this.natureOfCounterparty2Field = new FieldImpl(
			"natureOfCounterparty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "NatureOfTheCounterparty2")),
			Optional.of("1.11 Nature of the Counterparty 2"),
			Arrays.asList()
		);
		this.corporateSectorOfTheCounterparty2Field = new FieldImpl(
			"corporateSectorOfTheCounterparty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "CorporateSectorOfTheCounterparty2")),
			Optional.of("1.12 Corporate sector of the counterparty 2"),
			Arrays.asList()
		);
		this.counterparty2IdentifierTypeField = new FieldImpl(
			"counterparty2IdentifierType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Counterparty2IdentifierType")),
			Optional.of("1.8 Counterparty 2 Identifier Type"),
			Arrays.asList()
		);
		this.clearingMemberField = new FieldImpl(
			"clearingMember",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ClearingMember")),
			Optional.of("1.16 Clearing member"),
			Arrays.asList()
		);
		this.clearingThresholdOfCounterparty2Field = new FieldImpl(
			"clearingThresholdOfCounterparty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ClearingThresholdOfCounterparty2")),
			Optional.of("1.13 Clearing Threshold of Counterparty 2"),
			Arrays.asList()
		);
		this.reportingObligationOfTheCounterparty2Field = new FieldImpl(
			"reportingObligationOfTheCounterparty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ReportingObligationOfTheCounterparty2")),
			Optional.of("1.14 Reporting obligation of the counterparty 2"),
			Arrays.asList()
		);
		this.brokerIDField = new FieldImpl(
			"brokerID",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "BrokerID")),
			Optional.of("1.15 Broker ID"),
			Arrays.asList()
		);
		this.directionField = new FieldImpl(
			"direction",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Direction")),
			Optional.of("1.17 Direction"),
			Arrays.asList()
		);
		this.directionOfLeg1Field = new FieldImpl(
			"directionOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "DirectionOfLeg1")),
			Optional.of("1.18 Direction of leg 1"),
			Arrays.asList()
		);
		this.directionOfLeg2Field = new FieldImpl(
			"directionOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "DirectionOfLeg2")),
			Optional.of("1.19 Direction of leg 2"),
			Arrays.asList()
		);
		this.directlyLinkedToCommercialActivityOrTreasuryFinancingField = new FieldImpl(
			"directlyLinkedToCommercialActivityOrTreasuryFinancing",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "DirectlyLinkedToCommercialActivityOrTreasuryFinancing")),
			Optional.of("1.20 Directly linked to commercial activity or treasury financing"),
			Arrays.asList()
		);
		this.utiField = new FieldImpl(
			"uti",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Uti")),
			Optional.of("2.1 UTI"),
			Arrays.asList()
		);
		this.utiProprietaryField = new FieldImpl(
			"utiProprietary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "UtiProprietary")),
			Optional.of("2.1 UTI (Proprietary)"),
			Arrays.asList()
		);
		this.reportTrackingNumberField = new FieldImpl(
			"reportTrackingNumber",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ReportTrackingNumber")),
			Optional.of("2.2 Report Tracking Number"),
			Arrays.asList()
		);
		this.priorUtiField = new FieldImpl(
			"priorUti",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "PriorUti")),
			Optional.of("2.3 Prior UTI"),
			Arrays.asList()
		);
		this.priorUtiProprietaryField = new FieldImpl(
			"priorUtiProprietary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "PriorUtiProprietary")),
			Optional.of("2.3 Prior UTI (Proprietary)"),
			Arrays.asList()
		);
		this.ptrrIdField = new FieldImpl(
			"ptrrId",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "PTRRID")),
			Optional.of("2.5 PTRR ID"),
			Arrays.asList()
		);
		this.packageIdentifierField = new FieldImpl(
			"packageIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "PackageIdentifier")),
			Optional.of("2.6 Package identifier"),
			Arrays.asList()
		);
		this.subsequentPositionUtiField = new FieldImpl(
			"subsequentPositionUti",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "SubsequentPositionUti")),
			Optional.of("2.4 Subsequent Position UTI"),
			Arrays.asList()
		);
		this.isinField = new FieldImpl(
			"isin",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ISIN")),
			Optional.of("2.7 ISIN"),
			Arrays.asList()
		);
		this.upiField = new FieldImpl(
			"upi",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "UPI")),
			Optional.of("2.8 UPI"),
			Arrays.asList()
		);
		this.productClassificationField = new FieldImpl(
			"productClassification",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ProductClassification")),
			Optional.of("2.9 Product classification"),
			Arrays.asList()
		);
		this.contractTypeField = new FieldImpl(
			"contractType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ContractType")),
			Optional.of("2.10 Contract Type"),
			Arrays.asList()
		);
		this.assetClassField = new FieldImpl(
			"assetClass",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "AssetClass")),
			Optional.of("2.11 Asset Class"),
			Arrays.asList()
		);
		this.isCryptoField = new FieldImpl(
			"isCrypto",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "DerivativeBasedOnCryptoAssets")),
			Optional.of("2.12 Derivative based on crypto-assets"),
			Arrays.asList()
		);
		this.underlyingIdentificationTypeField = new FieldImpl(
			"underlyingIdentificationType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "UnderlyingIdentificationType")),
			Optional.of("2.13 Underlying Identification Type"),
			Arrays.asList()
		);
		this.underlyingIdentificationField = new FieldImpl(
			"underlyingIdentification",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "UnderlyingIdentification")),
			Optional.of("2.14 Underlying Identification"),
			Arrays.asList()
		);
		this.nameOfTheUnderlyingIndexField = new FieldImpl(
			"nameOfTheUnderlyingIndex",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "NameOfTheUnderlyingIndex")),
			Optional.of("2.16 Name of the underlying index"),
			Arrays.asList()
		);
		this.indicatorOfTheUnderlyingIndexField = new FieldImpl(
			"indicatorOfTheUnderlyingIndex",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "IndicatorOfTheUnderlyingIndex")),
			Optional.of("2.15 Indicator of the underlying index"),
			Arrays.asList()
		);
		this.basketConstituentsField = new FieldImpl(
			"basketConstituents",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "BasketConstituents")),
			Optional.empty(),
			basketConstituentsReportTypeTabulator.getFields()
		);
		this.customBasketCodeField = new FieldImpl(
			"customBasketCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "CustomBasketCode")),
			Optional.of("2.17 Custom basket code"),
			Arrays.asList()
		);
		this.settlementCurrency1Field = new FieldImpl(
			"settlementCurrency1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "SettlementCurrency1")),
			Optional.of("2.19 Settlement Currency 1"),
			Arrays.asList()
		);
		this.settlementCurrency2Field = new FieldImpl(
			"settlementCurrency2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "SettlementCurrency2")),
			Optional.of("2.20 Settlement Currency 2"),
			Arrays.asList()
		);
		this.valuationAmountField = new FieldImpl(
			"valuationAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ValuationAmount")),
			Optional.of("2.21 Valuation amount"),
			Arrays.asList()
		);
		this.valuationCurrencyField = new FieldImpl(
			"valuationCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ValuationCurrency")),
			Optional.of("2.22 Valuation currency"),
			Arrays.asList()
		);
		this.valuationMethodField = new FieldImpl(
			"valuationMethod",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ValuationMethod")),
			Optional.of("2.24 Valuation method"),
			Arrays.asList()
		);
		this.deltaField = new FieldImpl(
			"delta",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Delta")),
			Optional.of("2.25 Delta"),
			Arrays.asList()
		);
		this.valuationTimestampField = new FieldImpl(
			"valuationTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ValuationTimestamp")),
			Optional.of("2.23 Valuation Timestamp"),
			Arrays.asList()
		);
		this.confirmationTimestampField = new FieldImpl(
			"confirmationTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ConfirmationTimestamp")),
			Optional.of("2.28 Confirmation Timestamp"),
			Arrays.asList()
		);
		this.confirmedField = new FieldImpl(
			"confirmed",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Confirmed")),
			Optional.of("2.29 Confirmed"),
			Arrays.asList()
		);
		this.clearingObligationField = new FieldImpl(
			"clearingObligation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ClearingObligation")),
			Optional.of("2.30 Clearing Obligation"),
			Arrays.asList()
		);
		this.clearedField = new FieldImpl(
			"cleared",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Cleared")),
			Optional.of("2.31 Cleared"),
			Arrays.asList()
		);
		this.clearingTimestampField = new FieldImpl(
			"clearingTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ClearingTimestamp")),
			Optional.of("2.32 Clearing Timestamp"),
			Arrays.asList()
		);
		this.centralCounterpartyField = new FieldImpl(
			"centralCounterparty",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "CentralCounterparty")),
			Optional.of("2.33 Central Counterparty"),
			Arrays.asList()
		);
		this.masterAgreementTypeField = new FieldImpl(
			"masterAgreementType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "MasterAgreementType")),
			Optional.of("2.34 Master Agreement Type"),
			Arrays.asList()
		);
		this.otherMasterAgreementTypeField = new FieldImpl(
			"otherMasterAgreementType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "OtherMasterAgreementType")),
			Optional.of("2.35 Other Master Agreement Type"),
			Arrays.asList()
		);
		this.masterAgreementVersionField = new FieldImpl(
			"masterAgreementVersion",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "MasterAgreementVersion")),
			Optional.of("2.36 Master Agreement version"),
			Arrays.asList()
		);
		this.intragroupField = new FieldImpl(
			"intragroup",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Intragroup")),
			Optional.of("2.37 Intragroup"),
			Arrays.asList()
		);
		this.ptrrField = new FieldImpl(
			"ptrr",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "PTRR")),
			Optional.of("2.38 PTRR"),
			Arrays.asList()
		);
		this.typeOfPTRRTechniqueField = new FieldImpl(
			"typeOfPTRRTechnique",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "TypeOfPTRRTechnique")),
			Optional.of("2.39 Type of PTRR technique"),
			Arrays.asList()
		);
		this.ptrrServiceProviderField = new FieldImpl(
			"ptrrServiceProvider",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "PTRRServiceProvider")),
			Optional.of("2.40 PTRR service provider"),
			Arrays.asList()
		);
		this.venueOfExecutionField = new FieldImpl(
			"venueOfExecution",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "VenueOfExecution")),
			Optional.of("2.41 Venue of Execution"),
			Arrays.asList()
		);
		this.executionTimestampField = new FieldImpl(
			"executionTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ExecutionTimestamp")),
			Optional.of("2.42 Execution Timestamp"),
			Arrays.asList()
		);
		this.effectiveDateField = new FieldImpl(
			"effectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "EffectiveDate")),
			Optional.of("2.43 Effective Date"),
			Arrays.asList()
		);
		this.expirationDateField = new FieldImpl(
			"expirationDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ExpirationDate")),
			Optional.of("2.44 Expiration Date"),
			Arrays.asList()
		);
		this.earlyTerminationDateField = new FieldImpl(
			"earlyTerminationDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "EarlyTerminationDate")),
			Optional.of("2.45 Early termination date"),
			Arrays.asList()
		);
		this.finalContractualSettlementDateField = new FieldImpl(
			"finalContractualSettlementDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FinalContractualSettlementDate")),
			Optional.of("2.46 Final Contractual Settlement Date"),
			Arrays.asList()
		);
		this.deliveryTypeField = new FieldImpl(
			"deliveryType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "DeliveryType")),
			Optional.of("2.47 Delivery Type"),
			Arrays.asList()
		);
		this.priceField = new FieldImpl(
			"price",
			false,
			Optional.empty(),
			Optional.empty(),
			priceAndNotationTypeTabulator.getFields()
		);
		this.priceScheduleField = new FieldImpl(
			"priceSchedule",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ContractPriceSchedule")),
			Optional.empty(),
			priceScheduleReportTypeTabulator.getFields()
		);
		this.packageTransactionPriceField = new FieldImpl(
			"packageTransactionPrice",
			false,
			Optional.empty(),
			Optional.empty(),
			packageTransactionPriceTypeTabulator.getFields()
		);
		this.notionalLeg1Field = new FieldImpl(
			"notionalLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "NotionalAmountOfLeg1")),
			Optional.of("2.55 Notional Amount 1"),
			Arrays.asList()
		);
		this.notionalCurrency1Field = new FieldImpl(
			"notionalCurrency1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "NotionalCurrency1")),
			Optional.of("2.56 Notional Currency 1"),
			Arrays.asList()
		);
		this.notionalAmountScheduleLeg1Field = new FieldImpl(
			"notionalAmountScheduleLeg1",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "NotionalAmountScheduleLeg1")),
			Optional.empty(),
			notionalAmountScheduleLeg1ReportTypeTabulator.getFields()
		);
		this.totalNotionalQuantityLeg1Field = new FieldImpl(
			"totalNotionalQuantityLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "TotalNotionalQuantityOfLeg1")),
			Optional.of("2.60 Total notional quantity-Leg 1"),
			Arrays.asList()
		);
		this.notionalQuantityScheduleLeg1Field = new FieldImpl(
			"notionalQuantityScheduleLeg1",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "NotionalQuantityScheduleLeg1")),
			Optional.empty(),
			notionalQuantityScheduleLeg1ReportTypeTabulator.getFields()
		);
		this.totalNotionalQuantityLeg2Field = new FieldImpl(
			"totalNotionalQuantityLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "TotalNotionalQuantityOfLeg2")),
			Optional.of("2.69 Total notional quantity-Leg 2"),
			Arrays.asList()
		);
		this.notionalLeg2Field = new FieldImpl(
			"notionalLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "NotionalAmountOfLeg2")),
			Optional.of("2.64 Notional Amount 2"),
			Arrays.asList()
		);
		this.notionalCurrency2Field = new FieldImpl(
			"notionalCurrency2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "NotionalCurrency2")),
			Optional.of("2.65 Notional Currency 2"),
			Arrays.asList()
		);
		this.notionalAmountScheduleLeg2Field = new FieldImpl(
			"notionalAmountScheduleLeg2",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "NotionalAmountScheduleLeg2")),
			Optional.empty(),
			notionalAmountScheduleLeg2ReportTypeTabulator.getFields()
		);
		this.notionalQuantityScheduleLeg2Field = new FieldImpl(
			"notionalQuantityScheduleLeg2",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "NotionalQuantityScheduleLeg2")),
			Optional.empty(),
			notionalQuantityScheduleLeg2ReportTypeTabulator.getFields()
		);
		this.otherPaymentField = new FieldImpl(
			"otherPayment",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "OtherPayment")),
			Optional.empty(),
			otherPaymentReportTypeTabulator.getFields()
		);
		this.fixedRateOfLeg1Field = new FieldImpl(
			"fixedRateOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FixedRateOfLeg1OrCoupon")),
			Optional.of("2.79 Fixed Rate of leg 1 or coupon"),
			Arrays.asList()
		);
		this.fixedRateDayCountConventionLeg1Field = new FieldImpl(
			"fixedRateDayCountConventionLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FixedRateOrCouponDayCountConventionLeg1")),
			Optional.of("2.80 Fixed rate or coupon day count convention leg 1"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodLeg1Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FixedRateOrCouponPaymentFrequencyPeriodLeg1")),
			Optional.of("2.81 Fixed rate or coupon payment frequency period leg 1"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodMultiplierLeg1Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodMultiplierLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FixedRateOrCouponPaymentFrequencyPeriodMultiplierLeg1")),
			Optional.of("2.82 Fixed rate or coupon payment frequency period multiplier leg 1"),
			Arrays.asList()
		);
		this.identifierOfFloatingRateOfLeg1Field = new FieldImpl(
			"identifierOfFloatingRateOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "IdentifierOfFloatingRateOfLeg1")),
			Optional.of("2.83 - Identifier of the floating rate of leg 1"),
			Arrays.asList()
		);
		this.indicatorOfTheFloatingRateOfLeg1Field = new FieldImpl(
			"indicatorOfTheFloatingRateOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "IndicatorOfTheFloatingRateOfLeg1")),
			Optional.of("2.84 Indicator of the floating rate of leg 1"),
			Arrays.asList()
		);
		this.nameOfTheFloatingRateOfLeg1Field = new FieldImpl(
			"nameOfTheFloatingRateOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "NameOfTheFloatingRateOfLeg1")),
			Optional.of("2.85 Name of the Floating Rate of Leg 1"),
			Arrays.asList()
		);
		this.floatingRateDayCountConventionOfLeg1Field = new FieldImpl(
			"floatingRateDayCountConventionOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FloatingRateDayCountConventionOfLeg1")),
			Optional.of("2.86 Floating rate day count convention of leg 1"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodOfLeg1Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FloatingRatePaymentFrequencyPeriodOfLeg1")),
			Optional.of("2.87 Floating rate payment frequency period of leg 1"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg1Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodMultiplierOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FloatingRatePaymentFrequencyPeriodMultiplierOfLeg1")),
			Optional.of("2.88 Floating rate payment frequency period multiplier of leg 1"),
			Arrays.asList()
		);
		this.floatingRateReferencePeriodOfLeg1TimePeriodField = new FieldImpl(
			"floatingRateReferencePeriodOfLeg1TimePeriod",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FloatingRateReferencePeriodOfLeg1TimePeriod")),
			Optional.of("2.89 Floating rate reference period of leg 1 - time period"),
			Arrays.asList()
		);
		this.floatingRateReferencePeriodOfLeg1MultiplierField = new FieldImpl(
			"floatingRateReferencePeriodOfLeg1Multiplier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FloatingRateReferencePeriodOfLeg1Multiplier")),
			Optional.of("2.90 Floating rate reference period of leg 1 - multiplier"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodOfLeg1Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FloatingRateResetFrequencyPeriodOfLeg1")),
			Optional.of("2.91 Floating rate reset frequency period of Leg 1"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodMultiplierLeg1Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodMultiplierLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FloatingRateResetFrequencyMultiplierOfLeg1")),
			Optional.of("2.92 Floating rate reset frequency multiplier of leg 1"),
			Arrays.asList()
		);
		this.spreadOfLeg1Field = new FieldImpl(
			"spreadOfLeg1",
			false,
			Optional.empty(),
			Optional.empty(),
			spreadOfLeg1TypeTabulator.getFields()
		);
		this.fixedRateOfLeg2Field = new FieldImpl(
			"fixedRateOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FixedRateOfLeg2")),
			Optional.of("2.95 Fixed Rate of leg 2"),
			Arrays.asList()
		);
		this.fixedRateDayCountConventionLeg2Field = new FieldImpl(
			"fixedRateDayCountConventionLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FixedRateDayCountConventionLeg2")),
			Optional.of("2.96 Fixed rate day count convention leg 2"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodLeg2Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FixedRatePaymentFrequencyPeriodLeg2")),
			Optional.of("2.97 Fixed rate payment frequency period leg 2"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodMultiplierLeg2Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodMultiplierLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FixedRatePaymentFrequencyPeriodMultiplierLeg2")),
			Optional.of("2.98 Fixed rate payment frequency period multiplier of leg 2"),
			Arrays.asList()
		);
		this.identifierOfFloatingRateOfLeg2Field = new FieldImpl(
			"identifierOfFloatingRateOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "IdentifierOfFloatingRateOfLeg2")),
			Optional.of("2.99 - Identifier of the floating rate of leg 2"),
			Arrays.asList()
		);
		this.indicatorOfTheFloatingRateOfLeg2Field = new FieldImpl(
			"indicatorOfTheFloatingRateOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "IndicatorOfTheFloatingRateOfLeg2")),
			Optional.of("2.100 Indicator of the floating rate of leg 2"),
			Arrays.asList()
		);
		this.nameOfTheFloatingRateOfLeg2Field = new FieldImpl(
			"nameOfTheFloatingRateOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "NameOfTheFloatingRateOfLeg2")),
			Optional.of("2.101 Name of the Floating Rate of Leg 2"),
			Arrays.asList()
		);
		this.floatingRateDayCountConventionOfLeg2Field = new FieldImpl(
			"floatingRateDayCountConventionOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FloatingRateDayCountConventionOfLeg2")),
			Optional.of("2.102 Floating rate day count convention of leg 2"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodOfLeg2Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FloatingRatePaymentFrequencyPeriodOfLeg2")),
			Optional.of("2.103 Floating rate payment frequency period of leg 2"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg2Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodMultiplierOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FloatingRatePaymentFrequencyPeriodMultiplierOfLeg2")),
			Optional.of("2.104 Floating rate payment frequency period multiplier of leg 2"),
			Arrays.asList()
		);
		this.floatingRateReferencePeriodOfLeg2TimePeriodField = new FieldImpl(
			"floatingRateReferencePeriodOfLeg2TimePeriod",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FloatingRateReferencePeriodOfLeg2TimePeriod")),
			Optional.of("2.105 Floating rate reference period of leg 2 - time period"),
			Arrays.asList()
		);
		this.floatingRateReferencePeriodOfLeg2MultiplierField = new FieldImpl(
			"floatingRateReferencePeriodOfLeg2Multiplier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FloatingRateReferencePeriodOfLeg2Multiplier")),
			Optional.of("2.106 Floating rate reference period of leg 2 - multiplier"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodOfLeg2Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FloatingRateResetFrequencyPeriodOfLeg2")),
			Optional.of("2.107 Floating rate reset frequency period of Leg 2"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodMultiplierLeg2Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodMultiplierLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FloatingRateResetFrequencyMultiplierOfLeg2")),
			Optional.of("2.108 Floating rate reset frequency multiplier of leg 2"),
			Arrays.asList()
		);
		this.spreadOfLeg2Field = new FieldImpl(
			"spreadOfLeg2",
			false,
			Optional.empty(),
			Optional.empty(),
			spreadOfLeg2TypeTabulator.getFields()
		);
		this.packageTransactionSpreadField = new FieldImpl(
			"packageTransactionSpread",
			false,
			Optional.empty(),
			Optional.empty(),
			packageTransactionSpreadTypeTabulator.getFields()
		);
		this.exchangeRateField = new FieldImpl(
			"exchangeRate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ExchangeRate")),
			Optional.of("2.113 Exchange Rate 1"),
			Arrays.asList()
		);
		this.forwardExchangeRateField = new FieldImpl(
			"forwardExchangeRate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ForwardExchangeRate")),
			Optional.of("2.114 Forward exchange rate"),
			Arrays.asList()
		);
		this.exchangeRateBasisField = new FieldImpl(
			"exchangeRateBasis",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ExchangeRateBasis")),
			Optional.of("2.115 Exchange Rate Basis"),
			Arrays.asList()
		);
		this.baseProductField = new FieldImpl(
			"baseProduct",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "BaseProduct")),
			Optional.of("2.116 Base Product"),
			Arrays.asList()
		);
		this.subProductField = new FieldImpl(
			"subProduct",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "SubProduct")),
			Optional.of("2.117 Sub Product"),
			Arrays.asList()
		);
		this.furtherSubProductField = new FieldImpl(
			"furtherSubProduct",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "FurtherSubProduct")),
			Optional.of("2.118 Further sub-product"),
			Arrays.asList()
		);
		this.deliveryPointField = new FieldImpl(
			"deliveryPoint",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "DeliveryPointOrZone")),
			Optional.of("2.119 Delivery point or zone"),
			Arrays.asList()
		);
		this.interconnectionPointField = new FieldImpl(
			"interconnectionPoint",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "InterconnectionPoint")),
			Optional.of("2.120 Interconnection Point"),
			Arrays.asList()
		);
		this.loadTypeField = new FieldImpl(
			"loadType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "LoadType")),
			Optional.of("2.121 Load type"),
			Arrays.asList()
		);
		this.deliveryReportField = new FieldImpl(
			"deliveryReport",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Delivery")),
			Optional.empty(),
			deliveryReportTypeTabulator.getFields()
		);
		this.optionTypeField = new FieldImpl(
			"optionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "OptionType")),
			Optional.of("2.132 Option type"),
			Arrays.asList()
		);
		this.optionStyleField = new FieldImpl(
			"optionStyle",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "OptionStyle")),
			Optional.of("2.133 Option Style"),
			Arrays.asList()
		);
		this.strikePriceField = new FieldImpl(
			"strikePrice",
			false,
			Optional.empty(),
			Optional.empty(),
			strikePriceAndNotationTypeTabulator.getFields()
		);
		this.strikePriceScheduleField = new FieldImpl(
			"strikePriceSchedule",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "StrikePriceSchedule")),
			Optional.empty(),
			strikePriceScheduleReportTypeTabulator.getFields()
		);
		this.optionPremiumAmountField = new FieldImpl(
			"optionPremiumAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "OptionPremiumAmount")),
			Optional.of("2.139 Option Premium Amount"),
			Arrays.asList()
		);
		this.optionPremiumCurrencyField = new FieldImpl(
			"optionPremiumCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "OptionPremiumCurrency")),
			Optional.of("2.140 Option Premium Currency"),
			Arrays.asList()
		);
		this.optionPremiumPaymentDateField = new FieldImpl(
			"optionPremiumPaymentDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "OptionPremiumPaymentDate")),
			Optional.of("2.141 Option Premium Payment Date"),
			Arrays.asList()
		);
		this.maturityDateOfTheUnderlyingField = new FieldImpl(
			"maturityDateOfTheUnderlying",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "MaturityDateOfTheUnderlying")),
			Optional.of("2.142 Maturity date of the underlying"),
			Arrays.asList()
		);
		this.seniorityField = new FieldImpl(
			"seniority",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Seniority")),
			Optional.of("2.143 Seniority"),
			Arrays.asList()
		);
		this.referenceEntityField = new FieldImpl(
			"referenceEntity",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ReferenceEntity")),
			Optional.of("2.144 Reference Entity"),
			Arrays.asList()
		);
		this.seriesField = new FieldImpl(
			"series",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Series")),
			Optional.of("2.145 Series"),
			Arrays.asList()
		);
		this.versionField = new FieldImpl(
			"version",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Version")),
			Optional.of("2.146 Version"),
			Arrays.asList()
		);
		this.indexFactorField = new FieldImpl(
			"indexFactor",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "IndexFactor")),
			Optional.of("2.147 Index factor"),
			Arrays.asList()
		);
		this.trancheField = new FieldImpl(
			"tranche",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Tranche")),
			Optional.of("2.148 Tranche"),
			Arrays.asList()
		);
		this.cdSIndexAttachmentPointField = new FieldImpl(
			"cdSIndexAttachmentPoint",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "CDSIndexAttachmentPoint")),
			Optional.of("2.149 CDS Index Attachment Point"),
			Arrays.asList()
		);
		this.cdSIndexDetachmentPointField = new FieldImpl(
			"cdSIndexDetachmentPoint",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "CDSIndexDetachmentPoint")),
			Optional.of("2.150 CDS Index Detachment Point"),
			Arrays.asList()
		);
		this.collateralPortfolioIndicatorField = new FieldImpl(
			"collateralPortfolioIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "CollateralPortfolioIndicator")),
			Optional.of("2.26 Collateral Portfolio Indicator"),
			Arrays.asList()
		);
		this.collateralPortfolioCodeField = new FieldImpl(
			"collateralPortfolioCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "CollateralPortfolioCode")),
			Optional.of("2.27 Collateral Portfolio Code"),
			Arrays.asList()
		);
		this.actionTypeField = new FieldImpl(
			"actionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "ActionType")),
			Optional.of("2.151 Action type"),
			Arrays.asList()
		);
		this.eventTypeField = new FieldImpl(
			"eventType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "EventType")),
			Optional.of("2.152 Event Type"),
			Arrays.asList()
		);
		this.eventDateField = new FieldImpl(
			"eventDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "EventDate")),
			Optional.of("2.153 Event Date"),
			Arrays.asList()
		);
		this.levelField = new FieldImpl(
			"level",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "Level")),
			Optional.of("2.154 Level"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(reportingTimestampField, reportSubmittingEntityIDField, entityResponsibleForReportingField, counterparty1Field, natureOfCounterparty1Field, corporateSectorOfTheCounterparty1Field, clearingThresholdOfCounterparty1Field, counterparty2Field, countryOfTheCounterParty2Field, natureOfCounterparty2Field, corporateSectorOfTheCounterparty2Field, counterparty2IdentifierTypeField, clearingMemberField, clearingThresholdOfCounterparty2Field, reportingObligationOfTheCounterparty2Field, brokerIDField, directionField, directionOfLeg1Field, directionOfLeg2Field, directlyLinkedToCommercialActivityOrTreasuryFinancingField, utiField, utiProprietaryField, reportTrackingNumberField, priorUtiField, priorUtiProprietaryField, ptrrIdField, packageIdentifierField, subsequentPositionUtiField, isinField, upiField, productClassificationField, contractTypeField, assetClassField, isCryptoField, underlyingIdentificationTypeField, underlyingIdentificationField, nameOfTheUnderlyingIndexField, indicatorOfTheUnderlyingIndexField, basketConstituentsField, customBasketCodeField, settlementCurrency1Field, settlementCurrency2Field, valuationAmountField, valuationCurrencyField, valuationMethodField, deltaField, valuationTimestampField, confirmationTimestampField, confirmedField, clearingObligationField, clearedField, clearingTimestampField, centralCounterpartyField, masterAgreementTypeField, otherMasterAgreementTypeField, masterAgreementVersionField, intragroupField, ptrrField, typeOfPTRRTechniqueField, ptrrServiceProviderField, venueOfExecutionField, executionTimestampField, effectiveDateField, expirationDateField, earlyTerminationDateField, finalContractualSettlementDateField, deliveryTypeField, priceField, priceScheduleField, packageTransactionPriceField, notionalLeg1Field, notionalCurrency1Field, notionalAmountScheduleLeg1Field, totalNotionalQuantityLeg1Field, notionalQuantityScheduleLeg1Field, totalNotionalQuantityLeg2Field, notionalLeg2Field, notionalCurrency2Field, notionalAmountScheduleLeg2Field, notionalQuantityScheduleLeg2Field, otherPaymentField, fixedRateOfLeg1Field, fixedRateDayCountConventionLeg1Field, fixedRatePaymentFrequencyPeriodLeg1Field, fixedRatePaymentFrequencyPeriodMultiplierLeg1Field, identifierOfFloatingRateOfLeg1Field, indicatorOfTheFloatingRateOfLeg1Field, nameOfTheFloatingRateOfLeg1Field, floatingRateDayCountConventionOfLeg1Field, floatingRatePaymentFrequencyPeriodOfLeg1Field, floatingRatePaymentFrequencyPeriodMultiplierOfLeg1Field, floatingRateReferencePeriodOfLeg1TimePeriodField, floatingRateReferencePeriodOfLeg1MultiplierField, floatingRateResetFrequencyPeriodOfLeg1Field, floatingRateResetFrequencyPeriodMultiplierLeg1Field, spreadOfLeg1Field, fixedRateOfLeg2Field, fixedRateDayCountConventionLeg2Field, fixedRatePaymentFrequencyPeriodLeg2Field, fixedRatePaymentFrequencyPeriodMultiplierLeg2Field, identifierOfFloatingRateOfLeg2Field, indicatorOfTheFloatingRateOfLeg2Field, nameOfTheFloatingRateOfLeg2Field, floatingRateDayCountConventionOfLeg2Field, floatingRatePaymentFrequencyPeriodOfLeg2Field, floatingRatePaymentFrequencyPeriodMultiplierOfLeg2Field, floatingRateReferencePeriodOfLeg2TimePeriodField, floatingRateReferencePeriodOfLeg2MultiplierField, floatingRateResetFrequencyPeriodOfLeg2Field, floatingRateResetFrequencyPeriodMultiplierLeg2Field, spreadOfLeg2Field, packageTransactionSpreadField, exchangeRateField, forwardExchangeRateField, exchangeRateBasisField, baseProductField, subProductField, furtherSubProductField, deliveryPointField, interconnectionPointField, loadTypeField, deliveryReportField, optionTypeField, optionStyleField, strikePriceField, strikePriceScheduleField, optionPremiumAmountField, optionPremiumCurrencyField, optionPremiumPaymentDateField, maturityDateOfTheUnderlyingField, seniorityField, referenceEntityField, seriesField, versionField, indexFactorField, trancheField, cdSIndexAttachmentPointField, cdSIndexDetachmentPointField, collateralPortfolioIndicatorField, collateralPortfolioCodeField, actionTypeField, eventTypeField, eventDateField, levelField);
	}
	
	@Override
	public List<FieldValue> tabulate(ESMAEMIRTransactionReport input) {
		Optional<ZonedDateTime> reportingTimestamp = Optional.ofNullable(input.getReportingTimestamp());
		Optional<String> reportSubmittingEntityID = Optional.ofNullable(input.getReportSubmittingEntityID());
		Optional<String> entityResponsibleForReporting = Optional.ofNullable(input.getEntityResponsibleForReporting());
		Optional<String> counterparty1 = Optional.ofNullable(input.getCounterparty1());
		Optional<String> natureOfCounterparty1 = Optional.ofNullable(input.getNatureOfCounterparty1());
		Optional<String> corporateSectorOfTheCounterparty1 = Optional.ofNullable(input.getCorporateSectorOfTheCounterparty1());
		Optional<Boolean> clearingThresholdOfCounterparty1 = Optional.ofNullable(input.getClearingThresholdOfCounterparty1());
		Optional<String> counterparty2 = Optional.ofNullable(input.getCounterparty2());
		Optional<ISOCountryCodeEnum> countryOfTheCounterParty2 = Optional.ofNullable(input.getCountryOfTheCounterParty2());
		Optional<String> natureOfCounterparty2 = Optional.ofNullable(input.getNatureOfCounterparty2());
		Optional<String> corporateSectorOfTheCounterparty2 = Optional.ofNullable(input.getCorporateSectorOfTheCounterparty2());
		Optional<Boolean> counterparty2IdentifierType = Optional.ofNullable(input.getCounterparty2IdentifierType());
		Optional<String> clearingMember = Optional.ofNullable(input.getClearingMember());
		Optional<Boolean> clearingThresholdOfCounterparty2 = Optional.ofNullable(input.getClearingThresholdOfCounterparty2());
		Optional<Boolean> reportingObligationOfTheCounterparty2 = Optional.ofNullable(input.getReportingObligationOfTheCounterparty2());
		Optional<String> brokerID = Optional.ofNullable(input.getBrokerID());
		Optional<String> direction = Optional.ofNullable(input.getDirection());
		Optional<String> directionOfLeg1 = Optional.ofNullable(input.getDirectionOfLeg1());
		Optional<String> directionOfLeg2 = Optional.ofNullable(input.getDirectionOfLeg2());
		Optional<Boolean> directlyLinkedToCommercialActivityOrTreasuryFinancing = Optional.ofNullable(input.getDirectlyLinkedToCommercialActivityOrTreasuryFinancing());
		Optional<String> uti = Optional.ofNullable(input.getUti());
		Optional<String> utiProprietary = Optional.ofNullable(input.getUtiProprietary());
		Optional<String> reportTrackingNumber = Optional.ofNullable(input.getReportTrackingNumber());
		Optional<String> priorUti = Optional.ofNullable(input.getPriorUti());
		Optional<String> priorUtiProprietary = Optional.ofNullable(input.getPriorUtiProprietary());
		Optional<String> ptrrId = Optional.ofNullable(input.getPtrrId());
		Optional<String> packageIdentifier = Optional.ofNullable(input.getPackageIdentifier());
		Optional<String> subsequentPositionUti = Optional.ofNullable(input.getSubsequentPositionUti());
		Optional<String> isin = Optional.ofNullable(input.getIsin());
		Optional<String> upi = Optional.ofNullable(input.getUpi());
		Optional<String> productClassification = Optional.ofNullable(input.getProductClassification());
		Optional<String> contractType = Optional.ofNullable(input.getContractType());
		Optional<String> assetClass = Optional.ofNullable(input.getAssetClass());
		Optional<Boolean> isCrypto = Optional.ofNullable(input.getIsCrypto());
		Optional<String> underlyingIdentificationType = Optional.ofNullable(input.getUnderlyingIdentificationType());
		Optional<String> underlyingIdentification = Optional.ofNullable(input.getUnderlyingIdentification());
		Optional<List<? extends String>> nameOfTheUnderlyingIndex = Optional.ofNullable(input.getNameOfTheUnderlyingIndex());
		Optional<List<? extends String>> indicatorOfTheUnderlyingIndex = Optional.ofNullable(input.getIndicatorOfTheUnderlyingIndex());
		Optional<List<List<FieldValue>>> basketConstituents = Optional.ofNullable(input.getBasketConstituents())
			.map(x -> x.stream()
				.map(_x -> basketConstituentsReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<String> customBasketCode = Optional.ofNullable(input.getCustomBasketCode());
		Optional<ISOCurrencyCodeEnum> settlementCurrency1 = Optional.ofNullable(input.getSettlementCurrency1());
		Optional<ISOCurrencyCodeEnum> settlementCurrency2 = Optional.ofNullable(input.getSettlementCurrency2());
		Optional<BigDecimal> valuationAmount = Optional.ofNullable(input.getValuationAmount());
		Optional<ISOCurrencyCodeEnum> valuationCurrency = Optional.ofNullable(input.getValuationCurrency());
		Optional<ValuationType1Code> valuationMethod = Optional.ofNullable(input.getValuationMethod());
		Optional<BigDecimal> delta = Optional.ofNullable(input.getDelta());
		Optional<ZonedDateTime> valuationTimestamp = Optional.ofNullable(input.getValuationTimestamp());
		Optional<ZonedDateTime> confirmationTimestamp = Optional.ofNullable(input.getConfirmationTimestamp());
		Optional<String> confirmed = Optional.ofNullable(input.getConfirmed());
		Optional<String> clearingObligation = Optional.ofNullable(input.getClearingObligation());
		Optional<String> cleared = Optional.ofNullable(input.getCleared());
		Optional<ZonedDateTime> clearingTimestamp = Optional.ofNullable(input.getClearingTimestamp());
		Optional<String> centralCounterparty = Optional.ofNullable(input.getCentralCounterparty());
		Optional<String> masterAgreementType = Optional.ofNullable(input.getMasterAgreementType());
		Optional<String> otherMasterAgreementType = Optional.ofNullable(input.getOtherMasterAgreementType());
		Optional<Integer> masterAgreementVersion = Optional.ofNullable(input.getMasterAgreementVersion());
		Optional<Boolean> intragroup = Optional.ofNullable(input.getIntragroup());
		Optional<Boolean> ptrr = Optional.ofNullable(input.getPtrr());
		Optional<String> typeOfPTRRTechnique = Optional.ofNullable(input.getTypeOfPTRRTechnique());
		Optional<String> ptrrServiceProvider = Optional.ofNullable(input.getPtrrServiceProvider());
		Optional<String> venueOfExecution = Optional.ofNullable(input.getVenueOfExecution());
		Optional<ZonedDateTime> executionTimestamp = Optional.ofNullable(input.getExecutionTimestamp());
		Optional<Date> effectiveDate = Optional.ofNullable(input.getEffectiveDate());
		Optional<Date> expirationDate = Optional.ofNullable(input.getExpirationDate());
		Optional<Date> earlyTerminationDate = Optional.ofNullable(input.getEarlyTerminationDate());
		Optional<Date> finalContractualSettlementDate = Optional.ofNullable(input.getFinalContractualSettlementDate());
		Optional<String> deliveryType = Optional.ofNullable(input.getDeliveryType());
		Optional<List<FieldValue>> price = Optional.ofNullable(input.getPrice())
			.map(x -> priceAndNotationTypeTabulator.tabulate(x));
		Optional<List<List<FieldValue>>> priceSchedule = Optional.ofNullable(input.getPriceSchedule())
			.map(x -> x.stream()
				.map(_x -> priceScheduleReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<List<FieldValue>> packageTransactionPrice = Optional.ofNullable(input.getPackageTransactionPrice())
			.map(x -> packageTransactionPriceTypeTabulator.tabulate(x));
		Optional<BigDecimal> notionalLeg1 = Optional.ofNullable(input.getNotionalLeg1());
		Optional<ISOCurrencyCodeEnum> notionalCurrency1 = Optional.ofNullable(input.getNotionalCurrency1());
		Optional<List<List<FieldValue>>> notionalAmountScheduleLeg1 = Optional.ofNullable(input.getNotionalAmountScheduleLeg1())
			.map(x -> x.stream()
				.map(_x -> notionalAmountScheduleLeg1ReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<BigDecimal> totalNotionalQuantityLeg1 = Optional.ofNullable(input.getTotalNotionalQuantityLeg1());
		Optional<List<List<FieldValue>>> notionalQuantityScheduleLeg1 = Optional.ofNullable(input.getNotionalQuantityScheduleLeg1())
			.map(x -> x.stream()
				.map(_x -> notionalQuantityScheduleLeg1ReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<BigDecimal> totalNotionalQuantityLeg2 = Optional.ofNullable(input.getTotalNotionalQuantityLeg2());
		Optional<BigDecimal> notionalLeg2 = Optional.ofNullable(input.getNotionalLeg2());
		Optional<ISOCurrencyCodeEnum> notionalCurrency2 = Optional.ofNullable(input.getNotionalCurrency2());
		Optional<List<List<FieldValue>>> notionalAmountScheduleLeg2 = Optional.ofNullable(input.getNotionalAmountScheduleLeg2())
			.map(x -> x.stream()
				.map(_x -> notionalAmountScheduleLeg2ReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<List<List<FieldValue>>> notionalQuantityScheduleLeg2 = Optional.ofNullable(input.getNotionalQuantityScheduleLeg2())
			.map(x -> x.stream()
				.map(_x -> notionalQuantityScheduleLeg2ReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<List<List<FieldValue>>> otherPayment = Optional.ofNullable(input.getOtherPayment())
			.map(x -> x.stream()
				.map(_x -> otherPaymentReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<BigDecimal> fixedRateOfLeg1 = Optional.ofNullable(input.getFixedRateOfLeg1());
		Optional<InterestComputationMethod4Code> fixedRateDayCountConventionLeg1 = Optional.ofNullable(input.getFixedRateDayCountConventionLeg1());
		Optional<String> fixedRatePaymentFrequencyPeriodLeg1 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodLeg1());
		Optional<Integer> fixedRatePaymentFrequencyPeriodMultiplierLeg1 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodMultiplierLeg1());
		Optional<String> identifierOfFloatingRateOfLeg1 = Optional.ofNullable(input.getIdentifierOfFloatingRateOfLeg1());
		Optional<String> indicatorOfTheFloatingRateOfLeg1 = Optional.ofNullable(input.getIndicatorOfTheFloatingRateOfLeg1());
		Optional<String> nameOfTheFloatingRateOfLeg1 = Optional.ofNullable(input.getNameOfTheFloatingRateOfLeg1());
		Optional<InterestComputationMethod4Code> floatingRateDayCountConventionOfLeg1 = Optional.ofNullable(input.getFloatingRateDayCountConventionOfLeg1());
		Optional<Frequency13Code__1> floatingRatePaymentFrequencyPeriodOfLeg1 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodOfLeg1());
		Optional<Integer> floatingRatePaymentFrequencyPeriodMultiplierOfLeg1 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1());
		Optional<String> floatingRateReferencePeriodOfLeg1TimePeriod = Optional.ofNullable(input.getFloatingRateReferencePeriodOfLeg1TimePeriod());
		Optional<Integer> floatingRateReferencePeriodOfLeg1Multiplier = Optional.ofNullable(input.getFloatingRateReferencePeriodOfLeg1Multiplier());
		Optional<String> floatingRateResetFrequencyPeriodOfLeg1 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodOfLeg1());
		Optional<Integer> floatingRateResetFrequencyPeriodMultiplierLeg1 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodMultiplierLeg1());
		Optional<List<FieldValue>> spreadOfLeg1 = Optional.ofNullable(input.getSpreadOfLeg1())
			.map(x -> spreadOfLeg1TypeTabulator.tabulate(x));
		Optional<BigDecimal> fixedRateOfLeg2 = Optional.ofNullable(input.getFixedRateOfLeg2());
		Optional<InterestComputationMethod4Code> fixedRateDayCountConventionLeg2 = Optional.ofNullable(input.getFixedRateDayCountConventionLeg2());
		Optional<String> fixedRatePaymentFrequencyPeriodLeg2 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodLeg2());
		Optional<Integer> fixedRatePaymentFrequencyPeriodMultiplierLeg2 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodMultiplierLeg2());
		Optional<String> identifierOfFloatingRateOfLeg2 = Optional.ofNullable(input.getIdentifierOfFloatingRateOfLeg2());
		Optional<String> indicatorOfTheFloatingRateOfLeg2 = Optional.ofNullable(input.getIndicatorOfTheFloatingRateOfLeg2());
		Optional<String> nameOfTheFloatingRateOfLeg2 = Optional.ofNullable(input.getNameOfTheFloatingRateOfLeg2());
		Optional<InterestComputationMethod4Code> floatingRateDayCountConventionOfLeg2 = Optional.ofNullable(input.getFloatingRateDayCountConventionOfLeg2());
		Optional<String> floatingRatePaymentFrequencyPeriodOfLeg2 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodOfLeg2());
		Optional<Integer> floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2());
		Optional<String> floatingRateReferencePeriodOfLeg2TimePeriod = Optional.ofNullable(input.getFloatingRateReferencePeriodOfLeg2TimePeriod());
		Optional<Integer> floatingRateReferencePeriodOfLeg2Multiplier = Optional.ofNullable(input.getFloatingRateReferencePeriodOfLeg2Multiplier());
		Optional<String> floatingRateResetFrequencyPeriodOfLeg2 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodOfLeg2());
		Optional<Integer> floatingRateResetFrequencyPeriodMultiplierLeg2 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodMultiplierLeg2());
		Optional<List<FieldValue>> spreadOfLeg2 = Optional.ofNullable(input.getSpreadOfLeg2())
			.map(x -> spreadOfLeg2TypeTabulator.tabulate(x));
		Optional<List<FieldValue>> packageTransactionSpread = Optional.ofNullable(input.getPackageTransactionSpread())
			.map(x -> packageTransactionSpreadTypeTabulator.tabulate(x));
		Optional<BigDecimal> exchangeRate = Optional.ofNullable(input.getExchangeRate());
		Optional<BigDecimal> forwardExchangeRate = Optional.ofNullable(input.getForwardExchangeRate());
		Optional<String> exchangeRateBasis = Optional.ofNullable(input.getExchangeRateBasis());
		Optional<String> baseProduct = Optional.ofNullable(input.getBaseProduct());
		Optional<String> subProduct = Optional.ofNullable(input.getSubProduct());
		Optional<String> furtherSubProduct = Optional.ofNullable(input.getFurtherSubProduct());
		Optional<List<? extends String>> deliveryPoint = Optional.ofNullable(input.getDeliveryPoint());
		Optional<String> interconnectionPoint = Optional.ofNullable(input.getInterconnectionPoint());
		Optional<EnergyLoadType1Code> loadType = Optional.ofNullable(input.getLoadType());
		Optional<List<List<FieldValue>>> deliveryReport = Optional.ofNullable(input.getDeliveryReport())
			.map(x -> x.stream()
				.map(_x -> deliveryReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<String> optionType = Optional.ofNullable(input.getOptionType());
		Optional<String> optionStyle = Optional.ofNullable(input.getOptionStyle());
		Optional<List<FieldValue>> strikePrice = Optional.ofNullable(input.getStrikePrice())
			.map(x -> strikePriceAndNotationTypeTabulator.tabulate(x));
		Optional<List<List<FieldValue>>> strikePriceSchedule = Optional.ofNullable(input.getStrikePriceSchedule())
			.map(x -> x.stream()
				.map(_x -> strikePriceScheduleReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<BigDecimal> optionPremiumAmount = Optional.ofNullable(input.getOptionPremiumAmount());
		Optional<ISOCurrencyCodeEnum> optionPremiumCurrency = Optional.ofNullable(input.getOptionPremiumCurrency());
		Optional<Date> optionPremiumPaymentDate = Optional.ofNullable(input.getOptionPremiumPaymentDate());
		Optional<Date> maturityDateOfTheUnderlying = Optional.ofNullable(input.getMaturityDateOfTheUnderlying());
		Optional<String> seniority = Optional.ofNullable(input.getSeniority());
		Optional<String> referenceEntity = Optional.ofNullable(input.getReferenceEntity());
		Optional<Integer> series = Optional.ofNullable(input.getSeries());
		Optional<Integer> version = Optional.ofNullable(input.getVersion());
		Optional<BigDecimal> indexFactor = Optional.ofNullable(input.getIndexFactor());
		Optional<Boolean> tranche = Optional.ofNullable(input.getTranche());
		Optional<BigDecimal> cdSIndexAttachmentPoint = Optional.ofNullable(input.getCdSIndexAttachmentPoint());
		Optional<BigDecimal> cdSIndexDetachmentPoint = Optional.ofNullable(input.getCdSIndexDetachmentPoint());
		Optional<Boolean> collateralPortfolioIndicator = Optional.ofNullable(input.getCollateralPortfolioIndicator());
		Optional<String> collateralPortfolioCode = Optional.ofNullable(input.getCollateralPortfolioCode());
		Optional<String> actionType = Optional.ofNullable(input.getActionType());
		Optional<String> eventType = Optional.ofNullable(input.getEventType());
		Optional<Date> eventDate = Optional.ofNullable(input.getEventDate());
		Optional<String> level = Optional.ofNullable(input.getLevel());
		return Arrays.asList(
			new FieldValueImpl(reportingTimestampField, reportingTimestamp),
			new FieldValueImpl(reportSubmittingEntityIDField, reportSubmittingEntityID),
			new FieldValueImpl(entityResponsibleForReportingField, entityResponsibleForReporting),
			new FieldValueImpl(counterparty1Field, counterparty1),
			new FieldValueImpl(natureOfCounterparty1Field, natureOfCounterparty1),
			new FieldValueImpl(corporateSectorOfTheCounterparty1Field, corporateSectorOfTheCounterparty1),
			new FieldValueImpl(clearingThresholdOfCounterparty1Field, clearingThresholdOfCounterparty1),
			new FieldValueImpl(counterparty2Field, counterparty2),
			new FieldValueImpl(countryOfTheCounterParty2Field, countryOfTheCounterParty2),
			new FieldValueImpl(natureOfCounterparty2Field, natureOfCounterparty2),
			new FieldValueImpl(corporateSectorOfTheCounterparty2Field, corporateSectorOfTheCounterparty2),
			new FieldValueImpl(counterparty2IdentifierTypeField, counterparty2IdentifierType),
			new FieldValueImpl(clearingMemberField, clearingMember),
			new FieldValueImpl(clearingThresholdOfCounterparty2Field, clearingThresholdOfCounterparty2),
			new FieldValueImpl(reportingObligationOfTheCounterparty2Field, reportingObligationOfTheCounterparty2),
			new FieldValueImpl(brokerIDField, brokerID),
			new FieldValueImpl(directionField, direction),
			new FieldValueImpl(directionOfLeg1Field, directionOfLeg1),
			new FieldValueImpl(directionOfLeg2Field, directionOfLeg2),
			new FieldValueImpl(directlyLinkedToCommercialActivityOrTreasuryFinancingField, directlyLinkedToCommercialActivityOrTreasuryFinancing),
			new FieldValueImpl(utiField, uti),
			new FieldValueImpl(utiProprietaryField, utiProprietary),
			new FieldValueImpl(reportTrackingNumberField, reportTrackingNumber),
			new FieldValueImpl(priorUtiField, priorUti),
			new FieldValueImpl(priorUtiProprietaryField, priorUtiProprietary),
			new FieldValueImpl(ptrrIdField, ptrrId),
			new FieldValueImpl(packageIdentifierField, packageIdentifier),
			new FieldValueImpl(subsequentPositionUtiField, subsequentPositionUti),
			new FieldValueImpl(isinField, isin),
			new FieldValueImpl(upiField, upi),
			new FieldValueImpl(productClassificationField, productClassification),
			new FieldValueImpl(contractTypeField, contractType),
			new FieldValueImpl(assetClassField, assetClass),
			new FieldValueImpl(isCryptoField, isCrypto),
			new FieldValueImpl(underlyingIdentificationTypeField, underlyingIdentificationType),
			new FieldValueImpl(underlyingIdentificationField, underlyingIdentification),
			new FieldValueImpl(nameOfTheUnderlyingIndexField, nameOfTheUnderlyingIndex),
			new FieldValueImpl(indicatorOfTheUnderlyingIndexField, indicatorOfTheUnderlyingIndex),
			new MultiNestedFieldValueImpl(basketConstituentsField, basketConstituents),
			new FieldValueImpl(customBasketCodeField, customBasketCode),
			new FieldValueImpl(settlementCurrency1Field, settlementCurrency1),
			new FieldValueImpl(settlementCurrency2Field, settlementCurrency2),
			new FieldValueImpl(valuationAmountField, valuationAmount),
			new FieldValueImpl(valuationCurrencyField, valuationCurrency),
			new FieldValueImpl(valuationMethodField, valuationMethod),
			new FieldValueImpl(deltaField, delta),
			new FieldValueImpl(valuationTimestampField, valuationTimestamp),
			new FieldValueImpl(confirmationTimestampField, confirmationTimestamp),
			new FieldValueImpl(confirmedField, confirmed),
			new FieldValueImpl(clearingObligationField, clearingObligation),
			new FieldValueImpl(clearedField, cleared),
			new FieldValueImpl(clearingTimestampField, clearingTimestamp),
			new FieldValueImpl(centralCounterpartyField, centralCounterparty),
			new FieldValueImpl(masterAgreementTypeField, masterAgreementType),
			new FieldValueImpl(otherMasterAgreementTypeField, otherMasterAgreementType),
			new FieldValueImpl(masterAgreementVersionField, masterAgreementVersion),
			new FieldValueImpl(intragroupField, intragroup),
			new FieldValueImpl(ptrrField, ptrr),
			new FieldValueImpl(typeOfPTRRTechniqueField, typeOfPTRRTechnique),
			new FieldValueImpl(ptrrServiceProviderField, ptrrServiceProvider),
			new FieldValueImpl(venueOfExecutionField, venueOfExecution),
			new FieldValueImpl(executionTimestampField, executionTimestamp),
			new FieldValueImpl(effectiveDateField, effectiveDate),
			new FieldValueImpl(expirationDateField, expirationDate),
			new FieldValueImpl(earlyTerminationDateField, earlyTerminationDate),
			new FieldValueImpl(finalContractualSettlementDateField, finalContractualSettlementDate),
			new FieldValueImpl(deliveryTypeField, deliveryType),
			new NestedFieldValueImpl(priceField, price),
			new MultiNestedFieldValueImpl(priceScheduleField, priceSchedule),
			new NestedFieldValueImpl(packageTransactionPriceField, packageTransactionPrice),
			new FieldValueImpl(notionalLeg1Field, notionalLeg1),
			new FieldValueImpl(notionalCurrency1Field, notionalCurrency1),
			new MultiNestedFieldValueImpl(notionalAmountScheduleLeg1Field, notionalAmountScheduleLeg1),
			new FieldValueImpl(totalNotionalQuantityLeg1Field, totalNotionalQuantityLeg1),
			new MultiNestedFieldValueImpl(notionalQuantityScheduleLeg1Field, notionalQuantityScheduleLeg1),
			new FieldValueImpl(totalNotionalQuantityLeg2Field, totalNotionalQuantityLeg2),
			new FieldValueImpl(notionalLeg2Field, notionalLeg2),
			new FieldValueImpl(notionalCurrency2Field, notionalCurrency2),
			new MultiNestedFieldValueImpl(notionalAmountScheduleLeg2Field, notionalAmountScheduleLeg2),
			new MultiNestedFieldValueImpl(notionalQuantityScheduleLeg2Field, notionalQuantityScheduleLeg2),
			new MultiNestedFieldValueImpl(otherPaymentField, otherPayment),
			new FieldValueImpl(fixedRateOfLeg1Field, fixedRateOfLeg1),
			new FieldValueImpl(fixedRateDayCountConventionLeg1Field, fixedRateDayCountConventionLeg1),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodLeg1Field, fixedRatePaymentFrequencyPeriodLeg1),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodMultiplierLeg1Field, fixedRatePaymentFrequencyPeriodMultiplierLeg1),
			new FieldValueImpl(identifierOfFloatingRateOfLeg1Field, identifierOfFloatingRateOfLeg1),
			new FieldValueImpl(indicatorOfTheFloatingRateOfLeg1Field, indicatorOfTheFloatingRateOfLeg1),
			new FieldValueImpl(nameOfTheFloatingRateOfLeg1Field, nameOfTheFloatingRateOfLeg1),
			new FieldValueImpl(floatingRateDayCountConventionOfLeg1Field, floatingRateDayCountConventionOfLeg1),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodOfLeg1Field, floatingRatePaymentFrequencyPeriodOfLeg1),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodMultiplierOfLeg1Field, floatingRatePaymentFrequencyPeriodMultiplierOfLeg1),
			new FieldValueImpl(floatingRateReferencePeriodOfLeg1TimePeriodField, floatingRateReferencePeriodOfLeg1TimePeriod),
			new FieldValueImpl(floatingRateReferencePeriodOfLeg1MultiplierField, floatingRateReferencePeriodOfLeg1Multiplier),
			new FieldValueImpl(floatingRateResetFrequencyPeriodOfLeg1Field, floatingRateResetFrequencyPeriodOfLeg1),
			new FieldValueImpl(floatingRateResetFrequencyPeriodMultiplierLeg1Field, floatingRateResetFrequencyPeriodMultiplierLeg1),
			new NestedFieldValueImpl(spreadOfLeg1Field, spreadOfLeg1),
			new FieldValueImpl(fixedRateOfLeg2Field, fixedRateOfLeg2),
			new FieldValueImpl(fixedRateDayCountConventionLeg2Field, fixedRateDayCountConventionLeg2),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodLeg2Field, fixedRatePaymentFrequencyPeriodLeg2),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodMultiplierLeg2Field, fixedRatePaymentFrequencyPeriodMultiplierLeg2),
			new FieldValueImpl(identifierOfFloatingRateOfLeg2Field, identifierOfFloatingRateOfLeg2),
			new FieldValueImpl(indicatorOfTheFloatingRateOfLeg2Field, indicatorOfTheFloatingRateOfLeg2),
			new FieldValueImpl(nameOfTheFloatingRateOfLeg2Field, nameOfTheFloatingRateOfLeg2),
			new FieldValueImpl(floatingRateDayCountConventionOfLeg2Field, floatingRateDayCountConventionOfLeg2),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodOfLeg2Field, floatingRatePaymentFrequencyPeriodOfLeg2),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodMultiplierOfLeg2Field, floatingRatePaymentFrequencyPeriodMultiplierOfLeg2),
			new FieldValueImpl(floatingRateReferencePeriodOfLeg2TimePeriodField, floatingRateReferencePeriodOfLeg2TimePeriod),
			new FieldValueImpl(floatingRateReferencePeriodOfLeg2MultiplierField, floatingRateReferencePeriodOfLeg2Multiplier),
			new FieldValueImpl(floatingRateResetFrequencyPeriodOfLeg2Field, floatingRateResetFrequencyPeriodOfLeg2),
			new FieldValueImpl(floatingRateResetFrequencyPeriodMultiplierLeg2Field, floatingRateResetFrequencyPeriodMultiplierLeg2),
			new NestedFieldValueImpl(spreadOfLeg2Field, spreadOfLeg2),
			new NestedFieldValueImpl(packageTransactionSpreadField, packageTransactionSpread),
			new FieldValueImpl(exchangeRateField, exchangeRate),
			new FieldValueImpl(forwardExchangeRateField, forwardExchangeRate),
			new FieldValueImpl(exchangeRateBasisField, exchangeRateBasis),
			new FieldValueImpl(baseProductField, baseProduct),
			new FieldValueImpl(subProductField, subProduct),
			new FieldValueImpl(furtherSubProductField, furtherSubProduct),
			new FieldValueImpl(deliveryPointField, deliveryPoint),
			new FieldValueImpl(interconnectionPointField, interconnectionPoint),
			new FieldValueImpl(loadTypeField, loadType),
			new MultiNestedFieldValueImpl(deliveryReportField, deliveryReport),
			new FieldValueImpl(optionTypeField, optionType),
			new FieldValueImpl(optionStyleField, optionStyle),
			new NestedFieldValueImpl(strikePriceField, strikePrice),
			new MultiNestedFieldValueImpl(strikePriceScheduleField, strikePriceSchedule),
			new FieldValueImpl(optionPremiumAmountField, optionPremiumAmount),
			new FieldValueImpl(optionPremiumCurrencyField, optionPremiumCurrency),
			new FieldValueImpl(optionPremiumPaymentDateField, optionPremiumPaymentDate),
			new FieldValueImpl(maturityDateOfTheUnderlyingField, maturityDateOfTheUnderlying),
			new FieldValueImpl(seniorityField, seniority),
			new FieldValueImpl(referenceEntityField, referenceEntity),
			new FieldValueImpl(seriesField, series),
			new FieldValueImpl(versionField, version),
			new FieldValueImpl(indexFactorField, indexFactor),
			new FieldValueImpl(trancheField, tranche),
			new FieldValueImpl(cdSIndexAttachmentPointField, cdSIndexAttachmentPoint),
			new FieldValueImpl(cdSIndexDetachmentPointField, cdSIndexDetachmentPoint),
			new FieldValueImpl(collateralPortfolioIndicatorField, collateralPortfolioIndicator),
			new FieldValueImpl(collateralPortfolioCodeField, collateralPortfolioCode),
			new FieldValueImpl(actionTypeField, actionType),
			new FieldValueImpl(eventTypeField, eventType),
			new FieldValueImpl(eventDateField, eventDate),
			new FieldValueImpl(levelField, level)
		);
	}
}
