package drr.regulation.esma.emir.refit.trade.reports;

import cdm.event.common.ContractDetails;
import cdm.event.common.metafields.ReferenceWithMetaContractDetails;
import cdm.legaldocumentation.common.AgreementName;
import cdm.legaldocumentation.common.LegalAgreement;
import cdm.legaldocumentation.common.LegalAgreementIdentification;
import cdm.legaldocumentation.common.LegalAgreementTypeEnum;
import cdm.legaldocumentation.master.MasterAgreementTypeEnum;
import cdm.legaldocumentation.master.metafields.FieldWithMetaMasterAgreementTypeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.IsAllowableAction;
import drr.regulation.common.functions.PositionForEvent;
import drr.regulation.common.functions.TradeForEvent;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(MasterAgreementTypeRule.MasterAgreementTypeRuleDefault.class)
public abstract class MasterAgreementTypeRule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected IsAllowableAction isAllowableAction;
	@Inject protected PositionForEvent positionForEvent;
	@Inject protected TradeForEvent tradeForEvent;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class MasterAgreementTypeRuleDefault extends MasterAgreementTypeRule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult0 = MapperS.of(input)
				.filterSingleNullSafe(item -> isAllowableAction.evaluate(item.get()));
			final MapperS<ContractDetails> thenResult1 = thenResult0
				.mapSingleToItem(item -> {
					if (exists(MapperS.of(tradeForEvent.evaluate(item.get()))).getOrDefault(false)) {
						return MapperS.of(tradeForEvent.evaluate(item.get())).<ContractDetails>map("getContractDetails", trade -> trade.getContractDetails());
					}
					if (exists(MapperS.of(positionForEvent.evaluate(item.get()))).getOrDefault(false)) {
						return MapperS.of(positionForEvent.evaluate(item.get())).<ReferenceWithMetaContractDetails>map("getContractDetails", contractBase -> contractBase.getContractDetails()).<ContractDetails>map("getValue", _f->_f.getValue());
					}
					return MapperS.<ContractDetails>ofNull();
				});
			final MapperC<LegalAgreement> thenResult2 = thenResult1
				.mapSingleToList(item -> item.<LegalAgreement>mapC("getDocumentation", contractDetails -> contractDetails.getDocumentation()));
			final MapperC<LegalAgreement> thenResult3 = thenResult2
				.filterItemNullSafe(item -> areEqual(item.<LegalAgreementIdentification>map("getLegalAgreementIdentification", legalAgreementBase -> legalAgreementBase.getLegalAgreementIdentification()).<AgreementName>map("getAgreementName", legalAgreementIdentification -> legalAgreementIdentification.getAgreementName()).<LegalAgreementTypeEnum>map("getAgreementType", agreementName -> agreementName.getAgreementType()), MapperS.of(LegalAgreementTypeEnum.MASTER_AGREEMENT), CardinalityOperator.All).get());
			final MapperS<LegalAgreement> thenResult4 = MapperS.of(thenResult3.get());
			final MapperS<AgreementName> thenResult5 = thenResult4
				.mapSingleToItem(item -> item.<LegalAgreementIdentification>map("getLegalAgreementIdentification", legalAgreementBase -> legalAgreementBase.getLegalAgreementIdentification()).<AgreementName>map("getAgreementName", legalAgreementIdentification -> legalAgreementIdentification.getAgreementName()));
			output = thenResult5
				.mapSingleToItem(item -> {
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.ISDA_MASTER), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("ISDA");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.ISDAFIA_CDEA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("CDEA");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.EMA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("EUMA");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.FBF), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("FMAT");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.GERMAN), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("DERV");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.CMOF), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("CMOP");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.SWISS), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("CHMA");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.EFET_ELECTRICITY), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("EFMA");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.EFET_GAS), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("EFMA");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.GMRA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("GMRA");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.GMSLA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("GMSL");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.BESPOKE), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("BIAG");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.ISDAIIFM_TMA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("IDMA");
					}
					return MapperS.of("OTHR");
				}).get();
			
			return output;
		}
	}
}
