package drr.regulation.esma.emir.refit.trade.reports;

import cdm.base.math.DatedValue;
import cdm.base.math.RoundingDirectionEnum;
import cdm.base.math.functions.RoundToPrecision;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.standards.iosco.cde.reports.CDENotionalAmountScheduleAmountRule;
import java.math.BigDecimal;
import javax.inject.Inject;


@ImplementedBy(NotionalAmountInEffectOnAssociatedEffectiveDateOfLeg1Rule.NotionalAmountInEffectOnAssociatedEffectiveDateOfLeg1RuleDefault.class)
public abstract class NotionalAmountInEffectOnAssociatedEffectiveDateOfLeg1Rule implements ReportFunction<DatedValue, BigDecimal> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDENotionalAmountScheduleAmountRule cDENotionalAmountScheduleAmount;
	@Inject protected RoundToPrecision roundToPrecision;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public BigDecimal evaluate(DatedValue input) {
		BigDecimal output = doEvaluate(input);
		
		return output;
	}

	protected abstract BigDecimal doEvaluate(DatedValue input);

	public static class NotionalAmountInEffectOnAssociatedEffectiveDateOfLeg1RuleDefault extends NotionalAmountInEffectOnAssociatedEffectiveDateOfLeg1Rule {
		@Override
		protected BigDecimal doEvaluate(DatedValue input) {
			BigDecimal output = null;
			return assignOutput(output, input);
		}
		
		protected BigDecimal assignOutput(BigDecimal output, DatedValue input) {
			final MapperS<BigDecimal> thenResult = MapperS.of(cDENotionalAmountScheduleAmount.evaluate(input));
			output = thenResult
				.mapSingleToItem(item -> MapperS.of(roundToPrecision.evaluate(item.get(), 5, RoundingDirectionEnum.NEAREST))).get();
			
			return output;
		}
	}
}
