package drr.regulation.esma.emir.refit.trade.reports;

import cdm.base.math.DatedValue;
import cdm.base.math.NonNegativeQuantitySchedule;
import cdm.base.math.UnitType;
import cdm.base.math.metafields.ReferenceWithMetaNonNegativeQuantitySchedule;
import cdm.event.common.Trade;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import com.rosetta.model.metafields.FieldWithMetaString;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.GetLeg2ResolvablePriceQuantity;
import drr.regulation.common.functions.IsAllowableAction;
import drr.regulation.common.functions.TradeForEvent;
import drr.regulation.esma.emir.refit.trade.NotionalAmountScheduleLeg2Report;
import drr.regulation.esma.emir.refit.trade.NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(NotionalAmountScheduleLeg2Rule.NotionalAmountScheduleLeg2RuleDefault.class)
public abstract class NotionalAmountScheduleLeg2Rule implements ReportFunction<TransactionReportInstruction, List<? extends NotionalAmountScheduleLeg2Report>> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected EffectiveDateOfTheNotionalAmountOfLeg2Rule effectiveDateOfTheNotionalAmountOfLeg2;
	@Inject protected GetLeg2ResolvablePriceQuantity getLeg2ResolvablePriceQuantity;
	@Inject protected IsAllowableAction isAllowableAction;
	@Inject protected NotionalAmountInEffectOnAssociatedEffectiveDateOfLeg2Rule notionalAmountInEffectOnAssociatedEffectiveDateOfLeg2;
	@Inject protected TradeForEvent tradeForEvent;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public List<? extends NotionalAmountScheduleLeg2Report> evaluate(TransactionReportInstruction input) {
		List<NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> outputBuilder = doEvaluate(input);
		
		final List<? extends NotionalAmountScheduleLeg2Report> output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.stream().map(NotionalAmountScheduleLeg2Report::build).collect(Collectors.toList());
			objectValidator.validate(NotionalAmountScheduleLeg2Report.class, output);
		}
		
		return output;
	}

	protected abstract List<NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> doEvaluate(TransactionReportInstruction input);

	public static class NotionalAmountScheduleLeg2RuleDefault extends NotionalAmountScheduleLeg2Rule {
		@Override
		protected List<NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> doEvaluate(TransactionReportInstruction input) {
			List<NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> output = new ArrayList<>();
			return assignOutput(output, input);
		}
		
		protected List<NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> assignOutput(List<NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult0 = MapperS.of(input)
				.filterSingleNullSafe(item -> isAllowableAction.evaluate(item.get()));
			final MapperS<Trade> thenResult1 = thenResult0
				.mapSingleToItem(item -> MapperS.of(tradeForEvent.evaluate(item.get())));
			final MapperS<NonNegativeQuantitySchedule> thenResult2 = thenResult1
				.mapSingleToItem(item -> MapperS.of(getLeg2ResolvablePriceQuantity.evaluate(item.get())).<ReferenceWithMetaNonNegativeQuantitySchedule>map("getQuantitySchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getQuantitySchedule()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()));
			final MapperS<NonNegativeQuantitySchedule> thenResult3 = thenResult2
				.filterSingleNullSafe(item -> exists(item.<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue())).get());
			final MapperC<DatedValue> thenResult4 = thenResult3
				.mapSingleToList(item -> item.<DatedValue>mapC("getDatedValue", measureSchedule -> measureSchedule.getDatedValue()));
			output = toBuilder(thenResult4
				.mapItem(item -> MapperS.of(NotionalAmountScheduleLeg2Report.builder()
					.setAmount(notionalAmountInEffectOnAssociatedEffectiveDateOfLeg2.evaluate(item.get()))
					.setEffectiveDate(effectiveDateOfTheNotionalAmountOfLeg2.evaluate(item.get()))
					.build()
				)).getMulti());
			
			return Optional.ofNullable(output)
				.map(o -> o.stream().map(i -> i.prune()).collect(Collectors.toList()))
				.orElse(null);
		}
	}
}
