package drr.regulation.esma.emir.refit.trade.reports;

import cdm.base.math.RoundingDirectionEnum;
import cdm.base.math.functions.RoundToPrecision;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.QuantityReportablePeriod;
import drr.standards.iosco.cde.reports.CDENotionalQuantityScheduleValueRule;
import java.math.BigDecimal;
import javax.inject.Inject;


@ImplementedBy(NotionalQuantityInEffectOnAssociatedEffectiveDateOfLeg2Rule.NotionalQuantityInEffectOnAssociatedEffectiveDateOfLeg2RuleDefault.class)
public abstract class NotionalQuantityInEffectOnAssociatedEffectiveDateOfLeg2Rule implements ReportFunction<QuantityReportablePeriod, BigDecimal> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDENotionalQuantityScheduleValueRule cDENotionalQuantityScheduleValue;
	@Inject protected RoundToPrecision roundToPrecision;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public BigDecimal evaluate(QuantityReportablePeriod input) {
		BigDecimal output = doEvaluate(input);
		
		return output;
	}

	protected abstract BigDecimal doEvaluate(QuantityReportablePeriod input);

	public static class NotionalQuantityInEffectOnAssociatedEffectiveDateOfLeg2RuleDefault extends NotionalQuantityInEffectOnAssociatedEffectiveDateOfLeg2Rule {
		@Override
		protected BigDecimal doEvaluate(QuantityReportablePeriod input) {
			BigDecimal output = null;
			return assignOutput(output, input);
		}
		
		protected BigDecimal assignOutput(BigDecimal output, QuantityReportablePeriod input) {
			final MapperS<BigDecimal> thenResult = MapperS.of(cDENotionalQuantityScheduleValue.evaluate(input));
			output = thenResult
				.mapSingleToItem(item -> MapperS.of(roundToPrecision.evaluate(item.get(), 5, RoundingDirectionEnum.NEAREST))).get();
			
			return output;
		}
	}
}
