package drr.regulation.esma.emir.refit.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.esma.emir.refit.trade.NotionalQuantityScheduleLeg2Report;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class NotionalQuantityScheduleLeg2ReportTypeTabulator implements Tabulator<NotionalQuantityScheduleLeg2Report> {
	private final Field effectiveDateField;
	private final Field endDateField;
	private final Field valueField;
	
	public NotionalQuantityScheduleLeg2ReportTypeTabulator() {
		this.effectiveDateField = new FieldImpl(
			"effectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "EffectiveDateOfTheNotionalQuantityOfLeg2")),
			Optional.of("2.70/2.72-$ 2.70 Effective date of the notional quantity of leg 2"),
			Arrays.asList()
		);
		this.endDateField = new FieldImpl(
			"endDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "EndDateOfTheNotionalQuantityOfLeg2")),
			Optional.of("2.70/2.72-$ 2.71 End date of the notional quantity of leg 2"),
			Arrays.asList()
		);
		this.valueField = new FieldImpl(
			"value",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "NotionalQuantityInEffectOnAssociatedEffectiveDateOfLeg2")),
			Optional.of("2.70/2.72-$ 2.72 Notional quantity in effect on associated effective date of leg 2"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(effectiveDateField, endDateField, valueField);
	}
	
	@Override
	public List<FieldValue> tabulate(NotionalQuantityScheduleLeg2Report input) {
		Optional<Date> effectiveDate = Optional.ofNullable(input.getEffectiveDate());
		Optional<Date> endDate = Optional.ofNullable(input.getEndDate());
		Optional<BigDecimal> value = Optional.ofNullable(input.getValue());
		return Arrays.asList(
			new FieldValueImpl(effectiveDateField, effectiveDate),
			new FieldValueImpl(endDateField, endDate),
			new FieldValueImpl(valueField, value)
		);
	}
}
