package drr.regulation.esma.emir.refit.trade.reports;

import cdm.event.common.ContractDetails;
import cdm.event.common.metafields.ReferenceWithMetaContractDetails;
import cdm.legaldocumentation.common.AgreementName;
import cdm.legaldocumentation.common.LegalAgreement;
import cdm.legaldocumentation.common.LegalAgreementIdentification;
import cdm.legaldocumentation.common.LegalAgreementTypeEnum;
import cdm.legaldocumentation.master.MasterAgreementTypeEnum;
import cdm.legaldocumentation.master.metafields.FieldWithMetaMasterAgreementTypeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.IsAllowableAction;
import drr.regulation.common.functions.PositionForEvent;
import drr.regulation.common.functions.TradeForEvent;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(OtherMasterAgreementTypeRule.OtherMasterAgreementTypeRuleDefault.class)
public abstract class OtherMasterAgreementTypeRule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected IsAllowableAction isAllowableAction;
	@Inject protected PositionForEvent positionForEvent;
	@Inject protected TradeForEvent tradeForEvent;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class OtherMasterAgreementTypeRuleDefault extends OtherMasterAgreementTypeRule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult0 = MapperS.of(input)
				.filterSingleNullSafe(item -> isAllowableAction.evaluate(item.get()));
			final MapperS<ContractDetails> thenResult1 = thenResult0
				.mapSingleToItem(item -> {
					if (exists(MapperS.of(tradeForEvent.evaluate(item.get()))).getOrDefault(false)) {
						return MapperS.of(tradeForEvent.evaluate(item.get())).<ContractDetails>map("getContractDetails", trade -> trade.getContractDetails());
					}
					if (exists(MapperS.of(positionForEvent.evaluate(item.get()))).getOrDefault(false)) {
						return MapperS.of(positionForEvent.evaluate(item.get())).<ReferenceWithMetaContractDetails>map("getContractDetails", contractBase -> contractBase.getContractDetails()).<ContractDetails>map("getValue", _f->_f.getValue());
					}
					return MapperS.<ContractDetails>ofNull();
				});
			final MapperC<LegalAgreement> thenResult2 = thenResult1
				.mapSingleToList(item -> item.<LegalAgreement>mapC("getDocumentation", contractDetails -> contractDetails.getDocumentation()));
			final MapperC<LegalAgreement> thenResult3 = thenResult2
				.filterItemNullSafe(item -> areEqual(item.<LegalAgreementIdentification>map("getLegalAgreementIdentification", legalAgreementBase -> legalAgreementBase.getLegalAgreementIdentification()).<AgreementName>map("getAgreementName", legalAgreementIdentification -> legalAgreementIdentification.getAgreementName()).<LegalAgreementTypeEnum>map("getAgreementType", agreementName -> agreementName.getAgreementType()), MapperS.of(LegalAgreementTypeEnum.MASTER_AGREEMENT), CardinalityOperator.All).get());
			final MapperS<LegalAgreement> thenResult4 = MapperS.of(thenResult3.get());
			final MapperS<AgreementName> thenResult5 = thenResult4
				.mapSingleToItem(item -> item.<LegalAgreementIdentification>map("getLegalAgreementIdentification", legalAgreementBase -> legalAgreementBase.getLegalAgreementIdentification()).<AgreementName>map("getAgreementName", legalAgreementIdentification -> legalAgreementIdentification.getAgreementName()));
			output = thenResult5
				.mapSingleToItem(item -> {
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.AFB), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("AFBMAForFXAndDerivativesTransactions");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.CMA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("ClearingMA");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.IFEMA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("InternationalForeignExchangeMA");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.IFEOMA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("InternationalForeignExchangeAndOptionsMA");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.ICOM), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("InternationalCurrencyOptionsMarketMA");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.JSCC), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("MasterAgreementOfJapanSecuritiesClearingCorp");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.EEI_POWER), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("EEIMasterPowerPurchaseAndSaleAgreement");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.GTMA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("FOAGridTradeMasterAgreement");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.GAS_EDI), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("GasEDIBaseCntrctShortTermSaleAndPurNatGas");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.IETA_ERPA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("IntlEmissionTAEmissionReductionPurchaseAgrmt");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.IETA_ETMA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("IntlEmissionsTradingAssocEmissionsTradingMA");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.IETA_IETMA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("IntlEmissionsTradingAssocAndTradingMA");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.LBMA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("IntlBullionMATermsByTheLdnBullionMktAssoc");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.LEAP), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("LeadershipInEnergyAutomatedProcessing");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.MCPSA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("CTAMasterCoalPurchaseAndSalesAgreement");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.NAESB_GAS), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("NAESBBaseContractSaleAndPurchaseOfNaturalGas");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.NBP), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("ShortTermFlatNBPTradingTermsAndConditions");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.RUSSIAN_DERIVATIVES), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("StdDocForDerivTxnsOnTheRussianFinMarkets");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.RUSSIAN_REPO), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("MAForRepurchaseAgreementOnTheRussianFinMkt");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.S_CO_TA), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("globalCOALStandardCoalTradingAgreement");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.TTF), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("TTFHubNaturalGasTradingTermsAndConditions");
					}
					if (areEqual(item.<FieldWithMetaMasterAgreementTypeEnum>map("getMasterAgreementType", agreementName -> agreementName.getMasterAgreementType()).<MasterAgreementTypeEnum>map("getValue", _f->_f.getValue()), MapperS.of(MasterAgreementTypeEnum.ZBT), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("ZeebruggeHubNaturalGasTradingTandC");
					}
					return item.<String>map("getOtherAgreement", agreementName -> agreementName.getOtherAgreement());
				}).get();
			
			return output;
		}
	}
}
