package drr.regulation.esma.emir.refit.trade.reports;

import cdm.base.staticdata.identifier.IdentifiedList;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.GetPackageInformation;
import drr.regulation.common.functions.IsAllowableAction;
import drr.standards.iosco.cde.reports.CDEPackageIdentifierRule;
import javax.inject.Inject;


@ImplementedBy(PackageIdentifierRule.PackageIdentifierRuleDefault.class)
public abstract class PackageIdentifierRule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDEPackageIdentifierRule cDEPackageIdentifier;
	@Inject protected GetPackageInformation getPackageInformation;
	@Inject protected IsAllowableAction isAllowableAction;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class PackageIdentifierRuleDefault extends PackageIdentifierRule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult0 = MapperS.of(input)
				.filterSingleNullSafe(item -> isAllowableAction.evaluate(item.get()));
			final MapperS<IdentifiedList> thenResult1 = thenResult0
				.mapSingleToItem(item -> MapperS.of(getPackageInformation.evaluate(item.get())));
			final MapperC<String> thenResult2 = thenResult1
				.mapSingleToList(item -> MapperC.<String>of(cDEPackageIdentifier.evaluate(item.get())));
			output = MapperS.of(thenResult2.get()).get();
			
			return output;
		}
	}
}
