package drr.regulation.esma.emir.refit.trade.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.esma.emir.refit.trade.PackageTransactionPrice;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class PackageTransactionPriceTypeTabulator implements Tabulator<PackageTransactionPrice> {
	private final Field packageTransactionPriceMonetaryField;
	private final Field packageTransactionPricePercentageField;
	private final Field packageTransactionPriceBasisField;
	private final Field packageTransactionPriceNotationField;
	private final Field packageTransactionPriceCurrencyField;
	
	public PackageTransactionPriceTypeTabulator() {
		this.packageTransactionPriceMonetaryField = new FieldImpl(
			"packageTransactionPriceMonetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "PackageTransactionPriceMonetary")),
			Optional.of("2.53 Package Transaction Price (Monetary)"),
			Arrays.asList()
		);
		this.packageTransactionPricePercentageField = new FieldImpl(
			"packageTransactionPricePercentage",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "PackageTransactionPricePercentage")),
			Optional.of("2.53 Package Transaction Price (Percentage)"),
			Arrays.asList()
		);
		this.packageTransactionPriceBasisField = new FieldImpl(
			"packageTransactionPriceBasis",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "PackageTransactionPriceBasis")),
			Optional.of("2.53 Package Transaction Price (Basis)"),
			Arrays.asList()
		);
		this.packageTransactionPriceNotationField = new FieldImpl(
			"packageTransactionPriceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "PackageTransactionPriceNotation")),
			Optional.of("2.53 Package Transaction Price Notation"),
			Arrays.asList()
		);
		this.packageTransactionPriceCurrencyField = new FieldImpl(
			"packageTransactionPriceCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "PackageTransactionPriceCurrency")),
			Optional.of("2.54 Package Transaction Price Currency"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(packageTransactionPriceMonetaryField, packageTransactionPricePercentageField, packageTransactionPriceBasisField, packageTransactionPriceNotationField, packageTransactionPriceCurrencyField);
	}
	
	@Override
	public List<FieldValue> tabulate(PackageTransactionPrice input) {
		Optional<BigDecimal> packageTransactionPriceMonetary = Optional.ofNullable(input.getPackageTransactionPriceMonetary());
		Optional<BigDecimal> packageTransactionPricePercentage = Optional.ofNullable(input.getPackageTransactionPricePercentage());
		Optional<BigDecimal> packageTransactionPriceBasis = Optional.ofNullable(input.getPackageTransactionPriceBasis());
		Optional<PriceNotationEnum> packageTransactionPriceNotation = Optional.ofNullable(input.getPackageTransactionPriceNotation());
		Optional<ISOCurrencyCodeEnum> packageTransactionPriceCurrency = Optional.ofNullable(input.getPackageTransactionPriceCurrency());
		return Arrays.asList(
			new FieldValueImpl(packageTransactionPriceMonetaryField, packageTransactionPriceMonetary),
			new FieldValueImpl(packageTransactionPricePercentageField, packageTransactionPricePercentage),
			new FieldValueImpl(packageTransactionPriceBasisField, packageTransactionPriceBasis),
			new FieldValueImpl(packageTransactionPriceNotationField, packageTransactionPriceNotation),
			new FieldValueImpl(packageTransactionPriceCurrencyField, packageTransactionPriceCurrency)
		);
	}
}
