package drr.regulation.esma.emir.refit.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.esma.emir.refit.trade.PriceScheduleReport;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class PriceScheduleReportTypeTabulator implements Tabulator<PriceScheduleReport> {
	private final Field effectiveDateField;
	private final Field endDateField;
	private final Field priceMonetaryField;
	private final Field pricePercentageField;
	
	public PriceScheduleReportTypeTabulator() {
		this.effectiveDateField = new FieldImpl(
			"effectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "UnadjustedEffectiveDateOfThePrice")),
			Optional.of("2.50/2.52-$ 2.50 Unadjusted effective date of the price"),
			Arrays.asList()
		);
		this.endDateField = new FieldImpl(
			"endDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "UnadjustedEndDateOfThePrice")),
			Optional.of("2.50/2.52-$ 2.51 Unadjusted end date of the price"),
			Arrays.asList()
		);
		this.priceMonetaryField = new FieldImpl(
			"priceMonetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "PriceInEffectBetweenTheUnadjustedEffectiveAndEndDateFormatMonetary")),
			Optional.of("2.50/2.52-$ 2.52 Price in effect between the unadjusted effective and end date (Monetary)"),
			Arrays.asList()
		);
		this.pricePercentageField = new FieldImpl(
			"pricePercentage",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "PriceInEffectBetweenTheUnadjustedEffectiveAndEndDatePercentage")),
			Optional.of("2.50/2.52-$ 2.52 Price in effect between the unadjusted effective and end date (Percentage)"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(effectiveDateField, endDateField, priceMonetaryField, pricePercentageField);
	}
	
	@Override
	public List<FieldValue> tabulate(PriceScheduleReport input) {
		Optional<Date> effectiveDate = Optional.ofNullable(input.getEffectiveDate());
		Optional<Date> endDate = Optional.ofNullable(input.getEndDate());
		Optional<BigDecimal> priceMonetary = Optional.ofNullable(input.getPriceMonetary());
		Optional<BigDecimal> pricePercentage = Optional.ofNullable(input.getPricePercentage());
		return Arrays.asList(
			new FieldValueImpl(effectiveDateField, effectiveDate),
			new FieldValueImpl(endDateField, endDate),
			new FieldValueImpl(priceMonetaryField, priceMonetary),
			new FieldValueImpl(pricePercentageField, pricePercentage)
		);
	}
}
