package drr.regulation.esma.emir.refit.trade.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.IsAllowableAction;
import drr.regulation.common.util.functions.StringContains;
import drr.standards.iosco.cde.reports.CDEPriorUTIRule;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(PriorUtiProprietaryRule.PriorUtiProprietaryRuleDefault.class)
public abstract class PriorUtiProprietaryRule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDEPriorUTIRule cDEPriorUTI;
	@Inject protected IsAllowableAction isAllowableAction;
	@Inject protected StringContains stringContains;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class PriorUtiProprietaryRuleDefault extends PriorUtiProprietaryRule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult0 = MapperS.of(input)
				.filterSingleNullSafe(item -> isAllowableAction.evaluate(item.get()));
			final MapperS<String> thenResult1 = thenResult0
				.mapSingleToItem(item -> MapperS.of(cDEPriorUTI.evaluate(item.get())));
			output = thenResult1
				.filterSingleNullSafe(item -> areEqual(MapperS.of(stringContains.evaluate(item.get(), "^[A-Z0-9]{18}[0-9]{2}[A-Z0-9]{0,32}$")), MapperS.of(false), CardinalityOperator.All).get()).get();
			
			return output;
		}
	}
}
