package drr.regulation.esma.emir.refit.trade.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.GetNameOfTheFloatingRateOfLeg2;
import drr.regulation.common.functions.GetNotationEnum;
import drr.regulation.common.functions.IsAllowableAction;
import drr.standards.iosco.cde.reports.CDESpreadLeg2NotationRule;
import drr.standards.iosco.cde.reports.CDESpreadLeg2Rule;
import drr.standards.iso.PriceNotationEnum;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(SpreadOfLeg2NotationRule.SpreadOfLeg2NotationRuleDefault.class)
public abstract class SpreadOfLeg2NotationRule implements ReportFunction<TransactionReportInstruction, PriceNotationEnum> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDESpreadLeg2Rule cDESpreadLeg2;
	@Inject protected CDESpreadLeg2NotationRule cDESpreadLeg2Notation;
	@Inject protected GetNameOfTheFloatingRateOfLeg2 getNameOfTheFloatingRateOfLeg2;
	@Inject protected GetNotationEnum getNotationEnum;
	@Inject protected IsAllowableAction isAllowableAction;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public PriceNotationEnum evaluate(TransactionReportInstruction input) {
		PriceNotationEnum output = doEvaluate(input);
		
		return output;
	}

	protected abstract PriceNotationEnum doEvaluate(TransactionReportInstruction input);

	public static class SpreadOfLeg2NotationRuleDefault extends SpreadOfLeg2NotationRule {
		@Override
		protected PriceNotationEnum doEvaluate(TransactionReportInstruction input) {
			PriceNotationEnum output = null;
			return assignOutput(output, input);
		}
		
		protected PriceNotationEnum assignOutput(PriceNotationEnum output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult = MapperS.of(input)
				.filterSingleNullSafe(item -> isAllowableAction.evaluate(item.get()));
			output = thenResult
				.mapSingleToItem(item -> {
					if (exists(MapperS.of(cDESpreadLeg2.evaluate(item.get()))).getOrDefault(false)) {
						return MapperS.of(getNotationEnum.evaluate(cDESpreadLeg2Notation.evaluate(item.get())));
					}
					if (exists(MapperS.of(getNameOfTheFloatingRateOfLeg2.evaluate(item.get()))).getOrDefault(false)) {
						return MapperS.of(PriceNotationEnum.BASIS);
					}
					return MapperS.<PriceNotationEnum>ofNull();
				}).get();
			
			return output;
		}
	}
}
