package drr.regulation.esma.emir.refit.trade.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.esma.emir.refit.trade.SpreadOfLeg2;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class SpreadOfLeg2TypeTabulator implements Tabulator<SpreadOfLeg2> {
	private final Field spreadOfLeg2MonetaryField;
	private final Field spreadOfLeg2PercentageField;
	private final Field spreadOfLeg2BasisField;
	private final Field spreadCurrencyOfLeg2Field;
	private final Field spreadOfLeg2NotationField;
	
	public SpreadOfLeg2TypeTabulator() {
		this.spreadOfLeg2MonetaryField = new FieldImpl(
			"spreadOfLeg2Monetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "SpreadOfLeg2Monetary")),
			Optional.of("2.109 Spread of Leg 2 (Monetary)"),
			Arrays.asList()
		);
		this.spreadOfLeg2PercentageField = new FieldImpl(
			"spreadOfLeg2Percentage",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "SpreadOfLeg2Percentage")),
			Optional.of("2.109 Spread of Leg 2 (Percentage)"),
			Arrays.asList()
		);
		this.spreadOfLeg2BasisField = new FieldImpl(
			"spreadOfLeg2Basis",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "SpreadOfLeg2Basis")),
			Optional.of("2.109 Spread of Leg 2 (Basis)"),
			Arrays.asList()
		);
		this.spreadCurrencyOfLeg2Field = new FieldImpl(
			"spreadCurrencyOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "SpreadCurrencyOfLeg2")),
			Optional.of("2.110 Spread Currency of Leg 2"),
			Arrays.asList()
		);
		this.spreadOfLeg2NotationField = new FieldImpl(
			"spreadOfLeg2Notation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "SpreadOfLeg2Notation")),
			Optional.of("2.109 Spread of Leg 2 Notation (Non Reportable)"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(spreadOfLeg2MonetaryField, spreadOfLeg2PercentageField, spreadOfLeg2BasisField, spreadCurrencyOfLeg2Field, spreadOfLeg2NotationField);
	}
	
	@Override
	public List<FieldValue> tabulate(SpreadOfLeg2 input) {
		Optional<BigDecimal> spreadOfLeg2Monetary = Optional.ofNullable(input.getSpreadOfLeg2Monetary());
		Optional<BigDecimal> spreadOfLeg2Percentage = Optional.ofNullable(input.getSpreadOfLeg2Percentage());
		Optional<BigDecimal> spreadOfLeg2Basis = Optional.ofNullable(input.getSpreadOfLeg2Basis());
		Optional<ISOCurrencyCodeEnum> spreadCurrencyOfLeg2 = Optional.ofNullable(input.getSpreadCurrencyOfLeg2());
		Optional<PriceNotationEnum> spreadOfLeg2Notation = Optional.ofNullable(input.getSpreadOfLeg2Notation());
		return Arrays.asList(
			new FieldValueImpl(spreadOfLeg2MonetaryField, spreadOfLeg2Monetary),
			new FieldValueImpl(spreadOfLeg2PercentageField, spreadOfLeg2Percentage),
			new FieldValueImpl(spreadOfLeg2BasisField, spreadOfLeg2Basis),
			new FieldValueImpl(spreadCurrencyOfLeg2Field, spreadCurrencyOfLeg2),
			new FieldValueImpl(spreadOfLeg2NotationField, spreadOfLeg2Notation)
		);
	}
}
