package drr.regulation.esma.emir.refit.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.esma.emir.refit.trade.StrikePriceScheduleReport;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class StrikePriceScheduleReportTypeTabulator implements Tabulator<StrikePriceScheduleReport> {
	private final Field effectiveDateStrikePriceField;
	private final Field endDateStrikePriceField;
	private final Field strikePriceInEffectOnAssociatedEffectiveDateMonetaryField;
	private final Field strikePriceInEffectOnAssociatedEffectiveDatePercentageField;
	private final Field strikePriceNotationInEffectOnAssociatedEffectiveDateField;
	
	public StrikePriceScheduleReportTypeTabulator() {
		this.effectiveDateStrikePriceField = new FieldImpl(
			"effectiveDateStrikePrice",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "EffectiveDateOfTheStrikePrice")),
			Optional.of("2.135/2.137-$ 2.135 Effective date of the strike price"),
			Arrays.asList()
		);
		this.endDateStrikePriceField = new FieldImpl(
			"endDateStrikePrice",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "EndDateOfTheStrikePrice")),
			Optional.of("2.135/2.137-$ 2.136 End date of the strike price"),
			Arrays.asList()
		);
		this.strikePriceInEffectOnAssociatedEffectiveDateMonetaryField = new FieldImpl(
			"strikePriceInEffectOnAssociatedEffectiveDateMonetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "StrikePriceInEffectOnAssociatedEffectiveDateMonetary")),
			Optional.of("2.135/2.137-$ 2.137 Strike price in effect on associated effective date (Monetary)"),
			Arrays.asList()
		);
		this.strikePriceInEffectOnAssociatedEffectiveDatePercentageField = new FieldImpl(
			"strikePriceInEffectOnAssociatedEffectiveDatePercentage",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "StrikePriceInEffectOnAssociatedEffectiveDatePercentage")),
			Optional.of("2.135/2.137-$ 2.137 Strike price in effect on associated effective date (Percentage)"),
			Arrays.asList()
		);
		this.strikePriceNotationInEffectOnAssociatedEffectiveDateField = new FieldImpl(
			"strikePriceNotationInEffectOnAssociatedEffectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "esma", "emir", "refit", "trade"), "StrikePriceNotationInEffectOnAssociatedEffectiveDate")),
			Optional.of("2.135/2.137-$ 2.137 Strike price notation in effect on associated effective date (Non Reportable)"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(effectiveDateStrikePriceField, endDateStrikePriceField, strikePriceInEffectOnAssociatedEffectiveDateMonetaryField, strikePriceInEffectOnAssociatedEffectiveDatePercentageField, strikePriceNotationInEffectOnAssociatedEffectiveDateField);
	}
	
	@Override
	public List<FieldValue> tabulate(StrikePriceScheduleReport input) {
		Optional<Date> effectiveDateStrikePrice = Optional.ofNullable(input.getEffectiveDateStrikePrice());
		Optional<Date> endDateStrikePrice = Optional.ofNullable(input.getEndDateStrikePrice());
		Optional<BigDecimal> strikePriceInEffectOnAssociatedEffectiveDateMonetary = Optional.ofNullable(input.getStrikePriceInEffectOnAssociatedEffectiveDateMonetary());
		Optional<BigDecimal> strikePriceInEffectOnAssociatedEffectiveDatePercentage = Optional.ofNullable(input.getStrikePriceInEffectOnAssociatedEffectiveDatePercentage());
		Optional<PriceNotationEnum> strikePriceNotationInEffectOnAssociatedEffectiveDate = Optional.ofNullable(input.getStrikePriceNotationInEffectOnAssociatedEffectiveDate());
		return Arrays.asList(
			new FieldValueImpl(effectiveDateStrikePriceField, effectiveDateStrikePrice),
			new FieldValueImpl(endDateStrikePriceField, endDateStrikePrice),
			new FieldValueImpl(strikePriceInEffectOnAssociatedEffectiveDateMonetaryField, strikePriceInEffectOnAssociatedEffectiveDateMonetary),
			new FieldValueImpl(strikePriceInEffectOnAssociatedEffectiveDatePercentageField, strikePriceInEffectOnAssociatedEffectiveDatePercentage),
			new FieldValueImpl(strikePriceNotationInEffectOnAssociatedEffectiveDateField, strikePriceNotationInEffectOnAssociatedEffectiveDate)
		);
	}
}
