package drr.regulation.esma.emir.refit.trade.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.enrichment.common.EnrichmentData;
import drr.regulation.common.ReportableInformation;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.GetVenueOfExecution;
import drr.regulation.common.functions.IsAllowableAction;
import drr.standards.iosco.cde.functions.GetExecutionTimestamp;
import drr.standards.iso.MicData;
import drr.standards.iso.MicTypeEnum;
import drr.standards.iso.functions.GetOrFetchMicData;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(VenueOfExecutionRule.VenueOfExecutionRuleDefault.class)
public abstract class VenueOfExecutionRule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected GetExecutionTimestamp getExecutionTimestamp;
	@Inject protected GetOrFetchMicData getOrFetchMicData;
	@Inject protected GetVenueOfExecution getVenueOfExecution;
	@Inject protected IsAllowableAction isAllowableAction;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class VenueOfExecutionRuleDefault extends VenueOfExecutionRule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult = MapperS.of(input)
				.filterSingleNullSafe(item -> isAllowableAction.evaluate(item.get()));
			output = thenResult
				.mapSingleToItem(reportableEvent -> {
					final MapperS<MicData> _thenResult = MapperS.of(getVenueOfExecution.evaluate(thenResult.<ReportableInformation>map("getReportableInformation", _reportableEvent -> _reportableEvent.getReportableInformation()).get()))
						.mapSingleToItem(item -> MapperS.of(getOrFetchMicData.evaluate(reportableEvent.<ReportableInformation>map("getReportableInformation", _reportableEvent -> _reportableEvent.getReportableInformation()).<EnrichmentData>map("getEnrichment", reportableInformation -> reportableInformation.getEnrichment()).<MicData>mapC("getMicData", enrichmentData -> enrichmentData.getMicData()).getMulti(), item.get())));
					return _thenResult
						.mapSingleToItem(item -> {
							if (areEqual(item.<String>map("getStatus", micData -> micData.getStatus()), MapperS.of("EXPIRED"), CardinalityOperator.All).and(lessThanEquals(item.<Date>map("getExpiryDate", micData -> micData.getExpiryDate()), MapperS.of(getExecutionTimestamp.evaluate(reportableEvent.get())).<Date>map("Date", zdt -> Date.of(zdt.toLocalDate())), CardinalityOperator.All)).or(notEqual(item.<String>map("getStatus", micData -> micData.getStatus()), MapperS.of("EXPIRED"), CardinalityOperator.Any)).getOrDefault(false)) {
								if (areEqual(item.<MicTypeEnum>map("getMicType", micData -> micData.getMicType()), MapperS.of(MicTypeEnum.SGMT), CardinalityOperator.All).getOrDefault(false)) {
									return item.<String>map("getMic", micData -> micData.getMic());
								}
								return item.<String>map("getOperatingMic", micData -> micData.getOperatingMic());
							}
							return MapperS.<String>ofNull();
						});
				}).get();
			
			return output;
		}
	}
}
