package drr.regulation.esma.emir.refit.trade.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.esma.emir.refit.trade.NotionalQuantityScheduleLeg2Report;
import java.math.BigDecimal;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class NotionalQuantityScheduleLeg2ReportValidator implements Validator<NotionalQuantityScheduleLeg2Report> {

	private List<ComparisonResult> getComparisonResults(NotionalQuantityScheduleLeg2Report o) {
		return Lists.<ComparisonResult>newArrayList(
				checkCardinality("effectiveDate", (Date) o.getEffectiveDate() != null ? 1 : 0, 0, 1), 
				checkCardinality("endDate", (Date) o.getEndDate() != null ? 1 : 0, 0, 1), 
				checkCardinality("value", (BigDecimal) o.getValue() != null ? 1 : 0, 0, 1)
			);
	}

	@Override
	public ValidationResult<NotionalQuantityScheduleLeg2Report> validate(RosettaPath path, NotionalQuantityScheduleLeg2Report o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("NotionalQuantityScheduleLeg2Report", ValidationType.CARDINALITY, "NotionalQuantityScheduleLeg2Report", path, "", error);
		}
		return success("NotionalQuantityScheduleLeg2Report", ValidationType.CARDINALITY, "NotionalQuantityScheduleLeg2Report", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, NotionalQuantityScheduleLeg2Report o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("NotionalQuantityScheduleLeg2Report", ValidationType.CARDINALITY, "NotionalQuantityScheduleLeg2Report", path, "", res.getError());
				}
				return success("NotionalQuantityScheduleLeg2Report", ValidationType.CARDINALITY, "NotionalQuantityScheduleLeg2Report", path, "");
			})
			.collect(toList());
	}

}
