package drr.regulation.esma.emir.refit.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRTransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ESMAEMIRTransactionReportEMIR_VR_2001_04")
@ImplementedBy(ESMAEMIRTransactionReportEMIRVR200104.Default.class)
public interface ESMAEMIRTransactionReportEMIRVR200104 extends Validator<ESMAEMIRTransactionReport> {
	
	String NAME = "ESMAEMIRTransactionReportEMIR_VR_2001_04";
	String DEFINITION = "if uti exists then [reportTrackingNumber, subsequentPositionUti, priorUti, priorUtiProprietary] all <> uti else [reportTrackingNumber, subsequentPositionUti, priorUti, priorUtiProprietary] all <> utiProprietary";
	
	ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport);
	
	class Default implements ESMAEMIRTransactionReportEMIRVR200104 {
	
		@Override
		public ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			ComparisonResult result = executeDataRule(eSMAEMIRTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			try {
				if (exists(MapperS.of(eSMAEMIRTransactionReport).<String>map("getUti", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getUti())).getOrDefault(false)) {
					return notEqual(MapperC.<String>of(MapperS.of(eSMAEMIRTransactionReport).<String>map("getReportTrackingNumber", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getReportTrackingNumber()), MapperS.of(eSMAEMIRTransactionReport).<String>map("getSubsequentPositionUti", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getSubsequentPositionUti()), MapperS.of(eSMAEMIRTransactionReport).<String>map("getPriorUti", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getPriorUti()), MapperS.of(eSMAEMIRTransactionReport).<String>map("getPriorUtiProprietary", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getPriorUtiProprietary())), MapperS.of(eSMAEMIRTransactionReport).<String>map("getUti", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getUti()), CardinalityOperator.All);
				}
				return notEqual(MapperC.<String>of(MapperS.of(eSMAEMIRTransactionReport).<String>map("getReportTrackingNumber", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getReportTrackingNumber()), MapperS.of(eSMAEMIRTransactionReport).<String>map("getSubsequentPositionUti", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getSubsequentPositionUti()), MapperS.of(eSMAEMIRTransactionReport).<String>map("getPriorUti", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getPriorUti()), MapperS.of(eSMAEMIRTransactionReport).<String>map("getPriorUtiProprietary", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getPriorUtiProprietary())), MapperS.of(eSMAEMIRTransactionReport).<String>map("getUtiProprietary", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getUtiProprietary()), CardinalityOperator.All);
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ESMAEMIRTransactionReportEMIRVR200104 {
	
		@Override
		public ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION);
		}
	}
}
