package drr.regulation.esma.emir.refit.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.functions.IsVRAllowedForActionType;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRTransactionReport;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ESMAEMIRTransactionReportEMIR_VR_2002_02")
@ImplementedBy(ESMAEMIRTransactionReportEMIRVR200202.Default.class)
public interface ESMAEMIRTransactionReportEMIRVR200202 extends Validator<ESMAEMIRTransactionReport> {
	
	String NAME = "ESMAEMIRTransactionReportEMIR_VR_2002_02";
	String DEFINITION = "if IsVRAllowedForActionType(actionType) then if reportTrackingNumber exists then [uti, utiProprietary, subsequentPositionUti, priorUti, priorUtiProprietary] all <> reportTrackingNumber";
	
	ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport);
	
	class Default implements ESMAEMIRTransactionReportEMIRVR200202 {
	
		@Inject protected IsVRAllowedForActionType isVRAllowedForActionType;
		
		@Override
		public ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			ComparisonResult result = executeDataRule(eSMAEMIRTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			try {
				final Boolean _boolean = isVRAllowedForActionType.evaluate(MapperS.of(eSMAEMIRTransactionReport).<String>map("getActionType", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getActionType()).get());
				if ((_boolean == null ? false : _boolean)) {
					if (exists(MapperS.of(eSMAEMIRTransactionReport).<String>map("getReportTrackingNumber", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getReportTrackingNumber())).getOrDefault(false)) {
						return notEqual(MapperC.<String>of(MapperS.of(eSMAEMIRTransactionReport).<String>map("getUti", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getUti()), MapperS.of(eSMAEMIRTransactionReport).<String>map("getUtiProprietary", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getUtiProprietary()), MapperS.of(eSMAEMIRTransactionReport).<String>map("getSubsequentPositionUti", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getSubsequentPositionUti()), MapperS.of(eSMAEMIRTransactionReport).<String>map("getPriorUti", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getPriorUti()), MapperS.of(eSMAEMIRTransactionReport).<String>map("getPriorUtiProprietary", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getPriorUtiProprietary())), MapperS.of(eSMAEMIRTransactionReport).<String>map("getReportTrackingNumber", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getReportTrackingNumber()), CardinalityOperator.All);
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ESMAEMIRTransactionReportEMIRVR200202 {
	
		@Override
		public ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION);
		}
	}
}
