package drr.regulation.esma.emir.refit.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.functions.IsVRAllowedForActionType;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRTransactionReport;
import iso20022.auth030.esma.InterestComputationMethod4Code;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ESMAEMIRTransactionReportEMIR_VR_2086_01")
@ImplementedBy(ESMAEMIRTransactionReportEMIRVR208601.Default.class)
public interface ESMAEMIRTransactionReportEMIRVR208601 extends Validator<ESMAEMIRTransactionReport> {
	
	String NAME = "ESMAEMIRTransactionReportEMIR_VR_2086_01";
	String DEFINITION = "if IsVRAllowedForActionType(actionType) then if nameOfTheFloatingRateOfLeg1 exists then floatingRateDayCountConventionOfLeg1 exists and floatingRateDayCountConventionOfLeg1 exists else floatingRateDayCountConventionOfLeg1 is absent";
	
	ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport);
	
	class Default implements ESMAEMIRTransactionReportEMIRVR208601 {
	
		@Inject protected IsVRAllowedForActionType isVRAllowedForActionType;
		
		@Override
		public ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			ComparisonResult result = executeDataRule(eSMAEMIRTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			try {
				final Boolean _boolean = isVRAllowedForActionType.evaluate(MapperS.of(eSMAEMIRTransactionReport).<String>map("getActionType", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getActionType()).get());
				if ((_boolean == null ? false : _boolean)) {
					if (exists(MapperS.of(eSMAEMIRTransactionReport).<String>map("getNameOfTheFloatingRateOfLeg1", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getNameOfTheFloatingRateOfLeg1())).getOrDefault(false)) {
						return exists(MapperS.of(eSMAEMIRTransactionReport).<InterestComputationMethod4Code>map("getFloatingRateDayCountConventionOfLeg1", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRateDayCountConventionOfLeg1())).and(exists(MapperS.of(eSMAEMIRTransactionReport).<InterestComputationMethod4Code>map("getFloatingRateDayCountConventionOfLeg1", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRateDayCountConventionOfLeg1())));
					}
					return notExists(MapperS.of(eSMAEMIRTransactionReport).<InterestComputationMethod4Code>map("getFloatingRateDayCountConventionOfLeg1", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRateDayCountConventionOfLeg1()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ESMAEMIRTransactionReportEMIRVR208601 {
	
		@Override
		public ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION);
		}
	}
}
