package drr.regulation.esma.emir.refit.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.functions.IsVRAllowedForActionType;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRTransactionReport;
import iso20022.auth030.esma.Frequency13Code__1;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ESMAEMIRTransactionReportEMIR_VR_2087_01")
@ImplementedBy(ESMAEMIRTransactionReportEMIRVR208701.Default.class)
public interface ESMAEMIRTransactionReportEMIRVR208701 extends Validator<ESMAEMIRTransactionReport> {
	
	String NAME = "ESMAEMIRTransactionReportEMIR_VR_2087_01";
	String DEFINITION = "if IsVRAllowedForActionType(actionType) then if nameOfTheFloatingRateOfLeg1 exists then floatingRatePaymentFrequencyPeriodOfLeg1 exists and [Frequency13Code__1 -> YEAR, Frequency13Code__1 -> MNTH, Frequency13Code__1 -> WEEK, Frequency13Code__1 -> DAIL, Frequency13Code__1 -> EXPI, Frequency13Code__1 -> ADHO] any = floatingRatePaymentFrequencyPeriodOfLeg1 else floatingRatePaymentFrequencyPeriodOfLeg1 is absent";
	
	ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport);
	
	class Default implements ESMAEMIRTransactionReportEMIRVR208701 {
	
		@Inject protected IsVRAllowedForActionType isVRAllowedForActionType;
		
		@Override
		public ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			ComparisonResult result = executeDataRule(eSMAEMIRTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			try {
				final Boolean _boolean = isVRAllowedForActionType.evaluate(MapperS.of(eSMAEMIRTransactionReport).<String>map("getActionType", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getActionType()).get());
				if ((_boolean == null ? false : _boolean)) {
					if (exists(MapperS.of(eSMAEMIRTransactionReport).<String>map("getNameOfTheFloatingRateOfLeg1", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getNameOfTheFloatingRateOfLeg1())).getOrDefault(false)) {
						return exists(MapperS.of(eSMAEMIRTransactionReport).<Frequency13Code__1>map("getFloatingRatePaymentFrequencyPeriodOfLeg1", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRatePaymentFrequencyPeriodOfLeg1())).and(areEqual(MapperC.<Frequency13Code__1>of(MapperS.of(Frequency13Code__1.YEAR), MapperS.of(Frequency13Code__1.MNTH), MapperS.of(Frequency13Code__1.WEEK), MapperS.of(Frequency13Code__1.DAIL), MapperS.of(Frequency13Code__1.EXPI), MapperS.of(Frequency13Code__1.ADHO)), MapperS.of(eSMAEMIRTransactionReport).<Frequency13Code__1>map("getFloatingRatePaymentFrequencyPeriodOfLeg1", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRatePaymentFrequencyPeriodOfLeg1()), CardinalityOperator.Any));
					}
					return notExists(MapperS.of(eSMAEMIRTransactionReport).<Frequency13Code__1>map("getFloatingRatePaymentFrequencyPeriodOfLeg1", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRatePaymentFrequencyPeriodOfLeg1()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ESMAEMIRTransactionReportEMIRVR208701 {
	
		@Override
		public ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION);
		}
	}
}
