package drr.regulation.esma.emir.refit.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.functions.IsVRAllowedForActionType;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRTransactionReport;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ESMAEMIRTransactionReportEMIR_VR_2106_01")
@ImplementedBy(ESMAEMIRTransactionReportEMIRVR210601.Default.class)
public interface ESMAEMIRTransactionReportEMIRVR210601 extends Validator<ESMAEMIRTransactionReport> {
	
	String NAME = "ESMAEMIRTransactionReportEMIR_VR_2106_01";
	String DEFINITION = "if IsVRAllowedForActionType(actionType) then if floatingRateReferencePeriodOfLeg2TimePeriod exists then floatingRateReferencePeriodOfLeg2Multiplier exists and floatingRateReferencePeriodOfLeg2Multiplier >= 0 and floatingRateReferencePeriodOfLeg2Multiplier < 1000 else floatingRateReferencePeriodOfLeg2Multiplier is absent";
	
	ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport);
	
	class Default implements ESMAEMIRTransactionReportEMIRVR210601 {
	
		@Inject protected IsVRAllowedForActionType isVRAllowedForActionType;
		
		@Override
		public ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			ComparisonResult result = executeDataRule(eSMAEMIRTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			try {
				final Boolean _boolean = isVRAllowedForActionType.evaluate(MapperS.of(eSMAEMIRTransactionReport).<String>map("getActionType", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getActionType()).get());
				if ((_boolean == null ? false : _boolean)) {
					if (exists(MapperS.of(eSMAEMIRTransactionReport).<String>map("getFloatingRateReferencePeriodOfLeg2TimePeriod", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRateReferencePeriodOfLeg2TimePeriod())).getOrDefault(false)) {
						return exists(MapperS.of(eSMAEMIRTransactionReport).<Integer>map("getFloatingRateReferencePeriodOfLeg2Multiplier", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRateReferencePeriodOfLeg2Multiplier())).and(greaterThanEquals(MapperS.of(eSMAEMIRTransactionReport).<Integer>map("getFloatingRateReferencePeriodOfLeg2Multiplier", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRateReferencePeriodOfLeg2Multiplier()), MapperS.of(0), CardinalityOperator.All)).and(lessThan(MapperS.of(eSMAEMIRTransactionReport).<Integer>map("getFloatingRateReferencePeriodOfLeg2Multiplier", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRateReferencePeriodOfLeg2Multiplier()), MapperS.of(1000), CardinalityOperator.All));
					}
					return notExists(MapperS.of(eSMAEMIRTransactionReport).<Integer>map("getFloatingRateReferencePeriodOfLeg2Multiplier", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRateReferencePeriodOfLeg2Multiplier()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ESMAEMIRTransactionReportEMIRVR210601 {
	
		@Override
		public ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION);
		}
	}
}
