package drr.regulation.esma.emir.refit.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.functions.IsVRAllowedForActionType;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRTransactionReport;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ESMAEMIRTransactionReportEMIR_VR_2108_01")
@ImplementedBy(ESMAEMIRTransactionReportEMIRVR210801.Default.class)
public interface ESMAEMIRTransactionReportEMIRVR210801 extends Validator<ESMAEMIRTransactionReport> {
	
	String NAME = "ESMAEMIRTransactionReportEMIR_VR_2108_01";
	String DEFINITION = "if IsVRAllowedForActionType(actionType) then if [\"YEAR\", \"MNTH\", \"WEEK\", \"DAIL\", \"EXPI\"] any = floatingRateResetFrequencyPeriodOfLeg2 then floatingRateResetFrequencyPeriodMultiplierLeg2 exists and 0 <= floatingRateResetFrequencyPeriodMultiplierLeg2 and floatingRateResetFrequencyPeriodMultiplierLeg2 < 1000 else floatingRateResetFrequencyPeriodMultiplierLeg2 is absent";
	
	ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport);
	
	class Default implements ESMAEMIRTransactionReportEMIRVR210801 {
	
		@Inject protected IsVRAllowedForActionType isVRAllowedForActionType;
		
		@Override
		public ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			ComparisonResult result = executeDataRule(eSMAEMIRTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			try {
				final Boolean _boolean = isVRAllowedForActionType.evaluate(MapperS.of(eSMAEMIRTransactionReport).<String>map("getActionType", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getActionType()).get());
				if ((_boolean == null ? false : _boolean)) {
					if (areEqual(MapperC.<String>of(MapperS.of("YEAR"), MapperS.of("MNTH"), MapperS.of("WEEK"), MapperS.of("DAIL"), MapperS.of("EXPI")), MapperS.of(eSMAEMIRTransactionReport).<String>map("getFloatingRateResetFrequencyPeriodOfLeg2", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRateResetFrequencyPeriodOfLeg2()), CardinalityOperator.Any).getOrDefault(false)) {
						return exists(MapperS.of(eSMAEMIRTransactionReport).<Integer>map("getFloatingRateResetFrequencyPeriodMultiplierLeg2", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRateResetFrequencyPeriodMultiplierLeg2())).and(lessThanEquals(MapperS.of(0), MapperS.of(eSMAEMIRTransactionReport).<Integer>map("getFloatingRateResetFrequencyPeriodMultiplierLeg2", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRateResetFrequencyPeriodMultiplierLeg2()), CardinalityOperator.All)).and(lessThan(MapperS.of(eSMAEMIRTransactionReport).<Integer>map("getFloatingRateResetFrequencyPeriodMultiplierLeg2", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRateResetFrequencyPeriodMultiplierLeg2()), MapperS.of(1000), CardinalityOperator.All));
					}
					return notExists(MapperS.of(eSMAEMIRTransactionReport).<Integer>map("getFloatingRateResetFrequencyPeriodMultiplierLeg2", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getFloatingRateResetFrequencyPeriodMultiplierLeg2()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ESMAEMIRTransactionReportEMIRVR210801 {
	
		@Override
		public ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION);
		}
	}
}
