package drr.regulation.esma.emir.refit.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.functions.IsVRAllowedForActionType;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRTransactionReport;
import java.math.BigDecimal;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("ESMAEMIRTransactionReportEMIR_VR_2150_01")
@ImplementedBy(ESMAEMIRTransactionReportEMIRVR215001.Default.class)
public interface ESMAEMIRTransactionReportEMIRVR215001 extends Validator<ESMAEMIRTransactionReport> {
	
	String NAME = "ESMAEMIRTransactionReportEMIR_VR_2150_01";
	String DEFINITION = "if IsVRAllowedForActionType(actionType) then if tranche = True then cdSIndexDetachmentPoint exists and cdSIndexDetachmentPoint >= 0 and cdSIndexDetachmentPoint <= 1 else if tranche = False then cdSIndexDetachmentPoint is absent";
	
	ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport);
	
	class Default implements ESMAEMIRTransactionReportEMIRVR215001 {
	
		@Inject protected IsVRAllowedForActionType isVRAllowedForActionType;
		
		@Override
		public ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			ComparisonResult result = executeDataRule(eSMAEMIRTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			try {
				final Boolean _boolean = isVRAllowedForActionType.evaluate(MapperS.of(eSMAEMIRTransactionReport).<String>map("getActionType", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getActionType()).get());
				if ((_boolean == null ? false : _boolean)) {
					if (areEqual(MapperS.of(eSMAEMIRTransactionReport).<Boolean>map("getTranche", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getTranche()), MapperS.of(true), CardinalityOperator.All).getOrDefault(false)) {
						return exists(MapperS.of(eSMAEMIRTransactionReport).<BigDecimal>map("getCdSIndexDetachmentPoint", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getCdSIndexDetachmentPoint())).and(greaterThanEquals(MapperS.of(eSMAEMIRTransactionReport).<BigDecimal>map("getCdSIndexDetachmentPoint", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getCdSIndexDetachmentPoint()), MapperS.of(BigDecimal.valueOf(0)), CardinalityOperator.All)).and(lessThanEquals(MapperS.of(eSMAEMIRTransactionReport).<BigDecimal>map("getCdSIndexDetachmentPoint", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getCdSIndexDetachmentPoint()), MapperS.of(BigDecimal.valueOf(1)), CardinalityOperator.All));
					}
					if (areEqual(MapperS.of(eSMAEMIRTransactionReport).<Boolean>map("getTranche", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getTranche()), MapperS.of(false), CardinalityOperator.All).getOrDefault(false)) {
						return notExists(MapperS.of(eSMAEMIRTransactionReport).<BigDecimal>map("getCdSIndexDetachmentPoint", _eSMAEMIRTransactionReport -> _eSMAEMIRTransactionReport.getCdSIndexDetachmentPoint()));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements ESMAEMIRTransactionReportEMIRVR215001 {
	
		@Override
		public ValidationResult<ESMAEMIRTransactionReport> validate(RosettaPath path, ESMAEMIRTransactionReport eSMAEMIRTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "ESMAEMIRTransactionReport", path, DEFINITION);
		}
	}
}
