package drr.regulation.esma.emir.refit.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.esma.emir.refit.trade.PackageTransactionSpread;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("PackageTransactionSpreadPercentage")
@ImplementedBy(PackageTransactionSpreadPercentage.Default.class)
public interface PackageTransactionSpreadPercentage extends Validator<PackageTransactionSpread> {
	
	String NAME = "PackageTransactionSpreadPercentage";
	String DEFINITION = "if packageTransactionSpreadNotation = PriceNotationEnum -> Percentage then packageTransactionSpreadPercentage exists";
	
	ValidationResult<PackageTransactionSpread> validate(RosettaPath path, PackageTransactionSpread packageTransactionSpread);
	
	class Default implements PackageTransactionSpreadPercentage {
	
		@Override
		public ValidationResult<PackageTransactionSpread> validate(RosettaPath path, PackageTransactionSpread packageTransactionSpread) {
			ComparisonResult result = executeDataRule(packageTransactionSpread);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PackageTransactionSpread", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "PackageTransactionSpread", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(PackageTransactionSpread packageTransactionSpread) {
			try {
				if (areEqual(MapperS.of(packageTransactionSpread).<PriceNotationEnum>map("getPackageTransactionSpreadNotation", _packageTransactionSpread -> _packageTransactionSpread.getPackageTransactionSpreadNotation()), MapperS.of(PriceNotationEnum.PERCENTAGE), CardinalityOperator.All).getOrDefault(false)) {
					return exists(MapperS.of(packageTransactionSpread).<BigDecimal>map("getPackageTransactionSpreadPercentage", _packageTransactionSpread -> _packageTransactionSpread.getPackageTransactionSpreadPercentage()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements PackageTransactionSpreadPercentage {
	
		@Override
		public ValidationResult<PackageTransactionSpread> validate(RosettaPath path, PackageTransactionSpread packageTransactionSpread) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PackageTransactionSpread", path, DEFINITION);
		}
	}
}
