package drr.regulation.fca.ukemir.refit.margin.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.common.MarginActionEnum;
import drr.regulation.fca.ukemir.refit.margin.CollateralReport;
import iso20022.auth108.esma.CollateralisationType3Code__1;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class CollateralReportTypeTabulator implements Tabulator<CollateralReport> {
	private final Field collateralTimestampField;
	private final Field collateralPortfolioCodeField;
	private final Field collateralPortfolioIndicatorField;
	private final Field utiField;
	private final Field collateralisationCategoryField;
	private final Field initialMarginPostedByTheCounterparty1PreHaircutField;
	private final Field initialMarginPostedByTheCounterparty1PostHaircutField;
	private final Field initialMarginPostedByCounterparty1CurrencyField;
	private final Field variationMarginPostedByTheCounterparty1PreHaircutField;
	private final Field variationMarginPostedByTheCounterparty1PostHaircutField;
	private final Field variationMarginPostedByCounterparty1CurrencyField;
	private final Field initialMarginCollectedByCounterparty1PreHaircutField;
	private final Field initialMarginCollectedByCounterparty1PostHaircutField;
	private final Field initialMarginCollectedByCounterparty1CurrencyField;
	private final Field variationMarginCollectedByTheCounterparty1PreHaircutField;
	private final Field variationMarginCollectedByTheCounterparty1PostHaircutField;
	private final Field variationMarginCollectedByCounterparty1CurrencyField;
	private final Field excessCollateralPostedByTheCounterparty1Field;
	private final Field currencyOfTheExcessCollateralPostedField;
	private final Field excessCollateralCollectedByTheCounterparty1Field;
	private final Field currencyOfTheExcessCollateralCollectedField;
	private final Field actionTypeField;
	private final Field eventDateField;
	
	public CollateralReportTypeTabulator() {
		this.collateralTimestampField = new FieldImpl(
			"collateralTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "CollateralTimestamp")),
			Optional.of("3.7 Collateral timestamp"),
			Arrays.asList()
		);
		this.collateralPortfolioCodeField = new FieldImpl(
			"collateralPortfolioCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "CollateralPortfolioCode")),
			Optional.of("3.9 Collateral portfolio code"),
			Arrays.asList()
		);
		this.collateralPortfolioIndicatorField = new FieldImpl(
			"collateralPortfolioIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "CollateralPortfolioIndicator")),
			Optional.of("3.8 Collateral portfolio indicator"),
			Arrays.asList()
		);
		this.utiField = new FieldImpl(
			"uti",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "UTI")),
			Optional.of("3.10 UTI"),
			Arrays.asList()
		);
		this.collateralisationCategoryField = new FieldImpl(
			"collateralisationCategory",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "CollateralisationCategory")),
			Optional.of("3.11 Collateralisation category"),
			Arrays.asList()
		);
		this.initialMarginPostedByTheCounterparty1PreHaircutField = new FieldImpl(
			"initialMarginPostedByTheCounterparty1PreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "InitialMarginPostedByTheCounterparty1PreHaircut")),
			Optional.of("3.12 Initial margin posted by the counterparty 1 (pre-haircut)"),
			Arrays.asList()
		);
		this.initialMarginPostedByTheCounterparty1PostHaircutField = new FieldImpl(
			"initialMarginPostedByTheCounterparty1PostHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "InitialMarginPostedByTheCounterparty1PostHaircut")),
			Optional.of("3.13 Initial margin posted by the counterparty 1 (post-haircut)"),
			Arrays.asList()
		);
		this.initialMarginPostedByCounterparty1CurrencyField = new FieldImpl(
			"initialMarginPostedByCounterparty1Currency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "InitialMarginPostedByCounterparty1Currency")),
			Optional.of("3.14 Currency of the initial margins posted"),
			Arrays.asList()
		);
		this.variationMarginPostedByTheCounterparty1PreHaircutField = new FieldImpl(
			"variationMarginPostedByTheCounterparty1PreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "VariationMarginPostedByTheCounterparty1PreHaircut")),
			Optional.of("3.15 Variation margin posted by the counterparty 1 (pre-haircut)"),
			Arrays.asList()
		);
		this.variationMarginPostedByTheCounterparty1PostHaircutField = new FieldImpl(
			"variationMarginPostedByTheCounterparty1PostHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "VariationMarginPostedByTheCounterparty1PostHaircut")),
			Optional.of("3.16 Variation margin posted by the counterparty 1 (post-haircut)"),
			Arrays.asList()
		);
		this.variationMarginPostedByCounterparty1CurrencyField = new FieldImpl(
			"variationMarginPostedByCounterparty1Currency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "VariationMarginPostedByCounterparty1Currency")),
			Optional.of("3.17 Currency of the variation margins posted"),
			Arrays.asList()
		);
		this.initialMarginCollectedByCounterparty1PreHaircutField = new FieldImpl(
			"initialMarginCollectedByCounterparty1PreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "InitialMarginCollectedByCounterparty1PreHaircut")),
			Optional.of("3.20 Initial margin collected by the counterparty 1 (pre-haircut)"),
			Arrays.asList()
		);
		this.initialMarginCollectedByCounterparty1PostHaircutField = new FieldImpl(
			"initialMarginCollectedByCounterparty1PostHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "InitialMarginCollectedByCounterparty1PostHaircut")),
			Optional.of("3.21 Initial margin collected by the counterparty 1 (post-haircut)"),
			Arrays.asList()
		);
		this.initialMarginCollectedByCounterparty1CurrencyField = new FieldImpl(
			"initialMarginCollectedByCounterparty1Currency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "InitialMarginCollectedByCounterparty1Currency")),
			Optional.of("3.22 Currency of initial margin collected"),
			Arrays.asList()
		);
		this.variationMarginCollectedByTheCounterparty1PreHaircutField = new FieldImpl(
			"variationMarginCollectedByTheCounterparty1PreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "VariationMarginCollectedByTheCounterparty1PreHaircut")),
			Optional.of("3.23 Variation margin collected by the counterparty 1 (pre-haircut)"),
			Arrays.asList()
		);
		this.variationMarginCollectedByTheCounterparty1PostHaircutField = new FieldImpl(
			"variationMarginCollectedByTheCounterparty1PostHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "VariationMarginCollectedByTheCounterparty1PostHaircut")),
			Optional.of("3.24 Variation margin collected by the counterparty 1 (post-haircut)"),
			Arrays.asList()
		);
		this.variationMarginCollectedByCounterparty1CurrencyField = new FieldImpl(
			"variationMarginCollectedByCounterparty1Currency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "VariationMarginCollectedByCounterparty1Currency")),
			Optional.of("3.25 Currency of variation margin collected"),
			Arrays.asList()
		);
		this.excessCollateralPostedByTheCounterparty1Field = new FieldImpl(
			"excessCollateralPostedByTheCounterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "ExcessCollateralPostedByTheCounterparty1")),
			Optional.of("3.18 Excess collateral posted by the counterparty 1"),
			Arrays.asList()
		);
		this.currencyOfTheExcessCollateralPostedField = new FieldImpl(
			"currencyOfTheExcessCollateralPosted",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "CurrencyOfTheExcessCollateralPosted")),
			Optional.of("3.19 Currency of the excess collateral posted"),
			Arrays.asList()
		);
		this.excessCollateralCollectedByTheCounterparty1Field = new FieldImpl(
			"excessCollateralCollectedByTheCounterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "ExcessCollateralCollectedByTheCounterparty1")),
			Optional.of("3.26 Excess collateral collected by the counterparty 1"),
			Arrays.asList()
		);
		this.currencyOfTheExcessCollateralCollectedField = new FieldImpl(
			"currencyOfTheExcessCollateralCollected",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "CurrencyOfExcessCollateralCollected")),
			Optional.of("3.27 Currency of excess collateral collected"),
			Arrays.asList()
		);
		this.actionTypeField = new FieldImpl(
			"actionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "ActionType")),
			Optional.of("3.28 Action Type"),
			Arrays.asList()
		);
		this.eventDateField = new FieldImpl(
			"eventDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "EventDate")),
			Optional.of("3.29 Event date"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(collateralTimestampField, collateralPortfolioCodeField, collateralPortfolioIndicatorField, utiField, collateralisationCategoryField, initialMarginPostedByTheCounterparty1PreHaircutField, initialMarginPostedByTheCounterparty1PostHaircutField, initialMarginPostedByCounterparty1CurrencyField, variationMarginPostedByTheCounterparty1PreHaircutField, variationMarginPostedByTheCounterparty1PostHaircutField, variationMarginPostedByCounterparty1CurrencyField, initialMarginCollectedByCounterparty1PreHaircutField, initialMarginCollectedByCounterparty1PostHaircutField, initialMarginCollectedByCounterparty1CurrencyField, variationMarginCollectedByTheCounterparty1PreHaircutField, variationMarginCollectedByTheCounterparty1PostHaircutField, variationMarginCollectedByCounterparty1CurrencyField, excessCollateralPostedByTheCounterparty1Field, currencyOfTheExcessCollateralPostedField, excessCollateralCollectedByTheCounterparty1Field, currencyOfTheExcessCollateralCollectedField, actionTypeField, eventDateField);
	}
	
	@Override
	public List<FieldValue> tabulate(CollateralReport input) {
		Optional<ZonedDateTime> collateralTimestamp = Optional.ofNullable(input.getCollateralTimestamp());
		Optional<String> collateralPortfolioCode = Optional.ofNullable(input.getCollateralPortfolioCode());
		Optional<Boolean> collateralPortfolioIndicator = Optional.ofNullable(input.getCollateralPortfolioIndicator());
		Optional<String> uti = Optional.ofNullable(input.getUti());
		Optional<CollateralisationType3Code__1> collateralisationCategory = Optional.ofNullable(input.getCollateralisationCategory());
		Optional<BigDecimal> initialMarginPostedByTheCounterparty1PreHaircut = Optional.ofNullable(input.getInitialMarginPostedByTheCounterparty1PreHaircut());
		Optional<BigDecimal> initialMarginPostedByTheCounterparty1PostHaircut = Optional.ofNullable(input.getInitialMarginPostedByTheCounterparty1PostHaircut());
		Optional<ISOCurrencyCodeEnum> initialMarginPostedByCounterparty1Currency = Optional.ofNullable(input.getInitialMarginPostedByCounterparty1Currency());
		Optional<BigDecimal> variationMarginPostedByTheCounterparty1PreHaircut = Optional.ofNullable(input.getVariationMarginPostedByTheCounterparty1PreHaircut());
		Optional<BigDecimal> variationMarginPostedByTheCounterparty1PostHaircut = Optional.ofNullable(input.getVariationMarginPostedByTheCounterparty1PostHaircut());
		Optional<ISOCurrencyCodeEnum> variationMarginPostedByCounterparty1Currency = Optional.ofNullable(input.getVariationMarginPostedByCounterparty1Currency());
		Optional<BigDecimal> initialMarginCollectedByCounterparty1PreHaircut = Optional.ofNullable(input.getInitialMarginCollectedByCounterparty1PreHaircut());
		Optional<BigDecimal> initialMarginCollectedByCounterparty1PostHaircut = Optional.ofNullable(input.getInitialMarginCollectedByCounterparty1PostHaircut());
		Optional<ISOCurrencyCodeEnum> initialMarginCollectedByCounterparty1Currency = Optional.ofNullable(input.getInitialMarginCollectedByCounterparty1Currency());
		Optional<BigDecimal> variationMarginCollectedByTheCounterparty1PreHaircut = Optional.ofNullable(input.getVariationMarginCollectedByTheCounterparty1PreHaircut());
		Optional<BigDecimal> variationMarginCollectedByTheCounterparty1PostHaircut = Optional.ofNullable(input.getVariationMarginCollectedByTheCounterparty1PostHaircut());
		Optional<ISOCurrencyCodeEnum> variationMarginCollectedByCounterparty1Currency = Optional.ofNullable(input.getVariationMarginCollectedByCounterparty1Currency());
		Optional<BigDecimal> excessCollateralPostedByTheCounterparty1 = Optional.ofNullable(input.getExcessCollateralPostedByTheCounterparty1());
		Optional<String> currencyOfTheExcessCollateralPosted = Optional.ofNullable(input.getCurrencyOfTheExcessCollateralPosted());
		Optional<BigDecimal> excessCollateralCollectedByTheCounterparty1 = Optional.ofNullable(input.getExcessCollateralCollectedByTheCounterparty1());
		Optional<String> currencyOfTheExcessCollateralCollected = Optional.ofNullable(input.getCurrencyOfTheExcessCollateralCollected());
		Optional<MarginActionEnum> actionType = Optional.ofNullable(input.getActionType());
		Optional<Date> eventDate = Optional.ofNullable(input.getEventDate());
		return Arrays.asList(
			new FieldValueImpl(collateralTimestampField, collateralTimestamp),
			new FieldValueImpl(collateralPortfolioCodeField, collateralPortfolioCode),
			new FieldValueImpl(collateralPortfolioIndicatorField, collateralPortfolioIndicator),
			new FieldValueImpl(utiField, uti),
			new FieldValueImpl(collateralisationCategoryField, collateralisationCategory),
			new FieldValueImpl(initialMarginPostedByTheCounterparty1PreHaircutField, initialMarginPostedByTheCounterparty1PreHaircut),
			new FieldValueImpl(initialMarginPostedByTheCounterparty1PostHaircutField, initialMarginPostedByTheCounterparty1PostHaircut),
			new FieldValueImpl(initialMarginPostedByCounterparty1CurrencyField, initialMarginPostedByCounterparty1Currency),
			new FieldValueImpl(variationMarginPostedByTheCounterparty1PreHaircutField, variationMarginPostedByTheCounterparty1PreHaircut),
			new FieldValueImpl(variationMarginPostedByTheCounterparty1PostHaircutField, variationMarginPostedByTheCounterparty1PostHaircut),
			new FieldValueImpl(variationMarginPostedByCounterparty1CurrencyField, variationMarginPostedByCounterparty1Currency),
			new FieldValueImpl(initialMarginCollectedByCounterparty1PreHaircutField, initialMarginCollectedByCounterparty1PreHaircut),
			new FieldValueImpl(initialMarginCollectedByCounterparty1PostHaircutField, initialMarginCollectedByCounterparty1PostHaircut),
			new FieldValueImpl(initialMarginCollectedByCounterparty1CurrencyField, initialMarginCollectedByCounterparty1Currency),
			new FieldValueImpl(variationMarginCollectedByTheCounterparty1PreHaircutField, variationMarginCollectedByTheCounterparty1PreHaircut),
			new FieldValueImpl(variationMarginCollectedByTheCounterparty1PostHaircutField, variationMarginCollectedByTheCounterparty1PostHaircut),
			new FieldValueImpl(variationMarginCollectedByCounterparty1CurrencyField, variationMarginCollectedByCounterparty1Currency),
			new FieldValueImpl(excessCollateralPostedByTheCounterparty1Field, excessCollateralPostedByTheCounterparty1),
			new FieldValueImpl(currencyOfTheExcessCollateralPostedField, currencyOfTheExcessCollateralPosted),
			new FieldValueImpl(excessCollateralCollectedByTheCounterparty1Field, excessCollateralCollectedByTheCounterparty1),
			new FieldValueImpl(currencyOfTheExcessCollateralCollectedField, currencyOfTheExcessCollateralCollected),
			new FieldValueImpl(actionTypeField, actionType),
			new FieldValueImpl(eventDateField, eventDate)
		);
	}
}
