package drr.regulation.fca.ukemir.refit.margin.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.fca.ukemir.refit.margin.PartiesToTheDerivative;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class PartiesToTheDerivativeTypeTabulator implements Tabulator<PartiesToTheDerivative> {
	private final Field reportSubmittingEntityIDField;
	private final Field counterparty2IdentifierTypeField;
	
	public PartiesToTheDerivativeTypeTabulator() {
		this.reportSubmittingEntityIDField = new FieldImpl(
			"reportSubmittingEntityID",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "ReportSubmittingEntityID")),
			Optional.of("3.2 Report Submitting entity ID"),
			Arrays.asList()
		);
		this.counterparty2IdentifierTypeField = new FieldImpl(
			"counterparty2IdentifierType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "margin"), "Counterparty2IdentifierType")),
			Optional.of("3.5 Counterparty 2 identifier type"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(reportSubmittingEntityIDField, counterparty2IdentifierTypeField);
	}
	
	@Override
	public List<FieldValue> tabulate(PartiesToTheDerivative input) {
		Optional<String> reportSubmittingEntityID = Optional.ofNullable(input.getReportSubmittingEntityID());
		Optional<Boolean> counterparty2IdentifierType = Optional.ofNullable(input.getCounterparty2IdentifierType());
		return Arrays.asList(
			new FieldValueImpl(reportSubmittingEntityIDField, reportSubmittingEntityID),
			new FieldValueImpl(counterparty2IdentifierTypeField, counterparty2IdentifierType)
		);
	}
}
