package drr.regulation.fca.ukemir.refit.trade;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.common.collect.ImmutableList;
import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import com.rosetta.model.lib.records.Date;
import com.rosetta.util.ListEquals;
import drr.regulation.fca.ukemir.refit.trade.DeliveryReport;
import drr.regulation.fca.ukemir.refit.trade.DeliveryReport.DeliveryReportBuilder;
import drr.regulation.fca.ukemir.refit.trade.DeliveryReport.DeliveryReportBuilderImpl;
import drr.regulation.fca.ukemir.refit.trade.DeliveryReport.DeliveryReportImpl;
import drr.regulation.fca.ukemir.refit.trade.meta.DeliveryReportMeta;
import iso20022.auth030.esma.DurationType1Code;
import iso20022.auth030.esma.EnergyQuantityUnit2Code;
import java.math.BigDecimal;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.function.Consumer;
import java.util.stream.Collectors;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="DeliveryReport", builder=DeliveryReport.DeliveryReportBuilderImpl.class, version="4.2.1")
public interface DeliveryReport extends RosettaModelObject {

	DeliveryReportMeta metaData = new DeliveryReportMeta();

	/*********************** Getter Methods  ***********************/
	LocalTime getDeliveryStartTime();
	LocalTime getDeliveryEndTime();
	Date getDeliveryStartDate();
	Date getDeliveryEndDate();
	DurationType1Code getDuration();
	List<String> getDaysOfTheWeek();
	BigDecimal getDeliveryCapacity();
	EnergyQuantityUnit2Code getQuantityUnit();
	BigDecimal getPriceTimeIntervalQuantity();
	ISOCurrencyCodeEnum getCurrencyOfThePriceTimeIntervalQuantity();

	/*********************** Build Methods  ***********************/
	DeliveryReport build();
	
	DeliveryReport.DeliveryReportBuilder toBuilder();
	
	static DeliveryReport.DeliveryReportBuilder builder() {
		return new DeliveryReport.DeliveryReportBuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends DeliveryReport> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends DeliveryReport> getType() {
		return DeliveryReport.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("deliveryStartTime"), LocalTime.class, getDeliveryStartTime(), this);
		processor.processBasic(path.newSubPath("deliveryEndTime"), LocalTime.class, getDeliveryEndTime(), this);
		processor.processBasic(path.newSubPath("deliveryStartDate"), Date.class, getDeliveryStartDate(), this);
		processor.processBasic(path.newSubPath("deliveryEndDate"), Date.class, getDeliveryEndDate(), this);
		processor.processBasic(path.newSubPath("duration"), DurationType1Code.class, getDuration(), this);
		processor.processBasic(path.newSubPath("daysOfTheWeek"), String.class, getDaysOfTheWeek(), this);
		processor.processBasic(path.newSubPath("deliveryCapacity"), BigDecimal.class, getDeliveryCapacity(), this);
		processor.processBasic(path.newSubPath("quantityUnit"), EnergyQuantityUnit2Code.class, getQuantityUnit(), this);
		processor.processBasic(path.newSubPath("priceTimeIntervalQuantity"), BigDecimal.class, getPriceTimeIntervalQuantity(), this);
		processor.processBasic(path.newSubPath("currencyOfThePriceTimeIntervalQuantity"), ISOCurrencyCodeEnum.class, getCurrencyOfThePriceTimeIntervalQuantity(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface DeliveryReportBuilder extends DeliveryReport, RosettaModelObjectBuilder {
		DeliveryReport.DeliveryReportBuilder setDeliveryStartTime(LocalTime deliveryStartTime);
		DeliveryReport.DeliveryReportBuilder setDeliveryEndTime(LocalTime deliveryEndTime);
		DeliveryReport.DeliveryReportBuilder setDeliveryStartDate(Date deliveryStartDate);
		DeliveryReport.DeliveryReportBuilder setDeliveryEndDate(Date deliveryEndDate);
		DeliveryReport.DeliveryReportBuilder setDuration(DurationType1Code duration);
		DeliveryReport.DeliveryReportBuilder addDaysOfTheWeek(String daysOfTheWeek0);
		DeliveryReport.DeliveryReportBuilder addDaysOfTheWeek(String daysOfTheWeek1, int _idx);
		DeliveryReport.DeliveryReportBuilder addDaysOfTheWeek(List<? extends String> daysOfTheWeek2);
		DeliveryReport.DeliveryReportBuilder setDaysOfTheWeek(List<? extends String> daysOfTheWeek3);
		DeliveryReport.DeliveryReportBuilder setDeliveryCapacity(BigDecimal deliveryCapacity);
		DeliveryReport.DeliveryReportBuilder setQuantityUnit(EnergyQuantityUnit2Code quantityUnit);
		DeliveryReport.DeliveryReportBuilder setPriceTimeIntervalQuantity(BigDecimal priceTimeIntervalQuantity);
		DeliveryReport.DeliveryReportBuilder setCurrencyOfThePriceTimeIntervalQuantity(ISOCurrencyCodeEnum currencyOfThePriceTimeIntervalQuantity);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("deliveryStartTime"), LocalTime.class, getDeliveryStartTime(), this);
			processor.processBasic(path.newSubPath("deliveryEndTime"), LocalTime.class, getDeliveryEndTime(), this);
			processor.processBasic(path.newSubPath("deliveryStartDate"), Date.class, getDeliveryStartDate(), this);
			processor.processBasic(path.newSubPath("deliveryEndDate"), Date.class, getDeliveryEndDate(), this);
			processor.processBasic(path.newSubPath("duration"), DurationType1Code.class, getDuration(), this);
			processor.processBasic(path.newSubPath("daysOfTheWeek"), String.class, getDaysOfTheWeek(), this);
			processor.processBasic(path.newSubPath("deliveryCapacity"), BigDecimal.class, getDeliveryCapacity(), this);
			processor.processBasic(path.newSubPath("quantityUnit"), EnergyQuantityUnit2Code.class, getQuantityUnit(), this);
			processor.processBasic(path.newSubPath("priceTimeIntervalQuantity"), BigDecimal.class, getPriceTimeIntervalQuantity(), this);
			processor.processBasic(path.newSubPath("currencyOfThePriceTimeIntervalQuantity"), ISOCurrencyCodeEnum.class, getCurrencyOfThePriceTimeIntervalQuantity(), this);
		}
		

		DeliveryReport.DeliveryReportBuilder prune();
	}

	/*********************** Immutable Implementation of DeliveryReport  ***********************/
	class DeliveryReportImpl implements DeliveryReport {
		private final LocalTime deliveryStartTime;
		private final LocalTime deliveryEndTime;
		private final Date deliveryStartDate;
		private final Date deliveryEndDate;
		private final DurationType1Code duration;
		private final List<String> daysOfTheWeek;
		private final BigDecimal deliveryCapacity;
		private final EnergyQuantityUnit2Code quantityUnit;
		private final BigDecimal priceTimeIntervalQuantity;
		private final ISOCurrencyCodeEnum currencyOfThePriceTimeIntervalQuantity;
		
		protected DeliveryReportImpl(DeliveryReport.DeliveryReportBuilder builder) {
			this.deliveryStartTime = builder.getDeliveryStartTime();
			this.deliveryEndTime = builder.getDeliveryEndTime();
			this.deliveryStartDate = builder.getDeliveryStartDate();
			this.deliveryEndDate = builder.getDeliveryEndDate();
			this.duration = builder.getDuration();
			this.daysOfTheWeek = ofNullable(builder.getDaysOfTheWeek()).filter(_l->!_l.isEmpty()).map(ImmutableList::copyOf).orElse(null);
			this.deliveryCapacity = builder.getDeliveryCapacity();
			this.quantityUnit = builder.getQuantityUnit();
			this.priceTimeIntervalQuantity = builder.getPriceTimeIntervalQuantity();
			this.currencyOfThePriceTimeIntervalQuantity = builder.getCurrencyOfThePriceTimeIntervalQuantity();
		}
		
		@Override
		@RosettaAttribute("deliveryStartTime")
		public LocalTime getDeliveryStartTime() {
			return deliveryStartTime;
		}
		
		@Override
		@RosettaAttribute("deliveryEndTime")
		public LocalTime getDeliveryEndTime() {
			return deliveryEndTime;
		}
		
		@Override
		@RosettaAttribute("deliveryStartDate")
		public Date getDeliveryStartDate() {
			return deliveryStartDate;
		}
		
		@Override
		@RosettaAttribute("deliveryEndDate")
		public Date getDeliveryEndDate() {
			return deliveryEndDate;
		}
		
		@Override
		@RosettaAttribute("duration")
		public DurationType1Code getDuration() {
			return duration;
		}
		
		@Override
		@RosettaAttribute("daysOfTheWeek")
		public List<String> getDaysOfTheWeek() {
			return daysOfTheWeek;
		}
		
		@Override
		@RosettaAttribute("deliveryCapacity")
		public BigDecimal getDeliveryCapacity() {
			return deliveryCapacity;
		}
		
		@Override
		@RosettaAttribute("quantityUnit")
		public EnergyQuantityUnit2Code getQuantityUnit() {
			return quantityUnit;
		}
		
		@Override
		@RosettaAttribute("priceTimeIntervalQuantity")
		public BigDecimal getPriceTimeIntervalQuantity() {
			return priceTimeIntervalQuantity;
		}
		
		@Override
		@RosettaAttribute("currencyOfThePriceTimeIntervalQuantity")
		public ISOCurrencyCodeEnum getCurrencyOfThePriceTimeIntervalQuantity() {
			return currencyOfThePriceTimeIntervalQuantity;
		}
		
		@Override
		public DeliveryReport build() {
			return this;
		}
		
		@Override
		public DeliveryReport.DeliveryReportBuilder toBuilder() {
			DeliveryReport.DeliveryReportBuilder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(DeliveryReport.DeliveryReportBuilder builder) {
			ofNullable(getDeliveryStartTime()).ifPresent(builder::setDeliveryStartTime);
			ofNullable(getDeliveryEndTime()).ifPresent(builder::setDeliveryEndTime);
			ofNullable(getDeliveryStartDate()).ifPresent(builder::setDeliveryStartDate);
			ofNullable(getDeliveryEndDate()).ifPresent(builder::setDeliveryEndDate);
			ofNullable(getDuration()).ifPresent(builder::setDuration);
			ofNullable(getDaysOfTheWeek()).ifPresent(builder::setDaysOfTheWeek);
			ofNullable(getDeliveryCapacity()).ifPresent(builder::setDeliveryCapacity);
			ofNullable(getQuantityUnit()).ifPresent(builder::setQuantityUnit);
			ofNullable(getPriceTimeIntervalQuantity()).ifPresent(builder::setPriceTimeIntervalQuantity);
			ofNullable(getCurrencyOfThePriceTimeIntervalQuantity()).ifPresent(builder::setCurrencyOfThePriceTimeIntervalQuantity);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			DeliveryReport _that = getType().cast(o);
		
			if (!Objects.equals(deliveryStartTime, _that.getDeliveryStartTime())) return false;
			if (!Objects.equals(deliveryEndTime, _that.getDeliveryEndTime())) return false;
			if (!Objects.equals(deliveryStartDate, _that.getDeliveryStartDate())) return false;
			if (!Objects.equals(deliveryEndDate, _that.getDeliveryEndDate())) return false;
			if (!Objects.equals(duration, _that.getDuration())) return false;
			if (!ListEquals.listEquals(daysOfTheWeek, _that.getDaysOfTheWeek())) return false;
			if (!Objects.equals(deliveryCapacity, _that.getDeliveryCapacity())) return false;
			if (!Objects.equals(quantityUnit, _that.getQuantityUnit())) return false;
			if (!Objects.equals(priceTimeIntervalQuantity, _that.getPriceTimeIntervalQuantity())) return false;
			if (!Objects.equals(currencyOfThePriceTimeIntervalQuantity, _that.getCurrencyOfThePriceTimeIntervalQuantity())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (deliveryStartTime != null ? deliveryStartTime.hashCode() : 0);
			_result = 31 * _result + (deliveryEndTime != null ? deliveryEndTime.hashCode() : 0);
			_result = 31 * _result + (deliveryStartDate != null ? deliveryStartDate.hashCode() : 0);
			_result = 31 * _result + (deliveryEndDate != null ? deliveryEndDate.hashCode() : 0);
			_result = 31 * _result + (duration != null ? duration.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (daysOfTheWeek != null ? daysOfTheWeek.hashCode() : 0);
			_result = 31 * _result + (deliveryCapacity != null ? deliveryCapacity.hashCode() : 0);
			_result = 31 * _result + (quantityUnit != null ? quantityUnit.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (priceTimeIntervalQuantity != null ? priceTimeIntervalQuantity.hashCode() : 0);
			_result = 31 * _result + (currencyOfThePriceTimeIntervalQuantity != null ? currencyOfThePriceTimeIntervalQuantity.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DeliveryReport {" +
				"deliveryStartTime=" + this.deliveryStartTime + ", " +
				"deliveryEndTime=" + this.deliveryEndTime + ", " +
				"deliveryStartDate=" + this.deliveryStartDate + ", " +
				"deliveryEndDate=" + this.deliveryEndDate + ", " +
				"duration=" + this.duration + ", " +
				"daysOfTheWeek=" + this.daysOfTheWeek + ", " +
				"deliveryCapacity=" + this.deliveryCapacity + ", " +
				"quantityUnit=" + this.quantityUnit + ", " +
				"priceTimeIntervalQuantity=" + this.priceTimeIntervalQuantity + ", " +
				"currencyOfThePriceTimeIntervalQuantity=" + this.currencyOfThePriceTimeIntervalQuantity +
			'}';
		}
	}

	/*********************** Builder Implementation of DeliveryReport  ***********************/
	class DeliveryReportBuilderImpl implements DeliveryReport.DeliveryReportBuilder {
	
		protected LocalTime deliveryStartTime;
		protected LocalTime deliveryEndTime;
		protected Date deliveryStartDate;
		protected Date deliveryEndDate;
		protected DurationType1Code duration;
		protected List<String> daysOfTheWeek = new ArrayList<>();
		protected BigDecimal deliveryCapacity;
		protected EnergyQuantityUnit2Code quantityUnit;
		protected BigDecimal priceTimeIntervalQuantity;
		protected ISOCurrencyCodeEnum currencyOfThePriceTimeIntervalQuantity;
	
		public DeliveryReportBuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("deliveryStartTime")
		public LocalTime getDeliveryStartTime() {
			return deliveryStartTime;
		}
		
		@Override
		@RosettaAttribute("deliveryEndTime")
		public LocalTime getDeliveryEndTime() {
			return deliveryEndTime;
		}
		
		@Override
		@RosettaAttribute("deliveryStartDate")
		public Date getDeliveryStartDate() {
			return deliveryStartDate;
		}
		
		@Override
		@RosettaAttribute("deliveryEndDate")
		public Date getDeliveryEndDate() {
			return deliveryEndDate;
		}
		
		@Override
		@RosettaAttribute("duration")
		public DurationType1Code getDuration() {
			return duration;
		}
		
		@Override
		@RosettaAttribute("daysOfTheWeek")
		public List<String> getDaysOfTheWeek() {
			return daysOfTheWeek;
		}
		
		@Override
		@RosettaAttribute("deliveryCapacity")
		public BigDecimal getDeliveryCapacity() {
			return deliveryCapacity;
		}
		
		@Override
		@RosettaAttribute("quantityUnit")
		public EnergyQuantityUnit2Code getQuantityUnit() {
			return quantityUnit;
		}
		
		@Override
		@RosettaAttribute("priceTimeIntervalQuantity")
		public BigDecimal getPriceTimeIntervalQuantity() {
			return priceTimeIntervalQuantity;
		}
		
		@Override
		@RosettaAttribute("currencyOfThePriceTimeIntervalQuantity")
		public ISOCurrencyCodeEnum getCurrencyOfThePriceTimeIntervalQuantity() {
			return currencyOfThePriceTimeIntervalQuantity;
		}
		
	
		@Override
		@RosettaAttribute("deliveryStartTime")
		public DeliveryReport.DeliveryReportBuilder setDeliveryStartTime(LocalTime deliveryStartTime) {
			this.deliveryStartTime = deliveryStartTime==null?null:deliveryStartTime;
			return this;
		}
		@Override
		@RosettaAttribute("deliveryEndTime")
		public DeliveryReport.DeliveryReportBuilder setDeliveryEndTime(LocalTime deliveryEndTime) {
			this.deliveryEndTime = deliveryEndTime==null?null:deliveryEndTime;
			return this;
		}
		@Override
		@RosettaAttribute("deliveryStartDate")
		public DeliveryReport.DeliveryReportBuilder setDeliveryStartDate(Date deliveryStartDate) {
			this.deliveryStartDate = deliveryStartDate==null?null:deliveryStartDate;
			return this;
		}
		@Override
		@RosettaAttribute("deliveryEndDate")
		public DeliveryReport.DeliveryReportBuilder setDeliveryEndDate(Date deliveryEndDate) {
			this.deliveryEndDate = deliveryEndDate==null?null:deliveryEndDate;
			return this;
		}
		@Override
		@RosettaAttribute("duration")
		public DeliveryReport.DeliveryReportBuilder setDuration(DurationType1Code duration) {
			this.duration = duration==null?null:duration;
			return this;
		}
		@Override
		public DeliveryReport.DeliveryReportBuilder addDaysOfTheWeek(String daysOfTheWeek) {
			if (daysOfTheWeek!=null) this.daysOfTheWeek.add(daysOfTheWeek);
			return this;
		}
		
		@Override
		public DeliveryReport.DeliveryReportBuilder addDaysOfTheWeek(String daysOfTheWeek, int _idx) {
			getIndex(this.daysOfTheWeek, _idx, () -> daysOfTheWeek);
			return this;
		}
		@Override 
		public DeliveryReport.DeliveryReportBuilder addDaysOfTheWeek(List<? extends String> daysOfTheWeeks) {
			if (daysOfTheWeeks != null) {
				for (String toAdd : daysOfTheWeeks) {
					this.daysOfTheWeek.add(toAdd);
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("daysOfTheWeek")
		public DeliveryReport.DeliveryReportBuilder setDaysOfTheWeek(List<? extends String> daysOfTheWeeks) {
			if (daysOfTheWeeks == null)  {
				this.daysOfTheWeek = new ArrayList<>();
			}
			else {
				this.daysOfTheWeek = daysOfTheWeeks.stream()
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("deliveryCapacity")
		public DeliveryReport.DeliveryReportBuilder setDeliveryCapacity(BigDecimal deliveryCapacity) {
			this.deliveryCapacity = deliveryCapacity==null?null:deliveryCapacity;
			return this;
		}
		@Override
		@RosettaAttribute("quantityUnit")
		public DeliveryReport.DeliveryReportBuilder setQuantityUnit(EnergyQuantityUnit2Code quantityUnit) {
			this.quantityUnit = quantityUnit==null?null:quantityUnit;
			return this;
		}
		@Override
		@RosettaAttribute("priceTimeIntervalQuantity")
		public DeliveryReport.DeliveryReportBuilder setPriceTimeIntervalQuantity(BigDecimal priceTimeIntervalQuantity) {
			this.priceTimeIntervalQuantity = priceTimeIntervalQuantity==null?null:priceTimeIntervalQuantity;
			return this;
		}
		@Override
		@RosettaAttribute("currencyOfThePriceTimeIntervalQuantity")
		public DeliveryReport.DeliveryReportBuilder setCurrencyOfThePriceTimeIntervalQuantity(ISOCurrencyCodeEnum currencyOfThePriceTimeIntervalQuantity) {
			this.currencyOfThePriceTimeIntervalQuantity = currencyOfThePriceTimeIntervalQuantity==null?null:currencyOfThePriceTimeIntervalQuantity;
			return this;
		}
		
		@Override
		public DeliveryReport build() {
			return new DeliveryReport.DeliveryReportImpl(this);
		}
		
		@Override
		public DeliveryReport.DeliveryReportBuilder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public DeliveryReport.DeliveryReportBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getDeliveryStartTime()!=null) return true;
			if (getDeliveryEndTime()!=null) return true;
			if (getDeliveryStartDate()!=null) return true;
			if (getDeliveryEndDate()!=null) return true;
			if (getDuration()!=null) return true;
			if (getDaysOfTheWeek()!=null && !getDaysOfTheWeek().isEmpty()) return true;
			if (getDeliveryCapacity()!=null) return true;
			if (getQuantityUnit()!=null) return true;
			if (getPriceTimeIntervalQuantity()!=null) return true;
			if (getCurrencyOfThePriceTimeIntervalQuantity()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public DeliveryReport.DeliveryReportBuilder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			DeliveryReport.DeliveryReportBuilder o = (DeliveryReport.DeliveryReportBuilder) other;
			
			
			merger.mergeBasic(getDeliveryStartTime(), o.getDeliveryStartTime(), this::setDeliveryStartTime);
			merger.mergeBasic(getDeliveryEndTime(), o.getDeliveryEndTime(), this::setDeliveryEndTime);
			merger.mergeBasic(getDeliveryStartDate(), o.getDeliveryStartDate(), this::setDeliveryStartDate);
			merger.mergeBasic(getDeliveryEndDate(), o.getDeliveryEndDate(), this::setDeliveryEndDate);
			merger.mergeBasic(getDuration(), o.getDuration(), this::setDuration);
			merger.mergeBasic(getDaysOfTheWeek(), o.getDaysOfTheWeek(), (Consumer<String>) this::addDaysOfTheWeek);
			merger.mergeBasic(getDeliveryCapacity(), o.getDeliveryCapacity(), this::setDeliveryCapacity);
			merger.mergeBasic(getQuantityUnit(), o.getQuantityUnit(), this::setQuantityUnit);
			merger.mergeBasic(getPriceTimeIntervalQuantity(), o.getPriceTimeIntervalQuantity(), this::setPriceTimeIntervalQuantity);
			merger.mergeBasic(getCurrencyOfThePriceTimeIntervalQuantity(), o.getCurrencyOfThePriceTimeIntervalQuantity(), this::setCurrencyOfThePriceTimeIntervalQuantity);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			DeliveryReport _that = getType().cast(o);
		
			if (!Objects.equals(deliveryStartTime, _that.getDeliveryStartTime())) return false;
			if (!Objects.equals(deliveryEndTime, _that.getDeliveryEndTime())) return false;
			if (!Objects.equals(deliveryStartDate, _that.getDeliveryStartDate())) return false;
			if (!Objects.equals(deliveryEndDate, _that.getDeliveryEndDate())) return false;
			if (!Objects.equals(duration, _that.getDuration())) return false;
			if (!ListEquals.listEquals(daysOfTheWeek, _that.getDaysOfTheWeek())) return false;
			if (!Objects.equals(deliveryCapacity, _that.getDeliveryCapacity())) return false;
			if (!Objects.equals(quantityUnit, _that.getQuantityUnit())) return false;
			if (!Objects.equals(priceTimeIntervalQuantity, _that.getPriceTimeIntervalQuantity())) return false;
			if (!Objects.equals(currencyOfThePriceTimeIntervalQuantity, _that.getCurrencyOfThePriceTimeIntervalQuantity())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (deliveryStartTime != null ? deliveryStartTime.hashCode() : 0);
			_result = 31 * _result + (deliveryEndTime != null ? deliveryEndTime.hashCode() : 0);
			_result = 31 * _result + (deliveryStartDate != null ? deliveryStartDate.hashCode() : 0);
			_result = 31 * _result + (deliveryEndDate != null ? deliveryEndDate.hashCode() : 0);
			_result = 31 * _result + (duration != null ? duration.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (daysOfTheWeek != null ? daysOfTheWeek.hashCode() : 0);
			_result = 31 * _result + (deliveryCapacity != null ? deliveryCapacity.hashCode() : 0);
			_result = 31 * _result + (quantityUnit != null ? quantityUnit.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (priceTimeIntervalQuantity != null ? priceTimeIntervalQuantity.hashCode() : 0);
			_result = 31 * _result + (currencyOfThePriceTimeIntervalQuantity != null ? currencyOfThePriceTimeIntervalQuantity.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "DeliveryReportBuilder {" +
				"deliveryStartTime=" + this.deliveryStartTime + ", " +
				"deliveryEndTime=" + this.deliveryEndTime + ", " +
				"deliveryStartDate=" + this.deliveryStartDate + ", " +
				"deliveryEndDate=" + this.deliveryEndDate + ", " +
				"duration=" + this.duration + ", " +
				"daysOfTheWeek=" + this.daysOfTheWeek + ", " +
				"deliveryCapacity=" + this.deliveryCapacity + ", " +
				"quantityUnit=" + this.quantityUnit + ", " +
				"priceTimeIntervalQuantity=" + this.priceTimeIntervalQuantity + ", " +
				"currencyOfThePriceTimeIntervalQuantity=" + this.currencyOfThePriceTimeIntervalQuantity +
			'}';
		}
	}
}
