package drr.regulation.fca.ukemir.refit.trade.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.fca.ukemir.refit.trade.DeliveryReport;
import iso20022.auth030.esma.DurationType1Code;
import iso20022.auth030.esma.EnergyQuantityUnit2Code;
import java.math.BigDecimal;
import java.time.LocalTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class DeliveryReportTypeTabulator implements Tabulator<DeliveryReport> {
	private final Field deliveryStartTimeField;
	private final Field deliveryEndTimeField;
	private final Field deliveryStartDateField;
	private final Field deliveryEndDateField;
	private final Field durationField;
	private final Field daysOfTheWeekField;
	private final Field deliveryCapacityField;
	private final Field quantityUnitField;
	private final Field priceTimeIntervalQuantityField;
	private final Field currencyOfThePriceTimeIntervalQuantityField;
	
	public DeliveryReportTypeTabulator() {
		this.deliveryStartTimeField = new FieldImpl(
			"deliveryStartTime",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "DeliveryIntervalStartTime")),
			Optional.of("2.122/2.131-$ 2.122 Delivery interval start time"),
			Arrays.asList()
		);
		this.deliveryEndTimeField = new FieldImpl(
			"deliveryEndTime",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "DeliveryIntervalEndTime")),
			Optional.of("2.122/2.131-$ 2.123 Delivery interval end time"),
			Arrays.asList()
		);
		this.deliveryStartDateField = new FieldImpl(
			"deliveryStartDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "DeliveryStartDate")),
			Optional.of("2.122/2.131-$ 2.124 Delivery start date"),
			Arrays.asList()
		);
		this.deliveryEndDateField = new FieldImpl(
			"deliveryEndDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "DeliveryEndDate")),
			Optional.of("2.122/2.131-$ 2.125 Delivery end date"),
			Arrays.asList()
		);
		this.durationField = new FieldImpl(
			"duration",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "Duration")),
			Optional.of("2.122/2.131-$ 2.126 Duration"),
			Arrays.asList()
		);
		this.daysOfTheWeekField = new FieldImpl(
			"daysOfTheWeek",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "DaysOfTheWeek")),
			Optional.of("2.122/2.131-$ 2.127 Days of the week"),
			Arrays.asList()
		);
		this.deliveryCapacityField = new FieldImpl(
			"deliveryCapacity",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "DeliveryCapacity")),
			Optional.of("2.122/2.131-$ 2.128 Delivery capacity"),
			Arrays.asList()
		);
		this.quantityUnitField = new FieldImpl(
			"quantityUnit",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "QuantityUnit")),
			Optional.of("2.122/2.131-$ 2.129 Quantity unit"),
			Arrays.asList()
		);
		this.priceTimeIntervalQuantityField = new FieldImpl(
			"priceTimeIntervalQuantity",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "PriceTimeIntervalQuantity")),
			Optional.of("2.122/2.131-$ 2.130 Price time interval quantity"),
			Arrays.asList()
		);
		this.currencyOfThePriceTimeIntervalQuantityField = new FieldImpl(
			"currencyOfThePriceTimeIntervalQuantity",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "CurrencyOfThePriceTimeIntervalQuantity")),
			Optional.of("2.122/2.131-$ 2.131 Currency of the price time interval quantity"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(deliveryStartTimeField, deliveryEndTimeField, deliveryStartDateField, deliveryEndDateField, durationField, daysOfTheWeekField, deliveryCapacityField, quantityUnitField, priceTimeIntervalQuantityField, currencyOfThePriceTimeIntervalQuantityField);
	}
	
	@Override
	public List<FieldValue> tabulate(DeliveryReport input) {
		Optional<LocalTime> deliveryStartTime = Optional.ofNullable(input.getDeliveryStartTime());
		Optional<LocalTime> deliveryEndTime = Optional.ofNullable(input.getDeliveryEndTime());
		Optional<Date> deliveryStartDate = Optional.ofNullable(input.getDeliveryStartDate());
		Optional<Date> deliveryEndDate = Optional.ofNullable(input.getDeliveryEndDate());
		Optional<DurationType1Code> duration = Optional.ofNullable(input.getDuration());
		Optional<List<? extends String>> daysOfTheWeek = Optional.ofNullable(input.getDaysOfTheWeek());
		Optional<BigDecimal> deliveryCapacity = Optional.ofNullable(input.getDeliveryCapacity());
		Optional<EnergyQuantityUnit2Code> quantityUnit = Optional.ofNullable(input.getQuantityUnit());
		Optional<BigDecimal> priceTimeIntervalQuantity = Optional.ofNullable(input.getPriceTimeIntervalQuantity());
		Optional<ISOCurrencyCodeEnum> currencyOfThePriceTimeIntervalQuantity = Optional.ofNullable(input.getCurrencyOfThePriceTimeIntervalQuantity());
		return Arrays.asList(
			new FieldValueImpl(deliveryStartTimeField, deliveryStartTime),
			new FieldValueImpl(deliveryEndTimeField, deliveryEndTime),
			new FieldValueImpl(deliveryStartDateField, deliveryStartDate),
			new FieldValueImpl(deliveryEndDateField, deliveryEndDate),
			new FieldValueImpl(durationField, duration),
			new FieldValueImpl(daysOfTheWeekField, daysOfTheWeek),
			new FieldValueImpl(deliveryCapacityField, deliveryCapacity),
			new FieldValueImpl(quantityUnitField, quantityUnit),
			new FieldValueImpl(priceTimeIntervalQuantityField, priceTimeIntervalQuantity),
			new FieldValueImpl(currencyOfThePriceTimeIntervalQuantityField, currencyOfThePriceTimeIntervalQuantity)
		);
	}
}
