package drr.regulation.fca.ukemir.refit.trade.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaReport;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.enrichment.common.reports.EnrichmentDataRule;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.fca.ukemir.refit.trade.FCAUKEMIRTransactionReport;
import drr.regulation.fca.ukemir.refit.trade.FCAUKEMIRTransactionReport.FCAUKEMIRTransactionReportBuilder;
import java.util.Optional;
import javax.inject.Inject;


@RosettaReport(namespace="drr.regulation.fca.ukemir.refit.trade", body="FCA", corpusList={"UKEMIR", "Trade"})
@ImplementedBy(FCAUKEMIRTradeReportFunction.FCAUKEMIRTradeReportFunctionDefault.class)
public abstract class FCAUKEMIRTradeReportFunction implements ReportFunction<TransactionReportInstruction, FCAUKEMIRTransactionReport> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ActionTypeRule actionType;
	@Inject protected AssetClassRule assetClass;
	@Inject protected BasketConstituentsRule basketConstituents;
	@Inject protected BrokerIDRule brokerID;
	@Inject protected CDSIndexAttachmentPointRule cDSIndexAttachmentPoint;
	@Inject protected CDSIndexDetachmentPointRule cDSIndexDetachmentPoint;
	@Inject protected CentralCounterpartyRule centralCounterparty;
	@Inject protected ClearingMemberRule clearingMember;
	@Inject protected ClearingThresholdOfCounterparty1Rule clearingThresholdOfCounterparty1;
	@Inject protected ClearingThresholdOfCounterparty2Rule clearingThresholdOfCounterparty2;
	@Inject protected ClearingTimestampRule clearingTimestamp;
	@Inject protected CollateralPortfolioCodeRule collateralPortfolioCode;
	@Inject protected CollateralPortfolioIndicatorRule collateralPortfolioIndicator;
	@Inject protected ConfirmationTimestampRule confirmationTimestamp;
	@Inject protected ConfirmedRule confirmed;
	@Inject protected ContractPriceScheduleRule contractPriceSchedule;
	@Inject protected ContractTypeRule contractType;
	@Inject protected CorporateSectorOfTheCounterparty1Rule corporateSectorOfTheCounterparty1;
	@Inject protected CorporateSectorOfTheCounterparty2Rule corporateSectorOfTheCounterparty2;
	@Inject protected Counterparty1Rule counterparty1;
	@Inject protected Counterparty2Rule counterparty2;
	@Inject protected Counterparty2IdentifierTypeRule counterparty2IdentifierType;
	@Inject protected CountryOfTheCounterParty2Rule countryOfTheCounterParty2;
	@Inject protected CustomBasketCodeRule customBasketCode;
	@Inject protected DeliveryRule delivery;
	@Inject protected DeliveryPointOrZoneRule deliveryPointOrZone;
	@Inject protected DeliveryTypeRule deliveryType;
	@Inject protected DeltaRule delta;
	@Inject protected DerivativeBasedOnCryptoAssetsRule derivativeBasedOnCryptoAssets;
	@Inject protected DirectionRule direction;
	@Inject protected DirectionOfLeg1Rule directionOfLeg1;
	@Inject protected DirectionOfLeg2Rule directionOfLeg2;
	@Inject protected DirectlyLinkedToCommercialActivityOrTreasuryFinancingRule directlyLinkedToCommercialActivityOrTreasuryFinancing;
	@Inject protected EarlyTerminationDateRule earlyTerminationDate;
	@Inject protected EffectiveDateRule effectiveDate;
	@Inject protected EnrichmentDataRule enrichmentData;
	@Inject protected EntityResponsibleForReportingRule entityResponsibleForReporting;
	@Inject protected EventTypeRule eventType;
	@Inject protected ExchangeRateRule exchangeRate;
	@Inject protected ExchangeRateBasisRule exchangeRateBasis;
	@Inject protected ExecutionAgentRule executionAgent;
	@Inject protected ExecutionTimestampRule executionTimestamp;
	@Inject protected ExpirationDateRule expirationDate;
	@Inject protected FinalContractualSettlementDateRule finalContractualSettlementDate;
	@Inject protected FixedRateDayCountConventionLeg2Rule fixedRateDayCountConventionLeg2;
	@Inject protected FixedRateOfLeg1OrCouponRule fixedRateOfLeg1OrCoupon;
	@Inject protected FixedRateOfLeg2Rule fixedRateOfLeg2;
	@Inject protected FixedRateOrCouponDayCountConventionLeg1Rule fixedRateOrCouponDayCountConventionLeg1;
	@Inject protected FixedRateOrCouponPaymentFrequencyPeriodLeg1Rule fixedRateOrCouponPaymentFrequencyPeriodLeg1;
	@Inject protected FixedRatePaymentFrequencyPeriodLeg2Rule fixedRatePaymentFrequencyPeriodLeg2;
	@Inject protected FloatingRateDayCountConventionOfLeg1Rule floatingRateDayCountConventionOfLeg1;
	@Inject protected FloatingRateDayCountConventionOfLeg2Rule floatingRateDayCountConventionOfLeg2;
	@Inject protected FloatingRatePaymentFrequencyPeriodOfLeg1Rule floatingRatePaymentFrequencyPeriodOfLeg1;
	@Inject protected FloatingRatePaymentFrequencyPeriodOfLeg2Rule floatingRatePaymentFrequencyPeriodOfLeg2;
	@Inject protected FloatingRateReferencePeriodOfLeg1TimePeriodRule floatingRateReferencePeriodOfLeg1TimePeriod;
	@Inject protected FloatingRateReferencePeriodOfLeg2TimePeriodRule floatingRateReferencePeriodOfLeg2TimePeriod;
	@Inject protected FloatingRateResetFrequencyPeriodOfLeg1Rule floatingRateResetFrequencyPeriodOfLeg1;
	@Inject protected FloatingRateResetFrequencyPeriodOfLeg2Rule floatingRateResetFrequencyPeriodOfLeg2;
	@Inject protected ForwardExchangeRateRule forwardExchangeRate;
	@Inject protected FurtherSubProductRule furtherSubProduct;
	@Inject protected ISINRule iSIN;
	@Inject protected IdentifierOfFloatingRateOfLeg1Rule identifierOfFloatingRateOfLeg1;
	@Inject protected IdentifierOfFloatingRateOfLeg2Rule identifierOfFloatingRateOfLeg2;
	@Inject protected IndexFactorRule indexFactor;
	@Inject protected IntragroupRule intragroup;
	@Inject protected LevelRule level;
	@Inject protected LoadTypeRule loadType;
	@Inject protected MasterAgreementTypeRule masterAgreementType;
	@Inject protected MasterAgreementVersionRule masterAgreementVersion;
	@Inject protected MaturityDateOfTheUnderlyingRule maturityDateOfTheUnderlying;
	@Inject protected NameOfTheFloatingRateOfLeg1Rule nameOfTheFloatingRateOfLeg1;
	@Inject protected NameOfTheUnderlyingIndexRule nameOfTheUnderlyingIndex;
	@Inject protected NatureOfCounterparty1Rule natureOfCounterparty1;
	@Inject protected NotionalAmountOfLeg1Rule notionalAmountOfLeg1;
	@Inject protected NotionalAmountOfLeg2Rule notionalAmountOfLeg2;
	@Inject protected NotionalAmountScheduleLeg1Rule notionalAmountScheduleLeg1;
	@Inject protected NotionalAmountScheduleLeg2Rule notionalAmountScheduleLeg2;
	@Inject protected NotionalCurrency1Rule notionalCurrency1;
	@Inject protected NotionalCurrency2Rule notionalCurrency2;
	@Inject protected NotionalQuantityScheduleLeg1Rule notionalQuantityScheduleLeg1;
	@Inject protected NotionalQuantityScheduleLeg2Rule notionalQuantityScheduleLeg2;
	@Inject protected OptionPremiumAmountRule optionPremiumAmount;
	@Inject protected OptionPremiumCurrencyRule optionPremiumCurrency;
	@Inject protected OptionPremiumPaymentDateRule optionPremiumPaymentDate;
	@Inject protected OptionStyleRule optionStyle;
	@Inject protected OptionTypeRule optionType;
	@Inject protected OtherMasterAgreementTypeRule otherMasterAgreementType;
	@Inject protected OtherPaymentRule otherPayment;
	@Inject protected PTRRServiceProviderRule pTRRServiceProvider;
	@Inject protected PackageIdentifierRule packageIdentifier;
	@Inject protected PackageTransactionPriceBasisRule packageTransactionPriceBasis;
	@Inject protected PackageTransactionPriceCurrencyRule packageTransactionPriceCurrency;
	@Inject protected PackageTransactionPriceMonetaryRule packageTransactionPriceMonetary;
	@Inject protected PackageTransactionPriceNotationRule packageTransactionPriceNotation;
	@Inject protected PackageTransactionPricePercentageRule packageTransactionPricePercentage;
	@Inject protected PriceCurrencyRule priceCurrency;
	@Inject protected PriceFormatMonetaryRule priceFormatMonetary;
	@Inject protected PriceFormatPercentageRule priceFormatPercentage;
	@Inject protected PriceNotationRule priceNotation;
	@Inject protected PriorUTIRule priorUTI;
	@Inject protected ProductClassificationRule productClassification;
	@Inject protected ReferenceEntityRule referenceEntity;
	@Inject protected ReportSubmittingEntityIDRule reportSubmittingEntityID;
	@Inject protected ReportTrackingNumberRule reportTrackingNumber;
	@Inject protected ReportingObligationOfTheCounterparty2Rule reportingObligationOfTheCounterparty2;
	@Inject protected SeniorityRule seniority;
	@Inject protected SeriesRule series;
	@Inject protected SettlementCurrency1Rule settlementCurrency1;
	@Inject protected SettlementCurrency2Rule settlementCurrency2;
	@Inject protected SpreadCurrencyOfLeg1Rule spreadCurrencyOfLeg1;
	@Inject protected SpreadOfLeg1BasisRule spreadOfLeg1Basis;
	@Inject protected SpreadOfLeg1MonetaryRule spreadOfLeg1Monetary;
	@Inject protected SpreadOfLeg1NotationRule spreadOfLeg1Notation;
	@Inject protected SpreadOfLeg1PercentageRule spreadOfLeg1Percentage;
	@Inject protected StrikePriceCurrencyCurrencyPairRule strikePriceCurrencyCurrencyPair;
	@Inject protected StrikePriceMonetaryRule strikePriceMonetary;
	@Inject protected StrikePriceNotationRule strikePriceNotation;
	@Inject protected StrikePricePercentageRule strikePricePercentage;
	@Inject protected StrikePriceScheduleRule strikePriceSchedule;
	@Inject protected SubsequentPositionUTIRule subsequentPositionUTI;
	@Inject protected TotalNotionalQuantityOfLeg1Rule totalNotionalQuantityOfLeg1;
	@Inject protected TotalNotionalQuantityOfLeg2Rule totalNotionalQuantityOfLeg2;
	@Inject protected TrancheRule tranche;
	@Inject protected TypeOfPTRRTechniqueRule typeOfPTRRTechnique;
	@Inject protected UPIRule uPI;
	@Inject protected UTIRule uTI;
	@Inject protected UnderlyingIdentificationRule underlyingIdentification;
	@Inject protected UnderlyingIdentificationTypeRule underlyingIdentificationType;
	@Inject protected ValuationAmountRule valuationAmount;
	@Inject protected ValuationCurrencyRule valuationCurrency;
	@Inject protected ValuationMethodRule valuationMethod;
	@Inject protected ValuationTimestampRule valuationTimestamp;
	@Inject protected VersionRule version;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public FCAUKEMIRTransactionReport evaluate(TransactionReportInstruction input) {
		FCAUKEMIRTransactionReport.FCAUKEMIRTransactionReportBuilder outputBuilder = doEvaluate(input);
		
		final FCAUKEMIRTransactionReport output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.build();
			objectValidator.validate(FCAUKEMIRTransactionReport.class, output);
		}
		
		return output;
	}

	protected abstract FCAUKEMIRTransactionReport.FCAUKEMIRTransactionReportBuilder doEvaluate(TransactionReportInstruction input);

	public static class FCAUKEMIRTradeReportFunctionDefault extends FCAUKEMIRTradeReportFunction {
		@Override
		protected FCAUKEMIRTransactionReport.FCAUKEMIRTransactionReportBuilder doEvaluate(TransactionReportInstruction input) {
			FCAUKEMIRTransactionReport.FCAUKEMIRTransactionReportBuilder output = FCAUKEMIRTransactionReport.builder();
			return assignOutput(output, input);
		}
		
		protected FCAUKEMIRTransactionReport.FCAUKEMIRTransactionReportBuilder assignOutput(FCAUKEMIRTransactionReport.FCAUKEMIRTransactionReportBuilder output, TransactionReportInstruction input) {
			output
				.setReportSubmittingEntityID(reportSubmittingEntityID.evaluate(input));
			
			output
				.setEntityResponsibleForReporting(entityResponsibleForReporting.evaluate(input));
			
			output
				.setCounterparty1(counterparty1.evaluate(input));
			
			output
				.setNatureOfCounterparty1(natureOfCounterparty1.evaluate(input));
			
			output
				.setCorporateSectorOfTheCounterparty1(corporateSectorOfTheCounterparty1.evaluate(input));
			
			output
				.setClearingThresholdOfCounterparty1(clearingThresholdOfCounterparty1.evaluate(input));
			
			output
				.setCounterparty2(counterparty2.evaluate(input));
			
			output
				.setCountryOfTheCounterParty2(countryOfTheCounterParty2.evaluate(input));
			
			output
				.setCorporateSectorOfTheCounterparty2(corporateSectorOfTheCounterparty2.evaluate(input));
			
			output
				.setCounterparty2IdentifierType(counterparty2IdentifierType.evaluate(input));
			
			output
				.setClearingMember(clearingMember.evaluate(input));
			
			output
				.setClearingThresholdOfCounterparty2(clearingThresholdOfCounterparty2.evaluate(input));
			
			output
				.setReportingObligationOfTheCounterparty2(reportingObligationOfTheCounterparty2.evaluate(input));
			
			output
				.setBrokerID(brokerID.evaluate(input));
			
			output
				.setDirection(direction.evaluate(input));
			
			output
				.setDirectionOfLeg1(directionOfLeg1.evaluate(input));
			
			output
				.setDirectionOfLeg2(directionOfLeg2.evaluate(input));
			
			output
				.setDirectlyLinkedToCommercialActivityOrTreasuryFinancing(directlyLinkedToCommercialActivityOrTreasuryFinancing.evaluate(input));
			
			output
				.setExecutionAgent(executionAgent.evaluate(input));
			
			output
				.setUti(uTI.evaluate(input));
			
			output
				.setReportTrackingNumber(reportTrackingNumber.evaluate(input));
			
			output
				.setPriorUTI(priorUTI.evaluate(input));
			
			output
				.setPackageIdentifier(packageIdentifier.evaluate(input));
			
			output
				.setSubsequentPositionUTI(subsequentPositionUTI.evaluate(input));
			
			output
				.setIsin(iSIN.evaluate(input));
			
			output
				.setUpi(uPI.evaluate(input));
			
			output
				.setProductClassification(productClassification.evaluate(input));
			
			output
				.setContractType(contractType.evaluate(input));
			
			output
				.setAssetClass(assetClass.evaluate(input));
			
			output
				.setIsCrypto(derivativeBasedOnCryptoAssets.evaluate(input));
			
			output
				.setUnderlyingIdentificationType(underlyingIdentificationType.evaluate(input));
			
			output
				.setUnderlyingIdentification(underlyingIdentification.evaluate(input));
			
			output
				.setNameOfTheUnderlyingIndex(nameOfTheUnderlyingIndex.evaluate(input));
			
			output
				.setBasketConstituents(basketConstituents.evaluate(input));
			
			output
				.setCustomBasketCode(customBasketCode.evaluate(input));
			
			output
				.setSettlementCurrency1(settlementCurrency1.evaluate(input));
			
			output
				.setSettlementCurrency2(settlementCurrency2.evaluate(input));
			
			output
				.setValuationAmount(valuationAmount.evaluate(input));
			
			output
				.setValuationCurrency(valuationCurrency.evaluate(input));
			
			output
				.setValuationMethod(valuationMethod.evaluate(input));
			
			output
				.setDelta(delta.evaluate(input));
			
			output
				.setValuationTimestamp(valuationTimestamp.evaluate(input));
			
			output
				.setConfirmationTimestamp(confirmationTimestamp.evaluate(input));
			
			output
				.setConfirmed(confirmed.evaluate(input));
			
			output
				.setClearingTimestamp(clearingTimestamp.evaluate(input));
			
			output
				.setCentralCounterparty(centralCounterparty.evaluate(input));
			
			output
				.setMasterAgreementType(masterAgreementType.evaluate(input));
			
			output
				.setOtherMasterAgreementType(otherMasterAgreementType.evaluate(input));
			
			output
				.setMasterAgreementVersion(masterAgreementVersion.evaluate(input));
			
			output
				.setIntragroup(intragroup.evaluate(input));
			
			output
				.setTypeOfPTRRTechnique(typeOfPTRRTechnique.evaluate(input));
			
			output
				.setPtrrServiceProvider(pTRRServiceProvider.evaluate(input));
			
			output
				.setExecutionTimestamp(executionTimestamp.evaluate(input));
			
			output
				.setEffectiveDate(effectiveDate.evaluate(input));
			
			output
				.setExpirationDate(expirationDate.evaluate(input));
			
			output
				.setEarlyTerminationDate(earlyTerminationDate.evaluate(input));
			
			output
				.setFinalContractualSettlementDate(finalContractualSettlementDate.evaluate(input));
			
			output
				.setDeliveryType(deliveryType.evaluate(input));
			
			output
				.getOrCreatePrice()
				.setPriceMonetary(priceFormatMonetary.evaluate(input));
			
			output
				.getOrCreatePrice()
				.setPricePercentage(priceFormatPercentage.evaluate(input));
			
			output
				.getOrCreatePrice()
				.setPriceCurrency(priceCurrency.evaluate(input));
			
			output
				.getOrCreatePrice()
				.setPriceNotation(priceNotation.evaluate(input));
			
			output
				.setPriceSchedule(contractPriceSchedule.evaluate(input));
			
			output
				.getOrCreatePackageTransactionPrice()
				.setPackageTransactionPriceMonetary(packageTransactionPriceMonetary.evaluate(input));
			
			output
				.getOrCreatePackageTransactionPrice()
				.setPackageTransactionPricePercentage(packageTransactionPricePercentage.evaluate(input));
			
			output
				.getOrCreatePackageTransactionPrice()
				.setPackageTransactionPriceBasis(packageTransactionPriceBasis.evaluate(input));
			
			output
				.getOrCreatePackageTransactionPrice()
				.setPackageTransactionPriceNotation(packageTransactionPriceNotation.evaluate(input));
			
			output
				.getOrCreatePackageTransactionPrice()
				.setPackageTransactionPriceCurrency(packageTransactionPriceCurrency.evaluate(input));
			
			output
				.setNotionalLeg1(notionalAmountOfLeg1.evaluate(input));
			
			output
				.setNotionalCurrency1(notionalCurrency1.evaluate(input));
			
			output
				.setNotionalAmountScheduleLeg1(notionalAmountScheduleLeg1.evaluate(input));
			
			output
				.setTotalNotionalQuantityLeg1(totalNotionalQuantityOfLeg1.evaluate(input));
			
			output
				.setNotionalQuantityScheduleLeg1(notionalQuantityScheduleLeg1.evaluate(input));
			
			output
				.setTotalNotionalQuantityLeg2(totalNotionalQuantityOfLeg2.evaluate(input));
			
			output
				.setNotionalLeg2(notionalAmountOfLeg2.evaluate(input));
			
			output
				.setNotionalCurrency2(notionalCurrency2.evaluate(input));
			
			output
				.setNotionalAmountScheduleLeg2(notionalAmountScheduleLeg2.evaluate(input));
			
			output
				.setNotionalQuantityScheduleLeg2(notionalQuantityScheduleLeg2.evaluate(input));
			
			output
				.setOtherPayment(otherPayment.evaluate(input));
			
			output
				.setFixedRateOfLeg1(fixedRateOfLeg1OrCoupon.evaluate(input));
			
			output
				.setFixedRateDayCountConventionLeg1(fixedRateOrCouponDayCountConventionLeg1.evaluate(input));
			
			output
				.setFixedRatePaymentFrequencyPeriodLeg1(fixedRateOrCouponPaymentFrequencyPeriodLeg1.evaluate(input));
			
			output
				.setIdentifierOfFloatingRateOfLeg1(identifierOfFloatingRateOfLeg1.evaluate(input));
			
			output
				.setNameOfTheFloatingRateOfLeg1(nameOfTheFloatingRateOfLeg1.evaluate(input));
			
			output
				.setFloatingRateDayCountConventionOfLeg1(floatingRateDayCountConventionOfLeg1.evaluate(input));
			
			output
				.setFloatingRatePaymentFrequencyPeriodOfLeg1(floatingRatePaymentFrequencyPeriodOfLeg1.evaluate(input));
			
			output
				.setFloatingRateReferencePeriodOfLeg1TimePeriod(floatingRateReferencePeriodOfLeg1TimePeriod.evaluate(input));
			
			output
				.setFloatingRateResetFrequencyPeriodOfLeg1(floatingRateResetFrequencyPeriodOfLeg1.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadOfLeg1Monetary(spreadOfLeg1Monetary.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadOfLeg1Percentage(spreadOfLeg1Percentage.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadOfLeg1Basis(spreadOfLeg1Basis.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadCurrencyOfLeg1(spreadCurrencyOfLeg1.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadOfLeg1Notation(spreadOfLeg1Notation.evaluate(input));
			
			output
				.setFixedRateOfLeg2(fixedRateOfLeg2.evaluate(input));
			
			output
				.setFixedRateDayCountConventionLeg2(fixedRateDayCountConventionLeg2.evaluate(input));
			
			output
				.setFixedRatePaymentFrequencyPeriodLeg2(fixedRatePaymentFrequencyPeriodLeg2.evaluate(input));
			
			output
				.setIdentifierOfFloatingRateOfLeg2(identifierOfFloatingRateOfLeg2.evaluate(input));
			
			output
				.setFloatingRateDayCountConventionOfLeg2(floatingRateDayCountConventionOfLeg2.evaluate(input));
			
			output
				.setFloatingRatePaymentFrequencyPeriodOfLeg2(floatingRatePaymentFrequencyPeriodOfLeg2.evaluate(input));
			
			output
				.setFloatingRateReferencePeriodOfLeg2TimePeriod(floatingRateReferencePeriodOfLeg2TimePeriod.evaluate(input));
			
			output
				.setFloatingRateResetFrequencyPeriodOfLeg2(floatingRateResetFrequencyPeriodOfLeg2.evaluate(input));
			
			output
				.setExchangeRate(exchangeRate.evaluate(input));
			
			output
				.setForwardExchangeRate(forwardExchangeRate.evaluate(input));
			
			output
				.setExchangeRateBasis(exchangeRateBasis.evaluate(input));
			
			output
				.setFurtherSubProduct(furtherSubProduct.evaluate(input));
			
			output
				.setDeliveryPoint(deliveryPointOrZone.evaluate(input));
			
			output
				.setLoadType(loadType.evaluate(input));
			
			output
				.setDeliveryReport(delivery.evaluate(input));
			
			output
				.setOptionType(optionType.evaluate(input));
			
			output
				.setOptionStyle(optionStyle.evaluate(input));
			
			output
				.getOrCreateStrikePrice()
				.setStrikePriceMonetary(strikePriceMonetary.evaluate(input));
			
			output
				.getOrCreateStrikePrice()
				.setStrikePricePercentage(strikePricePercentage.evaluate(input));
			
			output
				.getOrCreateStrikePrice()
				.setStrikePriceCurrency(strikePriceCurrencyCurrencyPair.evaluate(input));
			
			output
				.getOrCreateStrikePrice()
				.setStrikePriceNotation(strikePriceNotation.evaluate(input));
			
			output
				.setStrikePriceSchedule(strikePriceSchedule.evaluate(input));
			
			output
				.setOptionPremiumAmount(optionPremiumAmount.evaluate(input));
			
			output
				.setOptionPremiumCurrency(optionPremiumCurrency.evaluate(input));
			
			output
				.setOptionPremiumPaymentDate(optionPremiumPaymentDate.evaluate(input));
			
			output
				.setMaturityDateOfTheUnderlying(maturityDateOfTheUnderlying.evaluate(input));
			
			output
				.setSeniority(seniority.evaluate(input));
			
			output
				.setReferenceEntity(referenceEntity.evaluate(input));
			
			output
				.setSeries(series.evaluate(input));
			
			output
				.setVersion(version.evaluate(input));
			
			output
				.setIndexFactor(indexFactor.evaluate(input));
			
			output
				.setTranche(tranche.evaluate(input));
			
			output
				.setCdSIndexAttachmentPoint(cDSIndexAttachmentPoint.evaluate(input));
			
			output
				.setCdSIndexDetachmentPoint(cDSIndexDetachmentPoint.evaluate(input));
			
			output
				.setCollateralPortfolioIndicator(collateralPortfolioIndicator.evaluate(input));
			
			output
				.setCollateralPortfolioCode(collateralPortfolioCode.evaluate(input));
			
			output
				.setActionType(actionType.evaluate(input));
			
			output
				.setEventType(eventType.evaluate(input));
			
			output
				.setLevel(level.evaluate(input));
			
			output
				.getOrCreateNonReportable()
				.setEnrichment(enrichmentData.evaluate(input));
			
			return Optional.ofNullable(output)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
