package drr.regulation.fca.ukemir.refit.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.fca.ukemir.refit.trade.NotionalAmountScheduleLeg2Report;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class NotionalAmountScheduleLeg2ReportTypeTabulator implements Tabulator<NotionalAmountScheduleLeg2Report> {
	private final Field amountField;
	private final Field effectiveDateField;
	
	public NotionalAmountScheduleLeg2ReportTypeTabulator() {
		this.amountField = new FieldImpl(
			"amount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "NotionalAmountInEffectOnAssociatedEffectiveDateOfLeg2")),
			Optional.of("2.66/2.68-$ 2.68 Notional amount in effect on associated effective date of leg 2"),
			Arrays.asList()
		);
		this.effectiveDateField = new FieldImpl(
			"effectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "EffectiveDateOfTheNotionalAmountOfLeg2")),
			Optional.of("2.66/2.68-$ 2.66 Effective date of the notional amount of leg 2"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(amountField, effectiveDateField);
	}
	
	@Override
	public List<FieldValue> tabulate(NotionalAmountScheduleLeg2Report input) {
		Optional<BigDecimal> amount = Optional.ofNullable(input.getAmount());
		Optional<Date> effectiveDate = Optional.ofNullable(input.getEffectiveDate());
		return Arrays.asList(
			new FieldValueImpl(amountField, amount),
			new FieldValueImpl(effectiveDateField, effectiveDate)
		);
	}
}
