package drr.regulation.fca.ukemir.refit.trade.reports;

import cdm.event.common.TransferState;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.functions.IsOtherPayment;
import drr.standards.iosco.cde.reports.CDEOtherPaymentPayerRule;
import javax.inject.Inject;


@ImplementedBy(OtherPaymentPayerRule.OtherPaymentPayerRuleDefault.class)
public abstract class OtherPaymentPayerRule implements ReportFunction<TransferState, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDEOtherPaymentPayerRule cDEOtherPaymentPayer;
	@Inject protected IsOtherPayment isOtherPayment;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransferState input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransferState input);

	public static class OtherPaymentPayerRuleDefault extends OtherPaymentPayerRule {
		@Override
		protected String doEvaluate(TransferState input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransferState input) {
			final MapperS<TransferState> thenResult = MapperS.of(input)
				.filterSingleNullSafe(item -> isOtherPayment.evaluate(item.get()));
			output = thenResult
				.mapSingleToItem(item -> MapperS.of(cDEOtherPaymentPayer.evaluate(item.get()))).get();
			
			return output;
		}
	}
}
