package drr.regulation.fca.ukemir.refit.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.fca.ukemir.refit.trade.StrikePriceAndNotation;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class StrikePriceAndNotationTypeTabulator implements Tabulator<StrikePriceAndNotation> {
	private final Field strikePriceMonetaryField;
	private final Field strikePricePercentageField;
	private final Field strikePriceCurrencyField;
	private final Field strikePriceNotationField;
	
	public StrikePriceAndNotationTypeTabulator() {
		this.strikePriceMonetaryField = new FieldImpl(
			"strikePriceMonetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "StrikePriceMonetary")),
			Optional.of("2.134 Strike Price (Monetary)"),
			Arrays.asList()
		);
		this.strikePricePercentageField = new FieldImpl(
			"strikePricePercentage",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "StrikePricePercentage")),
			Optional.of("2.134 Strike Price (Percentage)"),
			Arrays.asList()
		);
		this.strikePriceCurrencyField = new FieldImpl(
			"strikePriceCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "StrikePriceCurrencyCurrencyPair")),
			Optional.of("2.138 Strike price currency/currency pair"),
			Arrays.asList()
		);
		this.strikePriceNotationField = new FieldImpl(
			"strikePriceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "fca", "ukemir", "refit", "trade"), "StrikePriceNotation")),
			Optional.of("2.134 Strike Price Notation (Non Reportable)"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(strikePriceMonetaryField, strikePricePercentageField, strikePriceCurrencyField, strikePriceNotationField);
	}
	
	@Override
	public List<FieldValue> tabulate(StrikePriceAndNotation input) {
		Optional<BigDecimal> strikePriceMonetary = Optional.ofNullable(input.getStrikePriceMonetary());
		Optional<BigDecimal> strikePricePercentage = Optional.ofNullable(input.getStrikePricePercentage());
		Optional<String> strikePriceCurrency = Optional.ofNullable(input.getStrikePriceCurrency());
		Optional<PriceNotationEnum> strikePriceNotation = Optional.ofNullable(input.getStrikePriceNotation());
		return Arrays.asList(
			new FieldValueImpl(strikePriceMonetaryField, strikePriceMonetary),
			new FieldValueImpl(strikePricePercentageField, strikePricePercentage),
			new FieldValueImpl(strikePriceCurrencyField, strikePriceCurrency),
			new FieldValueImpl(strikePriceNotationField, strikePriceNotation)
		);
	}
}
