package drr.regulation.fca.ukemir.refit.trade.reports;

import cdm.product.qualification.functions.Qualify_Commodity_Option;
import cdm.product.qualification.functions.Qualify_Commodity_Swap_Basis;
import cdm.product.qualification.functions.Qualify_Commodity_Swap_FixedFloat;
import cdm.product.qualification.functions.Qualify_Commodity_Swaption;
import cdm.product.template.OptionPayout;
import cdm.product.template.Payout;
import cdm.product.template.Product;
import cdm.product.template.TradableProduct;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.CommodityLeg1;
import drr.regulation.common.functions.EconomicTermsForProduct;
import drr.regulation.common.functions.FixedPriceLeg1;
import drr.regulation.common.functions.IsAllowableAction;
import drr.regulation.common.functions.IsCommodity;
import drr.regulation.common.functions.IsCommodityCommodity;
import drr.regulation.common.functions.IsEquity;
import drr.regulation.common.functions.TradableProductForEvent;
import drr.regulation.common.functions.UnderlierForProduct;
import drr.standards.iosco.cde.reports.CDECommodityTotalNotionalQuantityRule;
import drr.standards.iosco.cde.reports.CDEEquityTotalNotionalQuantityRule;
import drr.standards.iosco.cde.reports.CDEFixedPriceTotalNotionalQuantityRule;
import drr.standards.iosco.cde.reports.CDEOptionTotalNotionalQuantityRule;
import java.math.BigDecimal;
import javax.inject.Inject;


@ImplementedBy(TotalNotionalQuantityOfLeg1Rule.TotalNotionalQuantityOfLeg1RuleDefault.class)
public abstract class TotalNotionalQuantityOfLeg1Rule implements ReportFunction<TransactionReportInstruction, BigDecimal> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDECommodityTotalNotionalQuantityRule cDECommodityTotalNotionalQuantity;
	@Inject protected CDEEquityTotalNotionalQuantityRule cDEEquityTotalNotionalQuantity;
	@Inject protected CDEFixedPriceTotalNotionalQuantityRule cDEFixedPriceTotalNotionalQuantity;
	@Inject protected CDEOptionTotalNotionalQuantityRule cDEOptionTotalNotionalQuantity;
	@Inject protected CommodityLeg1 commodityLeg1;
	@Inject protected EconomicTermsForProduct economicTermsForProduct;
	@Inject protected FixedPriceLeg1 fixedPriceLeg1;
	@Inject protected IsAllowableAction isAllowableAction;
	@Inject protected IsCommodity isCommodity;
	@Inject protected IsCommodityCommodity isCommodityCommodity;
	@Inject protected IsEquity isEquity;
	@Inject protected Qualify_Commodity_Option qualify_Commodity_Option;
	@Inject protected Qualify_Commodity_Swap_Basis qualify_Commodity_Swap_Basis;
	@Inject protected Qualify_Commodity_Swap_FixedFloat qualify_Commodity_Swap_FixedFloat;
	@Inject protected Qualify_Commodity_Swaption qualify_Commodity_Swaption;
	@Inject protected TradableProductForEvent tradableProductForEvent;
	@Inject protected UnderlierForProduct underlierForProduct;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public BigDecimal evaluate(TransactionReportInstruction input) {
		BigDecimal output = doEvaluate(input);
		
		return output;
	}

	protected abstract BigDecimal doEvaluate(TransactionReportInstruction input);

	public static class TotalNotionalQuantityOfLeg1RuleDefault extends TotalNotionalQuantityOfLeg1Rule {
		@Override
		protected BigDecimal doEvaluate(TransactionReportInstruction input) {
			BigDecimal output = null;
			return assignOutput(output, input);
		}
		
		protected BigDecimal assignOutput(BigDecimal output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult0 = MapperS.of(input)
				.filterSingleNullSafe(item -> isAllowableAction.evaluate(item.get()));
			final MapperS<TradableProduct> thenResult1 = thenResult0
				.mapSingleToItem(item -> MapperS.of(tradableProductForEvent.evaluate(item.get())));
			output = thenResult1
				.mapSingleToItem(item -> {
					final Boolean boolean0 = isEquity.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get());
					if ((boolean0 == null ? false : boolean0)) {
						return MapperS.of(cDEEquityTotalNotionalQuantity.evaluate(item.get()));
					}
					final Boolean boolean1 = isCommodityCommodity.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get());
					if ((boolean1 == null ? false : boolean1)) {
						return MapperS.of(cDECommodityTotalNotionalQuantity.evaluate(commodityLeg1.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get())));
					}
					final Boolean boolean2 = isCommodity.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get());
					if ((boolean2 == null ? false : boolean2)) {
						return MapperS.of(cDEFixedPriceTotalNotionalQuantity.evaluate(fixedPriceLeg1.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get())));
					}
					if (ComparisonResult.of(MapperS.of(qualify_Commodity_Swaption.evaluate(economicTermsForProduct.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get())))).and(ComparisonResult.of(MapperS.of(qualify_Commodity_Swap_Basis.evaluate(economicTermsForProduct.evaluate(underlierForProduct.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get())))))).getOrDefault(false)) {
						return MapperS.of(cDECommodityTotalNotionalQuantity.evaluate(commodityLeg1.evaluate(underlierForProduct.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get()))));
					}
					if (ComparisonResult.of(MapperS.of(qualify_Commodity_Swaption.evaluate(economicTermsForProduct.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get())))).and(ComparisonResult.of(MapperS.of(qualify_Commodity_Swap_FixedFloat.evaluate(economicTermsForProduct.evaluate(underlierForProduct.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get())))))).getOrDefault(false)) {
						return MapperS.of(cDEFixedPriceTotalNotionalQuantity.evaluate(fixedPriceLeg1.evaluate(underlierForProduct.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get()))));
					}
					final Boolean boolean3 = qualify_Commodity_Option.evaluate(economicTermsForProduct.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get()));
					if ((boolean3 == null ? false : boolean3)) {
						return MapperS.of(cDEOptionTotalNotionalQuantity.evaluate(MapperS.of(economicTermsForProduct.evaluate(item.<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<OptionPayout>mapC("getOptionPayout", payout -> payout.getOptionPayout()).get()));
					}
					return MapperS.<BigDecimal>ofNull();
				}).get();
			
			return output;
		}
	}
}
