package drr.regulation.fca.ukemir.refit.trade.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.fca.ukemir.refit.trade.NotionalAmountScheduleLeg1Report;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkNumber;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class NotionalAmountScheduleLeg1ReportTypeFormatValidator implements Validator<NotionalAmountScheduleLeg1Report> {

	private List<ComparisonResult> getComparisonResults(NotionalAmountScheduleLeg1Report o) {
		return Lists.<ComparisonResult>newArrayList(
				checkNumber("amount", o.getAmount(), of(25), of(5), empty(), empty())
			);
	}

	@Override
	public ValidationResult<NotionalAmountScheduleLeg1Report> validate(RosettaPath path, NotionalAmountScheduleLeg1Report o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("NotionalAmountScheduleLeg1Report", ValidationType.TYPE_FORMAT, "NotionalAmountScheduleLeg1Report", path, "", error);
		}
		return success("NotionalAmountScheduleLeg1Report", ValidationType.TYPE_FORMAT, "NotionalAmountScheduleLeg1Report", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, NotionalAmountScheduleLeg1Report o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("NotionalAmountScheduleLeg1Report", ValidationType.TYPE_FORMAT, "NotionalAmountScheduleLeg1Report", path, "", res.getError());
				}
				return success("NotionalAmountScheduleLeg1Report", ValidationType.TYPE_FORMAT, "NotionalAmountScheduleLeg1Report", path, "");
			})
			.collect(toList());
	}

}
