package drr.regulation.fca.ukemir.refit.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.fca.ukemir.refit.trade.PriceAndNotation;
import java.util.Arrays;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("PriceAndNotationMonetaryOrPercentage")
@ImplementedBy(PriceAndNotationMonetaryOrPercentage.Default.class)
public interface PriceAndNotationMonetaryOrPercentage extends Validator<PriceAndNotation> {
	
	String NAME = "PriceAndNotationMonetaryOrPercentage";
	String DEFINITION = "optional choice priceMonetary, pricePercentage";
	
	ValidationResult<PriceAndNotation> validate(RosettaPath path, PriceAndNotation priceAndNotation);
	
	class Default implements PriceAndNotationMonetaryOrPercentage {
	
		@Override
		public ValidationResult<PriceAndNotation> validate(RosettaPath path, PriceAndNotation priceAndNotation) {
			ComparisonResult result = executeDataRule(priceAndNotation);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PriceAndNotation", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "PriceAndNotation", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(PriceAndNotation priceAndNotation) {
			try {
				return choice(MapperS.of(priceAndNotation), Arrays.asList("priceMonetary", "pricePercentage"), ChoiceRuleValidationMethod.OPTIONAL);
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements PriceAndNotationMonetaryOrPercentage {
	
		@Override
		public ValidationResult<PriceAndNotation> validate(RosettaPath path, PriceAndNotation priceAndNotation) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PriceAndNotation", path, DEFINITION);
		}
	}
}
