package drr.regulation.fca.ukemir.refit.trade.validation.exists;

import com.google.common.collect.ImmutableMap;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import drr.regulation.fca.ukemir.refit.trade.NotionalAmountScheduleLeg2Report;
import java.math.BigDecimal;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class NotionalAmountScheduleLeg2ReportOnlyExistsValidator implements ValidatorWithArg<NotionalAmountScheduleLeg2Report, Set<String>> {

	/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
	@Override
	public <T2 extends NotionalAmountScheduleLeg2Report> ValidationResult<NotionalAmountScheduleLeg2Report> validate(RosettaPath path, T2 o, Set<String> fields) {
		Map<String, Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("amount", ExistenceChecker.isSet((BigDecimal) o.getAmount()))
				.put("effectiveDate", ExistenceChecker.isSet((Date) o.getEffectiveDate()))
				.build();
		
		// Find the fields that are set
		Set<String> setFields = fieldExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.map(Map.Entry::getKey)
				.collect(Collectors.toSet());
		
		if (setFields.equals(fields)) {
			return success("NotionalAmountScheduleLeg2Report", ValidationType.ONLY_EXISTS, "NotionalAmountScheduleLeg2Report", path, "");
		}
		return failure("NotionalAmountScheduleLeg2Report", ValidationType.ONLY_EXISTS, "NotionalAmountScheduleLeg2Report", path, "",
				String.format("[%s] should only be set.  Set fields: %s", fields, setFields));
	}
}
