package drr.regulation.jfsa.rewrite.margin.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.common.MarginActionEnum;
import drr.regulation.jfsa.rewrite.margin.CollateralReport;
import iso20022.auth108.esma.CollateralisationType3Code__1;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class CollateralReportTypeTabulator implements Tabulator<CollateralReport> {
	private final Field collateralPortfolioIndicatorField;
	private final Field utiField;
	private final Field collateralisationCategoryField;
	private final Field initialMarginCollateralPortfolioCodeField;
	private final Field initialMarginPostedByTheReportingCounterpartyPreHaircutField;
	private final Field initialMarginPostedByTheReportingCounterpartyPostHaircutField;
	private final Field currencyofInitialMarginPostedField;
	private final Field initialMarginCollectedByReportingCounterpartyPreHaircutField;
	private final Field initialMarginCollectedByReportingCounterpartyPostHaircutField;
	private final Field currencyofInitialMarginCollectedField;
	private final Field variationMarginCollateralPortfolioCodeField;
	private final Field variationMarginPostedByTheReportingCounterpartyPreHaircutField;
	private final Field variationMarginPostedByTheReportingCounterpartyPostHaircutField;
	private final Field currencyOfVariationMarginPostedField;
	private final Field variationMarginCollectedByTheReportingCounterpartyPreHaircutField;
	private final Field variationMarginCollectedByTheReportingCounterpartyPostHaircutField;
	private final Field currencyOfVariationMarginCollectedField;
	private final Field excessCollateralPostedByTheReportingCounterpartyField;
	private final Field currencyOfTheExcessCollateralPostedField;
	private final Field excessCollateralCollectedByTheReportingCounterpartyField;
	private final Field currencyOfTheExcessCollateralCollectedField;
	private final Field eventTimestampField;
	private final Field actionTypeField;
	
	public CollateralReportTypeTabulator() {
		this.collateralPortfolioIndicatorField = new FieldImpl(
			"collateralPortfolioIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "CollateralPortfolioIndicator")),
			Optional.of("44 Collateral portfolio indicator"),
			Arrays.asList()
		);
		this.utiField = new FieldImpl(
			"uti",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "UTI")),
			Optional.of("25 Unique transaction identifier (UTI)"),
			Arrays.asList()
		);
		this.collateralisationCategoryField = new FieldImpl(
			"collateralisationCategory",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "CollateralisationCategory")),
			Optional.of("61 Collateralisation category"),
			Arrays.asList()
		);
		this.initialMarginCollateralPortfolioCodeField = new FieldImpl(
			"initialMarginCollateralPortfolioCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "InitialMarginCollateralPortfolioCode")),
			Optional.of("62 Initial margin Collateral portfolio code"),
			Arrays.asList()
		);
		this.initialMarginPostedByTheReportingCounterpartyPreHaircutField = new FieldImpl(
			"initialMarginPostedByTheReportingCounterpartyPreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "InitialMarginPostedByTheReportingCounterpartyPreHaircut")),
			Optional.of("45 Initial margin posted by the reporting counterparty (pre-haircut)"),
			Arrays.asList()
		);
		this.initialMarginPostedByTheReportingCounterpartyPostHaircutField = new FieldImpl(
			"initialMarginPostedByTheReportingCounterpartyPostHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "InitialMarginPostedByTheReportingCounterpartyPostHaircut")),
			Optional.of("46 Initial margin posted by the reporting counterparty (post-haircut)"),
			Arrays.asList()
		);
		this.currencyofInitialMarginPostedField = new FieldImpl(
			"currencyofInitialMarginPosted",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "CurrencyofInitialMarginPosted")),
			Optional.of("47 Currency of the initial margins posted"),
			Arrays.asList()
		);
		this.initialMarginCollectedByReportingCounterpartyPreHaircutField = new FieldImpl(
			"initialMarginCollectedByReportingCounterpartyPreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "InitialMarginCollectedByReportingCounterpartyPreHaircut")),
			Optional.of("48 Initial margin collected by the reporting counterparty (pre-haircut)"),
			Arrays.asList()
		);
		this.initialMarginCollectedByReportingCounterpartyPostHaircutField = new FieldImpl(
			"initialMarginCollectedByReportingCounterpartyPostHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "InitialMarginCollectedByReportingCounterpartyPostHaircut")),
			Optional.of("49 Initial margin collected by the reporting counterparty (post-haircut)"),
			Arrays.asList()
		);
		this.currencyofInitialMarginCollectedField = new FieldImpl(
			"currencyofInitialMarginCollected",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "CurrencyofInitialMarginCollected")),
			Optional.of("50 Currency of initial margin collected"),
			Arrays.asList()
		);
		this.variationMarginCollateralPortfolioCodeField = new FieldImpl(
			"variationMarginCollateralPortfolioCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "VariationMarginCollateralPortfolioCode")),
			Optional.of("63 Variation Margin Collateral portfolio code"),
			Arrays.asList()
		);
		this.variationMarginPostedByTheReportingCounterpartyPreHaircutField = new FieldImpl(
			"variationMarginPostedByTheReportingCounterpartyPreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "VariationMarginPostedByTheReportingCounterpartyPreHaircut")),
			Optional.of("51 Variation margin posted by the reporting counterparty (pre-haircut)"),
			Arrays.asList()
		);
		this.variationMarginPostedByTheReportingCounterpartyPostHaircutField = new FieldImpl(
			"variationMarginPostedByTheReportingCounterpartyPostHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "VariationMarginPostedByTheReportingCounterpartyPostHaircut")),
			Optional.of("52 Variation margin posted by the reporting counterparty (post-haircut)"),
			Arrays.asList()
		);
		this.currencyOfVariationMarginPostedField = new FieldImpl(
			"currencyOfVariationMarginPosted",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "CurrencyOfVariationMarginPosted")),
			Optional.of("53 Currency of the variation margins posted"),
			Arrays.asList()
		);
		this.variationMarginCollectedByTheReportingCounterpartyPreHaircutField = new FieldImpl(
			"variationMarginCollectedByTheReportingCounterpartyPreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "VariationMarginCollectedByTheReportingCounterpartyPreHaircut")),
			Optional.of("54 Variation margin collected by the reporting counterparty (pre-haircut)"),
			Arrays.asList()
		);
		this.variationMarginCollectedByTheReportingCounterpartyPostHaircutField = new FieldImpl(
			"variationMarginCollectedByTheReportingCounterpartyPostHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "VariationMarginCollectedByTheReportingCounterpartyPostHaircut")),
			Optional.of("55 Variation margin collected by the reporting counterparty (post-haircut)"),
			Arrays.asList()
		);
		this.currencyOfVariationMarginCollectedField = new FieldImpl(
			"currencyOfVariationMarginCollected",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "CurrencyOfVariationMarginCollected")),
			Optional.of("56 Currency of variation margin collected"),
			Arrays.asList()
		);
		this.excessCollateralPostedByTheReportingCounterpartyField = new FieldImpl(
			"excessCollateralPostedByTheReportingCounterparty",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "ExcessCollateralPostedByTheReportingCounterparty")),
			Optional.of("57 Excess collateral posted by the reporting counterparty"),
			Arrays.asList()
		);
		this.currencyOfTheExcessCollateralPostedField = new FieldImpl(
			"currencyOfTheExcessCollateralPosted",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "CurrencyOfTheExcessCollateralPosted")),
			Optional.of("58 Currency of the excess collateral posted"),
			Arrays.asList()
		);
		this.excessCollateralCollectedByTheReportingCounterpartyField = new FieldImpl(
			"excessCollateralCollectedByTheReportingCounterparty",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "ExcessCollateralCollectedByTheReportingCounterparty")),
			Optional.of("59 Excess Collateral Collected By The reporting counterparty"),
			Arrays.asList()
		);
		this.currencyOfTheExcessCollateralCollectedField = new FieldImpl(
			"currencyOfTheExcessCollateralCollected",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "CurrencyOfExcessCollateralCollected")),
			Optional.of("60 Currency of excess collateral collected"),
			Arrays.asList()
		);
		this.eventTimestampField = new FieldImpl(
			"eventTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "EventTimestamp")),
			Optional.of("104 Event timestamp"),
			Arrays.asList()
		);
		this.actionTypeField = new FieldImpl(
			"actionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "ActionType")),
			Optional.of("101 Action Type"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(collateralPortfolioIndicatorField, utiField, collateralisationCategoryField, initialMarginCollateralPortfolioCodeField, initialMarginPostedByTheReportingCounterpartyPreHaircutField, initialMarginPostedByTheReportingCounterpartyPostHaircutField, currencyofInitialMarginPostedField, initialMarginCollectedByReportingCounterpartyPreHaircutField, initialMarginCollectedByReportingCounterpartyPostHaircutField, currencyofInitialMarginCollectedField, variationMarginCollateralPortfolioCodeField, variationMarginPostedByTheReportingCounterpartyPreHaircutField, variationMarginPostedByTheReportingCounterpartyPostHaircutField, currencyOfVariationMarginPostedField, variationMarginCollectedByTheReportingCounterpartyPreHaircutField, variationMarginCollectedByTheReportingCounterpartyPostHaircutField, currencyOfVariationMarginCollectedField, excessCollateralPostedByTheReportingCounterpartyField, currencyOfTheExcessCollateralPostedField, excessCollateralCollectedByTheReportingCounterpartyField, currencyOfTheExcessCollateralCollectedField, eventTimestampField, actionTypeField);
	}
	
	@Override
	public List<FieldValue> tabulate(CollateralReport input) {
		Optional<Boolean> collateralPortfolioIndicator = Optional.ofNullable(input.getCollateralPortfolioIndicator());
		Optional<String> uti = Optional.ofNullable(input.getUti());
		Optional<CollateralisationType3Code__1> collateralisationCategory = Optional.ofNullable(input.getCollateralisationCategory());
		Optional<String> initialMarginCollateralPortfolioCode = Optional.ofNullable(input.getInitialMarginCollateralPortfolioCode());
		Optional<BigDecimal> initialMarginPostedByTheReportingCounterpartyPreHaircut = Optional.ofNullable(input.getInitialMarginPostedByTheReportingCounterpartyPreHaircut());
		Optional<BigDecimal> initialMarginPostedByTheReportingCounterpartyPostHaircut = Optional.ofNullable(input.getInitialMarginPostedByTheReportingCounterpartyPostHaircut());
		Optional<ISOCurrencyCodeEnum> currencyofInitialMarginPosted = Optional.ofNullable(input.getCurrencyofInitialMarginPosted());
		Optional<BigDecimal> initialMarginCollectedByReportingCounterpartyPreHaircut = Optional.ofNullable(input.getInitialMarginCollectedByReportingCounterpartyPreHaircut());
		Optional<BigDecimal> initialMarginCollectedByReportingCounterpartyPostHaircut = Optional.ofNullable(input.getInitialMarginCollectedByReportingCounterpartyPostHaircut());
		Optional<ISOCurrencyCodeEnum> currencyofInitialMarginCollected = Optional.ofNullable(input.getCurrencyofInitialMarginCollected());
		Optional<String> variationMarginCollateralPortfolioCode = Optional.ofNullable(input.getVariationMarginCollateralPortfolioCode());
		Optional<BigDecimal> variationMarginPostedByTheReportingCounterpartyPreHaircut = Optional.ofNullable(input.getVariationMarginPostedByTheReportingCounterpartyPreHaircut());
		Optional<BigDecimal> variationMarginPostedByTheReportingCounterpartyPostHaircut = Optional.ofNullable(input.getVariationMarginPostedByTheReportingCounterpartyPostHaircut());
		Optional<ISOCurrencyCodeEnum> currencyOfVariationMarginPosted = Optional.ofNullable(input.getCurrencyOfVariationMarginPosted());
		Optional<BigDecimal> variationMarginCollectedByTheReportingCounterpartyPreHaircut = Optional.ofNullable(input.getVariationMarginCollectedByTheReportingCounterpartyPreHaircut());
		Optional<BigDecimal> variationMarginCollectedByTheReportingCounterpartyPostHaircut = Optional.ofNullable(input.getVariationMarginCollectedByTheReportingCounterpartyPostHaircut());
		Optional<ISOCurrencyCodeEnum> currencyOfVariationMarginCollected = Optional.ofNullable(input.getCurrencyOfVariationMarginCollected());
		Optional<BigDecimal> excessCollateralPostedByTheReportingCounterparty = Optional.ofNullable(input.getExcessCollateralPostedByTheReportingCounterparty());
		Optional<String> currencyOfTheExcessCollateralPosted = Optional.ofNullable(input.getCurrencyOfTheExcessCollateralPosted());
		Optional<BigDecimal> excessCollateralCollectedByTheReportingCounterparty = Optional.ofNullable(input.getExcessCollateralCollectedByTheReportingCounterparty());
		Optional<String> currencyOfTheExcessCollateralCollected = Optional.ofNullable(input.getCurrencyOfTheExcessCollateralCollected());
		Optional<ZonedDateTime> eventTimestamp = Optional.ofNullable(input.getEventTimestamp());
		Optional<MarginActionEnum> actionType = Optional.ofNullable(input.getActionType());
		return Arrays.asList(
			new FieldValueImpl(collateralPortfolioIndicatorField, collateralPortfolioIndicator),
			new FieldValueImpl(utiField, uti),
			new FieldValueImpl(collateralisationCategoryField, collateralisationCategory),
			new FieldValueImpl(initialMarginCollateralPortfolioCodeField, initialMarginCollateralPortfolioCode),
			new FieldValueImpl(initialMarginPostedByTheReportingCounterpartyPreHaircutField, initialMarginPostedByTheReportingCounterpartyPreHaircut),
			new FieldValueImpl(initialMarginPostedByTheReportingCounterpartyPostHaircutField, initialMarginPostedByTheReportingCounterpartyPostHaircut),
			new FieldValueImpl(currencyofInitialMarginPostedField, currencyofInitialMarginPosted),
			new FieldValueImpl(initialMarginCollectedByReportingCounterpartyPreHaircutField, initialMarginCollectedByReportingCounterpartyPreHaircut),
			new FieldValueImpl(initialMarginCollectedByReportingCounterpartyPostHaircutField, initialMarginCollectedByReportingCounterpartyPostHaircut),
			new FieldValueImpl(currencyofInitialMarginCollectedField, currencyofInitialMarginCollected),
			new FieldValueImpl(variationMarginCollateralPortfolioCodeField, variationMarginCollateralPortfolioCode),
			new FieldValueImpl(variationMarginPostedByTheReportingCounterpartyPreHaircutField, variationMarginPostedByTheReportingCounterpartyPreHaircut),
			new FieldValueImpl(variationMarginPostedByTheReportingCounterpartyPostHaircutField, variationMarginPostedByTheReportingCounterpartyPostHaircut),
			new FieldValueImpl(currencyOfVariationMarginPostedField, currencyOfVariationMarginPosted),
			new FieldValueImpl(variationMarginCollectedByTheReportingCounterpartyPreHaircutField, variationMarginCollectedByTheReportingCounterpartyPreHaircut),
			new FieldValueImpl(variationMarginCollectedByTheReportingCounterpartyPostHaircutField, variationMarginCollectedByTheReportingCounterpartyPostHaircut),
			new FieldValueImpl(currencyOfVariationMarginCollectedField, currencyOfVariationMarginCollected),
			new FieldValueImpl(excessCollateralPostedByTheReportingCounterpartyField, excessCollateralPostedByTheReportingCounterparty),
			new FieldValueImpl(currencyOfTheExcessCollateralPostedField, currencyOfTheExcessCollateralPosted),
			new FieldValueImpl(excessCollateralCollectedByTheReportingCounterpartyField, excessCollateralCollectedByTheReportingCounterparty),
			new FieldValueImpl(currencyOfTheExcessCollateralCollectedField, currencyOfTheExcessCollateralCollected),
			new FieldValueImpl(eventTimestampField, eventTimestamp),
			new FieldValueImpl(actionTypeField, actionType)
		);
	}
}
