package drr.regulation.jfsa.rewrite.margin.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaReport;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.CollateralReportInstruction;
import drr.regulation.jfsa.rewrite.margin.JFSAMarginReport;
import drr.regulation.jfsa.rewrite.margin.JFSAMarginReport.JFSAMarginReportBuilder;
import java.math.BigDecimal;
import java.util.Optional;
import javax.inject.Inject;


@RosettaReport(namespace="drr.regulation.jfsa.rewrite.margin", body="JFSA", corpusList={"Margin"})
@ImplementedBy(JFSAMarginReportFunction.JFSAMarginReportFunctionDefault.class)
public abstract class JFSAMarginReportFunction implements ReportFunction<CollateralReportInstruction, JFSAMarginReport> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ActionTypeRule actionType;
	@Inject protected CollateralPortfolioIndicatorRule collateralPortfolioIndicator;
	@Inject protected CollateralisationCategoryRule collateralisationCategory;
	@Inject protected Counterparty1Rule counterparty1;
	@Inject protected Counterparty2Rule counterparty2;
	@Inject protected Counterparty2IdentifierTypeRule counterparty2IdentifierType;
	@Inject protected CurrencyOfExcessCollateralCollectedRule currencyOfExcessCollateralCollected;
	@Inject protected CurrencyOfTheExcessCollateralPostedRule currencyOfTheExcessCollateralPosted;
	@Inject protected CurrencyOfVariationMarginCollectedRule currencyOfVariationMarginCollected;
	@Inject protected CurrencyOfVariationMarginPostedRule currencyOfVariationMarginPosted;
	@Inject protected CurrencyofInitialMarginCollectedRule currencyofInitialMarginCollected;
	@Inject protected CurrencyofInitialMarginPostedRule currencyofInitialMarginPosted;
	@Inject protected EntityResponsibleForReportingRule entityResponsibleForReporting;
	@Inject protected EventTimestampRule eventTimestamp;
	@Inject protected ExcessCollateralCollectedByTheReportingCounterpartyRule excessCollateralCollectedByTheReportingCounterparty;
	@Inject protected ExcessCollateralPostedByTheReportingCounterpartyRule excessCollateralPostedByTheReportingCounterparty;
	@Inject protected InitialMarginCollateralPortfolioCodeRule initialMarginCollateralPortfolioCode;
	@Inject protected InitialMarginCollectedByReportingCounterpartyPostHaircutRule initialMarginCollectedByReportingCounterpartyPostHaircut;
	@Inject protected InitialMarginCollectedByReportingCounterpartyPreHaircutRule initialMarginCollectedByReportingCounterpartyPreHaircut;
	@Inject protected InitialMarginPostedByTheReportingCounterpartyPostHaircutRule initialMarginPostedByTheReportingCounterpartyPostHaircut;
	@Inject protected InitialMarginPostedByTheReportingCounterpartyPreHaircutRule initialMarginPostedByTheReportingCounterpartyPreHaircut;
	@Inject protected ReportingTimestampRule reportingTimestamp;
	@Inject protected SubmitterIdentifierRule submitterIdentifier;
	@Inject protected UTIRule uTI;
	@Inject protected VariationMarginCollateralPortfolioCodeRule variationMarginCollateralPortfolioCode;
	@Inject protected VariationMarginCollectedByTheReportingCounterpartyPostHaircutRule variationMarginCollectedByTheReportingCounterpartyPostHaircut;
	@Inject protected VariationMarginCollectedByTheReportingCounterpartyPreHaircutRule variationMarginCollectedByTheReportingCounterpartyPreHaircut;
	@Inject protected VariationMarginPostedByTheReportingCounterpartyPostHaircutRule variationMarginPostedByTheReportingCounterpartyPostHaircut;
	@Inject protected VariationMarginPostedByTheReportingCounterpartyPreHaircutRule variationMarginPostedByTheReportingCounterpartyPreHaircut;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public JFSAMarginReport evaluate(CollateralReportInstruction input) {
		JFSAMarginReport.JFSAMarginReportBuilder outputBuilder = doEvaluate(input);
		
		final JFSAMarginReport output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.build();
			objectValidator.validate(JFSAMarginReport.class, output);
		}
		
		return output;
	}

	protected abstract JFSAMarginReport.JFSAMarginReportBuilder doEvaluate(CollateralReportInstruction input);

	public static class JFSAMarginReportFunctionDefault extends JFSAMarginReportFunction {
		@Override
		protected JFSAMarginReport.JFSAMarginReportBuilder doEvaluate(CollateralReportInstruction input) {
			JFSAMarginReport.JFSAMarginReportBuilder output = JFSAMarginReport.builder();
			return assignOutput(output, input);
		}
		
		protected JFSAMarginReport.JFSAMarginReportBuilder assignOutput(JFSAMarginReport.JFSAMarginReportBuilder output, CollateralReportInstruction input) {
			output
				.getOrCreatePartiesToTheDerivative()
				.setReportingTimestamp(reportingTimestamp.evaluate(input));
			
			output
				.getOrCreatePartiesToTheDerivative()
				.setSubmitterIdentifier(submitterIdentifier.evaluate(input));
			
			output
				.getOrCreatePartiesToTheDerivative()
				.setEntityResponsibleForReporting(entityResponsibleForReporting.evaluate(input));
			
			output
				.getOrCreatePartiesToTheDerivative()
				.setCounterparty1(counterparty1.evaluate(input));
			
			output
				.getOrCreatePartiesToTheDerivative()
				.setCounterparty2IdentifierType(counterparty2IdentifierType.evaluate(input));
			
			output
				.getOrCreatePartiesToTheDerivative()
				.setCounterparty2(counterparty2.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setCollateralPortfolioIndicator(collateralPortfolioIndicator.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setUti(uTI.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setCollateralisationCategory(collateralisationCategory.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setInitialMarginCollateralPortfolioCode(initialMarginCollateralPortfolioCode.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setInitialMarginPostedByTheReportingCounterpartyPreHaircut(initialMarginPostedByTheReportingCounterpartyPreHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setInitialMarginPostedByTheReportingCounterpartyPostHaircut(initialMarginPostedByTheReportingCounterpartyPostHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setCurrencyofInitialMarginPosted(currencyofInitialMarginPosted.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setInitialMarginCollectedByReportingCounterpartyPreHaircut(initialMarginCollectedByReportingCounterpartyPreHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setInitialMarginCollectedByReportingCounterpartyPostHaircut(initialMarginCollectedByReportingCounterpartyPostHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setCurrencyofInitialMarginCollected(currencyofInitialMarginCollected.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setVariationMarginCollateralPortfolioCode(variationMarginCollateralPortfolioCode.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setVariationMarginPostedByTheReportingCounterpartyPreHaircut(variationMarginPostedByTheReportingCounterpartyPreHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setVariationMarginPostedByTheReportingCounterpartyPostHaircut(variationMarginPostedByTheReportingCounterpartyPostHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setCurrencyOfVariationMarginPosted(currencyOfVariationMarginPosted.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setVariationMarginCollectedByTheReportingCounterpartyPreHaircut(variationMarginCollectedByTheReportingCounterpartyPreHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setVariationMarginCollectedByTheReportingCounterpartyPostHaircut(variationMarginCollectedByTheReportingCounterpartyPostHaircut.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setCurrencyOfVariationMarginCollected(currencyOfVariationMarginCollected.evaluate(input));
			
			final Integer integer0 = excessCollateralPostedByTheReportingCounterparty.evaluate(input);
			output
				.getOrCreateCollateral()
				.setExcessCollateralPostedByTheReportingCounterparty((integer0 == null ? null : BigDecimal.valueOf(integer0)));
			
			output
				.getOrCreateCollateral()
				.setCurrencyOfTheExcessCollateralPosted(currencyOfTheExcessCollateralPosted.evaluate(input));
			
			final Integer integer1 = excessCollateralCollectedByTheReportingCounterparty.evaluate(input);
			output
				.getOrCreateCollateral()
				.setExcessCollateralCollectedByTheReportingCounterparty((integer1 == null ? null : BigDecimal.valueOf(integer1)));
			
			output
				.getOrCreateCollateral()
				.setCurrencyOfTheExcessCollateralCollected(currencyOfExcessCollateralCollected.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setEventTimestamp(eventTimestamp.evaluate(input));
			
			output
				.getOrCreateCollateral()
				.setActionType(actionType.evaluate(input));
			
			return Optional.ofNullable(output)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
