package drr.regulation.jfsa.rewrite.margin.reports;

import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.NestedFieldValueImpl;
import drr.regulation.jfsa.rewrite.margin.JFSAMarginReport;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import javax.inject.Inject;


public class JFSAMarginReportTypeTabulator implements Tabulator<JFSAMarginReport> {
	private final Field partiesToTheDerivativeField;
	private final Field collateralField;
	
	private final PartiesToTheDerivativeTypeTabulator partiesToTheDerivativeTypeTabulator;
	private final CollateralReportTypeTabulator collateralReportTypeTabulator;
	
	@Inject
	public JFSAMarginReportTypeTabulator(PartiesToTheDerivativeTypeTabulator partiesToTheDerivativeTypeTabulator, CollateralReportTypeTabulator collateralReportTypeTabulator) {
		this.partiesToTheDerivativeTypeTabulator = partiesToTheDerivativeTypeTabulator;
		this.collateralReportTypeTabulator = collateralReportTypeTabulator;
		this.partiesToTheDerivativeField = new FieldImpl(
			"partiesToTheDerivative",
			false,
			Optional.empty(),
			Optional.empty(),
			partiesToTheDerivativeTypeTabulator.getFields()
		);
		this.collateralField = new FieldImpl(
			"collateral",
			false,
			Optional.empty(),
			Optional.empty(),
			collateralReportTypeTabulator.getFields()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(partiesToTheDerivativeField, collateralField);
	}
	
	@Override
	public List<FieldValue> tabulate(JFSAMarginReport input) {
		Optional<List<FieldValue>> partiesToTheDerivative = Optional.ofNullable(input.getPartiesToTheDerivative())
			.map(x -> partiesToTheDerivativeTypeTabulator.tabulate(x));
		Optional<List<FieldValue>> collateral = Optional.ofNullable(input.getCollateral())
			.map(x -> collateralReportTypeTabulator.tabulate(x));
		return Arrays.asList(
			new NestedFieldValueImpl(partiesToTheDerivativeField, partiesToTheDerivative),
			new NestedFieldValueImpl(collateralField, collateral)
		);
	}
}
