package drr.regulation.jfsa.rewrite.margin.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.jfsa.rewrite.margin.PartiesToTheDerivative;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class PartiesToTheDerivativeTypeTabulator implements Tabulator<PartiesToTheDerivative> {
	private final Field reportingTimestampField;
	private final Field submitterIdentifierField;
	private final Field entityResponsibleForReportingField;
	private final Field counterparty1Field;
	private final Field counterparty2IdentifierTypeField;
	private final Field counterparty2Field;
	
	public PartiesToTheDerivativeTypeTabulator() {
		this.reportingTimestampField = new FieldImpl(
			"reportingTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "ReportingTimestamp")),
			Optional.of("4 Reporting Timestamp"),
			Arrays.asList()
		);
		this.submitterIdentifierField = new FieldImpl(
			"submitterIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "SubmitterIdentifier")),
			Optional.of("12 Submitter Identifier"),
			Arrays.asList()
		);
		this.entityResponsibleForReportingField = new FieldImpl(
			"entityResponsibleForReporting",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "EntityResponsibleForReporting")),
			Optional.of("6 Entity responsible for reporting"),
			Arrays.asList()
		);
		this.counterparty1Field = new FieldImpl(
			"counterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "Counterparty1")),
			Optional.of("7 Counterparty 1 (reporting counterparty)"),
			Arrays.asList()
		);
		this.counterparty2IdentifierTypeField = new FieldImpl(
			"counterparty2IdentifierType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "Counterparty2IdentifierType")),
			Optional.of("9 Counterparty 2 identifier type"),
			Arrays.asList()
		);
		this.counterparty2Field = new FieldImpl(
			"counterparty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "margin"), "Counterparty2")),
			Optional.of("8 Counterparty 2"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(reportingTimestampField, submitterIdentifierField, entityResponsibleForReportingField, counterparty1Field, counterparty2IdentifierTypeField, counterparty2Field);
	}
	
	@Override
	public List<FieldValue> tabulate(PartiesToTheDerivative input) {
		Optional<ZonedDateTime> reportingTimestamp = Optional.ofNullable(input.getReportingTimestamp());
		Optional<String> submitterIdentifier = Optional.ofNullable(input.getSubmitterIdentifier());
		Optional<String> entityResponsibleForReporting = Optional.ofNullable(input.getEntityResponsibleForReporting());
		Optional<String> counterparty1 = Optional.ofNullable(input.getCounterparty1());
		Optional<Boolean> counterparty2IdentifierType = Optional.ofNullable(input.getCounterparty2IdentifierType());
		Optional<String> counterparty2 = Optional.ofNullable(input.getCounterparty2());
		return Arrays.asList(
			new FieldValueImpl(reportingTimestampField, reportingTimestamp),
			new FieldValueImpl(submitterIdentifierField, submitterIdentifier),
			new FieldValueImpl(entityResponsibleForReportingField, entityResponsibleForReporting),
			new FieldValueImpl(counterparty1Field, counterparty1),
			new FieldValueImpl(counterparty2IdentifierTypeField, counterparty2IdentifierType),
			new FieldValueImpl(counterparty2Field, counterparty2)
		);
	}
}
