package drr.regulation.jfsa.rewrite.margin.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.MarginActionEnum;
import drr.regulation.jfsa.rewrite.margin.CollateralReport;
import drr.regulation.jfsa.rewrite.margin.JFSAMarginReport;
import drr.regulation.jfsa.rewrite.margin.PartiesToTheDerivative;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSAMarginReportDTCC_JFSA_BR_0008_01")
@ImplementedBy(JFSAMarginReportDTCCJFSABR000801.Default.class)
public interface JFSAMarginReportDTCCJFSABR000801 extends Validator<JFSAMarginReport> {
	
	String NAME = "JFSAMarginReportDTCC_JFSA_BR_0008_01";
	String DEFINITION = "if collateral -> actionType = MarginActionEnum -> MARU then (if partiesToTheDerivative -> counterparty2IdentifierType = True then partiesToTheDerivative -> counterparty2 <> partiesToTheDerivative -> counterparty1)";
	
	ValidationResult<JFSAMarginReport> validate(RosettaPath path, JFSAMarginReport jFSAMarginReport);
	
	class Default implements JFSAMarginReportDTCCJFSABR000801 {
	
		@Override
		public ValidationResult<JFSAMarginReport> validate(RosettaPath path, JFSAMarginReport jFSAMarginReport) {
			ComparisonResult result = executeDataRule(jFSAMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSAMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSAMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSAMarginReport jFSAMarginReport) {
			try {
				if (areEqual(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<MarginActionEnum>map("getActionType", collateralReport -> collateralReport.getActionType()), MapperS.of(MarginActionEnum.MARU), CardinalityOperator.All).getOrDefault(false)) {
					if (areEqual(MapperS.of(jFSAMarginReport).<PartiesToTheDerivative>map("getPartiesToTheDerivative", _jFSAMarginReport -> _jFSAMarginReport.getPartiesToTheDerivative()).<Boolean>map("getCounterparty2IdentifierType", partiesToTheDerivative -> partiesToTheDerivative.getCounterparty2IdentifierType()), MapperS.of(true), CardinalityOperator.All).getOrDefault(false)) {
						return notEqual(MapperS.of(jFSAMarginReport).<PartiesToTheDerivative>map("getPartiesToTheDerivative", _jFSAMarginReport -> _jFSAMarginReport.getPartiesToTheDerivative()).<String>map("getCounterparty2", partiesToTheDerivative -> partiesToTheDerivative.getCounterparty2()), MapperS.of(jFSAMarginReport).<PartiesToTheDerivative>map("getPartiesToTheDerivative", _jFSAMarginReport -> _jFSAMarginReport.getPartiesToTheDerivative()).<String>map("getCounterparty1", partiesToTheDerivative -> partiesToTheDerivative.getCounterparty1()), CardinalityOperator.Any);
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSAMarginReportDTCCJFSABR000801 {
	
		@Override
		public ValidationResult<JFSAMarginReport> validate(RosettaPath path, JFSAMarginReport jFSAMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSAMarginReport", path, DEFINITION);
		}
	}
}
