package drr.regulation.jfsa.rewrite.margin.validation.datarule;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.MarginActionEnum;
import drr.regulation.jfsa.rewrite.margin.CollateralReport;
import drr.regulation.jfsa.rewrite.margin.JFSAMarginReport;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSAMarginReportDTCC_JFSA_BR_0047_01")
@ImplementedBy(JFSAMarginReportDTCCJFSABR004701.Default.class)
public interface JFSAMarginReportDTCCJFSABR004701 extends Validator<JFSAMarginReport> {
	
	String NAME = "JFSAMarginReportDTCC_JFSA_BR_0047_01";
	String DEFINITION = "if collateral -> actionType = MarginActionEnum -> MARU then (if collateral -> initialMarginPostedByTheReportingCounterpartyPreHaircut exists or collateral -> initialMarginPostedByTheReportingCounterpartyPostHaircut exists then collateral -> currencyofInitialMarginPosted exists)";
	
	ValidationResult<JFSAMarginReport> validate(RosettaPath path, JFSAMarginReport jFSAMarginReport);
	
	class Default implements JFSAMarginReportDTCCJFSABR004701 {
	
		@Override
		public ValidationResult<JFSAMarginReport> validate(RosettaPath path, JFSAMarginReport jFSAMarginReport) {
			ComparisonResult result = executeDataRule(jFSAMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSAMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSAMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSAMarginReport jFSAMarginReport) {
			try {
				if (areEqual(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<MarginActionEnum>map("getActionType", collateralReport -> collateralReport.getActionType()), MapperS.of(MarginActionEnum.MARU), CardinalityOperator.All).getOrDefault(false)) {
					if (exists(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<BigDecimal>map("getInitialMarginPostedByTheReportingCounterpartyPreHaircut", collateralReport -> collateralReport.getInitialMarginPostedByTheReportingCounterpartyPreHaircut())).or(exists(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<BigDecimal>map("getInitialMarginPostedByTheReportingCounterpartyPostHaircut", collateralReport -> collateralReport.getInitialMarginPostedByTheReportingCounterpartyPostHaircut()))).getOrDefault(false)) {
						return exists(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<ISOCurrencyCodeEnum>map("getCurrencyofInitialMarginPosted", collateralReport -> collateralReport.getCurrencyofInitialMarginPosted()));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSAMarginReportDTCCJFSABR004701 {
	
		@Override
		public ValidationResult<JFSAMarginReport> validate(RosettaPath path, JFSAMarginReport jFSAMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSAMarginReport", path, DEFINITION);
		}
	}
}
