package drr.regulation.jfsa.rewrite.margin.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.MarginActionEnum;
import drr.regulation.jfsa.rewrite.margin.CollateralReport;
import drr.regulation.jfsa.rewrite.margin.JFSAMarginReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSAMarginReportDTCC_JFSA_BR_0063_03")
@ImplementedBy(JFSAMarginReportDTCCJFSABR006303.Default.class)
public interface JFSAMarginReportDTCCJFSABR006303 extends Validator<JFSAMarginReport> {
	
	String NAME = "JFSAMarginReportDTCC_JFSA_BR_0063_03";
	String DEFINITION = "if collateral -> actionType = MarginActionEnum -> MARU then (if collateral -> uti is absent and collateral -> initialMarginCollateralPortfolioCode = \"NOAP\" then collateral -> variationMarginCollateralPortfolioCode exists else if collateral -> collateralPortfolioIndicator = False then collateral -> variationMarginCollateralPortfolioCode is absent else if collateral -> uti exists and collateral -> variationMarginCollateralPortfolioCode = \"NOAP\" then False)";
	
	ValidationResult<JFSAMarginReport> validate(RosettaPath path, JFSAMarginReport jFSAMarginReport);
	
	class Default implements JFSAMarginReportDTCCJFSABR006303 {
	
		@Override
		public ValidationResult<JFSAMarginReport> validate(RosettaPath path, JFSAMarginReport jFSAMarginReport) {
			ComparisonResult result = executeDataRule(jFSAMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSAMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSAMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSAMarginReport jFSAMarginReport) {
			try {
				if (areEqual(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<MarginActionEnum>map("getActionType", collateralReport -> collateralReport.getActionType()), MapperS.of(MarginActionEnum.MARU), CardinalityOperator.All).getOrDefault(false)) {
					if (notExists(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<String>map("getUti", collateralReport -> collateralReport.getUti())).and(areEqual(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<String>map("getInitialMarginCollateralPortfolioCode", collateralReport -> collateralReport.getInitialMarginCollateralPortfolioCode()), MapperS.of("NOAP"), CardinalityOperator.All)).getOrDefault(false)) {
						return exists(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<String>map("getVariationMarginCollateralPortfolioCode", collateralReport -> collateralReport.getVariationMarginCollateralPortfolioCode()));
					}
					if (areEqual(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<Boolean>map("getCollateralPortfolioIndicator", collateralReport -> collateralReport.getCollateralPortfolioIndicator()), MapperS.of(false), CardinalityOperator.All).getOrDefault(false)) {
						return notExists(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<String>map("getVariationMarginCollateralPortfolioCode", collateralReport -> collateralReport.getVariationMarginCollateralPortfolioCode()));
					}
					if (exists(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<String>map("getUti", collateralReport -> collateralReport.getUti())).and(areEqual(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<String>map("getVariationMarginCollateralPortfolioCode", collateralReport -> collateralReport.getVariationMarginCollateralPortfolioCode()), MapperS.of("NOAP"), CardinalityOperator.All)).getOrDefault(false)) {
						return ComparisonResult.of(MapperS.of(false));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSAMarginReportDTCCJFSABR006303 {
	
		@Override
		public ValidationResult<JFSAMarginReport> validate(RosettaPath path, JFSAMarginReport jFSAMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSAMarginReport", path, DEFINITION);
		}
	}
}
