package drr.regulation.jfsa.rewrite.margin.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.MarginActionEnum;
import drr.regulation.jfsa.rewrite.margin.CollateralReport;
import drr.regulation.jfsa.rewrite.margin.JFSAMarginReport;
import drr.regulation.jfsa.rewrite.margin.PartiesToTheDerivative;
import java.time.ZonedDateTime;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSAMarginReportDTCC_JFSA_BR_0104_04")
@ImplementedBy(JFSAMarginReportDTCCJFSABR010404.Default.class)
public interface JFSAMarginReportDTCCJFSABR010404 extends Validator<JFSAMarginReport> {
	
	String NAME = "JFSAMarginReportDTCC_JFSA_BR_0104_04";
	String DEFINITION = "if collateral -> actionType = MarginActionEnum -> MARU then (collateral -> eventTimestamp <= partiesToTheDerivative -> reportingTimestamp)";
	
	ValidationResult<JFSAMarginReport> validate(RosettaPath path, JFSAMarginReport jFSAMarginReport);
	
	class Default implements JFSAMarginReportDTCCJFSABR010404 {
	
		@Override
		public ValidationResult<JFSAMarginReport> validate(RosettaPath path, JFSAMarginReport jFSAMarginReport) {
			ComparisonResult result = executeDataRule(jFSAMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSAMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSAMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSAMarginReport jFSAMarginReport) {
			try {
				if (areEqual(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<MarginActionEnum>map("getActionType", collateralReport -> collateralReport.getActionType()), MapperS.of(MarginActionEnum.MARU), CardinalityOperator.All).getOrDefault(false)) {
					return lessThanEquals(MapperS.of(jFSAMarginReport).<CollateralReport>map("getCollateral", _jFSAMarginReport -> _jFSAMarginReport.getCollateral()).<ZonedDateTime>map("getEventTimestamp", collateralReport -> collateralReport.getEventTimestamp()), MapperS.of(jFSAMarginReport).<PartiesToTheDerivative>map("getPartiesToTheDerivative", _jFSAMarginReport -> _jFSAMarginReport.getPartiesToTheDerivative()).<ZonedDateTime>map("getReportingTimestamp", partiesToTheDerivative -> partiesToTheDerivative.getReportingTimestamp()), CardinalityOperator.All);
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSAMarginReportDTCCJFSABR010404 {
	
		@Override
		public ValidationResult<JFSAMarginReport> validate(RosettaPath path, JFSAMarginReport jFSAMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSAMarginReport", path, DEFINITION);
		}
	}
}
