package drr.regulation.jfsa.rewrite.trade.meta;

import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyFunctionFactory;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorFactory;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;
import drr.regulation.jfsa.rewrite.trade.validation.JFSATransactionReportTypeFormatValidator;
import drr.regulation.jfsa.rewrite.trade.validation.JFSATransactionReportValidator;
import drr.regulation.jfsa.rewrite.trade.validation.exists.JFSATransactionReportOnlyExistsValidator;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Function;


/**
 * @version 4.2.1
 */
@RosettaMeta(model=JFSATransactionReport.class)
public class JFSATransactionReportMeta implements RosettaMetaData<JFSATransactionReport> {

	@Override
	public List<Validator<? super JFSATransactionReport>> dataRules(ValidatorFactory factory) {
		return Arrays.asList(
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportExpirationDate.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR000301.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR000401.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR000402.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR000701.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR000801.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR001001.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR001101.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR001102.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR001301.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR001601.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR001701.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR0019a01.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportFinalContractualSettlementDate.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportSettlementCurrencyLeg1.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportClearingReceiptTimestamp.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR002402.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportUti.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportPriorUti.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportFixedRateDayCountConventionLeg1.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportFloatingRateDayCountConventionOfLeg1.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportFixedRateDayCountConventionLeg2.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportFloatingRateDayCountConventionOfLeg2.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportFixedRatePaymentFrequencyPeriodLeg1.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportFloatingRatePaymentFrequencyPeriodOfLeg1.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportFixedRatePaymentFrequencyPeriodLeg2.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportFloatingRatePaymentFrequencyPeriodOfLeg2.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR002901.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR002902.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR002903.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR002904.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportFloatingRateResetFrequencyPeriodofLeg1.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportFloatingRateResetFrequencyPeriodLeg2.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportFloatingRateResetFrequencyPeriodMultiplierLeg1.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportFloatingRateResetFrequencyPeriodMultiplierLeg2.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR003301.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR003302.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR003401.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR003501.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR003601.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR003701.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR003801.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR006201.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR006301.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR006302.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR006401.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR006501.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR006601.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR006801.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR006901.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR007001.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR007101.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR007102.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR007301.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR007302.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR007401.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR007402.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR007601.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR007701.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR007801.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR007901.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR008001.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR008101.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR008201.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR008301.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR008701.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR008702.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportNotionalCurrencyLeg1.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportNotionalCurrencyLeg2.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportCallCurrencyCondition.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportPutCurrencyCondition.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR009301.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR009302.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR009401.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR009402.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR009501.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR009502.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR009601.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR009602.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR009801.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR009802.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR009901.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR009902.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR010001.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR010002.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR010101.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR010201.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR010301.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR010401.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR010402.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR010403.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR011001.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR011002.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR011003.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR011004.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR011101.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR011102.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportUnderlyingIndexIndicator.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR011301.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR011302.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR011401.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportIndicatorOfTheFloatingRateOfLeg1.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportIndicatorOfTheFloatingRateOfLeg2.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportNameofthefloatingrateofleg1.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportNameofthefloatingrateofleg2.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR011701.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR011702.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR011801.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR011802.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR012401.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR012501.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR012701.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR012801.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR013301.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR013401.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR013501.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR013601.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR013701.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.JFSATransactionReportDTCCJFSABR013801.class)
		);
	}
	
	@Override
	public List<Function<? super JFSATransactionReport, QualifyResult>> getQualifyFunctions(QualifyFunctionFactory factory) {
		return Collections.emptyList();
	}

	@Override
	public Validator<? super JFSATransactionReport> validator() {
		return new JFSATransactionReportValidator();
	}

	@Override
	public Validator<? super JFSATransactionReport> typeFormatValidator() {
		return new JFSATransactionReportTypeFormatValidator();
	}
	
	@Override
	public ValidatorWithArg<? super JFSATransactionReport, Set<String>> onlyExistsValidator() {
		return new JFSATransactionReportOnlyExistsValidator();
	}
}
