package drr.regulation.jfsa.rewrite.trade.meta;

import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyFunctionFactory;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorFactory;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import drr.regulation.jfsa.rewrite.trade.StrikePriceScheduleReport;
import drr.regulation.jfsa.rewrite.trade.validation.StrikePriceScheduleReportTypeFormatValidator;
import drr.regulation.jfsa.rewrite.trade.validation.StrikePriceScheduleReportValidator;
import drr.regulation.jfsa.rewrite.trade.validation.exists.StrikePriceScheduleReportOnlyExistsValidator;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Function;


/**
 * @version 4.2.1
 */
@RosettaMeta(model=StrikePriceScheduleReport.class)
public class StrikePriceScheduleReportMeta implements RosettaMetaData<StrikePriceScheduleReport> {

	@Override
	public List<Validator<? super StrikePriceScheduleReport>> dataRules(ValidatorFactory factory) {
		return Arrays.asList(
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.StrikePriceScheduleReportMonetaryOrDecimal.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.StrikePriceScheduleReportMonetary.class),
			factory.create(drr.regulation.jfsa.rewrite.trade.validation.datarule.StrikePriceScheduleReportDecimal.class)
		);
	}
	
	@Override
	public List<Function<? super StrikePriceScheduleReport, QualifyResult>> getQualifyFunctions(QualifyFunctionFactory factory) {
		return Collections.emptyList();
	}

	@Override
	public Validator<? super StrikePriceScheduleReport> validator() {
		return new StrikePriceScheduleReportValidator();
	}

	@Override
	public Validator<? super StrikePriceScheduleReport> typeFormatValidator() {
		return new StrikePriceScheduleReportTypeFormatValidator();
	}
	
	@Override
	public ValidatorWithArg<? super StrikePriceScheduleReport, Set<String>> onlyExistsValidator() {
		return new StrikePriceScheduleReportOnlyExistsValidator();
	}
}
