package drr.regulation.jfsa.rewrite.trade.reports;

import cdm.base.staticdata.asset.common.ProductIdTypeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.standards.iosco.cde.reports.CDESourceOfTheIdentifierOfTheBasketConstituentsRule;
import java.util.ArrayList;
import java.util.List;
import javax.inject.Inject;


@ImplementedBy(BasketConstituentIdentifierSourceRule.BasketConstituentIdentifierSourceRuleDefault.class)
public abstract class BasketConstituentIdentifierSourceRule implements ReportFunction<TransactionReportInstruction, List<ProductIdTypeEnum>> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDESourceOfTheIdentifierOfTheBasketConstituentsRule cDESourceOfTheIdentifierOfTheBasketConstituents;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public List<ProductIdTypeEnum> evaluate(TransactionReportInstruction input) {
		List<ProductIdTypeEnum> output = doEvaluate(input);
		
		return output;
	}

	protected abstract List<ProductIdTypeEnum> doEvaluate(TransactionReportInstruction input);

	public static class BasketConstituentIdentifierSourceRuleDefault extends BasketConstituentIdentifierSourceRule {
		@Override
		protected List<ProductIdTypeEnum> doEvaluate(TransactionReportInstruction input) {
			List<ProductIdTypeEnum> output = new ArrayList<>();
			return assignOutput(output, input);
		}
		
		protected List<ProductIdTypeEnum> assignOutput(List<ProductIdTypeEnum> output, TransactionReportInstruction input) {
			output = cDESourceOfTheIdentifierOfTheBasketConstituents.evaluate(input);
			
			return output;
		}
	}
}
