package drr.regulation.jfsa.rewrite.trade.reports;

import cdm.event.common.Trade;
import cdm.product.template.CancelableProvision;
import cdm.product.template.ContractualProduct;
import cdm.product.template.EarlyTerminationProvision;
import cdm.product.template.EconomicTerms;
import cdm.product.template.ExtendibleProvision;
import cdm.product.template.MandatoryEarlyTermination;
import cdm.product.template.OptionalEarlyTermination;
import cdm.product.template.Product;
import cdm.product.template.TerminationProvision;
import cdm.product.template.TradableProduct;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.TradeForEvent;
import java.util.Arrays;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(EmbeddedOptionTypeRule.EmbeddedOptionTypeRuleDefault.class)
public abstract class EmbeddedOptionTypeRule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected TradeForEvent tradeForEvent;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class EmbeddedOptionTypeRuleDefault extends EmbeddedOptionTypeRule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			final MapperS<Trade> thenResult0 = MapperS.of(input)
				.mapSingleToItem(item -> MapperS.of(tradeForEvent.evaluate(item.get())));
			final MapperS<EconomicTerms> thenResult1 = thenResult0
				.mapSingleToItem(item -> item.<TradableProduct>map("getTradableProduct", trade -> trade.getTradableProduct()).<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()));
			output = thenResult1
				.mapSingleToItem(item -> {
					if (onlyExists(Arrays.asList(item.<TerminationProvision>map("getTerminationProvision", economicTerms -> economicTerms.getTerminationProvision()).<ExtendibleProvision>map("getExtendibleProvision", terminationProvision -> terminationProvision.getExtendibleProvision()))).getOrDefault(false)) {
						return MapperS.of("EXTD");
					}
					if (onlyExists(Arrays.asList(item.<TerminationProvision>map("getTerminationProvision", economicTerms -> economicTerms.getTerminationProvision()).<CancelableProvision>map("getCancelableProvision", terminationProvision -> terminationProvision.getCancelableProvision()))).getOrDefault(false)) {
						return MapperS.of("CANC");
					}
					if (onlyExists(Arrays.asList(item.<TerminationProvision>map("getTerminationProvision", economicTerms -> economicTerms.getTerminationProvision()).<EarlyTerminationProvision>map("getEarlyTerminationProvision", terminationProvision -> terminationProvision.getEarlyTerminationProvision()))).and(onlyExists(Arrays.asList(item.<TerminationProvision>map("getTerminationProvision", economicTerms -> economicTerms.getTerminationProvision()).<EarlyTerminationProvision>map("getEarlyTerminationProvision", terminationProvision -> terminationProvision.getEarlyTerminationProvision()).<OptionalEarlyTermination>map("getOptionalEarlyTermination", earlyTerminationProvision -> earlyTerminationProvision.getOptionalEarlyTermination())))).getOrDefault(false)) {
						return MapperS.of("OPET");
					}
					if (onlyExists(Arrays.asList(item.<TerminationProvision>map("getTerminationProvision", economicTerms -> economicTerms.getTerminationProvision()).<EarlyTerminationProvision>map("getEarlyTerminationProvision", terminationProvision -> terminationProvision.getEarlyTerminationProvision()))).and(onlyExists(Arrays.asList(item.<TerminationProvision>map("getTerminationProvision", economicTerms -> economicTerms.getTerminationProvision()).<EarlyTerminationProvision>map("getEarlyTerminationProvision", terminationProvision -> terminationProvision.getEarlyTerminationProvision()).<MandatoryEarlyTermination>map("getMandatoryEarlyTermination", earlyTerminationProvision -> earlyTerminationProvision.getMandatoryEarlyTermination())))).getOrDefault(false)) {
						return MapperS.of("MDET");
					}
					if (exists(item.<TerminationProvision>map("getTerminationProvision", economicTerms -> economicTerms.getTerminationProvision()).<ExtendibleProvision>map("getExtendibleProvision", terminationProvision -> terminationProvision.getExtendibleProvision())).or(exists(item.<TerminationProvision>map("getTerminationProvision", economicTerms -> economicTerms.getTerminationProvision()).<CancelableProvision>map("getCancelableProvision", terminationProvision -> terminationProvision.getCancelableProvision()))).or(exists(item.<TerminationProvision>map("getTerminationProvision", economicTerms -> economicTerms.getTerminationProvision()).<EarlyTerminationProvision>map("getEarlyTerminationProvision", terminationProvision -> terminationProvision.getEarlyTerminationProvision()))).getOrDefault(false)) {
						return MapperS.of("OTHR");
					}
					return MapperS.<String>ofNull();
				}).get();
			
			return output;
		}
	}
}
