package drr.regulation.jfsa.rewrite.trade.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaReport;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.enrichment.common.reports.EnrichmentDataRule;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport.JFSATransactionReportBuilder;
import java.util.Optional;
import javax.inject.Inject;


@RosettaReport(namespace="drr.regulation.jfsa.rewrite.trade", body="JFSA", corpusList={"Trade"})
@ImplementedBy(JFSATradeReportFunction.JFSATradeReportFunctionDefault.class)
public abstract class JFSATradeReportFunction implements ReportFunction<TransactionReportInstruction, JFSATransactionReport> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ActionTypeRule actionType;
	@Inject protected AssetClassRule assetClass;
	@Inject protected BasketConstituentIdentifierSourceRule basketConstituentIdentifierSource;
	@Inject protected CDSIndexAttachmentPointRule cDSIndexAttachmentPoint;
	@Inject protected CDSIndexDetachmentPointRule cDSIndexDetachmentPoint;
	@Inject protected CallAmountRule callAmount;
	@Inject protected CallCurrencyRule callCurrency;
	@Inject protected CentralCounterpartyRule centralCounterparty;
	@Inject protected ClearedRule cleared;
	@Inject protected ClearingAccountOriginRule clearingAccountOrigin;
	@Inject protected ClearingMemberRule clearingMember;
	@Inject protected ClearingReceiptTimestampRule clearingReceiptTimestamp;
	@Inject protected CollateralPortfolioIndicatorRule collateralPortfolioIndicator;
	@Inject protected ConfirmationTimestampDTCCRule confirmationTimestampDTCC;
	@Inject protected ConfirmedRule confirmed;
	@Inject protected ContractTypeRule contractType;
	@Inject protected Counterparty1Rule counterparty1;
	@Inject protected Counterparty2Rule counterparty2;
	@Inject protected Counterparty2IdentifierTypeRule counterparty2IdentifierType;
	@Inject protected CustomBasketCodeRule customBasketCode;
	@Inject protected DeliveryTypeRule deliveryType;
	@Inject protected DerivativeBasedOnCryptoAssetsRule derivativeBasedOnCryptoAssets;
	@Inject protected Direction1Rule direction1;
	@Inject protected Direction2Leg1Rule direction2Leg1;
	@Inject protected Direction2Leg2Rule direction2Leg2;
	@Inject protected EarlyTerminationDateRule earlyTerminationDate;
	@Inject protected EffectiveDateRule effectiveDate;
	@Inject protected EmbeddedOptionTypeRule embeddedOptionType;
	@Inject protected EnrichmentDataRule enrichmentData;
	@Inject protected EntityResponsibleForReportingRule entityResponsibleForReporting;
	@Inject protected EventIdentifierRule eventIdentifier;
	@Inject protected EventTimestampRule eventTimestamp;
	@Inject protected EventTypeRule eventType;
	@Inject protected ExchangeRateRule exchangeRate;
	@Inject protected ExchangeRateBasisRule exchangeRateBasis;
	@Inject protected ExecutionTimestampRule executionTimestamp;
	@Inject protected ExpirationDateRule expirationDate;
	@Inject protected FinalContractualSettlementDateRule finalContractualSettlementDate;
	@Inject protected FirstExerciseDateRule firstExerciseDate;
	@Inject protected FixedRateDayCountConventionLeg1Rule fixedRateDayCountConventionLeg1;
	@Inject protected FixedRateDayCountConventionLeg2Rule fixedRateDayCountConventionLeg2;
	@Inject protected FixedRateLeg1Rule fixedRateLeg1;
	@Inject protected FixedRateLeg2Rule fixedRateLeg2;
	@Inject protected FixedRatePaymentFrequencyPeriodLeg1Rule fixedRatePaymentFrequencyPeriodLeg1;
	@Inject protected FixedRatePaymentFrequencyPeriodLeg2Rule fixedRatePaymentFrequencyPeriodLeg2;
	@Inject protected FixedRatePaymentFrequencyPeriodMultiplierLeg1Rule fixedRatePaymentFrequencyPeriodMultiplierLeg1;
	@Inject protected FixedRatePaymentFrequencyPeriodMultiplierLeg2Rule fixedRatePaymentFrequencyPeriodMultiplierLeg2;
	@Inject protected FixingDateLeg1Rule fixingDateLeg1;
	@Inject protected FixingDateLeg2Rule fixingDateLeg2;
	@Inject protected FloatingRateDayCountConventionOfLeg1Rule floatingRateDayCountConventionOfLeg1;
	@Inject protected FloatingRateDayCountConventionOfLeg2Rule floatingRateDayCountConventionOfLeg2;
	@Inject protected FloatingRatePaymentFrequencyPeriodLeg1Rule floatingRatePaymentFrequencyPeriodLeg1;
	@Inject protected FloatingRatePaymentFrequencyPeriodLeg2Rule floatingRatePaymentFrequencyPeriodLeg2;
	@Inject protected FloatingRatePaymentFrequencyPeriodMultiplierLeg1Rule floatingRatePaymentFrequencyPeriodMultiplierLeg1;
	@Inject protected FloatingRatePaymentFrequencyPeriodMultiplierLeg2Rule floatingRatePaymentFrequencyPeriodMultiplierLeg2;
	@Inject protected FloatingRateReferencePeriodOfLeg1MultiplierRule floatingRateReferencePeriodOfLeg1Multiplier;
	@Inject protected FloatingRateReferencePeriodOfLeg1TimePeriodRule floatingRateReferencePeriodOfLeg1TimePeriod;
	@Inject protected FloatingRateReferencePeriodOfLeg2MultiplierRule floatingRateReferencePeriodOfLeg2Multiplier;
	@Inject protected FloatingRateReferencePeriodOfLeg2TimePeriodRule floatingRateReferencePeriodOfLeg2TimePeriod;
	@Inject protected FloatingRateResetFrequencyPeriodLeg1Rule floatingRateResetFrequencyPeriodLeg1;
	@Inject protected FloatingRateResetFrequencyPeriodLeg2Rule floatingRateResetFrequencyPeriodLeg2;
	@Inject protected FloatingRateResetFrequencyPeriodMultiplierLeg1Rule floatingRateResetFrequencyPeriodMultiplierLeg1;
	@Inject protected FloatingRateResetFrequencyPeriodMultiplierLeg2Rule floatingRateResetFrequencyPeriodMultiplierLeg2;
	@Inject protected GetPackageTransactionPriceNotationEnumRule getPackageTransactionPriceNotationEnum;
	@Inject protected GetPackageTransactionSpreadNotationEnumRule getPackageTransactionSpreadNotationEnum;
	@Inject protected GetPriceNotationEnumRule getPriceNotationEnum;
	@Inject protected GetSpreadLeg1NotationEnumRule getSpreadLeg1NotationEnum;
	@Inject protected GetSpreadLeg2NotationEnumRule getSpreadLeg2NotationEnum;
	@Inject protected GetStrikePriceNotationEnumRule getStrikePriceNotationEnum;
	@Inject protected IdentifierOfBasketConstituentsRule identifierOfBasketConstituents;
	@Inject protected IndexFactorRule indexFactor;
	@Inject protected IndicatorOfTheFloatingRateOfLeg1Rule indicatorOfTheFloatingRateOfLeg1;
	@Inject protected IndicatorOfTheFloatingRateOfLeg2Rule indicatorOfTheFloatingRateOfLeg2;
	@Inject protected InitialMarginCollateralPortfolioCodeRule initialMarginCollateralPortfolioCode;
	@Inject protected MaturityDateOfTheUnderlyingRule maturityDateOfTheUnderlying;
	@Inject protected NameOfTheFloatingRateOfLeg1Rule nameOfTheFloatingRateOfLeg1;
	@Inject protected NameOfTheFloatingRateOfLeg2Rule nameOfTheFloatingRateOfLeg2;
	@Inject protected NameOfTheUnderlyingIndexRule nameOfTheUnderlyingIndex;
	@Inject protected NewSDRIdentifierRule newSDRIdentifier;
	@Inject protected NotionalAmountScheduleLeg1Rule notionalAmountScheduleLeg1;
	@Inject protected NotionalAmountScheduleLeg2Rule notionalAmountScheduleLeg2;
	@Inject protected NotionalCurrencyOfLeg1Rule notionalCurrencyOfLeg1;
	@Inject protected NotionalCurrencyOfLeg2Rule notionalCurrencyOfLeg2;
	@Inject protected NotionalLeg1Rule notionalLeg1;
	@Inject protected NotionalLeg2Rule notionalLeg2;
	@Inject protected NotionalQuantityScheduleLeg1Rule notionalQuantityScheduleLeg1;
	@Inject protected NotionalQuantityScheduleLeg2Rule notionalQuantityScheduleLeg2;
	@Inject protected OptionPremiumAmountRule optionPremiumAmount;
	@Inject protected OptionPremiumCurrencyRule optionPremiumCurrency;
	@Inject protected OptionPremiumPaymentDateRule optionPremiumPaymentDate;
	@Inject protected OptionStyleRule optionStyle;
	@Inject protected OptionTypeRule optionType;
	@Inject protected OriginalSwapSDRIdentifierRule originalSwapSDRIdentifier;
	@Inject protected OriginalSwapUTIRule originalSwapUTI;
	@Inject protected OtherPaymentRule otherPayment;
	@Inject protected PackageIdentifierRule packageIdentifier;
	@Inject protected PackageTransactionPriceBasisRule packageTransactionPriceBasis;
	@Inject protected PackageTransactionPriceCurrencyRule packageTransactionPriceCurrency;
	@Inject protected PackageTransactionPriceMonetaryRule packageTransactionPriceMonetary;
	@Inject protected PackageTransactionPriceNotationRule packageTransactionPriceNotation;
	@Inject protected PackageTransactionPricePercentageRule packageTransactionPricePercentage;
	@Inject protected PackageTransactionSpreadBasisRule packageTransactionSpreadBasis;
	@Inject protected PackageTransactionSpreadCurrencyRule packageTransactionSpreadCurrency;
	@Inject protected PackageTransactionSpreadMonetaryRule packageTransactionSpreadMonetary;
	@Inject protected PackageTransactionSpreadNotationRule packageTransactionSpreadNotation;
	@Inject protected PackageTransactionSpreadPercentageRule packageTransactionSpreadPercentage;
	@Inject protected PlatformIdentifierRule platformIdentifier;
	@Inject protected PriceCurrencyRule priceCurrency;
	@Inject protected PriceFormatMonetaryRule priceFormatMonetary;
	@Inject protected PriceFormatPercentageRule priceFormatPercentage;
	@Inject protected PriceNotationRule priceNotation;
	@Inject protected PriceScheduleRule priceSchedule;
	@Inject protected PriceUnitOfMeasureRule priceUnitOfMeasure;
	@Inject protected PriorUtiRule priorUti;
	@Inject protected PriorUtiProprietaryRule priorUtiProprietary;
	@Inject protected PutAmountRule putAmount;
	@Inject protected PutCurrencyRule putCurrency;
	@Inject protected QuantityUnitOfMeasureLeg1Rule quantityUnitOfMeasureLeg1;
	@Inject protected QuantityUnitOfMeasureLeg2Rule quantityUnitOfMeasureLeg2;
	@Inject protected ReferenceEntityRule referenceEntity;
	@Inject protected ReportingTimestampRule reportingTimestamp;
	@Inject protected SeniorityRule seniority;
	@Inject protected SeriesRule series;
	@Inject protected SettlementCurrencyLeg1Rule settlementCurrencyLeg1;
	@Inject protected SettlementCurrencyLeg2Rule settlementCurrencyLeg2;
	@Inject protected SpreadCurrencyOfLeg1Rule spreadCurrencyOfLeg1;
	@Inject protected SpreadCurrencyOfLeg2Rule spreadCurrencyOfLeg2;
	@Inject protected SpreadNotationOfLeg1Rule spreadNotationOfLeg1;
	@Inject protected SpreadNotationOfLeg2Rule spreadNotationOfLeg2;
	@Inject protected SpreadOfLeg1BasisRule spreadOfLeg1Basis;
	@Inject protected SpreadOfLeg1MonetaryRule spreadOfLeg1Monetary;
	@Inject protected SpreadOfLeg1PercentageRule spreadOfLeg1Percentage;
	@Inject protected SpreadOfLeg2BasisRule spreadOfLeg2Basis;
	@Inject protected SpreadOfLeg2MonetaryRule spreadOfLeg2Monetary;
	@Inject protected SpreadOfLeg2PercentageRule spreadOfLeg2Percentage;
	@Inject protected StrikePriceCurrencyCurrencyPairRule strikePriceCurrencyCurrencyPair;
	@Inject protected StrikePriceDecimalRule strikePriceDecimal;
	@Inject protected StrikePriceMonetaryRule strikePriceMonetary;
	@Inject protected StrikePriceNotationRule strikePriceNotation;
	@Inject protected StrikePriceScheduleRule strikePriceSchedule;
	@Inject protected SubmitterIdentifierRule submitterIdentifier;
	@Inject protected TechnicalRecordIdDTCCRule technicalRecordIdDTCC;
	@Inject protected TotalNotionalQuantityLeg1Rule totalNotionalQuantityLeg1;
	@Inject protected TotalNotionalQuantityLeg2Rule totalNotionalQuantityLeg2;
	@Inject protected UPIRule uPI;
	@Inject protected UnderlyingIdentificationRule underlyingIdentification;
	@Inject protected UnderlyingIdentificationTypeRule underlyingIdentificationType;
	@Inject protected UnderlyingIndexIndicatorRule underlyingIndexIndicator;
	@Inject protected UtiRule uti;
	@Inject protected UtiProprietaryRule utiProprietary;
	@Inject protected ValuationAmountRule valuationAmount;
	@Inject protected ValuationCurrencyRule valuationCurrency;
	@Inject protected ValuationMethodRule valuationMethod;
	@Inject protected ValuationTimestampRule valuationTimestamp;
	@Inject protected VariationMarginCollateralPortfolioCodeRule variationMarginCollateralPortfolioCode;
	@Inject protected VersionRule version;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public JFSATransactionReport evaluate(TransactionReportInstruction input) {
		JFSATransactionReport.JFSATransactionReportBuilder outputBuilder = doEvaluate(input);
		
		final JFSATransactionReport output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.build();
			objectValidator.validate(JFSATransactionReport.class, output);
		}
		
		return output;
	}

	protected abstract JFSATransactionReport.JFSATransactionReportBuilder doEvaluate(TransactionReportInstruction input);

	public static class JFSATradeReportFunctionDefault extends JFSATradeReportFunction {
		@Override
		protected JFSATransactionReport.JFSATransactionReportBuilder doEvaluate(TransactionReportInstruction input) {
			JFSATransactionReport.JFSATransactionReportBuilder output = JFSATransactionReport.builder();
			return assignOutput(output, input);
		}
		
		protected JFSATransactionReport.JFSATransactionReportBuilder assignOutput(JFSATransactionReport.JFSATransactionReportBuilder output, TransactionReportInstruction input) {
			output
				.setEffectiveDate(effectiveDate.evaluate(input));
			
			output
				.setExpirationDate(expirationDate.evaluate(input));
			
			output
				.setEarlyTerminationDate(earlyTerminationDate.evaluate(input));
			
			output
				.setReportingTimestamp(reportingTimestamp.evaluate(input));
			
			output
				.setExecutionTimestamp(executionTimestamp.evaluate(input));
			
			output
				.setEntityResponsibleForReporting(entityResponsibleForReporting.evaluate(input));
			
			output
				.setCounterparty1(counterparty1.evaluate(input));
			
			output
				.setCounterparty2(counterparty2.evaluate(input));
			
			output
				.setCounterparty2IdentifierType(counterparty2IdentifierType.evaluate(input));
			
			output
				.setDirection1(direction1.evaluate(input));
			
			output
				.setDirection2Leg1(direction2Leg1.evaluate(input));
			
			output
				.setDirection2Leg2(direction2Leg2.evaluate(input));
			
			output
				.setSubmitterIdentifier(submitterIdentifier.evaluate(input));
			
			output
				.setNewSDRIdentifier(newSDRIdentifier.evaluate(input));
			
			output
				.setOriginalSwapSDRIdentifier(originalSwapSDRIdentifier.evaluate(input));
			
			output
				.setCleared(cleared.evaluate(input));
			
			output
				.setCentralCounterparty(centralCounterparty.evaluate(input));
			
			output
				.setClearingMember(clearingMember.evaluate(input));
			
			output
				.setPlatformIdentifier(platformIdentifier.evaluate(input));
			
			output
				.setConfirmed(confirmed.evaluate(input));
			
			output
				.setFinalContractualSettlementDate(finalContractualSettlementDate.evaluate(input));
			
			output
				.setConfirmationTimestamp(confirmationTimestampDTCC.evaluate(input));
			
			output
				.setSettlementCurrencyLeg1(settlementCurrencyLeg1.evaluate(input));
			
			output
				.setSettlementCurrencyLeg2(settlementCurrencyLeg2.evaluate(input));
			
			output
				.setClearingAccountOrigin(clearingAccountOrigin.evaluate(input));
			
			output
				.setOriginalSwapUTI(originalSwapUTI.evaluate(input));
			
			output
				.setClearingReceiptTimestamp(clearingReceiptTimestamp.evaluate(input));
			
			output
				.setUti(uti.evaluate(input));
			
			output
				.setUtiProprietary(utiProprietary.evaluate(input));
			
			output
				.setPriorUti(priorUti.evaluate(input));
			
			output
				.setPriorUtiProprietary(priorUtiProprietary.evaluate(input));
			
			output
				.setFixedRateDayCountConventionLeg1(fixedRateDayCountConventionLeg1.evaluate(input));
			
			output
				.setFloatingRateDayCountConventionOfLeg1(floatingRateDayCountConventionOfLeg1.evaluate(input));
			
			output
				.setFixedRateDayCountConventionLeg2(fixedRateDayCountConventionLeg2.evaluate(input));
			
			output
				.setFloatingRateDayCountConventionOfLeg2(floatingRateDayCountConventionOfLeg2.evaluate(input));
			
			output
				.setFixedRatePaymentFrequencyPeriodLeg1(fixedRatePaymentFrequencyPeriodLeg1.evaluate(input));
			
			output
				.setFixedRatePaymentFrequencyPeriodLeg2(fixedRatePaymentFrequencyPeriodLeg2.evaluate(input));
			
			output
				.setFloatingRatePaymentFrequencyPeriodLeg1(floatingRatePaymentFrequencyPeriodLeg1.evaluate(input));
			
			output
				.setFloatingRatePaymentFrequencyPeriodLeg2(floatingRatePaymentFrequencyPeriodLeg2.evaluate(input));
			
			output
				.setFixedRatePaymentFrequencyPeriodMultiplierLeg1(fixedRatePaymentFrequencyPeriodMultiplierLeg1.evaluate(input));
			
			output
				.setFixedRatePaymentFrequencyPeriodMultiplierLeg2(fixedRatePaymentFrequencyPeriodMultiplierLeg2.evaluate(input));
			
			output
				.setFloatingRatePaymentFrequencyPeriodMultiplierLeg1(floatingRatePaymentFrequencyPeriodMultiplierLeg1.evaluate(input));
			
			output
				.setFloatingRatePaymentFrequencyPeriodMultiplierLeg2(floatingRatePaymentFrequencyPeriodMultiplierLeg2.evaluate(input));
			
			output
				.setFixingDateLeg1(fixingDateLeg1.evaluate(input));
			
			output
				.setFixingDateLeg2(fixingDateLeg2.evaluate(input));
			
			output
				.setFloatingRateResetFrequencyPeriodLeg1(floatingRateResetFrequencyPeriodLeg1.evaluate(input));
			
			output
				.setFloatingRateResetFrequencyPeriodOfLeg2(floatingRateResetFrequencyPeriodLeg2.evaluate(input));
			
			output
				.setFloatingRateResetFrequencyPeriodMultiplierLeg1(floatingRateResetFrequencyPeriodMultiplierLeg1.evaluate(input));
			
			output
				.setFloatingRateResetFrequencyPeriodMultiplierLeg2(floatingRateResetFrequencyPeriodMultiplierLeg2.evaluate(input));
			
			output
				.setOtherPayment(otherPayment.evaluate(input));
			
			output
				.setValuationAmount(valuationAmount.evaluate(input));
			
			output
				.setValuationCurrency(valuationCurrency.evaluate(input));
			
			output
				.setValuationMethod(valuationMethod.evaluate(input));
			
			output
				.setValuationTimestamp(valuationTimestamp.evaluate(input));
			
			output
				.setCollateralPortfolioIndicator(collateralPortfolioIndicator.evaluate(input));
			
			output
				.setInitialMarginCollateralPortfolioCode(initialMarginCollateralPortfolioCode.evaluate(input));
			
			output
				.setVariationMarginCollateralPortfolioCode(variationMarginCollateralPortfolioCode.evaluate(input));
			
			output
				.getOrCreatePrice()
				.setPriceMonetary(priceFormatMonetary.evaluate(input));
			
			output
				.getOrCreatePrice()
				.setPricePercentage(priceFormatPercentage.evaluate(input));
			
			output
				.getOrCreatePrice()
				.setPriceCurrency(priceCurrency.evaluate(input));
			
			output
				.getOrCreatePrice()
				.setPriceNotation(getPriceNotationEnum.evaluate(input));
			
			output
				.setPriceNotation(priceNotation.evaluate(input));
			
			output
				.setPriceUnitOfMeasure(priceUnitOfMeasure.evaluate(input));
			
			output
				.setPriceSchedule(priceSchedule.evaluate(input));
			
			output
				.setFixedRateLeg1(fixedRateLeg1.evaluate(input));
			
			output
				.setFixedRateLeg2(fixedRateLeg2.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadOfLeg1Monetary(spreadOfLeg1Monetary.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadOfLeg1Percentage(spreadOfLeg1Percentage.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadOfLeg1Basis(spreadOfLeg1Basis.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadCurrencyOfLeg1(spreadCurrencyOfLeg1.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadOfLeg1Notation(getSpreadLeg1NotationEnum.evaluate(input));
			
			output
				.setSpreadNotationOfLeg1(spreadNotationOfLeg1.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg2()
				.setSpreadOfLeg2Monetary(spreadOfLeg2Monetary.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg2()
				.setSpreadOfLeg2Percentage(spreadOfLeg2Percentage.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg2()
				.setSpreadOfLeg2Basis(spreadOfLeg2Basis.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg2()
				.setSpreadCurrencyOfLeg2(spreadCurrencyOfLeg2.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg2()
				.setSpreadOfLeg2Notation(getSpreadLeg2NotationEnum.evaluate(input));
			
			output
				.setSpreadNotationOfLeg2(spreadNotationOfLeg2.evaluate(input));
			
			output
				.getOrCreateStrikePrice()
				.setStrikePriceMonetary(strikePriceMonetary.evaluate(input));
			
			output
				.getOrCreateStrikePrice()
				.setStrikePriceDecimal(strikePriceDecimal.evaluate(input));
			
			output
				.getOrCreateStrikePrice()
				.setStrikePriceCurrency(strikePriceCurrencyCurrencyPair.evaluate(input));
			
			output
				.getOrCreateStrikePrice()
				.setStrikePriceNotation(getStrikePriceNotationEnum.evaluate(input));
			
			output
				.setStrikePriceNotation(strikePriceNotation.evaluate(input));
			
			output
				.setStrikePriceSchedule(strikePriceSchedule.evaluate(input));
			
			output
				.setOptionPremiumAmount(optionPremiumAmount.evaluate(input));
			
			output
				.setOptionPremiumCurrency(optionPremiumCurrency.evaluate(input));
			
			output
				.setOptionPremiumPaymentDate(optionPremiumPaymentDate.evaluate(input));
			
			output
				.setFirstExerciseDate(firstExerciseDate.evaluate(input));
			
			output
				.setExchangeRate(exchangeRate.evaluate(input));
			
			output
				.setExchangeRateBasis(exchangeRateBasis.evaluate(input));
			
			output
				.setNotionalLeg1(notionalLeg1.evaluate(input));
			
			output
				.setNotionalLeg2(notionalLeg2.evaluate(input));
			
			output
				.setCallAmount(callAmount.evaluate(input));
			
			output
				.setPutAmount(putAmount.evaluate(input));
			
			output
				.setNotionalCurrencyOfLeg1(notionalCurrencyOfLeg1.evaluate(input));
			
			output
				.setNotionalCurrencyOfLeg2(notionalCurrencyOfLeg2.evaluate(input));
			
			output
				.setCallCurrency(callCurrency.evaluate(input));
			
			output
				.setPutCurrency(putCurrency.evaluate(input));
			
			output
				.setQuantityUnitOfMeasureLeg1(quantityUnitOfMeasureLeg1.evaluate(input));
			
			output
				.setQuantityUnitOfMeasureLeg2(quantityUnitOfMeasureLeg2.evaluate(input));
			
			output
				.setNotionalAmountScheduleLeg1(notionalAmountScheduleLeg1.evaluate(input));
			
			output
				.setNotionalAmountScheduleLeg2(notionalAmountScheduleLeg2.evaluate(input));
			
			output
				.setTotalNotionalQuantityLeg1(totalNotionalQuantityLeg1.evaluate(input));
			
			output
				.setTotalNotionalQuantityLeg2(totalNotionalQuantityLeg2.evaluate(input));
			
			output
				.setNotionalQuantityScheduleLeg1(notionalQuantityScheduleLeg1.evaluate(input));
			
			output
				.setNotionalQuantityScheduleLeg2(notionalQuantityScheduleLeg2.evaluate(input));
			
			output
				.setActionType(actionType.evaluate(input));
			
			output
				.setEventType(eventType.evaluate(input));
			
			output
				.setEventTimestamp(eventTimestamp.evaluate(input));
			
			output
				.setEventIdentifier(eventIdentifier.evaluate(input));
			
			output
				.setIndexFactor(indexFactor.evaluate(input));
			
			output
				.setEmbeddedOptionType(embeddedOptionType.evaluate(input));
			
			output
				.setUpi(uPI.evaluate(input));
			
			output
				.setDeliveryType(deliveryType.evaluate(input));
			
			output
				.setAssetClass(assetClass.evaluate(input));
			
			output
				.setUnderlyingIdentificationType(underlyingIdentificationType.evaluate(input));
			
			output
				.setUnderlyingIdentification(underlyingIdentification.evaluate(input));
			
			output
				.setUnderlyingIndexIndicator(underlyingIndexIndicator.evaluate(input));
			
			output
				.setNameOfTheUnderlyingIndex(nameOfTheUnderlyingIndex.evaluate(input));
			
			output
				.setReferenceEntity(referenceEntity.evaluate(input));
			
			output
				.setIndicatorOfTheFloatingRateOfLeg1(indicatorOfTheFloatingRateOfLeg1.evaluate(input));
			
			output
				.setIndicatorOfTheFloatingRateOfLeg2(indicatorOfTheFloatingRateOfLeg2.evaluate(input));
			
			output
				.setNameOfTheFloatingRateOfLeg1(nameOfTheFloatingRateOfLeg1.evaluate(input));
			
			output
				.setNameOfTheFloatingRateOfLeg2(nameOfTheFloatingRateOfLeg2.evaluate(input));
			
			output
				.setFloatingRateReferencePeriodOfLeg1TimePeriod(floatingRateReferencePeriodOfLeg1TimePeriod.evaluate(input));
			
			output
				.setFloatingRateReferencePeriodOfLeg2TimePeriod(floatingRateReferencePeriodOfLeg2TimePeriod.evaluate(input));
			
			output
				.setFloatingRateReferencePeriodOfLeg1Multiplier(floatingRateReferencePeriodOfLeg1Multiplier.evaluate(input));
			
			output
				.setFloatingRateReferencePeriodOfLeg2Multiplier(floatingRateReferencePeriodOfLeg2Multiplier.evaluate(input));
			
			output
				.setIsCrypto(derivativeBasedOnCryptoAssets.evaluate(input));
			
			output
				.setMaturityDateOfTheUnderlying(maturityDateOfTheUnderlying.evaluate(input));
			
			output
				.setSeniority(seniority.evaluate(input));
			
			output
				.setSeries(series.evaluate(input));
			
			output
				.setVersion(version.evaluate(input));
			
			output
				.setCdSIndexAttachmentPoint(cDSIndexAttachmentPoint.evaluate(input));
			
			output
				.setCdSIndexDetachmentPoint(cDSIndexDetachmentPoint.evaluate(input));
			
			output
				.setCustomBasketCode(customBasketCode.evaluate(input));
			
			output
				.setIdentifierOfBasketConstituents(identifierOfBasketConstituents.evaluate(input));
			
			output
				.setBasketConstituentIdentifierSource(basketConstituentIdentifierSource.evaluate(input));
			
			output
				.setContractType(contractType.evaluate(input));
			
			output
				.setOptionStyle(optionStyle.evaluate(input));
			
			output
				.setOptionType(optionType.evaluate(input));
			
			output
				.setPackageIdentifier(packageIdentifier.evaluate(input));
			
			output
				.getOrCreatePackageTransactionPrice()
				.setPackageTransactionPriceMonetary(packageTransactionPriceMonetary.evaluate(input));
			
			output
				.getOrCreatePackageTransactionPrice()
				.setPackageTransactionPricePercentage(packageTransactionPricePercentage.evaluate(input));
			
			output
				.getOrCreatePackageTransactionPrice()
				.setPackageTransactionPriceBasis(packageTransactionPriceBasis.evaluate(input));
			
			output
				.getOrCreatePackageTransactionPrice()
				.setPackageTransactionPriceNotation(getPackageTransactionPriceNotationEnum.evaluate(input));
			
			output
				.setPackageTransactionPriceCurrency(packageTransactionPriceCurrency.evaluate(input));
			
			output
				.setPackageTransactionPriceNotation(packageTransactionPriceNotation.evaluate(input));
			
			output
				.getOrCreatePackageTransactionSpread()
				.setPackageTransactionSpreadMonetary(packageTransactionSpreadMonetary.evaluate(input));
			
			output
				.getOrCreatePackageTransactionSpread()
				.setPackageTransactionSpreadPercentage(packageTransactionSpreadPercentage.evaluate(input));
			
			output
				.getOrCreatePackageTransactionSpread()
				.setPackageTransactionSpreadBasis(packageTransactionSpreadBasis.evaluate(input));
			
			output
				.getOrCreatePackageTransactionSpread()
				.setPackageTransactionSpreadNotation(getPackageTransactionSpreadNotationEnum.evaluate(input));
			
			output
				.setPackageTransactionSpreadCurrency(packageTransactionSpreadCurrency.evaluate(input));
			
			output
				.setPackageTransactionSpreadNotation(packageTransactionSpreadNotation.evaluate(input));
			
			output
				.setTechnicalRecordId(technicalRecordIdDTCC.evaluate(input));
			
			output
				.getOrCreateNonReportable()
				.setEnrichment(enrichmentData.evaluate(input));
			
			return Optional.ofNullable(output)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
