package drr.regulation.jfsa.rewrite.trade.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import cdm.base.staticdata.asset.common.ProductIdTypeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.model.lib.reports.Tabulator.MultiNestedFieldValueImpl;
import com.rosetta.model.lib.reports.Tabulator.NestedFieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;
import iso20022.auth030.esma.InterestComputationMethod4Code;
import iso20022.auth030.esma.ValuationType1Code;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.inject.Inject;


public class JFSATransactionReportTypeTabulator implements Tabulator<JFSATransactionReport> {
	private final Field effectiveDateField;
	private final Field expirationDateField;
	private final Field earlyTerminationDateField;
	private final Field reportingTimestampField;
	private final Field executionTimestampField;
	private final Field entityResponsibleForReportingField;
	private final Field counterparty1Field;
	private final Field counterparty2Field;
	private final Field counterparty2IdentifierTypeField;
	private final Field direction1Field;
	private final Field direction2Leg1Field;
	private final Field direction2Leg2Field;
	private final Field submitterIdentifierField;
	private final Field newSDRIdentifierField;
	private final Field originalSwapSDRIdentifierField;
	private final Field clearedField;
	private final Field centralCounterpartyField;
	private final Field clearingMemberField;
	private final Field platformIdentifierField;
	private final Field confirmedField;
	private final Field finalContractualSettlementDateField;
	private final Field confirmationTimestampField;
	private final Field settlementCurrencyLeg1Field;
	private final Field settlementCurrencyLeg2Field;
	private final Field clearingAccountOriginField;
	private final Field originalSwapUTIField;
	private final Field clearingReceiptTimestampField;
	private final Field utiField;
	private final Field utiProprietaryField;
	private final Field priorUtiField;
	private final Field priorUtiProprietaryField;
	private final Field fixedRateDayCountConventionLeg1Field;
	private final Field floatingRateDayCountConventionOfLeg1Field;
	private final Field fixedRateDayCountConventionLeg2Field;
	private final Field floatingRateDayCountConventionOfLeg2Field;
	private final Field fixedRatePaymentFrequencyPeriodLeg1Field;
	private final Field fixedRatePaymentFrequencyPeriodLeg2Field;
	private final Field floatingRatePaymentFrequencyPeriodLeg1Field;
	private final Field floatingRatePaymentFrequencyPeriodLeg2Field;
	private final Field fixedRatePaymentFrequencyPeriodMultiplierLeg1Field;
	private final Field fixedRatePaymentFrequencyPeriodMultiplierLeg2Field;
	private final Field floatingRatePaymentFrequencyPeriodMultiplierLeg1Field;
	private final Field floatingRatePaymentFrequencyPeriodMultiplierLeg2Field;
	private final Field fixingDateLeg1Field;
	private final Field fixingDateLeg2Field;
	private final Field floatingRateResetFrequencyPeriodLeg1Field;
	private final Field floatingRateResetFrequencyPeriodOfLeg2Field;
	private final Field floatingRateResetFrequencyPeriodMultiplierLeg1Field;
	private final Field floatingRateResetFrequencyPeriodMultiplierLeg2Field;
	private final Field otherPaymentField;
	private final Field valuationAmountField;
	private final Field valuationCurrencyField;
	private final Field valuationMethodField;
	private final Field valuationTimestampField;
	private final Field collateralPortfolioIndicatorField;
	private final Field initialMarginCollateralPortfolioCodeField;
	private final Field variationMarginCollateralPortfolioCodeField;
	private final Field priceField;
	private final Field priceNotationField;
	private final Field priceUnitOfMeasureField;
	private final Field priceScheduleField;
	private final Field fixedRateLeg1Field;
	private final Field fixedRateLeg2Field;
	private final Field spreadOfLeg1Field;
	private final Field spreadNotationOfLeg1Field;
	private final Field spreadOfLeg2Field;
	private final Field spreadNotationOfLeg2Field;
	private final Field strikePriceField;
	private final Field strikePriceNotationField;
	private final Field strikePriceScheduleField;
	private final Field optionPremiumAmountField;
	private final Field optionPremiumCurrencyField;
	private final Field optionPremiumPaymentDateField;
	private final Field firstExerciseDateField;
	private final Field exchangeRateField;
	private final Field exchangeRateBasisField;
	private final Field notionalLeg1Field;
	private final Field notionalLeg2Field;
	private final Field callAmountField;
	private final Field putAmountField;
	private final Field notionalCurrencyOfLeg1Field;
	private final Field notionalCurrencyOfLeg2Field;
	private final Field callCurrencyField;
	private final Field putCurrencyField;
	private final Field quantityUnitOfMeasureLeg1Field;
	private final Field quantityUnitOfMeasureLeg2Field;
	private final Field notionalAmountScheduleLeg1Field;
	private final Field notionalAmountScheduleLeg2Field;
	private final Field totalNotionalQuantityLeg1Field;
	private final Field totalNotionalQuantityLeg2Field;
	private final Field notionalQuantityScheduleLeg1Field;
	private final Field notionalQuantityScheduleLeg2Field;
	private final Field actionTypeField;
	private final Field eventTypeField;
	private final Field eventTimestampField;
	private final Field eventIdentifierField;
	private final Field indexFactorField;
	private final Field embeddedOptionTypeField;
	private final Field upiField;
	private final Field deliveryTypeField;
	private final Field assetClassField;
	private final Field underlyingIdentificationTypeField;
	private final Field underlyingIdentificationField;
	private final Field underlyingIndexIndicatorField;
	private final Field nameOfTheUnderlyingIndexField;
	private final Field referenceEntityField;
	private final Field indicatorOfTheFloatingRateOfLeg1Field;
	private final Field indicatorOfTheFloatingRateOfLeg2Field;
	private final Field nameOfTheFloatingRateOfLeg1Field;
	private final Field nameOfTheFloatingRateOfLeg2Field;
	private final Field floatingRateReferencePeriodOfLeg1TimePeriodField;
	private final Field floatingRateReferencePeriodOfLeg2TimePeriodField;
	private final Field floatingRateReferencePeriodOfLeg1MultiplierField;
	private final Field floatingRateReferencePeriodOfLeg2MultiplierField;
	private final Field isCryptoField;
	private final Field maturityDateOfTheUnderlyingField;
	private final Field seniorityField;
	private final Field seriesField;
	private final Field versionField;
	private final Field cdSIndexAttachmentPointField;
	private final Field cdSIndexDetachmentPointField;
	private final Field customBasketCodeField;
	private final Field identifierOfBasketConstituentsField;
	private final Field basketConstituentIdentifierSourceField;
	private final Field contractTypeField;
	private final Field optionStyleField;
	private final Field optionTypeField;
	private final Field packageIdentifierField;
	private final Field packageTransactionPriceField;
	private final Field packageTransactionPriceCurrencyField;
	private final Field packageTransactionPriceNotationField;
	private final Field packageTransactionSpreadField;
	private final Field packageTransactionSpreadCurrencyField;
	private final Field packageTransactionSpreadNotationField;
	private final Field technicalRecordIdField;
	
	private final OtherPaymentReportTypeTabulator otherPaymentReportTypeTabulator;
	private final PriceAndNotationTypeTabulator priceAndNotationTypeTabulator;
	private final PriceScheduleReportTypeTabulator priceScheduleReportTypeTabulator;
	private final SpreadOfLeg1TypeTabulator spreadOfLeg1TypeTabulator;
	private final SpreadOfLeg2TypeTabulator spreadOfLeg2TypeTabulator;
	private final StrikePriceAndNotationTypeTabulator strikePriceAndNotationTypeTabulator;
	private final StrikePriceScheduleReportTypeTabulator strikePriceScheduleReportTypeTabulator;
	private final NotionalAmountScheduleLeg1ReportTypeTabulator notionalAmountScheduleLeg1ReportTypeTabulator;
	private final NotionalAmountScheduleLeg2ReportTypeTabulator notionalAmountScheduleLeg2ReportTypeTabulator;
	private final NotionalQuantityScheduleLeg1ReportTypeTabulator notionalQuantityScheduleLeg1ReportTypeTabulator;
	private final NotionalQuantityScheduleLeg2ReportTypeTabulator notionalQuantityScheduleLeg2ReportTypeTabulator;
	private final PackageTransactionPriceTypeTabulator packageTransactionPriceTypeTabulator;
	private final PackageTransactionSpreadTypeTabulator packageTransactionSpreadTypeTabulator;
	
	@Inject
	public JFSATransactionReportTypeTabulator(OtherPaymentReportTypeTabulator otherPaymentReportTypeTabulator, PriceAndNotationTypeTabulator priceAndNotationTypeTabulator, PriceScheduleReportTypeTabulator priceScheduleReportTypeTabulator, SpreadOfLeg1TypeTabulator spreadOfLeg1TypeTabulator, SpreadOfLeg2TypeTabulator spreadOfLeg2TypeTabulator, StrikePriceAndNotationTypeTabulator strikePriceAndNotationTypeTabulator, StrikePriceScheduleReportTypeTabulator strikePriceScheduleReportTypeTabulator, NotionalAmountScheduleLeg1ReportTypeTabulator notionalAmountScheduleLeg1ReportTypeTabulator, NotionalAmountScheduleLeg2ReportTypeTabulator notionalAmountScheduleLeg2ReportTypeTabulator, NotionalQuantityScheduleLeg1ReportTypeTabulator notionalQuantityScheduleLeg1ReportTypeTabulator, NotionalQuantityScheduleLeg2ReportTypeTabulator notionalQuantityScheduleLeg2ReportTypeTabulator, PackageTransactionPriceTypeTabulator packageTransactionPriceTypeTabulator, PackageTransactionSpreadTypeTabulator packageTransactionSpreadTypeTabulator) {
		this.otherPaymentReportTypeTabulator = otherPaymentReportTypeTabulator;
		this.priceAndNotationTypeTabulator = priceAndNotationTypeTabulator;
		this.priceScheduleReportTypeTabulator = priceScheduleReportTypeTabulator;
		this.spreadOfLeg1TypeTabulator = spreadOfLeg1TypeTabulator;
		this.spreadOfLeg2TypeTabulator = spreadOfLeg2TypeTabulator;
		this.strikePriceAndNotationTypeTabulator = strikePriceAndNotationTypeTabulator;
		this.strikePriceScheduleReportTypeTabulator = strikePriceScheduleReportTypeTabulator;
		this.notionalAmountScheduleLeg1ReportTypeTabulator = notionalAmountScheduleLeg1ReportTypeTabulator;
		this.notionalAmountScheduleLeg2ReportTypeTabulator = notionalAmountScheduleLeg2ReportTypeTabulator;
		this.notionalQuantityScheduleLeg1ReportTypeTabulator = notionalQuantityScheduleLeg1ReportTypeTabulator;
		this.notionalQuantityScheduleLeg2ReportTypeTabulator = notionalQuantityScheduleLeg2ReportTypeTabulator;
		this.packageTransactionPriceTypeTabulator = packageTransactionPriceTypeTabulator;
		this.packageTransactionSpreadTypeTabulator = packageTransactionSpreadTypeTabulator;
		this.effectiveDateField = new FieldImpl(
			"effectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "EffectiveDate")),
			Optional.of("1 Effective Date"),
			Arrays.asList()
		);
		this.expirationDateField = new FieldImpl(
			"expirationDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ExpirationDate")),
			Optional.of("2 Expiration date"),
			Arrays.asList()
		);
		this.earlyTerminationDateField = new FieldImpl(
			"earlyTerminationDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "EarlyTerminationDate")),
			Optional.of("3 Early termination date"),
			Arrays.asList()
		);
		this.reportingTimestampField = new FieldImpl(
			"reportingTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ReportingTimestamp")),
			Optional.of("4 Reporting timestamp"),
			Arrays.asList()
		);
		this.executionTimestampField = new FieldImpl(
			"executionTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ExecutionTimestamp")),
			Optional.of("5 Execution timestamp"),
			Arrays.asList()
		);
		this.entityResponsibleForReportingField = new FieldImpl(
			"entityResponsibleForReporting",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "EntityResponsibleForReporting")),
			Optional.of("6 Entity responsible for reporting"),
			Arrays.asList()
		);
		this.counterparty1Field = new FieldImpl(
			"counterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "Counterparty1")),
			Optional.of("7 Counterparty 1"),
			Arrays.asList()
		);
		this.counterparty2Field = new FieldImpl(
			"counterparty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "Counterparty2")),
			Optional.of("8 Counterparty 2"),
			Arrays.asList()
		);
		this.counterparty2IdentifierTypeField = new FieldImpl(
			"counterparty2IdentifierType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "Counterparty2IdentifierType")),
			Optional.of("9 Counterparty 2 identifier type"),
			Arrays.asList()
		);
		this.direction1Field = new FieldImpl(
			"direction1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "Direction1")),
			Optional.of("10 Direction 1"),
			Arrays.asList()
		);
		this.direction2Leg1Field = new FieldImpl(
			"direction2Leg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "Direction2Leg1")),
			Optional.of("11 Direction 2-Leg 1"),
			Arrays.asList()
		);
		this.direction2Leg2Field = new FieldImpl(
			"direction2Leg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "Direction2Leg2")),
			Optional.of("11 Direction 2-Leg 2"),
			Arrays.asList()
		);
		this.submitterIdentifierField = new FieldImpl(
			"submitterIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "SubmitterIdentifier")),
			Optional.of("12 Submitter identifier"),
			Arrays.asList()
		);
		this.newSDRIdentifierField = new FieldImpl(
			"newSDRIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NewSDRIdentifier")),
			Optional.of("13 New SDR identifier"),
			Arrays.asList()
		);
		this.originalSwapSDRIdentifierField = new FieldImpl(
			"originalSwapSDRIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "OriginalSwapSDRIdentifier")),
			Optional.of("14 Original swap SDR identifier"),
			Arrays.asList()
		);
		this.clearedField = new FieldImpl(
			"cleared",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "Cleared")),
			Optional.of("15 Cleared"),
			Arrays.asList()
		);
		this.centralCounterpartyField = new FieldImpl(
			"centralCounterparty",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "CentralCounterparty")),
			Optional.of("16 Central counterparty"),
			Arrays.asList()
		);
		this.clearingMemberField = new FieldImpl(
			"clearingMember",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ClearingMember")),
			Optional.of("17 Clearing member"),
			Arrays.asList()
		);
		this.platformIdentifierField = new FieldImpl(
			"platformIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PlatformIdentifier")),
			Optional.of("18 Platform Identifier"),
			Arrays.asList()
		);
		this.confirmedField = new FieldImpl(
			"confirmed",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "Confirmed")),
			Optional.of("19 Confirmed"),
			Arrays.asList()
		);
		this.finalContractualSettlementDateField = new FieldImpl(
			"finalContractualSettlementDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FinalContractualSettlementDate")),
			Optional.of("20 Final contractual settlement date"),
			Arrays.asList()
		);
		this.confirmationTimestampField = new FieldImpl(
			"confirmationTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ConfirmationTimestampDTCC")),
			Optional.of("[Internal] Confirmation Timestamp (DTCC)"),
			Arrays.asList()
		);
		this.settlementCurrencyLeg1Field = new FieldImpl(
			"settlementCurrencyLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "SettlementCurrencyLeg1")),
			Optional.of("21 Settlement Currency-Leg 1"),
			Arrays.asList()
		);
		this.settlementCurrencyLeg2Field = new FieldImpl(
			"settlementCurrencyLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "SettlementCurrencyLeg2")),
			Optional.of("21 Settlement Currency-Leg 2"),
			Arrays.asList()
		);
		this.clearingAccountOriginField = new FieldImpl(
			"clearingAccountOrigin",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ClearingAccountOrigin")),
			Optional.of("22 Clearing account origin"),
			Arrays.asList()
		);
		this.originalSwapUTIField = new FieldImpl(
			"originalSwapUTI",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "OriginalSwapUTI")),
			Optional.of("23 Original Swap UTI"),
			Arrays.asList()
		);
		this.clearingReceiptTimestampField = new FieldImpl(
			"clearingReceiptTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ClearingReceiptTimestamp")),
			Optional.of("24 Clearing receipt timestamp"),
			Arrays.asList()
		);
		this.utiField = new FieldImpl(
			"uti",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "Uti")),
			Optional.of("25 Unique transaction identifier (UTI)"),
			Arrays.asList()
		);
		this.utiProprietaryField = new FieldImpl(
			"utiProprietary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "UtiProprietary")),
			Optional.of("25 Unique transaction identifier (UTI Proprietary)"),
			Arrays.asList()
		);
		this.priorUtiField = new FieldImpl(
			"priorUti",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PriorUti")),
			Optional.of("26 Prior UTI"),
			Arrays.asList()
		);
		this.priorUtiProprietaryField = new FieldImpl(
			"priorUtiProprietary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PriorUtiProprietary")),
			Optional.of("26 Prior UTI (Proprietary)"),
			Arrays.asList()
		);
		this.fixedRateDayCountConventionLeg1Field = new FieldImpl(
			"fixedRateDayCountConventionLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FixedRateDayCountConventionLeg1")),
			Optional.of("27 Fixed rate day count convention-Leg 1"),
			Arrays.asList()
		);
		this.floatingRateDayCountConventionOfLeg1Field = new FieldImpl(
			"floatingRateDayCountConventionOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FloatingRateDayCountConventionOfLeg1")),
			Optional.of("27 Floating rate day count convention of leg 1"),
			Arrays.asList()
		);
		this.fixedRateDayCountConventionLeg2Field = new FieldImpl(
			"fixedRateDayCountConventionLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FixedRateDayCountConventionLeg2")),
			Optional.of("27 Fixed rate day count convention leg 2"),
			Arrays.asList()
		);
		this.floatingRateDayCountConventionOfLeg2Field = new FieldImpl(
			"floatingRateDayCountConventionOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FloatingRateDayCountConventionOfLeg2")),
			Optional.of("27 Floating rate day count convention of leg 2"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodLeg1Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FixedRatePaymentFrequencyPeriodLeg1")),
			Optional.of("28 Fixed rate payment frequency period-Leg 1"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodLeg2Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FixedRatePaymentFrequencyPeriodLeg2")),
			Optional.of("28 Fixed rate payment frequency period-Leg 2"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodLeg1Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FloatingRatePaymentFrequencyPeriodLeg1")),
			Optional.of("28 Floating rate payment frequency period-Leg 1"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodLeg2Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FloatingRatePaymentFrequencyPeriodLeg2")),
			Optional.of("28 Floating rate payment frequency period-Leg 2"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodMultiplierLeg1Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodMultiplierLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FixedRatePaymentFrequencyPeriodMultiplierLeg1")),
			Optional.of("29 Fixed rate payment frequency period multiplier-Leg 1"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodMultiplierLeg2Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodMultiplierLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FixedRatePaymentFrequencyPeriodMultiplierLeg2")),
			Optional.of("29 Fixed rate payment frequency period multiplier-Leg 2"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodMultiplierLeg1Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodMultiplierLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FloatingRatePaymentFrequencyPeriodMultiplierLeg1")),
			Optional.of("29 Floating rate payment frequency period multiplier-Leg 1"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodMultiplierLeg2Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodMultiplierLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FloatingRatePaymentFrequencyPeriodMultiplierLeg2")),
			Optional.of("29 Floating rate payment frequency period multiplier-Leg 2"),
			Arrays.asList()
		);
		this.fixingDateLeg1Field = new FieldImpl(
			"fixingDateLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FixingDateLeg1")),
			Optional.of("30 Fixing date-Leg 1"),
			Arrays.asList()
		);
		this.fixingDateLeg2Field = new FieldImpl(
			"fixingDateLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FixingDateLeg2")),
			Optional.of("30 Fixing date-Leg 2"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodLeg1Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FloatingRateResetFrequencyPeriodLeg1")),
			Optional.of("31 Floating rate reset frequency period-Leg 1"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodOfLeg2Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FloatingRateResetFrequencyPeriodLeg2")),
			Optional.of("31 Floating rate reset frequency period-Leg 2"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodMultiplierLeg1Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodMultiplierLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FloatingRateResetFrequencyPeriodMultiplierLeg1")),
			Optional.of("32 Floating rate reset frequency period multiplier-Leg 1"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodMultiplierLeg2Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodMultiplierLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FloatingRateResetFrequencyPeriodMultiplierLeg2")),
			Optional.of("32 Floating rate reset frequency period multiplier-Leg 2"),
			Arrays.asList()
		);
		this.otherPaymentField = new FieldImpl(
			"otherPayment",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "OtherPayment")),
			Optional.empty(),
			otherPaymentReportTypeTabulator.getFields()
		);
		this.valuationAmountField = new FieldImpl(
			"valuationAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ValuationAmount")),
			Optional.of("39 Valuation amount"),
			Arrays.asList()
		);
		this.valuationCurrencyField = new FieldImpl(
			"valuationCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ValuationCurrency")),
			Optional.of("40 Valuation currency"),
			Arrays.asList()
		);
		this.valuationMethodField = new FieldImpl(
			"valuationMethod",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ValuationMethod")),
			Optional.of("42 Valuation method"),
			Arrays.asList()
		);
		this.valuationTimestampField = new FieldImpl(
			"valuationTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ValuationTimestamp")),
			Optional.of("41 Valuation Timestamp"),
			Arrays.asList()
		);
		this.collateralPortfolioIndicatorField = new FieldImpl(
			"collateralPortfolioIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "CollateralPortfolioIndicator")),
			Optional.of("44 Collateral Portfolio Indicator"),
			Arrays.asList()
		);
		this.initialMarginCollateralPortfolioCodeField = new FieldImpl(
			"initialMarginCollateralPortfolioCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "InitialMarginCollateralPortfolioCode")),
			Optional.of("62 Initial margin collateral portfolio code"),
			Arrays.asList()
		);
		this.variationMarginCollateralPortfolioCodeField = new FieldImpl(
			"variationMarginCollateralPortfolioCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "VariationMarginCollateralPortfolioCode")),
			Optional.of("63 Variation margin collateral portfolio code"),
			Arrays.asList()
		);
		this.priceField = new FieldImpl(
			"price",
			false,
			Optional.empty(),
			Optional.empty(),
			priceAndNotationTypeTabulator.getFields()
		);
		this.priceNotationField = new FieldImpl(
			"priceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PriceNotation")),
			Optional.of("66 Price notation"),
			Arrays.asList()
		);
		this.priceUnitOfMeasureField = new FieldImpl(
			"priceUnitOfMeasure",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PriceUnitOfMeasure")),
			Optional.of("67 Price unit of measure"),
			Arrays.asList()
		);
		this.priceScheduleField = new FieldImpl(
			"priceSchedule",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PriceSchedule")),
			Optional.empty(),
			priceScheduleReportTypeTabulator.getFields()
		);
		this.fixedRateLeg1Field = new FieldImpl(
			"fixedRateLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FixedRateLeg1")),
			Optional.of("71 Fixed rate-Leg 1"),
			Arrays.asList()
		);
		this.fixedRateLeg2Field = new FieldImpl(
			"fixedRateLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FixedRateLeg2")),
			Optional.of("71 Fixed rate-Leg 2"),
			Arrays.asList()
		);
		this.spreadOfLeg1Field = new FieldImpl(
			"spreadOfLeg1",
			false,
			Optional.empty(),
			Optional.empty(),
			spreadOfLeg1TypeTabulator.getFields()
		);
		this.spreadNotationOfLeg1Field = new FieldImpl(
			"spreadNotationOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "SpreadNotationOfLeg1")),
			Optional.of("74 Spread notation-Leg 1"),
			Arrays.asList()
		);
		this.spreadOfLeg2Field = new FieldImpl(
			"spreadOfLeg2",
			false,
			Optional.empty(),
			Optional.empty(),
			spreadOfLeg2TypeTabulator.getFields()
		);
		this.spreadNotationOfLeg2Field = new FieldImpl(
			"spreadNotationOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "SpreadNotationOfLeg2")),
			Optional.of("74 Spread notation-Leg 2"),
			Arrays.asList()
		);
		this.strikePriceField = new FieldImpl(
			"strikePrice",
			false,
			Optional.empty(),
			Optional.empty(),
			strikePriceAndNotationTypeTabulator.getFields()
		);
		this.strikePriceNotationField = new FieldImpl(
			"strikePriceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "StrikePriceNotation")),
			Optional.of("77 Strike price notation"),
			Arrays.asList()
		);
		this.strikePriceScheduleField = new FieldImpl(
			"strikePriceSchedule",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "StrikePriceSchedule")),
			Optional.empty(),
			strikePriceScheduleReportTypeTabulator.getFields()
		);
		this.optionPremiumAmountField = new FieldImpl(
			"optionPremiumAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "OptionPremiumAmount")),
			Optional.of("81 Option premium amount"),
			Arrays.asList()
		);
		this.optionPremiumCurrencyField = new FieldImpl(
			"optionPremiumCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "OptionPremiumCurrency")),
			Optional.of("82 Option premium currency"),
			Arrays.asList()
		);
		this.optionPremiumPaymentDateField = new FieldImpl(
			"optionPremiumPaymentDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "OptionPremiumPaymentDate")),
			Optional.of("83 Option premium payment date"),
			Arrays.asList()
		);
		this.firstExerciseDateField = new FieldImpl(
			"firstExerciseDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FirstExerciseDate")),
			Optional.of("84 First exercise date"),
			Arrays.asList()
		);
		this.exchangeRateField = new FieldImpl(
			"exchangeRate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ExchangeRate")),
			Optional.of("85 Exchange rate"),
			Arrays.asList()
		);
		this.exchangeRateBasisField = new FieldImpl(
			"exchangeRateBasis",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ExchangeRateBasis")),
			Optional.of("86 Exchange rate basis"),
			Arrays.asList()
		);
		this.notionalLeg1Field = new FieldImpl(
			"notionalLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NotionalLeg1")),
			Optional.of("87 Notional Amount 1"),
			Arrays.asList()
		);
		this.notionalLeg2Field = new FieldImpl(
			"notionalLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NotionalLeg2")),
			Optional.of("87 Notional Amount 2"),
			Arrays.asList()
		);
		this.callAmountField = new FieldImpl(
			"callAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "CallAmount")),
			Optional.of("88 Call amount"),
			Arrays.asList()
		);
		this.putAmountField = new FieldImpl(
			"putAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PutAmount")),
			Optional.of("89 Put Amount"),
			Arrays.asList()
		);
		this.notionalCurrencyOfLeg1Field = new FieldImpl(
			"notionalCurrencyOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NotionalCurrencyOfLeg1")),
			Optional.of("90 Notional currency-Leg 1"),
			Arrays.asList()
		);
		this.notionalCurrencyOfLeg2Field = new FieldImpl(
			"notionalCurrencyOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NotionalCurrencyOfLeg2")),
			Optional.of("90 Notional currency-Leg 2"),
			Arrays.asList()
		);
		this.callCurrencyField = new FieldImpl(
			"callCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "CallCurrency")),
			Optional.of("91 Call currency"),
			Arrays.asList()
		);
		this.putCurrencyField = new FieldImpl(
			"putCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PutCurrency")),
			Optional.of("92 Put currency"),
			Arrays.asList()
		);
		this.quantityUnitOfMeasureLeg1Field = new FieldImpl(
			"quantityUnitOfMeasureLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "QuantityUnitOfMeasureLeg1")),
			Optional.of("93 Quantity unit of measure-Leg 1"),
			Arrays.asList()
		);
		this.quantityUnitOfMeasureLeg2Field = new FieldImpl(
			"quantityUnitOfMeasureLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "QuantityUnitOfMeasureLeg2")),
			Optional.of("93 Quantity unit of measure-Leg 2"),
			Arrays.asList()
		);
		this.notionalAmountScheduleLeg1Field = new FieldImpl(
			"notionalAmountScheduleLeg1",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NotionalAmountScheduleLeg1")),
			Optional.empty(),
			notionalAmountScheduleLeg1ReportTypeTabulator.getFields()
		);
		this.notionalAmountScheduleLeg2Field = new FieldImpl(
			"notionalAmountScheduleLeg2",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NotionalAmountScheduleLeg2")),
			Optional.empty(),
			notionalAmountScheduleLeg2ReportTypeTabulator.getFields()
		);
		this.totalNotionalQuantityLeg1Field = new FieldImpl(
			"totalNotionalQuantityLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "TotalNotionalQuantityLeg1")),
			Optional.of("97 Total notional quantity-Leg 1"),
			Arrays.asList()
		);
		this.totalNotionalQuantityLeg2Field = new FieldImpl(
			"totalNotionalQuantityLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "TotalNotionalQuantityLeg2")),
			Optional.of("97 Total notional quantity-Leg 2"),
			Arrays.asList()
		);
		this.notionalQuantityScheduleLeg1Field = new FieldImpl(
			"notionalQuantityScheduleLeg1",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NotionalQuantityScheduleLeg1")),
			Optional.empty(),
			notionalQuantityScheduleLeg1ReportTypeTabulator.getFields()
		);
		this.notionalQuantityScheduleLeg2Field = new FieldImpl(
			"notionalQuantityScheduleLeg2",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NotionalQuantityScheduleLeg2")),
			Optional.empty(),
			notionalQuantityScheduleLeg2ReportTypeTabulator.getFields()
		);
		this.actionTypeField = new FieldImpl(
			"actionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ActionType")),
			Optional.of("101 Action type"),
			Arrays.asList()
		);
		this.eventTypeField = new FieldImpl(
			"eventType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "EventType")),
			Optional.of("102 Event type"),
			Arrays.asList()
		);
		this.eventTimestampField = new FieldImpl(
			"eventTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "EventTimestamp")),
			Optional.of("104 Event timestamp"),
			Arrays.asList()
		);
		this.eventIdentifierField = new FieldImpl(
			"eventIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "EventIdentifier")),
			Optional.of("103 Event identifier"),
			Arrays.asList()
		);
		this.indexFactorField = new FieldImpl(
			"indexFactor",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "IndexFactor")),
			Optional.of("105 Index factor"),
			Arrays.asList()
		);
		this.embeddedOptionTypeField = new FieldImpl(
			"embeddedOptionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "EmbeddedOptionType")),
			Optional.of("106 Embedded option type"),
			Arrays.asList()
		);
		this.upiField = new FieldImpl(
			"upi",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "UPI")),
			Optional.of("107 UPI"),
			Arrays.asList()
		);
		this.deliveryTypeField = new FieldImpl(
			"deliveryType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "DeliveryType")),
			Optional.of("108 Delivery Type"),
			Arrays.asList()
		);
		this.assetClassField = new FieldImpl(
			"assetClass",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "AssetClass")),
			Optional.of("109 Asset Class"),
			Arrays.asList()
		);
		this.underlyingIdentificationTypeField = new FieldImpl(
			"underlyingIdentificationType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "UnderlyingIdentificationType")),
			Optional.of("110 Underlying identification type"),
			Arrays.asList()
		);
		this.underlyingIdentificationField = new FieldImpl(
			"underlyingIdentification",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "UnderlyingIdentification")),
			Optional.of("111 Underlying identification"),
			Arrays.asList()
		);
		this.underlyingIndexIndicatorField = new FieldImpl(
			"underlyingIndexIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "UnderlyingIndexIndicator")),
			Optional.of("112 Indicator of the underlying index"),
			Arrays.asList()
		);
		this.nameOfTheUnderlyingIndexField = new FieldImpl(
			"nameOfTheUnderlyingIndex",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NameOfTheUnderlyingIndex")),
			Optional.of("113 Name of the underlying index"),
			Arrays.asList()
		);
		this.referenceEntityField = new FieldImpl(
			"referenceEntity",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ReferenceEntity")),
			Optional.of("114 Reference entity"),
			Arrays.asList()
		);
		this.indicatorOfTheFloatingRateOfLeg1Field = new FieldImpl(
			"indicatorOfTheFloatingRateOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "IndicatorOfTheFloatingRateOfLeg1")),
			Optional.of("115 Indicator of the floating rate-Leg 1"),
			Arrays.asList()
		);
		this.indicatorOfTheFloatingRateOfLeg2Field = new FieldImpl(
			"indicatorOfTheFloatingRateOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "IndicatorOfTheFloatingRateOfLeg2")),
			Optional.of("115 Indicator of the floating rate-Leg 2"),
			Arrays.asList()
		);
		this.nameOfTheFloatingRateOfLeg1Field = new FieldImpl(
			"nameOfTheFloatingRateOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NameOfTheFloatingRateOfLeg1")),
			Optional.of("116 Name of the Floating Rate of Leg 1"),
			Arrays.asList()
		);
		this.nameOfTheFloatingRateOfLeg2Field = new FieldImpl(
			"nameOfTheFloatingRateOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NameOfTheFloatingRateOfLeg2")),
			Optional.of("116 Name of the Floating Rate of Leg 2"),
			Arrays.asList()
		);
		this.floatingRateReferencePeriodOfLeg1TimePeriodField = new FieldImpl(
			"floatingRateReferencePeriodOfLeg1TimePeriod",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FloatingRateReferencePeriodOfLeg1TimePeriod")),
			Optional.of("117 Floating rate reference period - time period-Leg 1"),
			Arrays.asList()
		);
		this.floatingRateReferencePeriodOfLeg2TimePeriodField = new FieldImpl(
			"floatingRateReferencePeriodOfLeg2TimePeriod",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FloatingRateReferencePeriodOfLeg2TimePeriod")),
			Optional.of("117 Floating rate reference period - time period-Leg 2"),
			Arrays.asList()
		);
		this.floatingRateReferencePeriodOfLeg1MultiplierField = new FieldImpl(
			"floatingRateReferencePeriodOfLeg1Multiplier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FloatingRateReferencePeriodOfLeg1Multiplier")),
			Optional.of("118 Floating rate reference period - multiplier-Leg 1"),
			Arrays.asList()
		);
		this.floatingRateReferencePeriodOfLeg2MultiplierField = new FieldImpl(
			"floatingRateReferencePeriodOfLeg2Multiplier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "FloatingRateReferencePeriodOfLeg2Multiplier")),
			Optional.of("118 Floating rate reference period - multiplier-Leg 2"),
			Arrays.asList()
		);
		this.isCryptoField = new FieldImpl(
			"isCrypto",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "DerivativeBasedOnCryptoAssets")),
			Optional.of("119 Derivative based on crypto-assets"),
			Arrays.asList()
		);
		this.maturityDateOfTheUnderlyingField = new FieldImpl(
			"maturityDateOfTheUnderlying",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "MaturityDateOfTheUnderlying")),
			Optional.of("120 Maturity date of the underlying"),
			Arrays.asList()
		);
		this.seniorityField = new FieldImpl(
			"seniority",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "Seniority")),
			Optional.of("121 Seniority"),
			Arrays.asList()
		);
		this.seriesField = new FieldImpl(
			"series",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "Series")),
			Optional.of("122 Series"),
			Arrays.asList()
		);
		this.versionField = new FieldImpl(
			"version",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "Version")),
			Optional.of("123 Version"),
			Arrays.asList()
		);
		this.cdSIndexAttachmentPointField = new FieldImpl(
			"cdSIndexAttachmentPoint",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "CDSIndexAttachmentPoint")),
			Optional.of("124 CDS index attachment point"),
			Arrays.asList()
		);
		this.cdSIndexDetachmentPointField = new FieldImpl(
			"cdSIndexDetachmentPoint",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "CDSIndexDetachmentPoint")),
			Optional.of("125 CDS index detachment point"),
			Arrays.asList()
		);
		this.customBasketCodeField = new FieldImpl(
			"customBasketCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "CustomBasketCode")),
			Optional.of("126 Custom basket code"),
			Arrays.asList()
		);
		this.identifierOfBasketConstituentsField = new FieldImpl(
			"identifierOfBasketConstituents",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "IdentifierOfBasketConstituents")),
			Optional.of("127 Identifier of the basket's constituents"),
			Arrays.asList()
		);
		this.basketConstituentIdentifierSourceField = new FieldImpl(
			"basketConstituentIdentifierSource",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "BasketConstituentIdentifierSource")),
			Optional.of("128 Basket constituent identifier source"),
			Arrays.asList()
		);
		this.contractTypeField = new FieldImpl(
			"contractType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "ContractType")),
			Optional.of("129 Contract Type"),
			Arrays.asList()
		);
		this.optionStyleField = new FieldImpl(
			"optionStyle",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "OptionStyle")),
			Optional.of("130 Option style"),
			Arrays.asList()
		);
		this.optionTypeField = new FieldImpl(
			"optionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "OptionType")),
			Optional.of("131 Option type"),
			Arrays.asList()
		);
		this.packageIdentifierField = new FieldImpl(
			"packageIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PackageIdentifier")),
			Optional.of("132 Package identifier"),
			Arrays.asList()
		);
		this.packageTransactionPriceField = new FieldImpl(
			"packageTransactionPrice",
			false,
			Optional.empty(),
			Optional.empty(),
			packageTransactionPriceTypeTabulator.getFields()
		);
		this.packageTransactionPriceCurrencyField = new FieldImpl(
			"packageTransactionPriceCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PackageTransactionPriceCurrency")),
			Optional.of("134 Package transaction price currency"),
			Arrays.asList()
		);
		this.packageTransactionPriceNotationField = new FieldImpl(
			"packageTransactionPriceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PackageTransactionPriceNotation")),
			Optional.of("135 Package Transaction Price Notation"),
			Arrays.asList()
		);
		this.packageTransactionSpreadField = new FieldImpl(
			"packageTransactionSpread",
			false,
			Optional.empty(),
			Optional.empty(),
			packageTransactionSpreadTypeTabulator.getFields()
		);
		this.packageTransactionSpreadCurrencyField = new FieldImpl(
			"packageTransactionSpreadCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PackageTransactionSpreadCurrency")),
			Optional.of("137 Package transaction spread currency"),
			Arrays.asList()
		);
		this.packageTransactionSpreadNotationField = new FieldImpl(
			"packageTransactionSpreadNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PackageTransactionSpreadNotation")),
			Optional.of("138 Package transaction spread notation"),
			Arrays.asList()
		);
		this.technicalRecordIdField = new FieldImpl(
			"technicalRecordId",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "TechnicalRecordIdDTCC")),
			Optional.of("[Internal] Technical Record Id (DTCC)"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(effectiveDateField, expirationDateField, earlyTerminationDateField, reportingTimestampField, executionTimestampField, entityResponsibleForReportingField, counterparty1Field, counterparty2Field, counterparty2IdentifierTypeField, direction1Field, direction2Leg1Field, direction2Leg2Field, submitterIdentifierField, newSDRIdentifierField, originalSwapSDRIdentifierField, clearedField, centralCounterpartyField, clearingMemberField, platformIdentifierField, confirmedField, finalContractualSettlementDateField, confirmationTimestampField, settlementCurrencyLeg1Field, settlementCurrencyLeg2Field, clearingAccountOriginField, originalSwapUTIField, clearingReceiptTimestampField, utiField, utiProprietaryField, priorUtiField, priorUtiProprietaryField, fixedRateDayCountConventionLeg1Field, floatingRateDayCountConventionOfLeg1Field, fixedRateDayCountConventionLeg2Field, floatingRateDayCountConventionOfLeg2Field, fixedRatePaymentFrequencyPeriodLeg1Field, fixedRatePaymentFrequencyPeriodLeg2Field, floatingRatePaymentFrequencyPeriodLeg1Field, floatingRatePaymentFrequencyPeriodLeg2Field, fixedRatePaymentFrequencyPeriodMultiplierLeg1Field, fixedRatePaymentFrequencyPeriodMultiplierLeg2Field, floatingRatePaymentFrequencyPeriodMultiplierLeg1Field, floatingRatePaymentFrequencyPeriodMultiplierLeg2Field, fixingDateLeg1Field, fixingDateLeg2Field, floatingRateResetFrequencyPeriodLeg1Field, floatingRateResetFrequencyPeriodOfLeg2Field, floatingRateResetFrequencyPeriodMultiplierLeg1Field, floatingRateResetFrequencyPeriodMultiplierLeg2Field, otherPaymentField, valuationAmountField, valuationCurrencyField, valuationMethodField, valuationTimestampField, collateralPortfolioIndicatorField, initialMarginCollateralPortfolioCodeField, variationMarginCollateralPortfolioCodeField, priceField, priceNotationField, priceUnitOfMeasureField, priceScheduleField, fixedRateLeg1Field, fixedRateLeg2Field, spreadOfLeg1Field, spreadNotationOfLeg1Field, spreadOfLeg2Field, spreadNotationOfLeg2Field, strikePriceField, strikePriceNotationField, strikePriceScheduleField, optionPremiumAmountField, optionPremiumCurrencyField, optionPremiumPaymentDateField, firstExerciseDateField, exchangeRateField, exchangeRateBasisField, notionalLeg1Field, notionalLeg2Field, callAmountField, putAmountField, notionalCurrencyOfLeg1Field, notionalCurrencyOfLeg2Field, callCurrencyField, putCurrencyField, quantityUnitOfMeasureLeg1Field, quantityUnitOfMeasureLeg2Field, notionalAmountScheduleLeg1Field, notionalAmountScheduleLeg2Field, totalNotionalQuantityLeg1Field, totalNotionalQuantityLeg2Field, notionalQuantityScheduleLeg1Field, notionalQuantityScheduleLeg2Field, actionTypeField, eventTypeField, eventTimestampField, eventIdentifierField, indexFactorField, embeddedOptionTypeField, upiField, deliveryTypeField, assetClassField, underlyingIdentificationTypeField, underlyingIdentificationField, underlyingIndexIndicatorField, nameOfTheUnderlyingIndexField, referenceEntityField, indicatorOfTheFloatingRateOfLeg1Field, indicatorOfTheFloatingRateOfLeg2Field, nameOfTheFloatingRateOfLeg1Field, nameOfTheFloatingRateOfLeg2Field, floatingRateReferencePeriodOfLeg1TimePeriodField, floatingRateReferencePeriodOfLeg2TimePeriodField, floatingRateReferencePeriodOfLeg1MultiplierField, floatingRateReferencePeriodOfLeg2MultiplierField, isCryptoField, maturityDateOfTheUnderlyingField, seniorityField, seriesField, versionField, cdSIndexAttachmentPointField, cdSIndexDetachmentPointField, customBasketCodeField, identifierOfBasketConstituentsField, basketConstituentIdentifierSourceField, contractTypeField, optionStyleField, optionTypeField, packageIdentifierField, packageTransactionPriceField, packageTransactionPriceCurrencyField, packageTransactionPriceNotationField, packageTransactionSpreadField, packageTransactionSpreadCurrencyField, packageTransactionSpreadNotationField, technicalRecordIdField);
	}
	
	@Override
	public List<FieldValue> tabulate(JFSATransactionReport input) {
		Optional<Date> effectiveDate = Optional.ofNullable(input.getEffectiveDate());
		Optional<Date> expirationDate = Optional.ofNullable(input.getExpirationDate());
		Optional<Date> earlyTerminationDate = Optional.ofNullable(input.getEarlyTerminationDate());
		Optional<ZonedDateTime> reportingTimestamp = Optional.ofNullable(input.getReportingTimestamp());
		Optional<ZonedDateTime> executionTimestamp = Optional.ofNullable(input.getExecutionTimestamp());
		Optional<String> entityResponsibleForReporting = Optional.ofNullable(input.getEntityResponsibleForReporting());
		Optional<String> counterparty1 = Optional.ofNullable(input.getCounterparty1());
		Optional<String> counterparty2 = Optional.ofNullable(input.getCounterparty2());
		Optional<Boolean> counterparty2IdentifierType = Optional.ofNullable(input.getCounterparty2IdentifierType());
		Optional<String> direction1 = Optional.ofNullable(input.getDirection1());
		Optional<String> direction2Leg1 = Optional.ofNullable(input.getDirection2Leg1());
		Optional<String> direction2Leg2 = Optional.ofNullable(input.getDirection2Leg2());
		Optional<String> submitterIdentifier = Optional.ofNullable(input.getSubmitterIdentifier());
		Optional<String> newSDRIdentifier = Optional.ofNullable(input.getNewSDRIdentifier());
		Optional<String> originalSwapSDRIdentifier = Optional.ofNullable(input.getOriginalSwapSDRIdentifier());
		Optional<String> cleared = Optional.ofNullable(input.getCleared());
		Optional<String> centralCounterparty = Optional.ofNullable(input.getCentralCounterparty());
		Optional<String> clearingMember = Optional.ofNullable(input.getClearingMember());
		Optional<String> platformIdentifier = Optional.ofNullable(input.getPlatformIdentifier());
		Optional<String> confirmed = Optional.ofNullable(input.getConfirmed());
		Optional<Date> finalContractualSettlementDate = Optional.ofNullable(input.getFinalContractualSettlementDate());
		Optional<ZonedDateTime> confirmationTimestamp = Optional.ofNullable(input.getConfirmationTimestamp());
		Optional<ISOCurrencyCodeEnum> settlementCurrencyLeg1 = Optional.ofNullable(input.getSettlementCurrencyLeg1());
		Optional<ISOCurrencyCodeEnum> settlementCurrencyLeg2 = Optional.ofNullable(input.getSettlementCurrencyLeg2());
		Optional<String> clearingAccountOrigin = Optional.ofNullable(input.getClearingAccountOrigin());
		Optional<String> originalSwapUTI = Optional.ofNullable(input.getOriginalSwapUTI());
		Optional<ZonedDateTime> clearingReceiptTimestamp = Optional.ofNullable(input.getClearingReceiptTimestamp());
		Optional<String> uti = Optional.ofNullable(input.getUti());
		Optional<String> utiProprietary = Optional.ofNullable(input.getUtiProprietary());
		Optional<String> priorUti = Optional.ofNullable(input.getPriorUti());
		Optional<String> priorUtiProprietary = Optional.ofNullable(input.getPriorUtiProprietary());
		Optional<InterestComputationMethod4Code> fixedRateDayCountConventionLeg1 = Optional.ofNullable(input.getFixedRateDayCountConventionLeg1());
		Optional<InterestComputationMethod4Code> floatingRateDayCountConventionOfLeg1 = Optional.ofNullable(input.getFloatingRateDayCountConventionOfLeg1());
		Optional<InterestComputationMethod4Code> fixedRateDayCountConventionLeg2 = Optional.ofNullable(input.getFixedRateDayCountConventionLeg2());
		Optional<InterestComputationMethod4Code> floatingRateDayCountConventionOfLeg2 = Optional.ofNullable(input.getFloatingRateDayCountConventionOfLeg2());
		Optional<String> fixedRatePaymentFrequencyPeriodLeg1 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodLeg1());
		Optional<String> fixedRatePaymentFrequencyPeriodLeg2 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodLeg2());
		Optional<String> floatingRatePaymentFrequencyPeriodLeg1 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodLeg1());
		Optional<String> floatingRatePaymentFrequencyPeriodLeg2 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodLeg2());
		Optional<Integer> fixedRatePaymentFrequencyPeriodMultiplierLeg1 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodMultiplierLeg1());
		Optional<Integer> fixedRatePaymentFrequencyPeriodMultiplierLeg2 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodMultiplierLeg2());
		Optional<Integer> floatingRatePaymentFrequencyPeriodMultiplierLeg1 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodMultiplierLeg1());
		Optional<Integer> floatingRatePaymentFrequencyPeriodMultiplierLeg2 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodMultiplierLeg2());
		Optional<ZonedDateTime> fixingDateLeg1 = Optional.ofNullable(input.getFixingDateLeg1());
		Optional<ZonedDateTime> fixingDateLeg2 = Optional.ofNullable(input.getFixingDateLeg2());
		Optional<String> floatingRateResetFrequencyPeriodLeg1 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodLeg1());
		Optional<String> floatingRateResetFrequencyPeriodOfLeg2 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodOfLeg2());
		Optional<Integer> floatingRateResetFrequencyPeriodMultiplierLeg1 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodMultiplierLeg1());
		Optional<Integer> floatingRateResetFrequencyPeriodMultiplierLeg2 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodMultiplierLeg2());
		Optional<List<List<FieldValue>>> otherPayment = Optional.ofNullable(input.getOtherPayment())
			.map(x -> x.stream()
				.map(_x -> otherPaymentReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<BigDecimal> valuationAmount = Optional.ofNullable(input.getValuationAmount());
		Optional<ISOCurrencyCodeEnum> valuationCurrency = Optional.ofNullable(input.getValuationCurrency());
		Optional<ValuationType1Code> valuationMethod = Optional.ofNullable(input.getValuationMethod());
		Optional<ZonedDateTime> valuationTimestamp = Optional.ofNullable(input.getValuationTimestamp());
		Optional<Boolean> collateralPortfolioIndicator = Optional.ofNullable(input.getCollateralPortfolioIndicator());
		Optional<String> initialMarginCollateralPortfolioCode = Optional.ofNullable(input.getInitialMarginCollateralPortfolioCode());
		Optional<String> variationMarginCollateralPortfolioCode = Optional.ofNullable(input.getVariationMarginCollateralPortfolioCode());
		Optional<List<FieldValue>> price = Optional.ofNullable(input.getPrice())
			.map(x -> priceAndNotationTypeTabulator.tabulate(x));
		Optional<String> priceNotation = Optional.ofNullable(input.getPriceNotation());
		Optional<String> priceUnitOfMeasure = Optional.ofNullable(input.getPriceUnitOfMeasure());
		Optional<List<List<FieldValue>>> priceSchedule = Optional.ofNullable(input.getPriceSchedule())
			.map(x -> x.stream()
				.map(_x -> priceScheduleReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<BigDecimal> fixedRateLeg1 = Optional.ofNullable(input.getFixedRateLeg1());
		Optional<BigDecimal> fixedRateLeg2 = Optional.ofNullable(input.getFixedRateLeg2());
		Optional<List<FieldValue>> spreadOfLeg1 = Optional.ofNullable(input.getSpreadOfLeg1())
			.map(x -> spreadOfLeg1TypeTabulator.tabulate(x));
		Optional<String> spreadNotationOfLeg1 = Optional.ofNullable(input.getSpreadNotationOfLeg1());
		Optional<List<FieldValue>> spreadOfLeg2 = Optional.ofNullable(input.getSpreadOfLeg2())
			.map(x -> spreadOfLeg2TypeTabulator.tabulate(x));
		Optional<String> spreadNotationOfLeg2 = Optional.ofNullable(input.getSpreadNotationOfLeg2());
		Optional<List<FieldValue>> strikePrice = Optional.ofNullable(input.getStrikePrice())
			.map(x -> strikePriceAndNotationTypeTabulator.tabulate(x));
		Optional<String> strikePriceNotation = Optional.ofNullable(input.getStrikePriceNotation());
		Optional<List<List<FieldValue>>> strikePriceSchedule = Optional.ofNullable(input.getStrikePriceSchedule())
			.map(x -> x.stream()
				.map(_x -> strikePriceScheduleReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<BigDecimal> optionPremiumAmount = Optional.ofNullable(input.getOptionPremiumAmount());
		Optional<ISOCurrencyCodeEnum> optionPremiumCurrency = Optional.ofNullable(input.getOptionPremiumCurrency());
		Optional<Date> optionPremiumPaymentDate = Optional.ofNullable(input.getOptionPremiumPaymentDate());
		Optional<Date> firstExerciseDate = Optional.ofNullable(input.getFirstExerciseDate());
		Optional<BigDecimal> exchangeRate = Optional.ofNullable(input.getExchangeRate());
		Optional<String> exchangeRateBasis = Optional.ofNullable(input.getExchangeRateBasis());
		Optional<BigDecimal> notionalLeg1 = Optional.ofNullable(input.getNotionalLeg1());
		Optional<BigDecimal> notionalLeg2 = Optional.ofNullable(input.getNotionalLeg2());
		Optional<BigDecimal> callAmount = Optional.ofNullable(input.getCallAmount());
		Optional<BigDecimal> putAmount = Optional.ofNullable(input.getPutAmount());
		Optional<ISOCurrencyCodeEnum> notionalCurrencyOfLeg1 = Optional.ofNullable(input.getNotionalCurrencyOfLeg1());
		Optional<ISOCurrencyCodeEnum> notionalCurrencyOfLeg2 = Optional.ofNullable(input.getNotionalCurrencyOfLeg2());
		Optional<ISOCurrencyCodeEnum> callCurrency = Optional.ofNullable(input.getCallCurrency());
		Optional<ISOCurrencyCodeEnum> putCurrency = Optional.ofNullable(input.getPutCurrency());
		Optional<String> quantityUnitOfMeasureLeg1 = Optional.ofNullable(input.getQuantityUnitOfMeasureLeg1());
		Optional<String> quantityUnitOfMeasureLeg2 = Optional.ofNullable(input.getQuantityUnitOfMeasureLeg2());
		Optional<List<List<FieldValue>>> notionalAmountScheduleLeg1 = Optional.ofNullable(input.getNotionalAmountScheduleLeg1())
			.map(x -> x.stream()
				.map(_x -> notionalAmountScheduleLeg1ReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<List<List<FieldValue>>> notionalAmountScheduleLeg2 = Optional.ofNullable(input.getNotionalAmountScheduleLeg2())
			.map(x -> x.stream()
				.map(_x -> notionalAmountScheduleLeg2ReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<BigDecimal> totalNotionalQuantityLeg1 = Optional.ofNullable(input.getTotalNotionalQuantityLeg1());
		Optional<BigDecimal> totalNotionalQuantityLeg2 = Optional.ofNullable(input.getTotalNotionalQuantityLeg2());
		Optional<List<List<FieldValue>>> notionalQuantityScheduleLeg1 = Optional.ofNullable(input.getNotionalQuantityScheduleLeg1())
			.map(x -> x.stream()
				.map(_x -> notionalQuantityScheduleLeg1ReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<List<List<FieldValue>>> notionalQuantityScheduleLeg2 = Optional.ofNullable(input.getNotionalQuantityScheduleLeg2())
			.map(x -> x.stream()
				.map(_x -> notionalQuantityScheduleLeg2ReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<String> actionType = Optional.ofNullable(input.getActionType());
		Optional<String> eventType = Optional.ofNullable(input.getEventType());
		Optional<ZonedDateTime> eventTimestamp = Optional.ofNullable(input.getEventTimestamp());
		Optional<String> eventIdentifier = Optional.ofNullable(input.getEventIdentifier());
		Optional<BigDecimal> indexFactor = Optional.ofNullable(input.getIndexFactor());
		Optional<String> embeddedOptionType = Optional.ofNullable(input.getEmbeddedOptionType());
		Optional<String> upi = Optional.ofNullable(input.getUpi());
		Optional<String> deliveryType = Optional.ofNullable(input.getDeliveryType());
		Optional<String> assetClass = Optional.ofNullable(input.getAssetClass());
		Optional<String> underlyingIdentificationType = Optional.ofNullable(input.getUnderlyingIdentificationType());
		Optional<String> underlyingIdentification = Optional.ofNullable(input.getUnderlyingIdentification());
		Optional<String> underlyingIndexIndicator = Optional.ofNullable(input.getUnderlyingIndexIndicator());
		Optional<String> nameOfTheUnderlyingIndex = Optional.ofNullable(input.getNameOfTheUnderlyingIndex());
		Optional<String> referenceEntity = Optional.ofNullable(input.getReferenceEntity());
		Optional<String> indicatorOfTheFloatingRateOfLeg1 = Optional.ofNullable(input.getIndicatorOfTheFloatingRateOfLeg1());
		Optional<String> indicatorOfTheFloatingRateOfLeg2 = Optional.ofNullable(input.getIndicatorOfTheFloatingRateOfLeg2());
		Optional<String> nameOfTheFloatingRateOfLeg1 = Optional.ofNullable(input.getNameOfTheFloatingRateOfLeg1());
		Optional<String> nameOfTheFloatingRateOfLeg2 = Optional.ofNullable(input.getNameOfTheFloatingRateOfLeg2());
		Optional<String> floatingRateReferencePeriodOfLeg1TimePeriod = Optional.ofNullable(input.getFloatingRateReferencePeriodOfLeg1TimePeriod());
		Optional<String> floatingRateReferencePeriodOfLeg2TimePeriod = Optional.ofNullable(input.getFloatingRateReferencePeriodOfLeg2TimePeriod());
		Optional<Integer> floatingRateReferencePeriodOfLeg1Multiplier = Optional.ofNullable(input.getFloatingRateReferencePeriodOfLeg1Multiplier());
		Optional<Integer> floatingRateReferencePeriodOfLeg2Multiplier = Optional.ofNullable(input.getFloatingRateReferencePeriodOfLeg2Multiplier());
		Optional<Boolean> isCrypto = Optional.ofNullable(input.getIsCrypto());
		Optional<Date> maturityDateOfTheUnderlying = Optional.ofNullable(input.getMaturityDateOfTheUnderlying());
		Optional<String> seniority = Optional.ofNullable(input.getSeniority());
		Optional<Integer> series = Optional.ofNullable(input.getSeries());
		Optional<Integer> version = Optional.ofNullable(input.getVersion());
		Optional<BigDecimal> cdSIndexAttachmentPoint = Optional.ofNullable(input.getCdSIndexAttachmentPoint());
		Optional<BigDecimal> cdSIndexDetachmentPoint = Optional.ofNullable(input.getCdSIndexDetachmentPoint());
		Optional<String> customBasketCode = Optional.ofNullable(input.getCustomBasketCode());
		Optional<List<? extends String>> identifierOfBasketConstituents = Optional.ofNullable(input.getIdentifierOfBasketConstituents());
		Optional<List<? extends ProductIdTypeEnum>> basketConstituentIdentifierSource = Optional.ofNullable(input.getBasketConstituentIdentifierSource());
		Optional<String> contractType = Optional.ofNullable(input.getContractType());
		Optional<String> optionStyle = Optional.ofNullable(input.getOptionStyle());
		Optional<String> optionType = Optional.ofNullable(input.getOptionType());
		Optional<String> packageIdentifier = Optional.ofNullable(input.getPackageIdentifier());
		Optional<List<FieldValue>> packageTransactionPrice = Optional.ofNullable(input.getPackageTransactionPrice())
			.map(x -> packageTransactionPriceTypeTabulator.tabulate(x));
		Optional<ISOCurrencyCodeEnum> packageTransactionPriceCurrency = Optional.ofNullable(input.getPackageTransactionPriceCurrency());
		Optional<String> packageTransactionPriceNotation = Optional.ofNullable(input.getPackageTransactionPriceNotation());
		Optional<List<FieldValue>> packageTransactionSpread = Optional.ofNullable(input.getPackageTransactionSpread())
			.map(x -> packageTransactionSpreadTypeTabulator.tabulate(x));
		Optional<ISOCurrencyCodeEnum> packageTransactionSpreadCurrency = Optional.ofNullable(input.getPackageTransactionSpreadCurrency());
		Optional<String> packageTransactionSpreadNotation = Optional.ofNullable(input.getPackageTransactionSpreadNotation());
		Optional<String> technicalRecordId = Optional.ofNullable(input.getTechnicalRecordId());
		return Arrays.asList(
			new FieldValueImpl(effectiveDateField, effectiveDate),
			new FieldValueImpl(expirationDateField, expirationDate),
			new FieldValueImpl(earlyTerminationDateField, earlyTerminationDate),
			new FieldValueImpl(reportingTimestampField, reportingTimestamp),
			new FieldValueImpl(executionTimestampField, executionTimestamp),
			new FieldValueImpl(entityResponsibleForReportingField, entityResponsibleForReporting),
			new FieldValueImpl(counterparty1Field, counterparty1),
			new FieldValueImpl(counterparty2Field, counterparty2),
			new FieldValueImpl(counterparty2IdentifierTypeField, counterparty2IdentifierType),
			new FieldValueImpl(direction1Field, direction1),
			new FieldValueImpl(direction2Leg1Field, direction2Leg1),
			new FieldValueImpl(direction2Leg2Field, direction2Leg2),
			new FieldValueImpl(submitterIdentifierField, submitterIdentifier),
			new FieldValueImpl(newSDRIdentifierField, newSDRIdentifier),
			new FieldValueImpl(originalSwapSDRIdentifierField, originalSwapSDRIdentifier),
			new FieldValueImpl(clearedField, cleared),
			new FieldValueImpl(centralCounterpartyField, centralCounterparty),
			new FieldValueImpl(clearingMemberField, clearingMember),
			new FieldValueImpl(platformIdentifierField, platformIdentifier),
			new FieldValueImpl(confirmedField, confirmed),
			new FieldValueImpl(finalContractualSettlementDateField, finalContractualSettlementDate),
			new FieldValueImpl(confirmationTimestampField, confirmationTimestamp),
			new FieldValueImpl(settlementCurrencyLeg1Field, settlementCurrencyLeg1),
			new FieldValueImpl(settlementCurrencyLeg2Field, settlementCurrencyLeg2),
			new FieldValueImpl(clearingAccountOriginField, clearingAccountOrigin),
			new FieldValueImpl(originalSwapUTIField, originalSwapUTI),
			new FieldValueImpl(clearingReceiptTimestampField, clearingReceiptTimestamp),
			new FieldValueImpl(utiField, uti),
			new FieldValueImpl(utiProprietaryField, utiProprietary),
			new FieldValueImpl(priorUtiField, priorUti),
			new FieldValueImpl(priorUtiProprietaryField, priorUtiProprietary),
			new FieldValueImpl(fixedRateDayCountConventionLeg1Field, fixedRateDayCountConventionLeg1),
			new FieldValueImpl(floatingRateDayCountConventionOfLeg1Field, floatingRateDayCountConventionOfLeg1),
			new FieldValueImpl(fixedRateDayCountConventionLeg2Field, fixedRateDayCountConventionLeg2),
			new FieldValueImpl(floatingRateDayCountConventionOfLeg2Field, floatingRateDayCountConventionOfLeg2),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodLeg1Field, fixedRatePaymentFrequencyPeriodLeg1),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodLeg2Field, fixedRatePaymentFrequencyPeriodLeg2),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodLeg1Field, floatingRatePaymentFrequencyPeriodLeg1),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodLeg2Field, floatingRatePaymentFrequencyPeriodLeg2),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodMultiplierLeg1Field, fixedRatePaymentFrequencyPeriodMultiplierLeg1),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodMultiplierLeg2Field, fixedRatePaymentFrequencyPeriodMultiplierLeg2),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodMultiplierLeg1Field, floatingRatePaymentFrequencyPeriodMultiplierLeg1),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodMultiplierLeg2Field, floatingRatePaymentFrequencyPeriodMultiplierLeg2),
			new FieldValueImpl(fixingDateLeg1Field, fixingDateLeg1),
			new FieldValueImpl(fixingDateLeg2Field, fixingDateLeg2),
			new FieldValueImpl(floatingRateResetFrequencyPeriodLeg1Field, floatingRateResetFrequencyPeriodLeg1),
			new FieldValueImpl(floatingRateResetFrequencyPeriodOfLeg2Field, floatingRateResetFrequencyPeriodOfLeg2),
			new FieldValueImpl(floatingRateResetFrequencyPeriodMultiplierLeg1Field, floatingRateResetFrequencyPeriodMultiplierLeg1),
			new FieldValueImpl(floatingRateResetFrequencyPeriodMultiplierLeg2Field, floatingRateResetFrequencyPeriodMultiplierLeg2),
			new MultiNestedFieldValueImpl(otherPaymentField, otherPayment),
			new FieldValueImpl(valuationAmountField, valuationAmount),
			new FieldValueImpl(valuationCurrencyField, valuationCurrency),
			new FieldValueImpl(valuationMethodField, valuationMethod),
			new FieldValueImpl(valuationTimestampField, valuationTimestamp),
			new FieldValueImpl(collateralPortfolioIndicatorField, collateralPortfolioIndicator),
			new FieldValueImpl(initialMarginCollateralPortfolioCodeField, initialMarginCollateralPortfolioCode),
			new FieldValueImpl(variationMarginCollateralPortfolioCodeField, variationMarginCollateralPortfolioCode),
			new NestedFieldValueImpl(priceField, price),
			new FieldValueImpl(priceNotationField, priceNotation),
			new FieldValueImpl(priceUnitOfMeasureField, priceUnitOfMeasure),
			new MultiNestedFieldValueImpl(priceScheduleField, priceSchedule),
			new FieldValueImpl(fixedRateLeg1Field, fixedRateLeg1),
			new FieldValueImpl(fixedRateLeg2Field, fixedRateLeg2),
			new NestedFieldValueImpl(spreadOfLeg1Field, spreadOfLeg1),
			new FieldValueImpl(spreadNotationOfLeg1Field, spreadNotationOfLeg1),
			new NestedFieldValueImpl(spreadOfLeg2Field, spreadOfLeg2),
			new FieldValueImpl(spreadNotationOfLeg2Field, spreadNotationOfLeg2),
			new NestedFieldValueImpl(strikePriceField, strikePrice),
			new FieldValueImpl(strikePriceNotationField, strikePriceNotation),
			new MultiNestedFieldValueImpl(strikePriceScheduleField, strikePriceSchedule),
			new FieldValueImpl(optionPremiumAmountField, optionPremiumAmount),
			new FieldValueImpl(optionPremiumCurrencyField, optionPremiumCurrency),
			new FieldValueImpl(optionPremiumPaymentDateField, optionPremiumPaymentDate),
			new FieldValueImpl(firstExerciseDateField, firstExerciseDate),
			new FieldValueImpl(exchangeRateField, exchangeRate),
			new FieldValueImpl(exchangeRateBasisField, exchangeRateBasis),
			new FieldValueImpl(notionalLeg1Field, notionalLeg1),
			new FieldValueImpl(notionalLeg2Field, notionalLeg2),
			new FieldValueImpl(callAmountField, callAmount),
			new FieldValueImpl(putAmountField, putAmount),
			new FieldValueImpl(notionalCurrencyOfLeg1Field, notionalCurrencyOfLeg1),
			new FieldValueImpl(notionalCurrencyOfLeg2Field, notionalCurrencyOfLeg2),
			new FieldValueImpl(callCurrencyField, callCurrency),
			new FieldValueImpl(putCurrencyField, putCurrency),
			new FieldValueImpl(quantityUnitOfMeasureLeg1Field, quantityUnitOfMeasureLeg1),
			new FieldValueImpl(quantityUnitOfMeasureLeg2Field, quantityUnitOfMeasureLeg2),
			new MultiNestedFieldValueImpl(notionalAmountScheduleLeg1Field, notionalAmountScheduleLeg1),
			new MultiNestedFieldValueImpl(notionalAmountScheduleLeg2Field, notionalAmountScheduleLeg2),
			new FieldValueImpl(totalNotionalQuantityLeg1Field, totalNotionalQuantityLeg1),
			new FieldValueImpl(totalNotionalQuantityLeg2Field, totalNotionalQuantityLeg2),
			new MultiNestedFieldValueImpl(notionalQuantityScheduleLeg1Field, notionalQuantityScheduleLeg1),
			new MultiNestedFieldValueImpl(notionalQuantityScheduleLeg2Field, notionalQuantityScheduleLeg2),
			new FieldValueImpl(actionTypeField, actionType),
			new FieldValueImpl(eventTypeField, eventType),
			new FieldValueImpl(eventTimestampField, eventTimestamp),
			new FieldValueImpl(eventIdentifierField, eventIdentifier),
			new FieldValueImpl(indexFactorField, indexFactor),
			new FieldValueImpl(embeddedOptionTypeField, embeddedOptionType),
			new FieldValueImpl(upiField, upi),
			new FieldValueImpl(deliveryTypeField, deliveryType),
			new FieldValueImpl(assetClassField, assetClass),
			new FieldValueImpl(underlyingIdentificationTypeField, underlyingIdentificationType),
			new FieldValueImpl(underlyingIdentificationField, underlyingIdentification),
			new FieldValueImpl(underlyingIndexIndicatorField, underlyingIndexIndicator),
			new FieldValueImpl(nameOfTheUnderlyingIndexField, nameOfTheUnderlyingIndex),
			new FieldValueImpl(referenceEntityField, referenceEntity),
			new FieldValueImpl(indicatorOfTheFloatingRateOfLeg1Field, indicatorOfTheFloatingRateOfLeg1),
			new FieldValueImpl(indicatorOfTheFloatingRateOfLeg2Field, indicatorOfTheFloatingRateOfLeg2),
			new FieldValueImpl(nameOfTheFloatingRateOfLeg1Field, nameOfTheFloatingRateOfLeg1),
			new FieldValueImpl(nameOfTheFloatingRateOfLeg2Field, nameOfTheFloatingRateOfLeg2),
			new FieldValueImpl(floatingRateReferencePeriodOfLeg1TimePeriodField, floatingRateReferencePeriodOfLeg1TimePeriod),
			new FieldValueImpl(floatingRateReferencePeriodOfLeg2TimePeriodField, floatingRateReferencePeriodOfLeg2TimePeriod),
			new FieldValueImpl(floatingRateReferencePeriodOfLeg1MultiplierField, floatingRateReferencePeriodOfLeg1Multiplier),
			new FieldValueImpl(floatingRateReferencePeriodOfLeg2MultiplierField, floatingRateReferencePeriodOfLeg2Multiplier),
			new FieldValueImpl(isCryptoField, isCrypto),
			new FieldValueImpl(maturityDateOfTheUnderlyingField, maturityDateOfTheUnderlying),
			new FieldValueImpl(seniorityField, seniority),
			new FieldValueImpl(seriesField, series),
			new FieldValueImpl(versionField, version),
			new FieldValueImpl(cdSIndexAttachmentPointField, cdSIndexAttachmentPoint),
			new FieldValueImpl(cdSIndexDetachmentPointField, cdSIndexDetachmentPoint),
			new FieldValueImpl(customBasketCodeField, customBasketCode),
			new FieldValueImpl(identifierOfBasketConstituentsField, identifierOfBasketConstituents),
			new FieldValueImpl(basketConstituentIdentifierSourceField, basketConstituentIdentifierSource),
			new FieldValueImpl(contractTypeField, contractType),
			new FieldValueImpl(optionStyleField, optionStyle),
			new FieldValueImpl(optionTypeField, optionType),
			new FieldValueImpl(packageIdentifierField, packageIdentifier),
			new NestedFieldValueImpl(packageTransactionPriceField, packageTransactionPrice),
			new FieldValueImpl(packageTransactionPriceCurrencyField, packageTransactionPriceCurrency),
			new FieldValueImpl(packageTransactionPriceNotationField, packageTransactionPriceNotation),
			new NestedFieldValueImpl(packageTransactionSpreadField, packageTransactionSpread),
			new FieldValueImpl(packageTransactionSpreadCurrencyField, packageTransactionSpreadCurrency),
			new FieldValueImpl(packageTransactionSpreadNotationField, packageTransactionSpreadNotation),
			new FieldValueImpl(technicalRecordIdField, technicalRecordId)
		);
	}
}
