package drr.regulation.jfsa.rewrite.trade.reports;

import cdm.base.math.DatedValue;
import cdm.base.math.NonNegativeQuantitySchedule;
import cdm.base.math.metafields.ReferenceWithMetaNonNegativeQuantitySchedule;
import cdm.event.common.Trade;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.GetLeg1ResolvablePriceQuantity;
import drr.regulation.common.functions.TradeForEvent;
import drr.regulation.jfsa.rewrite.trade.NotionalAmountScheduleLeg1Report;
import drr.regulation.jfsa.rewrite.trade.NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.inject.Inject;


@ImplementedBy(NotionalAmountScheduleLeg1Rule.NotionalAmountScheduleLeg1RuleDefault.class)
public abstract class NotionalAmountScheduleLeg1Rule implements ReportFunction<TransactionReportInstruction, List<? extends NotionalAmountScheduleLeg1Report>> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected GetLeg1ResolvablePriceQuantity getLeg1ResolvablePriceQuantity;
	@Inject protected NotionalAmountScheduleLeg1AmountRule notionalAmountScheduleLeg1Amount;
	@Inject protected NotionalAmountScheduleLeg1EffectiveDateRule notionalAmountScheduleLeg1EffectiveDate;
	@Inject protected TradeForEvent tradeForEvent;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public List<? extends NotionalAmountScheduleLeg1Report> evaluate(TransactionReportInstruction input) {
		List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> outputBuilder = doEvaluate(input);
		
		final List<? extends NotionalAmountScheduleLeg1Report> output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.stream().map(NotionalAmountScheduleLeg1Report::build).collect(Collectors.toList());
			objectValidator.validate(NotionalAmountScheduleLeg1Report.class, output);
		}
		
		return output;
	}

	protected abstract List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> doEvaluate(TransactionReportInstruction input);

	public static class NotionalAmountScheduleLeg1RuleDefault extends NotionalAmountScheduleLeg1Rule {
		@Override
		protected List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> doEvaluate(TransactionReportInstruction input) {
			List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> output = new ArrayList<>();
			return assignOutput(output, input);
		}
		
		protected List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> assignOutput(List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> output, TransactionReportInstruction input) {
			final MapperS<Trade> thenResult0 = MapperS.of(input)
				.mapSingleToItem(item -> MapperS.of(tradeForEvent.evaluate(item.get())));
			final MapperC<DatedValue> thenResult1 = thenResult0
				.mapSingleToList(item -> MapperS.of(getLeg1ResolvablePriceQuantity.evaluate(item.get())).<ReferenceWithMetaNonNegativeQuantitySchedule>map("getQuantitySchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getQuantitySchedule()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()).<DatedValue>mapC("getDatedValue", measureSchedule -> measureSchedule.getDatedValue()));
			final MapperC<NotionalAmountScheduleLeg1Report> thenResult2 = thenResult1
				.mapItem(item -> MapperS.of(NotionalAmountScheduleLeg1Report.builder()
					.setAmount(notionalAmountScheduleLeg1Amount.evaluate(item.get()))
					.setEffectiveDate(notionalAmountScheduleLeg1EffectiveDate.evaluate(item.get()))
					.build()
				));
			output = toBuilder(thenResult2
				.sort(item -> item.<Date>map("getEffectiveDate", notionalAmountScheduleLeg1Report -> notionalAmountScheduleLeg1Report.getEffectiveDate())).getMulti());
			
			return Optional.ofNullable(output)
				.map(o -> o.stream().map(i -> i.prune()).collect(Collectors.toList()))
				.orElse(null);
		}
	}
}
