package drr.regulation.jfsa.rewrite.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.jfsa.rewrite.trade.NotionalQuantityScheduleLeg1Report;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class NotionalQuantityScheduleLeg1ReportTypeTabulator implements Tabulator<NotionalQuantityScheduleLeg1Report> {
	private final Field effectiveDateField;
	private final Field endDateField;
	private final Field valueField;
	
	public NotionalQuantityScheduleLeg1ReportTypeTabulator() {
		this.effectiveDateField = new FieldImpl(
			"effectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NotionalQuantityScheduleLeg1EffectiveDate")),
			Optional.of("98/100-$ 98 Notional quantity schedules - Unadjusted date on which the associated notional quantity becomes effective of leg 1"),
			Arrays.asList()
		);
		this.endDateField = new FieldImpl(
			"endDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NotionalQuantityScheduleLeg1EndDate")),
			Optional.of("98/100-$ 99 Notional quantity schedules - Unadjusted end date of the notional quantity of leg 1"),
			Arrays.asList()
		);
		this.valueField = new FieldImpl(
			"value",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "NotionalQuantityScheduleLeg1Value")),
			Optional.of("98/100-$ 100 Notional quantity which becomes effective on the associated unadjusted effective date-Leg 1"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(effectiveDateField, endDateField, valueField);
	}
	
	@Override
	public List<FieldValue> tabulate(NotionalQuantityScheduleLeg1Report input) {
		Optional<Date> effectiveDate = Optional.ofNullable(input.getEffectiveDate());
		Optional<Date> endDate = Optional.ofNullable(input.getEndDate());
		Optional<BigDecimal> value = Optional.ofNullable(input.getValue());
		return Arrays.asList(
			new FieldValueImpl(effectiveDateField, effectiveDate),
			new FieldValueImpl(endDateField, endDate),
			new FieldValueImpl(valueField, value)
		);
	}
}
