package drr.regulation.jfsa.rewrite.trade.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.QuantityReportablePeriod;
import drr.standards.iosco.cde.reports.CDENotionalQuantityScheduleEffectiveDateRule;
import javax.inject.Inject;


@ImplementedBy(NotionalQuantityScheduleLeg2EffectiveDateRule.NotionalQuantityScheduleLeg2EffectiveDateRuleDefault.class)
public abstract class NotionalQuantityScheduleLeg2EffectiveDateRule implements ReportFunction<QuantityReportablePeriod, Date> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDENotionalQuantityScheduleEffectiveDateRule cDENotionalQuantityScheduleEffectiveDate;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public Date evaluate(QuantityReportablePeriod input) {
		Date output = doEvaluate(input);
		
		return output;
	}

	protected abstract Date doEvaluate(QuantityReportablePeriod input);

	public static class NotionalQuantityScheduleLeg2EffectiveDateRuleDefault extends NotionalQuantityScheduleLeg2EffectiveDateRule {
		@Override
		protected Date doEvaluate(QuantityReportablePeriod input) {
			Date output = null;
			return assignOutput(output, input);
		}
		
		protected Date assignOutput(Date output, QuantityReportablePeriod input) {
			output = cDENotionalQuantityScheduleEffectiveDate.evaluate(input);
			
			return output;
		}
	}
}
