package drr.regulation.jfsa.rewrite.trade.reports;

import cdm.base.staticdata.identifier.IdentifiedList;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.GetPackageInformation;
import drr.standards.iosco.cde.reports.CDEPackageIdentifierRule;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(PackageIdentifierRule.PackageIdentifierRuleDefault.class)
public abstract class PackageIdentifierRule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDEPackageIdentifierRule cDEPackageIdentifier;
	@Inject protected GetPackageInformation getPackageInformation;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class PackageIdentifierRuleDefault extends PackageIdentifierRule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			final MapperS<IdentifiedList> thenResult0 = MapperS.of(input)
				.mapSingleToItem(item -> {
					if (exists(MapperS.of(getPackageInformation.evaluate(item.get()))).getOrDefault(false)) {
						return MapperS.of(getPackageInformation.evaluate(item.get()));
					}
					return MapperS.<IdentifiedList>ofNull();
				});
			final MapperC<String> thenResult1 = MapperC.<String>of(cDEPackageIdentifier.evaluate(thenResult0.get()));
			output = MapperS.of(thenResult1.get()).get();
			
			return output;
		}
	}
}
