package drr.regulation.jfsa.rewrite.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.jfsa.rewrite.trade.PackageTransactionPrice;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class PackageTransactionPriceTypeTabulator implements Tabulator<PackageTransactionPrice> {
	private final Field packageTransactionPriceMonetaryField;
	private final Field packageTransactionPricePercentageField;
	private final Field packageTransactionPriceBasisField;
	private final Field packageTransactionPriceNotationField;
	
	public PackageTransactionPriceTypeTabulator() {
		this.packageTransactionPriceMonetaryField = new FieldImpl(
			"packageTransactionPriceMonetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PackageTransactionPriceMonetary")),
			Optional.of("133 Package Transaction Price (Monetary)"),
			Arrays.asList()
		);
		this.packageTransactionPricePercentageField = new FieldImpl(
			"packageTransactionPricePercentage",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PackageTransactionPricePercentage")),
			Optional.of("133 Package Transaction Price (Percentage)"),
			Arrays.asList()
		);
		this.packageTransactionPriceBasisField = new FieldImpl(
			"packageTransactionPriceBasis",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PackageTransactionPriceBasis")),
			Optional.of("133 Package Transaction Price (Basis)"),
			Arrays.asList()
		);
		this.packageTransactionPriceNotationField = new FieldImpl(
			"packageTransactionPriceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "GetPackageTransactionPriceNotationEnum")),
			Optional.empty(),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(packageTransactionPriceMonetaryField, packageTransactionPricePercentageField, packageTransactionPriceBasisField, packageTransactionPriceNotationField);
	}
	
	@Override
	public List<FieldValue> tabulate(PackageTransactionPrice input) {
		Optional<BigDecimal> packageTransactionPriceMonetary = Optional.ofNullable(input.getPackageTransactionPriceMonetary());
		Optional<BigDecimal> packageTransactionPricePercentage = Optional.ofNullable(input.getPackageTransactionPricePercentage());
		Optional<BigDecimal> packageTransactionPriceBasis = Optional.ofNullable(input.getPackageTransactionPriceBasis());
		Optional<PriceNotationEnum> packageTransactionPriceNotation = Optional.ofNullable(input.getPackageTransactionPriceNotation());
		return Arrays.asList(
			new FieldValueImpl(packageTransactionPriceMonetaryField, packageTransactionPriceMonetary),
			new FieldValueImpl(packageTransactionPricePercentageField, packageTransactionPricePercentage),
			new FieldValueImpl(packageTransactionPriceBasisField, packageTransactionPriceBasis),
			new FieldValueImpl(packageTransactionPriceNotationField, packageTransactionPriceNotation)
		);
	}
}
