package drr.regulation.jfsa.rewrite.trade.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.jfsa.rewrite.trade.PriceAndNotation;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class PriceAndNotationTypeTabulator implements Tabulator<PriceAndNotation> {
	private final Field priceMonetaryField;
	private final Field pricePercentageField;
	private final Field priceCurrencyField;
	private final Field priceNotationField;
	
	public PriceAndNotationTypeTabulator() {
		this.priceMonetaryField = new FieldImpl(
			"priceMonetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PriceFormatMonetary")),
			Optional.of("64 Price (Monetary)"),
			Arrays.asList()
		);
		this.pricePercentageField = new FieldImpl(
			"pricePercentage",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PriceFormatPercentage")),
			Optional.of("64 Price (Percentage)"),
			Arrays.asList()
		);
		this.priceCurrencyField = new FieldImpl(
			"priceCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "PriceCurrency")),
			Optional.of("65 Price currency"),
			Arrays.asList()
		);
		this.priceNotationField = new FieldImpl(
			"priceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "GetPriceNotationEnum")),
			Optional.empty(),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(priceMonetaryField, pricePercentageField, priceCurrencyField, priceNotationField);
	}
	
	@Override
	public List<FieldValue> tabulate(PriceAndNotation input) {
		Optional<BigDecimal> priceMonetary = Optional.ofNullable(input.getPriceMonetary());
		Optional<BigDecimal> pricePercentage = Optional.ofNullable(input.getPricePercentage());
		Optional<ISOCurrencyCodeEnum> priceCurrency = Optional.ofNullable(input.getPriceCurrency());
		Optional<PriceNotationEnum> priceNotation = Optional.ofNullable(input.getPriceNotation());
		return Arrays.asList(
			new FieldValueImpl(priceMonetaryField, priceMonetary),
			new FieldValueImpl(pricePercentageField, pricePercentage),
			new FieldValueImpl(priceCurrencyField, priceCurrency),
			new FieldValueImpl(priceNotationField, priceNotation)
		);
	}
}
