package drr.regulation.jfsa.rewrite.trade.reports;

import cdm.product.asset.CreditDefaultPayout;
import cdm.product.asset.CreditIndexReferenceInformation;
import cdm.product.asset.CreditSeniorityEnum;
import cdm.product.asset.GeneralTerms;
import cdm.product.template.ContractualProduct;
import cdm.product.template.EconomicTerms;
import cdm.product.template.Payout;
import cdm.product.template.Product;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.IsCreditDefaultSwap;
import drr.regulation.common.functions.IsCreditSwaption;
import drr.regulation.common.functions.ProductForEvent;
import drr.regulation.common.functions.UnderlierForProduct;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(SeniorityRule.SeniorityRuleDefault.class)
public abstract class SeniorityRule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected IsCreditDefaultSwap isCreditDefaultSwap;
	@Inject protected IsCreditSwaption isCreditSwaption;
	@Inject protected ProductForEvent productForEvent;
	@Inject protected UnderlierForProduct underlierForProduct;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class SeniorityRuleDefault extends SeniorityRule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			final MapperS<Product> thenResult0 = MapperS.of(input)
				.mapSingleToItem(item -> MapperS.of(productForEvent.evaluate(item.get())));
			final MapperS<CreditIndexReferenceInformation> thenResult1 = thenResult0
				.mapSingleToItem(item -> {
					final Boolean boolean0 = isCreditDefaultSwap.evaluate(item.get());
					if ((boolean0 == null ? false : boolean0)) {
						return item.<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<CreditDefaultPayout>map("getCreditDefaultPayout", payout -> payout.getCreditDefaultPayout()).<GeneralTerms>map("getGeneralTerms", creditDefaultPayout -> creditDefaultPayout.getGeneralTerms()).<CreditIndexReferenceInformation>map("getIndexReferenceInformation", generalTerms -> generalTerms.getIndexReferenceInformation());
					}
					final Boolean boolean1 = isCreditSwaption.evaluate(item.get());
					if ((boolean1 == null ? false : boolean1)) {
						return MapperS.of(underlierForProduct.evaluate(item.get())).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<CreditDefaultPayout>map("getCreditDefaultPayout", payout -> payout.getCreditDefaultPayout()).<GeneralTerms>map("getGeneralTerms", creditDefaultPayout -> creditDefaultPayout.getGeneralTerms()).<CreditIndexReferenceInformation>map("getIndexReferenceInformation", generalTerms -> generalTerms.getIndexReferenceInformation());
					}
					return MapperS.<CreditIndexReferenceInformation>ofNull();
				});
			output = thenResult1
				.mapSingleToItem(item -> {
					if (areEqual(item.<CreditSeniorityEnum>map("getSeniority", creditIndexReferenceInformation -> creditIndexReferenceInformation.getSeniority()), MapperS.of(CreditSeniorityEnum.SENIOR_UN_SEC), CardinalityOperator.All).or(areEqual(item.<CreditSeniorityEnum>map("getSeniority", creditIndexReferenceInformation -> creditIndexReferenceInformation.getSeniority()), MapperS.of(CreditSeniorityEnum.SENIOR_SEC), CardinalityOperator.All)).or(areEqual(item.<CreditSeniorityEnum>map("getSeniority", creditIndexReferenceInformation -> creditIndexReferenceInformation.getSeniority()), MapperS.of(CreditSeniorityEnum.SENIOR_LOSS_ABSORBING_CAPACITY), CardinalityOperator.All)).getOrDefault(false)) {
						return MapperS.of("SNDB");
					}
					if (areEqual(item.<CreditSeniorityEnum>map("getSeniority", creditIndexReferenceInformation -> creditIndexReferenceInformation.getSeniority()), MapperS.of(CreditSeniorityEnum.SUB_TIER_3), CardinalityOperator.All).or(areEqual(item.<CreditSeniorityEnum>map("getSeniority", creditIndexReferenceInformation -> creditIndexReferenceInformation.getSeniority()), MapperS.of(CreditSeniorityEnum.SUB_UPPER_TIER_2), CardinalityOperator.All)).or(areEqual(item.<CreditSeniorityEnum>map("getSeniority", creditIndexReferenceInformation -> creditIndexReferenceInformation.getSeniority()), MapperS.of(CreditSeniorityEnum.SUB_LOWER_TIER_2), CardinalityOperator.All)).or(areEqual(item.<CreditSeniorityEnum>map("getSeniority", creditIndexReferenceInformation -> creditIndexReferenceInformation.getSeniority()), MapperS.of(CreditSeniorityEnum.SUB_TIER_1), CardinalityOperator.All)).getOrDefault(false)) {
						return MapperS.of("SBOD");
					}
					if (areEqual(item.<CreditSeniorityEnum>map("getSeniority", creditIndexReferenceInformation -> creditIndexReferenceInformation.getSeniority()), MapperS.of(CreditSeniorityEnum.OTHER), CardinalityOperator.All).getOrDefault(false)) {
						return MapperS.of("OTHR");
					}
					return MapperS.<String>ofNull();
				}).get();
			
			return output;
		}
	}
}
