package drr.regulation.jfsa.rewrite.trade.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.ConvertNonISOToISOCurrency;
import drr.standards.iosco.cde.reports.CDESpreadLeg2CurrencyRule;
import javax.inject.Inject;


@ImplementedBy(SpreadCurrencyOfLeg2Rule.SpreadCurrencyOfLeg2RuleDefault.class)
public abstract class SpreadCurrencyOfLeg2Rule implements ReportFunction<TransactionReportInstruction, ISOCurrencyCodeEnum> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDESpreadLeg2CurrencyRule cDESpreadLeg2Currency;
	@Inject protected ConvertNonISOToISOCurrency convertNonISOToISOCurrency;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public ISOCurrencyCodeEnum evaluate(TransactionReportInstruction input) {
		ISOCurrencyCodeEnum output = doEvaluate(input);
		
		return output;
	}

	protected abstract ISOCurrencyCodeEnum doEvaluate(TransactionReportInstruction input);

	public static class SpreadCurrencyOfLeg2RuleDefault extends SpreadCurrencyOfLeg2Rule {
		@Override
		protected ISOCurrencyCodeEnum doEvaluate(TransactionReportInstruction input) {
			ISOCurrencyCodeEnum output = null;
			return assignOutput(output, input);
		}
		
		protected ISOCurrencyCodeEnum assignOutput(ISOCurrencyCodeEnum output, TransactionReportInstruction input) {
			final MapperS<String> thenResult = MapperS.of(input)
				.mapSingleToItem(item -> MapperS.of(cDESpreadLeg2Currency.evaluate(item.get())));
			output = thenResult
				.mapSingleToItem(item -> MapperS.of(convertNonISOToISOCurrency.evaluate(item.get()))).get();
			
			return output;
		}
	}
}
