package drr.regulation.jfsa.rewrite.trade.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.jfsa.rewrite.trade.SpreadOfLeg1;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class SpreadOfLeg1TypeTabulator implements Tabulator<SpreadOfLeg1> {
	private final Field spreadOfLeg1MonetaryField;
	private final Field spreadOfLeg1PercentageField;
	private final Field spreadOfLeg1BasisField;
	private final Field spreadCurrencyOfLeg1Field;
	private final Field spreadOfLeg1NotationField;
	
	public SpreadOfLeg1TypeTabulator() {
		this.spreadOfLeg1MonetaryField = new FieldImpl(
			"spreadOfLeg1Monetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "SpreadOfLeg1Monetary")),
			Optional.of("72 Spread-Leg 1 (Monetary)"),
			Arrays.asList()
		);
		this.spreadOfLeg1PercentageField = new FieldImpl(
			"spreadOfLeg1Percentage",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "SpreadOfLeg1Percentage")),
			Optional.of("72 Spread-Leg 1 (Percentage)"),
			Arrays.asList()
		);
		this.spreadOfLeg1BasisField = new FieldImpl(
			"spreadOfLeg1Basis",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "SpreadOfLeg1Basis")),
			Optional.of("72 Spread-Leg 1 (Basis)"),
			Arrays.asList()
		);
		this.spreadCurrencyOfLeg1Field = new FieldImpl(
			"spreadCurrencyOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "SpreadCurrencyOfLeg1")),
			Optional.of("73 Spread currency-Leg 1"),
			Arrays.asList()
		);
		this.spreadOfLeg1NotationField = new FieldImpl(
			"spreadOfLeg1Notation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "GetSpreadLeg1NotationEnum")),
			Optional.empty(),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(spreadOfLeg1MonetaryField, spreadOfLeg1PercentageField, spreadOfLeg1BasisField, spreadCurrencyOfLeg1Field, spreadOfLeg1NotationField);
	}
	
	@Override
	public List<FieldValue> tabulate(SpreadOfLeg1 input) {
		Optional<BigDecimal> spreadOfLeg1Monetary = Optional.ofNullable(input.getSpreadOfLeg1Monetary());
		Optional<BigDecimal> spreadOfLeg1Percentage = Optional.ofNullable(input.getSpreadOfLeg1Percentage());
		Optional<BigDecimal> spreadOfLeg1Basis = Optional.ofNullable(input.getSpreadOfLeg1Basis());
		Optional<ISOCurrencyCodeEnum> spreadCurrencyOfLeg1 = Optional.ofNullable(input.getSpreadCurrencyOfLeg1());
		Optional<PriceNotationEnum> spreadOfLeg1Notation = Optional.ofNullable(input.getSpreadOfLeg1Notation());
		return Arrays.asList(
			new FieldValueImpl(spreadOfLeg1MonetaryField, spreadOfLeg1Monetary),
			new FieldValueImpl(spreadOfLeg1PercentageField, spreadOfLeg1Percentage),
			new FieldValueImpl(spreadOfLeg1BasisField, spreadOfLeg1Basis),
			new FieldValueImpl(spreadCurrencyOfLeg1Field, spreadCurrencyOfLeg1),
			new FieldValueImpl(spreadOfLeg1NotationField, spreadOfLeg1Notation)
		);
	}
}
