package drr.regulation.jfsa.rewrite.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.jfsa.rewrite.trade.StrikePriceAndNotation;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class StrikePriceAndNotationTypeTabulator implements Tabulator<StrikePriceAndNotation> {
	private final Field strikePriceMonetaryField;
	private final Field strikePriceDecimalField;
	private final Field strikePriceCurrencyField;
	private final Field strikePriceNotationField;
	
	public StrikePriceAndNotationTypeTabulator() {
		this.strikePriceMonetaryField = new FieldImpl(
			"strikePriceMonetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "StrikePriceMonetary")),
			Optional.of("75 Strike Price (Monetary)"),
			Arrays.asList()
		);
		this.strikePriceDecimalField = new FieldImpl(
			"strikePriceDecimal",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "StrikePriceDecimal")),
			Optional.of("75 Strike Price (Decimal)"),
			Arrays.asList()
		);
		this.strikePriceCurrencyField = new FieldImpl(
			"strikePriceCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "StrikePriceCurrencyCurrencyPair")),
			Optional.of("76 Strike price currency"),
			Arrays.asList()
		);
		this.strikePriceNotationField = new FieldImpl(
			"strikePriceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "GetStrikePriceNotationEnum")),
			Optional.empty(),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(strikePriceMonetaryField, strikePriceDecimalField, strikePriceCurrencyField, strikePriceNotationField);
	}
	
	@Override
	public List<FieldValue> tabulate(StrikePriceAndNotation input) {
		Optional<BigDecimal> strikePriceMonetary = Optional.ofNullable(input.getStrikePriceMonetary());
		Optional<BigDecimal> strikePriceDecimal = Optional.ofNullable(input.getStrikePriceDecimal());
		Optional<String> strikePriceCurrency = Optional.ofNullable(input.getStrikePriceCurrency());
		Optional<PriceNotationEnum> strikePriceNotation = Optional.ofNullable(input.getStrikePriceNotation());
		return Arrays.asList(
			new FieldValueImpl(strikePriceMonetaryField, strikePriceMonetary),
			new FieldValueImpl(strikePriceDecimalField, strikePriceDecimal),
			new FieldValueImpl(strikePriceCurrencyField, strikePriceCurrency),
			new FieldValueImpl(strikePriceNotationField, strikePriceNotation)
		);
	}
}
