package drr.regulation.jfsa.rewrite.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.jfsa.rewrite.trade.StrikePriceScheduleReport;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class StrikePriceScheduleReportTypeTabulator implements Tabulator<StrikePriceScheduleReport> {
	private final Field effectiveDateStrikePriceField;
	private final Field endDateStrikePriceField;
	private final Field strikePriceInEffectOnAssociatedEffectiveDateMonetaryField;
	private final Field strikePriceInEffectOnAssociatedEffectiveDateDecimalField;
	private final Field strikePriceNotationInEffectOnAssociatedEffectiveDateField;
	
	public StrikePriceScheduleReportTypeTabulator() {
		this.effectiveDateStrikePriceField = new FieldImpl(
			"effectiveDateStrikePrice",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "EffectiveDateStrikePrice")),
			Optional.of("78 Effective date of the strike price"),
			Arrays.asList()
		);
		this.endDateStrikePriceField = new FieldImpl(
			"endDateStrikePrice",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "EndDateStrikePrice")),
			Optional.of("79 End date of the strike price"),
			Arrays.asList()
		);
		this.strikePriceInEffectOnAssociatedEffectiveDateMonetaryField = new FieldImpl(
			"strikePriceInEffectOnAssociatedEffectiveDateMonetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "StrikePriceInEffectOnAssociatedEffectiveDateMonetary")),
			Optional.of("78/80-$ 80 Strike price in effect between the unadjusted effective and end date (Monetary)"),
			Arrays.asList()
		);
		this.strikePriceInEffectOnAssociatedEffectiveDateDecimalField = new FieldImpl(
			"strikePriceInEffectOnAssociatedEffectiveDateDecimal",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "StrikePriceInEffectOnAssociatedEffectiveDateDecimal")),
			Optional.of("78/80-$ 80 Strike price in effect between the unadjusted effective and end date (Decimal)"),
			Arrays.asList()
		);
		this.strikePriceNotationInEffectOnAssociatedEffectiveDateField = new FieldImpl(
			"strikePriceNotationInEffectOnAssociatedEffectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "jfsa", "rewrite", "trade"), "StrikePriceNotationInEffectOnAssociatedEffectiveDate")),
			Optional.of("78/80-$ 80 Strike price in effect between the unadjusted effective and end date (Non Reportable)"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(effectiveDateStrikePriceField, endDateStrikePriceField, strikePriceInEffectOnAssociatedEffectiveDateMonetaryField, strikePriceInEffectOnAssociatedEffectiveDateDecimalField, strikePriceNotationInEffectOnAssociatedEffectiveDateField);
	}
	
	@Override
	public List<FieldValue> tabulate(StrikePriceScheduleReport input) {
		Optional<Date> effectiveDateStrikePrice = Optional.ofNullable(input.getEffectiveDateStrikePrice());
		Optional<Date> endDateStrikePrice = Optional.ofNullable(input.getEndDateStrikePrice());
		Optional<BigDecimal> strikePriceInEffectOnAssociatedEffectiveDateMonetary = Optional.ofNullable(input.getStrikePriceInEffectOnAssociatedEffectiveDateMonetary());
		Optional<BigDecimal> strikePriceInEffectOnAssociatedEffectiveDateDecimal = Optional.ofNullable(input.getStrikePriceInEffectOnAssociatedEffectiveDateDecimal());
		Optional<PriceNotationEnum> strikePriceNotationInEffectOnAssociatedEffectiveDate = Optional.ofNullable(input.getStrikePriceNotationInEffectOnAssociatedEffectiveDate());
		return Arrays.asList(
			new FieldValueImpl(effectiveDateStrikePriceField, effectiveDateStrikePrice),
			new FieldValueImpl(endDateStrikePriceField, endDateStrikePrice),
			new FieldValueImpl(strikePriceInEffectOnAssociatedEffectiveDateMonetaryField, strikePriceInEffectOnAssociatedEffectiveDateMonetary),
			new FieldValueImpl(strikePriceInEffectOnAssociatedEffectiveDateDecimalField, strikePriceInEffectOnAssociatedEffectiveDateDecimal),
			new FieldValueImpl(strikePriceNotationInEffectOnAssociatedEffectiveDateField, strikePriceNotationInEffectOnAssociatedEffectiveDate)
		);
	}
}
