package drr.regulation.jfsa.rewrite.trade.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.util.functions.StringContains;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(UtiProprietaryRule.UtiProprietaryRuleDefault.class)
public abstract class UtiProprietaryRule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected JfsaUtiRule jfsaUti;
	@Inject protected StringContains stringContains;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class UtiProprietaryRuleDefault extends UtiProprietaryRule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			final MapperS<String> thenResult = MapperS.of(jfsaUti.evaluate(input));
			output = thenResult
				.filterSingleNullSafe(item -> areEqual(MapperS.of(stringContains.evaluate(item.get(), "^[A-Z0-9]{18}[0-9]{2}[A-Z0-9]{0,32}$")), MapperS.of(false), CardinalityOperator.All).get()).get();
			
			return output;
		}
	}
}
