package drr.regulation.jfsa.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;
import java.time.ZonedDateTime;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSATransactionReportClearingReceiptTimestamp")
@ImplementedBy(JFSATransactionReportClearingReceiptTimestamp.Default.class)
public interface JFSATransactionReportClearingReceiptTimestamp extends Validator<JFSATransactionReport> {
	
	String NAME = "JFSATransactionReportClearingReceiptTimestamp";
	String DEFINITION = "if cleared = \"Y\" or (cleared = \"I\" and actionType = \"TERM\") and eventType = \"CLRG\" then clearingReceiptTimestamp exists";
	
	ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport);
	
	class Default implements JFSATransactionReportClearingReceiptTimestamp {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			ComparisonResult result = executeDataRule(jFSATransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSATransactionReport jFSATransactionReport) {
			try {
				if (areEqual(MapperS.of(jFSATransactionReport).<String>map("getCleared", _jFSATransactionReport -> _jFSATransactionReport.getCleared()), MapperS.of("Y"), CardinalityOperator.All).or(areEqual(MapperS.of(jFSATransactionReport).<String>map("getCleared", _jFSATransactionReport -> _jFSATransactionReport.getCleared()), MapperS.of("I"), CardinalityOperator.All).and(areEqual(MapperS.of(jFSATransactionReport).<String>map("getActionType", _jFSATransactionReport -> _jFSATransactionReport.getActionType()), MapperS.of("TERM"), CardinalityOperator.All)).and(areEqual(MapperS.of(jFSATransactionReport).<String>map("getEventType", _jFSATransactionReport -> _jFSATransactionReport.getEventType()), MapperS.of("CLRG"), CardinalityOperator.All))).getOrDefault(false)) {
					return exists(MapperS.of(jFSATransactionReport).<ZonedDateTime>map("getClearingReceiptTimestamp", _jFSATransactionReport -> _jFSATransactionReport.getClearingReceiptTimestamp()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSATransactionReportClearingReceiptTimestamp {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
		}
	}
}
