package drr.regulation.jfsa.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSATransactionReportDTCC_JFSA_BR_0010_01")
@ImplementedBy(JFSATransactionReportDTCCJFSABR001001.Default.class)
public interface JFSATransactionReportDTCCJFSABR001001 extends Validator<JFSATransactionReport> {
	
	String NAME = "JFSATransactionReportDTCC_JFSA_BR_0010_01";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\", \"PRTO\", \"EROR\", \"TERM\"] any = actionType then (if direction2Leg1 is absent and direction2Leg2 is absent then direction1 exists else if direction2Leg1 exists or direction2Leg2 exists then direction1 is absent)";
	
	ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport);
	
	class Default implements JFSATransactionReportDTCCJFSABR001001 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			ComparisonResult result = executeDataRule(jFSATransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSATransactionReport jFSATransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI"), MapperS.of("PRTO"), MapperS.of("EROR"), MapperS.of("TERM")), MapperS.of(jFSATransactionReport).<String>map("getActionType", _jFSATransactionReport -> _jFSATransactionReport.getActionType()), CardinalityOperator.Any).getOrDefault(false)) {
					if (notExists(MapperS.of(jFSATransactionReport).<String>map("getDirection2Leg1", _jFSATransactionReport -> _jFSATransactionReport.getDirection2Leg1())).and(notExists(MapperS.of(jFSATransactionReport).<String>map("getDirection2Leg2", _jFSATransactionReport -> _jFSATransactionReport.getDirection2Leg2()))).getOrDefault(false)) {
						return exists(MapperS.of(jFSATransactionReport).<String>map("getDirection1", _jFSATransactionReport -> _jFSATransactionReport.getDirection1()));
					}
					if (exists(MapperS.of(jFSATransactionReport).<String>map("getDirection2Leg1", _jFSATransactionReport -> _jFSATransactionReport.getDirection2Leg1())).or(exists(MapperS.of(jFSATransactionReport).<String>map("getDirection2Leg2", _jFSATransactionReport -> _jFSATransactionReport.getDirection2Leg2()))).getOrDefault(false)) {
						return notExists(MapperS.of(jFSATransactionReport).<String>map("getDirection1", _jFSATransactionReport -> _jFSATransactionReport.getDirection1()));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSATransactionReportDTCCJFSABR001001 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
		}
	}
}
