package drr.regulation.jfsa.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSATransactionReportDTCC_JFSA_BR_0016_01")
@ImplementedBy(JFSATransactionReportDTCCJFSABR001601.Default.class)
public interface JFSATransactionReportDTCCJFSABR001601 extends Validator<JFSATransactionReport> {
	
	String NAME = "JFSATransactionReportDTCC_JFSA_BR_0016_01";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\", \"PRTO\", \"EROR\", \"TERM\"] any = actionType then (if cleared = \"Y\" then centralCounterparty exists)";
	
	ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport);
	
	class Default implements JFSATransactionReportDTCCJFSABR001601 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			ComparisonResult result = executeDataRule(jFSATransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSATransactionReport jFSATransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI"), MapperS.of("PRTO"), MapperS.of("EROR"), MapperS.of("TERM")), MapperS.of(jFSATransactionReport).<String>map("getActionType", _jFSATransactionReport -> _jFSATransactionReport.getActionType()), CardinalityOperator.Any).getOrDefault(false)) {
					if (areEqual(MapperS.of(jFSATransactionReport).<String>map("getCleared", _jFSATransactionReport -> _jFSATransactionReport.getCleared()), MapperS.of("Y"), CardinalityOperator.All).getOrDefault(false)) {
						return exists(MapperS.of(jFSATransactionReport).<String>map("getCentralCounterparty", _jFSATransactionReport -> _jFSATransactionReport.getCentralCounterparty()));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSATransactionReportDTCCJFSABR001601 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
		}
	}
}
