package drr.regulation.jfsa.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSATransactionReportDTCC_JFSA_BR_0029_02")
@ImplementedBy(JFSATransactionReportDTCCJFSABR002902.Default.class)
public interface JFSATransactionReportDTCCJFSABR002902 extends Validator<JFSATransactionReport> {
	
	String NAME = "JFSATransactionReportDTCC_JFSA_BR_0029_02";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType then (if [\"YEAR\", \"MNTH\", \"WEEK\", \"DAIL\", \"EXPI\"] any = floatingRatePaymentFrequencyPeriodLeg1 then floatingRatePaymentFrequencyPeriodMultiplierLeg1 exists else if floatingRatePaymentFrequencyPeriodLeg1 = \"EXPI\" and floatingRatePaymentFrequencyPeriodMultiplierLeg1 <> 1 then floatingRatePaymentFrequencyPeriodMultiplierLeg1 is absent)";
	
	ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport);
	
	class Default implements JFSATransactionReportDTCCJFSABR002902 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			ComparisonResult result = executeDataRule(jFSATransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSATransactionReport jFSATransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(jFSATransactionReport).<String>map("getActionType", _jFSATransactionReport -> _jFSATransactionReport.getActionType()), CardinalityOperator.Any).getOrDefault(false)) {
					if (areEqual(MapperC.<String>of(MapperS.of("YEAR"), MapperS.of("MNTH"), MapperS.of("WEEK"), MapperS.of("DAIL"), MapperS.of("EXPI")), MapperS.of(jFSATransactionReport).<String>map("getFloatingRatePaymentFrequencyPeriodLeg1", _jFSATransactionReport -> _jFSATransactionReport.getFloatingRatePaymentFrequencyPeriodLeg1()), CardinalityOperator.Any).getOrDefault(false)) {
						return exists(MapperS.of(jFSATransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierLeg1", _jFSATransactionReport -> _jFSATransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierLeg1()));
					}
					if (areEqual(MapperS.of(jFSATransactionReport).<String>map("getFloatingRatePaymentFrequencyPeriodLeg1", _jFSATransactionReport -> _jFSATransactionReport.getFloatingRatePaymentFrequencyPeriodLeg1()), MapperS.of("EXPI"), CardinalityOperator.All).and(notEqual(MapperS.of(jFSATransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierLeg1", _jFSATransactionReport -> _jFSATransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierLeg1()), MapperS.of(1), CardinalityOperator.Any)).getOrDefault(false)) {
						return notExists(MapperS.of(jFSATransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierLeg1", _jFSATransactionReport -> _jFSATransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierLeg1()));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSATransactionReportDTCCJFSABR002902 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
		}
	}
}
