package drr.regulation.jfsa.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;
import drr.regulation.jfsa.rewrite.trade.NotionalQuantityScheduleLeg2Report;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSATransactionReportDTCC_JFSA_BR_0100_02")
@ImplementedBy(JFSATransactionReportDTCCJFSABR010002.Default.class)
public interface JFSATransactionReportDTCCJFSABR010002 extends Validator<JFSATransactionReport> {
	
	String NAME = "JFSATransactionReportDTCC_JFSA_BR_0100_02";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType then (if notionalQuantityScheduleLeg2 -> effectiveDate exists then notionalQuantityScheduleLeg2 -> value exists else if notionalQuantityScheduleLeg2 -> value count <> notionalQuantityScheduleLeg2 -> effectiveDate count then False)";
	
	ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport);
	
	class Default implements JFSATransactionReportDTCCJFSABR010002 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			ComparisonResult result = executeDataRule(jFSATransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSATransactionReport jFSATransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(jFSATransactionReport).<String>map("getActionType", _jFSATransactionReport -> _jFSATransactionReport.getActionType()), CardinalityOperator.Any).getOrDefault(false)) {
					if (exists(MapperS.of(jFSATransactionReport).<NotionalQuantityScheduleLeg2Report>mapC("getNotionalQuantityScheduleLeg2", _jFSATransactionReport -> _jFSATransactionReport.getNotionalQuantityScheduleLeg2()).<Date>map("getEffectiveDate", notionalQuantityScheduleLeg2Report -> notionalQuantityScheduleLeg2Report.getEffectiveDate())).getOrDefault(false)) {
						return exists(MapperS.of(jFSATransactionReport).<NotionalQuantityScheduleLeg2Report>mapC("getNotionalQuantityScheduleLeg2", _jFSATransactionReport -> _jFSATransactionReport.getNotionalQuantityScheduleLeg2()).<BigDecimal>map("getValue", notionalQuantityScheduleLeg2Report -> notionalQuantityScheduleLeg2Report.getValue()));
					}
					if (notEqual(MapperS.of(MapperS.of(jFSATransactionReport).<NotionalQuantityScheduleLeg2Report>mapC("getNotionalQuantityScheduleLeg2", _jFSATransactionReport -> _jFSATransactionReport.getNotionalQuantityScheduleLeg2()).<BigDecimal>map("getValue", notionalQuantityScheduleLeg2Report -> notionalQuantityScheduleLeg2Report.getValue()).resultCount()), MapperS.of(MapperS.of(jFSATransactionReport).<NotionalQuantityScheduleLeg2Report>mapC("getNotionalQuantityScheduleLeg2", _jFSATransactionReport -> _jFSATransactionReport.getNotionalQuantityScheduleLeg2()).<Date>map("getEffectiveDate", notionalQuantityScheduleLeg2Report -> notionalQuantityScheduleLeg2Report.getEffectiveDate()).resultCount()), CardinalityOperator.Any).getOrDefault(false)) {
						return ComparisonResult.of(MapperS.of(false));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSATransactionReportDTCCJFSABR010002 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
		}
	}
}
