package drr.regulation.jfsa.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSATransactionReportDTCC_JFSA_BR_0102_01")
@ImplementedBy(JFSATransactionReportDTCCJFSABR010201.Default.class)
public interface JFSATransactionReportDTCCJFSABR010201 extends Validator<JFSATransactionReport> {
	
	String NAME = "JFSATransactionReportDTCC_JFSA_BR_0102_01";
	String DEFINITION = "if actionType = \"NEWT\" then [\"TRAD\", \"NOVA\", \"COMP\", \"CLRG\", \"EXER\", \"ALOC\", \"CLAL\", \"PTNG\", \"CORP\"] any = eventType else if actionType = \"MODI\" then [\"TRAD\", \"NOVA\", \"COMP\", \"ETRM\", \"EXER\", \"ALOC\", \"CREV\", \"CORP\", \"UPDT\"] any = eventType else if actionType = \"PRTO\" then \"PTNG\" = eventType else if actionType = \"TERM\" then [\"NOVA\", \"COMP\", \"ETRM\", \"CLRG\", \"EXER\", \"ALOC\", \"CLAL\", \"CREV\", \"CORP\"] any = eventType else if [\"CORR\", \"REVI\", \"EROR\", \"VALU\", \"MARU\"] any = actionType then True";
	
	ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport);
	
	class Default implements JFSATransactionReportDTCCJFSABR010201 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			ComparisonResult result = executeDataRule(jFSATransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSATransactionReport jFSATransactionReport) {
			try {
				if (areEqual(MapperS.of(jFSATransactionReport).<String>map("getActionType", _jFSATransactionReport -> _jFSATransactionReport.getActionType()), MapperS.of("NEWT"), CardinalityOperator.All).getOrDefault(false)) {
					return areEqual(MapperC.<String>of(MapperS.of("TRAD"), MapperS.of("NOVA"), MapperS.of("COMP"), MapperS.of("CLRG"), MapperS.of("EXER"), MapperS.of("ALOC"), MapperS.of("CLAL"), MapperS.of("PTNG"), MapperS.of("CORP")), MapperS.of(jFSATransactionReport).<String>map("getEventType", _jFSATransactionReport -> _jFSATransactionReport.getEventType()), CardinalityOperator.Any);
				}
				if (areEqual(MapperS.of(jFSATransactionReport).<String>map("getActionType", _jFSATransactionReport -> _jFSATransactionReport.getActionType()), MapperS.of("MODI"), CardinalityOperator.All).getOrDefault(false)) {
					return areEqual(MapperC.<String>of(MapperS.of("TRAD"), MapperS.of("NOVA"), MapperS.of("COMP"), MapperS.of("ETRM"), MapperS.of("EXER"), MapperS.of("ALOC"), MapperS.of("CREV"), MapperS.of("CORP"), MapperS.of("UPDT")), MapperS.of(jFSATransactionReport).<String>map("getEventType", _jFSATransactionReport -> _jFSATransactionReport.getEventType()), CardinalityOperator.Any);
				}
				if (areEqual(MapperS.of(jFSATransactionReport).<String>map("getActionType", _jFSATransactionReport -> _jFSATransactionReport.getActionType()), MapperS.of("PRTO"), CardinalityOperator.All).getOrDefault(false)) {
					return areEqual(MapperS.of("PTNG"), MapperS.of(jFSATransactionReport).<String>map("getEventType", _jFSATransactionReport -> _jFSATransactionReport.getEventType()), CardinalityOperator.All);
				}
				if (areEqual(MapperS.of(jFSATransactionReport).<String>map("getActionType", _jFSATransactionReport -> _jFSATransactionReport.getActionType()), MapperS.of("TERM"), CardinalityOperator.All).getOrDefault(false)) {
					return areEqual(MapperC.<String>of(MapperS.of("NOVA"), MapperS.of("COMP"), MapperS.of("ETRM"), MapperS.of("CLRG"), MapperS.of("EXER"), MapperS.of("ALOC"), MapperS.of("CLAL"), MapperS.of("CREV"), MapperS.of("CORP")), MapperS.of(jFSATransactionReport).<String>map("getEventType", _jFSATransactionReport -> _jFSATransactionReport.getEventType()), CardinalityOperator.Any);
				}
				if (areEqual(MapperC.<String>of(MapperS.of("CORR"), MapperS.of("REVI"), MapperS.of("EROR"), MapperS.of("VALU"), MapperS.of("MARU")), MapperS.of(jFSATransactionReport).<String>map("getActionType", _jFSATransactionReport -> _jFSATransactionReport.getActionType()), CardinalityOperator.Any).getOrDefault(false)) {
					return ComparisonResult.of(MapperS.of(true));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSATransactionReportDTCCJFSABR010201 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
		}
	}
}
