package drr.regulation.jfsa.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSATransactionReportFixedRatePaymentFrequencyPeriodLeg2")
@ImplementedBy(JFSATransactionReportFixedRatePaymentFrequencyPeriodLeg2.Default.class)
public interface JFSATransactionReportFixedRatePaymentFrequencyPeriodLeg2 extends Validator<JFSATransactionReport> {
	
	String NAME = "JFSATransactionReportFixedRatePaymentFrequencyPeriodLeg2";
	String DEFINITION = "if fixedRatePaymentFrequencyPeriodMultiplierLeg2 exists then fixedRatePaymentFrequencyPeriodLeg2 exists";
	
	ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport);
	
	class Default implements JFSATransactionReportFixedRatePaymentFrequencyPeriodLeg2 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			ComparisonResult result = executeDataRule(jFSATransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSATransactionReport jFSATransactionReport) {
			try {
				if (exists(MapperS.of(jFSATransactionReport).<Integer>map("getFixedRatePaymentFrequencyPeriodMultiplierLeg2", _jFSATransactionReport -> _jFSATransactionReport.getFixedRatePaymentFrequencyPeriodMultiplierLeg2())).getOrDefault(false)) {
					return exists(MapperS.of(jFSATransactionReport).<String>map("getFixedRatePaymentFrequencyPeriodLeg2", _jFSATransactionReport -> _jFSATransactionReport.getFixedRatePaymentFrequencyPeriodLeg2()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSATransactionReportFixedRatePaymentFrequencyPeriodLeg2 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
		}
	}
}
