package drr.regulation.jfsa.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;
import iso20022.auth030.esma.InterestComputationMethod4Code;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSATransactionReportFloatingRateDayCountConventionOfLeg1")
@ImplementedBy(JFSATransactionReportFloatingRateDayCountConventionOfLeg1.Default.class)
public interface JFSATransactionReportFloatingRateDayCountConventionOfLeg1 extends Validator<JFSATransactionReport> {
	
	String NAME = "JFSATransactionReportFloatingRateDayCountConventionOfLeg1";
	String DEFINITION = "if floatingRatePaymentFrequencyPeriodLeg1 exists then floatingRateDayCountConventionOfLeg1 exists";
	
	ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport);
	
	class Default implements JFSATransactionReportFloatingRateDayCountConventionOfLeg1 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			ComparisonResult result = executeDataRule(jFSATransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSATransactionReport jFSATransactionReport) {
			try {
				if (exists(MapperS.of(jFSATransactionReport).<String>map("getFloatingRatePaymentFrequencyPeriodLeg1", _jFSATransactionReport -> _jFSATransactionReport.getFloatingRatePaymentFrequencyPeriodLeg1())).getOrDefault(false)) {
					return exists(MapperS.of(jFSATransactionReport).<InterestComputationMethod4Code>map("getFloatingRateDayCountConventionOfLeg1", _jFSATransactionReport -> _jFSATransactionReport.getFloatingRateDayCountConventionOfLeg1()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSATransactionReportFloatingRateDayCountConventionOfLeg1 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
		}
	}
}
