package drr.regulation.jfsa.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSATransactionReportFloatingRateResetFrequencyPeriodMultiplierLeg1")
@ImplementedBy(JFSATransactionReportFloatingRateResetFrequencyPeriodMultiplierLeg1.Default.class)
public interface JFSATransactionReportFloatingRateResetFrequencyPeriodMultiplierLeg1 extends Validator<JFSATransactionReport> {
	
	String NAME = "JFSATransactionReportFloatingRateResetFrequencyPeriodMultiplierLeg1";
	String DEFINITION = "if floatingRateResetFrequencyPeriodLeg1 = \"EXPI\" and floatingRatePaymentFrequencyPeriodMultiplierLeg1 <> 1 then floatingRateResetFrequencyPeriodMultiplierLeg1 is absent";
	
	ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport);
	
	class Default implements JFSATransactionReportFloatingRateResetFrequencyPeriodMultiplierLeg1 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			ComparisonResult result = executeDataRule(jFSATransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSATransactionReport jFSATransactionReport) {
			try {
				if (areEqual(MapperS.of(jFSATransactionReport).<String>map("getFloatingRateResetFrequencyPeriodLeg1", _jFSATransactionReport -> _jFSATransactionReport.getFloatingRateResetFrequencyPeriodLeg1()), MapperS.of("EXPI"), CardinalityOperator.All).and(notEqual(MapperS.of(jFSATransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierLeg1", _jFSATransactionReport -> _jFSATransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierLeg1()), MapperS.of(1), CardinalityOperator.Any)).getOrDefault(false)) {
					return notExists(MapperS.of(jFSATransactionReport).<Integer>map("getFloatingRateResetFrequencyPeriodMultiplierLeg1", _jFSATransactionReport -> _jFSATransactionReport.getFloatingRateResetFrequencyPeriodMultiplierLeg1()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSATransactionReportFloatingRateResetFrequencyPeriodMultiplierLeg1 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
		}
	}
}
