package drr.regulation.jfsa.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSATransactionReportIndicatorOfTheFloatingRateOfLeg2")
@ImplementedBy(JFSATransactionReportIndicatorOfTheFloatingRateOfLeg2.Default.class)
public interface JFSATransactionReportIndicatorOfTheFloatingRateOfLeg2 extends Validator<JFSATransactionReport> {
	
	String NAME = "JFSATransactionReportIndicatorOfTheFloatingRateOfLeg2";
	String DEFINITION = "if assetClass = \"INTR\" and (fixedRateLeg2 is absent and nameOfTheFloatingRateOfLeg2 is absent) then indicatorOfTheFloatingRateOfLeg2 exists";
	
	ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport);
	
	class Default implements JFSATransactionReportIndicatorOfTheFloatingRateOfLeg2 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			ComparisonResult result = executeDataRule(jFSATransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSATransactionReport jFSATransactionReport) {
			try {
				if (areEqual(MapperS.of(jFSATransactionReport).<String>map("getAssetClass", _jFSATransactionReport -> _jFSATransactionReport.getAssetClass()), MapperS.of("INTR"), CardinalityOperator.All).and(notExists(MapperS.of(jFSATransactionReport).<BigDecimal>map("getFixedRateLeg2", _jFSATransactionReport -> _jFSATransactionReport.getFixedRateLeg2())).and(notExists(MapperS.of(jFSATransactionReport).<String>map("getNameOfTheFloatingRateOfLeg2", _jFSATransactionReport -> _jFSATransactionReport.getNameOfTheFloatingRateOfLeg2())))).getOrDefault(false)) {
					return exists(MapperS.of(jFSATransactionReport).<String>map("getIndicatorOfTheFloatingRateOfLeg2", _jFSATransactionReport -> _jFSATransactionReport.getIndicatorOfTheFloatingRateOfLeg2()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSATransactionReportIndicatorOfTheFloatingRateOfLeg2 {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
		}
	}
}
