package drr.regulation.jfsa.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.jfsa.rewrite.trade.JFSATransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("JFSATransactionReportUnderlyingIndexIndicator")
@ImplementedBy(JFSATransactionReportUnderlyingIndexIndicator.Default.class)
public interface JFSATransactionReportUnderlyingIndexIndicator extends Validator<JFSATransactionReport> {
	
	String NAME = "JFSATransactionReportUnderlyingIndexIndicator";
	String DEFINITION = "if assetClass = \"INTR\" and underlyingIdentificationType = \"X\" and underlyingIdentification is absent and nameOfTheUnderlyingIndex is absent then underlyingIndexIndicator exists";
	
	ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport);
	
	class Default implements JFSATransactionReportUnderlyingIndexIndicator {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			ComparisonResult result = executeDataRule(jFSATransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(JFSATransactionReport jFSATransactionReport) {
			try {
				if (areEqual(MapperS.of(jFSATransactionReport).<String>map("getAssetClass", _jFSATransactionReport -> _jFSATransactionReport.getAssetClass()), MapperS.of("INTR"), CardinalityOperator.All).and(areEqual(MapperS.of(jFSATransactionReport).<String>map("getUnderlyingIdentificationType", _jFSATransactionReport -> _jFSATransactionReport.getUnderlyingIdentificationType()), MapperS.of("X"), CardinalityOperator.All)).and(notExists(MapperS.of(jFSATransactionReport).<String>map("getUnderlyingIdentification", _jFSATransactionReport -> _jFSATransactionReport.getUnderlyingIdentification()))).and(notExists(MapperS.of(jFSATransactionReport).<String>map("getNameOfTheUnderlyingIndex", _jFSATransactionReport -> _jFSATransactionReport.getNameOfTheUnderlyingIndex()))).getOrDefault(false)) {
					return exists(MapperS.of(jFSATransactionReport).<String>map("getUnderlyingIndexIndicator", _jFSATransactionReport -> _jFSATransactionReport.getUnderlyingIndexIndicator()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements JFSATransactionReportUnderlyingIndexIndicator {
	
		@Override
		public ValidationResult<JFSATransactionReport> validate(RosettaPath path, JFSATransactionReport jFSATransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "JFSATransactionReport", path, DEFINITION);
		}
	}
}
