package drr.regulation.jfsa.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.jfsa.rewrite.trade.PackageTransactionPrice;
import java.util.Arrays;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("PackageTransactionPriceMonetaryOrPercentageOrBasis")
@ImplementedBy(PackageTransactionPriceMonetaryOrPercentageOrBasis.Default.class)
public interface PackageTransactionPriceMonetaryOrPercentageOrBasis extends Validator<PackageTransactionPrice> {
	
	String NAME = "PackageTransactionPriceMonetaryOrPercentageOrBasis";
	String DEFINITION = "optional choice packageTransactionPriceMonetary, packageTransactionPricePercentage, packageTransactionPriceBasis";
	
	ValidationResult<PackageTransactionPrice> validate(RosettaPath path, PackageTransactionPrice packageTransactionPrice);
	
	class Default implements PackageTransactionPriceMonetaryOrPercentageOrBasis {
	
		@Override
		public ValidationResult<PackageTransactionPrice> validate(RosettaPath path, PackageTransactionPrice packageTransactionPrice) {
			ComparisonResult result = executeDataRule(packageTransactionPrice);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PackageTransactionPrice", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "PackageTransactionPrice", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(PackageTransactionPrice packageTransactionPrice) {
			try {
				return choice(MapperS.of(packageTransactionPrice), Arrays.asList("packageTransactionPriceMonetary", "packageTransactionPricePercentage", "packageTransactionPriceBasis"), ChoiceRuleValidationMethod.OPTIONAL);
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements PackageTransactionPriceMonetaryOrPercentageOrBasis {
	
		@Override
		public ValidationResult<PackageTransactionPrice> validate(RosettaPath path, PackageTransactionPrice packageTransactionPrice) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "PackageTransactionPrice", path, DEFINITION);
		}
	}
}
