package drr.regulation.jfsa.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.jfsa.rewrite.trade.StrikePriceAndNotation;
import java.util.Arrays;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("StrikePriceAndNotationMonetaryOrDecimal")
@ImplementedBy(StrikePriceAndNotationMonetaryOrDecimal.Default.class)
public interface StrikePriceAndNotationMonetaryOrDecimal extends Validator<StrikePriceAndNotation> {
	
	String NAME = "StrikePriceAndNotationMonetaryOrDecimal";
	String DEFINITION = "optional choice strikePriceMonetary, strikePriceDecimal";
	
	ValidationResult<StrikePriceAndNotation> validate(RosettaPath path, StrikePriceAndNotation strikePriceAndNotation);
	
	class Default implements StrikePriceAndNotationMonetaryOrDecimal {
	
		@Override
		public ValidationResult<StrikePriceAndNotation> validate(RosettaPath path, StrikePriceAndNotation strikePriceAndNotation) {
			ComparisonResult result = executeDataRule(strikePriceAndNotation);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "StrikePriceAndNotation", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "StrikePriceAndNotation", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(StrikePriceAndNotation strikePriceAndNotation) {
			try {
				return choice(MapperS.of(strikePriceAndNotation), Arrays.asList("strikePriceMonetary", "strikePriceDecimal"), ChoiceRuleValidationMethod.OPTIONAL);
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements StrikePriceAndNotationMonetaryOrDecimal {
	
		@Override
		public ValidationResult<StrikePriceAndNotation> validate(RosettaPath path, StrikePriceAndNotation strikePriceAndNotation) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "StrikePriceAndNotation", path, DEFINITION);
		}
	}
}
