package drr.regulation.jfsa.rewrite.trade.validation.exists;

import com.google.common.collect.ImmutableMap;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import drr.regulation.jfsa.rewrite.trade.PackageTransactionSpread;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class PackageTransactionSpreadOnlyExistsValidator implements ValidatorWithArg<PackageTransactionSpread, Set<String>> {

	/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
	@Override
	public <T2 extends PackageTransactionSpread> ValidationResult<PackageTransactionSpread> validate(RosettaPath path, T2 o, Set<String> fields) {
		Map<String, Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("packageTransactionSpreadMonetary", ExistenceChecker.isSet((BigDecimal) o.getPackageTransactionSpreadMonetary()))
				.put("packageTransactionSpreadPercentage", ExistenceChecker.isSet((BigDecimal) o.getPackageTransactionSpreadPercentage()))
				.put("packageTransactionSpreadBasis", ExistenceChecker.isSet((BigDecimal) o.getPackageTransactionSpreadBasis()))
				.put("packageTransactionSpreadNotation", ExistenceChecker.isSet((PriceNotationEnum) o.getPackageTransactionSpreadNotation()))
				.build();
		
		// Find the fields that are set
		Set<String> setFields = fieldExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.map(Map.Entry::getKey)
				.collect(Collectors.toSet());
		
		if (setFields.equals(fields)) {
			return success("PackageTransactionSpread", ValidationType.ONLY_EXISTS, "PackageTransactionSpread", path, "");
		}
		return failure("PackageTransactionSpread", ValidationType.ONLY_EXISTS, "PackageTransactionSpread", path, "",
				String.format("[%s] should only be set.  Set fields: %s", fields, setFields));
	}
}
