package drr.regulation.mas.rewrite.margin.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaReport;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.CollateralReportInstruction;
import drr.regulation.mas.rewrite.margin.MASMarginReport;
import drr.regulation.mas.rewrite.margin.MASMarginReport.MASMarginReportBuilder;
import java.math.BigDecimal;
import java.util.Optional;
import javax.inject.Inject;


@RosettaReport(namespace="drr.regulation.mas.rewrite.margin", body="MAS", corpusList={"Margin"})
@ImplementedBy(MASMarginReportFunction.MASMarginReportFunctionDefault.class)
public abstract class MASMarginReportFunction implements ReportFunction<CollateralReportInstruction, MASMarginReport> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ActionTypeRule actionType;
	@Inject protected CollateralPortfolioIndicatorRule collateralPortfolioIndicator;
	@Inject protected CollateralisationCategoryRule collateralisationCategory;
	@Inject protected Counterparty1Rule counterparty1;
	@Inject protected Counterparty2Rule counterparty2;
	@Inject protected Counterparty2IdentifierTypeRule counterparty2IdentifierType;
	@Inject protected CurrencyOfExcessCollateralCollectedRule currencyOfExcessCollateralCollected;
	@Inject protected CurrencyOfExcessCollateralPostedRule currencyOfExcessCollateralPosted;
	@Inject protected DataSubmitterRule dataSubmitter;
	@Inject protected EventDateRule eventDate;
	@Inject protected ExcessCollateralCollectedByCounterparty1Rule excessCollateralCollectedByCounterparty1;
	@Inject protected ExcessCollateralPostedByCounterparty1Rule excessCollateralPostedByCounterparty1;
	@Inject protected InitialMarginCollateralPortfolioCodeDTCCRule initialMarginCollateralPortfolioCodeDTCC;
	@Inject protected InitialMarginCollectedByCounterparty1CurrencyRule initialMarginCollectedByCounterparty1Currency;
	@Inject protected InitialMarginCollectedByCounterparty1PostHaircutRule initialMarginCollectedByCounterparty1PostHaircut;
	@Inject protected InitialMarginCollectedByCounterparty1PreHaircutRule initialMarginCollectedByCounterparty1PreHaircut;
	@Inject protected InitialMarginPostedByCounterparty1CurrencyRule initialMarginPostedByCounterparty1Currency;
	@Inject protected InitialMarginPostedByCounterparty1PostHaircutRule initialMarginPostedByCounterparty1PostHaircut;
	@Inject protected InitialMarginPostedByCounterparty1PreHaircutRule initialMarginPostedByCounterparty1PreHaircut;
	@Inject protected ReportingSpecifiedPersonRule reportingSpecifiedPerson;
	@Inject protected ReportingTimestampRule reportingTimestamp;
	@Inject protected UniqueTransactionIdentifierRule uniqueTransactionIdentifier;
	@Inject protected VariationMarginCollateralPortfolioCodeDTCCRule variationMarginCollateralPortfolioCodeDTCC;
	@Inject protected VariationMarginCollectedByCounterparty1CurrencyRule variationMarginCollectedByCounterparty1Currency;
	@Inject protected VariationMarginCollectedByCounterparty1PostHaircutRule variationMarginCollectedByCounterparty1PostHaircut;
	@Inject protected VariationMarginCollectedByCounterparty1PreHaircutRule variationMarginCollectedByCounterparty1PreHaircut;
	@Inject protected VariationMarginPostedByCounterparty1CurrencyRule variationMarginPostedByCounterparty1Currency;
	@Inject protected VariationMarginPostedByCounterparty1PostHaircutRule variationMarginPostedByCounterparty1PostHaircut;
	@Inject protected VariationMarginPostedByCounterparty1PreHaircutRule variationMarginPostedByCounterparty1PreHaircut;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public MASMarginReport evaluate(CollateralReportInstruction input) {
		MASMarginReport.MASMarginReportBuilder outputBuilder = doEvaluate(input);
		
		final MASMarginReport output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.build();
			objectValidator.validate(MASMarginReport.class, output);
		}
		
		return output;
	}

	protected abstract MASMarginReport.MASMarginReportBuilder doEvaluate(CollateralReportInstruction input);

	public static class MASMarginReportFunctionDefault extends MASMarginReportFunction {
		@Override
		protected MASMarginReport.MASMarginReportBuilder doEvaluate(CollateralReportInstruction input) {
			MASMarginReport.MASMarginReportBuilder output = MASMarginReport.builder();
			return assignOutput(output, input);
		}
		
		protected MASMarginReport.MASMarginReportBuilder assignOutput(MASMarginReport.MASMarginReportBuilder output, CollateralReportInstruction input) {
			output
				.setCounterparty1(counterparty1.evaluate(input));
			
			output
				.setCounterparty2(counterparty2.evaluate(input));
			
			output
				.setCounterparty2IdentifierType(counterparty2IdentifierType.evaluate(input));
			
			output
				.setReportingSpecifiedPerson(reportingSpecifiedPerson.evaluate(input));
			
			output
				.setDataSubmitter(dataSubmitter.evaluate(input));
			
			output
				.setUniqueTransactionIdentifier(uniqueTransactionIdentifier.evaluate(input));
			
			output
				.setReportingTimestamp(reportingTimestamp.evaluate(input));
			
			output
				.setCollateralPortfolioIndicator(collateralPortfolioIndicator.evaluate(input));
			
			output
				.setInitialMarginCollateralPortfolioCode(initialMarginCollateralPortfolioCodeDTCC.evaluate(input));
			
			output
				.setVariationMarginCollateralPortfolioCode(variationMarginCollateralPortfolioCodeDTCC.evaluate(input));
			
			output
				.setInitialMarginPostedByCounterparty1PreHaircut(initialMarginPostedByCounterparty1PreHaircut.evaluate(input));
			
			output
				.setInitialMarginPostedByCounterparty1PostHaircut(initialMarginPostedByCounterparty1PostHaircut.evaluate(input));
			
			output
				.setInitialMarginPostedByCounterparty1Currency(initialMarginPostedByCounterparty1Currency.evaluate(input));
			
			output
				.setInitialMarginCollectedByCounterparty1PreHaircut(initialMarginCollectedByCounterparty1PreHaircut.evaluate(input));
			
			output
				.setInitialMarginCollectedByCounterparty1PostHaircut(initialMarginCollectedByCounterparty1PostHaircut.evaluate(input));
			
			output
				.setInitialMarginCollectedByCounterparty1Currency(initialMarginCollectedByCounterparty1Currency.evaluate(input));
			
			output
				.setVariationMarginPostedByCounterparty1PreHaircut(variationMarginPostedByCounterparty1PreHaircut.evaluate(input));
			
			output
				.setVariationMarginPostedByCounterparty1PostHaircut(variationMarginPostedByCounterparty1PostHaircut.evaluate(input));
			
			output
				.setVariationMarginPostedByCounterparty1Currency(variationMarginPostedByCounterparty1Currency.evaluate(input));
			
			output
				.setVariationMarginCollectedByCounterparty1PreHaircut(variationMarginCollectedByCounterparty1PreHaircut.evaluate(input));
			
			output
				.setVariationMarginCollectedByCounterparty1PostHaircut(variationMarginCollectedByCounterparty1PostHaircut.evaluate(input));
			
			output
				.setVariationMarginCollectedByCounterparty1Currency(variationMarginCollectedByCounterparty1Currency.evaluate(input));
			
			final Integer integer0 = excessCollateralPostedByCounterparty1.evaluate(input);
			output
				.setExcessCollateralPostedByCounterparty1((integer0 == null ? null : BigDecimal.valueOf(integer0)));
			
			output
				.setCurrencyOfExcessCollateralPosted(currencyOfExcessCollateralPosted.evaluate(input));
			
			final Integer integer1 = excessCollateralCollectedByCounterparty1.evaluate(input);
			output
				.setExcessCollateralCollectedByCounterparty1((integer1 == null ? null : BigDecimal.valueOf(integer1)));
			
			output
				.setCurrencyOfExcessCollateralCollected(currencyOfExcessCollateralCollected.evaluate(input));
			
			output
				.setCollateralisationCategory(collateralisationCategory.evaluate(input));
			
			output
				.setActionType(actionType.evaluate(input));
			
			output
				.setEventDate(eventDate.evaluate(input));
			
			return Optional.ofNullable(output)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
