package drr.regulation.mas.rewrite.margin.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.common.MarginActionEnum;
import drr.regulation.mas.rewrite.margin.MASMarginReport;
import iso20022.auth108.esma.CollateralisationType3Code__1;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class MASMarginReportTypeTabulator implements Tabulator<MASMarginReport> {
	private final Field counterparty1Field;
	private final Field counterparty2Field;
	private final Field counterparty2IdentifierTypeField;
	private final Field reportingSpecifiedPersonField;
	private final Field dataSubmitterField;
	private final Field uniqueTransactionIdentifierField;
	private final Field reportingTimestampField;
	private final Field collateralPortfolioIndicatorField;
	private final Field initialMarginCollateralPortfolioCodeField;
	private final Field variationMarginCollateralPortfolioCodeField;
	private final Field initialMarginPostedByCounterparty1PreHaircutField;
	private final Field initialMarginPostedByCounterparty1PostHaircutField;
	private final Field initialMarginPostedByCounterparty1CurrencyField;
	private final Field initialMarginCollectedByCounterparty1PreHaircutField;
	private final Field initialMarginCollectedByCounterparty1PostHaircutField;
	private final Field initialMarginCollectedByCounterparty1CurrencyField;
	private final Field variationMarginPostedByCounterparty1PreHaircutField;
	private final Field variationMarginPostedByCounterparty1PostHaircutField;
	private final Field variationMarginPostedByCounterparty1CurrencyField;
	private final Field variationMarginCollectedByCounterparty1PreHaircutField;
	private final Field variationMarginCollectedByCounterparty1PostHaircutField;
	private final Field variationMarginCollectedByCounterparty1CurrencyField;
	private final Field excessCollateralPostedByCounterparty1Field;
	private final Field currencyOfExcessCollateralPostedField;
	private final Field excessCollateralCollectedByCounterparty1Field;
	private final Field currencyOfExcessCollateralCollectedField;
	private final Field collateralisationCategoryField;
	private final Field actionTypeField;
	private final Field eventDateField;
	
	public MASMarginReportTypeTabulator() {
		this.counterparty1Field = new FieldImpl(
			"counterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "Counterparty1")),
			Optional.of("1 Counterparty 1"),
			Arrays.asList()
		);
		this.counterparty2Field = new FieldImpl(
			"counterparty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "Counterparty2")),
			Optional.of("2 Counterparty 2"),
			Arrays.asList()
		);
		this.counterparty2IdentifierTypeField = new FieldImpl(
			"counterparty2IdentifierType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "Counterparty2IdentifierType")),
			Optional.of("3 Counterparty 2 Identifier Type"),
			Arrays.asList()
		);
		this.reportingSpecifiedPersonField = new FieldImpl(
			"reportingSpecifiedPerson",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "ReportingSpecifiedPerson")),
			Optional.of("8 Reporting specified person"),
			Arrays.asList()
		);
		this.dataSubmitterField = new FieldImpl(
			"dataSubmitter",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "DataSubmitter")),
			Optional.of("9 Data submitter"),
			Arrays.asList()
		);
		this.uniqueTransactionIdentifierField = new FieldImpl(
			"uniqueTransactionIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "UniqueTransactionIdentifier")),
			Optional.of("10 Unique Transaction Identifier (UTI)"),
			Arrays.asList()
		);
		this.reportingTimestampField = new FieldImpl(
			"reportingTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "ReportingTimestamp")),
			Optional.of("50 Reporting Timestamp"),
			Arrays.asList()
		);
		this.collateralPortfolioIndicatorField = new FieldImpl(
			"collateralPortfolioIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "CollateralPortfolioIndicator")),
			Optional.of("57 Collateral Portfolio Indicator"),
			Arrays.asList()
		);
		this.initialMarginCollateralPortfolioCodeField = new FieldImpl(
			"initialMarginCollateralPortfolioCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "InitialMarginCollateralPortfolioCodeDTCC")),
			Optional.of("58.1 Initial margin collateral portfolio code (DTCC)"),
			Arrays.asList()
		);
		this.variationMarginCollateralPortfolioCodeField = new FieldImpl(
			"variationMarginCollateralPortfolioCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "VariationMarginCollateralPortfolioCodeDTCC")),
			Optional.of("58.2 Variation margin collateral portfolio code (DTCC)"),
			Arrays.asList()
		);
		this.initialMarginPostedByCounterparty1PreHaircutField = new FieldImpl(
			"initialMarginPostedByCounterparty1PreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "InitialMarginPostedByCounterparty1PreHaircut")),
			Optional.of("59 Initial margin posted by Counterparty 1 (pre-haircut)"),
			Arrays.asList()
		);
		this.initialMarginPostedByCounterparty1PostHaircutField = new FieldImpl(
			"initialMarginPostedByCounterparty1PostHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "InitialMarginPostedByCounterparty1PostHaircut")),
			Optional.of("60 Initial margin posted by Counterparty 1 (post-haircut)"),
			Arrays.asList()
		);
		this.initialMarginPostedByCounterparty1CurrencyField = new FieldImpl(
			"initialMarginPostedByCounterparty1Currency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "InitialMarginPostedByCounterparty1Currency")),
			Optional.of("61 Currency of initial margin posted"),
			Arrays.asList()
		);
		this.initialMarginCollectedByCounterparty1PreHaircutField = new FieldImpl(
			"initialMarginCollectedByCounterparty1PreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "InitialMarginCollectedByCounterparty1PreHaircut")),
			Optional.of("62 Initial margin collected by Counterparty 1 (pre-haircut)"),
			Arrays.asList()
		);
		this.initialMarginCollectedByCounterparty1PostHaircutField = new FieldImpl(
			"initialMarginCollectedByCounterparty1PostHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "InitialMarginCollectedByCounterparty1PostHaircut")),
			Optional.of("63 Initial margin collected by the counterparty 1 (post-haircut)"),
			Arrays.asList()
		);
		this.initialMarginCollectedByCounterparty1CurrencyField = new FieldImpl(
			"initialMarginCollectedByCounterparty1Currency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "InitialMarginCollectedByCounterparty1Currency")),
			Optional.of("64 Currency of initial margin collected"),
			Arrays.asList()
		);
		this.variationMarginPostedByCounterparty1PreHaircutField = new FieldImpl(
			"variationMarginPostedByCounterparty1PreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "VariationMarginPostedByCounterparty1PreHaircut")),
			Optional.of("65 Variation margin posted by Counterparty 1 (pre-haircut)"),
			Arrays.asList()
		);
		this.variationMarginPostedByCounterparty1PostHaircutField = new FieldImpl(
			"variationMarginPostedByCounterparty1PostHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "VariationMarginPostedByCounterparty1PostHaircut")),
			Optional.of("66 Variation margin posted by Counterparty 1 (post-haircut)"),
			Arrays.asList()
		);
		this.variationMarginPostedByCounterparty1CurrencyField = new FieldImpl(
			"variationMarginPostedByCounterparty1Currency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "VariationMarginPostedByCounterparty1Currency")),
			Optional.of("67 Currency of variation margin posted"),
			Arrays.asList()
		);
		this.variationMarginCollectedByCounterparty1PreHaircutField = new FieldImpl(
			"variationMarginCollectedByCounterparty1PreHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "VariationMarginCollectedByCounterparty1PreHaircut")),
			Optional.of("68 Variation margin collected by Counterparty 1 (pre-haircut)"),
			Arrays.asList()
		);
		this.variationMarginCollectedByCounterparty1PostHaircutField = new FieldImpl(
			"variationMarginCollectedByCounterparty1PostHaircut",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "VariationMarginCollectedByCounterparty1PostHaircut")),
			Optional.of("69 Variation margin collected by Counterparty 1 (post-haircut)"),
			Arrays.asList()
		);
		this.variationMarginCollectedByCounterparty1CurrencyField = new FieldImpl(
			"variationMarginCollectedByCounterparty1Currency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "VariationMarginCollectedByCounterparty1Currency")),
			Optional.of("70 Currency of variation margin collected"),
			Arrays.asList()
		);
		this.excessCollateralPostedByCounterparty1Field = new FieldImpl(
			"excessCollateralPostedByCounterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "ExcessCollateralPostedByCounterparty1")),
			Optional.of("71 Excess collateral posted by Counterparty 1"),
			Arrays.asList()
		);
		this.currencyOfExcessCollateralPostedField = new FieldImpl(
			"currencyOfExcessCollateralPosted",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "CurrencyOfExcessCollateralPosted")),
			Optional.of("72 Currency of excess collateral posted"),
			Arrays.asList()
		);
		this.excessCollateralCollectedByCounterparty1Field = new FieldImpl(
			"excessCollateralCollectedByCounterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "ExcessCollateralCollectedByCounterparty1")),
			Optional.of("73 Excess collateral collected by Counterparty 1"),
			Arrays.asList()
		);
		this.currencyOfExcessCollateralCollectedField = new FieldImpl(
			"currencyOfExcessCollateralCollected",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "CurrencyOfExcessCollateralCollected")),
			Optional.of("74 Currency of excess collateral collected"),
			Arrays.asList()
		);
		this.collateralisationCategoryField = new FieldImpl(
			"collateralisationCategory",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "CollateralisationCategory")),
			Optional.of("75 Collateralisation category"),
			Arrays.asList()
		);
		this.actionTypeField = new FieldImpl(
			"actionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "ActionType")),
			Optional.of("132 Action Type"),
			Arrays.asList()
		);
		this.eventDateField = new FieldImpl(
			"eventDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "margin"), "EventDate")),
			Optional.of("134 Event date"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(counterparty1Field, counterparty2Field, counterparty2IdentifierTypeField, reportingSpecifiedPersonField, dataSubmitterField, uniqueTransactionIdentifierField, reportingTimestampField, collateralPortfolioIndicatorField, initialMarginCollateralPortfolioCodeField, variationMarginCollateralPortfolioCodeField, initialMarginPostedByCounterparty1PreHaircutField, initialMarginPostedByCounterparty1PostHaircutField, initialMarginPostedByCounterparty1CurrencyField, initialMarginCollectedByCounterparty1PreHaircutField, initialMarginCollectedByCounterparty1PostHaircutField, initialMarginCollectedByCounterparty1CurrencyField, variationMarginPostedByCounterparty1PreHaircutField, variationMarginPostedByCounterparty1PostHaircutField, variationMarginPostedByCounterparty1CurrencyField, variationMarginCollectedByCounterparty1PreHaircutField, variationMarginCollectedByCounterparty1PostHaircutField, variationMarginCollectedByCounterparty1CurrencyField, excessCollateralPostedByCounterparty1Field, currencyOfExcessCollateralPostedField, excessCollateralCollectedByCounterparty1Field, currencyOfExcessCollateralCollectedField, collateralisationCategoryField, actionTypeField, eventDateField);
	}
	
	@Override
	public List<FieldValue> tabulate(MASMarginReport input) {
		Optional<String> counterparty1 = Optional.ofNullable(input.getCounterparty1());
		Optional<String> counterparty2 = Optional.ofNullable(input.getCounterparty2());
		Optional<Boolean> counterparty2IdentifierType = Optional.ofNullable(input.getCounterparty2IdentifierType());
		Optional<String> reportingSpecifiedPerson = Optional.ofNullable(input.getReportingSpecifiedPerson());
		Optional<String> dataSubmitter = Optional.ofNullable(input.getDataSubmitter());
		Optional<String> uniqueTransactionIdentifier = Optional.ofNullable(input.getUniqueTransactionIdentifier());
		Optional<ZonedDateTime> reportingTimestamp = Optional.ofNullable(input.getReportingTimestamp());
		Optional<Boolean> collateralPortfolioIndicator = Optional.ofNullable(input.getCollateralPortfolioIndicator());
		Optional<String> initialMarginCollateralPortfolioCode = Optional.ofNullable(input.getInitialMarginCollateralPortfolioCode());
		Optional<String> variationMarginCollateralPortfolioCode = Optional.ofNullable(input.getVariationMarginCollateralPortfolioCode());
		Optional<BigDecimal> initialMarginPostedByCounterparty1PreHaircut = Optional.ofNullable(input.getInitialMarginPostedByCounterparty1PreHaircut());
		Optional<BigDecimal> initialMarginPostedByCounterparty1PostHaircut = Optional.ofNullable(input.getInitialMarginPostedByCounterparty1PostHaircut());
		Optional<ISOCurrencyCodeEnum> initialMarginPostedByCounterparty1Currency = Optional.ofNullable(input.getInitialMarginPostedByCounterparty1Currency());
		Optional<BigDecimal> initialMarginCollectedByCounterparty1PreHaircut = Optional.ofNullable(input.getInitialMarginCollectedByCounterparty1PreHaircut());
		Optional<BigDecimal> initialMarginCollectedByCounterparty1PostHaircut = Optional.ofNullable(input.getInitialMarginCollectedByCounterparty1PostHaircut());
		Optional<ISOCurrencyCodeEnum> initialMarginCollectedByCounterparty1Currency = Optional.ofNullable(input.getInitialMarginCollectedByCounterparty1Currency());
		Optional<BigDecimal> variationMarginPostedByCounterparty1PreHaircut = Optional.ofNullable(input.getVariationMarginPostedByCounterparty1PreHaircut());
		Optional<BigDecimal> variationMarginPostedByCounterparty1PostHaircut = Optional.ofNullable(input.getVariationMarginPostedByCounterparty1PostHaircut());
		Optional<ISOCurrencyCodeEnum> variationMarginPostedByCounterparty1Currency = Optional.ofNullable(input.getVariationMarginPostedByCounterparty1Currency());
		Optional<BigDecimal> variationMarginCollectedByCounterparty1PreHaircut = Optional.ofNullable(input.getVariationMarginCollectedByCounterparty1PreHaircut());
		Optional<BigDecimal> variationMarginCollectedByCounterparty1PostHaircut = Optional.ofNullable(input.getVariationMarginCollectedByCounterparty1PostHaircut());
		Optional<ISOCurrencyCodeEnum> variationMarginCollectedByCounterparty1Currency = Optional.ofNullable(input.getVariationMarginCollectedByCounterparty1Currency());
		Optional<BigDecimal> excessCollateralPostedByCounterparty1 = Optional.ofNullable(input.getExcessCollateralPostedByCounterparty1());
		Optional<String> currencyOfExcessCollateralPosted = Optional.ofNullable(input.getCurrencyOfExcessCollateralPosted());
		Optional<BigDecimal> excessCollateralCollectedByCounterparty1 = Optional.ofNullable(input.getExcessCollateralCollectedByCounterparty1());
		Optional<String> currencyOfExcessCollateralCollected = Optional.ofNullable(input.getCurrencyOfExcessCollateralCollected());
		Optional<CollateralisationType3Code__1> collateralisationCategory = Optional.ofNullable(input.getCollateralisationCategory());
		Optional<MarginActionEnum> actionType = Optional.ofNullable(input.getActionType());
		Optional<Date> eventDate = Optional.ofNullable(input.getEventDate());
		return Arrays.asList(
			new FieldValueImpl(counterparty1Field, counterparty1),
			new FieldValueImpl(counterparty2Field, counterparty2),
			new FieldValueImpl(counterparty2IdentifierTypeField, counterparty2IdentifierType),
			new FieldValueImpl(reportingSpecifiedPersonField, reportingSpecifiedPerson),
			new FieldValueImpl(dataSubmitterField, dataSubmitter),
			new FieldValueImpl(uniqueTransactionIdentifierField, uniqueTransactionIdentifier),
			new FieldValueImpl(reportingTimestampField, reportingTimestamp),
			new FieldValueImpl(collateralPortfolioIndicatorField, collateralPortfolioIndicator),
			new FieldValueImpl(initialMarginCollateralPortfolioCodeField, initialMarginCollateralPortfolioCode),
			new FieldValueImpl(variationMarginCollateralPortfolioCodeField, variationMarginCollateralPortfolioCode),
			new FieldValueImpl(initialMarginPostedByCounterparty1PreHaircutField, initialMarginPostedByCounterparty1PreHaircut),
			new FieldValueImpl(initialMarginPostedByCounterparty1PostHaircutField, initialMarginPostedByCounterparty1PostHaircut),
			new FieldValueImpl(initialMarginPostedByCounterparty1CurrencyField, initialMarginPostedByCounterparty1Currency),
			new FieldValueImpl(initialMarginCollectedByCounterparty1PreHaircutField, initialMarginCollectedByCounterparty1PreHaircut),
			new FieldValueImpl(initialMarginCollectedByCounterparty1PostHaircutField, initialMarginCollectedByCounterparty1PostHaircut),
			new FieldValueImpl(initialMarginCollectedByCounterparty1CurrencyField, initialMarginCollectedByCounterparty1Currency),
			new FieldValueImpl(variationMarginPostedByCounterparty1PreHaircutField, variationMarginPostedByCounterparty1PreHaircut),
			new FieldValueImpl(variationMarginPostedByCounterparty1PostHaircutField, variationMarginPostedByCounterparty1PostHaircut),
			new FieldValueImpl(variationMarginPostedByCounterparty1CurrencyField, variationMarginPostedByCounterparty1Currency),
			new FieldValueImpl(variationMarginCollectedByCounterparty1PreHaircutField, variationMarginCollectedByCounterparty1PreHaircut),
			new FieldValueImpl(variationMarginCollectedByCounterparty1PostHaircutField, variationMarginCollectedByCounterparty1PostHaircut),
			new FieldValueImpl(variationMarginCollectedByCounterparty1CurrencyField, variationMarginCollectedByCounterparty1Currency),
			new FieldValueImpl(excessCollateralPostedByCounterparty1Field, excessCollateralPostedByCounterparty1),
			new FieldValueImpl(currencyOfExcessCollateralPostedField, currencyOfExcessCollateralPosted),
			new FieldValueImpl(excessCollateralCollectedByCounterparty1Field, excessCollateralCollectedByCounterparty1),
			new FieldValueImpl(currencyOfExcessCollateralCollectedField, currencyOfExcessCollateralCollected),
			new FieldValueImpl(collateralisationCategoryField, collateralisationCategory),
			new FieldValueImpl(actionTypeField, actionType),
			new FieldValueImpl(eventDateField, eventDate)
		);
	}
}
