package drr.regulation.mas.rewrite.margin.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.mas.rewrite.margin.MASMarginReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASMarginReportDTCC_MAS_BR_0001_01")
@ImplementedBy(MASMarginReportDTCCMASBR000101.Default.class)
public interface MASMarginReportDTCCMASBR000101 extends Validator<MASMarginReport> {
	
	String NAME = "MASMarginReportDTCC_MAS_BR_0001_01";
	String DEFINITION = "if counterparty2IdentifierType = True then counterparty1 <> counterparty2";
	
	ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport);
	
	class Default implements MASMarginReportDTCCMASBR000101 {
	
		@Override
		public ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport) {
			ComparisonResult result = executeDataRule(mASMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASMarginReport mASMarginReport) {
			try {
				if (areEqual(MapperS.of(mASMarginReport).<Boolean>map("getCounterparty2IdentifierType", _mASMarginReport -> _mASMarginReport.getCounterparty2IdentifierType()), MapperS.of(true), CardinalityOperator.All).getOrDefault(false)) {
					return notEqual(MapperS.of(mASMarginReport).<String>map("getCounterparty1", _mASMarginReport -> _mASMarginReport.getCounterparty1()), MapperS.of(mASMarginReport).<String>map("getCounterparty2", _mASMarginReport -> _mASMarginReport.getCounterparty2()), CardinalityOperator.Any);
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASMarginReportDTCCMASBR000101 {
	
		@Override
		public ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION);
		}
	}
}
