package drr.regulation.mas.rewrite.margin.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.MarginActionEnum;
import drr.regulation.mas.rewrite.margin.MASMarginReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASMarginReportDTCC_MAS_BR_0058_02")
@ImplementedBy(MASMarginReportDTCCMASBR005802.Default.class)
public interface MASMarginReportDTCCMASBR005802 extends Validator<MASMarginReport> {
	
	String NAME = "MASMarginReportDTCC_MAS_BR_0058_02";
	String DEFINITION = "if actionType = MarginActionEnum -> MARU then (if uniqueTransactionIdentifier exists and initialMarginCollateralPortfolioCode <> \"NOAP\" then initialMarginCollateralPortfolioCode is absent else if collateralPortfolioIndicator = False then initialMarginCollateralPortfolioCode is absent)";
	
	ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport);
	
	class Default implements MASMarginReportDTCCMASBR005802 {
	
		@Override
		public ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport) {
			ComparisonResult result = executeDataRule(mASMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASMarginReport mASMarginReport) {
			try {
				if (areEqual(MapperS.of(mASMarginReport).<MarginActionEnum>map("getActionType", _mASMarginReport -> _mASMarginReport.getActionType()), MapperS.of(MarginActionEnum.MARU), CardinalityOperator.All).getOrDefault(false)) {
					if (exists(MapperS.of(mASMarginReport).<String>map("getUniqueTransactionIdentifier", _mASMarginReport -> _mASMarginReport.getUniqueTransactionIdentifier())).and(notEqual(MapperS.of(mASMarginReport).<String>map("getInitialMarginCollateralPortfolioCode", _mASMarginReport -> _mASMarginReport.getInitialMarginCollateralPortfolioCode()), MapperS.of("NOAP"), CardinalityOperator.Any)).getOrDefault(false)) {
						return notExists(MapperS.of(mASMarginReport).<String>map("getInitialMarginCollateralPortfolioCode", _mASMarginReport -> _mASMarginReport.getInitialMarginCollateralPortfolioCode()));
					}
					if (areEqual(MapperS.of(mASMarginReport).<Boolean>map("getCollateralPortfolioIndicator", _mASMarginReport -> _mASMarginReport.getCollateralPortfolioIndicator()), MapperS.of(false), CardinalityOperator.All).getOrDefault(false)) {
						return notExists(MapperS.of(mASMarginReport).<String>map("getInitialMarginCollateralPortfolioCode", _mASMarginReport -> _mASMarginReport.getInitialMarginCollateralPortfolioCode()));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASMarginReportDTCCMASBR005802 {
	
		@Override
		public ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION);
		}
	}
}
