package drr.regulation.mas.rewrite.margin.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.MarginActionEnum;
import drr.regulation.mas.rewrite.margin.MASMarginReport;
import iso20022.auth108.esma.CollateralisationType3Code__1;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASMarginReportDTCC_MAS_BR_0069_01")
@ImplementedBy(MASMarginReportDTCCMASBR006901.Default.class)
public interface MASMarginReportDTCCMASBR006901 extends Validator<MASMarginReport> {
	
	String NAME = "MASMarginReportDTCC_MAS_BR_0069_01";
	String DEFINITION = "if actionType = MarginActionEnum -> MARU then if [CollateralisationType3Code__1 -> PRC2, CollateralisationType3Code__1 -> PRCL, CollateralisationType3Code__1 -> OWC2, CollateralisationType3Code__1 -> OWP1, CollateralisationType3Code__1 -> OWP2, CollateralisationType3Code__1 -> FLCL] any = collateralisationCategory then variationMarginCollectedByCounterparty1PostHaircut exists and variationMarginCollectedByCounterparty1PostHaircut >= 0 else if [CollateralisationType3Code__1 -> PRC2, CollateralisationType3Code__1 -> PRCL, CollateralisationType3Code__1 -> OWC2, CollateralisationType3Code__1 -> OWP1, CollateralisationType3Code__1 -> OWP2, CollateralisationType3Code__1 -> FLCL] all <> collateralisationCategory then variationMarginCollectedByCounterparty1PostHaircut is absent";
	
	ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport);
	
	class Default implements MASMarginReportDTCCMASBR006901 {
	
		@Override
		public ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport) {
			ComparisonResult result = executeDataRule(mASMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASMarginReport mASMarginReport) {
			try {
				if (areEqual(MapperS.of(mASMarginReport).<MarginActionEnum>map("getActionType", _mASMarginReport -> _mASMarginReport.getActionType()), MapperS.of(MarginActionEnum.MARU), CardinalityOperator.All).getOrDefault(false)) {
					if (areEqual(MapperC.<CollateralisationType3Code__1>of(MapperS.of(CollateralisationType3Code__1.PRC2), MapperS.of(CollateralisationType3Code__1.PRCL), MapperS.of(CollateralisationType3Code__1.OWC2), MapperS.of(CollateralisationType3Code__1.OWP1), MapperS.of(CollateralisationType3Code__1.OWP2), MapperS.of(CollateralisationType3Code__1.FLCL)), MapperS.of(mASMarginReport).<CollateralisationType3Code__1>map("getCollateralisationCategory", _mASMarginReport -> _mASMarginReport.getCollateralisationCategory()), CardinalityOperator.Any).getOrDefault(false)) {
						return exists(MapperS.of(mASMarginReport).<BigDecimal>map("getVariationMarginCollectedByCounterparty1PostHaircut", _mASMarginReport -> _mASMarginReport.getVariationMarginCollectedByCounterparty1PostHaircut())).and(greaterThanEquals(MapperS.of(mASMarginReport).<BigDecimal>map("getVariationMarginCollectedByCounterparty1PostHaircut", _mASMarginReport -> _mASMarginReport.getVariationMarginCollectedByCounterparty1PostHaircut()), MapperS.of(BigDecimal.valueOf(0)), CardinalityOperator.All));
					}
					if (notEqual(MapperC.<CollateralisationType3Code__1>of(MapperS.of(CollateralisationType3Code__1.PRC2), MapperS.of(CollateralisationType3Code__1.PRCL), MapperS.of(CollateralisationType3Code__1.OWC2), MapperS.of(CollateralisationType3Code__1.OWP1), MapperS.of(CollateralisationType3Code__1.OWP2), MapperS.of(CollateralisationType3Code__1.FLCL)), MapperS.of(mASMarginReport).<CollateralisationType3Code__1>map("getCollateralisationCategory", _mASMarginReport -> _mASMarginReport.getCollateralisationCategory()), CardinalityOperator.All).getOrDefault(false)) {
						return notExists(MapperS.of(mASMarginReport).<BigDecimal>map("getVariationMarginCollectedByCounterparty1PostHaircut", _mASMarginReport -> _mASMarginReport.getVariationMarginCollectedByCounterparty1PostHaircut()));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASMarginReportDTCCMASBR006901 {
	
		@Override
		public ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION);
		}
	}
}
