package drr.regulation.mas.rewrite.margin.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.MarginActionEnum;
import drr.regulation.mas.rewrite.margin.MASMarginReport;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASMarginReportDTCC_MAS_BR_0071_01")
@ImplementedBy(MASMarginReportDTCCMASBR007101.Default.class)
public interface MASMarginReportDTCCMASBR007101 extends Validator<MASMarginReport> {
	
	String NAME = "MASMarginReportDTCC_MAS_BR_0071_01";
	String DEFINITION = "if actionType = MarginActionEnum -> MARU then excessCollateralPostedByCounterparty1 exists and excessCollateralPostedByCounterparty1 >= 0";
	
	ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport);
	
	class Default implements MASMarginReportDTCCMASBR007101 {
	
		@Override
		public ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport) {
			ComparisonResult result = executeDataRule(mASMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASMarginReport mASMarginReport) {
			try {
				if (areEqual(MapperS.of(mASMarginReport).<MarginActionEnum>map("getActionType", _mASMarginReport -> _mASMarginReport.getActionType()), MapperS.of(MarginActionEnum.MARU), CardinalityOperator.All).getOrDefault(false)) {
					return exists(MapperS.of(mASMarginReport).<BigDecimal>map("getExcessCollateralPostedByCounterparty1", _mASMarginReport -> _mASMarginReport.getExcessCollateralPostedByCounterparty1())).and(greaterThanEquals(MapperS.of(mASMarginReport).<BigDecimal>map("getExcessCollateralPostedByCounterparty1", _mASMarginReport -> _mASMarginReport.getExcessCollateralPostedByCounterparty1()), MapperS.of(BigDecimal.valueOf(0)), CardinalityOperator.All));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASMarginReportDTCCMASBR007101 {
	
		@Override
		public ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION);
		}
	}
}
