package drr.regulation.mas.rewrite.margin.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.MarginActionEnum;
import drr.regulation.mas.rewrite.margin.MASMarginReport;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASMarginReportDTCC_MAS_BR_0072_01")
@ImplementedBy(MASMarginReportDTCCMASBR007201.Default.class)
public interface MASMarginReportDTCCMASBR007201 extends Validator<MASMarginReport> {
	
	String NAME = "MASMarginReportDTCC_MAS_BR_0072_01";
	String DEFINITION = "if actionType = MarginActionEnum -> MARU and excessCollateralPostedByCounterparty1 exists then currencyOfExcessCollateralPosted exists and [\"XAG\", \"XAU\", \"XBA\", \"XBB\", \"XBC\", \"XBD\", \"XDR\", \"XEU\", \"XFU\", \"XPD\", \"XPT\", \"XXX\"] all <> currencyOfExcessCollateralPosted";
	
	ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport);
	
	class Default implements MASMarginReportDTCCMASBR007201 {
	
		@Override
		public ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport) {
			ComparisonResult result = executeDataRule(mASMarginReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASMarginReport mASMarginReport) {
			try {
				if (areEqual(MapperS.of(mASMarginReport).<MarginActionEnum>map("getActionType", _mASMarginReport -> _mASMarginReport.getActionType()), MapperS.of(MarginActionEnum.MARU), CardinalityOperator.All).and(exists(MapperS.of(mASMarginReport).<BigDecimal>map("getExcessCollateralPostedByCounterparty1", _mASMarginReport -> _mASMarginReport.getExcessCollateralPostedByCounterparty1()))).getOrDefault(false)) {
					return exists(MapperS.of(mASMarginReport).<String>map("getCurrencyOfExcessCollateralPosted", _mASMarginReport -> _mASMarginReport.getCurrencyOfExcessCollateralPosted())).and(notEqual(MapperC.<String>of(MapperS.of("XAG"), MapperS.of("XAU"), MapperS.of("XBA"), MapperS.of("XBB"), MapperS.of("XBC"), MapperS.of("XBD"), MapperS.of("XDR"), MapperS.of("XEU"), MapperS.of("XFU"), MapperS.of("XPD"), MapperS.of("XPT"), MapperS.of("XXX")), MapperS.of(mASMarginReport).<String>map("getCurrencyOfExcessCollateralPosted", _mASMarginReport -> _mASMarginReport.getCurrencyOfExcessCollateralPosted()), CardinalityOperator.All));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASMarginReportDTCCMASBR007201 {
	
		@Override
		public ValidationResult<MASMarginReport> validate(RosettaPath path, MASMarginReport mASMarginReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASMarginReport", path, DEFINITION);
		}
	}
}
