package drr.regulation.mas.rewrite.trade;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import drr.regulation.mas.rewrite.trade.SpreadOfLeg1;
import drr.regulation.mas.rewrite.trade.SpreadOfLeg1.SpreadOfLeg1Builder;
import drr.regulation.mas.rewrite.trade.SpreadOfLeg1.SpreadOfLeg1BuilderImpl;
import drr.regulation.mas.rewrite.trade.SpreadOfLeg1.SpreadOfLeg1Impl;
import drr.regulation.mas.rewrite.trade.meta.SpreadOfLeg1Meta;
import java.math.BigDecimal;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="SpreadOfLeg1", builder=SpreadOfLeg1.SpreadOfLeg1BuilderImpl.class, version="4.2.1")
public interface SpreadOfLeg1 extends RosettaModelObject {

	SpreadOfLeg1Meta metaData = new SpreadOfLeg1Meta();

	/*********************** Getter Methods  ***********************/
	BigDecimal getSpreadOfLeg1Monetary();
	BigDecimal getSpreadOfLeg1Decimal();
	BigDecimal getSpreadOfLeg1Basis();
	String getSpreadNotationOfLeg1();
	String getSpreadCurrencyOfLeg1();

	/*********************** Build Methods  ***********************/
	SpreadOfLeg1 build();
	
	SpreadOfLeg1.SpreadOfLeg1Builder toBuilder();
	
	static SpreadOfLeg1.SpreadOfLeg1Builder builder() {
		return new SpreadOfLeg1.SpreadOfLeg1BuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends SpreadOfLeg1> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends SpreadOfLeg1> getType() {
		return SpreadOfLeg1.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("spreadOfLeg1Monetary"), BigDecimal.class, getSpreadOfLeg1Monetary(), this);
		processor.processBasic(path.newSubPath("spreadOfLeg1Decimal"), BigDecimal.class, getSpreadOfLeg1Decimal(), this);
		processor.processBasic(path.newSubPath("spreadOfLeg1Basis"), BigDecimal.class, getSpreadOfLeg1Basis(), this);
		processor.processBasic(path.newSubPath("spreadNotationOfLeg1"), String.class, getSpreadNotationOfLeg1(), this);
		processor.processBasic(path.newSubPath("spreadCurrencyOfLeg1"), String.class, getSpreadCurrencyOfLeg1(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface SpreadOfLeg1Builder extends SpreadOfLeg1, RosettaModelObjectBuilder {
		SpreadOfLeg1.SpreadOfLeg1Builder setSpreadOfLeg1Monetary(BigDecimal spreadOfLeg1Monetary);
		SpreadOfLeg1.SpreadOfLeg1Builder setSpreadOfLeg1Decimal(BigDecimal spreadOfLeg1Decimal);
		SpreadOfLeg1.SpreadOfLeg1Builder setSpreadOfLeg1Basis(BigDecimal spreadOfLeg1Basis);
		SpreadOfLeg1.SpreadOfLeg1Builder setSpreadNotationOfLeg1(String spreadNotationOfLeg1);
		SpreadOfLeg1.SpreadOfLeg1Builder setSpreadCurrencyOfLeg1(String spreadCurrencyOfLeg1);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("spreadOfLeg1Monetary"), BigDecimal.class, getSpreadOfLeg1Monetary(), this);
			processor.processBasic(path.newSubPath("spreadOfLeg1Decimal"), BigDecimal.class, getSpreadOfLeg1Decimal(), this);
			processor.processBasic(path.newSubPath("spreadOfLeg1Basis"), BigDecimal.class, getSpreadOfLeg1Basis(), this);
			processor.processBasic(path.newSubPath("spreadNotationOfLeg1"), String.class, getSpreadNotationOfLeg1(), this);
			processor.processBasic(path.newSubPath("spreadCurrencyOfLeg1"), String.class, getSpreadCurrencyOfLeg1(), this);
		}
		

		SpreadOfLeg1.SpreadOfLeg1Builder prune();
	}

	/*********************** Immutable Implementation of SpreadOfLeg1  ***********************/
	class SpreadOfLeg1Impl implements SpreadOfLeg1 {
		private final BigDecimal spreadOfLeg1Monetary;
		private final BigDecimal spreadOfLeg1Decimal;
		private final BigDecimal spreadOfLeg1Basis;
		private final String spreadNotationOfLeg1;
		private final String spreadCurrencyOfLeg1;
		
		protected SpreadOfLeg1Impl(SpreadOfLeg1.SpreadOfLeg1Builder builder) {
			this.spreadOfLeg1Monetary = builder.getSpreadOfLeg1Monetary();
			this.spreadOfLeg1Decimal = builder.getSpreadOfLeg1Decimal();
			this.spreadOfLeg1Basis = builder.getSpreadOfLeg1Basis();
			this.spreadNotationOfLeg1 = builder.getSpreadNotationOfLeg1();
			this.spreadCurrencyOfLeg1 = builder.getSpreadCurrencyOfLeg1();
		}
		
		@Override
		@RosettaAttribute("spreadOfLeg1Monetary")
		public BigDecimal getSpreadOfLeg1Monetary() {
			return spreadOfLeg1Monetary;
		}
		
		@Override
		@RosettaAttribute("spreadOfLeg1Decimal")
		public BigDecimal getSpreadOfLeg1Decimal() {
			return spreadOfLeg1Decimal;
		}
		
		@Override
		@RosettaAttribute("spreadOfLeg1Basis")
		public BigDecimal getSpreadOfLeg1Basis() {
			return spreadOfLeg1Basis;
		}
		
		@Override
		@RosettaAttribute("spreadNotationOfLeg1")
		public String getSpreadNotationOfLeg1() {
			return spreadNotationOfLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadCurrencyOfLeg1")
		public String getSpreadCurrencyOfLeg1() {
			return spreadCurrencyOfLeg1;
		}
		
		@Override
		public SpreadOfLeg1 build() {
			return this;
		}
		
		@Override
		public SpreadOfLeg1.SpreadOfLeg1Builder toBuilder() {
			SpreadOfLeg1.SpreadOfLeg1Builder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(SpreadOfLeg1.SpreadOfLeg1Builder builder) {
			ofNullable(getSpreadOfLeg1Monetary()).ifPresent(builder::setSpreadOfLeg1Monetary);
			ofNullable(getSpreadOfLeg1Decimal()).ifPresent(builder::setSpreadOfLeg1Decimal);
			ofNullable(getSpreadOfLeg1Basis()).ifPresent(builder::setSpreadOfLeg1Basis);
			ofNullable(getSpreadNotationOfLeg1()).ifPresent(builder::setSpreadNotationOfLeg1);
			ofNullable(getSpreadCurrencyOfLeg1()).ifPresent(builder::setSpreadCurrencyOfLeg1);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			SpreadOfLeg1 _that = getType().cast(o);
		
			if (!Objects.equals(spreadOfLeg1Monetary, _that.getSpreadOfLeg1Monetary())) return false;
			if (!Objects.equals(spreadOfLeg1Decimal, _that.getSpreadOfLeg1Decimal())) return false;
			if (!Objects.equals(spreadOfLeg1Basis, _that.getSpreadOfLeg1Basis())) return false;
			if (!Objects.equals(spreadNotationOfLeg1, _that.getSpreadNotationOfLeg1())) return false;
			if (!Objects.equals(spreadCurrencyOfLeg1, _that.getSpreadCurrencyOfLeg1())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (spreadOfLeg1Monetary != null ? spreadOfLeg1Monetary.hashCode() : 0);
			_result = 31 * _result + (spreadOfLeg1Decimal != null ? spreadOfLeg1Decimal.hashCode() : 0);
			_result = 31 * _result + (spreadOfLeg1Basis != null ? spreadOfLeg1Basis.hashCode() : 0);
			_result = 31 * _result + (spreadNotationOfLeg1 != null ? spreadNotationOfLeg1.hashCode() : 0);
			_result = 31 * _result + (spreadCurrencyOfLeg1 != null ? spreadCurrencyOfLeg1.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SpreadOfLeg1 {" +
				"spreadOfLeg1Monetary=" + this.spreadOfLeg1Monetary + ", " +
				"spreadOfLeg1Decimal=" + this.spreadOfLeg1Decimal + ", " +
				"spreadOfLeg1Basis=" + this.spreadOfLeg1Basis + ", " +
				"spreadNotationOfLeg1=" + this.spreadNotationOfLeg1 + ", " +
				"spreadCurrencyOfLeg1=" + this.spreadCurrencyOfLeg1 +
			'}';
		}
	}

	/*********************** Builder Implementation of SpreadOfLeg1  ***********************/
	class SpreadOfLeg1BuilderImpl implements SpreadOfLeg1.SpreadOfLeg1Builder {
	
		protected BigDecimal spreadOfLeg1Monetary;
		protected BigDecimal spreadOfLeg1Decimal;
		protected BigDecimal spreadOfLeg1Basis;
		protected String spreadNotationOfLeg1;
		protected String spreadCurrencyOfLeg1;
	
		public SpreadOfLeg1BuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("spreadOfLeg1Monetary")
		public BigDecimal getSpreadOfLeg1Monetary() {
			return spreadOfLeg1Monetary;
		}
		
		@Override
		@RosettaAttribute("spreadOfLeg1Decimal")
		public BigDecimal getSpreadOfLeg1Decimal() {
			return spreadOfLeg1Decimal;
		}
		
		@Override
		@RosettaAttribute("spreadOfLeg1Basis")
		public BigDecimal getSpreadOfLeg1Basis() {
			return spreadOfLeg1Basis;
		}
		
		@Override
		@RosettaAttribute("spreadNotationOfLeg1")
		public String getSpreadNotationOfLeg1() {
			return spreadNotationOfLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadCurrencyOfLeg1")
		public String getSpreadCurrencyOfLeg1() {
			return spreadCurrencyOfLeg1;
		}
		
	
		@Override
		@RosettaAttribute("spreadOfLeg1Monetary")
		public SpreadOfLeg1.SpreadOfLeg1Builder setSpreadOfLeg1Monetary(BigDecimal spreadOfLeg1Monetary) {
			this.spreadOfLeg1Monetary = spreadOfLeg1Monetary==null?null:spreadOfLeg1Monetary;
			return this;
		}
		@Override
		@RosettaAttribute("spreadOfLeg1Decimal")
		public SpreadOfLeg1.SpreadOfLeg1Builder setSpreadOfLeg1Decimal(BigDecimal spreadOfLeg1Decimal) {
			this.spreadOfLeg1Decimal = spreadOfLeg1Decimal==null?null:spreadOfLeg1Decimal;
			return this;
		}
		@Override
		@RosettaAttribute("spreadOfLeg1Basis")
		public SpreadOfLeg1.SpreadOfLeg1Builder setSpreadOfLeg1Basis(BigDecimal spreadOfLeg1Basis) {
			this.spreadOfLeg1Basis = spreadOfLeg1Basis==null?null:spreadOfLeg1Basis;
			return this;
		}
		@Override
		@RosettaAttribute("spreadNotationOfLeg1")
		public SpreadOfLeg1.SpreadOfLeg1Builder setSpreadNotationOfLeg1(String spreadNotationOfLeg1) {
			this.spreadNotationOfLeg1 = spreadNotationOfLeg1==null?null:spreadNotationOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("spreadCurrencyOfLeg1")
		public SpreadOfLeg1.SpreadOfLeg1Builder setSpreadCurrencyOfLeg1(String spreadCurrencyOfLeg1) {
			this.spreadCurrencyOfLeg1 = spreadCurrencyOfLeg1==null?null:spreadCurrencyOfLeg1;
			return this;
		}
		
		@Override
		public SpreadOfLeg1 build() {
			return new SpreadOfLeg1.SpreadOfLeg1Impl(this);
		}
		
		@Override
		public SpreadOfLeg1.SpreadOfLeg1Builder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public SpreadOfLeg1.SpreadOfLeg1Builder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getSpreadOfLeg1Monetary()!=null) return true;
			if (getSpreadOfLeg1Decimal()!=null) return true;
			if (getSpreadOfLeg1Basis()!=null) return true;
			if (getSpreadNotationOfLeg1()!=null) return true;
			if (getSpreadCurrencyOfLeg1()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public SpreadOfLeg1.SpreadOfLeg1Builder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			SpreadOfLeg1.SpreadOfLeg1Builder o = (SpreadOfLeg1.SpreadOfLeg1Builder) other;
			
			
			merger.mergeBasic(getSpreadOfLeg1Monetary(), o.getSpreadOfLeg1Monetary(), this::setSpreadOfLeg1Monetary);
			merger.mergeBasic(getSpreadOfLeg1Decimal(), o.getSpreadOfLeg1Decimal(), this::setSpreadOfLeg1Decimal);
			merger.mergeBasic(getSpreadOfLeg1Basis(), o.getSpreadOfLeg1Basis(), this::setSpreadOfLeg1Basis);
			merger.mergeBasic(getSpreadNotationOfLeg1(), o.getSpreadNotationOfLeg1(), this::setSpreadNotationOfLeg1);
			merger.mergeBasic(getSpreadCurrencyOfLeg1(), o.getSpreadCurrencyOfLeg1(), this::setSpreadCurrencyOfLeg1);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			SpreadOfLeg1 _that = getType().cast(o);
		
			if (!Objects.equals(spreadOfLeg1Monetary, _that.getSpreadOfLeg1Monetary())) return false;
			if (!Objects.equals(spreadOfLeg1Decimal, _that.getSpreadOfLeg1Decimal())) return false;
			if (!Objects.equals(spreadOfLeg1Basis, _that.getSpreadOfLeg1Basis())) return false;
			if (!Objects.equals(spreadNotationOfLeg1, _that.getSpreadNotationOfLeg1())) return false;
			if (!Objects.equals(spreadCurrencyOfLeg1, _that.getSpreadCurrencyOfLeg1())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (spreadOfLeg1Monetary != null ? spreadOfLeg1Monetary.hashCode() : 0);
			_result = 31 * _result + (spreadOfLeg1Decimal != null ? spreadOfLeg1Decimal.hashCode() : 0);
			_result = 31 * _result + (spreadOfLeg1Basis != null ? spreadOfLeg1Basis.hashCode() : 0);
			_result = 31 * _result + (spreadNotationOfLeg1 != null ? spreadNotationOfLeg1.hashCode() : 0);
			_result = 31 * _result + (spreadCurrencyOfLeg1 != null ? spreadCurrencyOfLeg1.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SpreadOfLeg1Builder {" +
				"spreadOfLeg1Monetary=" + this.spreadOfLeg1Monetary + ", " +
				"spreadOfLeg1Decimal=" + this.spreadOfLeg1Decimal + ", " +
				"spreadOfLeg1Basis=" + this.spreadOfLeg1Basis + ", " +
				"spreadNotationOfLeg1=" + this.spreadNotationOfLeg1 + ", " +
				"spreadCurrencyOfLeg1=" + this.spreadCurrencyOfLeg1 +
			'}';
		}
	}
}
