package drr.regulation.mas.rewrite.trade;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import com.rosetta.model.lib.records.Date;
import drr.regulation.mas.rewrite.trade.StrikePriceScheduleReport;
import drr.regulation.mas.rewrite.trade.StrikePriceScheduleReport.StrikePriceScheduleReportBuilder;
import drr.regulation.mas.rewrite.trade.StrikePriceScheduleReport.StrikePriceScheduleReportBuilderImpl;
import drr.regulation.mas.rewrite.trade.StrikePriceScheduleReport.StrikePriceScheduleReportImpl;
import drr.regulation.mas.rewrite.trade.meta.StrikePriceScheduleReportMeta;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="StrikePriceScheduleReport", builder=StrikePriceScheduleReport.StrikePriceScheduleReportBuilderImpl.class, version="4.2.1")
public interface StrikePriceScheduleReport extends RosettaModelObject {

	StrikePriceScheduleReportMeta metaData = new StrikePriceScheduleReportMeta();

	/*********************** Getter Methods  ***********************/
	Date getEffectiveDate();
	Date getEndDate();
	BigDecimal getPriceMonetary();
	BigDecimal getPriceDecimal();
	PriceNotationEnum getPriceNotation();

	/*********************** Build Methods  ***********************/
	StrikePriceScheduleReport build();
	
	StrikePriceScheduleReport.StrikePriceScheduleReportBuilder toBuilder();
	
	static StrikePriceScheduleReport.StrikePriceScheduleReportBuilder builder() {
		return new StrikePriceScheduleReport.StrikePriceScheduleReportBuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends StrikePriceScheduleReport> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends StrikePriceScheduleReport> getType() {
		return StrikePriceScheduleReport.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("effectiveDate"), Date.class, getEffectiveDate(), this);
		processor.processBasic(path.newSubPath("endDate"), Date.class, getEndDate(), this);
		processor.processBasic(path.newSubPath("priceMonetary"), BigDecimal.class, getPriceMonetary(), this);
		processor.processBasic(path.newSubPath("priceDecimal"), BigDecimal.class, getPriceDecimal(), this);
		processor.processBasic(path.newSubPath("priceNotation"), PriceNotationEnum.class, getPriceNotation(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface StrikePriceScheduleReportBuilder extends StrikePriceScheduleReport, RosettaModelObjectBuilder {
		StrikePriceScheduleReport.StrikePriceScheduleReportBuilder setEffectiveDate(Date effectiveDate);
		StrikePriceScheduleReport.StrikePriceScheduleReportBuilder setEndDate(Date endDate);
		StrikePriceScheduleReport.StrikePriceScheduleReportBuilder setPriceMonetary(BigDecimal priceMonetary);
		StrikePriceScheduleReport.StrikePriceScheduleReportBuilder setPriceDecimal(BigDecimal priceDecimal);
		StrikePriceScheduleReport.StrikePriceScheduleReportBuilder setPriceNotation(PriceNotationEnum priceNotation);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("effectiveDate"), Date.class, getEffectiveDate(), this);
			processor.processBasic(path.newSubPath("endDate"), Date.class, getEndDate(), this);
			processor.processBasic(path.newSubPath("priceMonetary"), BigDecimal.class, getPriceMonetary(), this);
			processor.processBasic(path.newSubPath("priceDecimal"), BigDecimal.class, getPriceDecimal(), this);
			processor.processBasic(path.newSubPath("priceNotation"), PriceNotationEnum.class, getPriceNotation(), this);
		}
		

		StrikePriceScheduleReport.StrikePriceScheduleReportBuilder prune();
	}

	/*********************** Immutable Implementation of StrikePriceScheduleReport  ***********************/
	class StrikePriceScheduleReportImpl implements StrikePriceScheduleReport {
		private final Date effectiveDate;
		private final Date endDate;
		private final BigDecimal priceMonetary;
		private final BigDecimal priceDecimal;
		private final PriceNotationEnum priceNotation;
		
		protected StrikePriceScheduleReportImpl(StrikePriceScheduleReport.StrikePriceScheduleReportBuilder builder) {
			this.effectiveDate = builder.getEffectiveDate();
			this.endDate = builder.getEndDate();
			this.priceMonetary = builder.getPriceMonetary();
			this.priceDecimal = builder.getPriceDecimal();
			this.priceNotation = builder.getPriceNotation();
		}
		
		@Override
		@RosettaAttribute("effectiveDate")
		public Date getEffectiveDate() {
			return effectiveDate;
		}
		
		@Override
		@RosettaAttribute("endDate")
		public Date getEndDate() {
			return endDate;
		}
		
		@Override
		@RosettaAttribute("priceMonetary")
		public BigDecimal getPriceMonetary() {
			return priceMonetary;
		}
		
		@Override
		@RosettaAttribute("priceDecimal")
		public BigDecimal getPriceDecimal() {
			return priceDecimal;
		}
		
		@Override
		@RosettaAttribute("priceNotation")
		public PriceNotationEnum getPriceNotation() {
			return priceNotation;
		}
		
		@Override
		public StrikePriceScheduleReport build() {
			return this;
		}
		
		@Override
		public StrikePriceScheduleReport.StrikePriceScheduleReportBuilder toBuilder() {
			StrikePriceScheduleReport.StrikePriceScheduleReportBuilder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(StrikePriceScheduleReport.StrikePriceScheduleReportBuilder builder) {
			ofNullable(getEffectiveDate()).ifPresent(builder::setEffectiveDate);
			ofNullable(getEndDate()).ifPresent(builder::setEndDate);
			ofNullable(getPriceMonetary()).ifPresent(builder::setPriceMonetary);
			ofNullable(getPriceDecimal()).ifPresent(builder::setPriceDecimal);
			ofNullable(getPriceNotation()).ifPresent(builder::setPriceNotation);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			StrikePriceScheduleReport _that = getType().cast(o);
		
			if (!Objects.equals(effectiveDate, _that.getEffectiveDate())) return false;
			if (!Objects.equals(endDate, _that.getEndDate())) return false;
			if (!Objects.equals(priceMonetary, _that.getPriceMonetary())) return false;
			if (!Objects.equals(priceDecimal, _that.getPriceDecimal())) return false;
			if (!Objects.equals(priceNotation, _that.getPriceNotation())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (endDate != null ? endDate.hashCode() : 0);
			_result = 31 * _result + (priceMonetary != null ? priceMonetary.hashCode() : 0);
			_result = 31 * _result + (priceDecimal != null ? priceDecimal.hashCode() : 0);
			_result = 31 * _result + (priceNotation != null ? priceNotation.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "StrikePriceScheduleReport {" +
				"effectiveDate=" + this.effectiveDate + ", " +
				"endDate=" + this.endDate + ", " +
				"priceMonetary=" + this.priceMonetary + ", " +
				"priceDecimal=" + this.priceDecimal + ", " +
				"priceNotation=" + this.priceNotation +
			'}';
		}
	}

	/*********************** Builder Implementation of StrikePriceScheduleReport  ***********************/
	class StrikePriceScheduleReportBuilderImpl implements StrikePriceScheduleReport.StrikePriceScheduleReportBuilder {
	
		protected Date effectiveDate;
		protected Date endDate;
		protected BigDecimal priceMonetary;
		protected BigDecimal priceDecimal;
		protected PriceNotationEnum priceNotation;
	
		public StrikePriceScheduleReportBuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("effectiveDate")
		public Date getEffectiveDate() {
			return effectiveDate;
		}
		
		@Override
		@RosettaAttribute("endDate")
		public Date getEndDate() {
			return endDate;
		}
		
		@Override
		@RosettaAttribute("priceMonetary")
		public BigDecimal getPriceMonetary() {
			return priceMonetary;
		}
		
		@Override
		@RosettaAttribute("priceDecimal")
		public BigDecimal getPriceDecimal() {
			return priceDecimal;
		}
		
		@Override
		@RosettaAttribute("priceNotation")
		public PriceNotationEnum getPriceNotation() {
			return priceNotation;
		}
		
	
		@Override
		@RosettaAttribute("effectiveDate")
		public StrikePriceScheduleReport.StrikePriceScheduleReportBuilder setEffectiveDate(Date effectiveDate) {
			this.effectiveDate = effectiveDate==null?null:effectiveDate;
			return this;
		}
		@Override
		@RosettaAttribute("endDate")
		public StrikePriceScheduleReport.StrikePriceScheduleReportBuilder setEndDate(Date endDate) {
			this.endDate = endDate==null?null:endDate;
			return this;
		}
		@Override
		@RosettaAttribute("priceMonetary")
		public StrikePriceScheduleReport.StrikePriceScheduleReportBuilder setPriceMonetary(BigDecimal priceMonetary) {
			this.priceMonetary = priceMonetary==null?null:priceMonetary;
			return this;
		}
		@Override
		@RosettaAttribute("priceDecimal")
		public StrikePriceScheduleReport.StrikePriceScheduleReportBuilder setPriceDecimal(BigDecimal priceDecimal) {
			this.priceDecimal = priceDecimal==null?null:priceDecimal;
			return this;
		}
		@Override
		@RosettaAttribute("priceNotation")
		public StrikePriceScheduleReport.StrikePriceScheduleReportBuilder setPriceNotation(PriceNotationEnum priceNotation) {
			this.priceNotation = priceNotation==null?null:priceNotation;
			return this;
		}
		
		@Override
		public StrikePriceScheduleReport build() {
			return new StrikePriceScheduleReport.StrikePriceScheduleReportImpl(this);
		}
		
		@Override
		public StrikePriceScheduleReport.StrikePriceScheduleReportBuilder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public StrikePriceScheduleReport.StrikePriceScheduleReportBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getEffectiveDate()!=null) return true;
			if (getEndDate()!=null) return true;
			if (getPriceMonetary()!=null) return true;
			if (getPriceDecimal()!=null) return true;
			if (getPriceNotation()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public StrikePriceScheduleReport.StrikePriceScheduleReportBuilder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			StrikePriceScheduleReport.StrikePriceScheduleReportBuilder o = (StrikePriceScheduleReport.StrikePriceScheduleReportBuilder) other;
			
			
			merger.mergeBasic(getEffectiveDate(), o.getEffectiveDate(), this::setEffectiveDate);
			merger.mergeBasic(getEndDate(), o.getEndDate(), this::setEndDate);
			merger.mergeBasic(getPriceMonetary(), o.getPriceMonetary(), this::setPriceMonetary);
			merger.mergeBasic(getPriceDecimal(), o.getPriceDecimal(), this::setPriceDecimal);
			merger.mergeBasic(getPriceNotation(), o.getPriceNotation(), this::setPriceNotation);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			StrikePriceScheduleReport _that = getType().cast(o);
		
			if (!Objects.equals(effectiveDate, _that.getEffectiveDate())) return false;
			if (!Objects.equals(endDate, _that.getEndDate())) return false;
			if (!Objects.equals(priceMonetary, _that.getPriceMonetary())) return false;
			if (!Objects.equals(priceDecimal, _that.getPriceDecimal())) return false;
			if (!Objects.equals(priceNotation, _that.getPriceNotation())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (endDate != null ? endDate.hashCode() : 0);
			_result = 31 * _result + (priceMonetary != null ? priceMonetary.hashCode() : 0);
			_result = 31 * _result + (priceDecimal != null ? priceDecimal.hashCode() : 0);
			_result = 31 * _result + (priceNotation != null ? priceNotation.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "StrikePriceScheduleReportBuilder {" +
				"effectiveDate=" + this.effectiveDate + ", " +
				"endDate=" + this.endDate + ", " +
				"priceMonetary=" + this.priceMonetary + ", " +
				"priceDecimal=" + this.priceDecimal + ", " +
				"priceNotation=" + this.priceNotation +
			'}';
		}
	}
}
