package drr.regulation.mas.rewrite.trade.meta;

import com.rosetta.model.lib.annotations.RosettaMeta;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.qualify.QualifyFunctionFactory;
import com.rosetta.model.lib.qualify.QualifyResult;
import com.rosetta.model.lib.validation.Validator;
import com.rosetta.model.lib.validation.ValidatorFactory;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;
import drr.regulation.mas.rewrite.trade.validation.MASTransactionReportTypeFormatValidator;
import drr.regulation.mas.rewrite.trade.validation.MASTransactionReportValidator;
import drr.regulation.mas.rewrite.trade.validation.exists.MASTransactionReportOnlyExistsValidator;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.function.Function;


/**
 * @version 4.2.1
 */
@RosettaMeta(model=MASTransactionReport.class)
public class MASTransactionReportMeta implements RosettaMetaData<MASTransactionReport> {

	@Override
	public List<Validator<? super MASTransactionReport>> dataRules(ValidatorFactory factory) {
		return Arrays.asList(
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR000101.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR000201.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR000501.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR000601.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR000701.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR001001.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR001101.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR001201.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR001501.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR001502.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR001503.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR001504.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR001601.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR001602.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR001901.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR001902.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR001903.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR002001.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR002101.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR002201.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR002501.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR002502.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR002601.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR002701.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR003001.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR003201.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR003601.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR003701.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR003801.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR004001.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR004101.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR004201.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR004801.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR004901.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR005001.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR005201.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR005401.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR005801.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR005803.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR007601.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR007701.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR007801.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR007901.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR008001.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR008101.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR008201.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR008401.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR008501.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR008601.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR008701.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR008801.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR008901.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR009001.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR009101.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR009201.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR009301.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR009401.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR009402.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR009403.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR009501.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR009601.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR009701.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR009801.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR009901.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR010001.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR010101.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR010201.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR010202.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR010203.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR010301.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR010401.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR010501.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR010601.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR010701.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR010801.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR010901.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR011001.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR011101.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR011201.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR011301.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR011401.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR011501.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR011601.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR011701.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR011801.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR011901.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR012001.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR012101.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR012201.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR012301.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR012401.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR012501.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR012502.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR012601.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR012701.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR012702.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR012703.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR012704.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR012801.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR012901.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR013001.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR013101.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR013201.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR013301.class),
			factory.create(drr.regulation.mas.rewrite.trade.validation.datarule.MASTransactionReportDTCCMASBR013401.class)
		);
	}
	
	@Override
	public List<Function<? super MASTransactionReport, QualifyResult>> getQualifyFunctions(QualifyFunctionFactory factory) {
		return Collections.emptyList();
	}

	@Override
	public Validator<? super MASTransactionReport> validator() {
		return new MASTransactionReportValidator();
	}

	@Override
	public Validator<? super MASTransactionReport> typeFormatValidator() {
		return new MASTransactionReportTypeFormatValidator();
	}
	
	@Override
	public ValidatorWithArg<? super MASTransactionReport, Set<String>> onlyExistsValidator() {
		return new MASTransactionReportOnlyExistsValidator();
	}
}
