package drr.regulation.mas.rewrite.trade.reports;

import cdm.base.staticdata.asset.common.ProductIdTypeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.mas.rewrite.trade.BasketConstituentsReport;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class BasketConstituentsReportTypeTabulator implements Tabulator<BasketConstituentsReport> {
	private final Field identifierField;
	private final Field sourceField;
	
	public BasketConstituentsReportTypeTabulator() {
		this.identifierField = new FieldImpl(
			"identifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "IdentifierOfBasketConstituents")),
			Optional.of("51/52-$ 51 Identifier of the basket's constituents"),
			Arrays.asList()
		);
		this.sourceField = new FieldImpl(
			"source",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "SourceOfTheIdentifierOfTheBasketConstituents")),
			Optional.of("51/52-$ 52 Source of the identifier of the basket constituents"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(identifierField, sourceField);
	}
	
	@Override
	public List<FieldValue> tabulate(BasketConstituentsReport input) {
		Optional<String> identifier = Optional.ofNullable(input.getIdentifier());
		Optional<ProductIdTypeEnum> source = Optional.ofNullable(input.getSource());
		return Arrays.asList(
			new FieldValueImpl(identifierField, identifier),
			new FieldValueImpl(sourceField, source)
		);
	}
}
