package drr.regulation.mas.rewrite.trade.reports;

import cdm.base.staticdata.party.PayerReceiver;
import cdm.product.qualification.functions.Qualify_AssetClass_Credit;
import cdm.product.qualification.functions.Qualify_InterestRate_CapFloor;
import cdm.product.qualification.functions.Qualify_InterestRate_Option_Swaption;
import cdm.product.template.ContractualProduct;
import cdm.product.template.EconomicTerms;
import cdm.product.template.Product;
import cdm.product.template.TradableProduct;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.CommodityLeg2;
import drr.regulation.common.functions.FXLeg2;
import drr.regulation.common.functions.InterestRateLeg2;
import drr.regulation.common.functions.IsAllowableAction;
import drr.regulation.common.functions.IsCommoditySwapProduct;
import drr.regulation.common.functions.IsFXForward;
import drr.regulation.common.functions.TradeForEvent;
import drr.standards.iosco.cde.functions.CDEDirection2;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(DirectionOfLeg2Rule.DirectionOfLeg2RuleDefault.class)
public abstract class DirectionOfLeg2Rule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDEDirection2 cDEDirection2;
	@Inject protected CommodityLeg2 commodityLeg2;
	@Inject protected FXLeg2 fXLeg2;
	@Inject protected InterestRateLeg2 interestRateLeg2;
	@Inject protected IsAllowableAction isAllowableAction;
	@Inject protected IsCommoditySwapProduct isCommoditySwapProduct;
	@Inject protected IsFXForward isFXForward;
	@Inject protected Qualify_AssetClass_Credit qualify_AssetClass_Credit;
	@Inject protected Qualify_InterestRate_CapFloor qualify_InterestRate_CapFloor;
	@Inject protected Qualify_InterestRate_Option_Swaption qualify_InterestRate_Option_Swaption;
	@Inject protected TradeForEvent tradeForEvent;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class DirectionOfLeg2RuleDefault extends DirectionOfLeg2Rule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult = MapperS.of(input)
				.filterSingleNullSafe(item -> isAllowableAction.evaluate(item.get()));
			output = thenResult
				.mapSingleToItem(item -> MapperS.of(cDEDirection2.evaluate(item.get(), MapperS.of(tradeForEvent.evaluate(item.get()))
					.mapSingleToItem(trade -> trade.<TradableProduct>map("getTradableProduct", _trade -> _trade.getTradableProduct()).<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct())
						.mapSingleToItem(_item -> {
							final Boolean boolean0 = isCommoditySwapProduct.evaluate(_item.get());
							if ((boolean0 == null ? false : boolean0)) {
								return MapperS.of(commodityLeg2.evaluate(_item.get())).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver());
							}
							final Boolean boolean1 = isFXForward.evaluate(_item.get());
							if ((boolean1 == null ? false : boolean1)) {
								return MapperS.of(fXLeg2.evaluate(trade.get())).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver());
							}
							if (areEqual(MapperS.of(qualify_InterestRate_Option_Swaption.evaluate(_item.<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get())), MapperS.of(false), CardinalityOperator.All).and(areEqual(MapperS.of(qualify_InterestRate_CapFloor.evaluate(_item.<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get())), MapperS.of(false), CardinalityOperator.All)).and(areEqual(MapperS.of(qualify_AssetClass_Credit.evaluate(_item.<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get())), MapperS.of(false), CardinalityOperator.All)).getOrDefault(false)) {
								return MapperS.of(interestRateLeg2.evaluate(_item.get())).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver());
							}
							return MapperS.<PayerReceiver>ofNull();
						})).get()))).get();
			
			return output;
		}
	}
}
