package drr.regulation.mas.rewrite.trade.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaReport;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;
import drr.regulation.mas.rewrite.trade.MASTransactionReport.MASTransactionReportBuilder;
import java.util.Optional;
import javax.inject.Inject;


@RosettaReport(namespace="drr.regulation.mas.rewrite.trade", body="MAS", corpusList={"Trade"})
@ImplementedBy(MASTradeReportFunction.MASTradeReportFunctionDefault.class)
public abstract class MASTradeReportFunction implements ReportFunction<TransactionReportInstruction, MASTransactionReport> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ActionTypeRule actionType;
	@Inject protected AssetClassRule assetClass;
	@Inject protected BasketConstituentsRule basketConstituents;
	@Inject protected BookingLocationRule bookingLocation;
	@Inject protected CallAmountRule callAmount;
	@Inject protected CallCurrencyRule callCurrency;
	@Inject protected CentralCounterpartyRule centralCounterparty;
	@Inject protected ClearedRule cleared;
	@Inject protected ClearingMemberRule clearingMember;
	@Inject protected ClearingTimestampRule clearingTimestamp;
	@Inject protected CollateralPortfolioIndicatorRule collateralPortfolioIndicator;
	@Inject protected ConfirmationTimestampRule confirmationTimestamp;
	@Inject protected ConfirmedRule confirmed;
	@Inject protected ContractPriceScheduleRule contractPriceSchedule;
	@Inject protected ContractTypeRule contractType;
	@Inject protected Counterparty1Rule counterparty1;
	@Inject protected Counterparty2Rule counterparty2;
	@Inject protected Counterparty2IdentifierTypeRule counterparty2IdentifierType;
	@Inject protected DataSubmitterRule dataSubmitter;
	@Inject protected DeliveryTypeRule deliveryType;
	@Inject protected DirectionRule direction;
	@Inject protected DirectionOfLeg1Rule directionOfLeg1;
	@Inject protected DirectionOfLeg2Rule directionOfLeg2;
	@Inject protected EffectiveDateRule effectiveDate;
	@Inject protected EventDateRule eventDate;
	@Inject protected EventTypeRule eventType;
	@Inject protected ExchangeRateRule exchangeRate;
	@Inject protected ExchangeRateBasisRule exchangeRateBasis;
	@Inject protected ExecutionTimestampRule executionTimestamp;
	@Inject protected ExpirationDateRule expirationDate;
	@Inject protected FinalContractualSettlementDateRule finalContractualSettlementDate;
	@Inject protected FirstExerciseDateRule firstExerciseDate;
	@Inject protected FixedRateDayCountConventionOfLeg1Rule fixedRateDayCountConventionOfLeg1;
	@Inject protected FixedRateDayCountConventionOfLeg2Rule fixedRateDayCountConventionOfLeg2;
	@Inject protected FixedRateOfLeg1Rule fixedRateOfLeg1;
	@Inject protected FixedRateOfLeg2Rule fixedRateOfLeg2;
	@Inject protected FixedRatePaymentFrequencyPeriodMultiplierOfLeg1Rule fixedRatePaymentFrequencyPeriodMultiplierOfLeg1;
	@Inject protected FixedRatePaymentFrequencyPeriodMultiplierOfLeg2Rule fixedRatePaymentFrequencyPeriodMultiplierOfLeg2;
	@Inject protected FixedRatePaymentFrequencyPeriodOfLeg1Rule fixedRatePaymentFrequencyPeriodOfLeg1;
	@Inject protected FixedRatePaymentFrequencyPeriodOfLeg2Rule fixedRatePaymentFrequencyPeriodOfLeg2;
	@Inject protected FloatingRateDayCountConventionOfLeg1Rule floatingRateDayCountConventionOfLeg1;
	@Inject protected FloatingRateDayCountConventionOfLeg2Rule floatingRateDayCountConventionOfLeg2;
	@Inject protected FloatingRatePaymentFrequencyPeriodMultiplierOfLeg1Rule floatingRatePaymentFrequencyPeriodMultiplierOfLeg1;
	@Inject protected FloatingRatePaymentFrequencyPeriodMultiplierOfLeg2Rule floatingRatePaymentFrequencyPeriodMultiplierOfLeg2;
	@Inject protected FloatingRatePaymentFrequencyPeriodOfLeg1Rule floatingRatePaymentFrequencyPeriodOfLeg1;
	@Inject protected FloatingRatePaymentFrequencyPeriodOfLeg2Rule floatingRatePaymentFrequencyPeriodOfLeg2;
	@Inject protected FloatingRateResetFrequencyPeriodMultiplierOfLeg1Rule floatingRateResetFrequencyPeriodMultiplierOfLeg1;
	@Inject protected FloatingRateResetFrequencyPeriodMultiplierOfLeg2Rule floatingRateResetFrequencyPeriodMultiplierOfLeg2;
	@Inject protected FloatingRateResetFrequencyPeriodOfLeg1Rule floatingRateResetFrequencyPeriodOfLeg1;
	@Inject protected FloatingRateResetFrequencyPeriodOfLeg2Rule floatingRateResetFrequencyPeriodOfLeg2;
	@Inject protected IdentifierOfFloatingRateOfLeg1Rule identifierOfFloatingRateOfLeg1;
	@Inject protected IdentifierOfFloatingRateOfLeg2Rule identifierOfFloatingRateOfLeg2;
	@Inject protected InitialMarginCollateralPortfolioCodeDTCCRule initialMarginCollateralPortfolioCodeDTCC;
	@Inject protected NameOfTheFloatingRateOfLeg1Rule nameOfTheFloatingRateOfLeg1;
	@Inject protected NameOfTheFloatingRateOfLeg2Rule nameOfTheFloatingRateOfLeg2;
	@Inject protected NameOfTheUnderlyingIndexDTCCRule nameOfTheUnderlyingIndexDTCC;
	@Inject protected NotionalAmountOfLeg2Rule notionalAmountOfLeg2;
	@Inject protected NotionalAmountOfTheContractOrLeg1Rule notionalAmountOfTheContractOrLeg1;
	@Inject protected NotionalCurrencyOfLeg2Rule notionalCurrencyOfLeg2;
	@Inject protected NotionalCurrencyOfTheContractOrLeg1Rule notionalCurrencyOfTheContractOrLeg1;
	@Inject protected NotionalQuantityOfLeg2Rule notionalQuantityOfLeg2;
	@Inject protected NotionalQuantityOfTheContractOrLeg1Rule notionalQuantityOfTheContractOrLeg1;
	@Inject protected OptionPremiumAmountRule optionPremiumAmount;
	@Inject protected OptionPremiumCurrencyRule optionPremiumCurrency;
	@Inject protected OptionStyleRule optionStyle;
	@Inject protected OptionTypeRule optionType;
	@Inject protected OtherPaymentRule otherPayment;
	@Inject protected PackageIdentifierRule packageIdentifier;
	@Inject protected PlatformIdentifierRule platformIdentifier;
	@Inject protected PriceRule price;
	@Inject protected PriceCurrencyRule priceCurrency;
	@Inject protected PriceNotationRule priceNotation;
	@Inject protected PriceUnitOfMeasureRule priceUnitOfMeasure;
	@Inject protected PriorUTIRule priorUTI;
	@Inject protected PutAmountRule putAmount;
	@Inject protected PutCurrencyRule putCurrency;
	@Inject protected QuantityFrequencyMultiplierOfLeg2Rule quantityFrequencyMultiplierOfLeg2;
	@Inject protected QuantityFrequencyMultiplierOfTheContractOrLeg1Rule quantityFrequencyMultiplierOfTheContractOrLeg1;
	@Inject protected QuantityFrequencyOfLeg2Rule quantityFrequencyOfLeg2;
	@Inject protected QuantityFrequencyOfTheContractOrLeg1Rule quantityFrequencyOfTheContractOrLeg1;
	@Inject protected QuantityUnitOfMeasureForTheContractOrLeg1Rule quantityUnitOfMeasureForTheContractOrLeg1;
	@Inject protected QuantityUnitOfMeasureLeg2Rule quantityUnitOfMeasureLeg2;
	@Inject protected ReportingObligationOfSpecifiedPersonRule reportingObligationOfSpecifiedPerson;
	@Inject protected ReportingSpecifiedPersonRule reportingSpecifiedPerson;
	@Inject protected ReportingTimestampRule reportingTimestamp;
	@Inject protected SettlementCurrency1Rule settlementCurrency1;
	@Inject protected SettlementCurrency2Rule settlementCurrency2;
	@Inject protected SpreadCurrencyOfLeg1Rule spreadCurrencyOfLeg1;
	@Inject protected SpreadCurrencyOfLeg2Rule spreadCurrencyOfLeg2;
	@Inject protected SpreadNotationOfLeg1Rule spreadNotationOfLeg1;
	@Inject protected SpreadNotationOfLeg2Rule spreadNotationOfLeg2;
	@Inject protected SpreadOfLeg1BasisRule spreadOfLeg1Basis;
	@Inject protected SpreadOfLeg1DecimalRule spreadOfLeg1Decimal;
	@Inject protected SpreadOfLeg1MonetaryRule spreadOfLeg1Monetary;
	@Inject protected SpreadOfLeg2BasisRule spreadOfLeg2Basis;
	@Inject protected SpreadOfLeg2DecimalRule spreadOfLeg2Decimal;
	@Inject protected SpreadOfLeg2MonetaryRule spreadOfLeg2Monetary;
	@Inject protected StrikePriceRule strikePrice;
	@Inject protected StrikePriceCurrencyRule strikePriceCurrency;
	@Inject protected StrikePriceNotationRule strikePriceNotation;
	@Inject protected StrikePriceScheduleRule strikePriceSchedule;
	@Inject protected SwapLinkIDRule swapLinkID;
	@Inject protected TotalNotionalQuantityOfLeg2Rule totalNotionalQuantityOfLeg2;
	@Inject protected TotalNotionalQuantityOfTheContractOrLeg1Rule totalNotionalQuantityOfTheContractOrLeg1;
	@Inject protected TraderLocationRule traderLocation;
	@Inject protected UnderlyingRule underlying;
	@Inject protected UnderlyingIdOtherDTCCRule underlyingIdOtherDTCC;
	@Inject protected UnderlyingIdOtherSourceDTCCRule underlyingIdOtherSourceDTCC;
	@Inject protected UnderlyingIdentificationTypeDTCCRule underlyingIdentificationTypeDTCC;
	@Inject protected UniqueProductIdentifierRule uniqueProductIdentifier;
	@Inject protected UniqueTransactionIdentifierRule uniqueTransactionIdentifier;
	@Inject protected ValuationAmountRule valuationAmount;
	@Inject protected ValuationCurrencyRule valuationCurrency;
	@Inject protected ValuationMethodRule valuationMethod;
	@Inject protected ValuationTimestampRule valuationTimestamp;
	@Inject protected VariationMarginCollateralPortfolioCodeDTCCRule variationMarginCollateralPortfolioCodeDTCC;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public MASTransactionReport evaluate(TransactionReportInstruction input) {
		MASTransactionReport.MASTransactionReportBuilder outputBuilder = doEvaluate(input);
		
		final MASTransactionReport output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.build();
			objectValidator.validate(MASTransactionReport.class, output);
		}
		
		return output;
	}

	protected abstract MASTransactionReport.MASTransactionReportBuilder doEvaluate(TransactionReportInstruction input);

	public static class MASTradeReportFunctionDefault extends MASTradeReportFunction {
		@Override
		protected MASTransactionReport.MASTransactionReportBuilder doEvaluate(TransactionReportInstruction input) {
			MASTransactionReport.MASTransactionReportBuilder output = MASTransactionReport.builder();
			return assignOutput(output, input);
		}
		
		protected MASTransactionReport.MASTransactionReportBuilder assignOutput(MASTransactionReport.MASTransactionReportBuilder output, TransactionReportInstruction input) {
			output
				.setCounterparty1(counterparty1.evaluate(input));
			
			output
				.setCounterparty2(counterparty2.evaluate(input));
			
			output
				.setCounterparty2IdentifierType(counterparty2IdentifierType.evaluate(input));
			
			output
				.setDirection(direction.evaluate(input));
			
			output
				.setDirectionOfLeg1(directionOfLeg1.evaluate(input));
			
			output
				.setDirectionOfLeg2(directionOfLeg2.evaluate(input));
			
			output
				.setReportingSpecifiedPerson(reportingSpecifiedPerson.evaluate(input));
			
			output
				.setDataSubmitter(dataSubmitter.evaluate(input));
			
			output
				.setUniqueTransactionIdentifier(uniqueTransactionIdentifier.evaluate(input));
			
			output
				.setUniqueProductIdentifier(uniqueProductIdentifier.evaluate(input));
			
			output
				.setPriorUTI(priorUTI.evaluate(input));
			
			output
				.setContractType(contractType.evaluate(input));
			
			output
				.setAssetClass(assetClass.evaluate(input));
			
			output
				.setUnderlying(underlying.evaluate(input));
			
			output
				.setUnderlyingIdentificationType(underlyingIdentificationTypeDTCC.evaluate(input));
			
			output
				.setUnderlyingIdOther(underlyingIdOtherDTCC.evaluate(input));
			
			output
				.setUnderlyingIdOtherSource(underlyingIdOtherSourceDTCC.evaluate(input));
			
			output
				.setNameOfTheUnderlyingIndex(nameOfTheUnderlyingIndexDTCC.evaluate(input));
			
			output
				.setFixedRateOfLeg1(fixedRateOfLeg1.evaluate(input));
			
			output
				.setIdentifierOfFloatingRateOfLeg1(identifierOfFloatingRateOfLeg1.evaluate(input));
			
			output
				.setNameOfTheFloatingRateOfLeg1(nameOfTheFloatingRateOfLeg1.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadOfLeg1Monetary(spreadOfLeg1Monetary.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadOfLeg1Decimal(spreadOfLeg1Decimal.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadOfLeg1Basis(spreadOfLeg1Basis.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadNotationOfLeg1(spreadNotationOfLeg1.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg1()
				.setSpreadCurrencyOfLeg1(spreadCurrencyOfLeg1.evaluate(input));
			
			output
				.setFixedRateOfLeg2(fixedRateOfLeg2.evaluate(input));
			
			output
				.setIdentifierOfFloatingRateOfLeg2(identifierOfFloatingRateOfLeg2.evaluate(input));
			
			output
				.setNameOfTheFloatingRateOfLeg2(nameOfTheFloatingRateOfLeg2.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg2()
				.setSpreadOfLeg2Monetary(spreadOfLeg2Monetary.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg2()
				.setSpreadOfLeg2Decimal(spreadOfLeg2Decimal.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg2()
				.setSpreadOfLeg2Basis(spreadOfLeg2Basis.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg2()
				.setSpreadNotationOfLeg2(spreadNotationOfLeg2.evaluate(input));
			
			output
				.getOrCreateSpreadOfLeg2()
				.setSpreadCurrencyOfLeg2(spreadCurrencyOfLeg2.evaluate(input));
			
			output
				.setOptionType(optionType.evaluate(input));
			
			output
				.setOptionStyle(optionStyle.evaluate(input));
			
			output
				.setFirstExerciseDate(firstExerciseDate.evaluate(input));
			
			output
				.setEffectiveDate(effectiveDate.evaluate(input));
			
			output
				.setExpirationDate(expirationDate.evaluate(input));
			
			output
				.setSwapLinkID(swapLinkID.evaluate(input));
			
			output
				.setPackageIdentifier(packageIdentifier.evaluate(input));
			
			output
				.setDeliveryType(deliveryType.evaluate(input));
			
			output
				.setSettlementCurrency1(settlementCurrency1.evaluate(input));
			
			output
				.setSettlementCurrency2(settlementCurrency2.evaluate(input));
			
			output
				.setFinalContractualSettlementDate(finalContractualSettlementDate.evaluate(input));
			
			output
				.setCleared(cleared.evaluate(input));
			
			output
				.setCentralCounterparty(centralCounterparty.evaluate(input));
			
			output
				.setClearingMember(clearingMember.evaluate(input));
			
			output
				.setClearingTimestamp(clearingTimestamp.evaluate(input));
			
			output
				.setPlatformIdentifier(platformIdentifier.evaluate(input));
			
			output
				.setReportingObligationOfSpecifiedPerson(reportingObligationOfSpecifiedPerson.evaluate(input));
			
			output
				.setBookingLocation(bookingLocation.evaluate(input));
			
			output
				.setTraderLocation(traderLocation.evaluate(input));
			
			output
				.setConfirmed(confirmed.evaluate(input));
			
			output
				.setConfirmationTimestamp(confirmationTimestamp.evaluate(input));
			
			output
				.setExecutionTimestamp(executionTimestamp.evaluate(input));
			
			output
				.setReportingTimestamp(reportingTimestamp.evaluate(input));
			
			output
				.setBasketConstituents(basketConstituents.evaluate(input));
			
			output
				.setValuationAmount(valuationAmount.evaluate(input));
			
			output
				.setValuationCurrency(valuationCurrency.evaluate(input));
			
			output
				.setValuationMethod(valuationMethod.evaluate(input));
			
			output
				.setValuationTimestamp(valuationTimestamp.evaluate(input));
			
			output
				.setCollateralPortfolioIndicator(collateralPortfolioIndicator.evaluate(input));
			
			output
				.setInitialMarginCollateralPortfolioCode(initialMarginCollateralPortfolioCodeDTCC.evaluate(input));
			
			output
				.setVariationMarginCollateralPortfolioCode(variationMarginCollateralPortfolioCodeDTCC.evaluate(input));
			
			output
				.setNotionalAmountOfTheContractOrLeg1(notionalAmountOfTheContractOrLeg1.evaluate(input));
			
			output
				.setNotionalCurrencyOfTheContractOrLeg1(notionalCurrencyOfTheContractOrLeg1.evaluate(input));
			
			output
				.setNotionalAmountOfLeg2(notionalAmountOfLeg2.evaluate(input));
			
			output
				.setNotionalCurrencyOfLeg2(notionalCurrencyOfLeg2.evaluate(input));
			
			output
				.setTotalNotionalQuantityOfTheContractOrLeg1(totalNotionalQuantityOfTheContractOrLeg1.evaluate(input));
			
			output
				.setQuantityUnitOfMeasureForTheContractOrLeg1(quantityUnitOfMeasureForTheContractOrLeg1.evaluate(input));
			
			output
				.setNotionalQuantityOfTheContractOrLeg1(notionalQuantityOfTheContractOrLeg1.evaluate(input));
			
			output
				.setQuantityFrequencyOfTheContractOrLeg1(quantityFrequencyOfTheContractOrLeg1.evaluate(input));
			
			output
				.setQuantityFrequencyMultiplierOfTheContractOrLeg1(quantityFrequencyMultiplierOfTheContractOrLeg1.evaluate(input));
			
			output
				.setTotalNotionalQuantityLeg2(totalNotionalQuantityOfLeg2.evaluate(input));
			
			output
				.setQuantityUnitOfMeasureLeg2(quantityUnitOfMeasureLeg2.evaluate(input));
			
			output
				.setNotionalQuantityOfLeg2(notionalQuantityOfLeg2.evaluate(input));
			
			output
				.setQuantityFrequencyOfLeg2(quantityFrequencyOfLeg2.evaluate(input));
			
			output
				.setQuantityFrequencyMultiplierOfLeg2(quantityFrequencyMultiplierOfLeg2.evaluate(input));
			
			output
				.setCallAmount(callAmount.evaluate(input));
			
			output
				.setCallCurrency(callCurrency.evaluate(input));
			
			output
				.setPutAmount(putAmount.evaluate(input));
			
			output
				.setPutCurrency(putCurrency.evaluate(input));
			
			output
				.setPrice(price.evaluate(input));
			
			output
				.setPriceCurrency(priceCurrency.evaluate(input));
			
			output
				.setPriceNotation(priceNotation.evaluate(input));
			
			output
				.setPriceUnitOfMeasure(priceUnitOfMeasure.evaluate(input));
			
			output
				.setPriceSchedule(contractPriceSchedule.evaluate(input));
			
			output
				.setStrikePrice(strikePrice.evaluate(input));
			
			output
				.setStrikePriceCurrency(strikePriceCurrency.evaluate(input));
			
			output
				.setStrikePriceNotation(strikePriceNotation.evaluate(input));
			
			output
				.setStrikePriceSchedule(strikePriceSchedule.evaluate(input));
			
			output
				.setOptionPremiumAmount(optionPremiumAmount.evaluate(input));
			
			output
				.setOptionPremiumCurrency(optionPremiumCurrency.evaluate(input));
			
			output
				.setExchangeRate(exchangeRate.evaluate(input));
			
			output
				.setExchangeRateBasis(exchangeRateBasis.evaluate(input));
			
			output
				.setFixedRateDayCountConventionOfLeg1(fixedRateDayCountConventionOfLeg1.evaluate(input));
			
			output
				.setFixedRateDayCountConventionOfLeg2(fixedRateDayCountConventionOfLeg2.evaluate(input));
			
			output
				.setFloatingRateDayCountConventionOfLeg1(floatingRateDayCountConventionOfLeg1.evaluate(input));
			
			output
				.setFloatingRateDayCountConventionOfLeg2(floatingRateDayCountConventionOfLeg2.evaluate(input));
			
			output
				.setFloatingRateResetFrequencyPeriodOfLeg1(floatingRateResetFrequencyPeriodOfLeg1.evaluate(input));
			
			output
				.setFloatingRateResetFrequencyPeriodMultiplierOfLeg1(floatingRateResetFrequencyPeriodMultiplierOfLeg1.evaluate(input));
			
			output
				.setFloatingRateResetFrequencyPeriodOfLeg2(floatingRateResetFrequencyPeriodOfLeg2.evaluate(input));
			
			output
				.setFloatingRateResetFrequencyPeriodMultiplierOfLeg2(floatingRateResetFrequencyPeriodMultiplierOfLeg2.evaluate(input));
			
			output
				.setFixedRatePaymentFrequencyPeriodOfLeg1(fixedRatePaymentFrequencyPeriodOfLeg1.evaluate(input));
			
			output
				.setFixedRatePaymentFrequencyPeriodMultiplierOfLeg1(fixedRatePaymentFrequencyPeriodMultiplierOfLeg1.evaluate(input));
			
			output
				.setFixedRatePaymentFrequencyPeriodOfLeg2(fixedRatePaymentFrequencyPeriodOfLeg2.evaluate(input));
			
			output
				.setFixedRatePaymentFrequencyPeriodMultiplierOfLeg2(fixedRatePaymentFrequencyPeriodMultiplierOfLeg2.evaluate(input));
			
			output
				.setFloatingRatePaymentFrequencyPeriodOfLeg1(floatingRatePaymentFrequencyPeriodOfLeg1.evaluate(input));
			
			output
				.setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1(floatingRatePaymentFrequencyPeriodMultiplierOfLeg1.evaluate(input));
			
			output
				.setFloatingRatePaymentFrequencyPeriodOfLeg2(floatingRatePaymentFrequencyPeriodOfLeg2.evaluate(input));
			
			output
				.setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2(floatingRatePaymentFrequencyPeriodMultiplierOfLeg2.evaluate(input));
			
			output
				.setOtherPayment(otherPayment.evaluate(input));
			
			output
				.setActionType(actionType.evaluate(input));
			
			output
				.setEventType(eventType.evaluate(input));
			
			output
				.setEventDate(eventDate.evaluate(input));
			
			return Optional.ofNullable(output)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
