package drr.regulation.mas.rewrite.trade.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import cdm.base.staticdata.asset.common.ProductIdTypeEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.model.lib.reports.Tabulator.MultiNestedFieldValueImpl;
import com.rosetta.model.lib.reports.Tabulator.NestedFieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;
import iso20022.auth030.esma.Frequency13Code__1;
import iso20022.auth030.esma.InterestComputationMethod4Code;
import iso20022.auth030.esma.ValuationType1Code;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import javax.inject.Inject;


public class MASTransactionReportTypeTabulator implements Tabulator<MASTransactionReport> {
	private final Field counterparty1Field;
	private final Field counterparty2Field;
	private final Field counterparty2IdentifierTypeField;
	private final Field directionField;
	private final Field directionOfLeg1Field;
	private final Field directionOfLeg2Field;
	private final Field reportingSpecifiedPersonField;
	private final Field dataSubmitterField;
	private final Field uniqueTransactionIdentifierField;
	private final Field uniqueProductIdentifierField;
	private final Field priorUTIField;
	private final Field contractTypeField;
	private final Field assetClassField;
	private final Field underlyingField;
	private final Field underlyingIdentificationTypeField;
	private final Field underlyingIdOtherField;
	private final Field underlyingIdOtherSourceField;
	private final Field nameOfTheUnderlyingIndexField;
	private final Field fixedRateOfLeg1Field;
	private final Field identifierOfFloatingRateOfLeg1Field;
	private final Field nameOfTheFloatingRateOfLeg1Field;
	private final Field spreadOfLeg1Field;
	private final Field fixedRateOfLeg2Field;
	private final Field identifierOfFloatingRateOfLeg2Field;
	private final Field nameOfTheFloatingRateOfLeg2Field;
	private final Field spreadOfLeg2Field;
	private final Field optionTypeField;
	private final Field optionStyleField;
	private final Field firstExerciseDateField;
	private final Field effectiveDateField;
	private final Field expirationDateField;
	private final Field swapLinkIDField;
	private final Field packageIdentifierField;
	private final Field deliveryTypeField;
	private final Field settlementCurrency1Field;
	private final Field settlementCurrency2Field;
	private final Field finalContractualSettlementDateField;
	private final Field clearedField;
	private final Field centralCounterpartyField;
	private final Field clearingMemberField;
	private final Field clearingTimestampField;
	private final Field platformIdentifierField;
	private final Field reportingObligationOfSpecifiedPersonField;
	private final Field bookingLocationField;
	private final Field traderLocationField;
	private final Field confirmedField;
	private final Field confirmationTimestampField;
	private final Field executionTimestampField;
	private final Field reportingTimestampField;
	private final Field basketConstituentsField;
	private final Field valuationAmountField;
	private final Field valuationCurrencyField;
	private final Field valuationMethodField;
	private final Field valuationTimestampField;
	private final Field collateralPortfolioIndicatorField;
	private final Field initialMarginCollateralPortfolioCodeField;
	private final Field variationMarginCollateralPortfolioCodeField;
	private final Field notionalAmountOfTheContractOrLeg1Field;
	private final Field notionalCurrencyOfTheContractOrLeg1Field;
	private final Field notionalAmountOfLeg2Field;
	private final Field notionalCurrencyOfLeg2Field;
	private final Field totalNotionalQuantityOfTheContractOrLeg1Field;
	private final Field quantityUnitOfMeasureForTheContractOrLeg1Field;
	private final Field notionalQuantityOfTheContractOrLeg1Field;
	private final Field quantityFrequencyOfTheContractOrLeg1Field;
	private final Field quantityFrequencyMultiplierOfTheContractOrLeg1Field;
	private final Field totalNotionalQuantityLeg2Field;
	private final Field quantityUnitOfMeasureLeg2Field;
	private final Field notionalQuantityOfLeg2Field;
	private final Field quantityFrequencyOfLeg2Field;
	private final Field quantityFrequencyMultiplierOfLeg2Field;
	private final Field callAmountField;
	private final Field callCurrencyField;
	private final Field putAmountField;
	private final Field putCurrencyField;
	private final Field priceField;
	private final Field priceCurrencyField;
	private final Field priceNotationField;
	private final Field priceUnitOfMeasureField;
	private final Field priceScheduleField;
	private final Field strikePriceField;
	private final Field strikePriceCurrencyField;
	private final Field strikePriceNotationField;
	private final Field strikePriceScheduleField;
	private final Field optionPremiumAmountField;
	private final Field optionPremiumCurrencyField;
	private final Field exchangeRateField;
	private final Field exchangeRateBasisField;
	private final Field fixedRateDayCountConventionOfLeg1Field;
	private final Field fixedRateDayCountConventionOfLeg2Field;
	private final Field floatingRateDayCountConventionOfLeg1Field;
	private final Field floatingRateDayCountConventionOfLeg2Field;
	private final Field floatingRateResetFrequencyPeriodOfLeg1Field;
	private final Field floatingRateResetFrequencyPeriodMultiplierOfLeg1Field;
	private final Field floatingRateResetFrequencyPeriodOfLeg2Field;
	private final Field floatingRateResetFrequencyPeriodMultiplierOfLeg2Field;
	private final Field fixedRatePaymentFrequencyPeriodOfLeg1Field;
	private final Field fixedRatePaymentFrequencyPeriodMultiplierOfLeg1Field;
	private final Field fixedRatePaymentFrequencyPeriodOfLeg2Field;
	private final Field fixedRatePaymentFrequencyPeriodMultiplierOfLeg2Field;
	private final Field floatingRatePaymentFrequencyPeriodOfLeg1Field;
	private final Field floatingRatePaymentFrequencyPeriodMultiplierOfLeg1Field;
	private final Field floatingRatePaymentFrequencyPeriodOfLeg2Field;
	private final Field floatingRatePaymentFrequencyPeriodMultiplierOfLeg2Field;
	private final Field otherPaymentField;
	private final Field actionTypeField;
	private final Field eventTypeField;
	private final Field eventDateField;
	
	private final SpreadOfLeg1TypeTabulator spreadOfLeg1TypeTabulator;
	private final SpreadOfLeg2TypeTabulator spreadOfLeg2TypeTabulator;
	private final BasketConstituentsReportTypeTabulator basketConstituentsReportTypeTabulator;
	private final PriceScheduleReportTypeTabulator priceScheduleReportTypeTabulator;
	private final StrikePriceScheduleReportTypeTabulator strikePriceScheduleReportTypeTabulator;
	private final OtherPaymentReportTypeTabulator otherPaymentReportTypeTabulator;
	
	@Inject
	public MASTransactionReportTypeTabulator(SpreadOfLeg1TypeTabulator spreadOfLeg1TypeTabulator, SpreadOfLeg2TypeTabulator spreadOfLeg2TypeTabulator, BasketConstituentsReportTypeTabulator basketConstituentsReportTypeTabulator, PriceScheduleReportTypeTabulator priceScheduleReportTypeTabulator, StrikePriceScheduleReportTypeTabulator strikePriceScheduleReportTypeTabulator, OtherPaymentReportTypeTabulator otherPaymentReportTypeTabulator) {
		this.spreadOfLeg1TypeTabulator = spreadOfLeg1TypeTabulator;
		this.spreadOfLeg2TypeTabulator = spreadOfLeg2TypeTabulator;
		this.basketConstituentsReportTypeTabulator = basketConstituentsReportTypeTabulator;
		this.priceScheduleReportTypeTabulator = priceScheduleReportTypeTabulator;
		this.strikePriceScheduleReportTypeTabulator = strikePriceScheduleReportTypeTabulator;
		this.otherPaymentReportTypeTabulator = otherPaymentReportTypeTabulator;
		this.counterparty1Field = new FieldImpl(
			"counterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "Counterparty1")),
			Optional.of("1 Counterparty 1"),
			Arrays.asList()
		);
		this.counterparty2Field = new FieldImpl(
			"counterparty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "Counterparty2")),
			Optional.of("2 Counterparty 2"),
			Arrays.asList()
		);
		this.counterparty2IdentifierTypeField = new FieldImpl(
			"counterparty2IdentifierType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "Counterparty2IdentifierType")),
			Optional.of("3 Counterparty 2 Identifier Type"),
			Arrays.asList()
		);
		this.directionField = new FieldImpl(
			"direction",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "Direction")),
			Optional.of("5 Direction"),
			Arrays.asList()
		);
		this.directionOfLeg1Field = new FieldImpl(
			"directionOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "DirectionOfLeg1")),
			Optional.of("6 Direction of leg 1"),
			Arrays.asList()
		);
		this.directionOfLeg2Field = new FieldImpl(
			"directionOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "DirectionOfLeg2")),
			Optional.of("7 Direction of leg 2"),
			Arrays.asList()
		);
		this.reportingSpecifiedPersonField = new FieldImpl(
			"reportingSpecifiedPerson",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ReportingSpecifiedPerson")),
			Optional.of("8 Reporting specified person"),
			Arrays.asList()
		);
		this.dataSubmitterField = new FieldImpl(
			"dataSubmitter",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "DataSubmitter")),
			Optional.of("9 Data submitter"),
			Arrays.asList()
		);
		this.uniqueTransactionIdentifierField = new FieldImpl(
			"uniqueTransactionIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "UniqueTransactionIdentifier")),
			Optional.of("10 Unique Transaction Identifier (UTI)"),
			Arrays.asList()
		);
		this.uniqueProductIdentifierField = new FieldImpl(
			"uniqueProductIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "UniqueProductIdentifier")),
			Optional.of("11 Unique product identifier (UPI)"),
			Arrays.asList()
		);
		this.priorUTIField = new FieldImpl(
			"priorUTI",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "PriorUTI")),
			Optional.of("12 Prior UTI"),
			Arrays.asList()
		);
		this.contractTypeField = new FieldImpl(
			"contractType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ContractType")),
			Optional.of("13 Contract Type"),
			Arrays.asList()
		);
		this.assetClassField = new FieldImpl(
			"assetClass",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "AssetClass")),
			Optional.of("14 Asset Class"),
			Arrays.asList()
		);
		this.underlyingField = new FieldImpl(
			"underlying",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "Underlying")),
			Optional.of("15 Underlying"),
			Arrays.asList()
		);
		this.underlyingIdentificationTypeField = new FieldImpl(
			"underlyingIdentificationType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "UnderlyingIdentificationTypeDTCC")),
			Optional.of("15.2 Underlying Identification Type (DTCC)"),
			Arrays.asList()
		);
		this.underlyingIdOtherField = new FieldImpl(
			"underlyingIdOther",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "UnderlyingIdOtherDTCC")),
			Optional.of("15.3 Underlying ID (Other) (DTCC)"),
			Arrays.asList()
		);
		this.underlyingIdOtherSourceField = new FieldImpl(
			"underlyingIdOtherSource",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "UnderlyingIdOtherSourceDTCC")),
			Optional.of("15.4 Underlying ID (Other) Source (DTCC)"),
			Arrays.asList()
		);
		this.nameOfTheUnderlyingIndexField = new FieldImpl(
			"nameOfTheUnderlyingIndex",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "NameOfTheUnderlyingIndexDTCC")),
			Optional.of("15.5 Name of the underlying index (DTCC)"),
			Arrays.asList()
		);
		this.fixedRateOfLeg1Field = new FieldImpl(
			"fixedRateOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FixedRateOfLeg1")),
			Optional.of("16 Fixed Rate of leg 1 or coupon"),
			Arrays.asList()
		);
		this.identifierOfFloatingRateOfLeg1Field = new FieldImpl(
			"identifierOfFloatingRateOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "IdentifierOfFloatingRateOfLeg1")),
			Optional.of("17 Identifier of the floating rate of leg 1"),
			Arrays.asList()
		);
		this.nameOfTheFloatingRateOfLeg1Field = new FieldImpl(
			"nameOfTheFloatingRateOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "NameOfTheFloatingRateOfLeg1")),
			Optional.of("18 Name of the Floating Rate of Leg 1"),
			Arrays.asList()
		);
		this.spreadOfLeg1Field = new FieldImpl(
			"spreadOfLeg1",
			false,
			Optional.empty(),
			Optional.empty(),
			spreadOfLeg1TypeTabulator.getFields()
		);
		this.fixedRateOfLeg2Field = new FieldImpl(
			"fixedRateOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FixedRateOfLeg2")),
			Optional.of("22 Fixed Rate of leg 2"),
			Arrays.asList()
		);
		this.identifierOfFloatingRateOfLeg2Field = new FieldImpl(
			"identifierOfFloatingRateOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "IdentifierOfFloatingRateOfLeg2")),
			Optional.of("23 Identifier of the floating rate of leg 2"),
			Arrays.asList()
		);
		this.nameOfTheFloatingRateOfLeg2Field = new FieldImpl(
			"nameOfTheFloatingRateOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "NameOfTheFloatingRateOfLeg2")),
			Optional.of("24 Name of the Floating Rate of Leg 2"),
			Arrays.asList()
		);
		this.spreadOfLeg2Field = new FieldImpl(
			"spreadOfLeg2",
			false,
			Optional.empty(),
			Optional.empty(),
			spreadOfLeg2TypeTabulator.getFields()
		);
		this.optionTypeField = new FieldImpl(
			"optionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "OptionType")),
			Optional.of("28 Option type"),
			Arrays.asList()
		);
		this.optionStyleField = new FieldImpl(
			"optionStyle",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "OptionStyle")),
			Optional.of("29 Option Style"),
			Arrays.asList()
		);
		this.firstExerciseDateField = new FieldImpl(
			"firstExerciseDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FirstExerciseDate")),
			Optional.of("30 First exercise date"),
			Arrays.asList()
		);
		this.effectiveDateField = new FieldImpl(
			"effectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "EffectiveDate")),
			Optional.of("31 Effective Date"),
			Arrays.asList()
		);
		this.expirationDateField = new FieldImpl(
			"expirationDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ExpirationDate")),
			Optional.of("32 Expiration Date"),
			Arrays.asList()
		);
		this.swapLinkIDField = new FieldImpl(
			"swapLinkID",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "SwapLinkID")),
			Optional.of("33 Swap link ID"),
			Arrays.asList()
		);
		this.packageIdentifierField = new FieldImpl(
			"packageIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "PackageIdentifier")),
			Optional.of("34 Package identifier"),
			Arrays.asList()
		);
		this.deliveryTypeField = new FieldImpl(
			"deliveryType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "DeliveryType")),
			Optional.of("35 Delivery Type"),
			Arrays.asList()
		);
		this.settlementCurrency1Field = new FieldImpl(
			"settlementCurrency1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "SettlementCurrency1")),
			Optional.of("36 Settlement Currency 1"),
			Arrays.asList()
		);
		this.settlementCurrency2Field = new FieldImpl(
			"settlementCurrency2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "SettlementCurrency2")),
			Optional.of("37 Settlement Currency 2"),
			Arrays.asList()
		);
		this.finalContractualSettlementDateField = new FieldImpl(
			"finalContractualSettlementDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FinalContractualSettlementDate")),
			Optional.of("38 Final Contractual Settlement Date"),
			Arrays.asList()
		);
		this.clearedField = new FieldImpl(
			"cleared",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "Cleared")),
			Optional.of("39 Cleared"),
			Arrays.asList()
		);
		this.centralCounterpartyField = new FieldImpl(
			"centralCounterparty",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "CentralCounterparty")),
			Optional.of("40 Central Counterparty"),
			Arrays.asList()
		);
		this.clearingMemberField = new FieldImpl(
			"clearingMember",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ClearingMember")),
			Optional.of("41 Clearing member"),
			Arrays.asList()
		);
		this.clearingTimestampField = new FieldImpl(
			"clearingTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ClearingTimestamp")),
			Optional.of("42 Clearing Timestamp"),
			Arrays.asList()
		);
		this.platformIdentifierField = new FieldImpl(
			"platformIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "PlatformIdentifier")),
			Optional.of("43 Platform Identifier"),
			Arrays.asList()
		);
		this.reportingObligationOfSpecifiedPersonField = new FieldImpl(
			"reportingObligationOfSpecifiedPerson",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ReportingObligationOfSpecifiedPerson")),
			Optional.of("44 Reporting obligation of specified person"),
			Arrays.asList()
		);
		this.bookingLocationField = new FieldImpl(
			"bookingLocation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "BookingLocation")),
			Optional.of("45 Booking location"),
			Arrays.asList()
		);
		this.traderLocationField = new FieldImpl(
			"traderLocation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "TraderLocation")),
			Optional.of("46 Trader location"),
			Arrays.asList()
		);
		this.confirmedField = new FieldImpl(
			"confirmed",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "Confirmed")),
			Optional.of("47 Confirmed"),
			Arrays.asList()
		);
		this.confirmationTimestampField = new FieldImpl(
			"confirmationTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ConfirmationTimestamp")),
			Optional.of("48 Confirmation Timestamp"),
			Arrays.asList()
		);
		this.executionTimestampField = new FieldImpl(
			"executionTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ExecutionTimestamp")),
			Optional.of("49 Execution Timestamp"),
			Arrays.asList()
		);
		this.reportingTimestampField = new FieldImpl(
			"reportingTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ReportingTimestamp")),
			Optional.of("50 Reporting Timestamp"),
			Arrays.asList()
		);
		this.basketConstituentsField = new FieldImpl(
			"basketConstituents",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "BasketConstituents")),
			Optional.empty(),
			basketConstituentsReportTypeTabulator.getFields()
		);
		this.valuationAmountField = new FieldImpl(
			"valuationAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ValuationAmount")),
			Optional.of("53 Valuation amount"),
			Arrays.asList()
		);
		this.valuationCurrencyField = new FieldImpl(
			"valuationCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ValuationCurrency")),
			Optional.of("54 Valuation currency"),
			Arrays.asList()
		);
		this.valuationMethodField = new FieldImpl(
			"valuationMethod",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ValuationMethod")),
			Optional.of("55 Valuation method"),
			Arrays.asList()
		);
		this.valuationTimestampField = new FieldImpl(
			"valuationTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ValuationTimestamp")),
			Optional.of("56 Valuation Timestamp"),
			Arrays.asList()
		);
		this.collateralPortfolioIndicatorField = new FieldImpl(
			"collateralPortfolioIndicator",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "CollateralPortfolioIndicator")),
			Optional.of("57 Collateral Portfolio Indicator"),
			Arrays.asList()
		);
		this.initialMarginCollateralPortfolioCodeField = new FieldImpl(
			"initialMarginCollateralPortfolioCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "InitialMarginCollateralPortfolioCodeDTCC")),
			Optional.of("58.1 Initial margin collateral portfolio code (DTCC)"),
			Arrays.asList()
		);
		this.variationMarginCollateralPortfolioCodeField = new FieldImpl(
			"variationMarginCollateralPortfolioCode",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "VariationMarginCollateralPortfolioCodeDTCC")),
			Optional.of("58.2 Variation margin collateral portfolio code (DTCC)"),
			Arrays.asList()
		);
		this.notionalAmountOfTheContractOrLeg1Field = new FieldImpl(
			"notionalAmountOfTheContractOrLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "NotionalAmountOfTheContractOrLeg1")),
			Optional.of("76 Notional amount of the contract or leg 1"),
			Arrays.asList()
		);
		this.notionalCurrencyOfTheContractOrLeg1Field = new FieldImpl(
			"notionalCurrencyOfTheContractOrLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "NotionalCurrencyOfTheContractOrLeg1")),
			Optional.of("77 Notional currency of the contract or leg 1"),
			Arrays.asList()
		);
		this.notionalAmountOfLeg2Field = new FieldImpl(
			"notionalAmountOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "NotionalAmountOfLeg2")),
			Optional.of("78 Notional amount of leg 2"),
			Arrays.asList()
		);
		this.notionalCurrencyOfLeg2Field = new FieldImpl(
			"notionalCurrencyOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "NotionalCurrencyOfLeg2")),
			Optional.of("79 Notional currency of leg 2"),
			Arrays.asList()
		);
		this.totalNotionalQuantityOfTheContractOrLeg1Field = new FieldImpl(
			"totalNotionalQuantityOfTheContractOrLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "TotalNotionalQuantityOfTheContractOrLeg1")),
			Optional.of("80 Total notional quantity of the contract or leg 1"),
			Arrays.asList()
		);
		this.quantityUnitOfMeasureForTheContractOrLeg1Field = new FieldImpl(
			"quantityUnitOfMeasureForTheContractOrLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "QuantityUnitOfMeasureForTheContractOrLeg1")),
			Optional.of("81 Quantity unit of measure for the contract or leg 1"),
			Arrays.asList()
		);
		this.notionalQuantityOfTheContractOrLeg1Field = new FieldImpl(
			"notionalQuantityOfTheContractOrLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "NotionalQuantityOfTheContractOrLeg1")),
			Optional.of("82 Notional quantity of the contract or leg 1"),
			Arrays.asList()
		);
		this.quantityFrequencyOfTheContractOrLeg1Field = new FieldImpl(
			"quantityFrequencyOfTheContractOrLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "QuantityFrequencyOfTheContractOrLeg1")),
			Optional.of("83 Quantity frequency of the contract or leg 1"),
			Arrays.asList()
		);
		this.quantityFrequencyMultiplierOfTheContractOrLeg1Field = new FieldImpl(
			"quantityFrequencyMultiplierOfTheContractOrLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "QuantityFrequencyMultiplierOfTheContractOrLeg1")),
			Optional.of("84 Quantity frequency multiplier of the contract or leg 1"),
			Arrays.asList()
		);
		this.totalNotionalQuantityLeg2Field = new FieldImpl(
			"totalNotionalQuantityLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "TotalNotionalQuantityOfLeg2")),
			Optional.of("85 Total notional quantity of leg 2"),
			Arrays.asList()
		);
		this.quantityUnitOfMeasureLeg2Field = new FieldImpl(
			"quantityUnitOfMeasureLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "QuantityUnitOfMeasureLeg2")),
			Optional.of("86 Quantity Unit Of Measure Leg 2"),
			Arrays.asList()
		);
		this.notionalQuantityOfLeg2Field = new FieldImpl(
			"notionalQuantityOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "NotionalQuantityOfLeg2")),
			Optional.of("87 Notional quantity of leg 2"),
			Arrays.asList()
		);
		this.quantityFrequencyOfLeg2Field = new FieldImpl(
			"quantityFrequencyOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "QuantityFrequencyOfLeg2")),
			Optional.of("88 Quantity frequency of leg 2"),
			Arrays.asList()
		);
		this.quantityFrequencyMultiplierOfLeg2Field = new FieldImpl(
			"quantityFrequencyMultiplierOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "QuantityFrequencyMultiplierOfLeg2")),
			Optional.of("89 Quantity frequency multiplier of leg 2"),
			Arrays.asList()
		);
		this.callAmountField = new FieldImpl(
			"callAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "CallAmount")),
			Optional.of("90 Call amount"),
			Arrays.asList()
		);
		this.callCurrencyField = new FieldImpl(
			"callCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "CallCurrency")),
			Optional.of("91 Call currency"),
			Arrays.asList()
		);
		this.putAmountField = new FieldImpl(
			"putAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "PutAmount")),
			Optional.of("92 Put Amount"),
			Arrays.asList()
		);
		this.putCurrencyField = new FieldImpl(
			"putCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "PutCurrency")),
			Optional.of("93 Put currency"),
			Arrays.asList()
		);
		this.priceField = new FieldImpl(
			"price",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "Price")),
			Optional.of("94 Price"),
			Arrays.asList()
		);
		this.priceCurrencyField = new FieldImpl(
			"priceCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "PriceCurrency")),
			Optional.of("95 Price currency"),
			Arrays.asList()
		);
		this.priceNotationField = new FieldImpl(
			"priceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "PriceNotation")),
			Optional.of("96 Price notation"),
			Arrays.asList()
		);
		this.priceUnitOfMeasureField = new FieldImpl(
			"priceUnitOfMeasure",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "PriceUnitOfMeasure")),
			Optional.of("97 Price unit of measure"),
			Arrays.asList()
		);
		this.priceScheduleField = new FieldImpl(
			"priceSchedule",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ContractPriceSchedule")),
			Optional.empty(),
			priceScheduleReportTypeTabulator.getFields()
		);
		this.strikePriceField = new FieldImpl(
			"strikePrice",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "StrikePrice")),
			Optional.of("101 Strike Price"),
			Arrays.asList()
		);
		this.strikePriceCurrencyField = new FieldImpl(
			"strikePriceCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "StrikePriceCurrency")),
			Optional.of("102 Strike price currency/currency pair"),
			Arrays.asList()
		);
		this.strikePriceNotationField = new FieldImpl(
			"strikePriceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "StrikePriceNotation")),
			Optional.of("103 Strike Price notation"),
			Arrays.asList()
		);
		this.strikePriceScheduleField = new FieldImpl(
			"strikePriceSchedule",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "StrikePriceSchedule")),
			Optional.empty(),
			strikePriceScheduleReportTypeTabulator.getFields()
		);
		this.optionPremiumAmountField = new FieldImpl(
			"optionPremiumAmount",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "OptionPremiumAmount")),
			Optional.of("107 Option Premium Amount"),
			Arrays.asList()
		);
		this.optionPremiumCurrencyField = new FieldImpl(
			"optionPremiumCurrency",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "OptionPremiumCurrency")),
			Optional.of("108 Option Premium Currency"),
			Arrays.asList()
		);
		this.exchangeRateField = new FieldImpl(
			"exchangeRate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ExchangeRate")),
			Optional.of("109 Exchange rate"),
			Arrays.asList()
		);
		this.exchangeRateBasisField = new FieldImpl(
			"exchangeRateBasis",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ExchangeRateBasis")),
			Optional.of("110 Exchange Rate Basis"),
			Arrays.asList()
		);
		this.fixedRateDayCountConventionOfLeg1Field = new FieldImpl(
			"fixedRateDayCountConventionOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FixedRateDayCountConventionOfLeg1")),
			Optional.of("111 Fixed rate day count convention of leg 1"),
			Arrays.asList()
		);
		this.fixedRateDayCountConventionOfLeg2Field = new FieldImpl(
			"fixedRateDayCountConventionOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FixedRateDayCountConventionOfLeg2")),
			Optional.of("112 Fixed rate day count convention of leg 2"),
			Arrays.asList()
		);
		this.floatingRateDayCountConventionOfLeg1Field = new FieldImpl(
			"floatingRateDayCountConventionOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FloatingRateDayCountConventionOfLeg1")),
			Optional.of("113 Floating rate day count convention of leg 1"),
			Arrays.asList()
		);
		this.floatingRateDayCountConventionOfLeg2Field = new FieldImpl(
			"floatingRateDayCountConventionOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FloatingRateDayCountConventionOfLeg2")),
			Optional.of("114 Floating rate day count convention of leg 2"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodOfLeg1Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FloatingRateResetFrequencyPeriodOfLeg1")),
			Optional.of("115 Floating rate reset frequency period of leg 1"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodMultiplierOfLeg1Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodMultiplierOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FloatingRateResetFrequencyPeriodMultiplierOfLeg1")),
			Optional.of("116 Floating rate reset frequency period multiplier of leg 1"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodOfLeg2Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FloatingRateResetFrequencyPeriodOfLeg2")),
			Optional.of("117 Floating rate reset frequency period of Leg 2"),
			Arrays.asList()
		);
		this.floatingRateResetFrequencyPeriodMultiplierOfLeg2Field = new FieldImpl(
			"floatingRateResetFrequencyPeriodMultiplierOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FloatingRateResetFrequencyPeriodMultiplierOfLeg2")),
			Optional.of("118 Floating rate reset frequency period multiplier of leg 2"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodOfLeg1Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FixedRatePaymentFrequencyPeriodOfLeg1")),
			Optional.of("119 Fixed rate payment frequency period of leg 1"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodMultiplierOfLeg1Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodMultiplierOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FixedRatePaymentFrequencyPeriodMultiplierOfLeg1")),
			Optional.of("120 Fixed rate payment frequency period multiplier of leg 1"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodOfLeg2Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FixedRatePaymentFrequencyPeriodOfLeg2")),
			Optional.of("121 Fixed rate payment frequency period of leg 2"),
			Arrays.asList()
		);
		this.fixedRatePaymentFrequencyPeriodMultiplierOfLeg2Field = new FieldImpl(
			"fixedRatePaymentFrequencyPeriodMultiplierOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FixedRatePaymentFrequencyPeriodMultiplierOfLeg2")),
			Optional.of("122 Fixed rate payment frequency period multiplier of leg 2"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodOfLeg1Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FloatingRatePaymentFrequencyPeriodOfLeg1")),
			Optional.of("123 Floating rate payment frequency period of leg 1"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg1Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodMultiplierOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FloatingRatePaymentFrequencyPeriodMultiplierOfLeg1")),
			Optional.of("124 Floating rate payment frequency period multiplier of leg 1"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodOfLeg2Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FloatingRatePaymentFrequencyPeriodOfLeg2")),
			Optional.of("125 Floating rate payment frequency period of leg 2"),
			Arrays.asList()
		);
		this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg2Field = new FieldImpl(
			"floatingRatePaymentFrequencyPeriodMultiplierOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "FloatingRatePaymentFrequencyPeriodMultiplierOfLeg2")),
			Optional.of("126 Floating rate payment frequency period multiplier of leg 2"),
			Arrays.asList()
		);
		this.otherPaymentField = new FieldImpl(
			"otherPayment",
			true,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "OtherPayment")),
			Optional.empty(),
			otherPaymentReportTypeTabulator.getFields()
		);
		this.actionTypeField = new FieldImpl(
			"actionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "ActionType")),
			Optional.of("132 Action type"),
			Arrays.asList()
		);
		this.eventTypeField = new FieldImpl(
			"eventType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "EventType")),
			Optional.of("133 Event Type"),
			Arrays.asList()
		);
		this.eventDateField = new FieldImpl(
			"eventDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "EventDate")),
			Optional.of("134 Event Date"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(counterparty1Field, counterparty2Field, counterparty2IdentifierTypeField, directionField, directionOfLeg1Field, directionOfLeg2Field, reportingSpecifiedPersonField, dataSubmitterField, uniqueTransactionIdentifierField, uniqueProductIdentifierField, priorUTIField, contractTypeField, assetClassField, underlyingField, underlyingIdentificationTypeField, underlyingIdOtherField, underlyingIdOtherSourceField, nameOfTheUnderlyingIndexField, fixedRateOfLeg1Field, identifierOfFloatingRateOfLeg1Field, nameOfTheFloatingRateOfLeg1Field, spreadOfLeg1Field, fixedRateOfLeg2Field, identifierOfFloatingRateOfLeg2Field, nameOfTheFloatingRateOfLeg2Field, spreadOfLeg2Field, optionTypeField, optionStyleField, firstExerciseDateField, effectiveDateField, expirationDateField, swapLinkIDField, packageIdentifierField, deliveryTypeField, settlementCurrency1Field, settlementCurrency2Field, finalContractualSettlementDateField, clearedField, centralCounterpartyField, clearingMemberField, clearingTimestampField, platformIdentifierField, reportingObligationOfSpecifiedPersonField, bookingLocationField, traderLocationField, confirmedField, confirmationTimestampField, executionTimestampField, reportingTimestampField, basketConstituentsField, valuationAmountField, valuationCurrencyField, valuationMethodField, valuationTimestampField, collateralPortfolioIndicatorField, initialMarginCollateralPortfolioCodeField, variationMarginCollateralPortfolioCodeField, notionalAmountOfTheContractOrLeg1Field, notionalCurrencyOfTheContractOrLeg1Field, notionalAmountOfLeg2Field, notionalCurrencyOfLeg2Field, totalNotionalQuantityOfTheContractOrLeg1Field, quantityUnitOfMeasureForTheContractOrLeg1Field, notionalQuantityOfTheContractOrLeg1Field, quantityFrequencyOfTheContractOrLeg1Field, quantityFrequencyMultiplierOfTheContractOrLeg1Field, totalNotionalQuantityLeg2Field, quantityUnitOfMeasureLeg2Field, notionalQuantityOfLeg2Field, quantityFrequencyOfLeg2Field, quantityFrequencyMultiplierOfLeg2Field, callAmountField, callCurrencyField, putAmountField, putCurrencyField, priceField, priceCurrencyField, priceNotationField, priceUnitOfMeasureField, priceScheduleField, strikePriceField, strikePriceCurrencyField, strikePriceNotationField, strikePriceScheduleField, optionPremiumAmountField, optionPremiumCurrencyField, exchangeRateField, exchangeRateBasisField, fixedRateDayCountConventionOfLeg1Field, fixedRateDayCountConventionOfLeg2Field, floatingRateDayCountConventionOfLeg1Field, floatingRateDayCountConventionOfLeg2Field, floatingRateResetFrequencyPeriodOfLeg1Field, floatingRateResetFrequencyPeriodMultiplierOfLeg1Field, floatingRateResetFrequencyPeriodOfLeg2Field, floatingRateResetFrequencyPeriodMultiplierOfLeg2Field, fixedRatePaymentFrequencyPeriodOfLeg1Field, fixedRatePaymentFrequencyPeriodMultiplierOfLeg1Field, fixedRatePaymentFrequencyPeriodOfLeg2Field, fixedRatePaymentFrequencyPeriodMultiplierOfLeg2Field, floatingRatePaymentFrequencyPeriodOfLeg1Field, floatingRatePaymentFrequencyPeriodMultiplierOfLeg1Field, floatingRatePaymentFrequencyPeriodOfLeg2Field, floatingRatePaymentFrequencyPeriodMultiplierOfLeg2Field, otherPaymentField, actionTypeField, eventTypeField, eventDateField);
	}
	
	@Override
	public List<FieldValue> tabulate(MASTransactionReport input) {
		Optional<String> counterparty1 = Optional.ofNullable(input.getCounterparty1());
		Optional<String> counterparty2 = Optional.ofNullable(input.getCounterparty2());
		Optional<Boolean> counterparty2IdentifierType = Optional.ofNullable(input.getCounterparty2IdentifierType());
		Optional<String> direction = Optional.ofNullable(input.getDirection());
		Optional<String> directionOfLeg1 = Optional.ofNullable(input.getDirectionOfLeg1());
		Optional<String> directionOfLeg2 = Optional.ofNullable(input.getDirectionOfLeg2());
		Optional<String> reportingSpecifiedPerson = Optional.ofNullable(input.getReportingSpecifiedPerson());
		Optional<String> dataSubmitter = Optional.ofNullable(input.getDataSubmitter());
		Optional<String> uniqueTransactionIdentifier = Optional.ofNullable(input.getUniqueTransactionIdentifier());
		Optional<String> uniqueProductIdentifier = Optional.ofNullable(input.getUniqueProductIdentifier());
		Optional<String> priorUTI = Optional.ofNullable(input.getPriorUTI());
		Optional<String> contractType = Optional.ofNullable(input.getContractType());
		Optional<String> assetClass = Optional.ofNullable(input.getAssetClass());
		Optional<String> underlying = Optional.ofNullable(input.getUnderlying());
		Optional<String> underlyingIdentificationType = Optional.ofNullable(input.getUnderlyingIdentificationType());
		Optional<String> underlyingIdOther = Optional.ofNullable(input.getUnderlyingIdOther());
		Optional<ProductIdTypeEnum> underlyingIdOtherSource = Optional.ofNullable(input.getUnderlyingIdOtherSource());
		Optional<String> nameOfTheUnderlyingIndex = Optional.ofNullable(input.getNameOfTheUnderlyingIndex());
		Optional<BigDecimal> fixedRateOfLeg1 = Optional.ofNullable(input.getFixedRateOfLeg1());
		Optional<String> identifierOfFloatingRateOfLeg1 = Optional.ofNullable(input.getIdentifierOfFloatingRateOfLeg1());
		Optional<String> nameOfTheFloatingRateOfLeg1 = Optional.ofNullable(input.getNameOfTheFloatingRateOfLeg1());
		Optional<List<FieldValue>> spreadOfLeg1 = Optional.ofNullable(input.getSpreadOfLeg1())
			.map(x -> spreadOfLeg1TypeTabulator.tabulate(x));
		Optional<BigDecimal> fixedRateOfLeg2 = Optional.ofNullable(input.getFixedRateOfLeg2());
		Optional<String> identifierOfFloatingRateOfLeg2 = Optional.ofNullable(input.getIdentifierOfFloatingRateOfLeg2());
		Optional<String> nameOfTheFloatingRateOfLeg2 = Optional.ofNullable(input.getNameOfTheFloatingRateOfLeg2());
		Optional<List<FieldValue>> spreadOfLeg2 = Optional.ofNullable(input.getSpreadOfLeg2())
			.map(x -> spreadOfLeg2TypeTabulator.tabulate(x));
		Optional<String> optionType = Optional.ofNullable(input.getOptionType());
		Optional<String> optionStyle = Optional.ofNullable(input.getOptionStyle());
		Optional<Date> firstExerciseDate = Optional.ofNullable(input.getFirstExerciseDate());
		Optional<Date> effectiveDate = Optional.ofNullable(input.getEffectiveDate());
		Optional<Date> expirationDate = Optional.ofNullable(input.getExpirationDate());
		Optional<String> swapLinkID = Optional.ofNullable(input.getSwapLinkID());
		Optional<String> packageIdentifier = Optional.ofNullable(input.getPackageIdentifier());
		Optional<String> deliveryType = Optional.ofNullable(input.getDeliveryType());
		Optional<ISOCurrencyCodeEnum> settlementCurrency1 = Optional.ofNullable(input.getSettlementCurrency1());
		Optional<ISOCurrencyCodeEnum> settlementCurrency2 = Optional.ofNullable(input.getSettlementCurrency2());
		Optional<Date> finalContractualSettlementDate = Optional.ofNullable(input.getFinalContractualSettlementDate());
		Optional<String> cleared = Optional.ofNullable(input.getCleared());
		Optional<String> centralCounterparty = Optional.ofNullable(input.getCentralCounterparty());
		Optional<String> clearingMember = Optional.ofNullable(input.getClearingMember());
		Optional<ZonedDateTime> clearingTimestamp = Optional.ofNullable(input.getClearingTimestamp());
		Optional<String> platformIdentifier = Optional.ofNullable(input.getPlatformIdentifier());
		Optional<String> reportingObligationOfSpecifiedPerson = Optional.ofNullable(input.getReportingObligationOfSpecifiedPerson());
		Optional<String> bookingLocation = Optional.ofNullable(input.getBookingLocation());
		Optional<String> traderLocation = Optional.ofNullable(input.getTraderLocation());
		Optional<String> confirmed = Optional.ofNullable(input.getConfirmed());
		Optional<ZonedDateTime> confirmationTimestamp = Optional.ofNullable(input.getConfirmationTimestamp());
		Optional<ZonedDateTime> executionTimestamp = Optional.ofNullable(input.getExecutionTimestamp());
		Optional<ZonedDateTime> reportingTimestamp = Optional.ofNullable(input.getReportingTimestamp());
		Optional<List<List<FieldValue>>> basketConstituents = Optional.ofNullable(input.getBasketConstituents())
			.map(x -> x.stream()
				.map(_x -> basketConstituentsReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<BigDecimal> valuationAmount = Optional.ofNullable(input.getValuationAmount());
		Optional<ISOCurrencyCodeEnum> valuationCurrency = Optional.ofNullable(input.getValuationCurrency());
		Optional<ValuationType1Code> valuationMethod = Optional.ofNullable(input.getValuationMethod());
		Optional<ZonedDateTime> valuationTimestamp = Optional.ofNullable(input.getValuationTimestamp());
		Optional<Boolean> collateralPortfolioIndicator = Optional.ofNullable(input.getCollateralPortfolioIndicator());
		Optional<String> initialMarginCollateralPortfolioCode = Optional.ofNullable(input.getInitialMarginCollateralPortfolioCode());
		Optional<String> variationMarginCollateralPortfolioCode = Optional.ofNullable(input.getVariationMarginCollateralPortfolioCode());
		Optional<BigDecimal> notionalAmountOfTheContractOrLeg1 = Optional.ofNullable(input.getNotionalAmountOfTheContractOrLeg1());
		Optional<ISOCurrencyCodeEnum> notionalCurrencyOfTheContractOrLeg1 = Optional.ofNullable(input.getNotionalCurrencyOfTheContractOrLeg1());
		Optional<BigDecimal> notionalAmountOfLeg2 = Optional.ofNullable(input.getNotionalAmountOfLeg2());
		Optional<ISOCurrencyCodeEnum> notionalCurrencyOfLeg2 = Optional.ofNullable(input.getNotionalCurrencyOfLeg2());
		Optional<BigDecimal> totalNotionalQuantityOfTheContractOrLeg1 = Optional.ofNullable(input.getTotalNotionalQuantityOfTheContractOrLeg1());
		Optional<String> quantityUnitOfMeasureForTheContractOrLeg1 = Optional.ofNullable(input.getQuantityUnitOfMeasureForTheContractOrLeg1());
		Optional<BigDecimal> notionalQuantityOfTheContractOrLeg1 = Optional.ofNullable(input.getNotionalQuantityOfTheContractOrLeg1());
		Optional<String> quantityFrequencyOfTheContractOrLeg1 = Optional.ofNullable(input.getQuantityFrequencyOfTheContractOrLeg1());
		Optional<Integer> quantityFrequencyMultiplierOfTheContractOrLeg1 = Optional.ofNullable(input.getQuantityFrequencyMultiplierOfTheContractOrLeg1());
		Optional<BigDecimal> totalNotionalQuantityLeg2 = Optional.ofNullable(input.getTotalNotionalQuantityLeg2());
		Optional<String> quantityUnitOfMeasureLeg2 = Optional.ofNullable(input.getQuantityUnitOfMeasureLeg2());
		Optional<BigDecimal> notionalQuantityOfLeg2 = Optional.ofNullable(input.getNotionalQuantityOfLeg2());
		Optional<String> quantityFrequencyOfLeg2 = Optional.ofNullable(input.getQuantityFrequencyOfLeg2());
		Optional<Integer> quantityFrequencyMultiplierOfLeg2 = Optional.ofNullable(input.getQuantityFrequencyMultiplierOfLeg2());
		Optional<BigDecimal> callAmount = Optional.ofNullable(input.getCallAmount());
		Optional<ISOCurrencyCodeEnum> callCurrency = Optional.ofNullable(input.getCallCurrency());
		Optional<BigDecimal> putAmount = Optional.ofNullable(input.getPutAmount());
		Optional<ISOCurrencyCodeEnum> putCurrency = Optional.ofNullable(input.getPutCurrency());
		Optional<BigDecimal> price = Optional.ofNullable(input.getPrice());
		Optional<ISOCurrencyCodeEnum> priceCurrency = Optional.ofNullable(input.getPriceCurrency());
		Optional<String> priceNotation = Optional.ofNullable(input.getPriceNotation());
		Optional<String> priceUnitOfMeasure = Optional.ofNullable(input.getPriceUnitOfMeasure());
		Optional<List<List<FieldValue>>> priceSchedule = Optional.ofNullable(input.getPriceSchedule())
			.map(x -> x.stream()
				.map(_x -> priceScheduleReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<BigDecimal> strikePrice = Optional.ofNullable(input.getStrikePrice());
		Optional<String> strikePriceCurrency = Optional.ofNullable(input.getStrikePriceCurrency());
		Optional<String> strikePriceNotation = Optional.ofNullable(input.getStrikePriceNotation());
		Optional<List<List<FieldValue>>> strikePriceSchedule = Optional.ofNullable(input.getStrikePriceSchedule())
			.map(x -> x.stream()
				.map(_x -> strikePriceScheduleReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<BigDecimal> optionPremiumAmount = Optional.ofNullable(input.getOptionPremiumAmount());
		Optional<ISOCurrencyCodeEnum> optionPremiumCurrency = Optional.ofNullable(input.getOptionPremiumCurrency());
		Optional<BigDecimal> exchangeRate = Optional.ofNullable(input.getExchangeRate());
		Optional<String> exchangeRateBasis = Optional.ofNullable(input.getExchangeRateBasis());
		Optional<InterestComputationMethod4Code> fixedRateDayCountConventionOfLeg1 = Optional.ofNullable(input.getFixedRateDayCountConventionOfLeg1());
		Optional<InterestComputationMethod4Code> fixedRateDayCountConventionOfLeg2 = Optional.ofNullable(input.getFixedRateDayCountConventionOfLeg2());
		Optional<InterestComputationMethod4Code> floatingRateDayCountConventionOfLeg1 = Optional.ofNullable(input.getFloatingRateDayCountConventionOfLeg1());
		Optional<InterestComputationMethod4Code> floatingRateDayCountConventionOfLeg2 = Optional.ofNullable(input.getFloatingRateDayCountConventionOfLeg2());
		Optional<String> floatingRateResetFrequencyPeriodOfLeg1 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodOfLeg1());
		Optional<Integer> floatingRateResetFrequencyPeriodMultiplierOfLeg1 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodMultiplierOfLeg1());
		Optional<String> floatingRateResetFrequencyPeriodOfLeg2 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodOfLeg2());
		Optional<Integer> floatingRateResetFrequencyPeriodMultiplierOfLeg2 = Optional.ofNullable(input.getFloatingRateResetFrequencyPeriodMultiplierOfLeg2());
		Optional<String> fixedRatePaymentFrequencyPeriodOfLeg1 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodOfLeg1());
		Optional<Integer> fixedRatePaymentFrequencyPeriodMultiplierOfLeg1 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg1());
		Optional<String> fixedRatePaymentFrequencyPeriodOfLeg2 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodOfLeg2());
		Optional<Integer> fixedRatePaymentFrequencyPeriodMultiplierOfLeg2 = Optional.ofNullable(input.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2());
		Optional<Frequency13Code__1> floatingRatePaymentFrequencyPeriodOfLeg1 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodOfLeg1());
		Optional<Integer> floatingRatePaymentFrequencyPeriodMultiplierOfLeg1 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1());
		Optional<Frequency13Code__1> floatingRatePaymentFrequencyPeriodOfLeg2 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodOfLeg2());
		Optional<Integer> floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 = Optional.ofNullable(input.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2());
		Optional<List<List<FieldValue>>> otherPayment = Optional.ofNullable(input.getOtherPayment())
			.map(x -> x.stream()
				.map(_x -> otherPaymentReportTypeTabulator.tabulate(_x))
				.collect(Collectors.toList()));
		Optional<String> actionType = Optional.ofNullable(input.getActionType());
		Optional<String> eventType = Optional.ofNullable(input.getEventType());
		Optional<Date> eventDate = Optional.ofNullable(input.getEventDate());
		return Arrays.asList(
			new FieldValueImpl(counterparty1Field, counterparty1),
			new FieldValueImpl(counterparty2Field, counterparty2),
			new FieldValueImpl(counterparty2IdentifierTypeField, counterparty2IdentifierType),
			new FieldValueImpl(directionField, direction),
			new FieldValueImpl(directionOfLeg1Field, directionOfLeg1),
			new FieldValueImpl(directionOfLeg2Field, directionOfLeg2),
			new FieldValueImpl(reportingSpecifiedPersonField, reportingSpecifiedPerson),
			new FieldValueImpl(dataSubmitterField, dataSubmitter),
			new FieldValueImpl(uniqueTransactionIdentifierField, uniqueTransactionIdentifier),
			new FieldValueImpl(uniqueProductIdentifierField, uniqueProductIdentifier),
			new FieldValueImpl(priorUTIField, priorUTI),
			new FieldValueImpl(contractTypeField, contractType),
			new FieldValueImpl(assetClassField, assetClass),
			new FieldValueImpl(underlyingField, underlying),
			new FieldValueImpl(underlyingIdentificationTypeField, underlyingIdentificationType),
			new FieldValueImpl(underlyingIdOtherField, underlyingIdOther),
			new FieldValueImpl(underlyingIdOtherSourceField, underlyingIdOtherSource),
			new FieldValueImpl(nameOfTheUnderlyingIndexField, nameOfTheUnderlyingIndex),
			new FieldValueImpl(fixedRateOfLeg1Field, fixedRateOfLeg1),
			new FieldValueImpl(identifierOfFloatingRateOfLeg1Field, identifierOfFloatingRateOfLeg1),
			new FieldValueImpl(nameOfTheFloatingRateOfLeg1Field, nameOfTheFloatingRateOfLeg1),
			new NestedFieldValueImpl(spreadOfLeg1Field, spreadOfLeg1),
			new FieldValueImpl(fixedRateOfLeg2Field, fixedRateOfLeg2),
			new FieldValueImpl(identifierOfFloatingRateOfLeg2Field, identifierOfFloatingRateOfLeg2),
			new FieldValueImpl(nameOfTheFloatingRateOfLeg2Field, nameOfTheFloatingRateOfLeg2),
			new NestedFieldValueImpl(spreadOfLeg2Field, spreadOfLeg2),
			new FieldValueImpl(optionTypeField, optionType),
			new FieldValueImpl(optionStyleField, optionStyle),
			new FieldValueImpl(firstExerciseDateField, firstExerciseDate),
			new FieldValueImpl(effectiveDateField, effectiveDate),
			new FieldValueImpl(expirationDateField, expirationDate),
			new FieldValueImpl(swapLinkIDField, swapLinkID),
			new FieldValueImpl(packageIdentifierField, packageIdentifier),
			new FieldValueImpl(deliveryTypeField, deliveryType),
			new FieldValueImpl(settlementCurrency1Field, settlementCurrency1),
			new FieldValueImpl(settlementCurrency2Field, settlementCurrency2),
			new FieldValueImpl(finalContractualSettlementDateField, finalContractualSettlementDate),
			new FieldValueImpl(clearedField, cleared),
			new FieldValueImpl(centralCounterpartyField, centralCounterparty),
			new FieldValueImpl(clearingMemberField, clearingMember),
			new FieldValueImpl(clearingTimestampField, clearingTimestamp),
			new FieldValueImpl(platformIdentifierField, platformIdentifier),
			new FieldValueImpl(reportingObligationOfSpecifiedPersonField, reportingObligationOfSpecifiedPerson),
			new FieldValueImpl(bookingLocationField, bookingLocation),
			new FieldValueImpl(traderLocationField, traderLocation),
			new FieldValueImpl(confirmedField, confirmed),
			new FieldValueImpl(confirmationTimestampField, confirmationTimestamp),
			new FieldValueImpl(executionTimestampField, executionTimestamp),
			new FieldValueImpl(reportingTimestampField, reportingTimestamp),
			new MultiNestedFieldValueImpl(basketConstituentsField, basketConstituents),
			new FieldValueImpl(valuationAmountField, valuationAmount),
			new FieldValueImpl(valuationCurrencyField, valuationCurrency),
			new FieldValueImpl(valuationMethodField, valuationMethod),
			new FieldValueImpl(valuationTimestampField, valuationTimestamp),
			new FieldValueImpl(collateralPortfolioIndicatorField, collateralPortfolioIndicator),
			new FieldValueImpl(initialMarginCollateralPortfolioCodeField, initialMarginCollateralPortfolioCode),
			new FieldValueImpl(variationMarginCollateralPortfolioCodeField, variationMarginCollateralPortfolioCode),
			new FieldValueImpl(notionalAmountOfTheContractOrLeg1Field, notionalAmountOfTheContractOrLeg1),
			new FieldValueImpl(notionalCurrencyOfTheContractOrLeg1Field, notionalCurrencyOfTheContractOrLeg1),
			new FieldValueImpl(notionalAmountOfLeg2Field, notionalAmountOfLeg2),
			new FieldValueImpl(notionalCurrencyOfLeg2Field, notionalCurrencyOfLeg2),
			new FieldValueImpl(totalNotionalQuantityOfTheContractOrLeg1Field, totalNotionalQuantityOfTheContractOrLeg1),
			new FieldValueImpl(quantityUnitOfMeasureForTheContractOrLeg1Field, quantityUnitOfMeasureForTheContractOrLeg1),
			new FieldValueImpl(notionalQuantityOfTheContractOrLeg1Field, notionalQuantityOfTheContractOrLeg1),
			new FieldValueImpl(quantityFrequencyOfTheContractOrLeg1Field, quantityFrequencyOfTheContractOrLeg1),
			new FieldValueImpl(quantityFrequencyMultiplierOfTheContractOrLeg1Field, quantityFrequencyMultiplierOfTheContractOrLeg1),
			new FieldValueImpl(totalNotionalQuantityLeg2Field, totalNotionalQuantityLeg2),
			new FieldValueImpl(quantityUnitOfMeasureLeg2Field, quantityUnitOfMeasureLeg2),
			new FieldValueImpl(notionalQuantityOfLeg2Field, notionalQuantityOfLeg2),
			new FieldValueImpl(quantityFrequencyOfLeg2Field, quantityFrequencyOfLeg2),
			new FieldValueImpl(quantityFrequencyMultiplierOfLeg2Field, quantityFrequencyMultiplierOfLeg2),
			new FieldValueImpl(callAmountField, callAmount),
			new FieldValueImpl(callCurrencyField, callCurrency),
			new FieldValueImpl(putAmountField, putAmount),
			new FieldValueImpl(putCurrencyField, putCurrency),
			new FieldValueImpl(priceField, price),
			new FieldValueImpl(priceCurrencyField, priceCurrency),
			new FieldValueImpl(priceNotationField, priceNotation),
			new FieldValueImpl(priceUnitOfMeasureField, priceUnitOfMeasure),
			new MultiNestedFieldValueImpl(priceScheduleField, priceSchedule),
			new FieldValueImpl(strikePriceField, strikePrice),
			new FieldValueImpl(strikePriceCurrencyField, strikePriceCurrency),
			new FieldValueImpl(strikePriceNotationField, strikePriceNotation),
			new MultiNestedFieldValueImpl(strikePriceScheduleField, strikePriceSchedule),
			new FieldValueImpl(optionPremiumAmountField, optionPremiumAmount),
			new FieldValueImpl(optionPremiumCurrencyField, optionPremiumCurrency),
			new FieldValueImpl(exchangeRateField, exchangeRate),
			new FieldValueImpl(exchangeRateBasisField, exchangeRateBasis),
			new FieldValueImpl(fixedRateDayCountConventionOfLeg1Field, fixedRateDayCountConventionOfLeg1),
			new FieldValueImpl(fixedRateDayCountConventionOfLeg2Field, fixedRateDayCountConventionOfLeg2),
			new FieldValueImpl(floatingRateDayCountConventionOfLeg1Field, floatingRateDayCountConventionOfLeg1),
			new FieldValueImpl(floatingRateDayCountConventionOfLeg2Field, floatingRateDayCountConventionOfLeg2),
			new FieldValueImpl(floatingRateResetFrequencyPeriodOfLeg1Field, floatingRateResetFrequencyPeriodOfLeg1),
			new FieldValueImpl(floatingRateResetFrequencyPeriodMultiplierOfLeg1Field, floatingRateResetFrequencyPeriodMultiplierOfLeg1),
			new FieldValueImpl(floatingRateResetFrequencyPeriodOfLeg2Field, floatingRateResetFrequencyPeriodOfLeg2),
			new FieldValueImpl(floatingRateResetFrequencyPeriodMultiplierOfLeg2Field, floatingRateResetFrequencyPeriodMultiplierOfLeg2),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodOfLeg1Field, fixedRatePaymentFrequencyPeriodOfLeg1),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodMultiplierOfLeg1Field, fixedRatePaymentFrequencyPeriodMultiplierOfLeg1),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodOfLeg2Field, fixedRatePaymentFrequencyPeriodOfLeg2),
			new FieldValueImpl(fixedRatePaymentFrequencyPeriodMultiplierOfLeg2Field, fixedRatePaymentFrequencyPeriodMultiplierOfLeg2),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodOfLeg1Field, floatingRatePaymentFrequencyPeriodOfLeg1),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodMultiplierOfLeg1Field, floatingRatePaymentFrequencyPeriodMultiplierOfLeg1),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodOfLeg2Field, floatingRatePaymentFrequencyPeriodOfLeg2),
			new FieldValueImpl(floatingRatePaymentFrequencyPeriodMultiplierOfLeg2Field, floatingRatePaymentFrequencyPeriodMultiplierOfLeg2),
			new MultiNestedFieldValueImpl(otherPaymentField, otherPayment),
			new FieldValueImpl(actionTypeField, actionType),
			new FieldValueImpl(eventTypeField, eventType),
			new FieldValueImpl(eventDateField, eventDate)
		);
	}
}
