package drr.regulation.mas.rewrite.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.mas.rewrite.trade.PriceScheduleReport;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class PriceScheduleReportTypeTabulator implements Tabulator<PriceScheduleReport> {
	private final Field effectiveDateField;
	private final Field endDateField;
	private final Field priceMonetaryField;
	private final Field priceDecimalField;
	
	public PriceScheduleReportTypeTabulator() {
		this.effectiveDateField = new FieldImpl(
			"effectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "UnadjustedEffectiveDateOfThePrice")),
			Optional.of("98/100-$ 98 Unadjusted effective date of the price"),
			Arrays.asList()
		);
		this.endDateField = new FieldImpl(
			"endDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "UnadjustedEndDateOfThePrice")),
			Optional.of("98/100-$ 99 Unadjusted end date of the price"),
			Arrays.asList()
		);
		this.priceMonetaryField = new FieldImpl(
			"priceMonetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "PriceInEffectBetweenTheUnadjustedEffectiveDateAndEndDateMonetary")),
			Optional.of("98/100-$ 100 Price in effect between the unadjusted effective and end date (Monetary)"),
			Arrays.asList()
		);
		this.priceDecimalField = new FieldImpl(
			"priceDecimal",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "PriceInEffectBetweenTheUnadjustedEffectiveDateAndEndDateDecimal")),
			Optional.of("98/100-$ 100 Price in effect between the unadjusted effective and end date (Decimal)"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(effectiveDateField, endDateField, priceMonetaryField, priceDecimalField);
	}
	
	@Override
	public List<FieldValue> tabulate(PriceScheduleReport input) {
		Optional<Date> effectiveDate = Optional.ofNullable(input.getEffectiveDate());
		Optional<Date> endDate = Optional.ofNullable(input.getEndDate());
		Optional<BigDecimal> priceMonetary = Optional.ofNullable(input.getPriceMonetary());
		Optional<BigDecimal> priceDecimal = Optional.ofNullable(input.getPriceDecimal());
		return Arrays.asList(
			new FieldValueImpl(effectiveDateField, effectiveDate),
			new FieldValueImpl(endDateField, endDate),
			new FieldValueImpl(priceMonetaryField, priceMonetary),
			new FieldValueImpl(priceDecimalField, priceDecimal)
		);
	}
}
