package drr.regulation.mas.rewrite.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.mas.rewrite.trade.SpreadOfLeg1;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class SpreadOfLeg1TypeTabulator implements Tabulator<SpreadOfLeg1> {
	private final Field spreadOfLeg1MonetaryField;
	private final Field spreadOfLeg1DecimalField;
	private final Field spreadOfLeg1BasisField;
	private final Field spreadNotationOfLeg1Field;
	private final Field spreadCurrencyOfLeg1Field;
	
	public SpreadOfLeg1TypeTabulator() {
		this.spreadOfLeg1MonetaryField = new FieldImpl(
			"spreadOfLeg1Monetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "SpreadOfLeg1Monetary")),
			Optional.of("19 Spread of Leg 1 (Monetary)"),
			Arrays.asList()
		);
		this.spreadOfLeg1DecimalField = new FieldImpl(
			"spreadOfLeg1Decimal",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "SpreadOfLeg1Decimal")),
			Optional.of("19 Spread of Leg 1 (Percentage)"),
			Arrays.asList()
		);
		this.spreadOfLeg1BasisField = new FieldImpl(
			"spreadOfLeg1Basis",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "SpreadOfLeg1Basis")),
			Optional.of("19 Spread of Leg 1 (Basis)"),
			Arrays.asList()
		);
		this.spreadNotationOfLeg1Field = new FieldImpl(
			"spreadNotationOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "SpreadNotationOfLeg1")),
			Optional.of("19 Spread Notation of Leg 1"),
			Arrays.asList()
		);
		this.spreadCurrencyOfLeg1Field = new FieldImpl(
			"spreadCurrencyOfLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "SpreadCurrencyOfLeg1")),
			Optional.of("2.94 Spread Currency of Leg 1"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(spreadOfLeg1MonetaryField, spreadOfLeg1DecimalField, spreadOfLeg1BasisField, spreadNotationOfLeg1Field, spreadCurrencyOfLeg1Field);
	}
	
	@Override
	public List<FieldValue> tabulate(SpreadOfLeg1 input) {
		Optional<BigDecimal> spreadOfLeg1Monetary = Optional.ofNullable(input.getSpreadOfLeg1Monetary());
		Optional<BigDecimal> spreadOfLeg1Decimal = Optional.ofNullable(input.getSpreadOfLeg1Decimal());
		Optional<BigDecimal> spreadOfLeg1Basis = Optional.ofNullable(input.getSpreadOfLeg1Basis());
		Optional<String> spreadNotationOfLeg1 = Optional.ofNullable(input.getSpreadNotationOfLeg1());
		Optional<String> spreadCurrencyOfLeg1 = Optional.ofNullable(input.getSpreadCurrencyOfLeg1());
		return Arrays.asList(
			new FieldValueImpl(spreadOfLeg1MonetaryField, spreadOfLeg1Monetary),
			new FieldValueImpl(spreadOfLeg1DecimalField, spreadOfLeg1Decimal),
			new FieldValueImpl(spreadOfLeg1BasisField, spreadOfLeg1Basis),
			new FieldValueImpl(spreadNotationOfLeg1Field, spreadNotationOfLeg1),
			new FieldValueImpl(spreadCurrencyOfLeg1Field, spreadCurrencyOfLeg1)
		);
	}
}
