package drr.regulation.mas.rewrite.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.mas.rewrite.trade.SpreadOfLeg2;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class SpreadOfLeg2TypeTabulator implements Tabulator<SpreadOfLeg2> {
	private final Field spreadOfLeg2MonetaryField;
	private final Field spreadOfLeg2DecimalField;
	private final Field spreadOfLeg2BasisField;
	private final Field spreadNotationOfLeg2Field;
	private final Field spreadCurrencyOfLeg2Field;
	
	public SpreadOfLeg2TypeTabulator() {
		this.spreadOfLeg2MonetaryField = new FieldImpl(
			"spreadOfLeg2Monetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "SpreadOfLeg2Monetary")),
			Optional.of("25 Spread of Leg 2 (Monetary)"),
			Arrays.asList()
		);
		this.spreadOfLeg2DecimalField = new FieldImpl(
			"spreadOfLeg2Decimal",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "SpreadOfLeg2Decimal")),
			Optional.of("25 Spread of Leg 2 (Percentage)"),
			Arrays.asList()
		);
		this.spreadOfLeg2BasisField = new FieldImpl(
			"spreadOfLeg2Basis",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "SpreadOfLeg2Basis")),
			Optional.of("25 Spread of Leg 2 (Basis)"),
			Arrays.asList()
		);
		this.spreadNotationOfLeg2Field = new FieldImpl(
			"spreadNotationOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "SpreadNotationOfLeg2")),
			Optional.of("26 Spread notation of leg 2"),
			Arrays.asList()
		);
		this.spreadCurrencyOfLeg2Field = new FieldImpl(
			"spreadCurrencyOfLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "SpreadCurrencyOfLeg2")),
			Optional.of("27 Spread Currency of Leg 2"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(spreadOfLeg2MonetaryField, spreadOfLeg2DecimalField, spreadOfLeg2BasisField, spreadNotationOfLeg2Field, spreadCurrencyOfLeg2Field);
	}
	
	@Override
	public List<FieldValue> tabulate(SpreadOfLeg2 input) {
		Optional<BigDecimal> spreadOfLeg2Monetary = Optional.ofNullable(input.getSpreadOfLeg2Monetary());
		Optional<BigDecimal> spreadOfLeg2Decimal = Optional.ofNullable(input.getSpreadOfLeg2Decimal());
		Optional<BigDecimal> spreadOfLeg2Basis = Optional.ofNullable(input.getSpreadOfLeg2Basis());
		Optional<String> spreadNotationOfLeg2 = Optional.ofNullable(input.getSpreadNotationOfLeg2());
		Optional<String> spreadCurrencyOfLeg2 = Optional.ofNullable(input.getSpreadCurrencyOfLeg2());
		return Arrays.asList(
			new FieldValueImpl(spreadOfLeg2MonetaryField, spreadOfLeg2Monetary),
			new FieldValueImpl(spreadOfLeg2DecimalField, spreadOfLeg2Decimal),
			new FieldValueImpl(spreadOfLeg2BasisField, spreadOfLeg2Basis),
			new FieldValueImpl(spreadNotationOfLeg2Field, spreadNotationOfLeg2),
			new FieldValueImpl(spreadCurrencyOfLeg2Field, spreadCurrencyOfLeg2)
		);
	}
}
