package drr.regulation.mas.rewrite.trade.reports;

import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.mas.rewrite.trade.StrikePriceScheduleReport;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class StrikePriceScheduleReportTypeTabulator implements Tabulator<StrikePriceScheduleReport> {
	private final Field effectiveDateField;
	private final Field endDateField;
	private final Field priceMonetaryField;
	private final Field priceDecimalField;
	private final Field priceNotationField;
	
	public StrikePriceScheduleReportTypeTabulator() {
		this.effectiveDateField = new FieldImpl(
			"effectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "EffectiveDateOfStrikePrice")),
			Optional.of("104/106-$ 104 Effective date of strike price"),
			Arrays.asList()
		);
		this.endDateField = new FieldImpl(
			"endDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "EndDateOfStrikePrice")),
			Optional.of("104/106-$ 105 End date of strike price"),
			Arrays.asList()
		);
		this.priceMonetaryField = new FieldImpl(
			"priceMonetary",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "StrikePriceInEffectOnAssociatedEffectiveDateMonetary")),
			Optional.of("104/106-$ 106 Strike price in effect on associated effective date (Monetary)"),
			Arrays.asList()
		);
		this.priceDecimalField = new FieldImpl(
			"priceDecimal",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "StrikePriceInEffectOnAssociatedEffectiveDateDecimal")),
			Optional.of("104/106-$ 106 Strike price in effect on associated effective date (Decimal)"),
			Arrays.asList()
		);
		this.priceNotationField = new FieldImpl(
			"priceNotation",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "mas", "rewrite", "trade"), "StrikePriceNotationInEffectOnAssociatedEffectiveDate")),
			Optional.of("104/106-$ 106 Strike price notation in effect on associated effective date (Non Reportable)"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(effectiveDateField, endDateField, priceMonetaryField, priceDecimalField, priceNotationField);
	}
	
	@Override
	public List<FieldValue> tabulate(StrikePriceScheduleReport input) {
		Optional<Date> effectiveDate = Optional.ofNullable(input.getEffectiveDate());
		Optional<Date> endDate = Optional.ofNullable(input.getEndDate());
		Optional<BigDecimal> priceMonetary = Optional.ofNullable(input.getPriceMonetary());
		Optional<BigDecimal> priceDecimal = Optional.ofNullable(input.getPriceDecimal());
		Optional<PriceNotationEnum> priceNotation = Optional.ofNullable(input.getPriceNotation());
		return Arrays.asList(
			new FieldValueImpl(effectiveDateField, effectiveDate),
			new FieldValueImpl(endDateField, endDate),
			new FieldValueImpl(priceMonetaryField, priceMonetary),
			new FieldValueImpl(priceDecimalField, priceDecimal),
			new FieldValueImpl(priceNotationField, priceNotation)
		);
	}
}
