package drr.regulation.mas.rewrite.trade.reports;

import cdm.base.staticdata.party.Party;
import cdm.base.staticdata.party.metafields.ReferenceWithMetaParty;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperListOfLists;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.PartyInformation;
import drr.regulation.common.ReportableInformation;
import drr.regulation.common.ReportingRegime;
import drr.regulation.common.ReportingRoleEnum;
import drr.regulation.common.ReportingSide;
import drr.regulation.common.SupervisoryBodyEnum;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.IsAllowableAction;
import drr.regulation.common.metafields.FieldWithMetaSupervisoryBodyEnum;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(TradingCapacityOfSpecifiedPersonRule.TradingCapacityOfSpecifiedPersonRuleDefault.class)
public abstract class TradingCapacityOfSpecifiedPersonRule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected IsAllowableAction isAllowableAction;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class TradingCapacityOfSpecifiedPersonRuleDefault extends TradingCapacityOfSpecifiedPersonRule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult = MapperS.of(input)
				.filterSingleNullSafe(item -> isAllowableAction.evaluate(item.get()));
			output = thenResult
				.mapSingleToItem(reportInstruction -> {
					final MapperC<PartyInformation> thenResult0 = thenResult.<ReportableInformation>map("getReportableInformation", reportableEvent -> reportableEvent.getReportableInformation()).<PartyInformation>mapC("getPartyInformation", reportableInformation -> reportableInformation.getPartyInformation());
					final MapperC<PartyInformation> thenResult1 = thenResult0
						.filterItemNullSafe(item -> areEqual(item.<ReferenceWithMetaParty>map("getPartyReference", partyInformation -> partyInformation.getPartyReference()).<Party>map("getValue", _f->_f.getValue()), reportInstruction.<ReportingSide>map("getReportingSide", transactionReportInstruction -> transactionReportInstruction.getReportingSide()).<ReferenceWithMetaParty>map("getReportingParty", reportingSide -> reportingSide.getReportingParty()).<Party>map("getValue", _f->_f.getValue()), CardinalityOperator.All).get());
					final MapperListOfLists<ReportingRegime> thenResult2 = thenResult1
						.mapItemToList(item -> item.<ReportingRegime>mapC("getRegimeInformation", partyInformation -> partyInformation.getRegimeInformation()));
					final MapperC<ReportingRegime> thenResult3 = thenResult2
						.flattenList();
					final MapperC<ReportingRegime> thenResult4 = thenResult3
						.filterItemNullSafe(item -> areEqual(item.<FieldWithMetaSupervisoryBodyEnum>map("getSupervisoryBody", reportingRegime -> reportingRegime.getSupervisoryBody()).<SupervisoryBodyEnum>map("getValue", _f->_f.getValue()), MapperS.of(SupervisoryBodyEnum.MAS), CardinalityOperator.All).get());
					final MapperS<ReportingRegime> thenResult5 = MapperS.of(thenResult4.get());
					return thenResult5
						.mapSingleToItem(item -> {
							if (areEqual(item.<ReportingRoleEnum>map("getReportingRole", reportingRegime -> reportingRegime.getReportingRole()), MapperS.of(ReportingRoleEnum.AGENT), CardinalityOperator.All).getOrDefault(false)) {
								return MapperS.of("Agent");
							}
							if (areEqual(item.<ReportingRoleEnum>map("getReportingRole", reportingRegime -> reportingRegime.getReportingRole()), MapperS.of(ReportingRoleEnum.PRINCIPAL), CardinalityOperator.All).getOrDefault(false)) {
								return MapperS.of("Principal");
							}
							return MapperS.<String>ofNull();
						});
				}).get();
			
			return output;
		}
	}
}
