package drr.regulation.mas.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASTransactionReportDTCC_MAS_BR_0007_01")
@ImplementedBy(MASTransactionReportDTCCMASBR000701.Default.class)
public interface MASTransactionReportDTCCMASBR000701 extends Validator<MASTransactionReport> {
	
	String NAME = "MASTransactionReportDTCC_MAS_BR_0007_01";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType and directionOfLeg1 exists then directionOfLeg2 exists and directionOfLeg1 <> directionOfLeg2 else if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType and direction is absent and directionOfLeg1 is absent then directionOfLeg2 is absent";
	
	ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport);
	
	class Default implements MASTransactionReportDTCCMASBR000701 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			ComparisonResult result = executeDataRule(mASTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASTransactionReport mASTransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), CardinalityOperator.Any).and(exists(MapperS.of(mASTransactionReport).<String>map("getDirectionOfLeg1", _mASTransactionReport -> _mASTransactionReport.getDirectionOfLeg1()))).getOrDefault(false)) {
					return exists(MapperS.of(mASTransactionReport).<String>map("getDirectionOfLeg2", _mASTransactionReport -> _mASTransactionReport.getDirectionOfLeg2())).and(notEqual(MapperS.of(mASTransactionReport).<String>map("getDirectionOfLeg1", _mASTransactionReport -> _mASTransactionReport.getDirectionOfLeg1()), MapperS.of(mASTransactionReport).<String>map("getDirectionOfLeg2", _mASTransactionReport -> _mASTransactionReport.getDirectionOfLeg2()), CardinalityOperator.Any));
				}
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), CardinalityOperator.Any).and(notExists(MapperS.of(mASTransactionReport).<String>map("getDirection", _mASTransactionReport -> _mASTransactionReport.getDirection()))).and(notExists(MapperS.of(mASTransactionReport).<String>map("getDirectionOfLeg1", _mASTransactionReport -> _mASTransactionReport.getDirectionOfLeg1()))).getOrDefault(false)) {
					return notExists(MapperS.of(mASTransactionReport).<String>map("getDirectionOfLeg2", _mASTransactionReport -> _mASTransactionReport.getDirectionOfLeg2()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASTransactionReportDTCCMASBR000701 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
		}
	}
}
