package drr.regulation.mas.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASTransactionReportDTCC_MAS_BR_0012_01")
@ImplementedBy(MASTransactionReportDTCCMASBR001201.Default.class)
public interface MASTransactionReportDTCCMASBR001201 extends Validator<MASTransactionReport> {
	
	String NAME = "MASTransactionReportDTCC_MAS_BR_0012_01";
	String DEFINITION = "if actionType = \"NEWT\" and eventType = \"ALOC\" then priorUTI exists and priorUTI <> uniqueTransactionIdentifier else if actionType = \"NEWT\" and eventType = \"TRAD\" then priorUTI is absent else if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType and priorUTI exists then priorUTI <> uniqueTransactionIdentifier";
	
	ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport);
	
	class Default implements MASTransactionReportDTCCMASBR001201 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			ComparisonResult result = executeDataRule(mASTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASTransactionReport mASTransactionReport) {
			try {
				if (areEqual(MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), MapperS.of("NEWT"), CardinalityOperator.All).and(areEqual(MapperS.of(mASTransactionReport).<String>map("getEventType", _mASTransactionReport -> _mASTransactionReport.getEventType()), MapperS.of("ALOC"), CardinalityOperator.All)).getOrDefault(false)) {
					return exists(MapperS.of(mASTransactionReport).<String>map("getPriorUTI", _mASTransactionReport -> _mASTransactionReport.getPriorUTI())).and(notEqual(MapperS.of(mASTransactionReport).<String>map("getPriorUTI", _mASTransactionReport -> _mASTransactionReport.getPriorUTI()), MapperS.of(mASTransactionReport).<String>map("getUniqueTransactionIdentifier", _mASTransactionReport -> _mASTransactionReport.getUniqueTransactionIdentifier()), CardinalityOperator.Any));
				}
				if (areEqual(MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), MapperS.of("NEWT"), CardinalityOperator.All).and(areEqual(MapperS.of(mASTransactionReport).<String>map("getEventType", _mASTransactionReport -> _mASTransactionReport.getEventType()), MapperS.of("TRAD"), CardinalityOperator.All)).getOrDefault(false)) {
					return notExists(MapperS.of(mASTransactionReport).<String>map("getPriorUTI", _mASTransactionReport -> _mASTransactionReport.getPriorUTI()));
				}
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), CardinalityOperator.Any).and(exists(MapperS.of(mASTransactionReport).<String>map("getPriorUTI", _mASTransactionReport -> _mASTransactionReport.getPriorUTI()))).getOrDefault(false)) {
					return notEqual(MapperS.of(mASTransactionReport).<String>map("getPriorUTI", _mASTransactionReport -> _mASTransactionReport.getPriorUTI()), MapperS.of(mASTransactionReport).<String>map("getUniqueTransactionIdentifier", _mASTransactionReport -> _mASTransactionReport.getUniqueTransactionIdentifier()), CardinalityOperator.Any);
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASTransactionReportDTCCMASBR001201 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
		}
	}
}
