package drr.regulation.mas.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;
import drr.regulation.mas.rewrite.trade.SpreadOfLeg1;
import drr.regulation.mas.rewrite.trade.SpreadOfLeg2;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASTransactionReportDTCC_MAS_BR_0019_01")
@ImplementedBy(MASTransactionReportDTCCMASBR001901.Default.class)
public interface MASTransactionReportDTCCMASBR001901 extends Validator<MASTransactionReport> {
	
	String NAME = "MASTransactionReportDTCC_MAS_BR_0019_01";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType and fixedRateOfLeg1 is absent and fixedRateOfLeg2 is absent and spreadOfLeg2 is absent then (spreadOfLeg1 -> spreadOfLeg1Monetary exists or spreadOfLeg1 -> spreadOfLeg1Decimal exists or spreadOfLeg1 -> spreadOfLeg1Basis exists) else if fixedRateOfLeg1 exists then (spreadOfLeg1 -> spreadOfLeg1Monetary is absent and spreadOfLeg1 -> spreadOfLeg1Decimal is absent and spreadOfLeg1 -> spreadOfLeg1Basis is absent)";
	
	ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport);
	
	class Default implements MASTransactionReportDTCCMASBR001901 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			ComparisonResult result = executeDataRule(mASTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASTransactionReport mASTransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), CardinalityOperator.Any).and(notExists(MapperS.of(mASTransactionReport).<BigDecimal>map("getFixedRateOfLeg1", _mASTransactionReport -> _mASTransactionReport.getFixedRateOfLeg1()))).and(notExists(MapperS.of(mASTransactionReport).<BigDecimal>map("getFixedRateOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFixedRateOfLeg2()))).and(notExists(MapperS.of(mASTransactionReport).<SpreadOfLeg2>map("getSpreadOfLeg2", _mASTransactionReport -> _mASTransactionReport.getSpreadOfLeg2()))).getOrDefault(false)) {
					return exists(MapperS.of(mASTransactionReport).<SpreadOfLeg1>map("getSpreadOfLeg1", _mASTransactionReport -> _mASTransactionReport.getSpreadOfLeg1()).<BigDecimal>map("getSpreadOfLeg1Monetary", spreadOfLeg1 -> spreadOfLeg1.getSpreadOfLeg1Monetary())).or(exists(MapperS.of(mASTransactionReport).<SpreadOfLeg1>map("getSpreadOfLeg1", _mASTransactionReport -> _mASTransactionReport.getSpreadOfLeg1()).<BigDecimal>map("getSpreadOfLeg1Decimal", spreadOfLeg1 -> spreadOfLeg1.getSpreadOfLeg1Decimal()))).or(exists(MapperS.of(mASTransactionReport).<SpreadOfLeg1>map("getSpreadOfLeg1", _mASTransactionReport -> _mASTransactionReport.getSpreadOfLeg1()).<BigDecimal>map("getSpreadOfLeg1Basis", spreadOfLeg1 -> spreadOfLeg1.getSpreadOfLeg1Basis())));
				}
				if (exists(MapperS.of(mASTransactionReport).<BigDecimal>map("getFixedRateOfLeg1", _mASTransactionReport -> _mASTransactionReport.getFixedRateOfLeg1())).getOrDefault(false)) {
					return notExists(MapperS.of(mASTransactionReport).<SpreadOfLeg1>map("getSpreadOfLeg1", _mASTransactionReport -> _mASTransactionReport.getSpreadOfLeg1()).<BigDecimal>map("getSpreadOfLeg1Monetary", spreadOfLeg1 -> spreadOfLeg1.getSpreadOfLeg1Monetary())).and(notExists(MapperS.of(mASTransactionReport).<SpreadOfLeg1>map("getSpreadOfLeg1", _mASTransactionReport -> _mASTransactionReport.getSpreadOfLeg1()).<BigDecimal>map("getSpreadOfLeg1Decimal", spreadOfLeg1 -> spreadOfLeg1.getSpreadOfLeg1Decimal()))).and(notExists(MapperS.of(mASTransactionReport).<SpreadOfLeg1>map("getSpreadOfLeg1", _mASTransactionReport -> _mASTransactionReport.getSpreadOfLeg1()).<BigDecimal>map("getSpreadOfLeg1Basis", spreadOfLeg1 -> spreadOfLeg1.getSpreadOfLeg1Basis())));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASTransactionReportDTCCMASBR001901 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
		}
	}
}
