package drr.regulation.mas.rewrite.trade.validation.datarule;

import cdm.base.staticdata.asset.common.ProductIdTypeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.mas.rewrite.trade.BasketConstituentsReport;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASTransactionReportDTCC_MAS_BR_0052_01")
@ImplementedBy(MASTransactionReportDTCCMASBR005201.Default.class)
public interface MASTransactionReportDTCCMASBR005201 extends Validator<MASTransactionReport> {
	
	String NAME = "MASTransactionReportDTCC_MAS_BR_0052_01";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType then if basketConstituents -> source any <> ProductIdTypeEnum -> ISIN then basketConstituents -> source exists";
	
	ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport);
	
	class Default implements MASTransactionReportDTCCMASBR005201 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			ComparisonResult result = executeDataRule(mASTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASTransactionReport mASTransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), CardinalityOperator.Any).getOrDefault(false)) {
					if (notEqual(MapperS.of(mASTransactionReport).<BasketConstituentsReport>mapC("getBasketConstituents", _mASTransactionReport -> _mASTransactionReport.getBasketConstituents()).<ProductIdTypeEnum>map("getSource", basketConstituentsReport -> basketConstituentsReport.getSource()), MapperS.of(ProductIdTypeEnum.ISIN), CardinalityOperator.Any).getOrDefault(false)) {
						return exists(MapperS.of(mASTransactionReport).<BasketConstituentsReport>mapC("getBasketConstituents", _mASTransactionReport -> _mASTransactionReport.getBasketConstituents()).<ProductIdTypeEnum>map("getSource", basketConstituentsReport -> basketConstituentsReport.getSource()));
					}
					return ComparisonResult.successEmptyOperand("");
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASTransactionReportDTCCMASBR005201 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
		}
	}
}
