package drr.regulation.mas.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;
import java.math.BigDecimal;
import java.math.BigInteger;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASTransactionReportDTCC_MAS_BR_0097_01")
@ImplementedBy(MASTransactionReportDTCCMASBR009701.Default.class)
public interface MASTransactionReportDTCCMASBR009701 extends Validator<MASTransactionReport> {
	
	String NAME = "MASTransactionReportDTCC_MAS_BR_0097_01";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType and price exists and [999999999999999999, 9999999999999] all <> price then priceUnitOfMeasure exists";
	
	ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport);
	
	class Default implements MASTransactionReportDTCCMASBR009701 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			ComparisonResult result = executeDataRule(mASTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASTransactionReport mASTransactionReport) {
			try {
				final BigInteger bigInteger0 = new BigInteger("999999999999999999");
				final BigInteger bigInteger1 = new BigInteger("9999999999999");
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), CardinalityOperator.Any).and(exists(MapperS.of(mASTransactionReport).<BigDecimal>map("getPrice", _mASTransactionReport -> _mASTransactionReport.getPrice()))).and(notEqual(MapperC.<Long>of((bigInteger0 == null ? MapperS.<Long>ofNull() : MapperS.of(bigInteger0.longValueExact())), (bigInteger1 == null ? MapperS.<Long>ofNull() : MapperS.of(bigInteger1.longValueExact()))).<BigDecimal>map("Type coercion", _long -> BigDecimal.valueOf(_long)), MapperS.of(mASTransactionReport).<BigDecimal>map("getPrice", _mASTransactionReport -> _mASTransactionReport.getPrice()), CardinalityOperator.All)).getOrDefault(false)) {
					return exists(MapperS.of(mASTransactionReport).<String>map("getPriceUnitOfMeasure", _mASTransactionReport -> _mASTransactionReport.getPriceUnitOfMeasure()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASTransactionReportDTCCMASBR009701 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
		}
	}
}
