package drr.regulation.mas.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;
import drr.regulation.mas.rewrite.trade.PriceScheduleReport;
import java.math.BigDecimal;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASTransactionReportDTCC_MAS_BR_0100_01")
@ImplementedBy(MASTransactionReportDTCCMASBR010001.Default.class)
public interface MASTransactionReportDTCCMASBR010001 extends Validator<MASTransactionReport> {
	
	String NAME = "MASTransactionReportDTCC_MAS_BR_0100_01";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType then ((priceSchedule -> effectiveDate count = priceSchedule -> priceMonetary count or priceSchedule -> effectiveDate count = priceSchedule -> priceDecimal count) and (priceSchedule -> endDate count = priceSchedule -> priceMonetary count or priceSchedule -> endDate count = priceSchedule -> priceDecimal count))";
	
	ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport);
	
	class Default implements MASTransactionReportDTCCMASBR010001 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			ComparisonResult result = executeDataRule(mASTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASTransactionReport mASTransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), CardinalityOperator.Any).getOrDefault(false)) {
					return areEqual(MapperS.of(MapperS.of(mASTransactionReport).<PriceScheduleReport>mapC("getPriceSchedule", _mASTransactionReport -> _mASTransactionReport.getPriceSchedule()).<Date>map("getEffectiveDate", priceScheduleReport -> priceScheduleReport.getEffectiveDate()).resultCount()), MapperS.of(MapperS.of(mASTransactionReport).<PriceScheduleReport>mapC("getPriceSchedule", _mASTransactionReport -> _mASTransactionReport.getPriceSchedule()).<BigDecimal>map("getPriceMonetary", priceScheduleReport -> priceScheduleReport.getPriceMonetary()).resultCount()), CardinalityOperator.All).or(areEqual(MapperS.of(MapperS.of(mASTransactionReport).<PriceScheduleReport>mapC("getPriceSchedule", _mASTransactionReport -> _mASTransactionReport.getPriceSchedule()).<Date>map("getEffectiveDate", priceScheduleReport -> priceScheduleReport.getEffectiveDate()).resultCount()), MapperS.of(MapperS.of(mASTransactionReport).<PriceScheduleReport>mapC("getPriceSchedule", _mASTransactionReport -> _mASTransactionReport.getPriceSchedule()).<BigDecimal>map("getPriceDecimal", priceScheduleReport -> priceScheduleReport.getPriceDecimal()).resultCount()), CardinalityOperator.All)).and(areEqual(MapperS.of(MapperS.of(mASTransactionReport).<PriceScheduleReport>mapC("getPriceSchedule", _mASTransactionReport -> _mASTransactionReport.getPriceSchedule()).<Date>map("getEndDate", priceScheduleReport -> priceScheduleReport.getEndDate()).resultCount()), MapperS.of(MapperS.of(mASTransactionReport).<PriceScheduleReport>mapC("getPriceSchedule", _mASTransactionReport -> _mASTransactionReport.getPriceSchedule()).<BigDecimal>map("getPriceMonetary", priceScheduleReport -> priceScheduleReport.getPriceMonetary()).resultCount()), CardinalityOperator.All).or(areEqual(MapperS.of(MapperS.of(mASTransactionReport).<PriceScheduleReport>mapC("getPriceSchedule", _mASTransactionReport -> _mASTransactionReport.getPriceSchedule()).<Date>map("getEndDate", priceScheduleReport -> priceScheduleReport.getEndDate()).resultCount()), MapperS.of(MapperS.of(mASTransactionReport).<PriceScheduleReport>mapC("getPriceSchedule", _mASTransactionReport -> _mASTransactionReport.getPriceSchedule()).<BigDecimal>map("getPriceDecimal", priceScheduleReport -> priceScheduleReport.getPriceDecimal()).resultCount()), CardinalityOperator.All)));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASTransactionReportDTCCMASBR010001 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
		}
	}
}
