package drr.regulation.mas.rewrite.trade.validation.datarule;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.common.util.functions.SubString;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASTransactionReportDTCC_MAS_BR_0110_01")
@ImplementedBy(MASTransactionReportDTCCMASBR011001.Default.class)
public interface MASTransactionReportDTCCMASBR011001 extends Validator<MASTransactionReport> {
	
	String NAME = "MASTransactionReportDTCC_MAS_BR_0110_01";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType then (SubString(exchangeRateBasis, 1, 3) = notionalCurrencyOfTheContractOrLeg1 to-string or SubString(exchangeRateBasis, 1, 3) = notionalCurrencyOfLeg2 to-string) and (SubString(exchangeRateBasis, 1, 3) <> SubString(exchangeRateBasis, 5, 3)) and ([\"XAG\", \"XAU\", \"XBA\", \"XBB\", \"XBC\", \"XBD\", \"XDR\", \"XEU\", \"XFU\", \"XPD\", \"XPT\", \"XXX\"] all <> SubString(exchangeRateBasis, 1, 3))";
	
	ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport);
	
	class Default implements MASTransactionReportDTCCMASBR011001 {
	
		@Inject protected SubString subString;
		
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			ComparisonResult result = executeDataRule(mASTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASTransactionReport mASTransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), CardinalityOperator.Any).getOrDefault(false)) {
					return areEqual(MapperS.of(subString.evaluate(MapperS.of(mASTransactionReport).<String>map("getExchangeRateBasis", _mASTransactionReport -> _mASTransactionReport.getExchangeRateBasis()).get(), 1, 3)), MapperS.of(mASTransactionReport).<ISOCurrencyCodeEnum>map("getNotionalCurrencyOfTheContractOrLeg1", _mASTransactionReport -> _mASTransactionReport.getNotionalCurrencyOfTheContractOrLeg1()).map("to-string", ISOCurrencyCodeEnum::toDisplayString), CardinalityOperator.All).or(areEqual(MapperS.of(subString.evaluate(MapperS.of(mASTransactionReport).<String>map("getExchangeRateBasis", _mASTransactionReport -> _mASTransactionReport.getExchangeRateBasis()).get(), 1, 3)), MapperS.of(mASTransactionReport).<ISOCurrencyCodeEnum>map("getNotionalCurrencyOfLeg2", _mASTransactionReport -> _mASTransactionReport.getNotionalCurrencyOfLeg2()).map("to-string", ISOCurrencyCodeEnum::toDisplayString), CardinalityOperator.All)).and(notEqual(MapperS.of(subString.evaluate(MapperS.of(mASTransactionReport).<String>map("getExchangeRateBasis", _mASTransactionReport -> _mASTransactionReport.getExchangeRateBasis()).get(), 1, 3)), MapperS.of(subString.evaluate(MapperS.of(mASTransactionReport).<String>map("getExchangeRateBasis", _mASTransactionReport -> _mASTransactionReport.getExchangeRateBasis()).get(), 5, 3)), CardinalityOperator.Any)).and(notEqual(MapperC.<String>of(MapperS.of("XAG"), MapperS.of("XAU"), MapperS.of("XBA"), MapperS.of("XBB"), MapperS.of("XBC"), MapperS.of("XBD"), MapperS.of("XDR"), MapperS.of("XEU"), MapperS.of("XFU"), MapperS.of("XPD"), MapperS.of("XPT"), MapperS.of("XXX")), MapperS.of(subString.evaluate(MapperS.of(mASTransactionReport).<String>map("getExchangeRateBasis", _mASTransactionReport -> _mASTransactionReport.getExchangeRateBasis()).get(), 1, 3)), CardinalityOperator.All));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASTransactionReportDTCCMASBR011001 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
		}
	}
}
