package drr.regulation.mas.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASTransactionReportDTCC_MAS_BR_0116_01")
@ImplementedBy(MASTransactionReportDTCCMASBR011601.Default.class)
public interface MASTransactionReportDTCCMASBR011601 extends Validator<MASTransactionReport> {
	
	String NAME = "MASTransactionReportDTCC_MAS_BR_0116_01";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType and floatingRateResetFrequencyPeriodOfLeg1 exists and [\"YEAR\", \"MNTH\", \"WEEK\", \"DAIL\", \"QURT\", \"EXPI\"] any = floatingRateResetFrequencyPeriodOfLeg1 then floatingRateResetFrequencyPeriodMultiplierOfLeg1 exists and floatingRateResetFrequencyPeriodMultiplierOfLeg1 > 0";
	
	ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport);
	
	class Default implements MASTransactionReportDTCCMASBR011601 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			ComparisonResult result = executeDataRule(mASTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASTransactionReport mASTransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), CardinalityOperator.Any).and(exists(MapperS.of(mASTransactionReport).<String>map("getFloatingRateResetFrequencyPeriodOfLeg1", _mASTransactionReport -> _mASTransactionReport.getFloatingRateResetFrequencyPeriodOfLeg1()))).and(areEqual(MapperC.<String>of(MapperS.of("YEAR"), MapperS.of("MNTH"), MapperS.of("WEEK"), MapperS.of("DAIL"), MapperS.of("QURT"), MapperS.of("EXPI")), MapperS.of(mASTransactionReport).<String>map("getFloatingRateResetFrequencyPeriodOfLeg1", _mASTransactionReport -> _mASTransactionReport.getFloatingRateResetFrequencyPeriodOfLeg1()), CardinalityOperator.Any)).getOrDefault(false)) {
					return exists(MapperS.of(mASTransactionReport).<Integer>map("getFloatingRateResetFrequencyPeriodMultiplierOfLeg1", _mASTransactionReport -> _mASTransactionReport.getFloatingRateResetFrequencyPeriodMultiplierOfLeg1())).and(greaterThan(MapperS.of(mASTransactionReport).<Integer>map("getFloatingRateResetFrequencyPeriodMultiplierOfLeg1", _mASTransactionReport -> _mASTransactionReport.getFloatingRateResetFrequencyPeriodMultiplierOfLeg1()), MapperS.of(0), CardinalityOperator.All));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASTransactionReportDTCCMASBR011601 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
		}
	}
}
