package drr.regulation.mas.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASTransactionReportDTCC_MAS_BR_0122_01")
@ImplementedBy(MASTransactionReportDTCCMASBR012201.Default.class)
public interface MASTransactionReportDTCCMASBR012201 extends Validator<MASTransactionReport> {
	
	String NAME = "MASTransactionReportDTCC_MAS_BR_0122_01";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType and fixedRatePaymentFrequencyPeriodOfLeg2 exists and [\"YEAR\", \"MNTH\", \"WEEK\", \"DAIL\", \"EXPI\"] any = fixedRatePaymentFrequencyPeriodOfLeg2 then (fixedRatePaymentFrequencyPeriodMultiplierOfLeg2 exists and fixedRatePaymentFrequencyPeriodMultiplierOfLeg2 > 0 and fixedRatePaymentFrequencyPeriodMultiplierOfLeg2 < 1000 and if fixedRatePaymentFrequencyPeriodOfLeg2 = \"EXPI\" then fixedRatePaymentFrequencyPeriodMultiplierOfLeg2 = 1) else if fixedRatePaymentFrequencyPeriodOfLeg2 is absent or fixedRatePaymentFrequencyPeriodOfLeg2 = \"ADHO\" then fixedRatePaymentFrequencyPeriodMultiplierOfLeg2 is absent";
	
	ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport);
	
	class Default implements MASTransactionReportDTCCMASBR012201 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			ComparisonResult result = executeDataRule(mASTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASTransactionReport mASTransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), CardinalityOperator.Any).and(exists(MapperS.of(mASTransactionReport).<String>map("getFixedRatePaymentFrequencyPeriodOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFixedRatePaymentFrequencyPeriodOfLeg2()))).and(areEqual(MapperC.<String>of(MapperS.of("YEAR"), MapperS.of("MNTH"), MapperS.of("WEEK"), MapperS.of("DAIL"), MapperS.of("EXPI")), MapperS.of(mASTransactionReport).<String>map("getFixedRatePaymentFrequencyPeriodOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFixedRatePaymentFrequencyPeriodOfLeg2()), CardinalityOperator.Any)).getOrDefault(false)) {
					final ComparisonResult ifThenElseResult;
					if (areEqual(MapperS.of(mASTransactionReport).<String>map("getFixedRatePaymentFrequencyPeriodOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFixedRatePaymentFrequencyPeriodOfLeg2()), MapperS.of("EXPI"), CardinalityOperator.All).getOrDefault(false)) {
						ifThenElseResult = areEqual(MapperS.of(mASTransactionReport).<Integer>map("getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2()), MapperS.of(1), CardinalityOperator.All);
					} else {
						ifThenElseResult = ComparisonResult.successEmptyOperand("");
					}
					return exists(MapperS.of(mASTransactionReport).<Integer>map("getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2())).and(greaterThan(MapperS.of(mASTransactionReport).<Integer>map("getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2()), MapperS.of(0), CardinalityOperator.All)).and(lessThan(MapperS.of(mASTransactionReport).<Integer>map("getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2()), MapperS.of(1000), CardinalityOperator.All)).and(ifThenElseResult);
				}
				if (notExists(MapperS.of(mASTransactionReport).<String>map("getFixedRatePaymentFrequencyPeriodOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFixedRatePaymentFrequencyPeriodOfLeg2())).or(areEqual(MapperS.of(mASTransactionReport).<String>map("getFixedRatePaymentFrequencyPeriodOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFixedRatePaymentFrequencyPeriodOfLeg2()), MapperS.of("ADHO"), CardinalityOperator.All)).getOrDefault(false)) {
					return notExists(MapperS.of(mASTransactionReport).<Integer>map("getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASTransactionReportDTCCMASBR012201 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
		}
	}
}
