package drr.regulation.mas.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;
import iso20022.auth030.esma.Frequency13Code__1;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASTransactionReportDTCC_MAS_BR_0126_01")
@ImplementedBy(MASTransactionReportDTCCMASBR012601.Default.class)
public interface MASTransactionReportDTCCMASBR012601 extends Validator<MASTransactionReport> {
	
	String NAME = "MASTransactionReportDTCC_MAS_BR_0126_01";
	String DEFINITION = "if [\"NEWT\", \"MODI\", \"CORR\", \"REVI\"] any = actionType and floatingRatePaymentFrequencyPeriodOfLeg2 exists and [Frequency13Code__1 -> YEAR, Frequency13Code__1 -> MNTH, Frequency13Code__1 -> WEEK, Frequency13Code__1 -> DAIL, Frequency13Code__1 -> EXPI] any = floatingRatePaymentFrequencyPeriodOfLeg2 then (floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 exists and floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 > 0 and floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 < 1000 and if floatingRatePaymentFrequencyPeriodOfLeg2 = Frequency13Code__1 -> EXPI then floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 = 1) else if floatingRatePaymentFrequencyPeriodOfLeg2 is absent or floatingRatePaymentFrequencyPeriodOfLeg2 = Frequency13Code__1 -> ADHO then floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 is absent";
	
	ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport);
	
	class Default implements MASTransactionReportDTCCMASBR012601 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			ComparisonResult result = executeDataRule(mASTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASTransactionReport mASTransactionReport) {
			try {
				if (areEqual(MapperC.<String>of(MapperS.of("NEWT"), MapperS.of("MODI"), MapperS.of("CORR"), MapperS.of("REVI")), MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), CardinalityOperator.Any).and(exists(MapperS.of(mASTransactionReport).<Frequency13Code__1>map("getFloatingRatePaymentFrequencyPeriodOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFloatingRatePaymentFrequencyPeriodOfLeg2()))).and(areEqual(MapperC.<Frequency13Code__1>of(MapperS.of(Frequency13Code__1.YEAR), MapperS.of(Frequency13Code__1.MNTH), MapperS.of(Frequency13Code__1.WEEK), MapperS.of(Frequency13Code__1.DAIL), MapperS.of(Frequency13Code__1.EXPI)), MapperS.of(mASTransactionReport).<Frequency13Code__1>map("getFloatingRatePaymentFrequencyPeriodOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFloatingRatePaymentFrequencyPeriodOfLeg2()), CardinalityOperator.Any)).getOrDefault(false)) {
					final ComparisonResult ifThenElseResult;
					if (areEqual(MapperS.of(mASTransactionReport).<Frequency13Code__1>map("getFloatingRatePaymentFrequencyPeriodOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFloatingRatePaymentFrequencyPeriodOfLeg2()), MapperS.of(Frequency13Code__1.EXPI), CardinalityOperator.All).getOrDefault(false)) {
						ifThenElseResult = areEqual(MapperS.of(mASTransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2()), MapperS.of(1), CardinalityOperator.All);
					} else {
						ifThenElseResult = ComparisonResult.successEmptyOperand("");
					}
					return exists(MapperS.of(mASTransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2())).and(greaterThan(MapperS.of(mASTransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2()), MapperS.of(0), CardinalityOperator.All)).and(lessThan(MapperS.of(mASTransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2()), MapperS.of(1000), CardinalityOperator.All)).and(ifThenElseResult);
				}
				if (notExists(MapperS.of(mASTransactionReport).<Frequency13Code__1>map("getFloatingRatePaymentFrequencyPeriodOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFloatingRatePaymentFrequencyPeriodOfLeg2())).or(areEqual(MapperS.of(mASTransactionReport).<Frequency13Code__1>map("getFloatingRatePaymentFrequencyPeriodOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFloatingRatePaymentFrequencyPeriodOfLeg2()), MapperS.of(Frequency13Code__1.ADHO), CardinalityOperator.All)).getOrDefault(false)) {
					return notExists(MapperS.of(mASTransactionReport).<Integer>map("getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2", _mASTransactionReport -> _mASTransactionReport.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2()));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASTransactionReportDTCCMASBR012601 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
		}
	}
}
