package drr.regulation.mas.rewrite.trade.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MASTransactionReportDTCC_MAS_BR_0133_01")
@ImplementedBy(MASTransactionReportDTCCMASBR013301.Default.class)
public interface MASTransactionReportDTCCMASBR013301 extends Validator<MASTransactionReport> {
	
	String NAME = "MASTransactionReportDTCC_MAS_BR_0133_01";
	String DEFINITION = "if actionType = \"NEWT\" then [\"TRAD\", \"NOVA\", \"COMP\", \"CLRG\", \"EXER\", \"ALOC\", \"CLAL\", \"INCP\", \"CORP\"] any = eventType else if actionType = \"MODI\" then [\"TRAD\", \"NOVA\", \"COMP\", \"ETRM\", \"EXER\", \"ALOC\", \"CREV\", \"CORP\", \"INCP\", \"UPDT\"] any = eventType else if actionType = \"TERM\" then [\"NOVA\", \"COMP\", \"ETRM\", \"CLRG\", \"EXER\", \"ALOC\", \"CLAL\", \"CREV\", \"CORP\", \"INCP\"] any = eventType else if [\"CORR\", \"REVI\", \"EROR\", \"VALU\"] any = actionType then True";
	
	ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport);
	
	class Default implements MASTransactionReportDTCCMASBR013301 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			ComparisonResult result = executeDataRule(mASTransactionReport);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MASTransactionReport mASTransactionReport) {
			try {
				if (areEqual(MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), MapperS.of("NEWT"), CardinalityOperator.All).getOrDefault(false)) {
					return areEqual(MapperC.<String>of(MapperS.of("TRAD"), MapperS.of("NOVA"), MapperS.of("COMP"), MapperS.of("CLRG"), MapperS.of("EXER"), MapperS.of("ALOC"), MapperS.of("CLAL"), MapperS.of("INCP"), MapperS.of("CORP")), MapperS.of(mASTransactionReport).<String>map("getEventType", _mASTransactionReport -> _mASTransactionReport.getEventType()), CardinalityOperator.Any);
				}
				if (areEqual(MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), MapperS.of("MODI"), CardinalityOperator.All).getOrDefault(false)) {
					return areEqual(MapperC.<String>of(MapperS.of("TRAD"), MapperS.of("NOVA"), MapperS.of("COMP"), MapperS.of("ETRM"), MapperS.of("EXER"), MapperS.of("ALOC"), MapperS.of("CREV"), MapperS.of("CORP"), MapperS.of("INCP"), MapperS.of("UPDT")), MapperS.of(mASTransactionReport).<String>map("getEventType", _mASTransactionReport -> _mASTransactionReport.getEventType()), CardinalityOperator.Any);
				}
				if (areEqual(MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), MapperS.of("TERM"), CardinalityOperator.All).getOrDefault(false)) {
					return areEqual(MapperC.<String>of(MapperS.of("NOVA"), MapperS.of("COMP"), MapperS.of("ETRM"), MapperS.of("CLRG"), MapperS.of("EXER"), MapperS.of("ALOC"), MapperS.of("CLAL"), MapperS.of("CREV"), MapperS.of("CORP"), MapperS.of("INCP")), MapperS.of(mASTransactionReport).<String>map("getEventType", _mASTransactionReport -> _mASTransactionReport.getEventType()), CardinalityOperator.Any);
				}
				if (areEqual(MapperC.<String>of(MapperS.of("CORR"), MapperS.of("REVI"), MapperS.of("EROR"), MapperS.of("VALU")), MapperS.of(mASTransactionReport).<String>map("getActionType", _mASTransactionReport -> _mASTransactionReport.getActionType()), CardinalityOperator.Any).getOrDefault(false)) {
					return ComparisonResult.of(MapperS.of(true));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MASTransactionReportDTCCMASBR013301 {
	
		@Override
		public ValidationResult<MASTransactionReport> validate(RosettaPath path, MASTransactionReport mASTransactionReport) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MASTransactionReport", path, DEFINITION);
		}
	}
}
