package drr.regulation.techsprint.g20.mas.reports;

import cdm.event.common.Trade;
import cdm.observable.asset.PriceSchedule;
import cdm.observable.asset.PriceTypeEnum;
import cdm.observable.asset.metafields.FieldWithMetaPriceSchedule;
import cdm.product.common.settlement.PriceQuantity;
import cdm.product.template.TradeLot;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.SingleTradeLot;
import java.math.BigDecimal;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(FXExchangeRateRule.FXExchangeRateRuleDefault.class)
public abstract class FXExchangeRateRule implements ReportFunction<TransactionReportInstruction, BigDecimal> {
	
	// RosettaFunction dependencies
	//
	@Inject protected ContractForEventRule contractForEvent;
	@Inject protected IsFxPayoutRule isFxPayout;
	@Inject protected SingleTradeLot singleTradeLot;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public BigDecimal evaluate(TransactionReportInstruction input) {
		BigDecimal output = doEvaluate(input);
		
		return output;
	}

	protected abstract BigDecimal doEvaluate(TransactionReportInstruction input);

	public static class FXExchangeRateRuleDefault extends FXExchangeRateRule {
		@Override
		protected BigDecimal doEvaluate(TransactionReportInstruction input) {
			BigDecimal output = null;
			return assignOutput(output, input);
		}
		
		protected BigDecimal assignOutput(BigDecimal output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult0 = MapperS.of(input)
				.filterSingleNullSafe(item -> isFxPayout.evaluate(item.get()));
			final MapperS<Trade> thenResult1 = thenResult0
				.mapSingleToItem(item -> MapperS.of(contractForEvent.evaluate(item.get())));
			final MapperS<TradeLot> thenResult2 = thenResult1
				.mapSingleToItem(item -> MapperS.of(singleTradeLot.evaluate(item.get())));
			final MapperC<PriceSchedule> thenResult3 = thenResult2
				.mapSingleToList(item -> item.<PriceQuantity>mapC("getPriceQuantity", tradeLot -> tradeLot.getPriceQuantity()).<FieldWithMetaPriceSchedule>mapC("getPrice", priceQuantity -> priceQuantity.getPrice()).<PriceSchedule>map("getValue", _f->_f.getValue()));
			final MapperC<PriceSchedule> thenResult4 = thenResult3
				.filterItemNullSafe(item -> areEqual(item.<PriceTypeEnum>map("getPriceType", priceSchedule -> priceSchedule.getPriceType()), MapperS.of(PriceTypeEnum.EXCHANGE_RATE), CardinalityOperator.All).get());
			final MapperC<BigDecimal> thenResult5 = thenResult4
				.mapItem(item -> item.<BigDecimal>map("getValue", measureBase -> measureBase.getValue()));
			output = thenResult5
				.last().get();
			
			return output;
		}
	}
}
