package drr.regulation.techsprint.g20.mas.reports;

import cdm.event.common.Trade;
import cdm.observable.asset.FloatingRateOption;
import cdm.observable.asset.Observable;
import cdm.observable.asset.PriceSchedule;
import cdm.observable.asset.metafields.FieldWithMetaFloatingRateOption;
import cdm.observable.asset.metafields.FieldWithMetaPriceSchedule;
import cdm.product.common.settlement.PriceQuantity;
import cdm.product.template.TradeLot;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperListOfLists;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.IsFixedInterestRate;
import drr.regulation.common.functions.SingleTradeLot;
import java.math.BigDecimal;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(FixedFloatRateLeg1Rule.FixedFloatRateLeg1RuleDefault.class)
public abstract class FixedFloatRateLeg1Rule implements ReportFunction<TransactionReportInstruction, BigDecimal> {
	
	// RosettaFunction dependencies
	//
	@Inject protected ContractForEventRule contractForEvent;
	@Inject protected IsFixedInterestRate isFixedInterestRate;
	@Inject protected IsInterestRatePayoutRule isInterestRatePayout;
	@Inject protected SingleTradeLot singleTradeLot;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public BigDecimal evaluate(TransactionReportInstruction input) {
		BigDecimal output = doEvaluate(input);
		
		return output;
	}

	protected abstract BigDecimal doEvaluate(TransactionReportInstruction input);

	public static class FixedFloatRateLeg1RuleDefault extends FixedFloatRateLeg1Rule {
		@Override
		protected BigDecimal doEvaluate(TransactionReportInstruction input) {
			BigDecimal output = null;
			return assignOutput(output, input);
		}
		
		protected BigDecimal assignOutput(BigDecimal output, TransactionReportInstruction input) {
			final MapperS<TransactionReportInstruction> thenResult0 = MapperS.of(input)
				.filterSingleNullSafe(item -> isInterestRatePayout.evaluate(item.get()));
			final MapperS<Trade> thenResult1 = thenResult0
				.mapSingleToItem(item -> MapperS.of(contractForEvent.evaluate(item.get())));
			final MapperS<TradeLot> thenResult2 = thenResult1
				.mapSingleToItem(item -> MapperS.of(singleTradeLot.evaluate(item.get())));
			final MapperC<PriceQuantity> thenResult3 = thenResult2
				.mapSingleToList(item -> item.<PriceQuantity>mapC("getPriceQuantity", tradeLot -> tradeLot.getPriceQuantity()));
			final MapperC<PriceQuantity> thenResult4 = thenResult3
				.filterItemNullSafe(item -> notExists(item.<Observable>map("getObservable", priceQuantity -> priceQuantity.getObservable()).<FieldWithMetaFloatingRateOption>map("getRateOption", observable -> observable.getRateOption()).<FloatingRateOption>map("getValue", _f->_f.getValue())).get());
			final MapperListOfLists<PriceSchedule> thenResult5 = thenResult4
				.mapItemToList(item -> item.<FieldWithMetaPriceSchedule>mapC("getPrice", priceQuantity -> priceQuantity.getPrice()).<PriceSchedule>map("getValue", _f->_f.getValue()));
			final MapperC<PriceSchedule> thenResult6 = thenResult5
				.flattenList();
			final MapperC<PriceSchedule> thenResult7 = thenResult6
				.filterItemNullSafe(item -> isFixedInterestRate.evaluate(item.get()));
			final MapperS<PriceSchedule> thenResult8 = MapperS.of(thenResult7.get());
			output = thenResult8
				.mapSingleToItem(item -> item.<BigDecimal>map("getValue", measureBase -> measureBase.getValue())).get();
			
			return output;
		}
	}
}
