package drr.regulation.techsprint.g20.mas.reports;

import cdm.base.staticdata.party.Counterparty;
import cdm.base.staticdata.party.CounterpartyRoleEnum;
import cdm.base.staticdata.party.Party;
import cdm.base.staticdata.party.Party.PartyBuilder;
import cdm.base.staticdata.party.PayerReceiver;
import cdm.base.staticdata.party.functions.ExtractCounterpartyByRole;
import cdm.base.staticdata.party.metafields.ReferenceWithMetaParty;
import cdm.event.common.Trade;
import cdm.product.asset.InterestRatePayout;
import cdm.product.asset.functions.ExtractFixedLeg;
import cdm.product.template.ContractualProduct;
import cdm.product.template.EconomicTerms;
import cdm.product.template.Payout;
import cdm.product.template.Product;
import cdm.product.template.TradableProduct;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import java.util.Optional;
import javax.inject.Inject;


@ImplementedBy(InterestRatePayoutPayerRule.InterestRatePayoutPayerRuleDefault.class)
public abstract class InterestRatePayoutPayerRule implements ReportFunction<TransactionReportInstruction, Party> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ContractForEventRule contractForEvent;
	@Inject protected ExtractCounterpartyByRole extractCounterpartyByRole;
	@Inject protected ExtractFixedLeg extractFixedLeg;
	@Inject protected IsFixedFloatRule isFixedFloat;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public Party evaluate(TransactionReportInstruction input) {
		Party.PartyBuilder outputBuilder = doEvaluate(input);
		
		final Party output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.build();
			objectValidator.validate(Party.class, output);
		}
		
		return output;
	}

	protected abstract Party.PartyBuilder doEvaluate(TransactionReportInstruction input);

	public static class InterestRatePayoutPayerRuleDefault extends InterestRatePayoutPayerRule {
		@Override
		protected Party.PartyBuilder doEvaluate(TransactionReportInstruction input) {
			Party.PartyBuilder output = Party.builder();
			return assignOutput(output, input);
		}
		
		protected Party.PartyBuilder assignOutput(Party.PartyBuilder output, TransactionReportInstruction input) {
			final MapperS<Trade> thenResult0 = MapperS.of(contractForEvent.evaluate(input));
			final MapperS<Trade> thenResult1 = thenResult0
				.filterSingleNullSafe(item -> isFixedFloat.evaluate(item.get()));
			output = toBuilder(thenResult1
				.mapSingleToItem(item -> MapperS.of(extractCounterpartyByRole.evaluate(item.<TradableProduct>map("getTradableProduct", trade -> trade.getTradableProduct()).<Counterparty>mapC("getCounterparty", tradableProduct -> tradableProduct.getCounterparty()).getMulti(), MapperS.of(extractFixedLeg.evaluate(item.<TradableProduct>map("getTradableProduct", trade -> trade.getTradableProduct()).<Product>map("getProduct", tradableProduct -> tradableProduct.getProduct()).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<InterestRatePayout>mapC("getInterestRatePayout", payout -> payout.getInterestRatePayout()).getMulti())).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getPayer", payerReceiver -> payerReceiver.getPayer()).get())).<ReferenceWithMetaParty>map("getPartyReference", counterparty -> counterparty.getPartyReference()).<Party>map("getValue", _f->_f.getValue())).get());
			
			return Optional.ofNullable(output)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
