package drr.regulation.techsprint.g20.mas.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaReport;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.techsprint.g20.mas.MASSFATransactionReport;
import drr.regulation.techsprint.g20.mas.MASSFATransactionReport.MASSFATransactionReportBuilder;
import java.util.Optional;
import javax.inject.Inject;


@RosettaReport(namespace="drr.regulation.techsprint.g20.mas", body="MAS", corpusList={"SFA", "MAS_2013"})
@ImplementedBy(MASSFAMAS_2013ReportFunction.MASSFAMAS_2013ReportFunctionDefault.class)
public abstract class MASSFAMAS_2013ReportFunction implements ReportFunction<TransactionReportInstruction, MASSFATransactionReport> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected ClearedRule cleared;
	@Inject protected ClearedEntityRule clearedEntity;
	@Inject protected Counterparty1Rule counterparty1;
	@Inject protected Counterparty2Rule counterparty2;
	@Inject protected EffectiveDateRule effectiveDate;
	@Inject protected FXCurrency1Rule fXCurrency1;
	@Inject protected FXCurrency2Rule fXCurrency2;
	@Inject protected FXExchangeRateRule fXExchangeRate;
	@Inject protected InterestRateDayCountLeg1Rule interestRateDayCountLeg1;
	@Inject protected InterestRateDayCountLeg2Rule interestRateDayCountLeg2;
	@Inject protected InterestRateNotionalAmountLeg1Rule interestRateNotionalAmountLeg1;
	@Inject protected InterestRateNotionalAmountLeg2Rule interestRateNotionalAmountLeg2;
	@Inject protected InterestRateNotionalCurrencyLeg1Rule interestRateNotionalCurrencyLeg1;
	@Inject protected InterestRateNotionalCurrencyLeg2Rule interestRateNotionalCurrencyLeg2;
	@Inject protected InterestRatePayerLeg1Rule interestRatePayerLeg1;
	@Inject protected InterestRatePayerLeg2Rule interestRatePayerLeg2;
	@Inject protected InterestRatePaymentFrequencyMultiplierLeg1Rule interestRatePaymentFrequencyMultiplierLeg1;
	@Inject protected InterestRatePaymentFrequencyMultiplierLeg2Rule interestRatePaymentFrequencyMultiplierLeg2;
	@Inject protected InterestRatePaymentFrequencyPeriodLeg1Rule interestRatePaymentFrequencyPeriodLeg1;
	@Inject protected InterestRatePaymentFrequencyPeriodLeg2Rule interestRatePaymentFrequencyPeriodLeg2;
	@Inject protected InterestRatePriceRule interestRatePrice;
	@Inject protected InterestRateRateLeg1Rule interestRateRateLeg1;
	@Inject protected InterestRateRateLeg2Rule interestRateRateLeg2;
	@Inject protected InterestRateResetFrequencyMultiplierLeg1Rule interestRateResetFrequencyMultiplierLeg1;
	@Inject protected InterestRateResetFrequencyMultiplierLeg2Rule interestRateResetFrequencyMultiplierLeg2;
	@Inject protected InterestRateResetFrequencyPeriodLeg1Rule interestRateResetFrequencyPeriodLeg1;
	@Inject protected InterestRateResetFrequencyPeriodLeg2Rule interestRateResetFrequencyPeriodLeg2;
	@Inject protected OptionExpirationDateRule optionExpirationDate;
	@Inject protected OptionPremiumRule optionPremium;
	@Inject protected OptionTypeRule optionType;
	@Inject protected PriorUniqueTransactionIdentifierRule priorUniqueTransactionIdentifier;
	@Inject protected ReportingEntityRule reportingEntity;
	@Inject protected ReportingTimestampRule reportingTimestamp;
	@Inject protected TerminationDateRule terminationDate;
	@Inject protected UniqueProductIdentifierRule uniqueProductIdentifier;
	@Inject protected UniqueTransactionIdentifierRule uniqueTransactionIdentifier;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public MASSFATransactionReport evaluate(TransactionReportInstruction input) {
		MASSFATransactionReport.MASSFATransactionReportBuilder outputBuilder = doEvaluate(input);
		
		final MASSFATransactionReport output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.build();
			objectValidator.validate(MASSFATransactionReport.class, output);
		}
		
		return output;
	}

	protected abstract MASSFATransactionReport.MASSFATransactionReportBuilder doEvaluate(TransactionReportInstruction input);

	public static class MASSFAMAS_2013ReportFunctionDefault extends MASSFAMAS_2013ReportFunction {
		@Override
		protected MASSFATransactionReport.MASSFATransactionReportBuilder doEvaluate(TransactionReportInstruction input) {
			MASSFATransactionReport.MASSFATransactionReportBuilder output = MASSFATransactionReport.builder();
			return assignOutput(output, input);
		}
		
		protected MASSFATransactionReport.MASSFATransactionReportBuilder assignOutput(MASSFATransactionReport.MASSFATransactionReportBuilder output, TransactionReportInstruction input) {
			output
				.setUniqueTransactionIdentifier(uniqueTransactionIdentifier.evaluate(input));
			
			output
				.setUniqueProductIdentifier(uniqueProductIdentifier.evaluate(input));
			
			output
				.setPriorUniqueTransactionIdentifier(priorUniqueTransactionIdentifier.evaluate(input));
			
			output
				.setCounterparty1(counterparty1.evaluate(input));
			
			output
				.setCounterparty2(counterparty2.evaluate(input));
			
			output
				.setReportingEntity(reportingEntity.evaluate(input));
			
			output
				.setCleared(cleared.evaluate(input));
			
			output
				.setClearedEntity(clearedEntity.evaluate(input));
			
			output
				.setEffectiveDate(effectiveDate.evaluate(input));
			
			output
				.setTerminationDate(terminationDate.evaluate(input));
			
			output
				.setReportingTimestamp(reportingTimestamp.evaluate(input));
			
			output
				.setOptionType(optionType.evaluate(input));
			
			output
				.setOptionExpirationDate(optionExpirationDate.evaluate(input));
			
			output
				.setOptionPremium(optionPremium.evaluate(input));
			
			output
				.setInterestRateNotionalAmountLeg1(interestRateNotionalAmountLeg1.evaluate(input));
			
			output
				.setInterestRateNotionalCurrencyLeg1(interestRateNotionalCurrencyLeg1.evaluate(input));
			
			output
				.setInterestRateNotionalAmountLeg2(interestRateNotionalAmountLeg2.evaluate(input));
			
			output
				.setInterestRateNotionalCurrencyLeg2(interestRateNotionalCurrencyLeg2.evaluate(input));
			
			output
				.setInterestRatePayerLeg1(interestRatePayerLeg1.evaluate(input));
			
			output
				.setInterestRatePayerLeg2(interestRatePayerLeg2.evaluate(input));
			
			output
				.setInterestRateRateLeg1(interestRateRateLeg1.evaluate(input));
			
			output
				.setInterestRateRateLeg2(interestRateRateLeg2.evaluate(input));
			
			output
				.setInterestRatePaymentFrequencyPeriodLeg1(interestRatePaymentFrequencyPeriodLeg1.evaluate(input));
			
			output
				.setInterestRatePaymentFrequencyMultiplierLeg1(interestRatePaymentFrequencyMultiplierLeg1.evaluate(input));
			
			output
				.setInterestRatePaymentFrequencyPeriodLeg2(interestRatePaymentFrequencyPeriodLeg2.evaluate(input));
			
			output
				.setInterestRatePaymentFrequencyMultiplierLeg2(interestRatePaymentFrequencyMultiplierLeg2.evaluate(input));
			
			output
				.setInterestRateDayCountLeg1(interestRateDayCountLeg1.evaluate(input));
			
			output
				.setInterestRateDayCountLeg2(interestRateDayCountLeg2.evaluate(input));
			
			output
				.setInterestRateResetFrequencyPeriodLeg1(interestRateResetFrequencyPeriodLeg1.evaluate(input));
			
			output
				.setInterestRateResetFrequencyMultiplierLeg1(interestRateResetFrequencyMultiplierLeg1.evaluate(input));
			
			output
				.setInterestRateResetFrequencyPeriodLeg2(interestRateResetFrequencyPeriodLeg2.evaluate(input));
			
			output
				.setInterestRateResetFrequencyMultiplierLeg2(interestRateResetFrequencyMultiplierLeg2.evaluate(input));
			
			output
				.setInterestRatePrice(interestRatePrice.evaluate(input));
			
			output
				.setFXCurrency1(fXCurrency1.evaluate(input));
			
			output
				.setFXCurrency2(fXCurrency2.evaluate(input));
			
			output
				.setFXExchangeRate(fXExchangeRate.evaluate(input));
			
			return Optional.ofNullable(output)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
