package drr.regulation.techsprint.g20.mas.reports;

import cdm.base.datetime.PeriodExtendedEnum;
import cdm.base.datetime.daycount.DayCountFractionEnum;
import com.rosetta.model.lib.ModelSymbolId;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.Tabulator;
import com.rosetta.model.lib.reports.Tabulator.Field;
import com.rosetta.model.lib.reports.Tabulator.FieldImpl;
import com.rosetta.model.lib.reports.Tabulator.FieldValue;
import com.rosetta.model.lib.reports.Tabulator.FieldValueImpl;
import com.rosetta.util.DottedPath;
import drr.regulation.techsprint.g20.mas.MASSFATransactionReport;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;


public class MASSFATransactionReportTypeTabulator implements Tabulator<MASSFATransactionReport> {
	private final Field uniqueTransactionIdentifierField;
	private final Field uniqueProductIdentifierField;
	private final Field priorUniqueTransactionIdentifierField;
	private final Field counterparty1Field;
	private final Field counterparty2Field;
	private final Field reportingEntityField;
	private final Field clearedField;
	private final Field clearedEntityField;
	private final Field effectiveDateField;
	private final Field terminationDateField;
	private final Field reportingTimestampField;
	private final Field optionTypeField;
	private final Field optionExpirationDateField;
	private final Field optionPremiumField;
	private final Field interestRateNotionalAmountLeg1Field;
	private final Field interestRateNotionalCurrencyLeg1Field;
	private final Field interestRateNotionalAmountLeg2Field;
	private final Field interestRateNotionalCurrencyLeg2Field;
	private final Field interestRatePayerLeg1Field;
	private final Field interestRatePayerLeg2Field;
	private final Field interestRateRateLeg1Field;
	private final Field interestRateRateLeg2Field;
	private final Field interestRatePaymentFrequencyPeriodLeg1Field;
	private final Field interestRatePaymentFrequencyMultiplierLeg1Field;
	private final Field interestRatePaymentFrequencyPeriodLeg2Field;
	private final Field interestRatePaymentFrequencyMultiplierLeg2Field;
	private final Field interestRateDayCountLeg1Field;
	private final Field interestRateDayCountLeg2Field;
	private final Field interestRateResetFrequencyPeriodLeg1Field;
	private final Field interestRateResetFrequencyMultiplierLeg1Field;
	private final Field interestRateResetFrequencyPeriodLeg2Field;
	private final Field interestRateResetFrequencyMultiplierLeg2Field;
	private final Field interestRatePriceField;
	private final Field fXCurrency1Field;
	private final Field fXCurrency2Field;
	private final Field fXExchangeRateField;
	
	public MASSFATransactionReportTypeTabulator() {
		this.uniqueTransactionIdentifierField = new FieldImpl(
			"uniqueTransactionIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "UniqueTransactionIdentifier")),
			Optional.of("I.1.1 UTI"),
			Arrays.asList()
		);
		this.uniqueProductIdentifierField = new FieldImpl(
			"uniqueProductIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "UniqueProductIdentifier")),
			Optional.of("I.1.2 UPI"),
			Arrays.asList()
		);
		this.priorUniqueTransactionIdentifierField = new FieldImpl(
			"priorUniqueTransactionIdentifier",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "PriorUniqueTransactionIdentifier")),
			Optional.of("I.1.3 Prior UTI"),
			Arrays.asList()
		);
		this.counterparty1Field = new FieldImpl(
			"counterparty1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "Counterparty1")),
			Optional.of("I.2.1 Counterparty 1"),
			Arrays.asList()
		);
		this.counterparty2Field = new FieldImpl(
			"counterparty2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "Counterparty2")),
			Optional.of("I.2.2 Counterparty 2"),
			Arrays.asList()
		);
		this.reportingEntityField = new FieldImpl(
			"reportingEntity",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "ReportingEntity")),
			Optional.of("I.2.3 Reporting Counterparty ID"),
			Arrays.asList()
		);
		this.clearedField = new FieldImpl(
			"cleared",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "Cleared")),
			Optional.of("I.3.1 Cleared"),
			Arrays.asList()
		);
		this.clearedEntityField = new FieldImpl(
			"clearedEntity",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "ClearedEntity")),
			Optional.of("I.3.2 Identifier of Clearing Entity"),
			Arrays.asList()
		);
		this.effectiveDateField = new FieldImpl(
			"effectiveDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "EffectiveDate")),
			Optional.of("I.4.1 Effective Date"),
			Arrays.asList()
		);
		this.terminationDateField = new FieldImpl(
			"terminationDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "TerminationDate")),
			Optional.of("I.4.1 Termination Date"),
			Arrays.asList()
		);
		this.reportingTimestampField = new FieldImpl(
			"reportingTimestamp",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "ReportingTimestamp")),
			Optional.of("I.5 Reporting Timestamp"),
			Arrays.asList()
		);
		this.optionTypeField = new FieldImpl(
			"optionType",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "OptionType")),
			Optional.of("I.6.1 Option Type"),
			Arrays.asList()
		);
		this.optionExpirationDateField = new FieldImpl(
			"optionExpirationDate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "OptionExpirationDate")),
			Optional.of("I.6.2 Option Expiration Date"),
			Arrays.asList()
		);
		this.optionPremiumField = new FieldImpl(
			"optionPremium",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "OptionPremium")),
			Optional.of("I.6.3 Option Premium"),
			Arrays.asList()
		);
		this.interestRateNotionalAmountLeg1Field = new FieldImpl(
			"interestRateNotionalAmountLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRateNotionalAmountLeg1")),
			Optional.of("II.1.1 Notional amount leg 1"),
			Arrays.asList()
		);
		this.interestRateNotionalCurrencyLeg1Field = new FieldImpl(
			"interestRateNotionalCurrencyLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRateNotionalCurrencyLeg1")),
			Optional.of("II.1.2 Notional currency leg 1"),
			Arrays.asList()
		);
		this.interestRateNotionalAmountLeg2Field = new FieldImpl(
			"interestRateNotionalAmountLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRateNotionalAmountLeg2")),
			Optional.of("II.1.3 Notional amount leg 2"),
			Arrays.asList()
		);
		this.interestRateNotionalCurrencyLeg2Field = new FieldImpl(
			"interestRateNotionalCurrencyLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRateNotionalCurrencyLeg2")),
			Optional.of("II.1.4 Notional currency leg 2"),
			Arrays.asList()
		);
		this.interestRatePayerLeg1Field = new FieldImpl(
			"interestRatePayerLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRatePayerLeg1")),
			Optional.of("II.1.7 Payer leg 1"),
			Arrays.asList()
		);
		this.interestRatePayerLeg2Field = new FieldImpl(
			"interestRatePayerLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRatePayerLeg2")),
			Optional.of("II.1.8 Payer leg 2"),
			Arrays.asList()
		);
		this.interestRateRateLeg1Field = new FieldImpl(
			"interestRateRateLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRateRateLeg1")),
			Optional.of("II.1.9 Rate leg 1"),
			Arrays.asList()
		);
		this.interestRateRateLeg2Field = new FieldImpl(
			"interestRateRateLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRateRateLeg2")),
			Optional.of("II.1.10 Rate of leg 2"),
			Arrays.asList()
		);
		this.interestRatePaymentFrequencyPeriodLeg1Field = new FieldImpl(
			"interestRatePaymentFrequencyPeriodLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRatePaymentFrequencyPeriodLeg1")),
			Optional.of("II.1.11 Payment frequency period leg 1"),
			Arrays.asList()
		);
		this.interestRatePaymentFrequencyMultiplierLeg1Field = new FieldImpl(
			"interestRatePaymentFrequencyMultiplierLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRatePaymentFrequencyMultiplierLeg1")),
			Optional.of("II.1.12 Payment frequency multiplier leg 1"),
			Arrays.asList()
		);
		this.interestRatePaymentFrequencyPeriodLeg2Field = new FieldImpl(
			"interestRatePaymentFrequencyPeriodLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRatePaymentFrequencyPeriodLeg2")),
			Optional.of("II.1.13 Payment frequency period leg 2"),
			Arrays.asList()
		);
		this.interestRatePaymentFrequencyMultiplierLeg2Field = new FieldImpl(
			"interestRatePaymentFrequencyMultiplierLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRatePaymentFrequencyMultiplierLeg2")),
			Optional.of("II.1.14 Payment frequency multiplier leg 2"),
			Arrays.asList()
		);
		this.interestRateDayCountLeg1Field = new FieldImpl(
			"interestRateDayCountLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRateDayCountLeg1")),
			Optional.of("II.1.15 Day count leg 1"),
			Arrays.asList()
		);
		this.interestRateDayCountLeg2Field = new FieldImpl(
			"interestRateDayCountLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRateDayCountLeg2")),
			Optional.of("II.1.16 Day count leg 2"),
			Arrays.asList()
		);
		this.interestRateResetFrequencyPeriodLeg1Field = new FieldImpl(
			"interestRateResetFrequencyPeriodLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRateResetFrequencyPeriodLeg1")),
			Optional.of("II.1.17 Reset frequency period leg 1"),
			Arrays.asList()
		);
		this.interestRateResetFrequencyMultiplierLeg1Field = new FieldImpl(
			"interestRateResetFrequencyMultiplierLeg1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRateResetFrequencyMultiplierLeg1")),
			Optional.of("II.1.18 Reset frequency multiplier leg 1"),
			Arrays.asList()
		);
		this.interestRateResetFrequencyPeriodLeg2Field = new FieldImpl(
			"interestRateResetFrequencyPeriodLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRateResetFrequencyPeriodLeg2")),
			Optional.of("II.1.19 Reset frequency period leg 2"),
			Arrays.asList()
		);
		this.interestRateResetFrequencyMultiplierLeg2Field = new FieldImpl(
			"interestRateResetFrequencyMultiplierLeg2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRateResetFrequencyMultiplierLeg2")),
			Optional.of("II.1.20 Reset frequency multiplier leg 2"),
			Arrays.asList()
		);
		this.interestRatePriceField = new FieldImpl(
			"interestRatePrice",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "InterestRatePrice")),
			Optional.of("II.1.21 Price/spread"),
			Arrays.asList()
		);
		this.fXCurrency1Field = new FieldImpl(
			"fXCurrency1",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "FXCurrency1")),
			Optional.of("IV.1.2 Currency 1"),
			Arrays.asList()
		);
		this.fXCurrency2Field = new FieldImpl(
			"fXCurrency2",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "FXCurrency2")),
			Optional.of("IV.1.4 Currency 2"),
			Arrays.asList()
		);
		this.fXExchangeRateField = new FieldImpl(
			"fXExchangeRate",
			false,
			Optional.of(new ModelSymbolId(DottedPath.of("drr", "regulation", "techsprint", "g20", "mas"), "FXExchangeRate")),
			Optional.of("IV.1.5 Exchange Rate"),
			Arrays.asList()
		);
	}
	
	@Override
	public List<Field> getFields() {
		return Arrays.asList(uniqueTransactionIdentifierField, uniqueProductIdentifierField, priorUniqueTransactionIdentifierField, counterparty1Field, counterparty2Field, reportingEntityField, clearedField, clearedEntityField, effectiveDateField, terminationDateField, reportingTimestampField, optionTypeField, optionExpirationDateField, optionPremiumField, interestRateNotionalAmountLeg1Field, interestRateNotionalCurrencyLeg1Field, interestRateNotionalAmountLeg2Field, interestRateNotionalCurrencyLeg2Field, interestRatePayerLeg1Field, interestRatePayerLeg2Field, interestRateRateLeg1Field, interestRateRateLeg2Field, interestRatePaymentFrequencyPeriodLeg1Field, interestRatePaymentFrequencyMultiplierLeg1Field, interestRatePaymentFrequencyPeriodLeg2Field, interestRatePaymentFrequencyMultiplierLeg2Field, interestRateDayCountLeg1Field, interestRateDayCountLeg2Field, interestRateResetFrequencyPeriodLeg1Field, interestRateResetFrequencyMultiplierLeg1Field, interestRateResetFrequencyPeriodLeg2Field, interestRateResetFrequencyMultiplierLeg2Field, interestRatePriceField, fXCurrency1Field, fXCurrency2Field, fXExchangeRateField);
	}
	
	@Override
	public List<FieldValue> tabulate(MASSFATransactionReport input) {
		Optional<String> uniqueTransactionIdentifier = Optional.ofNullable(input.getUniqueTransactionIdentifier());
		Optional<String> uniqueProductIdentifier = Optional.ofNullable(input.getUniqueProductIdentifier());
		Optional<String> priorUniqueTransactionIdentifier = Optional.ofNullable(input.getPriorUniqueTransactionIdentifier());
		Optional<String> counterparty1 = Optional.ofNullable(input.getCounterparty1());
		Optional<String> counterparty2 = Optional.ofNullable(input.getCounterparty2());
		Optional<String> reportingEntity = Optional.ofNullable(input.getReportingEntity());
		Optional<String> cleared = Optional.ofNullable(input.getCleared());
		Optional<String> clearedEntity = Optional.ofNullable(input.getClearedEntity());
		Optional<Date> effectiveDate = Optional.ofNullable(input.getEffectiveDate());
		Optional<Date> terminationDate = Optional.ofNullable(input.getTerminationDate());
		Optional<ZonedDateTime> reportingTimestamp = Optional.ofNullable(input.getReportingTimestamp());
		Optional<String> optionType = Optional.ofNullable(input.getOptionType());
		Optional<String> optionExpirationDate = Optional.ofNullable(input.getOptionExpirationDate());
		Optional<String> optionPremium = Optional.ofNullable(input.getOptionPremium());
		Optional<BigDecimal> interestRateNotionalAmountLeg1 = Optional.ofNullable(input.getInterestRateNotionalAmountLeg1());
		Optional<String> interestRateNotionalCurrencyLeg1 = Optional.ofNullable(input.getInterestRateNotionalCurrencyLeg1());
		Optional<BigDecimal> interestRateNotionalAmountLeg2 = Optional.ofNullable(input.getInterestRateNotionalAmountLeg2());
		Optional<String> interestRateNotionalCurrencyLeg2 = Optional.ofNullable(input.getInterestRateNotionalCurrencyLeg2());
		Optional<String> interestRatePayerLeg1 = Optional.ofNullable(input.getInterestRatePayerLeg1());
		Optional<String> interestRatePayerLeg2 = Optional.ofNullable(input.getInterestRatePayerLeg2());
		Optional<BigDecimal> interestRateRateLeg1 = Optional.ofNullable(input.getInterestRateRateLeg1());
		Optional<BigDecimal> interestRateRateLeg2 = Optional.ofNullable(input.getInterestRateRateLeg2());
		Optional<PeriodExtendedEnum> interestRatePaymentFrequencyPeriodLeg1 = Optional.ofNullable(input.getInterestRatePaymentFrequencyPeriodLeg1());
		Optional<Integer> interestRatePaymentFrequencyMultiplierLeg1 = Optional.ofNullable(input.getInterestRatePaymentFrequencyMultiplierLeg1());
		Optional<PeriodExtendedEnum> interestRatePaymentFrequencyPeriodLeg2 = Optional.ofNullable(input.getInterestRatePaymentFrequencyPeriodLeg2());
		Optional<Integer> interestRatePaymentFrequencyMultiplierLeg2 = Optional.ofNullable(input.getInterestRatePaymentFrequencyMultiplierLeg2());
		Optional<DayCountFractionEnum> interestRateDayCountLeg1 = Optional.ofNullable(input.getInterestRateDayCountLeg1());
		Optional<DayCountFractionEnum> interestRateDayCountLeg2 = Optional.ofNullable(input.getInterestRateDayCountLeg2());
		Optional<PeriodExtendedEnum> interestRateResetFrequencyPeriodLeg1 = Optional.ofNullable(input.getInterestRateResetFrequencyPeriodLeg1());
		Optional<Integer> interestRateResetFrequencyMultiplierLeg1 = Optional.ofNullable(input.getInterestRateResetFrequencyMultiplierLeg1());
		Optional<PeriodExtendedEnum> interestRateResetFrequencyPeriodLeg2 = Optional.ofNullable(input.getInterestRateResetFrequencyPeriodLeg2());
		Optional<Integer> interestRateResetFrequencyMultiplierLeg2 = Optional.ofNullable(input.getInterestRateResetFrequencyMultiplierLeg2());
		Optional<BigDecimal> interestRatePrice = Optional.ofNullable(input.getInterestRatePrice());
		Optional<String> fXCurrency1 = Optional.ofNullable(input.getFXCurrency1());
		Optional<String> fXCurrency2 = Optional.ofNullable(input.getFXCurrency2());
		Optional<BigDecimal> fXExchangeRate = Optional.ofNullable(input.getFXExchangeRate());
		return Arrays.asList(
			new FieldValueImpl(uniqueTransactionIdentifierField, uniqueTransactionIdentifier),
			new FieldValueImpl(uniqueProductIdentifierField, uniqueProductIdentifier),
			new FieldValueImpl(priorUniqueTransactionIdentifierField, priorUniqueTransactionIdentifier),
			new FieldValueImpl(counterparty1Field, counterparty1),
			new FieldValueImpl(counterparty2Field, counterparty2),
			new FieldValueImpl(reportingEntityField, reportingEntity),
			new FieldValueImpl(clearedField, cleared),
			new FieldValueImpl(clearedEntityField, clearedEntity),
			new FieldValueImpl(effectiveDateField, effectiveDate),
			new FieldValueImpl(terminationDateField, terminationDate),
			new FieldValueImpl(reportingTimestampField, reportingTimestamp),
			new FieldValueImpl(optionTypeField, optionType),
			new FieldValueImpl(optionExpirationDateField, optionExpirationDate),
			new FieldValueImpl(optionPremiumField, optionPremium),
			new FieldValueImpl(interestRateNotionalAmountLeg1Field, interestRateNotionalAmountLeg1),
			new FieldValueImpl(interestRateNotionalCurrencyLeg1Field, interestRateNotionalCurrencyLeg1),
			new FieldValueImpl(interestRateNotionalAmountLeg2Field, interestRateNotionalAmountLeg2),
			new FieldValueImpl(interestRateNotionalCurrencyLeg2Field, interestRateNotionalCurrencyLeg2),
			new FieldValueImpl(interestRatePayerLeg1Field, interestRatePayerLeg1),
			new FieldValueImpl(interestRatePayerLeg2Field, interestRatePayerLeg2),
			new FieldValueImpl(interestRateRateLeg1Field, interestRateRateLeg1),
			new FieldValueImpl(interestRateRateLeg2Field, interestRateRateLeg2),
			new FieldValueImpl(interestRatePaymentFrequencyPeriodLeg1Field, interestRatePaymentFrequencyPeriodLeg1),
			new FieldValueImpl(interestRatePaymentFrequencyMultiplierLeg1Field, interestRatePaymentFrequencyMultiplierLeg1),
			new FieldValueImpl(interestRatePaymentFrequencyPeriodLeg2Field, interestRatePaymentFrequencyPeriodLeg2),
			new FieldValueImpl(interestRatePaymentFrequencyMultiplierLeg2Field, interestRatePaymentFrequencyMultiplierLeg2),
			new FieldValueImpl(interestRateDayCountLeg1Field, interestRateDayCountLeg1),
			new FieldValueImpl(interestRateDayCountLeg2Field, interestRateDayCountLeg2),
			new FieldValueImpl(interestRateResetFrequencyPeriodLeg1Field, interestRateResetFrequencyPeriodLeg1),
			new FieldValueImpl(interestRateResetFrequencyMultiplierLeg1Field, interestRateResetFrequencyMultiplierLeg1),
			new FieldValueImpl(interestRateResetFrequencyPeriodLeg2Field, interestRateResetFrequencyPeriodLeg2),
			new FieldValueImpl(interestRateResetFrequencyMultiplierLeg2Field, interestRateResetFrequencyMultiplierLeg2),
			new FieldValueImpl(interestRatePriceField, interestRatePrice),
			new FieldValueImpl(fXCurrency1Field, fXCurrency1),
			new FieldValueImpl(fXCurrency2Field, fXCurrency2),
			new FieldValueImpl(fXExchangeRateField, fXExchangeRate)
		);
	}
}
