package drr.regulation.techsprint.g20.mas.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.functions.ModelObjectValidator;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.TransactionReportInstruction.TransactionReportInstructionBuilder;
import java.util.Optional;
import javax.inject.Inject;


@ImplementedBy(NexusCompliantRule.NexusCompliantRuleDefault.class)
public abstract class NexusCompliantRule implements ReportFunction<TransactionReportInstruction, TransactionReportInstruction> {
	
	@Inject protected ModelObjectValidator objectValidator;
	
	// RosettaFunction dependencies
	//
	@Inject protected BookedInSingaporeRule bookedInSingapore;
	@Inject protected TradedInSingaporeRule tradedInSingapore;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public TransactionReportInstruction evaluate(TransactionReportInstruction input) {
		TransactionReportInstruction.TransactionReportInstructionBuilder outputBuilder = doEvaluate(input);
		
		final TransactionReportInstruction output;
		if (outputBuilder == null) {
			output = null;
		} else {
			output = outputBuilder.build();
			objectValidator.validate(TransactionReportInstruction.class, output);
		}
		
		return output;
	}

	protected abstract TransactionReportInstruction.TransactionReportInstructionBuilder doEvaluate(TransactionReportInstruction input);

	public static class NexusCompliantRuleDefault extends NexusCompliantRule {
		@Override
		protected TransactionReportInstruction.TransactionReportInstructionBuilder doEvaluate(TransactionReportInstruction input) {
			TransactionReportInstruction.TransactionReportInstructionBuilder output = TransactionReportInstruction.builder();
			return assignOutput(output, input);
		}
		
		protected TransactionReportInstruction.TransactionReportInstructionBuilder assignOutput(TransactionReportInstruction.TransactionReportInstructionBuilder output, TransactionReportInstruction input) {
			output = toBuilder(MapperC.<TransactionReportInstruction>of(MapperS.of(bookedInSingapore.evaluate(input)), MapperS.of(tradedInSingapore.evaluate(input)))
				.last().get());
			
			return Optional.ofNullable(output)
				.map(o -> o.prune())
				.orElse(null);
		}
	}
}
