package drr.standards.iosco.cde.functions;

import cdm.base.staticdata.party.BuyerSeller;
import cdm.base.staticdata.party.Counterparty;
import cdm.base.staticdata.party.CounterpartyRoleEnum;
import cdm.base.staticdata.party.Party;
import cdm.base.staticdata.party.PartyIdentifier;
import cdm.base.staticdata.party.PayerReceiver;
import cdm.base.staticdata.party.functions.ExtractCounterpartyByRole;
import cdm.base.staticdata.party.metafields.ReferenceWithMetaParty;
import cdm.product.asset.CommodityPayout;
import cdm.product.asset.CreditDefaultPayout;
import cdm.product.asset.InterestRatePayout;
import cdm.product.qualification.functions.Qualify_InterestRate_Forward_Debt;
import cdm.product.template.ContractualProduct;
import cdm.product.template.EconomicTerms;
import cdm.product.template.ForwardPayout;
import cdm.product.template.OptionPayout;
import cdm.product.template.Payout;
import cdm.product.template.PerformancePayout;
import cdm.product.template.Product;
import cdm.product.template.TradableProduct;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.EconomicTermsForProduct;
import drr.regulation.common.functions.IsCapFloor;
import drr.regulation.common.functions.IsCommodityForward;
import drr.regulation.common.functions.IsCreditDefaultSwap;
import drr.regulation.common.functions.IsProductETD;
import drr.regulation.common.functions.IsSingleOnlyCommodityPayout;
import drr.regulation.common.functions.IsVarianceSwap;
import drr.regulation.common.functions.IsVolatilitySwap;
import drr.regulation.common.functions.PartyLei;
import drr.regulation.common.functions.TradableProductForEvent;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDEDirection1.CDEDirection1Default.class)
public abstract class CDEDirection1 implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDECounterparty1 cDECounterparty1;
	@Inject protected EconomicTermsForProduct economicTermsForProduct;
	@Inject protected ExtractCounterpartyByRole extractCounterpartyByRole;
	@Inject protected IsCapFloor isCapFloor;
	@Inject protected IsCommodityForward isCommodityForward;
	@Inject protected IsCreditDefaultSwap isCreditDefaultSwap;
	@Inject protected IsProductETD isProductETD;
	@Inject protected IsSingleOnlyCommodityPayout isSingleOnlyCommodityPayout;
	@Inject protected IsVarianceSwap isVarianceSwap;
	@Inject protected IsVolatilitySwap isVolatilitySwap;
	@Inject protected PartyLei partyLei;
	@Inject protected Qualify_InterestRate_Forward_Debt qualify_InterestRate_Forward_Debt;
	@Inject protected TradableProductForEvent tradableProductForEvent;

	/**
	* @param event 
	* @return direction 
	*/
	public String evaluate(TransactionReportInstruction event) {
		String direction = doEvaluate(event);
		
		return direction;
	}

	protected abstract String doEvaluate(TransactionReportInstruction event);

	protected abstract MapperS<? extends TradableProduct> tradableProduct(TransactionReportInstruction event);

	protected abstract MapperS<String> reportingParty(TransactionReportInstruction event);

	protected abstract MapperS<? extends Counterparty> buyer(TransactionReportInstruction event);

	protected abstract MapperS<? extends Counterparty> seller(TransactionReportInstruction event);

	public static class CDEDirection1Default extends CDEDirection1 {
		@Override
		protected String doEvaluate(TransactionReportInstruction event) {
			String direction = null;
			return assignOutput(direction, event);
		}
		
		protected String assignOutput(String direction, TransactionReportInstruction event) {
			if (areEqual(reportingParty(event), MapperS.of(partyLei.evaluate(buyer(event).<ReferenceWithMetaParty>map("getPartyReference", counterparty -> counterparty.getPartyReference()).<Party>map("getValue", _f->_f.getValue()).<PartyIdentifier>mapC("getPartyId", party -> party.getPartyId()).getMulti())), CardinalityOperator.All).getOrDefault(false)) {
				direction = "BYER";
			} else if (areEqual(reportingParty(event), MapperS.of(partyLei.evaluate(seller(event).<ReferenceWithMetaParty>map("getPartyReference", counterparty -> counterparty.getPartyReference()).<Party>map("getValue", _f->_f.getValue()).<PartyIdentifier>mapC("getPartyId", party -> party.getPartyId()).getMulti())), CardinalityOperator.All).getOrDefault(false)) {
				direction = "SLLR";
			} else {
				direction = null;
			}
			
			return direction;
		}
		
		@Override
		protected MapperS<? extends TradableProduct> tradableProduct(TransactionReportInstruction event) {
			return MapperS.of(tradableProductForEvent.evaluate(event));
		}
		
		@Override
		protected MapperS<String> reportingParty(TransactionReportInstruction event) {
			return MapperS.of(cDECounterparty1.evaluate(event));
		}
		
		@Override
		protected MapperS<? extends Counterparty> buyer(TransactionReportInstruction event) {
			if (exists(MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<OptionPayout>mapC("getOptionPayout", payout -> payout.getOptionPayout())).getOrDefault(false)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<OptionPayout>mapC("getOptionPayout", payout -> payout.getOptionPayout()).get()).<BuyerSeller>map("getBuyerSeller", optionPayout -> optionPayout.getBuyerSeller()).<CounterpartyRoleEnum>map("getBuyer", buyerSeller -> buyerSeller.getBuyer()).get()));
			}
			final Boolean boolean0 = isCreditDefaultSwap.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get());
			if ((boolean0 == null ? false : boolean0)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<CreditDefaultPayout>map("getCreditDefaultPayout", payout -> payout.getCreditDefaultPayout()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getReceiver", payerReceiver -> payerReceiver.getReceiver()).get()));
			}
			if (ComparisonResult.of(MapperS.of(isVarianceSwap.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get()))).or(ComparisonResult.of(MapperS.of(isVolatilitySwap.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())))).getOrDefault(false)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<PerformancePayout>mapC("getPerformancePayout", payout -> payout.getPerformancePayout()).get()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getReceiver", payerReceiver -> payerReceiver.getReceiver()).get()));
			}
			final Boolean boolean1 = isCapFloor.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get());
			if ((boolean1 == null ? false : boolean1)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<InterestRatePayout>mapC("getInterestRatePayout", payout -> payout.getInterestRatePayout()).get()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getPayer", payerReceiver -> payerReceiver.getPayer()).get()));
			}
			if (ComparisonResult.of(MapperS.of(isProductETD.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get()))).and(exists(MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<ForwardPayout>mapC("getForwardPayout", payout -> payout.getForwardPayout()))).getOrDefault(false)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<ForwardPayout>mapC("getForwardPayout", payout -> payout.getForwardPayout()).get()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getPayer", payerReceiver -> payerReceiver.getPayer()).get()));
			}
			final Boolean boolean2 = isSingleOnlyCommodityPayout.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get());
			if ((boolean2 == null ? false : boolean2)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<CommodityPayout>mapC("getCommodityPayout", payout -> payout.getCommodityPayout()).get()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getPayer", payerReceiver -> payerReceiver.getPayer()).get()));
			}
			final Boolean boolean3 = isCommodityForward.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get());
			if ((boolean3 == null ? false : boolean3)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<ForwardPayout>mapC("getForwardPayout", payout -> payout.getForwardPayout()).get()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getReceiver", payerReceiver -> payerReceiver.getReceiver()).get()));
			}
			final Boolean boolean4 = qualify_InterestRate_Forward_Debt.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get());
			if ((boolean4 == null ? false : boolean4)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<ForwardPayout>mapC("getForwardPayout", payout -> payout.getForwardPayout()).get()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getPayer", payerReceiver -> payerReceiver.getPayer()).get()));
			}
			return MapperS.<Counterparty>ofNull();
		}
		
		@Override
		protected MapperS<? extends Counterparty> seller(TransactionReportInstruction event) {
			if (exists(MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<OptionPayout>mapC("getOptionPayout", payout -> payout.getOptionPayout())).getOrDefault(false)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<OptionPayout>mapC("getOptionPayout", payout -> payout.getOptionPayout()).get()).<BuyerSeller>map("getBuyerSeller", optionPayout -> optionPayout.getBuyerSeller()).<CounterpartyRoleEnum>map("getSeller", buyerSeller -> buyerSeller.getSeller()).get()));
			}
			final Boolean boolean0 = isCreditDefaultSwap.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get());
			if ((boolean0 == null ? false : boolean0)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<CreditDefaultPayout>map("getCreditDefaultPayout", payout -> payout.getCreditDefaultPayout()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getPayer", payerReceiver -> payerReceiver.getPayer()).get()));
			}
			if (ComparisonResult.of(MapperS.of(isVarianceSwap.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get()))).or(ComparisonResult.of(MapperS.of(isVolatilitySwap.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())))).getOrDefault(false)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<PerformancePayout>mapC("getPerformancePayout", payout -> payout.getPerformancePayout()).get()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getPayer", payerReceiver -> payerReceiver.getPayer()).get()));
			}
			final Boolean boolean1 = isCapFloor.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get());
			if ((boolean1 == null ? false : boolean1)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<InterestRatePayout>mapC("getInterestRatePayout", payout -> payout.getInterestRatePayout()).get()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getReceiver", payerReceiver -> payerReceiver.getReceiver()).get()));
			}
			if (ComparisonResult.of(MapperS.of(isProductETD.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get()))).and(exists(MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<ForwardPayout>mapC("getForwardPayout", payout -> payout.getForwardPayout()))).getOrDefault(false)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<ForwardPayout>mapC("getForwardPayout", payout -> payout.getForwardPayout()).get()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getReceiver", payerReceiver -> payerReceiver.getReceiver()).get()));
			}
			final Boolean boolean2 = isSingleOnlyCommodityPayout.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get());
			if ((boolean2 == null ? false : boolean2)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(MapperS.of(economicTermsForProduct.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get())).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<CommodityPayout>mapC("getCommodityPayout", payout -> payout.getCommodityPayout()).get()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getReceiver", payerReceiver -> payerReceiver.getReceiver()).get()));
			}
			final Boolean boolean3 = isCommodityForward.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).get());
			if ((boolean3 == null ? false : boolean3)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<ForwardPayout>mapC("getForwardPayout", payout -> payout.getForwardPayout()).get()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getPayer", payerReceiver -> payerReceiver.getPayer()).get()));
			}
			final Boolean boolean4 = qualify_InterestRate_Forward_Debt.evaluate(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).get());
			if ((boolean4 == null ? false : boolean4)) {
				return MapperS.of(extractCounterpartyByRole.evaluate(tradableProduct(event).<Counterparty>mapC("getCounterparty", _tradableProduct -> _tradableProduct.getCounterparty()).getMulti(), MapperS.of(tradableProduct(event).<Product>map("getProduct", _tradableProduct -> _tradableProduct.getProduct()).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<ForwardPayout>mapC("getForwardPayout", payout -> payout.getForwardPayout()).get()).<PayerReceiver>map("getPayerReceiver", payoutBase -> payoutBase.getPayerReceiver()).<CounterpartyRoleEnum>map("getReceiver", payerReceiver -> payerReceiver.getReceiver()).get()));
			}
			return MapperS.<Counterparty>ofNull();
		}
	}
}
