package drr.standards.iosco.cde.functions;

import cdm.base.staticdata.party.Party;
import cdm.event.common.Trade;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.common.PartyInformation;
import drr.regulation.common.functions.PartyLeiAndPersonByRoles;
import java.util.Collections;
import java.util.List;
import javax.inject.Inject;


@ImplementedBy(CDEDirection1SellerIdentifier.CDEDirection1SellerIdentifierDefault.class)
public abstract class CDEDirection1SellerIdentifier implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDEDirection1SellerParty cDEDirection1SellerParty;
	@Inject protected PartyLeiAndPersonByRoles partyLeiAndPersonByRoles;

	/**
	* @param trade 
	* @param partyInformation 
	* @return leiAndPerson 
	*/
	public String evaluate(Trade trade, List<? extends PartyInformation> partyInformation) {
		String leiAndPerson = doEvaluate(trade, partyInformation);
		
		return leiAndPerson;
	}

	protected abstract String doEvaluate(Trade trade, List<? extends PartyInformation> partyInformation);

	protected abstract MapperS<? extends Party> counterparty(Trade trade, List<? extends PartyInformation> partyInformation);

	public static class CDEDirection1SellerIdentifierDefault extends CDEDirection1SellerIdentifier {
		@Override
		protected String doEvaluate(Trade trade, List<? extends PartyInformation> partyInformation) {
			if (partyInformation == null) {
				partyInformation = Collections.emptyList();
			}
			String leiAndPerson = null;
			return assignOutput(leiAndPerson, trade, partyInformation);
		}
		
		protected String assignOutput(String leiAndPerson, Trade trade, List<? extends PartyInformation> partyInformation) {
			leiAndPerson = partyLeiAndPersonByRoles.evaluate(counterparty(trade, partyInformation).get(), partyInformation);
			
			return leiAndPerson;
		}
		
		@Override
		protected MapperS<? extends Party> counterparty(Trade trade, List<? extends PartyInformation> partyInformation) {
			return MapperS.of(cDEDirection1SellerParty.evaluate(trade));
		}
	}
}
