package drr.standards.iosco.cde.functions;

import cdm.base.staticdata.party.Party;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.functions.RosettaFunction;
import drr.regulation.common.PartyInformation;
import drr.regulation.common.functions.PartyLeiAndPersonByRoles;
import java.util.Collections;
import java.util.List;
import javax.inject.Inject;


@ImplementedBy(CDEDirection2PartyRoleIdentifier.CDEDirection2PartyRoleIdentifierDefault.class)
public abstract class CDEDirection2PartyRoleIdentifier implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected PartyLeiAndPersonByRoles partyLeiAndPersonByRoles;

	/**
	* @param targetParty 
	* @param partyInformation 
	* @return leiAndPerson 
	*/
	public String evaluate(Party targetParty, List<? extends PartyInformation> partyInformation) {
		String leiAndPerson = doEvaluate(targetParty, partyInformation);
		
		return leiAndPerson;
	}

	protected abstract String doEvaluate(Party targetParty, List<? extends PartyInformation> partyInformation);

	public static class CDEDirection2PartyRoleIdentifierDefault extends CDEDirection2PartyRoleIdentifier {
		@Override
		protected String doEvaluate(Party targetParty, List<? extends PartyInformation> partyInformation) {
			if (partyInformation == null) {
				partyInformation = Collections.emptyList();
			}
			String leiAndPerson = null;
			return assignOutput(leiAndPerson, targetParty, partyInformation);
		}
		
		protected String assignOutput(String leiAndPerson, Party targetParty, List<? extends PartyInformation> partyInformation) {
			leiAndPerson = partyLeiAndPersonByRoles.evaluate(targetParty, partyInformation);
			
			return leiAndPerson;
		}
	}
}
