package drr.standards.iosco.cde.functions;

import cdm.base.math.FinancialUnitEnum;
import cdm.base.math.NonNegativeQuantitySchedule;
import cdm.base.math.UnitType;
import cdm.base.math.metafields.ReferenceWithMetaNonNegativeQuantitySchedule;
import cdm.observable.asset.PriceSchedule;
import cdm.observable.asset.metafields.ReferenceWithMetaPriceSchedule;
import cdm.product.common.settlement.ResolvablePriceQuantity;
import cdm.product.template.ForwardPayout;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.MapperMaths;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.metafields.FieldWithMetaString;
import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDEEquityForwardNotional.CDEEquityForwardNotionalDefault.class)
public abstract class CDEEquityForwardNotional implements RosettaFunction {

	/**
	* @param forwardPayout 
	* @return amount 
	*/
	public BigDecimal evaluate(List<? extends ForwardPayout> forwardPayout) {
		BigDecimal amount = doEvaluate(forwardPayout);
		
		return amount;
	}

	protected abstract BigDecimal doEvaluate(List<? extends ForwardPayout> forwardPayout);

	public static class CDEEquityForwardNotionalDefault extends CDEEquityForwardNotional {
		@Override
		protected BigDecimal doEvaluate(List<? extends ForwardPayout> forwardPayout) {
			if (forwardPayout == null) {
				forwardPayout = Collections.emptyList();
			}
			BigDecimal amount = null;
			return assignOutput(amount, forwardPayout);
		}
		
		protected BigDecimal assignOutput(BigDecimal amount, List<? extends ForwardPayout> forwardPayout) {
			if (exists(MapperS.of(MapperC.of(forwardPayout).get()).<ResolvablePriceQuantity>map("getPriceQuantity", payoutBase -> payoutBase.getPriceQuantity()).<ReferenceWithMetaNonNegativeQuantitySchedule>map("getQuantitySchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getQuantitySchedule()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()).<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue())).getOrDefault(false)) {
				amount = MapperS.of(MapperC.of(forwardPayout).get()).<ResolvablePriceQuantity>map("getPriceQuantity", payoutBase -> payoutBase.getPriceQuantity()).<ReferenceWithMetaNonNegativeQuantitySchedule>map("getQuantitySchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getQuantitySchedule()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()).<BigDecimal>map("getValue", measureBase -> measureBase.getValue()).get();
			} else if (exists(MapperS.of(MapperC.of(forwardPayout).get()).<ResolvablePriceQuantity>map("getPriceQuantity", payoutBase -> payoutBase.getPriceQuantity()).<ReferenceWithMetaNonNegativeQuantitySchedule>map("getQuantitySchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getQuantitySchedule()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()).<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FinancialUnitEnum>map("getFinancialUnit", unitType -> unitType.getFinancialUnit())).getOrDefault(false)) {
				amount = MapperMaths.<BigDecimal, BigDecimal, BigDecimal>multiply(MapperS.of(MapperC.of(forwardPayout).get()).<ResolvablePriceQuantity>map("getPriceQuantity", payoutBase -> payoutBase.getPriceQuantity()).<ReferenceWithMetaNonNegativeQuantitySchedule>map("getQuantitySchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getQuantitySchedule()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()).<BigDecimal>map("getValue", measureBase -> measureBase.getValue()), MapperS.of(MapperC.<ForwardPayout>of(forwardPayout).<ResolvablePriceQuantity>map("getPriceQuantity", payoutBase -> payoutBase.getPriceQuantity()).<ReferenceWithMetaPriceSchedule>mapC("getPriceSchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getPriceSchedule()).<PriceSchedule>map("getValue", _f->_f.getValue()).get()).<BigDecimal>map("getValue", measureBase -> measureBase.getValue())).get();
			} else {
				amount = null;
			}
			
			return amount;
		}
	}
}
