package drr.standards.iosco.cde.functions;

import cdm.base.math.NonNegativeQuantitySchedule;
import cdm.base.math.metafields.ReferenceWithMetaNonNegativeQuantitySchedule;
import cdm.product.common.settlement.ResolvablePriceQuantity;
import cdm.product.template.PerformancePayout;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import java.math.BigDecimal;
import java.util.Collections;
import java.util.List;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDEPerformanceNotional.CDEPerformanceNotionalDefault.class)
public abstract class CDEPerformanceNotional implements RosettaFunction {

	/**
	* @param performancePayout 
	* @return amount 
	*/
	public BigDecimal evaluate(List<? extends PerformancePayout> performancePayout) {
		BigDecimal amount = doEvaluate(performancePayout);
		
		return amount;
	}

	protected abstract BigDecimal doEvaluate(List<? extends PerformancePayout> performancePayout);

	public static class CDEPerformanceNotionalDefault extends CDEPerformanceNotional {
		@Override
		protected BigDecimal doEvaluate(List<? extends PerformancePayout> performancePayout) {
			if (performancePayout == null) {
				performancePayout = Collections.emptyList();
			}
			BigDecimal amount = null;
			return assignOutput(amount, performancePayout);
		}
		
		protected BigDecimal assignOutput(BigDecimal amount, List<? extends PerformancePayout> performancePayout) {
			if (areEqual(MapperS.of(MapperC.<PerformancePayout>of(performancePayout).<ResolvablePriceQuantity>map("getPriceQuantity", payoutBase -> payoutBase.getPriceQuantity()).<ReferenceWithMetaNonNegativeQuantitySchedule>map("getQuantitySchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getQuantitySchedule()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()).resultCount()), MapperS.of(1), CardinalityOperator.All).getOrDefault(false)) {
				amount = MapperS.of(MapperC.of(performancePayout).get()).<ResolvablePriceQuantity>map("getPriceQuantity", payoutBase -> payoutBase.getPriceQuantity()).<ReferenceWithMetaNonNegativeQuantitySchedule>map("getQuantitySchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getQuantitySchedule()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()).<BigDecimal>map("getValue", measureBase -> measureBase.getValue()).get();
			} else {
				amount = null;
			}
			
			return amount;
		}
	}
}
