package drr.standards.iosco.cde.functions;

import cdm.base.math.NonNegativeQuantitySchedule;
import cdm.base.math.UnitType;
import cdm.base.math.metafields.ReferenceWithMetaNonNegativeQuantitySchedule;
import cdm.product.common.settlement.ResolvablePriceQuantity;
import cdm.product.template.PerformancePayout;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.metafields.FieldWithMetaString;
import java.util.Collections;
import java.util.List;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDEPerformanceNotionalCurrency.CDEPerformanceNotionalCurrencyDefault.class)
public abstract class CDEPerformanceNotionalCurrency implements RosettaFunction {

	/**
	* @param performancePayout 
	* @return currency 
	*/
	public String evaluate(List<? extends PerformancePayout> performancePayout) {
		String currency = doEvaluate(performancePayout);
		
		return currency;
	}

	protected abstract String doEvaluate(List<? extends PerformancePayout> performancePayout);

	public static class CDEPerformanceNotionalCurrencyDefault extends CDEPerformanceNotionalCurrency {
		@Override
		protected String doEvaluate(List<? extends PerformancePayout> performancePayout) {
			if (performancePayout == null) {
				performancePayout = Collections.emptyList();
			}
			String currency = null;
			return assignOutput(currency, performancePayout);
		}
		
		protected String assignOutput(String currency, List<? extends PerformancePayout> performancePayout) {
			if (areEqual(MapperS.of(MapperC.<PerformancePayout>of(performancePayout).resultCount()), MapperS.of(1), CardinalityOperator.All).getOrDefault(false)) {
				currency = MapperS.of(MapperC.of(performancePayout).get()).<ResolvablePriceQuantity>map("getPriceQuantity", payoutBase -> payoutBase.getPriceQuantity()).<ReferenceWithMetaNonNegativeQuantitySchedule>map("getQuantitySchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getQuantitySchedule()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()).<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue()).get();
			} else {
				currency = null;
			}
			
			return currency;
		}
	}
}
