package drr.standards.iosco.cde.functions;

import cdm.event.common.Valuation;
import cdm.event.common.ValuationSourceEnum;
import cdm.event.common.ValuationTypeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import iso20022.auth030.esma.ValuationType1Code;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDEValuationMethod.CDEValuationMethodDefault.class)
public abstract class CDEValuationMethod implements RosettaFunction {

	/**
	* @param valuation 
	* @return valuationType 
	*/
	public ValuationType1Code evaluate(Valuation valuation) {
		ValuationType1Code valuationType = doEvaluate(valuation);
		
		return valuationType;
	}

	protected abstract ValuationType1Code doEvaluate(Valuation valuation);

	protected abstract MapperS<ValuationTypeEnum> valuationMethod(Valuation valuation);

	protected abstract MapperS<ValuationSourceEnum> valuationSource(Valuation valuation);

	public static class CDEValuationMethodDefault extends CDEValuationMethod {
		@Override
		protected ValuationType1Code doEvaluate(Valuation valuation) {
			ValuationType1Code valuationType = null;
			return assignOutput(valuationType, valuation);
		}
		
		protected ValuationType1Code assignOutput(ValuationType1Code valuationType, Valuation valuation) {
			if (exists(valuationSource(valuation)).and(areEqual(valuationSource(valuation), MapperS.of(ValuationSourceEnum.CENTRAL_COUNTERPARTY), CardinalityOperator.All)).getOrDefault(false)) {
				valuationType = ValuationType1Code.CCPV;
			} else if (exists(valuationMethod(valuation)).and(areEqual(valuationMethod(valuation), MapperS.of(ValuationTypeEnum.MARK_TO_MARKET), CardinalityOperator.All)).getOrDefault(false)) {
				valuationType = ValuationType1Code.MTMA;
			} else if (exists(valuationMethod(valuation)).and(areEqual(valuationMethod(valuation), MapperS.of(ValuationTypeEnum.MARK_TO_MODEL), CardinalityOperator.All)).getOrDefault(false)) {
				valuationType = ValuationType1Code.MTMO;
			} else {
				valuationType = null;
			}
			
			return valuationType;
		}
		
		@Override
		protected MapperS<ValuationTypeEnum> valuationMethod(Valuation valuation) {
			if (exists(MapperS.of(valuation)).getOrDefault(false)) {
				return MapperS.of(valuation).<ValuationTypeEnum>map("getMethod", _valuation -> _valuation.getMethod());
			}
			return MapperS.<ValuationTypeEnum>ofNull();
		}
		
		@Override
		protected MapperS<ValuationSourceEnum> valuationSource(Valuation valuation) {
			if (exists(MapperS.of(valuation)).getOrDefault(false)) {
				return MapperS.of(valuation).<ValuationSourceEnum>map("getSource", _valuation -> _valuation.getSource());
			}
			return MapperS.<ValuationSourceEnum>ofNull();
		}
	}
}
