package drr.standards.iosco.cde.functions;

import cdm.base.staticdata.party.Party;
import cdm.base.staticdata.party.PartyReferencePayerReceiver;
import cdm.base.staticdata.party.metafields.ReferenceWithMetaParty;
import cdm.event.common.CollateralBalance;
import cdm.event.common.HaircutIndicatorEnum;
import cdm.product.collateral.CollateralMarginTypeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.common.ReportableCollateral;
import java.math.BigDecimal;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDEVariationMarginCollectedByReportingCounterpartyPostHaircut.CDEVariationMarginCollectedByReportingCounterpartyPostHaircutDefault.class)
public abstract class CDEVariationMarginCollectedByReportingCounterpartyPostHaircut implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected GetCollateralBalancesForMarginType getCollateralBalancesForMarginType;
	@Inject protected GetMarginValue getMarginValue;

	/**
	* @param reportableCollateral 
	* @param reportingCounterparty 
	* @return marginValue 
	*/
	public BigDecimal evaluate(ReportableCollateral reportableCollateral, Party reportingCounterparty) {
		BigDecimal marginValue = doEvaluate(reportableCollateral, reportingCounterparty);
		
		return marginValue;
	}

	protected abstract BigDecimal doEvaluate(ReportableCollateral reportableCollateral, Party reportingCounterparty);

	public static class CDEVariationMarginCollectedByReportingCounterpartyPostHaircutDefault extends CDEVariationMarginCollectedByReportingCounterpartyPostHaircut {
		@Override
		protected BigDecimal doEvaluate(ReportableCollateral reportableCollateral, Party reportingCounterparty) {
			BigDecimal marginValue = null;
			return assignOutput(marginValue, reportableCollateral, reportingCounterparty);
		}
		
		protected BigDecimal assignOutput(BigDecimal marginValue, ReportableCollateral reportableCollateral, Party reportingCounterparty) {
			marginValue = MapperS.of(getMarginValue.evaluate(MapperC.<CollateralBalance>of(getCollateralBalancesForMarginType.evaluate(reportableCollateral, CollateralMarginTypeEnum.VARIATION_MARGIN))
				.filterItemNullSafe(item -> areEqual(item.<PartyReferencePayerReceiver>map("getPayerReceiver", collateralBalance -> collateralBalance.getPayerReceiver()).<ReferenceWithMetaParty>map("getReceiverPartyReference", partyReferencePayerReceiver -> partyReferencePayerReceiver.getReceiverPartyReference()).<Party>map("getValue", _f->_f.getValue()), MapperS.of(reportingCounterparty), CardinalityOperator.All).get()).getMulti(), HaircutIndicatorEnum.PRE_HAIRCUT)).<BigDecimal>map("getValue", measureBase -> measureBase.getValue()).get();
			
			return marginValue;
		}
	}
}
