package drr.standards.iosco.cde.functions;

import cdm.event.workflow.EventTimestamp;
import cdm.event.workflow.EventTimestampQualificationEnum;
import cdm.event.workflow.WorkflowStep;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import drr.regulation.common.ReportableEvent;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(ExistConfirmationDate.ExistConfirmationDateDefault.class)
public abstract class ExistConfirmationDate implements RosettaFunction {

	/**
	* @param reportableEvent 
	* @return hasConfirmationDate 
	*/
	public Boolean evaluate(ReportableEvent reportableEvent) {
		Boolean hasConfirmationDate = doEvaluate(reportableEvent);
		
		return hasConfirmationDate;
	}

	protected abstract Boolean doEvaluate(ReportableEvent reportableEvent);

	public static class ExistConfirmationDateDefault extends ExistConfirmationDate {
		@Override
		protected Boolean doEvaluate(ReportableEvent reportableEvent) {
			Boolean hasConfirmationDate = null;
			return assignOutput(hasConfirmationDate, reportableEvent);
		}
		
		protected Boolean assignOutput(Boolean hasConfirmationDate, ReportableEvent reportableEvent) {
			final MapperC<EventTimestamp> thenResult0 = MapperS.of(reportableEvent).<WorkflowStep>map("getOriginatingWorkflowStep", _reportableEvent -> _reportableEvent.getOriginatingWorkflowStep()).<EventTimestamp>mapC("getTimestamp", workflowStep -> workflowStep.getTimestamp());
			final MapperC<EventTimestamp> thenResult1 = thenResult0
				.filterItemNullSafe(item -> areEqual(item.<EventTimestampQualificationEnum>map("getQualification", eventTimestamp -> eventTimestamp.getQualification()), MapperS.of(EventTimestampQualificationEnum.CONFIRMATION_DATE_TIME), CardinalityOperator.All).get());
			hasConfirmationDate = exists(thenResult1).asMapper().get();
			
			return hasConfirmationDate;
		}
	}
}
