package drr.standards.iosco.cde.functions;

import cdm.base.math.UnitType;
import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import cdm.event.common.CollateralBalance;
import cdm.event.common.HaircutIndicatorEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.functions.RosettaFunction;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.metafields.FieldWithMetaString;
import drr.regulation.common.functions.ConvertNonISOToISOCurrency;
import java.util.Collections;
import java.util.List;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(GetMarginCurrency.GetMarginCurrencyDefault.class)
public abstract class GetMarginCurrency implements RosettaFunction {
	
	// RosettaFunction dependencies
	//
	@Inject protected ConvertNonISOToISOCurrency convertNonISOToISOCurrency;
	@Inject protected GetMarginValue getMarginValue;

	/**
	* @param partyCollateralBalances 
	* @return marginCurrency 
	*/
	public ISOCurrencyCodeEnum evaluate(List<? extends CollateralBalance> partyCollateralBalances) {
		ISOCurrencyCodeEnum marginCurrency = doEvaluate(partyCollateralBalances);
		
		return marginCurrency;
	}

	protected abstract ISOCurrencyCodeEnum doEvaluate(List<? extends CollateralBalance> partyCollateralBalances);

	public static class GetMarginCurrencyDefault extends GetMarginCurrency {
		@Override
		protected ISOCurrencyCodeEnum doEvaluate(List<? extends CollateralBalance> partyCollateralBalances) {
			if (partyCollateralBalances == null) {
				partyCollateralBalances = Collections.emptyList();
			}
			ISOCurrencyCodeEnum marginCurrency = null;
			return assignOutput(marginCurrency, partyCollateralBalances);
		}
		
		protected ISOCurrencyCodeEnum assignOutput(ISOCurrencyCodeEnum marginCurrency, List<? extends CollateralBalance> partyCollateralBalances) {
			if (areEqual(MapperS.of(getMarginValue.evaluate(partyCollateralBalances, HaircutIndicatorEnum.PRE_HAIRCUT)).<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue()), MapperS.of(getMarginValue.evaluate(partyCollateralBalances, HaircutIndicatorEnum.POST_HAIRCUT)).<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue()), CardinalityOperator.All).getOrDefault(false)) {
				final MapperS<String> thenResult = MapperS.of(getMarginValue.evaluate(partyCollateralBalances, HaircutIndicatorEnum.PRE_HAIRCUT)).<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue());
				marginCurrency = thenResult
					.mapSingleToItem(item -> MapperS.of(convertNonISOToISOCurrency.evaluate(item.get()))).get();
			} else {
				marginCurrency = null;
			}
			
			return marginCurrency;
		}
	}
}
