package drr.standards.iosco.cde.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.ConfirmationMethodEnum;
import drr.regulation.common.ReportableInformation;
import drr.regulation.common.TransactionReportInstruction;
import drr.standards.iosco.cde.functions.ExistConfirmationDate;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDEConfirmedRule.CDEConfirmedRuleDefault.class)
public abstract class CDEConfirmedRule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected ExistConfirmationDate existConfirmationDate;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class CDEConfirmedRuleDefault extends CDEConfirmedRule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			if (areEqual(MapperS.of(existConfirmationDate.evaluate(input)), MapperS.of(false), CardinalityOperator.All).or(areEqual(MapperS.of(input).<ReportableInformation>map("getReportableInformation", reportableEvent -> reportableEvent.getReportableInformation()).<ConfirmationMethodEnum>map("getConfirmationMethod", reportableInformation -> reportableInformation.getConfirmationMethod()), MapperS.of(ConfirmationMethodEnum.NOT_CONFIRMED), CardinalityOperator.All)).getOrDefault(false)) {
				output = "NCNF";
			} else {
				output = MapperS.of(input)
					.mapSingleToItem(item -> {
						if (areEqual(item.<ReportableInformation>map("getReportableInformation", reportableEvent -> reportableEvent.getReportableInformation()).<ConfirmationMethodEnum>map("getConfirmationMethod", reportableInformation -> reportableInformation.getConfirmationMethod()), MapperS.of(ConfirmationMethodEnum.ELECTRONIC), CardinalityOperator.All).getOrDefault(false)) {
							return MapperS.of("ECNF");
						}
						if (areEqual(item.<ReportableInformation>map("getReportableInformation", reportableEvent -> reportableEvent.getReportableInformation()).<ConfirmationMethodEnum>map("getConfirmationMethod", reportableInformation -> reportableInformation.getConfirmationMethod()), MapperS.of(ConfirmationMethodEnum.NON_ELECTRONIC), CardinalityOperator.All).getOrDefault(false)) {
							return MapperS.of("YCNF");
						}
						return MapperS.<String>ofNull();
					}).get();
			}
			
			return output;
		}
	}
}
