package drr.standards.iosco.cde.reports;

import cdm.base.datetime.daycount.DayCountFractionEnum;
import cdm.base.datetime.daycount.metafields.FieldWithMetaDayCountFractionEnum;
import cdm.product.asset.InterestRatePayout;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import iso20022.auth030.esma.InterestComputationMethod4Code;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDEDayCountConventionRule.CDEDayCountConventionRuleDefault.class)
public abstract class CDEDayCountConventionRule implements ReportFunction<InterestRatePayout, InterestComputationMethod4Code> {

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public InterestComputationMethod4Code evaluate(InterestRatePayout input) {
		InterestComputationMethod4Code output = doEvaluate(input);
		
		return output;
	}

	protected abstract InterestComputationMethod4Code doEvaluate(InterestRatePayout input);

	public static class CDEDayCountConventionRuleDefault extends CDEDayCountConventionRule {
		@Override
		protected InterestComputationMethod4Code doEvaluate(InterestRatePayout input) {
			InterestComputationMethod4Code output = null;
			return assignOutput(output, input);
		}
		
		protected InterestComputationMethod4Code assignOutput(InterestComputationMethod4Code output, InterestRatePayout input) {
			if (areEqual(MapperS.of(input).<FieldWithMetaDayCountFractionEnum>map("getDayCountFraction", interestRatePayout -> interestRatePayout.getDayCountFraction()).<DayCountFractionEnum>map("getValue", _f->_f.getValue()), MapperS.of(DayCountFractionEnum._30_360), CardinalityOperator.All).getOrDefault(false)) {
				output = InterestComputationMethod4Code.A001;
			} else if (areEqual(MapperS.of(input).<FieldWithMetaDayCountFractionEnum>map("getDayCountFraction", interestRatePayout -> interestRatePayout.getDayCountFraction()).<DayCountFractionEnum>map("getValue", _f->_f.getValue()), MapperS.of(DayCountFractionEnum.ACT_360), CardinalityOperator.All).getOrDefault(false)) {
				output = InterestComputationMethod4Code.A004;
			} else if (areEqual(MapperS.of(input).<FieldWithMetaDayCountFractionEnum>map("getDayCountFraction", interestRatePayout -> interestRatePayout.getDayCountFraction()).<DayCountFractionEnum>map("getValue", _f->_f.getValue()), MapperS.of(DayCountFractionEnum.ACT_365_FIXED), CardinalityOperator.All).getOrDefault(false)) {
				output = InterestComputationMethod4Code.A005;
			} else if (areEqual(MapperS.of(input).<FieldWithMetaDayCountFractionEnum>map("getDayCountFraction", interestRatePayout -> interestRatePayout.getDayCountFraction()).<DayCountFractionEnum>map("getValue", _f->_f.getValue()), MapperS.of(DayCountFractionEnum.ACT_ACT_ICMA), CardinalityOperator.All).getOrDefault(false)) {
				output = InterestComputationMethod4Code.A006;
			} else if (areEqual(MapperS.of(input).<FieldWithMetaDayCountFractionEnum>map("getDayCountFraction", interestRatePayout -> interestRatePayout.getDayCountFraction()).<DayCountFractionEnum>map("getValue", _f->_f.getValue()), MapperS.of(DayCountFractionEnum._30E_360_ISDA), CardinalityOperator.All).getOrDefault(false)) {
				output = InterestComputationMethod4Code.A007;
			} else if (areEqual(MapperS.of(input).<FieldWithMetaDayCountFractionEnum>map("getDayCountFraction", interestRatePayout -> interestRatePayout.getDayCountFraction()).<DayCountFractionEnum>map("getValue", _f->_f.getValue()), MapperS.of(DayCountFractionEnum.ACT_ACT_ISDA), CardinalityOperator.All).getOrDefault(false)) {
				output = InterestComputationMethod4Code.A008;
			} else if (areEqual(MapperS.of(input).<FieldWithMetaDayCountFractionEnum>map("getDayCountFraction", interestRatePayout -> interestRatePayout.getDayCountFraction()).<DayCountFractionEnum>map("getValue", _f->_f.getValue()), MapperS.of(DayCountFractionEnum.ACT_365L), CardinalityOperator.All).getOrDefault(false)) {
				output = InterestComputationMethod4Code.A009;
			} else if (areEqual(MapperS.of(input).<FieldWithMetaDayCountFractionEnum>map("getDayCountFraction", interestRatePayout -> interestRatePayout.getDayCountFraction()).<DayCountFractionEnum>map("getValue", _f->_f.getValue()), MapperS.of(DayCountFractionEnum.ACT_ACT_AFB), CardinalityOperator.All).getOrDefault(false)) {
				output = InterestComputationMethod4Code.A010;
			} else if (areEqual(MapperS.of(input).<FieldWithMetaDayCountFractionEnum>map("getDayCountFraction", interestRatePayout -> interestRatePayout.getDayCountFraction()).<DayCountFractionEnum>map("getValue", _f->_f.getValue()), MapperS.of(DayCountFractionEnum._30E_360), CardinalityOperator.All).getOrDefault(false)) {
				output = InterestComputationMethod4Code.A011;
			} else if (areEqual(MapperS.of(input).<FieldWithMetaDayCountFractionEnum>map("getDayCountFraction", interestRatePayout -> interestRatePayout.getDayCountFraction()).<DayCountFractionEnum>map("getValue", _f->_f.getValue()), MapperS.of(DayCountFractionEnum.ACT_ACT_ISMA), CardinalityOperator.All).getOrDefault(false)) {
				output = InterestComputationMethod4Code.A015;
			} else if (areEqual(MapperS.of(input).<FieldWithMetaDayCountFractionEnum>map("getDayCountFraction", interestRatePayout -> interestRatePayout.getDayCountFraction()).<DayCountFractionEnum>map("getValue", _f->_f.getValue()), MapperS.of(DayCountFractionEnum.ACT_364), CardinalityOperator.All).getOrDefault(false)) {
				output = InterestComputationMethod4Code.A017;
			} else if (areEqual(MapperS.of(input).<FieldWithMetaDayCountFractionEnum>map("getDayCountFraction", interestRatePayout -> interestRatePayout.getDayCountFraction()).<DayCountFractionEnum>map("getValue", _f->_f.getValue()), MapperS.of(DayCountFractionEnum.CAL_252), CardinalityOperator.All).getOrDefault(false)) {
				output = InterestComputationMethod4Code.A018;
			} else if (areEqual(MapperS.of(input).<FieldWithMetaDayCountFractionEnum>map("getDayCountFraction", interestRatePayout -> interestRatePayout.getDayCountFraction()).<DayCountFractionEnum>map("getValue", _f->_f.getValue()), MapperS.of(DayCountFractionEnum._1_1), CardinalityOperator.All).getOrDefault(false)) {
				output = InterestComputationMethod4Code.A020;
			} else if (exists(MapperS.of(input).<FieldWithMetaDayCountFractionEnum>map("getDayCountFraction", interestRatePayout -> interestRatePayout.getDayCountFraction()).<DayCountFractionEnum>map("getValue", _f->_f.getValue())).getOrDefault(false)) {
				output = InterestComputationMethod4Code.NARR;
			} else {
				output = null;
			}
			
			return output;
		}
	}
}
