package drr.standards.iosco.cde.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.ReportableEvent;
import drr.regulation.common.functions.GetValuation;
import java.math.BigDecimal;
import javax.inject.Inject;


@ImplementedBy(CDEDeltaRule.CDEDeltaRuleDefault.class)
public abstract class CDEDeltaRule implements ReportFunction<ReportableEvent, BigDecimal> {
	
	// RosettaFunction dependencies
	//
	@Inject protected GetValuation getValuation;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public BigDecimal evaluate(ReportableEvent input) {
		BigDecimal output = doEvaluate(input);
		
		return output;
	}

	protected abstract BigDecimal doEvaluate(ReportableEvent input);

	public static class CDEDeltaRuleDefault extends CDEDeltaRule {
		@Override
		protected BigDecimal doEvaluate(ReportableEvent input) {
			BigDecimal output = null;
			return assignOutput(output, input);
		}
		
		protected BigDecimal assignOutput(BigDecimal output, ReportableEvent input) {
			output = MapperS.of(input)
				.mapSingleToItem(item -> MapperS.of(getValuation.evaluate(item.get())).<BigDecimal>map("getDelta", valuation -> valuation.getDelta())).get();
			
			return output;
		}
	}
}
