package drr.standards.iosco.cde.reports;

import cdm.base.math.UnitType;
import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import cdm.observable.asset.PriceSchedule;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.MapperMaths;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import com.rosetta.model.metafields.FieldWithMetaString;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDEExchangeRateBasisRule.CDEExchangeRateBasisRuleDefault.class)
public abstract class CDEExchangeRateBasisRule implements ReportFunction<PriceSchedule, String> {

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(PriceSchedule input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(PriceSchedule input);

	public static class CDEExchangeRateBasisRuleDefault extends CDEExchangeRateBasisRule {
		@Override
		protected String doEvaluate(PriceSchedule input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, PriceSchedule input) {
			if (exists(MapperS.of(input).<UnitType>map("getPerUnitOf", priceSchedule -> priceSchedule.getPerUnitOf()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue()).checkedMap("to-enum", ISOCurrencyCodeEnum::fromDisplayName, IllegalArgumentException.class)).and(exists(MapperS.of(input).<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue()).checkedMap("to-enum", ISOCurrencyCodeEnum::fromDisplayName, IllegalArgumentException.class))).getOrDefault(false)) {
				output = MapperMaths.<String, String, String>add(MapperMaths.<String, String, String>add(MapperS.of(input).<UnitType>map("getPerUnitOf", priceSchedule -> priceSchedule.getPerUnitOf()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue()), MapperS.of("/")), MapperS.of(input).<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue())).get();
			} else {
				output = null;
			}
			
			return output;
		}
	}
}
