package drr.standards.iosco.cde.reports;

import cdm.base.datetime.AdjustableDate;
import cdm.base.datetime.AdjustableDates;
import cdm.base.datetime.AdjustableOrRelativeDate;
import cdm.base.datetime.AdjustableOrRelativeDates;
import cdm.event.common.Trade;
import cdm.product.template.AmericanExercise;
import cdm.product.template.BermudaExercise;
import cdm.product.template.ContractualProduct;
import cdm.product.template.EconomicTerms;
import cdm.product.template.EuropeanExercise;
import cdm.product.template.OptionExercise;
import cdm.product.template.OptionPayout;
import cdm.product.template.OptionStyle;
import cdm.product.template.Payout;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.ReportFunction;
import com.rosetta.model.metafields.FieldWithMetaDate;
import drr.regulation.common.functions.CFTCAdjustableDateResolution;
import drr.regulation.common.functions.CFTCAdjustableDatesResolution;
import drr.regulation.common.functions.CFTCMinAdjustableDateResolution;
import drr.regulation.common.functions.ProductForTrade;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDEFirstExerciseDateRule.CDEFirstExerciseDateRuleDefault.class)
public abstract class CDEFirstExerciseDateRule implements ReportFunction<Trade, Date> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CFTCAdjustableDateResolution cFTCAdjustableDateResolution;
	@Inject protected CFTCAdjustableDatesResolution cFTCAdjustableDatesResolution;
	@Inject protected CFTCMinAdjustableDateResolution cFTCMinAdjustableDateResolution;
	@Inject protected ProductForTrade productForTrade;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public Date evaluate(Trade input) {
		Date output = doEvaluate(input);
		
		return output;
	}

	protected abstract Date doEvaluate(Trade input);

	public static class CDEFirstExerciseDateRuleDefault extends CDEFirstExerciseDateRule {
		@Override
		protected Date doEvaluate(Trade input) {
			Date output = null;
			return assignOutput(output, input);
		}
		
		protected Date assignOutput(Date output, Trade input) {
			output = MapperS.of(input)
				.mapSingleToItem(trade -> MapperS.of(MapperS.of(productForTrade.evaluate(trade.get())).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<Payout>map("getPayout", economicTerms -> economicTerms.getPayout()).<OptionPayout>mapC("getOptionPayout", payout -> payout.getOptionPayout()).get()).<OptionExercise>map("getExerciseTerms", optionPayout -> optionPayout.getExerciseTerms()).<OptionStyle>map("getOptionStyle", optionExercise -> optionExercise.getOptionStyle())
					.mapSingleToItem(item -> {
						if (exists(item.<AmericanExercise>map("getAmericanExercise", optionStyle -> optionStyle.getAmericanExercise())).getOrDefault(false)) {
							if (exists(MapperS.of(cFTCAdjustableDateResolution.evaluate(item.<AmericanExercise>map("getAmericanExercise", optionStyle -> optionStyle.getAmericanExercise()).<AdjustableOrRelativeDate>map("getCommencementDate", americanExercise -> americanExercise.getCommencementDate()).<AdjustableDate>map("getAdjustableDate", adjustableOrRelativeDate -> adjustableOrRelativeDate.getAdjustableDate()).get()))).getOrDefault(false)) {
								return MapperS.of(cFTCAdjustableDateResolution.evaluate(item.<AmericanExercise>map("getAmericanExercise", optionStyle -> optionStyle.getAmericanExercise()).<AdjustableOrRelativeDate>map("getCommencementDate", americanExercise -> americanExercise.getCommencementDate()).<AdjustableDate>map("getAdjustableDate", adjustableOrRelativeDate -> adjustableOrRelativeDate.getAdjustableDate()).get()));
							}
							return trade.<FieldWithMetaDate>map("getTradeDate", _trade -> _trade.getTradeDate()).<Date>map("getValue", _f->_f.getValue());
						}
						if (exists(item.<EuropeanExercise>map("getEuropeanExercise", optionStyle -> optionStyle.getEuropeanExercise())).getOrDefault(false)) {
							if (exists(MapperS.of(cFTCMinAdjustableDateResolution.evaluate(item.<EuropeanExercise>map("getEuropeanExercise", optionStyle -> optionStyle.getEuropeanExercise()).<AdjustableOrRelativeDate>mapC("getExpirationDate", europeanExercise -> europeanExercise.getExpirationDate()).<AdjustableDate>map("getAdjustableDate", adjustableOrRelativeDate -> adjustableOrRelativeDate.getAdjustableDate()).getMulti()))).getOrDefault(false)) {
								return MapperS.of(cFTCMinAdjustableDateResolution.evaluate(item.<EuropeanExercise>map("getEuropeanExercise", optionStyle -> optionStyle.getEuropeanExercise()).<AdjustableOrRelativeDate>mapC("getExpirationDate", europeanExercise -> europeanExercise.getExpirationDate()).<AdjustableDate>map("getAdjustableDate", adjustableOrRelativeDate -> adjustableOrRelativeDate.getAdjustableDate()).getMulti()));
							}
							return MapperS.of(cFTCAdjustableDateResolution.evaluate(MapperS.of(productForTrade.evaluate(trade.get())).<ContractualProduct>map("getContractualProduct", product -> product.getContractualProduct()).<EconomicTerms>map("getEconomicTerms", contractualProduct -> contractualProduct.getEconomicTerms()).<AdjustableOrRelativeDate>map("getTerminationDate", economicTerms -> economicTerms.getTerminationDate()).<AdjustableDate>map("getAdjustableDate", adjustableOrRelativeDate -> adjustableOrRelativeDate.getAdjustableDate()).get()));
						}
						if (exists(item.<BermudaExercise>map("getBermudaExercise", optionStyle -> optionStyle.getBermudaExercise())).getOrDefault(false)) {
							return MapperC.<Date>of(cFTCAdjustableDatesResolution.evaluate(item.<BermudaExercise>map("getBermudaExercise", optionStyle -> optionStyle.getBermudaExercise()).<AdjustableOrRelativeDates>map("getBermudaExerciseDates", bermudaExercise -> bermudaExercise.getBermudaExerciseDates()).<AdjustableDates>map("getAdjustableDates", adjustableOrRelativeDates -> adjustableOrRelativeDates.getAdjustableDates()).get()))
								.min();
						}
						return MapperS.<Date>ofNull();
					})).get();
			
			return output;
		}
	}
}
