package drr.standards.iosco.cde.reports;

import cdm.base.math.NonNegativeQuantitySchedule;
import cdm.base.math.UnitType;
import cdm.base.math.metafields.ReferenceWithMetaNonNegativeQuantitySchedule;
import cdm.product.common.settlement.ResolvablePriceQuantity;
import cdm.product.template.ForwardPayout;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import com.rosetta.model.metafields.FieldWithMetaString;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDEForwardNotionalCurrencyRule.CDEForwardNotionalCurrencyRuleDefault.class)
public abstract class CDEForwardNotionalCurrencyRule implements ReportFunction<ForwardPayout, String> {

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(ForwardPayout input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(ForwardPayout input);

	public static class CDEForwardNotionalCurrencyRuleDefault extends CDEForwardNotionalCurrencyRule {
		@Override
		protected String doEvaluate(ForwardPayout input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, ForwardPayout input) {
			if (exists(MapperS.of(input).<ResolvablePriceQuantity>map("getPriceQuantity", payoutBase -> payoutBase.getPriceQuantity()).<ReferenceWithMetaNonNegativeQuantitySchedule>map("getQuantitySchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getQuantitySchedule()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue())).getOrDefault(false)) {
				output = MapperS.of(input).<ResolvablePriceQuantity>map("getPriceQuantity", payoutBase -> payoutBase.getPriceQuantity()).<ReferenceWithMetaNonNegativeQuantitySchedule>map("getQuantitySchedule", resolvablePriceQuantity -> resolvablePriceQuantity.getQuantitySchedule()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()).<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue()).get();
			} else {
				output = null;
			}
			
			return output;
		}
	}
}
