package drr.standards.iosco.cde.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.QuantityReportablePeriod;
import java.math.BigDecimal;


@ImplementedBy(CDENotionalQuantityScheduleValueRule.CDENotionalQuantityScheduleValueRuleDefault.class)
public abstract class CDENotionalQuantityScheduleValueRule implements ReportFunction<QuantityReportablePeriod, BigDecimal> {

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public BigDecimal evaluate(QuantityReportablePeriod input) {
		BigDecimal output = doEvaluate(input);
		
		return output;
	}

	protected abstract BigDecimal doEvaluate(QuantityReportablePeriod input);

	public static class CDENotionalQuantityScheduleValueRuleDefault extends CDENotionalQuantityScheduleValueRule {
		@Override
		protected BigDecimal doEvaluate(QuantityReportablePeriod input) {
			BigDecimal output = null;
			return assignOutput(output, input);
		}
		
		protected BigDecimal assignOutput(BigDecimal output, QuantityReportablePeriod input) {
			output = MapperS.of(input)
				.mapSingleToItem(item -> item.<BigDecimal>map("getPeriodValue", quantityReportablePeriod -> quantityReportablePeriod.getPeriodValue())).get();
			
			return output;
		}
	}
}
