package drr.standards.iosco.cde.reports;

import cdm.base.math.NonNegativeQuantity;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.Get_OptionPremiumOnEventDate;
import java.math.BigDecimal;
import javax.inject.Inject;


@ImplementedBy(CDEOptionPremiumAmountRule.CDEOptionPremiumAmountRuleDefault.class)
public abstract class CDEOptionPremiumAmountRule implements ReportFunction<TransactionReportInstruction, BigDecimal> {
	
	// RosettaFunction dependencies
	//
	@Inject protected Get_OptionPremiumOnEventDate get_OptionPremiumOnEventDate;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public BigDecimal evaluate(TransactionReportInstruction input) {
		BigDecimal output = doEvaluate(input);
		
		return output;
	}

	protected abstract BigDecimal doEvaluate(TransactionReportInstruction input);

	public static class CDEOptionPremiumAmountRuleDefault extends CDEOptionPremiumAmountRule {
		@Override
		protected BigDecimal doEvaluate(TransactionReportInstruction input) {
			BigDecimal output = null;
			return assignOutput(output, input);
		}
		
		protected BigDecimal assignOutput(BigDecimal output, TransactionReportInstruction input) {
			output = MapperS.of(get_OptionPremiumOnEventDate.evaluate(input)).<NonNegativeQuantity>map("getQuantity", transferBase -> transferBase.getQuantity()).<BigDecimal>map("getValue", measureBase -> measureBase.getValue()).get();
			
			return output;
		}
	}
}
