package drr.standards.iosco.cde.reports;

import cdm.base.datetime.AdjustableOrAdjustedOrRelativeDate;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.AdjustableOrAdjustedOrRelativeDateResolution;
import drr.regulation.common.functions.Get_OptionPremiumOnEventDate;
import javax.inject.Inject;


@ImplementedBy(CDEOptionPremiumPaymentDateRule.CDEOptionPremiumPaymentDateRuleDefault.class)
public abstract class CDEOptionPremiumPaymentDateRule implements ReportFunction<TransactionReportInstruction, Date> {
	
	// RosettaFunction dependencies
	//
	@Inject protected AdjustableOrAdjustedOrRelativeDateResolution adjustableOrAdjustedOrRelativeDateResolution;
	@Inject protected Get_OptionPremiumOnEventDate get_OptionPremiumOnEventDate;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public Date evaluate(TransactionReportInstruction input) {
		Date output = doEvaluate(input);
		
		return output;
	}

	protected abstract Date doEvaluate(TransactionReportInstruction input);

	public static class CDEOptionPremiumPaymentDateRuleDefault extends CDEOptionPremiumPaymentDateRule {
		@Override
		protected Date doEvaluate(TransactionReportInstruction input) {
			Date output = null;
			return assignOutput(output, input);
		}
		
		protected Date assignOutput(Date output, TransactionReportInstruction input) {
			output = adjustableOrAdjustedOrRelativeDateResolution.evaluate(MapperS.of(get_OptionPremiumOnEventDate.evaluate(input)).<AdjustableOrAdjustedOrRelativeDate>map("getSettlementDate", transferBase -> transferBase.getSettlementDate()).get());
			
			return output;
		}
	}
}
