package drr.standards.iosco.cde.reports;

import cdm.base.staticdata.party.Party;
import cdm.base.staticdata.party.PartyReferencePayerReceiver;
import cdm.base.staticdata.party.metafields.ReferenceWithMetaParty;
import cdm.event.common.Transfer;
import cdm.event.common.TransferState;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.functions.PartyLeiAndPersonByPartyRoles;
import javax.inject.Inject;


@ImplementedBy(CDEOtherPaymentReceiverRule.CDEOtherPaymentReceiverRuleDefault.class)
public abstract class CDEOtherPaymentReceiverRule implements ReportFunction<TransferState, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected PartyLeiAndPersonByPartyRoles partyLeiAndPersonByPartyRoles;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransferState input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransferState input);

	public static class CDEOtherPaymentReceiverRuleDefault extends CDEOtherPaymentReceiverRule {
		@Override
		protected String doEvaluate(TransferState input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransferState input) {
			final MapperS<Party> thenResult = MapperS.of(input)
				.mapSingleToItem(item -> item.<Transfer>map("getTransfer", transferState -> transferState.getTransfer()).<PartyReferencePayerReceiver>map("getPayerReceiver", transferBase -> transferBase.getPayerReceiver()).<ReferenceWithMetaParty>map("getReceiverPartyReference", partyReferencePayerReceiver -> partyReferencePayerReceiver.getReceiverPartyReference()).<Party>map("getValue", _f->_f.getValue()));
			output = thenResult
				.mapSingleToItem(item -> MapperS.of(partyLeiAndPersonByPartyRoles.evaluate(item.get()))).get();
			
			return output;
		}
	}
}
