package drr.standards.iosco.cde.reports;

import cdm.base.datetime.Frequency;
import cdm.product.asset.InterestRatePayout;
import cdm.product.common.schedule.PaymentDates;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;


@ImplementedBy(CDEPaymentFrequencyPeriodMultiplierRule.CDEPaymentFrequencyPeriodMultiplierRuleDefault.class)
public abstract class CDEPaymentFrequencyPeriodMultiplierRule implements ReportFunction<InterestRatePayout, Integer> {

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public Integer evaluate(InterestRatePayout input) {
		Integer output = doEvaluate(input);
		
		return output;
	}

	protected abstract Integer doEvaluate(InterestRatePayout input);

	public static class CDEPaymentFrequencyPeriodMultiplierRuleDefault extends CDEPaymentFrequencyPeriodMultiplierRule {
		@Override
		protected Integer doEvaluate(InterestRatePayout input) {
			Integer output = null;
			return assignOutput(output, input);
		}
		
		protected Integer assignOutput(Integer output, InterestRatePayout input) {
			output = MapperS.of(input)
				.mapSingleToItem(item -> item.<PaymentDates>map("getPaymentDates", interestRatePayout -> interestRatePayout.getPaymentDates()).<Frequency>map("getPaymentFrequency", paymentDates -> paymentDates.getPaymentFrequency()).<Integer>map("getPeriodMultiplier", frequency -> frequency.getPeriodMultiplier())).get();
			
			return output;
		}
	}
}
