package drr.standards.iosco.cde.reports;

import cdm.base.math.NonNegativeQuantitySchedule;
import cdm.base.math.UnitType;
import cdm.base.math.metafields.FieldWithMetaNonNegativeQuantitySchedule;
import cdm.observable.asset.Observable;
import cdm.product.common.settlement.PriceQuantity;
import cdm.product.template.TradableProduct;
import cdm.product.template.TradeLot;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperListOfLists;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import com.rosetta.model.metafields.FieldWithMetaString;
import drr.regulation.common.ReportableEvent;
import drr.regulation.common.functions.Contract_Price_Monetary;
import drr.regulation.common.functions.GetPriceNotation;
import drr.regulation.common.functions.IsFixedFloatZeroCouponSwapKnownAmount;
import drr.regulation.common.functions.ProductForEvent;
import drr.regulation.common.functions.TradableProductForEvent;
import drr.standards.iso.PriceNotationEnum;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDEPriceNotationEnumRule.CDEPriceNotationEnumRuleDefault.class)
public abstract class CDEPriceNotationEnumRule implements ReportFunction<ReportableEvent, PriceNotationEnum> {
	
	// RosettaFunction dependencies
	//
	@Inject protected Contract_Price_Monetary contract_Price_Monetary;
	@Inject protected GetPriceNotation getPriceNotation;
	@Inject protected IsFixedFloatZeroCouponSwapKnownAmount isFixedFloatZeroCouponSwapKnownAmount;
	@Inject protected ProductForEvent productForEvent;
	@Inject protected TradableProductForEvent tradableProductForEvent;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public PriceNotationEnum evaluate(ReportableEvent input) {
		PriceNotationEnum output = doEvaluate(input);
		
		return output;
	}

	protected abstract PriceNotationEnum doEvaluate(ReportableEvent input);

	public static class CDEPriceNotationEnumRuleDefault extends CDEPriceNotationEnumRule {
		@Override
		protected PriceNotationEnum doEvaluate(ReportableEvent input) {
			PriceNotationEnum output = null;
			return assignOutput(output, input);
		}
		
		protected PriceNotationEnum assignOutput(PriceNotationEnum output, ReportableEvent input) {
			final Boolean _boolean = isFixedFloatZeroCouponSwapKnownAmount.evaluate(productForEvent.evaluate(input));
			if ((_boolean == null ? false : _boolean)) {
				final MapperS<TradableProduct> thenResult0 = MapperS.of(tradableProductForEvent.evaluate(input));
				final MapperC<PriceQuantity> thenResult1 = thenResult0
					.mapSingleToList(item -> item.<TradeLot>mapC("getTradeLot", tradableProduct -> tradableProduct.getTradeLot()).<PriceQuantity>mapC("getPriceQuantity", tradeLot -> tradeLot.getPriceQuantity()));
				final MapperC<PriceQuantity> thenResult2 = thenResult1
					.filterItemNullSafe(item -> notExists(item.<Observable>map("getObservable", priceQuantity -> priceQuantity.getObservable())).get());
				final MapperListOfLists<NonNegativeQuantitySchedule> thenResult3 = thenResult2
					.mapItemToList(item -> item.<FieldWithMetaNonNegativeQuantitySchedule>mapC("getQuantity", priceQuantity -> priceQuantity.getQuantity()).<NonNegativeQuantitySchedule>map("getValue", _f->_f.getValue()));
				final MapperC<PriceNotationEnum> thenResult4 = thenResult3
					.mapListToItem(item -> {
						if (exists(item.<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue())).getOrDefault(false)) {
							return MapperS.of(PriceNotationEnum.MONETARY);
						}
						return MapperS.of(PriceNotationEnum.DECIMAL);
					});
				output = MapperS.of(thenResult4.get()).get();
			} else {
				output = MapperS.of(MapperC.of(contract_Price_Monetary.evaluate(input)).get())
					.mapSingleToItem(item -> MapperS.of(getPriceNotation.evaluate(item.get()))).get();
			}
			
			return output;
		}
	}
}
