package drr.standards.iosco.cde.reports;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import cdm.product.common.settlement.CashSettlementTerms;
import cdm.product.common.settlement.SettlementTerms;
import cdm.product.common.settlement.SettlementTypeEnum;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import com.rosetta.model.metafields.FieldWithMetaString;
import drr.regulation.common.functions.ConvertNonISOToISOCurrency;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDESettlementCurrencyRule.CDESettlementCurrencyRuleDefault.class)
public abstract class CDESettlementCurrencyRule implements ReportFunction<SettlementTerms, ISOCurrencyCodeEnum> {
	
	// RosettaFunction dependencies
	//
	@Inject protected ConvertNonISOToISOCurrency convertNonISOToISOCurrency;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public ISOCurrencyCodeEnum evaluate(SettlementTerms input) {
		ISOCurrencyCodeEnum output = doEvaluate(input);
		
		return output;
	}

	protected abstract ISOCurrencyCodeEnum doEvaluate(SettlementTerms input);

	public static class CDESettlementCurrencyRuleDefault extends CDESettlementCurrencyRule {
		@Override
		protected ISOCurrencyCodeEnum doEvaluate(SettlementTerms input) {
			ISOCurrencyCodeEnum output = null;
			return assignOutput(output, input);
		}
		
		protected ISOCurrencyCodeEnum assignOutput(ISOCurrencyCodeEnum output, SettlementTerms input) {
			final MapperS<String> thenResult = MapperS.of(input)
				.mapSingleToItem(item -> {
					if (areEqual(item.<SettlementTypeEnum>map("getSettlementType", settlementBase -> settlementBase.getSettlementType()), MapperS.of(SettlementTypeEnum.CASH), CardinalityOperator.All).or(exists(item.<CashSettlementTerms>mapC("getCashSettlementTerms", settlementTerms -> settlementTerms.getCashSettlementTerms()))).getOrDefault(false)) {
						return item.<FieldWithMetaString>map("getSettlementCurrency", settlementBase -> settlementBase.getSettlementCurrency()).<String>map("getValue", _f->_f.getValue());
					}
					return MapperS.<String>ofNull();
				});
			output = thenResult
				.mapSingleToItem(item -> MapperS.of(convertNonISOToISOCurrency.evaluate(item.get()))).get();
			
			return output;
		}
	}
}
