package drr.standards.iosco.cde.reports;

import cdm.base.staticdata.asset.common.ProductIdTypeEnum;
import cdm.base.staticdata.asset.common.ProductIdentifier;
import cdm.event.common.Trade;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.mapper.MapperC;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.ReportableEvent;
import drr.regulation.common.functions.GetBasketConstituentsProductIdentifier;
import drr.regulation.common.functions.TradeForEvent;
import java.util.ArrayList;
import java.util.List;
import javax.inject.Inject;


@ImplementedBy(CDESourceOfTheIdentifierOfTheBasketConstituentsRule.CDESourceOfTheIdentifierOfTheBasketConstituentsRuleDefault.class)
public abstract class CDESourceOfTheIdentifierOfTheBasketConstituentsRule implements ReportFunction<ReportableEvent, List<ProductIdTypeEnum>> {
	
	// RosettaFunction dependencies
	//
	@Inject protected GetBasketConstituentsProductIdentifier getBasketConstituentsProductIdentifier;
	@Inject protected TradeForEvent tradeForEvent;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public List<ProductIdTypeEnum> evaluate(ReportableEvent input) {
		List<ProductIdTypeEnum> output = doEvaluate(input);
		
		return output;
	}

	protected abstract List<ProductIdTypeEnum> doEvaluate(ReportableEvent input);

	public static class CDESourceOfTheIdentifierOfTheBasketConstituentsRuleDefault extends CDESourceOfTheIdentifierOfTheBasketConstituentsRule {
		@Override
		protected List<ProductIdTypeEnum> doEvaluate(ReportableEvent input) {
			List<ProductIdTypeEnum> output = new ArrayList<>();
			return assignOutput(output, input);
		}
		
		protected List<ProductIdTypeEnum> assignOutput(List<ProductIdTypeEnum> output, ReportableEvent input) {
			final MapperS<Trade> thenResult = MapperS.of(input)
				.mapSingleToItem(item -> MapperS.of(tradeForEvent.evaluate(item.get())));
			output = thenResult
				.mapSingleToList(item -> MapperC.<ProductIdentifier>of(getBasketConstituentsProductIdentifier.evaluate(item.get())).<ProductIdTypeEnum>map("getSource", productIdentifier -> productIdentifier.getSource())).getMulti();
			
			return output;
		}
	}
}
