package drr.standards.iosco.cde.reports;

import cdm.base.math.UnitType;
import cdm.observable.asset.PriceSchedule;
import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import com.rosetta.model.metafields.FieldWithMetaString;
import drr.regulation.common.TransactionReportInstruction;
import drr.regulation.common.functions.GetNotationString;
import drr.regulation.common.functions.GetPriceNotation;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDESpreadLeg2CurrencyRule.CDESpreadLeg2CurrencyRuleDefault.class)
public abstract class CDESpreadLeg2CurrencyRule implements ReportFunction<TransactionReportInstruction, String> {
	
	// RosettaFunction dependencies
	//
	@Inject protected CDESpreadLeg2Rule cDESpreadLeg2;
	@Inject protected GetNotationString getNotationString;
	@Inject protected GetPriceNotation getPriceNotation;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public String evaluate(TransactionReportInstruction input) {
		String output = doEvaluate(input);
		
		return output;
	}

	protected abstract String doEvaluate(TransactionReportInstruction input);

	public static class CDESpreadLeg2CurrencyRuleDefault extends CDESpreadLeg2CurrencyRule {
		@Override
		protected String doEvaluate(TransactionReportInstruction input) {
			String output = null;
			return assignOutput(output, input);
		}
		
		protected String assignOutput(String output, TransactionReportInstruction input) {
			final MapperS<PriceSchedule> thenResult = MapperS.of(cDESpreadLeg2.evaluate(input));
			output = thenResult
				.mapSingleToItem(item -> {
					if (areEqual(MapperS.of(getNotationString.evaluate(getPriceNotation.evaluate(item.get()))), MapperS.of("1"), CardinalityOperator.All).getOrDefault(false)) {
						return item.<UnitType>map("getUnit", measureBase -> measureBase.getUnit()).<FieldWithMetaString>map("getCurrency", unitType -> unitType.getCurrency()).<String>map("getValue", _f->_f.getValue());
					}
					return MapperS.<String>ofNull();
				}).get();
			
			return output;
		}
	}
}
