package drr.standards.iosco.cde.reports;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.reports.ReportFunction;
import drr.regulation.common.PriceReportablePeriod;
import drr.regulation.common.functions.MultiplyPrice;
import drr.regulation.esma.emir.refit.trade.PriceAndNotation;
import drr.standards.iso.PriceNotationEnum;
import java.math.BigDecimal;
import javax.inject.Inject;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

@ImplementedBy(CDEStrikePriceScheduleAmountFormatPercentageRule.CDEStrikePriceScheduleAmountFormatPercentageRuleDefault.class)
public abstract class CDEStrikePriceScheduleAmountFormatPercentageRule implements ReportFunction<PriceReportablePeriod, BigDecimal> {
	
	// RosettaFunction dependencies
	//
	@Inject protected MultiplyPrice multiplyPrice;

	/**
	* @param input 
	* @return output 
	*/
	@Override
	public BigDecimal evaluate(PriceReportablePeriod input) {
		BigDecimal output = doEvaluate(input);
		
		return output;
	}

	protected abstract BigDecimal doEvaluate(PriceReportablePeriod input);

	public static class CDEStrikePriceScheduleAmountFormatPercentageRuleDefault extends CDEStrikePriceScheduleAmountFormatPercentageRule {
		@Override
		protected BigDecimal doEvaluate(PriceReportablePeriod input) {
			BigDecimal output = null;
			return assignOutput(output, input);
		}
		
		protected BigDecimal assignOutput(BigDecimal output, PriceReportablePeriod input) {
			final MapperS<PriceAndNotation> thenResult = MapperS.of(input)
				.mapSingleToItem(item -> item.<PriceAndNotation>map("getPeriodValue", priceReportablePeriod -> priceReportablePeriod.getPeriodValue()));
			final MapperS<BigDecimal> ifThenElseResult;
			if (areEqual(thenResult.<PriceNotationEnum>map("getPriceNotation", priceAndNotation -> priceAndNotation.getPriceNotation()), MapperS.of(PriceNotationEnum.PERCENTAGE), CardinalityOperator.All).getOrDefault(false)) {
				ifThenElseResult = MapperS.of(multiplyPrice.evaluate(thenResult.<BigDecimal>map("getPricePercentage", priceAndNotation -> priceAndNotation.getPricePercentage()).get(), thenResult.<PriceNotationEnum>map("getPriceNotation", priceAndNotation -> priceAndNotation.getPriceNotation()).get()));
			} else {
				ifThenElseResult = MapperS.<BigDecimal>ofNull();
			}
			output = ifThenElseResult.get();
			
			return output;
		}
	}
}
