package drr.standards.iso;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import com.rosetta.model.lib.records.Date;
import drr.standards.iso.MicData;
import drr.standards.iso.MicData.MicDataBuilder;
import drr.standards.iso.MicData.MicDataBuilderImpl;
import drr.standards.iso.MicData.MicDataImpl;
import drr.standards.iso.MicMarketCategoryEnum;
import drr.standards.iso.MicTypeEnum;
import drr.standards.iso.meta.MicDataMeta;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="MicData", builder=MicData.MicDataBuilderImpl.class, version="4.2.1")
public interface MicData extends RosettaModelObject {

	MicDataMeta metaData = new MicDataMeta();

	/*********************** Getter Methods  ***********************/
	String getMic();
	String getOperatingMic();
	MicTypeEnum getMicType();
	String getNameInstitutionDescription();
	String getLegalEntityName();
	MicMarketCategoryEnum getMarketCategory();
	String getAcronym();
	String getCountryCode();
	String getCity();
	String getWebsite();
	String getStatus();
	Date getCreationDate();
	Date getLastUpdateDate();
	Date getLastValidationDate();
	Date getExpiryDate();

	/*********************** Build Methods  ***********************/
	MicData build();
	
	MicData.MicDataBuilder toBuilder();
	
	static MicData.MicDataBuilder builder() {
		return new MicData.MicDataBuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends MicData> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends MicData> getType() {
		return MicData.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("mic"), String.class, getMic(), this);
		processor.processBasic(path.newSubPath("operatingMic"), String.class, getOperatingMic(), this);
		processor.processBasic(path.newSubPath("micType"), MicTypeEnum.class, getMicType(), this);
		processor.processBasic(path.newSubPath("nameInstitutionDescription"), String.class, getNameInstitutionDescription(), this);
		processor.processBasic(path.newSubPath("legalEntityName"), String.class, getLegalEntityName(), this);
		processor.processBasic(path.newSubPath("marketCategory"), MicMarketCategoryEnum.class, getMarketCategory(), this);
		processor.processBasic(path.newSubPath("acronym"), String.class, getAcronym(), this);
		processor.processBasic(path.newSubPath("countryCode"), String.class, getCountryCode(), this);
		processor.processBasic(path.newSubPath("city"), String.class, getCity(), this);
		processor.processBasic(path.newSubPath("website"), String.class, getWebsite(), this);
		processor.processBasic(path.newSubPath("status"), String.class, getStatus(), this);
		processor.processBasic(path.newSubPath("creationDate"), Date.class, getCreationDate(), this);
		processor.processBasic(path.newSubPath("lastUpdateDate"), Date.class, getLastUpdateDate(), this);
		processor.processBasic(path.newSubPath("lastValidationDate"), Date.class, getLastValidationDate(), this);
		processor.processBasic(path.newSubPath("expiryDate"), Date.class, getExpiryDate(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface MicDataBuilder extends MicData, RosettaModelObjectBuilder {
		MicData.MicDataBuilder setMic(String mic);
		MicData.MicDataBuilder setOperatingMic(String operatingMic);
		MicData.MicDataBuilder setMicType(MicTypeEnum micType);
		MicData.MicDataBuilder setNameInstitutionDescription(String nameInstitutionDescription);
		MicData.MicDataBuilder setLegalEntityName(String legalEntityName);
		MicData.MicDataBuilder setMarketCategory(MicMarketCategoryEnum marketCategory);
		MicData.MicDataBuilder setAcronym(String acronym);
		MicData.MicDataBuilder setCountryCode(String countryCode);
		MicData.MicDataBuilder setCity(String city);
		MicData.MicDataBuilder setWebsite(String website);
		MicData.MicDataBuilder setStatus(String status);
		MicData.MicDataBuilder setCreationDate(Date creationDate);
		MicData.MicDataBuilder setLastUpdateDate(Date lastUpdateDate);
		MicData.MicDataBuilder setLastValidationDate(Date lastValidationDate);
		MicData.MicDataBuilder setExpiryDate(Date expiryDate);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("mic"), String.class, getMic(), this);
			processor.processBasic(path.newSubPath("operatingMic"), String.class, getOperatingMic(), this);
			processor.processBasic(path.newSubPath("micType"), MicTypeEnum.class, getMicType(), this);
			processor.processBasic(path.newSubPath("nameInstitutionDescription"), String.class, getNameInstitutionDescription(), this);
			processor.processBasic(path.newSubPath("legalEntityName"), String.class, getLegalEntityName(), this);
			processor.processBasic(path.newSubPath("marketCategory"), MicMarketCategoryEnum.class, getMarketCategory(), this);
			processor.processBasic(path.newSubPath("acronym"), String.class, getAcronym(), this);
			processor.processBasic(path.newSubPath("countryCode"), String.class, getCountryCode(), this);
			processor.processBasic(path.newSubPath("city"), String.class, getCity(), this);
			processor.processBasic(path.newSubPath("website"), String.class, getWebsite(), this);
			processor.processBasic(path.newSubPath("status"), String.class, getStatus(), this);
			processor.processBasic(path.newSubPath("creationDate"), Date.class, getCreationDate(), this);
			processor.processBasic(path.newSubPath("lastUpdateDate"), Date.class, getLastUpdateDate(), this);
			processor.processBasic(path.newSubPath("lastValidationDate"), Date.class, getLastValidationDate(), this);
			processor.processBasic(path.newSubPath("expiryDate"), Date.class, getExpiryDate(), this);
		}
		

		MicData.MicDataBuilder prune();
	}

	/*********************** Immutable Implementation of MicData  ***********************/
	class MicDataImpl implements MicData {
		private final String mic;
		private final String operatingMic;
		private final MicTypeEnum micType;
		private final String nameInstitutionDescription;
		private final String legalEntityName;
		private final MicMarketCategoryEnum marketCategory;
		private final String acronym;
		private final String countryCode;
		private final String city;
		private final String website;
		private final String status;
		private final Date creationDate;
		private final Date lastUpdateDate;
		private final Date lastValidationDate;
		private final Date expiryDate;
		
		protected MicDataImpl(MicData.MicDataBuilder builder) {
			this.mic = builder.getMic();
			this.operatingMic = builder.getOperatingMic();
			this.micType = builder.getMicType();
			this.nameInstitutionDescription = builder.getNameInstitutionDescription();
			this.legalEntityName = builder.getLegalEntityName();
			this.marketCategory = builder.getMarketCategory();
			this.acronym = builder.getAcronym();
			this.countryCode = builder.getCountryCode();
			this.city = builder.getCity();
			this.website = builder.getWebsite();
			this.status = builder.getStatus();
			this.creationDate = builder.getCreationDate();
			this.lastUpdateDate = builder.getLastUpdateDate();
			this.lastValidationDate = builder.getLastValidationDate();
			this.expiryDate = builder.getExpiryDate();
		}
		
		@Override
		@RosettaAttribute("mic")
		public String getMic() {
			return mic;
		}
		
		@Override
		@RosettaAttribute("operatingMic")
		public String getOperatingMic() {
			return operatingMic;
		}
		
		@Override
		@RosettaAttribute("micType")
		public MicTypeEnum getMicType() {
			return micType;
		}
		
		@Override
		@RosettaAttribute("nameInstitutionDescription")
		public String getNameInstitutionDescription() {
			return nameInstitutionDescription;
		}
		
		@Override
		@RosettaAttribute("legalEntityName")
		public String getLegalEntityName() {
			return legalEntityName;
		}
		
		@Override
		@RosettaAttribute("marketCategory")
		public MicMarketCategoryEnum getMarketCategory() {
			return marketCategory;
		}
		
		@Override
		@RosettaAttribute("acronym")
		public String getAcronym() {
			return acronym;
		}
		
		@Override
		@RosettaAttribute("countryCode")
		public String getCountryCode() {
			return countryCode;
		}
		
		@Override
		@RosettaAttribute("city")
		public String getCity() {
			return city;
		}
		
		@Override
		@RosettaAttribute("website")
		public String getWebsite() {
			return website;
		}
		
		@Override
		@RosettaAttribute("status")
		public String getStatus() {
			return status;
		}
		
		@Override
		@RosettaAttribute("creationDate")
		public Date getCreationDate() {
			return creationDate;
		}
		
		@Override
		@RosettaAttribute("lastUpdateDate")
		public Date getLastUpdateDate() {
			return lastUpdateDate;
		}
		
		@Override
		@RosettaAttribute("lastValidationDate")
		public Date getLastValidationDate() {
			return lastValidationDate;
		}
		
		@Override
		@RosettaAttribute("expiryDate")
		public Date getExpiryDate() {
			return expiryDate;
		}
		
		@Override
		public MicData build() {
			return this;
		}
		
		@Override
		public MicData.MicDataBuilder toBuilder() {
			MicData.MicDataBuilder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(MicData.MicDataBuilder builder) {
			ofNullable(getMic()).ifPresent(builder::setMic);
			ofNullable(getOperatingMic()).ifPresent(builder::setOperatingMic);
			ofNullable(getMicType()).ifPresent(builder::setMicType);
			ofNullable(getNameInstitutionDescription()).ifPresent(builder::setNameInstitutionDescription);
			ofNullable(getLegalEntityName()).ifPresent(builder::setLegalEntityName);
			ofNullable(getMarketCategory()).ifPresent(builder::setMarketCategory);
			ofNullable(getAcronym()).ifPresent(builder::setAcronym);
			ofNullable(getCountryCode()).ifPresent(builder::setCountryCode);
			ofNullable(getCity()).ifPresent(builder::setCity);
			ofNullable(getWebsite()).ifPresent(builder::setWebsite);
			ofNullable(getStatus()).ifPresent(builder::setStatus);
			ofNullable(getCreationDate()).ifPresent(builder::setCreationDate);
			ofNullable(getLastUpdateDate()).ifPresent(builder::setLastUpdateDate);
			ofNullable(getLastValidationDate()).ifPresent(builder::setLastValidationDate);
			ofNullable(getExpiryDate()).ifPresent(builder::setExpiryDate);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			MicData _that = getType().cast(o);
		
			if (!Objects.equals(mic, _that.getMic())) return false;
			if (!Objects.equals(operatingMic, _that.getOperatingMic())) return false;
			if (!Objects.equals(micType, _that.getMicType())) return false;
			if (!Objects.equals(nameInstitutionDescription, _that.getNameInstitutionDescription())) return false;
			if (!Objects.equals(legalEntityName, _that.getLegalEntityName())) return false;
			if (!Objects.equals(marketCategory, _that.getMarketCategory())) return false;
			if (!Objects.equals(acronym, _that.getAcronym())) return false;
			if (!Objects.equals(countryCode, _that.getCountryCode())) return false;
			if (!Objects.equals(city, _that.getCity())) return false;
			if (!Objects.equals(website, _that.getWebsite())) return false;
			if (!Objects.equals(status, _that.getStatus())) return false;
			if (!Objects.equals(creationDate, _that.getCreationDate())) return false;
			if (!Objects.equals(lastUpdateDate, _that.getLastUpdateDate())) return false;
			if (!Objects.equals(lastValidationDate, _that.getLastValidationDate())) return false;
			if (!Objects.equals(expiryDate, _that.getExpiryDate())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (mic != null ? mic.hashCode() : 0);
			_result = 31 * _result + (operatingMic != null ? operatingMic.hashCode() : 0);
			_result = 31 * _result + (micType != null ? micType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (nameInstitutionDescription != null ? nameInstitutionDescription.hashCode() : 0);
			_result = 31 * _result + (legalEntityName != null ? legalEntityName.hashCode() : 0);
			_result = 31 * _result + (marketCategory != null ? marketCategory.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (acronym != null ? acronym.hashCode() : 0);
			_result = 31 * _result + (countryCode != null ? countryCode.hashCode() : 0);
			_result = 31 * _result + (city != null ? city.hashCode() : 0);
			_result = 31 * _result + (website != null ? website.hashCode() : 0);
			_result = 31 * _result + (status != null ? status.hashCode() : 0);
			_result = 31 * _result + (creationDate != null ? creationDate.hashCode() : 0);
			_result = 31 * _result + (lastUpdateDate != null ? lastUpdateDate.hashCode() : 0);
			_result = 31 * _result + (lastValidationDate != null ? lastValidationDate.hashCode() : 0);
			_result = 31 * _result + (expiryDate != null ? expiryDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MicData {" +
				"mic=" + this.mic + ", " +
				"operatingMic=" + this.operatingMic + ", " +
				"micType=" + this.micType + ", " +
				"nameInstitutionDescription=" + this.nameInstitutionDescription + ", " +
				"legalEntityName=" + this.legalEntityName + ", " +
				"marketCategory=" + this.marketCategory + ", " +
				"acronym=" + this.acronym + ", " +
				"countryCode=" + this.countryCode + ", " +
				"city=" + this.city + ", " +
				"website=" + this.website + ", " +
				"status=" + this.status + ", " +
				"creationDate=" + this.creationDate + ", " +
				"lastUpdateDate=" + this.lastUpdateDate + ", " +
				"lastValidationDate=" + this.lastValidationDate + ", " +
				"expiryDate=" + this.expiryDate +
			'}';
		}
	}

	/*********************** Builder Implementation of MicData  ***********************/
	class MicDataBuilderImpl implements MicData.MicDataBuilder {
	
		protected String mic;
		protected String operatingMic;
		protected MicTypeEnum micType;
		protected String nameInstitutionDescription;
		protected String legalEntityName;
		protected MicMarketCategoryEnum marketCategory;
		protected String acronym;
		protected String countryCode;
		protected String city;
		protected String website;
		protected String status;
		protected Date creationDate;
		protected Date lastUpdateDate;
		protected Date lastValidationDate;
		protected Date expiryDate;
	
		public MicDataBuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("mic")
		public String getMic() {
			return mic;
		}
		
		@Override
		@RosettaAttribute("operatingMic")
		public String getOperatingMic() {
			return operatingMic;
		}
		
		@Override
		@RosettaAttribute("micType")
		public MicTypeEnum getMicType() {
			return micType;
		}
		
		@Override
		@RosettaAttribute("nameInstitutionDescription")
		public String getNameInstitutionDescription() {
			return nameInstitutionDescription;
		}
		
		@Override
		@RosettaAttribute("legalEntityName")
		public String getLegalEntityName() {
			return legalEntityName;
		}
		
		@Override
		@RosettaAttribute("marketCategory")
		public MicMarketCategoryEnum getMarketCategory() {
			return marketCategory;
		}
		
		@Override
		@RosettaAttribute("acronym")
		public String getAcronym() {
			return acronym;
		}
		
		@Override
		@RosettaAttribute("countryCode")
		public String getCountryCode() {
			return countryCode;
		}
		
		@Override
		@RosettaAttribute("city")
		public String getCity() {
			return city;
		}
		
		@Override
		@RosettaAttribute("website")
		public String getWebsite() {
			return website;
		}
		
		@Override
		@RosettaAttribute("status")
		public String getStatus() {
			return status;
		}
		
		@Override
		@RosettaAttribute("creationDate")
		public Date getCreationDate() {
			return creationDate;
		}
		
		@Override
		@RosettaAttribute("lastUpdateDate")
		public Date getLastUpdateDate() {
			return lastUpdateDate;
		}
		
		@Override
		@RosettaAttribute("lastValidationDate")
		public Date getLastValidationDate() {
			return lastValidationDate;
		}
		
		@Override
		@RosettaAttribute("expiryDate")
		public Date getExpiryDate() {
			return expiryDate;
		}
		
	
		@Override
		@RosettaAttribute("mic")
		public MicData.MicDataBuilder setMic(String mic) {
			this.mic = mic==null?null:mic;
			return this;
		}
		@Override
		@RosettaAttribute("operatingMic")
		public MicData.MicDataBuilder setOperatingMic(String operatingMic) {
			this.operatingMic = operatingMic==null?null:operatingMic;
			return this;
		}
		@Override
		@RosettaAttribute("micType")
		public MicData.MicDataBuilder setMicType(MicTypeEnum micType) {
			this.micType = micType==null?null:micType;
			return this;
		}
		@Override
		@RosettaAttribute("nameInstitutionDescription")
		public MicData.MicDataBuilder setNameInstitutionDescription(String nameInstitutionDescription) {
			this.nameInstitutionDescription = nameInstitutionDescription==null?null:nameInstitutionDescription;
			return this;
		}
		@Override
		@RosettaAttribute("legalEntityName")
		public MicData.MicDataBuilder setLegalEntityName(String legalEntityName) {
			this.legalEntityName = legalEntityName==null?null:legalEntityName;
			return this;
		}
		@Override
		@RosettaAttribute("marketCategory")
		public MicData.MicDataBuilder setMarketCategory(MicMarketCategoryEnum marketCategory) {
			this.marketCategory = marketCategory==null?null:marketCategory;
			return this;
		}
		@Override
		@RosettaAttribute("acronym")
		public MicData.MicDataBuilder setAcronym(String acronym) {
			this.acronym = acronym==null?null:acronym;
			return this;
		}
		@Override
		@RosettaAttribute("countryCode")
		public MicData.MicDataBuilder setCountryCode(String countryCode) {
			this.countryCode = countryCode==null?null:countryCode;
			return this;
		}
		@Override
		@RosettaAttribute("city")
		public MicData.MicDataBuilder setCity(String city) {
			this.city = city==null?null:city;
			return this;
		}
		@Override
		@RosettaAttribute("website")
		public MicData.MicDataBuilder setWebsite(String website) {
			this.website = website==null?null:website;
			return this;
		}
		@Override
		@RosettaAttribute("status")
		public MicData.MicDataBuilder setStatus(String status) {
			this.status = status==null?null:status;
			return this;
		}
		@Override
		@RosettaAttribute("creationDate")
		public MicData.MicDataBuilder setCreationDate(Date creationDate) {
			this.creationDate = creationDate==null?null:creationDate;
			return this;
		}
		@Override
		@RosettaAttribute("lastUpdateDate")
		public MicData.MicDataBuilder setLastUpdateDate(Date lastUpdateDate) {
			this.lastUpdateDate = lastUpdateDate==null?null:lastUpdateDate;
			return this;
		}
		@Override
		@RosettaAttribute("lastValidationDate")
		public MicData.MicDataBuilder setLastValidationDate(Date lastValidationDate) {
			this.lastValidationDate = lastValidationDate==null?null:lastValidationDate;
			return this;
		}
		@Override
		@RosettaAttribute("expiryDate")
		public MicData.MicDataBuilder setExpiryDate(Date expiryDate) {
			this.expiryDate = expiryDate==null?null:expiryDate;
			return this;
		}
		
		@Override
		public MicData build() {
			return new MicData.MicDataImpl(this);
		}
		
		@Override
		public MicData.MicDataBuilder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public MicData.MicDataBuilder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getMic()!=null) return true;
			if (getOperatingMic()!=null) return true;
			if (getMicType()!=null) return true;
			if (getNameInstitutionDescription()!=null) return true;
			if (getLegalEntityName()!=null) return true;
			if (getMarketCategory()!=null) return true;
			if (getAcronym()!=null) return true;
			if (getCountryCode()!=null) return true;
			if (getCity()!=null) return true;
			if (getWebsite()!=null) return true;
			if (getStatus()!=null) return true;
			if (getCreationDate()!=null) return true;
			if (getLastUpdateDate()!=null) return true;
			if (getLastValidationDate()!=null) return true;
			if (getExpiryDate()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public MicData.MicDataBuilder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			MicData.MicDataBuilder o = (MicData.MicDataBuilder) other;
			
			
			merger.mergeBasic(getMic(), o.getMic(), this::setMic);
			merger.mergeBasic(getOperatingMic(), o.getOperatingMic(), this::setOperatingMic);
			merger.mergeBasic(getMicType(), o.getMicType(), this::setMicType);
			merger.mergeBasic(getNameInstitutionDescription(), o.getNameInstitutionDescription(), this::setNameInstitutionDescription);
			merger.mergeBasic(getLegalEntityName(), o.getLegalEntityName(), this::setLegalEntityName);
			merger.mergeBasic(getMarketCategory(), o.getMarketCategory(), this::setMarketCategory);
			merger.mergeBasic(getAcronym(), o.getAcronym(), this::setAcronym);
			merger.mergeBasic(getCountryCode(), o.getCountryCode(), this::setCountryCode);
			merger.mergeBasic(getCity(), o.getCity(), this::setCity);
			merger.mergeBasic(getWebsite(), o.getWebsite(), this::setWebsite);
			merger.mergeBasic(getStatus(), o.getStatus(), this::setStatus);
			merger.mergeBasic(getCreationDate(), o.getCreationDate(), this::setCreationDate);
			merger.mergeBasic(getLastUpdateDate(), o.getLastUpdateDate(), this::setLastUpdateDate);
			merger.mergeBasic(getLastValidationDate(), o.getLastValidationDate(), this::setLastValidationDate);
			merger.mergeBasic(getExpiryDate(), o.getExpiryDate(), this::setExpiryDate);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			MicData _that = getType().cast(o);
		
			if (!Objects.equals(mic, _that.getMic())) return false;
			if (!Objects.equals(operatingMic, _that.getOperatingMic())) return false;
			if (!Objects.equals(micType, _that.getMicType())) return false;
			if (!Objects.equals(nameInstitutionDescription, _that.getNameInstitutionDescription())) return false;
			if (!Objects.equals(legalEntityName, _that.getLegalEntityName())) return false;
			if (!Objects.equals(marketCategory, _that.getMarketCategory())) return false;
			if (!Objects.equals(acronym, _that.getAcronym())) return false;
			if (!Objects.equals(countryCode, _that.getCountryCode())) return false;
			if (!Objects.equals(city, _that.getCity())) return false;
			if (!Objects.equals(website, _that.getWebsite())) return false;
			if (!Objects.equals(status, _that.getStatus())) return false;
			if (!Objects.equals(creationDate, _that.getCreationDate())) return false;
			if (!Objects.equals(lastUpdateDate, _that.getLastUpdateDate())) return false;
			if (!Objects.equals(lastValidationDate, _that.getLastValidationDate())) return false;
			if (!Objects.equals(expiryDate, _that.getExpiryDate())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (mic != null ? mic.hashCode() : 0);
			_result = 31 * _result + (operatingMic != null ? operatingMic.hashCode() : 0);
			_result = 31 * _result + (micType != null ? micType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (nameInstitutionDescription != null ? nameInstitutionDescription.hashCode() : 0);
			_result = 31 * _result + (legalEntityName != null ? legalEntityName.hashCode() : 0);
			_result = 31 * _result + (marketCategory != null ? marketCategory.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (acronym != null ? acronym.hashCode() : 0);
			_result = 31 * _result + (countryCode != null ? countryCode.hashCode() : 0);
			_result = 31 * _result + (city != null ? city.hashCode() : 0);
			_result = 31 * _result + (website != null ? website.hashCode() : 0);
			_result = 31 * _result + (status != null ? status.hashCode() : 0);
			_result = 31 * _result + (creationDate != null ? creationDate.hashCode() : 0);
			_result = 31 * _result + (lastUpdateDate != null ? lastUpdateDate.hashCode() : 0);
			_result = 31 * _result + (lastValidationDate != null ? lastValidationDate.hashCode() : 0);
			_result = 31 * _result + (expiryDate != null ? expiryDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MicDataBuilder {" +
				"mic=" + this.mic + ", " +
				"operatingMic=" + this.operatingMic + ", " +
				"micType=" + this.micType + ", " +
				"nameInstitutionDescription=" + this.nameInstitutionDescription + ", " +
				"legalEntityName=" + this.legalEntityName + ", " +
				"marketCategory=" + this.marketCategory + ", " +
				"acronym=" + this.acronym + ", " +
				"countryCode=" + this.countryCode + ", " +
				"city=" + this.city + ", " +
				"website=" + this.website + ", " +
				"status=" + this.status + ", " +
				"creationDate=" + this.creationDate + ", " +
				"lastUpdateDate=" + this.lastUpdateDate + ", " +
				"lastValidationDate=" + this.lastValidationDate + ", " +
				"expiryDate=" + this.expiryDate +
			'}';
		}
	}
}
