package drr.standards.iso;

import com.rosetta.model.lib.annotations.RosettaEnum;
import com.rosetta.model.lib.annotations.RosettaEnumValue;
import java.util.Collections;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;


/**
 * @version 4.2.1
 */
@RosettaEnum("PriceNotationEnum")
public enum PriceNotationEnum {

	@RosettaEnumValue(value = "Monetary") MONETARY("Monetary"),
	
	@RosettaEnumValue(value = "Percentage") PERCENTAGE("Percentage"),
	
	@RosettaEnumValue(value = "Decimal") DECIMAL("Decimal"),
	
	@RosettaEnumValue(value = "Basis") BASIS("Basis")
;
	private static Map<String, PriceNotationEnum> values;
	static {
        Map<String, PriceNotationEnum> map = new ConcurrentHashMap<>();
		for (PriceNotationEnum instance : PriceNotationEnum.values()) {
			map.put(instance.toDisplayString(), instance);
		}
		values = Collections.unmodifiableMap(map);
    }

	private final String rosettaName;
	private final String displayName;

	PriceNotationEnum(String rosettaName) {
		this(rosettaName, null);
	}

	PriceNotationEnum(String rosettaName, String displayName) {
		this.rosettaName = rosettaName;
		this.displayName = displayName;
	}

	public static PriceNotationEnum fromDisplayName(String name) {
		PriceNotationEnum value = values.get(name);
		if (value == null) {
			throw new IllegalArgumentException("No enum constant with display name \"" + name + "\".");
		}
		return value;
	}

	@Override
	public String toString() {
		return toDisplayString();
	}

	public String toDisplayString() {
		return displayName != null ?  displayName : rosettaName;
	}
}
