package drr.standards.iso.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.CardinalityOperator;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import drr.standards.iso.MicData;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 4.2.1
 */
@RosettaDataRule("MicDataMICIsNotXOFFOrXXXX")
@ImplementedBy(MicDataMICIsNotXOFFOrXXXX.Default.class)
public interface MicDataMICIsNotXOFFOrXXXX extends Validator<MicData> {
	
	String NAME = "MicDataMICIsNotXOFFOrXXXX";
	String DEFINITION = "if mic exists and mic <> \"XXXX\" and mic <> \"XOFF\" then nameInstitutionDescription exists and countryCode exists and city exists and status exists and creationDate exists";
	
	ValidationResult<MicData> validate(RosettaPath path, MicData micData);
	
	class Default implements MicDataMICIsNotXOFFOrXXXX {
	
		@Override
		public ValidationResult<MicData> validate(RosettaPath path, MicData micData) {
			ComparisonResult result = executeDataRule(micData);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MicData", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "MicData", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(MicData micData) {
			try {
				if (exists(MapperS.of(micData).<String>map("getMic", _micData -> _micData.getMic())).and(notEqual(MapperS.of(micData).<String>map("getMic", _micData -> _micData.getMic()), MapperS.of("XXXX"), CardinalityOperator.Any)).and(notEqual(MapperS.of(micData).<String>map("getMic", _micData -> _micData.getMic()), MapperS.of("XOFF"), CardinalityOperator.Any)).getOrDefault(false)) {
					return exists(MapperS.of(micData).<String>map("getNameInstitutionDescription", _micData -> _micData.getNameInstitutionDescription())).and(exists(MapperS.of(micData).<String>map("getCountryCode", _micData -> _micData.getCountryCode()))).and(exists(MapperS.of(micData).<String>map("getCity", _micData -> _micData.getCity()))).and(exists(MapperS.of(micData).<String>map("getStatus", _micData -> _micData.getStatus()))).and(exists(MapperS.of(micData).<Date>map("getCreationDate", _micData -> _micData.getCreationDate())));
				}
				return ComparisonResult.successEmptyOperand("");
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements MicDataMICIsNotXOFFOrXXXX {
	
		@Override
		public ValidationResult<MicData> validate(RosettaPath path, MicData micData) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "MicData", path, DEFINITION);
		}
	}
}
