package iso20022.auth030.esma;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import iso20022.auth030.esma.AgriculturalCommodityOther2;
import iso20022.auth030.esma.AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder;
import iso20022.auth030.esma.AgriculturalCommodityOther2.AgriculturalCommodityOther2BuilderImpl;
import iso20022.auth030.esma.AgriculturalCommodityOther2.AgriculturalCommodityOther2Impl;
import iso20022.auth030.esma.AssetClassProductType1Code;
import iso20022.auth030.esma.AssetClassSubProductType49Code;
import iso20022.auth030.esma.meta.AgriculturalCommodityOther2Meta;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * Other agricultural commodity derivative.
 * @version ${project.version}
 */
@RosettaDataType(value="AgriculturalCommodityOther2", builder=AgriculturalCommodityOther2.AgriculturalCommodityOther2BuilderImpl.class, version="${project.version}")
public interface AgriculturalCommodityOther2 extends RosettaModelObject {

	AgriculturalCommodityOther2Meta metaData = new AgriculturalCommodityOther2Meta();

	/*********************** Getter Methods  ***********************/
	/**
	 * Base product for the underlying asset class as specified in the classification of commodities derivatives table.
	 */
	AssetClassProductType1Code getBasePdct();
	/**
	 * Sub-product for the underlying asset class.
	 */
	AssetClassSubProductType49Code getSubPdct();

	/*********************** Build Methods  ***********************/
	AgriculturalCommodityOther2 build();
	
	AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder toBuilder();
	
	static AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder builder() {
		return new AgriculturalCommodityOther2.AgriculturalCommodityOther2BuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends AgriculturalCommodityOther2> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends AgriculturalCommodityOther2> getType() {
		return AgriculturalCommodityOther2.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("basePdct"), AssetClassProductType1Code.class, getBasePdct(), this);
		processor.processBasic(path.newSubPath("subPdct"), AssetClassSubProductType49Code.class, getSubPdct(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface AgriculturalCommodityOther2Builder extends AgriculturalCommodityOther2, RosettaModelObjectBuilder {
		AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder setBasePdct(AssetClassProductType1Code basePdct);
		AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder setSubPdct(AssetClassSubProductType49Code subPdct);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("basePdct"), AssetClassProductType1Code.class, getBasePdct(), this);
			processor.processBasic(path.newSubPath("subPdct"), AssetClassSubProductType49Code.class, getSubPdct(), this);
		}
		

		AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder prune();
	}

	/*********************** Immutable Implementation of AgriculturalCommodityOther2  ***********************/
	class AgriculturalCommodityOther2Impl implements AgriculturalCommodityOther2 {
		private final AssetClassProductType1Code basePdct;
		private final AssetClassSubProductType49Code subPdct;
		
		protected AgriculturalCommodityOther2Impl(AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder builder) {
			this.basePdct = builder.getBasePdct();
			this.subPdct = builder.getSubPdct();
		}
		
		@Override
		@RosettaAttribute("basePdct")
		public AssetClassProductType1Code getBasePdct() {
			return basePdct;
		}
		
		@Override
		@RosettaAttribute("subPdct")
		public AssetClassSubProductType49Code getSubPdct() {
			return subPdct;
		}
		
		@Override
		public AgriculturalCommodityOther2 build() {
			return this;
		}
		
		@Override
		public AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder toBuilder() {
			AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder builder) {
			ofNullable(getBasePdct()).ifPresent(builder::setBasePdct);
			ofNullable(getSubPdct()).ifPresent(builder::setSubPdct);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			AgriculturalCommodityOther2 _that = getType().cast(o);
		
			if (!Objects.equals(basePdct, _that.getBasePdct())) return false;
			if (!Objects.equals(subPdct, _that.getSubPdct())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (basePdct != null ? basePdct.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (subPdct != null ? subPdct.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AgriculturalCommodityOther2 {" +
				"basePdct=" + this.basePdct + ", " +
				"subPdct=" + this.subPdct +
			'}';
		}
	}

	/*********************** Builder Implementation of AgriculturalCommodityOther2  ***********************/
	class AgriculturalCommodityOther2BuilderImpl implements AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder {
	
		protected AssetClassProductType1Code basePdct;
		protected AssetClassSubProductType49Code subPdct;
	
		public AgriculturalCommodityOther2BuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("basePdct")
		public AssetClassProductType1Code getBasePdct() {
			return basePdct;
		}
		
		@Override
		@RosettaAttribute("subPdct")
		public AssetClassSubProductType49Code getSubPdct() {
			return subPdct;
		}
		
	
		@Override
		@RosettaAttribute("basePdct")
		public AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder setBasePdct(AssetClassProductType1Code basePdct) {
			this.basePdct = basePdct==null?null:basePdct;
			return this;
		}
		@Override
		@RosettaAttribute("subPdct")
		public AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder setSubPdct(AssetClassSubProductType49Code subPdct) {
			this.subPdct = subPdct==null?null:subPdct;
			return this;
		}
		
		@Override
		public AgriculturalCommodityOther2 build() {
			return new AgriculturalCommodityOther2.AgriculturalCommodityOther2Impl(this);
		}
		
		@Override
		public AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBasePdct()!=null) return true;
			if (getSubPdct()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder o = (AgriculturalCommodityOther2.AgriculturalCommodityOther2Builder) other;
			
			
			merger.mergeBasic(getBasePdct(), o.getBasePdct(), this::setBasePdct);
			merger.mergeBasic(getSubPdct(), o.getSubPdct(), this::setSubPdct);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			AgriculturalCommodityOther2 _that = getType().cast(o);
		
			if (!Objects.equals(basePdct, _that.getBasePdct())) return false;
			if (!Objects.equals(subPdct, _that.getSubPdct())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (basePdct != null ? basePdct.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (subPdct != null ? subPdct.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AgriculturalCommodityOther2Builder {" +
				"basePdct=" + this.basePdct + ", " +
				"subPdct=" + this.subPdct +
			'}';
		}
	}
}
