package iso20022.auth030.esma;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import iso20022.auth030.esma.ActiveOrHistoricCurrencyAnd19DecimalAmount;
import iso20022.auth030.esma.AmountAndDirection106;
import iso20022.auth030.esma.AmountAndDirection106.AmountAndDirection106Builder;
import iso20022.auth030.esma.AmountAndDirection106.AmountAndDirection106BuilderImpl;
import iso20022.auth030.esma.AmountAndDirection106.AmountAndDirection106Impl;
import iso20022.auth030.esma.meta.AmountAndDirection106Meta;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * Posting of an item to a cash account, in the context of a cash transaction, that results in an increase or decrease to the balance of the account.
 * @version ${project.version}
 */
@RosettaDataType(value="AmountAndDirection106", builder=AmountAndDirection106.AmountAndDirection106BuilderImpl.class, version="${project.version}")
public interface AmountAndDirection106 extends RosettaModelObject {

	AmountAndDirection106Meta metaData = new AmountAndDirection106Meta();

	/*********************** Getter Methods  ***********************/
	/**
	 * Amount of money in the cash entry.
	 */
	ActiveOrHistoricCurrencyAnd19DecimalAmount getAmt();
	/**
	 * Indicates that the amount value is positive or negative.
	 */
	Boolean getSgn();

	/*********************** Build Methods  ***********************/
	AmountAndDirection106 build();
	
	AmountAndDirection106.AmountAndDirection106Builder toBuilder();
	
	static AmountAndDirection106.AmountAndDirection106Builder builder() {
		return new AmountAndDirection106.AmountAndDirection106BuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends AmountAndDirection106> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends AmountAndDirection106> getType() {
		return AmountAndDirection106.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processRosetta(path.newSubPath("amt"), processor, ActiveOrHistoricCurrencyAnd19DecimalAmount.class, getAmt());
		processor.processBasic(path.newSubPath("sgn"), Boolean.class, getSgn(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface AmountAndDirection106Builder extends AmountAndDirection106, RosettaModelObjectBuilder {
		ActiveOrHistoricCurrencyAnd19DecimalAmount.ActiveOrHistoricCurrencyAnd19DecimalAmountBuilder getOrCreateAmt();
		ActiveOrHistoricCurrencyAnd19DecimalAmount.ActiveOrHistoricCurrencyAnd19DecimalAmountBuilder getAmt();
		AmountAndDirection106.AmountAndDirection106Builder setAmt(ActiveOrHistoricCurrencyAnd19DecimalAmount amt);
		AmountAndDirection106.AmountAndDirection106Builder setSgn(Boolean sgn);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processRosetta(path.newSubPath("amt"), processor, ActiveOrHistoricCurrencyAnd19DecimalAmount.ActiveOrHistoricCurrencyAnd19DecimalAmountBuilder.class, getAmt());
			processor.processBasic(path.newSubPath("sgn"), Boolean.class, getSgn(), this);
		}
		

		AmountAndDirection106.AmountAndDirection106Builder prune();
	}

	/*********************** Immutable Implementation of AmountAndDirection106  ***********************/
	class AmountAndDirection106Impl implements AmountAndDirection106 {
		private final ActiveOrHistoricCurrencyAnd19DecimalAmount amt;
		private final Boolean sgn;
		
		protected AmountAndDirection106Impl(AmountAndDirection106.AmountAndDirection106Builder builder) {
			this.amt = ofNullable(builder.getAmt()).map(f->f.build()).orElse(null);
			this.sgn = builder.getSgn();
		}
		
		@Override
		@RosettaAttribute("amt")
		public ActiveOrHistoricCurrencyAnd19DecimalAmount getAmt() {
			return amt;
		}
		
		@Override
		@RosettaAttribute("sgn")
		public Boolean getSgn() {
			return sgn;
		}
		
		@Override
		public AmountAndDirection106 build() {
			return this;
		}
		
		@Override
		public AmountAndDirection106.AmountAndDirection106Builder toBuilder() {
			AmountAndDirection106.AmountAndDirection106Builder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(AmountAndDirection106.AmountAndDirection106Builder builder) {
			ofNullable(getAmt()).ifPresent(builder::setAmt);
			ofNullable(getSgn()).ifPresent(builder::setSgn);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			AmountAndDirection106 _that = getType().cast(o);
		
			if (!Objects.equals(amt, _that.getAmt())) return false;
			if (!Objects.equals(sgn, _that.getSgn())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (amt != null ? amt.hashCode() : 0);
			_result = 31 * _result + (sgn != null ? sgn.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AmountAndDirection106 {" +
				"amt=" + this.amt + ", " +
				"sgn=" + this.sgn +
			'}';
		}
	}

	/*********************** Builder Implementation of AmountAndDirection106  ***********************/
	class AmountAndDirection106BuilderImpl implements AmountAndDirection106.AmountAndDirection106Builder {
	
		protected ActiveOrHistoricCurrencyAnd19DecimalAmount.ActiveOrHistoricCurrencyAnd19DecimalAmountBuilder amt;
		protected Boolean sgn;
	
		public AmountAndDirection106BuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("amt")
		public ActiveOrHistoricCurrencyAnd19DecimalAmount.ActiveOrHistoricCurrencyAnd19DecimalAmountBuilder getAmt() {
			return amt;
		}
		
		@Override
		public ActiveOrHistoricCurrencyAnd19DecimalAmount.ActiveOrHistoricCurrencyAnd19DecimalAmountBuilder getOrCreateAmt() {
			ActiveOrHistoricCurrencyAnd19DecimalAmount.ActiveOrHistoricCurrencyAnd19DecimalAmountBuilder result;
			if (amt!=null) {
				result = amt;
			}
			else {
				result = amt = ActiveOrHistoricCurrencyAnd19DecimalAmount.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("sgn")
		public Boolean getSgn() {
			return sgn;
		}
		
	
		@Override
		@RosettaAttribute("amt")
		public AmountAndDirection106.AmountAndDirection106Builder setAmt(ActiveOrHistoricCurrencyAnd19DecimalAmount amt) {
			this.amt = amt==null?null:amt.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("sgn")
		public AmountAndDirection106.AmountAndDirection106Builder setSgn(Boolean sgn) {
			this.sgn = sgn==null?null:sgn;
			return this;
		}
		
		@Override
		public AmountAndDirection106 build() {
			return new AmountAndDirection106.AmountAndDirection106Impl(this);
		}
		
		@Override
		public AmountAndDirection106.AmountAndDirection106Builder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public AmountAndDirection106.AmountAndDirection106Builder prune() {
			if (amt!=null && !amt.prune().hasData()) amt = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAmt()!=null && getAmt().hasData()) return true;
			if (getSgn()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public AmountAndDirection106.AmountAndDirection106Builder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			AmountAndDirection106.AmountAndDirection106Builder o = (AmountAndDirection106.AmountAndDirection106Builder) other;
			
			merger.mergeRosetta(getAmt(), o.getAmt(), this::setAmt);
			
			merger.mergeBasic(getSgn(), o.getSgn(), this::setSgn);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			AmountAndDirection106 _that = getType().cast(o);
		
			if (!Objects.equals(amt, _that.getAmt())) return false;
			if (!Objects.equals(sgn, _that.getSgn())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (amt != null ? amt.hashCode() : 0);
			_result = 31 * _result + (sgn != null ? sgn.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "AmountAndDirection106Builder {" +
				"amt=" + this.amt + ", " +
				"sgn=" + this.sgn +
			'}';
		}
	}
}
