package iso20022.auth030.esma;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import iso20022.auth030.esma.AssetClassProductType5Code;
import iso20022.auth030.esma.AssetClassSubProductType39Code;
import iso20022.auth030.esma.FertilizerCommodityAmmonia2;
import iso20022.auth030.esma.FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder;
import iso20022.auth030.esma.FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2BuilderImpl;
import iso20022.auth030.esma.FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Impl;
import iso20022.auth030.esma.meta.FertilizerCommodityAmmonia2Meta;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * Defines commodity sub-product attributes of a fertilizer derivative of type ammonia.
 * @version ${project.version}
 */
@RosettaDataType(value="FertilizerCommodityAmmonia2", builder=FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2BuilderImpl.class, version="${project.version}")
public interface FertilizerCommodityAmmonia2 extends RosettaModelObject {

	FertilizerCommodityAmmonia2Meta metaData = new FertilizerCommodityAmmonia2Meta();

	/*********************** Getter Methods  ***********************/
	/**
	 * Base product for the underlying asset class as specified in the classification of commodities derivatives table.
	 */
	AssetClassProductType5Code getBasePdct();
	/**
	 * Sub-product for the underlying asset class.
	 */
	AssetClassSubProductType39Code getSubPdct();

	/*********************** Build Methods  ***********************/
	FertilizerCommodityAmmonia2 build();
	
	FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder toBuilder();
	
	static FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder builder() {
		return new FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2BuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends FertilizerCommodityAmmonia2> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends FertilizerCommodityAmmonia2> getType() {
		return FertilizerCommodityAmmonia2.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("basePdct"), AssetClassProductType5Code.class, getBasePdct(), this);
		processor.processBasic(path.newSubPath("subPdct"), AssetClassSubProductType39Code.class, getSubPdct(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface FertilizerCommodityAmmonia2Builder extends FertilizerCommodityAmmonia2, RosettaModelObjectBuilder {
		FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder setBasePdct(AssetClassProductType5Code basePdct);
		FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder setSubPdct(AssetClassSubProductType39Code subPdct);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("basePdct"), AssetClassProductType5Code.class, getBasePdct(), this);
			processor.processBasic(path.newSubPath("subPdct"), AssetClassSubProductType39Code.class, getSubPdct(), this);
		}
		

		FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder prune();
	}

	/*********************** Immutable Implementation of FertilizerCommodityAmmonia2  ***********************/
	class FertilizerCommodityAmmonia2Impl implements FertilizerCommodityAmmonia2 {
		private final AssetClassProductType5Code basePdct;
		private final AssetClassSubProductType39Code subPdct;
		
		protected FertilizerCommodityAmmonia2Impl(FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder builder) {
			this.basePdct = builder.getBasePdct();
			this.subPdct = builder.getSubPdct();
		}
		
		@Override
		@RosettaAttribute("basePdct")
		public AssetClassProductType5Code getBasePdct() {
			return basePdct;
		}
		
		@Override
		@RosettaAttribute("subPdct")
		public AssetClassSubProductType39Code getSubPdct() {
			return subPdct;
		}
		
		@Override
		public FertilizerCommodityAmmonia2 build() {
			return this;
		}
		
		@Override
		public FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder toBuilder() {
			FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder builder) {
			ofNullable(getBasePdct()).ifPresent(builder::setBasePdct);
			ofNullable(getSubPdct()).ifPresent(builder::setSubPdct);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			FertilizerCommodityAmmonia2 _that = getType().cast(o);
		
			if (!Objects.equals(basePdct, _that.getBasePdct())) return false;
			if (!Objects.equals(subPdct, _that.getSubPdct())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (basePdct != null ? basePdct.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (subPdct != null ? subPdct.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FertilizerCommodityAmmonia2 {" +
				"basePdct=" + this.basePdct + ", " +
				"subPdct=" + this.subPdct +
			'}';
		}
	}

	/*********************** Builder Implementation of FertilizerCommodityAmmonia2  ***********************/
	class FertilizerCommodityAmmonia2BuilderImpl implements FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder {
	
		protected AssetClassProductType5Code basePdct;
		protected AssetClassSubProductType39Code subPdct;
	
		public FertilizerCommodityAmmonia2BuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("basePdct")
		public AssetClassProductType5Code getBasePdct() {
			return basePdct;
		}
		
		@Override
		@RosettaAttribute("subPdct")
		public AssetClassSubProductType39Code getSubPdct() {
			return subPdct;
		}
		
	
		@Override
		@RosettaAttribute("basePdct")
		public FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder setBasePdct(AssetClassProductType5Code basePdct) {
			this.basePdct = basePdct==null?null:basePdct;
			return this;
		}
		@Override
		@RosettaAttribute("subPdct")
		public FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder setSubPdct(AssetClassSubProductType39Code subPdct) {
			this.subPdct = subPdct==null?null:subPdct;
			return this;
		}
		
		@Override
		public FertilizerCommodityAmmonia2 build() {
			return new FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Impl(this);
		}
		
		@Override
		public FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBasePdct()!=null) return true;
			if (getSubPdct()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder o = (FertilizerCommodityAmmonia2.FertilizerCommodityAmmonia2Builder) other;
			
			
			merger.mergeBasic(getBasePdct(), o.getBasePdct(), this::setBasePdct);
			merger.mergeBasic(getSubPdct(), o.getSubPdct(), this::setSubPdct);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			FertilizerCommodityAmmonia2 _that = getType().cast(o);
		
			if (!Objects.equals(basePdct, _that.getBasePdct())) return false;
			if (!Objects.equals(subPdct, _that.getSubPdct())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (basePdct != null ? basePdct.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (subPdct != null ? subPdct.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FertilizerCommodityAmmonia2Builder {" +
				"basePdct=" + this.basePdct + ", " +
				"subPdct=" + this.subPdct +
			'}';
		}
	}
}
