package iso20022.auth030.esma;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import iso20022.auth030.esma.AssetClassDetailedSubProductType34Code;
import iso20022.auth030.esma.AssetClassProductType4Code;
import iso20022.auth030.esma.AssetClassSubProductType32Code;
import iso20022.auth030.esma.FreightCommodityWet3;
import iso20022.auth030.esma.FreightCommodityWet3.FreightCommodityWet3Builder;
import iso20022.auth030.esma.FreightCommodityWet3.FreightCommodityWet3BuilderImpl;
import iso20022.auth030.esma.FreightCommodityWet3.FreightCommodityWet3Impl;
import iso20022.auth030.esma.meta.FreightCommodityWet3Meta;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * Defines commodity sub-product attributes of a freight derivative of type wet.
 * @version ${project.version}
 */
@RosettaDataType(value="FreightCommodityWet3", builder=FreightCommodityWet3.FreightCommodityWet3BuilderImpl.class, version="${project.version}")
public interface FreightCommodityWet3 extends RosettaModelObject {

	FreightCommodityWet3Meta metaData = new FreightCommodityWet3Meta();

	/*********************** Getter Methods  ***********************/
	/**
	 * Base product for the underlying asset class as specified in the classification of commodities derivatives table.
	 */
	AssetClassProductType4Code getBasePdct();
	/**
	 * Sub-product for the underlying asset class.
	 */
	AssetClassSubProductType32Code getSubPdct();
	/**
	 * Further subproduct type related to instruments that have a non-financial instrument or commodity as underlying.
	 */
	AssetClassDetailedSubProductType34Code getAddtlSubPdct();

	/*********************** Build Methods  ***********************/
	FreightCommodityWet3 build();
	
	FreightCommodityWet3.FreightCommodityWet3Builder toBuilder();
	
	static FreightCommodityWet3.FreightCommodityWet3Builder builder() {
		return new FreightCommodityWet3.FreightCommodityWet3BuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends FreightCommodityWet3> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends FreightCommodityWet3> getType() {
		return FreightCommodityWet3.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("basePdct"), AssetClassProductType4Code.class, getBasePdct(), this);
		processor.processBasic(path.newSubPath("subPdct"), AssetClassSubProductType32Code.class, getSubPdct(), this);
		processor.processBasic(path.newSubPath("addtlSubPdct"), AssetClassDetailedSubProductType34Code.class, getAddtlSubPdct(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface FreightCommodityWet3Builder extends FreightCommodityWet3, RosettaModelObjectBuilder {
		FreightCommodityWet3.FreightCommodityWet3Builder setBasePdct(AssetClassProductType4Code basePdct);
		FreightCommodityWet3.FreightCommodityWet3Builder setSubPdct(AssetClassSubProductType32Code subPdct);
		FreightCommodityWet3.FreightCommodityWet3Builder setAddtlSubPdct(AssetClassDetailedSubProductType34Code addtlSubPdct);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("basePdct"), AssetClassProductType4Code.class, getBasePdct(), this);
			processor.processBasic(path.newSubPath("subPdct"), AssetClassSubProductType32Code.class, getSubPdct(), this);
			processor.processBasic(path.newSubPath("addtlSubPdct"), AssetClassDetailedSubProductType34Code.class, getAddtlSubPdct(), this);
		}
		

		FreightCommodityWet3.FreightCommodityWet3Builder prune();
	}

	/*********************** Immutable Implementation of FreightCommodityWet3  ***********************/
	class FreightCommodityWet3Impl implements FreightCommodityWet3 {
		private final AssetClassProductType4Code basePdct;
		private final AssetClassSubProductType32Code subPdct;
		private final AssetClassDetailedSubProductType34Code addtlSubPdct;
		
		protected FreightCommodityWet3Impl(FreightCommodityWet3.FreightCommodityWet3Builder builder) {
			this.basePdct = builder.getBasePdct();
			this.subPdct = builder.getSubPdct();
			this.addtlSubPdct = builder.getAddtlSubPdct();
		}
		
		@Override
		@RosettaAttribute("basePdct")
		public AssetClassProductType4Code getBasePdct() {
			return basePdct;
		}
		
		@Override
		@RosettaAttribute("subPdct")
		public AssetClassSubProductType32Code getSubPdct() {
			return subPdct;
		}
		
		@Override
		@RosettaAttribute("addtlSubPdct")
		public AssetClassDetailedSubProductType34Code getAddtlSubPdct() {
			return addtlSubPdct;
		}
		
		@Override
		public FreightCommodityWet3 build() {
			return this;
		}
		
		@Override
		public FreightCommodityWet3.FreightCommodityWet3Builder toBuilder() {
			FreightCommodityWet3.FreightCommodityWet3Builder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(FreightCommodityWet3.FreightCommodityWet3Builder builder) {
			ofNullable(getBasePdct()).ifPresent(builder::setBasePdct);
			ofNullable(getSubPdct()).ifPresent(builder::setSubPdct);
			ofNullable(getAddtlSubPdct()).ifPresent(builder::setAddtlSubPdct);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			FreightCommodityWet3 _that = getType().cast(o);
		
			if (!Objects.equals(basePdct, _that.getBasePdct())) return false;
			if (!Objects.equals(subPdct, _that.getSubPdct())) return false;
			if (!Objects.equals(addtlSubPdct, _that.getAddtlSubPdct())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (basePdct != null ? basePdct.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (subPdct != null ? subPdct.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (addtlSubPdct != null ? addtlSubPdct.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FreightCommodityWet3 {" +
				"basePdct=" + this.basePdct + ", " +
				"subPdct=" + this.subPdct + ", " +
				"addtlSubPdct=" + this.addtlSubPdct +
			'}';
		}
	}

	/*********************** Builder Implementation of FreightCommodityWet3  ***********************/
	class FreightCommodityWet3BuilderImpl implements FreightCommodityWet3.FreightCommodityWet3Builder {
	
		protected AssetClassProductType4Code basePdct;
		protected AssetClassSubProductType32Code subPdct;
		protected AssetClassDetailedSubProductType34Code addtlSubPdct;
	
		public FreightCommodityWet3BuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("basePdct")
		public AssetClassProductType4Code getBasePdct() {
			return basePdct;
		}
		
		@Override
		@RosettaAttribute("subPdct")
		public AssetClassSubProductType32Code getSubPdct() {
			return subPdct;
		}
		
		@Override
		@RosettaAttribute("addtlSubPdct")
		public AssetClassDetailedSubProductType34Code getAddtlSubPdct() {
			return addtlSubPdct;
		}
		
	
		@Override
		@RosettaAttribute("basePdct")
		public FreightCommodityWet3.FreightCommodityWet3Builder setBasePdct(AssetClassProductType4Code basePdct) {
			this.basePdct = basePdct==null?null:basePdct;
			return this;
		}
		@Override
		@RosettaAttribute("subPdct")
		public FreightCommodityWet3.FreightCommodityWet3Builder setSubPdct(AssetClassSubProductType32Code subPdct) {
			this.subPdct = subPdct==null?null:subPdct;
			return this;
		}
		@Override
		@RosettaAttribute("addtlSubPdct")
		public FreightCommodityWet3.FreightCommodityWet3Builder setAddtlSubPdct(AssetClassDetailedSubProductType34Code addtlSubPdct) {
			this.addtlSubPdct = addtlSubPdct==null?null:addtlSubPdct;
			return this;
		}
		
		@Override
		public FreightCommodityWet3 build() {
			return new FreightCommodityWet3.FreightCommodityWet3Impl(this);
		}
		
		@Override
		public FreightCommodityWet3.FreightCommodityWet3Builder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public FreightCommodityWet3.FreightCommodityWet3Builder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBasePdct()!=null) return true;
			if (getSubPdct()!=null) return true;
			if (getAddtlSubPdct()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public FreightCommodityWet3.FreightCommodityWet3Builder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			FreightCommodityWet3.FreightCommodityWet3Builder o = (FreightCommodityWet3.FreightCommodityWet3Builder) other;
			
			
			merger.mergeBasic(getBasePdct(), o.getBasePdct(), this::setBasePdct);
			merger.mergeBasic(getSubPdct(), o.getSubPdct(), this::setSubPdct);
			merger.mergeBasic(getAddtlSubPdct(), o.getAddtlSubPdct(), this::setAddtlSubPdct);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			FreightCommodityWet3 _that = getType().cast(o);
		
			if (!Objects.equals(basePdct, _that.getBasePdct())) return false;
			if (!Objects.equals(subPdct, _that.getSubPdct())) return false;
			if (!Objects.equals(addtlSubPdct, _that.getAddtlSubPdct())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (basePdct != null ? basePdct.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (subPdct != null ? subPdct.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (addtlSubPdct != null ? addtlSubPdct.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "FreightCommodityWet3Builder {" +
				"basePdct=" + this.basePdct + ", " +
				"subPdct=" + this.subPdct + ", " +
				"addtlSubPdct=" + this.addtlSubPdct +
			'}';
		}
	}
}
