package iso20022.auth030.esma;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import iso20022.auth030.esma.CustomBasket4__1;
import iso20022.auth030.esma.IndexIdentification1__1;
import iso20022.auth030.esma.SecurityIdentification41Choice__1;
import iso20022.auth030.esma.SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder;
import iso20022.auth030.esma.SecurityIdentification41Choice__1.SecurityIdentification41Choice__1BuilderImpl;
import iso20022.auth030.esma.SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Impl;
import iso20022.auth030.esma.meta.SecurityIdentification41Choice__1Meta;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * Choice between ISIN and an alternative format for the identification of a financial instrument. ISIN is the preferred format.
 * @version ${project.version}
 */
@RosettaDataType(value="SecurityIdentification41Choice__1", builder=SecurityIdentification41Choice__1.SecurityIdentification41Choice__1BuilderImpl.class, version="${project.version}")
public interface SecurityIdentification41Choice__1 extends RosettaModelObject {

	SecurityIdentification41Choice__1Meta metaData = new SecurityIdentification41Choice__1Meta();

	/*********************** Getter Methods  ***********************/
	/**
	 * International Securities Identification Number (ISIN). A numbering system designed by the United Nation&#39;s International Organisation for Standardisation (ISO). The ISIN is composed of a 2-character prefix representing the country of issue, followed by the national security number (if one exists), and a check digit. Each country has a national numbering agency that assigns ISIN numbers for securities in that country.
	 */
	String getIsin();
	/**
	 * Identification of constituents for a basket of indexes.
	 */
	CustomBasket4__1 getBskt();
	/**
	 * Indicates the index upon which the financial instrument is based.
	 */
	IndexIdentification1__1 getIndx();

	/*********************** Build Methods  ***********************/
	SecurityIdentification41Choice__1 build();
	
	SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder toBuilder();
	
	static SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder builder() {
		return new SecurityIdentification41Choice__1.SecurityIdentification41Choice__1BuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends SecurityIdentification41Choice__1> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends SecurityIdentification41Choice__1> getType() {
		return SecurityIdentification41Choice__1.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("isin"), String.class, getIsin(), this);
		processRosetta(path.newSubPath("bskt"), processor, CustomBasket4__1.class, getBskt());
		processRosetta(path.newSubPath("indx"), processor, IndexIdentification1__1.class, getIndx());
	}
	

	/*********************** Builder Interface  ***********************/
	interface SecurityIdentification41Choice__1Builder extends SecurityIdentification41Choice__1, RosettaModelObjectBuilder {
		CustomBasket4__1.CustomBasket4__1Builder getOrCreateBskt();
		CustomBasket4__1.CustomBasket4__1Builder getBskt();
		IndexIdentification1__1.IndexIdentification1__1Builder getOrCreateIndx();
		IndexIdentification1__1.IndexIdentification1__1Builder getIndx();
		SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder setIsin(String isin);
		SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder setBskt(CustomBasket4__1 bskt);
		SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder setIndx(IndexIdentification1__1 indx);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("isin"), String.class, getIsin(), this);
			processRosetta(path.newSubPath("bskt"), processor, CustomBasket4__1.CustomBasket4__1Builder.class, getBskt());
			processRosetta(path.newSubPath("indx"), processor, IndexIdentification1__1.IndexIdentification1__1Builder.class, getIndx());
		}
		

		SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder prune();
	}

	/*********************** Immutable Implementation of SecurityIdentification41Choice__1  ***********************/
	class SecurityIdentification41Choice__1Impl implements SecurityIdentification41Choice__1 {
		private final String isin;
		private final CustomBasket4__1 bskt;
		private final IndexIdentification1__1 indx;
		
		protected SecurityIdentification41Choice__1Impl(SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder builder) {
			this.isin = builder.getIsin();
			this.bskt = ofNullable(builder.getBskt()).map(f->f.build()).orElse(null);
			this.indx = ofNullable(builder.getIndx()).map(f->f.build()).orElse(null);
		}
		
		@Override
		@RosettaAttribute("isin")
		public String getIsin() {
			return isin;
		}
		
		@Override
		@RosettaAttribute("bskt")
		public CustomBasket4__1 getBskt() {
			return bskt;
		}
		
		@Override
		@RosettaAttribute("indx")
		public IndexIdentification1__1 getIndx() {
			return indx;
		}
		
		@Override
		public SecurityIdentification41Choice__1 build() {
			return this;
		}
		
		@Override
		public SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder toBuilder() {
			SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder builder) {
			ofNullable(getIsin()).ifPresent(builder::setIsin);
			ofNullable(getBskt()).ifPresent(builder::setBskt);
			ofNullable(getIndx()).ifPresent(builder::setIndx);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			SecurityIdentification41Choice__1 _that = getType().cast(o);
		
			if (!Objects.equals(isin, _that.getIsin())) return false;
			if (!Objects.equals(bskt, _that.getBskt())) return false;
			if (!Objects.equals(indx, _that.getIndx())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (isin != null ? isin.hashCode() : 0);
			_result = 31 * _result + (bskt != null ? bskt.hashCode() : 0);
			_result = 31 * _result + (indx != null ? indx.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SecurityIdentification41Choice__1 {" +
				"isin=" + this.isin + ", " +
				"bskt=" + this.bskt + ", " +
				"indx=" + this.indx +
			'}';
		}
	}

	/*********************** Builder Implementation of SecurityIdentification41Choice__1  ***********************/
	class SecurityIdentification41Choice__1BuilderImpl implements SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder {
	
		protected String isin;
		protected CustomBasket4__1.CustomBasket4__1Builder bskt;
		protected IndexIdentification1__1.IndexIdentification1__1Builder indx;
	
		public SecurityIdentification41Choice__1BuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("isin")
		public String getIsin() {
			return isin;
		}
		
		@Override
		@RosettaAttribute("bskt")
		public CustomBasket4__1.CustomBasket4__1Builder getBskt() {
			return bskt;
		}
		
		@Override
		public CustomBasket4__1.CustomBasket4__1Builder getOrCreateBskt() {
			CustomBasket4__1.CustomBasket4__1Builder result;
			if (bskt!=null) {
				result = bskt;
			}
			else {
				result = bskt = CustomBasket4__1.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("indx")
		public IndexIdentification1__1.IndexIdentification1__1Builder getIndx() {
			return indx;
		}
		
		@Override
		public IndexIdentification1__1.IndexIdentification1__1Builder getOrCreateIndx() {
			IndexIdentification1__1.IndexIdentification1__1Builder result;
			if (indx!=null) {
				result = indx;
			}
			else {
				result = indx = IndexIdentification1__1.builder();
			}
			
			return result;
		}
	
		@Override
		@RosettaAttribute("isin")
		public SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder setIsin(String isin) {
			this.isin = isin==null?null:isin;
			return this;
		}
		@Override
		@RosettaAttribute("bskt")
		public SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder setBskt(CustomBasket4__1 bskt) {
			this.bskt = bskt==null?null:bskt.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("indx")
		public SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder setIndx(IndexIdentification1__1 indx) {
			this.indx = indx==null?null:indx.toBuilder();
			return this;
		}
		
		@Override
		public SecurityIdentification41Choice__1 build() {
			return new SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Impl(this);
		}
		
		@Override
		public SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder prune() {
			if (bskt!=null && !bskt.prune().hasData()) bskt = null;
			if (indx!=null && !indx.prune().hasData()) indx = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getIsin()!=null) return true;
			if (getBskt()!=null && getBskt().hasData()) return true;
			if (getIndx()!=null && getIndx().hasData()) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder o = (SecurityIdentification41Choice__1.SecurityIdentification41Choice__1Builder) other;
			
			merger.mergeRosetta(getBskt(), o.getBskt(), this::setBskt);
			merger.mergeRosetta(getIndx(), o.getIndx(), this::setIndx);
			
			merger.mergeBasic(getIsin(), o.getIsin(), this::setIsin);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			SecurityIdentification41Choice__1 _that = getType().cast(o);
		
			if (!Objects.equals(isin, _that.getIsin())) return false;
			if (!Objects.equals(bskt, _that.getBskt())) return false;
			if (!Objects.equals(indx, _that.getIndx())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (isin != null ? isin.hashCode() : 0);
			_result = 31 * _result + (bskt != null ? bskt.hashCode() : 0);
			_result = 31 * _result + (indx != null ? indx.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "SecurityIdentification41Choice__1Builder {" +
				"isin=" + this.isin + ", " +
				"bskt=" + this.bskt + ", " +
				"indx=" + this.indx +
			'}';
		}
	}
}
