package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.ActiveOrHistoricCurrencyAnd13DecimalAmount__1;
import java.math.BigDecimal;
import java.util.regex.Pattern;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkNumber;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkString;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.stream.Collectors.joining;

public class ActiveOrHistoricCurrencyAnd13DecimalAmount__1TypeFormatValidator implements Validator<ActiveOrHistoricCurrencyAnd13DecimalAmount__1> {

	@Override
	public ValidationResult<ActiveOrHistoricCurrencyAnd13DecimalAmount__1> validate(RosettaPath path, ActiveOrHistoricCurrencyAnd13DecimalAmount__1 o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkNumber("value", o.getValue(), of(18), of(13), of(new BigDecimal("0")), empty()), 
				checkString("ccy", o.getCcy(), 0, empty(), of(Pattern.compile("[A-Z]{3,3}")))
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ActiveOrHistoricCurrencyAnd13DecimalAmount__1", ValidationType.TYPE_FORMAT, "ActiveOrHistoricCurrencyAnd13DecimalAmount__1", path, "", error);
		}
		return success("ActiveOrHistoricCurrencyAnd13DecimalAmount__1", ValidationType.TYPE_FORMAT, "ActiveOrHistoricCurrencyAnd13DecimalAmount__1", path, "");
	}

}
