package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.AgriculturalCommodityDairy2;
import iso20022.auth030.esma.AgriculturalCommodityForestry2;
import iso20022.auth030.esma.AgriculturalCommodityGrain3;
import iso20022.auth030.esma.AgriculturalCommodityLiveStock2;
import iso20022.auth030.esma.AgriculturalCommodityOilSeed2;
import iso20022.auth030.esma.AgriculturalCommodityOliveOil3;
import iso20022.auth030.esma.AgriculturalCommodityOther2;
import iso20022.auth030.esma.AgriculturalCommodityPotato2;
import iso20022.auth030.esma.AgriculturalCommoditySeafood2;
import iso20022.auth030.esma.AgriculturalCommoditySoft2;
import iso20022.auth030.esma.AssetClassCommodityAgricultural6Choice;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class AssetClassCommodityAgricultural6ChoiceValidator implements Validator<AssetClassCommodityAgricultural6Choice> {

	@Override
	public ValidationResult<AssetClassCommodityAgricultural6Choice> validate(RosettaPath path, AssetClassCommodityAgricultural6Choice o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("grnOilSeed", (AgriculturalCommodityOilSeed2) o.getGrnOilSeed() != null ? 1 : 0, 0, 1), 
				checkCardinality("soft", (AgriculturalCommoditySoft2) o.getSoft() != null ? 1 : 0, 0, 1), 
				checkCardinality("ptt", (AgriculturalCommodityPotato2) o.getPtt() != null ? 1 : 0, 0, 1), 
				checkCardinality("olvOil", (AgriculturalCommodityOliveOil3) o.getOlvOil() != null ? 1 : 0, 0, 1), 
				checkCardinality("dairy", (AgriculturalCommodityDairy2) o.getDairy() != null ? 1 : 0, 0, 1), 
				checkCardinality("frstry", (AgriculturalCommodityForestry2) o.getFrstry() != null ? 1 : 0, 0, 1), 
				checkCardinality("sfd", (AgriculturalCommoditySeafood2) o.getSfd() != null ? 1 : 0, 0, 1), 
				checkCardinality("liveStock", (AgriculturalCommodityLiveStock2) o.getLiveStock() != null ? 1 : 0, 0, 1), 
				checkCardinality("grn", (AgriculturalCommodityGrain3) o.getGrn() != null ? 1 : 0, 0, 1), 
				checkCardinality("othr", (AgriculturalCommodityOther2) o.getOthr() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("AssetClassCommodityAgricultural6Choice", ValidationType.CARDINALITY, "AssetClassCommodityAgricultural6Choice", path, "", error);
		}
		return success("AssetClassCommodityAgricultural6Choice", ValidationType.CARDINALITY, "AssetClassCommodityAgricultural6Choice", path, "");
	}

}
