package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.AssetClassCommodityFertilizer4Choice;
import iso20022.auth030.esma.FertilizerCommodityAmmonia2;
import iso20022.auth030.esma.FertilizerCommodityDiammoniumPhosphate2;
import iso20022.auth030.esma.FertilizerCommodityOther2;
import iso20022.auth030.esma.FertilizerCommodityPotash2;
import iso20022.auth030.esma.FertilizerCommoditySulphur2;
import iso20022.auth030.esma.FertilizerCommodityUrea2;
import iso20022.auth030.esma.FertilizerCommodityUreaAndAmmoniumNitrate2;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class AssetClassCommodityFertilizer4ChoiceValidator implements Validator<AssetClassCommodityFertilizer4Choice> {

	@Override
	public ValidationResult<AssetClassCommodityFertilizer4Choice> validate(RosettaPath path, AssetClassCommodityFertilizer4Choice o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("ammn", (FertilizerCommodityAmmonia2) o.getAmmn() != null ? 1 : 0, 0, 1), 
				checkCardinality("dmmnmPhspht", (FertilizerCommodityDiammoniumPhosphate2) o.getDmmnmPhspht() != null ? 1 : 0, 0, 1), 
				checkCardinality("ptsh", (FertilizerCommodityPotash2) o.getPtsh() != null ? 1 : 0, 0, 1), 
				checkCardinality("slphr", (FertilizerCommoditySulphur2) o.getSlphr() != null ? 1 : 0, 0, 1), 
				checkCardinality("urea", (FertilizerCommodityUrea2) o.getUrea() != null ? 1 : 0, 0, 1), 
				checkCardinality("ureaAndAmmnmNtrt", (FertilizerCommodityUreaAndAmmoniumNitrate2) o.getUreaAndAmmnmNtrt() != null ? 1 : 0, 0, 1), 
				checkCardinality("othr", (FertilizerCommodityOther2) o.getOthr() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("AssetClassCommodityFertilizer4Choice", ValidationType.CARDINALITY, "AssetClassCommodityFertilizer4Choice", path, "", error);
		}
		return success("AssetClassCommodityFertilizer4Choice", ValidationType.CARDINALITY, "AssetClassCommodityFertilizer4Choice", path, "");
	}

}
