package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.AssetClassCommodityPolypropylene4Choice;
import iso20022.auth030.esma.PolypropyleneCommodityOther2;
import iso20022.auth030.esma.PolypropyleneCommodityPlastic2;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class AssetClassCommodityPolypropylene4ChoiceValidator implements Validator<AssetClassCommodityPolypropylene4Choice> {

	@Override
	public ValidationResult<AssetClassCommodityPolypropylene4Choice> validate(RosettaPath path, AssetClassCommodityPolypropylene4Choice o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("plstc", (PolypropyleneCommodityPlastic2) o.getPlstc() != null ? 1 : 0, 0, 1), 
				checkCardinality("othr", (PolypropyleneCommodityOther2) o.getOthr() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("AssetClassCommodityPolypropylene4Choice", ValidationType.CARDINALITY, "AssetClassCommodityPolypropylene4Choice", path, "", error);
		}
		return success("AssetClassCommodityPolypropylene4Choice", ValidationType.CARDINALITY, "AssetClassCommodityPolypropylene4Choice", path, "");
	}

}
