package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.AmountAndDirection109__1;
import iso20022.auth030.esma.ContractValuationData8__1;
import iso20022.auth030.esma.ValuationType1Code;
import java.math.BigDecimal;
import java.time.ZonedDateTime;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class ContractValuationData8__1Validator implements Validator<ContractValuationData8__1> {

	@Override
	public ValidationResult<ContractValuationData8__1> validate(RosettaPath path, ContractValuationData8__1 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("ctrctVal", (AmountAndDirection109__1) o.getCtrctVal() != null ? 1 : 0, 0, 1), 
				checkCardinality("tmStmp", (ZonedDateTime) o.getTmStmp() != null ? 1 : 0, 0, 1), 
				checkCardinality("tp", (ValuationType1Code) o.getTp() != null ? 1 : 0, 0, 1), 
				checkCardinality("dlta", (BigDecimal) o.getDlta() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ContractValuationData8__1", ValidationType.CARDINALITY, "ContractValuationData8__1", path, "", error);
		}
		return success("ContractValuationData8__1", ValidationType.CARDINALITY, "ContractValuationData8__1", path, "");
	}

}
