package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.ContractValuationData8__2;
import iso20022.auth030.esma.CounterpartySpecificData36__4;
import iso20022.auth030.esma.TradeCounterpartyReport20__2;
import java.time.ZonedDateTime;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class CounterpartySpecificData36__4Validator implements Validator<CounterpartySpecificData36__4> {

	@Override
	public ValidationResult<CounterpartySpecificData36__4> validate(RosettaPath path, CounterpartySpecificData36__4 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("ctrPty", (TradeCounterpartyReport20__2) o.getCtrPty() != null ? 1 : 0, 1, 1), 
				checkCardinality("valtn", (ContractValuationData8__2) o.getValtn() != null ? 1 : 0, 0, 1), 
				checkCardinality("rptgTmStmp", (ZonedDateTime) o.getRptgTmStmp() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CounterpartySpecificData36__4", ValidationType.CARDINALITY, "CounterpartySpecificData36__4", path, "", error);
		}
		return success("CounterpartySpecificData36__4", ValidationType.CARDINALITY, "CounterpartySpecificData36__4", path, "");
	}

}
