package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.CounterpartyTradeNature15Choice__2;
import iso20022.auth030.esma.FinancialInstitutionSector1__1;
import iso20022.auth030.esma.NoReasonCode;
import iso20022.auth030.esma.NonFinancialInstitutionSector10__2;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class CounterpartyTradeNature15Choice__2Validator implements Validator<CounterpartyTradeNature15Choice__2> {

	@Override
	public ValidationResult<CounterpartyTradeNature15Choice__2> validate(RosettaPath path, CounterpartyTradeNature15Choice__2 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("fi", (FinancialInstitutionSector1__1) o.getFi() != null ? 1 : 0, 0, 1), 
				checkCardinality("nfi", (NonFinancialInstitutionSector10__2) o.getNfi() != null ? 1 : 0, 0, 1), 
				checkCardinality("cntrlCntrPty", (NoReasonCode) o.getCntrlCntrPty() != null ? 1 : 0, 0, 1), 
				checkCardinality("othr", (NoReasonCode) o.getOthr() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CounterpartyTradeNature15Choice__2", ValidationType.CARDINALITY, "CounterpartyTradeNature15Choice__2", path, "", error);
		}
		return success("CounterpartyTradeNature15Choice__2", ValidationType.CARDINALITY, "CounterpartyTradeNature15Choice__2", path, "");
	}

}
