package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.CurrencyExchange23__1;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class CurrencyExchange23__1Validator implements Validator<CurrencyExchange23__1> {

	@Override
	public ValidationResult<CurrencyExchange23__1> validate(RosettaPath path, CurrencyExchange23__1 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("ccy", (String) o.getCcy() != null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("CurrencyExchange23__1", ValidationType.CARDINALITY, "CurrencyExchange23__1", path, "", error);
		}
		return success("CurrencyExchange23__1", ValidationType.CARDINALITY, "CurrencyExchange23__1", path, "");
	}

}
