package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.DateAndDateTime2Choice__1;
import iso20022.auth030.esma.DerivativeEvent6__4;
import iso20022.auth030.esma.DerivativeEventType3Code__1;
import iso20022.auth030.esma.EventIdentifier1Choice__2;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class DerivativeEvent6__4Validator implements Validator<DerivativeEvent6__4> {

	@Override
	public ValidationResult<DerivativeEvent6__4> validate(RosettaPath path, DerivativeEvent6__4 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("tp", (DerivativeEventType3Code__1) o.getTp() != null ? 1 : 0, 1, 1), 
				checkCardinality("id", (EventIdentifier1Choice__2) o.getId() != null ? 1 : 0, 0, 1), 
				checkCardinality("tmStmp", (DateAndDateTime2Choice__1) o.getTmStmp() != null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("DerivativeEvent6__4", ValidationType.CARDINALITY, "DerivativeEvent6__4", path, "", error);
		}
		return success("DerivativeEvent6__4", ValidationType.CARDINALITY, "DerivativeEvent6__4", path, "");
	}

}
