package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.Direction2__1;
import iso20022.auth030.esma.Direction4Choice__1;
import iso20022.auth030.esma.OptionParty1Code;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class Direction4Choice__1Validator implements Validator<Direction4Choice__1> {

	@Override
	public ValidationResult<Direction4Choice__1> validate(RosettaPath path, Direction4Choice__1 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("drctn", (Direction2__1) o.getDrctn() != null ? 1 : 0, 0, 1), 
				checkCardinality("ctrPtySd", (OptionParty1Code) o.getCtrPtySd() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Direction4Choice__1", ValidationType.CARDINALITY, "Direction4Choice__1", path, "", error);
		}
		return success("Direction4Choice__1", ValidationType.CARDINALITY, "Direction4Choice__1", path, "");
	}

}
