package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.AssetClassProductType2Code;
import iso20022.auth030.esma.AssetClassSubProductType28Code;
import iso20022.auth030.esma.EnergyCommodityRenewableEnergy2;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class EnergyCommodityRenewableEnergy2Validator implements Validator<EnergyCommodityRenewableEnergy2> {

	@Override
	public ValidationResult<EnergyCommodityRenewableEnergy2> validate(RosettaPath path, EnergyCommodityRenewableEnergy2 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("basePdct", (AssetClassProductType2Code) o.getBasePdct() != null ? 1 : 0, 1, 1), 
				checkCardinality("subPdct", (AssetClassSubProductType28Code) o.getSubPdct() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("EnergyCommodityRenewableEnergy2", ValidationType.CARDINALITY, "EnergyCommodityRenewableEnergy2", path, "", error);
		}
		return success("EnergyCommodityRenewableEnergy2", ValidationType.CARDINALITY, "EnergyCommodityRenewableEnergy2", path, "");
	}

}
