package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.AssetClassProductType5Code;
import iso20022.auth030.esma.AssetClassSubProductType40Code;
import iso20022.auth030.esma.FertilizerCommodityDiammoniumPhosphate2;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class FertilizerCommodityDiammoniumPhosphate2Validator implements Validator<FertilizerCommodityDiammoniumPhosphate2> {

	@Override
	public ValidationResult<FertilizerCommodityDiammoniumPhosphate2> validate(RosettaPath path, FertilizerCommodityDiammoniumPhosphate2 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("basePdct", (AssetClassProductType5Code) o.getBasePdct() != null ? 1 : 0, 1, 1), 
				checkCardinality("subPdct", (AssetClassSubProductType40Code) o.getSubPdct() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("FertilizerCommodityDiammoniumPhosphate2", ValidationType.CARDINALITY, "FertilizerCommodityDiammoniumPhosphate2", path, "", error);
		}
		return success("FertilizerCommodityDiammoniumPhosphate2", ValidationType.CARDINALITY, "FertilizerCommodityDiammoniumPhosphate2", path, "");
	}

}
