package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.MasterAgreement8__1;
import java.math.BigDecimal;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkNumber;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkString;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.stream.Collectors.joining;

public class MasterAgreement8__1TypeFormatValidator implements Validator<MasterAgreement8__1> {

	@Override
	public ValidationResult<MasterAgreement8__1> validate(RosettaPath path, MasterAgreement8__1 o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkNumber("vrsn", o.getVrsn(), empty(), of(0), of(new BigDecimal("1.9E+3")), of(new BigDecimal("2099"))), 
				checkString("othrMstrAgrmtDtls", o.getOthrMstrAgrmtDtls(), 1, of(50), empty())
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("MasterAgreement8__1", ValidationType.TYPE_FORMAT, "MasterAgreement8__1", path, "", error);
		}
		return success("MasterAgreement8__1", ValidationType.TYPE_FORMAT, "MasterAgreement8__1", path, "");
	}

}
