package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.NaturalPersonIdentification3__1;
import java.util.regex.Pattern;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkString;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.stream.Collectors.joining;

public class NaturalPersonIdentification3__1TypeFormatValidator implements Validator<NaturalPersonIdentification3__1> {

	@Override
	public ValidationResult<NaturalPersonIdentification3__1> validate(RosettaPath path, NaturalPersonIdentification3__1 o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkString("ctry", o.getCtry(), 0, empty(), of(Pattern.compile("[A-Z]{2,2}")))
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("NaturalPersonIdentification3__1", ValidationType.TYPE_FORMAT, "NaturalPersonIdentification3__1", path, "", error);
		}
		return success("NaturalPersonIdentification3__1", ValidationType.TYPE_FORMAT, "NaturalPersonIdentification3__1", path, "");
	}

}
