package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.NotionalQuantity9__1;
import iso20022.auth030.esma.QuantityOrTerm1Choice__1;
import java.math.BigDecimal;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class NotionalQuantity9__1Validator implements Validator<NotionalQuantity9__1> {

	@Override
	public ValidationResult<NotionalQuantity9__1> validate(RosettaPath path, NotionalQuantity9__1 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("ttlQty", (BigDecimal) o.getTtlQty() != null ? 1 : 0, 0, 1), 
				checkCardinality("dtls", (QuantityOrTerm1Choice__1) o.getDtls() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("NotionalQuantity9__1", ValidationType.CARDINALITY, "NotionalQuantity9__1", path, "", error);
		}
		return success("NotionalQuantity9__1", ValidationType.CARDINALITY, "NotionalQuantity9__1", path, "");
	}

}
