package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.records.Date;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.AmountAndDirection106__2;
import iso20022.auth030.esma.OtherPayment5__1;
import iso20022.auth030.esma.PartyIdentification236Choice__1;
import iso20022.auth030.esma.PaymentType5Choice__1;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class OtherPayment5__1Validator implements Validator<OtherPayment5__1> {

	@Override
	public ValidationResult<OtherPayment5__1> validate(RosettaPath path, OtherPayment5__1 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("pmtAmt", (AmountAndDirection106__2) o.getPmtAmt() != null ? 1 : 0, 0, 1), 
				checkCardinality("pmtTp", (PaymentType5Choice__1) o.getPmtTp() != null ? 1 : 0, 0, 1), 
				checkCardinality("pmtDt", (Date) o.getPmtDt() != null ? 1 : 0, 0, 1), 
				checkCardinality("pmtPyer", (PartyIdentification236Choice__1) o.getPmtPyer() != null ? 1 : 0, 0, 1), 
				checkCardinality("pmtRcvr", (PartyIdentification236Choice__1) o.getPmtRcvr() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("OtherPayment5__1", ValidationType.CARDINALITY, "OtherPayment5__1", path, "", error);
		}
		return success("OtherPayment5__1", ValidationType.CARDINALITY, "OtherPayment5__1", path, "");
	}

}
