package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.Schedule10__1;
import java.math.BigDecimal;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkNumber;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.stream.Collectors.joining;

public class Schedule10__1TypeFormatValidator implements Validator<Schedule10__1> {

	@Override
	public ValidationResult<Schedule10__1> validate(RosettaPath path, Schedule10__1 o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkNumber("qty", o.getQty(), of(25), of(5), of(new BigDecimal("0")), empty())
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Schedule10__1", ValidationType.TYPE_FORMAT, "Schedule10__1", path, "", error);
		}
		return success("Schedule10__1", ValidationType.TYPE_FORMAT, "Schedule10__1", path, "");
	}

}
