package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.SecuritiesTransactionPrice17Choice;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkNumber;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.stream.Collectors.joining;

public class SecuritiesTransactionPrice17ChoiceTypeFormatValidator implements Validator<SecuritiesTransactionPrice17Choice> {

	@Override
	public ValidationResult<SecuritiesTransactionPrice17Choice> validate(RosettaPath path, SecuritiesTransactionPrice17Choice o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkNumber("unit", o.getUnit(), of(25), of(19), empty(), empty()), 
				checkNumber("pctg", o.getPctg(), of(11), of(10), empty(), empty()), 
				checkNumber("yld", o.getYld(), of(11), of(10), empty(), empty()), 
				checkNumber("dcml", o.getDcml(), of(11), of(10), empty(), empty())
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("SecuritiesTransactionPrice17Choice", ValidationType.TYPE_FORMAT, "SecuritiesTransactionPrice17Choice", path, "", error);
		}
		return success("SecuritiesTransactionPrice17Choice", ValidationType.TYPE_FORMAT, "SecuritiesTransactionPrice17Choice", path, "");
	}

}
