package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.SecuritiesTransactionPrice5;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkNumber;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkString;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.stream.Collectors.joining;

public class SecuritiesTransactionPrice5TypeFormatValidator implements Validator<SecuritiesTransactionPrice5> {

	@Override
	public ValidationResult<SecuritiesTransactionPrice5> validate(RosettaPath path, SecuritiesTransactionPrice5 o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkNumber("val", o.getVal(), of(25), of(19), empty(), empty()), 
				checkString("tp", o.getTp(), 1, of(35), empty())
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("SecuritiesTransactionPrice5", ValidationType.TYPE_FORMAT, "SecuritiesTransactionPrice5", path, "", error);
		}
		return success("SecuritiesTransactionPrice5", ValidationType.TYPE_FORMAT, "SecuritiesTransactionPrice5", path, "");
	}

}
