package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.TradeConfirmationType2Code;
import iso20022.auth030.esma.TradeNonConfirmation1;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class TradeNonConfirmation1Validator implements Validator<TradeNonConfirmation1> {

	@Override
	public ValidationResult<TradeNonConfirmation1> validate(RosettaPath path, TradeNonConfirmation1 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("tp", (TradeConfirmationType2Code) o.getTp() != null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("TradeNonConfirmation1", ValidationType.CARDINALITY, "TradeNonConfirmation1", path, "", error);
		}
		return success("TradeNonConfirmation1", ValidationType.CARDINALITY, "TradeNonConfirmation1", path, "");
	}

}
