package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.Tranche3;
import java.math.BigDecimal;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class Tranche3Validator implements Validator<Tranche3> {

	@Override
	public ValidationResult<Tranche3> validate(RosettaPath path, Tranche3 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("attchmntPt", (BigDecimal) o.getAttchmntPt() != null ? 1 : 0, 0, 1), 
				checkCardinality("dtchmntPt", (BigDecimal) o.getDtchmntPt() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Tranche3", ValidationType.CARDINALITY, "Tranche3", path, "", error);
		}
		return success("Tranche3", ValidationType.CARDINALITY, "Tranche3", path, "");
	}

}
