package iso20022.auth030.esma.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.CounterpartyTradeNature15Choice__2;
import java.util.Arrays;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version ${project.version}
 */
@RosettaDataRule("CounterpartyTradeNature15Choice__2Choice")
@ImplementedBy(CounterpartyTradeNature15Choice2Choice.Default.class)
public interface CounterpartyTradeNature15Choice2Choice extends Validator<CounterpartyTradeNature15Choice__2> {
	
	String NAME = "CounterpartyTradeNature15Choice__2Choice";
	String DEFINITION = "one-of";
	
	ValidationResult<CounterpartyTradeNature15Choice__2> validate(RosettaPath path, CounterpartyTradeNature15Choice__2 counterpartyTradeNature15Choice__2);
	
	class Default implements CounterpartyTradeNature15Choice2Choice {
	
		@Override
		public ValidationResult<CounterpartyTradeNature15Choice__2> validate(RosettaPath path, CounterpartyTradeNature15Choice__2 counterpartyTradeNature15Choice__2) {
			ComparisonResult result = executeDataRule(counterpartyTradeNature15Choice__2);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CounterpartyTradeNature15Choice__2", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "CounterpartyTradeNature15Choice__2", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(CounterpartyTradeNature15Choice__2 counterpartyTradeNature15Choice__2) {
			try {
				return choice(MapperS.of(counterpartyTradeNature15Choice__2), Arrays.asList("fi", "nfi", "cntrlCntrPty", "othr"), ChoiceRuleValidationMethod.REQUIRED);
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements CounterpartyTradeNature15Choice2Choice {
	
		@Override
		public ValidationResult<CounterpartyTradeNature15Choice__2> validate(RosettaPath path, CounterpartyTradeNature15Choice__2 counterpartyTradeNature15Choice__2) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "CounterpartyTradeNature15Choice__2", path, DEFINITION);
		}
	}
}
