package iso20022.auth030.esma.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.DateAndDateTime2Choice__1;
import java.util.Arrays;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version ${project.version}
 */
@RosettaDataRule("DateAndDateTime2Choice__1Choice")
@ImplementedBy(DateAndDateTime2Choice1Choice.Default.class)
public interface DateAndDateTime2Choice1Choice extends Validator<DateAndDateTime2Choice__1> {
	
	String NAME = "DateAndDateTime2Choice__1Choice";
	String DEFINITION = "one-of";
	
	ValidationResult<DateAndDateTime2Choice__1> validate(RosettaPath path, DateAndDateTime2Choice__1 dateAndDateTime2Choice__1);
	
	class Default implements DateAndDateTime2Choice1Choice {
	
		@Override
		public ValidationResult<DateAndDateTime2Choice__1> validate(RosettaPath path, DateAndDateTime2Choice__1 dateAndDateTime2Choice__1) {
			ComparisonResult result = executeDataRule(dateAndDateTime2Choice__1);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "DateAndDateTime2Choice__1", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "DateAndDateTime2Choice__1", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(DateAndDateTime2Choice__1 dateAndDateTime2Choice__1) {
			try {
				return choice(MapperS.of(dateAndDateTime2Choice__1), Arrays.asList("dt"), ChoiceRuleValidationMethod.REQUIRED);
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements DateAndDateTime2Choice1Choice {
	
		@Override
		public ValidationResult<DateAndDateTime2Choice__1> validate(RosettaPath path, DateAndDateTime2Choice__1 dateAndDateTime2Choice__1) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "DateAndDateTime2Choice__1", path, DEFINITION);
		}
	}
}
