package iso20022.auth030.esma.validation.exists;

import com.google.common.collect.ImmutableMap;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ExistenceChecker;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.ValidatorWithArg;
import iso20022.auth030.esma.NoReasonCode;
import iso20022.auth030.esma.Tranche3;
import iso20022.auth030.esma.TrancheIndicator3Choice;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;

public class TrancheIndicator3ChoiceOnlyExistsValidator implements ValidatorWithArg<TrancheIndicator3Choice, Set<String>> {

	/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
	@Override
	public <T2 extends TrancheIndicator3Choice> ValidationResult<TrancheIndicator3Choice> validate(RosettaPath path, T2 o, Set<String> fields) {
		Map<String, Boolean> fieldExistenceMap = ImmutableMap.<String, Boolean>builder()
				.put("trnchd", ExistenceChecker.isSet((Tranche3) o.getTrnchd()))
				.put("utrnchd", ExistenceChecker.isSet((NoReasonCode) o.getUtrnchd()))
				.build();
		
		// Find the fields that are set
		Set<String> setFields = fieldExistenceMap.entrySet().stream()
				.filter(Map.Entry::getValue)
				.map(Map.Entry::getKey)
				.collect(Collectors.toSet());
		
		if (setFields.equals(fields)) {
			return success("TrancheIndicator3Choice", ValidationType.ONLY_EXISTS, "TrancheIndicator3Choice", path, "");
		}
		return failure("TrancheIndicator3Choice", ValidationType.ONLY_EXISTS, "TrancheIndicator3Choice", path, "",
				String.format("[%s] should only be set.  Set fields: %s", fields, setFields));
	}
}
