package iso20022.auth030.jfsa.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.jfsa.ActiveOrHistoricCurrencyAnd13DecimalAmount;
import java.math.BigDecimal;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class ActiveOrHistoricCurrencyAnd13DecimalAmountValidator implements Validator<ActiveOrHistoricCurrencyAnd13DecimalAmount> {

	@Override
	public ValidationResult<ActiveOrHistoricCurrencyAnd13DecimalAmount> validate(RosettaPath path, ActiveOrHistoricCurrencyAnd13DecimalAmount o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("value", (BigDecimal) o.getValue() != null ? 1 : 0, 1, 1), 
				checkCardinality("ccy", (String) o.getCcy() != null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ActiveOrHistoricCurrencyAnd13DecimalAmount", ValidationType.CARDINALITY, "ActiveOrHistoricCurrencyAnd13DecimalAmount", path, "", error);
		}
		return success("ActiveOrHistoricCurrencyAnd13DecimalAmount", ValidationType.CARDINALITY, "ActiveOrHistoricCurrencyAnd13DecimalAmount", path, "");
	}

}
