package iso20022.auth030.jfsa.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.jfsa.ClearingPartyAndTime23__2;
import iso20022.auth030.jfsa.OrganisationIdentification15Choice__1;
import iso20022.auth030.jfsa.UniqueTransactionIdentifier1Choice__1;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class ClearingPartyAndTime23__2Validator implements Validator<ClearingPartyAndTime23__2> {

	@Override
	public ValidationResult<ClearingPartyAndTime23__2> validate(RosettaPath path, ClearingPartyAndTime23__2 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("ccp", (OrganisationIdentification15Choice__1) o.getCcp() != null ? 1 : 0, 0, 1), 
				checkCardinality("orgnlIdr", (UniqueTransactionIdentifier1Choice__1) o.getOrgnlIdr() != null ? 1 : 0, 0, 1), 
				checkCardinality("orgnlTradRpstryIdr", (OrganisationIdentification15Choice__1) o.getOrgnlTradRpstryIdr() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ClearingPartyAndTime23__2", ValidationType.CARDINALITY, "ClearingPartyAndTime23__2", path, "", error);
		}
		return success("ClearingPartyAndTime23__2", ValidationType.CARDINALITY, "ClearingPartyAndTime23__2", path, "");
	}

}
