package iso20022.auth030.jfsa.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.jfsa.ContractType14__2;
import iso20022.auth030.jfsa.CurrencyExchange23__1;
import iso20022.auth030.jfsa.FinancialInstrumentContractType2Code;
import iso20022.auth030.jfsa.ProductType4Code__1;
import iso20022.auth030.jfsa.SecurityIdentification41Choice__1;
import iso20022.auth030.jfsa.SecurityIdentification46__1;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class ContractType14__2Validator implements Validator<ContractType14__2> {

	@Override
	public ValidationResult<ContractType14__2> validate(RosettaPath path, ContractType14__2 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("ctrctTp", (FinancialInstrumentContractType2Code) o.getCtrctTp() != null ? 1 : 0, 1, 1), 
				checkCardinality("asstClss", (ProductType4Code__1) o.getAsstClss() != null ? 1 : 0, 1, 1), 
				checkCardinality("pdctId", (SecurityIdentification46__1) o.getPdctId() != null ? 1 : 0, 0, 1), 
				checkCardinality("undrlygInstrm", (SecurityIdentification41Choice__1) o.getUndrlygInstrm() != null ? 1 : 0, 0, 1), 
				checkCardinality("sttlmCcy", (CurrencyExchange23__1) o.getSttlmCcy() != null ? 1 : 0, 0, 1), 
				checkCardinality("sttlmCcyScndLeg", (CurrencyExchange23__1) o.getSttlmCcyScndLeg() != null ? 1 : 0, 0, 1), 
				checkCardinality("derivBasedOnCrptAsst", (Boolean) o.getDerivBasedOnCrptAsst() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("ContractType14__2", ValidationType.CARDINALITY, "ContractType14__2", path, "", error);
		}
		return success("ContractType14__2", ValidationType.CARDINALITY, "ContractType14__2", path, "");
	}

}
