package iso20022.auth030.jfsa.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.jfsa.Direction2;
import iso20022.auth030.jfsa.OptionParty3Code;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class Direction2Validator implements Validator<Direction2> {

	@Override
	public ValidationResult<Direction2> validate(RosettaPath path, Direction2 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("drctnOfTheFrstLeg", (OptionParty3Code) o.getDrctnOfTheFrstLeg() != null ? 1 : 0, 1, 1), 
				checkCardinality("drctnOfTheScndLeg", (OptionParty3Code) o.getDrctnOfTheScndLeg() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("Direction2", ValidationType.CARDINALITY, "Direction2", path, "", error);
		}
		return success("Direction2", ValidationType.CARDINALITY, "Direction2", path, "");
	}

}
