package iso20022.auth030.jfsa.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.jfsa.GenericIdentification184;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkString;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.stream.Collectors.joining;

public class GenericIdentification184TypeFormatValidator implements Validator<GenericIdentification184> {

	@Override
	public ValidationResult<GenericIdentification184> validate(RosettaPath path, GenericIdentification184 o) {
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkString("id", o.getId(), 1, of(210), empty()), 
				checkString("src", o.getSrc(), 1, of(100), empty())
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("GenericIdentification184", ValidationType.TYPE_FORMAT, "GenericIdentification184", path, "", error);
		}
		return success("GenericIdentification184", ValidationType.TYPE_FORMAT, "GenericIdentification184", path, "");
	}

}
