package iso20022.auth030.jfsa.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.jfsa.NotionalAmount5__2;
import iso20022.auth030.jfsa.NotionalAmount6__2;
import iso20022.auth030.jfsa.NotionalAmountLegs5__2;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class NotionalAmountLegs5__2Validator implements Validator<NotionalAmountLegs5__2> {

	@Override
	public ValidationResult<NotionalAmountLegs5__2> validate(RosettaPath path, NotionalAmountLegs5__2 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("frstLeg", (NotionalAmount5__2) o.getFrstLeg() != null ? 1 : 0, 0, 1), 
				checkCardinality("scndLeg", (NotionalAmount6__2) o.getScndLeg() != null ? 1 : 0, 0, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("NotionalAmountLegs5__2", ValidationType.CARDINALITY, "NotionalAmountLegs5__2", path, "", error);
		}
		return success("NotionalAmountLegs5__2", ValidationType.CARDINALITY, "NotionalAmountLegs5__2", path, "");
	}

}
