package iso20022.auth108.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth108.esma.TradeReportHeader4__1;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class TradeReportHeader4__1Validator implements Validator<TradeReportHeader4__1> {

	@Override
	public ValidationResult<TradeReportHeader4__1> validate(RosettaPath path, TradeReportHeader4__1 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("nbRcrds", (Long) o.getNbRcrds() != null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("TradeReportHeader4__1", ValidationType.CARDINALITY, "TradeReportHeader4__1", path, "", error);
		}
		return success("TradeReportHeader4__1", ValidationType.CARDINALITY, "TradeReportHeader4__1", path, "");
	}

}
