package iso20022.auth108.jfsa.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth108.jfsa.DerivativesTradeMarginDataReportV01;
import iso20022.auth108.jfsa.TradeData55Choice__1;
import iso20022.auth108.jfsa.TradeReportHeader4__1;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class DerivativesTradeMarginDataReportV01Validator implements Validator<DerivativesTradeMarginDataReportV01> {

	@Override
	public ValidationResult<DerivativesTradeMarginDataReportV01> validate(RosettaPath path, DerivativesTradeMarginDataReportV01 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("rptHdr", (TradeReportHeader4__1) o.getRptHdr() != null ? 1 : 0, 1, 1), 
				checkCardinality("tradData", (TradeData55Choice__1) o.getTradData() != null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("DerivativesTradeMarginDataReportV01", ValidationType.CARDINALITY, "DerivativesTradeMarginDataReportV01", path, "", error);
		}
		return success("DerivativesTradeMarginDataReportV01", ValidationType.CARDINALITY, "DerivativesTradeMarginDataReportV01", path, "");
	}

}
