package iso20022.auth108.jfsa.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth108.jfsa.CollateralPortfolioCode5Choice__1;
import iso20022.auth108.jfsa.CollateralisationType3Code;
import iso20022.auth108.jfsa.MarginCollateralReport4__1;
import java.time.ZonedDateTime;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;

public class MarginCollateralReport4__1Validator implements Validator<MarginCollateralReport4__1> {

	@Override
	public ValidationResult<MarginCollateralReport4__1> validate(RosettaPath path, MarginCollateralReport4__1 o) {
		/* Casting is required to ensure types are output to ensure recompilation in Rosetta */
		String error = 
			Lists.<ComparisonResult>newArrayList(
				checkCardinality("collPrtflCd", (CollateralPortfolioCode5Choice__1) o.getCollPrtflCd() != null ? 1 : 0, 1, 1), 
				checkCardinality("collstnCtgy", (CollateralisationType3Code) o.getCollstnCtgy() != null ? 1 : 0, 1, 1), 
				checkCardinality("tmStmp", (ZonedDateTime) o.getTmStmp() != null ? 1 : 0, 1, 1)
			).stream().filter(res -> !res.get()).map(res -> res.getError()).collect(joining("; "));
		
		if (!isNullOrEmpty(error)) {
			return failure("MarginCollateralReport4__1", ValidationType.CARDINALITY, "MarginCollateralReport4__1", path, "", error);
		}
		return success("MarginCollateralReport4__1", ValidationType.CARDINALITY, "MarginCollateralReport4__1", path, "");
	}

}
