package iso20022.auth030.esma;

import com.rosetta.model.lib.annotations.RosettaEnum;
import com.rosetta.model.lib.annotations.RosettaEnumValue;
import java.util.Collections;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;


/**
 * Further sub product code list for commodity derivative Oil.
 * @version 0.5.1
 */
@RosettaEnum("AssetClassDetailedSubProductType32Code")
public enum AssetClassDetailedSubProductType32Code {

	/**
	 * Commodity attribute of type bakken.
	 */
	@RosettaEnumValue(value = "BAKK") BAKK("BAKK"),
	
	/**
	 * Commodity attribute of type biodiesel.
	 */
	@RosettaEnumValue(value = "BDSL") BDSL("BDSL"),
	
	/**
	 * Commodity attribute of type Brent.
	 */
	@RosettaEnumValue(value = "BRNT") BRNT("BRNT"),
	
	/**
	 * Commodity attribute of type Brent NX (New Expiry).
	 */
	@RosettaEnumValue(value = "BRNX") BRNX("BRNX"),
	
	/**
	 * Commodity attribute of type canadian.
	 */
	@RosettaEnumValue(value = "CNDA") CNDA("CNDA"),
	
	/**
	 * Commodity attribute of type condensate.
	 */
	@RosettaEnumValue(value = "COND") COND("COND"),
	
	/**
	 * Commodity attribute of type diesel.
	 */
	@RosettaEnumValue(value = "DSEL") DSEL("DSEL"),
	
	/**
	 * Commodity attribute of type Dubai.
	 */
	@RosettaEnumValue(value = "DUBA") DUBA("DUBA"),
	
	/**
	 * Commodity attribute of type ESPO (Eastern Siberia Pacific Ocean).
	 */
	@RosettaEnumValue(value = "ESPO") ESPO("ESPO"),
	
	/**
	 * Commodity attribute of type ethanol.
	 */
	@RosettaEnumValue(value = "ETHA") ETHA("ETHA"),
	
	/**
	 * Commodity attribute of type fuel.
	 */
	@RosettaEnumValue(value = "FUEL") FUEL("FUEL"),
	
	/**
	 * Commodity attribute of type fuel oil.
	 */
	@RosettaEnumValue(value = "FOIL") FOIL("FOIL"),
	
	/**
	 * Commodity attribute of type gasoil.
	 */
	@RosettaEnumValue(value = "GOIL") GOIL("GOIL"),
	
	/**
	 * Commodity attribute of type gasoline.
	 */
	@RosettaEnumValue(value = "GSLN") GSLN("GSLN"),
	
	/**
	 * Commodity attribute of type heating oil.
	 */
	@RosettaEnumValue(value = "HEAT") HEAT("HEAT"),
	
	/**
	 * Commodity attribute of type jet fuel.
	 */
	@RosettaEnumValue(value = "JTFL") JTFL("JTFL"),
	
	/**
	 * Commodity attribute of type kerosene.
	 */
	@RosettaEnumValue(value = "KERO") KERO("KERO"),
	
	/**
	 * Commodity attribute of type light Louisiana sweet (LLS).
	 */
	@RosettaEnumValue(value = "LLSO") LLSO("LLSO"),
	
	/**
	 * Commodity attribute of type mars.
	 */
	@RosettaEnumValue(value = "MARS") MARS("MARS"),
	
	/**
	 * Commodity attribute of type naptha.
	 */
	@RosettaEnumValue(value = "NAPH") NAPH("NAPH"),
	
	/**
	 * Commodity attribute of type NGL (Natural Gas Liquids).
	 */
	@RosettaEnumValue(value = "NGLO") NGLO("NGLO"),
	
	/**
	 * Commodity attribute of type tapis.
	 */
	@RosettaEnumValue(value = "TAPI") TAPI("TAPI"),
	
	/**
	 * Commodity attribute of type WTI (West Texas Intermediate).
	 */
	@RosettaEnumValue(value = "WTIO") WTIO("WTIO"),
	
	/**
	 * Commodity attribute of type urals.
	 */
	@RosettaEnumValue(value = "URAL") URAL("URAL"),
	
	/**
	 * Commodity attribute of other type.
	 */
	@RosettaEnumValue(value = "OTHR") OTHR("OTHR")
;
	private static Map<String, AssetClassDetailedSubProductType32Code> values;
	static {
        Map<String, AssetClassDetailedSubProductType32Code> map = new ConcurrentHashMap<>();
		for (AssetClassDetailedSubProductType32Code instance : AssetClassDetailedSubProductType32Code.values()) {
			map.put(instance.toDisplayString(), instance);
		}
		values = Collections.unmodifiableMap(map);
    }

	private final String rosettaName;
	private final String displayName;

	AssetClassDetailedSubProductType32Code(String rosettaName) {
		this(rosettaName, null);
	}

	AssetClassDetailedSubProductType32Code(String rosettaName, String displayName) {
		this.rosettaName = rosettaName;
		this.displayName = displayName;
	}

	public static AssetClassDetailedSubProductType32Code fromDisplayName(String name) {
		AssetClassDetailedSubProductType32Code value = values.get(name);
		if (value == null) {
			throw new IllegalArgumentException("No enum constant with display name \"" + name + "\".");
		}
		return value;
	}

	@Override
	public String toString() {
		return toDisplayString();
	}

	public String toDisplayString() {
		return displayName != null ?  displayName : rosettaName;
	}
}
