package iso20022.auth030.esma;

import com.rosetta.model.lib.annotations.RosettaEnum;
import com.rosetta.model.lib.annotations.RosettaEnumValue;
import java.util.Collections;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;


/**
 * Specifies an energy quantity unit.
 * @version 0.5.1
 */
@RosettaEnum("EnergyQuantityUnit2Code")
public enum EnergyQuantityUnit2Code {

	/**
	 * British Thermal Unit Per Day
	 */
	@RosettaEnumValue(value = "BTUD") BTUD("BTUD"),
	
	/**
	 * Cm per day.
	 */
	@RosettaEnumValue(value = "CMPD") CMPD("CMPD"),
	
	/**
	 * GigaJoule Per Day.
	 */
	@RosettaEnumValue(value = "GJDD") GJDD("GJDD"),
	
	/**
	 * Giga Watt.
	 */
	@RosettaEnumValue(value = "GWAT") GWAT("GWAT"),
	
	/**
	 * Giga Watt hour per day.
	 */
	@RosettaEnumValue(value = "GWHD") GWHD("GWHD"),
	
	/**
	 * Giga Watt hour per hour.
	 */
	@RosettaEnumValue(value = "GWHH") GWHH("GWHH"),
	
	/**
	 * Hundred MegaJoule Per Day.
	 */
	@RosettaEnumValue(value = "HMJD") HMJD("HMJD"),
	
	/**
	 * KTherm per day.
	 */
	@RosettaEnumValue(value = "KTMD") KTMD("KTMD"),
	
	/**
	 * Kilo Watt.
	 */
	@RosettaEnumValue(value = "KWAT") KWAT("KWAT"),
	
	/**
	 * Kilo Watt hour per day.
	 */
	@RosettaEnumValue(value = "KWHD") KWHD("KWHD"),
	
	/**
	 * Kilo Watt hour per hour.
	 */
	@RosettaEnumValue(value = "KWHH") KWHH("KWHH"),
	
	/**
	 * Mcm per day.
	 */
	@RosettaEnumValue(value = "MCMD") MCMD("MCMD"),
	
	/**
	 * MegaJoule Per Day.
	 */
	@RosettaEnumValue(value = "MJDD") MJDD("MJDD"),
	
	/**
	 * Million British Thermal Unit Per Day.
	 */
	@RosettaEnumValue(value = "MBTD") MBTD("MBTD"),
	
	/**
	 * Million MegaJoule Per Day.
	 */
	@RosettaEnumValue(value = "MMJD") MMJD("MMJD"),
	
	/**
	 * MTherm per day.
	 */
	@RosettaEnumValue(value = "MTMD") MTMD("MTMD"),
	
	/**
	 * Mega Watt.
	 */
	@RosettaEnumValue(value = "MWAT") MWAT("MWAT"),
	
	/**
	 * Mega Watt hour per day.
	 */
	@RosettaEnumValue(value = "MWHD") MWHD("MWHD"),
	
	/**
	 * Mega Watt hour per hour.
	 */
	@RosettaEnumValue(value = "MWHH") MWHH("MWHH"),
	
	/**
	 * Therm per day.
	 */
	@RosettaEnumValue(value = "THMD") THMD("THMD")
;
	private static Map<String, EnergyQuantityUnit2Code> values;
	static {
        Map<String, EnergyQuantityUnit2Code> map = new ConcurrentHashMap<>();
		for (EnergyQuantityUnit2Code instance : EnergyQuantityUnit2Code.values()) {
			map.put(instance.toDisplayString(), instance);
		}
		values = Collections.unmodifiableMap(map);
    }

	private final String rosettaName;
	private final String displayName;

	EnergyQuantityUnit2Code(String rosettaName) {
		this(rosettaName, null);
	}

	EnergyQuantityUnit2Code(String rosettaName, String displayName) {
		this.rosettaName = rosettaName;
		this.displayName = displayName;
	}

	public static EnergyQuantityUnit2Code fromDisplayName(String name) {
		EnergyQuantityUnit2Code value = values.get(name);
		if (value == null) {
			throw new IllegalArgumentException("No enum constant with display name \"" + name + "\".");
		}
		return value;
	}

	@Override
	public String toString() {
		return toDisplayString();
	}

	public String toDisplayString() {
		return displayName != null ?  displayName : rosettaName;
	}
}
