package iso20022.auth030.esma;

import com.rosetta.model.lib.annotations.RosettaEnum;
import com.rosetta.model.lib.annotations.RosettaEnumValue;
import java.util.Collections;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;


/**
 * Specifies the external benchmark curve name code in the format of character string with a maximum length of 4 characters.
 * @version 0.5.1
 */
@RosettaEnum("FloatingRateIndex1Code")
public enum FloatingRateIndex1Code {

	/**
	 * BBSW
	 */
	@RosettaEnumValue(value = "BBSW") BBSW("BBSW"),
	
	/**
	 * BUBOR
	 */
	@RosettaEnumValue(value = "BUBO") BUBO("BUBO"),
	
	/**
	 * CDOR
	 */
	@RosettaEnumValue(value = "CDOR") CDOR("CDOR"),
	
	/**
	 * CIBOR
	 */
	@RosettaEnumValue(value = "CIBO") CIBO("CIBO"),
	
	/**
	 * CZEONIA
	 */
	@RosettaEnumValue(value = "CZNA") CZNA("CZNA"),
	
	/**
	 * Effective Federal Funds Rate
	 */
	@RosettaEnumValue(value = "EFFR") EFFR("EFFR"),
	
	/**
	 * EONIA
	 */
	@RosettaEnumValue(value = "EONA") EONA("EONA"),
	
	/**
	 * EONIA SWAP
	 */
	@RosettaEnumValue(value = "EONS") EONS("EONS"),
	
	/**
	 * €STR
	 */
	@RosettaEnumValue(value = "ESTR") ESTR("ESTR"),
	
	/**
	 * EuroSwiss
	 */
	@RosettaEnumValue(value = "EUCH") EUCH("EUCH"),
	
	/**
	 * EURIBOR
	 */
	@RosettaEnumValue(value = "EURI") EURI("EURI"),
	
	/**
	 * EURODOLLAR
	 */
	@RosettaEnumValue(value = "EUUS") EUUS("EUUS"),
	
	/**
	 * Future SWAP
	 */
	@RosettaEnumValue(value = "FUSW") FUSW("FUSW"),
	
	/**
	 * GCF REPO
	 */
	@RosettaEnumValue(value = "GCFR") GCFR("GCFR"),
	
	/**
	 * ISDAFIX
	 */
	@RosettaEnumValue(value = "ISDA") ISDA("ISDA"),
	
	/**
	 * JIBAR
	 */
	@RosettaEnumValue(value = "JIBA") JIBA("JIBA"),
	
	/**
	 * LIBID
	 */
	@RosettaEnumValue(value = "LIBI") LIBI("LIBI"),
	
	/**
	 * LIBOR
	 */
	@RosettaEnumValue(value = "LIBO") LIBO("LIBO"),
	
	/**
	 * Muni AAA
	 */
	@RosettaEnumValue(value = "MAAA") MAAA("MAAA"),
	
	/**
	 * MOSPRIM
	 */
	@RosettaEnumValue(value = "MOSP") MOSP("MOSP"),
	
	/**
	 * NIBOR
	 */
	@RosettaEnumValue(value = "NIBO") NIBO("NIBO"),
	
	/**
	 * Overnight Bank Funding Rate
	 */
	@RosettaEnumValue(value = "OBFR") OBFR("OBFR"),
	
	/**
	 * Pfandbriefe
	 */
	@RosettaEnumValue(value = "PFAN") PFAN("PFAN"),
	
	/**
	 * PRIBOR
	 */
	@RosettaEnumValue(value = "PRBO") PRBO("PRBO"),
	
	/**
	 * SOFR
	 */
	@RosettaEnumValue(value = "SOFR") SOFR("SOFR"),
	
	/**
	 * SONIA
	 */
	@RosettaEnumValue(value = "SONA") SONA("SONA"),
	
	/**
	 * STIBOR
	 */
	@RosettaEnumValue(value = "STBO") STBO("STBO"),
	
	/**
	 * SWAP
	 */
	@RosettaEnumValue(value = "SWAP") SWAP("SWAP"),
	
	/**
	 * TIBO
	 */
	@RosettaEnumValue(value = "TIBO") TIBO("TIBO"),
	
	/**
	 * TELBOR
	 */
	@RosettaEnumValue(value = "TLBO") TLBO("TLBO"),
	
	/**
	 * Treasury
	 */
	@RosettaEnumValue(value = "TREA") TREA("TREA"),
	
	/**
	 * WIBOR
	 */
	@RosettaEnumValue(value = "WIBO") WIBO("WIBO")
;
	private static Map<String, FloatingRateIndex1Code> values;
	static {
        Map<String, FloatingRateIndex1Code> map = new ConcurrentHashMap<>();
		for (FloatingRateIndex1Code instance : FloatingRateIndex1Code.values()) {
			map.put(instance.toDisplayString(), instance);
		}
		values = Collections.unmodifiableMap(map);
    }

	private final String rosettaName;
	private final String displayName;

	FloatingRateIndex1Code(String rosettaName) {
		this(rosettaName, null);
	}

	FloatingRateIndex1Code(String rosettaName, String displayName) {
		this.rosettaName = rosettaName;
		this.displayName = displayName;
	}

	public static FloatingRateIndex1Code fromDisplayName(String name) {
		FloatingRateIndex1Code value = values.get(name);
		if (value == null) {
			throw new IllegalArgumentException("No enum constant with display name \"" + name + "\".");
		}
		return value;
	}

	@Override
	public String toString() {
		return toDisplayString();
	}

	public String toDisplayString() {
		return displayName != null ?  displayName : rosettaName;
	}
}
