package iso20022.auth030.esma;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import iso20022.auth030.esma.AssetClassProductType8Code;
import iso20022.auth030.esma.AssetClassSubProductType49Code;
import iso20022.auth030.esma.PaperCommodityOther1;
import iso20022.auth030.esma.PaperCommodityOther1.PaperCommodityOther1Builder;
import iso20022.auth030.esma.PaperCommodityOther1.PaperCommodityOther1BuilderImpl;
import iso20022.auth030.esma.PaperCommodityOther1.PaperCommodityOther1Impl;
import iso20022.auth030.esma.meta.PaperCommodityOther1Meta;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * Defines commodity sub-product attributes of a paper derivative of type other.
 * @version 0.5.1
 */
@RosettaDataType(value="PaperCommodityOther1", builder=PaperCommodityOther1.PaperCommodityOther1BuilderImpl.class, version="0.5.1")
public interface PaperCommodityOther1 extends RosettaModelObject {

	PaperCommodityOther1Meta metaData = new PaperCommodityOther1Meta();

	/*********************** Getter Methods  ***********************/
	/**
	 * Base product for the underlying asset class as specified in the classification of commodities derivatives table.
	 */
	AssetClassProductType8Code getBasePdct();
	/**
	 * Sub-product for the underlying asset class.
	 */
	AssetClassSubProductType49Code getSubPdct();

	/*********************** Build Methods  ***********************/
	PaperCommodityOther1 build();
	
	PaperCommodityOther1.PaperCommodityOther1Builder toBuilder();
	
	static PaperCommodityOther1.PaperCommodityOther1Builder builder() {
		return new PaperCommodityOther1.PaperCommodityOther1BuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends PaperCommodityOther1> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends PaperCommodityOther1> getType() {
		return PaperCommodityOther1.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("basePdct"), AssetClassProductType8Code.class, getBasePdct(), this);
		processor.processBasic(path.newSubPath("subPdct"), AssetClassSubProductType49Code.class, getSubPdct(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface PaperCommodityOther1Builder extends PaperCommodityOther1, RosettaModelObjectBuilder {
		PaperCommodityOther1.PaperCommodityOther1Builder setBasePdct(AssetClassProductType8Code basePdct);
		PaperCommodityOther1.PaperCommodityOther1Builder setSubPdct(AssetClassSubProductType49Code subPdct);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("basePdct"), AssetClassProductType8Code.class, getBasePdct(), this);
			processor.processBasic(path.newSubPath("subPdct"), AssetClassSubProductType49Code.class, getSubPdct(), this);
		}
		

		PaperCommodityOther1.PaperCommodityOther1Builder prune();
	}

	/*********************** Immutable Implementation of PaperCommodityOther1  ***********************/
	class PaperCommodityOther1Impl implements PaperCommodityOther1 {
		private final AssetClassProductType8Code basePdct;
		private final AssetClassSubProductType49Code subPdct;
		
		protected PaperCommodityOther1Impl(PaperCommodityOther1.PaperCommodityOther1Builder builder) {
			this.basePdct = builder.getBasePdct();
			this.subPdct = builder.getSubPdct();
		}
		
		@Override
		@RosettaAttribute("basePdct")
		public AssetClassProductType8Code getBasePdct() {
			return basePdct;
		}
		
		@Override
		@RosettaAttribute("subPdct")
		public AssetClassSubProductType49Code getSubPdct() {
			return subPdct;
		}
		
		@Override
		public PaperCommodityOther1 build() {
			return this;
		}
		
		@Override
		public PaperCommodityOther1.PaperCommodityOther1Builder toBuilder() {
			PaperCommodityOther1.PaperCommodityOther1Builder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(PaperCommodityOther1.PaperCommodityOther1Builder builder) {
			ofNullable(getBasePdct()).ifPresent(builder::setBasePdct);
			ofNullable(getSubPdct()).ifPresent(builder::setSubPdct);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			PaperCommodityOther1 _that = getType().cast(o);
		
			if (!Objects.equals(basePdct, _that.getBasePdct())) return false;
			if (!Objects.equals(subPdct, _that.getSubPdct())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (basePdct != null ? basePdct.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (subPdct != null ? subPdct.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PaperCommodityOther1 {" +
				"basePdct=" + this.basePdct + ", " +
				"subPdct=" + this.subPdct +
			'}';
		}
	}

	/*********************** Builder Implementation of PaperCommodityOther1  ***********************/
	class PaperCommodityOther1BuilderImpl implements PaperCommodityOther1.PaperCommodityOther1Builder {
	
		protected AssetClassProductType8Code basePdct;
		protected AssetClassSubProductType49Code subPdct;
	
		public PaperCommodityOther1BuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("basePdct")
		public AssetClassProductType8Code getBasePdct() {
			return basePdct;
		}
		
		@Override
		@RosettaAttribute("subPdct")
		public AssetClassSubProductType49Code getSubPdct() {
			return subPdct;
		}
		
	
		@Override
		@RosettaAttribute("basePdct")
		public PaperCommodityOther1.PaperCommodityOther1Builder setBasePdct(AssetClassProductType8Code basePdct) {
			this.basePdct = basePdct==null?null:basePdct;
			return this;
		}
		@Override
		@RosettaAttribute("subPdct")
		public PaperCommodityOther1.PaperCommodityOther1Builder setSubPdct(AssetClassSubProductType49Code subPdct) {
			this.subPdct = subPdct==null?null:subPdct;
			return this;
		}
		
		@Override
		public PaperCommodityOther1 build() {
			return new PaperCommodityOther1.PaperCommodityOther1Impl(this);
		}
		
		@Override
		public PaperCommodityOther1.PaperCommodityOther1Builder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public PaperCommodityOther1.PaperCommodityOther1Builder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getBasePdct()!=null) return true;
			if (getSubPdct()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public PaperCommodityOther1.PaperCommodityOther1Builder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			PaperCommodityOther1.PaperCommodityOther1Builder o = (PaperCommodityOther1.PaperCommodityOther1Builder) other;
			
			
			merger.mergeBasic(getBasePdct(), o.getBasePdct(), this::setBasePdct);
			merger.mergeBasic(getSubPdct(), o.getSubPdct(), this::setSubPdct);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			PaperCommodityOther1 _that = getType().cast(o);
		
			if (!Objects.equals(basePdct, _that.getBasePdct())) return false;
			if (!Objects.equals(subPdct, _that.getSubPdct())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (basePdct != null ? basePdct.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (subPdct != null ? subPdct.getClass().getName().hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "PaperCommodityOther1Builder {" +
				"basePdct=" + this.basePdct + ", " +
				"subPdct=" + this.subPdct +
			'}';
		}
	}
}
