package iso20022.auth030.esma;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import iso20022.auth030.esma.Tranche3;
import iso20022.auth030.esma.Tranche3.Tranche3Builder;
import iso20022.auth030.esma.Tranche3.Tranche3BuilderImpl;
import iso20022.auth030.esma.Tranche3.Tranche3Impl;
import iso20022.auth030.esma.meta.Tranche3Meta;
import java.math.BigDecimal;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * Indicates derivative contract was tranched.
 * @version 0.5.1
 */
@RosettaDataType(value="Tranche3", builder=Tranche3.Tranche3BuilderImpl.class, version="0.5.1")
public interface Tranche3 extends RosettaModelObject {

	Tranche3Meta metaData = new Tranche3Meta();

	/*********************** Getter Methods  ***********************/
	/**
	 * Indicates the lower point at which the level of losses in the underlying portfolio reduces the notional of the tranche.
	 */
	BigDecimal getAttchmntPt();
	/**
	 * Indicates the point beyond which the losses in the underlying portfolio no longer reduce the notional of the tranche.
	 */
	BigDecimal getDtchmntPt();

	/*********************** Build Methods  ***********************/
	Tranche3 build();
	
	Tranche3.Tranche3Builder toBuilder();
	
	static Tranche3.Tranche3Builder builder() {
		return new Tranche3.Tranche3BuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends Tranche3> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends Tranche3> getType() {
		return Tranche3.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("attchmntPt"), BigDecimal.class, getAttchmntPt(), this);
		processor.processBasic(path.newSubPath("dtchmntPt"), BigDecimal.class, getDtchmntPt(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface Tranche3Builder extends Tranche3, RosettaModelObjectBuilder {
		Tranche3.Tranche3Builder setAttchmntPt(BigDecimal attchmntPt);
		Tranche3.Tranche3Builder setDtchmntPt(BigDecimal dtchmntPt);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("attchmntPt"), BigDecimal.class, getAttchmntPt(), this);
			processor.processBasic(path.newSubPath("dtchmntPt"), BigDecimal.class, getDtchmntPt(), this);
		}
		

		Tranche3.Tranche3Builder prune();
	}

	/*********************** Immutable Implementation of Tranche3  ***********************/
	class Tranche3Impl implements Tranche3 {
		private final BigDecimal attchmntPt;
		private final BigDecimal dtchmntPt;
		
		protected Tranche3Impl(Tranche3.Tranche3Builder builder) {
			this.attchmntPt = builder.getAttchmntPt();
			this.dtchmntPt = builder.getDtchmntPt();
		}
		
		@Override
		@RosettaAttribute("attchmntPt")
		public BigDecimal getAttchmntPt() {
			return attchmntPt;
		}
		
		@Override
		@RosettaAttribute("dtchmntPt")
		public BigDecimal getDtchmntPt() {
			return dtchmntPt;
		}
		
		@Override
		public Tranche3 build() {
			return this;
		}
		
		@Override
		public Tranche3.Tranche3Builder toBuilder() {
			Tranche3.Tranche3Builder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(Tranche3.Tranche3Builder builder) {
			ofNullable(getAttchmntPt()).ifPresent(builder::setAttchmntPt);
			ofNullable(getDtchmntPt()).ifPresent(builder::setDtchmntPt);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			Tranche3 _that = getType().cast(o);
		
			if (!Objects.equals(attchmntPt, _that.getAttchmntPt())) return false;
			if (!Objects.equals(dtchmntPt, _that.getDtchmntPt())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (attchmntPt != null ? attchmntPt.hashCode() : 0);
			_result = 31 * _result + (dtchmntPt != null ? dtchmntPt.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "Tranche3 {" +
				"attchmntPt=" + this.attchmntPt + ", " +
				"dtchmntPt=" + this.dtchmntPt +
			'}';
		}
	}

	/*********************** Builder Implementation of Tranche3  ***********************/
	class Tranche3BuilderImpl implements Tranche3.Tranche3Builder {
	
		protected BigDecimal attchmntPt;
		protected BigDecimal dtchmntPt;
	
		public Tranche3BuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("attchmntPt")
		public BigDecimal getAttchmntPt() {
			return attchmntPt;
		}
		
		@Override
		@RosettaAttribute("dtchmntPt")
		public BigDecimal getDtchmntPt() {
			return dtchmntPt;
		}
		
	
		@Override
		@RosettaAttribute("attchmntPt")
		public Tranche3.Tranche3Builder setAttchmntPt(BigDecimal attchmntPt) {
			this.attchmntPt = attchmntPt==null?null:attchmntPt;
			return this;
		}
		@Override
		@RosettaAttribute("dtchmntPt")
		public Tranche3.Tranche3Builder setDtchmntPt(BigDecimal dtchmntPt) {
			this.dtchmntPt = dtchmntPt==null?null:dtchmntPt;
			return this;
		}
		
		@Override
		public Tranche3 build() {
			return new Tranche3.Tranche3Impl(this);
		}
		
		@Override
		public Tranche3.Tranche3Builder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public Tranche3.Tranche3Builder prune() {
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getAttchmntPt()!=null) return true;
			if (getDtchmntPt()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public Tranche3.Tranche3Builder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			Tranche3.Tranche3Builder o = (Tranche3.Tranche3Builder) other;
			
			
			merger.mergeBasic(getAttchmntPt(), o.getAttchmntPt(), this::setAttchmntPt);
			merger.mergeBasic(getDtchmntPt(), o.getDtchmntPt(), this::setDtchmntPt);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			Tranche3 _that = getType().cast(o);
		
			if (!Objects.equals(attchmntPt, _that.getAttchmntPt())) return false;
			if (!Objects.equals(dtchmntPt, _that.getDtchmntPt())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (attchmntPt != null ? attchmntPt.hashCode() : 0);
			_result = 31 * _result + (dtchmntPt != null ? dtchmntPt.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "Tranche3Builder {" +
				"attchmntPt=" + this.attchmntPt + ", " +
				"dtchmntPt=" + this.dtchmntPt +
			'}';
		}
	}
}
