package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.AssetClassCommodityEnvironmental3Choice;
import iso20022.auth030.esma.EnvironmentCommodityOther2;
import iso20022.auth030.esma.EnvironmentalCommodityCarbonRelated2;
import iso20022.auth030.esma.EnvironmentalCommodityEmission3;
import iso20022.auth030.esma.EnvironmentalCommodityWeather2;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class AssetClassCommodityEnvironmental3ChoiceValidator implements Validator<AssetClassCommodityEnvironmental3Choice> {

	private List<ComparisonResult> getComparisonResults(AssetClassCommodityEnvironmental3Choice o) {
		return Lists.<ComparisonResult>newArrayList(
				checkCardinality("emssns", (EnvironmentalCommodityEmission3) o.getEmssns() != null ? 1 : 0, 0, 1), 
				checkCardinality("wthr", (EnvironmentalCommodityWeather2) o.getWthr() != null ? 1 : 0, 0, 1), 
				checkCardinality("crbnRltd", (EnvironmentalCommodityCarbonRelated2) o.getCrbnRltd() != null ? 1 : 0, 0, 1), 
				checkCardinality("othr", (EnvironmentCommodityOther2) o.getOthr() != null ? 1 : 0, 0, 1)
			);
	}

	@Override
	public ValidationResult<AssetClassCommodityEnvironmental3Choice> validate(RosettaPath path, AssetClassCommodityEnvironmental3Choice o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("AssetClassCommodityEnvironmental3Choice", ValidationType.CARDINALITY, "AssetClassCommodityEnvironmental3Choice", path, "", error);
		}
		return success("AssetClassCommodityEnvironmental3Choice", ValidationType.CARDINALITY, "AssetClassCommodityEnvironmental3Choice", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, AssetClassCommodityEnvironmental3Choice o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("AssetClassCommodityEnvironmental3Choice", ValidationType.CARDINALITY, "AssetClassCommodityEnvironmental3Choice", path, "", res.getError());
				}
				return success("AssetClassCommodityEnvironmental3Choice", ValidationType.CARDINALITY, "AssetClassCommodityEnvironmental3Choice", path, "");
			})
			.collect(toList());
	}

}
