package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.InterestRateContractTerm4__1;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkNumber;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class InterestRateContractTerm4__1TypeFormatValidator implements Validator<InterestRateContractTerm4__1> {

	private List<ComparisonResult> getComparisonResults(InterestRateContractTerm4__1 o) {
		return Lists.<ComparisonResult>newArrayList(
				checkNumber("val", o.getVal(), of(3), of(0), empty(), empty())
			);
	}

	@Override
	public ValidationResult<InterestRateContractTerm4__1> validate(RosettaPath path, InterestRateContractTerm4__1 o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("InterestRateContractTerm4__1", ValidationType.TYPE_FORMAT, "InterestRateContractTerm4__1", path, "", error);
		}
		return success("InterestRateContractTerm4__1", ValidationType.TYPE_FORMAT, "InterestRateContractTerm4__1", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, InterestRateContractTerm4__1 o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("InterestRateContractTerm4__1", ValidationType.TYPE_FORMAT, "InterestRateContractTerm4__1", path, "", res.getError());
				}
				return success("InterestRateContractTerm4__1", ValidationType.TYPE_FORMAT, "InterestRateContractTerm4__1", path, "");
			})
			.collect(toList());
	}

}
