package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.NaturalPersonIdentification3__2;
import java.util.List;
import java.util.regex.Pattern;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkString;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class NaturalPersonIdentification3__2TypeFormatValidator implements Validator<NaturalPersonIdentification3__2> {

	private List<ComparisonResult> getComparisonResults(NaturalPersonIdentification3__2 o) {
		return Lists.<ComparisonResult>newArrayList(
				checkString("ctry", o.getCtry(), 0, empty(), of(Pattern.compile("[A-Z]{2,2}")))
			);
	}

	@Override
	public ValidationResult<NaturalPersonIdentification3__2> validate(RosettaPath path, NaturalPersonIdentification3__2 o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("NaturalPersonIdentification3__2", ValidationType.TYPE_FORMAT, "NaturalPersonIdentification3__2", path, "", error);
		}
		return success("NaturalPersonIdentification3__2", ValidationType.TYPE_FORMAT, "NaturalPersonIdentification3__2", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, NaturalPersonIdentification3__2 o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("NaturalPersonIdentification3__2", ValidationType.TYPE_FORMAT, "NaturalPersonIdentification3__2", path, "", res.getError());
				}
				return success("NaturalPersonIdentification3__2", ValidationType.TYPE_FORMAT, "NaturalPersonIdentification3__2", path, "");
			})
			.collect(toList());
	}

}
