package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.PostTradeRiskReductionIdentifier1__1;
import java.util.List;
import java.util.regex.Pattern;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkString;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.Optional.empty;
import static java.util.Optional.of;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class PostTradeRiskReductionIdentifier1__1TypeFormatValidator implements Validator<PostTradeRiskReductionIdentifier1__1> {

	private List<ComparisonResult> getComparisonResults(PostTradeRiskReductionIdentifier1__1 o) {
		return Lists.<ComparisonResult>newArrayList(
				checkString("strr", o.getStrr(), 0, empty(), of(Pattern.compile("[A-Z0-9]{18,18}[0-9]{2,2}"))), 
				checkString("id", o.getId(), 1, of(32), empty())
			);
	}

	@Override
	public ValidationResult<PostTradeRiskReductionIdentifier1__1> validate(RosettaPath path, PostTradeRiskReductionIdentifier1__1 o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("PostTradeRiskReductionIdentifier1__1", ValidationType.TYPE_FORMAT, "PostTradeRiskReductionIdentifier1__1", path, "", error);
		}
		return success("PostTradeRiskReductionIdentifier1__1", ValidationType.TYPE_FORMAT, "PostTradeRiskReductionIdentifier1__1", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, PostTradeRiskReductionIdentifier1__1 o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("PostTradeRiskReductionIdentifier1__1", ValidationType.TYPE_FORMAT, "PostTradeRiskReductionIdentifier1__1", path, "", res.getError());
				}
				return success("PostTradeRiskReductionIdentifier1__1", ValidationType.TYPE_FORMAT, "PostTradeRiskReductionIdentifier1__1", path, "");
			})
			.collect(toList());
	}

}
