package iso20022.auth030.esma.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.GenericIdentification175__2;
import iso20022.auth030.esma.NoReasonCode;
import iso20022.auth030.esma.UniqueTransactionIdentifier3Choice__1;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class UniqueTransactionIdentifier3Choice__1Validator implements Validator<UniqueTransactionIdentifier3Choice__1> {

	private List<ComparisonResult> getComparisonResults(UniqueTransactionIdentifier3Choice__1 o) {
		return Lists.<ComparisonResult>newArrayList(
				checkCardinality("unqTxIdr", (String) o.getUnqTxIdr() != null ? 1 : 0, 0, 1), 
				checkCardinality("prtry", (GenericIdentification175__2) o.getPrtry() != null ? 1 : 0, 0, 1), 
				checkCardinality("notAvlbl", (NoReasonCode) o.getNotAvlbl() != null ? 1 : 0, 0, 1)
			);
	}

	@Override
	public ValidationResult<UniqueTransactionIdentifier3Choice__1> validate(RosettaPath path, UniqueTransactionIdentifier3Choice__1 o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("UniqueTransactionIdentifier3Choice__1", ValidationType.CARDINALITY, "UniqueTransactionIdentifier3Choice__1", path, "", error);
		}
		return success("UniqueTransactionIdentifier3Choice__1", ValidationType.CARDINALITY, "UniqueTransactionIdentifier3Choice__1", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, UniqueTransactionIdentifier3Choice__1 o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("UniqueTransactionIdentifier3Choice__1", ValidationType.CARDINALITY, "UniqueTransactionIdentifier3Choice__1", path, "", res.getError());
				}
				return success("UniqueTransactionIdentifier3Choice__1", ValidationType.CARDINALITY, "UniqueTransactionIdentifier3Choice__1", path, "");
			})
			.collect(toList());
	}

}
