package iso20022.auth030.esma.validation.datarule;

import com.google.inject.ImplementedBy;
import com.rosetta.model.lib.annotations.RosettaDataRule;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.mapper.MapperS;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ChoiceRuleValidationMethod;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.esma.AssetClassCommodityAgricultural6Choice;
import java.util.Arrays;

import static com.rosetta.model.lib.expression.ExpressionOperators.*;

/**
 * @version 0.5.1
 */
@RosettaDataRule("AssetClassCommodityAgricultural6ChoiceChoice")
@ImplementedBy(AssetClassCommodityAgricultural6ChoiceChoice.Default.class)
public interface AssetClassCommodityAgricultural6ChoiceChoice extends Validator<AssetClassCommodityAgricultural6Choice> {
	
	String NAME = "AssetClassCommodityAgricultural6ChoiceChoice";
	String DEFINITION = "one-of";
	
	ValidationResult<AssetClassCommodityAgricultural6Choice> validate(RosettaPath path, AssetClassCommodityAgricultural6Choice assetClassCommodityAgricultural6Choice);
	
	class Default implements AssetClassCommodityAgricultural6ChoiceChoice {
	
		@Override
		public ValidationResult<AssetClassCommodityAgricultural6Choice> validate(RosettaPath path, AssetClassCommodityAgricultural6Choice assetClassCommodityAgricultural6Choice) {
			ComparisonResult result = executeDataRule(assetClassCommodityAgricultural6Choice);
			if (result.get()) {
				return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "AssetClassCommodityAgricultural6Choice", path, DEFINITION);
			}
			
			String failureMessage = result.getError();
			if (failureMessage == null || failureMessage.contains("Null") || failureMessage == "") {
				failureMessage = "Condition has failed.";
			}
			return ValidationResult.failure(NAME, ValidationType.DATA_RULE, "AssetClassCommodityAgricultural6Choice", path, DEFINITION, failureMessage);
		}
		
		private ComparisonResult executeDataRule(AssetClassCommodityAgricultural6Choice assetClassCommodityAgricultural6Choice) {
			try {
				return choice(MapperS.of(assetClassCommodityAgricultural6Choice), Arrays.asList("grnOilSeed", "soft", "ptt", "olvOil", "dairy", "frstry", "sfd", "liveStock", "grn", "othr"), ChoiceRuleValidationMethod.REQUIRED);
			}
			catch (Exception ex) {
				return ComparisonResult.failure(ex.getMessage());
			}
		}
	}
	
	@SuppressWarnings("unused")
	class NoOp implements AssetClassCommodityAgricultural6ChoiceChoice {
	
		@Override
		public ValidationResult<AssetClassCommodityAgricultural6Choice> validate(RosettaPath path, AssetClassCommodityAgricultural6Choice assetClassCommodityAgricultural6Choice) {
			return ValidationResult.success(NAME, ValidationResult.ValidationType.DATA_RULE, "AssetClassCommodityAgricultural6Choice", path, DEFINITION);
		}
	}
}
