package iso20022.auth030.jfsa.validation;

import com.google.common.collect.Lists;
import com.rosetta.model.lib.expression.ComparisonResult;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.validation.ValidationResult;
import com.rosetta.model.lib.validation.ValidationResult.ValidationType;
import com.rosetta.model.lib.validation.Validator;
import iso20022.auth030.jfsa.CommonTradeDataReport69__2;
import iso20022.auth030.jfsa.ContractType14__1;
import iso20022.auth030.jfsa.TradeTransaction49__2;
import java.util.List;

import static com.google.common.base.Strings.isNullOrEmpty;
import static com.rosetta.model.lib.expression.ExpressionOperators.checkCardinality;
import static com.rosetta.model.lib.validation.ValidationResult.failure;
import static com.rosetta.model.lib.validation.ValidationResult.success;
import static java.util.stream.Collectors.joining;
import static java.util.stream.Collectors.toList;

public class CommonTradeDataReport69__2Validator implements Validator<CommonTradeDataReport69__2> {

	private List<ComparisonResult> getComparisonResults(CommonTradeDataReport69__2 o) {
		return Lists.<ComparisonResult>newArrayList(
				checkCardinality("ctrctData", (ContractType14__1) o.getCtrctData() != null ? 1 : 0, 0, 1), 
				checkCardinality("txData", (TradeTransaction49__2) o.getTxData() != null ? 1 : 0, 1, 1)
			);
	}

	@Override
	public ValidationResult<CommonTradeDataReport69__2> validate(RosettaPath path, CommonTradeDataReport69__2 o) {
		String error = getComparisonResults(o)
			.stream()
			.filter(res -> !res.get())
			.map(res -> res.getError())
			.collect(joining("; "));

		if (!isNullOrEmpty(error)) {
			return failure("CommonTradeDataReport69__2", ValidationType.CARDINALITY, "CommonTradeDataReport69__2", path, "", error);
		}
		return success("CommonTradeDataReport69__2", ValidationType.CARDINALITY, "CommonTradeDataReport69__2", path, "");
	}

	@Override
	public List<ValidationResult<?>> getValidationResults(RosettaPath path, CommonTradeDataReport69__2 o) {
		return getComparisonResults(o)
			.stream()
			.map(res -> {
				if (!isNullOrEmpty(res.getError())) {
					return failure("CommonTradeDataReport69__2", ValidationType.CARDINALITY, "CommonTradeDataReport69__2", path, "", res.getError());
				}
				return success("CommonTradeDataReport69__2", ValidationType.CARDINALITY, "CommonTradeDataReport69__2", path, "");
			})
			.collect(toList());
	}

}
