package iso20022.auth108.esma;

import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import com.rosetta.model.lib.records.Date;
import iso20022.auth108.esma.MarginCollateralReport4__1;
import iso20022.auth108.esma.MarginReportData7__1;
import iso20022.auth108.esma.MarginReportData7__1.MarginReportData7__1Builder;
import iso20022.auth108.esma.MarginReportData7__1.MarginReportData7__1BuilderImpl;
import iso20022.auth108.esma.MarginReportData7__1.MarginReportData7__1Impl;
import iso20022.auth108.esma.PostedMarginOrCollateral6__1;
import iso20022.auth108.esma.ReceivedMarginOrCollateral6__1;
import iso20022.auth108.esma.TradeCounterpartyReport20__1;
import iso20022.auth108.esma.UniqueTransactionIdentifier2Choice__1;
import iso20022.auth108.esma.meta.MarginReportData7__1Meta;
import java.time.ZonedDateTime;
import java.util.Objects;

import static java.util.Optional.ofNullable;

/**
 * Specifies the details of the margin data.
 * @version 0.5.1
 */
@RosettaDataType(value="MarginReportData7__1", builder=MarginReportData7__1.MarginReportData7__1BuilderImpl.class, version="0.5.1")
public interface MarginReportData7__1 extends RosettaModelObject {

	MarginReportData7__1Meta metaData = new MarginReportData7__1Meta();

	/*********************** Getter Methods  ***********************/
	/**
	 * Date and time of submission of the report to the trade repository.
	 */
	ZonedDateTime getRptgTmStmp();
	/**
	 * Data specific to counterparties and related fields.
	 */
	TradeCounterpartyReport20__1 getCtrPtyId();
	/**
	 * Date on which the reportable event pertaining to the transaction and captured by the report took place.
	 */
	Date getEvtDt();
	/**
	 * Choice between a Unique Transaction Identifier (UTI) or a proprietary identifier as agreed with the counterparty.
	 */
	UniqueTransactionIdentifier2Choice__1 getTxId();
	/**
	 * Information related to collateral agreement existing between counterparties.
	 */
	MarginCollateralReport4__1 getColl();
	/**
	 * Information on posted collateral and margin.
	 */
	PostedMarginOrCollateral6__1 getPstdMrgnOrColl();
	/**
	 * Information on received collateral and margin.
	 */
	ReceivedMarginOrCollateral6__1 getRcvdMrgnOrColl();

	/*********************** Build Methods  ***********************/
	MarginReportData7__1 build();
	
	MarginReportData7__1.MarginReportData7__1Builder toBuilder();
	
	static MarginReportData7__1.MarginReportData7__1Builder builder() {
		return new MarginReportData7__1.MarginReportData7__1BuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends MarginReportData7__1> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends MarginReportData7__1> getType() {
		return MarginReportData7__1.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("rptgTmStmp"), ZonedDateTime.class, getRptgTmStmp(), this);
		processRosetta(path.newSubPath("ctrPtyId"), processor, TradeCounterpartyReport20__1.class, getCtrPtyId());
		processor.processBasic(path.newSubPath("evtDt"), Date.class, getEvtDt(), this);
		processRosetta(path.newSubPath("txId"), processor, UniqueTransactionIdentifier2Choice__1.class, getTxId());
		processRosetta(path.newSubPath("coll"), processor, MarginCollateralReport4__1.class, getColl());
		processRosetta(path.newSubPath("pstdMrgnOrColl"), processor, PostedMarginOrCollateral6__1.class, getPstdMrgnOrColl());
		processRosetta(path.newSubPath("rcvdMrgnOrColl"), processor, ReceivedMarginOrCollateral6__1.class, getRcvdMrgnOrColl());
	}
	

	/*********************** Builder Interface  ***********************/
	interface MarginReportData7__1Builder extends MarginReportData7__1, RosettaModelObjectBuilder {
		TradeCounterpartyReport20__1.TradeCounterpartyReport20__1Builder getOrCreateCtrPtyId();
		TradeCounterpartyReport20__1.TradeCounterpartyReport20__1Builder getCtrPtyId();
		UniqueTransactionIdentifier2Choice__1.UniqueTransactionIdentifier2Choice__1Builder getOrCreateTxId();
		UniqueTransactionIdentifier2Choice__1.UniqueTransactionIdentifier2Choice__1Builder getTxId();
		MarginCollateralReport4__1.MarginCollateralReport4__1Builder getOrCreateColl();
		MarginCollateralReport4__1.MarginCollateralReport4__1Builder getColl();
		PostedMarginOrCollateral6__1.PostedMarginOrCollateral6__1Builder getOrCreatePstdMrgnOrColl();
		PostedMarginOrCollateral6__1.PostedMarginOrCollateral6__1Builder getPstdMrgnOrColl();
		ReceivedMarginOrCollateral6__1.ReceivedMarginOrCollateral6__1Builder getOrCreateRcvdMrgnOrColl();
		ReceivedMarginOrCollateral6__1.ReceivedMarginOrCollateral6__1Builder getRcvdMrgnOrColl();
		MarginReportData7__1.MarginReportData7__1Builder setRptgTmStmp(ZonedDateTime rptgTmStmp);
		MarginReportData7__1.MarginReportData7__1Builder setCtrPtyId(TradeCounterpartyReport20__1 ctrPtyId);
		MarginReportData7__1.MarginReportData7__1Builder setEvtDt(Date evtDt);
		MarginReportData7__1.MarginReportData7__1Builder setTxId(UniqueTransactionIdentifier2Choice__1 txId);
		MarginReportData7__1.MarginReportData7__1Builder setColl(MarginCollateralReport4__1 coll);
		MarginReportData7__1.MarginReportData7__1Builder setPstdMrgnOrColl(PostedMarginOrCollateral6__1 pstdMrgnOrColl);
		MarginReportData7__1.MarginReportData7__1Builder setRcvdMrgnOrColl(ReceivedMarginOrCollateral6__1 rcvdMrgnOrColl);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("rptgTmStmp"), ZonedDateTime.class, getRptgTmStmp(), this);
			processRosetta(path.newSubPath("ctrPtyId"), processor, TradeCounterpartyReport20__1.TradeCounterpartyReport20__1Builder.class, getCtrPtyId());
			processor.processBasic(path.newSubPath("evtDt"), Date.class, getEvtDt(), this);
			processRosetta(path.newSubPath("txId"), processor, UniqueTransactionIdentifier2Choice__1.UniqueTransactionIdentifier2Choice__1Builder.class, getTxId());
			processRosetta(path.newSubPath("coll"), processor, MarginCollateralReport4__1.MarginCollateralReport4__1Builder.class, getColl());
			processRosetta(path.newSubPath("pstdMrgnOrColl"), processor, PostedMarginOrCollateral6__1.PostedMarginOrCollateral6__1Builder.class, getPstdMrgnOrColl());
			processRosetta(path.newSubPath("rcvdMrgnOrColl"), processor, ReceivedMarginOrCollateral6__1.ReceivedMarginOrCollateral6__1Builder.class, getRcvdMrgnOrColl());
		}
		

		MarginReportData7__1.MarginReportData7__1Builder prune();
	}

	/*********************** Immutable Implementation of MarginReportData7__1  ***********************/
	class MarginReportData7__1Impl implements MarginReportData7__1 {
		private final ZonedDateTime rptgTmStmp;
		private final TradeCounterpartyReport20__1 ctrPtyId;
		private final Date evtDt;
		private final UniqueTransactionIdentifier2Choice__1 txId;
		private final MarginCollateralReport4__1 coll;
		private final PostedMarginOrCollateral6__1 pstdMrgnOrColl;
		private final ReceivedMarginOrCollateral6__1 rcvdMrgnOrColl;
		
		protected MarginReportData7__1Impl(MarginReportData7__1.MarginReportData7__1Builder builder) {
			this.rptgTmStmp = builder.getRptgTmStmp();
			this.ctrPtyId = ofNullable(builder.getCtrPtyId()).map(f->f.build()).orElse(null);
			this.evtDt = builder.getEvtDt();
			this.txId = ofNullable(builder.getTxId()).map(f->f.build()).orElse(null);
			this.coll = ofNullable(builder.getColl()).map(f->f.build()).orElse(null);
			this.pstdMrgnOrColl = ofNullable(builder.getPstdMrgnOrColl()).map(f->f.build()).orElse(null);
			this.rcvdMrgnOrColl = ofNullable(builder.getRcvdMrgnOrColl()).map(f->f.build()).orElse(null);
		}
		
		@Override
		@RosettaAttribute("rptgTmStmp")
		public ZonedDateTime getRptgTmStmp() {
			return rptgTmStmp;
		}
		
		@Override
		@RosettaAttribute("ctrPtyId")
		public TradeCounterpartyReport20__1 getCtrPtyId() {
			return ctrPtyId;
		}
		
		@Override
		@RosettaAttribute("evtDt")
		public Date getEvtDt() {
			return evtDt;
		}
		
		@Override
		@RosettaAttribute("txId")
		public UniqueTransactionIdentifier2Choice__1 getTxId() {
			return txId;
		}
		
		@Override
		@RosettaAttribute("coll")
		public MarginCollateralReport4__1 getColl() {
			return coll;
		}
		
		@Override
		@RosettaAttribute("pstdMrgnOrColl")
		public PostedMarginOrCollateral6__1 getPstdMrgnOrColl() {
			return pstdMrgnOrColl;
		}
		
		@Override
		@RosettaAttribute("rcvdMrgnOrColl")
		public ReceivedMarginOrCollateral6__1 getRcvdMrgnOrColl() {
			return rcvdMrgnOrColl;
		}
		
		@Override
		public MarginReportData7__1 build() {
			return this;
		}
		
		@Override
		public MarginReportData7__1.MarginReportData7__1Builder toBuilder() {
			MarginReportData7__1.MarginReportData7__1Builder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(MarginReportData7__1.MarginReportData7__1Builder builder) {
			ofNullable(getRptgTmStmp()).ifPresent(builder::setRptgTmStmp);
			ofNullable(getCtrPtyId()).ifPresent(builder::setCtrPtyId);
			ofNullable(getEvtDt()).ifPresent(builder::setEvtDt);
			ofNullable(getTxId()).ifPresent(builder::setTxId);
			ofNullable(getColl()).ifPresent(builder::setColl);
			ofNullable(getPstdMrgnOrColl()).ifPresent(builder::setPstdMrgnOrColl);
			ofNullable(getRcvdMrgnOrColl()).ifPresent(builder::setRcvdMrgnOrColl);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			MarginReportData7__1 _that = getType().cast(o);
		
			if (!Objects.equals(rptgTmStmp, _that.getRptgTmStmp())) return false;
			if (!Objects.equals(ctrPtyId, _that.getCtrPtyId())) return false;
			if (!Objects.equals(evtDt, _that.getEvtDt())) return false;
			if (!Objects.equals(txId, _that.getTxId())) return false;
			if (!Objects.equals(coll, _that.getColl())) return false;
			if (!Objects.equals(pstdMrgnOrColl, _that.getPstdMrgnOrColl())) return false;
			if (!Objects.equals(rcvdMrgnOrColl, _that.getRcvdMrgnOrColl())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (rptgTmStmp != null ? rptgTmStmp.hashCode() : 0);
			_result = 31 * _result + (ctrPtyId != null ? ctrPtyId.hashCode() : 0);
			_result = 31 * _result + (evtDt != null ? evtDt.hashCode() : 0);
			_result = 31 * _result + (txId != null ? txId.hashCode() : 0);
			_result = 31 * _result + (coll != null ? coll.hashCode() : 0);
			_result = 31 * _result + (pstdMrgnOrColl != null ? pstdMrgnOrColl.hashCode() : 0);
			_result = 31 * _result + (rcvdMrgnOrColl != null ? rcvdMrgnOrColl.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MarginReportData7__1 {" +
				"rptgTmStmp=" + this.rptgTmStmp + ", " +
				"ctrPtyId=" + this.ctrPtyId + ", " +
				"evtDt=" + this.evtDt + ", " +
				"txId=" + this.txId + ", " +
				"coll=" + this.coll + ", " +
				"pstdMrgnOrColl=" + this.pstdMrgnOrColl + ", " +
				"rcvdMrgnOrColl=" + this.rcvdMrgnOrColl +
			'}';
		}
	}

	/*********************** Builder Implementation of MarginReportData7__1  ***********************/
	class MarginReportData7__1BuilderImpl implements MarginReportData7__1.MarginReportData7__1Builder {
	
		protected ZonedDateTime rptgTmStmp;
		protected TradeCounterpartyReport20__1.TradeCounterpartyReport20__1Builder ctrPtyId;
		protected Date evtDt;
		protected UniqueTransactionIdentifier2Choice__1.UniqueTransactionIdentifier2Choice__1Builder txId;
		protected MarginCollateralReport4__1.MarginCollateralReport4__1Builder coll;
		protected PostedMarginOrCollateral6__1.PostedMarginOrCollateral6__1Builder pstdMrgnOrColl;
		protected ReceivedMarginOrCollateral6__1.ReceivedMarginOrCollateral6__1Builder rcvdMrgnOrColl;
	
		public MarginReportData7__1BuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("rptgTmStmp")
		public ZonedDateTime getRptgTmStmp() {
			return rptgTmStmp;
		}
		
		@Override
		@RosettaAttribute("ctrPtyId")
		public TradeCounterpartyReport20__1.TradeCounterpartyReport20__1Builder getCtrPtyId() {
			return ctrPtyId;
		}
		
		@Override
		public TradeCounterpartyReport20__1.TradeCounterpartyReport20__1Builder getOrCreateCtrPtyId() {
			TradeCounterpartyReport20__1.TradeCounterpartyReport20__1Builder result;
			if (ctrPtyId!=null) {
				result = ctrPtyId;
			}
			else {
				result = ctrPtyId = TradeCounterpartyReport20__1.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("evtDt")
		public Date getEvtDt() {
			return evtDt;
		}
		
		@Override
		@RosettaAttribute("txId")
		public UniqueTransactionIdentifier2Choice__1.UniqueTransactionIdentifier2Choice__1Builder getTxId() {
			return txId;
		}
		
		@Override
		public UniqueTransactionIdentifier2Choice__1.UniqueTransactionIdentifier2Choice__1Builder getOrCreateTxId() {
			UniqueTransactionIdentifier2Choice__1.UniqueTransactionIdentifier2Choice__1Builder result;
			if (txId!=null) {
				result = txId;
			}
			else {
				result = txId = UniqueTransactionIdentifier2Choice__1.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("coll")
		public MarginCollateralReport4__1.MarginCollateralReport4__1Builder getColl() {
			return coll;
		}
		
		@Override
		public MarginCollateralReport4__1.MarginCollateralReport4__1Builder getOrCreateColl() {
			MarginCollateralReport4__1.MarginCollateralReport4__1Builder result;
			if (coll!=null) {
				result = coll;
			}
			else {
				result = coll = MarginCollateralReport4__1.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("pstdMrgnOrColl")
		public PostedMarginOrCollateral6__1.PostedMarginOrCollateral6__1Builder getPstdMrgnOrColl() {
			return pstdMrgnOrColl;
		}
		
		@Override
		public PostedMarginOrCollateral6__1.PostedMarginOrCollateral6__1Builder getOrCreatePstdMrgnOrColl() {
			PostedMarginOrCollateral6__1.PostedMarginOrCollateral6__1Builder result;
			if (pstdMrgnOrColl!=null) {
				result = pstdMrgnOrColl;
			}
			else {
				result = pstdMrgnOrColl = PostedMarginOrCollateral6__1.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("rcvdMrgnOrColl")
		public ReceivedMarginOrCollateral6__1.ReceivedMarginOrCollateral6__1Builder getRcvdMrgnOrColl() {
			return rcvdMrgnOrColl;
		}
		
		@Override
		public ReceivedMarginOrCollateral6__1.ReceivedMarginOrCollateral6__1Builder getOrCreateRcvdMrgnOrColl() {
			ReceivedMarginOrCollateral6__1.ReceivedMarginOrCollateral6__1Builder result;
			if (rcvdMrgnOrColl!=null) {
				result = rcvdMrgnOrColl;
			}
			else {
				result = rcvdMrgnOrColl = ReceivedMarginOrCollateral6__1.builder();
			}
			
			return result;
		}
	
		@Override
		@RosettaAttribute("rptgTmStmp")
		public MarginReportData7__1.MarginReportData7__1Builder setRptgTmStmp(ZonedDateTime rptgTmStmp) {
			this.rptgTmStmp = rptgTmStmp==null?null:rptgTmStmp;
			return this;
		}
		@Override
		@RosettaAttribute("ctrPtyId")
		public MarginReportData7__1.MarginReportData7__1Builder setCtrPtyId(TradeCounterpartyReport20__1 ctrPtyId) {
			this.ctrPtyId = ctrPtyId==null?null:ctrPtyId.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("evtDt")
		public MarginReportData7__1.MarginReportData7__1Builder setEvtDt(Date evtDt) {
			this.evtDt = evtDt==null?null:evtDt;
			return this;
		}
		@Override
		@RosettaAttribute("txId")
		public MarginReportData7__1.MarginReportData7__1Builder setTxId(UniqueTransactionIdentifier2Choice__1 txId) {
			this.txId = txId==null?null:txId.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("coll")
		public MarginReportData7__1.MarginReportData7__1Builder setColl(MarginCollateralReport4__1 coll) {
			this.coll = coll==null?null:coll.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("pstdMrgnOrColl")
		public MarginReportData7__1.MarginReportData7__1Builder setPstdMrgnOrColl(PostedMarginOrCollateral6__1 pstdMrgnOrColl) {
			this.pstdMrgnOrColl = pstdMrgnOrColl==null?null:pstdMrgnOrColl.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("rcvdMrgnOrColl")
		public MarginReportData7__1.MarginReportData7__1Builder setRcvdMrgnOrColl(ReceivedMarginOrCollateral6__1 rcvdMrgnOrColl) {
			this.rcvdMrgnOrColl = rcvdMrgnOrColl==null?null:rcvdMrgnOrColl.toBuilder();
			return this;
		}
		
		@Override
		public MarginReportData7__1 build() {
			return new MarginReportData7__1.MarginReportData7__1Impl(this);
		}
		
		@Override
		public MarginReportData7__1.MarginReportData7__1Builder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public MarginReportData7__1.MarginReportData7__1Builder prune() {
			if (ctrPtyId!=null && !ctrPtyId.prune().hasData()) ctrPtyId = null;
			if (txId!=null && !txId.prune().hasData()) txId = null;
			if (coll!=null && !coll.prune().hasData()) coll = null;
			if (pstdMrgnOrColl!=null && !pstdMrgnOrColl.prune().hasData()) pstdMrgnOrColl = null;
			if (rcvdMrgnOrColl!=null && !rcvdMrgnOrColl.prune().hasData()) rcvdMrgnOrColl = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getRptgTmStmp()!=null) return true;
			if (getCtrPtyId()!=null && getCtrPtyId().hasData()) return true;
			if (getEvtDt()!=null) return true;
			if (getTxId()!=null && getTxId().hasData()) return true;
			if (getColl()!=null && getColl().hasData()) return true;
			if (getPstdMrgnOrColl()!=null && getPstdMrgnOrColl().hasData()) return true;
			if (getRcvdMrgnOrColl()!=null && getRcvdMrgnOrColl().hasData()) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public MarginReportData7__1.MarginReportData7__1Builder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			MarginReportData7__1.MarginReportData7__1Builder o = (MarginReportData7__1.MarginReportData7__1Builder) other;
			
			merger.mergeRosetta(getCtrPtyId(), o.getCtrPtyId(), this::setCtrPtyId);
			merger.mergeRosetta(getTxId(), o.getTxId(), this::setTxId);
			merger.mergeRosetta(getColl(), o.getColl(), this::setColl);
			merger.mergeRosetta(getPstdMrgnOrColl(), o.getPstdMrgnOrColl(), this::setPstdMrgnOrColl);
			merger.mergeRosetta(getRcvdMrgnOrColl(), o.getRcvdMrgnOrColl(), this::setRcvdMrgnOrColl);
			
			merger.mergeBasic(getRptgTmStmp(), o.getRptgTmStmp(), this::setRptgTmStmp);
			merger.mergeBasic(getEvtDt(), o.getEvtDt(), this::setEvtDt);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			MarginReportData7__1 _that = getType().cast(o);
		
			if (!Objects.equals(rptgTmStmp, _that.getRptgTmStmp())) return false;
			if (!Objects.equals(ctrPtyId, _that.getCtrPtyId())) return false;
			if (!Objects.equals(evtDt, _that.getEvtDt())) return false;
			if (!Objects.equals(txId, _that.getTxId())) return false;
			if (!Objects.equals(coll, _that.getColl())) return false;
			if (!Objects.equals(pstdMrgnOrColl, _that.getPstdMrgnOrColl())) return false;
			if (!Objects.equals(rcvdMrgnOrColl, _that.getRcvdMrgnOrColl())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (rptgTmStmp != null ? rptgTmStmp.hashCode() : 0);
			_result = 31 * _result + (ctrPtyId != null ? ctrPtyId.hashCode() : 0);
			_result = 31 * _result + (evtDt != null ? evtDt.hashCode() : 0);
			_result = 31 * _result + (txId != null ? txId.hashCode() : 0);
			_result = 31 * _result + (coll != null ? coll.hashCode() : 0);
			_result = 31 * _result + (pstdMrgnOrColl != null ? pstdMrgnOrColl.hashCode() : 0);
			_result = 31 * _result + (rcvdMrgnOrColl != null ? rcvdMrgnOrColl.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MarginReportData7__1Builder {" +
				"rptgTmStmp=" + this.rptgTmStmp + ", " +
				"ctrPtyId=" + this.ctrPtyId + ", " +
				"evtDt=" + this.evtDt + ", " +
				"txId=" + this.txId + ", " +
				"coll=" + this.coll + ", " +
				"pstdMrgnOrColl=" + this.pstdMrgnOrColl + ", " +
				"rcvdMrgnOrColl=" + this.rcvdMrgnOrColl +
			'}';
		}
	}
}
