package drr.regulation.asic.rewrite.trade;

import cdm.base.staticdata.asset.common.ISOCountryCodeEnum;
import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import cdm.base.staticdata.asset.common.ProductIdTypeEnum;
import com.google.common.collect.ImmutableList;
import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import com.rosetta.model.lib.records.Date;
import com.rosetta.util.ListEquals;
import drr.regulation.asic.rewrite.trade.ASICNonReportable;
import drr.regulation.asic.rewrite.trade.ASICTransactionReport;
import drr.regulation.asic.rewrite.trade.ASICTransactionReport.ASICTransactionReportBuilder;
import drr.regulation.asic.rewrite.trade.ASICTransactionReport.ASICTransactionReportBuilderImpl;
import drr.regulation.asic.rewrite.trade.ASICTransactionReport.ASICTransactionReportImpl;
import drr.regulation.asic.rewrite.trade.BasketConstituentsReport;
import drr.regulation.asic.rewrite.trade.NotionalAmountScheduleLeg1Report;
import drr.regulation.asic.rewrite.trade.NotionalAmountScheduleLeg2Report;
import drr.regulation.asic.rewrite.trade.OtherPaymentReport;
import drr.regulation.asic.rewrite.trade.PackageTransactionPrice;
import drr.regulation.asic.rewrite.trade.PackageTransactionSpread;
import drr.regulation.asic.rewrite.trade.SpreadLeg1;
import drr.regulation.asic.rewrite.trade.SpreadLeg2;
import drr.regulation.asic.rewrite.trade.StrikePriceAndNotation;
import drr.regulation.asic.rewrite.trade.meta.ASICTransactionReportMeta;
import iso20022.auth030.esma.InterestComputationMethod4Code;
import iso20022.auth030.esma.ValuationType1Code;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.function.Consumer;
import java.util.stream.Collectors;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="ASICTransactionReport", builder=ASICTransactionReport.ASICTransactionReportBuilderImpl.class, version="4.2.1")
public interface ASICTransactionReport extends RosettaModelObject {

	ASICTransactionReportMeta metaData = new ASICTransactionReportMeta();

	/*********************** Getter Methods  ***********************/
	String getUniqueTransactionIdentifier();
	String getUniqueProductIdentifier();
	String getAssetClass();
	String getContractType();
	String getReportingEntity();
	String getCounterparty1();
	String getCounterparty2();
	Boolean getCounterpartyIdentifierTypeIndicator();
	ISOCountryCodeEnum getCountryOfTheCounterParty2();
	String getBeneficiary1();
	Boolean getBeneficiary1IdentifierTypeIndicator();
	String getBroker();
	String getExecutionAgentOfTheCounterparty1();
	String getDirection();
	String getDirection2OfLeg1();
	String getDirection2OfLeg2();
	Date getEffectiveDate();
	Date getExpirationDate();
	ZonedDateTime getExecutionTimestamp();
	ZonedDateTime getEventTimestamp();
	ZonedDateTime getClearingTimestamp();
	String getCleared();
	String getCentralCounterparty();
	String getClearingMember();
	String getPlatformIdentifier();
	BigDecimal getNotionalLeg1();
	BigDecimal getNotionalLeg2();
	BigDecimal getTotalNotionalQuantityLeg1();
	BigDecimal getTotalNotionalQuantityLeg2();
	BigDecimal getNotionalQuantityLeg1();
	BigDecimal getNotionalQuantityLeg2();
	BigDecimal getCallAmount();
	BigDecimal getPutAmount();
	ISOCurrencyCodeEnum getNotionalCurrencyLeg1();
	ISOCurrencyCodeEnum getNotionalCurrencyLeg2();
	String getQuantityUnitOfMeasureLeg1();
	String getQuantityUnitOfMeasureLeg2();
	ISOCurrencyCodeEnum getCallCurrency();
	ISOCurrencyCodeEnum getPutCurrency();
	List<? extends NotionalAmountScheduleLeg1Report> getNotionalAmountScheduleLeg1();
	List<? extends NotionalAmountScheduleLeg2Report> getNotionalAmountScheduleLeg2();
	String getPriceNotation();
	BigDecimal getPrice();
	ISOCurrencyCodeEnum getPriceCurrency();
	String getPriceUnitOfMeasure();
	BigDecimal getFixedRateLeg1();
	BigDecimal getFixedRateLeg2();
	String getSpreadNotationLeg1();
	SpreadLeg1 getSpreadLeg1();
	String getSpreadNotationLeg2();
	SpreadLeg2 getSpreadLeg2();
	ISOCurrencyCodeEnum getSpreadCurrencyLeg1();
	ISOCurrencyCodeEnum getSpreadCurrencyLeg2();
	String getStrikePriceNotation();
	StrikePriceAndNotation getStrikePrice();
	String getStrikePriceCurrency();
	BigDecimal getOptionPremiumAmount();
	ISOCurrencyCodeEnum getOptionPremiumCurrency();
	Date getOptionPremiumPaymentDate();
	BigDecimal getExchangeRate();
	String getExchangeRateBasis();
	InterestComputationMethod4Code getDayCountConventionLeg1();
	InterestComputationMethod4Code getDayCountConventionLeg2();
	String getPaymentFrequencyPeriodLeg1();
	String getPaymentFrequencyPeriodLeg2();
	Integer getPaymentFrequencyPeriodMultiplierLeg1();
	Integer getPaymentFrequencyPeriodMultiplierLeg2();
	ISOCurrencyCodeEnum getSettlementCurrencyLeg1();
	String getSettlementCurrencyLeg2();
	List<? extends OtherPaymentReport> getOtherPayment();
	String getCustomBasketCode();
	List<? extends BasketConstituentsReport> getBasketConstituents();
	List<ProductIdTypeEnum> getBasketConstituentIdentifierSource();
	String getUnderlyingIdentification();
	String getUnderlierIDSource();
	Date getMaturityDateOfTheUnderlying();
	String getIdentifierOfTheFloatingRateLeg2();
	String getFloatingRateReferencePeriodOfLeg2TimePeriod();
	Integer getFloatingRateReferencePeriodOfLeg2Multiplier();
	BigDecimal getCdSIndexAttachmentPoint();
	BigDecimal getCdSIndexDetachmentPoint();
	BigDecimal getIndexFactor();
	String getPackageIdentifier();
	PackageTransactionPrice getPackageTransactionPrice();
	PackageTransactionSpread getPackageTransactionSpread();
	String getPriorUTI();
	String getEventIdentifier();
	String getActionType();
	String getEventType();
	ZonedDateTime getReportingTimestamp();
	String getReportSubmittingEntityID();
	ZonedDateTime getValuationTimestamp();
	BigDecimal getValuationAmount();
	ISOCurrencyCodeEnum getValuationCurrency();
	ValuationType1Code getValuationMethod();
	BigDecimal getDelta();
	Date getNextFloatingReferenceResetDateLeg1();
	Date getNextFloatingReferenceResetDateLeg2();
	String getSmallScaleBuySideEntityIndicator();
	ASICNonReportable getNonReportable();

	/*********************** Build Methods  ***********************/
	ASICTransactionReport build();
	
	ASICTransactionReport.ASICTransactionReportBuilder toBuilder();
	
	static ASICTransactionReport.ASICTransactionReportBuilder builder() {
		return new ASICTransactionReport.ASICTransactionReportBuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends ASICTransactionReport> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends ASICTransactionReport> getType() {
		return ASICTransactionReport.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("uniqueTransactionIdentifier"), String.class, getUniqueTransactionIdentifier(), this);
		processor.processBasic(path.newSubPath("uniqueProductIdentifier"), String.class, getUniqueProductIdentifier(), this);
		processor.processBasic(path.newSubPath("assetClass"), String.class, getAssetClass(), this);
		processor.processBasic(path.newSubPath("contractType"), String.class, getContractType(), this);
		processor.processBasic(path.newSubPath("reportingEntity"), String.class, getReportingEntity(), this);
		processor.processBasic(path.newSubPath("counterparty1"), String.class, getCounterparty1(), this);
		processor.processBasic(path.newSubPath("counterparty2"), String.class, getCounterparty2(), this);
		processor.processBasic(path.newSubPath("counterpartyIdentifierTypeIndicator"), Boolean.class, getCounterpartyIdentifierTypeIndicator(), this);
		processor.processBasic(path.newSubPath("countryOfTheCounterParty2"), ISOCountryCodeEnum.class, getCountryOfTheCounterParty2(), this);
		processor.processBasic(path.newSubPath("beneficiary1"), String.class, getBeneficiary1(), this);
		processor.processBasic(path.newSubPath("beneficiary1IdentifierTypeIndicator"), Boolean.class, getBeneficiary1IdentifierTypeIndicator(), this);
		processor.processBasic(path.newSubPath("broker"), String.class, getBroker(), this);
		processor.processBasic(path.newSubPath("executionAgentOfTheCounterparty1"), String.class, getExecutionAgentOfTheCounterparty1(), this);
		processor.processBasic(path.newSubPath("direction"), String.class, getDirection(), this);
		processor.processBasic(path.newSubPath("direction2OfLeg1"), String.class, getDirection2OfLeg1(), this);
		processor.processBasic(path.newSubPath("direction2OfLeg2"), String.class, getDirection2OfLeg2(), this);
		processor.processBasic(path.newSubPath("effectiveDate"), Date.class, getEffectiveDate(), this);
		processor.processBasic(path.newSubPath("expirationDate"), Date.class, getExpirationDate(), this);
		processor.processBasic(path.newSubPath("executionTimestamp"), ZonedDateTime.class, getExecutionTimestamp(), this);
		processor.processBasic(path.newSubPath("eventTimestamp"), ZonedDateTime.class, getEventTimestamp(), this);
		processor.processBasic(path.newSubPath("clearingTimestamp"), ZonedDateTime.class, getClearingTimestamp(), this);
		processor.processBasic(path.newSubPath("cleared"), String.class, getCleared(), this);
		processor.processBasic(path.newSubPath("centralCounterparty"), String.class, getCentralCounterparty(), this);
		processor.processBasic(path.newSubPath("clearingMember"), String.class, getClearingMember(), this);
		processor.processBasic(path.newSubPath("platformIdentifier"), String.class, getPlatformIdentifier(), this);
		processor.processBasic(path.newSubPath("notionalLeg1"), BigDecimal.class, getNotionalLeg1(), this);
		processor.processBasic(path.newSubPath("notionalLeg2"), BigDecimal.class, getNotionalLeg2(), this);
		processor.processBasic(path.newSubPath("totalNotionalQuantityLeg1"), BigDecimal.class, getTotalNotionalQuantityLeg1(), this);
		processor.processBasic(path.newSubPath("totalNotionalQuantityLeg2"), BigDecimal.class, getTotalNotionalQuantityLeg2(), this);
		processor.processBasic(path.newSubPath("notionalQuantityLeg1"), BigDecimal.class, getNotionalQuantityLeg1(), this);
		processor.processBasic(path.newSubPath("notionalQuantityLeg2"), BigDecimal.class, getNotionalQuantityLeg2(), this);
		processor.processBasic(path.newSubPath("callAmount"), BigDecimal.class, getCallAmount(), this);
		processor.processBasic(path.newSubPath("putAmount"), BigDecimal.class, getPutAmount(), this);
		processor.processBasic(path.newSubPath("notionalCurrencyLeg1"), ISOCurrencyCodeEnum.class, getNotionalCurrencyLeg1(), this);
		processor.processBasic(path.newSubPath("notionalCurrencyLeg2"), ISOCurrencyCodeEnum.class, getNotionalCurrencyLeg2(), this);
		processor.processBasic(path.newSubPath("quantityUnitOfMeasureLeg1"), String.class, getQuantityUnitOfMeasureLeg1(), this);
		processor.processBasic(path.newSubPath("quantityUnitOfMeasureLeg2"), String.class, getQuantityUnitOfMeasureLeg2(), this);
		processor.processBasic(path.newSubPath("callCurrency"), ISOCurrencyCodeEnum.class, getCallCurrency(), this);
		processor.processBasic(path.newSubPath("putCurrency"), ISOCurrencyCodeEnum.class, getPutCurrency(), this);
		processRosetta(path.newSubPath("notionalAmountScheduleLeg1"), processor, NotionalAmountScheduleLeg1Report.class, getNotionalAmountScheduleLeg1());
		processRosetta(path.newSubPath("notionalAmountScheduleLeg2"), processor, NotionalAmountScheduleLeg2Report.class, getNotionalAmountScheduleLeg2());
		processor.processBasic(path.newSubPath("priceNotation"), String.class, getPriceNotation(), this);
		processor.processBasic(path.newSubPath("price"), BigDecimal.class, getPrice(), this);
		processor.processBasic(path.newSubPath("priceCurrency"), ISOCurrencyCodeEnum.class, getPriceCurrency(), this);
		processor.processBasic(path.newSubPath("priceUnitOfMeasure"), String.class, getPriceUnitOfMeasure(), this);
		processor.processBasic(path.newSubPath("fixedRateLeg1"), BigDecimal.class, getFixedRateLeg1(), this);
		processor.processBasic(path.newSubPath("fixedRateLeg2"), BigDecimal.class, getFixedRateLeg2(), this);
		processor.processBasic(path.newSubPath("spreadNotationLeg1"), String.class, getSpreadNotationLeg1(), this);
		processRosetta(path.newSubPath("spreadLeg1"), processor, SpreadLeg1.class, getSpreadLeg1());
		processor.processBasic(path.newSubPath("spreadNotationLeg2"), String.class, getSpreadNotationLeg2(), this);
		processRosetta(path.newSubPath("spreadLeg2"), processor, SpreadLeg2.class, getSpreadLeg2());
		processor.processBasic(path.newSubPath("spreadCurrencyLeg1"), ISOCurrencyCodeEnum.class, getSpreadCurrencyLeg1(), this);
		processor.processBasic(path.newSubPath("spreadCurrencyLeg2"), ISOCurrencyCodeEnum.class, getSpreadCurrencyLeg2(), this);
		processor.processBasic(path.newSubPath("strikePriceNotation"), String.class, getStrikePriceNotation(), this);
		processRosetta(path.newSubPath("strikePrice"), processor, StrikePriceAndNotation.class, getStrikePrice());
		processor.processBasic(path.newSubPath("strikePriceCurrency"), String.class, getStrikePriceCurrency(), this);
		processor.processBasic(path.newSubPath("optionPremiumAmount"), BigDecimal.class, getOptionPremiumAmount(), this);
		processor.processBasic(path.newSubPath("optionPremiumCurrency"), ISOCurrencyCodeEnum.class, getOptionPremiumCurrency(), this);
		processor.processBasic(path.newSubPath("optionPremiumPaymentDate"), Date.class, getOptionPremiumPaymentDate(), this);
		processor.processBasic(path.newSubPath("exchangeRate"), BigDecimal.class, getExchangeRate(), this);
		processor.processBasic(path.newSubPath("exchangeRateBasis"), String.class, getExchangeRateBasis(), this);
		processor.processBasic(path.newSubPath("dayCountConventionLeg1"), InterestComputationMethod4Code.class, getDayCountConventionLeg1(), this);
		processor.processBasic(path.newSubPath("dayCountConventionLeg2"), InterestComputationMethod4Code.class, getDayCountConventionLeg2(), this);
		processor.processBasic(path.newSubPath("paymentFrequencyPeriodLeg1"), String.class, getPaymentFrequencyPeriodLeg1(), this);
		processor.processBasic(path.newSubPath("paymentFrequencyPeriodLeg2"), String.class, getPaymentFrequencyPeriodLeg2(), this);
		processor.processBasic(path.newSubPath("paymentFrequencyPeriodMultiplierLeg1"), Integer.class, getPaymentFrequencyPeriodMultiplierLeg1(), this);
		processor.processBasic(path.newSubPath("paymentFrequencyPeriodMultiplierLeg2"), Integer.class, getPaymentFrequencyPeriodMultiplierLeg2(), this);
		processor.processBasic(path.newSubPath("settlementCurrencyLeg1"), ISOCurrencyCodeEnum.class, getSettlementCurrencyLeg1(), this);
		processor.processBasic(path.newSubPath("settlementCurrencyLeg2"), String.class, getSettlementCurrencyLeg2(), this);
		processRosetta(path.newSubPath("otherPayment"), processor, OtherPaymentReport.class, getOtherPayment());
		processor.processBasic(path.newSubPath("customBasketCode"), String.class, getCustomBasketCode(), this);
		processRosetta(path.newSubPath("basketConstituents"), processor, BasketConstituentsReport.class, getBasketConstituents());
		processor.processBasic(path.newSubPath("basketConstituentIdentifierSource"), ProductIdTypeEnum.class, getBasketConstituentIdentifierSource(), this);
		processor.processBasic(path.newSubPath("underlyingIdentification"), String.class, getUnderlyingIdentification(), this);
		processor.processBasic(path.newSubPath("underlierIDSource"), String.class, getUnderlierIDSource(), this);
		processor.processBasic(path.newSubPath("maturityDateOfTheUnderlying"), Date.class, getMaturityDateOfTheUnderlying(), this);
		processor.processBasic(path.newSubPath("identifierOfTheFloatingRateLeg2"), String.class, getIdentifierOfTheFloatingRateLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRateReferencePeriodOfLeg2TimePeriod"), String.class, getFloatingRateReferencePeriodOfLeg2TimePeriod(), this);
		processor.processBasic(path.newSubPath("floatingRateReferencePeriodOfLeg2Multiplier"), Integer.class, getFloatingRateReferencePeriodOfLeg2Multiplier(), this);
		processor.processBasic(path.newSubPath("cdSIndexAttachmentPoint"), BigDecimal.class, getCdSIndexAttachmentPoint(), this);
		processor.processBasic(path.newSubPath("cdSIndexDetachmentPoint"), BigDecimal.class, getCdSIndexDetachmentPoint(), this);
		processor.processBasic(path.newSubPath("indexFactor"), BigDecimal.class, getIndexFactor(), this);
		processor.processBasic(path.newSubPath("packageIdentifier"), String.class, getPackageIdentifier(), this);
		processRosetta(path.newSubPath("packageTransactionPrice"), processor, PackageTransactionPrice.class, getPackageTransactionPrice());
		processRosetta(path.newSubPath("packageTransactionSpread"), processor, PackageTransactionSpread.class, getPackageTransactionSpread());
		processor.processBasic(path.newSubPath("priorUTI"), String.class, getPriorUTI(), this);
		processor.processBasic(path.newSubPath("eventIdentifier"), String.class, getEventIdentifier(), this);
		processor.processBasic(path.newSubPath("actionType"), String.class, getActionType(), this);
		processor.processBasic(path.newSubPath("eventType"), String.class, getEventType(), this);
		processor.processBasic(path.newSubPath("reportingTimestamp"), ZonedDateTime.class, getReportingTimestamp(), this);
		processor.processBasic(path.newSubPath("reportSubmittingEntityID"), String.class, getReportSubmittingEntityID(), this);
		processor.processBasic(path.newSubPath("valuationTimestamp"), ZonedDateTime.class, getValuationTimestamp(), this);
		processor.processBasic(path.newSubPath("valuationAmount"), BigDecimal.class, getValuationAmount(), this);
		processor.processBasic(path.newSubPath("valuationCurrency"), ISOCurrencyCodeEnum.class, getValuationCurrency(), this);
		processor.processBasic(path.newSubPath("valuationMethod"), ValuationType1Code.class, getValuationMethod(), this);
		processor.processBasic(path.newSubPath("delta"), BigDecimal.class, getDelta(), this);
		processor.processBasic(path.newSubPath("nextFloatingReferenceResetDateLeg1"), Date.class, getNextFloatingReferenceResetDateLeg1(), this);
		processor.processBasic(path.newSubPath("nextFloatingReferenceResetDateLeg2"), Date.class, getNextFloatingReferenceResetDateLeg2(), this);
		processor.processBasic(path.newSubPath("smallScaleBuySideEntityIndicator"), String.class, getSmallScaleBuySideEntityIndicator(), this);
		processRosetta(path.newSubPath("nonReportable"), processor, ASICNonReportable.class, getNonReportable());
	}
	

	/*********************** Builder Interface  ***********************/
	interface ASICTransactionReportBuilder extends ASICTransactionReport, RosettaModelObjectBuilder {
		NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder getOrCreateNotionalAmountScheduleLeg1(int _index);
		List<? extends NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> getNotionalAmountScheduleLeg1();
		NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder getOrCreateNotionalAmountScheduleLeg2(int _index);
		List<? extends NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> getNotionalAmountScheduleLeg2();
		SpreadLeg1.SpreadLeg1Builder getOrCreateSpreadLeg1();
		SpreadLeg1.SpreadLeg1Builder getSpreadLeg1();
		SpreadLeg2.SpreadLeg2Builder getOrCreateSpreadLeg2();
		SpreadLeg2.SpreadLeg2Builder getSpreadLeg2();
		StrikePriceAndNotation.StrikePriceAndNotationBuilder getOrCreateStrikePrice();
		StrikePriceAndNotation.StrikePriceAndNotationBuilder getStrikePrice();
		OtherPaymentReport.OtherPaymentReportBuilder getOrCreateOtherPayment(int _index);
		List<? extends OtherPaymentReport.OtherPaymentReportBuilder> getOtherPayment();
		BasketConstituentsReport.BasketConstituentsReportBuilder getOrCreateBasketConstituents(int _index);
		List<? extends BasketConstituentsReport.BasketConstituentsReportBuilder> getBasketConstituents();
		PackageTransactionPrice.PackageTransactionPriceBuilder getOrCreatePackageTransactionPrice();
		PackageTransactionPrice.PackageTransactionPriceBuilder getPackageTransactionPrice();
		PackageTransactionSpread.PackageTransactionSpreadBuilder getOrCreatePackageTransactionSpread();
		PackageTransactionSpread.PackageTransactionSpreadBuilder getPackageTransactionSpread();
		ASICNonReportable.ASICNonReportableBuilder getOrCreateNonReportable();
		ASICNonReportable.ASICNonReportableBuilder getNonReportable();
		ASICTransactionReport.ASICTransactionReportBuilder setUniqueTransactionIdentifier(String uniqueTransactionIdentifier);
		ASICTransactionReport.ASICTransactionReportBuilder setUniqueProductIdentifier(String uniqueProductIdentifier);
		ASICTransactionReport.ASICTransactionReportBuilder setAssetClass(String assetClass);
		ASICTransactionReport.ASICTransactionReportBuilder setContractType(String contractType);
		ASICTransactionReport.ASICTransactionReportBuilder setReportingEntity(String reportingEntity);
		ASICTransactionReport.ASICTransactionReportBuilder setCounterparty1(String counterparty1);
		ASICTransactionReport.ASICTransactionReportBuilder setCounterparty2(String counterparty2);
		ASICTransactionReport.ASICTransactionReportBuilder setCounterpartyIdentifierTypeIndicator(Boolean counterpartyIdentifierTypeIndicator);
		ASICTransactionReport.ASICTransactionReportBuilder setCountryOfTheCounterParty2(ISOCountryCodeEnum countryOfTheCounterParty2);
		ASICTransactionReport.ASICTransactionReportBuilder setBeneficiary1(String beneficiary1);
		ASICTransactionReport.ASICTransactionReportBuilder setBeneficiary1IdentifierTypeIndicator(Boolean beneficiary1IdentifierTypeIndicator);
		ASICTransactionReport.ASICTransactionReportBuilder setBroker(String broker);
		ASICTransactionReport.ASICTransactionReportBuilder setExecutionAgentOfTheCounterparty1(String executionAgentOfTheCounterparty1);
		ASICTransactionReport.ASICTransactionReportBuilder setDirection(String direction);
		ASICTransactionReport.ASICTransactionReportBuilder setDirection2OfLeg1(String direction2OfLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setDirection2OfLeg2(String direction2OfLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setEffectiveDate(Date effectiveDate);
		ASICTransactionReport.ASICTransactionReportBuilder setExpirationDate(Date expirationDate);
		ASICTransactionReport.ASICTransactionReportBuilder setExecutionTimestamp(ZonedDateTime executionTimestamp);
		ASICTransactionReport.ASICTransactionReportBuilder setEventTimestamp(ZonedDateTime eventTimestamp);
		ASICTransactionReport.ASICTransactionReportBuilder setClearingTimestamp(ZonedDateTime clearingTimestamp);
		ASICTransactionReport.ASICTransactionReportBuilder setCleared(String cleared);
		ASICTransactionReport.ASICTransactionReportBuilder setCentralCounterparty(String centralCounterparty);
		ASICTransactionReport.ASICTransactionReportBuilder setClearingMember(String clearingMember);
		ASICTransactionReport.ASICTransactionReportBuilder setPlatformIdentifier(String platformIdentifier);
		ASICTransactionReport.ASICTransactionReportBuilder setNotionalLeg1(BigDecimal notionalLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setNotionalLeg2(BigDecimal notionalLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setTotalNotionalQuantityLeg1(BigDecimal totalNotionalQuantityLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setTotalNotionalQuantityLeg2(BigDecimal totalNotionalQuantityLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setNotionalQuantityLeg1(BigDecimal notionalQuantityLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setNotionalQuantityLeg2(BigDecimal notionalQuantityLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setCallAmount(BigDecimal callAmount);
		ASICTransactionReport.ASICTransactionReportBuilder setPutAmount(BigDecimal putAmount);
		ASICTransactionReport.ASICTransactionReportBuilder setNotionalCurrencyLeg1(ISOCurrencyCodeEnum notionalCurrencyLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setNotionalCurrencyLeg2(ISOCurrencyCodeEnum notionalCurrencyLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setQuantityUnitOfMeasureLeg1(String quantityUnitOfMeasureLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setQuantityUnitOfMeasureLeg2(String quantityUnitOfMeasureLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setCallCurrency(ISOCurrencyCodeEnum callCurrency);
		ASICTransactionReport.ASICTransactionReportBuilder setPutCurrency(ISOCurrencyCodeEnum putCurrency);
		ASICTransactionReport.ASICTransactionReportBuilder addNotionalAmountScheduleLeg1(NotionalAmountScheduleLeg1Report notionalAmountScheduleLeg10);
		ASICTransactionReport.ASICTransactionReportBuilder addNotionalAmountScheduleLeg1(NotionalAmountScheduleLeg1Report notionalAmountScheduleLeg11, int _idx);
		ASICTransactionReport.ASICTransactionReportBuilder addNotionalAmountScheduleLeg1(List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg12);
		ASICTransactionReport.ASICTransactionReportBuilder setNotionalAmountScheduleLeg1(List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg13);
		ASICTransactionReport.ASICTransactionReportBuilder addNotionalAmountScheduleLeg2(NotionalAmountScheduleLeg2Report notionalAmountScheduleLeg20);
		ASICTransactionReport.ASICTransactionReportBuilder addNotionalAmountScheduleLeg2(NotionalAmountScheduleLeg2Report notionalAmountScheduleLeg21, int _idx);
		ASICTransactionReport.ASICTransactionReportBuilder addNotionalAmountScheduleLeg2(List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg22);
		ASICTransactionReport.ASICTransactionReportBuilder setNotionalAmountScheduleLeg2(List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg23);
		ASICTransactionReport.ASICTransactionReportBuilder setPriceNotation(String priceNotation);
		ASICTransactionReport.ASICTransactionReportBuilder setPrice(BigDecimal price);
		ASICTransactionReport.ASICTransactionReportBuilder setPriceCurrency(ISOCurrencyCodeEnum priceCurrency);
		ASICTransactionReport.ASICTransactionReportBuilder setPriceUnitOfMeasure(String priceUnitOfMeasure);
		ASICTransactionReport.ASICTransactionReportBuilder setFixedRateLeg1(BigDecimal fixedRateLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setFixedRateLeg2(BigDecimal fixedRateLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setSpreadNotationLeg1(String spreadNotationLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setSpreadLeg1(SpreadLeg1 spreadLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setSpreadNotationLeg2(String spreadNotationLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setSpreadLeg2(SpreadLeg2 spreadLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setSpreadCurrencyLeg1(ISOCurrencyCodeEnum spreadCurrencyLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setSpreadCurrencyLeg2(ISOCurrencyCodeEnum spreadCurrencyLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setStrikePriceNotation(String strikePriceNotation);
		ASICTransactionReport.ASICTransactionReportBuilder setStrikePrice(StrikePriceAndNotation strikePrice);
		ASICTransactionReport.ASICTransactionReportBuilder setStrikePriceCurrency(String strikePriceCurrency);
		ASICTransactionReport.ASICTransactionReportBuilder setOptionPremiumAmount(BigDecimal optionPremiumAmount);
		ASICTransactionReport.ASICTransactionReportBuilder setOptionPremiumCurrency(ISOCurrencyCodeEnum optionPremiumCurrency);
		ASICTransactionReport.ASICTransactionReportBuilder setOptionPremiumPaymentDate(Date optionPremiumPaymentDate);
		ASICTransactionReport.ASICTransactionReportBuilder setExchangeRate(BigDecimal exchangeRate);
		ASICTransactionReport.ASICTransactionReportBuilder setExchangeRateBasis(String exchangeRateBasis);
		ASICTransactionReport.ASICTransactionReportBuilder setDayCountConventionLeg1(InterestComputationMethod4Code dayCountConventionLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setDayCountConventionLeg2(InterestComputationMethod4Code dayCountConventionLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setPaymentFrequencyPeriodLeg1(String paymentFrequencyPeriodLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setPaymentFrequencyPeriodLeg2(String paymentFrequencyPeriodLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setPaymentFrequencyPeriodMultiplierLeg1(Integer paymentFrequencyPeriodMultiplierLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setPaymentFrequencyPeriodMultiplierLeg2(Integer paymentFrequencyPeriodMultiplierLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setSettlementCurrencyLeg1(ISOCurrencyCodeEnum settlementCurrencyLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setSettlementCurrencyLeg2(String settlementCurrencyLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment0);
		ASICTransactionReport.ASICTransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment1, int _idx);
		ASICTransactionReport.ASICTransactionReportBuilder addOtherPayment(List<? extends OtherPaymentReport> otherPayment2);
		ASICTransactionReport.ASICTransactionReportBuilder setOtherPayment(List<? extends OtherPaymentReport> otherPayment3);
		ASICTransactionReport.ASICTransactionReportBuilder setCustomBasketCode(String customBasketCode);
		ASICTransactionReport.ASICTransactionReportBuilder addBasketConstituents(BasketConstituentsReport basketConstituents0);
		ASICTransactionReport.ASICTransactionReportBuilder addBasketConstituents(BasketConstituentsReport basketConstituents1, int _idx);
		ASICTransactionReport.ASICTransactionReportBuilder addBasketConstituents(List<? extends BasketConstituentsReport> basketConstituents2);
		ASICTransactionReport.ASICTransactionReportBuilder setBasketConstituents(List<? extends BasketConstituentsReport> basketConstituents3);
		ASICTransactionReport.ASICTransactionReportBuilder addBasketConstituentIdentifierSource(ProductIdTypeEnum basketConstituentIdentifierSource0);
		ASICTransactionReport.ASICTransactionReportBuilder addBasketConstituentIdentifierSource(ProductIdTypeEnum basketConstituentIdentifierSource1, int _idx);
		ASICTransactionReport.ASICTransactionReportBuilder addBasketConstituentIdentifierSource(List<? extends ProductIdTypeEnum> basketConstituentIdentifierSource2);
		ASICTransactionReport.ASICTransactionReportBuilder setBasketConstituentIdentifierSource(List<? extends ProductIdTypeEnum> basketConstituentIdentifierSource3);
		ASICTransactionReport.ASICTransactionReportBuilder setUnderlyingIdentification(String underlyingIdentification);
		ASICTransactionReport.ASICTransactionReportBuilder setUnderlierIDSource(String underlierIDSource);
		ASICTransactionReport.ASICTransactionReportBuilder setMaturityDateOfTheUnderlying(Date maturityDateOfTheUnderlying);
		ASICTransactionReport.ASICTransactionReportBuilder setIdentifierOfTheFloatingRateLeg2(String identifierOfTheFloatingRateLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setFloatingRateReferencePeriodOfLeg2TimePeriod(String floatingRateReferencePeriodOfLeg2TimePeriod);
		ASICTransactionReport.ASICTransactionReportBuilder setFloatingRateReferencePeriodOfLeg2Multiplier(Integer floatingRateReferencePeriodOfLeg2Multiplier);
		ASICTransactionReport.ASICTransactionReportBuilder setCdSIndexAttachmentPoint(BigDecimal cdSIndexAttachmentPoint);
		ASICTransactionReport.ASICTransactionReportBuilder setCdSIndexDetachmentPoint(BigDecimal cdSIndexDetachmentPoint);
		ASICTransactionReport.ASICTransactionReportBuilder setIndexFactor(BigDecimal indexFactor);
		ASICTransactionReport.ASICTransactionReportBuilder setPackageIdentifier(String packageIdentifier);
		ASICTransactionReport.ASICTransactionReportBuilder setPackageTransactionPrice(PackageTransactionPrice packageTransactionPrice);
		ASICTransactionReport.ASICTransactionReportBuilder setPackageTransactionSpread(PackageTransactionSpread packageTransactionSpread);
		ASICTransactionReport.ASICTransactionReportBuilder setPriorUTI(String priorUTI);
		ASICTransactionReport.ASICTransactionReportBuilder setEventIdentifier(String eventIdentifier);
		ASICTransactionReport.ASICTransactionReportBuilder setActionType(String actionType);
		ASICTransactionReport.ASICTransactionReportBuilder setEventType(String eventType);
		ASICTransactionReport.ASICTransactionReportBuilder setReportingTimestamp(ZonedDateTime reportingTimestamp);
		ASICTransactionReport.ASICTransactionReportBuilder setReportSubmittingEntityID(String reportSubmittingEntityID);
		ASICTransactionReport.ASICTransactionReportBuilder setValuationTimestamp(ZonedDateTime valuationTimestamp);
		ASICTransactionReport.ASICTransactionReportBuilder setValuationAmount(BigDecimal valuationAmount);
		ASICTransactionReport.ASICTransactionReportBuilder setValuationCurrency(ISOCurrencyCodeEnum valuationCurrency);
		ASICTransactionReport.ASICTransactionReportBuilder setValuationMethod(ValuationType1Code valuationMethod);
		ASICTransactionReport.ASICTransactionReportBuilder setDelta(BigDecimal delta);
		ASICTransactionReport.ASICTransactionReportBuilder setNextFloatingReferenceResetDateLeg1(Date nextFloatingReferenceResetDateLeg1);
		ASICTransactionReport.ASICTransactionReportBuilder setNextFloatingReferenceResetDateLeg2(Date nextFloatingReferenceResetDateLeg2);
		ASICTransactionReport.ASICTransactionReportBuilder setSmallScaleBuySideEntityIndicator(String smallScaleBuySideEntityIndicator);
		ASICTransactionReport.ASICTransactionReportBuilder setNonReportable(ASICNonReportable nonReportable);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("uniqueTransactionIdentifier"), String.class, getUniqueTransactionIdentifier(), this);
			processor.processBasic(path.newSubPath("uniqueProductIdentifier"), String.class, getUniqueProductIdentifier(), this);
			processor.processBasic(path.newSubPath("assetClass"), String.class, getAssetClass(), this);
			processor.processBasic(path.newSubPath("contractType"), String.class, getContractType(), this);
			processor.processBasic(path.newSubPath("reportingEntity"), String.class, getReportingEntity(), this);
			processor.processBasic(path.newSubPath("counterparty1"), String.class, getCounterparty1(), this);
			processor.processBasic(path.newSubPath("counterparty2"), String.class, getCounterparty2(), this);
			processor.processBasic(path.newSubPath("counterpartyIdentifierTypeIndicator"), Boolean.class, getCounterpartyIdentifierTypeIndicator(), this);
			processor.processBasic(path.newSubPath("countryOfTheCounterParty2"), ISOCountryCodeEnum.class, getCountryOfTheCounterParty2(), this);
			processor.processBasic(path.newSubPath("beneficiary1"), String.class, getBeneficiary1(), this);
			processor.processBasic(path.newSubPath("beneficiary1IdentifierTypeIndicator"), Boolean.class, getBeneficiary1IdentifierTypeIndicator(), this);
			processor.processBasic(path.newSubPath("broker"), String.class, getBroker(), this);
			processor.processBasic(path.newSubPath("executionAgentOfTheCounterparty1"), String.class, getExecutionAgentOfTheCounterparty1(), this);
			processor.processBasic(path.newSubPath("direction"), String.class, getDirection(), this);
			processor.processBasic(path.newSubPath("direction2OfLeg1"), String.class, getDirection2OfLeg1(), this);
			processor.processBasic(path.newSubPath("direction2OfLeg2"), String.class, getDirection2OfLeg2(), this);
			processor.processBasic(path.newSubPath("effectiveDate"), Date.class, getEffectiveDate(), this);
			processor.processBasic(path.newSubPath("expirationDate"), Date.class, getExpirationDate(), this);
			processor.processBasic(path.newSubPath("executionTimestamp"), ZonedDateTime.class, getExecutionTimestamp(), this);
			processor.processBasic(path.newSubPath("eventTimestamp"), ZonedDateTime.class, getEventTimestamp(), this);
			processor.processBasic(path.newSubPath("clearingTimestamp"), ZonedDateTime.class, getClearingTimestamp(), this);
			processor.processBasic(path.newSubPath("cleared"), String.class, getCleared(), this);
			processor.processBasic(path.newSubPath("centralCounterparty"), String.class, getCentralCounterparty(), this);
			processor.processBasic(path.newSubPath("clearingMember"), String.class, getClearingMember(), this);
			processor.processBasic(path.newSubPath("platformIdentifier"), String.class, getPlatformIdentifier(), this);
			processor.processBasic(path.newSubPath("notionalLeg1"), BigDecimal.class, getNotionalLeg1(), this);
			processor.processBasic(path.newSubPath("notionalLeg2"), BigDecimal.class, getNotionalLeg2(), this);
			processor.processBasic(path.newSubPath("totalNotionalQuantityLeg1"), BigDecimal.class, getTotalNotionalQuantityLeg1(), this);
			processor.processBasic(path.newSubPath("totalNotionalQuantityLeg2"), BigDecimal.class, getTotalNotionalQuantityLeg2(), this);
			processor.processBasic(path.newSubPath("notionalQuantityLeg1"), BigDecimal.class, getNotionalQuantityLeg1(), this);
			processor.processBasic(path.newSubPath("notionalQuantityLeg2"), BigDecimal.class, getNotionalQuantityLeg2(), this);
			processor.processBasic(path.newSubPath("callAmount"), BigDecimal.class, getCallAmount(), this);
			processor.processBasic(path.newSubPath("putAmount"), BigDecimal.class, getPutAmount(), this);
			processor.processBasic(path.newSubPath("notionalCurrencyLeg1"), ISOCurrencyCodeEnum.class, getNotionalCurrencyLeg1(), this);
			processor.processBasic(path.newSubPath("notionalCurrencyLeg2"), ISOCurrencyCodeEnum.class, getNotionalCurrencyLeg2(), this);
			processor.processBasic(path.newSubPath("quantityUnitOfMeasureLeg1"), String.class, getQuantityUnitOfMeasureLeg1(), this);
			processor.processBasic(path.newSubPath("quantityUnitOfMeasureLeg2"), String.class, getQuantityUnitOfMeasureLeg2(), this);
			processor.processBasic(path.newSubPath("callCurrency"), ISOCurrencyCodeEnum.class, getCallCurrency(), this);
			processor.processBasic(path.newSubPath("putCurrency"), ISOCurrencyCodeEnum.class, getPutCurrency(), this);
			processRosetta(path.newSubPath("notionalAmountScheduleLeg1"), processor, NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder.class, getNotionalAmountScheduleLeg1());
			processRosetta(path.newSubPath("notionalAmountScheduleLeg2"), processor, NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder.class, getNotionalAmountScheduleLeg2());
			processor.processBasic(path.newSubPath("priceNotation"), String.class, getPriceNotation(), this);
			processor.processBasic(path.newSubPath("price"), BigDecimal.class, getPrice(), this);
			processor.processBasic(path.newSubPath("priceCurrency"), ISOCurrencyCodeEnum.class, getPriceCurrency(), this);
			processor.processBasic(path.newSubPath("priceUnitOfMeasure"), String.class, getPriceUnitOfMeasure(), this);
			processor.processBasic(path.newSubPath("fixedRateLeg1"), BigDecimal.class, getFixedRateLeg1(), this);
			processor.processBasic(path.newSubPath("fixedRateLeg2"), BigDecimal.class, getFixedRateLeg2(), this);
			processor.processBasic(path.newSubPath("spreadNotationLeg1"), String.class, getSpreadNotationLeg1(), this);
			processRosetta(path.newSubPath("spreadLeg1"), processor, SpreadLeg1.SpreadLeg1Builder.class, getSpreadLeg1());
			processor.processBasic(path.newSubPath("spreadNotationLeg2"), String.class, getSpreadNotationLeg2(), this);
			processRosetta(path.newSubPath("spreadLeg2"), processor, SpreadLeg2.SpreadLeg2Builder.class, getSpreadLeg2());
			processor.processBasic(path.newSubPath("spreadCurrencyLeg1"), ISOCurrencyCodeEnum.class, getSpreadCurrencyLeg1(), this);
			processor.processBasic(path.newSubPath("spreadCurrencyLeg2"), ISOCurrencyCodeEnum.class, getSpreadCurrencyLeg2(), this);
			processor.processBasic(path.newSubPath("strikePriceNotation"), String.class, getStrikePriceNotation(), this);
			processRosetta(path.newSubPath("strikePrice"), processor, StrikePriceAndNotation.StrikePriceAndNotationBuilder.class, getStrikePrice());
			processor.processBasic(path.newSubPath("strikePriceCurrency"), String.class, getStrikePriceCurrency(), this);
			processor.processBasic(path.newSubPath("optionPremiumAmount"), BigDecimal.class, getOptionPremiumAmount(), this);
			processor.processBasic(path.newSubPath("optionPremiumCurrency"), ISOCurrencyCodeEnum.class, getOptionPremiumCurrency(), this);
			processor.processBasic(path.newSubPath("optionPremiumPaymentDate"), Date.class, getOptionPremiumPaymentDate(), this);
			processor.processBasic(path.newSubPath("exchangeRate"), BigDecimal.class, getExchangeRate(), this);
			processor.processBasic(path.newSubPath("exchangeRateBasis"), String.class, getExchangeRateBasis(), this);
			processor.processBasic(path.newSubPath("dayCountConventionLeg1"), InterestComputationMethod4Code.class, getDayCountConventionLeg1(), this);
			processor.processBasic(path.newSubPath("dayCountConventionLeg2"), InterestComputationMethod4Code.class, getDayCountConventionLeg2(), this);
			processor.processBasic(path.newSubPath("paymentFrequencyPeriodLeg1"), String.class, getPaymentFrequencyPeriodLeg1(), this);
			processor.processBasic(path.newSubPath("paymentFrequencyPeriodLeg2"), String.class, getPaymentFrequencyPeriodLeg2(), this);
			processor.processBasic(path.newSubPath("paymentFrequencyPeriodMultiplierLeg1"), Integer.class, getPaymentFrequencyPeriodMultiplierLeg1(), this);
			processor.processBasic(path.newSubPath("paymentFrequencyPeriodMultiplierLeg2"), Integer.class, getPaymentFrequencyPeriodMultiplierLeg2(), this);
			processor.processBasic(path.newSubPath("settlementCurrencyLeg1"), ISOCurrencyCodeEnum.class, getSettlementCurrencyLeg1(), this);
			processor.processBasic(path.newSubPath("settlementCurrencyLeg2"), String.class, getSettlementCurrencyLeg2(), this);
			processRosetta(path.newSubPath("otherPayment"), processor, OtherPaymentReport.OtherPaymentReportBuilder.class, getOtherPayment());
			processor.processBasic(path.newSubPath("customBasketCode"), String.class, getCustomBasketCode(), this);
			processRosetta(path.newSubPath("basketConstituents"), processor, BasketConstituentsReport.BasketConstituentsReportBuilder.class, getBasketConstituents());
			processor.processBasic(path.newSubPath("basketConstituentIdentifierSource"), ProductIdTypeEnum.class, getBasketConstituentIdentifierSource(), this);
			processor.processBasic(path.newSubPath("underlyingIdentification"), String.class, getUnderlyingIdentification(), this);
			processor.processBasic(path.newSubPath("underlierIDSource"), String.class, getUnderlierIDSource(), this);
			processor.processBasic(path.newSubPath("maturityDateOfTheUnderlying"), Date.class, getMaturityDateOfTheUnderlying(), this);
			processor.processBasic(path.newSubPath("identifierOfTheFloatingRateLeg2"), String.class, getIdentifierOfTheFloatingRateLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRateReferencePeriodOfLeg2TimePeriod"), String.class, getFloatingRateReferencePeriodOfLeg2TimePeriod(), this);
			processor.processBasic(path.newSubPath("floatingRateReferencePeriodOfLeg2Multiplier"), Integer.class, getFloatingRateReferencePeriodOfLeg2Multiplier(), this);
			processor.processBasic(path.newSubPath("cdSIndexAttachmentPoint"), BigDecimal.class, getCdSIndexAttachmentPoint(), this);
			processor.processBasic(path.newSubPath("cdSIndexDetachmentPoint"), BigDecimal.class, getCdSIndexDetachmentPoint(), this);
			processor.processBasic(path.newSubPath("indexFactor"), BigDecimal.class, getIndexFactor(), this);
			processor.processBasic(path.newSubPath("packageIdentifier"), String.class, getPackageIdentifier(), this);
			processRosetta(path.newSubPath("packageTransactionPrice"), processor, PackageTransactionPrice.PackageTransactionPriceBuilder.class, getPackageTransactionPrice());
			processRosetta(path.newSubPath("packageTransactionSpread"), processor, PackageTransactionSpread.PackageTransactionSpreadBuilder.class, getPackageTransactionSpread());
			processor.processBasic(path.newSubPath("priorUTI"), String.class, getPriorUTI(), this);
			processor.processBasic(path.newSubPath("eventIdentifier"), String.class, getEventIdentifier(), this);
			processor.processBasic(path.newSubPath("actionType"), String.class, getActionType(), this);
			processor.processBasic(path.newSubPath("eventType"), String.class, getEventType(), this);
			processor.processBasic(path.newSubPath("reportingTimestamp"), ZonedDateTime.class, getReportingTimestamp(), this);
			processor.processBasic(path.newSubPath("reportSubmittingEntityID"), String.class, getReportSubmittingEntityID(), this);
			processor.processBasic(path.newSubPath("valuationTimestamp"), ZonedDateTime.class, getValuationTimestamp(), this);
			processor.processBasic(path.newSubPath("valuationAmount"), BigDecimal.class, getValuationAmount(), this);
			processor.processBasic(path.newSubPath("valuationCurrency"), ISOCurrencyCodeEnum.class, getValuationCurrency(), this);
			processor.processBasic(path.newSubPath("valuationMethod"), ValuationType1Code.class, getValuationMethod(), this);
			processor.processBasic(path.newSubPath("delta"), BigDecimal.class, getDelta(), this);
			processor.processBasic(path.newSubPath("nextFloatingReferenceResetDateLeg1"), Date.class, getNextFloatingReferenceResetDateLeg1(), this);
			processor.processBasic(path.newSubPath("nextFloatingReferenceResetDateLeg2"), Date.class, getNextFloatingReferenceResetDateLeg2(), this);
			processor.processBasic(path.newSubPath("smallScaleBuySideEntityIndicator"), String.class, getSmallScaleBuySideEntityIndicator(), this);
			processRosetta(path.newSubPath("nonReportable"), processor, ASICNonReportable.ASICNonReportableBuilder.class, getNonReportable());
		}
		

		ASICTransactionReport.ASICTransactionReportBuilder prune();
	}

	/*********************** Immutable Implementation of ASICTransactionReport  ***********************/
	class ASICTransactionReportImpl implements ASICTransactionReport {
		private final String uniqueTransactionIdentifier;
		private final String uniqueProductIdentifier;
		private final String assetClass;
		private final String contractType;
		private final String reportingEntity;
		private final String counterparty1;
		private final String counterparty2;
		private final Boolean counterpartyIdentifierTypeIndicator;
		private final ISOCountryCodeEnum countryOfTheCounterParty2;
		private final String beneficiary1;
		private final Boolean beneficiary1IdentifierTypeIndicator;
		private final String broker;
		private final String executionAgentOfTheCounterparty1;
		private final String direction;
		private final String direction2OfLeg1;
		private final String direction2OfLeg2;
		private final Date effectiveDate;
		private final Date expirationDate;
		private final ZonedDateTime executionTimestamp;
		private final ZonedDateTime eventTimestamp;
		private final ZonedDateTime clearingTimestamp;
		private final String cleared;
		private final String centralCounterparty;
		private final String clearingMember;
		private final String platformIdentifier;
		private final BigDecimal notionalLeg1;
		private final BigDecimal notionalLeg2;
		private final BigDecimal totalNotionalQuantityLeg1;
		private final BigDecimal totalNotionalQuantityLeg2;
		private final BigDecimal notionalQuantityLeg1;
		private final BigDecimal notionalQuantityLeg2;
		private final BigDecimal callAmount;
		private final BigDecimal putAmount;
		private final ISOCurrencyCodeEnum notionalCurrencyLeg1;
		private final ISOCurrencyCodeEnum notionalCurrencyLeg2;
		private final String quantityUnitOfMeasureLeg1;
		private final String quantityUnitOfMeasureLeg2;
		private final ISOCurrencyCodeEnum callCurrency;
		private final ISOCurrencyCodeEnum putCurrency;
		private final List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg1;
		private final List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg2;
		private final String priceNotation;
		private final BigDecimal price;
		private final ISOCurrencyCodeEnum priceCurrency;
		private final String priceUnitOfMeasure;
		private final BigDecimal fixedRateLeg1;
		private final BigDecimal fixedRateLeg2;
		private final String spreadNotationLeg1;
		private final SpreadLeg1 spreadLeg1;
		private final String spreadNotationLeg2;
		private final SpreadLeg2 spreadLeg2;
		private final ISOCurrencyCodeEnum spreadCurrencyLeg1;
		private final ISOCurrencyCodeEnum spreadCurrencyLeg2;
		private final String strikePriceNotation;
		private final StrikePriceAndNotation strikePrice;
		private final String strikePriceCurrency;
		private final BigDecimal optionPremiumAmount;
		private final ISOCurrencyCodeEnum optionPremiumCurrency;
		private final Date optionPremiumPaymentDate;
		private final BigDecimal exchangeRate;
		private final String exchangeRateBasis;
		private final InterestComputationMethod4Code dayCountConventionLeg1;
		private final InterestComputationMethod4Code dayCountConventionLeg2;
		private final String paymentFrequencyPeriodLeg1;
		private final String paymentFrequencyPeriodLeg2;
		private final Integer paymentFrequencyPeriodMultiplierLeg1;
		private final Integer paymentFrequencyPeriodMultiplierLeg2;
		private final ISOCurrencyCodeEnum settlementCurrencyLeg1;
		private final String settlementCurrencyLeg2;
		private final List<? extends OtherPaymentReport> otherPayment;
		private final String customBasketCode;
		private final List<? extends BasketConstituentsReport> basketConstituents;
		private final List<ProductIdTypeEnum> basketConstituentIdentifierSource;
		private final String underlyingIdentification;
		private final String underlierIDSource;
		private final Date maturityDateOfTheUnderlying;
		private final String identifierOfTheFloatingRateLeg2;
		private final String floatingRateReferencePeriodOfLeg2TimePeriod;
		private final Integer floatingRateReferencePeriodOfLeg2Multiplier;
		private final BigDecimal cdSIndexAttachmentPoint;
		private final BigDecimal cdSIndexDetachmentPoint;
		private final BigDecimal indexFactor;
		private final String packageIdentifier;
		private final PackageTransactionPrice packageTransactionPrice;
		private final PackageTransactionSpread packageTransactionSpread;
		private final String priorUTI;
		private final String eventIdentifier;
		private final String actionType;
		private final String eventType;
		private final ZonedDateTime reportingTimestamp;
		private final String reportSubmittingEntityID;
		private final ZonedDateTime valuationTimestamp;
		private final BigDecimal valuationAmount;
		private final ISOCurrencyCodeEnum valuationCurrency;
		private final ValuationType1Code valuationMethod;
		private final BigDecimal delta;
		private final Date nextFloatingReferenceResetDateLeg1;
		private final Date nextFloatingReferenceResetDateLeg2;
		private final String smallScaleBuySideEntityIndicator;
		private final ASICNonReportable nonReportable;
		
		protected ASICTransactionReportImpl(ASICTransactionReport.ASICTransactionReportBuilder builder) {
			this.uniqueTransactionIdentifier = builder.getUniqueTransactionIdentifier();
			this.uniqueProductIdentifier = builder.getUniqueProductIdentifier();
			this.assetClass = builder.getAssetClass();
			this.contractType = builder.getContractType();
			this.reportingEntity = builder.getReportingEntity();
			this.counterparty1 = builder.getCounterparty1();
			this.counterparty2 = builder.getCounterparty2();
			this.counterpartyIdentifierTypeIndicator = builder.getCounterpartyIdentifierTypeIndicator();
			this.countryOfTheCounterParty2 = builder.getCountryOfTheCounterParty2();
			this.beneficiary1 = builder.getBeneficiary1();
			this.beneficiary1IdentifierTypeIndicator = builder.getBeneficiary1IdentifierTypeIndicator();
			this.broker = builder.getBroker();
			this.executionAgentOfTheCounterparty1 = builder.getExecutionAgentOfTheCounterparty1();
			this.direction = builder.getDirection();
			this.direction2OfLeg1 = builder.getDirection2OfLeg1();
			this.direction2OfLeg2 = builder.getDirection2OfLeg2();
			this.effectiveDate = builder.getEffectiveDate();
			this.expirationDate = builder.getExpirationDate();
			this.executionTimestamp = builder.getExecutionTimestamp();
			this.eventTimestamp = builder.getEventTimestamp();
			this.clearingTimestamp = builder.getClearingTimestamp();
			this.cleared = builder.getCleared();
			this.centralCounterparty = builder.getCentralCounterparty();
			this.clearingMember = builder.getClearingMember();
			this.platformIdentifier = builder.getPlatformIdentifier();
			this.notionalLeg1 = builder.getNotionalLeg1();
			this.notionalLeg2 = builder.getNotionalLeg2();
			this.totalNotionalQuantityLeg1 = builder.getTotalNotionalQuantityLeg1();
			this.totalNotionalQuantityLeg2 = builder.getTotalNotionalQuantityLeg2();
			this.notionalQuantityLeg1 = builder.getNotionalQuantityLeg1();
			this.notionalQuantityLeg2 = builder.getNotionalQuantityLeg2();
			this.callAmount = builder.getCallAmount();
			this.putAmount = builder.getPutAmount();
			this.notionalCurrencyLeg1 = builder.getNotionalCurrencyLeg1();
			this.notionalCurrencyLeg2 = builder.getNotionalCurrencyLeg2();
			this.quantityUnitOfMeasureLeg1 = builder.getQuantityUnitOfMeasureLeg1();
			this.quantityUnitOfMeasureLeg2 = builder.getQuantityUnitOfMeasureLeg2();
			this.callCurrency = builder.getCallCurrency();
			this.putCurrency = builder.getPutCurrency();
			this.notionalAmountScheduleLeg1 = ofNullable(builder.getNotionalAmountScheduleLeg1()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.notionalAmountScheduleLeg2 = ofNullable(builder.getNotionalAmountScheduleLeg2()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.priceNotation = builder.getPriceNotation();
			this.price = builder.getPrice();
			this.priceCurrency = builder.getPriceCurrency();
			this.priceUnitOfMeasure = builder.getPriceUnitOfMeasure();
			this.fixedRateLeg1 = builder.getFixedRateLeg1();
			this.fixedRateLeg2 = builder.getFixedRateLeg2();
			this.spreadNotationLeg1 = builder.getSpreadNotationLeg1();
			this.spreadLeg1 = ofNullable(builder.getSpreadLeg1()).map(f->f.build()).orElse(null);
			this.spreadNotationLeg2 = builder.getSpreadNotationLeg2();
			this.spreadLeg2 = ofNullable(builder.getSpreadLeg2()).map(f->f.build()).orElse(null);
			this.spreadCurrencyLeg1 = builder.getSpreadCurrencyLeg1();
			this.spreadCurrencyLeg2 = builder.getSpreadCurrencyLeg2();
			this.strikePriceNotation = builder.getStrikePriceNotation();
			this.strikePrice = ofNullable(builder.getStrikePrice()).map(f->f.build()).orElse(null);
			this.strikePriceCurrency = builder.getStrikePriceCurrency();
			this.optionPremiumAmount = builder.getOptionPremiumAmount();
			this.optionPremiumCurrency = builder.getOptionPremiumCurrency();
			this.optionPremiumPaymentDate = builder.getOptionPremiumPaymentDate();
			this.exchangeRate = builder.getExchangeRate();
			this.exchangeRateBasis = builder.getExchangeRateBasis();
			this.dayCountConventionLeg1 = builder.getDayCountConventionLeg1();
			this.dayCountConventionLeg2 = builder.getDayCountConventionLeg2();
			this.paymentFrequencyPeriodLeg1 = builder.getPaymentFrequencyPeriodLeg1();
			this.paymentFrequencyPeriodLeg2 = builder.getPaymentFrequencyPeriodLeg2();
			this.paymentFrequencyPeriodMultiplierLeg1 = builder.getPaymentFrequencyPeriodMultiplierLeg1();
			this.paymentFrequencyPeriodMultiplierLeg2 = builder.getPaymentFrequencyPeriodMultiplierLeg2();
			this.settlementCurrencyLeg1 = builder.getSettlementCurrencyLeg1();
			this.settlementCurrencyLeg2 = builder.getSettlementCurrencyLeg2();
			this.otherPayment = ofNullable(builder.getOtherPayment()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.customBasketCode = builder.getCustomBasketCode();
			this.basketConstituents = ofNullable(builder.getBasketConstituents()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.basketConstituentIdentifierSource = ofNullable(builder.getBasketConstituentIdentifierSource()).filter(_l->!_l.isEmpty()).map(ImmutableList::copyOf).orElse(null);
			this.underlyingIdentification = builder.getUnderlyingIdentification();
			this.underlierIDSource = builder.getUnderlierIDSource();
			this.maturityDateOfTheUnderlying = builder.getMaturityDateOfTheUnderlying();
			this.identifierOfTheFloatingRateLeg2 = builder.getIdentifierOfTheFloatingRateLeg2();
			this.floatingRateReferencePeriodOfLeg2TimePeriod = builder.getFloatingRateReferencePeriodOfLeg2TimePeriod();
			this.floatingRateReferencePeriodOfLeg2Multiplier = builder.getFloatingRateReferencePeriodOfLeg2Multiplier();
			this.cdSIndexAttachmentPoint = builder.getCdSIndexAttachmentPoint();
			this.cdSIndexDetachmentPoint = builder.getCdSIndexDetachmentPoint();
			this.indexFactor = builder.getIndexFactor();
			this.packageIdentifier = builder.getPackageIdentifier();
			this.packageTransactionPrice = ofNullable(builder.getPackageTransactionPrice()).map(f->f.build()).orElse(null);
			this.packageTransactionSpread = ofNullable(builder.getPackageTransactionSpread()).map(f->f.build()).orElse(null);
			this.priorUTI = builder.getPriorUTI();
			this.eventIdentifier = builder.getEventIdentifier();
			this.actionType = builder.getActionType();
			this.eventType = builder.getEventType();
			this.reportingTimestamp = builder.getReportingTimestamp();
			this.reportSubmittingEntityID = builder.getReportSubmittingEntityID();
			this.valuationTimestamp = builder.getValuationTimestamp();
			this.valuationAmount = builder.getValuationAmount();
			this.valuationCurrency = builder.getValuationCurrency();
			this.valuationMethod = builder.getValuationMethod();
			this.delta = builder.getDelta();
			this.nextFloatingReferenceResetDateLeg1 = builder.getNextFloatingReferenceResetDateLeg1();
			this.nextFloatingReferenceResetDateLeg2 = builder.getNextFloatingReferenceResetDateLeg2();
			this.smallScaleBuySideEntityIndicator = builder.getSmallScaleBuySideEntityIndicator();
			this.nonReportable = ofNullable(builder.getNonReportable()).map(f->f.build()).orElse(null);
		}
		
		@Override
		@RosettaAttribute("uniqueTransactionIdentifier")
		public String getUniqueTransactionIdentifier() {
			return uniqueTransactionIdentifier;
		}
		
		@Override
		@RosettaAttribute("uniqueProductIdentifier")
		public String getUniqueProductIdentifier() {
			return uniqueProductIdentifier;
		}
		
		@Override
		@RosettaAttribute("assetClass")
		public String getAssetClass() {
			return assetClass;
		}
		
		@Override
		@RosettaAttribute("contractType")
		public String getContractType() {
			return contractType;
		}
		
		@Override
		@RosettaAttribute("reportingEntity")
		public String getReportingEntity() {
			return reportingEntity;
		}
		
		@Override
		@RosettaAttribute("counterparty1")
		public String getCounterparty1() {
			return counterparty1;
		}
		
		@Override
		@RosettaAttribute("counterparty2")
		public String getCounterparty2() {
			return counterparty2;
		}
		
		@Override
		@RosettaAttribute("counterpartyIdentifierTypeIndicator")
		public Boolean getCounterpartyIdentifierTypeIndicator() {
			return counterpartyIdentifierTypeIndicator;
		}
		
		@Override
		@RosettaAttribute("countryOfTheCounterParty2")
		public ISOCountryCodeEnum getCountryOfTheCounterParty2() {
			return countryOfTheCounterParty2;
		}
		
		@Override
		@RosettaAttribute("beneficiary1")
		public String getBeneficiary1() {
			return beneficiary1;
		}
		
		@Override
		@RosettaAttribute("beneficiary1IdentifierTypeIndicator")
		public Boolean getBeneficiary1IdentifierTypeIndicator() {
			return beneficiary1IdentifierTypeIndicator;
		}
		
		@Override
		@RosettaAttribute("broker")
		public String getBroker() {
			return broker;
		}
		
		@Override
		@RosettaAttribute("executionAgentOfTheCounterparty1")
		public String getExecutionAgentOfTheCounterparty1() {
			return executionAgentOfTheCounterparty1;
		}
		
		@Override
		@RosettaAttribute("direction")
		public String getDirection() {
			return direction;
		}
		
		@Override
		@RosettaAttribute("direction2OfLeg1")
		public String getDirection2OfLeg1() {
			return direction2OfLeg1;
		}
		
		@Override
		@RosettaAttribute("direction2OfLeg2")
		public String getDirection2OfLeg2() {
			return direction2OfLeg2;
		}
		
		@Override
		@RosettaAttribute("effectiveDate")
		public Date getEffectiveDate() {
			return effectiveDate;
		}
		
		@Override
		@RosettaAttribute("expirationDate")
		public Date getExpirationDate() {
			return expirationDate;
		}
		
		@Override
		@RosettaAttribute("executionTimestamp")
		public ZonedDateTime getExecutionTimestamp() {
			return executionTimestamp;
		}
		
		@Override
		@RosettaAttribute("eventTimestamp")
		public ZonedDateTime getEventTimestamp() {
			return eventTimestamp;
		}
		
		@Override
		@RosettaAttribute("clearingTimestamp")
		public ZonedDateTime getClearingTimestamp() {
			return clearingTimestamp;
		}
		
		@Override
		@RosettaAttribute("cleared")
		public String getCleared() {
			return cleared;
		}
		
		@Override
		@RosettaAttribute("centralCounterparty")
		public String getCentralCounterparty() {
			return centralCounterparty;
		}
		
		@Override
		@RosettaAttribute("clearingMember")
		public String getClearingMember() {
			return clearingMember;
		}
		
		@Override
		@RosettaAttribute("platformIdentifier")
		public String getPlatformIdentifier() {
			return platformIdentifier;
		}
		
		@Override
		@RosettaAttribute("notionalLeg1")
		public BigDecimal getNotionalLeg1() {
			return notionalLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalLeg2")
		public BigDecimal getNotionalLeg2() {
			return notionalLeg2;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg1")
		public BigDecimal getTotalNotionalQuantityLeg1() {
			return totalNotionalQuantityLeg1;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg2")
		public BigDecimal getTotalNotionalQuantityLeg2() {
			return totalNotionalQuantityLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityLeg1")
		public BigDecimal getNotionalQuantityLeg1() {
			return notionalQuantityLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityLeg2")
		public BigDecimal getNotionalQuantityLeg2() {
			return notionalQuantityLeg2;
		}
		
		@Override
		@RosettaAttribute("callAmount")
		public BigDecimal getCallAmount() {
			return callAmount;
		}
		
		@Override
		@RosettaAttribute("putAmount")
		public BigDecimal getPutAmount() {
			return putAmount;
		}
		
		@Override
		@RosettaAttribute("notionalCurrencyLeg1")
		public ISOCurrencyCodeEnum getNotionalCurrencyLeg1() {
			return notionalCurrencyLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalCurrencyLeg2")
		public ISOCurrencyCodeEnum getNotionalCurrencyLeg2() {
			return notionalCurrencyLeg2;
		}
		
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg1")
		public String getQuantityUnitOfMeasureLeg1() {
			return quantityUnitOfMeasureLeg1;
		}
		
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg2")
		public String getQuantityUnitOfMeasureLeg2() {
			return quantityUnitOfMeasureLeg2;
		}
		
		@Override
		@RosettaAttribute("callCurrency")
		public ISOCurrencyCodeEnum getCallCurrency() {
			return callCurrency;
		}
		
		@Override
		@RosettaAttribute("putCurrency")
		public ISOCurrencyCodeEnum getPutCurrency() {
			return putCurrency;
		}
		
		@Override
		@RosettaAttribute("notionalAmountScheduleLeg1")
		public List<? extends NotionalAmountScheduleLeg1Report> getNotionalAmountScheduleLeg1() {
			return notionalAmountScheduleLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalAmountScheduleLeg2")
		public List<? extends NotionalAmountScheduleLeg2Report> getNotionalAmountScheduleLeg2() {
			return notionalAmountScheduleLeg2;
		}
		
		@Override
		@RosettaAttribute("priceNotation")
		public String getPriceNotation() {
			return priceNotation;
		}
		
		@Override
		@RosettaAttribute("price")
		public BigDecimal getPrice() {
			return price;
		}
		
		@Override
		@RosettaAttribute("priceCurrency")
		public ISOCurrencyCodeEnum getPriceCurrency() {
			return priceCurrency;
		}
		
		@Override
		@RosettaAttribute("priceUnitOfMeasure")
		public String getPriceUnitOfMeasure() {
			return priceUnitOfMeasure;
		}
		
		@Override
		@RosettaAttribute("fixedRateLeg1")
		public BigDecimal getFixedRateLeg1() {
			return fixedRateLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRateLeg2")
		public BigDecimal getFixedRateLeg2() {
			return fixedRateLeg2;
		}
		
		@Override
		@RosettaAttribute("spreadNotationLeg1")
		public String getSpreadNotationLeg1() {
			return spreadNotationLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadLeg1")
		public SpreadLeg1 getSpreadLeg1() {
			return spreadLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadNotationLeg2")
		public String getSpreadNotationLeg2() {
			return spreadNotationLeg2;
		}
		
		@Override
		@RosettaAttribute("spreadLeg2")
		public SpreadLeg2 getSpreadLeg2() {
			return spreadLeg2;
		}
		
		@Override
		@RosettaAttribute("spreadCurrencyLeg1")
		public ISOCurrencyCodeEnum getSpreadCurrencyLeg1() {
			return spreadCurrencyLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadCurrencyLeg2")
		public ISOCurrencyCodeEnum getSpreadCurrencyLeg2() {
			return spreadCurrencyLeg2;
		}
		
		@Override
		@RosettaAttribute("strikePriceNotation")
		public String getStrikePriceNotation() {
			return strikePriceNotation;
		}
		
		@Override
		@RosettaAttribute("strikePrice")
		public StrikePriceAndNotation getStrikePrice() {
			return strikePrice;
		}
		
		@Override
		@RosettaAttribute("strikePriceCurrency")
		public String getStrikePriceCurrency() {
			return strikePriceCurrency;
		}
		
		@Override
		@RosettaAttribute("optionPremiumAmount")
		public BigDecimal getOptionPremiumAmount() {
			return optionPremiumAmount;
		}
		
		@Override
		@RosettaAttribute("optionPremiumCurrency")
		public ISOCurrencyCodeEnum getOptionPremiumCurrency() {
			return optionPremiumCurrency;
		}
		
		@Override
		@RosettaAttribute("optionPremiumPaymentDate")
		public Date getOptionPremiumPaymentDate() {
			return optionPremiumPaymentDate;
		}
		
		@Override
		@RosettaAttribute("exchangeRate")
		public BigDecimal getExchangeRate() {
			return exchangeRate;
		}
		
		@Override
		@RosettaAttribute("exchangeRateBasis")
		public String getExchangeRateBasis() {
			return exchangeRateBasis;
		}
		
		@Override
		@RosettaAttribute("dayCountConventionLeg1")
		public InterestComputationMethod4Code getDayCountConventionLeg1() {
			return dayCountConventionLeg1;
		}
		
		@Override
		@RosettaAttribute("dayCountConventionLeg2")
		public InterestComputationMethod4Code getDayCountConventionLeg2() {
			return dayCountConventionLeg2;
		}
		
		@Override
		@RosettaAttribute("paymentFrequencyPeriodLeg1")
		public String getPaymentFrequencyPeriodLeg1() {
			return paymentFrequencyPeriodLeg1;
		}
		
		@Override
		@RosettaAttribute("paymentFrequencyPeriodLeg2")
		public String getPaymentFrequencyPeriodLeg2() {
			return paymentFrequencyPeriodLeg2;
		}
		
		@Override
		@RosettaAttribute("paymentFrequencyPeriodMultiplierLeg1")
		public Integer getPaymentFrequencyPeriodMultiplierLeg1() {
			return paymentFrequencyPeriodMultiplierLeg1;
		}
		
		@Override
		@RosettaAttribute("paymentFrequencyPeriodMultiplierLeg2")
		public Integer getPaymentFrequencyPeriodMultiplierLeg2() {
			return paymentFrequencyPeriodMultiplierLeg2;
		}
		
		@Override
		@RosettaAttribute("settlementCurrencyLeg1")
		public ISOCurrencyCodeEnum getSettlementCurrencyLeg1() {
			return settlementCurrencyLeg1;
		}
		
		@Override
		@RosettaAttribute("settlementCurrencyLeg2")
		public String getSettlementCurrencyLeg2() {
			return settlementCurrencyLeg2;
		}
		
		@Override
		@RosettaAttribute("otherPayment")
		public List<? extends OtherPaymentReport> getOtherPayment() {
			return otherPayment;
		}
		
		@Override
		@RosettaAttribute("customBasketCode")
		public String getCustomBasketCode() {
			return customBasketCode;
		}
		
		@Override
		@RosettaAttribute("basketConstituents")
		public List<? extends BasketConstituentsReport> getBasketConstituents() {
			return basketConstituents;
		}
		
		@Override
		@RosettaAttribute("basketConstituentIdentifierSource")
		public List<ProductIdTypeEnum> getBasketConstituentIdentifierSource() {
			return basketConstituentIdentifierSource;
		}
		
		@Override
		@RosettaAttribute("underlyingIdentification")
		public String getUnderlyingIdentification() {
			return underlyingIdentification;
		}
		
		@Override
		@RosettaAttribute("underlierIDSource")
		public String getUnderlierIDSource() {
			return underlierIDSource;
		}
		
		@Override
		@RosettaAttribute("maturityDateOfTheUnderlying")
		public Date getMaturityDateOfTheUnderlying() {
			return maturityDateOfTheUnderlying;
		}
		
		@Override
		@RosettaAttribute("identifierOfTheFloatingRateLeg2")
		public String getIdentifierOfTheFloatingRateLeg2() {
			return identifierOfTheFloatingRateLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg2TimePeriod")
		public String getFloatingRateReferencePeriodOfLeg2TimePeriod() {
			return floatingRateReferencePeriodOfLeg2TimePeriod;
		}
		
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg2Multiplier")
		public Integer getFloatingRateReferencePeriodOfLeg2Multiplier() {
			return floatingRateReferencePeriodOfLeg2Multiplier;
		}
		
		@Override
		@RosettaAttribute("cdSIndexAttachmentPoint")
		public BigDecimal getCdSIndexAttachmentPoint() {
			return cdSIndexAttachmentPoint;
		}
		
		@Override
		@RosettaAttribute("cdSIndexDetachmentPoint")
		public BigDecimal getCdSIndexDetachmentPoint() {
			return cdSIndexDetachmentPoint;
		}
		
		@Override
		@RosettaAttribute("indexFactor")
		public BigDecimal getIndexFactor() {
			return indexFactor;
		}
		
		@Override
		@RosettaAttribute("packageIdentifier")
		public String getPackageIdentifier() {
			return packageIdentifier;
		}
		
		@Override
		@RosettaAttribute("packageTransactionPrice")
		public PackageTransactionPrice getPackageTransactionPrice() {
			return packageTransactionPrice;
		}
		
		@Override
		@RosettaAttribute("packageTransactionSpread")
		public PackageTransactionSpread getPackageTransactionSpread() {
			return packageTransactionSpread;
		}
		
		@Override
		@RosettaAttribute("priorUTI")
		public String getPriorUTI() {
			return priorUTI;
		}
		
		@Override
		@RosettaAttribute("eventIdentifier")
		public String getEventIdentifier() {
			return eventIdentifier;
		}
		
		@Override
		@RosettaAttribute("actionType")
		public String getActionType() {
			return actionType;
		}
		
		@Override
		@RosettaAttribute("eventType")
		public String getEventType() {
			return eventType;
		}
		
		@Override
		@RosettaAttribute("reportingTimestamp")
		public ZonedDateTime getReportingTimestamp() {
			return reportingTimestamp;
		}
		
		@Override
		@RosettaAttribute("reportSubmittingEntityID")
		public String getReportSubmittingEntityID() {
			return reportSubmittingEntityID;
		}
		
		@Override
		@RosettaAttribute("valuationTimestamp")
		public ZonedDateTime getValuationTimestamp() {
			return valuationTimestamp;
		}
		
		@Override
		@RosettaAttribute("valuationAmount")
		public BigDecimal getValuationAmount() {
			return valuationAmount;
		}
		
		@Override
		@RosettaAttribute("valuationCurrency")
		public ISOCurrencyCodeEnum getValuationCurrency() {
			return valuationCurrency;
		}
		
		@Override
		@RosettaAttribute("valuationMethod")
		public ValuationType1Code getValuationMethod() {
			return valuationMethod;
		}
		
		@Override
		@RosettaAttribute("delta")
		public BigDecimal getDelta() {
			return delta;
		}
		
		@Override
		@RosettaAttribute("nextFloatingReferenceResetDateLeg1")
		public Date getNextFloatingReferenceResetDateLeg1() {
			return nextFloatingReferenceResetDateLeg1;
		}
		
		@Override
		@RosettaAttribute("nextFloatingReferenceResetDateLeg2")
		public Date getNextFloatingReferenceResetDateLeg2() {
			return nextFloatingReferenceResetDateLeg2;
		}
		
		@Override
		@RosettaAttribute("smallScaleBuySideEntityIndicator")
		public String getSmallScaleBuySideEntityIndicator() {
			return smallScaleBuySideEntityIndicator;
		}
		
		@Override
		@RosettaAttribute("nonReportable")
		public ASICNonReportable getNonReportable() {
			return nonReportable;
		}
		
		@Override
		public ASICTransactionReport build() {
			return this;
		}
		
		@Override
		public ASICTransactionReport.ASICTransactionReportBuilder toBuilder() {
			ASICTransactionReport.ASICTransactionReportBuilder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(ASICTransactionReport.ASICTransactionReportBuilder builder) {
			ofNullable(getUniqueTransactionIdentifier()).ifPresent(builder::setUniqueTransactionIdentifier);
			ofNullable(getUniqueProductIdentifier()).ifPresent(builder::setUniqueProductIdentifier);
			ofNullable(getAssetClass()).ifPresent(builder::setAssetClass);
			ofNullable(getContractType()).ifPresent(builder::setContractType);
			ofNullable(getReportingEntity()).ifPresent(builder::setReportingEntity);
			ofNullable(getCounterparty1()).ifPresent(builder::setCounterparty1);
			ofNullable(getCounterparty2()).ifPresent(builder::setCounterparty2);
			ofNullable(getCounterpartyIdentifierTypeIndicator()).ifPresent(builder::setCounterpartyIdentifierTypeIndicator);
			ofNullable(getCountryOfTheCounterParty2()).ifPresent(builder::setCountryOfTheCounterParty2);
			ofNullable(getBeneficiary1()).ifPresent(builder::setBeneficiary1);
			ofNullable(getBeneficiary1IdentifierTypeIndicator()).ifPresent(builder::setBeneficiary1IdentifierTypeIndicator);
			ofNullable(getBroker()).ifPresent(builder::setBroker);
			ofNullable(getExecutionAgentOfTheCounterparty1()).ifPresent(builder::setExecutionAgentOfTheCounterparty1);
			ofNullable(getDirection()).ifPresent(builder::setDirection);
			ofNullable(getDirection2OfLeg1()).ifPresent(builder::setDirection2OfLeg1);
			ofNullable(getDirection2OfLeg2()).ifPresent(builder::setDirection2OfLeg2);
			ofNullable(getEffectiveDate()).ifPresent(builder::setEffectiveDate);
			ofNullable(getExpirationDate()).ifPresent(builder::setExpirationDate);
			ofNullable(getExecutionTimestamp()).ifPresent(builder::setExecutionTimestamp);
			ofNullable(getEventTimestamp()).ifPresent(builder::setEventTimestamp);
			ofNullable(getClearingTimestamp()).ifPresent(builder::setClearingTimestamp);
			ofNullable(getCleared()).ifPresent(builder::setCleared);
			ofNullable(getCentralCounterparty()).ifPresent(builder::setCentralCounterparty);
			ofNullable(getClearingMember()).ifPresent(builder::setClearingMember);
			ofNullable(getPlatformIdentifier()).ifPresent(builder::setPlatformIdentifier);
			ofNullable(getNotionalLeg1()).ifPresent(builder::setNotionalLeg1);
			ofNullable(getNotionalLeg2()).ifPresent(builder::setNotionalLeg2);
			ofNullable(getTotalNotionalQuantityLeg1()).ifPresent(builder::setTotalNotionalQuantityLeg1);
			ofNullable(getTotalNotionalQuantityLeg2()).ifPresent(builder::setTotalNotionalQuantityLeg2);
			ofNullable(getNotionalQuantityLeg1()).ifPresent(builder::setNotionalQuantityLeg1);
			ofNullable(getNotionalQuantityLeg2()).ifPresent(builder::setNotionalQuantityLeg2);
			ofNullable(getCallAmount()).ifPresent(builder::setCallAmount);
			ofNullable(getPutAmount()).ifPresent(builder::setPutAmount);
			ofNullable(getNotionalCurrencyLeg1()).ifPresent(builder::setNotionalCurrencyLeg1);
			ofNullable(getNotionalCurrencyLeg2()).ifPresent(builder::setNotionalCurrencyLeg2);
			ofNullable(getQuantityUnitOfMeasureLeg1()).ifPresent(builder::setQuantityUnitOfMeasureLeg1);
			ofNullable(getQuantityUnitOfMeasureLeg2()).ifPresent(builder::setQuantityUnitOfMeasureLeg2);
			ofNullable(getCallCurrency()).ifPresent(builder::setCallCurrency);
			ofNullable(getPutCurrency()).ifPresent(builder::setPutCurrency);
			ofNullable(getNotionalAmountScheduleLeg1()).ifPresent(builder::setNotionalAmountScheduleLeg1);
			ofNullable(getNotionalAmountScheduleLeg2()).ifPresent(builder::setNotionalAmountScheduleLeg2);
			ofNullable(getPriceNotation()).ifPresent(builder::setPriceNotation);
			ofNullable(getPrice()).ifPresent(builder::setPrice);
			ofNullable(getPriceCurrency()).ifPresent(builder::setPriceCurrency);
			ofNullable(getPriceUnitOfMeasure()).ifPresent(builder::setPriceUnitOfMeasure);
			ofNullable(getFixedRateLeg1()).ifPresent(builder::setFixedRateLeg1);
			ofNullable(getFixedRateLeg2()).ifPresent(builder::setFixedRateLeg2);
			ofNullable(getSpreadNotationLeg1()).ifPresent(builder::setSpreadNotationLeg1);
			ofNullable(getSpreadLeg1()).ifPresent(builder::setSpreadLeg1);
			ofNullable(getSpreadNotationLeg2()).ifPresent(builder::setSpreadNotationLeg2);
			ofNullable(getSpreadLeg2()).ifPresent(builder::setSpreadLeg2);
			ofNullable(getSpreadCurrencyLeg1()).ifPresent(builder::setSpreadCurrencyLeg1);
			ofNullable(getSpreadCurrencyLeg2()).ifPresent(builder::setSpreadCurrencyLeg2);
			ofNullable(getStrikePriceNotation()).ifPresent(builder::setStrikePriceNotation);
			ofNullable(getStrikePrice()).ifPresent(builder::setStrikePrice);
			ofNullable(getStrikePriceCurrency()).ifPresent(builder::setStrikePriceCurrency);
			ofNullable(getOptionPremiumAmount()).ifPresent(builder::setOptionPremiumAmount);
			ofNullable(getOptionPremiumCurrency()).ifPresent(builder::setOptionPremiumCurrency);
			ofNullable(getOptionPremiumPaymentDate()).ifPresent(builder::setOptionPremiumPaymentDate);
			ofNullable(getExchangeRate()).ifPresent(builder::setExchangeRate);
			ofNullable(getExchangeRateBasis()).ifPresent(builder::setExchangeRateBasis);
			ofNullable(getDayCountConventionLeg1()).ifPresent(builder::setDayCountConventionLeg1);
			ofNullable(getDayCountConventionLeg2()).ifPresent(builder::setDayCountConventionLeg2);
			ofNullable(getPaymentFrequencyPeriodLeg1()).ifPresent(builder::setPaymentFrequencyPeriodLeg1);
			ofNullable(getPaymentFrequencyPeriodLeg2()).ifPresent(builder::setPaymentFrequencyPeriodLeg2);
			ofNullable(getPaymentFrequencyPeriodMultiplierLeg1()).ifPresent(builder::setPaymentFrequencyPeriodMultiplierLeg1);
			ofNullable(getPaymentFrequencyPeriodMultiplierLeg2()).ifPresent(builder::setPaymentFrequencyPeriodMultiplierLeg2);
			ofNullable(getSettlementCurrencyLeg1()).ifPresent(builder::setSettlementCurrencyLeg1);
			ofNullable(getSettlementCurrencyLeg2()).ifPresent(builder::setSettlementCurrencyLeg2);
			ofNullable(getOtherPayment()).ifPresent(builder::setOtherPayment);
			ofNullable(getCustomBasketCode()).ifPresent(builder::setCustomBasketCode);
			ofNullable(getBasketConstituents()).ifPresent(builder::setBasketConstituents);
			ofNullable(getBasketConstituentIdentifierSource()).ifPresent(builder::setBasketConstituentIdentifierSource);
			ofNullable(getUnderlyingIdentification()).ifPresent(builder::setUnderlyingIdentification);
			ofNullable(getUnderlierIDSource()).ifPresent(builder::setUnderlierIDSource);
			ofNullable(getMaturityDateOfTheUnderlying()).ifPresent(builder::setMaturityDateOfTheUnderlying);
			ofNullable(getIdentifierOfTheFloatingRateLeg2()).ifPresent(builder::setIdentifierOfTheFloatingRateLeg2);
			ofNullable(getFloatingRateReferencePeriodOfLeg2TimePeriod()).ifPresent(builder::setFloatingRateReferencePeriodOfLeg2TimePeriod);
			ofNullable(getFloatingRateReferencePeriodOfLeg2Multiplier()).ifPresent(builder::setFloatingRateReferencePeriodOfLeg2Multiplier);
			ofNullable(getCdSIndexAttachmentPoint()).ifPresent(builder::setCdSIndexAttachmentPoint);
			ofNullable(getCdSIndexDetachmentPoint()).ifPresent(builder::setCdSIndexDetachmentPoint);
			ofNullable(getIndexFactor()).ifPresent(builder::setIndexFactor);
			ofNullable(getPackageIdentifier()).ifPresent(builder::setPackageIdentifier);
			ofNullable(getPackageTransactionPrice()).ifPresent(builder::setPackageTransactionPrice);
			ofNullable(getPackageTransactionSpread()).ifPresent(builder::setPackageTransactionSpread);
			ofNullable(getPriorUTI()).ifPresent(builder::setPriorUTI);
			ofNullable(getEventIdentifier()).ifPresent(builder::setEventIdentifier);
			ofNullable(getActionType()).ifPresent(builder::setActionType);
			ofNullable(getEventType()).ifPresent(builder::setEventType);
			ofNullable(getReportingTimestamp()).ifPresent(builder::setReportingTimestamp);
			ofNullable(getReportSubmittingEntityID()).ifPresent(builder::setReportSubmittingEntityID);
			ofNullable(getValuationTimestamp()).ifPresent(builder::setValuationTimestamp);
			ofNullable(getValuationAmount()).ifPresent(builder::setValuationAmount);
			ofNullable(getValuationCurrency()).ifPresent(builder::setValuationCurrency);
			ofNullable(getValuationMethod()).ifPresent(builder::setValuationMethod);
			ofNullable(getDelta()).ifPresent(builder::setDelta);
			ofNullable(getNextFloatingReferenceResetDateLeg1()).ifPresent(builder::setNextFloatingReferenceResetDateLeg1);
			ofNullable(getNextFloatingReferenceResetDateLeg2()).ifPresent(builder::setNextFloatingReferenceResetDateLeg2);
			ofNullable(getSmallScaleBuySideEntityIndicator()).ifPresent(builder::setSmallScaleBuySideEntityIndicator);
			ofNullable(getNonReportable()).ifPresent(builder::setNonReportable);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			ASICTransactionReport _that = getType().cast(o);
		
			if (!Objects.equals(uniqueTransactionIdentifier, _that.getUniqueTransactionIdentifier())) return false;
			if (!Objects.equals(uniqueProductIdentifier, _that.getUniqueProductIdentifier())) return false;
			if (!Objects.equals(assetClass, _that.getAssetClass())) return false;
			if (!Objects.equals(contractType, _that.getContractType())) return false;
			if (!Objects.equals(reportingEntity, _that.getReportingEntity())) return false;
			if (!Objects.equals(counterparty1, _that.getCounterparty1())) return false;
			if (!Objects.equals(counterparty2, _that.getCounterparty2())) return false;
			if (!Objects.equals(counterpartyIdentifierTypeIndicator, _that.getCounterpartyIdentifierTypeIndicator())) return false;
			if (!Objects.equals(countryOfTheCounterParty2, _that.getCountryOfTheCounterParty2())) return false;
			if (!Objects.equals(beneficiary1, _that.getBeneficiary1())) return false;
			if (!Objects.equals(beneficiary1IdentifierTypeIndicator, _that.getBeneficiary1IdentifierTypeIndicator())) return false;
			if (!Objects.equals(broker, _that.getBroker())) return false;
			if (!Objects.equals(executionAgentOfTheCounterparty1, _that.getExecutionAgentOfTheCounterparty1())) return false;
			if (!Objects.equals(direction, _that.getDirection())) return false;
			if (!Objects.equals(direction2OfLeg1, _that.getDirection2OfLeg1())) return false;
			if (!Objects.equals(direction2OfLeg2, _that.getDirection2OfLeg2())) return false;
			if (!Objects.equals(effectiveDate, _that.getEffectiveDate())) return false;
			if (!Objects.equals(expirationDate, _that.getExpirationDate())) return false;
			if (!Objects.equals(executionTimestamp, _that.getExecutionTimestamp())) return false;
			if (!Objects.equals(eventTimestamp, _that.getEventTimestamp())) return false;
			if (!Objects.equals(clearingTimestamp, _that.getClearingTimestamp())) return false;
			if (!Objects.equals(cleared, _that.getCleared())) return false;
			if (!Objects.equals(centralCounterparty, _that.getCentralCounterparty())) return false;
			if (!Objects.equals(clearingMember, _that.getClearingMember())) return false;
			if (!Objects.equals(platformIdentifier, _that.getPlatformIdentifier())) return false;
			if (!Objects.equals(notionalLeg1, _that.getNotionalLeg1())) return false;
			if (!Objects.equals(notionalLeg2, _that.getNotionalLeg2())) return false;
			if (!Objects.equals(totalNotionalQuantityLeg1, _that.getTotalNotionalQuantityLeg1())) return false;
			if (!Objects.equals(totalNotionalQuantityLeg2, _that.getTotalNotionalQuantityLeg2())) return false;
			if (!Objects.equals(notionalQuantityLeg1, _that.getNotionalQuantityLeg1())) return false;
			if (!Objects.equals(notionalQuantityLeg2, _that.getNotionalQuantityLeg2())) return false;
			if (!Objects.equals(callAmount, _that.getCallAmount())) return false;
			if (!Objects.equals(putAmount, _that.getPutAmount())) return false;
			if (!Objects.equals(notionalCurrencyLeg1, _that.getNotionalCurrencyLeg1())) return false;
			if (!Objects.equals(notionalCurrencyLeg2, _that.getNotionalCurrencyLeg2())) return false;
			if (!Objects.equals(quantityUnitOfMeasureLeg1, _that.getQuantityUnitOfMeasureLeg1())) return false;
			if (!Objects.equals(quantityUnitOfMeasureLeg2, _that.getQuantityUnitOfMeasureLeg2())) return false;
			if (!Objects.equals(callCurrency, _that.getCallCurrency())) return false;
			if (!Objects.equals(putCurrency, _that.getPutCurrency())) return false;
			if (!ListEquals.listEquals(notionalAmountScheduleLeg1, _that.getNotionalAmountScheduleLeg1())) return false;
			if (!ListEquals.listEquals(notionalAmountScheduleLeg2, _that.getNotionalAmountScheduleLeg2())) return false;
			if (!Objects.equals(priceNotation, _that.getPriceNotation())) return false;
			if (!Objects.equals(price, _that.getPrice())) return false;
			if (!Objects.equals(priceCurrency, _that.getPriceCurrency())) return false;
			if (!Objects.equals(priceUnitOfMeasure, _that.getPriceUnitOfMeasure())) return false;
			if (!Objects.equals(fixedRateLeg1, _that.getFixedRateLeg1())) return false;
			if (!Objects.equals(fixedRateLeg2, _that.getFixedRateLeg2())) return false;
			if (!Objects.equals(spreadNotationLeg1, _that.getSpreadNotationLeg1())) return false;
			if (!Objects.equals(spreadLeg1, _that.getSpreadLeg1())) return false;
			if (!Objects.equals(spreadNotationLeg2, _that.getSpreadNotationLeg2())) return false;
			if (!Objects.equals(spreadLeg2, _that.getSpreadLeg2())) return false;
			if (!Objects.equals(spreadCurrencyLeg1, _that.getSpreadCurrencyLeg1())) return false;
			if (!Objects.equals(spreadCurrencyLeg2, _that.getSpreadCurrencyLeg2())) return false;
			if (!Objects.equals(strikePriceNotation, _that.getStrikePriceNotation())) return false;
			if (!Objects.equals(strikePrice, _that.getStrikePrice())) return false;
			if (!Objects.equals(strikePriceCurrency, _that.getStrikePriceCurrency())) return false;
			if (!Objects.equals(optionPremiumAmount, _that.getOptionPremiumAmount())) return false;
			if (!Objects.equals(optionPremiumCurrency, _that.getOptionPremiumCurrency())) return false;
			if (!Objects.equals(optionPremiumPaymentDate, _that.getOptionPremiumPaymentDate())) return false;
			if (!Objects.equals(exchangeRate, _that.getExchangeRate())) return false;
			if (!Objects.equals(exchangeRateBasis, _that.getExchangeRateBasis())) return false;
			if (!Objects.equals(dayCountConventionLeg1, _that.getDayCountConventionLeg1())) return false;
			if (!Objects.equals(dayCountConventionLeg2, _that.getDayCountConventionLeg2())) return false;
			if (!Objects.equals(paymentFrequencyPeriodLeg1, _that.getPaymentFrequencyPeriodLeg1())) return false;
			if (!Objects.equals(paymentFrequencyPeriodLeg2, _that.getPaymentFrequencyPeriodLeg2())) return false;
			if (!Objects.equals(paymentFrequencyPeriodMultiplierLeg1, _that.getPaymentFrequencyPeriodMultiplierLeg1())) return false;
			if (!Objects.equals(paymentFrequencyPeriodMultiplierLeg2, _that.getPaymentFrequencyPeriodMultiplierLeg2())) return false;
			if (!Objects.equals(settlementCurrencyLeg1, _that.getSettlementCurrencyLeg1())) return false;
			if (!Objects.equals(settlementCurrencyLeg2, _that.getSettlementCurrencyLeg2())) return false;
			if (!ListEquals.listEquals(otherPayment, _that.getOtherPayment())) return false;
			if (!Objects.equals(customBasketCode, _that.getCustomBasketCode())) return false;
			if (!ListEquals.listEquals(basketConstituents, _that.getBasketConstituents())) return false;
			if (!ListEquals.listEquals(basketConstituentIdentifierSource, _that.getBasketConstituentIdentifierSource())) return false;
			if (!Objects.equals(underlyingIdentification, _that.getUnderlyingIdentification())) return false;
			if (!Objects.equals(underlierIDSource, _that.getUnderlierIDSource())) return false;
			if (!Objects.equals(maturityDateOfTheUnderlying, _that.getMaturityDateOfTheUnderlying())) return false;
			if (!Objects.equals(identifierOfTheFloatingRateLeg2, _that.getIdentifierOfTheFloatingRateLeg2())) return false;
			if (!Objects.equals(floatingRateReferencePeriodOfLeg2TimePeriod, _that.getFloatingRateReferencePeriodOfLeg2TimePeriod())) return false;
			if (!Objects.equals(floatingRateReferencePeriodOfLeg2Multiplier, _that.getFloatingRateReferencePeriodOfLeg2Multiplier())) return false;
			if (!Objects.equals(cdSIndexAttachmentPoint, _that.getCdSIndexAttachmentPoint())) return false;
			if (!Objects.equals(cdSIndexDetachmentPoint, _that.getCdSIndexDetachmentPoint())) return false;
			if (!Objects.equals(indexFactor, _that.getIndexFactor())) return false;
			if (!Objects.equals(packageIdentifier, _that.getPackageIdentifier())) return false;
			if (!Objects.equals(packageTransactionPrice, _that.getPackageTransactionPrice())) return false;
			if (!Objects.equals(packageTransactionSpread, _that.getPackageTransactionSpread())) return false;
			if (!Objects.equals(priorUTI, _that.getPriorUTI())) return false;
			if (!Objects.equals(eventIdentifier, _that.getEventIdentifier())) return false;
			if (!Objects.equals(actionType, _that.getActionType())) return false;
			if (!Objects.equals(eventType, _that.getEventType())) return false;
			if (!Objects.equals(reportingTimestamp, _that.getReportingTimestamp())) return false;
			if (!Objects.equals(reportSubmittingEntityID, _that.getReportSubmittingEntityID())) return false;
			if (!Objects.equals(valuationTimestamp, _that.getValuationTimestamp())) return false;
			if (!Objects.equals(valuationAmount, _that.getValuationAmount())) return false;
			if (!Objects.equals(valuationCurrency, _that.getValuationCurrency())) return false;
			if (!Objects.equals(valuationMethod, _that.getValuationMethod())) return false;
			if (!Objects.equals(delta, _that.getDelta())) return false;
			if (!Objects.equals(nextFloatingReferenceResetDateLeg1, _that.getNextFloatingReferenceResetDateLeg1())) return false;
			if (!Objects.equals(nextFloatingReferenceResetDateLeg2, _that.getNextFloatingReferenceResetDateLeg2())) return false;
			if (!Objects.equals(smallScaleBuySideEntityIndicator, _that.getSmallScaleBuySideEntityIndicator())) return false;
			if (!Objects.equals(nonReportable, _that.getNonReportable())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (uniqueTransactionIdentifier != null ? uniqueTransactionIdentifier.hashCode() : 0);
			_result = 31 * _result + (uniqueProductIdentifier != null ? uniqueProductIdentifier.hashCode() : 0);
			_result = 31 * _result + (assetClass != null ? assetClass.hashCode() : 0);
			_result = 31 * _result + (contractType != null ? contractType.hashCode() : 0);
			_result = 31 * _result + (reportingEntity != null ? reportingEntity.hashCode() : 0);
			_result = 31 * _result + (counterparty1 != null ? counterparty1.hashCode() : 0);
			_result = 31 * _result + (counterparty2 != null ? counterparty2.hashCode() : 0);
			_result = 31 * _result + (counterpartyIdentifierTypeIndicator != null ? counterpartyIdentifierTypeIndicator.hashCode() : 0);
			_result = 31 * _result + (countryOfTheCounterParty2 != null ? countryOfTheCounterParty2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (beneficiary1 != null ? beneficiary1.hashCode() : 0);
			_result = 31 * _result + (beneficiary1IdentifierTypeIndicator != null ? beneficiary1IdentifierTypeIndicator.hashCode() : 0);
			_result = 31 * _result + (broker != null ? broker.hashCode() : 0);
			_result = 31 * _result + (executionAgentOfTheCounterparty1 != null ? executionAgentOfTheCounterparty1.hashCode() : 0);
			_result = 31 * _result + (direction != null ? direction.hashCode() : 0);
			_result = 31 * _result + (direction2OfLeg1 != null ? direction2OfLeg1.hashCode() : 0);
			_result = 31 * _result + (direction2OfLeg2 != null ? direction2OfLeg2.hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (expirationDate != null ? expirationDate.hashCode() : 0);
			_result = 31 * _result + (executionTimestamp != null ? executionTimestamp.hashCode() : 0);
			_result = 31 * _result + (eventTimestamp != null ? eventTimestamp.hashCode() : 0);
			_result = 31 * _result + (clearingTimestamp != null ? clearingTimestamp.hashCode() : 0);
			_result = 31 * _result + (cleared != null ? cleared.hashCode() : 0);
			_result = 31 * _result + (centralCounterparty != null ? centralCounterparty.hashCode() : 0);
			_result = 31 * _result + (clearingMember != null ? clearingMember.hashCode() : 0);
			_result = 31 * _result + (platformIdentifier != null ? platformIdentifier.hashCode() : 0);
			_result = 31 * _result + (notionalLeg1 != null ? notionalLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalLeg2 != null ? notionalLeg2.hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityLeg1 != null ? totalNotionalQuantityLeg1.hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityLeg2 != null ? totalNotionalQuantityLeg2.hashCode() : 0);
			_result = 31 * _result + (notionalQuantityLeg1 != null ? notionalQuantityLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalQuantityLeg2 != null ? notionalQuantityLeg2.hashCode() : 0);
			_result = 31 * _result + (callAmount != null ? callAmount.hashCode() : 0);
			_result = 31 * _result + (putAmount != null ? putAmount.hashCode() : 0);
			_result = 31 * _result + (notionalCurrencyLeg1 != null ? notionalCurrencyLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalCurrencyLeg2 != null ? notionalCurrencyLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (quantityUnitOfMeasureLeg1 != null ? quantityUnitOfMeasureLeg1.hashCode() : 0);
			_result = 31 * _result + (quantityUnitOfMeasureLeg2 != null ? quantityUnitOfMeasureLeg2.hashCode() : 0);
			_result = 31 * _result + (callCurrency != null ? callCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (putCurrency != null ? putCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalAmountScheduleLeg1 != null ? notionalAmountScheduleLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalAmountScheduleLeg2 != null ? notionalAmountScheduleLeg2.hashCode() : 0);
			_result = 31 * _result + (priceNotation != null ? priceNotation.hashCode() : 0);
			_result = 31 * _result + (price != null ? price.hashCode() : 0);
			_result = 31 * _result + (priceCurrency != null ? priceCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (priceUnitOfMeasure != null ? priceUnitOfMeasure.hashCode() : 0);
			_result = 31 * _result + (fixedRateLeg1 != null ? fixedRateLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRateLeg2 != null ? fixedRateLeg2.hashCode() : 0);
			_result = 31 * _result + (spreadNotationLeg1 != null ? spreadNotationLeg1.hashCode() : 0);
			_result = 31 * _result + (spreadLeg1 != null ? spreadLeg1.hashCode() : 0);
			_result = 31 * _result + (spreadNotationLeg2 != null ? spreadNotationLeg2.hashCode() : 0);
			_result = 31 * _result + (spreadLeg2 != null ? spreadLeg2.hashCode() : 0);
			_result = 31 * _result + (spreadCurrencyLeg1 != null ? spreadCurrencyLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (spreadCurrencyLeg2 != null ? spreadCurrencyLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (strikePriceNotation != null ? strikePriceNotation.hashCode() : 0);
			_result = 31 * _result + (strikePrice != null ? strikePrice.hashCode() : 0);
			_result = 31 * _result + (strikePriceCurrency != null ? strikePriceCurrency.hashCode() : 0);
			_result = 31 * _result + (optionPremiumAmount != null ? optionPremiumAmount.hashCode() : 0);
			_result = 31 * _result + (optionPremiumCurrency != null ? optionPremiumCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (optionPremiumPaymentDate != null ? optionPremiumPaymentDate.hashCode() : 0);
			_result = 31 * _result + (exchangeRate != null ? exchangeRate.hashCode() : 0);
			_result = 31 * _result + (exchangeRateBasis != null ? exchangeRateBasis.hashCode() : 0);
			_result = 31 * _result + (dayCountConventionLeg1 != null ? dayCountConventionLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (dayCountConventionLeg2 != null ? dayCountConventionLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (paymentFrequencyPeriodLeg1 != null ? paymentFrequencyPeriodLeg1.hashCode() : 0);
			_result = 31 * _result + (paymentFrequencyPeriodLeg2 != null ? paymentFrequencyPeriodLeg2.hashCode() : 0);
			_result = 31 * _result + (paymentFrequencyPeriodMultiplierLeg1 != null ? paymentFrequencyPeriodMultiplierLeg1.hashCode() : 0);
			_result = 31 * _result + (paymentFrequencyPeriodMultiplierLeg2 != null ? paymentFrequencyPeriodMultiplierLeg2.hashCode() : 0);
			_result = 31 * _result + (settlementCurrencyLeg1 != null ? settlementCurrencyLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (settlementCurrencyLeg2 != null ? settlementCurrencyLeg2.hashCode() : 0);
			_result = 31 * _result + (otherPayment != null ? otherPayment.hashCode() : 0);
			_result = 31 * _result + (customBasketCode != null ? customBasketCode.hashCode() : 0);
			_result = 31 * _result + (basketConstituents != null ? basketConstituents.hashCode() : 0);
			_result = 31 * _result + (basketConstituentIdentifierSource != null ? basketConstituentIdentifierSource.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
			_result = 31 * _result + (underlyingIdentification != null ? underlyingIdentification.hashCode() : 0);
			_result = 31 * _result + (underlierIDSource != null ? underlierIDSource.hashCode() : 0);
			_result = 31 * _result + (maturityDateOfTheUnderlying != null ? maturityDateOfTheUnderlying.hashCode() : 0);
			_result = 31 * _result + (identifierOfTheFloatingRateLeg2 != null ? identifierOfTheFloatingRateLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRateReferencePeriodOfLeg2TimePeriod != null ? floatingRateReferencePeriodOfLeg2TimePeriod.hashCode() : 0);
			_result = 31 * _result + (floatingRateReferencePeriodOfLeg2Multiplier != null ? floatingRateReferencePeriodOfLeg2Multiplier.hashCode() : 0);
			_result = 31 * _result + (cdSIndexAttachmentPoint != null ? cdSIndexAttachmentPoint.hashCode() : 0);
			_result = 31 * _result + (cdSIndexDetachmentPoint != null ? cdSIndexDetachmentPoint.hashCode() : 0);
			_result = 31 * _result + (indexFactor != null ? indexFactor.hashCode() : 0);
			_result = 31 * _result + (packageIdentifier != null ? packageIdentifier.hashCode() : 0);
			_result = 31 * _result + (packageTransactionPrice != null ? packageTransactionPrice.hashCode() : 0);
			_result = 31 * _result + (packageTransactionSpread != null ? packageTransactionSpread.hashCode() : 0);
			_result = 31 * _result + (priorUTI != null ? priorUTI.hashCode() : 0);
			_result = 31 * _result + (eventIdentifier != null ? eventIdentifier.hashCode() : 0);
			_result = 31 * _result + (actionType != null ? actionType.hashCode() : 0);
			_result = 31 * _result + (eventType != null ? eventType.hashCode() : 0);
			_result = 31 * _result + (reportingTimestamp != null ? reportingTimestamp.hashCode() : 0);
			_result = 31 * _result + (reportSubmittingEntityID != null ? reportSubmittingEntityID.hashCode() : 0);
			_result = 31 * _result + (valuationTimestamp != null ? valuationTimestamp.hashCode() : 0);
			_result = 31 * _result + (valuationAmount != null ? valuationAmount.hashCode() : 0);
			_result = 31 * _result + (valuationCurrency != null ? valuationCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (valuationMethod != null ? valuationMethod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (delta != null ? delta.hashCode() : 0);
			_result = 31 * _result + (nextFloatingReferenceResetDateLeg1 != null ? nextFloatingReferenceResetDateLeg1.hashCode() : 0);
			_result = 31 * _result + (nextFloatingReferenceResetDateLeg2 != null ? nextFloatingReferenceResetDateLeg2.hashCode() : 0);
			_result = 31 * _result + (smallScaleBuySideEntityIndicator != null ? smallScaleBuySideEntityIndicator.hashCode() : 0);
			_result = 31 * _result + (nonReportable != null ? nonReportable.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ASICTransactionReport {" +
				"uniqueTransactionIdentifier=" + this.uniqueTransactionIdentifier + ", " +
				"uniqueProductIdentifier=" + this.uniqueProductIdentifier + ", " +
				"assetClass=" + this.assetClass + ", " +
				"contractType=" + this.contractType + ", " +
				"reportingEntity=" + this.reportingEntity + ", " +
				"counterparty1=" + this.counterparty1 + ", " +
				"counterparty2=" + this.counterparty2 + ", " +
				"counterpartyIdentifierTypeIndicator=" + this.counterpartyIdentifierTypeIndicator + ", " +
				"countryOfTheCounterParty2=" + this.countryOfTheCounterParty2 + ", " +
				"beneficiary1=" + this.beneficiary1 + ", " +
				"beneficiary1IdentifierTypeIndicator=" + this.beneficiary1IdentifierTypeIndicator + ", " +
				"broker=" + this.broker + ", " +
				"executionAgentOfTheCounterparty1=" + this.executionAgentOfTheCounterparty1 + ", " +
				"direction=" + this.direction + ", " +
				"direction2OfLeg1=" + this.direction2OfLeg1 + ", " +
				"direction2OfLeg2=" + this.direction2OfLeg2 + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"expirationDate=" + this.expirationDate + ", " +
				"executionTimestamp=" + this.executionTimestamp + ", " +
				"eventTimestamp=" + this.eventTimestamp + ", " +
				"clearingTimestamp=" + this.clearingTimestamp + ", " +
				"cleared=" + this.cleared + ", " +
				"centralCounterparty=" + this.centralCounterparty + ", " +
				"clearingMember=" + this.clearingMember + ", " +
				"platformIdentifier=" + this.platformIdentifier + ", " +
				"notionalLeg1=" + this.notionalLeg1 + ", " +
				"notionalLeg2=" + this.notionalLeg2 + ", " +
				"totalNotionalQuantityLeg1=" + this.totalNotionalQuantityLeg1 + ", " +
				"totalNotionalQuantityLeg2=" + this.totalNotionalQuantityLeg2 + ", " +
				"notionalQuantityLeg1=" + this.notionalQuantityLeg1 + ", " +
				"notionalQuantityLeg2=" + this.notionalQuantityLeg2 + ", " +
				"callAmount=" + this.callAmount + ", " +
				"putAmount=" + this.putAmount + ", " +
				"notionalCurrencyLeg1=" + this.notionalCurrencyLeg1 + ", " +
				"notionalCurrencyLeg2=" + this.notionalCurrencyLeg2 + ", " +
				"quantityUnitOfMeasureLeg1=" + this.quantityUnitOfMeasureLeg1 + ", " +
				"quantityUnitOfMeasureLeg2=" + this.quantityUnitOfMeasureLeg2 + ", " +
				"callCurrency=" + this.callCurrency + ", " +
				"putCurrency=" + this.putCurrency + ", " +
				"notionalAmountScheduleLeg1=" + this.notionalAmountScheduleLeg1 + ", " +
				"notionalAmountScheduleLeg2=" + this.notionalAmountScheduleLeg2 + ", " +
				"priceNotation=" + this.priceNotation + ", " +
				"price=" + this.price + ", " +
				"priceCurrency=" + this.priceCurrency + ", " +
				"priceUnitOfMeasure=" + this.priceUnitOfMeasure + ", " +
				"fixedRateLeg1=" + this.fixedRateLeg1 + ", " +
				"fixedRateLeg2=" + this.fixedRateLeg2 + ", " +
				"spreadNotationLeg1=" + this.spreadNotationLeg1 + ", " +
				"spreadLeg1=" + this.spreadLeg1 + ", " +
				"spreadNotationLeg2=" + this.spreadNotationLeg2 + ", " +
				"spreadLeg2=" + this.spreadLeg2 + ", " +
				"spreadCurrencyLeg1=" + this.spreadCurrencyLeg1 + ", " +
				"spreadCurrencyLeg2=" + this.spreadCurrencyLeg2 + ", " +
				"strikePriceNotation=" + this.strikePriceNotation + ", " +
				"strikePrice=" + this.strikePrice + ", " +
				"strikePriceCurrency=" + this.strikePriceCurrency + ", " +
				"optionPremiumAmount=" + this.optionPremiumAmount + ", " +
				"optionPremiumCurrency=" + this.optionPremiumCurrency + ", " +
				"optionPremiumPaymentDate=" + this.optionPremiumPaymentDate + ", " +
				"exchangeRate=" + this.exchangeRate + ", " +
				"exchangeRateBasis=" + this.exchangeRateBasis + ", " +
				"dayCountConventionLeg1=" + this.dayCountConventionLeg1 + ", " +
				"dayCountConventionLeg2=" + this.dayCountConventionLeg2 + ", " +
				"paymentFrequencyPeriodLeg1=" + this.paymentFrequencyPeriodLeg1 + ", " +
				"paymentFrequencyPeriodLeg2=" + this.paymentFrequencyPeriodLeg2 + ", " +
				"paymentFrequencyPeriodMultiplierLeg1=" + this.paymentFrequencyPeriodMultiplierLeg1 + ", " +
				"paymentFrequencyPeriodMultiplierLeg2=" + this.paymentFrequencyPeriodMultiplierLeg2 + ", " +
				"settlementCurrencyLeg1=" + this.settlementCurrencyLeg1 + ", " +
				"settlementCurrencyLeg2=" + this.settlementCurrencyLeg2 + ", " +
				"otherPayment=" + this.otherPayment + ", " +
				"customBasketCode=" + this.customBasketCode + ", " +
				"basketConstituents=" + this.basketConstituents + ", " +
				"basketConstituentIdentifierSource=" + this.basketConstituentIdentifierSource + ", " +
				"underlyingIdentification=" + this.underlyingIdentification + ", " +
				"underlierIDSource=" + this.underlierIDSource + ", " +
				"maturityDateOfTheUnderlying=" + this.maturityDateOfTheUnderlying + ", " +
				"identifierOfTheFloatingRateLeg2=" + this.identifierOfTheFloatingRateLeg2 + ", " +
				"floatingRateReferencePeriodOfLeg2TimePeriod=" + this.floatingRateReferencePeriodOfLeg2TimePeriod + ", " +
				"floatingRateReferencePeriodOfLeg2Multiplier=" + this.floatingRateReferencePeriodOfLeg2Multiplier + ", " +
				"cdSIndexAttachmentPoint=" + this.cdSIndexAttachmentPoint + ", " +
				"cdSIndexDetachmentPoint=" + this.cdSIndexDetachmentPoint + ", " +
				"indexFactor=" + this.indexFactor + ", " +
				"packageIdentifier=" + this.packageIdentifier + ", " +
				"packageTransactionPrice=" + this.packageTransactionPrice + ", " +
				"packageTransactionSpread=" + this.packageTransactionSpread + ", " +
				"priorUTI=" + this.priorUTI + ", " +
				"eventIdentifier=" + this.eventIdentifier + ", " +
				"actionType=" + this.actionType + ", " +
				"eventType=" + this.eventType + ", " +
				"reportingTimestamp=" + this.reportingTimestamp + ", " +
				"reportSubmittingEntityID=" + this.reportSubmittingEntityID + ", " +
				"valuationTimestamp=" + this.valuationTimestamp + ", " +
				"valuationAmount=" + this.valuationAmount + ", " +
				"valuationCurrency=" + this.valuationCurrency + ", " +
				"valuationMethod=" + this.valuationMethod + ", " +
				"delta=" + this.delta + ", " +
				"nextFloatingReferenceResetDateLeg1=" + this.nextFloatingReferenceResetDateLeg1 + ", " +
				"nextFloatingReferenceResetDateLeg2=" + this.nextFloatingReferenceResetDateLeg2 + ", " +
				"smallScaleBuySideEntityIndicator=" + this.smallScaleBuySideEntityIndicator + ", " +
				"nonReportable=" + this.nonReportable +
			'}';
		}
	}

	/*********************** Builder Implementation of ASICTransactionReport  ***********************/
	class ASICTransactionReportBuilderImpl implements ASICTransactionReport.ASICTransactionReportBuilder {
	
		protected String uniqueTransactionIdentifier;
		protected String uniqueProductIdentifier;
		protected String assetClass;
		protected String contractType;
		protected String reportingEntity;
		protected String counterparty1;
		protected String counterparty2;
		protected Boolean counterpartyIdentifierTypeIndicator;
		protected ISOCountryCodeEnum countryOfTheCounterParty2;
		protected String beneficiary1;
		protected Boolean beneficiary1IdentifierTypeIndicator;
		protected String broker;
		protected String executionAgentOfTheCounterparty1;
		protected String direction;
		protected String direction2OfLeg1;
		protected String direction2OfLeg2;
		protected Date effectiveDate;
		protected Date expirationDate;
		protected ZonedDateTime executionTimestamp;
		protected ZonedDateTime eventTimestamp;
		protected ZonedDateTime clearingTimestamp;
		protected String cleared;
		protected String centralCounterparty;
		protected String clearingMember;
		protected String platformIdentifier;
		protected BigDecimal notionalLeg1;
		protected BigDecimal notionalLeg2;
		protected BigDecimal totalNotionalQuantityLeg1;
		protected BigDecimal totalNotionalQuantityLeg2;
		protected BigDecimal notionalQuantityLeg1;
		protected BigDecimal notionalQuantityLeg2;
		protected BigDecimal callAmount;
		protected BigDecimal putAmount;
		protected ISOCurrencyCodeEnum notionalCurrencyLeg1;
		protected ISOCurrencyCodeEnum notionalCurrencyLeg2;
		protected String quantityUnitOfMeasureLeg1;
		protected String quantityUnitOfMeasureLeg2;
		protected ISOCurrencyCodeEnum callCurrency;
		protected ISOCurrencyCodeEnum putCurrency;
		protected List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> notionalAmountScheduleLeg1 = new ArrayList<>();
		protected List<NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> notionalAmountScheduleLeg2 = new ArrayList<>();
		protected String priceNotation;
		protected BigDecimal price;
		protected ISOCurrencyCodeEnum priceCurrency;
		protected String priceUnitOfMeasure;
		protected BigDecimal fixedRateLeg1;
		protected BigDecimal fixedRateLeg2;
		protected String spreadNotationLeg1;
		protected SpreadLeg1.SpreadLeg1Builder spreadLeg1;
		protected String spreadNotationLeg2;
		protected SpreadLeg2.SpreadLeg2Builder spreadLeg2;
		protected ISOCurrencyCodeEnum spreadCurrencyLeg1;
		protected ISOCurrencyCodeEnum spreadCurrencyLeg2;
		protected String strikePriceNotation;
		protected StrikePriceAndNotation.StrikePriceAndNotationBuilder strikePrice;
		protected String strikePriceCurrency;
		protected BigDecimal optionPremiumAmount;
		protected ISOCurrencyCodeEnum optionPremiumCurrency;
		protected Date optionPremiumPaymentDate;
		protected BigDecimal exchangeRate;
		protected String exchangeRateBasis;
		protected InterestComputationMethod4Code dayCountConventionLeg1;
		protected InterestComputationMethod4Code dayCountConventionLeg2;
		protected String paymentFrequencyPeriodLeg1;
		protected String paymentFrequencyPeriodLeg2;
		protected Integer paymentFrequencyPeriodMultiplierLeg1;
		protected Integer paymentFrequencyPeriodMultiplierLeg2;
		protected ISOCurrencyCodeEnum settlementCurrencyLeg1;
		protected String settlementCurrencyLeg2;
		protected List<OtherPaymentReport.OtherPaymentReportBuilder> otherPayment = new ArrayList<>();
		protected String customBasketCode;
		protected List<BasketConstituentsReport.BasketConstituentsReportBuilder> basketConstituents = new ArrayList<>();
		protected List<ProductIdTypeEnum> basketConstituentIdentifierSource = new ArrayList<>();
		protected String underlyingIdentification;
		protected String underlierIDSource;
		protected Date maturityDateOfTheUnderlying;
		protected String identifierOfTheFloatingRateLeg2;
		protected String floatingRateReferencePeriodOfLeg2TimePeriod;
		protected Integer floatingRateReferencePeriodOfLeg2Multiplier;
		protected BigDecimal cdSIndexAttachmentPoint;
		protected BigDecimal cdSIndexDetachmentPoint;
		protected BigDecimal indexFactor;
		protected String packageIdentifier;
		protected PackageTransactionPrice.PackageTransactionPriceBuilder packageTransactionPrice;
		protected PackageTransactionSpread.PackageTransactionSpreadBuilder packageTransactionSpread;
		protected String priorUTI;
		protected String eventIdentifier;
		protected String actionType;
		protected String eventType;
		protected ZonedDateTime reportingTimestamp;
		protected String reportSubmittingEntityID;
		protected ZonedDateTime valuationTimestamp;
		protected BigDecimal valuationAmount;
		protected ISOCurrencyCodeEnum valuationCurrency;
		protected ValuationType1Code valuationMethod;
		protected BigDecimal delta;
		protected Date nextFloatingReferenceResetDateLeg1;
		protected Date nextFloatingReferenceResetDateLeg2;
		protected String smallScaleBuySideEntityIndicator;
		protected ASICNonReportable.ASICNonReportableBuilder nonReportable;
	
		public ASICTransactionReportBuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("uniqueTransactionIdentifier")
		public String getUniqueTransactionIdentifier() {
			return uniqueTransactionIdentifier;
		}
		
		@Override
		@RosettaAttribute("uniqueProductIdentifier")
		public String getUniqueProductIdentifier() {
			return uniqueProductIdentifier;
		}
		
		@Override
		@RosettaAttribute("assetClass")
		public String getAssetClass() {
			return assetClass;
		}
		
		@Override
		@RosettaAttribute("contractType")
		public String getContractType() {
			return contractType;
		}
		
		@Override
		@RosettaAttribute("reportingEntity")
		public String getReportingEntity() {
			return reportingEntity;
		}
		
		@Override
		@RosettaAttribute("counterparty1")
		public String getCounterparty1() {
			return counterparty1;
		}
		
		@Override
		@RosettaAttribute("counterparty2")
		public String getCounterparty2() {
			return counterparty2;
		}
		
		@Override
		@RosettaAttribute("counterpartyIdentifierTypeIndicator")
		public Boolean getCounterpartyIdentifierTypeIndicator() {
			return counterpartyIdentifierTypeIndicator;
		}
		
		@Override
		@RosettaAttribute("countryOfTheCounterParty2")
		public ISOCountryCodeEnum getCountryOfTheCounterParty2() {
			return countryOfTheCounterParty2;
		}
		
		@Override
		@RosettaAttribute("beneficiary1")
		public String getBeneficiary1() {
			return beneficiary1;
		}
		
		@Override
		@RosettaAttribute("beneficiary1IdentifierTypeIndicator")
		public Boolean getBeneficiary1IdentifierTypeIndicator() {
			return beneficiary1IdentifierTypeIndicator;
		}
		
		@Override
		@RosettaAttribute("broker")
		public String getBroker() {
			return broker;
		}
		
		@Override
		@RosettaAttribute("executionAgentOfTheCounterparty1")
		public String getExecutionAgentOfTheCounterparty1() {
			return executionAgentOfTheCounterparty1;
		}
		
		@Override
		@RosettaAttribute("direction")
		public String getDirection() {
			return direction;
		}
		
		@Override
		@RosettaAttribute("direction2OfLeg1")
		public String getDirection2OfLeg1() {
			return direction2OfLeg1;
		}
		
		@Override
		@RosettaAttribute("direction2OfLeg2")
		public String getDirection2OfLeg2() {
			return direction2OfLeg2;
		}
		
		@Override
		@RosettaAttribute("effectiveDate")
		public Date getEffectiveDate() {
			return effectiveDate;
		}
		
		@Override
		@RosettaAttribute("expirationDate")
		public Date getExpirationDate() {
			return expirationDate;
		}
		
		@Override
		@RosettaAttribute("executionTimestamp")
		public ZonedDateTime getExecutionTimestamp() {
			return executionTimestamp;
		}
		
		@Override
		@RosettaAttribute("eventTimestamp")
		public ZonedDateTime getEventTimestamp() {
			return eventTimestamp;
		}
		
		@Override
		@RosettaAttribute("clearingTimestamp")
		public ZonedDateTime getClearingTimestamp() {
			return clearingTimestamp;
		}
		
		@Override
		@RosettaAttribute("cleared")
		public String getCleared() {
			return cleared;
		}
		
		@Override
		@RosettaAttribute("centralCounterparty")
		public String getCentralCounterparty() {
			return centralCounterparty;
		}
		
		@Override
		@RosettaAttribute("clearingMember")
		public String getClearingMember() {
			return clearingMember;
		}
		
		@Override
		@RosettaAttribute("platformIdentifier")
		public String getPlatformIdentifier() {
			return platformIdentifier;
		}
		
		@Override
		@RosettaAttribute("notionalLeg1")
		public BigDecimal getNotionalLeg1() {
			return notionalLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalLeg2")
		public BigDecimal getNotionalLeg2() {
			return notionalLeg2;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg1")
		public BigDecimal getTotalNotionalQuantityLeg1() {
			return totalNotionalQuantityLeg1;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg2")
		public BigDecimal getTotalNotionalQuantityLeg2() {
			return totalNotionalQuantityLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityLeg1")
		public BigDecimal getNotionalQuantityLeg1() {
			return notionalQuantityLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityLeg2")
		public BigDecimal getNotionalQuantityLeg2() {
			return notionalQuantityLeg2;
		}
		
		@Override
		@RosettaAttribute("callAmount")
		public BigDecimal getCallAmount() {
			return callAmount;
		}
		
		@Override
		@RosettaAttribute("putAmount")
		public BigDecimal getPutAmount() {
			return putAmount;
		}
		
		@Override
		@RosettaAttribute("notionalCurrencyLeg1")
		public ISOCurrencyCodeEnum getNotionalCurrencyLeg1() {
			return notionalCurrencyLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalCurrencyLeg2")
		public ISOCurrencyCodeEnum getNotionalCurrencyLeg2() {
			return notionalCurrencyLeg2;
		}
		
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg1")
		public String getQuantityUnitOfMeasureLeg1() {
			return quantityUnitOfMeasureLeg1;
		}
		
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg2")
		public String getQuantityUnitOfMeasureLeg2() {
			return quantityUnitOfMeasureLeg2;
		}
		
		@Override
		@RosettaAttribute("callCurrency")
		public ISOCurrencyCodeEnum getCallCurrency() {
			return callCurrency;
		}
		
		@Override
		@RosettaAttribute("putCurrency")
		public ISOCurrencyCodeEnum getPutCurrency() {
			return putCurrency;
		}
		
		@Override
		@RosettaAttribute("notionalAmountScheduleLeg1")
		public List<? extends NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> getNotionalAmountScheduleLeg1() {
			return notionalAmountScheduleLeg1;
		}
		
		public NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder getOrCreateNotionalAmountScheduleLeg1(int _index) {
		
			if (notionalAmountScheduleLeg1==null) {
				this.notionalAmountScheduleLeg1 = new ArrayList<>();
			}
			NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder result;
			return getIndex(notionalAmountScheduleLeg1, _index, () -> {
						NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder newNotionalAmountScheduleLeg1 = NotionalAmountScheduleLeg1Report.builder();
						return newNotionalAmountScheduleLeg1;
					});
		}
		
		@Override
		@RosettaAttribute("notionalAmountScheduleLeg2")
		public List<? extends NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> getNotionalAmountScheduleLeg2() {
			return notionalAmountScheduleLeg2;
		}
		
		public NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder getOrCreateNotionalAmountScheduleLeg2(int _index) {
		
			if (notionalAmountScheduleLeg2==null) {
				this.notionalAmountScheduleLeg2 = new ArrayList<>();
			}
			NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder result;
			return getIndex(notionalAmountScheduleLeg2, _index, () -> {
						NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder newNotionalAmountScheduleLeg2 = NotionalAmountScheduleLeg2Report.builder();
						return newNotionalAmountScheduleLeg2;
					});
		}
		
		@Override
		@RosettaAttribute("priceNotation")
		public String getPriceNotation() {
			return priceNotation;
		}
		
		@Override
		@RosettaAttribute("price")
		public BigDecimal getPrice() {
			return price;
		}
		
		@Override
		@RosettaAttribute("priceCurrency")
		public ISOCurrencyCodeEnum getPriceCurrency() {
			return priceCurrency;
		}
		
		@Override
		@RosettaAttribute("priceUnitOfMeasure")
		public String getPriceUnitOfMeasure() {
			return priceUnitOfMeasure;
		}
		
		@Override
		@RosettaAttribute("fixedRateLeg1")
		public BigDecimal getFixedRateLeg1() {
			return fixedRateLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRateLeg2")
		public BigDecimal getFixedRateLeg2() {
			return fixedRateLeg2;
		}
		
		@Override
		@RosettaAttribute("spreadNotationLeg1")
		public String getSpreadNotationLeg1() {
			return spreadNotationLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadLeg1")
		public SpreadLeg1.SpreadLeg1Builder getSpreadLeg1() {
			return spreadLeg1;
		}
		
		@Override
		public SpreadLeg1.SpreadLeg1Builder getOrCreateSpreadLeg1() {
			SpreadLeg1.SpreadLeg1Builder result;
			if (spreadLeg1!=null) {
				result = spreadLeg1;
			}
			else {
				result = spreadLeg1 = SpreadLeg1.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("spreadNotationLeg2")
		public String getSpreadNotationLeg2() {
			return spreadNotationLeg2;
		}
		
		@Override
		@RosettaAttribute("spreadLeg2")
		public SpreadLeg2.SpreadLeg2Builder getSpreadLeg2() {
			return spreadLeg2;
		}
		
		@Override
		public SpreadLeg2.SpreadLeg2Builder getOrCreateSpreadLeg2() {
			SpreadLeg2.SpreadLeg2Builder result;
			if (spreadLeg2!=null) {
				result = spreadLeg2;
			}
			else {
				result = spreadLeg2 = SpreadLeg2.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("spreadCurrencyLeg1")
		public ISOCurrencyCodeEnum getSpreadCurrencyLeg1() {
			return spreadCurrencyLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadCurrencyLeg2")
		public ISOCurrencyCodeEnum getSpreadCurrencyLeg2() {
			return spreadCurrencyLeg2;
		}
		
		@Override
		@RosettaAttribute("strikePriceNotation")
		public String getStrikePriceNotation() {
			return strikePriceNotation;
		}
		
		@Override
		@RosettaAttribute("strikePrice")
		public StrikePriceAndNotation.StrikePriceAndNotationBuilder getStrikePrice() {
			return strikePrice;
		}
		
		@Override
		public StrikePriceAndNotation.StrikePriceAndNotationBuilder getOrCreateStrikePrice() {
			StrikePriceAndNotation.StrikePriceAndNotationBuilder result;
			if (strikePrice!=null) {
				result = strikePrice;
			}
			else {
				result = strikePrice = StrikePriceAndNotation.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("strikePriceCurrency")
		public String getStrikePriceCurrency() {
			return strikePriceCurrency;
		}
		
		@Override
		@RosettaAttribute("optionPremiumAmount")
		public BigDecimal getOptionPremiumAmount() {
			return optionPremiumAmount;
		}
		
		@Override
		@RosettaAttribute("optionPremiumCurrency")
		public ISOCurrencyCodeEnum getOptionPremiumCurrency() {
			return optionPremiumCurrency;
		}
		
		@Override
		@RosettaAttribute("optionPremiumPaymentDate")
		public Date getOptionPremiumPaymentDate() {
			return optionPremiumPaymentDate;
		}
		
		@Override
		@RosettaAttribute("exchangeRate")
		public BigDecimal getExchangeRate() {
			return exchangeRate;
		}
		
		@Override
		@RosettaAttribute("exchangeRateBasis")
		public String getExchangeRateBasis() {
			return exchangeRateBasis;
		}
		
		@Override
		@RosettaAttribute("dayCountConventionLeg1")
		public InterestComputationMethod4Code getDayCountConventionLeg1() {
			return dayCountConventionLeg1;
		}
		
		@Override
		@RosettaAttribute("dayCountConventionLeg2")
		public InterestComputationMethod4Code getDayCountConventionLeg2() {
			return dayCountConventionLeg2;
		}
		
		@Override
		@RosettaAttribute("paymentFrequencyPeriodLeg1")
		public String getPaymentFrequencyPeriodLeg1() {
			return paymentFrequencyPeriodLeg1;
		}
		
		@Override
		@RosettaAttribute("paymentFrequencyPeriodLeg2")
		public String getPaymentFrequencyPeriodLeg2() {
			return paymentFrequencyPeriodLeg2;
		}
		
		@Override
		@RosettaAttribute("paymentFrequencyPeriodMultiplierLeg1")
		public Integer getPaymentFrequencyPeriodMultiplierLeg1() {
			return paymentFrequencyPeriodMultiplierLeg1;
		}
		
		@Override
		@RosettaAttribute("paymentFrequencyPeriodMultiplierLeg2")
		public Integer getPaymentFrequencyPeriodMultiplierLeg2() {
			return paymentFrequencyPeriodMultiplierLeg2;
		}
		
		@Override
		@RosettaAttribute("settlementCurrencyLeg1")
		public ISOCurrencyCodeEnum getSettlementCurrencyLeg1() {
			return settlementCurrencyLeg1;
		}
		
		@Override
		@RosettaAttribute("settlementCurrencyLeg2")
		public String getSettlementCurrencyLeg2() {
			return settlementCurrencyLeg2;
		}
		
		@Override
		@RosettaAttribute("otherPayment")
		public List<? extends OtherPaymentReport.OtherPaymentReportBuilder> getOtherPayment() {
			return otherPayment;
		}
		
		public OtherPaymentReport.OtherPaymentReportBuilder getOrCreateOtherPayment(int _index) {
		
			if (otherPayment==null) {
				this.otherPayment = new ArrayList<>();
			}
			OtherPaymentReport.OtherPaymentReportBuilder result;
			return getIndex(otherPayment, _index, () -> {
						OtherPaymentReport.OtherPaymentReportBuilder newOtherPayment = OtherPaymentReport.builder();
						return newOtherPayment;
					});
		}
		
		@Override
		@RosettaAttribute("customBasketCode")
		public String getCustomBasketCode() {
			return customBasketCode;
		}
		
		@Override
		@RosettaAttribute("basketConstituents")
		public List<? extends BasketConstituentsReport.BasketConstituentsReportBuilder> getBasketConstituents() {
			return basketConstituents;
		}
		
		public BasketConstituentsReport.BasketConstituentsReportBuilder getOrCreateBasketConstituents(int _index) {
		
			if (basketConstituents==null) {
				this.basketConstituents = new ArrayList<>();
			}
			BasketConstituentsReport.BasketConstituentsReportBuilder result;
			return getIndex(basketConstituents, _index, () -> {
						BasketConstituentsReport.BasketConstituentsReportBuilder newBasketConstituents = BasketConstituentsReport.builder();
						return newBasketConstituents;
					});
		}
		
		@Override
		@RosettaAttribute("basketConstituentIdentifierSource")
		public List<ProductIdTypeEnum> getBasketConstituentIdentifierSource() {
			return basketConstituentIdentifierSource;
		}
		
		@Override
		@RosettaAttribute("underlyingIdentification")
		public String getUnderlyingIdentification() {
			return underlyingIdentification;
		}
		
		@Override
		@RosettaAttribute("underlierIDSource")
		public String getUnderlierIDSource() {
			return underlierIDSource;
		}
		
		@Override
		@RosettaAttribute("maturityDateOfTheUnderlying")
		public Date getMaturityDateOfTheUnderlying() {
			return maturityDateOfTheUnderlying;
		}
		
		@Override
		@RosettaAttribute("identifierOfTheFloatingRateLeg2")
		public String getIdentifierOfTheFloatingRateLeg2() {
			return identifierOfTheFloatingRateLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg2TimePeriod")
		public String getFloatingRateReferencePeriodOfLeg2TimePeriod() {
			return floatingRateReferencePeriodOfLeg2TimePeriod;
		}
		
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg2Multiplier")
		public Integer getFloatingRateReferencePeriodOfLeg2Multiplier() {
			return floatingRateReferencePeriodOfLeg2Multiplier;
		}
		
		@Override
		@RosettaAttribute("cdSIndexAttachmentPoint")
		public BigDecimal getCdSIndexAttachmentPoint() {
			return cdSIndexAttachmentPoint;
		}
		
		@Override
		@RosettaAttribute("cdSIndexDetachmentPoint")
		public BigDecimal getCdSIndexDetachmentPoint() {
			return cdSIndexDetachmentPoint;
		}
		
		@Override
		@RosettaAttribute("indexFactor")
		public BigDecimal getIndexFactor() {
			return indexFactor;
		}
		
		@Override
		@RosettaAttribute("packageIdentifier")
		public String getPackageIdentifier() {
			return packageIdentifier;
		}
		
		@Override
		@RosettaAttribute("packageTransactionPrice")
		public PackageTransactionPrice.PackageTransactionPriceBuilder getPackageTransactionPrice() {
			return packageTransactionPrice;
		}
		
		@Override
		public PackageTransactionPrice.PackageTransactionPriceBuilder getOrCreatePackageTransactionPrice() {
			PackageTransactionPrice.PackageTransactionPriceBuilder result;
			if (packageTransactionPrice!=null) {
				result = packageTransactionPrice;
			}
			else {
				result = packageTransactionPrice = PackageTransactionPrice.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("packageTransactionSpread")
		public PackageTransactionSpread.PackageTransactionSpreadBuilder getPackageTransactionSpread() {
			return packageTransactionSpread;
		}
		
		@Override
		public PackageTransactionSpread.PackageTransactionSpreadBuilder getOrCreatePackageTransactionSpread() {
			PackageTransactionSpread.PackageTransactionSpreadBuilder result;
			if (packageTransactionSpread!=null) {
				result = packageTransactionSpread;
			}
			else {
				result = packageTransactionSpread = PackageTransactionSpread.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("priorUTI")
		public String getPriorUTI() {
			return priorUTI;
		}
		
		@Override
		@RosettaAttribute("eventIdentifier")
		public String getEventIdentifier() {
			return eventIdentifier;
		}
		
		@Override
		@RosettaAttribute("actionType")
		public String getActionType() {
			return actionType;
		}
		
		@Override
		@RosettaAttribute("eventType")
		public String getEventType() {
			return eventType;
		}
		
		@Override
		@RosettaAttribute("reportingTimestamp")
		public ZonedDateTime getReportingTimestamp() {
			return reportingTimestamp;
		}
		
		@Override
		@RosettaAttribute("reportSubmittingEntityID")
		public String getReportSubmittingEntityID() {
			return reportSubmittingEntityID;
		}
		
		@Override
		@RosettaAttribute("valuationTimestamp")
		public ZonedDateTime getValuationTimestamp() {
			return valuationTimestamp;
		}
		
		@Override
		@RosettaAttribute("valuationAmount")
		public BigDecimal getValuationAmount() {
			return valuationAmount;
		}
		
		@Override
		@RosettaAttribute("valuationCurrency")
		public ISOCurrencyCodeEnum getValuationCurrency() {
			return valuationCurrency;
		}
		
		@Override
		@RosettaAttribute("valuationMethod")
		public ValuationType1Code getValuationMethod() {
			return valuationMethod;
		}
		
		@Override
		@RosettaAttribute("delta")
		public BigDecimal getDelta() {
			return delta;
		}
		
		@Override
		@RosettaAttribute("nextFloatingReferenceResetDateLeg1")
		public Date getNextFloatingReferenceResetDateLeg1() {
			return nextFloatingReferenceResetDateLeg1;
		}
		
		@Override
		@RosettaAttribute("nextFloatingReferenceResetDateLeg2")
		public Date getNextFloatingReferenceResetDateLeg2() {
			return nextFloatingReferenceResetDateLeg2;
		}
		
		@Override
		@RosettaAttribute("smallScaleBuySideEntityIndicator")
		public String getSmallScaleBuySideEntityIndicator() {
			return smallScaleBuySideEntityIndicator;
		}
		
		@Override
		@RosettaAttribute("nonReportable")
		public ASICNonReportable.ASICNonReportableBuilder getNonReportable() {
			return nonReportable;
		}
		
		@Override
		public ASICNonReportable.ASICNonReportableBuilder getOrCreateNonReportable() {
			ASICNonReportable.ASICNonReportableBuilder result;
			if (nonReportable!=null) {
				result = nonReportable;
			}
			else {
				result = nonReportable = ASICNonReportable.builder();
			}
			
			return result;
		}
	
		@Override
		@RosettaAttribute("uniqueTransactionIdentifier")
		public ASICTransactionReport.ASICTransactionReportBuilder setUniqueTransactionIdentifier(String uniqueTransactionIdentifier) {
			this.uniqueTransactionIdentifier = uniqueTransactionIdentifier==null?null:uniqueTransactionIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("uniqueProductIdentifier")
		public ASICTransactionReport.ASICTransactionReportBuilder setUniqueProductIdentifier(String uniqueProductIdentifier) {
			this.uniqueProductIdentifier = uniqueProductIdentifier==null?null:uniqueProductIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("assetClass")
		public ASICTransactionReport.ASICTransactionReportBuilder setAssetClass(String assetClass) {
			this.assetClass = assetClass==null?null:assetClass;
			return this;
		}
		@Override
		@RosettaAttribute("contractType")
		public ASICTransactionReport.ASICTransactionReportBuilder setContractType(String contractType) {
			this.contractType = contractType==null?null:contractType;
			return this;
		}
		@Override
		@RosettaAttribute("reportingEntity")
		public ASICTransactionReport.ASICTransactionReportBuilder setReportingEntity(String reportingEntity) {
			this.reportingEntity = reportingEntity==null?null:reportingEntity;
			return this;
		}
		@Override
		@RosettaAttribute("counterparty1")
		public ASICTransactionReport.ASICTransactionReportBuilder setCounterparty1(String counterparty1) {
			this.counterparty1 = counterparty1==null?null:counterparty1;
			return this;
		}
		@Override
		@RosettaAttribute("counterparty2")
		public ASICTransactionReport.ASICTransactionReportBuilder setCounterparty2(String counterparty2) {
			this.counterparty2 = counterparty2==null?null:counterparty2;
			return this;
		}
		@Override
		@RosettaAttribute("counterpartyIdentifierTypeIndicator")
		public ASICTransactionReport.ASICTransactionReportBuilder setCounterpartyIdentifierTypeIndicator(Boolean counterpartyIdentifierTypeIndicator) {
			this.counterpartyIdentifierTypeIndicator = counterpartyIdentifierTypeIndicator==null?null:counterpartyIdentifierTypeIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("countryOfTheCounterParty2")
		public ASICTransactionReport.ASICTransactionReportBuilder setCountryOfTheCounterParty2(ISOCountryCodeEnum countryOfTheCounterParty2) {
			this.countryOfTheCounterParty2 = countryOfTheCounterParty2==null?null:countryOfTheCounterParty2;
			return this;
		}
		@Override
		@RosettaAttribute("beneficiary1")
		public ASICTransactionReport.ASICTransactionReportBuilder setBeneficiary1(String beneficiary1) {
			this.beneficiary1 = beneficiary1==null?null:beneficiary1;
			return this;
		}
		@Override
		@RosettaAttribute("beneficiary1IdentifierTypeIndicator")
		public ASICTransactionReport.ASICTransactionReportBuilder setBeneficiary1IdentifierTypeIndicator(Boolean beneficiary1IdentifierTypeIndicator) {
			this.beneficiary1IdentifierTypeIndicator = beneficiary1IdentifierTypeIndicator==null?null:beneficiary1IdentifierTypeIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("broker")
		public ASICTransactionReport.ASICTransactionReportBuilder setBroker(String broker) {
			this.broker = broker==null?null:broker;
			return this;
		}
		@Override
		@RosettaAttribute("executionAgentOfTheCounterparty1")
		public ASICTransactionReport.ASICTransactionReportBuilder setExecutionAgentOfTheCounterparty1(String executionAgentOfTheCounterparty1) {
			this.executionAgentOfTheCounterparty1 = executionAgentOfTheCounterparty1==null?null:executionAgentOfTheCounterparty1;
			return this;
		}
		@Override
		@RosettaAttribute("direction")
		public ASICTransactionReport.ASICTransactionReportBuilder setDirection(String direction) {
			this.direction = direction==null?null:direction;
			return this;
		}
		@Override
		@RosettaAttribute("direction2OfLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setDirection2OfLeg1(String direction2OfLeg1) {
			this.direction2OfLeg1 = direction2OfLeg1==null?null:direction2OfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("direction2OfLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setDirection2OfLeg2(String direction2OfLeg2) {
			this.direction2OfLeg2 = direction2OfLeg2==null?null:direction2OfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("effectiveDate")
		public ASICTransactionReport.ASICTransactionReportBuilder setEffectiveDate(Date effectiveDate) {
			this.effectiveDate = effectiveDate==null?null:effectiveDate;
			return this;
		}
		@Override
		@RosettaAttribute("expirationDate")
		public ASICTransactionReport.ASICTransactionReportBuilder setExpirationDate(Date expirationDate) {
			this.expirationDate = expirationDate==null?null:expirationDate;
			return this;
		}
		@Override
		@RosettaAttribute("executionTimestamp")
		public ASICTransactionReport.ASICTransactionReportBuilder setExecutionTimestamp(ZonedDateTime executionTimestamp) {
			this.executionTimestamp = executionTimestamp==null?null:executionTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("eventTimestamp")
		public ASICTransactionReport.ASICTransactionReportBuilder setEventTimestamp(ZonedDateTime eventTimestamp) {
			this.eventTimestamp = eventTimestamp==null?null:eventTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("clearingTimestamp")
		public ASICTransactionReport.ASICTransactionReportBuilder setClearingTimestamp(ZonedDateTime clearingTimestamp) {
			this.clearingTimestamp = clearingTimestamp==null?null:clearingTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("cleared")
		public ASICTransactionReport.ASICTransactionReportBuilder setCleared(String cleared) {
			this.cleared = cleared==null?null:cleared;
			return this;
		}
		@Override
		@RosettaAttribute("centralCounterparty")
		public ASICTransactionReport.ASICTransactionReportBuilder setCentralCounterparty(String centralCounterparty) {
			this.centralCounterparty = centralCounterparty==null?null:centralCounterparty;
			return this;
		}
		@Override
		@RosettaAttribute("clearingMember")
		public ASICTransactionReport.ASICTransactionReportBuilder setClearingMember(String clearingMember) {
			this.clearingMember = clearingMember==null?null:clearingMember;
			return this;
		}
		@Override
		@RosettaAttribute("platformIdentifier")
		public ASICTransactionReport.ASICTransactionReportBuilder setPlatformIdentifier(String platformIdentifier) {
			this.platformIdentifier = platformIdentifier==null?null:platformIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("notionalLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setNotionalLeg1(BigDecimal notionalLeg1) {
			this.notionalLeg1 = notionalLeg1==null?null:notionalLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("notionalLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setNotionalLeg2(BigDecimal notionalLeg2) {
			this.notionalLeg2 = notionalLeg2==null?null:notionalLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setTotalNotionalQuantityLeg1(BigDecimal totalNotionalQuantityLeg1) {
			this.totalNotionalQuantityLeg1 = totalNotionalQuantityLeg1==null?null:totalNotionalQuantityLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setTotalNotionalQuantityLeg2(BigDecimal totalNotionalQuantityLeg2) {
			this.totalNotionalQuantityLeg2 = totalNotionalQuantityLeg2==null?null:totalNotionalQuantityLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("notionalQuantityLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setNotionalQuantityLeg1(BigDecimal notionalQuantityLeg1) {
			this.notionalQuantityLeg1 = notionalQuantityLeg1==null?null:notionalQuantityLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("notionalQuantityLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setNotionalQuantityLeg2(BigDecimal notionalQuantityLeg2) {
			this.notionalQuantityLeg2 = notionalQuantityLeg2==null?null:notionalQuantityLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("callAmount")
		public ASICTransactionReport.ASICTransactionReportBuilder setCallAmount(BigDecimal callAmount) {
			this.callAmount = callAmount==null?null:callAmount;
			return this;
		}
		@Override
		@RosettaAttribute("putAmount")
		public ASICTransactionReport.ASICTransactionReportBuilder setPutAmount(BigDecimal putAmount) {
			this.putAmount = putAmount==null?null:putAmount;
			return this;
		}
		@Override
		@RosettaAttribute("notionalCurrencyLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setNotionalCurrencyLeg1(ISOCurrencyCodeEnum notionalCurrencyLeg1) {
			this.notionalCurrencyLeg1 = notionalCurrencyLeg1==null?null:notionalCurrencyLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("notionalCurrencyLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setNotionalCurrencyLeg2(ISOCurrencyCodeEnum notionalCurrencyLeg2) {
			this.notionalCurrencyLeg2 = notionalCurrencyLeg2==null?null:notionalCurrencyLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setQuantityUnitOfMeasureLeg1(String quantityUnitOfMeasureLeg1) {
			this.quantityUnitOfMeasureLeg1 = quantityUnitOfMeasureLeg1==null?null:quantityUnitOfMeasureLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setQuantityUnitOfMeasureLeg2(String quantityUnitOfMeasureLeg2) {
			this.quantityUnitOfMeasureLeg2 = quantityUnitOfMeasureLeg2==null?null:quantityUnitOfMeasureLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("callCurrency")
		public ASICTransactionReport.ASICTransactionReportBuilder setCallCurrency(ISOCurrencyCodeEnum callCurrency) {
			this.callCurrency = callCurrency==null?null:callCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("putCurrency")
		public ASICTransactionReport.ASICTransactionReportBuilder setPutCurrency(ISOCurrencyCodeEnum putCurrency) {
			this.putCurrency = putCurrency==null?null:putCurrency;
			return this;
		}
		@Override
		public ASICTransactionReport.ASICTransactionReportBuilder addNotionalAmountScheduleLeg1(NotionalAmountScheduleLeg1Report notionalAmountScheduleLeg1) {
			if (notionalAmountScheduleLeg1!=null) this.notionalAmountScheduleLeg1.add(notionalAmountScheduleLeg1.toBuilder());
			return this;
		}
		
		@Override
		public ASICTransactionReport.ASICTransactionReportBuilder addNotionalAmountScheduleLeg1(NotionalAmountScheduleLeg1Report notionalAmountScheduleLeg1, int _idx) {
			getIndex(this.notionalAmountScheduleLeg1, _idx, () -> notionalAmountScheduleLeg1.toBuilder());
			return this;
		}
		@Override 
		public ASICTransactionReport.ASICTransactionReportBuilder addNotionalAmountScheduleLeg1(List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg1s) {
			if (notionalAmountScheduleLeg1s != null) {
				for (NotionalAmountScheduleLeg1Report toAdd : notionalAmountScheduleLeg1s) {
					this.notionalAmountScheduleLeg1.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("notionalAmountScheduleLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setNotionalAmountScheduleLeg1(List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg1s) {
			if (notionalAmountScheduleLeg1s == null)  {
				this.notionalAmountScheduleLeg1 = new ArrayList<>();
			}
			else {
				this.notionalAmountScheduleLeg1 = notionalAmountScheduleLeg1s.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		public ASICTransactionReport.ASICTransactionReportBuilder addNotionalAmountScheduleLeg2(NotionalAmountScheduleLeg2Report notionalAmountScheduleLeg2) {
			if (notionalAmountScheduleLeg2!=null) this.notionalAmountScheduleLeg2.add(notionalAmountScheduleLeg2.toBuilder());
			return this;
		}
		
		@Override
		public ASICTransactionReport.ASICTransactionReportBuilder addNotionalAmountScheduleLeg2(NotionalAmountScheduleLeg2Report notionalAmountScheduleLeg2, int _idx) {
			getIndex(this.notionalAmountScheduleLeg2, _idx, () -> notionalAmountScheduleLeg2.toBuilder());
			return this;
		}
		@Override 
		public ASICTransactionReport.ASICTransactionReportBuilder addNotionalAmountScheduleLeg2(List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg2s) {
			if (notionalAmountScheduleLeg2s != null) {
				for (NotionalAmountScheduleLeg2Report toAdd : notionalAmountScheduleLeg2s) {
					this.notionalAmountScheduleLeg2.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("notionalAmountScheduleLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setNotionalAmountScheduleLeg2(List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg2s) {
			if (notionalAmountScheduleLeg2s == null)  {
				this.notionalAmountScheduleLeg2 = new ArrayList<>();
			}
			else {
				this.notionalAmountScheduleLeg2 = notionalAmountScheduleLeg2s.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("priceNotation")
		public ASICTransactionReport.ASICTransactionReportBuilder setPriceNotation(String priceNotation) {
			this.priceNotation = priceNotation==null?null:priceNotation;
			return this;
		}
		@Override
		@RosettaAttribute("price")
		public ASICTransactionReport.ASICTransactionReportBuilder setPrice(BigDecimal price) {
			this.price = price==null?null:price;
			return this;
		}
		@Override
		@RosettaAttribute("priceCurrency")
		public ASICTransactionReport.ASICTransactionReportBuilder setPriceCurrency(ISOCurrencyCodeEnum priceCurrency) {
			this.priceCurrency = priceCurrency==null?null:priceCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("priceUnitOfMeasure")
		public ASICTransactionReport.ASICTransactionReportBuilder setPriceUnitOfMeasure(String priceUnitOfMeasure) {
			this.priceUnitOfMeasure = priceUnitOfMeasure==null?null:priceUnitOfMeasure;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRateLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setFixedRateLeg1(BigDecimal fixedRateLeg1) {
			this.fixedRateLeg1 = fixedRateLeg1==null?null:fixedRateLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRateLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setFixedRateLeg2(BigDecimal fixedRateLeg2) {
			this.fixedRateLeg2 = fixedRateLeg2==null?null:fixedRateLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("spreadNotationLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setSpreadNotationLeg1(String spreadNotationLeg1) {
			this.spreadNotationLeg1 = spreadNotationLeg1==null?null:spreadNotationLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("spreadLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setSpreadLeg1(SpreadLeg1 spreadLeg1) {
			this.spreadLeg1 = spreadLeg1==null?null:spreadLeg1.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("spreadNotationLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setSpreadNotationLeg2(String spreadNotationLeg2) {
			this.spreadNotationLeg2 = spreadNotationLeg2==null?null:spreadNotationLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("spreadLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setSpreadLeg2(SpreadLeg2 spreadLeg2) {
			this.spreadLeg2 = spreadLeg2==null?null:spreadLeg2.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("spreadCurrencyLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setSpreadCurrencyLeg1(ISOCurrencyCodeEnum spreadCurrencyLeg1) {
			this.spreadCurrencyLeg1 = spreadCurrencyLeg1==null?null:spreadCurrencyLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("spreadCurrencyLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setSpreadCurrencyLeg2(ISOCurrencyCodeEnum spreadCurrencyLeg2) {
			this.spreadCurrencyLeg2 = spreadCurrencyLeg2==null?null:spreadCurrencyLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("strikePriceNotation")
		public ASICTransactionReport.ASICTransactionReportBuilder setStrikePriceNotation(String strikePriceNotation) {
			this.strikePriceNotation = strikePriceNotation==null?null:strikePriceNotation;
			return this;
		}
		@Override
		@RosettaAttribute("strikePrice")
		public ASICTransactionReport.ASICTransactionReportBuilder setStrikePrice(StrikePriceAndNotation strikePrice) {
			this.strikePrice = strikePrice==null?null:strikePrice.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("strikePriceCurrency")
		public ASICTransactionReport.ASICTransactionReportBuilder setStrikePriceCurrency(String strikePriceCurrency) {
			this.strikePriceCurrency = strikePriceCurrency==null?null:strikePriceCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("optionPremiumAmount")
		public ASICTransactionReport.ASICTransactionReportBuilder setOptionPremiumAmount(BigDecimal optionPremiumAmount) {
			this.optionPremiumAmount = optionPremiumAmount==null?null:optionPremiumAmount;
			return this;
		}
		@Override
		@RosettaAttribute("optionPremiumCurrency")
		public ASICTransactionReport.ASICTransactionReportBuilder setOptionPremiumCurrency(ISOCurrencyCodeEnum optionPremiumCurrency) {
			this.optionPremiumCurrency = optionPremiumCurrency==null?null:optionPremiumCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("optionPremiumPaymentDate")
		public ASICTransactionReport.ASICTransactionReportBuilder setOptionPremiumPaymentDate(Date optionPremiumPaymentDate) {
			this.optionPremiumPaymentDate = optionPremiumPaymentDate==null?null:optionPremiumPaymentDate;
			return this;
		}
		@Override
		@RosettaAttribute("exchangeRate")
		public ASICTransactionReport.ASICTransactionReportBuilder setExchangeRate(BigDecimal exchangeRate) {
			this.exchangeRate = exchangeRate==null?null:exchangeRate;
			return this;
		}
		@Override
		@RosettaAttribute("exchangeRateBasis")
		public ASICTransactionReport.ASICTransactionReportBuilder setExchangeRateBasis(String exchangeRateBasis) {
			this.exchangeRateBasis = exchangeRateBasis==null?null:exchangeRateBasis;
			return this;
		}
		@Override
		@RosettaAttribute("dayCountConventionLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setDayCountConventionLeg1(InterestComputationMethod4Code dayCountConventionLeg1) {
			this.dayCountConventionLeg1 = dayCountConventionLeg1==null?null:dayCountConventionLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("dayCountConventionLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setDayCountConventionLeg2(InterestComputationMethod4Code dayCountConventionLeg2) {
			this.dayCountConventionLeg2 = dayCountConventionLeg2==null?null:dayCountConventionLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("paymentFrequencyPeriodLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setPaymentFrequencyPeriodLeg1(String paymentFrequencyPeriodLeg1) {
			this.paymentFrequencyPeriodLeg1 = paymentFrequencyPeriodLeg1==null?null:paymentFrequencyPeriodLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("paymentFrequencyPeriodLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setPaymentFrequencyPeriodLeg2(String paymentFrequencyPeriodLeg2) {
			this.paymentFrequencyPeriodLeg2 = paymentFrequencyPeriodLeg2==null?null:paymentFrequencyPeriodLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("paymentFrequencyPeriodMultiplierLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setPaymentFrequencyPeriodMultiplierLeg1(Integer paymentFrequencyPeriodMultiplierLeg1) {
			this.paymentFrequencyPeriodMultiplierLeg1 = paymentFrequencyPeriodMultiplierLeg1==null?null:paymentFrequencyPeriodMultiplierLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("paymentFrequencyPeriodMultiplierLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setPaymentFrequencyPeriodMultiplierLeg2(Integer paymentFrequencyPeriodMultiplierLeg2) {
			this.paymentFrequencyPeriodMultiplierLeg2 = paymentFrequencyPeriodMultiplierLeg2==null?null:paymentFrequencyPeriodMultiplierLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("settlementCurrencyLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setSettlementCurrencyLeg1(ISOCurrencyCodeEnum settlementCurrencyLeg1) {
			this.settlementCurrencyLeg1 = settlementCurrencyLeg1==null?null:settlementCurrencyLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("settlementCurrencyLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setSettlementCurrencyLeg2(String settlementCurrencyLeg2) {
			this.settlementCurrencyLeg2 = settlementCurrencyLeg2==null?null:settlementCurrencyLeg2;
			return this;
		}
		@Override
		public ASICTransactionReport.ASICTransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment) {
			if (otherPayment!=null) this.otherPayment.add(otherPayment.toBuilder());
			return this;
		}
		
		@Override
		public ASICTransactionReport.ASICTransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment, int _idx) {
			getIndex(this.otherPayment, _idx, () -> otherPayment.toBuilder());
			return this;
		}
		@Override 
		public ASICTransactionReport.ASICTransactionReportBuilder addOtherPayment(List<? extends OtherPaymentReport> otherPayments) {
			if (otherPayments != null) {
				for (OtherPaymentReport toAdd : otherPayments) {
					this.otherPayment.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("otherPayment")
		public ASICTransactionReport.ASICTransactionReportBuilder setOtherPayment(List<? extends OtherPaymentReport> otherPayments) {
			if (otherPayments == null)  {
				this.otherPayment = new ArrayList<>();
			}
			else {
				this.otherPayment = otherPayments.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("customBasketCode")
		public ASICTransactionReport.ASICTransactionReportBuilder setCustomBasketCode(String customBasketCode) {
			this.customBasketCode = customBasketCode==null?null:customBasketCode;
			return this;
		}
		@Override
		public ASICTransactionReport.ASICTransactionReportBuilder addBasketConstituents(BasketConstituentsReport basketConstituents) {
			if (basketConstituents!=null) this.basketConstituents.add(basketConstituents.toBuilder());
			return this;
		}
		
		@Override
		public ASICTransactionReport.ASICTransactionReportBuilder addBasketConstituents(BasketConstituentsReport basketConstituents, int _idx) {
			getIndex(this.basketConstituents, _idx, () -> basketConstituents.toBuilder());
			return this;
		}
		@Override 
		public ASICTransactionReport.ASICTransactionReportBuilder addBasketConstituents(List<? extends BasketConstituentsReport> basketConstituentss) {
			if (basketConstituentss != null) {
				for (BasketConstituentsReport toAdd : basketConstituentss) {
					this.basketConstituents.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("basketConstituents")
		public ASICTransactionReport.ASICTransactionReportBuilder setBasketConstituents(List<? extends BasketConstituentsReport> basketConstituentss) {
			if (basketConstituentss == null)  {
				this.basketConstituents = new ArrayList<>();
			}
			else {
				this.basketConstituents = basketConstituentss.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		public ASICTransactionReport.ASICTransactionReportBuilder addBasketConstituentIdentifierSource(ProductIdTypeEnum basketConstituentIdentifierSource) {
			if (basketConstituentIdentifierSource!=null) this.basketConstituentIdentifierSource.add(basketConstituentIdentifierSource);
			return this;
		}
		
		@Override
		public ASICTransactionReport.ASICTransactionReportBuilder addBasketConstituentIdentifierSource(ProductIdTypeEnum basketConstituentIdentifierSource, int _idx) {
			getIndex(this.basketConstituentIdentifierSource, _idx, () -> basketConstituentIdentifierSource);
			return this;
		}
		@Override 
		public ASICTransactionReport.ASICTransactionReportBuilder addBasketConstituentIdentifierSource(List<? extends ProductIdTypeEnum> basketConstituentIdentifierSources) {
			if (basketConstituentIdentifierSources != null) {
				for (ProductIdTypeEnum toAdd : basketConstituentIdentifierSources) {
					this.basketConstituentIdentifierSource.add(toAdd);
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("basketConstituentIdentifierSource")
		public ASICTransactionReport.ASICTransactionReportBuilder setBasketConstituentIdentifierSource(List<? extends ProductIdTypeEnum> basketConstituentIdentifierSources) {
			if (basketConstituentIdentifierSources == null)  {
				this.basketConstituentIdentifierSource = new ArrayList<>();
			}
			else {
				this.basketConstituentIdentifierSource = basketConstituentIdentifierSources.stream()
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("underlyingIdentification")
		public ASICTransactionReport.ASICTransactionReportBuilder setUnderlyingIdentification(String underlyingIdentification) {
			this.underlyingIdentification = underlyingIdentification==null?null:underlyingIdentification;
			return this;
		}
		@Override
		@RosettaAttribute("underlierIDSource")
		public ASICTransactionReport.ASICTransactionReportBuilder setUnderlierIDSource(String underlierIDSource) {
			this.underlierIDSource = underlierIDSource==null?null:underlierIDSource;
			return this;
		}
		@Override
		@RosettaAttribute("maturityDateOfTheUnderlying")
		public ASICTransactionReport.ASICTransactionReportBuilder setMaturityDateOfTheUnderlying(Date maturityDateOfTheUnderlying) {
			this.maturityDateOfTheUnderlying = maturityDateOfTheUnderlying==null?null:maturityDateOfTheUnderlying;
			return this;
		}
		@Override
		@RosettaAttribute("identifierOfTheFloatingRateLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setIdentifierOfTheFloatingRateLeg2(String identifierOfTheFloatingRateLeg2) {
			this.identifierOfTheFloatingRateLeg2 = identifierOfTheFloatingRateLeg2==null?null:identifierOfTheFloatingRateLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg2TimePeriod")
		public ASICTransactionReport.ASICTransactionReportBuilder setFloatingRateReferencePeriodOfLeg2TimePeriod(String floatingRateReferencePeriodOfLeg2TimePeriod) {
			this.floatingRateReferencePeriodOfLeg2TimePeriod = floatingRateReferencePeriodOfLeg2TimePeriod==null?null:floatingRateReferencePeriodOfLeg2TimePeriod;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg2Multiplier")
		public ASICTransactionReport.ASICTransactionReportBuilder setFloatingRateReferencePeriodOfLeg2Multiplier(Integer floatingRateReferencePeriodOfLeg2Multiplier) {
			this.floatingRateReferencePeriodOfLeg2Multiplier = floatingRateReferencePeriodOfLeg2Multiplier==null?null:floatingRateReferencePeriodOfLeg2Multiplier;
			return this;
		}
		@Override
		@RosettaAttribute("cdSIndexAttachmentPoint")
		public ASICTransactionReport.ASICTransactionReportBuilder setCdSIndexAttachmentPoint(BigDecimal cdSIndexAttachmentPoint) {
			this.cdSIndexAttachmentPoint = cdSIndexAttachmentPoint==null?null:cdSIndexAttachmentPoint;
			return this;
		}
		@Override
		@RosettaAttribute("cdSIndexDetachmentPoint")
		public ASICTransactionReport.ASICTransactionReportBuilder setCdSIndexDetachmentPoint(BigDecimal cdSIndexDetachmentPoint) {
			this.cdSIndexDetachmentPoint = cdSIndexDetachmentPoint==null?null:cdSIndexDetachmentPoint;
			return this;
		}
		@Override
		@RosettaAttribute("indexFactor")
		public ASICTransactionReport.ASICTransactionReportBuilder setIndexFactor(BigDecimal indexFactor) {
			this.indexFactor = indexFactor==null?null:indexFactor;
			return this;
		}
		@Override
		@RosettaAttribute("packageIdentifier")
		public ASICTransactionReport.ASICTransactionReportBuilder setPackageIdentifier(String packageIdentifier) {
			this.packageIdentifier = packageIdentifier==null?null:packageIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("packageTransactionPrice")
		public ASICTransactionReport.ASICTransactionReportBuilder setPackageTransactionPrice(PackageTransactionPrice packageTransactionPrice) {
			this.packageTransactionPrice = packageTransactionPrice==null?null:packageTransactionPrice.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("packageTransactionSpread")
		public ASICTransactionReport.ASICTransactionReportBuilder setPackageTransactionSpread(PackageTransactionSpread packageTransactionSpread) {
			this.packageTransactionSpread = packageTransactionSpread==null?null:packageTransactionSpread.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("priorUTI")
		public ASICTransactionReport.ASICTransactionReportBuilder setPriorUTI(String priorUTI) {
			this.priorUTI = priorUTI==null?null:priorUTI;
			return this;
		}
		@Override
		@RosettaAttribute("eventIdentifier")
		public ASICTransactionReport.ASICTransactionReportBuilder setEventIdentifier(String eventIdentifier) {
			this.eventIdentifier = eventIdentifier==null?null:eventIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("actionType")
		public ASICTransactionReport.ASICTransactionReportBuilder setActionType(String actionType) {
			this.actionType = actionType==null?null:actionType;
			return this;
		}
		@Override
		@RosettaAttribute("eventType")
		public ASICTransactionReport.ASICTransactionReportBuilder setEventType(String eventType) {
			this.eventType = eventType==null?null:eventType;
			return this;
		}
		@Override
		@RosettaAttribute("reportingTimestamp")
		public ASICTransactionReport.ASICTransactionReportBuilder setReportingTimestamp(ZonedDateTime reportingTimestamp) {
			this.reportingTimestamp = reportingTimestamp==null?null:reportingTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("reportSubmittingEntityID")
		public ASICTransactionReport.ASICTransactionReportBuilder setReportSubmittingEntityID(String reportSubmittingEntityID) {
			this.reportSubmittingEntityID = reportSubmittingEntityID==null?null:reportSubmittingEntityID;
			return this;
		}
		@Override
		@RosettaAttribute("valuationTimestamp")
		public ASICTransactionReport.ASICTransactionReportBuilder setValuationTimestamp(ZonedDateTime valuationTimestamp) {
			this.valuationTimestamp = valuationTimestamp==null?null:valuationTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("valuationAmount")
		public ASICTransactionReport.ASICTransactionReportBuilder setValuationAmount(BigDecimal valuationAmount) {
			this.valuationAmount = valuationAmount==null?null:valuationAmount;
			return this;
		}
		@Override
		@RosettaAttribute("valuationCurrency")
		public ASICTransactionReport.ASICTransactionReportBuilder setValuationCurrency(ISOCurrencyCodeEnum valuationCurrency) {
			this.valuationCurrency = valuationCurrency==null?null:valuationCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("valuationMethod")
		public ASICTransactionReport.ASICTransactionReportBuilder setValuationMethod(ValuationType1Code valuationMethod) {
			this.valuationMethod = valuationMethod==null?null:valuationMethod;
			return this;
		}
		@Override
		@RosettaAttribute("delta")
		public ASICTransactionReport.ASICTransactionReportBuilder setDelta(BigDecimal delta) {
			this.delta = delta==null?null:delta;
			return this;
		}
		@Override
		@RosettaAttribute("nextFloatingReferenceResetDateLeg1")
		public ASICTransactionReport.ASICTransactionReportBuilder setNextFloatingReferenceResetDateLeg1(Date nextFloatingReferenceResetDateLeg1) {
			this.nextFloatingReferenceResetDateLeg1 = nextFloatingReferenceResetDateLeg1==null?null:nextFloatingReferenceResetDateLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("nextFloatingReferenceResetDateLeg2")
		public ASICTransactionReport.ASICTransactionReportBuilder setNextFloatingReferenceResetDateLeg2(Date nextFloatingReferenceResetDateLeg2) {
			this.nextFloatingReferenceResetDateLeg2 = nextFloatingReferenceResetDateLeg2==null?null:nextFloatingReferenceResetDateLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("smallScaleBuySideEntityIndicator")
		public ASICTransactionReport.ASICTransactionReportBuilder setSmallScaleBuySideEntityIndicator(String smallScaleBuySideEntityIndicator) {
			this.smallScaleBuySideEntityIndicator = smallScaleBuySideEntityIndicator==null?null:smallScaleBuySideEntityIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("nonReportable")
		public ASICTransactionReport.ASICTransactionReportBuilder setNonReportable(ASICNonReportable nonReportable) {
			this.nonReportable = nonReportable==null?null:nonReportable.toBuilder();
			return this;
		}
		
		@Override
		public ASICTransactionReport build() {
			return new ASICTransactionReport.ASICTransactionReportImpl(this);
		}
		
		@Override
		public ASICTransactionReport.ASICTransactionReportBuilder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public ASICTransactionReport.ASICTransactionReportBuilder prune() {
			notionalAmountScheduleLeg1 = notionalAmountScheduleLeg1.stream().filter(b->b!=null).<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			notionalAmountScheduleLeg2 = notionalAmountScheduleLeg2.stream().filter(b->b!=null).<NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (spreadLeg1!=null && !spreadLeg1.prune().hasData()) spreadLeg1 = null;
			if (spreadLeg2!=null && !spreadLeg2.prune().hasData()) spreadLeg2 = null;
			if (strikePrice!=null && !strikePrice.prune().hasData()) strikePrice = null;
			otherPayment = otherPayment.stream().filter(b->b!=null).<OtherPaymentReport.OtherPaymentReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			basketConstituents = basketConstituents.stream().filter(b->b!=null).<BasketConstituentsReport.BasketConstituentsReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (packageTransactionPrice!=null && !packageTransactionPrice.prune().hasData()) packageTransactionPrice = null;
			if (packageTransactionSpread!=null && !packageTransactionSpread.prune().hasData()) packageTransactionSpread = null;
			if (nonReportable!=null && !nonReportable.prune().hasData()) nonReportable = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getUniqueTransactionIdentifier()!=null) return true;
			if (getUniqueProductIdentifier()!=null) return true;
			if (getAssetClass()!=null) return true;
			if (getContractType()!=null) return true;
			if (getReportingEntity()!=null) return true;
			if (getCounterparty1()!=null) return true;
			if (getCounterparty2()!=null) return true;
			if (getCounterpartyIdentifierTypeIndicator()!=null) return true;
			if (getCountryOfTheCounterParty2()!=null) return true;
			if (getBeneficiary1()!=null) return true;
			if (getBeneficiary1IdentifierTypeIndicator()!=null) return true;
			if (getBroker()!=null) return true;
			if (getExecutionAgentOfTheCounterparty1()!=null) return true;
			if (getDirection()!=null) return true;
			if (getDirection2OfLeg1()!=null) return true;
			if (getDirection2OfLeg2()!=null) return true;
			if (getEffectiveDate()!=null) return true;
			if (getExpirationDate()!=null) return true;
			if (getExecutionTimestamp()!=null) return true;
			if (getEventTimestamp()!=null) return true;
			if (getClearingTimestamp()!=null) return true;
			if (getCleared()!=null) return true;
			if (getCentralCounterparty()!=null) return true;
			if (getClearingMember()!=null) return true;
			if (getPlatformIdentifier()!=null) return true;
			if (getNotionalLeg1()!=null) return true;
			if (getNotionalLeg2()!=null) return true;
			if (getTotalNotionalQuantityLeg1()!=null) return true;
			if (getTotalNotionalQuantityLeg2()!=null) return true;
			if (getNotionalQuantityLeg1()!=null) return true;
			if (getNotionalQuantityLeg2()!=null) return true;
			if (getCallAmount()!=null) return true;
			if (getPutAmount()!=null) return true;
			if (getNotionalCurrencyLeg1()!=null) return true;
			if (getNotionalCurrencyLeg2()!=null) return true;
			if (getQuantityUnitOfMeasureLeg1()!=null) return true;
			if (getQuantityUnitOfMeasureLeg2()!=null) return true;
			if (getCallCurrency()!=null) return true;
			if (getPutCurrency()!=null) return true;
			if (getNotionalAmountScheduleLeg1()!=null && getNotionalAmountScheduleLeg1().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getNotionalAmountScheduleLeg2()!=null && getNotionalAmountScheduleLeg2().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPriceNotation()!=null) return true;
			if (getPrice()!=null) return true;
			if (getPriceCurrency()!=null) return true;
			if (getPriceUnitOfMeasure()!=null) return true;
			if (getFixedRateLeg1()!=null) return true;
			if (getFixedRateLeg2()!=null) return true;
			if (getSpreadNotationLeg1()!=null) return true;
			if (getSpreadLeg1()!=null && getSpreadLeg1().hasData()) return true;
			if (getSpreadNotationLeg2()!=null) return true;
			if (getSpreadLeg2()!=null && getSpreadLeg2().hasData()) return true;
			if (getSpreadCurrencyLeg1()!=null) return true;
			if (getSpreadCurrencyLeg2()!=null) return true;
			if (getStrikePriceNotation()!=null) return true;
			if (getStrikePrice()!=null && getStrikePrice().hasData()) return true;
			if (getStrikePriceCurrency()!=null) return true;
			if (getOptionPremiumAmount()!=null) return true;
			if (getOptionPremiumCurrency()!=null) return true;
			if (getOptionPremiumPaymentDate()!=null) return true;
			if (getExchangeRate()!=null) return true;
			if (getExchangeRateBasis()!=null) return true;
			if (getDayCountConventionLeg1()!=null) return true;
			if (getDayCountConventionLeg2()!=null) return true;
			if (getPaymentFrequencyPeriodLeg1()!=null) return true;
			if (getPaymentFrequencyPeriodLeg2()!=null) return true;
			if (getPaymentFrequencyPeriodMultiplierLeg1()!=null) return true;
			if (getPaymentFrequencyPeriodMultiplierLeg2()!=null) return true;
			if (getSettlementCurrencyLeg1()!=null) return true;
			if (getSettlementCurrencyLeg2()!=null) return true;
			if (getOtherPayment()!=null && getOtherPayment().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getCustomBasketCode()!=null) return true;
			if (getBasketConstituents()!=null && getBasketConstituents().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getBasketConstituentIdentifierSource()!=null && !getBasketConstituentIdentifierSource().isEmpty()) return true;
			if (getUnderlyingIdentification()!=null) return true;
			if (getUnderlierIDSource()!=null) return true;
			if (getMaturityDateOfTheUnderlying()!=null) return true;
			if (getIdentifierOfTheFloatingRateLeg2()!=null) return true;
			if (getFloatingRateReferencePeriodOfLeg2TimePeriod()!=null) return true;
			if (getFloatingRateReferencePeriodOfLeg2Multiplier()!=null) return true;
			if (getCdSIndexAttachmentPoint()!=null) return true;
			if (getCdSIndexDetachmentPoint()!=null) return true;
			if (getIndexFactor()!=null) return true;
			if (getPackageIdentifier()!=null) return true;
			if (getPackageTransactionPrice()!=null && getPackageTransactionPrice().hasData()) return true;
			if (getPackageTransactionSpread()!=null && getPackageTransactionSpread().hasData()) return true;
			if (getPriorUTI()!=null) return true;
			if (getEventIdentifier()!=null) return true;
			if (getActionType()!=null) return true;
			if (getEventType()!=null) return true;
			if (getReportingTimestamp()!=null) return true;
			if (getReportSubmittingEntityID()!=null) return true;
			if (getValuationTimestamp()!=null) return true;
			if (getValuationAmount()!=null) return true;
			if (getValuationCurrency()!=null) return true;
			if (getValuationMethod()!=null) return true;
			if (getDelta()!=null) return true;
			if (getNextFloatingReferenceResetDateLeg1()!=null) return true;
			if (getNextFloatingReferenceResetDateLeg2()!=null) return true;
			if (getSmallScaleBuySideEntityIndicator()!=null) return true;
			if (getNonReportable()!=null && getNonReportable().hasData()) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public ASICTransactionReport.ASICTransactionReportBuilder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			ASICTransactionReport.ASICTransactionReportBuilder o = (ASICTransactionReport.ASICTransactionReportBuilder) other;
			
			merger.mergeRosetta(getNotionalAmountScheduleLeg1(), o.getNotionalAmountScheduleLeg1(), this::getOrCreateNotionalAmountScheduleLeg1);
			merger.mergeRosetta(getNotionalAmountScheduleLeg2(), o.getNotionalAmountScheduleLeg2(), this::getOrCreateNotionalAmountScheduleLeg2);
			merger.mergeRosetta(getSpreadLeg1(), o.getSpreadLeg1(), this::setSpreadLeg1);
			merger.mergeRosetta(getSpreadLeg2(), o.getSpreadLeg2(), this::setSpreadLeg2);
			merger.mergeRosetta(getStrikePrice(), o.getStrikePrice(), this::setStrikePrice);
			merger.mergeRosetta(getOtherPayment(), o.getOtherPayment(), this::getOrCreateOtherPayment);
			merger.mergeRosetta(getBasketConstituents(), o.getBasketConstituents(), this::getOrCreateBasketConstituents);
			merger.mergeRosetta(getPackageTransactionPrice(), o.getPackageTransactionPrice(), this::setPackageTransactionPrice);
			merger.mergeRosetta(getPackageTransactionSpread(), o.getPackageTransactionSpread(), this::setPackageTransactionSpread);
			merger.mergeRosetta(getNonReportable(), o.getNonReportable(), this::setNonReportable);
			
			merger.mergeBasic(getUniqueTransactionIdentifier(), o.getUniqueTransactionIdentifier(), this::setUniqueTransactionIdentifier);
			merger.mergeBasic(getUniqueProductIdentifier(), o.getUniqueProductIdentifier(), this::setUniqueProductIdentifier);
			merger.mergeBasic(getAssetClass(), o.getAssetClass(), this::setAssetClass);
			merger.mergeBasic(getContractType(), o.getContractType(), this::setContractType);
			merger.mergeBasic(getReportingEntity(), o.getReportingEntity(), this::setReportingEntity);
			merger.mergeBasic(getCounterparty1(), o.getCounterparty1(), this::setCounterparty1);
			merger.mergeBasic(getCounterparty2(), o.getCounterparty2(), this::setCounterparty2);
			merger.mergeBasic(getCounterpartyIdentifierTypeIndicator(), o.getCounterpartyIdentifierTypeIndicator(), this::setCounterpartyIdentifierTypeIndicator);
			merger.mergeBasic(getCountryOfTheCounterParty2(), o.getCountryOfTheCounterParty2(), this::setCountryOfTheCounterParty2);
			merger.mergeBasic(getBeneficiary1(), o.getBeneficiary1(), this::setBeneficiary1);
			merger.mergeBasic(getBeneficiary1IdentifierTypeIndicator(), o.getBeneficiary1IdentifierTypeIndicator(), this::setBeneficiary1IdentifierTypeIndicator);
			merger.mergeBasic(getBroker(), o.getBroker(), this::setBroker);
			merger.mergeBasic(getExecutionAgentOfTheCounterparty1(), o.getExecutionAgentOfTheCounterparty1(), this::setExecutionAgentOfTheCounterparty1);
			merger.mergeBasic(getDirection(), o.getDirection(), this::setDirection);
			merger.mergeBasic(getDirection2OfLeg1(), o.getDirection2OfLeg1(), this::setDirection2OfLeg1);
			merger.mergeBasic(getDirection2OfLeg2(), o.getDirection2OfLeg2(), this::setDirection2OfLeg2);
			merger.mergeBasic(getEffectiveDate(), o.getEffectiveDate(), this::setEffectiveDate);
			merger.mergeBasic(getExpirationDate(), o.getExpirationDate(), this::setExpirationDate);
			merger.mergeBasic(getExecutionTimestamp(), o.getExecutionTimestamp(), this::setExecutionTimestamp);
			merger.mergeBasic(getEventTimestamp(), o.getEventTimestamp(), this::setEventTimestamp);
			merger.mergeBasic(getClearingTimestamp(), o.getClearingTimestamp(), this::setClearingTimestamp);
			merger.mergeBasic(getCleared(), o.getCleared(), this::setCleared);
			merger.mergeBasic(getCentralCounterparty(), o.getCentralCounterparty(), this::setCentralCounterparty);
			merger.mergeBasic(getClearingMember(), o.getClearingMember(), this::setClearingMember);
			merger.mergeBasic(getPlatformIdentifier(), o.getPlatformIdentifier(), this::setPlatformIdentifier);
			merger.mergeBasic(getNotionalLeg1(), o.getNotionalLeg1(), this::setNotionalLeg1);
			merger.mergeBasic(getNotionalLeg2(), o.getNotionalLeg2(), this::setNotionalLeg2);
			merger.mergeBasic(getTotalNotionalQuantityLeg1(), o.getTotalNotionalQuantityLeg1(), this::setTotalNotionalQuantityLeg1);
			merger.mergeBasic(getTotalNotionalQuantityLeg2(), o.getTotalNotionalQuantityLeg2(), this::setTotalNotionalQuantityLeg2);
			merger.mergeBasic(getNotionalQuantityLeg1(), o.getNotionalQuantityLeg1(), this::setNotionalQuantityLeg1);
			merger.mergeBasic(getNotionalQuantityLeg2(), o.getNotionalQuantityLeg2(), this::setNotionalQuantityLeg2);
			merger.mergeBasic(getCallAmount(), o.getCallAmount(), this::setCallAmount);
			merger.mergeBasic(getPutAmount(), o.getPutAmount(), this::setPutAmount);
			merger.mergeBasic(getNotionalCurrencyLeg1(), o.getNotionalCurrencyLeg1(), this::setNotionalCurrencyLeg1);
			merger.mergeBasic(getNotionalCurrencyLeg2(), o.getNotionalCurrencyLeg2(), this::setNotionalCurrencyLeg2);
			merger.mergeBasic(getQuantityUnitOfMeasureLeg1(), o.getQuantityUnitOfMeasureLeg1(), this::setQuantityUnitOfMeasureLeg1);
			merger.mergeBasic(getQuantityUnitOfMeasureLeg2(), o.getQuantityUnitOfMeasureLeg2(), this::setQuantityUnitOfMeasureLeg2);
			merger.mergeBasic(getCallCurrency(), o.getCallCurrency(), this::setCallCurrency);
			merger.mergeBasic(getPutCurrency(), o.getPutCurrency(), this::setPutCurrency);
			merger.mergeBasic(getPriceNotation(), o.getPriceNotation(), this::setPriceNotation);
			merger.mergeBasic(getPrice(), o.getPrice(), this::setPrice);
			merger.mergeBasic(getPriceCurrency(), o.getPriceCurrency(), this::setPriceCurrency);
			merger.mergeBasic(getPriceUnitOfMeasure(), o.getPriceUnitOfMeasure(), this::setPriceUnitOfMeasure);
			merger.mergeBasic(getFixedRateLeg1(), o.getFixedRateLeg1(), this::setFixedRateLeg1);
			merger.mergeBasic(getFixedRateLeg2(), o.getFixedRateLeg2(), this::setFixedRateLeg2);
			merger.mergeBasic(getSpreadNotationLeg1(), o.getSpreadNotationLeg1(), this::setSpreadNotationLeg1);
			merger.mergeBasic(getSpreadNotationLeg2(), o.getSpreadNotationLeg2(), this::setSpreadNotationLeg2);
			merger.mergeBasic(getSpreadCurrencyLeg1(), o.getSpreadCurrencyLeg1(), this::setSpreadCurrencyLeg1);
			merger.mergeBasic(getSpreadCurrencyLeg2(), o.getSpreadCurrencyLeg2(), this::setSpreadCurrencyLeg2);
			merger.mergeBasic(getStrikePriceNotation(), o.getStrikePriceNotation(), this::setStrikePriceNotation);
			merger.mergeBasic(getStrikePriceCurrency(), o.getStrikePriceCurrency(), this::setStrikePriceCurrency);
			merger.mergeBasic(getOptionPremiumAmount(), o.getOptionPremiumAmount(), this::setOptionPremiumAmount);
			merger.mergeBasic(getOptionPremiumCurrency(), o.getOptionPremiumCurrency(), this::setOptionPremiumCurrency);
			merger.mergeBasic(getOptionPremiumPaymentDate(), o.getOptionPremiumPaymentDate(), this::setOptionPremiumPaymentDate);
			merger.mergeBasic(getExchangeRate(), o.getExchangeRate(), this::setExchangeRate);
			merger.mergeBasic(getExchangeRateBasis(), o.getExchangeRateBasis(), this::setExchangeRateBasis);
			merger.mergeBasic(getDayCountConventionLeg1(), o.getDayCountConventionLeg1(), this::setDayCountConventionLeg1);
			merger.mergeBasic(getDayCountConventionLeg2(), o.getDayCountConventionLeg2(), this::setDayCountConventionLeg2);
			merger.mergeBasic(getPaymentFrequencyPeriodLeg1(), o.getPaymentFrequencyPeriodLeg1(), this::setPaymentFrequencyPeriodLeg1);
			merger.mergeBasic(getPaymentFrequencyPeriodLeg2(), o.getPaymentFrequencyPeriodLeg2(), this::setPaymentFrequencyPeriodLeg2);
			merger.mergeBasic(getPaymentFrequencyPeriodMultiplierLeg1(), o.getPaymentFrequencyPeriodMultiplierLeg1(), this::setPaymentFrequencyPeriodMultiplierLeg1);
			merger.mergeBasic(getPaymentFrequencyPeriodMultiplierLeg2(), o.getPaymentFrequencyPeriodMultiplierLeg2(), this::setPaymentFrequencyPeriodMultiplierLeg2);
			merger.mergeBasic(getSettlementCurrencyLeg1(), o.getSettlementCurrencyLeg1(), this::setSettlementCurrencyLeg1);
			merger.mergeBasic(getSettlementCurrencyLeg2(), o.getSettlementCurrencyLeg2(), this::setSettlementCurrencyLeg2);
			merger.mergeBasic(getCustomBasketCode(), o.getCustomBasketCode(), this::setCustomBasketCode);
			merger.mergeBasic(getBasketConstituentIdentifierSource(), o.getBasketConstituentIdentifierSource(), (Consumer<ProductIdTypeEnum>) this::addBasketConstituentIdentifierSource);
			merger.mergeBasic(getUnderlyingIdentification(), o.getUnderlyingIdentification(), this::setUnderlyingIdentification);
			merger.mergeBasic(getUnderlierIDSource(), o.getUnderlierIDSource(), this::setUnderlierIDSource);
			merger.mergeBasic(getMaturityDateOfTheUnderlying(), o.getMaturityDateOfTheUnderlying(), this::setMaturityDateOfTheUnderlying);
			merger.mergeBasic(getIdentifierOfTheFloatingRateLeg2(), o.getIdentifierOfTheFloatingRateLeg2(), this::setIdentifierOfTheFloatingRateLeg2);
			merger.mergeBasic(getFloatingRateReferencePeriodOfLeg2TimePeriod(), o.getFloatingRateReferencePeriodOfLeg2TimePeriod(), this::setFloatingRateReferencePeriodOfLeg2TimePeriod);
			merger.mergeBasic(getFloatingRateReferencePeriodOfLeg2Multiplier(), o.getFloatingRateReferencePeriodOfLeg2Multiplier(), this::setFloatingRateReferencePeriodOfLeg2Multiplier);
			merger.mergeBasic(getCdSIndexAttachmentPoint(), o.getCdSIndexAttachmentPoint(), this::setCdSIndexAttachmentPoint);
			merger.mergeBasic(getCdSIndexDetachmentPoint(), o.getCdSIndexDetachmentPoint(), this::setCdSIndexDetachmentPoint);
			merger.mergeBasic(getIndexFactor(), o.getIndexFactor(), this::setIndexFactor);
			merger.mergeBasic(getPackageIdentifier(), o.getPackageIdentifier(), this::setPackageIdentifier);
			merger.mergeBasic(getPriorUTI(), o.getPriorUTI(), this::setPriorUTI);
			merger.mergeBasic(getEventIdentifier(), o.getEventIdentifier(), this::setEventIdentifier);
			merger.mergeBasic(getActionType(), o.getActionType(), this::setActionType);
			merger.mergeBasic(getEventType(), o.getEventType(), this::setEventType);
			merger.mergeBasic(getReportingTimestamp(), o.getReportingTimestamp(), this::setReportingTimestamp);
			merger.mergeBasic(getReportSubmittingEntityID(), o.getReportSubmittingEntityID(), this::setReportSubmittingEntityID);
			merger.mergeBasic(getValuationTimestamp(), o.getValuationTimestamp(), this::setValuationTimestamp);
			merger.mergeBasic(getValuationAmount(), o.getValuationAmount(), this::setValuationAmount);
			merger.mergeBasic(getValuationCurrency(), o.getValuationCurrency(), this::setValuationCurrency);
			merger.mergeBasic(getValuationMethod(), o.getValuationMethod(), this::setValuationMethod);
			merger.mergeBasic(getDelta(), o.getDelta(), this::setDelta);
			merger.mergeBasic(getNextFloatingReferenceResetDateLeg1(), o.getNextFloatingReferenceResetDateLeg1(), this::setNextFloatingReferenceResetDateLeg1);
			merger.mergeBasic(getNextFloatingReferenceResetDateLeg2(), o.getNextFloatingReferenceResetDateLeg2(), this::setNextFloatingReferenceResetDateLeg2);
			merger.mergeBasic(getSmallScaleBuySideEntityIndicator(), o.getSmallScaleBuySideEntityIndicator(), this::setSmallScaleBuySideEntityIndicator);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			ASICTransactionReport _that = getType().cast(o);
		
			if (!Objects.equals(uniqueTransactionIdentifier, _that.getUniqueTransactionIdentifier())) return false;
			if (!Objects.equals(uniqueProductIdentifier, _that.getUniqueProductIdentifier())) return false;
			if (!Objects.equals(assetClass, _that.getAssetClass())) return false;
			if (!Objects.equals(contractType, _that.getContractType())) return false;
			if (!Objects.equals(reportingEntity, _that.getReportingEntity())) return false;
			if (!Objects.equals(counterparty1, _that.getCounterparty1())) return false;
			if (!Objects.equals(counterparty2, _that.getCounterparty2())) return false;
			if (!Objects.equals(counterpartyIdentifierTypeIndicator, _that.getCounterpartyIdentifierTypeIndicator())) return false;
			if (!Objects.equals(countryOfTheCounterParty2, _that.getCountryOfTheCounterParty2())) return false;
			if (!Objects.equals(beneficiary1, _that.getBeneficiary1())) return false;
			if (!Objects.equals(beneficiary1IdentifierTypeIndicator, _that.getBeneficiary1IdentifierTypeIndicator())) return false;
			if (!Objects.equals(broker, _that.getBroker())) return false;
			if (!Objects.equals(executionAgentOfTheCounterparty1, _that.getExecutionAgentOfTheCounterparty1())) return false;
			if (!Objects.equals(direction, _that.getDirection())) return false;
			if (!Objects.equals(direction2OfLeg1, _that.getDirection2OfLeg1())) return false;
			if (!Objects.equals(direction2OfLeg2, _that.getDirection2OfLeg2())) return false;
			if (!Objects.equals(effectiveDate, _that.getEffectiveDate())) return false;
			if (!Objects.equals(expirationDate, _that.getExpirationDate())) return false;
			if (!Objects.equals(executionTimestamp, _that.getExecutionTimestamp())) return false;
			if (!Objects.equals(eventTimestamp, _that.getEventTimestamp())) return false;
			if (!Objects.equals(clearingTimestamp, _that.getClearingTimestamp())) return false;
			if (!Objects.equals(cleared, _that.getCleared())) return false;
			if (!Objects.equals(centralCounterparty, _that.getCentralCounterparty())) return false;
			if (!Objects.equals(clearingMember, _that.getClearingMember())) return false;
			if (!Objects.equals(platformIdentifier, _that.getPlatformIdentifier())) return false;
			if (!Objects.equals(notionalLeg1, _that.getNotionalLeg1())) return false;
			if (!Objects.equals(notionalLeg2, _that.getNotionalLeg2())) return false;
			if (!Objects.equals(totalNotionalQuantityLeg1, _that.getTotalNotionalQuantityLeg1())) return false;
			if (!Objects.equals(totalNotionalQuantityLeg2, _that.getTotalNotionalQuantityLeg2())) return false;
			if (!Objects.equals(notionalQuantityLeg1, _that.getNotionalQuantityLeg1())) return false;
			if (!Objects.equals(notionalQuantityLeg2, _that.getNotionalQuantityLeg2())) return false;
			if (!Objects.equals(callAmount, _that.getCallAmount())) return false;
			if (!Objects.equals(putAmount, _that.getPutAmount())) return false;
			if (!Objects.equals(notionalCurrencyLeg1, _that.getNotionalCurrencyLeg1())) return false;
			if (!Objects.equals(notionalCurrencyLeg2, _that.getNotionalCurrencyLeg2())) return false;
			if (!Objects.equals(quantityUnitOfMeasureLeg1, _that.getQuantityUnitOfMeasureLeg1())) return false;
			if (!Objects.equals(quantityUnitOfMeasureLeg2, _that.getQuantityUnitOfMeasureLeg2())) return false;
			if (!Objects.equals(callCurrency, _that.getCallCurrency())) return false;
			if (!Objects.equals(putCurrency, _that.getPutCurrency())) return false;
			if (!ListEquals.listEquals(notionalAmountScheduleLeg1, _that.getNotionalAmountScheduleLeg1())) return false;
			if (!ListEquals.listEquals(notionalAmountScheduleLeg2, _that.getNotionalAmountScheduleLeg2())) return false;
			if (!Objects.equals(priceNotation, _that.getPriceNotation())) return false;
			if (!Objects.equals(price, _that.getPrice())) return false;
			if (!Objects.equals(priceCurrency, _that.getPriceCurrency())) return false;
			if (!Objects.equals(priceUnitOfMeasure, _that.getPriceUnitOfMeasure())) return false;
			if (!Objects.equals(fixedRateLeg1, _that.getFixedRateLeg1())) return false;
			if (!Objects.equals(fixedRateLeg2, _that.getFixedRateLeg2())) return false;
			if (!Objects.equals(spreadNotationLeg1, _that.getSpreadNotationLeg1())) return false;
			if (!Objects.equals(spreadLeg1, _that.getSpreadLeg1())) return false;
			if (!Objects.equals(spreadNotationLeg2, _that.getSpreadNotationLeg2())) return false;
			if (!Objects.equals(spreadLeg2, _that.getSpreadLeg2())) return false;
			if (!Objects.equals(spreadCurrencyLeg1, _that.getSpreadCurrencyLeg1())) return false;
			if (!Objects.equals(spreadCurrencyLeg2, _that.getSpreadCurrencyLeg2())) return false;
			if (!Objects.equals(strikePriceNotation, _that.getStrikePriceNotation())) return false;
			if (!Objects.equals(strikePrice, _that.getStrikePrice())) return false;
			if (!Objects.equals(strikePriceCurrency, _that.getStrikePriceCurrency())) return false;
			if (!Objects.equals(optionPremiumAmount, _that.getOptionPremiumAmount())) return false;
			if (!Objects.equals(optionPremiumCurrency, _that.getOptionPremiumCurrency())) return false;
			if (!Objects.equals(optionPremiumPaymentDate, _that.getOptionPremiumPaymentDate())) return false;
			if (!Objects.equals(exchangeRate, _that.getExchangeRate())) return false;
			if (!Objects.equals(exchangeRateBasis, _that.getExchangeRateBasis())) return false;
			if (!Objects.equals(dayCountConventionLeg1, _that.getDayCountConventionLeg1())) return false;
			if (!Objects.equals(dayCountConventionLeg2, _that.getDayCountConventionLeg2())) return false;
			if (!Objects.equals(paymentFrequencyPeriodLeg1, _that.getPaymentFrequencyPeriodLeg1())) return false;
			if (!Objects.equals(paymentFrequencyPeriodLeg2, _that.getPaymentFrequencyPeriodLeg2())) return false;
			if (!Objects.equals(paymentFrequencyPeriodMultiplierLeg1, _that.getPaymentFrequencyPeriodMultiplierLeg1())) return false;
			if (!Objects.equals(paymentFrequencyPeriodMultiplierLeg2, _that.getPaymentFrequencyPeriodMultiplierLeg2())) return false;
			if (!Objects.equals(settlementCurrencyLeg1, _that.getSettlementCurrencyLeg1())) return false;
			if (!Objects.equals(settlementCurrencyLeg2, _that.getSettlementCurrencyLeg2())) return false;
			if (!ListEquals.listEquals(otherPayment, _that.getOtherPayment())) return false;
			if (!Objects.equals(customBasketCode, _that.getCustomBasketCode())) return false;
			if (!ListEquals.listEquals(basketConstituents, _that.getBasketConstituents())) return false;
			if (!ListEquals.listEquals(basketConstituentIdentifierSource, _that.getBasketConstituentIdentifierSource())) return false;
			if (!Objects.equals(underlyingIdentification, _that.getUnderlyingIdentification())) return false;
			if (!Objects.equals(underlierIDSource, _that.getUnderlierIDSource())) return false;
			if (!Objects.equals(maturityDateOfTheUnderlying, _that.getMaturityDateOfTheUnderlying())) return false;
			if (!Objects.equals(identifierOfTheFloatingRateLeg2, _that.getIdentifierOfTheFloatingRateLeg2())) return false;
			if (!Objects.equals(floatingRateReferencePeriodOfLeg2TimePeriod, _that.getFloatingRateReferencePeriodOfLeg2TimePeriod())) return false;
			if (!Objects.equals(floatingRateReferencePeriodOfLeg2Multiplier, _that.getFloatingRateReferencePeriodOfLeg2Multiplier())) return false;
			if (!Objects.equals(cdSIndexAttachmentPoint, _that.getCdSIndexAttachmentPoint())) return false;
			if (!Objects.equals(cdSIndexDetachmentPoint, _that.getCdSIndexDetachmentPoint())) return false;
			if (!Objects.equals(indexFactor, _that.getIndexFactor())) return false;
			if (!Objects.equals(packageIdentifier, _that.getPackageIdentifier())) return false;
			if (!Objects.equals(packageTransactionPrice, _that.getPackageTransactionPrice())) return false;
			if (!Objects.equals(packageTransactionSpread, _that.getPackageTransactionSpread())) return false;
			if (!Objects.equals(priorUTI, _that.getPriorUTI())) return false;
			if (!Objects.equals(eventIdentifier, _that.getEventIdentifier())) return false;
			if (!Objects.equals(actionType, _that.getActionType())) return false;
			if (!Objects.equals(eventType, _that.getEventType())) return false;
			if (!Objects.equals(reportingTimestamp, _that.getReportingTimestamp())) return false;
			if (!Objects.equals(reportSubmittingEntityID, _that.getReportSubmittingEntityID())) return false;
			if (!Objects.equals(valuationTimestamp, _that.getValuationTimestamp())) return false;
			if (!Objects.equals(valuationAmount, _that.getValuationAmount())) return false;
			if (!Objects.equals(valuationCurrency, _that.getValuationCurrency())) return false;
			if (!Objects.equals(valuationMethod, _that.getValuationMethod())) return false;
			if (!Objects.equals(delta, _that.getDelta())) return false;
			if (!Objects.equals(nextFloatingReferenceResetDateLeg1, _that.getNextFloatingReferenceResetDateLeg1())) return false;
			if (!Objects.equals(nextFloatingReferenceResetDateLeg2, _that.getNextFloatingReferenceResetDateLeg2())) return false;
			if (!Objects.equals(smallScaleBuySideEntityIndicator, _that.getSmallScaleBuySideEntityIndicator())) return false;
			if (!Objects.equals(nonReportable, _that.getNonReportable())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (uniqueTransactionIdentifier != null ? uniqueTransactionIdentifier.hashCode() : 0);
			_result = 31 * _result + (uniqueProductIdentifier != null ? uniqueProductIdentifier.hashCode() : 0);
			_result = 31 * _result + (assetClass != null ? assetClass.hashCode() : 0);
			_result = 31 * _result + (contractType != null ? contractType.hashCode() : 0);
			_result = 31 * _result + (reportingEntity != null ? reportingEntity.hashCode() : 0);
			_result = 31 * _result + (counterparty1 != null ? counterparty1.hashCode() : 0);
			_result = 31 * _result + (counterparty2 != null ? counterparty2.hashCode() : 0);
			_result = 31 * _result + (counterpartyIdentifierTypeIndicator != null ? counterpartyIdentifierTypeIndicator.hashCode() : 0);
			_result = 31 * _result + (countryOfTheCounterParty2 != null ? countryOfTheCounterParty2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (beneficiary1 != null ? beneficiary1.hashCode() : 0);
			_result = 31 * _result + (beneficiary1IdentifierTypeIndicator != null ? beneficiary1IdentifierTypeIndicator.hashCode() : 0);
			_result = 31 * _result + (broker != null ? broker.hashCode() : 0);
			_result = 31 * _result + (executionAgentOfTheCounterparty1 != null ? executionAgentOfTheCounterparty1.hashCode() : 0);
			_result = 31 * _result + (direction != null ? direction.hashCode() : 0);
			_result = 31 * _result + (direction2OfLeg1 != null ? direction2OfLeg1.hashCode() : 0);
			_result = 31 * _result + (direction2OfLeg2 != null ? direction2OfLeg2.hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (expirationDate != null ? expirationDate.hashCode() : 0);
			_result = 31 * _result + (executionTimestamp != null ? executionTimestamp.hashCode() : 0);
			_result = 31 * _result + (eventTimestamp != null ? eventTimestamp.hashCode() : 0);
			_result = 31 * _result + (clearingTimestamp != null ? clearingTimestamp.hashCode() : 0);
			_result = 31 * _result + (cleared != null ? cleared.hashCode() : 0);
			_result = 31 * _result + (centralCounterparty != null ? centralCounterparty.hashCode() : 0);
			_result = 31 * _result + (clearingMember != null ? clearingMember.hashCode() : 0);
			_result = 31 * _result + (platformIdentifier != null ? platformIdentifier.hashCode() : 0);
			_result = 31 * _result + (notionalLeg1 != null ? notionalLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalLeg2 != null ? notionalLeg2.hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityLeg1 != null ? totalNotionalQuantityLeg1.hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityLeg2 != null ? totalNotionalQuantityLeg2.hashCode() : 0);
			_result = 31 * _result + (notionalQuantityLeg1 != null ? notionalQuantityLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalQuantityLeg2 != null ? notionalQuantityLeg2.hashCode() : 0);
			_result = 31 * _result + (callAmount != null ? callAmount.hashCode() : 0);
			_result = 31 * _result + (putAmount != null ? putAmount.hashCode() : 0);
			_result = 31 * _result + (notionalCurrencyLeg1 != null ? notionalCurrencyLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalCurrencyLeg2 != null ? notionalCurrencyLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (quantityUnitOfMeasureLeg1 != null ? quantityUnitOfMeasureLeg1.hashCode() : 0);
			_result = 31 * _result + (quantityUnitOfMeasureLeg2 != null ? quantityUnitOfMeasureLeg2.hashCode() : 0);
			_result = 31 * _result + (callCurrency != null ? callCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (putCurrency != null ? putCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalAmountScheduleLeg1 != null ? notionalAmountScheduleLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalAmountScheduleLeg2 != null ? notionalAmountScheduleLeg2.hashCode() : 0);
			_result = 31 * _result + (priceNotation != null ? priceNotation.hashCode() : 0);
			_result = 31 * _result + (price != null ? price.hashCode() : 0);
			_result = 31 * _result + (priceCurrency != null ? priceCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (priceUnitOfMeasure != null ? priceUnitOfMeasure.hashCode() : 0);
			_result = 31 * _result + (fixedRateLeg1 != null ? fixedRateLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRateLeg2 != null ? fixedRateLeg2.hashCode() : 0);
			_result = 31 * _result + (spreadNotationLeg1 != null ? spreadNotationLeg1.hashCode() : 0);
			_result = 31 * _result + (spreadLeg1 != null ? spreadLeg1.hashCode() : 0);
			_result = 31 * _result + (spreadNotationLeg2 != null ? spreadNotationLeg2.hashCode() : 0);
			_result = 31 * _result + (spreadLeg2 != null ? spreadLeg2.hashCode() : 0);
			_result = 31 * _result + (spreadCurrencyLeg1 != null ? spreadCurrencyLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (spreadCurrencyLeg2 != null ? spreadCurrencyLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (strikePriceNotation != null ? strikePriceNotation.hashCode() : 0);
			_result = 31 * _result + (strikePrice != null ? strikePrice.hashCode() : 0);
			_result = 31 * _result + (strikePriceCurrency != null ? strikePriceCurrency.hashCode() : 0);
			_result = 31 * _result + (optionPremiumAmount != null ? optionPremiumAmount.hashCode() : 0);
			_result = 31 * _result + (optionPremiumCurrency != null ? optionPremiumCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (optionPremiumPaymentDate != null ? optionPremiumPaymentDate.hashCode() : 0);
			_result = 31 * _result + (exchangeRate != null ? exchangeRate.hashCode() : 0);
			_result = 31 * _result + (exchangeRateBasis != null ? exchangeRateBasis.hashCode() : 0);
			_result = 31 * _result + (dayCountConventionLeg1 != null ? dayCountConventionLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (dayCountConventionLeg2 != null ? dayCountConventionLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (paymentFrequencyPeriodLeg1 != null ? paymentFrequencyPeriodLeg1.hashCode() : 0);
			_result = 31 * _result + (paymentFrequencyPeriodLeg2 != null ? paymentFrequencyPeriodLeg2.hashCode() : 0);
			_result = 31 * _result + (paymentFrequencyPeriodMultiplierLeg1 != null ? paymentFrequencyPeriodMultiplierLeg1.hashCode() : 0);
			_result = 31 * _result + (paymentFrequencyPeriodMultiplierLeg2 != null ? paymentFrequencyPeriodMultiplierLeg2.hashCode() : 0);
			_result = 31 * _result + (settlementCurrencyLeg1 != null ? settlementCurrencyLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (settlementCurrencyLeg2 != null ? settlementCurrencyLeg2.hashCode() : 0);
			_result = 31 * _result + (otherPayment != null ? otherPayment.hashCode() : 0);
			_result = 31 * _result + (customBasketCode != null ? customBasketCode.hashCode() : 0);
			_result = 31 * _result + (basketConstituents != null ? basketConstituents.hashCode() : 0);
			_result = 31 * _result + (basketConstituentIdentifierSource != null ? basketConstituentIdentifierSource.stream().map(Object::getClass).map(Class::getName).mapToInt(String::hashCode).sum() : 0);
			_result = 31 * _result + (underlyingIdentification != null ? underlyingIdentification.hashCode() : 0);
			_result = 31 * _result + (underlierIDSource != null ? underlierIDSource.hashCode() : 0);
			_result = 31 * _result + (maturityDateOfTheUnderlying != null ? maturityDateOfTheUnderlying.hashCode() : 0);
			_result = 31 * _result + (identifierOfTheFloatingRateLeg2 != null ? identifierOfTheFloatingRateLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRateReferencePeriodOfLeg2TimePeriod != null ? floatingRateReferencePeriodOfLeg2TimePeriod.hashCode() : 0);
			_result = 31 * _result + (floatingRateReferencePeriodOfLeg2Multiplier != null ? floatingRateReferencePeriodOfLeg2Multiplier.hashCode() : 0);
			_result = 31 * _result + (cdSIndexAttachmentPoint != null ? cdSIndexAttachmentPoint.hashCode() : 0);
			_result = 31 * _result + (cdSIndexDetachmentPoint != null ? cdSIndexDetachmentPoint.hashCode() : 0);
			_result = 31 * _result + (indexFactor != null ? indexFactor.hashCode() : 0);
			_result = 31 * _result + (packageIdentifier != null ? packageIdentifier.hashCode() : 0);
			_result = 31 * _result + (packageTransactionPrice != null ? packageTransactionPrice.hashCode() : 0);
			_result = 31 * _result + (packageTransactionSpread != null ? packageTransactionSpread.hashCode() : 0);
			_result = 31 * _result + (priorUTI != null ? priorUTI.hashCode() : 0);
			_result = 31 * _result + (eventIdentifier != null ? eventIdentifier.hashCode() : 0);
			_result = 31 * _result + (actionType != null ? actionType.hashCode() : 0);
			_result = 31 * _result + (eventType != null ? eventType.hashCode() : 0);
			_result = 31 * _result + (reportingTimestamp != null ? reportingTimestamp.hashCode() : 0);
			_result = 31 * _result + (reportSubmittingEntityID != null ? reportSubmittingEntityID.hashCode() : 0);
			_result = 31 * _result + (valuationTimestamp != null ? valuationTimestamp.hashCode() : 0);
			_result = 31 * _result + (valuationAmount != null ? valuationAmount.hashCode() : 0);
			_result = 31 * _result + (valuationCurrency != null ? valuationCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (valuationMethod != null ? valuationMethod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (delta != null ? delta.hashCode() : 0);
			_result = 31 * _result + (nextFloatingReferenceResetDateLeg1 != null ? nextFloatingReferenceResetDateLeg1.hashCode() : 0);
			_result = 31 * _result + (nextFloatingReferenceResetDateLeg2 != null ? nextFloatingReferenceResetDateLeg2.hashCode() : 0);
			_result = 31 * _result + (smallScaleBuySideEntityIndicator != null ? smallScaleBuySideEntityIndicator.hashCode() : 0);
			_result = 31 * _result + (nonReportable != null ? nonReportable.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ASICTransactionReportBuilder {" +
				"uniqueTransactionIdentifier=" + this.uniqueTransactionIdentifier + ", " +
				"uniqueProductIdentifier=" + this.uniqueProductIdentifier + ", " +
				"assetClass=" + this.assetClass + ", " +
				"contractType=" + this.contractType + ", " +
				"reportingEntity=" + this.reportingEntity + ", " +
				"counterparty1=" + this.counterparty1 + ", " +
				"counterparty2=" + this.counterparty2 + ", " +
				"counterpartyIdentifierTypeIndicator=" + this.counterpartyIdentifierTypeIndicator + ", " +
				"countryOfTheCounterParty2=" + this.countryOfTheCounterParty2 + ", " +
				"beneficiary1=" + this.beneficiary1 + ", " +
				"beneficiary1IdentifierTypeIndicator=" + this.beneficiary1IdentifierTypeIndicator + ", " +
				"broker=" + this.broker + ", " +
				"executionAgentOfTheCounterparty1=" + this.executionAgentOfTheCounterparty1 + ", " +
				"direction=" + this.direction + ", " +
				"direction2OfLeg1=" + this.direction2OfLeg1 + ", " +
				"direction2OfLeg2=" + this.direction2OfLeg2 + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"expirationDate=" + this.expirationDate + ", " +
				"executionTimestamp=" + this.executionTimestamp + ", " +
				"eventTimestamp=" + this.eventTimestamp + ", " +
				"clearingTimestamp=" + this.clearingTimestamp + ", " +
				"cleared=" + this.cleared + ", " +
				"centralCounterparty=" + this.centralCounterparty + ", " +
				"clearingMember=" + this.clearingMember + ", " +
				"platformIdentifier=" + this.platformIdentifier + ", " +
				"notionalLeg1=" + this.notionalLeg1 + ", " +
				"notionalLeg2=" + this.notionalLeg2 + ", " +
				"totalNotionalQuantityLeg1=" + this.totalNotionalQuantityLeg1 + ", " +
				"totalNotionalQuantityLeg2=" + this.totalNotionalQuantityLeg2 + ", " +
				"notionalQuantityLeg1=" + this.notionalQuantityLeg1 + ", " +
				"notionalQuantityLeg2=" + this.notionalQuantityLeg2 + ", " +
				"callAmount=" + this.callAmount + ", " +
				"putAmount=" + this.putAmount + ", " +
				"notionalCurrencyLeg1=" + this.notionalCurrencyLeg1 + ", " +
				"notionalCurrencyLeg2=" + this.notionalCurrencyLeg2 + ", " +
				"quantityUnitOfMeasureLeg1=" + this.quantityUnitOfMeasureLeg1 + ", " +
				"quantityUnitOfMeasureLeg2=" + this.quantityUnitOfMeasureLeg2 + ", " +
				"callCurrency=" + this.callCurrency + ", " +
				"putCurrency=" + this.putCurrency + ", " +
				"notionalAmountScheduleLeg1=" + this.notionalAmountScheduleLeg1 + ", " +
				"notionalAmountScheduleLeg2=" + this.notionalAmountScheduleLeg2 + ", " +
				"priceNotation=" + this.priceNotation + ", " +
				"price=" + this.price + ", " +
				"priceCurrency=" + this.priceCurrency + ", " +
				"priceUnitOfMeasure=" + this.priceUnitOfMeasure + ", " +
				"fixedRateLeg1=" + this.fixedRateLeg1 + ", " +
				"fixedRateLeg2=" + this.fixedRateLeg2 + ", " +
				"spreadNotationLeg1=" + this.spreadNotationLeg1 + ", " +
				"spreadLeg1=" + this.spreadLeg1 + ", " +
				"spreadNotationLeg2=" + this.spreadNotationLeg2 + ", " +
				"spreadLeg2=" + this.spreadLeg2 + ", " +
				"spreadCurrencyLeg1=" + this.spreadCurrencyLeg1 + ", " +
				"spreadCurrencyLeg2=" + this.spreadCurrencyLeg2 + ", " +
				"strikePriceNotation=" + this.strikePriceNotation + ", " +
				"strikePrice=" + this.strikePrice + ", " +
				"strikePriceCurrency=" + this.strikePriceCurrency + ", " +
				"optionPremiumAmount=" + this.optionPremiumAmount + ", " +
				"optionPremiumCurrency=" + this.optionPremiumCurrency + ", " +
				"optionPremiumPaymentDate=" + this.optionPremiumPaymentDate + ", " +
				"exchangeRate=" + this.exchangeRate + ", " +
				"exchangeRateBasis=" + this.exchangeRateBasis + ", " +
				"dayCountConventionLeg1=" + this.dayCountConventionLeg1 + ", " +
				"dayCountConventionLeg2=" + this.dayCountConventionLeg2 + ", " +
				"paymentFrequencyPeriodLeg1=" + this.paymentFrequencyPeriodLeg1 + ", " +
				"paymentFrequencyPeriodLeg2=" + this.paymentFrequencyPeriodLeg2 + ", " +
				"paymentFrequencyPeriodMultiplierLeg1=" + this.paymentFrequencyPeriodMultiplierLeg1 + ", " +
				"paymentFrequencyPeriodMultiplierLeg2=" + this.paymentFrequencyPeriodMultiplierLeg2 + ", " +
				"settlementCurrencyLeg1=" + this.settlementCurrencyLeg1 + ", " +
				"settlementCurrencyLeg2=" + this.settlementCurrencyLeg2 + ", " +
				"otherPayment=" + this.otherPayment + ", " +
				"customBasketCode=" + this.customBasketCode + ", " +
				"basketConstituents=" + this.basketConstituents + ", " +
				"basketConstituentIdentifierSource=" + this.basketConstituentIdentifierSource + ", " +
				"underlyingIdentification=" + this.underlyingIdentification + ", " +
				"underlierIDSource=" + this.underlierIDSource + ", " +
				"maturityDateOfTheUnderlying=" + this.maturityDateOfTheUnderlying + ", " +
				"identifierOfTheFloatingRateLeg2=" + this.identifierOfTheFloatingRateLeg2 + ", " +
				"floatingRateReferencePeriodOfLeg2TimePeriod=" + this.floatingRateReferencePeriodOfLeg2TimePeriod + ", " +
				"floatingRateReferencePeriodOfLeg2Multiplier=" + this.floatingRateReferencePeriodOfLeg2Multiplier + ", " +
				"cdSIndexAttachmentPoint=" + this.cdSIndexAttachmentPoint + ", " +
				"cdSIndexDetachmentPoint=" + this.cdSIndexDetachmentPoint + ", " +
				"indexFactor=" + this.indexFactor + ", " +
				"packageIdentifier=" + this.packageIdentifier + ", " +
				"packageTransactionPrice=" + this.packageTransactionPrice + ", " +
				"packageTransactionSpread=" + this.packageTransactionSpread + ", " +
				"priorUTI=" + this.priorUTI + ", " +
				"eventIdentifier=" + this.eventIdentifier + ", " +
				"actionType=" + this.actionType + ", " +
				"eventType=" + this.eventType + ", " +
				"reportingTimestamp=" + this.reportingTimestamp + ", " +
				"reportSubmittingEntityID=" + this.reportSubmittingEntityID + ", " +
				"valuationTimestamp=" + this.valuationTimestamp + ", " +
				"valuationAmount=" + this.valuationAmount + ", " +
				"valuationCurrency=" + this.valuationCurrency + ", " +
				"valuationMethod=" + this.valuationMethod + ", " +
				"delta=" + this.delta + ", " +
				"nextFloatingReferenceResetDateLeg1=" + this.nextFloatingReferenceResetDateLeg1 + ", " +
				"nextFloatingReferenceResetDateLeg2=" + this.nextFloatingReferenceResetDateLeg2 + ", " +
				"smallScaleBuySideEntityIndicator=" + this.smallScaleBuySideEntityIndicator + ", " +
				"nonReportable=" + this.nonReportable +
			'}';
		}
	}
}
