package drr.regulation.cftc.rewrite;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.common.collect.ImmutableList;
import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import com.rosetta.model.lib.records.Date;
import com.rosetta.util.ListEquals;
import drr.regulation.cftc.rewrite.CFTCGuidebookPart43;
import drr.regulation.cftc.rewrite.CFTCNonReportablePart43;
import drr.regulation.cftc.rewrite.CFTCPart43TransactionReport;
import drr.regulation.cftc.rewrite.CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder;
import drr.regulation.cftc.rewrite.CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilderImpl;
import drr.regulation.cftc.rewrite.CFTCPart43TransactionReport.CFTCPart43TransactionReportImpl;
import drr.regulation.cftc.rewrite.NotionalAmountScheduleLeg1Report;
import drr.regulation.cftc.rewrite.NotionalAmountScheduleLeg2Report;
import drr.regulation.cftc.rewrite.OtherPaymentReport;
import drr.regulation.cftc.rewrite.meta.CFTCPart43TransactionReportMeta;
import iso20022.auth030.esma.InterestComputationMethod4Code;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="CFTCPart43TransactionReport", builder=CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilderImpl.class, version="4.2.1")
public interface CFTCPart43TransactionReport extends RosettaModelObject {

	CFTCPart43TransactionReportMeta metaData = new CFTCPart43TransactionReportMeta();

	/*********************** Getter Methods  ***********************/
	String getCleared();
	String getCounterparty1();
	String getCounterparty2();
	String getCounterparty2IdentifierSource();
	String getSubmitterIdentifier();
	Boolean getCustomBasketIndicator();
	String getActionType();
	String getEventType();
	Boolean getAmendmentIndicator();
	ZonedDateTime getEventTimestamp();
	BigDecimal getNotionalAmountLeg1();
	BigDecimal getNotionalAmountLeg2();
	ISOCurrencyCodeEnum getNotionalCurrencyLeg1();
	ISOCurrencyCodeEnum getNotionalCurrencyLeg2();
	List<? extends NotionalAmountScheduleLeg1Report> getNotionalAmountScheduleLeg1();
	List<? extends NotionalAmountScheduleLeg2Report> getNotionalAmountScheduleLeg2();
	BigDecimal getCallAmount();
	ISOCurrencyCodeEnum getCallCurrency();
	BigDecimal getPutAmount();
	ISOCurrencyCodeEnum getPutCurrency();
	BigDecimal getNotionalQuantityLeg1();
	BigDecimal getNotionalQuantityLeg2();
	String getQuantityFrequencyLeg1();
	String getQuantityFrequencyLeg2();
	Integer getQuantityFrequencyMultiplierLeg1();
	Integer getQuantityFrequencyMultiplierLeg2();
	String getQuantityUnitOfMeasureLeg1();
	String getQuantityUnitOfMeasureLeg2();
	BigDecimal getTotalNotionalQuantityLeg1();
	BigDecimal getTotalNotionalQuantityLeg2();
	Boolean getPackageIndicator();
	BigDecimal getPackageTransactionPrice();
	ISOCurrencyCodeEnum getPackageTransactionPriceCurrency();
	String getPackageTransactionPriceNotation();
	BigDecimal getPackageTransactionSpread();
	ISOCurrencyCodeEnum getPackageTransactionSpreadCurrency();
	String getPackageTransactionSpreadNotation();
	InterestComputationMethod4Code getFixedRateDayCountConventionLeg1();
	InterestComputationMethod4Code getFixedRateDayCountConventionLeg2();
	InterestComputationMethod4Code getFloatingRateDayCountConventionLeg1();
	InterestComputationMethod4Code getFloatingRateDayCountConventionLeg2();
	String getFloatingRateResetFrequencyPeriodLeg1();
	String getFloatingRateResetFrequencyPeriodLeg2();
	Integer getFloatingRateResetFrequencyPeriodMultiplierLeg1();
	Integer getFloatingRateResetFrequencyPeriodMultiplierLeg2();
	List<? extends OtherPaymentReport> getOtherPayment();
	String getFixedRatePaymentFrequencyPeriodLeg1();
	String getFixedRatePaymentFrequencyPeriodLeg2();
	String getFloatingRatePaymentFrequencyPeriodLeg1();
	String getFloatingRatePaymentFrequencyPeriodLeg2();
	Integer getFixedRatePaymentFrequencyPeriodMultiplierLeg1();
	Integer getFixedRatePaymentFrequencyPeriodMultiplierLeg2();
	Integer getFloatingRatePaymentFrequencyPeriodMultiplierLeg1();
	Integer getFloatingRatePaymentFrequencyPeriodMultiplierLeg2();
	BigDecimal getExchangeRateLeg1();
	String getExchangeRateBasis();
	BigDecimal getFixedRateLeg1();
	BigDecimal getFixedRateLeg2();
	Boolean getPostPricedSwapIndicator();
	BigDecimal getPrice();
	ISOCurrencyCodeEnum getPriceCurrency();
	String getPriceNotation();
	String getPriceUnitOfMeasure();
	BigDecimal getSpreadLeg1();
	BigDecimal getSpreadLeg2();
	ISOCurrencyCodeEnum getSpreadCurrencyLeg1();
	ISOCurrencyCodeEnum getSpreadCurrencyLeg2();
	String getSpreadNotationLeg1();
	String getSpreadNotationLeg2();
	BigDecimal getStrikePrice();
	String getStrikePriceCurrency();
	String getStrikePriceNotation();
	BigDecimal getOptionPremiumAmount();
	ISOCurrencyCodeEnum getOptionPremiumCurrency();
	Date getFirstExerciseDate();
	BigDecimal getIndexFactor();
	String getEmbeddedOptionType();
	String getUniqueProductIdentifier();
	ISOCurrencyCodeEnum getSettlementCurrencyLeg1();
	ISOCurrencyCodeEnum getSettlementCurrencyLeg2();
	String getSettlementLocationLeg1();
	String getSettlementLocationLeg2();
	Boolean getNonStandardizedTermIndicator();
	Boolean getBlockTradeElectionIndicator();
	Date getEffectiveDate();
	Date getExpirationDate();
	ZonedDateTime getExecutionTimestamp();
	ZonedDateTime getReportingTimestamp();
	String getPlatformIdentifier();
	Boolean getPrimeBrokerageTransactionIndicator();
	String getUniqueSwapIdentifier();
	String getUniqueTransactionIdentifier();
	CFTCGuidebookPart43 getCftcGuidebook();
	CFTCNonReportablePart43 getNonReportable();

	/*********************** Build Methods  ***********************/
	CFTCPart43TransactionReport build();
	
	CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder toBuilder();
	
	static CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder builder() {
		return new CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends CFTCPart43TransactionReport> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends CFTCPart43TransactionReport> getType() {
		return CFTCPart43TransactionReport.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("cleared"), String.class, getCleared(), this);
		processor.processBasic(path.newSubPath("counterparty1"), String.class, getCounterparty1(), this);
		processor.processBasic(path.newSubPath("counterparty2"), String.class, getCounterparty2(), this);
		processor.processBasic(path.newSubPath("counterparty2IdentifierSource"), String.class, getCounterparty2IdentifierSource(), this);
		processor.processBasic(path.newSubPath("submitterIdentifier"), String.class, getSubmitterIdentifier(), this);
		processor.processBasic(path.newSubPath("customBasketIndicator"), Boolean.class, getCustomBasketIndicator(), this);
		processor.processBasic(path.newSubPath("actionType"), String.class, getActionType(), this);
		processor.processBasic(path.newSubPath("eventType"), String.class, getEventType(), this);
		processor.processBasic(path.newSubPath("amendmentIndicator"), Boolean.class, getAmendmentIndicator(), this);
		processor.processBasic(path.newSubPath("eventTimestamp"), ZonedDateTime.class, getEventTimestamp(), this);
		processor.processBasic(path.newSubPath("notionalAmountLeg1"), BigDecimal.class, getNotionalAmountLeg1(), this);
		processor.processBasic(path.newSubPath("notionalAmountLeg2"), BigDecimal.class, getNotionalAmountLeg2(), this);
		processor.processBasic(path.newSubPath("notionalCurrencyLeg1"), ISOCurrencyCodeEnum.class, getNotionalCurrencyLeg1(), this);
		processor.processBasic(path.newSubPath("notionalCurrencyLeg2"), ISOCurrencyCodeEnum.class, getNotionalCurrencyLeg2(), this);
		processRosetta(path.newSubPath("notionalAmountScheduleLeg1"), processor, NotionalAmountScheduleLeg1Report.class, getNotionalAmountScheduleLeg1());
		processRosetta(path.newSubPath("notionalAmountScheduleLeg2"), processor, NotionalAmountScheduleLeg2Report.class, getNotionalAmountScheduleLeg2());
		processor.processBasic(path.newSubPath("callAmount"), BigDecimal.class, getCallAmount(), this);
		processor.processBasic(path.newSubPath("callCurrency"), ISOCurrencyCodeEnum.class, getCallCurrency(), this);
		processor.processBasic(path.newSubPath("putAmount"), BigDecimal.class, getPutAmount(), this);
		processor.processBasic(path.newSubPath("putCurrency"), ISOCurrencyCodeEnum.class, getPutCurrency(), this);
		processor.processBasic(path.newSubPath("notionalQuantityLeg1"), BigDecimal.class, getNotionalQuantityLeg1(), this);
		processor.processBasic(path.newSubPath("notionalQuantityLeg2"), BigDecimal.class, getNotionalQuantityLeg2(), this);
		processor.processBasic(path.newSubPath("quantityFrequencyLeg1"), String.class, getQuantityFrequencyLeg1(), this);
		processor.processBasic(path.newSubPath("quantityFrequencyLeg2"), String.class, getQuantityFrequencyLeg2(), this);
		processor.processBasic(path.newSubPath("quantityFrequencyMultiplierLeg1"), Integer.class, getQuantityFrequencyMultiplierLeg1(), this);
		processor.processBasic(path.newSubPath("quantityFrequencyMultiplierLeg2"), Integer.class, getQuantityFrequencyMultiplierLeg2(), this);
		processor.processBasic(path.newSubPath("quantityUnitOfMeasureLeg1"), String.class, getQuantityUnitOfMeasureLeg1(), this);
		processor.processBasic(path.newSubPath("quantityUnitOfMeasureLeg2"), String.class, getQuantityUnitOfMeasureLeg2(), this);
		processor.processBasic(path.newSubPath("totalNotionalQuantityLeg1"), BigDecimal.class, getTotalNotionalQuantityLeg1(), this);
		processor.processBasic(path.newSubPath("totalNotionalQuantityLeg2"), BigDecimal.class, getTotalNotionalQuantityLeg2(), this);
		processor.processBasic(path.newSubPath("packageIndicator"), Boolean.class, getPackageIndicator(), this);
		processor.processBasic(path.newSubPath("packageTransactionPrice"), BigDecimal.class, getPackageTransactionPrice(), this);
		processor.processBasic(path.newSubPath("packageTransactionPriceCurrency"), ISOCurrencyCodeEnum.class, getPackageTransactionPriceCurrency(), this);
		processor.processBasic(path.newSubPath("packageTransactionPriceNotation"), String.class, getPackageTransactionPriceNotation(), this);
		processor.processBasic(path.newSubPath("packageTransactionSpread"), BigDecimal.class, getPackageTransactionSpread(), this);
		processor.processBasic(path.newSubPath("packageTransactionSpreadCurrency"), ISOCurrencyCodeEnum.class, getPackageTransactionSpreadCurrency(), this);
		processor.processBasic(path.newSubPath("packageTransactionSpreadNotation"), String.class, getPackageTransactionSpreadNotation(), this);
		processor.processBasic(path.newSubPath("fixedRateDayCountConventionLeg1"), InterestComputationMethod4Code.class, getFixedRateDayCountConventionLeg1(), this);
		processor.processBasic(path.newSubPath("fixedRateDayCountConventionLeg2"), InterestComputationMethod4Code.class, getFixedRateDayCountConventionLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRateDayCountConventionLeg1"), InterestComputationMethod4Code.class, getFloatingRateDayCountConventionLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRateDayCountConventionLeg2"), InterestComputationMethod4Code.class, getFloatingRateDayCountConventionLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodLeg1"), String.class, getFloatingRateResetFrequencyPeriodLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodLeg2"), String.class, getFloatingRateResetFrequencyPeriodLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodMultiplierLeg1"), Integer.class, getFloatingRateResetFrequencyPeriodMultiplierLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodMultiplierLeg2"), Integer.class, getFloatingRateResetFrequencyPeriodMultiplierLeg2(), this);
		processRosetta(path.newSubPath("otherPayment"), processor, OtherPaymentReport.class, getOtherPayment());
		processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodLeg1"), String.class, getFixedRatePaymentFrequencyPeriodLeg1(), this);
		processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodLeg2"), String.class, getFixedRatePaymentFrequencyPeriodLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodLeg1"), String.class, getFloatingRatePaymentFrequencyPeriodLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodLeg2"), String.class, getFloatingRatePaymentFrequencyPeriodLeg2(), this);
		processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodMultiplierLeg1"), Integer.class, getFixedRatePaymentFrequencyPeriodMultiplierLeg1(), this);
		processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodMultiplierLeg2"), Integer.class, getFixedRatePaymentFrequencyPeriodMultiplierLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodMultiplierLeg1"), Integer.class, getFloatingRatePaymentFrequencyPeriodMultiplierLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodMultiplierLeg2"), Integer.class, getFloatingRatePaymentFrequencyPeriodMultiplierLeg2(), this);
		processor.processBasic(path.newSubPath("exchangeRateLeg1"), BigDecimal.class, getExchangeRateLeg1(), this);
		processor.processBasic(path.newSubPath("exchangeRateBasis"), String.class, getExchangeRateBasis(), this);
		processor.processBasic(path.newSubPath("fixedRateLeg1"), BigDecimal.class, getFixedRateLeg1(), this);
		processor.processBasic(path.newSubPath("fixedRateLeg2"), BigDecimal.class, getFixedRateLeg2(), this);
		processor.processBasic(path.newSubPath("postPricedSwapIndicator"), Boolean.class, getPostPricedSwapIndicator(), this);
		processor.processBasic(path.newSubPath("price"), BigDecimal.class, getPrice(), this);
		processor.processBasic(path.newSubPath("priceCurrency"), ISOCurrencyCodeEnum.class, getPriceCurrency(), this);
		processor.processBasic(path.newSubPath("priceNotation"), String.class, getPriceNotation(), this);
		processor.processBasic(path.newSubPath("priceUnitOfMeasure"), String.class, getPriceUnitOfMeasure(), this);
		processor.processBasic(path.newSubPath("spreadLeg1"), BigDecimal.class, getSpreadLeg1(), this);
		processor.processBasic(path.newSubPath("spreadLeg2"), BigDecimal.class, getSpreadLeg2(), this);
		processor.processBasic(path.newSubPath("spreadCurrencyLeg1"), ISOCurrencyCodeEnum.class, getSpreadCurrencyLeg1(), this);
		processor.processBasic(path.newSubPath("spreadCurrencyLeg2"), ISOCurrencyCodeEnum.class, getSpreadCurrencyLeg2(), this);
		processor.processBasic(path.newSubPath("spreadNotationLeg1"), String.class, getSpreadNotationLeg1(), this);
		processor.processBasic(path.newSubPath("spreadNotationLeg2"), String.class, getSpreadNotationLeg2(), this);
		processor.processBasic(path.newSubPath("strikePrice"), BigDecimal.class, getStrikePrice(), this);
		processor.processBasic(path.newSubPath("strikePriceCurrency"), String.class, getStrikePriceCurrency(), this);
		processor.processBasic(path.newSubPath("strikePriceNotation"), String.class, getStrikePriceNotation(), this);
		processor.processBasic(path.newSubPath("optionPremiumAmount"), BigDecimal.class, getOptionPremiumAmount(), this);
		processor.processBasic(path.newSubPath("optionPremiumCurrency"), ISOCurrencyCodeEnum.class, getOptionPremiumCurrency(), this);
		processor.processBasic(path.newSubPath("firstExerciseDate"), Date.class, getFirstExerciseDate(), this);
		processor.processBasic(path.newSubPath("indexFactor"), BigDecimal.class, getIndexFactor(), this);
		processor.processBasic(path.newSubPath("embeddedOptionType"), String.class, getEmbeddedOptionType(), this);
		processor.processBasic(path.newSubPath("uniqueProductIdentifier"), String.class, getUniqueProductIdentifier(), this);
		processor.processBasic(path.newSubPath("settlementCurrencyLeg1"), ISOCurrencyCodeEnum.class, getSettlementCurrencyLeg1(), this);
		processor.processBasic(path.newSubPath("settlementCurrencyLeg2"), ISOCurrencyCodeEnum.class, getSettlementCurrencyLeg2(), this);
		processor.processBasic(path.newSubPath("settlementLocationLeg1"), String.class, getSettlementLocationLeg1(), this);
		processor.processBasic(path.newSubPath("settlementLocationLeg2"), String.class, getSettlementLocationLeg2(), this);
		processor.processBasic(path.newSubPath("nonStandardizedTermIndicator"), Boolean.class, getNonStandardizedTermIndicator(), this);
		processor.processBasic(path.newSubPath("blockTradeElectionIndicator"), Boolean.class, getBlockTradeElectionIndicator(), this);
		processor.processBasic(path.newSubPath("effectiveDate"), Date.class, getEffectiveDate(), this);
		processor.processBasic(path.newSubPath("expirationDate"), Date.class, getExpirationDate(), this);
		processor.processBasic(path.newSubPath("executionTimestamp"), ZonedDateTime.class, getExecutionTimestamp(), this);
		processor.processBasic(path.newSubPath("reportingTimestamp"), ZonedDateTime.class, getReportingTimestamp(), this);
		processor.processBasic(path.newSubPath("platformIdentifier"), String.class, getPlatformIdentifier(), this);
		processor.processBasic(path.newSubPath("primeBrokerageTransactionIndicator"), Boolean.class, getPrimeBrokerageTransactionIndicator(), this);
		processor.processBasic(path.newSubPath("uniqueSwapIdentifier"), String.class, getUniqueSwapIdentifier(), this);
		processor.processBasic(path.newSubPath("uniqueTransactionIdentifier"), String.class, getUniqueTransactionIdentifier(), this);
		processRosetta(path.newSubPath("cftcGuidebook"), processor, CFTCGuidebookPart43.class, getCftcGuidebook());
		processRosetta(path.newSubPath("nonReportable"), processor, CFTCNonReportablePart43.class, getNonReportable());
	}
	

	/*********************** Builder Interface  ***********************/
	interface CFTCPart43TransactionReportBuilder extends CFTCPart43TransactionReport, RosettaModelObjectBuilder {
		NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder getOrCreateNotionalAmountScheduleLeg1(int _index);
		List<? extends NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> getNotionalAmountScheduleLeg1();
		NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder getOrCreateNotionalAmountScheduleLeg2(int _index);
		List<? extends NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> getNotionalAmountScheduleLeg2();
		OtherPaymentReport.OtherPaymentReportBuilder getOrCreateOtherPayment(int _index);
		List<? extends OtherPaymentReport.OtherPaymentReportBuilder> getOtherPayment();
		CFTCGuidebookPart43.CFTCGuidebookPart43Builder getOrCreateCftcGuidebook();
		CFTCGuidebookPart43.CFTCGuidebookPart43Builder getCftcGuidebook();
		CFTCNonReportablePart43.CFTCNonReportablePart43Builder getOrCreateNonReportable();
		CFTCNonReportablePart43.CFTCNonReportablePart43Builder getNonReportable();
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCleared(String cleared);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCounterparty1(String counterparty1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCounterparty2(String counterparty2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCounterparty2IdentifierSource(String counterparty2IdentifierSource);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSubmitterIdentifier(String submitterIdentifier);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCustomBasketIndicator(Boolean customBasketIndicator);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setActionType(String actionType);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setEventType(String eventType);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setAmendmentIndicator(Boolean amendmentIndicator);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setEventTimestamp(ZonedDateTime eventTimestamp);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalAmountLeg1(BigDecimal notionalAmountLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalAmountLeg2(BigDecimal notionalAmountLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalCurrencyLeg1(ISOCurrencyCodeEnum notionalCurrencyLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalCurrencyLeg2(ISOCurrencyCodeEnum notionalCurrencyLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addNotionalAmountScheduleLeg1(NotionalAmountScheduleLeg1Report notionalAmountScheduleLeg10);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addNotionalAmountScheduleLeg1(NotionalAmountScheduleLeg1Report notionalAmountScheduleLeg11, int _idx);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addNotionalAmountScheduleLeg1(List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg12);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalAmountScheduleLeg1(List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg13);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addNotionalAmountScheduleLeg2(NotionalAmountScheduleLeg2Report notionalAmountScheduleLeg20);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addNotionalAmountScheduleLeg2(NotionalAmountScheduleLeg2Report notionalAmountScheduleLeg21, int _idx);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addNotionalAmountScheduleLeg2(List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg22);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalAmountScheduleLeg2(List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg23);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCallAmount(BigDecimal callAmount);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCallCurrency(ISOCurrencyCodeEnum callCurrency);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPutAmount(BigDecimal putAmount);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPutCurrency(ISOCurrencyCodeEnum putCurrency);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalQuantityLeg1(BigDecimal notionalQuantityLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalQuantityLeg2(BigDecimal notionalQuantityLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setQuantityFrequencyLeg1(String quantityFrequencyLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setQuantityFrequencyLeg2(String quantityFrequencyLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setQuantityFrequencyMultiplierLeg1(Integer quantityFrequencyMultiplierLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setQuantityFrequencyMultiplierLeg2(Integer quantityFrequencyMultiplierLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setQuantityUnitOfMeasureLeg1(String quantityUnitOfMeasureLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setQuantityUnitOfMeasureLeg2(String quantityUnitOfMeasureLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setTotalNotionalQuantityLeg1(BigDecimal totalNotionalQuantityLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setTotalNotionalQuantityLeg2(BigDecimal totalNotionalQuantityLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPackageIndicator(Boolean packageIndicator);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPackageTransactionPrice(BigDecimal packageTransactionPrice);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPackageTransactionPriceCurrency(ISOCurrencyCodeEnum packageTransactionPriceCurrency);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPackageTransactionPriceNotation(String packageTransactionPriceNotation);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPackageTransactionSpread(BigDecimal packageTransactionSpread);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPackageTransactionSpreadCurrency(ISOCurrencyCodeEnum packageTransactionSpreadCurrency);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPackageTransactionSpreadNotation(String packageTransactionSpreadNotation);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRateDayCountConventionLeg1(InterestComputationMethod4Code fixedRateDayCountConventionLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRateDayCountConventionLeg2(InterestComputationMethod4Code fixedRateDayCountConventionLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRateDayCountConventionLeg1(InterestComputationMethod4Code floatingRateDayCountConventionLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRateDayCountConventionLeg2(InterestComputationMethod4Code floatingRateDayCountConventionLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRateResetFrequencyPeriodLeg1(String floatingRateResetFrequencyPeriodLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRateResetFrequencyPeriodLeg2(String floatingRateResetFrequencyPeriodLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRateResetFrequencyPeriodMultiplierLeg1(Integer floatingRateResetFrequencyPeriodMultiplierLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRateResetFrequencyPeriodMultiplierLeg2(Integer floatingRateResetFrequencyPeriodMultiplierLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment0);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment1, int _idx);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addOtherPayment(List<? extends OtherPaymentReport> otherPayment2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setOtherPayment(List<? extends OtherPaymentReport> otherPayment3);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRatePaymentFrequencyPeriodLeg1(String fixedRatePaymentFrequencyPeriodLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRatePaymentFrequencyPeriodLeg2(String fixedRatePaymentFrequencyPeriodLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRatePaymentFrequencyPeriodLeg1(String floatingRatePaymentFrequencyPeriodLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRatePaymentFrequencyPeriodLeg2(String floatingRatePaymentFrequencyPeriodLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRatePaymentFrequencyPeriodMultiplierLeg1(Integer fixedRatePaymentFrequencyPeriodMultiplierLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRatePaymentFrequencyPeriodMultiplierLeg2(Integer fixedRatePaymentFrequencyPeriodMultiplierLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRatePaymentFrequencyPeriodMultiplierLeg1(Integer floatingRatePaymentFrequencyPeriodMultiplierLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRatePaymentFrequencyPeriodMultiplierLeg2(Integer floatingRatePaymentFrequencyPeriodMultiplierLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setExchangeRateLeg1(BigDecimal exchangeRateLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setExchangeRateBasis(String exchangeRateBasis);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRateLeg1(BigDecimal fixedRateLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRateLeg2(BigDecimal fixedRateLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPostPricedSwapIndicator(Boolean postPricedSwapIndicator);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPrice(BigDecimal price);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPriceCurrency(ISOCurrencyCodeEnum priceCurrency);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPriceNotation(String priceNotation);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPriceUnitOfMeasure(String priceUnitOfMeasure);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSpreadLeg1(BigDecimal spreadLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSpreadLeg2(BigDecimal spreadLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSpreadCurrencyLeg1(ISOCurrencyCodeEnum spreadCurrencyLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSpreadCurrencyLeg2(ISOCurrencyCodeEnum spreadCurrencyLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSpreadNotationLeg1(String spreadNotationLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSpreadNotationLeg2(String spreadNotationLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setStrikePrice(BigDecimal strikePrice);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setStrikePriceCurrency(String strikePriceCurrency);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setStrikePriceNotation(String strikePriceNotation);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setOptionPremiumAmount(BigDecimal optionPremiumAmount);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setOptionPremiumCurrency(ISOCurrencyCodeEnum optionPremiumCurrency);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFirstExerciseDate(Date firstExerciseDate);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setIndexFactor(BigDecimal indexFactor);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setEmbeddedOptionType(String embeddedOptionType);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setUniqueProductIdentifier(String uniqueProductIdentifier);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSettlementCurrencyLeg1(ISOCurrencyCodeEnum settlementCurrencyLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSettlementCurrencyLeg2(ISOCurrencyCodeEnum settlementCurrencyLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSettlementLocationLeg1(String settlementLocationLeg1);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSettlementLocationLeg2(String settlementLocationLeg2);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNonStandardizedTermIndicator(Boolean nonStandardizedTermIndicator);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setBlockTradeElectionIndicator(Boolean blockTradeElectionIndicator);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setEffectiveDate(Date effectiveDate);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setExpirationDate(Date expirationDate);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setExecutionTimestamp(ZonedDateTime executionTimestamp);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setReportingTimestamp(ZonedDateTime reportingTimestamp);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPlatformIdentifier(String platformIdentifier);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPrimeBrokerageTransactionIndicator(Boolean primeBrokerageTransactionIndicator);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setUniqueSwapIdentifier(String uniqueSwapIdentifier);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setUniqueTransactionIdentifier(String uniqueTransactionIdentifier);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCftcGuidebook(CFTCGuidebookPart43 cftcGuidebook);
		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNonReportable(CFTCNonReportablePart43 nonReportable);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("cleared"), String.class, getCleared(), this);
			processor.processBasic(path.newSubPath("counterparty1"), String.class, getCounterparty1(), this);
			processor.processBasic(path.newSubPath("counterparty2"), String.class, getCounterparty2(), this);
			processor.processBasic(path.newSubPath("counterparty2IdentifierSource"), String.class, getCounterparty2IdentifierSource(), this);
			processor.processBasic(path.newSubPath("submitterIdentifier"), String.class, getSubmitterIdentifier(), this);
			processor.processBasic(path.newSubPath("customBasketIndicator"), Boolean.class, getCustomBasketIndicator(), this);
			processor.processBasic(path.newSubPath("actionType"), String.class, getActionType(), this);
			processor.processBasic(path.newSubPath("eventType"), String.class, getEventType(), this);
			processor.processBasic(path.newSubPath("amendmentIndicator"), Boolean.class, getAmendmentIndicator(), this);
			processor.processBasic(path.newSubPath("eventTimestamp"), ZonedDateTime.class, getEventTimestamp(), this);
			processor.processBasic(path.newSubPath("notionalAmountLeg1"), BigDecimal.class, getNotionalAmountLeg1(), this);
			processor.processBasic(path.newSubPath("notionalAmountLeg2"), BigDecimal.class, getNotionalAmountLeg2(), this);
			processor.processBasic(path.newSubPath("notionalCurrencyLeg1"), ISOCurrencyCodeEnum.class, getNotionalCurrencyLeg1(), this);
			processor.processBasic(path.newSubPath("notionalCurrencyLeg2"), ISOCurrencyCodeEnum.class, getNotionalCurrencyLeg2(), this);
			processRosetta(path.newSubPath("notionalAmountScheduleLeg1"), processor, NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder.class, getNotionalAmountScheduleLeg1());
			processRosetta(path.newSubPath("notionalAmountScheduleLeg2"), processor, NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder.class, getNotionalAmountScheduleLeg2());
			processor.processBasic(path.newSubPath("callAmount"), BigDecimal.class, getCallAmount(), this);
			processor.processBasic(path.newSubPath("callCurrency"), ISOCurrencyCodeEnum.class, getCallCurrency(), this);
			processor.processBasic(path.newSubPath("putAmount"), BigDecimal.class, getPutAmount(), this);
			processor.processBasic(path.newSubPath("putCurrency"), ISOCurrencyCodeEnum.class, getPutCurrency(), this);
			processor.processBasic(path.newSubPath("notionalQuantityLeg1"), BigDecimal.class, getNotionalQuantityLeg1(), this);
			processor.processBasic(path.newSubPath("notionalQuantityLeg2"), BigDecimal.class, getNotionalQuantityLeg2(), this);
			processor.processBasic(path.newSubPath("quantityFrequencyLeg1"), String.class, getQuantityFrequencyLeg1(), this);
			processor.processBasic(path.newSubPath("quantityFrequencyLeg2"), String.class, getQuantityFrequencyLeg2(), this);
			processor.processBasic(path.newSubPath("quantityFrequencyMultiplierLeg1"), Integer.class, getQuantityFrequencyMultiplierLeg1(), this);
			processor.processBasic(path.newSubPath("quantityFrequencyMultiplierLeg2"), Integer.class, getQuantityFrequencyMultiplierLeg2(), this);
			processor.processBasic(path.newSubPath("quantityUnitOfMeasureLeg1"), String.class, getQuantityUnitOfMeasureLeg1(), this);
			processor.processBasic(path.newSubPath("quantityUnitOfMeasureLeg2"), String.class, getQuantityUnitOfMeasureLeg2(), this);
			processor.processBasic(path.newSubPath("totalNotionalQuantityLeg1"), BigDecimal.class, getTotalNotionalQuantityLeg1(), this);
			processor.processBasic(path.newSubPath("totalNotionalQuantityLeg2"), BigDecimal.class, getTotalNotionalQuantityLeg2(), this);
			processor.processBasic(path.newSubPath("packageIndicator"), Boolean.class, getPackageIndicator(), this);
			processor.processBasic(path.newSubPath("packageTransactionPrice"), BigDecimal.class, getPackageTransactionPrice(), this);
			processor.processBasic(path.newSubPath("packageTransactionPriceCurrency"), ISOCurrencyCodeEnum.class, getPackageTransactionPriceCurrency(), this);
			processor.processBasic(path.newSubPath("packageTransactionPriceNotation"), String.class, getPackageTransactionPriceNotation(), this);
			processor.processBasic(path.newSubPath("packageTransactionSpread"), BigDecimal.class, getPackageTransactionSpread(), this);
			processor.processBasic(path.newSubPath("packageTransactionSpreadCurrency"), ISOCurrencyCodeEnum.class, getPackageTransactionSpreadCurrency(), this);
			processor.processBasic(path.newSubPath("packageTransactionSpreadNotation"), String.class, getPackageTransactionSpreadNotation(), this);
			processor.processBasic(path.newSubPath("fixedRateDayCountConventionLeg1"), InterestComputationMethod4Code.class, getFixedRateDayCountConventionLeg1(), this);
			processor.processBasic(path.newSubPath("fixedRateDayCountConventionLeg2"), InterestComputationMethod4Code.class, getFixedRateDayCountConventionLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRateDayCountConventionLeg1"), InterestComputationMethod4Code.class, getFloatingRateDayCountConventionLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRateDayCountConventionLeg2"), InterestComputationMethod4Code.class, getFloatingRateDayCountConventionLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodLeg1"), String.class, getFloatingRateResetFrequencyPeriodLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodLeg2"), String.class, getFloatingRateResetFrequencyPeriodLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodMultiplierLeg1"), Integer.class, getFloatingRateResetFrequencyPeriodMultiplierLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodMultiplierLeg2"), Integer.class, getFloatingRateResetFrequencyPeriodMultiplierLeg2(), this);
			processRosetta(path.newSubPath("otherPayment"), processor, OtherPaymentReport.OtherPaymentReportBuilder.class, getOtherPayment());
			processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodLeg1"), String.class, getFixedRatePaymentFrequencyPeriodLeg1(), this);
			processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodLeg2"), String.class, getFixedRatePaymentFrequencyPeriodLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodLeg1"), String.class, getFloatingRatePaymentFrequencyPeriodLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodLeg2"), String.class, getFloatingRatePaymentFrequencyPeriodLeg2(), this);
			processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodMultiplierLeg1"), Integer.class, getFixedRatePaymentFrequencyPeriodMultiplierLeg1(), this);
			processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodMultiplierLeg2"), Integer.class, getFixedRatePaymentFrequencyPeriodMultiplierLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodMultiplierLeg1"), Integer.class, getFloatingRatePaymentFrequencyPeriodMultiplierLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodMultiplierLeg2"), Integer.class, getFloatingRatePaymentFrequencyPeriodMultiplierLeg2(), this);
			processor.processBasic(path.newSubPath("exchangeRateLeg1"), BigDecimal.class, getExchangeRateLeg1(), this);
			processor.processBasic(path.newSubPath("exchangeRateBasis"), String.class, getExchangeRateBasis(), this);
			processor.processBasic(path.newSubPath("fixedRateLeg1"), BigDecimal.class, getFixedRateLeg1(), this);
			processor.processBasic(path.newSubPath("fixedRateLeg2"), BigDecimal.class, getFixedRateLeg2(), this);
			processor.processBasic(path.newSubPath("postPricedSwapIndicator"), Boolean.class, getPostPricedSwapIndicator(), this);
			processor.processBasic(path.newSubPath("price"), BigDecimal.class, getPrice(), this);
			processor.processBasic(path.newSubPath("priceCurrency"), ISOCurrencyCodeEnum.class, getPriceCurrency(), this);
			processor.processBasic(path.newSubPath("priceNotation"), String.class, getPriceNotation(), this);
			processor.processBasic(path.newSubPath("priceUnitOfMeasure"), String.class, getPriceUnitOfMeasure(), this);
			processor.processBasic(path.newSubPath("spreadLeg1"), BigDecimal.class, getSpreadLeg1(), this);
			processor.processBasic(path.newSubPath("spreadLeg2"), BigDecimal.class, getSpreadLeg2(), this);
			processor.processBasic(path.newSubPath("spreadCurrencyLeg1"), ISOCurrencyCodeEnum.class, getSpreadCurrencyLeg1(), this);
			processor.processBasic(path.newSubPath("spreadCurrencyLeg2"), ISOCurrencyCodeEnum.class, getSpreadCurrencyLeg2(), this);
			processor.processBasic(path.newSubPath("spreadNotationLeg1"), String.class, getSpreadNotationLeg1(), this);
			processor.processBasic(path.newSubPath("spreadNotationLeg2"), String.class, getSpreadNotationLeg2(), this);
			processor.processBasic(path.newSubPath("strikePrice"), BigDecimal.class, getStrikePrice(), this);
			processor.processBasic(path.newSubPath("strikePriceCurrency"), String.class, getStrikePriceCurrency(), this);
			processor.processBasic(path.newSubPath("strikePriceNotation"), String.class, getStrikePriceNotation(), this);
			processor.processBasic(path.newSubPath("optionPremiumAmount"), BigDecimal.class, getOptionPremiumAmount(), this);
			processor.processBasic(path.newSubPath("optionPremiumCurrency"), ISOCurrencyCodeEnum.class, getOptionPremiumCurrency(), this);
			processor.processBasic(path.newSubPath("firstExerciseDate"), Date.class, getFirstExerciseDate(), this);
			processor.processBasic(path.newSubPath("indexFactor"), BigDecimal.class, getIndexFactor(), this);
			processor.processBasic(path.newSubPath("embeddedOptionType"), String.class, getEmbeddedOptionType(), this);
			processor.processBasic(path.newSubPath("uniqueProductIdentifier"), String.class, getUniqueProductIdentifier(), this);
			processor.processBasic(path.newSubPath("settlementCurrencyLeg1"), ISOCurrencyCodeEnum.class, getSettlementCurrencyLeg1(), this);
			processor.processBasic(path.newSubPath("settlementCurrencyLeg2"), ISOCurrencyCodeEnum.class, getSettlementCurrencyLeg2(), this);
			processor.processBasic(path.newSubPath("settlementLocationLeg1"), String.class, getSettlementLocationLeg1(), this);
			processor.processBasic(path.newSubPath("settlementLocationLeg2"), String.class, getSettlementLocationLeg2(), this);
			processor.processBasic(path.newSubPath("nonStandardizedTermIndicator"), Boolean.class, getNonStandardizedTermIndicator(), this);
			processor.processBasic(path.newSubPath("blockTradeElectionIndicator"), Boolean.class, getBlockTradeElectionIndicator(), this);
			processor.processBasic(path.newSubPath("effectiveDate"), Date.class, getEffectiveDate(), this);
			processor.processBasic(path.newSubPath("expirationDate"), Date.class, getExpirationDate(), this);
			processor.processBasic(path.newSubPath("executionTimestamp"), ZonedDateTime.class, getExecutionTimestamp(), this);
			processor.processBasic(path.newSubPath("reportingTimestamp"), ZonedDateTime.class, getReportingTimestamp(), this);
			processor.processBasic(path.newSubPath("platformIdentifier"), String.class, getPlatformIdentifier(), this);
			processor.processBasic(path.newSubPath("primeBrokerageTransactionIndicator"), Boolean.class, getPrimeBrokerageTransactionIndicator(), this);
			processor.processBasic(path.newSubPath("uniqueSwapIdentifier"), String.class, getUniqueSwapIdentifier(), this);
			processor.processBasic(path.newSubPath("uniqueTransactionIdentifier"), String.class, getUniqueTransactionIdentifier(), this);
			processRosetta(path.newSubPath("cftcGuidebook"), processor, CFTCGuidebookPart43.CFTCGuidebookPart43Builder.class, getCftcGuidebook());
			processRosetta(path.newSubPath("nonReportable"), processor, CFTCNonReportablePart43.CFTCNonReportablePart43Builder.class, getNonReportable());
		}
		

		CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder prune();
	}

	/*********************** Immutable Implementation of CFTCPart43TransactionReport  ***********************/
	class CFTCPart43TransactionReportImpl implements CFTCPart43TransactionReport {
		private final String cleared;
		private final String counterparty1;
		private final String counterparty2;
		private final String counterparty2IdentifierSource;
		private final String submitterIdentifier;
		private final Boolean customBasketIndicator;
		private final String actionType;
		private final String eventType;
		private final Boolean amendmentIndicator;
		private final ZonedDateTime eventTimestamp;
		private final BigDecimal notionalAmountLeg1;
		private final BigDecimal notionalAmountLeg2;
		private final ISOCurrencyCodeEnum notionalCurrencyLeg1;
		private final ISOCurrencyCodeEnum notionalCurrencyLeg2;
		private final List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg1;
		private final List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg2;
		private final BigDecimal callAmount;
		private final ISOCurrencyCodeEnum callCurrency;
		private final BigDecimal putAmount;
		private final ISOCurrencyCodeEnum putCurrency;
		private final BigDecimal notionalQuantityLeg1;
		private final BigDecimal notionalQuantityLeg2;
		private final String quantityFrequencyLeg1;
		private final String quantityFrequencyLeg2;
		private final Integer quantityFrequencyMultiplierLeg1;
		private final Integer quantityFrequencyMultiplierLeg2;
		private final String quantityUnitOfMeasureLeg1;
		private final String quantityUnitOfMeasureLeg2;
		private final BigDecimal totalNotionalQuantityLeg1;
		private final BigDecimal totalNotionalQuantityLeg2;
		private final Boolean packageIndicator;
		private final BigDecimal packageTransactionPrice;
		private final ISOCurrencyCodeEnum packageTransactionPriceCurrency;
		private final String packageTransactionPriceNotation;
		private final BigDecimal packageTransactionSpread;
		private final ISOCurrencyCodeEnum packageTransactionSpreadCurrency;
		private final String packageTransactionSpreadNotation;
		private final InterestComputationMethod4Code fixedRateDayCountConventionLeg1;
		private final InterestComputationMethod4Code fixedRateDayCountConventionLeg2;
		private final InterestComputationMethod4Code floatingRateDayCountConventionLeg1;
		private final InterestComputationMethod4Code floatingRateDayCountConventionLeg2;
		private final String floatingRateResetFrequencyPeriodLeg1;
		private final String floatingRateResetFrequencyPeriodLeg2;
		private final Integer floatingRateResetFrequencyPeriodMultiplierLeg1;
		private final Integer floatingRateResetFrequencyPeriodMultiplierLeg2;
		private final List<? extends OtherPaymentReport> otherPayment;
		private final String fixedRatePaymentFrequencyPeriodLeg1;
		private final String fixedRatePaymentFrequencyPeriodLeg2;
		private final String floatingRatePaymentFrequencyPeriodLeg1;
		private final String floatingRatePaymentFrequencyPeriodLeg2;
		private final Integer fixedRatePaymentFrequencyPeriodMultiplierLeg1;
		private final Integer fixedRatePaymentFrequencyPeriodMultiplierLeg2;
		private final Integer floatingRatePaymentFrequencyPeriodMultiplierLeg1;
		private final Integer floatingRatePaymentFrequencyPeriodMultiplierLeg2;
		private final BigDecimal exchangeRateLeg1;
		private final String exchangeRateBasis;
		private final BigDecimal fixedRateLeg1;
		private final BigDecimal fixedRateLeg2;
		private final Boolean postPricedSwapIndicator;
		private final BigDecimal price;
		private final ISOCurrencyCodeEnum priceCurrency;
		private final String priceNotation;
		private final String priceUnitOfMeasure;
		private final BigDecimal spreadLeg1;
		private final BigDecimal spreadLeg2;
		private final ISOCurrencyCodeEnum spreadCurrencyLeg1;
		private final ISOCurrencyCodeEnum spreadCurrencyLeg2;
		private final String spreadNotationLeg1;
		private final String spreadNotationLeg2;
		private final BigDecimal strikePrice;
		private final String strikePriceCurrency;
		private final String strikePriceNotation;
		private final BigDecimal optionPremiumAmount;
		private final ISOCurrencyCodeEnum optionPremiumCurrency;
		private final Date firstExerciseDate;
		private final BigDecimal indexFactor;
		private final String embeddedOptionType;
		private final String uniqueProductIdentifier;
		private final ISOCurrencyCodeEnum settlementCurrencyLeg1;
		private final ISOCurrencyCodeEnum settlementCurrencyLeg2;
		private final String settlementLocationLeg1;
		private final String settlementLocationLeg2;
		private final Boolean nonStandardizedTermIndicator;
		private final Boolean blockTradeElectionIndicator;
		private final Date effectiveDate;
		private final Date expirationDate;
		private final ZonedDateTime executionTimestamp;
		private final ZonedDateTime reportingTimestamp;
		private final String platformIdentifier;
		private final Boolean primeBrokerageTransactionIndicator;
		private final String uniqueSwapIdentifier;
		private final String uniqueTransactionIdentifier;
		private final CFTCGuidebookPart43 cftcGuidebook;
		private final CFTCNonReportablePart43 nonReportable;
		
		protected CFTCPart43TransactionReportImpl(CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder builder) {
			this.cleared = builder.getCleared();
			this.counterparty1 = builder.getCounterparty1();
			this.counterparty2 = builder.getCounterparty2();
			this.counterparty2IdentifierSource = builder.getCounterparty2IdentifierSource();
			this.submitterIdentifier = builder.getSubmitterIdentifier();
			this.customBasketIndicator = builder.getCustomBasketIndicator();
			this.actionType = builder.getActionType();
			this.eventType = builder.getEventType();
			this.amendmentIndicator = builder.getAmendmentIndicator();
			this.eventTimestamp = builder.getEventTimestamp();
			this.notionalAmountLeg1 = builder.getNotionalAmountLeg1();
			this.notionalAmountLeg2 = builder.getNotionalAmountLeg2();
			this.notionalCurrencyLeg1 = builder.getNotionalCurrencyLeg1();
			this.notionalCurrencyLeg2 = builder.getNotionalCurrencyLeg2();
			this.notionalAmountScheduleLeg1 = ofNullable(builder.getNotionalAmountScheduleLeg1()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.notionalAmountScheduleLeg2 = ofNullable(builder.getNotionalAmountScheduleLeg2()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.callAmount = builder.getCallAmount();
			this.callCurrency = builder.getCallCurrency();
			this.putAmount = builder.getPutAmount();
			this.putCurrency = builder.getPutCurrency();
			this.notionalQuantityLeg1 = builder.getNotionalQuantityLeg1();
			this.notionalQuantityLeg2 = builder.getNotionalQuantityLeg2();
			this.quantityFrequencyLeg1 = builder.getQuantityFrequencyLeg1();
			this.quantityFrequencyLeg2 = builder.getQuantityFrequencyLeg2();
			this.quantityFrequencyMultiplierLeg1 = builder.getQuantityFrequencyMultiplierLeg1();
			this.quantityFrequencyMultiplierLeg2 = builder.getQuantityFrequencyMultiplierLeg2();
			this.quantityUnitOfMeasureLeg1 = builder.getQuantityUnitOfMeasureLeg1();
			this.quantityUnitOfMeasureLeg2 = builder.getQuantityUnitOfMeasureLeg2();
			this.totalNotionalQuantityLeg1 = builder.getTotalNotionalQuantityLeg1();
			this.totalNotionalQuantityLeg2 = builder.getTotalNotionalQuantityLeg2();
			this.packageIndicator = builder.getPackageIndicator();
			this.packageTransactionPrice = builder.getPackageTransactionPrice();
			this.packageTransactionPriceCurrency = builder.getPackageTransactionPriceCurrency();
			this.packageTransactionPriceNotation = builder.getPackageTransactionPriceNotation();
			this.packageTransactionSpread = builder.getPackageTransactionSpread();
			this.packageTransactionSpreadCurrency = builder.getPackageTransactionSpreadCurrency();
			this.packageTransactionSpreadNotation = builder.getPackageTransactionSpreadNotation();
			this.fixedRateDayCountConventionLeg1 = builder.getFixedRateDayCountConventionLeg1();
			this.fixedRateDayCountConventionLeg2 = builder.getFixedRateDayCountConventionLeg2();
			this.floatingRateDayCountConventionLeg1 = builder.getFloatingRateDayCountConventionLeg1();
			this.floatingRateDayCountConventionLeg2 = builder.getFloatingRateDayCountConventionLeg2();
			this.floatingRateResetFrequencyPeriodLeg1 = builder.getFloatingRateResetFrequencyPeriodLeg1();
			this.floatingRateResetFrequencyPeriodLeg2 = builder.getFloatingRateResetFrequencyPeriodLeg2();
			this.floatingRateResetFrequencyPeriodMultiplierLeg1 = builder.getFloatingRateResetFrequencyPeriodMultiplierLeg1();
			this.floatingRateResetFrequencyPeriodMultiplierLeg2 = builder.getFloatingRateResetFrequencyPeriodMultiplierLeg2();
			this.otherPayment = ofNullable(builder.getOtherPayment()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.fixedRatePaymentFrequencyPeriodLeg1 = builder.getFixedRatePaymentFrequencyPeriodLeg1();
			this.fixedRatePaymentFrequencyPeriodLeg2 = builder.getFixedRatePaymentFrequencyPeriodLeg2();
			this.floatingRatePaymentFrequencyPeriodLeg1 = builder.getFloatingRatePaymentFrequencyPeriodLeg1();
			this.floatingRatePaymentFrequencyPeriodLeg2 = builder.getFloatingRatePaymentFrequencyPeriodLeg2();
			this.fixedRatePaymentFrequencyPeriodMultiplierLeg1 = builder.getFixedRatePaymentFrequencyPeriodMultiplierLeg1();
			this.fixedRatePaymentFrequencyPeriodMultiplierLeg2 = builder.getFixedRatePaymentFrequencyPeriodMultiplierLeg2();
			this.floatingRatePaymentFrequencyPeriodMultiplierLeg1 = builder.getFloatingRatePaymentFrequencyPeriodMultiplierLeg1();
			this.floatingRatePaymentFrequencyPeriodMultiplierLeg2 = builder.getFloatingRatePaymentFrequencyPeriodMultiplierLeg2();
			this.exchangeRateLeg1 = builder.getExchangeRateLeg1();
			this.exchangeRateBasis = builder.getExchangeRateBasis();
			this.fixedRateLeg1 = builder.getFixedRateLeg1();
			this.fixedRateLeg2 = builder.getFixedRateLeg2();
			this.postPricedSwapIndicator = builder.getPostPricedSwapIndicator();
			this.price = builder.getPrice();
			this.priceCurrency = builder.getPriceCurrency();
			this.priceNotation = builder.getPriceNotation();
			this.priceUnitOfMeasure = builder.getPriceUnitOfMeasure();
			this.spreadLeg1 = builder.getSpreadLeg1();
			this.spreadLeg2 = builder.getSpreadLeg2();
			this.spreadCurrencyLeg1 = builder.getSpreadCurrencyLeg1();
			this.spreadCurrencyLeg2 = builder.getSpreadCurrencyLeg2();
			this.spreadNotationLeg1 = builder.getSpreadNotationLeg1();
			this.spreadNotationLeg2 = builder.getSpreadNotationLeg2();
			this.strikePrice = builder.getStrikePrice();
			this.strikePriceCurrency = builder.getStrikePriceCurrency();
			this.strikePriceNotation = builder.getStrikePriceNotation();
			this.optionPremiumAmount = builder.getOptionPremiumAmount();
			this.optionPremiumCurrency = builder.getOptionPremiumCurrency();
			this.firstExerciseDate = builder.getFirstExerciseDate();
			this.indexFactor = builder.getIndexFactor();
			this.embeddedOptionType = builder.getEmbeddedOptionType();
			this.uniqueProductIdentifier = builder.getUniqueProductIdentifier();
			this.settlementCurrencyLeg1 = builder.getSettlementCurrencyLeg1();
			this.settlementCurrencyLeg2 = builder.getSettlementCurrencyLeg2();
			this.settlementLocationLeg1 = builder.getSettlementLocationLeg1();
			this.settlementLocationLeg2 = builder.getSettlementLocationLeg2();
			this.nonStandardizedTermIndicator = builder.getNonStandardizedTermIndicator();
			this.blockTradeElectionIndicator = builder.getBlockTradeElectionIndicator();
			this.effectiveDate = builder.getEffectiveDate();
			this.expirationDate = builder.getExpirationDate();
			this.executionTimestamp = builder.getExecutionTimestamp();
			this.reportingTimestamp = builder.getReportingTimestamp();
			this.platformIdentifier = builder.getPlatformIdentifier();
			this.primeBrokerageTransactionIndicator = builder.getPrimeBrokerageTransactionIndicator();
			this.uniqueSwapIdentifier = builder.getUniqueSwapIdentifier();
			this.uniqueTransactionIdentifier = builder.getUniqueTransactionIdentifier();
			this.cftcGuidebook = ofNullable(builder.getCftcGuidebook()).map(f->f.build()).orElse(null);
			this.nonReportable = ofNullable(builder.getNonReportable()).map(f->f.build()).orElse(null);
		}
		
		@Override
		@RosettaAttribute("cleared")
		public String getCleared() {
			return cleared;
		}
		
		@Override
		@RosettaAttribute("counterparty1")
		public String getCounterparty1() {
			return counterparty1;
		}
		
		@Override
		@RosettaAttribute("counterparty2")
		public String getCounterparty2() {
			return counterparty2;
		}
		
		@Override
		@RosettaAttribute("counterparty2IdentifierSource")
		public String getCounterparty2IdentifierSource() {
			return counterparty2IdentifierSource;
		}
		
		@Override
		@RosettaAttribute("submitterIdentifier")
		public String getSubmitterIdentifier() {
			return submitterIdentifier;
		}
		
		@Override
		@RosettaAttribute("customBasketIndicator")
		public Boolean getCustomBasketIndicator() {
			return customBasketIndicator;
		}
		
		@Override
		@RosettaAttribute("actionType")
		public String getActionType() {
			return actionType;
		}
		
		@Override
		@RosettaAttribute("eventType")
		public String getEventType() {
			return eventType;
		}
		
		@Override
		@RosettaAttribute("amendmentIndicator")
		public Boolean getAmendmentIndicator() {
			return amendmentIndicator;
		}
		
		@Override
		@RosettaAttribute("eventTimestamp")
		public ZonedDateTime getEventTimestamp() {
			return eventTimestamp;
		}
		
		@Override
		@RosettaAttribute("notionalAmountLeg1")
		public BigDecimal getNotionalAmountLeg1() {
			return notionalAmountLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalAmountLeg2")
		public BigDecimal getNotionalAmountLeg2() {
			return notionalAmountLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalCurrencyLeg1")
		public ISOCurrencyCodeEnum getNotionalCurrencyLeg1() {
			return notionalCurrencyLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalCurrencyLeg2")
		public ISOCurrencyCodeEnum getNotionalCurrencyLeg2() {
			return notionalCurrencyLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalAmountScheduleLeg1")
		public List<? extends NotionalAmountScheduleLeg1Report> getNotionalAmountScheduleLeg1() {
			return notionalAmountScheduleLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalAmountScheduleLeg2")
		public List<? extends NotionalAmountScheduleLeg2Report> getNotionalAmountScheduleLeg2() {
			return notionalAmountScheduleLeg2;
		}
		
		@Override
		@RosettaAttribute("callAmount")
		public BigDecimal getCallAmount() {
			return callAmount;
		}
		
		@Override
		@RosettaAttribute("callCurrency")
		public ISOCurrencyCodeEnum getCallCurrency() {
			return callCurrency;
		}
		
		@Override
		@RosettaAttribute("putAmount")
		public BigDecimal getPutAmount() {
			return putAmount;
		}
		
		@Override
		@RosettaAttribute("putCurrency")
		public ISOCurrencyCodeEnum getPutCurrency() {
			return putCurrency;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityLeg1")
		public BigDecimal getNotionalQuantityLeg1() {
			return notionalQuantityLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityLeg2")
		public BigDecimal getNotionalQuantityLeg2() {
			return notionalQuantityLeg2;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyLeg1")
		public String getQuantityFrequencyLeg1() {
			return quantityFrequencyLeg1;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyLeg2")
		public String getQuantityFrequencyLeg2() {
			return quantityFrequencyLeg2;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyMultiplierLeg1")
		public Integer getQuantityFrequencyMultiplierLeg1() {
			return quantityFrequencyMultiplierLeg1;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyMultiplierLeg2")
		public Integer getQuantityFrequencyMultiplierLeg2() {
			return quantityFrequencyMultiplierLeg2;
		}
		
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg1")
		public String getQuantityUnitOfMeasureLeg1() {
			return quantityUnitOfMeasureLeg1;
		}
		
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg2")
		public String getQuantityUnitOfMeasureLeg2() {
			return quantityUnitOfMeasureLeg2;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg1")
		public BigDecimal getTotalNotionalQuantityLeg1() {
			return totalNotionalQuantityLeg1;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg2")
		public BigDecimal getTotalNotionalQuantityLeg2() {
			return totalNotionalQuantityLeg2;
		}
		
		@Override
		@RosettaAttribute("packageIndicator")
		public Boolean getPackageIndicator() {
			return packageIndicator;
		}
		
		@Override
		@RosettaAttribute("packageTransactionPrice")
		public BigDecimal getPackageTransactionPrice() {
			return packageTransactionPrice;
		}
		
		@Override
		@RosettaAttribute("packageTransactionPriceCurrency")
		public ISOCurrencyCodeEnum getPackageTransactionPriceCurrency() {
			return packageTransactionPriceCurrency;
		}
		
		@Override
		@RosettaAttribute("packageTransactionPriceNotation")
		public String getPackageTransactionPriceNotation() {
			return packageTransactionPriceNotation;
		}
		
		@Override
		@RosettaAttribute("packageTransactionSpread")
		public BigDecimal getPackageTransactionSpread() {
			return packageTransactionSpread;
		}
		
		@Override
		@RosettaAttribute("packageTransactionSpreadCurrency")
		public ISOCurrencyCodeEnum getPackageTransactionSpreadCurrency() {
			return packageTransactionSpreadCurrency;
		}
		
		@Override
		@RosettaAttribute("packageTransactionSpreadNotation")
		public String getPackageTransactionSpreadNotation() {
			return packageTransactionSpreadNotation;
		}
		
		@Override
		@RosettaAttribute("fixedRateDayCountConventionLeg1")
		public InterestComputationMethod4Code getFixedRateDayCountConventionLeg1() {
			return fixedRateDayCountConventionLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRateDayCountConventionLeg2")
		public InterestComputationMethod4Code getFixedRateDayCountConventionLeg2() {
			return fixedRateDayCountConventionLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateDayCountConventionLeg1")
		public InterestComputationMethod4Code getFloatingRateDayCountConventionLeg1() {
			return floatingRateDayCountConventionLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateDayCountConventionLeg2")
		public InterestComputationMethod4Code getFloatingRateDayCountConventionLeg2() {
			return floatingRateDayCountConventionLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodLeg1")
		public String getFloatingRateResetFrequencyPeriodLeg1() {
			return floatingRateResetFrequencyPeriodLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodLeg2")
		public String getFloatingRateResetFrequencyPeriodLeg2() {
			return floatingRateResetFrequencyPeriodLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierLeg1")
		public Integer getFloatingRateResetFrequencyPeriodMultiplierLeg1() {
			return floatingRateResetFrequencyPeriodMultiplierLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierLeg2")
		public Integer getFloatingRateResetFrequencyPeriodMultiplierLeg2() {
			return floatingRateResetFrequencyPeriodMultiplierLeg2;
		}
		
		@Override
		@RosettaAttribute("otherPayment")
		public List<? extends OtherPaymentReport> getOtherPayment() {
			return otherPayment;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodLeg1")
		public String getFixedRatePaymentFrequencyPeriodLeg1() {
			return fixedRatePaymentFrequencyPeriodLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodLeg2")
		public String getFixedRatePaymentFrequencyPeriodLeg2() {
			return fixedRatePaymentFrequencyPeriodLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodLeg1")
		public String getFloatingRatePaymentFrequencyPeriodLeg1() {
			return floatingRatePaymentFrequencyPeriodLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodLeg2")
		public String getFloatingRatePaymentFrequencyPeriodLeg2() {
			return floatingRatePaymentFrequencyPeriodLeg2;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierLeg1")
		public Integer getFixedRatePaymentFrequencyPeriodMultiplierLeg1() {
			return fixedRatePaymentFrequencyPeriodMultiplierLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierLeg2")
		public Integer getFixedRatePaymentFrequencyPeriodMultiplierLeg2() {
			return fixedRatePaymentFrequencyPeriodMultiplierLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierLeg1")
		public Integer getFloatingRatePaymentFrequencyPeriodMultiplierLeg1() {
			return floatingRatePaymentFrequencyPeriodMultiplierLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierLeg2")
		public Integer getFloatingRatePaymentFrequencyPeriodMultiplierLeg2() {
			return floatingRatePaymentFrequencyPeriodMultiplierLeg2;
		}
		
		@Override
		@RosettaAttribute("exchangeRateLeg1")
		public BigDecimal getExchangeRateLeg1() {
			return exchangeRateLeg1;
		}
		
		@Override
		@RosettaAttribute("exchangeRateBasis")
		public String getExchangeRateBasis() {
			return exchangeRateBasis;
		}
		
		@Override
		@RosettaAttribute("fixedRateLeg1")
		public BigDecimal getFixedRateLeg1() {
			return fixedRateLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRateLeg2")
		public BigDecimal getFixedRateLeg2() {
			return fixedRateLeg2;
		}
		
		@Override
		@RosettaAttribute("postPricedSwapIndicator")
		public Boolean getPostPricedSwapIndicator() {
			return postPricedSwapIndicator;
		}
		
		@Override
		@RosettaAttribute("price")
		public BigDecimal getPrice() {
			return price;
		}
		
		@Override
		@RosettaAttribute("priceCurrency")
		public ISOCurrencyCodeEnum getPriceCurrency() {
			return priceCurrency;
		}
		
		@Override
		@RosettaAttribute("priceNotation")
		public String getPriceNotation() {
			return priceNotation;
		}
		
		@Override
		@RosettaAttribute("priceUnitOfMeasure")
		public String getPriceUnitOfMeasure() {
			return priceUnitOfMeasure;
		}
		
		@Override
		@RosettaAttribute("spreadLeg1")
		public BigDecimal getSpreadLeg1() {
			return spreadLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadLeg2")
		public BigDecimal getSpreadLeg2() {
			return spreadLeg2;
		}
		
		@Override
		@RosettaAttribute("spreadCurrencyLeg1")
		public ISOCurrencyCodeEnum getSpreadCurrencyLeg1() {
			return spreadCurrencyLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadCurrencyLeg2")
		public ISOCurrencyCodeEnum getSpreadCurrencyLeg2() {
			return spreadCurrencyLeg2;
		}
		
		@Override
		@RosettaAttribute("spreadNotationLeg1")
		public String getSpreadNotationLeg1() {
			return spreadNotationLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadNotationLeg2")
		public String getSpreadNotationLeg2() {
			return spreadNotationLeg2;
		}
		
		@Override
		@RosettaAttribute("strikePrice")
		public BigDecimal getStrikePrice() {
			return strikePrice;
		}
		
		@Override
		@RosettaAttribute("strikePriceCurrency")
		public String getStrikePriceCurrency() {
			return strikePriceCurrency;
		}
		
		@Override
		@RosettaAttribute("strikePriceNotation")
		public String getStrikePriceNotation() {
			return strikePriceNotation;
		}
		
		@Override
		@RosettaAttribute("optionPremiumAmount")
		public BigDecimal getOptionPremiumAmount() {
			return optionPremiumAmount;
		}
		
		@Override
		@RosettaAttribute("optionPremiumCurrency")
		public ISOCurrencyCodeEnum getOptionPremiumCurrency() {
			return optionPremiumCurrency;
		}
		
		@Override
		@RosettaAttribute("firstExerciseDate")
		public Date getFirstExerciseDate() {
			return firstExerciseDate;
		}
		
		@Override
		@RosettaAttribute("indexFactor")
		public BigDecimal getIndexFactor() {
			return indexFactor;
		}
		
		@Override
		@RosettaAttribute("embeddedOptionType")
		public String getEmbeddedOptionType() {
			return embeddedOptionType;
		}
		
		@Override
		@RosettaAttribute("uniqueProductIdentifier")
		public String getUniqueProductIdentifier() {
			return uniqueProductIdentifier;
		}
		
		@Override
		@RosettaAttribute("settlementCurrencyLeg1")
		public ISOCurrencyCodeEnum getSettlementCurrencyLeg1() {
			return settlementCurrencyLeg1;
		}
		
		@Override
		@RosettaAttribute("settlementCurrencyLeg2")
		public ISOCurrencyCodeEnum getSettlementCurrencyLeg2() {
			return settlementCurrencyLeg2;
		}
		
		@Override
		@RosettaAttribute("settlementLocationLeg1")
		public String getSettlementLocationLeg1() {
			return settlementLocationLeg1;
		}
		
		@Override
		@RosettaAttribute("settlementLocationLeg2")
		public String getSettlementLocationLeg2() {
			return settlementLocationLeg2;
		}
		
		@Override
		@RosettaAttribute("nonStandardizedTermIndicator")
		public Boolean getNonStandardizedTermIndicator() {
			return nonStandardizedTermIndicator;
		}
		
		@Override
		@RosettaAttribute("blockTradeElectionIndicator")
		public Boolean getBlockTradeElectionIndicator() {
			return blockTradeElectionIndicator;
		}
		
		@Override
		@RosettaAttribute("effectiveDate")
		public Date getEffectiveDate() {
			return effectiveDate;
		}
		
		@Override
		@RosettaAttribute("expirationDate")
		public Date getExpirationDate() {
			return expirationDate;
		}
		
		@Override
		@RosettaAttribute("executionTimestamp")
		public ZonedDateTime getExecutionTimestamp() {
			return executionTimestamp;
		}
		
		@Override
		@RosettaAttribute("reportingTimestamp")
		public ZonedDateTime getReportingTimestamp() {
			return reportingTimestamp;
		}
		
		@Override
		@RosettaAttribute("platformIdentifier")
		public String getPlatformIdentifier() {
			return platformIdentifier;
		}
		
		@Override
		@RosettaAttribute("primeBrokerageTransactionIndicator")
		public Boolean getPrimeBrokerageTransactionIndicator() {
			return primeBrokerageTransactionIndicator;
		}
		
		@Override
		@RosettaAttribute("uniqueSwapIdentifier")
		public String getUniqueSwapIdentifier() {
			return uniqueSwapIdentifier;
		}
		
		@Override
		@RosettaAttribute("uniqueTransactionIdentifier")
		public String getUniqueTransactionIdentifier() {
			return uniqueTransactionIdentifier;
		}
		
		@Override
		@RosettaAttribute("cftcGuidebook")
		public CFTCGuidebookPart43 getCftcGuidebook() {
			return cftcGuidebook;
		}
		
		@Override
		@RosettaAttribute("nonReportable")
		public CFTCNonReportablePart43 getNonReportable() {
			return nonReportable;
		}
		
		@Override
		public CFTCPart43TransactionReport build() {
			return this;
		}
		
		@Override
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder toBuilder() {
			CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder builder) {
			ofNullable(getCleared()).ifPresent(builder::setCleared);
			ofNullable(getCounterparty1()).ifPresent(builder::setCounterparty1);
			ofNullable(getCounterparty2()).ifPresent(builder::setCounterparty2);
			ofNullable(getCounterparty2IdentifierSource()).ifPresent(builder::setCounterparty2IdentifierSource);
			ofNullable(getSubmitterIdentifier()).ifPresent(builder::setSubmitterIdentifier);
			ofNullable(getCustomBasketIndicator()).ifPresent(builder::setCustomBasketIndicator);
			ofNullable(getActionType()).ifPresent(builder::setActionType);
			ofNullable(getEventType()).ifPresent(builder::setEventType);
			ofNullable(getAmendmentIndicator()).ifPresent(builder::setAmendmentIndicator);
			ofNullable(getEventTimestamp()).ifPresent(builder::setEventTimestamp);
			ofNullable(getNotionalAmountLeg1()).ifPresent(builder::setNotionalAmountLeg1);
			ofNullable(getNotionalAmountLeg2()).ifPresent(builder::setNotionalAmountLeg2);
			ofNullable(getNotionalCurrencyLeg1()).ifPresent(builder::setNotionalCurrencyLeg1);
			ofNullable(getNotionalCurrencyLeg2()).ifPresent(builder::setNotionalCurrencyLeg2);
			ofNullable(getNotionalAmountScheduleLeg1()).ifPresent(builder::setNotionalAmountScheduleLeg1);
			ofNullable(getNotionalAmountScheduleLeg2()).ifPresent(builder::setNotionalAmountScheduleLeg2);
			ofNullable(getCallAmount()).ifPresent(builder::setCallAmount);
			ofNullable(getCallCurrency()).ifPresent(builder::setCallCurrency);
			ofNullable(getPutAmount()).ifPresent(builder::setPutAmount);
			ofNullable(getPutCurrency()).ifPresent(builder::setPutCurrency);
			ofNullable(getNotionalQuantityLeg1()).ifPresent(builder::setNotionalQuantityLeg1);
			ofNullable(getNotionalQuantityLeg2()).ifPresent(builder::setNotionalQuantityLeg2);
			ofNullable(getQuantityFrequencyLeg1()).ifPresent(builder::setQuantityFrequencyLeg1);
			ofNullable(getQuantityFrequencyLeg2()).ifPresent(builder::setQuantityFrequencyLeg2);
			ofNullable(getQuantityFrequencyMultiplierLeg1()).ifPresent(builder::setQuantityFrequencyMultiplierLeg1);
			ofNullable(getQuantityFrequencyMultiplierLeg2()).ifPresent(builder::setQuantityFrequencyMultiplierLeg2);
			ofNullable(getQuantityUnitOfMeasureLeg1()).ifPresent(builder::setQuantityUnitOfMeasureLeg1);
			ofNullable(getQuantityUnitOfMeasureLeg2()).ifPresent(builder::setQuantityUnitOfMeasureLeg2);
			ofNullable(getTotalNotionalQuantityLeg1()).ifPresent(builder::setTotalNotionalQuantityLeg1);
			ofNullable(getTotalNotionalQuantityLeg2()).ifPresent(builder::setTotalNotionalQuantityLeg2);
			ofNullable(getPackageIndicator()).ifPresent(builder::setPackageIndicator);
			ofNullable(getPackageTransactionPrice()).ifPresent(builder::setPackageTransactionPrice);
			ofNullable(getPackageTransactionPriceCurrency()).ifPresent(builder::setPackageTransactionPriceCurrency);
			ofNullable(getPackageTransactionPriceNotation()).ifPresent(builder::setPackageTransactionPriceNotation);
			ofNullable(getPackageTransactionSpread()).ifPresent(builder::setPackageTransactionSpread);
			ofNullable(getPackageTransactionSpreadCurrency()).ifPresent(builder::setPackageTransactionSpreadCurrency);
			ofNullable(getPackageTransactionSpreadNotation()).ifPresent(builder::setPackageTransactionSpreadNotation);
			ofNullable(getFixedRateDayCountConventionLeg1()).ifPresent(builder::setFixedRateDayCountConventionLeg1);
			ofNullable(getFixedRateDayCountConventionLeg2()).ifPresent(builder::setFixedRateDayCountConventionLeg2);
			ofNullable(getFloatingRateDayCountConventionLeg1()).ifPresent(builder::setFloatingRateDayCountConventionLeg1);
			ofNullable(getFloatingRateDayCountConventionLeg2()).ifPresent(builder::setFloatingRateDayCountConventionLeg2);
			ofNullable(getFloatingRateResetFrequencyPeriodLeg1()).ifPresent(builder::setFloatingRateResetFrequencyPeriodLeg1);
			ofNullable(getFloatingRateResetFrequencyPeriodLeg2()).ifPresent(builder::setFloatingRateResetFrequencyPeriodLeg2);
			ofNullable(getFloatingRateResetFrequencyPeriodMultiplierLeg1()).ifPresent(builder::setFloatingRateResetFrequencyPeriodMultiplierLeg1);
			ofNullable(getFloatingRateResetFrequencyPeriodMultiplierLeg2()).ifPresent(builder::setFloatingRateResetFrequencyPeriodMultiplierLeg2);
			ofNullable(getOtherPayment()).ifPresent(builder::setOtherPayment);
			ofNullable(getFixedRatePaymentFrequencyPeriodLeg1()).ifPresent(builder::setFixedRatePaymentFrequencyPeriodLeg1);
			ofNullable(getFixedRatePaymentFrequencyPeriodLeg2()).ifPresent(builder::setFixedRatePaymentFrequencyPeriodLeg2);
			ofNullable(getFloatingRatePaymentFrequencyPeriodLeg1()).ifPresent(builder::setFloatingRatePaymentFrequencyPeriodLeg1);
			ofNullable(getFloatingRatePaymentFrequencyPeriodLeg2()).ifPresent(builder::setFloatingRatePaymentFrequencyPeriodLeg2);
			ofNullable(getFixedRatePaymentFrequencyPeriodMultiplierLeg1()).ifPresent(builder::setFixedRatePaymentFrequencyPeriodMultiplierLeg1);
			ofNullable(getFixedRatePaymentFrequencyPeriodMultiplierLeg2()).ifPresent(builder::setFixedRatePaymentFrequencyPeriodMultiplierLeg2);
			ofNullable(getFloatingRatePaymentFrequencyPeriodMultiplierLeg1()).ifPresent(builder::setFloatingRatePaymentFrequencyPeriodMultiplierLeg1);
			ofNullable(getFloatingRatePaymentFrequencyPeriodMultiplierLeg2()).ifPresent(builder::setFloatingRatePaymentFrequencyPeriodMultiplierLeg2);
			ofNullable(getExchangeRateLeg1()).ifPresent(builder::setExchangeRateLeg1);
			ofNullable(getExchangeRateBasis()).ifPresent(builder::setExchangeRateBasis);
			ofNullable(getFixedRateLeg1()).ifPresent(builder::setFixedRateLeg1);
			ofNullable(getFixedRateLeg2()).ifPresent(builder::setFixedRateLeg2);
			ofNullable(getPostPricedSwapIndicator()).ifPresent(builder::setPostPricedSwapIndicator);
			ofNullable(getPrice()).ifPresent(builder::setPrice);
			ofNullable(getPriceCurrency()).ifPresent(builder::setPriceCurrency);
			ofNullable(getPriceNotation()).ifPresent(builder::setPriceNotation);
			ofNullable(getPriceUnitOfMeasure()).ifPresent(builder::setPriceUnitOfMeasure);
			ofNullable(getSpreadLeg1()).ifPresent(builder::setSpreadLeg1);
			ofNullable(getSpreadLeg2()).ifPresent(builder::setSpreadLeg2);
			ofNullable(getSpreadCurrencyLeg1()).ifPresent(builder::setSpreadCurrencyLeg1);
			ofNullable(getSpreadCurrencyLeg2()).ifPresent(builder::setSpreadCurrencyLeg2);
			ofNullable(getSpreadNotationLeg1()).ifPresent(builder::setSpreadNotationLeg1);
			ofNullable(getSpreadNotationLeg2()).ifPresent(builder::setSpreadNotationLeg2);
			ofNullable(getStrikePrice()).ifPresent(builder::setStrikePrice);
			ofNullable(getStrikePriceCurrency()).ifPresent(builder::setStrikePriceCurrency);
			ofNullable(getStrikePriceNotation()).ifPresent(builder::setStrikePriceNotation);
			ofNullable(getOptionPremiumAmount()).ifPresent(builder::setOptionPremiumAmount);
			ofNullable(getOptionPremiumCurrency()).ifPresent(builder::setOptionPremiumCurrency);
			ofNullable(getFirstExerciseDate()).ifPresent(builder::setFirstExerciseDate);
			ofNullable(getIndexFactor()).ifPresent(builder::setIndexFactor);
			ofNullable(getEmbeddedOptionType()).ifPresent(builder::setEmbeddedOptionType);
			ofNullable(getUniqueProductIdentifier()).ifPresent(builder::setUniqueProductIdentifier);
			ofNullable(getSettlementCurrencyLeg1()).ifPresent(builder::setSettlementCurrencyLeg1);
			ofNullable(getSettlementCurrencyLeg2()).ifPresent(builder::setSettlementCurrencyLeg2);
			ofNullable(getSettlementLocationLeg1()).ifPresent(builder::setSettlementLocationLeg1);
			ofNullable(getSettlementLocationLeg2()).ifPresent(builder::setSettlementLocationLeg2);
			ofNullable(getNonStandardizedTermIndicator()).ifPresent(builder::setNonStandardizedTermIndicator);
			ofNullable(getBlockTradeElectionIndicator()).ifPresent(builder::setBlockTradeElectionIndicator);
			ofNullable(getEffectiveDate()).ifPresent(builder::setEffectiveDate);
			ofNullable(getExpirationDate()).ifPresent(builder::setExpirationDate);
			ofNullable(getExecutionTimestamp()).ifPresent(builder::setExecutionTimestamp);
			ofNullable(getReportingTimestamp()).ifPresent(builder::setReportingTimestamp);
			ofNullable(getPlatformIdentifier()).ifPresent(builder::setPlatformIdentifier);
			ofNullable(getPrimeBrokerageTransactionIndicator()).ifPresent(builder::setPrimeBrokerageTransactionIndicator);
			ofNullable(getUniqueSwapIdentifier()).ifPresent(builder::setUniqueSwapIdentifier);
			ofNullable(getUniqueTransactionIdentifier()).ifPresent(builder::setUniqueTransactionIdentifier);
			ofNullable(getCftcGuidebook()).ifPresent(builder::setCftcGuidebook);
			ofNullable(getNonReportable()).ifPresent(builder::setNonReportable);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			CFTCPart43TransactionReport _that = getType().cast(o);
		
			if (!Objects.equals(cleared, _that.getCleared())) return false;
			if (!Objects.equals(counterparty1, _that.getCounterparty1())) return false;
			if (!Objects.equals(counterparty2, _that.getCounterparty2())) return false;
			if (!Objects.equals(counterparty2IdentifierSource, _that.getCounterparty2IdentifierSource())) return false;
			if (!Objects.equals(submitterIdentifier, _that.getSubmitterIdentifier())) return false;
			if (!Objects.equals(customBasketIndicator, _that.getCustomBasketIndicator())) return false;
			if (!Objects.equals(actionType, _that.getActionType())) return false;
			if (!Objects.equals(eventType, _that.getEventType())) return false;
			if (!Objects.equals(amendmentIndicator, _that.getAmendmentIndicator())) return false;
			if (!Objects.equals(eventTimestamp, _that.getEventTimestamp())) return false;
			if (!Objects.equals(notionalAmountLeg1, _that.getNotionalAmountLeg1())) return false;
			if (!Objects.equals(notionalAmountLeg2, _that.getNotionalAmountLeg2())) return false;
			if (!Objects.equals(notionalCurrencyLeg1, _that.getNotionalCurrencyLeg1())) return false;
			if (!Objects.equals(notionalCurrencyLeg2, _that.getNotionalCurrencyLeg2())) return false;
			if (!ListEquals.listEquals(notionalAmountScheduleLeg1, _that.getNotionalAmountScheduleLeg1())) return false;
			if (!ListEquals.listEquals(notionalAmountScheduleLeg2, _that.getNotionalAmountScheduleLeg2())) return false;
			if (!Objects.equals(callAmount, _that.getCallAmount())) return false;
			if (!Objects.equals(callCurrency, _that.getCallCurrency())) return false;
			if (!Objects.equals(putAmount, _that.getPutAmount())) return false;
			if (!Objects.equals(putCurrency, _that.getPutCurrency())) return false;
			if (!Objects.equals(notionalQuantityLeg1, _that.getNotionalQuantityLeg1())) return false;
			if (!Objects.equals(notionalQuantityLeg2, _that.getNotionalQuantityLeg2())) return false;
			if (!Objects.equals(quantityFrequencyLeg1, _that.getQuantityFrequencyLeg1())) return false;
			if (!Objects.equals(quantityFrequencyLeg2, _that.getQuantityFrequencyLeg2())) return false;
			if (!Objects.equals(quantityFrequencyMultiplierLeg1, _that.getQuantityFrequencyMultiplierLeg1())) return false;
			if (!Objects.equals(quantityFrequencyMultiplierLeg2, _that.getQuantityFrequencyMultiplierLeg2())) return false;
			if (!Objects.equals(quantityUnitOfMeasureLeg1, _that.getQuantityUnitOfMeasureLeg1())) return false;
			if (!Objects.equals(quantityUnitOfMeasureLeg2, _that.getQuantityUnitOfMeasureLeg2())) return false;
			if (!Objects.equals(totalNotionalQuantityLeg1, _that.getTotalNotionalQuantityLeg1())) return false;
			if (!Objects.equals(totalNotionalQuantityLeg2, _that.getTotalNotionalQuantityLeg2())) return false;
			if (!Objects.equals(packageIndicator, _that.getPackageIndicator())) return false;
			if (!Objects.equals(packageTransactionPrice, _that.getPackageTransactionPrice())) return false;
			if (!Objects.equals(packageTransactionPriceCurrency, _that.getPackageTransactionPriceCurrency())) return false;
			if (!Objects.equals(packageTransactionPriceNotation, _that.getPackageTransactionPriceNotation())) return false;
			if (!Objects.equals(packageTransactionSpread, _that.getPackageTransactionSpread())) return false;
			if (!Objects.equals(packageTransactionSpreadCurrency, _that.getPackageTransactionSpreadCurrency())) return false;
			if (!Objects.equals(packageTransactionSpreadNotation, _that.getPackageTransactionSpreadNotation())) return false;
			if (!Objects.equals(fixedRateDayCountConventionLeg1, _that.getFixedRateDayCountConventionLeg1())) return false;
			if (!Objects.equals(fixedRateDayCountConventionLeg2, _that.getFixedRateDayCountConventionLeg2())) return false;
			if (!Objects.equals(floatingRateDayCountConventionLeg1, _that.getFloatingRateDayCountConventionLeg1())) return false;
			if (!Objects.equals(floatingRateDayCountConventionLeg2, _that.getFloatingRateDayCountConventionLeg2())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodLeg1, _that.getFloatingRateResetFrequencyPeriodLeg1())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodLeg2, _that.getFloatingRateResetFrequencyPeriodLeg2())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodMultiplierLeg1, _that.getFloatingRateResetFrequencyPeriodMultiplierLeg1())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodMultiplierLeg2, _that.getFloatingRateResetFrequencyPeriodMultiplierLeg2())) return false;
			if (!ListEquals.listEquals(otherPayment, _that.getOtherPayment())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodLeg1, _that.getFixedRatePaymentFrequencyPeriodLeg1())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodLeg2, _that.getFixedRatePaymentFrequencyPeriodLeg2())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodLeg1, _that.getFloatingRatePaymentFrequencyPeriodLeg1())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodLeg2, _that.getFloatingRatePaymentFrequencyPeriodLeg2())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodMultiplierLeg1, _that.getFixedRatePaymentFrequencyPeriodMultiplierLeg1())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodMultiplierLeg2, _that.getFixedRatePaymentFrequencyPeriodMultiplierLeg2())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodMultiplierLeg1, _that.getFloatingRatePaymentFrequencyPeriodMultiplierLeg1())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodMultiplierLeg2, _that.getFloatingRatePaymentFrequencyPeriodMultiplierLeg2())) return false;
			if (!Objects.equals(exchangeRateLeg1, _that.getExchangeRateLeg1())) return false;
			if (!Objects.equals(exchangeRateBasis, _that.getExchangeRateBasis())) return false;
			if (!Objects.equals(fixedRateLeg1, _that.getFixedRateLeg1())) return false;
			if (!Objects.equals(fixedRateLeg2, _that.getFixedRateLeg2())) return false;
			if (!Objects.equals(postPricedSwapIndicator, _that.getPostPricedSwapIndicator())) return false;
			if (!Objects.equals(price, _that.getPrice())) return false;
			if (!Objects.equals(priceCurrency, _that.getPriceCurrency())) return false;
			if (!Objects.equals(priceNotation, _that.getPriceNotation())) return false;
			if (!Objects.equals(priceUnitOfMeasure, _that.getPriceUnitOfMeasure())) return false;
			if (!Objects.equals(spreadLeg1, _that.getSpreadLeg1())) return false;
			if (!Objects.equals(spreadLeg2, _that.getSpreadLeg2())) return false;
			if (!Objects.equals(spreadCurrencyLeg1, _that.getSpreadCurrencyLeg1())) return false;
			if (!Objects.equals(spreadCurrencyLeg2, _that.getSpreadCurrencyLeg2())) return false;
			if (!Objects.equals(spreadNotationLeg1, _that.getSpreadNotationLeg1())) return false;
			if (!Objects.equals(spreadNotationLeg2, _that.getSpreadNotationLeg2())) return false;
			if (!Objects.equals(strikePrice, _that.getStrikePrice())) return false;
			if (!Objects.equals(strikePriceCurrency, _that.getStrikePriceCurrency())) return false;
			if (!Objects.equals(strikePriceNotation, _that.getStrikePriceNotation())) return false;
			if (!Objects.equals(optionPremiumAmount, _that.getOptionPremiumAmount())) return false;
			if (!Objects.equals(optionPremiumCurrency, _that.getOptionPremiumCurrency())) return false;
			if (!Objects.equals(firstExerciseDate, _that.getFirstExerciseDate())) return false;
			if (!Objects.equals(indexFactor, _that.getIndexFactor())) return false;
			if (!Objects.equals(embeddedOptionType, _that.getEmbeddedOptionType())) return false;
			if (!Objects.equals(uniqueProductIdentifier, _that.getUniqueProductIdentifier())) return false;
			if (!Objects.equals(settlementCurrencyLeg1, _that.getSettlementCurrencyLeg1())) return false;
			if (!Objects.equals(settlementCurrencyLeg2, _that.getSettlementCurrencyLeg2())) return false;
			if (!Objects.equals(settlementLocationLeg1, _that.getSettlementLocationLeg1())) return false;
			if (!Objects.equals(settlementLocationLeg2, _that.getSettlementLocationLeg2())) return false;
			if (!Objects.equals(nonStandardizedTermIndicator, _that.getNonStandardizedTermIndicator())) return false;
			if (!Objects.equals(blockTradeElectionIndicator, _that.getBlockTradeElectionIndicator())) return false;
			if (!Objects.equals(effectiveDate, _that.getEffectiveDate())) return false;
			if (!Objects.equals(expirationDate, _that.getExpirationDate())) return false;
			if (!Objects.equals(executionTimestamp, _that.getExecutionTimestamp())) return false;
			if (!Objects.equals(reportingTimestamp, _that.getReportingTimestamp())) return false;
			if (!Objects.equals(platformIdentifier, _that.getPlatformIdentifier())) return false;
			if (!Objects.equals(primeBrokerageTransactionIndicator, _that.getPrimeBrokerageTransactionIndicator())) return false;
			if (!Objects.equals(uniqueSwapIdentifier, _that.getUniqueSwapIdentifier())) return false;
			if (!Objects.equals(uniqueTransactionIdentifier, _that.getUniqueTransactionIdentifier())) return false;
			if (!Objects.equals(cftcGuidebook, _that.getCftcGuidebook())) return false;
			if (!Objects.equals(nonReportable, _that.getNonReportable())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cleared != null ? cleared.hashCode() : 0);
			_result = 31 * _result + (counterparty1 != null ? counterparty1.hashCode() : 0);
			_result = 31 * _result + (counterparty2 != null ? counterparty2.hashCode() : 0);
			_result = 31 * _result + (counterparty2IdentifierSource != null ? counterparty2IdentifierSource.hashCode() : 0);
			_result = 31 * _result + (submitterIdentifier != null ? submitterIdentifier.hashCode() : 0);
			_result = 31 * _result + (customBasketIndicator != null ? customBasketIndicator.hashCode() : 0);
			_result = 31 * _result + (actionType != null ? actionType.hashCode() : 0);
			_result = 31 * _result + (eventType != null ? eventType.hashCode() : 0);
			_result = 31 * _result + (amendmentIndicator != null ? amendmentIndicator.hashCode() : 0);
			_result = 31 * _result + (eventTimestamp != null ? eventTimestamp.hashCode() : 0);
			_result = 31 * _result + (notionalAmountLeg1 != null ? notionalAmountLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalAmountLeg2 != null ? notionalAmountLeg2.hashCode() : 0);
			_result = 31 * _result + (notionalCurrencyLeg1 != null ? notionalCurrencyLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalCurrencyLeg2 != null ? notionalCurrencyLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalAmountScheduleLeg1 != null ? notionalAmountScheduleLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalAmountScheduleLeg2 != null ? notionalAmountScheduleLeg2.hashCode() : 0);
			_result = 31 * _result + (callAmount != null ? callAmount.hashCode() : 0);
			_result = 31 * _result + (callCurrency != null ? callCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (putAmount != null ? putAmount.hashCode() : 0);
			_result = 31 * _result + (putCurrency != null ? putCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalQuantityLeg1 != null ? notionalQuantityLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalQuantityLeg2 != null ? notionalQuantityLeg2.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyLeg1 != null ? quantityFrequencyLeg1.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyLeg2 != null ? quantityFrequencyLeg2.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyMultiplierLeg1 != null ? quantityFrequencyMultiplierLeg1.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyMultiplierLeg2 != null ? quantityFrequencyMultiplierLeg2.hashCode() : 0);
			_result = 31 * _result + (quantityUnitOfMeasureLeg1 != null ? quantityUnitOfMeasureLeg1.hashCode() : 0);
			_result = 31 * _result + (quantityUnitOfMeasureLeg2 != null ? quantityUnitOfMeasureLeg2.hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityLeg1 != null ? totalNotionalQuantityLeg1.hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityLeg2 != null ? totalNotionalQuantityLeg2.hashCode() : 0);
			_result = 31 * _result + (packageIndicator != null ? packageIndicator.hashCode() : 0);
			_result = 31 * _result + (packageTransactionPrice != null ? packageTransactionPrice.hashCode() : 0);
			_result = 31 * _result + (packageTransactionPriceCurrency != null ? packageTransactionPriceCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (packageTransactionPriceNotation != null ? packageTransactionPriceNotation.hashCode() : 0);
			_result = 31 * _result + (packageTransactionSpread != null ? packageTransactionSpread.hashCode() : 0);
			_result = 31 * _result + (packageTransactionSpreadCurrency != null ? packageTransactionSpreadCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (packageTransactionSpreadNotation != null ? packageTransactionSpreadNotation.hashCode() : 0);
			_result = 31 * _result + (fixedRateDayCountConventionLeg1 != null ? fixedRateDayCountConventionLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (fixedRateDayCountConventionLeg2 != null ? fixedRateDayCountConventionLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRateDayCountConventionLeg1 != null ? floatingRateDayCountConventionLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRateDayCountConventionLeg2 != null ? floatingRateDayCountConventionLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodLeg1 != null ? floatingRateResetFrequencyPeriodLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodLeg2 != null ? floatingRateResetFrequencyPeriodLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodMultiplierLeg1 != null ? floatingRateResetFrequencyPeriodMultiplierLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodMultiplierLeg2 != null ? floatingRateResetFrequencyPeriodMultiplierLeg2.hashCode() : 0);
			_result = 31 * _result + (otherPayment != null ? otherPayment.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodLeg1 != null ? fixedRatePaymentFrequencyPeriodLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodLeg2 != null ? fixedRatePaymentFrequencyPeriodLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodLeg1 != null ? floatingRatePaymentFrequencyPeriodLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodLeg2 != null ? floatingRatePaymentFrequencyPeriodLeg2.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodMultiplierLeg1 != null ? fixedRatePaymentFrequencyPeriodMultiplierLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodMultiplierLeg2 != null ? fixedRatePaymentFrequencyPeriodMultiplierLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodMultiplierLeg1 != null ? floatingRatePaymentFrequencyPeriodMultiplierLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodMultiplierLeg2 != null ? floatingRatePaymentFrequencyPeriodMultiplierLeg2.hashCode() : 0);
			_result = 31 * _result + (exchangeRateLeg1 != null ? exchangeRateLeg1.hashCode() : 0);
			_result = 31 * _result + (exchangeRateBasis != null ? exchangeRateBasis.hashCode() : 0);
			_result = 31 * _result + (fixedRateLeg1 != null ? fixedRateLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRateLeg2 != null ? fixedRateLeg2.hashCode() : 0);
			_result = 31 * _result + (postPricedSwapIndicator != null ? postPricedSwapIndicator.hashCode() : 0);
			_result = 31 * _result + (price != null ? price.hashCode() : 0);
			_result = 31 * _result + (priceCurrency != null ? priceCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (priceNotation != null ? priceNotation.hashCode() : 0);
			_result = 31 * _result + (priceUnitOfMeasure != null ? priceUnitOfMeasure.hashCode() : 0);
			_result = 31 * _result + (spreadLeg1 != null ? spreadLeg1.hashCode() : 0);
			_result = 31 * _result + (spreadLeg2 != null ? spreadLeg2.hashCode() : 0);
			_result = 31 * _result + (spreadCurrencyLeg1 != null ? spreadCurrencyLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (spreadCurrencyLeg2 != null ? spreadCurrencyLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (spreadNotationLeg1 != null ? spreadNotationLeg1.hashCode() : 0);
			_result = 31 * _result + (spreadNotationLeg2 != null ? spreadNotationLeg2.hashCode() : 0);
			_result = 31 * _result + (strikePrice != null ? strikePrice.hashCode() : 0);
			_result = 31 * _result + (strikePriceCurrency != null ? strikePriceCurrency.hashCode() : 0);
			_result = 31 * _result + (strikePriceNotation != null ? strikePriceNotation.hashCode() : 0);
			_result = 31 * _result + (optionPremiumAmount != null ? optionPremiumAmount.hashCode() : 0);
			_result = 31 * _result + (optionPremiumCurrency != null ? optionPremiumCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (firstExerciseDate != null ? firstExerciseDate.hashCode() : 0);
			_result = 31 * _result + (indexFactor != null ? indexFactor.hashCode() : 0);
			_result = 31 * _result + (embeddedOptionType != null ? embeddedOptionType.hashCode() : 0);
			_result = 31 * _result + (uniqueProductIdentifier != null ? uniqueProductIdentifier.hashCode() : 0);
			_result = 31 * _result + (settlementCurrencyLeg1 != null ? settlementCurrencyLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (settlementCurrencyLeg2 != null ? settlementCurrencyLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (settlementLocationLeg1 != null ? settlementLocationLeg1.hashCode() : 0);
			_result = 31 * _result + (settlementLocationLeg2 != null ? settlementLocationLeg2.hashCode() : 0);
			_result = 31 * _result + (nonStandardizedTermIndicator != null ? nonStandardizedTermIndicator.hashCode() : 0);
			_result = 31 * _result + (blockTradeElectionIndicator != null ? blockTradeElectionIndicator.hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (expirationDate != null ? expirationDate.hashCode() : 0);
			_result = 31 * _result + (executionTimestamp != null ? executionTimestamp.hashCode() : 0);
			_result = 31 * _result + (reportingTimestamp != null ? reportingTimestamp.hashCode() : 0);
			_result = 31 * _result + (platformIdentifier != null ? platformIdentifier.hashCode() : 0);
			_result = 31 * _result + (primeBrokerageTransactionIndicator != null ? primeBrokerageTransactionIndicator.hashCode() : 0);
			_result = 31 * _result + (uniqueSwapIdentifier != null ? uniqueSwapIdentifier.hashCode() : 0);
			_result = 31 * _result + (uniqueTransactionIdentifier != null ? uniqueTransactionIdentifier.hashCode() : 0);
			_result = 31 * _result + (cftcGuidebook != null ? cftcGuidebook.hashCode() : 0);
			_result = 31 * _result + (nonReportable != null ? nonReportable.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CFTCPart43TransactionReport {" +
				"cleared=" + this.cleared + ", " +
				"counterparty1=" + this.counterparty1 + ", " +
				"counterparty2=" + this.counterparty2 + ", " +
				"counterparty2IdentifierSource=" + this.counterparty2IdentifierSource + ", " +
				"submitterIdentifier=" + this.submitterIdentifier + ", " +
				"customBasketIndicator=" + this.customBasketIndicator + ", " +
				"actionType=" + this.actionType + ", " +
				"eventType=" + this.eventType + ", " +
				"amendmentIndicator=" + this.amendmentIndicator + ", " +
				"eventTimestamp=" + this.eventTimestamp + ", " +
				"notionalAmountLeg1=" + this.notionalAmountLeg1 + ", " +
				"notionalAmountLeg2=" + this.notionalAmountLeg2 + ", " +
				"notionalCurrencyLeg1=" + this.notionalCurrencyLeg1 + ", " +
				"notionalCurrencyLeg2=" + this.notionalCurrencyLeg2 + ", " +
				"notionalAmountScheduleLeg1=" + this.notionalAmountScheduleLeg1 + ", " +
				"notionalAmountScheduleLeg2=" + this.notionalAmountScheduleLeg2 + ", " +
				"callAmount=" + this.callAmount + ", " +
				"callCurrency=" + this.callCurrency + ", " +
				"putAmount=" + this.putAmount + ", " +
				"putCurrency=" + this.putCurrency + ", " +
				"notionalQuantityLeg1=" + this.notionalQuantityLeg1 + ", " +
				"notionalQuantityLeg2=" + this.notionalQuantityLeg2 + ", " +
				"quantityFrequencyLeg1=" + this.quantityFrequencyLeg1 + ", " +
				"quantityFrequencyLeg2=" + this.quantityFrequencyLeg2 + ", " +
				"quantityFrequencyMultiplierLeg1=" + this.quantityFrequencyMultiplierLeg1 + ", " +
				"quantityFrequencyMultiplierLeg2=" + this.quantityFrequencyMultiplierLeg2 + ", " +
				"quantityUnitOfMeasureLeg1=" + this.quantityUnitOfMeasureLeg1 + ", " +
				"quantityUnitOfMeasureLeg2=" + this.quantityUnitOfMeasureLeg2 + ", " +
				"totalNotionalQuantityLeg1=" + this.totalNotionalQuantityLeg1 + ", " +
				"totalNotionalQuantityLeg2=" + this.totalNotionalQuantityLeg2 + ", " +
				"packageIndicator=" + this.packageIndicator + ", " +
				"packageTransactionPrice=" + this.packageTransactionPrice + ", " +
				"packageTransactionPriceCurrency=" + this.packageTransactionPriceCurrency + ", " +
				"packageTransactionPriceNotation=" + this.packageTransactionPriceNotation + ", " +
				"packageTransactionSpread=" + this.packageTransactionSpread + ", " +
				"packageTransactionSpreadCurrency=" + this.packageTransactionSpreadCurrency + ", " +
				"packageTransactionSpreadNotation=" + this.packageTransactionSpreadNotation + ", " +
				"fixedRateDayCountConventionLeg1=" + this.fixedRateDayCountConventionLeg1 + ", " +
				"fixedRateDayCountConventionLeg2=" + this.fixedRateDayCountConventionLeg2 + ", " +
				"floatingRateDayCountConventionLeg1=" + this.floatingRateDayCountConventionLeg1 + ", " +
				"floatingRateDayCountConventionLeg2=" + this.floatingRateDayCountConventionLeg2 + ", " +
				"floatingRateResetFrequencyPeriodLeg1=" + this.floatingRateResetFrequencyPeriodLeg1 + ", " +
				"floatingRateResetFrequencyPeriodLeg2=" + this.floatingRateResetFrequencyPeriodLeg2 + ", " +
				"floatingRateResetFrequencyPeriodMultiplierLeg1=" + this.floatingRateResetFrequencyPeriodMultiplierLeg1 + ", " +
				"floatingRateResetFrequencyPeriodMultiplierLeg2=" + this.floatingRateResetFrequencyPeriodMultiplierLeg2 + ", " +
				"otherPayment=" + this.otherPayment + ", " +
				"fixedRatePaymentFrequencyPeriodLeg1=" + this.fixedRatePaymentFrequencyPeriodLeg1 + ", " +
				"fixedRatePaymentFrequencyPeriodLeg2=" + this.fixedRatePaymentFrequencyPeriodLeg2 + ", " +
				"floatingRatePaymentFrequencyPeriodLeg1=" + this.floatingRatePaymentFrequencyPeriodLeg1 + ", " +
				"floatingRatePaymentFrequencyPeriodLeg2=" + this.floatingRatePaymentFrequencyPeriodLeg2 + ", " +
				"fixedRatePaymentFrequencyPeriodMultiplierLeg1=" + this.fixedRatePaymentFrequencyPeriodMultiplierLeg1 + ", " +
				"fixedRatePaymentFrequencyPeriodMultiplierLeg2=" + this.fixedRatePaymentFrequencyPeriodMultiplierLeg2 + ", " +
				"floatingRatePaymentFrequencyPeriodMultiplierLeg1=" + this.floatingRatePaymentFrequencyPeriodMultiplierLeg1 + ", " +
				"floatingRatePaymentFrequencyPeriodMultiplierLeg2=" + this.floatingRatePaymentFrequencyPeriodMultiplierLeg2 + ", " +
				"exchangeRateLeg1=" + this.exchangeRateLeg1 + ", " +
				"exchangeRateBasis=" + this.exchangeRateBasis + ", " +
				"fixedRateLeg1=" + this.fixedRateLeg1 + ", " +
				"fixedRateLeg2=" + this.fixedRateLeg2 + ", " +
				"postPricedSwapIndicator=" + this.postPricedSwapIndicator + ", " +
				"price=" + this.price + ", " +
				"priceCurrency=" + this.priceCurrency + ", " +
				"priceNotation=" + this.priceNotation + ", " +
				"priceUnitOfMeasure=" + this.priceUnitOfMeasure + ", " +
				"spreadLeg1=" + this.spreadLeg1 + ", " +
				"spreadLeg2=" + this.spreadLeg2 + ", " +
				"spreadCurrencyLeg1=" + this.spreadCurrencyLeg1 + ", " +
				"spreadCurrencyLeg2=" + this.spreadCurrencyLeg2 + ", " +
				"spreadNotationLeg1=" + this.spreadNotationLeg1 + ", " +
				"spreadNotationLeg2=" + this.spreadNotationLeg2 + ", " +
				"strikePrice=" + this.strikePrice + ", " +
				"strikePriceCurrency=" + this.strikePriceCurrency + ", " +
				"strikePriceNotation=" + this.strikePriceNotation + ", " +
				"optionPremiumAmount=" + this.optionPremiumAmount + ", " +
				"optionPremiumCurrency=" + this.optionPremiumCurrency + ", " +
				"firstExerciseDate=" + this.firstExerciseDate + ", " +
				"indexFactor=" + this.indexFactor + ", " +
				"embeddedOptionType=" + this.embeddedOptionType + ", " +
				"uniqueProductIdentifier=" + this.uniqueProductIdentifier + ", " +
				"settlementCurrencyLeg1=" + this.settlementCurrencyLeg1 + ", " +
				"settlementCurrencyLeg2=" + this.settlementCurrencyLeg2 + ", " +
				"settlementLocationLeg1=" + this.settlementLocationLeg1 + ", " +
				"settlementLocationLeg2=" + this.settlementLocationLeg2 + ", " +
				"nonStandardizedTermIndicator=" + this.nonStandardizedTermIndicator + ", " +
				"blockTradeElectionIndicator=" + this.blockTradeElectionIndicator + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"expirationDate=" + this.expirationDate + ", " +
				"executionTimestamp=" + this.executionTimestamp + ", " +
				"reportingTimestamp=" + this.reportingTimestamp + ", " +
				"platformIdentifier=" + this.platformIdentifier + ", " +
				"primeBrokerageTransactionIndicator=" + this.primeBrokerageTransactionIndicator + ", " +
				"uniqueSwapIdentifier=" + this.uniqueSwapIdentifier + ", " +
				"uniqueTransactionIdentifier=" + this.uniqueTransactionIdentifier + ", " +
				"cftcGuidebook=" + this.cftcGuidebook + ", " +
				"nonReportable=" + this.nonReportable +
			'}';
		}
	}

	/*********************** Builder Implementation of CFTCPart43TransactionReport  ***********************/
	class CFTCPart43TransactionReportBuilderImpl implements CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder {
	
		protected String cleared;
		protected String counterparty1;
		protected String counterparty2;
		protected String counterparty2IdentifierSource;
		protected String submitterIdentifier;
		protected Boolean customBasketIndicator;
		protected String actionType;
		protected String eventType;
		protected Boolean amendmentIndicator;
		protected ZonedDateTime eventTimestamp;
		protected BigDecimal notionalAmountLeg1;
		protected BigDecimal notionalAmountLeg2;
		protected ISOCurrencyCodeEnum notionalCurrencyLeg1;
		protected ISOCurrencyCodeEnum notionalCurrencyLeg2;
		protected List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> notionalAmountScheduleLeg1 = new ArrayList<>();
		protected List<NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> notionalAmountScheduleLeg2 = new ArrayList<>();
		protected BigDecimal callAmount;
		protected ISOCurrencyCodeEnum callCurrency;
		protected BigDecimal putAmount;
		protected ISOCurrencyCodeEnum putCurrency;
		protected BigDecimal notionalQuantityLeg1;
		protected BigDecimal notionalQuantityLeg2;
		protected String quantityFrequencyLeg1;
		protected String quantityFrequencyLeg2;
		protected Integer quantityFrequencyMultiplierLeg1;
		protected Integer quantityFrequencyMultiplierLeg2;
		protected String quantityUnitOfMeasureLeg1;
		protected String quantityUnitOfMeasureLeg2;
		protected BigDecimal totalNotionalQuantityLeg1;
		protected BigDecimal totalNotionalQuantityLeg2;
		protected Boolean packageIndicator;
		protected BigDecimal packageTransactionPrice;
		protected ISOCurrencyCodeEnum packageTransactionPriceCurrency;
		protected String packageTransactionPriceNotation;
		protected BigDecimal packageTransactionSpread;
		protected ISOCurrencyCodeEnum packageTransactionSpreadCurrency;
		protected String packageTransactionSpreadNotation;
		protected InterestComputationMethod4Code fixedRateDayCountConventionLeg1;
		protected InterestComputationMethod4Code fixedRateDayCountConventionLeg2;
		protected InterestComputationMethod4Code floatingRateDayCountConventionLeg1;
		protected InterestComputationMethod4Code floatingRateDayCountConventionLeg2;
		protected String floatingRateResetFrequencyPeriodLeg1;
		protected String floatingRateResetFrequencyPeriodLeg2;
		protected Integer floatingRateResetFrequencyPeriodMultiplierLeg1;
		protected Integer floatingRateResetFrequencyPeriodMultiplierLeg2;
		protected List<OtherPaymentReport.OtherPaymentReportBuilder> otherPayment = new ArrayList<>();
		protected String fixedRatePaymentFrequencyPeriodLeg1;
		protected String fixedRatePaymentFrequencyPeriodLeg2;
		protected String floatingRatePaymentFrequencyPeriodLeg1;
		protected String floatingRatePaymentFrequencyPeriodLeg2;
		protected Integer fixedRatePaymentFrequencyPeriodMultiplierLeg1;
		protected Integer fixedRatePaymentFrequencyPeriodMultiplierLeg2;
		protected Integer floatingRatePaymentFrequencyPeriodMultiplierLeg1;
		protected Integer floatingRatePaymentFrequencyPeriodMultiplierLeg2;
		protected BigDecimal exchangeRateLeg1;
		protected String exchangeRateBasis;
		protected BigDecimal fixedRateLeg1;
		protected BigDecimal fixedRateLeg2;
		protected Boolean postPricedSwapIndicator;
		protected BigDecimal price;
		protected ISOCurrencyCodeEnum priceCurrency;
		protected String priceNotation;
		protected String priceUnitOfMeasure;
		protected BigDecimal spreadLeg1;
		protected BigDecimal spreadLeg2;
		protected ISOCurrencyCodeEnum spreadCurrencyLeg1;
		protected ISOCurrencyCodeEnum spreadCurrencyLeg2;
		protected String spreadNotationLeg1;
		protected String spreadNotationLeg2;
		protected BigDecimal strikePrice;
		protected String strikePriceCurrency;
		protected String strikePriceNotation;
		protected BigDecimal optionPremiumAmount;
		protected ISOCurrencyCodeEnum optionPremiumCurrency;
		protected Date firstExerciseDate;
		protected BigDecimal indexFactor;
		protected String embeddedOptionType;
		protected String uniqueProductIdentifier;
		protected ISOCurrencyCodeEnum settlementCurrencyLeg1;
		protected ISOCurrencyCodeEnum settlementCurrencyLeg2;
		protected String settlementLocationLeg1;
		protected String settlementLocationLeg2;
		protected Boolean nonStandardizedTermIndicator;
		protected Boolean blockTradeElectionIndicator;
		protected Date effectiveDate;
		protected Date expirationDate;
		protected ZonedDateTime executionTimestamp;
		protected ZonedDateTime reportingTimestamp;
		protected String platformIdentifier;
		protected Boolean primeBrokerageTransactionIndicator;
		protected String uniqueSwapIdentifier;
		protected String uniqueTransactionIdentifier;
		protected CFTCGuidebookPart43.CFTCGuidebookPart43Builder cftcGuidebook;
		protected CFTCNonReportablePart43.CFTCNonReportablePart43Builder nonReportable;
	
		public CFTCPart43TransactionReportBuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("cleared")
		public String getCleared() {
			return cleared;
		}
		
		@Override
		@RosettaAttribute("counterparty1")
		public String getCounterparty1() {
			return counterparty1;
		}
		
		@Override
		@RosettaAttribute("counterparty2")
		public String getCounterparty2() {
			return counterparty2;
		}
		
		@Override
		@RosettaAttribute("counterparty2IdentifierSource")
		public String getCounterparty2IdentifierSource() {
			return counterparty2IdentifierSource;
		}
		
		@Override
		@RosettaAttribute("submitterIdentifier")
		public String getSubmitterIdentifier() {
			return submitterIdentifier;
		}
		
		@Override
		@RosettaAttribute("customBasketIndicator")
		public Boolean getCustomBasketIndicator() {
			return customBasketIndicator;
		}
		
		@Override
		@RosettaAttribute("actionType")
		public String getActionType() {
			return actionType;
		}
		
		@Override
		@RosettaAttribute("eventType")
		public String getEventType() {
			return eventType;
		}
		
		@Override
		@RosettaAttribute("amendmentIndicator")
		public Boolean getAmendmentIndicator() {
			return amendmentIndicator;
		}
		
		@Override
		@RosettaAttribute("eventTimestamp")
		public ZonedDateTime getEventTimestamp() {
			return eventTimestamp;
		}
		
		@Override
		@RosettaAttribute("notionalAmountLeg1")
		public BigDecimal getNotionalAmountLeg1() {
			return notionalAmountLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalAmountLeg2")
		public BigDecimal getNotionalAmountLeg2() {
			return notionalAmountLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalCurrencyLeg1")
		public ISOCurrencyCodeEnum getNotionalCurrencyLeg1() {
			return notionalCurrencyLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalCurrencyLeg2")
		public ISOCurrencyCodeEnum getNotionalCurrencyLeg2() {
			return notionalCurrencyLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalAmountScheduleLeg1")
		public List<? extends NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> getNotionalAmountScheduleLeg1() {
			return notionalAmountScheduleLeg1;
		}
		
		public NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder getOrCreateNotionalAmountScheduleLeg1(int _index) {
		
			if (notionalAmountScheduleLeg1==null) {
				this.notionalAmountScheduleLeg1 = new ArrayList<>();
			}
			NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder result;
			return getIndex(notionalAmountScheduleLeg1, _index, () -> {
						NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder newNotionalAmountScheduleLeg1 = NotionalAmountScheduleLeg1Report.builder();
						return newNotionalAmountScheduleLeg1;
					});
		}
		
		@Override
		@RosettaAttribute("notionalAmountScheduleLeg2")
		public List<? extends NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> getNotionalAmountScheduleLeg2() {
			return notionalAmountScheduleLeg2;
		}
		
		public NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder getOrCreateNotionalAmountScheduleLeg2(int _index) {
		
			if (notionalAmountScheduleLeg2==null) {
				this.notionalAmountScheduleLeg2 = new ArrayList<>();
			}
			NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder result;
			return getIndex(notionalAmountScheduleLeg2, _index, () -> {
						NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder newNotionalAmountScheduleLeg2 = NotionalAmountScheduleLeg2Report.builder();
						return newNotionalAmountScheduleLeg2;
					});
		}
		
		@Override
		@RosettaAttribute("callAmount")
		public BigDecimal getCallAmount() {
			return callAmount;
		}
		
		@Override
		@RosettaAttribute("callCurrency")
		public ISOCurrencyCodeEnum getCallCurrency() {
			return callCurrency;
		}
		
		@Override
		@RosettaAttribute("putAmount")
		public BigDecimal getPutAmount() {
			return putAmount;
		}
		
		@Override
		@RosettaAttribute("putCurrency")
		public ISOCurrencyCodeEnum getPutCurrency() {
			return putCurrency;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityLeg1")
		public BigDecimal getNotionalQuantityLeg1() {
			return notionalQuantityLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityLeg2")
		public BigDecimal getNotionalQuantityLeg2() {
			return notionalQuantityLeg2;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyLeg1")
		public String getQuantityFrequencyLeg1() {
			return quantityFrequencyLeg1;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyLeg2")
		public String getQuantityFrequencyLeg2() {
			return quantityFrequencyLeg2;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyMultiplierLeg1")
		public Integer getQuantityFrequencyMultiplierLeg1() {
			return quantityFrequencyMultiplierLeg1;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyMultiplierLeg2")
		public Integer getQuantityFrequencyMultiplierLeg2() {
			return quantityFrequencyMultiplierLeg2;
		}
		
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg1")
		public String getQuantityUnitOfMeasureLeg1() {
			return quantityUnitOfMeasureLeg1;
		}
		
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg2")
		public String getQuantityUnitOfMeasureLeg2() {
			return quantityUnitOfMeasureLeg2;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg1")
		public BigDecimal getTotalNotionalQuantityLeg1() {
			return totalNotionalQuantityLeg1;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg2")
		public BigDecimal getTotalNotionalQuantityLeg2() {
			return totalNotionalQuantityLeg2;
		}
		
		@Override
		@RosettaAttribute("packageIndicator")
		public Boolean getPackageIndicator() {
			return packageIndicator;
		}
		
		@Override
		@RosettaAttribute("packageTransactionPrice")
		public BigDecimal getPackageTransactionPrice() {
			return packageTransactionPrice;
		}
		
		@Override
		@RosettaAttribute("packageTransactionPriceCurrency")
		public ISOCurrencyCodeEnum getPackageTransactionPriceCurrency() {
			return packageTransactionPriceCurrency;
		}
		
		@Override
		@RosettaAttribute("packageTransactionPriceNotation")
		public String getPackageTransactionPriceNotation() {
			return packageTransactionPriceNotation;
		}
		
		@Override
		@RosettaAttribute("packageTransactionSpread")
		public BigDecimal getPackageTransactionSpread() {
			return packageTransactionSpread;
		}
		
		@Override
		@RosettaAttribute("packageTransactionSpreadCurrency")
		public ISOCurrencyCodeEnum getPackageTransactionSpreadCurrency() {
			return packageTransactionSpreadCurrency;
		}
		
		@Override
		@RosettaAttribute("packageTransactionSpreadNotation")
		public String getPackageTransactionSpreadNotation() {
			return packageTransactionSpreadNotation;
		}
		
		@Override
		@RosettaAttribute("fixedRateDayCountConventionLeg1")
		public InterestComputationMethod4Code getFixedRateDayCountConventionLeg1() {
			return fixedRateDayCountConventionLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRateDayCountConventionLeg2")
		public InterestComputationMethod4Code getFixedRateDayCountConventionLeg2() {
			return fixedRateDayCountConventionLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateDayCountConventionLeg1")
		public InterestComputationMethod4Code getFloatingRateDayCountConventionLeg1() {
			return floatingRateDayCountConventionLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateDayCountConventionLeg2")
		public InterestComputationMethod4Code getFloatingRateDayCountConventionLeg2() {
			return floatingRateDayCountConventionLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodLeg1")
		public String getFloatingRateResetFrequencyPeriodLeg1() {
			return floatingRateResetFrequencyPeriodLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodLeg2")
		public String getFloatingRateResetFrequencyPeriodLeg2() {
			return floatingRateResetFrequencyPeriodLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierLeg1")
		public Integer getFloatingRateResetFrequencyPeriodMultiplierLeg1() {
			return floatingRateResetFrequencyPeriodMultiplierLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierLeg2")
		public Integer getFloatingRateResetFrequencyPeriodMultiplierLeg2() {
			return floatingRateResetFrequencyPeriodMultiplierLeg2;
		}
		
		@Override
		@RosettaAttribute("otherPayment")
		public List<? extends OtherPaymentReport.OtherPaymentReportBuilder> getOtherPayment() {
			return otherPayment;
		}
		
		public OtherPaymentReport.OtherPaymentReportBuilder getOrCreateOtherPayment(int _index) {
		
			if (otherPayment==null) {
				this.otherPayment = new ArrayList<>();
			}
			OtherPaymentReport.OtherPaymentReportBuilder result;
			return getIndex(otherPayment, _index, () -> {
						OtherPaymentReport.OtherPaymentReportBuilder newOtherPayment = OtherPaymentReport.builder();
						return newOtherPayment;
					});
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodLeg1")
		public String getFixedRatePaymentFrequencyPeriodLeg1() {
			return fixedRatePaymentFrequencyPeriodLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodLeg2")
		public String getFixedRatePaymentFrequencyPeriodLeg2() {
			return fixedRatePaymentFrequencyPeriodLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodLeg1")
		public String getFloatingRatePaymentFrequencyPeriodLeg1() {
			return floatingRatePaymentFrequencyPeriodLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodLeg2")
		public String getFloatingRatePaymentFrequencyPeriodLeg2() {
			return floatingRatePaymentFrequencyPeriodLeg2;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierLeg1")
		public Integer getFixedRatePaymentFrequencyPeriodMultiplierLeg1() {
			return fixedRatePaymentFrequencyPeriodMultiplierLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierLeg2")
		public Integer getFixedRatePaymentFrequencyPeriodMultiplierLeg2() {
			return fixedRatePaymentFrequencyPeriodMultiplierLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierLeg1")
		public Integer getFloatingRatePaymentFrequencyPeriodMultiplierLeg1() {
			return floatingRatePaymentFrequencyPeriodMultiplierLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierLeg2")
		public Integer getFloatingRatePaymentFrequencyPeriodMultiplierLeg2() {
			return floatingRatePaymentFrequencyPeriodMultiplierLeg2;
		}
		
		@Override
		@RosettaAttribute("exchangeRateLeg1")
		public BigDecimal getExchangeRateLeg1() {
			return exchangeRateLeg1;
		}
		
		@Override
		@RosettaAttribute("exchangeRateBasis")
		public String getExchangeRateBasis() {
			return exchangeRateBasis;
		}
		
		@Override
		@RosettaAttribute("fixedRateLeg1")
		public BigDecimal getFixedRateLeg1() {
			return fixedRateLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRateLeg2")
		public BigDecimal getFixedRateLeg2() {
			return fixedRateLeg2;
		}
		
		@Override
		@RosettaAttribute("postPricedSwapIndicator")
		public Boolean getPostPricedSwapIndicator() {
			return postPricedSwapIndicator;
		}
		
		@Override
		@RosettaAttribute("price")
		public BigDecimal getPrice() {
			return price;
		}
		
		@Override
		@RosettaAttribute("priceCurrency")
		public ISOCurrencyCodeEnum getPriceCurrency() {
			return priceCurrency;
		}
		
		@Override
		@RosettaAttribute("priceNotation")
		public String getPriceNotation() {
			return priceNotation;
		}
		
		@Override
		@RosettaAttribute("priceUnitOfMeasure")
		public String getPriceUnitOfMeasure() {
			return priceUnitOfMeasure;
		}
		
		@Override
		@RosettaAttribute("spreadLeg1")
		public BigDecimal getSpreadLeg1() {
			return spreadLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadLeg2")
		public BigDecimal getSpreadLeg2() {
			return spreadLeg2;
		}
		
		@Override
		@RosettaAttribute("spreadCurrencyLeg1")
		public ISOCurrencyCodeEnum getSpreadCurrencyLeg1() {
			return spreadCurrencyLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadCurrencyLeg2")
		public ISOCurrencyCodeEnum getSpreadCurrencyLeg2() {
			return spreadCurrencyLeg2;
		}
		
		@Override
		@RosettaAttribute("spreadNotationLeg1")
		public String getSpreadNotationLeg1() {
			return spreadNotationLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadNotationLeg2")
		public String getSpreadNotationLeg2() {
			return spreadNotationLeg2;
		}
		
		@Override
		@RosettaAttribute("strikePrice")
		public BigDecimal getStrikePrice() {
			return strikePrice;
		}
		
		@Override
		@RosettaAttribute("strikePriceCurrency")
		public String getStrikePriceCurrency() {
			return strikePriceCurrency;
		}
		
		@Override
		@RosettaAttribute("strikePriceNotation")
		public String getStrikePriceNotation() {
			return strikePriceNotation;
		}
		
		@Override
		@RosettaAttribute("optionPremiumAmount")
		public BigDecimal getOptionPremiumAmount() {
			return optionPremiumAmount;
		}
		
		@Override
		@RosettaAttribute("optionPremiumCurrency")
		public ISOCurrencyCodeEnum getOptionPremiumCurrency() {
			return optionPremiumCurrency;
		}
		
		@Override
		@RosettaAttribute("firstExerciseDate")
		public Date getFirstExerciseDate() {
			return firstExerciseDate;
		}
		
		@Override
		@RosettaAttribute("indexFactor")
		public BigDecimal getIndexFactor() {
			return indexFactor;
		}
		
		@Override
		@RosettaAttribute("embeddedOptionType")
		public String getEmbeddedOptionType() {
			return embeddedOptionType;
		}
		
		@Override
		@RosettaAttribute("uniqueProductIdentifier")
		public String getUniqueProductIdentifier() {
			return uniqueProductIdentifier;
		}
		
		@Override
		@RosettaAttribute("settlementCurrencyLeg1")
		public ISOCurrencyCodeEnum getSettlementCurrencyLeg1() {
			return settlementCurrencyLeg1;
		}
		
		@Override
		@RosettaAttribute("settlementCurrencyLeg2")
		public ISOCurrencyCodeEnum getSettlementCurrencyLeg2() {
			return settlementCurrencyLeg2;
		}
		
		@Override
		@RosettaAttribute("settlementLocationLeg1")
		public String getSettlementLocationLeg1() {
			return settlementLocationLeg1;
		}
		
		@Override
		@RosettaAttribute("settlementLocationLeg2")
		public String getSettlementLocationLeg2() {
			return settlementLocationLeg2;
		}
		
		@Override
		@RosettaAttribute("nonStandardizedTermIndicator")
		public Boolean getNonStandardizedTermIndicator() {
			return nonStandardizedTermIndicator;
		}
		
		@Override
		@RosettaAttribute("blockTradeElectionIndicator")
		public Boolean getBlockTradeElectionIndicator() {
			return blockTradeElectionIndicator;
		}
		
		@Override
		@RosettaAttribute("effectiveDate")
		public Date getEffectiveDate() {
			return effectiveDate;
		}
		
		@Override
		@RosettaAttribute("expirationDate")
		public Date getExpirationDate() {
			return expirationDate;
		}
		
		@Override
		@RosettaAttribute("executionTimestamp")
		public ZonedDateTime getExecutionTimestamp() {
			return executionTimestamp;
		}
		
		@Override
		@RosettaAttribute("reportingTimestamp")
		public ZonedDateTime getReportingTimestamp() {
			return reportingTimestamp;
		}
		
		@Override
		@RosettaAttribute("platformIdentifier")
		public String getPlatformIdentifier() {
			return platformIdentifier;
		}
		
		@Override
		@RosettaAttribute("primeBrokerageTransactionIndicator")
		public Boolean getPrimeBrokerageTransactionIndicator() {
			return primeBrokerageTransactionIndicator;
		}
		
		@Override
		@RosettaAttribute("uniqueSwapIdentifier")
		public String getUniqueSwapIdentifier() {
			return uniqueSwapIdentifier;
		}
		
		@Override
		@RosettaAttribute("uniqueTransactionIdentifier")
		public String getUniqueTransactionIdentifier() {
			return uniqueTransactionIdentifier;
		}
		
		@Override
		@RosettaAttribute("cftcGuidebook")
		public CFTCGuidebookPart43.CFTCGuidebookPart43Builder getCftcGuidebook() {
			return cftcGuidebook;
		}
		
		@Override
		public CFTCGuidebookPart43.CFTCGuidebookPart43Builder getOrCreateCftcGuidebook() {
			CFTCGuidebookPart43.CFTCGuidebookPart43Builder result;
			if (cftcGuidebook!=null) {
				result = cftcGuidebook;
			}
			else {
				result = cftcGuidebook = CFTCGuidebookPart43.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("nonReportable")
		public CFTCNonReportablePart43.CFTCNonReportablePart43Builder getNonReportable() {
			return nonReportable;
		}
		
		@Override
		public CFTCNonReportablePart43.CFTCNonReportablePart43Builder getOrCreateNonReportable() {
			CFTCNonReportablePart43.CFTCNonReportablePart43Builder result;
			if (nonReportable!=null) {
				result = nonReportable;
			}
			else {
				result = nonReportable = CFTCNonReportablePart43.builder();
			}
			
			return result;
		}
	
		@Override
		@RosettaAttribute("cleared")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCleared(String cleared) {
			this.cleared = cleared==null?null:cleared;
			return this;
		}
		@Override
		@RosettaAttribute("counterparty1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCounterparty1(String counterparty1) {
			this.counterparty1 = counterparty1==null?null:counterparty1;
			return this;
		}
		@Override
		@RosettaAttribute("counterparty2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCounterparty2(String counterparty2) {
			this.counterparty2 = counterparty2==null?null:counterparty2;
			return this;
		}
		@Override
		@RosettaAttribute("counterparty2IdentifierSource")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCounterparty2IdentifierSource(String counterparty2IdentifierSource) {
			this.counterparty2IdentifierSource = counterparty2IdentifierSource==null?null:counterparty2IdentifierSource;
			return this;
		}
		@Override
		@RosettaAttribute("submitterIdentifier")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSubmitterIdentifier(String submitterIdentifier) {
			this.submitterIdentifier = submitterIdentifier==null?null:submitterIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("customBasketIndicator")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCustomBasketIndicator(Boolean customBasketIndicator) {
			this.customBasketIndicator = customBasketIndicator==null?null:customBasketIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("actionType")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setActionType(String actionType) {
			this.actionType = actionType==null?null:actionType;
			return this;
		}
		@Override
		@RosettaAttribute("eventType")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setEventType(String eventType) {
			this.eventType = eventType==null?null:eventType;
			return this;
		}
		@Override
		@RosettaAttribute("amendmentIndicator")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setAmendmentIndicator(Boolean amendmentIndicator) {
			this.amendmentIndicator = amendmentIndicator==null?null:amendmentIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("eventTimestamp")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setEventTimestamp(ZonedDateTime eventTimestamp) {
			this.eventTimestamp = eventTimestamp==null?null:eventTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("notionalAmountLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalAmountLeg1(BigDecimal notionalAmountLeg1) {
			this.notionalAmountLeg1 = notionalAmountLeg1==null?null:notionalAmountLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("notionalAmountLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalAmountLeg2(BigDecimal notionalAmountLeg2) {
			this.notionalAmountLeg2 = notionalAmountLeg2==null?null:notionalAmountLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("notionalCurrencyLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalCurrencyLeg1(ISOCurrencyCodeEnum notionalCurrencyLeg1) {
			this.notionalCurrencyLeg1 = notionalCurrencyLeg1==null?null:notionalCurrencyLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("notionalCurrencyLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalCurrencyLeg2(ISOCurrencyCodeEnum notionalCurrencyLeg2) {
			this.notionalCurrencyLeg2 = notionalCurrencyLeg2==null?null:notionalCurrencyLeg2;
			return this;
		}
		@Override
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addNotionalAmountScheduleLeg1(NotionalAmountScheduleLeg1Report notionalAmountScheduleLeg1) {
			if (notionalAmountScheduleLeg1!=null) this.notionalAmountScheduleLeg1.add(notionalAmountScheduleLeg1.toBuilder());
			return this;
		}
		
		@Override
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addNotionalAmountScheduleLeg1(NotionalAmountScheduleLeg1Report notionalAmountScheduleLeg1, int _idx) {
			getIndex(this.notionalAmountScheduleLeg1, _idx, () -> notionalAmountScheduleLeg1.toBuilder());
			return this;
		}
		@Override 
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addNotionalAmountScheduleLeg1(List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg1s) {
			if (notionalAmountScheduleLeg1s != null) {
				for (NotionalAmountScheduleLeg1Report toAdd : notionalAmountScheduleLeg1s) {
					this.notionalAmountScheduleLeg1.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("notionalAmountScheduleLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalAmountScheduleLeg1(List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg1s) {
			if (notionalAmountScheduleLeg1s == null)  {
				this.notionalAmountScheduleLeg1 = new ArrayList<>();
			}
			else {
				this.notionalAmountScheduleLeg1 = notionalAmountScheduleLeg1s.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addNotionalAmountScheduleLeg2(NotionalAmountScheduleLeg2Report notionalAmountScheduleLeg2) {
			if (notionalAmountScheduleLeg2!=null) this.notionalAmountScheduleLeg2.add(notionalAmountScheduleLeg2.toBuilder());
			return this;
		}
		
		@Override
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addNotionalAmountScheduleLeg2(NotionalAmountScheduleLeg2Report notionalAmountScheduleLeg2, int _idx) {
			getIndex(this.notionalAmountScheduleLeg2, _idx, () -> notionalAmountScheduleLeg2.toBuilder());
			return this;
		}
		@Override 
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addNotionalAmountScheduleLeg2(List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg2s) {
			if (notionalAmountScheduleLeg2s != null) {
				for (NotionalAmountScheduleLeg2Report toAdd : notionalAmountScheduleLeg2s) {
					this.notionalAmountScheduleLeg2.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("notionalAmountScheduleLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalAmountScheduleLeg2(List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg2s) {
			if (notionalAmountScheduleLeg2s == null)  {
				this.notionalAmountScheduleLeg2 = new ArrayList<>();
			}
			else {
				this.notionalAmountScheduleLeg2 = notionalAmountScheduleLeg2s.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("callAmount")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCallAmount(BigDecimal callAmount) {
			this.callAmount = callAmount==null?null:callAmount;
			return this;
		}
		@Override
		@RosettaAttribute("callCurrency")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCallCurrency(ISOCurrencyCodeEnum callCurrency) {
			this.callCurrency = callCurrency==null?null:callCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("putAmount")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPutAmount(BigDecimal putAmount) {
			this.putAmount = putAmount==null?null:putAmount;
			return this;
		}
		@Override
		@RosettaAttribute("putCurrency")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPutCurrency(ISOCurrencyCodeEnum putCurrency) {
			this.putCurrency = putCurrency==null?null:putCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("notionalQuantityLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalQuantityLeg1(BigDecimal notionalQuantityLeg1) {
			this.notionalQuantityLeg1 = notionalQuantityLeg1==null?null:notionalQuantityLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("notionalQuantityLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNotionalQuantityLeg2(BigDecimal notionalQuantityLeg2) {
			this.notionalQuantityLeg2 = notionalQuantityLeg2==null?null:notionalQuantityLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("quantityFrequencyLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setQuantityFrequencyLeg1(String quantityFrequencyLeg1) {
			this.quantityFrequencyLeg1 = quantityFrequencyLeg1==null?null:quantityFrequencyLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("quantityFrequencyLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setQuantityFrequencyLeg2(String quantityFrequencyLeg2) {
			this.quantityFrequencyLeg2 = quantityFrequencyLeg2==null?null:quantityFrequencyLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("quantityFrequencyMultiplierLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setQuantityFrequencyMultiplierLeg1(Integer quantityFrequencyMultiplierLeg1) {
			this.quantityFrequencyMultiplierLeg1 = quantityFrequencyMultiplierLeg1==null?null:quantityFrequencyMultiplierLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("quantityFrequencyMultiplierLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setQuantityFrequencyMultiplierLeg2(Integer quantityFrequencyMultiplierLeg2) {
			this.quantityFrequencyMultiplierLeg2 = quantityFrequencyMultiplierLeg2==null?null:quantityFrequencyMultiplierLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setQuantityUnitOfMeasureLeg1(String quantityUnitOfMeasureLeg1) {
			this.quantityUnitOfMeasureLeg1 = quantityUnitOfMeasureLeg1==null?null:quantityUnitOfMeasureLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setQuantityUnitOfMeasureLeg2(String quantityUnitOfMeasureLeg2) {
			this.quantityUnitOfMeasureLeg2 = quantityUnitOfMeasureLeg2==null?null:quantityUnitOfMeasureLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setTotalNotionalQuantityLeg1(BigDecimal totalNotionalQuantityLeg1) {
			this.totalNotionalQuantityLeg1 = totalNotionalQuantityLeg1==null?null:totalNotionalQuantityLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setTotalNotionalQuantityLeg2(BigDecimal totalNotionalQuantityLeg2) {
			this.totalNotionalQuantityLeg2 = totalNotionalQuantityLeg2==null?null:totalNotionalQuantityLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("packageIndicator")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPackageIndicator(Boolean packageIndicator) {
			this.packageIndicator = packageIndicator==null?null:packageIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("packageTransactionPrice")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPackageTransactionPrice(BigDecimal packageTransactionPrice) {
			this.packageTransactionPrice = packageTransactionPrice==null?null:packageTransactionPrice;
			return this;
		}
		@Override
		@RosettaAttribute("packageTransactionPriceCurrency")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPackageTransactionPriceCurrency(ISOCurrencyCodeEnum packageTransactionPriceCurrency) {
			this.packageTransactionPriceCurrency = packageTransactionPriceCurrency==null?null:packageTransactionPriceCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("packageTransactionPriceNotation")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPackageTransactionPriceNotation(String packageTransactionPriceNotation) {
			this.packageTransactionPriceNotation = packageTransactionPriceNotation==null?null:packageTransactionPriceNotation;
			return this;
		}
		@Override
		@RosettaAttribute("packageTransactionSpread")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPackageTransactionSpread(BigDecimal packageTransactionSpread) {
			this.packageTransactionSpread = packageTransactionSpread==null?null:packageTransactionSpread;
			return this;
		}
		@Override
		@RosettaAttribute("packageTransactionSpreadCurrency")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPackageTransactionSpreadCurrency(ISOCurrencyCodeEnum packageTransactionSpreadCurrency) {
			this.packageTransactionSpreadCurrency = packageTransactionSpreadCurrency==null?null:packageTransactionSpreadCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("packageTransactionSpreadNotation")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPackageTransactionSpreadNotation(String packageTransactionSpreadNotation) {
			this.packageTransactionSpreadNotation = packageTransactionSpreadNotation==null?null:packageTransactionSpreadNotation;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRateDayCountConventionLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRateDayCountConventionLeg1(InterestComputationMethod4Code fixedRateDayCountConventionLeg1) {
			this.fixedRateDayCountConventionLeg1 = fixedRateDayCountConventionLeg1==null?null:fixedRateDayCountConventionLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRateDayCountConventionLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRateDayCountConventionLeg2(InterestComputationMethod4Code fixedRateDayCountConventionLeg2) {
			this.fixedRateDayCountConventionLeg2 = fixedRateDayCountConventionLeg2==null?null:fixedRateDayCountConventionLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateDayCountConventionLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRateDayCountConventionLeg1(InterestComputationMethod4Code floatingRateDayCountConventionLeg1) {
			this.floatingRateDayCountConventionLeg1 = floatingRateDayCountConventionLeg1==null?null:floatingRateDayCountConventionLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateDayCountConventionLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRateDayCountConventionLeg2(InterestComputationMethod4Code floatingRateDayCountConventionLeg2) {
			this.floatingRateDayCountConventionLeg2 = floatingRateDayCountConventionLeg2==null?null:floatingRateDayCountConventionLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRateResetFrequencyPeriodLeg1(String floatingRateResetFrequencyPeriodLeg1) {
			this.floatingRateResetFrequencyPeriodLeg1 = floatingRateResetFrequencyPeriodLeg1==null?null:floatingRateResetFrequencyPeriodLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRateResetFrequencyPeriodLeg2(String floatingRateResetFrequencyPeriodLeg2) {
			this.floatingRateResetFrequencyPeriodLeg2 = floatingRateResetFrequencyPeriodLeg2==null?null:floatingRateResetFrequencyPeriodLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRateResetFrequencyPeriodMultiplierLeg1(Integer floatingRateResetFrequencyPeriodMultiplierLeg1) {
			this.floatingRateResetFrequencyPeriodMultiplierLeg1 = floatingRateResetFrequencyPeriodMultiplierLeg1==null?null:floatingRateResetFrequencyPeriodMultiplierLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRateResetFrequencyPeriodMultiplierLeg2(Integer floatingRateResetFrequencyPeriodMultiplierLeg2) {
			this.floatingRateResetFrequencyPeriodMultiplierLeg2 = floatingRateResetFrequencyPeriodMultiplierLeg2==null?null:floatingRateResetFrequencyPeriodMultiplierLeg2;
			return this;
		}
		@Override
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment) {
			if (otherPayment!=null) this.otherPayment.add(otherPayment.toBuilder());
			return this;
		}
		
		@Override
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment, int _idx) {
			getIndex(this.otherPayment, _idx, () -> otherPayment.toBuilder());
			return this;
		}
		@Override 
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder addOtherPayment(List<? extends OtherPaymentReport> otherPayments) {
			if (otherPayments != null) {
				for (OtherPaymentReport toAdd : otherPayments) {
					this.otherPayment.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("otherPayment")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setOtherPayment(List<? extends OtherPaymentReport> otherPayments) {
			if (otherPayments == null)  {
				this.otherPayment = new ArrayList<>();
			}
			else {
				this.otherPayment = otherPayments.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRatePaymentFrequencyPeriodLeg1(String fixedRatePaymentFrequencyPeriodLeg1) {
			this.fixedRatePaymentFrequencyPeriodLeg1 = fixedRatePaymentFrequencyPeriodLeg1==null?null:fixedRatePaymentFrequencyPeriodLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRatePaymentFrequencyPeriodLeg2(String fixedRatePaymentFrequencyPeriodLeg2) {
			this.fixedRatePaymentFrequencyPeriodLeg2 = fixedRatePaymentFrequencyPeriodLeg2==null?null:fixedRatePaymentFrequencyPeriodLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRatePaymentFrequencyPeriodLeg1(String floatingRatePaymentFrequencyPeriodLeg1) {
			this.floatingRatePaymentFrequencyPeriodLeg1 = floatingRatePaymentFrequencyPeriodLeg1==null?null:floatingRatePaymentFrequencyPeriodLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRatePaymentFrequencyPeriodLeg2(String floatingRatePaymentFrequencyPeriodLeg2) {
			this.floatingRatePaymentFrequencyPeriodLeg2 = floatingRatePaymentFrequencyPeriodLeg2==null?null:floatingRatePaymentFrequencyPeriodLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRatePaymentFrequencyPeriodMultiplierLeg1(Integer fixedRatePaymentFrequencyPeriodMultiplierLeg1) {
			this.fixedRatePaymentFrequencyPeriodMultiplierLeg1 = fixedRatePaymentFrequencyPeriodMultiplierLeg1==null?null:fixedRatePaymentFrequencyPeriodMultiplierLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRatePaymentFrequencyPeriodMultiplierLeg2(Integer fixedRatePaymentFrequencyPeriodMultiplierLeg2) {
			this.fixedRatePaymentFrequencyPeriodMultiplierLeg2 = fixedRatePaymentFrequencyPeriodMultiplierLeg2==null?null:fixedRatePaymentFrequencyPeriodMultiplierLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRatePaymentFrequencyPeriodMultiplierLeg1(Integer floatingRatePaymentFrequencyPeriodMultiplierLeg1) {
			this.floatingRatePaymentFrequencyPeriodMultiplierLeg1 = floatingRatePaymentFrequencyPeriodMultiplierLeg1==null?null:floatingRatePaymentFrequencyPeriodMultiplierLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFloatingRatePaymentFrequencyPeriodMultiplierLeg2(Integer floatingRatePaymentFrequencyPeriodMultiplierLeg2) {
			this.floatingRatePaymentFrequencyPeriodMultiplierLeg2 = floatingRatePaymentFrequencyPeriodMultiplierLeg2==null?null:floatingRatePaymentFrequencyPeriodMultiplierLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("exchangeRateLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setExchangeRateLeg1(BigDecimal exchangeRateLeg1) {
			this.exchangeRateLeg1 = exchangeRateLeg1==null?null:exchangeRateLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("exchangeRateBasis")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setExchangeRateBasis(String exchangeRateBasis) {
			this.exchangeRateBasis = exchangeRateBasis==null?null:exchangeRateBasis;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRateLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRateLeg1(BigDecimal fixedRateLeg1) {
			this.fixedRateLeg1 = fixedRateLeg1==null?null:fixedRateLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRateLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFixedRateLeg2(BigDecimal fixedRateLeg2) {
			this.fixedRateLeg2 = fixedRateLeg2==null?null:fixedRateLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("postPricedSwapIndicator")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPostPricedSwapIndicator(Boolean postPricedSwapIndicator) {
			this.postPricedSwapIndicator = postPricedSwapIndicator==null?null:postPricedSwapIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("price")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPrice(BigDecimal price) {
			this.price = price==null?null:price;
			return this;
		}
		@Override
		@RosettaAttribute("priceCurrency")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPriceCurrency(ISOCurrencyCodeEnum priceCurrency) {
			this.priceCurrency = priceCurrency==null?null:priceCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("priceNotation")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPriceNotation(String priceNotation) {
			this.priceNotation = priceNotation==null?null:priceNotation;
			return this;
		}
		@Override
		@RosettaAttribute("priceUnitOfMeasure")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPriceUnitOfMeasure(String priceUnitOfMeasure) {
			this.priceUnitOfMeasure = priceUnitOfMeasure==null?null:priceUnitOfMeasure;
			return this;
		}
		@Override
		@RosettaAttribute("spreadLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSpreadLeg1(BigDecimal spreadLeg1) {
			this.spreadLeg1 = spreadLeg1==null?null:spreadLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("spreadLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSpreadLeg2(BigDecimal spreadLeg2) {
			this.spreadLeg2 = spreadLeg2==null?null:spreadLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("spreadCurrencyLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSpreadCurrencyLeg1(ISOCurrencyCodeEnum spreadCurrencyLeg1) {
			this.spreadCurrencyLeg1 = spreadCurrencyLeg1==null?null:spreadCurrencyLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("spreadCurrencyLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSpreadCurrencyLeg2(ISOCurrencyCodeEnum spreadCurrencyLeg2) {
			this.spreadCurrencyLeg2 = spreadCurrencyLeg2==null?null:spreadCurrencyLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("spreadNotationLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSpreadNotationLeg1(String spreadNotationLeg1) {
			this.spreadNotationLeg1 = spreadNotationLeg1==null?null:spreadNotationLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("spreadNotationLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSpreadNotationLeg2(String spreadNotationLeg2) {
			this.spreadNotationLeg2 = spreadNotationLeg2==null?null:spreadNotationLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("strikePrice")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setStrikePrice(BigDecimal strikePrice) {
			this.strikePrice = strikePrice==null?null:strikePrice;
			return this;
		}
		@Override
		@RosettaAttribute("strikePriceCurrency")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setStrikePriceCurrency(String strikePriceCurrency) {
			this.strikePriceCurrency = strikePriceCurrency==null?null:strikePriceCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("strikePriceNotation")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setStrikePriceNotation(String strikePriceNotation) {
			this.strikePriceNotation = strikePriceNotation==null?null:strikePriceNotation;
			return this;
		}
		@Override
		@RosettaAttribute("optionPremiumAmount")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setOptionPremiumAmount(BigDecimal optionPremiumAmount) {
			this.optionPremiumAmount = optionPremiumAmount==null?null:optionPremiumAmount;
			return this;
		}
		@Override
		@RosettaAttribute("optionPremiumCurrency")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setOptionPremiumCurrency(ISOCurrencyCodeEnum optionPremiumCurrency) {
			this.optionPremiumCurrency = optionPremiumCurrency==null?null:optionPremiumCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("firstExerciseDate")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setFirstExerciseDate(Date firstExerciseDate) {
			this.firstExerciseDate = firstExerciseDate==null?null:firstExerciseDate;
			return this;
		}
		@Override
		@RosettaAttribute("indexFactor")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setIndexFactor(BigDecimal indexFactor) {
			this.indexFactor = indexFactor==null?null:indexFactor;
			return this;
		}
		@Override
		@RosettaAttribute("embeddedOptionType")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setEmbeddedOptionType(String embeddedOptionType) {
			this.embeddedOptionType = embeddedOptionType==null?null:embeddedOptionType;
			return this;
		}
		@Override
		@RosettaAttribute("uniqueProductIdentifier")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setUniqueProductIdentifier(String uniqueProductIdentifier) {
			this.uniqueProductIdentifier = uniqueProductIdentifier==null?null:uniqueProductIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("settlementCurrencyLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSettlementCurrencyLeg1(ISOCurrencyCodeEnum settlementCurrencyLeg1) {
			this.settlementCurrencyLeg1 = settlementCurrencyLeg1==null?null:settlementCurrencyLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("settlementCurrencyLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSettlementCurrencyLeg2(ISOCurrencyCodeEnum settlementCurrencyLeg2) {
			this.settlementCurrencyLeg2 = settlementCurrencyLeg2==null?null:settlementCurrencyLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("settlementLocationLeg1")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSettlementLocationLeg1(String settlementLocationLeg1) {
			this.settlementLocationLeg1 = settlementLocationLeg1==null?null:settlementLocationLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("settlementLocationLeg2")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setSettlementLocationLeg2(String settlementLocationLeg2) {
			this.settlementLocationLeg2 = settlementLocationLeg2==null?null:settlementLocationLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("nonStandardizedTermIndicator")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNonStandardizedTermIndicator(Boolean nonStandardizedTermIndicator) {
			this.nonStandardizedTermIndicator = nonStandardizedTermIndicator==null?null:nonStandardizedTermIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("blockTradeElectionIndicator")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setBlockTradeElectionIndicator(Boolean blockTradeElectionIndicator) {
			this.blockTradeElectionIndicator = blockTradeElectionIndicator==null?null:blockTradeElectionIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("effectiveDate")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setEffectiveDate(Date effectiveDate) {
			this.effectiveDate = effectiveDate==null?null:effectiveDate;
			return this;
		}
		@Override
		@RosettaAttribute("expirationDate")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setExpirationDate(Date expirationDate) {
			this.expirationDate = expirationDate==null?null:expirationDate;
			return this;
		}
		@Override
		@RosettaAttribute("executionTimestamp")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setExecutionTimestamp(ZonedDateTime executionTimestamp) {
			this.executionTimestamp = executionTimestamp==null?null:executionTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("reportingTimestamp")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setReportingTimestamp(ZonedDateTime reportingTimestamp) {
			this.reportingTimestamp = reportingTimestamp==null?null:reportingTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("platformIdentifier")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPlatformIdentifier(String platformIdentifier) {
			this.platformIdentifier = platformIdentifier==null?null:platformIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("primeBrokerageTransactionIndicator")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setPrimeBrokerageTransactionIndicator(Boolean primeBrokerageTransactionIndicator) {
			this.primeBrokerageTransactionIndicator = primeBrokerageTransactionIndicator==null?null:primeBrokerageTransactionIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("uniqueSwapIdentifier")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setUniqueSwapIdentifier(String uniqueSwapIdentifier) {
			this.uniqueSwapIdentifier = uniqueSwapIdentifier==null?null:uniqueSwapIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("uniqueTransactionIdentifier")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setUniqueTransactionIdentifier(String uniqueTransactionIdentifier) {
			this.uniqueTransactionIdentifier = uniqueTransactionIdentifier==null?null:uniqueTransactionIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("cftcGuidebook")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setCftcGuidebook(CFTCGuidebookPart43 cftcGuidebook) {
			this.cftcGuidebook = cftcGuidebook==null?null:cftcGuidebook.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("nonReportable")
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder setNonReportable(CFTCNonReportablePart43 nonReportable) {
			this.nonReportable = nonReportable==null?null:nonReportable.toBuilder();
			return this;
		}
		
		@Override
		public CFTCPart43TransactionReport build() {
			return new CFTCPart43TransactionReport.CFTCPart43TransactionReportImpl(this);
		}
		
		@Override
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder prune() {
			notionalAmountScheduleLeg1 = notionalAmountScheduleLeg1.stream().filter(b->b!=null).<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			notionalAmountScheduleLeg2 = notionalAmountScheduleLeg2.stream().filter(b->b!=null).<NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			otherPayment = otherPayment.stream().filter(b->b!=null).<OtherPaymentReport.OtherPaymentReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (cftcGuidebook!=null && !cftcGuidebook.prune().hasData()) cftcGuidebook = null;
			if (nonReportable!=null && !nonReportable.prune().hasData()) nonReportable = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCleared()!=null) return true;
			if (getCounterparty1()!=null) return true;
			if (getCounterparty2()!=null) return true;
			if (getCounterparty2IdentifierSource()!=null) return true;
			if (getSubmitterIdentifier()!=null) return true;
			if (getCustomBasketIndicator()!=null) return true;
			if (getActionType()!=null) return true;
			if (getEventType()!=null) return true;
			if (getAmendmentIndicator()!=null) return true;
			if (getEventTimestamp()!=null) return true;
			if (getNotionalAmountLeg1()!=null) return true;
			if (getNotionalAmountLeg2()!=null) return true;
			if (getNotionalCurrencyLeg1()!=null) return true;
			if (getNotionalCurrencyLeg2()!=null) return true;
			if (getNotionalAmountScheduleLeg1()!=null && getNotionalAmountScheduleLeg1().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getNotionalAmountScheduleLeg2()!=null && getNotionalAmountScheduleLeg2().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getCallAmount()!=null) return true;
			if (getCallCurrency()!=null) return true;
			if (getPutAmount()!=null) return true;
			if (getPutCurrency()!=null) return true;
			if (getNotionalQuantityLeg1()!=null) return true;
			if (getNotionalQuantityLeg2()!=null) return true;
			if (getQuantityFrequencyLeg1()!=null) return true;
			if (getQuantityFrequencyLeg2()!=null) return true;
			if (getQuantityFrequencyMultiplierLeg1()!=null) return true;
			if (getQuantityFrequencyMultiplierLeg2()!=null) return true;
			if (getQuantityUnitOfMeasureLeg1()!=null) return true;
			if (getQuantityUnitOfMeasureLeg2()!=null) return true;
			if (getTotalNotionalQuantityLeg1()!=null) return true;
			if (getTotalNotionalQuantityLeg2()!=null) return true;
			if (getPackageIndicator()!=null) return true;
			if (getPackageTransactionPrice()!=null) return true;
			if (getPackageTransactionPriceCurrency()!=null) return true;
			if (getPackageTransactionPriceNotation()!=null) return true;
			if (getPackageTransactionSpread()!=null) return true;
			if (getPackageTransactionSpreadCurrency()!=null) return true;
			if (getPackageTransactionSpreadNotation()!=null) return true;
			if (getFixedRateDayCountConventionLeg1()!=null) return true;
			if (getFixedRateDayCountConventionLeg2()!=null) return true;
			if (getFloatingRateDayCountConventionLeg1()!=null) return true;
			if (getFloatingRateDayCountConventionLeg2()!=null) return true;
			if (getFloatingRateResetFrequencyPeriodLeg1()!=null) return true;
			if (getFloatingRateResetFrequencyPeriodLeg2()!=null) return true;
			if (getFloatingRateResetFrequencyPeriodMultiplierLeg1()!=null) return true;
			if (getFloatingRateResetFrequencyPeriodMultiplierLeg2()!=null) return true;
			if (getOtherPayment()!=null && getOtherPayment().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getFixedRatePaymentFrequencyPeriodLeg1()!=null) return true;
			if (getFixedRatePaymentFrequencyPeriodLeg2()!=null) return true;
			if (getFloatingRatePaymentFrequencyPeriodLeg1()!=null) return true;
			if (getFloatingRatePaymentFrequencyPeriodLeg2()!=null) return true;
			if (getFixedRatePaymentFrequencyPeriodMultiplierLeg1()!=null) return true;
			if (getFixedRatePaymentFrequencyPeriodMultiplierLeg2()!=null) return true;
			if (getFloatingRatePaymentFrequencyPeriodMultiplierLeg1()!=null) return true;
			if (getFloatingRatePaymentFrequencyPeriodMultiplierLeg2()!=null) return true;
			if (getExchangeRateLeg1()!=null) return true;
			if (getExchangeRateBasis()!=null) return true;
			if (getFixedRateLeg1()!=null) return true;
			if (getFixedRateLeg2()!=null) return true;
			if (getPostPricedSwapIndicator()!=null) return true;
			if (getPrice()!=null) return true;
			if (getPriceCurrency()!=null) return true;
			if (getPriceNotation()!=null) return true;
			if (getPriceUnitOfMeasure()!=null) return true;
			if (getSpreadLeg1()!=null) return true;
			if (getSpreadLeg2()!=null) return true;
			if (getSpreadCurrencyLeg1()!=null) return true;
			if (getSpreadCurrencyLeg2()!=null) return true;
			if (getSpreadNotationLeg1()!=null) return true;
			if (getSpreadNotationLeg2()!=null) return true;
			if (getStrikePrice()!=null) return true;
			if (getStrikePriceCurrency()!=null) return true;
			if (getStrikePriceNotation()!=null) return true;
			if (getOptionPremiumAmount()!=null) return true;
			if (getOptionPremiumCurrency()!=null) return true;
			if (getFirstExerciseDate()!=null) return true;
			if (getIndexFactor()!=null) return true;
			if (getEmbeddedOptionType()!=null) return true;
			if (getUniqueProductIdentifier()!=null) return true;
			if (getSettlementCurrencyLeg1()!=null) return true;
			if (getSettlementCurrencyLeg2()!=null) return true;
			if (getSettlementLocationLeg1()!=null) return true;
			if (getSettlementLocationLeg2()!=null) return true;
			if (getNonStandardizedTermIndicator()!=null) return true;
			if (getBlockTradeElectionIndicator()!=null) return true;
			if (getEffectiveDate()!=null) return true;
			if (getExpirationDate()!=null) return true;
			if (getExecutionTimestamp()!=null) return true;
			if (getReportingTimestamp()!=null) return true;
			if (getPlatformIdentifier()!=null) return true;
			if (getPrimeBrokerageTransactionIndicator()!=null) return true;
			if (getUniqueSwapIdentifier()!=null) return true;
			if (getUniqueTransactionIdentifier()!=null) return true;
			if (getCftcGuidebook()!=null && getCftcGuidebook().hasData()) return true;
			if (getNonReportable()!=null && getNonReportable().hasData()) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder o = (CFTCPart43TransactionReport.CFTCPart43TransactionReportBuilder) other;
			
			merger.mergeRosetta(getNotionalAmountScheduleLeg1(), o.getNotionalAmountScheduleLeg1(), this::getOrCreateNotionalAmountScheduleLeg1);
			merger.mergeRosetta(getNotionalAmountScheduleLeg2(), o.getNotionalAmountScheduleLeg2(), this::getOrCreateNotionalAmountScheduleLeg2);
			merger.mergeRosetta(getOtherPayment(), o.getOtherPayment(), this::getOrCreateOtherPayment);
			merger.mergeRosetta(getCftcGuidebook(), o.getCftcGuidebook(), this::setCftcGuidebook);
			merger.mergeRosetta(getNonReportable(), o.getNonReportable(), this::setNonReportable);
			
			merger.mergeBasic(getCleared(), o.getCleared(), this::setCleared);
			merger.mergeBasic(getCounterparty1(), o.getCounterparty1(), this::setCounterparty1);
			merger.mergeBasic(getCounterparty2(), o.getCounterparty2(), this::setCounterparty2);
			merger.mergeBasic(getCounterparty2IdentifierSource(), o.getCounterparty2IdentifierSource(), this::setCounterparty2IdentifierSource);
			merger.mergeBasic(getSubmitterIdentifier(), o.getSubmitterIdentifier(), this::setSubmitterIdentifier);
			merger.mergeBasic(getCustomBasketIndicator(), o.getCustomBasketIndicator(), this::setCustomBasketIndicator);
			merger.mergeBasic(getActionType(), o.getActionType(), this::setActionType);
			merger.mergeBasic(getEventType(), o.getEventType(), this::setEventType);
			merger.mergeBasic(getAmendmentIndicator(), o.getAmendmentIndicator(), this::setAmendmentIndicator);
			merger.mergeBasic(getEventTimestamp(), o.getEventTimestamp(), this::setEventTimestamp);
			merger.mergeBasic(getNotionalAmountLeg1(), o.getNotionalAmountLeg1(), this::setNotionalAmountLeg1);
			merger.mergeBasic(getNotionalAmountLeg2(), o.getNotionalAmountLeg2(), this::setNotionalAmountLeg2);
			merger.mergeBasic(getNotionalCurrencyLeg1(), o.getNotionalCurrencyLeg1(), this::setNotionalCurrencyLeg1);
			merger.mergeBasic(getNotionalCurrencyLeg2(), o.getNotionalCurrencyLeg2(), this::setNotionalCurrencyLeg2);
			merger.mergeBasic(getCallAmount(), o.getCallAmount(), this::setCallAmount);
			merger.mergeBasic(getCallCurrency(), o.getCallCurrency(), this::setCallCurrency);
			merger.mergeBasic(getPutAmount(), o.getPutAmount(), this::setPutAmount);
			merger.mergeBasic(getPutCurrency(), o.getPutCurrency(), this::setPutCurrency);
			merger.mergeBasic(getNotionalQuantityLeg1(), o.getNotionalQuantityLeg1(), this::setNotionalQuantityLeg1);
			merger.mergeBasic(getNotionalQuantityLeg2(), o.getNotionalQuantityLeg2(), this::setNotionalQuantityLeg2);
			merger.mergeBasic(getQuantityFrequencyLeg1(), o.getQuantityFrequencyLeg1(), this::setQuantityFrequencyLeg1);
			merger.mergeBasic(getQuantityFrequencyLeg2(), o.getQuantityFrequencyLeg2(), this::setQuantityFrequencyLeg2);
			merger.mergeBasic(getQuantityFrequencyMultiplierLeg1(), o.getQuantityFrequencyMultiplierLeg1(), this::setQuantityFrequencyMultiplierLeg1);
			merger.mergeBasic(getQuantityFrequencyMultiplierLeg2(), o.getQuantityFrequencyMultiplierLeg2(), this::setQuantityFrequencyMultiplierLeg2);
			merger.mergeBasic(getQuantityUnitOfMeasureLeg1(), o.getQuantityUnitOfMeasureLeg1(), this::setQuantityUnitOfMeasureLeg1);
			merger.mergeBasic(getQuantityUnitOfMeasureLeg2(), o.getQuantityUnitOfMeasureLeg2(), this::setQuantityUnitOfMeasureLeg2);
			merger.mergeBasic(getTotalNotionalQuantityLeg1(), o.getTotalNotionalQuantityLeg1(), this::setTotalNotionalQuantityLeg1);
			merger.mergeBasic(getTotalNotionalQuantityLeg2(), o.getTotalNotionalQuantityLeg2(), this::setTotalNotionalQuantityLeg2);
			merger.mergeBasic(getPackageIndicator(), o.getPackageIndicator(), this::setPackageIndicator);
			merger.mergeBasic(getPackageTransactionPrice(), o.getPackageTransactionPrice(), this::setPackageTransactionPrice);
			merger.mergeBasic(getPackageTransactionPriceCurrency(), o.getPackageTransactionPriceCurrency(), this::setPackageTransactionPriceCurrency);
			merger.mergeBasic(getPackageTransactionPriceNotation(), o.getPackageTransactionPriceNotation(), this::setPackageTransactionPriceNotation);
			merger.mergeBasic(getPackageTransactionSpread(), o.getPackageTransactionSpread(), this::setPackageTransactionSpread);
			merger.mergeBasic(getPackageTransactionSpreadCurrency(), o.getPackageTransactionSpreadCurrency(), this::setPackageTransactionSpreadCurrency);
			merger.mergeBasic(getPackageTransactionSpreadNotation(), o.getPackageTransactionSpreadNotation(), this::setPackageTransactionSpreadNotation);
			merger.mergeBasic(getFixedRateDayCountConventionLeg1(), o.getFixedRateDayCountConventionLeg1(), this::setFixedRateDayCountConventionLeg1);
			merger.mergeBasic(getFixedRateDayCountConventionLeg2(), o.getFixedRateDayCountConventionLeg2(), this::setFixedRateDayCountConventionLeg2);
			merger.mergeBasic(getFloatingRateDayCountConventionLeg1(), o.getFloatingRateDayCountConventionLeg1(), this::setFloatingRateDayCountConventionLeg1);
			merger.mergeBasic(getFloatingRateDayCountConventionLeg2(), o.getFloatingRateDayCountConventionLeg2(), this::setFloatingRateDayCountConventionLeg2);
			merger.mergeBasic(getFloatingRateResetFrequencyPeriodLeg1(), o.getFloatingRateResetFrequencyPeriodLeg1(), this::setFloatingRateResetFrequencyPeriodLeg1);
			merger.mergeBasic(getFloatingRateResetFrequencyPeriodLeg2(), o.getFloatingRateResetFrequencyPeriodLeg2(), this::setFloatingRateResetFrequencyPeriodLeg2);
			merger.mergeBasic(getFloatingRateResetFrequencyPeriodMultiplierLeg1(), o.getFloatingRateResetFrequencyPeriodMultiplierLeg1(), this::setFloatingRateResetFrequencyPeriodMultiplierLeg1);
			merger.mergeBasic(getFloatingRateResetFrequencyPeriodMultiplierLeg2(), o.getFloatingRateResetFrequencyPeriodMultiplierLeg2(), this::setFloatingRateResetFrequencyPeriodMultiplierLeg2);
			merger.mergeBasic(getFixedRatePaymentFrequencyPeriodLeg1(), o.getFixedRatePaymentFrequencyPeriodLeg1(), this::setFixedRatePaymentFrequencyPeriodLeg1);
			merger.mergeBasic(getFixedRatePaymentFrequencyPeriodLeg2(), o.getFixedRatePaymentFrequencyPeriodLeg2(), this::setFixedRatePaymentFrequencyPeriodLeg2);
			merger.mergeBasic(getFloatingRatePaymentFrequencyPeriodLeg1(), o.getFloatingRatePaymentFrequencyPeriodLeg1(), this::setFloatingRatePaymentFrequencyPeriodLeg1);
			merger.mergeBasic(getFloatingRatePaymentFrequencyPeriodLeg2(), o.getFloatingRatePaymentFrequencyPeriodLeg2(), this::setFloatingRatePaymentFrequencyPeriodLeg2);
			merger.mergeBasic(getFixedRatePaymentFrequencyPeriodMultiplierLeg1(), o.getFixedRatePaymentFrequencyPeriodMultiplierLeg1(), this::setFixedRatePaymentFrequencyPeriodMultiplierLeg1);
			merger.mergeBasic(getFixedRatePaymentFrequencyPeriodMultiplierLeg2(), o.getFixedRatePaymentFrequencyPeriodMultiplierLeg2(), this::setFixedRatePaymentFrequencyPeriodMultiplierLeg2);
			merger.mergeBasic(getFloatingRatePaymentFrequencyPeriodMultiplierLeg1(), o.getFloatingRatePaymentFrequencyPeriodMultiplierLeg1(), this::setFloatingRatePaymentFrequencyPeriodMultiplierLeg1);
			merger.mergeBasic(getFloatingRatePaymentFrequencyPeriodMultiplierLeg2(), o.getFloatingRatePaymentFrequencyPeriodMultiplierLeg2(), this::setFloatingRatePaymentFrequencyPeriodMultiplierLeg2);
			merger.mergeBasic(getExchangeRateLeg1(), o.getExchangeRateLeg1(), this::setExchangeRateLeg1);
			merger.mergeBasic(getExchangeRateBasis(), o.getExchangeRateBasis(), this::setExchangeRateBasis);
			merger.mergeBasic(getFixedRateLeg1(), o.getFixedRateLeg1(), this::setFixedRateLeg1);
			merger.mergeBasic(getFixedRateLeg2(), o.getFixedRateLeg2(), this::setFixedRateLeg2);
			merger.mergeBasic(getPostPricedSwapIndicator(), o.getPostPricedSwapIndicator(), this::setPostPricedSwapIndicator);
			merger.mergeBasic(getPrice(), o.getPrice(), this::setPrice);
			merger.mergeBasic(getPriceCurrency(), o.getPriceCurrency(), this::setPriceCurrency);
			merger.mergeBasic(getPriceNotation(), o.getPriceNotation(), this::setPriceNotation);
			merger.mergeBasic(getPriceUnitOfMeasure(), o.getPriceUnitOfMeasure(), this::setPriceUnitOfMeasure);
			merger.mergeBasic(getSpreadLeg1(), o.getSpreadLeg1(), this::setSpreadLeg1);
			merger.mergeBasic(getSpreadLeg2(), o.getSpreadLeg2(), this::setSpreadLeg2);
			merger.mergeBasic(getSpreadCurrencyLeg1(), o.getSpreadCurrencyLeg1(), this::setSpreadCurrencyLeg1);
			merger.mergeBasic(getSpreadCurrencyLeg2(), o.getSpreadCurrencyLeg2(), this::setSpreadCurrencyLeg2);
			merger.mergeBasic(getSpreadNotationLeg1(), o.getSpreadNotationLeg1(), this::setSpreadNotationLeg1);
			merger.mergeBasic(getSpreadNotationLeg2(), o.getSpreadNotationLeg2(), this::setSpreadNotationLeg2);
			merger.mergeBasic(getStrikePrice(), o.getStrikePrice(), this::setStrikePrice);
			merger.mergeBasic(getStrikePriceCurrency(), o.getStrikePriceCurrency(), this::setStrikePriceCurrency);
			merger.mergeBasic(getStrikePriceNotation(), o.getStrikePriceNotation(), this::setStrikePriceNotation);
			merger.mergeBasic(getOptionPremiumAmount(), o.getOptionPremiumAmount(), this::setOptionPremiumAmount);
			merger.mergeBasic(getOptionPremiumCurrency(), o.getOptionPremiumCurrency(), this::setOptionPremiumCurrency);
			merger.mergeBasic(getFirstExerciseDate(), o.getFirstExerciseDate(), this::setFirstExerciseDate);
			merger.mergeBasic(getIndexFactor(), o.getIndexFactor(), this::setIndexFactor);
			merger.mergeBasic(getEmbeddedOptionType(), o.getEmbeddedOptionType(), this::setEmbeddedOptionType);
			merger.mergeBasic(getUniqueProductIdentifier(), o.getUniqueProductIdentifier(), this::setUniqueProductIdentifier);
			merger.mergeBasic(getSettlementCurrencyLeg1(), o.getSettlementCurrencyLeg1(), this::setSettlementCurrencyLeg1);
			merger.mergeBasic(getSettlementCurrencyLeg2(), o.getSettlementCurrencyLeg2(), this::setSettlementCurrencyLeg2);
			merger.mergeBasic(getSettlementLocationLeg1(), o.getSettlementLocationLeg1(), this::setSettlementLocationLeg1);
			merger.mergeBasic(getSettlementLocationLeg2(), o.getSettlementLocationLeg2(), this::setSettlementLocationLeg2);
			merger.mergeBasic(getNonStandardizedTermIndicator(), o.getNonStandardizedTermIndicator(), this::setNonStandardizedTermIndicator);
			merger.mergeBasic(getBlockTradeElectionIndicator(), o.getBlockTradeElectionIndicator(), this::setBlockTradeElectionIndicator);
			merger.mergeBasic(getEffectiveDate(), o.getEffectiveDate(), this::setEffectiveDate);
			merger.mergeBasic(getExpirationDate(), o.getExpirationDate(), this::setExpirationDate);
			merger.mergeBasic(getExecutionTimestamp(), o.getExecutionTimestamp(), this::setExecutionTimestamp);
			merger.mergeBasic(getReportingTimestamp(), o.getReportingTimestamp(), this::setReportingTimestamp);
			merger.mergeBasic(getPlatformIdentifier(), o.getPlatformIdentifier(), this::setPlatformIdentifier);
			merger.mergeBasic(getPrimeBrokerageTransactionIndicator(), o.getPrimeBrokerageTransactionIndicator(), this::setPrimeBrokerageTransactionIndicator);
			merger.mergeBasic(getUniqueSwapIdentifier(), o.getUniqueSwapIdentifier(), this::setUniqueSwapIdentifier);
			merger.mergeBasic(getUniqueTransactionIdentifier(), o.getUniqueTransactionIdentifier(), this::setUniqueTransactionIdentifier);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			CFTCPart43TransactionReport _that = getType().cast(o);
		
			if (!Objects.equals(cleared, _that.getCleared())) return false;
			if (!Objects.equals(counterparty1, _that.getCounterparty1())) return false;
			if (!Objects.equals(counterparty2, _that.getCounterparty2())) return false;
			if (!Objects.equals(counterparty2IdentifierSource, _that.getCounterparty2IdentifierSource())) return false;
			if (!Objects.equals(submitterIdentifier, _that.getSubmitterIdentifier())) return false;
			if (!Objects.equals(customBasketIndicator, _that.getCustomBasketIndicator())) return false;
			if (!Objects.equals(actionType, _that.getActionType())) return false;
			if (!Objects.equals(eventType, _that.getEventType())) return false;
			if (!Objects.equals(amendmentIndicator, _that.getAmendmentIndicator())) return false;
			if (!Objects.equals(eventTimestamp, _that.getEventTimestamp())) return false;
			if (!Objects.equals(notionalAmountLeg1, _that.getNotionalAmountLeg1())) return false;
			if (!Objects.equals(notionalAmountLeg2, _that.getNotionalAmountLeg2())) return false;
			if (!Objects.equals(notionalCurrencyLeg1, _that.getNotionalCurrencyLeg1())) return false;
			if (!Objects.equals(notionalCurrencyLeg2, _that.getNotionalCurrencyLeg2())) return false;
			if (!ListEquals.listEquals(notionalAmountScheduleLeg1, _that.getNotionalAmountScheduleLeg1())) return false;
			if (!ListEquals.listEquals(notionalAmountScheduleLeg2, _that.getNotionalAmountScheduleLeg2())) return false;
			if (!Objects.equals(callAmount, _that.getCallAmount())) return false;
			if (!Objects.equals(callCurrency, _that.getCallCurrency())) return false;
			if (!Objects.equals(putAmount, _that.getPutAmount())) return false;
			if (!Objects.equals(putCurrency, _that.getPutCurrency())) return false;
			if (!Objects.equals(notionalQuantityLeg1, _that.getNotionalQuantityLeg1())) return false;
			if (!Objects.equals(notionalQuantityLeg2, _that.getNotionalQuantityLeg2())) return false;
			if (!Objects.equals(quantityFrequencyLeg1, _that.getQuantityFrequencyLeg1())) return false;
			if (!Objects.equals(quantityFrequencyLeg2, _that.getQuantityFrequencyLeg2())) return false;
			if (!Objects.equals(quantityFrequencyMultiplierLeg1, _that.getQuantityFrequencyMultiplierLeg1())) return false;
			if (!Objects.equals(quantityFrequencyMultiplierLeg2, _that.getQuantityFrequencyMultiplierLeg2())) return false;
			if (!Objects.equals(quantityUnitOfMeasureLeg1, _that.getQuantityUnitOfMeasureLeg1())) return false;
			if (!Objects.equals(quantityUnitOfMeasureLeg2, _that.getQuantityUnitOfMeasureLeg2())) return false;
			if (!Objects.equals(totalNotionalQuantityLeg1, _that.getTotalNotionalQuantityLeg1())) return false;
			if (!Objects.equals(totalNotionalQuantityLeg2, _that.getTotalNotionalQuantityLeg2())) return false;
			if (!Objects.equals(packageIndicator, _that.getPackageIndicator())) return false;
			if (!Objects.equals(packageTransactionPrice, _that.getPackageTransactionPrice())) return false;
			if (!Objects.equals(packageTransactionPriceCurrency, _that.getPackageTransactionPriceCurrency())) return false;
			if (!Objects.equals(packageTransactionPriceNotation, _that.getPackageTransactionPriceNotation())) return false;
			if (!Objects.equals(packageTransactionSpread, _that.getPackageTransactionSpread())) return false;
			if (!Objects.equals(packageTransactionSpreadCurrency, _that.getPackageTransactionSpreadCurrency())) return false;
			if (!Objects.equals(packageTransactionSpreadNotation, _that.getPackageTransactionSpreadNotation())) return false;
			if (!Objects.equals(fixedRateDayCountConventionLeg1, _that.getFixedRateDayCountConventionLeg1())) return false;
			if (!Objects.equals(fixedRateDayCountConventionLeg2, _that.getFixedRateDayCountConventionLeg2())) return false;
			if (!Objects.equals(floatingRateDayCountConventionLeg1, _that.getFloatingRateDayCountConventionLeg1())) return false;
			if (!Objects.equals(floatingRateDayCountConventionLeg2, _that.getFloatingRateDayCountConventionLeg2())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodLeg1, _that.getFloatingRateResetFrequencyPeriodLeg1())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodLeg2, _that.getFloatingRateResetFrequencyPeriodLeg2())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodMultiplierLeg1, _that.getFloatingRateResetFrequencyPeriodMultiplierLeg1())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodMultiplierLeg2, _that.getFloatingRateResetFrequencyPeriodMultiplierLeg2())) return false;
			if (!ListEquals.listEquals(otherPayment, _that.getOtherPayment())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodLeg1, _that.getFixedRatePaymentFrequencyPeriodLeg1())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodLeg2, _that.getFixedRatePaymentFrequencyPeriodLeg2())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodLeg1, _that.getFloatingRatePaymentFrequencyPeriodLeg1())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodLeg2, _that.getFloatingRatePaymentFrequencyPeriodLeg2())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodMultiplierLeg1, _that.getFixedRatePaymentFrequencyPeriodMultiplierLeg1())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodMultiplierLeg2, _that.getFixedRatePaymentFrequencyPeriodMultiplierLeg2())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodMultiplierLeg1, _that.getFloatingRatePaymentFrequencyPeriodMultiplierLeg1())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodMultiplierLeg2, _that.getFloatingRatePaymentFrequencyPeriodMultiplierLeg2())) return false;
			if (!Objects.equals(exchangeRateLeg1, _that.getExchangeRateLeg1())) return false;
			if (!Objects.equals(exchangeRateBasis, _that.getExchangeRateBasis())) return false;
			if (!Objects.equals(fixedRateLeg1, _that.getFixedRateLeg1())) return false;
			if (!Objects.equals(fixedRateLeg2, _that.getFixedRateLeg2())) return false;
			if (!Objects.equals(postPricedSwapIndicator, _that.getPostPricedSwapIndicator())) return false;
			if (!Objects.equals(price, _that.getPrice())) return false;
			if (!Objects.equals(priceCurrency, _that.getPriceCurrency())) return false;
			if (!Objects.equals(priceNotation, _that.getPriceNotation())) return false;
			if (!Objects.equals(priceUnitOfMeasure, _that.getPriceUnitOfMeasure())) return false;
			if (!Objects.equals(spreadLeg1, _that.getSpreadLeg1())) return false;
			if (!Objects.equals(spreadLeg2, _that.getSpreadLeg2())) return false;
			if (!Objects.equals(spreadCurrencyLeg1, _that.getSpreadCurrencyLeg1())) return false;
			if (!Objects.equals(spreadCurrencyLeg2, _that.getSpreadCurrencyLeg2())) return false;
			if (!Objects.equals(spreadNotationLeg1, _that.getSpreadNotationLeg1())) return false;
			if (!Objects.equals(spreadNotationLeg2, _that.getSpreadNotationLeg2())) return false;
			if (!Objects.equals(strikePrice, _that.getStrikePrice())) return false;
			if (!Objects.equals(strikePriceCurrency, _that.getStrikePriceCurrency())) return false;
			if (!Objects.equals(strikePriceNotation, _that.getStrikePriceNotation())) return false;
			if (!Objects.equals(optionPremiumAmount, _that.getOptionPremiumAmount())) return false;
			if (!Objects.equals(optionPremiumCurrency, _that.getOptionPremiumCurrency())) return false;
			if (!Objects.equals(firstExerciseDate, _that.getFirstExerciseDate())) return false;
			if (!Objects.equals(indexFactor, _that.getIndexFactor())) return false;
			if (!Objects.equals(embeddedOptionType, _that.getEmbeddedOptionType())) return false;
			if (!Objects.equals(uniqueProductIdentifier, _that.getUniqueProductIdentifier())) return false;
			if (!Objects.equals(settlementCurrencyLeg1, _that.getSettlementCurrencyLeg1())) return false;
			if (!Objects.equals(settlementCurrencyLeg2, _that.getSettlementCurrencyLeg2())) return false;
			if (!Objects.equals(settlementLocationLeg1, _that.getSettlementLocationLeg1())) return false;
			if (!Objects.equals(settlementLocationLeg2, _that.getSettlementLocationLeg2())) return false;
			if (!Objects.equals(nonStandardizedTermIndicator, _that.getNonStandardizedTermIndicator())) return false;
			if (!Objects.equals(blockTradeElectionIndicator, _that.getBlockTradeElectionIndicator())) return false;
			if (!Objects.equals(effectiveDate, _that.getEffectiveDate())) return false;
			if (!Objects.equals(expirationDate, _that.getExpirationDate())) return false;
			if (!Objects.equals(executionTimestamp, _that.getExecutionTimestamp())) return false;
			if (!Objects.equals(reportingTimestamp, _that.getReportingTimestamp())) return false;
			if (!Objects.equals(platformIdentifier, _that.getPlatformIdentifier())) return false;
			if (!Objects.equals(primeBrokerageTransactionIndicator, _that.getPrimeBrokerageTransactionIndicator())) return false;
			if (!Objects.equals(uniqueSwapIdentifier, _that.getUniqueSwapIdentifier())) return false;
			if (!Objects.equals(uniqueTransactionIdentifier, _that.getUniqueTransactionIdentifier())) return false;
			if (!Objects.equals(cftcGuidebook, _that.getCftcGuidebook())) return false;
			if (!Objects.equals(nonReportable, _that.getNonReportable())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (cleared != null ? cleared.hashCode() : 0);
			_result = 31 * _result + (counterparty1 != null ? counterparty1.hashCode() : 0);
			_result = 31 * _result + (counterparty2 != null ? counterparty2.hashCode() : 0);
			_result = 31 * _result + (counterparty2IdentifierSource != null ? counterparty2IdentifierSource.hashCode() : 0);
			_result = 31 * _result + (submitterIdentifier != null ? submitterIdentifier.hashCode() : 0);
			_result = 31 * _result + (customBasketIndicator != null ? customBasketIndicator.hashCode() : 0);
			_result = 31 * _result + (actionType != null ? actionType.hashCode() : 0);
			_result = 31 * _result + (eventType != null ? eventType.hashCode() : 0);
			_result = 31 * _result + (amendmentIndicator != null ? amendmentIndicator.hashCode() : 0);
			_result = 31 * _result + (eventTimestamp != null ? eventTimestamp.hashCode() : 0);
			_result = 31 * _result + (notionalAmountLeg1 != null ? notionalAmountLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalAmountLeg2 != null ? notionalAmountLeg2.hashCode() : 0);
			_result = 31 * _result + (notionalCurrencyLeg1 != null ? notionalCurrencyLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalCurrencyLeg2 != null ? notionalCurrencyLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalAmountScheduleLeg1 != null ? notionalAmountScheduleLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalAmountScheduleLeg2 != null ? notionalAmountScheduleLeg2.hashCode() : 0);
			_result = 31 * _result + (callAmount != null ? callAmount.hashCode() : 0);
			_result = 31 * _result + (callCurrency != null ? callCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (putAmount != null ? putAmount.hashCode() : 0);
			_result = 31 * _result + (putCurrency != null ? putCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalQuantityLeg1 != null ? notionalQuantityLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalQuantityLeg2 != null ? notionalQuantityLeg2.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyLeg1 != null ? quantityFrequencyLeg1.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyLeg2 != null ? quantityFrequencyLeg2.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyMultiplierLeg1 != null ? quantityFrequencyMultiplierLeg1.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyMultiplierLeg2 != null ? quantityFrequencyMultiplierLeg2.hashCode() : 0);
			_result = 31 * _result + (quantityUnitOfMeasureLeg1 != null ? quantityUnitOfMeasureLeg1.hashCode() : 0);
			_result = 31 * _result + (quantityUnitOfMeasureLeg2 != null ? quantityUnitOfMeasureLeg2.hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityLeg1 != null ? totalNotionalQuantityLeg1.hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityLeg2 != null ? totalNotionalQuantityLeg2.hashCode() : 0);
			_result = 31 * _result + (packageIndicator != null ? packageIndicator.hashCode() : 0);
			_result = 31 * _result + (packageTransactionPrice != null ? packageTransactionPrice.hashCode() : 0);
			_result = 31 * _result + (packageTransactionPriceCurrency != null ? packageTransactionPriceCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (packageTransactionPriceNotation != null ? packageTransactionPriceNotation.hashCode() : 0);
			_result = 31 * _result + (packageTransactionSpread != null ? packageTransactionSpread.hashCode() : 0);
			_result = 31 * _result + (packageTransactionSpreadCurrency != null ? packageTransactionSpreadCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (packageTransactionSpreadNotation != null ? packageTransactionSpreadNotation.hashCode() : 0);
			_result = 31 * _result + (fixedRateDayCountConventionLeg1 != null ? fixedRateDayCountConventionLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (fixedRateDayCountConventionLeg2 != null ? fixedRateDayCountConventionLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRateDayCountConventionLeg1 != null ? floatingRateDayCountConventionLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRateDayCountConventionLeg2 != null ? floatingRateDayCountConventionLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodLeg1 != null ? floatingRateResetFrequencyPeriodLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodLeg2 != null ? floatingRateResetFrequencyPeriodLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodMultiplierLeg1 != null ? floatingRateResetFrequencyPeriodMultiplierLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodMultiplierLeg2 != null ? floatingRateResetFrequencyPeriodMultiplierLeg2.hashCode() : 0);
			_result = 31 * _result + (otherPayment != null ? otherPayment.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodLeg1 != null ? fixedRatePaymentFrequencyPeriodLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodLeg2 != null ? fixedRatePaymentFrequencyPeriodLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodLeg1 != null ? floatingRatePaymentFrequencyPeriodLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodLeg2 != null ? floatingRatePaymentFrequencyPeriodLeg2.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodMultiplierLeg1 != null ? fixedRatePaymentFrequencyPeriodMultiplierLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodMultiplierLeg2 != null ? fixedRatePaymentFrequencyPeriodMultiplierLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodMultiplierLeg1 != null ? floatingRatePaymentFrequencyPeriodMultiplierLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodMultiplierLeg2 != null ? floatingRatePaymentFrequencyPeriodMultiplierLeg2.hashCode() : 0);
			_result = 31 * _result + (exchangeRateLeg1 != null ? exchangeRateLeg1.hashCode() : 0);
			_result = 31 * _result + (exchangeRateBasis != null ? exchangeRateBasis.hashCode() : 0);
			_result = 31 * _result + (fixedRateLeg1 != null ? fixedRateLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRateLeg2 != null ? fixedRateLeg2.hashCode() : 0);
			_result = 31 * _result + (postPricedSwapIndicator != null ? postPricedSwapIndicator.hashCode() : 0);
			_result = 31 * _result + (price != null ? price.hashCode() : 0);
			_result = 31 * _result + (priceCurrency != null ? priceCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (priceNotation != null ? priceNotation.hashCode() : 0);
			_result = 31 * _result + (priceUnitOfMeasure != null ? priceUnitOfMeasure.hashCode() : 0);
			_result = 31 * _result + (spreadLeg1 != null ? spreadLeg1.hashCode() : 0);
			_result = 31 * _result + (spreadLeg2 != null ? spreadLeg2.hashCode() : 0);
			_result = 31 * _result + (spreadCurrencyLeg1 != null ? spreadCurrencyLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (spreadCurrencyLeg2 != null ? spreadCurrencyLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (spreadNotationLeg1 != null ? spreadNotationLeg1.hashCode() : 0);
			_result = 31 * _result + (spreadNotationLeg2 != null ? spreadNotationLeg2.hashCode() : 0);
			_result = 31 * _result + (strikePrice != null ? strikePrice.hashCode() : 0);
			_result = 31 * _result + (strikePriceCurrency != null ? strikePriceCurrency.hashCode() : 0);
			_result = 31 * _result + (strikePriceNotation != null ? strikePriceNotation.hashCode() : 0);
			_result = 31 * _result + (optionPremiumAmount != null ? optionPremiumAmount.hashCode() : 0);
			_result = 31 * _result + (optionPremiumCurrency != null ? optionPremiumCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (firstExerciseDate != null ? firstExerciseDate.hashCode() : 0);
			_result = 31 * _result + (indexFactor != null ? indexFactor.hashCode() : 0);
			_result = 31 * _result + (embeddedOptionType != null ? embeddedOptionType.hashCode() : 0);
			_result = 31 * _result + (uniqueProductIdentifier != null ? uniqueProductIdentifier.hashCode() : 0);
			_result = 31 * _result + (settlementCurrencyLeg1 != null ? settlementCurrencyLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (settlementCurrencyLeg2 != null ? settlementCurrencyLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (settlementLocationLeg1 != null ? settlementLocationLeg1.hashCode() : 0);
			_result = 31 * _result + (settlementLocationLeg2 != null ? settlementLocationLeg2.hashCode() : 0);
			_result = 31 * _result + (nonStandardizedTermIndicator != null ? nonStandardizedTermIndicator.hashCode() : 0);
			_result = 31 * _result + (blockTradeElectionIndicator != null ? blockTradeElectionIndicator.hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (expirationDate != null ? expirationDate.hashCode() : 0);
			_result = 31 * _result + (executionTimestamp != null ? executionTimestamp.hashCode() : 0);
			_result = 31 * _result + (reportingTimestamp != null ? reportingTimestamp.hashCode() : 0);
			_result = 31 * _result + (platformIdentifier != null ? platformIdentifier.hashCode() : 0);
			_result = 31 * _result + (primeBrokerageTransactionIndicator != null ? primeBrokerageTransactionIndicator.hashCode() : 0);
			_result = 31 * _result + (uniqueSwapIdentifier != null ? uniqueSwapIdentifier.hashCode() : 0);
			_result = 31 * _result + (uniqueTransactionIdentifier != null ? uniqueTransactionIdentifier.hashCode() : 0);
			_result = 31 * _result + (cftcGuidebook != null ? cftcGuidebook.hashCode() : 0);
			_result = 31 * _result + (nonReportable != null ? nonReportable.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "CFTCPart43TransactionReportBuilder {" +
				"cleared=" + this.cleared + ", " +
				"counterparty1=" + this.counterparty1 + ", " +
				"counterparty2=" + this.counterparty2 + ", " +
				"counterparty2IdentifierSource=" + this.counterparty2IdentifierSource + ", " +
				"submitterIdentifier=" + this.submitterIdentifier + ", " +
				"customBasketIndicator=" + this.customBasketIndicator + ", " +
				"actionType=" + this.actionType + ", " +
				"eventType=" + this.eventType + ", " +
				"amendmentIndicator=" + this.amendmentIndicator + ", " +
				"eventTimestamp=" + this.eventTimestamp + ", " +
				"notionalAmountLeg1=" + this.notionalAmountLeg1 + ", " +
				"notionalAmountLeg2=" + this.notionalAmountLeg2 + ", " +
				"notionalCurrencyLeg1=" + this.notionalCurrencyLeg1 + ", " +
				"notionalCurrencyLeg2=" + this.notionalCurrencyLeg2 + ", " +
				"notionalAmountScheduleLeg1=" + this.notionalAmountScheduleLeg1 + ", " +
				"notionalAmountScheduleLeg2=" + this.notionalAmountScheduleLeg2 + ", " +
				"callAmount=" + this.callAmount + ", " +
				"callCurrency=" + this.callCurrency + ", " +
				"putAmount=" + this.putAmount + ", " +
				"putCurrency=" + this.putCurrency + ", " +
				"notionalQuantityLeg1=" + this.notionalQuantityLeg1 + ", " +
				"notionalQuantityLeg2=" + this.notionalQuantityLeg2 + ", " +
				"quantityFrequencyLeg1=" + this.quantityFrequencyLeg1 + ", " +
				"quantityFrequencyLeg2=" + this.quantityFrequencyLeg2 + ", " +
				"quantityFrequencyMultiplierLeg1=" + this.quantityFrequencyMultiplierLeg1 + ", " +
				"quantityFrequencyMultiplierLeg2=" + this.quantityFrequencyMultiplierLeg2 + ", " +
				"quantityUnitOfMeasureLeg1=" + this.quantityUnitOfMeasureLeg1 + ", " +
				"quantityUnitOfMeasureLeg2=" + this.quantityUnitOfMeasureLeg2 + ", " +
				"totalNotionalQuantityLeg1=" + this.totalNotionalQuantityLeg1 + ", " +
				"totalNotionalQuantityLeg2=" + this.totalNotionalQuantityLeg2 + ", " +
				"packageIndicator=" + this.packageIndicator + ", " +
				"packageTransactionPrice=" + this.packageTransactionPrice + ", " +
				"packageTransactionPriceCurrency=" + this.packageTransactionPriceCurrency + ", " +
				"packageTransactionPriceNotation=" + this.packageTransactionPriceNotation + ", " +
				"packageTransactionSpread=" + this.packageTransactionSpread + ", " +
				"packageTransactionSpreadCurrency=" + this.packageTransactionSpreadCurrency + ", " +
				"packageTransactionSpreadNotation=" + this.packageTransactionSpreadNotation + ", " +
				"fixedRateDayCountConventionLeg1=" + this.fixedRateDayCountConventionLeg1 + ", " +
				"fixedRateDayCountConventionLeg2=" + this.fixedRateDayCountConventionLeg2 + ", " +
				"floatingRateDayCountConventionLeg1=" + this.floatingRateDayCountConventionLeg1 + ", " +
				"floatingRateDayCountConventionLeg2=" + this.floatingRateDayCountConventionLeg2 + ", " +
				"floatingRateResetFrequencyPeriodLeg1=" + this.floatingRateResetFrequencyPeriodLeg1 + ", " +
				"floatingRateResetFrequencyPeriodLeg2=" + this.floatingRateResetFrequencyPeriodLeg2 + ", " +
				"floatingRateResetFrequencyPeriodMultiplierLeg1=" + this.floatingRateResetFrequencyPeriodMultiplierLeg1 + ", " +
				"floatingRateResetFrequencyPeriodMultiplierLeg2=" + this.floatingRateResetFrequencyPeriodMultiplierLeg2 + ", " +
				"otherPayment=" + this.otherPayment + ", " +
				"fixedRatePaymentFrequencyPeriodLeg1=" + this.fixedRatePaymentFrequencyPeriodLeg1 + ", " +
				"fixedRatePaymentFrequencyPeriodLeg2=" + this.fixedRatePaymentFrequencyPeriodLeg2 + ", " +
				"floatingRatePaymentFrequencyPeriodLeg1=" + this.floatingRatePaymentFrequencyPeriodLeg1 + ", " +
				"floatingRatePaymentFrequencyPeriodLeg2=" + this.floatingRatePaymentFrequencyPeriodLeg2 + ", " +
				"fixedRatePaymentFrequencyPeriodMultiplierLeg1=" + this.fixedRatePaymentFrequencyPeriodMultiplierLeg1 + ", " +
				"fixedRatePaymentFrequencyPeriodMultiplierLeg2=" + this.fixedRatePaymentFrequencyPeriodMultiplierLeg2 + ", " +
				"floatingRatePaymentFrequencyPeriodMultiplierLeg1=" + this.floatingRatePaymentFrequencyPeriodMultiplierLeg1 + ", " +
				"floatingRatePaymentFrequencyPeriodMultiplierLeg2=" + this.floatingRatePaymentFrequencyPeriodMultiplierLeg2 + ", " +
				"exchangeRateLeg1=" + this.exchangeRateLeg1 + ", " +
				"exchangeRateBasis=" + this.exchangeRateBasis + ", " +
				"fixedRateLeg1=" + this.fixedRateLeg1 + ", " +
				"fixedRateLeg2=" + this.fixedRateLeg2 + ", " +
				"postPricedSwapIndicator=" + this.postPricedSwapIndicator + ", " +
				"price=" + this.price + ", " +
				"priceCurrency=" + this.priceCurrency + ", " +
				"priceNotation=" + this.priceNotation + ", " +
				"priceUnitOfMeasure=" + this.priceUnitOfMeasure + ", " +
				"spreadLeg1=" + this.spreadLeg1 + ", " +
				"spreadLeg2=" + this.spreadLeg2 + ", " +
				"spreadCurrencyLeg1=" + this.spreadCurrencyLeg1 + ", " +
				"spreadCurrencyLeg2=" + this.spreadCurrencyLeg2 + ", " +
				"spreadNotationLeg1=" + this.spreadNotationLeg1 + ", " +
				"spreadNotationLeg2=" + this.spreadNotationLeg2 + ", " +
				"strikePrice=" + this.strikePrice + ", " +
				"strikePriceCurrency=" + this.strikePriceCurrency + ", " +
				"strikePriceNotation=" + this.strikePriceNotation + ", " +
				"optionPremiumAmount=" + this.optionPremiumAmount + ", " +
				"optionPremiumCurrency=" + this.optionPremiumCurrency + ", " +
				"firstExerciseDate=" + this.firstExerciseDate + ", " +
				"indexFactor=" + this.indexFactor + ", " +
				"embeddedOptionType=" + this.embeddedOptionType + ", " +
				"uniqueProductIdentifier=" + this.uniqueProductIdentifier + ", " +
				"settlementCurrencyLeg1=" + this.settlementCurrencyLeg1 + ", " +
				"settlementCurrencyLeg2=" + this.settlementCurrencyLeg2 + ", " +
				"settlementLocationLeg1=" + this.settlementLocationLeg1 + ", " +
				"settlementLocationLeg2=" + this.settlementLocationLeg2 + ", " +
				"nonStandardizedTermIndicator=" + this.nonStandardizedTermIndicator + ", " +
				"blockTradeElectionIndicator=" + this.blockTradeElectionIndicator + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"expirationDate=" + this.expirationDate + ", " +
				"executionTimestamp=" + this.executionTimestamp + ", " +
				"reportingTimestamp=" + this.reportingTimestamp + ", " +
				"platformIdentifier=" + this.platformIdentifier + ", " +
				"primeBrokerageTransactionIndicator=" + this.primeBrokerageTransactionIndicator + ", " +
				"uniqueSwapIdentifier=" + this.uniqueSwapIdentifier + ", " +
				"uniqueTransactionIdentifier=" + this.uniqueTransactionIdentifier + ", " +
				"cftcGuidebook=" + this.cftcGuidebook + ", " +
				"nonReportable=" + this.nonReportable +
			'}';
		}
	}
}
