package drr.regulation.esma.emir.refit.trade;

import cdm.base.staticdata.asset.common.ISOCountryCodeEnum;
import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import com.google.common.collect.ImmutableList;
import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import com.rosetta.model.lib.records.Date;
import com.rosetta.util.ListEquals;
import drr.regulation.esma.emir.refit.trade.BasketConstituentsReport;
import drr.regulation.esma.emir.refit.trade.DeliveryReport;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRNonReportable;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRTransactionReport;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilderImpl;
import drr.regulation.esma.emir.refit.trade.ESMAEMIRTransactionReport.ESMAEMIRTransactionReportImpl;
import drr.regulation.esma.emir.refit.trade.NotionalAmountScheduleLeg1Report;
import drr.regulation.esma.emir.refit.trade.NotionalAmountScheduleLeg2Report;
import drr.regulation.esma.emir.refit.trade.NotionalQuantityScheduleLeg1Report;
import drr.regulation.esma.emir.refit.trade.NotionalQuantityScheduleLeg2Report;
import drr.regulation.esma.emir.refit.trade.OtherPaymentReport;
import drr.regulation.esma.emir.refit.trade.PackageTransactionPrice;
import drr.regulation.esma.emir.refit.trade.PackageTransactionSpread;
import drr.regulation.esma.emir.refit.trade.PriceAndNotation;
import drr.regulation.esma.emir.refit.trade.PriceScheduleReport;
import drr.regulation.esma.emir.refit.trade.SpreadOfLeg1;
import drr.regulation.esma.emir.refit.trade.SpreadOfLeg2;
import drr.regulation.esma.emir.refit.trade.StrikePriceAndNotation;
import drr.regulation.esma.emir.refit.trade.StrikePriceScheduleReport;
import drr.regulation.esma.emir.refit.trade.meta.ESMAEMIRTransactionReportMeta;
import iso20022.auth030.esma.EnergyLoadType1Code;
import iso20022.auth030.esma.Frequency13Code__1;
import iso20022.auth030.esma.InterestComputationMethod4Code;
import iso20022.auth030.esma.ValuationType1Code;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.function.Consumer;
import java.util.stream.Collectors;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="ESMAEMIRTransactionReport", builder=ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilderImpl.class, version="4.2.1")
public interface ESMAEMIRTransactionReport extends RosettaModelObject {

	ESMAEMIRTransactionReportMeta metaData = new ESMAEMIRTransactionReportMeta();

	/*********************** Getter Methods  ***********************/
	ZonedDateTime getReportingTimestamp();
	String getReportSubmittingEntityID();
	String getEntityResponsibleForReporting();
	String getCounterparty1();
	String getNatureOfCounterparty1();
	String getCorporateSectorOfTheCounterparty1();
	Boolean getClearingThresholdOfCounterparty1();
	String getCounterparty2();
	ISOCountryCodeEnum getCountryOfTheCounterParty2();
	String getNatureOfCounterparty2();
	String getCorporateSectorOfTheCounterparty2();
	Boolean getCounterparty2IdentifierType();
	String getClearingMember();
	Boolean getClearingThresholdOfCounterparty2();
	Boolean getReportingObligationOfTheCounterparty2();
	String getBrokerID();
	String getDirection();
	String getDirectionOfLeg1();
	String getDirectionOfLeg2();
	Boolean getDirectlyLinkedToCommercialActivityOrTreasuryFinancing();
	String getUti();
	String getUtiProprietary();
	String getReportTrackingNumber();
	String getPriorUti();
	String getPriorUtiProprietary();
	String getPtrrId();
	String getPackageIdentifier();
	String getSubsequentPositionUti();
	String getIsin();
	String getUpi();
	String getProductClassification();
	String getContractType();
	String getAssetClass();
	Boolean getIsCrypto();
	String getUnderlyingIdentificationType();
	String getUnderlyingIdentification();
	List<String> getNameOfTheUnderlyingIndex();
	List<String> getIndicatorOfTheUnderlyingIndex();
	List<? extends BasketConstituentsReport> getBasketConstituents();
	String getCustomBasketCode();
	ISOCurrencyCodeEnum getSettlementCurrency1();
	ISOCurrencyCodeEnum getSettlementCurrency2();
	BigDecimal getValuationAmount();
	ISOCurrencyCodeEnum getValuationCurrency();
	ValuationType1Code getValuationMethod();
	BigDecimal getDelta();
	ZonedDateTime getValuationTimestamp();
	ZonedDateTime getConfirmationTimestamp();
	String getConfirmed();
	String getClearingObligation();
	String getCleared();
	ZonedDateTime getClearingTimestamp();
	String getCentralCounterparty();
	String getMasterAgreementType();
	String getOtherMasterAgreementType();
	Integer getMasterAgreementVersion();
	Boolean getIntragroup();
	Boolean getPtrr();
	String getTypeOfPTRRTechnique();
	String getPtrrServiceProvider();
	String getVenueOfExecution();
	ZonedDateTime getExecutionTimestamp();
	Date getEffectiveDate();
	Date getExpirationDate();
	Date getEarlyTerminationDate();
	Date getFinalContractualSettlementDate();
	String getDeliveryType();
	PriceAndNotation getPrice();
	List<? extends PriceScheduleReport> getPriceSchedule();
	PackageTransactionPrice getPackageTransactionPrice();
	BigDecimal getNotionalLeg1();
	ISOCurrencyCodeEnum getNotionalCurrency1();
	List<? extends NotionalAmountScheduleLeg1Report> getNotionalAmountScheduleLeg1();
	BigDecimal getTotalNotionalQuantityLeg1();
	List<? extends NotionalQuantityScheduleLeg1Report> getNotionalQuantityScheduleLeg1();
	BigDecimal getTotalNotionalQuantityLeg2();
	BigDecimal getNotionalLeg2();
	ISOCurrencyCodeEnum getNotionalCurrency2();
	List<? extends NotionalAmountScheduleLeg2Report> getNotionalAmountScheduleLeg2();
	List<? extends NotionalQuantityScheduleLeg2Report> getNotionalQuantityScheduleLeg2();
	List<? extends OtherPaymentReport> getOtherPayment();
	BigDecimal getFixedRateOfLeg1();
	InterestComputationMethod4Code getFixedRateDayCountConventionLeg1();
	String getFixedRatePaymentFrequencyPeriodLeg1();
	Integer getFixedRatePaymentFrequencyPeriodMultiplierLeg1();
	String getIdentifierOfFloatingRateOfLeg1();
	String getIndicatorOfTheFloatingRateOfLeg1();
	String getNameOfTheFloatingRateOfLeg1();
	InterestComputationMethod4Code getFloatingRateDayCountConventionOfLeg1();
	Frequency13Code__1 getFloatingRatePaymentFrequencyPeriodOfLeg1();
	Integer getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1();
	String getFloatingRateReferencePeriodOfLeg1TimePeriod();
	Integer getFloatingRateReferencePeriodOfLeg1Multiplier();
	String getFloatingRateResetFrequencyPeriodOfLeg1();
	Integer getFloatingRateResetFrequencyPeriodMultiplierLeg1();
	SpreadOfLeg1 getSpreadOfLeg1();
	BigDecimal getFixedRateOfLeg2();
	InterestComputationMethod4Code getFixedRateDayCountConventionLeg2();
	String getFixedRatePaymentFrequencyPeriodLeg2();
	Integer getFixedRatePaymentFrequencyPeriodMultiplierLeg2();
	String getIdentifierOfFloatingRateOfLeg2();
	String getIndicatorOfTheFloatingRateOfLeg2();
	String getNameOfTheFloatingRateOfLeg2();
	InterestComputationMethod4Code getFloatingRateDayCountConventionOfLeg2();
	String getFloatingRatePaymentFrequencyPeriodOfLeg2();
	Integer getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2();
	String getFloatingRateReferencePeriodOfLeg2TimePeriod();
	Integer getFloatingRateReferencePeriodOfLeg2Multiplier();
	String getFloatingRateResetFrequencyPeriodOfLeg2();
	Integer getFloatingRateResetFrequencyPeriodMultiplierLeg2();
	SpreadOfLeg2 getSpreadOfLeg2();
	PackageTransactionSpread getPackageTransactionSpread();
	BigDecimal getExchangeRate();
	BigDecimal getForwardExchangeRate();
	String getExchangeRateBasis();
	String getBaseProduct();
	String getSubProduct();
	String getFurtherSubProduct();
	List<String> getDeliveryPoint();
	String getInterconnectionPoint();
	EnergyLoadType1Code getLoadType();
	List<? extends DeliveryReport> getDeliveryReport();
	String getOptionType();
	String getOptionStyle();
	StrikePriceAndNotation getStrikePrice();
	List<? extends StrikePriceScheduleReport> getStrikePriceSchedule();
	BigDecimal getOptionPremiumAmount();
	ISOCurrencyCodeEnum getOptionPremiumCurrency();
	Date getOptionPremiumPaymentDate();
	Date getMaturityDateOfTheUnderlying();
	String getSeniority();
	String getReferenceEntity();
	Integer getSeries();
	Integer getVersion();
	BigDecimal getIndexFactor();
	Boolean getTranche();
	BigDecimal getCdSIndexAttachmentPoint();
	BigDecimal getCdSIndexDetachmentPoint();
	Boolean getCollateralPortfolioIndicator();
	String getCollateralPortfolioCode();
	String getActionType();
	String getEventType();
	Date getEventDate();
	String getLevel();
	ESMAEMIRNonReportable getNonReportable();

	/*********************** Build Methods  ***********************/
	ESMAEMIRTransactionReport build();
	
	ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder toBuilder();
	
	static ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder builder() {
		return new ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends ESMAEMIRTransactionReport> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends ESMAEMIRTransactionReport> getType() {
		return ESMAEMIRTransactionReport.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("reportingTimestamp"), ZonedDateTime.class, getReportingTimestamp(), this);
		processor.processBasic(path.newSubPath("reportSubmittingEntityID"), String.class, getReportSubmittingEntityID(), this);
		processor.processBasic(path.newSubPath("entityResponsibleForReporting"), String.class, getEntityResponsibleForReporting(), this);
		processor.processBasic(path.newSubPath("counterparty1"), String.class, getCounterparty1(), this);
		processor.processBasic(path.newSubPath("natureOfCounterparty1"), String.class, getNatureOfCounterparty1(), this);
		processor.processBasic(path.newSubPath("corporateSectorOfTheCounterparty1"), String.class, getCorporateSectorOfTheCounterparty1(), this);
		processor.processBasic(path.newSubPath("clearingThresholdOfCounterparty1"), Boolean.class, getClearingThresholdOfCounterparty1(), this);
		processor.processBasic(path.newSubPath("counterparty2"), String.class, getCounterparty2(), this);
		processor.processBasic(path.newSubPath("countryOfTheCounterParty2"), ISOCountryCodeEnum.class, getCountryOfTheCounterParty2(), this);
		processor.processBasic(path.newSubPath("natureOfCounterparty2"), String.class, getNatureOfCounterparty2(), this);
		processor.processBasic(path.newSubPath("corporateSectorOfTheCounterparty2"), String.class, getCorporateSectorOfTheCounterparty2(), this);
		processor.processBasic(path.newSubPath("counterparty2IdentifierType"), Boolean.class, getCounterparty2IdentifierType(), this);
		processor.processBasic(path.newSubPath("clearingMember"), String.class, getClearingMember(), this);
		processor.processBasic(path.newSubPath("clearingThresholdOfCounterparty2"), Boolean.class, getClearingThresholdOfCounterparty2(), this);
		processor.processBasic(path.newSubPath("reportingObligationOfTheCounterparty2"), Boolean.class, getReportingObligationOfTheCounterparty2(), this);
		processor.processBasic(path.newSubPath("brokerID"), String.class, getBrokerID(), this);
		processor.processBasic(path.newSubPath("direction"), String.class, getDirection(), this);
		processor.processBasic(path.newSubPath("directionOfLeg1"), String.class, getDirectionOfLeg1(), this);
		processor.processBasic(path.newSubPath("directionOfLeg2"), String.class, getDirectionOfLeg2(), this);
		processor.processBasic(path.newSubPath("directlyLinkedToCommercialActivityOrTreasuryFinancing"), Boolean.class, getDirectlyLinkedToCommercialActivityOrTreasuryFinancing(), this);
		processor.processBasic(path.newSubPath("uti"), String.class, getUti(), this);
		processor.processBasic(path.newSubPath("utiProprietary"), String.class, getUtiProprietary(), this);
		processor.processBasic(path.newSubPath("reportTrackingNumber"), String.class, getReportTrackingNumber(), this);
		processor.processBasic(path.newSubPath("priorUti"), String.class, getPriorUti(), this);
		processor.processBasic(path.newSubPath("priorUtiProprietary"), String.class, getPriorUtiProprietary(), this);
		processor.processBasic(path.newSubPath("ptrrId"), String.class, getPtrrId(), this);
		processor.processBasic(path.newSubPath("packageIdentifier"), String.class, getPackageIdentifier(), this);
		processor.processBasic(path.newSubPath("subsequentPositionUti"), String.class, getSubsequentPositionUti(), this);
		processor.processBasic(path.newSubPath("isin"), String.class, getIsin(), this);
		processor.processBasic(path.newSubPath("upi"), String.class, getUpi(), this);
		processor.processBasic(path.newSubPath("productClassification"), String.class, getProductClassification(), this);
		processor.processBasic(path.newSubPath("contractType"), String.class, getContractType(), this);
		processor.processBasic(path.newSubPath("assetClass"), String.class, getAssetClass(), this);
		processor.processBasic(path.newSubPath("isCrypto"), Boolean.class, getIsCrypto(), this);
		processor.processBasic(path.newSubPath("underlyingIdentificationType"), String.class, getUnderlyingIdentificationType(), this);
		processor.processBasic(path.newSubPath("underlyingIdentification"), String.class, getUnderlyingIdentification(), this);
		processor.processBasic(path.newSubPath("nameOfTheUnderlyingIndex"), String.class, getNameOfTheUnderlyingIndex(), this);
		processor.processBasic(path.newSubPath("indicatorOfTheUnderlyingIndex"), String.class, getIndicatorOfTheUnderlyingIndex(), this);
		processRosetta(path.newSubPath("basketConstituents"), processor, BasketConstituentsReport.class, getBasketConstituents());
		processor.processBasic(path.newSubPath("customBasketCode"), String.class, getCustomBasketCode(), this);
		processor.processBasic(path.newSubPath("settlementCurrency1"), ISOCurrencyCodeEnum.class, getSettlementCurrency1(), this);
		processor.processBasic(path.newSubPath("settlementCurrency2"), ISOCurrencyCodeEnum.class, getSettlementCurrency2(), this);
		processor.processBasic(path.newSubPath("valuationAmount"), BigDecimal.class, getValuationAmount(), this);
		processor.processBasic(path.newSubPath("valuationCurrency"), ISOCurrencyCodeEnum.class, getValuationCurrency(), this);
		processor.processBasic(path.newSubPath("valuationMethod"), ValuationType1Code.class, getValuationMethod(), this);
		processor.processBasic(path.newSubPath("delta"), BigDecimal.class, getDelta(), this);
		processor.processBasic(path.newSubPath("valuationTimestamp"), ZonedDateTime.class, getValuationTimestamp(), this);
		processor.processBasic(path.newSubPath("confirmationTimestamp"), ZonedDateTime.class, getConfirmationTimestamp(), this);
		processor.processBasic(path.newSubPath("confirmed"), String.class, getConfirmed(), this);
		processor.processBasic(path.newSubPath("clearingObligation"), String.class, getClearingObligation(), this);
		processor.processBasic(path.newSubPath("cleared"), String.class, getCleared(), this);
		processor.processBasic(path.newSubPath("clearingTimestamp"), ZonedDateTime.class, getClearingTimestamp(), this);
		processor.processBasic(path.newSubPath("centralCounterparty"), String.class, getCentralCounterparty(), this);
		processor.processBasic(path.newSubPath("masterAgreementType"), String.class, getMasterAgreementType(), this);
		processor.processBasic(path.newSubPath("otherMasterAgreementType"), String.class, getOtherMasterAgreementType(), this);
		processor.processBasic(path.newSubPath("masterAgreementVersion"), Integer.class, getMasterAgreementVersion(), this);
		processor.processBasic(path.newSubPath("intragroup"), Boolean.class, getIntragroup(), this);
		processor.processBasic(path.newSubPath("ptrr"), Boolean.class, getPtrr(), this);
		processor.processBasic(path.newSubPath("typeOfPTRRTechnique"), String.class, getTypeOfPTRRTechnique(), this);
		processor.processBasic(path.newSubPath("ptrrServiceProvider"), String.class, getPtrrServiceProvider(), this);
		processor.processBasic(path.newSubPath("venueOfExecution"), String.class, getVenueOfExecution(), this);
		processor.processBasic(path.newSubPath("executionTimestamp"), ZonedDateTime.class, getExecutionTimestamp(), this);
		processor.processBasic(path.newSubPath("effectiveDate"), Date.class, getEffectiveDate(), this);
		processor.processBasic(path.newSubPath("expirationDate"), Date.class, getExpirationDate(), this);
		processor.processBasic(path.newSubPath("earlyTerminationDate"), Date.class, getEarlyTerminationDate(), this);
		processor.processBasic(path.newSubPath("finalContractualSettlementDate"), Date.class, getFinalContractualSettlementDate(), this);
		processor.processBasic(path.newSubPath("deliveryType"), String.class, getDeliveryType(), this);
		processRosetta(path.newSubPath("price"), processor, PriceAndNotation.class, getPrice());
		processRosetta(path.newSubPath("priceSchedule"), processor, PriceScheduleReport.class, getPriceSchedule());
		processRosetta(path.newSubPath("packageTransactionPrice"), processor, PackageTransactionPrice.class, getPackageTransactionPrice());
		processor.processBasic(path.newSubPath("notionalLeg1"), BigDecimal.class, getNotionalLeg1(), this);
		processor.processBasic(path.newSubPath("notionalCurrency1"), ISOCurrencyCodeEnum.class, getNotionalCurrency1(), this);
		processRosetta(path.newSubPath("notionalAmountScheduleLeg1"), processor, NotionalAmountScheduleLeg1Report.class, getNotionalAmountScheduleLeg1());
		processor.processBasic(path.newSubPath("totalNotionalQuantityLeg1"), BigDecimal.class, getTotalNotionalQuantityLeg1(), this);
		processRosetta(path.newSubPath("notionalQuantityScheduleLeg1"), processor, NotionalQuantityScheduleLeg1Report.class, getNotionalQuantityScheduleLeg1());
		processor.processBasic(path.newSubPath("totalNotionalQuantityLeg2"), BigDecimal.class, getTotalNotionalQuantityLeg2(), this);
		processor.processBasic(path.newSubPath("notionalLeg2"), BigDecimal.class, getNotionalLeg2(), this);
		processor.processBasic(path.newSubPath("notionalCurrency2"), ISOCurrencyCodeEnum.class, getNotionalCurrency2(), this);
		processRosetta(path.newSubPath("notionalAmountScheduleLeg2"), processor, NotionalAmountScheduleLeg2Report.class, getNotionalAmountScheduleLeg2());
		processRosetta(path.newSubPath("notionalQuantityScheduleLeg2"), processor, NotionalQuantityScheduleLeg2Report.class, getNotionalQuantityScheduleLeg2());
		processRosetta(path.newSubPath("otherPayment"), processor, OtherPaymentReport.class, getOtherPayment());
		processor.processBasic(path.newSubPath("fixedRateOfLeg1"), BigDecimal.class, getFixedRateOfLeg1(), this);
		processor.processBasic(path.newSubPath("fixedRateDayCountConventionLeg1"), InterestComputationMethod4Code.class, getFixedRateDayCountConventionLeg1(), this);
		processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodLeg1"), String.class, getFixedRatePaymentFrequencyPeriodLeg1(), this);
		processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodMultiplierLeg1"), Integer.class, getFixedRatePaymentFrequencyPeriodMultiplierLeg1(), this);
		processor.processBasic(path.newSubPath("identifierOfFloatingRateOfLeg1"), String.class, getIdentifierOfFloatingRateOfLeg1(), this);
		processor.processBasic(path.newSubPath("indicatorOfTheFloatingRateOfLeg1"), String.class, getIndicatorOfTheFloatingRateOfLeg1(), this);
		processor.processBasic(path.newSubPath("nameOfTheFloatingRateOfLeg1"), String.class, getNameOfTheFloatingRateOfLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRateDayCountConventionOfLeg1"), InterestComputationMethod4Code.class, getFloatingRateDayCountConventionOfLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodOfLeg1"), Frequency13Code__1.class, getFloatingRatePaymentFrequencyPeriodOfLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodMultiplierOfLeg1"), Integer.class, getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRateReferencePeriodOfLeg1TimePeriod"), String.class, getFloatingRateReferencePeriodOfLeg1TimePeriod(), this);
		processor.processBasic(path.newSubPath("floatingRateReferencePeriodOfLeg1Multiplier"), Integer.class, getFloatingRateReferencePeriodOfLeg1Multiplier(), this);
		processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodOfLeg1"), String.class, getFloatingRateResetFrequencyPeriodOfLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodMultiplierLeg1"), Integer.class, getFloatingRateResetFrequencyPeriodMultiplierLeg1(), this);
		processRosetta(path.newSubPath("spreadOfLeg1"), processor, SpreadOfLeg1.class, getSpreadOfLeg1());
		processor.processBasic(path.newSubPath("fixedRateOfLeg2"), BigDecimal.class, getFixedRateOfLeg2(), this);
		processor.processBasic(path.newSubPath("fixedRateDayCountConventionLeg2"), InterestComputationMethod4Code.class, getFixedRateDayCountConventionLeg2(), this);
		processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodLeg2"), String.class, getFixedRatePaymentFrequencyPeriodLeg2(), this);
		processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodMultiplierLeg2"), Integer.class, getFixedRatePaymentFrequencyPeriodMultiplierLeg2(), this);
		processor.processBasic(path.newSubPath("identifierOfFloatingRateOfLeg2"), String.class, getIdentifierOfFloatingRateOfLeg2(), this);
		processor.processBasic(path.newSubPath("indicatorOfTheFloatingRateOfLeg2"), String.class, getIndicatorOfTheFloatingRateOfLeg2(), this);
		processor.processBasic(path.newSubPath("nameOfTheFloatingRateOfLeg2"), String.class, getNameOfTheFloatingRateOfLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRateDayCountConventionOfLeg2"), InterestComputationMethod4Code.class, getFloatingRateDayCountConventionOfLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodOfLeg2"), String.class, getFloatingRatePaymentFrequencyPeriodOfLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodMultiplierOfLeg2"), Integer.class, getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRateReferencePeriodOfLeg2TimePeriod"), String.class, getFloatingRateReferencePeriodOfLeg2TimePeriod(), this);
		processor.processBasic(path.newSubPath("floatingRateReferencePeriodOfLeg2Multiplier"), Integer.class, getFloatingRateReferencePeriodOfLeg2Multiplier(), this);
		processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodOfLeg2"), String.class, getFloatingRateResetFrequencyPeriodOfLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodMultiplierLeg2"), Integer.class, getFloatingRateResetFrequencyPeriodMultiplierLeg2(), this);
		processRosetta(path.newSubPath("spreadOfLeg2"), processor, SpreadOfLeg2.class, getSpreadOfLeg2());
		processRosetta(path.newSubPath("packageTransactionSpread"), processor, PackageTransactionSpread.class, getPackageTransactionSpread());
		processor.processBasic(path.newSubPath("exchangeRate"), BigDecimal.class, getExchangeRate(), this);
		processor.processBasic(path.newSubPath("forwardExchangeRate"), BigDecimal.class, getForwardExchangeRate(), this);
		processor.processBasic(path.newSubPath("exchangeRateBasis"), String.class, getExchangeRateBasis(), this);
		processor.processBasic(path.newSubPath("baseProduct"), String.class, getBaseProduct(), this);
		processor.processBasic(path.newSubPath("subProduct"), String.class, getSubProduct(), this);
		processor.processBasic(path.newSubPath("furtherSubProduct"), String.class, getFurtherSubProduct(), this);
		processor.processBasic(path.newSubPath("deliveryPoint"), String.class, getDeliveryPoint(), this);
		processor.processBasic(path.newSubPath("interconnectionPoint"), String.class, getInterconnectionPoint(), this);
		processor.processBasic(path.newSubPath("loadType"), EnergyLoadType1Code.class, getLoadType(), this);
		processRosetta(path.newSubPath("deliveryReport"), processor, DeliveryReport.class, getDeliveryReport());
		processor.processBasic(path.newSubPath("optionType"), String.class, getOptionType(), this);
		processor.processBasic(path.newSubPath("optionStyle"), String.class, getOptionStyle(), this);
		processRosetta(path.newSubPath("strikePrice"), processor, StrikePriceAndNotation.class, getStrikePrice());
		processRosetta(path.newSubPath("strikePriceSchedule"), processor, StrikePriceScheduleReport.class, getStrikePriceSchedule());
		processor.processBasic(path.newSubPath("optionPremiumAmount"), BigDecimal.class, getOptionPremiumAmount(), this);
		processor.processBasic(path.newSubPath("optionPremiumCurrency"), ISOCurrencyCodeEnum.class, getOptionPremiumCurrency(), this);
		processor.processBasic(path.newSubPath("optionPremiumPaymentDate"), Date.class, getOptionPremiumPaymentDate(), this);
		processor.processBasic(path.newSubPath("maturityDateOfTheUnderlying"), Date.class, getMaturityDateOfTheUnderlying(), this);
		processor.processBasic(path.newSubPath("seniority"), String.class, getSeniority(), this);
		processor.processBasic(path.newSubPath("referenceEntity"), String.class, getReferenceEntity(), this);
		processor.processBasic(path.newSubPath("series"), Integer.class, getSeries(), this);
		processor.processBasic(path.newSubPath("version"), Integer.class, getVersion(), this);
		processor.processBasic(path.newSubPath("indexFactor"), BigDecimal.class, getIndexFactor(), this);
		processor.processBasic(path.newSubPath("tranche"), Boolean.class, getTranche(), this);
		processor.processBasic(path.newSubPath("cdSIndexAttachmentPoint"), BigDecimal.class, getCdSIndexAttachmentPoint(), this);
		processor.processBasic(path.newSubPath("cdSIndexDetachmentPoint"), BigDecimal.class, getCdSIndexDetachmentPoint(), this);
		processor.processBasic(path.newSubPath("collateralPortfolioIndicator"), Boolean.class, getCollateralPortfolioIndicator(), this);
		processor.processBasic(path.newSubPath("collateralPortfolioCode"), String.class, getCollateralPortfolioCode(), this);
		processor.processBasic(path.newSubPath("actionType"), String.class, getActionType(), this);
		processor.processBasic(path.newSubPath("eventType"), String.class, getEventType(), this);
		processor.processBasic(path.newSubPath("eventDate"), Date.class, getEventDate(), this);
		processor.processBasic(path.newSubPath("level"), String.class, getLevel(), this);
		processRosetta(path.newSubPath("nonReportable"), processor, ESMAEMIRNonReportable.class, getNonReportable());
	}
	

	/*********************** Builder Interface  ***********************/
	interface ESMAEMIRTransactionReportBuilder extends ESMAEMIRTransactionReport, RosettaModelObjectBuilder {
		BasketConstituentsReport.BasketConstituentsReportBuilder getOrCreateBasketConstituents(int _index);
		List<? extends BasketConstituentsReport.BasketConstituentsReportBuilder> getBasketConstituents();
		PriceAndNotation.PriceAndNotationBuilder getOrCreatePrice();
		PriceAndNotation.PriceAndNotationBuilder getPrice();
		PriceScheduleReport.PriceScheduleReportBuilder getOrCreatePriceSchedule(int _index);
		List<? extends PriceScheduleReport.PriceScheduleReportBuilder> getPriceSchedule();
		PackageTransactionPrice.PackageTransactionPriceBuilder getOrCreatePackageTransactionPrice();
		PackageTransactionPrice.PackageTransactionPriceBuilder getPackageTransactionPrice();
		NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder getOrCreateNotionalAmountScheduleLeg1(int _index);
		List<? extends NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> getNotionalAmountScheduleLeg1();
		NotionalQuantityScheduleLeg1Report.NotionalQuantityScheduleLeg1ReportBuilder getOrCreateNotionalQuantityScheduleLeg1(int _index);
		List<? extends NotionalQuantityScheduleLeg1Report.NotionalQuantityScheduleLeg1ReportBuilder> getNotionalQuantityScheduleLeg1();
		NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder getOrCreateNotionalAmountScheduleLeg2(int _index);
		List<? extends NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> getNotionalAmountScheduleLeg2();
		NotionalQuantityScheduleLeg2Report.NotionalQuantityScheduleLeg2ReportBuilder getOrCreateNotionalQuantityScheduleLeg2(int _index);
		List<? extends NotionalQuantityScheduleLeg2Report.NotionalQuantityScheduleLeg2ReportBuilder> getNotionalQuantityScheduleLeg2();
		OtherPaymentReport.OtherPaymentReportBuilder getOrCreateOtherPayment(int _index);
		List<? extends OtherPaymentReport.OtherPaymentReportBuilder> getOtherPayment();
		SpreadOfLeg1.SpreadOfLeg1Builder getOrCreateSpreadOfLeg1();
		SpreadOfLeg1.SpreadOfLeg1Builder getSpreadOfLeg1();
		SpreadOfLeg2.SpreadOfLeg2Builder getOrCreateSpreadOfLeg2();
		SpreadOfLeg2.SpreadOfLeg2Builder getSpreadOfLeg2();
		PackageTransactionSpread.PackageTransactionSpreadBuilder getOrCreatePackageTransactionSpread();
		PackageTransactionSpread.PackageTransactionSpreadBuilder getPackageTransactionSpread();
		DeliveryReport.DeliveryReportBuilder getOrCreateDeliveryReport(int _index);
		List<? extends DeliveryReport.DeliveryReportBuilder> getDeliveryReport();
		StrikePriceAndNotation.StrikePriceAndNotationBuilder getOrCreateStrikePrice();
		StrikePriceAndNotation.StrikePriceAndNotationBuilder getStrikePrice();
		StrikePriceScheduleReport.StrikePriceScheduleReportBuilder getOrCreateStrikePriceSchedule(int _index);
		List<? extends StrikePriceScheduleReport.StrikePriceScheduleReportBuilder> getStrikePriceSchedule();
		ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder getOrCreateNonReportable();
		ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder getNonReportable();
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setReportingTimestamp(ZonedDateTime reportingTimestamp);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setReportSubmittingEntityID(String reportSubmittingEntityID);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setEntityResponsibleForReporting(String entityResponsibleForReporting);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCounterparty1(String counterparty1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNatureOfCounterparty1(String natureOfCounterparty1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCorporateSectorOfTheCounterparty1(String corporateSectorOfTheCounterparty1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setClearingThresholdOfCounterparty1(Boolean clearingThresholdOfCounterparty1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCounterparty2(String counterparty2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCountryOfTheCounterParty2(ISOCountryCodeEnum countryOfTheCounterParty2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNatureOfCounterparty2(String natureOfCounterparty2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCorporateSectorOfTheCounterparty2(String corporateSectorOfTheCounterparty2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCounterparty2IdentifierType(Boolean counterparty2IdentifierType);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setClearingMember(String clearingMember);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setClearingThresholdOfCounterparty2(Boolean clearingThresholdOfCounterparty2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setReportingObligationOfTheCounterparty2(Boolean reportingObligationOfTheCounterparty2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setBrokerID(String brokerID);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDirection(String direction);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDirectionOfLeg1(String directionOfLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDirectionOfLeg2(String directionOfLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDirectlyLinkedToCommercialActivityOrTreasuryFinancing(Boolean directlyLinkedToCommercialActivityOrTreasuryFinancing);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setUti(String uti);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setUtiProprietary(String utiProprietary);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setReportTrackingNumber(String reportTrackingNumber);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPriorUti(String priorUti);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPriorUtiProprietary(String priorUtiProprietary);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPtrrId(String ptrrId);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPackageIdentifier(String packageIdentifier);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSubsequentPositionUti(String subsequentPositionUti);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIsin(String isin);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setUpi(String upi);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setProductClassification(String productClassification);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setContractType(String contractType);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setAssetClass(String assetClass);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIsCrypto(Boolean isCrypto);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setUnderlyingIdentificationType(String underlyingIdentificationType);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setUnderlyingIdentification(String underlyingIdentification);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNameOfTheUnderlyingIndex(String nameOfTheUnderlyingIndex0);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNameOfTheUnderlyingIndex(String nameOfTheUnderlyingIndex1, int _idx);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNameOfTheUnderlyingIndex(List<? extends String> nameOfTheUnderlyingIndex2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNameOfTheUnderlyingIndex(List<? extends String> nameOfTheUnderlyingIndex3);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addIndicatorOfTheUnderlyingIndex(String indicatorOfTheUnderlyingIndex0);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addIndicatorOfTheUnderlyingIndex(String indicatorOfTheUnderlyingIndex1, int _idx);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addIndicatorOfTheUnderlyingIndex(List<? extends String> indicatorOfTheUnderlyingIndex2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIndicatorOfTheUnderlyingIndex(List<? extends String> indicatorOfTheUnderlyingIndex3);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addBasketConstituents(BasketConstituentsReport basketConstituents0);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addBasketConstituents(BasketConstituentsReport basketConstituents1, int _idx);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addBasketConstituents(List<? extends BasketConstituentsReport> basketConstituents2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setBasketConstituents(List<? extends BasketConstituentsReport> basketConstituents3);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCustomBasketCode(String customBasketCode);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSettlementCurrency1(ISOCurrencyCodeEnum settlementCurrency1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSettlementCurrency2(ISOCurrencyCodeEnum settlementCurrency2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setValuationAmount(BigDecimal valuationAmount);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setValuationCurrency(ISOCurrencyCodeEnum valuationCurrency);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setValuationMethod(ValuationType1Code valuationMethod);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDelta(BigDecimal delta);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setValuationTimestamp(ZonedDateTime valuationTimestamp);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setConfirmationTimestamp(ZonedDateTime confirmationTimestamp);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setConfirmed(String confirmed);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setClearingObligation(String clearingObligation);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCleared(String cleared);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setClearingTimestamp(ZonedDateTime clearingTimestamp);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCentralCounterparty(String centralCounterparty);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setMasterAgreementType(String masterAgreementType);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setOtherMasterAgreementType(String otherMasterAgreementType);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setMasterAgreementVersion(Integer masterAgreementVersion);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIntragroup(Boolean intragroup);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPtrr(Boolean ptrr);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setTypeOfPTRRTechnique(String typeOfPTRRTechnique);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPtrrServiceProvider(String ptrrServiceProvider);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setVenueOfExecution(String venueOfExecution);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setExecutionTimestamp(ZonedDateTime executionTimestamp);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setEffectiveDate(Date effectiveDate);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setExpirationDate(Date expirationDate);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setEarlyTerminationDate(Date earlyTerminationDate);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFinalContractualSettlementDate(Date finalContractualSettlementDate);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDeliveryType(String deliveryType);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPrice(PriceAndNotation price);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addPriceSchedule(PriceScheduleReport priceSchedule0);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addPriceSchedule(PriceScheduleReport priceSchedule1, int _idx);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addPriceSchedule(List<? extends PriceScheduleReport> priceSchedule2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPriceSchedule(List<? extends PriceScheduleReport> priceSchedule3);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPackageTransactionPrice(PackageTransactionPrice packageTransactionPrice);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalLeg1(BigDecimal notionalLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalCurrency1(ISOCurrencyCodeEnum notionalCurrency1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalAmountScheduleLeg1(NotionalAmountScheduleLeg1Report notionalAmountScheduleLeg10);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalAmountScheduleLeg1(NotionalAmountScheduleLeg1Report notionalAmountScheduleLeg11, int _idx);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalAmountScheduleLeg1(List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg12);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalAmountScheduleLeg1(List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg13);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setTotalNotionalQuantityLeg1(BigDecimal totalNotionalQuantityLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalQuantityScheduleLeg1(NotionalQuantityScheduleLeg1Report notionalQuantityScheduleLeg10);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalQuantityScheduleLeg1(NotionalQuantityScheduleLeg1Report notionalQuantityScheduleLeg11, int _idx);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalQuantityScheduleLeg1(List<? extends NotionalQuantityScheduleLeg1Report> notionalQuantityScheduleLeg12);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalQuantityScheduleLeg1(List<? extends NotionalQuantityScheduleLeg1Report> notionalQuantityScheduleLeg13);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setTotalNotionalQuantityLeg2(BigDecimal totalNotionalQuantityLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalLeg2(BigDecimal notionalLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalCurrency2(ISOCurrencyCodeEnum notionalCurrency2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalAmountScheduleLeg2(NotionalAmountScheduleLeg2Report notionalAmountScheduleLeg20);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalAmountScheduleLeg2(NotionalAmountScheduleLeg2Report notionalAmountScheduleLeg21, int _idx);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalAmountScheduleLeg2(List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg22);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalAmountScheduleLeg2(List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg23);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalQuantityScheduleLeg2(NotionalQuantityScheduleLeg2Report notionalQuantityScheduleLeg20);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalQuantityScheduleLeg2(NotionalQuantityScheduleLeg2Report notionalQuantityScheduleLeg21, int _idx);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalQuantityScheduleLeg2(List<? extends NotionalQuantityScheduleLeg2Report> notionalQuantityScheduleLeg22);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalQuantityScheduleLeg2(List<? extends NotionalQuantityScheduleLeg2Report> notionalQuantityScheduleLeg23);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment0);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment1, int _idx);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addOtherPayment(List<? extends OtherPaymentReport> otherPayment2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setOtherPayment(List<? extends OtherPaymentReport> otherPayment3);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRateOfLeg1(BigDecimal fixedRateOfLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRateDayCountConventionLeg1(InterestComputationMethod4Code fixedRateDayCountConventionLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRatePaymentFrequencyPeriodLeg1(String fixedRatePaymentFrequencyPeriodLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRatePaymentFrequencyPeriodMultiplierLeg1(Integer fixedRatePaymentFrequencyPeriodMultiplierLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIdentifierOfFloatingRateOfLeg1(String identifierOfFloatingRateOfLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIndicatorOfTheFloatingRateOfLeg1(String indicatorOfTheFloatingRateOfLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNameOfTheFloatingRateOfLeg1(String nameOfTheFloatingRateOfLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateDayCountConventionOfLeg1(InterestComputationMethod4Code floatingRateDayCountConventionOfLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodOfLeg1(Frequency13Code__1 floatingRatePaymentFrequencyPeriodOfLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1(Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateReferencePeriodOfLeg1TimePeriod(String floatingRateReferencePeriodOfLeg1TimePeriod);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateReferencePeriodOfLeg1Multiplier(Integer floatingRateReferencePeriodOfLeg1Multiplier);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateResetFrequencyPeriodOfLeg1(String floatingRateResetFrequencyPeriodOfLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateResetFrequencyPeriodMultiplierLeg1(Integer floatingRateResetFrequencyPeriodMultiplierLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSpreadOfLeg1(SpreadOfLeg1 spreadOfLeg1);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRateOfLeg2(BigDecimal fixedRateOfLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRateDayCountConventionLeg2(InterestComputationMethod4Code fixedRateDayCountConventionLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRatePaymentFrequencyPeriodLeg2(String fixedRatePaymentFrequencyPeriodLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRatePaymentFrequencyPeriodMultiplierLeg2(Integer fixedRatePaymentFrequencyPeriodMultiplierLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIdentifierOfFloatingRateOfLeg2(String identifierOfFloatingRateOfLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIndicatorOfTheFloatingRateOfLeg2(String indicatorOfTheFloatingRateOfLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNameOfTheFloatingRateOfLeg2(String nameOfTheFloatingRateOfLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateDayCountConventionOfLeg2(InterestComputationMethod4Code floatingRateDayCountConventionOfLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodOfLeg2(String floatingRatePaymentFrequencyPeriodOfLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2(Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateReferencePeriodOfLeg2TimePeriod(String floatingRateReferencePeriodOfLeg2TimePeriod);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateReferencePeriodOfLeg2Multiplier(Integer floatingRateReferencePeriodOfLeg2Multiplier);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateResetFrequencyPeriodOfLeg2(String floatingRateResetFrequencyPeriodOfLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateResetFrequencyPeriodMultiplierLeg2(Integer floatingRateResetFrequencyPeriodMultiplierLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSpreadOfLeg2(SpreadOfLeg2 spreadOfLeg2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPackageTransactionSpread(PackageTransactionSpread packageTransactionSpread);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setExchangeRate(BigDecimal exchangeRate);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setForwardExchangeRate(BigDecimal forwardExchangeRate);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setExchangeRateBasis(String exchangeRateBasis);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setBaseProduct(String baseProduct);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSubProduct(String subProduct);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFurtherSubProduct(String furtherSubProduct);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addDeliveryPoint(String deliveryPoint0);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addDeliveryPoint(String deliveryPoint1, int _idx);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addDeliveryPoint(List<? extends String> deliveryPoint2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDeliveryPoint(List<? extends String> deliveryPoint3);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setInterconnectionPoint(String interconnectionPoint);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setLoadType(EnergyLoadType1Code loadType);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addDeliveryReport(DeliveryReport deliveryReport0);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addDeliveryReport(DeliveryReport deliveryReport1, int _idx);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addDeliveryReport(List<? extends DeliveryReport> deliveryReport2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDeliveryReport(List<? extends DeliveryReport> deliveryReport3);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setOptionType(String optionType);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setOptionStyle(String optionStyle);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setStrikePrice(StrikePriceAndNotation strikePrice);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addStrikePriceSchedule(StrikePriceScheduleReport strikePriceSchedule0);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addStrikePriceSchedule(StrikePriceScheduleReport strikePriceSchedule1, int _idx);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addStrikePriceSchedule(List<? extends StrikePriceScheduleReport> strikePriceSchedule2);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setStrikePriceSchedule(List<? extends StrikePriceScheduleReport> strikePriceSchedule3);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setOptionPremiumAmount(BigDecimal optionPremiumAmount);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setOptionPremiumCurrency(ISOCurrencyCodeEnum optionPremiumCurrency);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setOptionPremiumPaymentDate(Date optionPremiumPaymentDate);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setMaturityDateOfTheUnderlying(Date maturityDateOfTheUnderlying);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSeniority(String seniority);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setReferenceEntity(String referenceEntity);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSeries(Integer series);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setVersion(Integer version);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIndexFactor(BigDecimal indexFactor);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setTranche(Boolean tranche);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCdSIndexAttachmentPoint(BigDecimal cdSIndexAttachmentPoint);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCdSIndexDetachmentPoint(BigDecimal cdSIndexDetachmentPoint);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCollateralPortfolioIndicator(Boolean collateralPortfolioIndicator);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCollateralPortfolioCode(String collateralPortfolioCode);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setActionType(String actionType);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setEventType(String eventType);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setEventDate(Date eventDate);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setLevel(String level);
		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNonReportable(ESMAEMIRNonReportable nonReportable);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("reportingTimestamp"), ZonedDateTime.class, getReportingTimestamp(), this);
			processor.processBasic(path.newSubPath("reportSubmittingEntityID"), String.class, getReportSubmittingEntityID(), this);
			processor.processBasic(path.newSubPath("entityResponsibleForReporting"), String.class, getEntityResponsibleForReporting(), this);
			processor.processBasic(path.newSubPath("counterparty1"), String.class, getCounterparty1(), this);
			processor.processBasic(path.newSubPath("natureOfCounterparty1"), String.class, getNatureOfCounterparty1(), this);
			processor.processBasic(path.newSubPath("corporateSectorOfTheCounterparty1"), String.class, getCorporateSectorOfTheCounterparty1(), this);
			processor.processBasic(path.newSubPath("clearingThresholdOfCounterparty1"), Boolean.class, getClearingThresholdOfCounterparty1(), this);
			processor.processBasic(path.newSubPath("counterparty2"), String.class, getCounterparty2(), this);
			processor.processBasic(path.newSubPath("countryOfTheCounterParty2"), ISOCountryCodeEnum.class, getCountryOfTheCounterParty2(), this);
			processor.processBasic(path.newSubPath("natureOfCounterparty2"), String.class, getNatureOfCounterparty2(), this);
			processor.processBasic(path.newSubPath("corporateSectorOfTheCounterparty2"), String.class, getCorporateSectorOfTheCounterparty2(), this);
			processor.processBasic(path.newSubPath("counterparty2IdentifierType"), Boolean.class, getCounterparty2IdentifierType(), this);
			processor.processBasic(path.newSubPath("clearingMember"), String.class, getClearingMember(), this);
			processor.processBasic(path.newSubPath("clearingThresholdOfCounterparty2"), Boolean.class, getClearingThresholdOfCounterparty2(), this);
			processor.processBasic(path.newSubPath("reportingObligationOfTheCounterparty2"), Boolean.class, getReportingObligationOfTheCounterparty2(), this);
			processor.processBasic(path.newSubPath("brokerID"), String.class, getBrokerID(), this);
			processor.processBasic(path.newSubPath("direction"), String.class, getDirection(), this);
			processor.processBasic(path.newSubPath("directionOfLeg1"), String.class, getDirectionOfLeg1(), this);
			processor.processBasic(path.newSubPath("directionOfLeg2"), String.class, getDirectionOfLeg2(), this);
			processor.processBasic(path.newSubPath("directlyLinkedToCommercialActivityOrTreasuryFinancing"), Boolean.class, getDirectlyLinkedToCommercialActivityOrTreasuryFinancing(), this);
			processor.processBasic(path.newSubPath("uti"), String.class, getUti(), this);
			processor.processBasic(path.newSubPath("utiProprietary"), String.class, getUtiProprietary(), this);
			processor.processBasic(path.newSubPath("reportTrackingNumber"), String.class, getReportTrackingNumber(), this);
			processor.processBasic(path.newSubPath("priorUti"), String.class, getPriorUti(), this);
			processor.processBasic(path.newSubPath("priorUtiProprietary"), String.class, getPriorUtiProprietary(), this);
			processor.processBasic(path.newSubPath("ptrrId"), String.class, getPtrrId(), this);
			processor.processBasic(path.newSubPath("packageIdentifier"), String.class, getPackageIdentifier(), this);
			processor.processBasic(path.newSubPath("subsequentPositionUti"), String.class, getSubsequentPositionUti(), this);
			processor.processBasic(path.newSubPath("isin"), String.class, getIsin(), this);
			processor.processBasic(path.newSubPath("upi"), String.class, getUpi(), this);
			processor.processBasic(path.newSubPath("productClassification"), String.class, getProductClassification(), this);
			processor.processBasic(path.newSubPath("contractType"), String.class, getContractType(), this);
			processor.processBasic(path.newSubPath("assetClass"), String.class, getAssetClass(), this);
			processor.processBasic(path.newSubPath("isCrypto"), Boolean.class, getIsCrypto(), this);
			processor.processBasic(path.newSubPath("underlyingIdentificationType"), String.class, getUnderlyingIdentificationType(), this);
			processor.processBasic(path.newSubPath("underlyingIdentification"), String.class, getUnderlyingIdentification(), this);
			processor.processBasic(path.newSubPath("nameOfTheUnderlyingIndex"), String.class, getNameOfTheUnderlyingIndex(), this);
			processor.processBasic(path.newSubPath("indicatorOfTheUnderlyingIndex"), String.class, getIndicatorOfTheUnderlyingIndex(), this);
			processRosetta(path.newSubPath("basketConstituents"), processor, BasketConstituentsReport.BasketConstituentsReportBuilder.class, getBasketConstituents());
			processor.processBasic(path.newSubPath("customBasketCode"), String.class, getCustomBasketCode(), this);
			processor.processBasic(path.newSubPath("settlementCurrency1"), ISOCurrencyCodeEnum.class, getSettlementCurrency1(), this);
			processor.processBasic(path.newSubPath("settlementCurrency2"), ISOCurrencyCodeEnum.class, getSettlementCurrency2(), this);
			processor.processBasic(path.newSubPath("valuationAmount"), BigDecimal.class, getValuationAmount(), this);
			processor.processBasic(path.newSubPath("valuationCurrency"), ISOCurrencyCodeEnum.class, getValuationCurrency(), this);
			processor.processBasic(path.newSubPath("valuationMethod"), ValuationType1Code.class, getValuationMethod(), this);
			processor.processBasic(path.newSubPath("delta"), BigDecimal.class, getDelta(), this);
			processor.processBasic(path.newSubPath("valuationTimestamp"), ZonedDateTime.class, getValuationTimestamp(), this);
			processor.processBasic(path.newSubPath("confirmationTimestamp"), ZonedDateTime.class, getConfirmationTimestamp(), this);
			processor.processBasic(path.newSubPath("confirmed"), String.class, getConfirmed(), this);
			processor.processBasic(path.newSubPath("clearingObligation"), String.class, getClearingObligation(), this);
			processor.processBasic(path.newSubPath("cleared"), String.class, getCleared(), this);
			processor.processBasic(path.newSubPath("clearingTimestamp"), ZonedDateTime.class, getClearingTimestamp(), this);
			processor.processBasic(path.newSubPath("centralCounterparty"), String.class, getCentralCounterparty(), this);
			processor.processBasic(path.newSubPath("masterAgreementType"), String.class, getMasterAgreementType(), this);
			processor.processBasic(path.newSubPath("otherMasterAgreementType"), String.class, getOtherMasterAgreementType(), this);
			processor.processBasic(path.newSubPath("masterAgreementVersion"), Integer.class, getMasterAgreementVersion(), this);
			processor.processBasic(path.newSubPath("intragroup"), Boolean.class, getIntragroup(), this);
			processor.processBasic(path.newSubPath("ptrr"), Boolean.class, getPtrr(), this);
			processor.processBasic(path.newSubPath("typeOfPTRRTechnique"), String.class, getTypeOfPTRRTechnique(), this);
			processor.processBasic(path.newSubPath("ptrrServiceProvider"), String.class, getPtrrServiceProvider(), this);
			processor.processBasic(path.newSubPath("venueOfExecution"), String.class, getVenueOfExecution(), this);
			processor.processBasic(path.newSubPath("executionTimestamp"), ZonedDateTime.class, getExecutionTimestamp(), this);
			processor.processBasic(path.newSubPath("effectiveDate"), Date.class, getEffectiveDate(), this);
			processor.processBasic(path.newSubPath("expirationDate"), Date.class, getExpirationDate(), this);
			processor.processBasic(path.newSubPath("earlyTerminationDate"), Date.class, getEarlyTerminationDate(), this);
			processor.processBasic(path.newSubPath("finalContractualSettlementDate"), Date.class, getFinalContractualSettlementDate(), this);
			processor.processBasic(path.newSubPath("deliveryType"), String.class, getDeliveryType(), this);
			processRosetta(path.newSubPath("price"), processor, PriceAndNotation.PriceAndNotationBuilder.class, getPrice());
			processRosetta(path.newSubPath("priceSchedule"), processor, PriceScheduleReport.PriceScheduleReportBuilder.class, getPriceSchedule());
			processRosetta(path.newSubPath("packageTransactionPrice"), processor, PackageTransactionPrice.PackageTransactionPriceBuilder.class, getPackageTransactionPrice());
			processor.processBasic(path.newSubPath("notionalLeg1"), BigDecimal.class, getNotionalLeg1(), this);
			processor.processBasic(path.newSubPath("notionalCurrency1"), ISOCurrencyCodeEnum.class, getNotionalCurrency1(), this);
			processRosetta(path.newSubPath("notionalAmountScheduleLeg1"), processor, NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder.class, getNotionalAmountScheduleLeg1());
			processor.processBasic(path.newSubPath("totalNotionalQuantityLeg1"), BigDecimal.class, getTotalNotionalQuantityLeg1(), this);
			processRosetta(path.newSubPath("notionalQuantityScheduleLeg1"), processor, NotionalQuantityScheduleLeg1Report.NotionalQuantityScheduleLeg1ReportBuilder.class, getNotionalQuantityScheduleLeg1());
			processor.processBasic(path.newSubPath("totalNotionalQuantityLeg2"), BigDecimal.class, getTotalNotionalQuantityLeg2(), this);
			processor.processBasic(path.newSubPath("notionalLeg2"), BigDecimal.class, getNotionalLeg2(), this);
			processor.processBasic(path.newSubPath("notionalCurrency2"), ISOCurrencyCodeEnum.class, getNotionalCurrency2(), this);
			processRosetta(path.newSubPath("notionalAmountScheduleLeg2"), processor, NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder.class, getNotionalAmountScheduleLeg2());
			processRosetta(path.newSubPath("notionalQuantityScheduleLeg2"), processor, NotionalQuantityScheduleLeg2Report.NotionalQuantityScheduleLeg2ReportBuilder.class, getNotionalQuantityScheduleLeg2());
			processRosetta(path.newSubPath("otherPayment"), processor, OtherPaymentReport.OtherPaymentReportBuilder.class, getOtherPayment());
			processor.processBasic(path.newSubPath("fixedRateOfLeg1"), BigDecimal.class, getFixedRateOfLeg1(), this);
			processor.processBasic(path.newSubPath("fixedRateDayCountConventionLeg1"), InterestComputationMethod4Code.class, getFixedRateDayCountConventionLeg1(), this);
			processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodLeg1"), String.class, getFixedRatePaymentFrequencyPeriodLeg1(), this);
			processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodMultiplierLeg1"), Integer.class, getFixedRatePaymentFrequencyPeriodMultiplierLeg1(), this);
			processor.processBasic(path.newSubPath("identifierOfFloatingRateOfLeg1"), String.class, getIdentifierOfFloatingRateOfLeg1(), this);
			processor.processBasic(path.newSubPath("indicatorOfTheFloatingRateOfLeg1"), String.class, getIndicatorOfTheFloatingRateOfLeg1(), this);
			processor.processBasic(path.newSubPath("nameOfTheFloatingRateOfLeg1"), String.class, getNameOfTheFloatingRateOfLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRateDayCountConventionOfLeg1"), InterestComputationMethod4Code.class, getFloatingRateDayCountConventionOfLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodOfLeg1"), Frequency13Code__1.class, getFloatingRatePaymentFrequencyPeriodOfLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodMultiplierOfLeg1"), Integer.class, getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRateReferencePeriodOfLeg1TimePeriod"), String.class, getFloatingRateReferencePeriodOfLeg1TimePeriod(), this);
			processor.processBasic(path.newSubPath("floatingRateReferencePeriodOfLeg1Multiplier"), Integer.class, getFloatingRateReferencePeriodOfLeg1Multiplier(), this);
			processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodOfLeg1"), String.class, getFloatingRateResetFrequencyPeriodOfLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodMultiplierLeg1"), Integer.class, getFloatingRateResetFrequencyPeriodMultiplierLeg1(), this);
			processRosetta(path.newSubPath("spreadOfLeg1"), processor, SpreadOfLeg1.SpreadOfLeg1Builder.class, getSpreadOfLeg1());
			processor.processBasic(path.newSubPath("fixedRateOfLeg2"), BigDecimal.class, getFixedRateOfLeg2(), this);
			processor.processBasic(path.newSubPath("fixedRateDayCountConventionLeg2"), InterestComputationMethod4Code.class, getFixedRateDayCountConventionLeg2(), this);
			processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodLeg2"), String.class, getFixedRatePaymentFrequencyPeriodLeg2(), this);
			processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodMultiplierLeg2"), Integer.class, getFixedRatePaymentFrequencyPeriodMultiplierLeg2(), this);
			processor.processBasic(path.newSubPath("identifierOfFloatingRateOfLeg2"), String.class, getIdentifierOfFloatingRateOfLeg2(), this);
			processor.processBasic(path.newSubPath("indicatorOfTheFloatingRateOfLeg2"), String.class, getIndicatorOfTheFloatingRateOfLeg2(), this);
			processor.processBasic(path.newSubPath("nameOfTheFloatingRateOfLeg2"), String.class, getNameOfTheFloatingRateOfLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRateDayCountConventionOfLeg2"), InterestComputationMethod4Code.class, getFloatingRateDayCountConventionOfLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodOfLeg2"), String.class, getFloatingRatePaymentFrequencyPeriodOfLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodMultiplierOfLeg2"), Integer.class, getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRateReferencePeriodOfLeg2TimePeriod"), String.class, getFloatingRateReferencePeriodOfLeg2TimePeriod(), this);
			processor.processBasic(path.newSubPath("floatingRateReferencePeriodOfLeg2Multiplier"), Integer.class, getFloatingRateReferencePeriodOfLeg2Multiplier(), this);
			processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodOfLeg2"), String.class, getFloatingRateResetFrequencyPeriodOfLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodMultiplierLeg2"), Integer.class, getFloatingRateResetFrequencyPeriodMultiplierLeg2(), this);
			processRosetta(path.newSubPath("spreadOfLeg2"), processor, SpreadOfLeg2.SpreadOfLeg2Builder.class, getSpreadOfLeg2());
			processRosetta(path.newSubPath("packageTransactionSpread"), processor, PackageTransactionSpread.PackageTransactionSpreadBuilder.class, getPackageTransactionSpread());
			processor.processBasic(path.newSubPath("exchangeRate"), BigDecimal.class, getExchangeRate(), this);
			processor.processBasic(path.newSubPath("forwardExchangeRate"), BigDecimal.class, getForwardExchangeRate(), this);
			processor.processBasic(path.newSubPath("exchangeRateBasis"), String.class, getExchangeRateBasis(), this);
			processor.processBasic(path.newSubPath("baseProduct"), String.class, getBaseProduct(), this);
			processor.processBasic(path.newSubPath("subProduct"), String.class, getSubProduct(), this);
			processor.processBasic(path.newSubPath("furtherSubProduct"), String.class, getFurtherSubProduct(), this);
			processor.processBasic(path.newSubPath("deliveryPoint"), String.class, getDeliveryPoint(), this);
			processor.processBasic(path.newSubPath("interconnectionPoint"), String.class, getInterconnectionPoint(), this);
			processor.processBasic(path.newSubPath("loadType"), EnergyLoadType1Code.class, getLoadType(), this);
			processRosetta(path.newSubPath("deliveryReport"), processor, DeliveryReport.DeliveryReportBuilder.class, getDeliveryReport());
			processor.processBasic(path.newSubPath("optionType"), String.class, getOptionType(), this);
			processor.processBasic(path.newSubPath("optionStyle"), String.class, getOptionStyle(), this);
			processRosetta(path.newSubPath("strikePrice"), processor, StrikePriceAndNotation.StrikePriceAndNotationBuilder.class, getStrikePrice());
			processRosetta(path.newSubPath("strikePriceSchedule"), processor, StrikePriceScheduleReport.StrikePriceScheduleReportBuilder.class, getStrikePriceSchedule());
			processor.processBasic(path.newSubPath("optionPremiumAmount"), BigDecimal.class, getOptionPremiumAmount(), this);
			processor.processBasic(path.newSubPath("optionPremiumCurrency"), ISOCurrencyCodeEnum.class, getOptionPremiumCurrency(), this);
			processor.processBasic(path.newSubPath("optionPremiumPaymentDate"), Date.class, getOptionPremiumPaymentDate(), this);
			processor.processBasic(path.newSubPath("maturityDateOfTheUnderlying"), Date.class, getMaturityDateOfTheUnderlying(), this);
			processor.processBasic(path.newSubPath("seniority"), String.class, getSeniority(), this);
			processor.processBasic(path.newSubPath("referenceEntity"), String.class, getReferenceEntity(), this);
			processor.processBasic(path.newSubPath("series"), Integer.class, getSeries(), this);
			processor.processBasic(path.newSubPath("version"), Integer.class, getVersion(), this);
			processor.processBasic(path.newSubPath("indexFactor"), BigDecimal.class, getIndexFactor(), this);
			processor.processBasic(path.newSubPath("tranche"), Boolean.class, getTranche(), this);
			processor.processBasic(path.newSubPath("cdSIndexAttachmentPoint"), BigDecimal.class, getCdSIndexAttachmentPoint(), this);
			processor.processBasic(path.newSubPath("cdSIndexDetachmentPoint"), BigDecimal.class, getCdSIndexDetachmentPoint(), this);
			processor.processBasic(path.newSubPath("collateralPortfolioIndicator"), Boolean.class, getCollateralPortfolioIndicator(), this);
			processor.processBasic(path.newSubPath("collateralPortfolioCode"), String.class, getCollateralPortfolioCode(), this);
			processor.processBasic(path.newSubPath("actionType"), String.class, getActionType(), this);
			processor.processBasic(path.newSubPath("eventType"), String.class, getEventType(), this);
			processor.processBasic(path.newSubPath("eventDate"), Date.class, getEventDate(), this);
			processor.processBasic(path.newSubPath("level"), String.class, getLevel(), this);
			processRosetta(path.newSubPath("nonReportable"), processor, ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder.class, getNonReportable());
		}
		

		ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder prune();
	}

	/*********************** Immutable Implementation of ESMAEMIRTransactionReport  ***********************/
	class ESMAEMIRTransactionReportImpl implements ESMAEMIRTransactionReport {
		private final ZonedDateTime reportingTimestamp;
		private final String reportSubmittingEntityID;
		private final String entityResponsibleForReporting;
		private final String counterparty1;
		private final String natureOfCounterparty1;
		private final String corporateSectorOfTheCounterparty1;
		private final Boolean clearingThresholdOfCounterparty1;
		private final String counterparty2;
		private final ISOCountryCodeEnum countryOfTheCounterParty2;
		private final String natureOfCounterparty2;
		private final String corporateSectorOfTheCounterparty2;
		private final Boolean counterparty2IdentifierType;
		private final String clearingMember;
		private final Boolean clearingThresholdOfCounterparty2;
		private final Boolean reportingObligationOfTheCounterparty2;
		private final String brokerID;
		private final String direction;
		private final String directionOfLeg1;
		private final String directionOfLeg2;
		private final Boolean directlyLinkedToCommercialActivityOrTreasuryFinancing;
		private final String uti;
		private final String utiProprietary;
		private final String reportTrackingNumber;
		private final String priorUti;
		private final String priorUtiProprietary;
		private final String ptrrId;
		private final String packageIdentifier;
		private final String subsequentPositionUti;
		private final String isin;
		private final String upi;
		private final String productClassification;
		private final String contractType;
		private final String assetClass;
		private final Boolean isCrypto;
		private final String underlyingIdentificationType;
		private final String underlyingIdentification;
		private final List<String> nameOfTheUnderlyingIndex;
		private final List<String> indicatorOfTheUnderlyingIndex;
		private final List<? extends BasketConstituentsReport> basketConstituents;
		private final String customBasketCode;
		private final ISOCurrencyCodeEnum settlementCurrency1;
		private final ISOCurrencyCodeEnum settlementCurrency2;
		private final BigDecimal valuationAmount;
		private final ISOCurrencyCodeEnum valuationCurrency;
		private final ValuationType1Code valuationMethod;
		private final BigDecimal delta;
		private final ZonedDateTime valuationTimestamp;
		private final ZonedDateTime confirmationTimestamp;
		private final String confirmed;
		private final String clearingObligation;
		private final String cleared;
		private final ZonedDateTime clearingTimestamp;
		private final String centralCounterparty;
		private final String masterAgreementType;
		private final String otherMasterAgreementType;
		private final Integer masterAgreementVersion;
		private final Boolean intragroup;
		private final Boolean ptrr;
		private final String typeOfPTRRTechnique;
		private final String ptrrServiceProvider;
		private final String venueOfExecution;
		private final ZonedDateTime executionTimestamp;
		private final Date effectiveDate;
		private final Date expirationDate;
		private final Date earlyTerminationDate;
		private final Date finalContractualSettlementDate;
		private final String deliveryType;
		private final PriceAndNotation price;
		private final List<? extends PriceScheduleReport> priceSchedule;
		private final PackageTransactionPrice packageTransactionPrice;
		private final BigDecimal notionalLeg1;
		private final ISOCurrencyCodeEnum notionalCurrency1;
		private final List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg1;
		private final BigDecimal totalNotionalQuantityLeg1;
		private final List<? extends NotionalQuantityScheduleLeg1Report> notionalQuantityScheduleLeg1;
		private final BigDecimal totalNotionalQuantityLeg2;
		private final BigDecimal notionalLeg2;
		private final ISOCurrencyCodeEnum notionalCurrency2;
		private final List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg2;
		private final List<? extends NotionalQuantityScheduleLeg2Report> notionalQuantityScheduleLeg2;
		private final List<? extends OtherPaymentReport> otherPayment;
		private final BigDecimal fixedRateOfLeg1;
		private final InterestComputationMethod4Code fixedRateDayCountConventionLeg1;
		private final String fixedRatePaymentFrequencyPeriodLeg1;
		private final Integer fixedRatePaymentFrequencyPeriodMultiplierLeg1;
		private final String identifierOfFloatingRateOfLeg1;
		private final String indicatorOfTheFloatingRateOfLeg1;
		private final String nameOfTheFloatingRateOfLeg1;
		private final InterestComputationMethod4Code floatingRateDayCountConventionOfLeg1;
		private final Frequency13Code__1 floatingRatePaymentFrequencyPeriodOfLeg1;
		private final Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg1;
		private final String floatingRateReferencePeriodOfLeg1TimePeriod;
		private final Integer floatingRateReferencePeriodOfLeg1Multiplier;
		private final String floatingRateResetFrequencyPeriodOfLeg1;
		private final Integer floatingRateResetFrequencyPeriodMultiplierLeg1;
		private final SpreadOfLeg1 spreadOfLeg1;
		private final BigDecimal fixedRateOfLeg2;
		private final InterestComputationMethod4Code fixedRateDayCountConventionLeg2;
		private final String fixedRatePaymentFrequencyPeriodLeg2;
		private final Integer fixedRatePaymentFrequencyPeriodMultiplierLeg2;
		private final String identifierOfFloatingRateOfLeg2;
		private final String indicatorOfTheFloatingRateOfLeg2;
		private final String nameOfTheFloatingRateOfLeg2;
		private final InterestComputationMethod4Code floatingRateDayCountConventionOfLeg2;
		private final String floatingRatePaymentFrequencyPeriodOfLeg2;
		private final Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg2;
		private final String floatingRateReferencePeriodOfLeg2TimePeriod;
		private final Integer floatingRateReferencePeriodOfLeg2Multiplier;
		private final String floatingRateResetFrequencyPeriodOfLeg2;
		private final Integer floatingRateResetFrequencyPeriodMultiplierLeg2;
		private final SpreadOfLeg2 spreadOfLeg2;
		private final PackageTransactionSpread packageTransactionSpread;
		private final BigDecimal exchangeRate;
		private final BigDecimal forwardExchangeRate;
		private final String exchangeRateBasis;
		private final String baseProduct;
		private final String subProduct;
		private final String furtherSubProduct;
		private final List<String> deliveryPoint;
		private final String interconnectionPoint;
		private final EnergyLoadType1Code loadType;
		private final List<? extends DeliveryReport> deliveryReport;
		private final String optionType;
		private final String optionStyle;
		private final StrikePriceAndNotation strikePrice;
		private final List<? extends StrikePriceScheduleReport> strikePriceSchedule;
		private final BigDecimal optionPremiumAmount;
		private final ISOCurrencyCodeEnum optionPremiumCurrency;
		private final Date optionPremiumPaymentDate;
		private final Date maturityDateOfTheUnderlying;
		private final String seniority;
		private final String referenceEntity;
		private final Integer series;
		private final Integer version;
		private final BigDecimal indexFactor;
		private final Boolean tranche;
		private final BigDecimal cdSIndexAttachmentPoint;
		private final BigDecimal cdSIndexDetachmentPoint;
		private final Boolean collateralPortfolioIndicator;
		private final String collateralPortfolioCode;
		private final String actionType;
		private final String eventType;
		private final Date eventDate;
		private final String level;
		private final ESMAEMIRNonReportable nonReportable;
		
		protected ESMAEMIRTransactionReportImpl(ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder builder) {
			this.reportingTimestamp = builder.getReportingTimestamp();
			this.reportSubmittingEntityID = builder.getReportSubmittingEntityID();
			this.entityResponsibleForReporting = builder.getEntityResponsibleForReporting();
			this.counterparty1 = builder.getCounterparty1();
			this.natureOfCounterparty1 = builder.getNatureOfCounterparty1();
			this.corporateSectorOfTheCounterparty1 = builder.getCorporateSectorOfTheCounterparty1();
			this.clearingThresholdOfCounterparty1 = builder.getClearingThresholdOfCounterparty1();
			this.counterparty2 = builder.getCounterparty2();
			this.countryOfTheCounterParty2 = builder.getCountryOfTheCounterParty2();
			this.natureOfCounterparty2 = builder.getNatureOfCounterparty2();
			this.corporateSectorOfTheCounterparty2 = builder.getCorporateSectorOfTheCounterparty2();
			this.counterparty2IdentifierType = builder.getCounterparty2IdentifierType();
			this.clearingMember = builder.getClearingMember();
			this.clearingThresholdOfCounterparty2 = builder.getClearingThresholdOfCounterparty2();
			this.reportingObligationOfTheCounterparty2 = builder.getReportingObligationOfTheCounterparty2();
			this.brokerID = builder.getBrokerID();
			this.direction = builder.getDirection();
			this.directionOfLeg1 = builder.getDirectionOfLeg1();
			this.directionOfLeg2 = builder.getDirectionOfLeg2();
			this.directlyLinkedToCommercialActivityOrTreasuryFinancing = builder.getDirectlyLinkedToCommercialActivityOrTreasuryFinancing();
			this.uti = builder.getUti();
			this.utiProprietary = builder.getUtiProprietary();
			this.reportTrackingNumber = builder.getReportTrackingNumber();
			this.priorUti = builder.getPriorUti();
			this.priorUtiProprietary = builder.getPriorUtiProprietary();
			this.ptrrId = builder.getPtrrId();
			this.packageIdentifier = builder.getPackageIdentifier();
			this.subsequentPositionUti = builder.getSubsequentPositionUti();
			this.isin = builder.getIsin();
			this.upi = builder.getUpi();
			this.productClassification = builder.getProductClassification();
			this.contractType = builder.getContractType();
			this.assetClass = builder.getAssetClass();
			this.isCrypto = builder.getIsCrypto();
			this.underlyingIdentificationType = builder.getUnderlyingIdentificationType();
			this.underlyingIdentification = builder.getUnderlyingIdentification();
			this.nameOfTheUnderlyingIndex = ofNullable(builder.getNameOfTheUnderlyingIndex()).filter(_l->!_l.isEmpty()).map(ImmutableList::copyOf).orElse(null);
			this.indicatorOfTheUnderlyingIndex = ofNullable(builder.getIndicatorOfTheUnderlyingIndex()).filter(_l->!_l.isEmpty()).map(ImmutableList::copyOf).orElse(null);
			this.basketConstituents = ofNullable(builder.getBasketConstituents()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.customBasketCode = builder.getCustomBasketCode();
			this.settlementCurrency1 = builder.getSettlementCurrency1();
			this.settlementCurrency2 = builder.getSettlementCurrency2();
			this.valuationAmount = builder.getValuationAmount();
			this.valuationCurrency = builder.getValuationCurrency();
			this.valuationMethod = builder.getValuationMethod();
			this.delta = builder.getDelta();
			this.valuationTimestamp = builder.getValuationTimestamp();
			this.confirmationTimestamp = builder.getConfirmationTimestamp();
			this.confirmed = builder.getConfirmed();
			this.clearingObligation = builder.getClearingObligation();
			this.cleared = builder.getCleared();
			this.clearingTimestamp = builder.getClearingTimestamp();
			this.centralCounterparty = builder.getCentralCounterparty();
			this.masterAgreementType = builder.getMasterAgreementType();
			this.otherMasterAgreementType = builder.getOtherMasterAgreementType();
			this.masterAgreementVersion = builder.getMasterAgreementVersion();
			this.intragroup = builder.getIntragroup();
			this.ptrr = builder.getPtrr();
			this.typeOfPTRRTechnique = builder.getTypeOfPTRRTechnique();
			this.ptrrServiceProvider = builder.getPtrrServiceProvider();
			this.venueOfExecution = builder.getVenueOfExecution();
			this.executionTimestamp = builder.getExecutionTimestamp();
			this.effectiveDate = builder.getEffectiveDate();
			this.expirationDate = builder.getExpirationDate();
			this.earlyTerminationDate = builder.getEarlyTerminationDate();
			this.finalContractualSettlementDate = builder.getFinalContractualSettlementDate();
			this.deliveryType = builder.getDeliveryType();
			this.price = ofNullable(builder.getPrice()).map(f->f.build()).orElse(null);
			this.priceSchedule = ofNullable(builder.getPriceSchedule()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.packageTransactionPrice = ofNullable(builder.getPackageTransactionPrice()).map(f->f.build()).orElse(null);
			this.notionalLeg1 = builder.getNotionalLeg1();
			this.notionalCurrency1 = builder.getNotionalCurrency1();
			this.notionalAmountScheduleLeg1 = ofNullable(builder.getNotionalAmountScheduleLeg1()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.totalNotionalQuantityLeg1 = builder.getTotalNotionalQuantityLeg1();
			this.notionalQuantityScheduleLeg1 = ofNullable(builder.getNotionalQuantityScheduleLeg1()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.totalNotionalQuantityLeg2 = builder.getTotalNotionalQuantityLeg2();
			this.notionalLeg2 = builder.getNotionalLeg2();
			this.notionalCurrency2 = builder.getNotionalCurrency2();
			this.notionalAmountScheduleLeg2 = ofNullable(builder.getNotionalAmountScheduleLeg2()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.notionalQuantityScheduleLeg2 = ofNullable(builder.getNotionalQuantityScheduleLeg2()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.otherPayment = ofNullable(builder.getOtherPayment()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.fixedRateOfLeg1 = builder.getFixedRateOfLeg1();
			this.fixedRateDayCountConventionLeg1 = builder.getFixedRateDayCountConventionLeg1();
			this.fixedRatePaymentFrequencyPeriodLeg1 = builder.getFixedRatePaymentFrequencyPeriodLeg1();
			this.fixedRatePaymentFrequencyPeriodMultiplierLeg1 = builder.getFixedRatePaymentFrequencyPeriodMultiplierLeg1();
			this.identifierOfFloatingRateOfLeg1 = builder.getIdentifierOfFloatingRateOfLeg1();
			this.indicatorOfTheFloatingRateOfLeg1 = builder.getIndicatorOfTheFloatingRateOfLeg1();
			this.nameOfTheFloatingRateOfLeg1 = builder.getNameOfTheFloatingRateOfLeg1();
			this.floatingRateDayCountConventionOfLeg1 = builder.getFloatingRateDayCountConventionOfLeg1();
			this.floatingRatePaymentFrequencyPeriodOfLeg1 = builder.getFloatingRatePaymentFrequencyPeriodOfLeg1();
			this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg1 = builder.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1();
			this.floatingRateReferencePeriodOfLeg1TimePeriod = builder.getFloatingRateReferencePeriodOfLeg1TimePeriod();
			this.floatingRateReferencePeriodOfLeg1Multiplier = builder.getFloatingRateReferencePeriodOfLeg1Multiplier();
			this.floatingRateResetFrequencyPeriodOfLeg1 = builder.getFloatingRateResetFrequencyPeriodOfLeg1();
			this.floatingRateResetFrequencyPeriodMultiplierLeg1 = builder.getFloatingRateResetFrequencyPeriodMultiplierLeg1();
			this.spreadOfLeg1 = ofNullable(builder.getSpreadOfLeg1()).map(f->f.build()).orElse(null);
			this.fixedRateOfLeg2 = builder.getFixedRateOfLeg2();
			this.fixedRateDayCountConventionLeg2 = builder.getFixedRateDayCountConventionLeg2();
			this.fixedRatePaymentFrequencyPeriodLeg2 = builder.getFixedRatePaymentFrequencyPeriodLeg2();
			this.fixedRatePaymentFrequencyPeriodMultiplierLeg2 = builder.getFixedRatePaymentFrequencyPeriodMultiplierLeg2();
			this.identifierOfFloatingRateOfLeg2 = builder.getIdentifierOfFloatingRateOfLeg2();
			this.indicatorOfTheFloatingRateOfLeg2 = builder.getIndicatorOfTheFloatingRateOfLeg2();
			this.nameOfTheFloatingRateOfLeg2 = builder.getNameOfTheFloatingRateOfLeg2();
			this.floatingRateDayCountConventionOfLeg2 = builder.getFloatingRateDayCountConventionOfLeg2();
			this.floatingRatePaymentFrequencyPeriodOfLeg2 = builder.getFloatingRatePaymentFrequencyPeriodOfLeg2();
			this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 = builder.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2();
			this.floatingRateReferencePeriodOfLeg2TimePeriod = builder.getFloatingRateReferencePeriodOfLeg2TimePeriod();
			this.floatingRateReferencePeriodOfLeg2Multiplier = builder.getFloatingRateReferencePeriodOfLeg2Multiplier();
			this.floatingRateResetFrequencyPeriodOfLeg2 = builder.getFloatingRateResetFrequencyPeriodOfLeg2();
			this.floatingRateResetFrequencyPeriodMultiplierLeg2 = builder.getFloatingRateResetFrequencyPeriodMultiplierLeg2();
			this.spreadOfLeg2 = ofNullable(builder.getSpreadOfLeg2()).map(f->f.build()).orElse(null);
			this.packageTransactionSpread = ofNullable(builder.getPackageTransactionSpread()).map(f->f.build()).orElse(null);
			this.exchangeRate = builder.getExchangeRate();
			this.forwardExchangeRate = builder.getForwardExchangeRate();
			this.exchangeRateBasis = builder.getExchangeRateBasis();
			this.baseProduct = builder.getBaseProduct();
			this.subProduct = builder.getSubProduct();
			this.furtherSubProduct = builder.getFurtherSubProduct();
			this.deliveryPoint = ofNullable(builder.getDeliveryPoint()).filter(_l->!_l.isEmpty()).map(ImmutableList::copyOf).orElse(null);
			this.interconnectionPoint = builder.getInterconnectionPoint();
			this.loadType = builder.getLoadType();
			this.deliveryReport = ofNullable(builder.getDeliveryReport()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.optionType = builder.getOptionType();
			this.optionStyle = builder.getOptionStyle();
			this.strikePrice = ofNullable(builder.getStrikePrice()).map(f->f.build()).orElse(null);
			this.strikePriceSchedule = ofNullable(builder.getStrikePriceSchedule()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.optionPremiumAmount = builder.getOptionPremiumAmount();
			this.optionPremiumCurrency = builder.getOptionPremiumCurrency();
			this.optionPremiumPaymentDate = builder.getOptionPremiumPaymentDate();
			this.maturityDateOfTheUnderlying = builder.getMaturityDateOfTheUnderlying();
			this.seniority = builder.getSeniority();
			this.referenceEntity = builder.getReferenceEntity();
			this.series = builder.getSeries();
			this.version = builder.getVersion();
			this.indexFactor = builder.getIndexFactor();
			this.tranche = builder.getTranche();
			this.cdSIndexAttachmentPoint = builder.getCdSIndexAttachmentPoint();
			this.cdSIndexDetachmentPoint = builder.getCdSIndexDetachmentPoint();
			this.collateralPortfolioIndicator = builder.getCollateralPortfolioIndicator();
			this.collateralPortfolioCode = builder.getCollateralPortfolioCode();
			this.actionType = builder.getActionType();
			this.eventType = builder.getEventType();
			this.eventDate = builder.getEventDate();
			this.level = builder.getLevel();
			this.nonReportable = ofNullable(builder.getNonReportable()).map(f->f.build()).orElse(null);
		}
		
		@Override
		@RosettaAttribute("reportingTimestamp")
		public ZonedDateTime getReportingTimestamp() {
			return reportingTimestamp;
		}
		
		@Override
		@RosettaAttribute("reportSubmittingEntityID")
		public String getReportSubmittingEntityID() {
			return reportSubmittingEntityID;
		}
		
		@Override
		@RosettaAttribute("entityResponsibleForReporting")
		public String getEntityResponsibleForReporting() {
			return entityResponsibleForReporting;
		}
		
		@Override
		@RosettaAttribute("counterparty1")
		public String getCounterparty1() {
			return counterparty1;
		}
		
		@Override
		@RosettaAttribute("natureOfCounterparty1")
		public String getNatureOfCounterparty1() {
			return natureOfCounterparty1;
		}
		
		@Override
		@RosettaAttribute("corporateSectorOfTheCounterparty1")
		public String getCorporateSectorOfTheCounterparty1() {
			return corporateSectorOfTheCounterparty1;
		}
		
		@Override
		@RosettaAttribute("clearingThresholdOfCounterparty1")
		public Boolean getClearingThresholdOfCounterparty1() {
			return clearingThresholdOfCounterparty1;
		}
		
		@Override
		@RosettaAttribute("counterparty2")
		public String getCounterparty2() {
			return counterparty2;
		}
		
		@Override
		@RosettaAttribute("countryOfTheCounterParty2")
		public ISOCountryCodeEnum getCountryOfTheCounterParty2() {
			return countryOfTheCounterParty2;
		}
		
		@Override
		@RosettaAttribute("natureOfCounterparty2")
		public String getNatureOfCounterparty2() {
			return natureOfCounterparty2;
		}
		
		@Override
		@RosettaAttribute("corporateSectorOfTheCounterparty2")
		public String getCorporateSectorOfTheCounterparty2() {
			return corporateSectorOfTheCounterparty2;
		}
		
		@Override
		@RosettaAttribute("counterparty2IdentifierType")
		public Boolean getCounterparty2IdentifierType() {
			return counterparty2IdentifierType;
		}
		
		@Override
		@RosettaAttribute("clearingMember")
		public String getClearingMember() {
			return clearingMember;
		}
		
		@Override
		@RosettaAttribute("clearingThresholdOfCounterparty2")
		public Boolean getClearingThresholdOfCounterparty2() {
			return clearingThresholdOfCounterparty2;
		}
		
		@Override
		@RosettaAttribute("reportingObligationOfTheCounterparty2")
		public Boolean getReportingObligationOfTheCounterparty2() {
			return reportingObligationOfTheCounterparty2;
		}
		
		@Override
		@RosettaAttribute("brokerID")
		public String getBrokerID() {
			return brokerID;
		}
		
		@Override
		@RosettaAttribute("direction")
		public String getDirection() {
			return direction;
		}
		
		@Override
		@RosettaAttribute("directionOfLeg1")
		public String getDirectionOfLeg1() {
			return directionOfLeg1;
		}
		
		@Override
		@RosettaAttribute("directionOfLeg2")
		public String getDirectionOfLeg2() {
			return directionOfLeg2;
		}
		
		@Override
		@RosettaAttribute("directlyLinkedToCommercialActivityOrTreasuryFinancing")
		public Boolean getDirectlyLinkedToCommercialActivityOrTreasuryFinancing() {
			return directlyLinkedToCommercialActivityOrTreasuryFinancing;
		}
		
		@Override
		@RosettaAttribute("uti")
		public String getUti() {
			return uti;
		}
		
		@Override
		@RosettaAttribute("utiProprietary")
		public String getUtiProprietary() {
			return utiProprietary;
		}
		
		@Override
		@RosettaAttribute("reportTrackingNumber")
		public String getReportTrackingNumber() {
			return reportTrackingNumber;
		}
		
		@Override
		@RosettaAttribute("priorUti")
		public String getPriorUti() {
			return priorUti;
		}
		
		@Override
		@RosettaAttribute("priorUtiProprietary")
		public String getPriorUtiProprietary() {
			return priorUtiProprietary;
		}
		
		@Override
		@RosettaAttribute("ptrrId")
		public String getPtrrId() {
			return ptrrId;
		}
		
		@Override
		@RosettaAttribute("packageIdentifier")
		public String getPackageIdentifier() {
			return packageIdentifier;
		}
		
		@Override
		@RosettaAttribute("subsequentPositionUti")
		public String getSubsequentPositionUti() {
			return subsequentPositionUti;
		}
		
		@Override
		@RosettaAttribute("isin")
		public String getIsin() {
			return isin;
		}
		
		@Override
		@RosettaAttribute("upi")
		public String getUpi() {
			return upi;
		}
		
		@Override
		@RosettaAttribute("productClassification")
		public String getProductClassification() {
			return productClassification;
		}
		
		@Override
		@RosettaAttribute("contractType")
		public String getContractType() {
			return contractType;
		}
		
		@Override
		@RosettaAttribute("assetClass")
		public String getAssetClass() {
			return assetClass;
		}
		
		@Override
		@RosettaAttribute("isCrypto")
		public Boolean getIsCrypto() {
			return isCrypto;
		}
		
		@Override
		@RosettaAttribute("underlyingIdentificationType")
		public String getUnderlyingIdentificationType() {
			return underlyingIdentificationType;
		}
		
		@Override
		@RosettaAttribute("underlyingIdentification")
		public String getUnderlyingIdentification() {
			return underlyingIdentification;
		}
		
		@Override
		@RosettaAttribute("nameOfTheUnderlyingIndex")
		public List<String> getNameOfTheUnderlyingIndex() {
			return nameOfTheUnderlyingIndex;
		}
		
		@Override
		@RosettaAttribute("indicatorOfTheUnderlyingIndex")
		public List<String> getIndicatorOfTheUnderlyingIndex() {
			return indicatorOfTheUnderlyingIndex;
		}
		
		@Override
		@RosettaAttribute("basketConstituents")
		public List<? extends BasketConstituentsReport> getBasketConstituents() {
			return basketConstituents;
		}
		
		@Override
		@RosettaAttribute("customBasketCode")
		public String getCustomBasketCode() {
			return customBasketCode;
		}
		
		@Override
		@RosettaAttribute("settlementCurrency1")
		public ISOCurrencyCodeEnum getSettlementCurrency1() {
			return settlementCurrency1;
		}
		
		@Override
		@RosettaAttribute("settlementCurrency2")
		public ISOCurrencyCodeEnum getSettlementCurrency2() {
			return settlementCurrency2;
		}
		
		@Override
		@RosettaAttribute("valuationAmount")
		public BigDecimal getValuationAmount() {
			return valuationAmount;
		}
		
		@Override
		@RosettaAttribute("valuationCurrency")
		public ISOCurrencyCodeEnum getValuationCurrency() {
			return valuationCurrency;
		}
		
		@Override
		@RosettaAttribute("valuationMethod")
		public ValuationType1Code getValuationMethod() {
			return valuationMethod;
		}
		
		@Override
		@RosettaAttribute("delta")
		public BigDecimal getDelta() {
			return delta;
		}
		
		@Override
		@RosettaAttribute("valuationTimestamp")
		public ZonedDateTime getValuationTimestamp() {
			return valuationTimestamp;
		}
		
		@Override
		@RosettaAttribute("confirmationTimestamp")
		public ZonedDateTime getConfirmationTimestamp() {
			return confirmationTimestamp;
		}
		
		@Override
		@RosettaAttribute("confirmed")
		public String getConfirmed() {
			return confirmed;
		}
		
		@Override
		@RosettaAttribute("clearingObligation")
		public String getClearingObligation() {
			return clearingObligation;
		}
		
		@Override
		@RosettaAttribute("cleared")
		public String getCleared() {
			return cleared;
		}
		
		@Override
		@RosettaAttribute("clearingTimestamp")
		public ZonedDateTime getClearingTimestamp() {
			return clearingTimestamp;
		}
		
		@Override
		@RosettaAttribute("centralCounterparty")
		public String getCentralCounterparty() {
			return centralCounterparty;
		}
		
		@Override
		@RosettaAttribute("masterAgreementType")
		public String getMasterAgreementType() {
			return masterAgreementType;
		}
		
		@Override
		@RosettaAttribute("otherMasterAgreementType")
		public String getOtherMasterAgreementType() {
			return otherMasterAgreementType;
		}
		
		@Override
		@RosettaAttribute("masterAgreementVersion")
		public Integer getMasterAgreementVersion() {
			return masterAgreementVersion;
		}
		
		@Override
		@RosettaAttribute("intragroup")
		public Boolean getIntragroup() {
			return intragroup;
		}
		
		@Override
		@RosettaAttribute("ptrr")
		public Boolean getPtrr() {
			return ptrr;
		}
		
		@Override
		@RosettaAttribute("typeOfPTRRTechnique")
		public String getTypeOfPTRRTechnique() {
			return typeOfPTRRTechnique;
		}
		
		@Override
		@RosettaAttribute("ptrrServiceProvider")
		public String getPtrrServiceProvider() {
			return ptrrServiceProvider;
		}
		
		@Override
		@RosettaAttribute("venueOfExecution")
		public String getVenueOfExecution() {
			return venueOfExecution;
		}
		
		@Override
		@RosettaAttribute("executionTimestamp")
		public ZonedDateTime getExecutionTimestamp() {
			return executionTimestamp;
		}
		
		@Override
		@RosettaAttribute("effectiveDate")
		public Date getEffectiveDate() {
			return effectiveDate;
		}
		
		@Override
		@RosettaAttribute("expirationDate")
		public Date getExpirationDate() {
			return expirationDate;
		}
		
		@Override
		@RosettaAttribute("earlyTerminationDate")
		public Date getEarlyTerminationDate() {
			return earlyTerminationDate;
		}
		
		@Override
		@RosettaAttribute("finalContractualSettlementDate")
		public Date getFinalContractualSettlementDate() {
			return finalContractualSettlementDate;
		}
		
		@Override
		@RosettaAttribute("deliveryType")
		public String getDeliveryType() {
			return deliveryType;
		}
		
		@Override
		@RosettaAttribute("price")
		public PriceAndNotation getPrice() {
			return price;
		}
		
		@Override
		@RosettaAttribute("priceSchedule")
		public List<? extends PriceScheduleReport> getPriceSchedule() {
			return priceSchedule;
		}
		
		@Override
		@RosettaAttribute("packageTransactionPrice")
		public PackageTransactionPrice getPackageTransactionPrice() {
			return packageTransactionPrice;
		}
		
		@Override
		@RosettaAttribute("notionalLeg1")
		public BigDecimal getNotionalLeg1() {
			return notionalLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalCurrency1")
		public ISOCurrencyCodeEnum getNotionalCurrency1() {
			return notionalCurrency1;
		}
		
		@Override
		@RosettaAttribute("notionalAmountScheduleLeg1")
		public List<? extends NotionalAmountScheduleLeg1Report> getNotionalAmountScheduleLeg1() {
			return notionalAmountScheduleLeg1;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg1")
		public BigDecimal getTotalNotionalQuantityLeg1() {
			return totalNotionalQuantityLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityScheduleLeg1")
		public List<? extends NotionalQuantityScheduleLeg1Report> getNotionalQuantityScheduleLeg1() {
			return notionalQuantityScheduleLeg1;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg2")
		public BigDecimal getTotalNotionalQuantityLeg2() {
			return totalNotionalQuantityLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalLeg2")
		public BigDecimal getNotionalLeg2() {
			return notionalLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalCurrency2")
		public ISOCurrencyCodeEnum getNotionalCurrency2() {
			return notionalCurrency2;
		}
		
		@Override
		@RosettaAttribute("notionalAmountScheduleLeg2")
		public List<? extends NotionalAmountScheduleLeg2Report> getNotionalAmountScheduleLeg2() {
			return notionalAmountScheduleLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityScheduleLeg2")
		public List<? extends NotionalQuantityScheduleLeg2Report> getNotionalQuantityScheduleLeg2() {
			return notionalQuantityScheduleLeg2;
		}
		
		@Override
		@RosettaAttribute("otherPayment")
		public List<? extends OtherPaymentReport> getOtherPayment() {
			return otherPayment;
		}
		
		@Override
		@RosettaAttribute("fixedRateOfLeg1")
		public BigDecimal getFixedRateOfLeg1() {
			return fixedRateOfLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRateDayCountConventionLeg1")
		public InterestComputationMethod4Code getFixedRateDayCountConventionLeg1() {
			return fixedRateDayCountConventionLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodLeg1")
		public String getFixedRatePaymentFrequencyPeriodLeg1() {
			return fixedRatePaymentFrequencyPeriodLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierLeg1")
		public Integer getFixedRatePaymentFrequencyPeriodMultiplierLeg1() {
			return fixedRatePaymentFrequencyPeriodMultiplierLeg1;
		}
		
		@Override
		@RosettaAttribute("identifierOfFloatingRateOfLeg1")
		public String getIdentifierOfFloatingRateOfLeg1() {
			return identifierOfFloatingRateOfLeg1;
		}
		
		@Override
		@RosettaAttribute("indicatorOfTheFloatingRateOfLeg1")
		public String getIndicatorOfTheFloatingRateOfLeg1() {
			return indicatorOfTheFloatingRateOfLeg1;
		}
		
		@Override
		@RosettaAttribute("nameOfTheFloatingRateOfLeg1")
		public String getNameOfTheFloatingRateOfLeg1() {
			return nameOfTheFloatingRateOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateDayCountConventionOfLeg1")
		public InterestComputationMethod4Code getFloatingRateDayCountConventionOfLeg1() {
			return floatingRateDayCountConventionOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodOfLeg1")
		public Frequency13Code__1 getFloatingRatePaymentFrequencyPeriodOfLeg1() {
			return floatingRatePaymentFrequencyPeriodOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierOfLeg1")
		public Integer getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1() {
			return floatingRatePaymentFrequencyPeriodMultiplierOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg1TimePeriod")
		public String getFloatingRateReferencePeriodOfLeg1TimePeriod() {
			return floatingRateReferencePeriodOfLeg1TimePeriod;
		}
		
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg1Multiplier")
		public Integer getFloatingRateReferencePeriodOfLeg1Multiplier() {
			return floatingRateReferencePeriodOfLeg1Multiplier;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodOfLeg1")
		public String getFloatingRateResetFrequencyPeriodOfLeg1() {
			return floatingRateResetFrequencyPeriodOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierLeg1")
		public Integer getFloatingRateResetFrequencyPeriodMultiplierLeg1() {
			return floatingRateResetFrequencyPeriodMultiplierLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadOfLeg1")
		public SpreadOfLeg1 getSpreadOfLeg1() {
			return spreadOfLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRateOfLeg2")
		public BigDecimal getFixedRateOfLeg2() {
			return fixedRateOfLeg2;
		}
		
		@Override
		@RosettaAttribute("fixedRateDayCountConventionLeg2")
		public InterestComputationMethod4Code getFixedRateDayCountConventionLeg2() {
			return fixedRateDayCountConventionLeg2;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodLeg2")
		public String getFixedRatePaymentFrequencyPeriodLeg2() {
			return fixedRatePaymentFrequencyPeriodLeg2;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierLeg2")
		public Integer getFixedRatePaymentFrequencyPeriodMultiplierLeg2() {
			return fixedRatePaymentFrequencyPeriodMultiplierLeg2;
		}
		
		@Override
		@RosettaAttribute("identifierOfFloatingRateOfLeg2")
		public String getIdentifierOfFloatingRateOfLeg2() {
			return identifierOfFloatingRateOfLeg2;
		}
		
		@Override
		@RosettaAttribute("indicatorOfTheFloatingRateOfLeg2")
		public String getIndicatorOfTheFloatingRateOfLeg2() {
			return indicatorOfTheFloatingRateOfLeg2;
		}
		
		@Override
		@RosettaAttribute("nameOfTheFloatingRateOfLeg2")
		public String getNameOfTheFloatingRateOfLeg2() {
			return nameOfTheFloatingRateOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateDayCountConventionOfLeg2")
		public InterestComputationMethod4Code getFloatingRateDayCountConventionOfLeg2() {
			return floatingRateDayCountConventionOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodOfLeg2")
		public String getFloatingRatePaymentFrequencyPeriodOfLeg2() {
			return floatingRatePaymentFrequencyPeriodOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierOfLeg2")
		public Integer getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2() {
			return floatingRatePaymentFrequencyPeriodMultiplierOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg2TimePeriod")
		public String getFloatingRateReferencePeriodOfLeg2TimePeriod() {
			return floatingRateReferencePeriodOfLeg2TimePeriod;
		}
		
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg2Multiplier")
		public Integer getFloatingRateReferencePeriodOfLeg2Multiplier() {
			return floatingRateReferencePeriodOfLeg2Multiplier;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodOfLeg2")
		public String getFloatingRateResetFrequencyPeriodOfLeg2() {
			return floatingRateResetFrequencyPeriodOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierLeg2")
		public Integer getFloatingRateResetFrequencyPeriodMultiplierLeg2() {
			return floatingRateResetFrequencyPeriodMultiplierLeg2;
		}
		
		@Override
		@RosettaAttribute("spreadOfLeg2")
		public SpreadOfLeg2 getSpreadOfLeg2() {
			return spreadOfLeg2;
		}
		
		@Override
		@RosettaAttribute("packageTransactionSpread")
		public PackageTransactionSpread getPackageTransactionSpread() {
			return packageTransactionSpread;
		}
		
		@Override
		@RosettaAttribute("exchangeRate")
		public BigDecimal getExchangeRate() {
			return exchangeRate;
		}
		
		@Override
		@RosettaAttribute("forwardExchangeRate")
		public BigDecimal getForwardExchangeRate() {
			return forwardExchangeRate;
		}
		
		@Override
		@RosettaAttribute("exchangeRateBasis")
		public String getExchangeRateBasis() {
			return exchangeRateBasis;
		}
		
		@Override
		@RosettaAttribute("baseProduct")
		public String getBaseProduct() {
			return baseProduct;
		}
		
		@Override
		@RosettaAttribute("subProduct")
		public String getSubProduct() {
			return subProduct;
		}
		
		@Override
		@RosettaAttribute("furtherSubProduct")
		public String getFurtherSubProduct() {
			return furtherSubProduct;
		}
		
		@Override
		@RosettaAttribute("deliveryPoint")
		public List<String> getDeliveryPoint() {
			return deliveryPoint;
		}
		
		@Override
		@RosettaAttribute("interconnectionPoint")
		public String getInterconnectionPoint() {
			return interconnectionPoint;
		}
		
		@Override
		@RosettaAttribute("loadType")
		public EnergyLoadType1Code getLoadType() {
			return loadType;
		}
		
		@Override
		@RosettaAttribute("deliveryReport")
		public List<? extends DeliveryReport> getDeliveryReport() {
			return deliveryReport;
		}
		
		@Override
		@RosettaAttribute("optionType")
		public String getOptionType() {
			return optionType;
		}
		
		@Override
		@RosettaAttribute("optionStyle")
		public String getOptionStyle() {
			return optionStyle;
		}
		
		@Override
		@RosettaAttribute("strikePrice")
		public StrikePriceAndNotation getStrikePrice() {
			return strikePrice;
		}
		
		@Override
		@RosettaAttribute("strikePriceSchedule")
		public List<? extends StrikePriceScheduleReport> getStrikePriceSchedule() {
			return strikePriceSchedule;
		}
		
		@Override
		@RosettaAttribute("optionPremiumAmount")
		public BigDecimal getOptionPremiumAmount() {
			return optionPremiumAmount;
		}
		
		@Override
		@RosettaAttribute("optionPremiumCurrency")
		public ISOCurrencyCodeEnum getOptionPremiumCurrency() {
			return optionPremiumCurrency;
		}
		
		@Override
		@RosettaAttribute("optionPremiumPaymentDate")
		public Date getOptionPremiumPaymentDate() {
			return optionPremiumPaymentDate;
		}
		
		@Override
		@RosettaAttribute("maturityDateOfTheUnderlying")
		public Date getMaturityDateOfTheUnderlying() {
			return maturityDateOfTheUnderlying;
		}
		
		@Override
		@RosettaAttribute("seniority")
		public String getSeniority() {
			return seniority;
		}
		
		@Override
		@RosettaAttribute("referenceEntity")
		public String getReferenceEntity() {
			return referenceEntity;
		}
		
		@Override
		@RosettaAttribute("series")
		public Integer getSeries() {
			return series;
		}
		
		@Override
		@RosettaAttribute("version")
		public Integer getVersion() {
			return version;
		}
		
		@Override
		@RosettaAttribute("indexFactor")
		public BigDecimal getIndexFactor() {
			return indexFactor;
		}
		
		@Override
		@RosettaAttribute("tranche")
		public Boolean getTranche() {
			return tranche;
		}
		
		@Override
		@RosettaAttribute("cdSIndexAttachmentPoint")
		public BigDecimal getCdSIndexAttachmentPoint() {
			return cdSIndexAttachmentPoint;
		}
		
		@Override
		@RosettaAttribute("cdSIndexDetachmentPoint")
		public BigDecimal getCdSIndexDetachmentPoint() {
			return cdSIndexDetachmentPoint;
		}
		
		@Override
		@RosettaAttribute("collateralPortfolioIndicator")
		public Boolean getCollateralPortfolioIndicator() {
			return collateralPortfolioIndicator;
		}
		
		@Override
		@RosettaAttribute("collateralPortfolioCode")
		public String getCollateralPortfolioCode() {
			return collateralPortfolioCode;
		}
		
		@Override
		@RosettaAttribute("actionType")
		public String getActionType() {
			return actionType;
		}
		
		@Override
		@RosettaAttribute("eventType")
		public String getEventType() {
			return eventType;
		}
		
		@Override
		@RosettaAttribute("eventDate")
		public Date getEventDate() {
			return eventDate;
		}
		
		@Override
		@RosettaAttribute("level")
		public String getLevel() {
			return level;
		}
		
		@Override
		@RosettaAttribute("nonReportable")
		public ESMAEMIRNonReportable getNonReportable() {
			return nonReportable;
		}
		
		@Override
		public ESMAEMIRTransactionReport build() {
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder toBuilder() {
			ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder builder) {
			ofNullable(getReportingTimestamp()).ifPresent(builder::setReportingTimestamp);
			ofNullable(getReportSubmittingEntityID()).ifPresent(builder::setReportSubmittingEntityID);
			ofNullable(getEntityResponsibleForReporting()).ifPresent(builder::setEntityResponsibleForReporting);
			ofNullable(getCounterparty1()).ifPresent(builder::setCounterparty1);
			ofNullable(getNatureOfCounterparty1()).ifPresent(builder::setNatureOfCounterparty1);
			ofNullable(getCorporateSectorOfTheCounterparty1()).ifPresent(builder::setCorporateSectorOfTheCounterparty1);
			ofNullable(getClearingThresholdOfCounterparty1()).ifPresent(builder::setClearingThresholdOfCounterparty1);
			ofNullable(getCounterparty2()).ifPresent(builder::setCounterparty2);
			ofNullable(getCountryOfTheCounterParty2()).ifPresent(builder::setCountryOfTheCounterParty2);
			ofNullable(getNatureOfCounterparty2()).ifPresent(builder::setNatureOfCounterparty2);
			ofNullable(getCorporateSectorOfTheCounterparty2()).ifPresent(builder::setCorporateSectorOfTheCounterparty2);
			ofNullable(getCounterparty2IdentifierType()).ifPresent(builder::setCounterparty2IdentifierType);
			ofNullable(getClearingMember()).ifPresent(builder::setClearingMember);
			ofNullable(getClearingThresholdOfCounterparty2()).ifPresent(builder::setClearingThresholdOfCounterparty2);
			ofNullable(getReportingObligationOfTheCounterparty2()).ifPresent(builder::setReportingObligationOfTheCounterparty2);
			ofNullable(getBrokerID()).ifPresent(builder::setBrokerID);
			ofNullable(getDirection()).ifPresent(builder::setDirection);
			ofNullable(getDirectionOfLeg1()).ifPresent(builder::setDirectionOfLeg1);
			ofNullable(getDirectionOfLeg2()).ifPresent(builder::setDirectionOfLeg2);
			ofNullable(getDirectlyLinkedToCommercialActivityOrTreasuryFinancing()).ifPresent(builder::setDirectlyLinkedToCommercialActivityOrTreasuryFinancing);
			ofNullable(getUti()).ifPresent(builder::setUti);
			ofNullable(getUtiProprietary()).ifPresent(builder::setUtiProprietary);
			ofNullable(getReportTrackingNumber()).ifPresent(builder::setReportTrackingNumber);
			ofNullable(getPriorUti()).ifPresent(builder::setPriorUti);
			ofNullable(getPriorUtiProprietary()).ifPresent(builder::setPriorUtiProprietary);
			ofNullable(getPtrrId()).ifPresent(builder::setPtrrId);
			ofNullable(getPackageIdentifier()).ifPresent(builder::setPackageIdentifier);
			ofNullable(getSubsequentPositionUti()).ifPresent(builder::setSubsequentPositionUti);
			ofNullable(getIsin()).ifPresent(builder::setIsin);
			ofNullable(getUpi()).ifPresent(builder::setUpi);
			ofNullable(getProductClassification()).ifPresent(builder::setProductClassification);
			ofNullable(getContractType()).ifPresent(builder::setContractType);
			ofNullable(getAssetClass()).ifPresent(builder::setAssetClass);
			ofNullable(getIsCrypto()).ifPresent(builder::setIsCrypto);
			ofNullable(getUnderlyingIdentificationType()).ifPresent(builder::setUnderlyingIdentificationType);
			ofNullable(getUnderlyingIdentification()).ifPresent(builder::setUnderlyingIdentification);
			ofNullable(getNameOfTheUnderlyingIndex()).ifPresent(builder::setNameOfTheUnderlyingIndex);
			ofNullable(getIndicatorOfTheUnderlyingIndex()).ifPresent(builder::setIndicatorOfTheUnderlyingIndex);
			ofNullable(getBasketConstituents()).ifPresent(builder::setBasketConstituents);
			ofNullable(getCustomBasketCode()).ifPresent(builder::setCustomBasketCode);
			ofNullable(getSettlementCurrency1()).ifPresent(builder::setSettlementCurrency1);
			ofNullable(getSettlementCurrency2()).ifPresent(builder::setSettlementCurrency2);
			ofNullable(getValuationAmount()).ifPresent(builder::setValuationAmount);
			ofNullable(getValuationCurrency()).ifPresent(builder::setValuationCurrency);
			ofNullable(getValuationMethod()).ifPresent(builder::setValuationMethod);
			ofNullable(getDelta()).ifPresent(builder::setDelta);
			ofNullable(getValuationTimestamp()).ifPresent(builder::setValuationTimestamp);
			ofNullable(getConfirmationTimestamp()).ifPresent(builder::setConfirmationTimestamp);
			ofNullable(getConfirmed()).ifPresent(builder::setConfirmed);
			ofNullable(getClearingObligation()).ifPresent(builder::setClearingObligation);
			ofNullable(getCleared()).ifPresent(builder::setCleared);
			ofNullable(getClearingTimestamp()).ifPresent(builder::setClearingTimestamp);
			ofNullable(getCentralCounterparty()).ifPresent(builder::setCentralCounterparty);
			ofNullable(getMasterAgreementType()).ifPresent(builder::setMasterAgreementType);
			ofNullable(getOtherMasterAgreementType()).ifPresent(builder::setOtherMasterAgreementType);
			ofNullable(getMasterAgreementVersion()).ifPresent(builder::setMasterAgreementVersion);
			ofNullable(getIntragroup()).ifPresent(builder::setIntragroup);
			ofNullable(getPtrr()).ifPresent(builder::setPtrr);
			ofNullable(getTypeOfPTRRTechnique()).ifPresent(builder::setTypeOfPTRRTechnique);
			ofNullable(getPtrrServiceProvider()).ifPresent(builder::setPtrrServiceProvider);
			ofNullable(getVenueOfExecution()).ifPresent(builder::setVenueOfExecution);
			ofNullable(getExecutionTimestamp()).ifPresent(builder::setExecutionTimestamp);
			ofNullable(getEffectiveDate()).ifPresent(builder::setEffectiveDate);
			ofNullable(getExpirationDate()).ifPresent(builder::setExpirationDate);
			ofNullable(getEarlyTerminationDate()).ifPresent(builder::setEarlyTerminationDate);
			ofNullable(getFinalContractualSettlementDate()).ifPresent(builder::setFinalContractualSettlementDate);
			ofNullable(getDeliveryType()).ifPresent(builder::setDeliveryType);
			ofNullable(getPrice()).ifPresent(builder::setPrice);
			ofNullable(getPriceSchedule()).ifPresent(builder::setPriceSchedule);
			ofNullable(getPackageTransactionPrice()).ifPresent(builder::setPackageTransactionPrice);
			ofNullable(getNotionalLeg1()).ifPresent(builder::setNotionalLeg1);
			ofNullable(getNotionalCurrency1()).ifPresent(builder::setNotionalCurrency1);
			ofNullable(getNotionalAmountScheduleLeg1()).ifPresent(builder::setNotionalAmountScheduleLeg1);
			ofNullable(getTotalNotionalQuantityLeg1()).ifPresent(builder::setTotalNotionalQuantityLeg1);
			ofNullable(getNotionalQuantityScheduleLeg1()).ifPresent(builder::setNotionalQuantityScheduleLeg1);
			ofNullable(getTotalNotionalQuantityLeg2()).ifPresent(builder::setTotalNotionalQuantityLeg2);
			ofNullable(getNotionalLeg2()).ifPresent(builder::setNotionalLeg2);
			ofNullable(getNotionalCurrency2()).ifPresent(builder::setNotionalCurrency2);
			ofNullable(getNotionalAmountScheduleLeg2()).ifPresent(builder::setNotionalAmountScheduleLeg2);
			ofNullable(getNotionalQuantityScheduleLeg2()).ifPresent(builder::setNotionalQuantityScheduleLeg2);
			ofNullable(getOtherPayment()).ifPresent(builder::setOtherPayment);
			ofNullable(getFixedRateOfLeg1()).ifPresent(builder::setFixedRateOfLeg1);
			ofNullable(getFixedRateDayCountConventionLeg1()).ifPresent(builder::setFixedRateDayCountConventionLeg1);
			ofNullable(getFixedRatePaymentFrequencyPeriodLeg1()).ifPresent(builder::setFixedRatePaymentFrequencyPeriodLeg1);
			ofNullable(getFixedRatePaymentFrequencyPeriodMultiplierLeg1()).ifPresent(builder::setFixedRatePaymentFrequencyPeriodMultiplierLeg1);
			ofNullable(getIdentifierOfFloatingRateOfLeg1()).ifPresent(builder::setIdentifierOfFloatingRateOfLeg1);
			ofNullable(getIndicatorOfTheFloatingRateOfLeg1()).ifPresent(builder::setIndicatorOfTheFloatingRateOfLeg1);
			ofNullable(getNameOfTheFloatingRateOfLeg1()).ifPresent(builder::setNameOfTheFloatingRateOfLeg1);
			ofNullable(getFloatingRateDayCountConventionOfLeg1()).ifPresent(builder::setFloatingRateDayCountConventionOfLeg1);
			ofNullable(getFloatingRatePaymentFrequencyPeriodOfLeg1()).ifPresent(builder::setFloatingRatePaymentFrequencyPeriodOfLeg1);
			ofNullable(getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1()).ifPresent(builder::setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1);
			ofNullable(getFloatingRateReferencePeriodOfLeg1TimePeriod()).ifPresent(builder::setFloatingRateReferencePeriodOfLeg1TimePeriod);
			ofNullable(getFloatingRateReferencePeriodOfLeg1Multiplier()).ifPresent(builder::setFloatingRateReferencePeriodOfLeg1Multiplier);
			ofNullable(getFloatingRateResetFrequencyPeriodOfLeg1()).ifPresent(builder::setFloatingRateResetFrequencyPeriodOfLeg1);
			ofNullable(getFloatingRateResetFrequencyPeriodMultiplierLeg1()).ifPresent(builder::setFloatingRateResetFrequencyPeriodMultiplierLeg1);
			ofNullable(getSpreadOfLeg1()).ifPresent(builder::setSpreadOfLeg1);
			ofNullable(getFixedRateOfLeg2()).ifPresent(builder::setFixedRateOfLeg2);
			ofNullable(getFixedRateDayCountConventionLeg2()).ifPresent(builder::setFixedRateDayCountConventionLeg2);
			ofNullable(getFixedRatePaymentFrequencyPeriodLeg2()).ifPresent(builder::setFixedRatePaymentFrequencyPeriodLeg2);
			ofNullable(getFixedRatePaymentFrequencyPeriodMultiplierLeg2()).ifPresent(builder::setFixedRatePaymentFrequencyPeriodMultiplierLeg2);
			ofNullable(getIdentifierOfFloatingRateOfLeg2()).ifPresent(builder::setIdentifierOfFloatingRateOfLeg2);
			ofNullable(getIndicatorOfTheFloatingRateOfLeg2()).ifPresent(builder::setIndicatorOfTheFloatingRateOfLeg2);
			ofNullable(getNameOfTheFloatingRateOfLeg2()).ifPresent(builder::setNameOfTheFloatingRateOfLeg2);
			ofNullable(getFloatingRateDayCountConventionOfLeg2()).ifPresent(builder::setFloatingRateDayCountConventionOfLeg2);
			ofNullable(getFloatingRatePaymentFrequencyPeriodOfLeg2()).ifPresent(builder::setFloatingRatePaymentFrequencyPeriodOfLeg2);
			ofNullable(getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2()).ifPresent(builder::setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2);
			ofNullable(getFloatingRateReferencePeriodOfLeg2TimePeriod()).ifPresent(builder::setFloatingRateReferencePeriodOfLeg2TimePeriod);
			ofNullable(getFloatingRateReferencePeriodOfLeg2Multiplier()).ifPresent(builder::setFloatingRateReferencePeriodOfLeg2Multiplier);
			ofNullable(getFloatingRateResetFrequencyPeriodOfLeg2()).ifPresent(builder::setFloatingRateResetFrequencyPeriodOfLeg2);
			ofNullable(getFloatingRateResetFrequencyPeriodMultiplierLeg2()).ifPresent(builder::setFloatingRateResetFrequencyPeriodMultiplierLeg2);
			ofNullable(getSpreadOfLeg2()).ifPresent(builder::setSpreadOfLeg2);
			ofNullable(getPackageTransactionSpread()).ifPresent(builder::setPackageTransactionSpread);
			ofNullable(getExchangeRate()).ifPresent(builder::setExchangeRate);
			ofNullable(getForwardExchangeRate()).ifPresent(builder::setForwardExchangeRate);
			ofNullable(getExchangeRateBasis()).ifPresent(builder::setExchangeRateBasis);
			ofNullable(getBaseProduct()).ifPresent(builder::setBaseProduct);
			ofNullable(getSubProduct()).ifPresent(builder::setSubProduct);
			ofNullable(getFurtherSubProduct()).ifPresent(builder::setFurtherSubProduct);
			ofNullable(getDeliveryPoint()).ifPresent(builder::setDeliveryPoint);
			ofNullable(getInterconnectionPoint()).ifPresent(builder::setInterconnectionPoint);
			ofNullable(getLoadType()).ifPresent(builder::setLoadType);
			ofNullable(getDeliveryReport()).ifPresent(builder::setDeliveryReport);
			ofNullable(getOptionType()).ifPresent(builder::setOptionType);
			ofNullable(getOptionStyle()).ifPresent(builder::setOptionStyle);
			ofNullable(getStrikePrice()).ifPresent(builder::setStrikePrice);
			ofNullable(getStrikePriceSchedule()).ifPresent(builder::setStrikePriceSchedule);
			ofNullable(getOptionPremiumAmount()).ifPresent(builder::setOptionPremiumAmount);
			ofNullable(getOptionPremiumCurrency()).ifPresent(builder::setOptionPremiumCurrency);
			ofNullable(getOptionPremiumPaymentDate()).ifPresent(builder::setOptionPremiumPaymentDate);
			ofNullable(getMaturityDateOfTheUnderlying()).ifPresent(builder::setMaturityDateOfTheUnderlying);
			ofNullable(getSeniority()).ifPresent(builder::setSeniority);
			ofNullable(getReferenceEntity()).ifPresent(builder::setReferenceEntity);
			ofNullable(getSeries()).ifPresent(builder::setSeries);
			ofNullable(getVersion()).ifPresent(builder::setVersion);
			ofNullable(getIndexFactor()).ifPresent(builder::setIndexFactor);
			ofNullable(getTranche()).ifPresent(builder::setTranche);
			ofNullable(getCdSIndexAttachmentPoint()).ifPresent(builder::setCdSIndexAttachmentPoint);
			ofNullable(getCdSIndexDetachmentPoint()).ifPresent(builder::setCdSIndexDetachmentPoint);
			ofNullable(getCollateralPortfolioIndicator()).ifPresent(builder::setCollateralPortfolioIndicator);
			ofNullable(getCollateralPortfolioCode()).ifPresent(builder::setCollateralPortfolioCode);
			ofNullable(getActionType()).ifPresent(builder::setActionType);
			ofNullable(getEventType()).ifPresent(builder::setEventType);
			ofNullable(getEventDate()).ifPresent(builder::setEventDate);
			ofNullable(getLevel()).ifPresent(builder::setLevel);
			ofNullable(getNonReportable()).ifPresent(builder::setNonReportable);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			ESMAEMIRTransactionReport _that = getType().cast(o);
		
			if (!Objects.equals(reportingTimestamp, _that.getReportingTimestamp())) return false;
			if (!Objects.equals(reportSubmittingEntityID, _that.getReportSubmittingEntityID())) return false;
			if (!Objects.equals(entityResponsibleForReporting, _that.getEntityResponsibleForReporting())) return false;
			if (!Objects.equals(counterparty1, _that.getCounterparty1())) return false;
			if (!Objects.equals(natureOfCounterparty1, _that.getNatureOfCounterparty1())) return false;
			if (!Objects.equals(corporateSectorOfTheCounterparty1, _that.getCorporateSectorOfTheCounterparty1())) return false;
			if (!Objects.equals(clearingThresholdOfCounterparty1, _that.getClearingThresholdOfCounterparty1())) return false;
			if (!Objects.equals(counterparty2, _that.getCounterparty2())) return false;
			if (!Objects.equals(countryOfTheCounterParty2, _that.getCountryOfTheCounterParty2())) return false;
			if (!Objects.equals(natureOfCounterparty2, _that.getNatureOfCounterparty2())) return false;
			if (!Objects.equals(corporateSectorOfTheCounterparty2, _that.getCorporateSectorOfTheCounterparty2())) return false;
			if (!Objects.equals(counterparty2IdentifierType, _that.getCounterparty2IdentifierType())) return false;
			if (!Objects.equals(clearingMember, _that.getClearingMember())) return false;
			if (!Objects.equals(clearingThresholdOfCounterparty2, _that.getClearingThresholdOfCounterparty2())) return false;
			if (!Objects.equals(reportingObligationOfTheCounterparty2, _that.getReportingObligationOfTheCounterparty2())) return false;
			if (!Objects.equals(brokerID, _that.getBrokerID())) return false;
			if (!Objects.equals(direction, _that.getDirection())) return false;
			if (!Objects.equals(directionOfLeg1, _that.getDirectionOfLeg1())) return false;
			if (!Objects.equals(directionOfLeg2, _that.getDirectionOfLeg2())) return false;
			if (!Objects.equals(directlyLinkedToCommercialActivityOrTreasuryFinancing, _that.getDirectlyLinkedToCommercialActivityOrTreasuryFinancing())) return false;
			if (!Objects.equals(uti, _that.getUti())) return false;
			if (!Objects.equals(utiProprietary, _that.getUtiProprietary())) return false;
			if (!Objects.equals(reportTrackingNumber, _that.getReportTrackingNumber())) return false;
			if (!Objects.equals(priorUti, _that.getPriorUti())) return false;
			if (!Objects.equals(priorUtiProprietary, _that.getPriorUtiProprietary())) return false;
			if (!Objects.equals(ptrrId, _that.getPtrrId())) return false;
			if (!Objects.equals(packageIdentifier, _that.getPackageIdentifier())) return false;
			if (!Objects.equals(subsequentPositionUti, _that.getSubsequentPositionUti())) return false;
			if (!Objects.equals(isin, _that.getIsin())) return false;
			if (!Objects.equals(upi, _that.getUpi())) return false;
			if (!Objects.equals(productClassification, _that.getProductClassification())) return false;
			if (!Objects.equals(contractType, _that.getContractType())) return false;
			if (!Objects.equals(assetClass, _that.getAssetClass())) return false;
			if (!Objects.equals(isCrypto, _that.getIsCrypto())) return false;
			if (!Objects.equals(underlyingIdentificationType, _that.getUnderlyingIdentificationType())) return false;
			if (!Objects.equals(underlyingIdentification, _that.getUnderlyingIdentification())) return false;
			if (!ListEquals.listEquals(nameOfTheUnderlyingIndex, _that.getNameOfTheUnderlyingIndex())) return false;
			if (!ListEquals.listEquals(indicatorOfTheUnderlyingIndex, _that.getIndicatorOfTheUnderlyingIndex())) return false;
			if (!ListEquals.listEquals(basketConstituents, _that.getBasketConstituents())) return false;
			if (!Objects.equals(customBasketCode, _that.getCustomBasketCode())) return false;
			if (!Objects.equals(settlementCurrency1, _that.getSettlementCurrency1())) return false;
			if (!Objects.equals(settlementCurrency2, _that.getSettlementCurrency2())) return false;
			if (!Objects.equals(valuationAmount, _that.getValuationAmount())) return false;
			if (!Objects.equals(valuationCurrency, _that.getValuationCurrency())) return false;
			if (!Objects.equals(valuationMethod, _that.getValuationMethod())) return false;
			if (!Objects.equals(delta, _that.getDelta())) return false;
			if (!Objects.equals(valuationTimestamp, _that.getValuationTimestamp())) return false;
			if (!Objects.equals(confirmationTimestamp, _that.getConfirmationTimestamp())) return false;
			if (!Objects.equals(confirmed, _that.getConfirmed())) return false;
			if (!Objects.equals(clearingObligation, _that.getClearingObligation())) return false;
			if (!Objects.equals(cleared, _that.getCleared())) return false;
			if (!Objects.equals(clearingTimestamp, _that.getClearingTimestamp())) return false;
			if (!Objects.equals(centralCounterparty, _that.getCentralCounterparty())) return false;
			if (!Objects.equals(masterAgreementType, _that.getMasterAgreementType())) return false;
			if (!Objects.equals(otherMasterAgreementType, _that.getOtherMasterAgreementType())) return false;
			if (!Objects.equals(masterAgreementVersion, _that.getMasterAgreementVersion())) return false;
			if (!Objects.equals(intragroup, _that.getIntragroup())) return false;
			if (!Objects.equals(ptrr, _that.getPtrr())) return false;
			if (!Objects.equals(typeOfPTRRTechnique, _that.getTypeOfPTRRTechnique())) return false;
			if (!Objects.equals(ptrrServiceProvider, _that.getPtrrServiceProvider())) return false;
			if (!Objects.equals(venueOfExecution, _that.getVenueOfExecution())) return false;
			if (!Objects.equals(executionTimestamp, _that.getExecutionTimestamp())) return false;
			if (!Objects.equals(effectiveDate, _that.getEffectiveDate())) return false;
			if (!Objects.equals(expirationDate, _that.getExpirationDate())) return false;
			if (!Objects.equals(earlyTerminationDate, _that.getEarlyTerminationDate())) return false;
			if (!Objects.equals(finalContractualSettlementDate, _that.getFinalContractualSettlementDate())) return false;
			if (!Objects.equals(deliveryType, _that.getDeliveryType())) return false;
			if (!Objects.equals(price, _that.getPrice())) return false;
			if (!ListEquals.listEquals(priceSchedule, _that.getPriceSchedule())) return false;
			if (!Objects.equals(packageTransactionPrice, _that.getPackageTransactionPrice())) return false;
			if (!Objects.equals(notionalLeg1, _that.getNotionalLeg1())) return false;
			if (!Objects.equals(notionalCurrency1, _that.getNotionalCurrency1())) return false;
			if (!ListEquals.listEquals(notionalAmountScheduleLeg1, _that.getNotionalAmountScheduleLeg1())) return false;
			if (!Objects.equals(totalNotionalQuantityLeg1, _that.getTotalNotionalQuantityLeg1())) return false;
			if (!ListEquals.listEquals(notionalQuantityScheduleLeg1, _that.getNotionalQuantityScheduleLeg1())) return false;
			if (!Objects.equals(totalNotionalQuantityLeg2, _that.getTotalNotionalQuantityLeg2())) return false;
			if (!Objects.equals(notionalLeg2, _that.getNotionalLeg2())) return false;
			if (!Objects.equals(notionalCurrency2, _that.getNotionalCurrency2())) return false;
			if (!ListEquals.listEquals(notionalAmountScheduleLeg2, _that.getNotionalAmountScheduleLeg2())) return false;
			if (!ListEquals.listEquals(notionalQuantityScheduleLeg2, _that.getNotionalQuantityScheduleLeg2())) return false;
			if (!ListEquals.listEquals(otherPayment, _that.getOtherPayment())) return false;
			if (!Objects.equals(fixedRateOfLeg1, _that.getFixedRateOfLeg1())) return false;
			if (!Objects.equals(fixedRateDayCountConventionLeg1, _that.getFixedRateDayCountConventionLeg1())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodLeg1, _that.getFixedRatePaymentFrequencyPeriodLeg1())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodMultiplierLeg1, _that.getFixedRatePaymentFrequencyPeriodMultiplierLeg1())) return false;
			if (!Objects.equals(identifierOfFloatingRateOfLeg1, _that.getIdentifierOfFloatingRateOfLeg1())) return false;
			if (!Objects.equals(indicatorOfTheFloatingRateOfLeg1, _that.getIndicatorOfTheFloatingRateOfLeg1())) return false;
			if (!Objects.equals(nameOfTheFloatingRateOfLeg1, _that.getNameOfTheFloatingRateOfLeg1())) return false;
			if (!Objects.equals(floatingRateDayCountConventionOfLeg1, _that.getFloatingRateDayCountConventionOfLeg1())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodOfLeg1, _that.getFloatingRatePaymentFrequencyPeriodOfLeg1())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodMultiplierOfLeg1, _that.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1())) return false;
			if (!Objects.equals(floatingRateReferencePeriodOfLeg1TimePeriod, _that.getFloatingRateReferencePeriodOfLeg1TimePeriod())) return false;
			if (!Objects.equals(floatingRateReferencePeriodOfLeg1Multiplier, _that.getFloatingRateReferencePeriodOfLeg1Multiplier())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodOfLeg1, _that.getFloatingRateResetFrequencyPeriodOfLeg1())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodMultiplierLeg1, _that.getFloatingRateResetFrequencyPeriodMultiplierLeg1())) return false;
			if (!Objects.equals(spreadOfLeg1, _that.getSpreadOfLeg1())) return false;
			if (!Objects.equals(fixedRateOfLeg2, _that.getFixedRateOfLeg2())) return false;
			if (!Objects.equals(fixedRateDayCountConventionLeg2, _that.getFixedRateDayCountConventionLeg2())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodLeg2, _that.getFixedRatePaymentFrequencyPeriodLeg2())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodMultiplierLeg2, _that.getFixedRatePaymentFrequencyPeriodMultiplierLeg2())) return false;
			if (!Objects.equals(identifierOfFloatingRateOfLeg2, _that.getIdentifierOfFloatingRateOfLeg2())) return false;
			if (!Objects.equals(indicatorOfTheFloatingRateOfLeg2, _that.getIndicatorOfTheFloatingRateOfLeg2())) return false;
			if (!Objects.equals(nameOfTheFloatingRateOfLeg2, _that.getNameOfTheFloatingRateOfLeg2())) return false;
			if (!Objects.equals(floatingRateDayCountConventionOfLeg2, _that.getFloatingRateDayCountConventionOfLeg2())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodOfLeg2, _that.getFloatingRatePaymentFrequencyPeriodOfLeg2())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodMultiplierOfLeg2, _that.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2())) return false;
			if (!Objects.equals(floatingRateReferencePeriodOfLeg2TimePeriod, _that.getFloatingRateReferencePeriodOfLeg2TimePeriod())) return false;
			if (!Objects.equals(floatingRateReferencePeriodOfLeg2Multiplier, _that.getFloatingRateReferencePeriodOfLeg2Multiplier())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodOfLeg2, _that.getFloatingRateResetFrequencyPeriodOfLeg2())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodMultiplierLeg2, _that.getFloatingRateResetFrequencyPeriodMultiplierLeg2())) return false;
			if (!Objects.equals(spreadOfLeg2, _that.getSpreadOfLeg2())) return false;
			if (!Objects.equals(packageTransactionSpread, _that.getPackageTransactionSpread())) return false;
			if (!Objects.equals(exchangeRate, _that.getExchangeRate())) return false;
			if (!Objects.equals(forwardExchangeRate, _that.getForwardExchangeRate())) return false;
			if (!Objects.equals(exchangeRateBasis, _that.getExchangeRateBasis())) return false;
			if (!Objects.equals(baseProduct, _that.getBaseProduct())) return false;
			if (!Objects.equals(subProduct, _that.getSubProduct())) return false;
			if (!Objects.equals(furtherSubProduct, _that.getFurtherSubProduct())) return false;
			if (!ListEquals.listEquals(deliveryPoint, _that.getDeliveryPoint())) return false;
			if (!Objects.equals(interconnectionPoint, _that.getInterconnectionPoint())) return false;
			if (!Objects.equals(loadType, _that.getLoadType())) return false;
			if (!ListEquals.listEquals(deliveryReport, _that.getDeliveryReport())) return false;
			if (!Objects.equals(optionType, _that.getOptionType())) return false;
			if (!Objects.equals(optionStyle, _that.getOptionStyle())) return false;
			if (!Objects.equals(strikePrice, _that.getStrikePrice())) return false;
			if (!ListEquals.listEquals(strikePriceSchedule, _that.getStrikePriceSchedule())) return false;
			if (!Objects.equals(optionPremiumAmount, _that.getOptionPremiumAmount())) return false;
			if (!Objects.equals(optionPremiumCurrency, _that.getOptionPremiumCurrency())) return false;
			if (!Objects.equals(optionPremiumPaymentDate, _that.getOptionPremiumPaymentDate())) return false;
			if (!Objects.equals(maturityDateOfTheUnderlying, _that.getMaturityDateOfTheUnderlying())) return false;
			if (!Objects.equals(seniority, _that.getSeniority())) return false;
			if (!Objects.equals(referenceEntity, _that.getReferenceEntity())) return false;
			if (!Objects.equals(series, _that.getSeries())) return false;
			if (!Objects.equals(version, _that.getVersion())) return false;
			if (!Objects.equals(indexFactor, _that.getIndexFactor())) return false;
			if (!Objects.equals(tranche, _that.getTranche())) return false;
			if (!Objects.equals(cdSIndexAttachmentPoint, _that.getCdSIndexAttachmentPoint())) return false;
			if (!Objects.equals(cdSIndexDetachmentPoint, _that.getCdSIndexDetachmentPoint())) return false;
			if (!Objects.equals(collateralPortfolioIndicator, _that.getCollateralPortfolioIndicator())) return false;
			if (!Objects.equals(collateralPortfolioCode, _that.getCollateralPortfolioCode())) return false;
			if (!Objects.equals(actionType, _that.getActionType())) return false;
			if (!Objects.equals(eventType, _that.getEventType())) return false;
			if (!Objects.equals(eventDate, _that.getEventDate())) return false;
			if (!Objects.equals(level, _that.getLevel())) return false;
			if (!Objects.equals(nonReportable, _that.getNonReportable())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (reportingTimestamp != null ? reportingTimestamp.hashCode() : 0);
			_result = 31 * _result + (reportSubmittingEntityID != null ? reportSubmittingEntityID.hashCode() : 0);
			_result = 31 * _result + (entityResponsibleForReporting != null ? entityResponsibleForReporting.hashCode() : 0);
			_result = 31 * _result + (counterparty1 != null ? counterparty1.hashCode() : 0);
			_result = 31 * _result + (natureOfCounterparty1 != null ? natureOfCounterparty1.hashCode() : 0);
			_result = 31 * _result + (corporateSectorOfTheCounterparty1 != null ? corporateSectorOfTheCounterparty1.hashCode() : 0);
			_result = 31 * _result + (clearingThresholdOfCounterparty1 != null ? clearingThresholdOfCounterparty1.hashCode() : 0);
			_result = 31 * _result + (counterparty2 != null ? counterparty2.hashCode() : 0);
			_result = 31 * _result + (countryOfTheCounterParty2 != null ? countryOfTheCounterParty2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (natureOfCounterparty2 != null ? natureOfCounterparty2.hashCode() : 0);
			_result = 31 * _result + (corporateSectorOfTheCounterparty2 != null ? corporateSectorOfTheCounterparty2.hashCode() : 0);
			_result = 31 * _result + (counterparty2IdentifierType != null ? counterparty2IdentifierType.hashCode() : 0);
			_result = 31 * _result + (clearingMember != null ? clearingMember.hashCode() : 0);
			_result = 31 * _result + (clearingThresholdOfCounterparty2 != null ? clearingThresholdOfCounterparty2.hashCode() : 0);
			_result = 31 * _result + (reportingObligationOfTheCounterparty2 != null ? reportingObligationOfTheCounterparty2.hashCode() : 0);
			_result = 31 * _result + (brokerID != null ? brokerID.hashCode() : 0);
			_result = 31 * _result + (direction != null ? direction.hashCode() : 0);
			_result = 31 * _result + (directionOfLeg1 != null ? directionOfLeg1.hashCode() : 0);
			_result = 31 * _result + (directionOfLeg2 != null ? directionOfLeg2.hashCode() : 0);
			_result = 31 * _result + (directlyLinkedToCommercialActivityOrTreasuryFinancing != null ? directlyLinkedToCommercialActivityOrTreasuryFinancing.hashCode() : 0);
			_result = 31 * _result + (uti != null ? uti.hashCode() : 0);
			_result = 31 * _result + (utiProprietary != null ? utiProprietary.hashCode() : 0);
			_result = 31 * _result + (reportTrackingNumber != null ? reportTrackingNumber.hashCode() : 0);
			_result = 31 * _result + (priorUti != null ? priorUti.hashCode() : 0);
			_result = 31 * _result + (priorUtiProprietary != null ? priorUtiProprietary.hashCode() : 0);
			_result = 31 * _result + (ptrrId != null ? ptrrId.hashCode() : 0);
			_result = 31 * _result + (packageIdentifier != null ? packageIdentifier.hashCode() : 0);
			_result = 31 * _result + (subsequentPositionUti != null ? subsequentPositionUti.hashCode() : 0);
			_result = 31 * _result + (isin != null ? isin.hashCode() : 0);
			_result = 31 * _result + (upi != null ? upi.hashCode() : 0);
			_result = 31 * _result + (productClassification != null ? productClassification.hashCode() : 0);
			_result = 31 * _result + (contractType != null ? contractType.hashCode() : 0);
			_result = 31 * _result + (assetClass != null ? assetClass.hashCode() : 0);
			_result = 31 * _result + (isCrypto != null ? isCrypto.hashCode() : 0);
			_result = 31 * _result + (underlyingIdentificationType != null ? underlyingIdentificationType.hashCode() : 0);
			_result = 31 * _result + (underlyingIdentification != null ? underlyingIdentification.hashCode() : 0);
			_result = 31 * _result + (nameOfTheUnderlyingIndex != null ? nameOfTheUnderlyingIndex.hashCode() : 0);
			_result = 31 * _result + (indicatorOfTheUnderlyingIndex != null ? indicatorOfTheUnderlyingIndex.hashCode() : 0);
			_result = 31 * _result + (basketConstituents != null ? basketConstituents.hashCode() : 0);
			_result = 31 * _result + (customBasketCode != null ? customBasketCode.hashCode() : 0);
			_result = 31 * _result + (settlementCurrency1 != null ? settlementCurrency1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (settlementCurrency2 != null ? settlementCurrency2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (valuationAmount != null ? valuationAmount.hashCode() : 0);
			_result = 31 * _result + (valuationCurrency != null ? valuationCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (valuationMethod != null ? valuationMethod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (delta != null ? delta.hashCode() : 0);
			_result = 31 * _result + (valuationTimestamp != null ? valuationTimestamp.hashCode() : 0);
			_result = 31 * _result + (confirmationTimestamp != null ? confirmationTimestamp.hashCode() : 0);
			_result = 31 * _result + (confirmed != null ? confirmed.hashCode() : 0);
			_result = 31 * _result + (clearingObligation != null ? clearingObligation.hashCode() : 0);
			_result = 31 * _result + (cleared != null ? cleared.hashCode() : 0);
			_result = 31 * _result + (clearingTimestamp != null ? clearingTimestamp.hashCode() : 0);
			_result = 31 * _result + (centralCounterparty != null ? centralCounterparty.hashCode() : 0);
			_result = 31 * _result + (masterAgreementType != null ? masterAgreementType.hashCode() : 0);
			_result = 31 * _result + (otherMasterAgreementType != null ? otherMasterAgreementType.hashCode() : 0);
			_result = 31 * _result + (masterAgreementVersion != null ? masterAgreementVersion.hashCode() : 0);
			_result = 31 * _result + (intragroup != null ? intragroup.hashCode() : 0);
			_result = 31 * _result + (ptrr != null ? ptrr.hashCode() : 0);
			_result = 31 * _result + (typeOfPTRRTechnique != null ? typeOfPTRRTechnique.hashCode() : 0);
			_result = 31 * _result + (ptrrServiceProvider != null ? ptrrServiceProvider.hashCode() : 0);
			_result = 31 * _result + (venueOfExecution != null ? venueOfExecution.hashCode() : 0);
			_result = 31 * _result + (executionTimestamp != null ? executionTimestamp.hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (expirationDate != null ? expirationDate.hashCode() : 0);
			_result = 31 * _result + (earlyTerminationDate != null ? earlyTerminationDate.hashCode() : 0);
			_result = 31 * _result + (finalContractualSettlementDate != null ? finalContractualSettlementDate.hashCode() : 0);
			_result = 31 * _result + (deliveryType != null ? deliveryType.hashCode() : 0);
			_result = 31 * _result + (price != null ? price.hashCode() : 0);
			_result = 31 * _result + (priceSchedule != null ? priceSchedule.hashCode() : 0);
			_result = 31 * _result + (packageTransactionPrice != null ? packageTransactionPrice.hashCode() : 0);
			_result = 31 * _result + (notionalLeg1 != null ? notionalLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalCurrency1 != null ? notionalCurrency1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalAmountScheduleLeg1 != null ? notionalAmountScheduleLeg1.hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityLeg1 != null ? totalNotionalQuantityLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalQuantityScheduleLeg1 != null ? notionalQuantityScheduleLeg1.hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityLeg2 != null ? totalNotionalQuantityLeg2.hashCode() : 0);
			_result = 31 * _result + (notionalLeg2 != null ? notionalLeg2.hashCode() : 0);
			_result = 31 * _result + (notionalCurrency2 != null ? notionalCurrency2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalAmountScheduleLeg2 != null ? notionalAmountScheduleLeg2.hashCode() : 0);
			_result = 31 * _result + (notionalQuantityScheduleLeg2 != null ? notionalQuantityScheduleLeg2.hashCode() : 0);
			_result = 31 * _result + (otherPayment != null ? otherPayment.hashCode() : 0);
			_result = 31 * _result + (fixedRateOfLeg1 != null ? fixedRateOfLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRateDayCountConventionLeg1 != null ? fixedRateDayCountConventionLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodLeg1 != null ? fixedRatePaymentFrequencyPeriodLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodMultiplierLeg1 != null ? fixedRatePaymentFrequencyPeriodMultiplierLeg1.hashCode() : 0);
			_result = 31 * _result + (identifierOfFloatingRateOfLeg1 != null ? identifierOfFloatingRateOfLeg1.hashCode() : 0);
			_result = 31 * _result + (indicatorOfTheFloatingRateOfLeg1 != null ? indicatorOfTheFloatingRateOfLeg1.hashCode() : 0);
			_result = 31 * _result + (nameOfTheFloatingRateOfLeg1 != null ? nameOfTheFloatingRateOfLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRateDayCountConventionOfLeg1 != null ? floatingRateDayCountConventionOfLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodOfLeg1 != null ? floatingRatePaymentFrequencyPeriodOfLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodMultiplierOfLeg1 != null ? floatingRatePaymentFrequencyPeriodMultiplierOfLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRateReferencePeriodOfLeg1TimePeriod != null ? floatingRateReferencePeriodOfLeg1TimePeriod.hashCode() : 0);
			_result = 31 * _result + (floatingRateReferencePeriodOfLeg1Multiplier != null ? floatingRateReferencePeriodOfLeg1Multiplier.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodOfLeg1 != null ? floatingRateResetFrequencyPeriodOfLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodMultiplierLeg1 != null ? floatingRateResetFrequencyPeriodMultiplierLeg1.hashCode() : 0);
			_result = 31 * _result + (spreadOfLeg1 != null ? spreadOfLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRateOfLeg2 != null ? fixedRateOfLeg2.hashCode() : 0);
			_result = 31 * _result + (fixedRateDayCountConventionLeg2 != null ? fixedRateDayCountConventionLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodLeg2 != null ? fixedRatePaymentFrequencyPeriodLeg2.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodMultiplierLeg2 != null ? fixedRatePaymentFrequencyPeriodMultiplierLeg2.hashCode() : 0);
			_result = 31 * _result + (identifierOfFloatingRateOfLeg2 != null ? identifierOfFloatingRateOfLeg2.hashCode() : 0);
			_result = 31 * _result + (indicatorOfTheFloatingRateOfLeg2 != null ? indicatorOfTheFloatingRateOfLeg2.hashCode() : 0);
			_result = 31 * _result + (nameOfTheFloatingRateOfLeg2 != null ? nameOfTheFloatingRateOfLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRateDayCountConventionOfLeg2 != null ? floatingRateDayCountConventionOfLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodOfLeg2 != null ? floatingRatePaymentFrequencyPeriodOfLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 != null ? floatingRatePaymentFrequencyPeriodMultiplierOfLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRateReferencePeriodOfLeg2TimePeriod != null ? floatingRateReferencePeriodOfLeg2TimePeriod.hashCode() : 0);
			_result = 31 * _result + (floatingRateReferencePeriodOfLeg2Multiplier != null ? floatingRateReferencePeriodOfLeg2Multiplier.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodOfLeg2 != null ? floatingRateResetFrequencyPeriodOfLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodMultiplierLeg2 != null ? floatingRateResetFrequencyPeriodMultiplierLeg2.hashCode() : 0);
			_result = 31 * _result + (spreadOfLeg2 != null ? spreadOfLeg2.hashCode() : 0);
			_result = 31 * _result + (packageTransactionSpread != null ? packageTransactionSpread.hashCode() : 0);
			_result = 31 * _result + (exchangeRate != null ? exchangeRate.hashCode() : 0);
			_result = 31 * _result + (forwardExchangeRate != null ? forwardExchangeRate.hashCode() : 0);
			_result = 31 * _result + (exchangeRateBasis != null ? exchangeRateBasis.hashCode() : 0);
			_result = 31 * _result + (baseProduct != null ? baseProduct.hashCode() : 0);
			_result = 31 * _result + (subProduct != null ? subProduct.hashCode() : 0);
			_result = 31 * _result + (furtherSubProduct != null ? furtherSubProduct.hashCode() : 0);
			_result = 31 * _result + (deliveryPoint != null ? deliveryPoint.hashCode() : 0);
			_result = 31 * _result + (interconnectionPoint != null ? interconnectionPoint.hashCode() : 0);
			_result = 31 * _result + (loadType != null ? loadType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (deliveryReport != null ? deliveryReport.hashCode() : 0);
			_result = 31 * _result + (optionType != null ? optionType.hashCode() : 0);
			_result = 31 * _result + (optionStyle != null ? optionStyle.hashCode() : 0);
			_result = 31 * _result + (strikePrice != null ? strikePrice.hashCode() : 0);
			_result = 31 * _result + (strikePriceSchedule != null ? strikePriceSchedule.hashCode() : 0);
			_result = 31 * _result + (optionPremiumAmount != null ? optionPremiumAmount.hashCode() : 0);
			_result = 31 * _result + (optionPremiumCurrency != null ? optionPremiumCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (optionPremiumPaymentDate != null ? optionPremiumPaymentDate.hashCode() : 0);
			_result = 31 * _result + (maturityDateOfTheUnderlying != null ? maturityDateOfTheUnderlying.hashCode() : 0);
			_result = 31 * _result + (seniority != null ? seniority.hashCode() : 0);
			_result = 31 * _result + (referenceEntity != null ? referenceEntity.hashCode() : 0);
			_result = 31 * _result + (series != null ? series.hashCode() : 0);
			_result = 31 * _result + (version != null ? version.hashCode() : 0);
			_result = 31 * _result + (indexFactor != null ? indexFactor.hashCode() : 0);
			_result = 31 * _result + (tranche != null ? tranche.hashCode() : 0);
			_result = 31 * _result + (cdSIndexAttachmentPoint != null ? cdSIndexAttachmentPoint.hashCode() : 0);
			_result = 31 * _result + (cdSIndexDetachmentPoint != null ? cdSIndexDetachmentPoint.hashCode() : 0);
			_result = 31 * _result + (collateralPortfolioIndicator != null ? collateralPortfolioIndicator.hashCode() : 0);
			_result = 31 * _result + (collateralPortfolioCode != null ? collateralPortfolioCode.hashCode() : 0);
			_result = 31 * _result + (actionType != null ? actionType.hashCode() : 0);
			_result = 31 * _result + (eventType != null ? eventType.hashCode() : 0);
			_result = 31 * _result + (eventDate != null ? eventDate.hashCode() : 0);
			_result = 31 * _result + (level != null ? level.hashCode() : 0);
			_result = 31 * _result + (nonReportable != null ? nonReportable.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ESMAEMIRTransactionReport {" +
				"reportingTimestamp=" + this.reportingTimestamp + ", " +
				"reportSubmittingEntityID=" + this.reportSubmittingEntityID + ", " +
				"entityResponsibleForReporting=" + this.entityResponsibleForReporting + ", " +
				"counterparty1=" + this.counterparty1 + ", " +
				"natureOfCounterparty1=" + this.natureOfCounterparty1 + ", " +
				"corporateSectorOfTheCounterparty1=" + this.corporateSectorOfTheCounterparty1 + ", " +
				"clearingThresholdOfCounterparty1=" + this.clearingThresholdOfCounterparty1 + ", " +
				"counterparty2=" + this.counterparty2 + ", " +
				"countryOfTheCounterParty2=" + this.countryOfTheCounterParty2 + ", " +
				"natureOfCounterparty2=" + this.natureOfCounterparty2 + ", " +
				"corporateSectorOfTheCounterparty2=" + this.corporateSectorOfTheCounterparty2 + ", " +
				"counterparty2IdentifierType=" + this.counterparty2IdentifierType + ", " +
				"clearingMember=" + this.clearingMember + ", " +
				"clearingThresholdOfCounterparty2=" + this.clearingThresholdOfCounterparty2 + ", " +
				"reportingObligationOfTheCounterparty2=" + this.reportingObligationOfTheCounterparty2 + ", " +
				"brokerID=" + this.brokerID + ", " +
				"direction=" + this.direction + ", " +
				"directionOfLeg1=" + this.directionOfLeg1 + ", " +
				"directionOfLeg2=" + this.directionOfLeg2 + ", " +
				"directlyLinkedToCommercialActivityOrTreasuryFinancing=" + this.directlyLinkedToCommercialActivityOrTreasuryFinancing + ", " +
				"uti=" + this.uti + ", " +
				"utiProprietary=" + this.utiProprietary + ", " +
				"reportTrackingNumber=" + this.reportTrackingNumber + ", " +
				"priorUti=" + this.priorUti + ", " +
				"priorUtiProprietary=" + this.priorUtiProprietary + ", " +
				"ptrrId=" + this.ptrrId + ", " +
				"packageIdentifier=" + this.packageIdentifier + ", " +
				"subsequentPositionUti=" + this.subsequentPositionUti + ", " +
				"isin=" + this.isin + ", " +
				"upi=" + this.upi + ", " +
				"productClassification=" + this.productClassification + ", " +
				"contractType=" + this.contractType + ", " +
				"assetClass=" + this.assetClass + ", " +
				"isCrypto=" + this.isCrypto + ", " +
				"underlyingIdentificationType=" + this.underlyingIdentificationType + ", " +
				"underlyingIdentification=" + this.underlyingIdentification + ", " +
				"nameOfTheUnderlyingIndex=" + this.nameOfTheUnderlyingIndex + ", " +
				"indicatorOfTheUnderlyingIndex=" + this.indicatorOfTheUnderlyingIndex + ", " +
				"basketConstituents=" + this.basketConstituents + ", " +
				"customBasketCode=" + this.customBasketCode + ", " +
				"settlementCurrency1=" + this.settlementCurrency1 + ", " +
				"settlementCurrency2=" + this.settlementCurrency2 + ", " +
				"valuationAmount=" + this.valuationAmount + ", " +
				"valuationCurrency=" + this.valuationCurrency + ", " +
				"valuationMethod=" + this.valuationMethod + ", " +
				"delta=" + this.delta + ", " +
				"valuationTimestamp=" + this.valuationTimestamp + ", " +
				"confirmationTimestamp=" + this.confirmationTimestamp + ", " +
				"confirmed=" + this.confirmed + ", " +
				"clearingObligation=" + this.clearingObligation + ", " +
				"cleared=" + this.cleared + ", " +
				"clearingTimestamp=" + this.clearingTimestamp + ", " +
				"centralCounterparty=" + this.centralCounterparty + ", " +
				"masterAgreementType=" + this.masterAgreementType + ", " +
				"otherMasterAgreementType=" + this.otherMasterAgreementType + ", " +
				"masterAgreementVersion=" + this.masterAgreementVersion + ", " +
				"intragroup=" + this.intragroup + ", " +
				"ptrr=" + this.ptrr + ", " +
				"typeOfPTRRTechnique=" + this.typeOfPTRRTechnique + ", " +
				"ptrrServiceProvider=" + this.ptrrServiceProvider + ", " +
				"venueOfExecution=" + this.venueOfExecution + ", " +
				"executionTimestamp=" + this.executionTimestamp + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"expirationDate=" + this.expirationDate + ", " +
				"earlyTerminationDate=" + this.earlyTerminationDate + ", " +
				"finalContractualSettlementDate=" + this.finalContractualSettlementDate + ", " +
				"deliveryType=" + this.deliveryType + ", " +
				"price=" + this.price + ", " +
				"priceSchedule=" + this.priceSchedule + ", " +
				"packageTransactionPrice=" + this.packageTransactionPrice + ", " +
				"notionalLeg1=" + this.notionalLeg1 + ", " +
				"notionalCurrency1=" + this.notionalCurrency1 + ", " +
				"notionalAmountScheduleLeg1=" + this.notionalAmountScheduleLeg1 + ", " +
				"totalNotionalQuantityLeg1=" + this.totalNotionalQuantityLeg1 + ", " +
				"notionalQuantityScheduleLeg1=" + this.notionalQuantityScheduleLeg1 + ", " +
				"totalNotionalQuantityLeg2=" + this.totalNotionalQuantityLeg2 + ", " +
				"notionalLeg2=" + this.notionalLeg2 + ", " +
				"notionalCurrency2=" + this.notionalCurrency2 + ", " +
				"notionalAmountScheduleLeg2=" + this.notionalAmountScheduleLeg2 + ", " +
				"notionalQuantityScheduleLeg2=" + this.notionalQuantityScheduleLeg2 + ", " +
				"otherPayment=" + this.otherPayment + ", " +
				"fixedRateOfLeg1=" + this.fixedRateOfLeg1 + ", " +
				"fixedRateDayCountConventionLeg1=" + this.fixedRateDayCountConventionLeg1 + ", " +
				"fixedRatePaymentFrequencyPeriodLeg1=" + this.fixedRatePaymentFrequencyPeriodLeg1 + ", " +
				"fixedRatePaymentFrequencyPeriodMultiplierLeg1=" + this.fixedRatePaymentFrequencyPeriodMultiplierLeg1 + ", " +
				"identifierOfFloatingRateOfLeg1=" + this.identifierOfFloatingRateOfLeg1 + ", " +
				"indicatorOfTheFloatingRateOfLeg1=" + this.indicatorOfTheFloatingRateOfLeg1 + ", " +
				"nameOfTheFloatingRateOfLeg1=" + this.nameOfTheFloatingRateOfLeg1 + ", " +
				"floatingRateDayCountConventionOfLeg1=" + this.floatingRateDayCountConventionOfLeg1 + ", " +
				"floatingRatePaymentFrequencyPeriodOfLeg1=" + this.floatingRatePaymentFrequencyPeriodOfLeg1 + ", " +
				"floatingRatePaymentFrequencyPeriodMultiplierOfLeg1=" + this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg1 + ", " +
				"floatingRateReferencePeriodOfLeg1TimePeriod=" + this.floatingRateReferencePeriodOfLeg1TimePeriod + ", " +
				"floatingRateReferencePeriodOfLeg1Multiplier=" + this.floatingRateReferencePeriodOfLeg1Multiplier + ", " +
				"floatingRateResetFrequencyPeriodOfLeg1=" + this.floatingRateResetFrequencyPeriodOfLeg1 + ", " +
				"floatingRateResetFrequencyPeriodMultiplierLeg1=" + this.floatingRateResetFrequencyPeriodMultiplierLeg1 + ", " +
				"spreadOfLeg1=" + this.spreadOfLeg1 + ", " +
				"fixedRateOfLeg2=" + this.fixedRateOfLeg2 + ", " +
				"fixedRateDayCountConventionLeg2=" + this.fixedRateDayCountConventionLeg2 + ", " +
				"fixedRatePaymentFrequencyPeriodLeg2=" + this.fixedRatePaymentFrequencyPeriodLeg2 + ", " +
				"fixedRatePaymentFrequencyPeriodMultiplierLeg2=" + this.fixedRatePaymentFrequencyPeriodMultiplierLeg2 + ", " +
				"identifierOfFloatingRateOfLeg2=" + this.identifierOfFloatingRateOfLeg2 + ", " +
				"indicatorOfTheFloatingRateOfLeg2=" + this.indicatorOfTheFloatingRateOfLeg2 + ", " +
				"nameOfTheFloatingRateOfLeg2=" + this.nameOfTheFloatingRateOfLeg2 + ", " +
				"floatingRateDayCountConventionOfLeg2=" + this.floatingRateDayCountConventionOfLeg2 + ", " +
				"floatingRatePaymentFrequencyPeriodOfLeg2=" + this.floatingRatePaymentFrequencyPeriodOfLeg2 + ", " +
				"floatingRatePaymentFrequencyPeriodMultiplierOfLeg2=" + this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 + ", " +
				"floatingRateReferencePeriodOfLeg2TimePeriod=" + this.floatingRateReferencePeriodOfLeg2TimePeriod + ", " +
				"floatingRateReferencePeriodOfLeg2Multiplier=" + this.floatingRateReferencePeriodOfLeg2Multiplier + ", " +
				"floatingRateResetFrequencyPeriodOfLeg2=" + this.floatingRateResetFrequencyPeriodOfLeg2 + ", " +
				"floatingRateResetFrequencyPeriodMultiplierLeg2=" + this.floatingRateResetFrequencyPeriodMultiplierLeg2 + ", " +
				"spreadOfLeg2=" + this.spreadOfLeg2 + ", " +
				"packageTransactionSpread=" + this.packageTransactionSpread + ", " +
				"exchangeRate=" + this.exchangeRate + ", " +
				"forwardExchangeRate=" + this.forwardExchangeRate + ", " +
				"exchangeRateBasis=" + this.exchangeRateBasis + ", " +
				"baseProduct=" + this.baseProduct + ", " +
				"subProduct=" + this.subProduct + ", " +
				"furtherSubProduct=" + this.furtherSubProduct + ", " +
				"deliveryPoint=" + this.deliveryPoint + ", " +
				"interconnectionPoint=" + this.interconnectionPoint + ", " +
				"loadType=" + this.loadType + ", " +
				"deliveryReport=" + this.deliveryReport + ", " +
				"optionType=" + this.optionType + ", " +
				"optionStyle=" + this.optionStyle + ", " +
				"strikePrice=" + this.strikePrice + ", " +
				"strikePriceSchedule=" + this.strikePriceSchedule + ", " +
				"optionPremiumAmount=" + this.optionPremiumAmount + ", " +
				"optionPremiumCurrency=" + this.optionPremiumCurrency + ", " +
				"optionPremiumPaymentDate=" + this.optionPremiumPaymentDate + ", " +
				"maturityDateOfTheUnderlying=" + this.maturityDateOfTheUnderlying + ", " +
				"seniority=" + this.seniority + ", " +
				"referenceEntity=" + this.referenceEntity + ", " +
				"series=" + this.series + ", " +
				"version=" + this.version + ", " +
				"indexFactor=" + this.indexFactor + ", " +
				"tranche=" + this.tranche + ", " +
				"cdSIndexAttachmentPoint=" + this.cdSIndexAttachmentPoint + ", " +
				"cdSIndexDetachmentPoint=" + this.cdSIndexDetachmentPoint + ", " +
				"collateralPortfolioIndicator=" + this.collateralPortfolioIndicator + ", " +
				"collateralPortfolioCode=" + this.collateralPortfolioCode + ", " +
				"actionType=" + this.actionType + ", " +
				"eventType=" + this.eventType + ", " +
				"eventDate=" + this.eventDate + ", " +
				"level=" + this.level + ", " +
				"nonReportable=" + this.nonReportable +
			'}';
		}
	}

	/*********************** Builder Implementation of ESMAEMIRTransactionReport  ***********************/
	class ESMAEMIRTransactionReportBuilderImpl implements ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder {
	
		protected ZonedDateTime reportingTimestamp;
		protected String reportSubmittingEntityID;
		protected String entityResponsibleForReporting;
		protected String counterparty1;
		protected String natureOfCounterparty1;
		protected String corporateSectorOfTheCounterparty1;
		protected Boolean clearingThresholdOfCounterparty1;
		protected String counterparty2;
		protected ISOCountryCodeEnum countryOfTheCounterParty2;
		protected String natureOfCounterparty2;
		protected String corporateSectorOfTheCounterparty2;
		protected Boolean counterparty2IdentifierType;
		protected String clearingMember;
		protected Boolean clearingThresholdOfCounterparty2;
		protected Boolean reportingObligationOfTheCounterparty2;
		protected String brokerID;
		protected String direction;
		protected String directionOfLeg1;
		protected String directionOfLeg2;
		protected Boolean directlyLinkedToCommercialActivityOrTreasuryFinancing;
		protected String uti;
		protected String utiProprietary;
		protected String reportTrackingNumber;
		protected String priorUti;
		protected String priorUtiProprietary;
		protected String ptrrId;
		protected String packageIdentifier;
		protected String subsequentPositionUti;
		protected String isin;
		protected String upi;
		protected String productClassification;
		protected String contractType;
		protected String assetClass;
		protected Boolean isCrypto;
		protected String underlyingIdentificationType;
		protected String underlyingIdentification;
		protected List<String> nameOfTheUnderlyingIndex = new ArrayList<>();
		protected List<String> indicatorOfTheUnderlyingIndex = new ArrayList<>();
		protected List<BasketConstituentsReport.BasketConstituentsReportBuilder> basketConstituents = new ArrayList<>();
		protected String customBasketCode;
		protected ISOCurrencyCodeEnum settlementCurrency1;
		protected ISOCurrencyCodeEnum settlementCurrency2;
		protected BigDecimal valuationAmount;
		protected ISOCurrencyCodeEnum valuationCurrency;
		protected ValuationType1Code valuationMethod;
		protected BigDecimal delta;
		protected ZonedDateTime valuationTimestamp;
		protected ZonedDateTime confirmationTimestamp;
		protected String confirmed;
		protected String clearingObligation;
		protected String cleared;
		protected ZonedDateTime clearingTimestamp;
		protected String centralCounterparty;
		protected String masterAgreementType;
		protected String otherMasterAgreementType;
		protected Integer masterAgreementVersion;
		protected Boolean intragroup;
		protected Boolean ptrr;
		protected String typeOfPTRRTechnique;
		protected String ptrrServiceProvider;
		protected String venueOfExecution;
		protected ZonedDateTime executionTimestamp;
		protected Date effectiveDate;
		protected Date expirationDate;
		protected Date earlyTerminationDate;
		protected Date finalContractualSettlementDate;
		protected String deliveryType;
		protected PriceAndNotation.PriceAndNotationBuilder price;
		protected List<PriceScheduleReport.PriceScheduleReportBuilder> priceSchedule = new ArrayList<>();
		protected PackageTransactionPrice.PackageTransactionPriceBuilder packageTransactionPrice;
		protected BigDecimal notionalLeg1;
		protected ISOCurrencyCodeEnum notionalCurrency1;
		protected List<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> notionalAmountScheduleLeg1 = new ArrayList<>();
		protected BigDecimal totalNotionalQuantityLeg1;
		protected List<NotionalQuantityScheduleLeg1Report.NotionalQuantityScheduleLeg1ReportBuilder> notionalQuantityScheduleLeg1 = new ArrayList<>();
		protected BigDecimal totalNotionalQuantityLeg2;
		protected BigDecimal notionalLeg2;
		protected ISOCurrencyCodeEnum notionalCurrency2;
		protected List<NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> notionalAmountScheduleLeg2 = new ArrayList<>();
		protected List<NotionalQuantityScheduleLeg2Report.NotionalQuantityScheduleLeg2ReportBuilder> notionalQuantityScheduleLeg2 = new ArrayList<>();
		protected List<OtherPaymentReport.OtherPaymentReportBuilder> otherPayment = new ArrayList<>();
		protected BigDecimal fixedRateOfLeg1;
		protected InterestComputationMethod4Code fixedRateDayCountConventionLeg1;
		protected String fixedRatePaymentFrequencyPeriodLeg1;
		protected Integer fixedRatePaymentFrequencyPeriodMultiplierLeg1;
		protected String identifierOfFloatingRateOfLeg1;
		protected String indicatorOfTheFloatingRateOfLeg1;
		protected String nameOfTheFloatingRateOfLeg1;
		protected InterestComputationMethod4Code floatingRateDayCountConventionOfLeg1;
		protected Frequency13Code__1 floatingRatePaymentFrequencyPeriodOfLeg1;
		protected Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg1;
		protected String floatingRateReferencePeriodOfLeg1TimePeriod;
		protected Integer floatingRateReferencePeriodOfLeg1Multiplier;
		protected String floatingRateResetFrequencyPeriodOfLeg1;
		protected Integer floatingRateResetFrequencyPeriodMultiplierLeg1;
		protected SpreadOfLeg1.SpreadOfLeg1Builder spreadOfLeg1;
		protected BigDecimal fixedRateOfLeg2;
		protected InterestComputationMethod4Code fixedRateDayCountConventionLeg2;
		protected String fixedRatePaymentFrequencyPeriodLeg2;
		protected Integer fixedRatePaymentFrequencyPeriodMultiplierLeg2;
		protected String identifierOfFloatingRateOfLeg2;
		protected String indicatorOfTheFloatingRateOfLeg2;
		protected String nameOfTheFloatingRateOfLeg2;
		protected InterestComputationMethod4Code floatingRateDayCountConventionOfLeg2;
		protected String floatingRatePaymentFrequencyPeriodOfLeg2;
		protected Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg2;
		protected String floatingRateReferencePeriodOfLeg2TimePeriod;
		protected Integer floatingRateReferencePeriodOfLeg2Multiplier;
		protected String floatingRateResetFrequencyPeriodOfLeg2;
		protected Integer floatingRateResetFrequencyPeriodMultiplierLeg2;
		protected SpreadOfLeg2.SpreadOfLeg2Builder spreadOfLeg2;
		protected PackageTransactionSpread.PackageTransactionSpreadBuilder packageTransactionSpread;
		protected BigDecimal exchangeRate;
		protected BigDecimal forwardExchangeRate;
		protected String exchangeRateBasis;
		protected String baseProduct;
		protected String subProduct;
		protected String furtherSubProduct;
		protected List<String> deliveryPoint = new ArrayList<>();
		protected String interconnectionPoint;
		protected EnergyLoadType1Code loadType;
		protected List<DeliveryReport.DeliveryReportBuilder> deliveryReport = new ArrayList<>();
		protected String optionType;
		protected String optionStyle;
		protected StrikePriceAndNotation.StrikePriceAndNotationBuilder strikePrice;
		protected List<StrikePriceScheduleReport.StrikePriceScheduleReportBuilder> strikePriceSchedule = new ArrayList<>();
		protected BigDecimal optionPremiumAmount;
		protected ISOCurrencyCodeEnum optionPremiumCurrency;
		protected Date optionPremiumPaymentDate;
		protected Date maturityDateOfTheUnderlying;
		protected String seniority;
		protected String referenceEntity;
		protected Integer series;
		protected Integer version;
		protected BigDecimal indexFactor;
		protected Boolean tranche;
		protected BigDecimal cdSIndexAttachmentPoint;
		protected BigDecimal cdSIndexDetachmentPoint;
		protected Boolean collateralPortfolioIndicator;
		protected String collateralPortfolioCode;
		protected String actionType;
		protected String eventType;
		protected Date eventDate;
		protected String level;
		protected ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder nonReportable;
	
		public ESMAEMIRTransactionReportBuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("reportingTimestamp")
		public ZonedDateTime getReportingTimestamp() {
			return reportingTimestamp;
		}
		
		@Override
		@RosettaAttribute("reportSubmittingEntityID")
		public String getReportSubmittingEntityID() {
			return reportSubmittingEntityID;
		}
		
		@Override
		@RosettaAttribute("entityResponsibleForReporting")
		public String getEntityResponsibleForReporting() {
			return entityResponsibleForReporting;
		}
		
		@Override
		@RosettaAttribute("counterparty1")
		public String getCounterparty1() {
			return counterparty1;
		}
		
		@Override
		@RosettaAttribute("natureOfCounterparty1")
		public String getNatureOfCounterparty1() {
			return natureOfCounterparty1;
		}
		
		@Override
		@RosettaAttribute("corporateSectorOfTheCounterparty1")
		public String getCorporateSectorOfTheCounterparty1() {
			return corporateSectorOfTheCounterparty1;
		}
		
		@Override
		@RosettaAttribute("clearingThresholdOfCounterparty1")
		public Boolean getClearingThresholdOfCounterparty1() {
			return clearingThresholdOfCounterparty1;
		}
		
		@Override
		@RosettaAttribute("counterparty2")
		public String getCounterparty2() {
			return counterparty2;
		}
		
		@Override
		@RosettaAttribute("countryOfTheCounterParty2")
		public ISOCountryCodeEnum getCountryOfTheCounterParty2() {
			return countryOfTheCounterParty2;
		}
		
		@Override
		@RosettaAttribute("natureOfCounterparty2")
		public String getNatureOfCounterparty2() {
			return natureOfCounterparty2;
		}
		
		@Override
		@RosettaAttribute("corporateSectorOfTheCounterparty2")
		public String getCorporateSectorOfTheCounterparty2() {
			return corporateSectorOfTheCounterparty2;
		}
		
		@Override
		@RosettaAttribute("counterparty2IdentifierType")
		public Boolean getCounterparty2IdentifierType() {
			return counterparty2IdentifierType;
		}
		
		@Override
		@RosettaAttribute("clearingMember")
		public String getClearingMember() {
			return clearingMember;
		}
		
		@Override
		@RosettaAttribute("clearingThresholdOfCounterparty2")
		public Boolean getClearingThresholdOfCounterparty2() {
			return clearingThresholdOfCounterparty2;
		}
		
		@Override
		@RosettaAttribute("reportingObligationOfTheCounterparty2")
		public Boolean getReportingObligationOfTheCounterparty2() {
			return reportingObligationOfTheCounterparty2;
		}
		
		@Override
		@RosettaAttribute("brokerID")
		public String getBrokerID() {
			return brokerID;
		}
		
		@Override
		@RosettaAttribute("direction")
		public String getDirection() {
			return direction;
		}
		
		@Override
		@RosettaAttribute("directionOfLeg1")
		public String getDirectionOfLeg1() {
			return directionOfLeg1;
		}
		
		@Override
		@RosettaAttribute("directionOfLeg2")
		public String getDirectionOfLeg2() {
			return directionOfLeg2;
		}
		
		@Override
		@RosettaAttribute("directlyLinkedToCommercialActivityOrTreasuryFinancing")
		public Boolean getDirectlyLinkedToCommercialActivityOrTreasuryFinancing() {
			return directlyLinkedToCommercialActivityOrTreasuryFinancing;
		}
		
		@Override
		@RosettaAttribute("uti")
		public String getUti() {
			return uti;
		}
		
		@Override
		@RosettaAttribute("utiProprietary")
		public String getUtiProprietary() {
			return utiProprietary;
		}
		
		@Override
		@RosettaAttribute("reportTrackingNumber")
		public String getReportTrackingNumber() {
			return reportTrackingNumber;
		}
		
		@Override
		@RosettaAttribute("priorUti")
		public String getPriorUti() {
			return priorUti;
		}
		
		@Override
		@RosettaAttribute("priorUtiProprietary")
		public String getPriorUtiProprietary() {
			return priorUtiProprietary;
		}
		
		@Override
		@RosettaAttribute("ptrrId")
		public String getPtrrId() {
			return ptrrId;
		}
		
		@Override
		@RosettaAttribute("packageIdentifier")
		public String getPackageIdentifier() {
			return packageIdentifier;
		}
		
		@Override
		@RosettaAttribute("subsequentPositionUti")
		public String getSubsequentPositionUti() {
			return subsequentPositionUti;
		}
		
		@Override
		@RosettaAttribute("isin")
		public String getIsin() {
			return isin;
		}
		
		@Override
		@RosettaAttribute("upi")
		public String getUpi() {
			return upi;
		}
		
		@Override
		@RosettaAttribute("productClassification")
		public String getProductClassification() {
			return productClassification;
		}
		
		@Override
		@RosettaAttribute("contractType")
		public String getContractType() {
			return contractType;
		}
		
		@Override
		@RosettaAttribute("assetClass")
		public String getAssetClass() {
			return assetClass;
		}
		
		@Override
		@RosettaAttribute("isCrypto")
		public Boolean getIsCrypto() {
			return isCrypto;
		}
		
		@Override
		@RosettaAttribute("underlyingIdentificationType")
		public String getUnderlyingIdentificationType() {
			return underlyingIdentificationType;
		}
		
		@Override
		@RosettaAttribute("underlyingIdentification")
		public String getUnderlyingIdentification() {
			return underlyingIdentification;
		}
		
		@Override
		@RosettaAttribute("nameOfTheUnderlyingIndex")
		public List<String> getNameOfTheUnderlyingIndex() {
			return nameOfTheUnderlyingIndex;
		}
		
		@Override
		@RosettaAttribute("indicatorOfTheUnderlyingIndex")
		public List<String> getIndicatorOfTheUnderlyingIndex() {
			return indicatorOfTheUnderlyingIndex;
		}
		
		@Override
		@RosettaAttribute("basketConstituents")
		public List<? extends BasketConstituentsReport.BasketConstituentsReportBuilder> getBasketConstituents() {
			return basketConstituents;
		}
		
		public BasketConstituentsReport.BasketConstituentsReportBuilder getOrCreateBasketConstituents(int _index) {
		
			if (basketConstituents==null) {
				this.basketConstituents = new ArrayList<>();
			}
			BasketConstituentsReport.BasketConstituentsReportBuilder result;
			return getIndex(basketConstituents, _index, () -> {
						BasketConstituentsReport.BasketConstituentsReportBuilder newBasketConstituents = BasketConstituentsReport.builder();
						return newBasketConstituents;
					});
		}
		
		@Override
		@RosettaAttribute("customBasketCode")
		public String getCustomBasketCode() {
			return customBasketCode;
		}
		
		@Override
		@RosettaAttribute("settlementCurrency1")
		public ISOCurrencyCodeEnum getSettlementCurrency1() {
			return settlementCurrency1;
		}
		
		@Override
		@RosettaAttribute("settlementCurrency2")
		public ISOCurrencyCodeEnum getSettlementCurrency2() {
			return settlementCurrency2;
		}
		
		@Override
		@RosettaAttribute("valuationAmount")
		public BigDecimal getValuationAmount() {
			return valuationAmount;
		}
		
		@Override
		@RosettaAttribute("valuationCurrency")
		public ISOCurrencyCodeEnum getValuationCurrency() {
			return valuationCurrency;
		}
		
		@Override
		@RosettaAttribute("valuationMethod")
		public ValuationType1Code getValuationMethod() {
			return valuationMethod;
		}
		
		@Override
		@RosettaAttribute("delta")
		public BigDecimal getDelta() {
			return delta;
		}
		
		@Override
		@RosettaAttribute("valuationTimestamp")
		public ZonedDateTime getValuationTimestamp() {
			return valuationTimestamp;
		}
		
		@Override
		@RosettaAttribute("confirmationTimestamp")
		public ZonedDateTime getConfirmationTimestamp() {
			return confirmationTimestamp;
		}
		
		@Override
		@RosettaAttribute("confirmed")
		public String getConfirmed() {
			return confirmed;
		}
		
		@Override
		@RosettaAttribute("clearingObligation")
		public String getClearingObligation() {
			return clearingObligation;
		}
		
		@Override
		@RosettaAttribute("cleared")
		public String getCleared() {
			return cleared;
		}
		
		@Override
		@RosettaAttribute("clearingTimestamp")
		public ZonedDateTime getClearingTimestamp() {
			return clearingTimestamp;
		}
		
		@Override
		@RosettaAttribute("centralCounterparty")
		public String getCentralCounterparty() {
			return centralCounterparty;
		}
		
		@Override
		@RosettaAttribute("masterAgreementType")
		public String getMasterAgreementType() {
			return masterAgreementType;
		}
		
		@Override
		@RosettaAttribute("otherMasterAgreementType")
		public String getOtherMasterAgreementType() {
			return otherMasterAgreementType;
		}
		
		@Override
		@RosettaAttribute("masterAgreementVersion")
		public Integer getMasterAgreementVersion() {
			return masterAgreementVersion;
		}
		
		@Override
		@RosettaAttribute("intragroup")
		public Boolean getIntragroup() {
			return intragroup;
		}
		
		@Override
		@RosettaAttribute("ptrr")
		public Boolean getPtrr() {
			return ptrr;
		}
		
		@Override
		@RosettaAttribute("typeOfPTRRTechnique")
		public String getTypeOfPTRRTechnique() {
			return typeOfPTRRTechnique;
		}
		
		@Override
		@RosettaAttribute("ptrrServiceProvider")
		public String getPtrrServiceProvider() {
			return ptrrServiceProvider;
		}
		
		@Override
		@RosettaAttribute("venueOfExecution")
		public String getVenueOfExecution() {
			return venueOfExecution;
		}
		
		@Override
		@RosettaAttribute("executionTimestamp")
		public ZonedDateTime getExecutionTimestamp() {
			return executionTimestamp;
		}
		
		@Override
		@RosettaAttribute("effectiveDate")
		public Date getEffectiveDate() {
			return effectiveDate;
		}
		
		@Override
		@RosettaAttribute("expirationDate")
		public Date getExpirationDate() {
			return expirationDate;
		}
		
		@Override
		@RosettaAttribute("earlyTerminationDate")
		public Date getEarlyTerminationDate() {
			return earlyTerminationDate;
		}
		
		@Override
		@RosettaAttribute("finalContractualSettlementDate")
		public Date getFinalContractualSettlementDate() {
			return finalContractualSettlementDate;
		}
		
		@Override
		@RosettaAttribute("deliveryType")
		public String getDeliveryType() {
			return deliveryType;
		}
		
		@Override
		@RosettaAttribute("price")
		public PriceAndNotation.PriceAndNotationBuilder getPrice() {
			return price;
		}
		
		@Override
		public PriceAndNotation.PriceAndNotationBuilder getOrCreatePrice() {
			PriceAndNotation.PriceAndNotationBuilder result;
			if (price!=null) {
				result = price;
			}
			else {
				result = price = PriceAndNotation.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("priceSchedule")
		public List<? extends PriceScheduleReport.PriceScheduleReportBuilder> getPriceSchedule() {
			return priceSchedule;
		}
		
		public PriceScheduleReport.PriceScheduleReportBuilder getOrCreatePriceSchedule(int _index) {
		
			if (priceSchedule==null) {
				this.priceSchedule = new ArrayList<>();
			}
			PriceScheduleReport.PriceScheduleReportBuilder result;
			return getIndex(priceSchedule, _index, () -> {
						PriceScheduleReport.PriceScheduleReportBuilder newPriceSchedule = PriceScheduleReport.builder();
						return newPriceSchedule;
					});
		}
		
		@Override
		@RosettaAttribute("packageTransactionPrice")
		public PackageTransactionPrice.PackageTransactionPriceBuilder getPackageTransactionPrice() {
			return packageTransactionPrice;
		}
		
		@Override
		public PackageTransactionPrice.PackageTransactionPriceBuilder getOrCreatePackageTransactionPrice() {
			PackageTransactionPrice.PackageTransactionPriceBuilder result;
			if (packageTransactionPrice!=null) {
				result = packageTransactionPrice;
			}
			else {
				result = packageTransactionPrice = PackageTransactionPrice.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("notionalLeg1")
		public BigDecimal getNotionalLeg1() {
			return notionalLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalCurrency1")
		public ISOCurrencyCodeEnum getNotionalCurrency1() {
			return notionalCurrency1;
		}
		
		@Override
		@RosettaAttribute("notionalAmountScheduleLeg1")
		public List<? extends NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder> getNotionalAmountScheduleLeg1() {
			return notionalAmountScheduleLeg1;
		}
		
		public NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder getOrCreateNotionalAmountScheduleLeg1(int _index) {
		
			if (notionalAmountScheduleLeg1==null) {
				this.notionalAmountScheduleLeg1 = new ArrayList<>();
			}
			NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder result;
			return getIndex(notionalAmountScheduleLeg1, _index, () -> {
						NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder newNotionalAmountScheduleLeg1 = NotionalAmountScheduleLeg1Report.builder();
						return newNotionalAmountScheduleLeg1;
					});
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg1")
		public BigDecimal getTotalNotionalQuantityLeg1() {
			return totalNotionalQuantityLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityScheduleLeg1")
		public List<? extends NotionalQuantityScheduleLeg1Report.NotionalQuantityScheduleLeg1ReportBuilder> getNotionalQuantityScheduleLeg1() {
			return notionalQuantityScheduleLeg1;
		}
		
		public NotionalQuantityScheduleLeg1Report.NotionalQuantityScheduleLeg1ReportBuilder getOrCreateNotionalQuantityScheduleLeg1(int _index) {
		
			if (notionalQuantityScheduleLeg1==null) {
				this.notionalQuantityScheduleLeg1 = new ArrayList<>();
			}
			NotionalQuantityScheduleLeg1Report.NotionalQuantityScheduleLeg1ReportBuilder result;
			return getIndex(notionalQuantityScheduleLeg1, _index, () -> {
						NotionalQuantityScheduleLeg1Report.NotionalQuantityScheduleLeg1ReportBuilder newNotionalQuantityScheduleLeg1 = NotionalQuantityScheduleLeg1Report.builder();
						return newNotionalQuantityScheduleLeg1;
					});
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg2")
		public BigDecimal getTotalNotionalQuantityLeg2() {
			return totalNotionalQuantityLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalLeg2")
		public BigDecimal getNotionalLeg2() {
			return notionalLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalCurrency2")
		public ISOCurrencyCodeEnum getNotionalCurrency2() {
			return notionalCurrency2;
		}
		
		@Override
		@RosettaAttribute("notionalAmountScheduleLeg2")
		public List<? extends NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder> getNotionalAmountScheduleLeg2() {
			return notionalAmountScheduleLeg2;
		}
		
		public NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder getOrCreateNotionalAmountScheduleLeg2(int _index) {
		
			if (notionalAmountScheduleLeg2==null) {
				this.notionalAmountScheduleLeg2 = new ArrayList<>();
			}
			NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder result;
			return getIndex(notionalAmountScheduleLeg2, _index, () -> {
						NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder newNotionalAmountScheduleLeg2 = NotionalAmountScheduleLeg2Report.builder();
						return newNotionalAmountScheduleLeg2;
					});
		}
		
		@Override
		@RosettaAttribute("notionalQuantityScheduleLeg2")
		public List<? extends NotionalQuantityScheduleLeg2Report.NotionalQuantityScheduleLeg2ReportBuilder> getNotionalQuantityScheduleLeg2() {
			return notionalQuantityScheduleLeg2;
		}
		
		public NotionalQuantityScheduleLeg2Report.NotionalQuantityScheduleLeg2ReportBuilder getOrCreateNotionalQuantityScheduleLeg2(int _index) {
		
			if (notionalQuantityScheduleLeg2==null) {
				this.notionalQuantityScheduleLeg2 = new ArrayList<>();
			}
			NotionalQuantityScheduleLeg2Report.NotionalQuantityScheduleLeg2ReportBuilder result;
			return getIndex(notionalQuantityScheduleLeg2, _index, () -> {
						NotionalQuantityScheduleLeg2Report.NotionalQuantityScheduleLeg2ReportBuilder newNotionalQuantityScheduleLeg2 = NotionalQuantityScheduleLeg2Report.builder();
						return newNotionalQuantityScheduleLeg2;
					});
		}
		
		@Override
		@RosettaAttribute("otherPayment")
		public List<? extends OtherPaymentReport.OtherPaymentReportBuilder> getOtherPayment() {
			return otherPayment;
		}
		
		public OtherPaymentReport.OtherPaymentReportBuilder getOrCreateOtherPayment(int _index) {
		
			if (otherPayment==null) {
				this.otherPayment = new ArrayList<>();
			}
			OtherPaymentReport.OtherPaymentReportBuilder result;
			return getIndex(otherPayment, _index, () -> {
						OtherPaymentReport.OtherPaymentReportBuilder newOtherPayment = OtherPaymentReport.builder();
						return newOtherPayment;
					});
		}
		
		@Override
		@RosettaAttribute("fixedRateOfLeg1")
		public BigDecimal getFixedRateOfLeg1() {
			return fixedRateOfLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRateDayCountConventionLeg1")
		public InterestComputationMethod4Code getFixedRateDayCountConventionLeg1() {
			return fixedRateDayCountConventionLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodLeg1")
		public String getFixedRatePaymentFrequencyPeriodLeg1() {
			return fixedRatePaymentFrequencyPeriodLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierLeg1")
		public Integer getFixedRatePaymentFrequencyPeriodMultiplierLeg1() {
			return fixedRatePaymentFrequencyPeriodMultiplierLeg1;
		}
		
		@Override
		@RosettaAttribute("identifierOfFloatingRateOfLeg1")
		public String getIdentifierOfFloatingRateOfLeg1() {
			return identifierOfFloatingRateOfLeg1;
		}
		
		@Override
		@RosettaAttribute("indicatorOfTheFloatingRateOfLeg1")
		public String getIndicatorOfTheFloatingRateOfLeg1() {
			return indicatorOfTheFloatingRateOfLeg1;
		}
		
		@Override
		@RosettaAttribute("nameOfTheFloatingRateOfLeg1")
		public String getNameOfTheFloatingRateOfLeg1() {
			return nameOfTheFloatingRateOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateDayCountConventionOfLeg1")
		public InterestComputationMethod4Code getFloatingRateDayCountConventionOfLeg1() {
			return floatingRateDayCountConventionOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodOfLeg1")
		public Frequency13Code__1 getFloatingRatePaymentFrequencyPeriodOfLeg1() {
			return floatingRatePaymentFrequencyPeriodOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierOfLeg1")
		public Integer getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1() {
			return floatingRatePaymentFrequencyPeriodMultiplierOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg1TimePeriod")
		public String getFloatingRateReferencePeriodOfLeg1TimePeriod() {
			return floatingRateReferencePeriodOfLeg1TimePeriod;
		}
		
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg1Multiplier")
		public Integer getFloatingRateReferencePeriodOfLeg1Multiplier() {
			return floatingRateReferencePeriodOfLeg1Multiplier;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodOfLeg1")
		public String getFloatingRateResetFrequencyPeriodOfLeg1() {
			return floatingRateResetFrequencyPeriodOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierLeg1")
		public Integer getFloatingRateResetFrequencyPeriodMultiplierLeg1() {
			return floatingRateResetFrequencyPeriodMultiplierLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadOfLeg1")
		public SpreadOfLeg1.SpreadOfLeg1Builder getSpreadOfLeg1() {
			return spreadOfLeg1;
		}
		
		@Override
		public SpreadOfLeg1.SpreadOfLeg1Builder getOrCreateSpreadOfLeg1() {
			SpreadOfLeg1.SpreadOfLeg1Builder result;
			if (spreadOfLeg1!=null) {
				result = spreadOfLeg1;
			}
			else {
				result = spreadOfLeg1 = SpreadOfLeg1.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("fixedRateOfLeg2")
		public BigDecimal getFixedRateOfLeg2() {
			return fixedRateOfLeg2;
		}
		
		@Override
		@RosettaAttribute("fixedRateDayCountConventionLeg2")
		public InterestComputationMethod4Code getFixedRateDayCountConventionLeg2() {
			return fixedRateDayCountConventionLeg2;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodLeg2")
		public String getFixedRatePaymentFrequencyPeriodLeg2() {
			return fixedRatePaymentFrequencyPeriodLeg2;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierLeg2")
		public Integer getFixedRatePaymentFrequencyPeriodMultiplierLeg2() {
			return fixedRatePaymentFrequencyPeriodMultiplierLeg2;
		}
		
		@Override
		@RosettaAttribute("identifierOfFloatingRateOfLeg2")
		public String getIdentifierOfFloatingRateOfLeg2() {
			return identifierOfFloatingRateOfLeg2;
		}
		
		@Override
		@RosettaAttribute("indicatorOfTheFloatingRateOfLeg2")
		public String getIndicatorOfTheFloatingRateOfLeg2() {
			return indicatorOfTheFloatingRateOfLeg2;
		}
		
		@Override
		@RosettaAttribute("nameOfTheFloatingRateOfLeg2")
		public String getNameOfTheFloatingRateOfLeg2() {
			return nameOfTheFloatingRateOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateDayCountConventionOfLeg2")
		public InterestComputationMethod4Code getFloatingRateDayCountConventionOfLeg2() {
			return floatingRateDayCountConventionOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodOfLeg2")
		public String getFloatingRatePaymentFrequencyPeriodOfLeg2() {
			return floatingRatePaymentFrequencyPeriodOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierOfLeg2")
		public Integer getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2() {
			return floatingRatePaymentFrequencyPeriodMultiplierOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg2TimePeriod")
		public String getFloatingRateReferencePeriodOfLeg2TimePeriod() {
			return floatingRateReferencePeriodOfLeg2TimePeriod;
		}
		
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg2Multiplier")
		public Integer getFloatingRateReferencePeriodOfLeg2Multiplier() {
			return floatingRateReferencePeriodOfLeg2Multiplier;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodOfLeg2")
		public String getFloatingRateResetFrequencyPeriodOfLeg2() {
			return floatingRateResetFrequencyPeriodOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierLeg2")
		public Integer getFloatingRateResetFrequencyPeriodMultiplierLeg2() {
			return floatingRateResetFrequencyPeriodMultiplierLeg2;
		}
		
		@Override
		@RosettaAttribute("spreadOfLeg2")
		public SpreadOfLeg2.SpreadOfLeg2Builder getSpreadOfLeg2() {
			return spreadOfLeg2;
		}
		
		@Override
		public SpreadOfLeg2.SpreadOfLeg2Builder getOrCreateSpreadOfLeg2() {
			SpreadOfLeg2.SpreadOfLeg2Builder result;
			if (spreadOfLeg2!=null) {
				result = spreadOfLeg2;
			}
			else {
				result = spreadOfLeg2 = SpreadOfLeg2.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("packageTransactionSpread")
		public PackageTransactionSpread.PackageTransactionSpreadBuilder getPackageTransactionSpread() {
			return packageTransactionSpread;
		}
		
		@Override
		public PackageTransactionSpread.PackageTransactionSpreadBuilder getOrCreatePackageTransactionSpread() {
			PackageTransactionSpread.PackageTransactionSpreadBuilder result;
			if (packageTransactionSpread!=null) {
				result = packageTransactionSpread;
			}
			else {
				result = packageTransactionSpread = PackageTransactionSpread.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("exchangeRate")
		public BigDecimal getExchangeRate() {
			return exchangeRate;
		}
		
		@Override
		@RosettaAttribute("forwardExchangeRate")
		public BigDecimal getForwardExchangeRate() {
			return forwardExchangeRate;
		}
		
		@Override
		@RosettaAttribute("exchangeRateBasis")
		public String getExchangeRateBasis() {
			return exchangeRateBasis;
		}
		
		@Override
		@RosettaAttribute("baseProduct")
		public String getBaseProduct() {
			return baseProduct;
		}
		
		@Override
		@RosettaAttribute("subProduct")
		public String getSubProduct() {
			return subProduct;
		}
		
		@Override
		@RosettaAttribute("furtherSubProduct")
		public String getFurtherSubProduct() {
			return furtherSubProduct;
		}
		
		@Override
		@RosettaAttribute("deliveryPoint")
		public List<String> getDeliveryPoint() {
			return deliveryPoint;
		}
		
		@Override
		@RosettaAttribute("interconnectionPoint")
		public String getInterconnectionPoint() {
			return interconnectionPoint;
		}
		
		@Override
		@RosettaAttribute("loadType")
		public EnergyLoadType1Code getLoadType() {
			return loadType;
		}
		
		@Override
		@RosettaAttribute("deliveryReport")
		public List<? extends DeliveryReport.DeliveryReportBuilder> getDeliveryReport() {
			return deliveryReport;
		}
		
		public DeliveryReport.DeliveryReportBuilder getOrCreateDeliveryReport(int _index) {
		
			if (deliveryReport==null) {
				this.deliveryReport = new ArrayList<>();
			}
			DeliveryReport.DeliveryReportBuilder result;
			return getIndex(deliveryReport, _index, () -> {
						DeliveryReport.DeliveryReportBuilder newDeliveryReport = DeliveryReport.builder();
						return newDeliveryReport;
					});
		}
		
		@Override
		@RosettaAttribute("optionType")
		public String getOptionType() {
			return optionType;
		}
		
		@Override
		@RosettaAttribute("optionStyle")
		public String getOptionStyle() {
			return optionStyle;
		}
		
		@Override
		@RosettaAttribute("strikePrice")
		public StrikePriceAndNotation.StrikePriceAndNotationBuilder getStrikePrice() {
			return strikePrice;
		}
		
		@Override
		public StrikePriceAndNotation.StrikePriceAndNotationBuilder getOrCreateStrikePrice() {
			StrikePriceAndNotation.StrikePriceAndNotationBuilder result;
			if (strikePrice!=null) {
				result = strikePrice;
			}
			else {
				result = strikePrice = StrikePriceAndNotation.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("strikePriceSchedule")
		public List<? extends StrikePriceScheduleReport.StrikePriceScheduleReportBuilder> getStrikePriceSchedule() {
			return strikePriceSchedule;
		}
		
		public StrikePriceScheduleReport.StrikePriceScheduleReportBuilder getOrCreateStrikePriceSchedule(int _index) {
		
			if (strikePriceSchedule==null) {
				this.strikePriceSchedule = new ArrayList<>();
			}
			StrikePriceScheduleReport.StrikePriceScheduleReportBuilder result;
			return getIndex(strikePriceSchedule, _index, () -> {
						StrikePriceScheduleReport.StrikePriceScheduleReportBuilder newStrikePriceSchedule = StrikePriceScheduleReport.builder();
						return newStrikePriceSchedule;
					});
		}
		
		@Override
		@RosettaAttribute("optionPremiumAmount")
		public BigDecimal getOptionPremiumAmount() {
			return optionPremiumAmount;
		}
		
		@Override
		@RosettaAttribute("optionPremiumCurrency")
		public ISOCurrencyCodeEnum getOptionPremiumCurrency() {
			return optionPremiumCurrency;
		}
		
		@Override
		@RosettaAttribute("optionPremiumPaymentDate")
		public Date getOptionPremiumPaymentDate() {
			return optionPremiumPaymentDate;
		}
		
		@Override
		@RosettaAttribute("maturityDateOfTheUnderlying")
		public Date getMaturityDateOfTheUnderlying() {
			return maturityDateOfTheUnderlying;
		}
		
		@Override
		@RosettaAttribute("seniority")
		public String getSeniority() {
			return seniority;
		}
		
		@Override
		@RosettaAttribute("referenceEntity")
		public String getReferenceEntity() {
			return referenceEntity;
		}
		
		@Override
		@RosettaAttribute("series")
		public Integer getSeries() {
			return series;
		}
		
		@Override
		@RosettaAttribute("version")
		public Integer getVersion() {
			return version;
		}
		
		@Override
		@RosettaAttribute("indexFactor")
		public BigDecimal getIndexFactor() {
			return indexFactor;
		}
		
		@Override
		@RosettaAttribute("tranche")
		public Boolean getTranche() {
			return tranche;
		}
		
		@Override
		@RosettaAttribute("cdSIndexAttachmentPoint")
		public BigDecimal getCdSIndexAttachmentPoint() {
			return cdSIndexAttachmentPoint;
		}
		
		@Override
		@RosettaAttribute("cdSIndexDetachmentPoint")
		public BigDecimal getCdSIndexDetachmentPoint() {
			return cdSIndexDetachmentPoint;
		}
		
		@Override
		@RosettaAttribute("collateralPortfolioIndicator")
		public Boolean getCollateralPortfolioIndicator() {
			return collateralPortfolioIndicator;
		}
		
		@Override
		@RosettaAttribute("collateralPortfolioCode")
		public String getCollateralPortfolioCode() {
			return collateralPortfolioCode;
		}
		
		@Override
		@RosettaAttribute("actionType")
		public String getActionType() {
			return actionType;
		}
		
		@Override
		@RosettaAttribute("eventType")
		public String getEventType() {
			return eventType;
		}
		
		@Override
		@RosettaAttribute("eventDate")
		public Date getEventDate() {
			return eventDate;
		}
		
		@Override
		@RosettaAttribute("level")
		public String getLevel() {
			return level;
		}
		
		@Override
		@RosettaAttribute("nonReportable")
		public ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder getNonReportable() {
			return nonReportable;
		}
		
		@Override
		public ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder getOrCreateNonReportable() {
			ESMAEMIRNonReportable.ESMAEMIRNonReportableBuilder result;
			if (nonReportable!=null) {
				result = nonReportable;
			}
			else {
				result = nonReportable = ESMAEMIRNonReportable.builder();
			}
			
			return result;
		}
	
		@Override
		@RosettaAttribute("reportingTimestamp")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setReportingTimestamp(ZonedDateTime reportingTimestamp) {
			this.reportingTimestamp = reportingTimestamp==null?null:reportingTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("reportSubmittingEntityID")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setReportSubmittingEntityID(String reportSubmittingEntityID) {
			this.reportSubmittingEntityID = reportSubmittingEntityID==null?null:reportSubmittingEntityID;
			return this;
		}
		@Override
		@RosettaAttribute("entityResponsibleForReporting")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setEntityResponsibleForReporting(String entityResponsibleForReporting) {
			this.entityResponsibleForReporting = entityResponsibleForReporting==null?null:entityResponsibleForReporting;
			return this;
		}
		@Override
		@RosettaAttribute("counterparty1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCounterparty1(String counterparty1) {
			this.counterparty1 = counterparty1==null?null:counterparty1;
			return this;
		}
		@Override
		@RosettaAttribute("natureOfCounterparty1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNatureOfCounterparty1(String natureOfCounterparty1) {
			this.natureOfCounterparty1 = natureOfCounterparty1==null?null:natureOfCounterparty1;
			return this;
		}
		@Override
		@RosettaAttribute("corporateSectorOfTheCounterparty1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCorporateSectorOfTheCounterparty1(String corporateSectorOfTheCounterparty1) {
			this.corporateSectorOfTheCounterparty1 = corporateSectorOfTheCounterparty1==null?null:corporateSectorOfTheCounterparty1;
			return this;
		}
		@Override
		@RosettaAttribute("clearingThresholdOfCounterparty1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setClearingThresholdOfCounterparty1(Boolean clearingThresholdOfCounterparty1) {
			this.clearingThresholdOfCounterparty1 = clearingThresholdOfCounterparty1==null?null:clearingThresholdOfCounterparty1;
			return this;
		}
		@Override
		@RosettaAttribute("counterparty2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCounterparty2(String counterparty2) {
			this.counterparty2 = counterparty2==null?null:counterparty2;
			return this;
		}
		@Override
		@RosettaAttribute("countryOfTheCounterParty2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCountryOfTheCounterParty2(ISOCountryCodeEnum countryOfTheCounterParty2) {
			this.countryOfTheCounterParty2 = countryOfTheCounterParty2==null?null:countryOfTheCounterParty2;
			return this;
		}
		@Override
		@RosettaAttribute("natureOfCounterparty2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNatureOfCounterparty2(String natureOfCounterparty2) {
			this.natureOfCounterparty2 = natureOfCounterparty2==null?null:natureOfCounterparty2;
			return this;
		}
		@Override
		@RosettaAttribute("corporateSectorOfTheCounterparty2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCorporateSectorOfTheCounterparty2(String corporateSectorOfTheCounterparty2) {
			this.corporateSectorOfTheCounterparty2 = corporateSectorOfTheCounterparty2==null?null:corporateSectorOfTheCounterparty2;
			return this;
		}
		@Override
		@RosettaAttribute("counterparty2IdentifierType")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCounterparty2IdentifierType(Boolean counterparty2IdentifierType) {
			this.counterparty2IdentifierType = counterparty2IdentifierType==null?null:counterparty2IdentifierType;
			return this;
		}
		@Override
		@RosettaAttribute("clearingMember")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setClearingMember(String clearingMember) {
			this.clearingMember = clearingMember==null?null:clearingMember;
			return this;
		}
		@Override
		@RosettaAttribute("clearingThresholdOfCounterparty2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setClearingThresholdOfCounterparty2(Boolean clearingThresholdOfCounterparty2) {
			this.clearingThresholdOfCounterparty2 = clearingThresholdOfCounterparty2==null?null:clearingThresholdOfCounterparty2;
			return this;
		}
		@Override
		@RosettaAttribute("reportingObligationOfTheCounterparty2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setReportingObligationOfTheCounterparty2(Boolean reportingObligationOfTheCounterparty2) {
			this.reportingObligationOfTheCounterparty2 = reportingObligationOfTheCounterparty2==null?null:reportingObligationOfTheCounterparty2;
			return this;
		}
		@Override
		@RosettaAttribute("brokerID")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setBrokerID(String brokerID) {
			this.brokerID = brokerID==null?null:brokerID;
			return this;
		}
		@Override
		@RosettaAttribute("direction")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDirection(String direction) {
			this.direction = direction==null?null:direction;
			return this;
		}
		@Override
		@RosettaAttribute("directionOfLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDirectionOfLeg1(String directionOfLeg1) {
			this.directionOfLeg1 = directionOfLeg1==null?null:directionOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("directionOfLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDirectionOfLeg2(String directionOfLeg2) {
			this.directionOfLeg2 = directionOfLeg2==null?null:directionOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("directlyLinkedToCommercialActivityOrTreasuryFinancing")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDirectlyLinkedToCommercialActivityOrTreasuryFinancing(Boolean directlyLinkedToCommercialActivityOrTreasuryFinancing) {
			this.directlyLinkedToCommercialActivityOrTreasuryFinancing = directlyLinkedToCommercialActivityOrTreasuryFinancing==null?null:directlyLinkedToCommercialActivityOrTreasuryFinancing;
			return this;
		}
		@Override
		@RosettaAttribute("uti")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setUti(String uti) {
			this.uti = uti==null?null:uti;
			return this;
		}
		@Override
		@RosettaAttribute("utiProprietary")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setUtiProprietary(String utiProprietary) {
			this.utiProprietary = utiProprietary==null?null:utiProprietary;
			return this;
		}
		@Override
		@RosettaAttribute("reportTrackingNumber")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setReportTrackingNumber(String reportTrackingNumber) {
			this.reportTrackingNumber = reportTrackingNumber==null?null:reportTrackingNumber;
			return this;
		}
		@Override
		@RosettaAttribute("priorUti")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPriorUti(String priorUti) {
			this.priorUti = priorUti==null?null:priorUti;
			return this;
		}
		@Override
		@RosettaAttribute("priorUtiProprietary")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPriorUtiProprietary(String priorUtiProprietary) {
			this.priorUtiProprietary = priorUtiProprietary==null?null:priorUtiProprietary;
			return this;
		}
		@Override
		@RosettaAttribute("ptrrId")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPtrrId(String ptrrId) {
			this.ptrrId = ptrrId==null?null:ptrrId;
			return this;
		}
		@Override
		@RosettaAttribute("packageIdentifier")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPackageIdentifier(String packageIdentifier) {
			this.packageIdentifier = packageIdentifier==null?null:packageIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("subsequentPositionUti")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSubsequentPositionUti(String subsequentPositionUti) {
			this.subsequentPositionUti = subsequentPositionUti==null?null:subsequentPositionUti;
			return this;
		}
		@Override
		@RosettaAttribute("isin")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIsin(String isin) {
			this.isin = isin==null?null:isin;
			return this;
		}
		@Override
		@RosettaAttribute("upi")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setUpi(String upi) {
			this.upi = upi==null?null:upi;
			return this;
		}
		@Override
		@RosettaAttribute("productClassification")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setProductClassification(String productClassification) {
			this.productClassification = productClassification==null?null:productClassification;
			return this;
		}
		@Override
		@RosettaAttribute("contractType")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setContractType(String contractType) {
			this.contractType = contractType==null?null:contractType;
			return this;
		}
		@Override
		@RosettaAttribute("assetClass")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setAssetClass(String assetClass) {
			this.assetClass = assetClass==null?null:assetClass;
			return this;
		}
		@Override
		@RosettaAttribute("isCrypto")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIsCrypto(Boolean isCrypto) {
			this.isCrypto = isCrypto==null?null:isCrypto;
			return this;
		}
		@Override
		@RosettaAttribute("underlyingIdentificationType")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setUnderlyingIdentificationType(String underlyingIdentificationType) {
			this.underlyingIdentificationType = underlyingIdentificationType==null?null:underlyingIdentificationType;
			return this;
		}
		@Override
		@RosettaAttribute("underlyingIdentification")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setUnderlyingIdentification(String underlyingIdentification) {
			this.underlyingIdentification = underlyingIdentification==null?null:underlyingIdentification;
			return this;
		}
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNameOfTheUnderlyingIndex(String nameOfTheUnderlyingIndex) {
			if (nameOfTheUnderlyingIndex!=null) this.nameOfTheUnderlyingIndex.add(nameOfTheUnderlyingIndex);
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNameOfTheUnderlyingIndex(String nameOfTheUnderlyingIndex, int _idx) {
			getIndex(this.nameOfTheUnderlyingIndex, _idx, () -> nameOfTheUnderlyingIndex);
			return this;
		}
		@Override 
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNameOfTheUnderlyingIndex(List<? extends String> nameOfTheUnderlyingIndexs) {
			if (nameOfTheUnderlyingIndexs != null) {
				for (String toAdd : nameOfTheUnderlyingIndexs) {
					this.nameOfTheUnderlyingIndex.add(toAdd);
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("nameOfTheUnderlyingIndex")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNameOfTheUnderlyingIndex(List<? extends String> nameOfTheUnderlyingIndexs) {
			if (nameOfTheUnderlyingIndexs == null)  {
				this.nameOfTheUnderlyingIndex = new ArrayList<>();
			}
			else {
				this.nameOfTheUnderlyingIndex = nameOfTheUnderlyingIndexs.stream()
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addIndicatorOfTheUnderlyingIndex(String indicatorOfTheUnderlyingIndex) {
			if (indicatorOfTheUnderlyingIndex!=null) this.indicatorOfTheUnderlyingIndex.add(indicatorOfTheUnderlyingIndex);
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addIndicatorOfTheUnderlyingIndex(String indicatorOfTheUnderlyingIndex, int _idx) {
			getIndex(this.indicatorOfTheUnderlyingIndex, _idx, () -> indicatorOfTheUnderlyingIndex);
			return this;
		}
		@Override 
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addIndicatorOfTheUnderlyingIndex(List<? extends String> indicatorOfTheUnderlyingIndexs) {
			if (indicatorOfTheUnderlyingIndexs != null) {
				for (String toAdd : indicatorOfTheUnderlyingIndexs) {
					this.indicatorOfTheUnderlyingIndex.add(toAdd);
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("indicatorOfTheUnderlyingIndex")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIndicatorOfTheUnderlyingIndex(List<? extends String> indicatorOfTheUnderlyingIndexs) {
			if (indicatorOfTheUnderlyingIndexs == null)  {
				this.indicatorOfTheUnderlyingIndex = new ArrayList<>();
			}
			else {
				this.indicatorOfTheUnderlyingIndex = indicatorOfTheUnderlyingIndexs.stream()
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addBasketConstituents(BasketConstituentsReport basketConstituents) {
			if (basketConstituents!=null) this.basketConstituents.add(basketConstituents.toBuilder());
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addBasketConstituents(BasketConstituentsReport basketConstituents, int _idx) {
			getIndex(this.basketConstituents, _idx, () -> basketConstituents.toBuilder());
			return this;
		}
		@Override 
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addBasketConstituents(List<? extends BasketConstituentsReport> basketConstituentss) {
			if (basketConstituentss != null) {
				for (BasketConstituentsReport toAdd : basketConstituentss) {
					this.basketConstituents.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("basketConstituents")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setBasketConstituents(List<? extends BasketConstituentsReport> basketConstituentss) {
			if (basketConstituentss == null)  {
				this.basketConstituents = new ArrayList<>();
			}
			else {
				this.basketConstituents = basketConstituentss.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("customBasketCode")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCustomBasketCode(String customBasketCode) {
			this.customBasketCode = customBasketCode==null?null:customBasketCode;
			return this;
		}
		@Override
		@RosettaAttribute("settlementCurrency1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSettlementCurrency1(ISOCurrencyCodeEnum settlementCurrency1) {
			this.settlementCurrency1 = settlementCurrency1==null?null:settlementCurrency1;
			return this;
		}
		@Override
		@RosettaAttribute("settlementCurrency2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSettlementCurrency2(ISOCurrencyCodeEnum settlementCurrency2) {
			this.settlementCurrency2 = settlementCurrency2==null?null:settlementCurrency2;
			return this;
		}
		@Override
		@RosettaAttribute("valuationAmount")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setValuationAmount(BigDecimal valuationAmount) {
			this.valuationAmount = valuationAmount==null?null:valuationAmount;
			return this;
		}
		@Override
		@RosettaAttribute("valuationCurrency")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setValuationCurrency(ISOCurrencyCodeEnum valuationCurrency) {
			this.valuationCurrency = valuationCurrency==null?null:valuationCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("valuationMethod")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setValuationMethod(ValuationType1Code valuationMethod) {
			this.valuationMethod = valuationMethod==null?null:valuationMethod;
			return this;
		}
		@Override
		@RosettaAttribute("delta")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDelta(BigDecimal delta) {
			this.delta = delta==null?null:delta;
			return this;
		}
		@Override
		@RosettaAttribute("valuationTimestamp")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setValuationTimestamp(ZonedDateTime valuationTimestamp) {
			this.valuationTimestamp = valuationTimestamp==null?null:valuationTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("confirmationTimestamp")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setConfirmationTimestamp(ZonedDateTime confirmationTimestamp) {
			this.confirmationTimestamp = confirmationTimestamp==null?null:confirmationTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("confirmed")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setConfirmed(String confirmed) {
			this.confirmed = confirmed==null?null:confirmed;
			return this;
		}
		@Override
		@RosettaAttribute("clearingObligation")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setClearingObligation(String clearingObligation) {
			this.clearingObligation = clearingObligation==null?null:clearingObligation;
			return this;
		}
		@Override
		@RosettaAttribute("cleared")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCleared(String cleared) {
			this.cleared = cleared==null?null:cleared;
			return this;
		}
		@Override
		@RosettaAttribute("clearingTimestamp")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setClearingTimestamp(ZonedDateTime clearingTimestamp) {
			this.clearingTimestamp = clearingTimestamp==null?null:clearingTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("centralCounterparty")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCentralCounterparty(String centralCounterparty) {
			this.centralCounterparty = centralCounterparty==null?null:centralCounterparty;
			return this;
		}
		@Override
		@RosettaAttribute("masterAgreementType")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setMasterAgreementType(String masterAgreementType) {
			this.masterAgreementType = masterAgreementType==null?null:masterAgreementType;
			return this;
		}
		@Override
		@RosettaAttribute("otherMasterAgreementType")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setOtherMasterAgreementType(String otherMasterAgreementType) {
			this.otherMasterAgreementType = otherMasterAgreementType==null?null:otherMasterAgreementType;
			return this;
		}
		@Override
		@RosettaAttribute("masterAgreementVersion")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setMasterAgreementVersion(Integer masterAgreementVersion) {
			this.masterAgreementVersion = masterAgreementVersion==null?null:masterAgreementVersion;
			return this;
		}
		@Override
		@RosettaAttribute("intragroup")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIntragroup(Boolean intragroup) {
			this.intragroup = intragroup==null?null:intragroup;
			return this;
		}
		@Override
		@RosettaAttribute("ptrr")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPtrr(Boolean ptrr) {
			this.ptrr = ptrr==null?null:ptrr;
			return this;
		}
		@Override
		@RosettaAttribute("typeOfPTRRTechnique")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setTypeOfPTRRTechnique(String typeOfPTRRTechnique) {
			this.typeOfPTRRTechnique = typeOfPTRRTechnique==null?null:typeOfPTRRTechnique;
			return this;
		}
		@Override
		@RosettaAttribute("ptrrServiceProvider")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPtrrServiceProvider(String ptrrServiceProvider) {
			this.ptrrServiceProvider = ptrrServiceProvider==null?null:ptrrServiceProvider;
			return this;
		}
		@Override
		@RosettaAttribute("venueOfExecution")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setVenueOfExecution(String venueOfExecution) {
			this.venueOfExecution = venueOfExecution==null?null:venueOfExecution;
			return this;
		}
		@Override
		@RosettaAttribute("executionTimestamp")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setExecutionTimestamp(ZonedDateTime executionTimestamp) {
			this.executionTimestamp = executionTimestamp==null?null:executionTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("effectiveDate")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setEffectiveDate(Date effectiveDate) {
			this.effectiveDate = effectiveDate==null?null:effectiveDate;
			return this;
		}
		@Override
		@RosettaAttribute("expirationDate")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setExpirationDate(Date expirationDate) {
			this.expirationDate = expirationDate==null?null:expirationDate;
			return this;
		}
		@Override
		@RosettaAttribute("earlyTerminationDate")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setEarlyTerminationDate(Date earlyTerminationDate) {
			this.earlyTerminationDate = earlyTerminationDate==null?null:earlyTerminationDate;
			return this;
		}
		@Override
		@RosettaAttribute("finalContractualSettlementDate")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFinalContractualSettlementDate(Date finalContractualSettlementDate) {
			this.finalContractualSettlementDate = finalContractualSettlementDate==null?null:finalContractualSettlementDate;
			return this;
		}
		@Override
		@RosettaAttribute("deliveryType")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDeliveryType(String deliveryType) {
			this.deliveryType = deliveryType==null?null:deliveryType;
			return this;
		}
		@Override
		@RosettaAttribute("price")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPrice(PriceAndNotation price) {
			this.price = price==null?null:price.toBuilder();
			return this;
		}
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addPriceSchedule(PriceScheduleReport priceSchedule) {
			if (priceSchedule!=null) this.priceSchedule.add(priceSchedule.toBuilder());
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addPriceSchedule(PriceScheduleReport priceSchedule, int _idx) {
			getIndex(this.priceSchedule, _idx, () -> priceSchedule.toBuilder());
			return this;
		}
		@Override 
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addPriceSchedule(List<? extends PriceScheduleReport> priceSchedules) {
			if (priceSchedules != null) {
				for (PriceScheduleReport toAdd : priceSchedules) {
					this.priceSchedule.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("priceSchedule")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPriceSchedule(List<? extends PriceScheduleReport> priceSchedules) {
			if (priceSchedules == null)  {
				this.priceSchedule = new ArrayList<>();
			}
			else {
				this.priceSchedule = priceSchedules.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("packageTransactionPrice")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPackageTransactionPrice(PackageTransactionPrice packageTransactionPrice) {
			this.packageTransactionPrice = packageTransactionPrice==null?null:packageTransactionPrice.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("notionalLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalLeg1(BigDecimal notionalLeg1) {
			this.notionalLeg1 = notionalLeg1==null?null:notionalLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("notionalCurrency1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalCurrency1(ISOCurrencyCodeEnum notionalCurrency1) {
			this.notionalCurrency1 = notionalCurrency1==null?null:notionalCurrency1;
			return this;
		}
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalAmountScheduleLeg1(NotionalAmountScheduleLeg1Report notionalAmountScheduleLeg1) {
			if (notionalAmountScheduleLeg1!=null) this.notionalAmountScheduleLeg1.add(notionalAmountScheduleLeg1.toBuilder());
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalAmountScheduleLeg1(NotionalAmountScheduleLeg1Report notionalAmountScheduleLeg1, int _idx) {
			getIndex(this.notionalAmountScheduleLeg1, _idx, () -> notionalAmountScheduleLeg1.toBuilder());
			return this;
		}
		@Override 
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalAmountScheduleLeg1(List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg1s) {
			if (notionalAmountScheduleLeg1s != null) {
				for (NotionalAmountScheduleLeg1Report toAdd : notionalAmountScheduleLeg1s) {
					this.notionalAmountScheduleLeg1.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("notionalAmountScheduleLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalAmountScheduleLeg1(List<? extends NotionalAmountScheduleLeg1Report> notionalAmountScheduleLeg1s) {
			if (notionalAmountScheduleLeg1s == null)  {
				this.notionalAmountScheduleLeg1 = new ArrayList<>();
			}
			else {
				this.notionalAmountScheduleLeg1 = notionalAmountScheduleLeg1s.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setTotalNotionalQuantityLeg1(BigDecimal totalNotionalQuantityLeg1) {
			this.totalNotionalQuantityLeg1 = totalNotionalQuantityLeg1==null?null:totalNotionalQuantityLeg1;
			return this;
		}
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalQuantityScheduleLeg1(NotionalQuantityScheduleLeg1Report notionalQuantityScheduleLeg1) {
			if (notionalQuantityScheduleLeg1!=null) this.notionalQuantityScheduleLeg1.add(notionalQuantityScheduleLeg1.toBuilder());
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalQuantityScheduleLeg1(NotionalQuantityScheduleLeg1Report notionalQuantityScheduleLeg1, int _idx) {
			getIndex(this.notionalQuantityScheduleLeg1, _idx, () -> notionalQuantityScheduleLeg1.toBuilder());
			return this;
		}
		@Override 
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalQuantityScheduleLeg1(List<? extends NotionalQuantityScheduleLeg1Report> notionalQuantityScheduleLeg1s) {
			if (notionalQuantityScheduleLeg1s != null) {
				for (NotionalQuantityScheduleLeg1Report toAdd : notionalQuantityScheduleLeg1s) {
					this.notionalQuantityScheduleLeg1.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("notionalQuantityScheduleLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalQuantityScheduleLeg1(List<? extends NotionalQuantityScheduleLeg1Report> notionalQuantityScheduleLeg1s) {
			if (notionalQuantityScheduleLeg1s == null)  {
				this.notionalQuantityScheduleLeg1 = new ArrayList<>();
			}
			else {
				this.notionalQuantityScheduleLeg1 = notionalQuantityScheduleLeg1s.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setTotalNotionalQuantityLeg2(BigDecimal totalNotionalQuantityLeg2) {
			this.totalNotionalQuantityLeg2 = totalNotionalQuantityLeg2==null?null:totalNotionalQuantityLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("notionalLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalLeg2(BigDecimal notionalLeg2) {
			this.notionalLeg2 = notionalLeg2==null?null:notionalLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("notionalCurrency2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalCurrency2(ISOCurrencyCodeEnum notionalCurrency2) {
			this.notionalCurrency2 = notionalCurrency2==null?null:notionalCurrency2;
			return this;
		}
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalAmountScheduleLeg2(NotionalAmountScheduleLeg2Report notionalAmountScheduleLeg2) {
			if (notionalAmountScheduleLeg2!=null) this.notionalAmountScheduleLeg2.add(notionalAmountScheduleLeg2.toBuilder());
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalAmountScheduleLeg2(NotionalAmountScheduleLeg2Report notionalAmountScheduleLeg2, int _idx) {
			getIndex(this.notionalAmountScheduleLeg2, _idx, () -> notionalAmountScheduleLeg2.toBuilder());
			return this;
		}
		@Override 
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalAmountScheduleLeg2(List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg2s) {
			if (notionalAmountScheduleLeg2s != null) {
				for (NotionalAmountScheduleLeg2Report toAdd : notionalAmountScheduleLeg2s) {
					this.notionalAmountScheduleLeg2.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("notionalAmountScheduleLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalAmountScheduleLeg2(List<? extends NotionalAmountScheduleLeg2Report> notionalAmountScheduleLeg2s) {
			if (notionalAmountScheduleLeg2s == null)  {
				this.notionalAmountScheduleLeg2 = new ArrayList<>();
			}
			else {
				this.notionalAmountScheduleLeg2 = notionalAmountScheduleLeg2s.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalQuantityScheduleLeg2(NotionalQuantityScheduleLeg2Report notionalQuantityScheduleLeg2) {
			if (notionalQuantityScheduleLeg2!=null) this.notionalQuantityScheduleLeg2.add(notionalQuantityScheduleLeg2.toBuilder());
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalQuantityScheduleLeg2(NotionalQuantityScheduleLeg2Report notionalQuantityScheduleLeg2, int _idx) {
			getIndex(this.notionalQuantityScheduleLeg2, _idx, () -> notionalQuantityScheduleLeg2.toBuilder());
			return this;
		}
		@Override 
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addNotionalQuantityScheduleLeg2(List<? extends NotionalQuantityScheduleLeg2Report> notionalQuantityScheduleLeg2s) {
			if (notionalQuantityScheduleLeg2s != null) {
				for (NotionalQuantityScheduleLeg2Report toAdd : notionalQuantityScheduleLeg2s) {
					this.notionalQuantityScheduleLeg2.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("notionalQuantityScheduleLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNotionalQuantityScheduleLeg2(List<? extends NotionalQuantityScheduleLeg2Report> notionalQuantityScheduleLeg2s) {
			if (notionalQuantityScheduleLeg2s == null)  {
				this.notionalQuantityScheduleLeg2 = new ArrayList<>();
			}
			else {
				this.notionalQuantityScheduleLeg2 = notionalQuantityScheduleLeg2s.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment) {
			if (otherPayment!=null) this.otherPayment.add(otherPayment.toBuilder());
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment, int _idx) {
			getIndex(this.otherPayment, _idx, () -> otherPayment.toBuilder());
			return this;
		}
		@Override 
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addOtherPayment(List<? extends OtherPaymentReport> otherPayments) {
			if (otherPayments != null) {
				for (OtherPaymentReport toAdd : otherPayments) {
					this.otherPayment.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("otherPayment")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setOtherPayment(List<? extends OtherPaymentReport> otherPayments) {
			if (otherPayments == null)  {
				this.otherPayment = new ArrayList<>();
			}
			else {
				this.otherPayment = otherPayments.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("fixedRateOfLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRateOfLeg1(BigDecimal fixedRateOfLeg1) {
			this.fixedRateOfLeg1 = fixedRateOfLeg1==null?null:fixedRateOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRateDayCountConventionLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRateDayCountConventionLeg1(InterestComputationMethod4Code fixedRateDayCountConventionLeg1) {
			this.fixedRateDayCountConventionLeg1 = fixedRateDayCountConventionLeg1==null?null:fixedRateDayCountConventionLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRatePaymentFrequencyPeriodLeg1(String fixedRatePaymentFrequencyPeriodLeg1) {
			this.fixedRatePaymentFrequencyPeriodLeg1 = fixedRatePaymentFrequencyPeriodLeg1==null?null:fixedRatePaymentFrequencyPeriodLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRatePaymentFrequencyPeriodMultiplierLeg1(Integer fixedRatePaymentFrequencyPeriodMultiplierLeg1) {
			this.fixedRatePaymentFrequencyPeriodMultiplierLeg1 = fixedRatePaymentFrequencyPeriodMultiplierLeg1==null?null:fixedRatePaymentFrequencyPeriodMultiplierLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("identifierOfFloatingRateOfLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIdentifierOfFloatingRateOfLeg1(String identifierOfFloatingRateOfLeg1) {
			this.identifierOfFloatingRateOfLeg1 = identifierOfFloatingRateOfLeg1==null?null:identifierOfFloatingRateOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("indicatorOfTheFloatingRateOfLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIndicatorOfTheFloatingRateOfLeg1(String indicatorOfTheFloatingRateOfLeg1) {
			this.indicatorOfTheFloatingRateOfLeg1 = indicatorOfTheFloatingRateOfLeg1==null?null:indicatorOfTheFloatingRateOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("nameOfTheFloatingRateOfLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNameOfTheFloatingRateOfLeg1(String nameOfTheFloatingRateOfLeg1) {
			this.nameOfTheFloatingRateOfLeg1 = nameOfTheFloatingRateOfLeg1==null?null:nameOfTheFloatingRateOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateDayCountConventionOfLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateDayCountConventionOfLeg1(InterestComputationMethod4Code floatingRateDayCountConventionOfLeg1) {
			this.floatingRateDayCountConventionOfLeg1 = floatingRateDayCountConventionOfLeg1==null?null:floatingRateDayCountConventionOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodOfLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodOfLeg1(Frequency13Code__1 floatingRatePaymentFrequencyPeriodOfLeg1) {
			this.floatingRatePaymentFrequencyPeriodOfLeg1 = floatingRatePaymentFrequencyPeriodOfLeg1==null?null:floatingRatePaymentFrequencyPeriodOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierOfLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1(Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg1) {
			this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg1 = floatingRatePaymentFrequencyPeriodMultiplierOfLeg1==null?null:floatingRatePaymentFrequencyPeriodMultiplierOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg1TimePeriod")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateReferencePeriodOfLeg1TimePeriod(String floatingRateReferencePeriodOfLeg1TimePeriod) {
			this.floatingRateReferencePeriodOfLeg1TimePeriod = floatingRateReferencePeriodOfLeg1TimePeriod==null?null:floatingRateReferencePeriodOfLeg1TimePeriod;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg1Multiplier")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateReferencePeriodOfLeg1Multiplier(Integer floatingRateReferencePeriodOfLeg1Multiplier) {
			this.floatingRateReferencePeriodOfLeg1Multiplier = floatingRateReferencePeriodOfLeg1Multiplier==null?null:floatingRateReferencePeriodOfLeg1Multiplier;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodOfLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateResetFrequencyPeriodOfLeg1(String floatingRateResetFrequencyPeriodOfLeg1) {
			this.floatingRateResetFrequencyPeriodOfLeg1 = floatingRateResetFrequencyPeriodOfLeg1==null?null:floatingRateResetFrequencyPeriodOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateResetFrequencyPeriodMultiplierLeg1(Integer floatingRateResetFrequencyPeriodMultiplierLeg1) {
			this.floatingRateResetFrequencyPeriodMultiplierLeg1 = floatingRateResetFrequencyPeriodMultiplierLeg1==null?null:floatingRateResetFrequencyPeriodMultiplierLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("spreadOfLeg1")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSpreadOfLeg1(SpreadOfLeg1 spreadOfLeg1) {
			this.spreadOfLeg1 = spreadOfLeg1==null?null:spreadOfLeg1.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("fixedRateOfLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRateOfLeg2(BigDecimal fixedRateOfLeg2) {
			this.fixedRateOfLeg2 = fixedRateOfLeg2==null?null:fixedRateOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRateDayCountConventionLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRateDayCountConventionLeg2(InterestComputationMethod4Code fixedRateDayCountConventionLeg2) {
			this.fixedRateDayCountConventionLeg2 = fixedRateDayCountConventionLeg2==null?null:fixedRateDayCountConventionLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRatePaymentFrequencyPeriodLeg2(String fixedRatePaymentFrequencyPeriodLeg2) {
			this.fixedRatePaymentFrequencyPeriodLeg2 = fixedRatePaymentFrequencyPeriodLeg2==null?null:fixedRatePaymentFrequencyPeriodLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFixedRatePaymentFrequencyPeriodMultiplierLeg2(Integer fixedRatePaymentFrequencyPeriodMultiplierLeg2) {
			this.fixedRatePaymentFrequencyPeriodMultiplierLeg2 = fixedRatePaymentFrequencyPeriodMultiplierLeg2==null?null:fixedRatePaymentFrequencyPeriodMultiplierLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("identifierOfFloatingRateOfLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIdentifierOfFloatingRateOfLeg2(String identifierOfFloatingRateOfLeg2) {
			this.identifierOfFloatingRateOfLeg2 = identifierOfFloatingRateOfLeg2==null?null:identifierOfFloatingRateOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("indicatorOfTheFloatingRateOfLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIndicatorOfTheFloatingRateOfLeg2(String indicatorOfTheFloatingRateOfLeg2) {
			this.indicatorOfTheFloatingRateOfLeg2 = indicatorOfTheFloatingRateOfLeg2==null?null:indicatorOfTheFloatingRateOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("nameOfTheFloatingRateOfLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNameOfTheFloatingRateOfLeg2(String nameOfTheFloatingRateOfLeg2) {
			this.nameOfTheFloatingRateOfLeg2 = nameOfTheFloatingRateOfLeg2==null?null:nameOfTheFloatingRateOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateDayCountConventionOfLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateDayCountConventionOfLeg2(InterestComputationMethod4Code floatingRateDayCountConventionOfLeg2) {
			this.floatingRateDayCountConventionOfLeg2 = floatingRateDayCountConventionOfLeg2==null?null:floatingRateDayCountConventionOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodOfLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodOfLeg2(String floatingRatePaymentFrequencyPeriodOfLeg2) {
			this.floatingRatePaymentFrequencyPeriodOfLeg2 = floatingRatePaymentFrequencyPeriodOfLeg2==null?null:floatingRatePaymentFrequencyPeriodOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierOfLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2(Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg2) {
			this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 = floatingRatePaymentFrequencyPeriodMultiplierOfLeg2==null?null:floatingRatePaymentFrequencyPeriodMultiplierOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg2TimePeriod")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateReferencePeriodOfLeg2TimePeriod(String floatingRateReferencePeriodOfLeg2TimePeriod) {
			this.floatingRateReferencePeriodOfLeg2TimePeriod = floatingRateReferencePeriodOfLeg2TimePeriod==null?null:floatingRateReferencePeriodOfLeg2TimePeriod;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateReferencePeriodOfLeg2Multiplier")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateReferencePeriodOfLeg2Multiplier(Integer floatingRateReferencePeriodOfLeg2Multiplier) {
			this.floatingRateReferencePeriodOfLeg2Multiplier = floatingRateReferencePeriodOfLeg2Multiplier==null?null:floatingRateReferencePeriodOfLeg2Multiplier;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodOfLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateResetFrequencyPeriodOfLeg2(String floatingRateResetFrequencyPeriodOfLeg2) {
			this.floatingRateResetFrequencyPeriodOfLeg2 = floatingRateResetFrequencyPeriodOfLeg2==null?null:floatingRateResetFrequencyPeriodOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFloatingRateResetFrequencyPeriodMultiplierLeg2(Integer floatingRateResetFrequencyPeriodMultiplierLeg2) {
			this.floatingRateResetFrequencyPeriodMultiplierLeg2 = floatingRateResetFrequencyPeriodMultiplierLeg2==null?null:floatingRateResetFrequencyPeriodMultiplierLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("spreadOfLeg2")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSpreadOfLeg2(SpreadOfLeg2 spreadOfLeg2) {
			this.spreadOfLeg2 = spreadOfLeg2==null?null:spreadOfLeg2.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("packageTransactionSpread")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setPackageTransactionSpread(PackageTransactionSpread packageTransactionSpread) {
			this.packageTransactionSpread = packageTransactionSpread==null?null:packageTransactionSpread.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("exchangeRate")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setExchangeRate(BigDecimal exchangeRate) {
			this.exchangeRate = exchangeRate==null?null:exchangeRate;
			return this;
		}
		@Override
		@RosettaAttribute("forwardExchangeRate")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setForwardExchangeRate(BigDecimal forwardExchangeRate) {
			this.forwardExchangeRate = forwardExchangeRate==null?null:forwardExchangeRate;
			return this;
		}
		@Override
		@RosettaAttribute("exchangeRateBasis")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setExchangeRateBasis(String exchangeRateBasis) {
			this.exchangeRateBasis = exchangeRateBasis==null?null:exchangeRateBasis;
			return this;
		}
		@Override
		@RosettaAttribute("baseProduct")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setBaseProduct(String baseProduct) {
			this.baseProduct = baseProduct==null?null:baseProduct;
			return this;
		}
		@Override
		@RosettaAttribute("subProduct")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSubProduct(String subProduct) {
			this.subProduct = subProduct==null?null:subProduct;
			return this;
		}
		@Override
		@RosettaAttribute("furtherSubProduct")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setFurtherSubProduct(String furtherSubProduct) {
			this.furtherSubProduct = furtherSubProduct==null?null:furtherSubProduct;
			return this;
		}
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addDeliveryPoint(String deliveryPoint) {
			if (deliveryPoint!=null) this.deliveryPoint.add(deliveryPoint);
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addDeliveryPoint(String deliveryPoint, int _idx) {
			getIndex(this.deliveryPoint, _idx, () -> deliveryPoint);
			return this;
		}
		@Override 
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addDeliveryPoint(List<? extends String> deliveryPoints) {
			if (deliveryPoints != null) {
				for (String toAdd : deliveryPoints) {
					this.deliveryPoint.add(toAdd);
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("deliveryPoint")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDeliveryPoint(List<? extends String> deliveryPoints) {
			if (deliveryPoints == null)  {
				this.deliveryPoint = new ArrayList<>();
			}
			else {
				this.deliveryPoint = deliveryPoints.stream()
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("interconnectionPoint")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setInterconnectionPoint(String interconnectionPoint) {
			this.interconnectionPoint = interconnectionPoint==null?null:interconnectionPoint;
			return this;
		}
		@Override
		@RosettaAttribute("loadType")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setLoadType(EnergyLoadType1Code loadType) {
			this.loadType = loadType==null?null:loadType;
			return this;
		}
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addDeliveryReport(DeliveryReport deliveryReport) {
			if (deliveryReport!=null) this.deliveryReport.add(deliveryReport.toBuilder());
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addDeliveryReport(DeliveryReport deliveryReport, int _idx) {
			getIndex(this.deliveryReport, _idx, () -> deliveryReport.toBuilder());
			return this;
		}
		@Override 
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addDeliveryReport(List<? extends DeliveryReport> deliveryReports) {
			if (deliveryReports != null) {
				for (DeliveryReport toAdd : deliveryReports) {
					this.deliveryReport.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("deliveryReport")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setDeliveryReport(List<? extends DeliveryReport> deliveryReports) {
			if (deliveryReports == null)  {
				this.deliveryReport = new ArrayList<>();
			}
			else {
				this.deliveryReport = deliveryReports.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("optionType")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setOptionType(String optionType) {
			this.optionType = optionType==null?null:optionType;
			return this;
		}
		@Override
		@RosettaAttribute("optionStyle")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setOptionStyle(String optionStyle) {
			this.optionStyle = optionStyle==null?null:optionStyle;
			return this;
		}
		@Override
		@RosettaAttribute("strikePrice")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setStrikePrice(StrikePriceAndNotation strikePrice) {
			this.strikePrice = strikePrice==null?null:strikePrice.toBuilder();
			return this;
		}
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addStrikePriceSchedule(StrikePriceScheduleReport strikePriceSchedule) {
			if (strikePriceSchedule!=null) this.strikePriceSchedule.add(strikePriceSchedule.toBuilder());
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addStrikePriceSchedule(StrikePriceScheduleReport strikePriceSchedule, int _idx) {
			getIndex(this.strikePriceSchedule, _idx, () -> strikePriceSchedule.toBuilder());
			return this;
		}
		@Override 
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder addStrikePriceSchedule(List<? extends StrikePriceScheduleReport> strikePriceSchedules) {
			if (strikePriceSchedules != null) {
				for (StrikePriceScheduleReport toAdd : strikePriceSchedules) {
					this.strikePriceSchedule.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("strikePriceSchedule")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setStrikePriceSchedule(List<? extends StrikePriceScheduleReport> strikePriceSchedules) {
			if (strikePriceSchedules == null)  {
				this.strikePriceSchedule = new ArrayList<>();
			}
			else {
				this.strikePriceSchedule = strikePriceSchedules.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("optionPremiumAmount")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setOptionPremiumAmount(BigDecimal optionPremiumAmount) {
			this.optionPremiumAmount = optionPremiumAmount==null?null:optionPremiumAmount;
			return this;
		}
		@Override
		@RosettaAttribute("optionPremiumCurrency")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setOptionPremiumCurrency(ISOCurrencyCodeEnum optionPremiumCurrency) {
			this.optionPremiumCurrency = optionPremiumCurrency==null?null:optionPremiumCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("optionPremiumPaymentDate")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setOptionPremiumPaymentDate(Date optionPremiumPaymentDate) {
			this.optionPremiumPaymentDate = optionPremiumPaymentDate==null?null:optionPremiumPaymentDate;
			return this;
		}
		@Override
		@RosettaAttribute("maturityDateOfTheUnderlying")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setMaturityDateOfTheUnderlying(Date maturityDateOfTheUnderlying) {
			this.maturityDateOfTheUnderlying = maturityDateOfTheUnderlying==null?null:maturityDateOfTheUnderlying;
			return this;
		}
		@Override
		@RosettaAttribute("seniority")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSeniority(String seniority) {
			this.seniority = seniority==null?null:seniority;
			return this;
		}
		@Override
		@RosettaAttribute("referenceEntity")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setReferenceEntity(String referenceEntity) {
			this.referenceEntity = referenceEntity==null?null:referenceEntity;
			return this;
		}
		@Override
		@RosettaAttribute("series")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setSeries(Integer series) {
			this.series = series==null?null:series;
			return this;
		}
		@Override
		@RosettaAttribute("version")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setVersion(Integer version) {
			this.version = version==null?null:version;
			return this;
		}
		@Override
		@RosettaAttribute("indexFactor")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setIndexFactor(BigDecimal indexFactor) {
			this.indexFactor = indexFactor==null?null:indexFactor;
			return this;
		}
		@Override
		@RosettaAttribute("tranche")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setTranche(Boolean tranche) {
			this.tranche = tranche==null?null:tranche;
			return this;
		}
		@Override
		@RosettaAttribute("cdSIndexAttachmentPoint")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCdSIndexAttachmentPoint(BigDecimal cdSIndexAttachmentPoint) {
			this.cdSIndexAttachmentPoint = cdSIndexAttachmentPoint==null?null:cdSIndexAttachmentPoint;
			return this;
		}
		@Override
		@RosettaAttribute("cdSIndexDetachmentPoint")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCdSIndexDetachmentPoint(BigDecimal cdSIndexDetachmentPoint) {
			this.cdSIndexDetachmentPoint = cdSIndexDetachmentPoint==null?null:cdSIndexDetachmentPoint;
			return this;
		}
		@Override
		@RosettaAttribute("collateralPortfolioIndicator")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCollateralPortfolioIndicator(Boolean collateralPortfolioIndicator) {
			this.collateralPortfolioIndicator = collateralPortfolioIndicator==null?null:collateralPortfolioIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("collateralPortfolioCode")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setCollateralPortfolioCode(String collateralPortfolioCode) {
			this.collateralPortfolioCode = collateralPortfolioCode==null?null:collateralPortfolioCode;
			return this;
		}
		@Override
		@RosettaAttribute("actionType")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setActionType(String actionType) {
			this.actionType = actionType==null?null:actionType;
			return this;
		}
		@Override
		@RosettaAttribute("eventType")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setEventType(String eventType) {
			this.eventType = eventType==null?null:eventType;
			return this;
		}
		@Override
		@RosettaAttribute("eventDate")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setEventDate(Date eventDate) {
			this.eventDate = eventDate==null?null:eventDate;
			return this;
		}
		@Override
		@RosettaAttribute("level")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setLevel(String level) {
			this.level = level==null?null:level;
			return this;
		}
		@Override
		@RosettaAttribute("nonReportable")
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder setNonReportable(ESMAEMIRNonReportable nonReportable) {
			this.nonReportable = nonReportable==null?null:nonReportable.toBuilder();
			return this;
		}
		
		@Override
		public ESMAEMIRTransactionReport build() {
			return new ESMAEMIRTransactionReport.ESMAEMIRTransactionReportImpl(this);
		}
		
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder prune() {
			basketConstituents = basketConstituents.stream().filter(b->b!=null).<BasketConstituentsReport.BasketConstituentsReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (price!=null && !price.prune().hasData()) price = null;
			priceSchedule = priceSchedule.stream().filter(b->b!=null).<PriceScheduleReport.PriceScheduleReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (packageTransactionPrice!=null && !packageTransactionPrice.prune().hasData()) packageTransactionPrice = null;
			notionalAmountScheduleLeg1 = notionalAmountScheduleLeg1.stream().filter(b->b!=null).<NotionalAmountScheduleLeg1Report.NotionalAmountScheduleLeg1ReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			notionalQuantityScheduleLeg1 = notionalQuantityScheduleLeg1.stream().filter(b->b!=null).<NotionalQuantityScheduleLeg1Report.NotionalQuantityScheduleLeg1ReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			notionalAmountScheduleLeg2 = notionalAmountScheduleLeg2.stream().filter(b->b!=null).<NotionalAmountScheduleLeg2Report.NotionalAmountScheduleLeg2ReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			notionalQuantityScheduleLeg2 = notionalQuantityScheduleLeg2.stream().filter(b->b!=null).<NotionalQuantityScheduleLeg2Report.NotionalQuantityScheduleLeg2ReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			otherPayment = otherPayment.stream().filter(b->b!=null).<OtherPaymentReport.OtherPaymentReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (spreadOfLeg1!=null && !spreadOfLeg1.prune().hasData()) spreadOfLeg1 = null;
			if (spreadOfLeg2!=null && !spreadOfLeg2.prune().hasData()) spreadOfLeg2 = null;
			if (packageTransactionSpread!=null && !packageTransactionSpread.prune().hasData()) packageTransactionSpread = null;
			deliveryReport = deliveryReport.stream().filter(b->b!=null).<DeliveryReport.DeliveryReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (strikePrice!=null && !strikePrice.prune().hasData()) strikePrice = null;
			strikePriceSchedule = strikePriceSchedule.stream().filter(b->b!=null).<StrikePriceScheduleReport.StrikePriceScheduleReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			if (nonReportable!=null && !nonReportable.prune().hasData()) nonReportable = null;
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getReportingTimestamp()!=null) return true;
			if (getReportSubmittingEntityID()!=null) return true;
			if (getEntityResponsibleForReporting()!=null) return true;
			if (getCounterparty1()!=null) return true;
			if (getNatureOfCounterparty1()!=null) return true;
			if (getCorporateSectorOfTheCounterparty1()!=null) return true;
			if (getClearingThresholdOfCounterparty1()!=null) return true;
			if (getCounterparty2()!=null) return true;
			if (getCountryOfTheCounterParty2()!=null) return true;
			if (getNatureOfCounterparty2()!=null) return true;
			if (getCorporateSectorOfTheCounterparty2()!=null) return true;
			if (getCounterparty2IdentifierType()!=null) return true;
			if (getClearingMember()!=null) return true;
			if (getClearingThresholdOfCounterparty2()!=null) return true;
			if (getReportingObligationOfTheCounterparty2()!=null) return true;
			if (getBrokerID()!=null) return true;
			if (getDirection()!=null) return true;
			if (getDirectionOfLeg1()!=null) return true;
			if (getDirectionOfLeg2()!=null) return true;
			if (getDirectlyLinkedToCommercialActivityOrTreasuryFinancing()!=null) return true;
			if (getUti()!=null) return true;
			if (getUtiProprietary()!=null) return true;
			if (getReportTrackingNumber()!=null) return true;
			if (getPriorUti()!=null) return true;
			if (getPriorUtiProprietary()!=null) return true;
			if (getPtrrId()!=null) return true;
			if (getPackageIdentifier()!=null) return true;
			if (getSubsequentPositionUti()!=null) return true;
			if (getIsin()!=null) return true;
			if (getUpi()!=null) return true;
			if (getProductClassification()!=null) return true;
			if (getContractType()!=null) return true;
			if (getAssetClass()!=null) return true;
			if (getIsCrypto()!=null) return true;
			if (getUnderlyingIdentificationType()!=null) return true;
			if (getUnderlyingIdentification()!=null) return true;
			if (getNameOfTheUnderlyingIndex()!=null && !getNameOfTheUnderlyingIndex().isEmpty()) return true;
			if (getIndicatorOfTheUnderlyingIndex()!=null && !getIndicatorOfTheUnderlyingIndex().isEmpty()) return true;
			if (getBasketConstituents()!=null && getBasketConstituents().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getCustomBasketCode()!=null) return true;
			if (getSettlementCurrency1()!=null) return true;
			if (getSettlementCurrency2()!=null) return true;
			if (getValuationAmount()!=null) return true;
			if (getValuationCurrency()!=null) return true;
			if (getValuationMethod()!=null) return true;
			if (getDelta()!=null) return true;
			if (getValuationTimestamp()!=null) return true;
			if (getConfirmationTimestamp()!=null) return true;
			if (getConfirmed()!=null) return true;
			if (getClearingObligation()!=null) return true;
			if (getCleared()!=null) return true;
			if (getClearingTimestamp()!=null) return true;
			if (getCentralCounterparty()!=null) return true;
			if (getMasterAgreementType()!=null) return true;
			if (getOtherMasterAgreementType()!=null) return true;
			if (getMasterAgreementVersion()!=null) return true;
			if (getIntragroup()!=null) return true;
			if (getPtrr()!=null) return true;
			if (getTypeOfPTRRTechnique()!=null) return true;
			if (getPtrrServiceProvider()!=null) return true;
			if (getVenueOfExecution()!=null) return true;
			if (getExecutionTimestamp()!=null) return true;
			if (getEffectiveDate()!=null) return true;
			if (getExpirationDate()!=null) return true;
			if (getEarlyTerminationDate()!=null) return true;
			if (getFinalContractualSettlementDate()!=null) return true;
			if (getDeliveryType()!=null) return true;
			if (getPrice()!=null && getPrice().hasData()) return true;
			if (getPriceSchedule()!=null && getPriceSchedule().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getPackageTransactionPrice()!=null && getPackageTransactionPrice().hasData()) return true;
			if (getNotionalLeg1()!=null) return true;
			if (getNotionalCurrency1()!=null) return true;
			if (getNotionalAmountScheduleLeg1()!=null && getNotionalAmountScheduleLeg1().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getTotalNotionalQuantityLeg1()!=null) return true;
			if (getNotionalQuantityScheduleLeg1()!=null && getNotionalQuantityScheduleLeg1().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getTotalNotionalQuantityLeg2()!=null) return true;
			if (getNotionalLeg2()!=null) return true;
			if (getNotionalCurrency2()!=null) return true;
			if (getNotionalAmountScheduleLeg2()!=null && getNotionalAmountScheduleLeg2().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getNotionalQuantityScheduleLeg2()!=null && getNotionalQuantityScheduleLeg2().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getOtherPayment()!=null && getOtherPayment().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getFixedRateOfLeg1()!=null) return true;
			if (getFixedRateDayCountConventionLeg1()!=null) return true;
			if (getFixedRatePaymentFrequencyPeriodLeg1()!=null) return true;
			if (getFixedRatePaymentFrequencyPeriodMultiplierLeg1()!=null) return true;
			if (getIdentifierOfFloatingRateOfLeg1()!=null) return true;
			if (getIndicatorOfTheFloatingRateOfLeg1()!=null) return true;
			if (getNameOfTheFloatingRateOfLeg1()!=null) return true;
			if (getFloatingRateDayCountConventionOfLeg1()!=null) return true;
			if (getFloatingRatePaymentFrequencyPeriodOfLeg1()!=null) return true;
			if (getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1()!=null) return true;
			if (getFloatingRateReferencePeriodOfLeg1TimePeriod()!=null) return true;
			if (getFloatingRateReferencePeriodOfLeg1Multiplier()!=null) return true;
			if (getFloatingRateResetFrequencyPeriodOfLeg1()!=null) return true;
			if (getFloatingRateResetFrequencyPeriodMultiplierLeg1()!=null) return true;
			if (getSpreadOfLeg1()!=null && getSpreadOfLeg1().hasData()) return true;
			if (getFixedRateOfLeg2()!=null) return true;
			if (getFixedRateDayCountConventionLeg2()!=null) return true;
			if (getFixedRatePaymentFrequencyPeriodLeg2()!=null) return true;
			if (getFixedRatePaymentFrequencyPeriodMultiplierLeg2()!=null) return true;
			if (getIdentifierOfFloatingRateOfLeg2()!=null) return true;
			if (getIndicatorOfTheFloatingRateOfLeg2()!=null) return true;
			if (getNameOfTheFloatingRateOfLeg2()!=null) return true;
			if (getFloatingRateDayCountConventionOfLeg2()!=null) return true;
			if (getFloatingRatePaymentFrequencyPeriodOfLeg2()!=null) return true;
			if (getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2()!=null) return true;
			if (getFloatingRateReferencePeriodOfLeg2TimePeriod()!=null) return true;
			if (getFloatingRateReferencePeriodOfLeg2Multiplier()!=null) return true;
			if (getFloatingRateResetFrequencyPeriodOfLeg2()!=null) return true;
			if (getFloatingRateResetFrequencyPeriodMultiplierLeg2()!=null) return true;
			if (getSpreadOfLeg2()!=null && getSpreadOfLeg2().hasData()) return true;
			if (getPackageTransactionSpread()!=null && getPackageTransactionSpread().hasData()) return true;
			if (getExchangeRate()!=null) return true;
			if (getForwardExchangeRate()!=null) return true;
			if (getExchangeRateBasis()!=null) return true;
			if (getBaseProduct()!=null) return true;
			if (getSubProduct()!=null) return true;
			if (getFurtherSubProduct()!=null) return true;
			if (getDeliveryPoint()!=null && !getDeliveryPoint().isEmpty()) return true;
			if (getInterconnectionPoint()!=null) return true;
			if (getLoadType()!=null) return true;
			if (getDeliveryReport()!=null && getDeliveryReport().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getOptionType()!=null) return true;
			if (getOptionStyle()!=null) return true;
			if (getStrikePrice()!=null && getStrikePrice().hasData()) return true;
			if (getStrikePriceSchedule()!=null && getStrikePriceSchedule().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getOptionPremiumAmount()!=null) return true;
			if (getOptionPremiumCurrency()!=null) return true;
			if (getOptionPremiumPaymentDate()!=null) return true;
			if (getMaturityDateOfTheUnderlying()!=null) return true;
			if (getSeniority()!=null) return true;
			if (getReferenceEntity()!=null) return true;
			if (getSeries()!=null) return true;
			if (getVersion()!=null) return true;
			if (getIndexFactor()!=null) return true;
			if (getTranche()!=null) return true;
			if (getCdSIndexAttachmentPoint()!=null) return true;
			if (getCdSIndexDetachmentPoint()!=null) return true;
			if (getCollateralPortfolioIndicator()!=null) return true;
			if (getCollateralPortfolioCode()!=null) return true;
			if (getActionType()!=null) return true;
			if (getEventType()!=null) return true;
			if (getEventDate()!=null) return true;
			if (getLevel()!=null) return true;
			if (getNonReportable()!=null && getNonReportable().hasData()) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder o = (ESMAEMIRTransactionReport.ESMAEMIRTransactionReportBuilder) other;
			
			merger.mergeRosetta(getBasketConstituents(), o.getBasketConstituents(), this::getOrCreateBasketConstituents);
			merger.mergeRosetta(getPrice(), o.getPrice(), this::setPrice);
			merger.mergeRosetta(getPriceSchedule(), o.getPriceSchedule(), this::getOrCreatePriceSchedule);
			merger.mergeRosetta(getPackageTransactionPrice(), o.getPackageTransactionPrice(), this::setPackageTransactionPrice);
			merger.mergeRosetta(getNotionalAmountScheduleLeg1(), o.getNotionalAmountScheduleLeg1(), this::getOrCreateNotionalAmountScheduleLeg1);
			merger.mergeRosetta(getNotionalQuantityScheduleLeg1(), o.getNotionalQuantityScheduleLeg1(), this::getOrCreateNotionalQuantityScheduleLeg1);
			merger.mergeRosetta(getNotionalAmountScheduleLeg2(), o.getNotionalAmountScheduleLeg2(), this::getOrCreateNotionalAmountScheduleLeg2);
			merger.mergeRosetta(getNotionalQuantityScheduleLeg2(), o.getNotionalQuantityScheduleLeg2(), this::getOrCreateNotionalQuantityScheduleLeg2);
			merger.mergeRosetta(getOtherPayment(), o.getOtherPayment(), this::getOrCreateOtherPayment);
			merger.mergeRosetta(getSpreadOfLeg1(), o.getSpreadOfLeg1(), this::setSpreadOfLeg1);
			merger.mergeRosetta(getSpreadOfLeg2(), o.getSpreadOfLeg2(), this::setSpreadOfLeg2);
			merger.mergeRosetta(getPackageTransactionSpread(), o.getPackageTransactionSpread(), this::setPackageTransactionSpread);
			merger.mergeRosetta(getDeliveryReport(), o.getDeliveryReport(), this::getOrCreateDeliveryReport);
			merger.mergeRosetta(getStrikePrice(), o.getStrikePrice(), this::setStrikePrice);
			merger.mergeRosetta(getStrikePriceSchedule(), o.getStrikePriceSchedule(), this::getOrCreateStrikePriceSchedule);
			merger.mergeRosetta(getNonReportable(), o.getNonReportable(), this::setNonReportable);
			
			merger.mergeBasic(getReportingTimestamp(), o.getReportingTimestamp(), this::setReportingTimestamp);
			merger.mergeBasic(getReportSubmittingEntityID(), o.getReportSubmittingEntityID(), this::setReportSubmittingEntityID);
			merger.mergeBasic(getEntityResponsibleForReporting(), o.getEntityResponsibleForReporting(), this::setEntityResponsibleForReporting);
			merger.mergeBasic(getCounterparty1(), o.getCounterparty1(), this::setCounterparty1);
			merger.mergeBasic(getNatureOfCounterparty1(), o.getNatureOfCounterparty1(), this::setNatureOfCounterparty1);
			merger.mergeBasic(getCorporateSectorOfTheCounterparty1(), o.getCorporateSectorOfTheCounterparty1(), this::setCorporateSectorOfTheCounterparty1);
			merger.mergeBasic(getClearingThresholdOfCounterparty1(), o.getClearingThresholdOfCounterparty1(), this::setClearingThresholdOfCounterparty1);
			merger.mergeBasic(getCounterparty2(), o.getCounterparty2(), this::setCounterparty2);
			merger.mergeBasic(getCountryOfTheCounterParty2(), o.getCountryOfTheCounterParty2(), this::setCountryOfTheCounterParty2);
			merger.mergeBasic(getNatureOfCounterparty2(), o.getNatureOfCounterparty2(), this::setNatureOfCounterparty2);
			merger.mergeBasic(getCorporateSectorOfTheCounterparty2(), o.getCorporateSectorOfTheCounterparty2(), this::setCorporateSectorOfTheCounterparty2);
			merger.mergeBasic(getCounterparty2IdentifierType(), o.getCounterparty2IdentifierType(), this::setCounterparty2IdentifierType);
			merger.mergeBasic(getClearingMember(), o.getClearingMember(), this::setClearingMember);
			merger.mergeBasic(getClearingThresholdOfCounterparty2(), o.getClearingThresholdOfCounterparty2(), this::setClearingThresholdOfCounterparty2);
			merger.mergeBasic(getReportingObligationOfTheCounterparty2(), o.getReportingObligationOfTheCounterparty2(), this::setReportingObligationOfTheCounterparty2);
			merger.mergeBasic(getBrokerID(), o.getBrokerID(), this::setBrokerID);
			merger.mergeBasic(getDirection(), o.getDirection(), this::setDirection);
			merger.mergeBasic(getDirectionOfLeg1(), o.getDirectionOfLeg1(), this::setDirectionOfLeg1);
			merger.mergeBasic(getDirectionOfLeg2(), o.getDirectionOfLeg2(), this::setDirectionOfLeg2);
			merger.mergeBasic(getDirectlyLinkedToCommercialActivityOrTreasuryFinancing(), o.getDirectlyLinkedToCommercialActivityOrTreasuryFinancing(), this::setDirectlyLinkedToCommercialActivityOrTreasuryFinancing);
			merger.mergeBasic(getUti(), o.getUti(), this::setUti);
			merger.mergeBasic(getUtiProprietary(), o.getUtiProprietary(), this::setUtiProprietary);
			merger.mergeBasic(getReportTrackingNumber(), o.getReportTrackingNumber(), this::setReportTrackingNumber);
			merger.mergeBasic(getPriorUti(), o.getPriorUti(), this::setPriorUti);
			merger.mergeBasic(getPriorUtiProprietary(), o.getPriorUtiProprietary(), this::setPriorUtiProprietary);
			merger.mergeBasic(getPtrrId(), o.getPtrrId(), this::setPtrrId);
			merger.mergeBasic(getPackageIdentifier(), o.getPackageIdentifier(), this::setPackageIdentifier);
			merger.mergeBasic(getSubsequentPositionUti(), o.getSubsequentPositionUti(), this::setSubsequentPositionUti);
			merger.mergeBasic(getIsin(), o.getIsin(), this::setIsin);
			merger.mergeBasic(getUpi(), o.getUpi(), this::setUpi);
			merger.mergeBasic(getProductClassification(), o.getProductClassification(), this::setProductClassification);
			merger.mergeBasic(getContractType(), o.getContractType(), this::setContractType);
			merger.mergeBasic(getAssetClass(), o.getAssetClass(), this::setAssetClass);
			merger.mergeBasic(getIsCrypto(), o.getIsCrypto(), this::setIsCrypto);
			merger.mergeBasic(getUnderlyingIdentificationType(), o.getUnderlyingIdentificationType(), this::setUnderlyingIdentificationType);
			merger.mergeBasic(getUnderlyingIdentification(), o.getUnderlyingIdentification(), this::setUnderlyingIdentification);
			merger.mergeBasic(getNameOfTheUnderlyingIndex(), o.getNameOfTheUnderlyingIndex(), (Consumer<String>) this::addNameOfTheUnderlyingIndex);
			merger.mergeBasic(getIndicatorOfTheUnderlyingIndex(), o.getIndicatorOfTheUnderlyingIndex(), (Consumer<String>) this::addIndicatorOfTheUnderlyingIndex);
			merger.mergeBasic(getCustomBasketCode(), o.getCustomBasketCode(), this::setCustomBasketCode);
			merger.mergeBasic(getSettlementCurrency1(), o.getSettlementCurrency1(), this::setSettlementCurrency1);
			merger.mergeBasic(getSettlementCurrency2(), o.getSettlementCurrency2(), this::setSettlementCurrency2);
			merger.mergeBasic(getValuationAmount(), o.getValuationAmount(), this::setValuationAmount);
			merger.mergeBasic(getValuationCurrency(), o.getValuationCurrency(), this::setValuationCurrency);
			merger.mergeBasic(getValuationMethod(), o.getValuationMethod(), this::setValuationMethod);
			merger.mergeBasic(getDelta(), o.getDelta(), this::setDelta);
			merger.mergeBasic(getValuationTimestamp(), o.getValuationTimestamp(), this::setValuationTimestamp);
			merger.mergeBasic(getConfirmationTimestamp(), o.getConfirmationTimestamp(), this::setConfirmationTimestamp);
			merger.mergeBasic(getConfirmed(), o.getConfirmed(), this::setConfirmed);
			merger.mergeBasic(getClearingObligation(), o.getClearingObligation(), this::setClearingObligation);
			merger.mergeBasic(getCleared(), o.getCleared(), this::setCleared);
			merger.mergeBasic(getClearingTimestamp(), o.getClearingTimestamp(), this::setClearingTimestamp);
			merger.mergeBasic(getCentralCounterparty(), o.getCentralCounterparty(), this::setCentralCounterparty);
			merger.mergeBasic(getMasterAgreementType(), o.getMasterAgreementType(), this::setMasterAgreementType);
			merger.mergeBasic(getOtherMasterAgreementType(), o.getOtherMasterAgreementType(), this::setOtherMasterAgreementType);
			merger.mergeBasic(getMasterAgreementVersion(), o.getMasterAgreementVersion(), this::setMasterAgreementVersion);
			merger.mergeBasic(getIntragroup(), o.getIntragroup(), this::setIntragroup);
			merger.mergeBasic(getPtrr(), o.getPtrr(), this::setPtrr);
			merger.mergeBasic(getTypeOfPTRRTechnique(), o.getTypeOfPTRRTechnique(), this::setTypeOfPTRRTechnique);
			merger.mergeBasic(getPtrrServiceProvider(), o.getPtrrServiceProvider(), this::setPtrrServiceProvider);
			merger.mergeBasic(getVenueOfExecution(), o.getVenueOfExecution(), this::setVenueOfExecution);
			merger.mergeBasic(getExecutionTimestamp(), o.getExecutionTimestamp(), this::setExecutionTimestamp);
			merger.mergeBasic(getEffectiveDate(), o.getEffectiveDate(), this::setEffectiveDate);
			merger.mergeBasic(getExpirationDate(), o.getExpirationDate(), this::setExpirationDate);
			merger.mergeBasic(getEarlyTerminationDate(), o.getEarlyTerminationDate(), this::setEarlyTerminationDate);
			merger.mergeBasic(getFinalContractualSettlementDate(), o.getFinalContractualSettlementDate(), this::setFinalContractualSettlementDate);
			merger.mergeBasic(getDeliveryType(), o.getDeliveryType(), this::setDeliveryType);
			merger.mergeBasic(getNotionalLeg1(), o.getNotionalLeg1(), this::setNotionalLeg1);
			merger.mergeBasic(getNotionalCurrency1(), o.getNotionalCurrency1(), this::setNotionalCurrency1);
			merger.mergeBasic(getTotalNotionalQuantityLeg1(), o.getTotalNotionalQuantityLeg1(), this::setTotalNotionalQuantityLeg1);
			merger.mergeBasic(getTotalNotionalQuantityLeg2(), o.getTotalNotionalQuantityLeg2(), this::setTotalNotionalQuantityLeg2);
			merger.mergeBasic(getNotionalLeg2(), o.getNotionalLeg2(), this::setNotionalLeg2);
			merger.mergeBasic(getNotionalCurrency2(), o.getNotionalCurrency2(), this::setNotionalCurrency2);
			merger.mergeBasic(getFixedRateOfLeg1(), o.getFixedRateOfLeg1(), this::setFixedRateOfLeg1);
			merger.mergeBasic(getFixedRateDayCountConventionLeg1(), o.getFixedRateDayCountConventionLeg1(), this::setFixedRateDayCountConventionLeg1);
			merger.mergeBasic(getFixedRatePaymentFrequencyPeriodLeg1(), o.getFixedRatePaymentFrequencyPeriodLeg1(), this::setFixedRatePaymentFrequencyPeriodLeg1);
			merger.mergeBasic(getFixedRatePaymentFrequencyPeriodMultiplierLeg1(), o.getFixedRatePaymentFrequencyPeriodMultiplierLeg1(), this::setFixedRatePaymentFrequencyPeriodMultiplierLeg1);
			merger.mergeBasic(getIdentifierOfFloatingRateOfLeg1(), o.getIdentifierOfFloatingRateOfLeg1(), this::setIdentifierOfFloatingRateOfLeg1);
			merger.mergeBasic(getIndicatorOfTheFloatingRateOfLeg1(), o.getIndicatorOfTheFloatingRateOfLeg1(), this::setIndicatorOfTheFloatingRateOfLeg1);
			merger.mergeBasic(getNameOfTheFloatingRateOfLeg1(), o.getNameOfTheFloatingRateOfLeg1(), this::setNameOfTheFloatingRateOfLeg1);
			merger.mergeBasic(getFloatingRateDayCountConventionOfLeg1(), o.getFloatingRateDayCountConventionOfLeg1(), this::setFloatingRateDayCountConventionOfLeg1);
			merger.mergeBasic(getFloatingRatePaymentFrequencyPeriodOfLeg1(), o.getFloatingRatePaymentFrequencyPeriodOfLeg1(), this::setFloatingRatePaymentFrequencyPeriodOfLeg1);
			merger.mergeBasic(getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1(), o.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1(), this::setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1);
			merger.mergeBasic(getFloatingRateReferencePeriodOfLeg1TimePeriod(), o.getFloatingRateReferencePeriodOfLeg1TimePeriod(), this::setFloatingRateReferencePeriodOfLeg1TimePeriod);
			merger.mergeBasic(getFloatingRateReferencePeriodOfLeg1Multiplier(), o.getFloatingRateReferencePeriodOfLeg1Multiplier(), this::setFloatingRateReferencePeriodOfLeg1Multiplier);
			merger.mergeBasic(getFloatingRateResetFrequencyPeriodOfLeg1(), o.getFloatingRateResetFrequencyPeriodOfLeg1(), this::setFloatingRateResetFrequencyPeriodOfLeg1);
			merger.mergeBasic(getFloatingRateResetFrequencyPeriodMultiplierLeg1(), o.getFloatingRateResetFrequencyPeriodMultiplierLeg1(), this::setFloatingRateResetFrequencyPeriodMultiplierLeg1);
			merger.mergeBasic(getFixedRateOfLeg2(), o.getFixedRateOfLeg2(), this::setFixedRateOfLeg2);
			merger.mergeBasic(getFixedRateDayCountConventionLeg2(), o.getFixedRateDayCountConventionLeg2(), this::setFixedRateDayCountConventionLeg2);
			merger.mergeBasic(getFixedRatePaymentFrequencyPeriodLeg2(), o.getFixedRatePaymentFrequencyPeriodLeg2(), this::setFixedRatePaymentFrequencyPeriodLeg2);
			merger.mergeBasic(getFixedRatePaymentFrequencyPeriodMultiplierLeg2(), o.getFixedRatePaymentFrequencyPeriodMultiplierLeg2(), this::setFixedRatePaymentFrequencyPeriodMultiplierLeg2);
			merger.mergeBasic(getIdentifierOfFloatingRateOfLeg2(), o.getIdentifierOfFloatingRateOfLeg2(), this::setIdentifierOfFloatingRateOfLeg2);
			merger.mergeBasic(getIndicatorOfTheFloatingRateOfLeg2(), o.getIndicatorOfTheFloatingRateOfLeg2(), this::setIndicatorOfTheFloatingRateOfLeg2);
			merger.mergeBasic(getNameOfTheFloatingRateOfLeg2(), o.getNameOfTheFloatingRateOfLeg2(), this::setNameOfTheFloatingRateOfLeg2);
			merger.mergeBasic(getFloatingRateDayCountConventionOfLeg2(), o.getFloatingRateDayCountConventionOfLeg2(), this::setFloatingRateDayCountConventionOfLeg2);
			merger.mergeBasic(getFloatingRatePaymentFrequencyPeriodOfLeg2(), o.getFloatingRatePaymentFrequencyPeriodOfLeg2(), this::setFloatingRatePaymentFrequencyPeriodOfLeg2);
			merger.mergeBasic(getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2(), o.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2(), this::setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2);
			merger.mergeBasic(getFloatingRateReferencePeriodOfLeg2TimePeriod(), o.getFloatingRateReferencePeriodOfLeg2TimePeriod(), this::setFloatingRateReferencePeriodOfLeg2TimePeriod);
			merger.mergeBasic(getFloatingRateReferencePeriodOfLeg2Multiplier(), o.getFloatingRateReferencePeriodOfLeg2Multiplier(), this::setFloatingRateReferencePeriodOfLeg2Multiplier);
			merger.mergeBasic(getFloatingRateResetFrequencyPeriodOfLeg2(), o.getFloatingRateResetFrequencyPeriodOfLeg2(), this::setFloatingRateResetFrequencyPeriodOfLeg2);
			merger.mergeBasic(getFloatingRateResetFrequencyPeriodMultiplierLeg2(), o.getFloatingRateResetFrequencyPeriodMultiplierLeg2(), this::setFloatingRateResetFrequencyPeriodMultiplierLeg2);
			merger.mergeBasic(getExchangeRate(), o.getExchangeRate(), this::setExchangeRate);
			merger.mergeBasic(getForwardExchangeRate(), o.getForwardExchangeRate(), this::setForwardExchangeRate);
			merger.mergeBasic(getExchangeRateBasis(), o.getExchangeRateBasis(), this::setExchangeRateBasis);
			merger.mergeBasic(getBaseProduct(), o.getBaseProduct(), this::setBaseProduct);
			merger.mergeBasic(getSubProduct(), o.getSubProduct(), this::setSubProduct);
			merger.mergeBasic(getFurtherSubProduct(), o.getFurtherSubProduct(), this::setFurtherSubProduct);
			merger.mergeBasic(getDeliveryPoint(), o.getDeliveryPoint(), (Consumer<String>) this::addDeliveryPoint);
			merger.mergeBasic(getInterconnectionPoint(), o.getInterconnectionPoint(), this::setInterconnectionPoint);
			merger.mergeBasic(getLoadType(), o.getLoadType(), this::setLoadType);
			merger.mergeBasic(getOptionType(), o.getOptionType(), this::setOptionType);
			merger.mergeBasic(getOptionStyle(), o.getOptionStyle(), this::setOptionStyle);
			merger.mergeBasic(getOptionPremiumAmount(), o.getOptionPremiumAmount(), this::setOptionPremiumAmount);
			merger.mergeBasic(getOptionPremiumCurrency(), o.getOptionPremiumCurrency(), this::setOptionPremiumCurrency);
			merger.mergeBasic(getOptionPremiumPaymentDate(), o.getOptionPremiumPaymentDate(), this::setOptionPremiumPaymentDate);
			merger.mergeBasic(getMaturityDateOfTheUnderlying(), o.getMaturityDateOfTheUnderlying(), this::setMaturityDateOfTheUnderlying);
			merger.mergeBasic(getSeniority(), o.getSeniority(), this::setSeniority);
			merger.mergeBasic(getReferenceEntity(), o.getReferenceEntity(), this::setReferenceEntity);
			merger.mergeBasic(getSeries(), o.getSeries(), this::setSeries);
			merger.mergeBasic(getVersion(), o.getVersion(), this::setVersion);
			merger.mergeBasic(getIndexFactor(), o.getIndexFactor(), this::setIndexFactor);
			merger.mergeBasic(getTranche(), o.getTranche(), this::setTranche);
			merger.mergeBasic(getCdSIndexAttachmentPoint(), o.getCdSIndexAttachmentPoint(), this::setCdSIndexAttachmentPoint);
			merger.mergeBasic(getCdSIndexDetachmentPoint(), o.getCdSIndexDetachmentPoint(), this::setCdSIndexDetachmentPoint);
			merger.mergeBasic(getCollateralPortfolioIndicator(), o.getCollateralPortfolioIndicator(), this::setCollateralPortfolioIndicator);
			merger.mergeBasic(getCollateralPortfolioCode(), o.getCollateralPortfolioCode(), this::setCollateralPortfolioCode);
			merger.mergeBasic(getActionType(), o.getActionType(), this::setActionType);
			merger.mergeBasic(getEventType(), o.getEventType(), this::setEventType);
			merger.mergeBasic(getEventDate(), o.getEventDate(), this::setEventDate);
			merger.mergeBasic(getLevel(), o.getLevel(), this::setLevel);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			ESMAEMIRTransactionReport _that = getType().cast(o);
		
			if (!Objects.equals(reportingTimestamp, _that.getReportingTimestamp())) return false;
			if (!Objects.equals(reportSubmittingEntityID, _that.getReportSubmittingEntityID())) return false;
			if (!Objects.equals(entityResponsibleForReporting, _that.getEntityResponsibleForReporting())) return false;
			if (!Objects.equals(counterparty1, _that.getCounterparty1())) return false;
			if (!Objects.equals(natureOfCounterparty1, _that.getNatureOfCounterparty1())) return false;
			if (!Objects.equals(corporateSectorOfTheCounterparty1, _that.getCorporateSectorOfTheCounterparty1())) return false;
			if (!Objects.equals(clearingThresholdOfCounterparty1, _that.getClearingThresholdOfCounterparty1())) return false;
			if (!Objects.equals(counterparty2, _that.getCounterparty2())) return false;
			if (!Objects.equals(countryOfTheCounterParty2, _that.getCountryOfTheCounterParty2())) return false;
			if (!Objects.equals(natureOfCounterparty2, _that.getNatureOfCounterparty2())) return false;
			if (!Objects.equals(corporateSectorOfTheCounterparty2, _that.getCorporateSectorOfTheCounterparty2())) return false;
			if (!Objects.equals(counterparty2IdentifierType, _that.getCounterparty2IdentifierType())) return false;
			if (!Objects.equals(clearingMember, _that.getClearingMember())) return false;
			if (!Objects.equals(clearingThresholdOfCounterparty2, _that.getClearingThresholdOfCounterparty2())) return false;
			if (!Objects.equals(reportingObligationOfTheCounterparty2, _that.getReportingObligationOfTheCounterparty2())) return false;
			if (!Objects.equals(brokerID, _that.getBrokerID())) return false;
			if (!Objects.equals(direction, _that.getDirection())) return false;
			if (!Objects.equals(directionOfLeg1, _that.getDirectionOfLeg1())) return false;
			if (!Objects.equals(directionOfLeg2, _that.getDirectionOfLeg2())) return false;
			if (!Objects.equals(directlyLinkedToCommercialActivityOrTreasuryFinancing, _that.getDirectlyLinkedToCommercialActivityOrTreasuryFinancing())) return false;
			if (!Objects.equals(uti, _that.getUti())) return false;
			if (!Objects.equals(utiProprietary, _that.getUtiProprietary())) return false;
			if (!Objects.equals(reportTrackingNumber, _that.getReportTrackingNumber())) return false;
			if (!Objects.equals(priorUti, _that.getPriorUti())) return false;
			if (!Objects.equals(priorUtiProprietary, _that.getPriorUtiProprietary())) return false;
			if (!Objects.equals(ptrrId, _that.getPtrrId())) return false;
			if (!Objects.equals(packageIdentifier, _that.getPackageIdentifier())) return false;
			if (!Objects.equals(subsequentPositionUti, _that.getSubsequentPositionUti())) return false;
			if (!Objects.equals(isin, _that.getIsin())) return false;
			if (!Objects.equals(upi, _that.getUpi())) return false;
			if (!Objects.equals(productClassification, _that.getProductClassification())) return false;
			if (!Objects.equals(contractType, _that.getContractType())) return false;
			if (!Objects.equals(assetClass, _that.getAssetClass())) return false;
			if (!Objects.equals(isCrypto, _that.getIsCrypto())) return false;
			if (!Objects.equals(underlyingIdentificationType, _that.getUnderlyingIdentificationType())) return false;
			if (!Objects.equals(underlyingIdentification, _that.getUnderlyingIdentification())) return false;
			if (!ListEquals.listEquals(nameOfTheUnderlyingIndex, _that.getNameOfTheUnderlyingIndex())) return false;
			if (!ListEquals.listEquals(indicatorOfTheUnderlyingIndex, _that.getIndicatorOfTheUnderlyingIndex())) return false;
			if (!ListEquals.listEquals(basketConstituents, _that.getBasketConstituents())) return false;
			if (!Objects.equals(customBasketCode, _that.getCustomBasketCode())) return false;
			if (!Objects.equals(settlementCurrency1, _that.getSettlementCurrency1())) return false;
			if (!Objects.equals(settlementCurrency2, _that.getSettlementCurrency2())) return false;
			if (!Objects.equals(valuationAmount, _that.getValuationAmount())) return false;
			if (!Objects.equals(valuationCurrency, _that.getValuationCurrency())) return false;
			if (!Objects.equals(valuationMethod, _that.getValuationMethod())) return false;
			if (!Objects.equals(delta, _that.getDelta())) return false;
			if (!Objects.equals(valuationTimestamp, _that.getValuationTimestamp())) return false;
			if (!Objects.equals(confirmationTimestamp, _that.getConfirmationTimestamp())) return false;
			if (!Objects.equals(confirmed, _that.getConfirmed())) return false;
			if (!Objects.equals(clearingObligation, _that.getClearingObligation())) return false;
			if (!Objects.equals(cleared, _that.getCleared())) return false;
			if (!Objects.equals(clearingTimestamp, _that.getClearingTimestamp())) return false;
			if (!Objects.equals(centralCounterparty, _that.getCentralCounterparty())) return false;
			if (!Objects.equals(masterAgreementType, _that.getMasterAgreementType())) return false;
			if (!Objects.equals(otherMasterAgreementType, _that.getOtherMasterAgreementType())) return false;
			if (!Objects.equals(masterAgreementVersion, _that.getMasterAgreementVersion())) return false;
			if (!Objects.equals(intragroup, _that.getIntragroup())) return false;
			if (!Objects.equals(ptrr, _that.getPtrr())) return false;
			if (!Objects.equals(typeOfPTRRTechnique, _that.getTypeOfPTRRTechnique())) return false;
			if (!Objects.equals(ptrrServiceProvider, _that.getPtrrServiceProvider())) return false;
			if (!Objects.equals(venueOfExecution, _that.getVenueOfExecution())) return false;
			if (!Objects.equals(executionTimestamp, _that.getExecutionTimestamp())) return false;
			if (!Objects.equals(effectiveDate, _that.getEffectiveDate())) return false;
			if (!Objects.equals(expirationDate, _that.getExpirationDate())) return false;
			if (!Objects.equals(earlyTerminationDate, _that.getEarlyTerminationDate())) return false;
			if (!Objects.equals(finalContractualSettlementDate, _that.getFinalContractualSettlementDate())) return false;
			if (!Objects.equals(deliveryType, _that.getDeliveryType())) return false;
			if (!Objects.equals(price, _that.getPrice())) return false;
			if (!ListEquals.listEquals(priceSchedule, _that.getPriceSchedule())) return false;
			if (!Objects.equals(packageTransactionPrice, _that.getPackageTransactionPrice())) return false;
			if (!Objects.equals(notionalLeg1, _that.getNotionalLeg1())) return false;
			if (!Objects.equals(notionalCurrency1, _that.getNotionalCurrency1())) return false;
			if (!ListEquals.listEquals(notionalAmountScheduleLeg1, _that.getNotionalAmountScheduleLeg1())) return false;
			if (!Objects.equals(totalNotionalQuantityLeg1, _that.getTotalNotionalQuantityLeg1())) return false;
			if (!ListEquals.listEquals(notionalQuantityScheduleLeg1, _that.getNotionalQuantityScheduleLeg1())) return false;
			if (!Objects.equals(totalNotionalQuantityLeg2, _that.getTotalNotionalQuantityLeg2())) return false;
			if (!Objects.equals(notionalLeg2, _that.getNotionalLeg2())) return false;
			if (!Objects.equals(notionalCurrency2, _that.getNotionalCurrency2())) return false;
			if (!ListEquals.listEquals(notionalAmountScheduleLeg2, _that.getNotionalAmountScheduleLeg2())) return false;
			if (!ListEquals.listEquals(notionalQuantityScheduleLeg2, _that.getNotionalQuantityScheduleLeg2())) return false;
			if (!ListEquals.listEquals(otherPayment, _that.getOtherPayment())) return false;
			if (!Objects.equals(fixedRateOfLeg1, _that.getFixedRateOfLeg1())) return false;
			if (!Objects.equals(fixedRateDayCountConventionLeg1, _that.getFixedRateDayCountConventionLeg1())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodLeg1, _that.getFixedRatePaymentFrequencyPeriodLeg1())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodMultiplierLeg1, _that.getFixedRatePaymentFrequencyPeriodMultiplierLeg1())) return false;
			if (!Objects.equals(identifierOfFloatingRateOfLeg1, _that.getIdentifierOfFloatingRateOfLeg1())) return false;
			if (!Objects.equals(indicatorOfTheFloatingRateOfLeg1, _that.getIndicatorOfTheFloatingRateOfLeg1())) return false;
			if (!Objects.equals(nameOfTheFloatingRateOfLeg1, _that.getNameOfTheFloatingRateOfLeg1())) return false;
			if (!Objects.equals(floatingRateDayCountConventionOfLeg1, _that.getFloatingRateDayCountConventionOfLeg1())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodOfLeg1, _that.getFloatingRatePaymentFrequencyPeriodOfLeg1())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodMultiplierOfLeg1, _that.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1())) return false;
			if (!Objects.equals(floatingRateReferencePeriodOfLeg1TimePeriod, _that.getFloatingRateReferencePeriodOfLeg1TimePeriod())) return false;
			if (!Objects.equals(floatingRateReferencePeriodOfLeg1Multiplier, _that.getFloatingRateReferencePeriodOfLeg1Multiplier())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodOfLeg1, _that.getFloatingRateResetFrequencyPeriodOfLeg1())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodMultiplierLeg1, _that.getFloatingRateResetFrequencyPeriodMultiplierLeg1())) return false;
			if (!Objects.equals(spreadOfLeg1, _that.getSpreadOfLeg1())) return false;
			if (!Objects.equals(fixedRateOfLeg2, _that.getFixedRateOfLeg2())) return false;
			if (!Objects.equals(fixedRateDayCountConventionLeg2, _that.getFixedRateDayCountConventionLeg2())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodLeg2, _that.getFixedRatePaymentFrequencyPeriodLeg2())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodMultiplierLeg2, _that.getFixedRatePaymentFrequencyPeriodMultiplierLeg2())) return false;
			if (!Objects.equals(identifierOfFloatingRateOfLeg2, _that.getIdentifierOfFloatingRateOfLeg2())) return false;
			if (!Objects.equals(indicatorOfTheFloatingRateOfLeg2, _that.getIndicatorOfTheFloatingRateOfLeg2())) return false;
			if (!Objects.equals(nameOfTheFloatingRateOfLeg2, _that.getNameOfTheFloatingRateOfLeg2())) return false;
			if (!Objects.equals(floatingRateDayCountConventionOfLeg2, _that.getFloatingRateDayCountConventionOfLeg2())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodOfLeg2, _that.getFloatingRatePaymentFrequencyPeriodOfLeg2())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodMultiplierOfLeg2, _that.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2())) return false;
			if (!Objects.equals(floatingRateReferencePeriodOfLeg2TimePeriod, _that.getFloatingRateReferencePeriodOfLeg2TimePeriod())) return false;
			if (!Objects.equals(floatingRateReferencePeriodOfLeg2Multiplier, _that.getFloatingRateReferencePeriodOfLeg2Multiplier())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodOfLeg2, _that.getFloatingRateResetFrequencyPeriodOfLeg2())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodMultiplierLeg2, _that.getFloatingRateResetFrequencyPeriodMultiplierLeg2())) return false;
			if (!Objects.equals(spreadOfLeg2, _that.getSpreadOfLeg2())) return false;
			if (!Objects.equals(packageTransactionSpread, _that.getPackageTransactionSpread())) return false;
			if (!Objects.equals(exchangeRate, _that.getExchangeRate())) return false;
			if (!Objects.equals(forwardExchangeRate, _that.getForwardExchangeRate())) return false;
			if (!Objects.equals(exchangeRateBasis, _that.getExchangeRateBasis())) return false;
			if (!Objects.equals(baseProduct, _that.getBaseProduct())) return false;
			if (!Objects.equals(subProduct, _that.getSubProduct())) return false;
			if (!Objects.equals(furtherSubProduct, _that.getFurtherSubProduct())) return false;
			if (!ListEquals.listEquals(deliveryPoint, _that.getDeliveryPoint())) return false;
			if (!Objects.equals(interconnectionPoint, _that.getInterconnectionPoint())) return false;
			if (!Objects.equals(loadType, _that.getLoadType())) return false;
			if (!ListEquals.listEquals(deliveryReport, _that.getDeliveryReport())) return false;
			if (!Objects.equals(optionType, _that.getOptionType())) return false;
			if (!Objects.equals(optionStyle, _that.getOptionStyle())) return false;
			if (!Objects.equals(strikePrice, _that.getStrikePrice())) return false;
			if (!ListEquals.listEquals(strikePriceSchedule, _that.getStrikePriceSchedule())) return false;
			if (!Objects.equals(optionPremiumAmount, _that.getOptionPremiumAmount())) return false;
			if (!Objects.equals(optionPremiumCurrency, _that.getOptionPremiumCurrency())) return false;
			if (!Objects.equals(optionPremiumPaymentDate, _that.getOptionPremiumPaymentDate())) return false;
			if (!Objects.equals(maturityDateOfTheUnderlying, _that.getMaturityDateOfTheUnderlying())) return false;
			if (!Objects.equals(seniority, _that.getSeniority())) return false;
			if (!Objects.equals(referenceEntity, _that.getReferenceEntity())) return false;
			if (!Objects.equals(series, _that.getSeries())) return false;
			if (!Objects.equals(version, _that.getVersion())) return false;
			if (!Objects.equals(indexFactor, _that.getIndexFactor())) return false;
			if (!Objects.equals(tranche, _that.getTranche())) return false;
			if (!Objects.equals(cdSIndexAttachmentPoint, _that.getCdSIndexAttachmentPoint())) return false;
			if (!Objects.equals(cdSIndexDetachmentPoint, _that.getCdSIndexDetachmentPoint())) return false;
			if (!Objects.equals(collateralPortfolioIndicator, _that.getCollateralPortfolioIndicator())) return false;
			if (!Objects.equals(collateralPortfolioCode, _that.getCollateralPortfolioCode())) return false;
			if (!Objects.equals(actionType, _that.getActionType())) return false;
			if (!Objects.equals(eventType, _that.getEventType())) return false;
			if (!Objects.equals(eventDate, _that.getEventDate())) return false;
			if (!Objects.equals(level, _that.getLevel())) return false;
			if (!Objects.equals(nonReportable, _that.getNonReportable())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (reportingTimestamp != null ? reportingTimestamp.hashCode() : 0);
			_result = 31 * _result + (reportSubmittingEntityID != null ? reportSubmittingEntityID.hashCode() : 0);
			_result = 31 * _result + (entityResponsibleForReporting != null ? entityResponsibleForReporting.hashCode() : 0);
			_result = 31 * _result + (counterparty1 != null ? counterparty1.hashCode() : 0);
			_result = 31 * _result + (natureOfCounterparty1 != null ? natureOfCounterparty1.hashCode() : 0);
			_result = 31 * _result + (corporateSectorOfTheCounterparty1 != null ? corporateSectorOfTheCounterparty1.hashCode() : 0);
			_result = 31 * _result + (clearingThresholdOfCounterparty1 != null ? clearingThresholdOfCounterparty1.hashCode() : 0);
			_result = 31 * _result + (counterparty2 != null ? counterparty2.hashCode() : 0);
			_result = 31 * _result + (countryOfTheCounterParty2 != null ? countryOfTheCounterParty2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (natureOfCounterparty2 != null ? natureOfCounterparty2.hashCode() : 0);
			_result = 31 * _result + (corporateSectorOfTheCounterparty2 != null ? corporateSectorOfTheCounterparty2.hashCode() : 0);
			_result = 31 * _result + (counterparty2IdentifierType != null ? counterparty2IdentifierType.hashCode() : 0);
			_result = 31 * _result + (clearingMember != null ? clearingMember.hashCode() : 0);
			_result = 31 * _result + (clearingThresholdOfCounterparty2 != null ? clearingThresholdOfCounterparty2.hashCode() : 0);
			_result = 31 * _result + (reportingObligationOfTheCounterparty2 != null ? reportingObligationOfTheCounterparty2.hashCode() : 0);
			_result = 31 * _result + (brokerID != null ? brokerID.hashCode() : 0);
			_result = 31 * _result + (direction != null ? direction.hashCode() : 0);
			_result = 31 * _result + (directionOfLeg1 != null ? directionOfLeg1.hashCode() : 0);
			_result = 31 * _result + (directionOfLeg2 != null ? directionOfLeg2.hashCode() : 0);
			_result = 31 * _result + (directlyLinkedToCommercialActivityOrTreasuryFinancing != null ? directlyLinkedToCommercialActivityOrTreasuryFinancing.hashCode() : 0);
			_result = 31 * _result + (uti != null ? uti.hashCode() : 0);
			_result = 31 * _result + (utiProprietary != null ? utiProprietary.hashCode() : 0);
			_result = 31 * _result + (reportTrackingNumber != null ? reportTrackingNumber.hashCode() : 0);
			_result = 31 * _result + (priorUti != null ? priorUti.hashCode() : 0);
			_result = 31 * _result + (priorUtiProprietary != null ? priorUtiProprietary.hashCode() : 0);
			_result = 31 * _result + (ptrrId != null ? ptrrId.hashCode() : 0);
			_result = 31 * _result + (packageIdentifier != null ? packageIdentifier.hashCode() : 0);
			_result = 31 * _result + (subsequentPositionUti != null ? subsequentPositionUti.hashCode() : 0);
			_result = 31 * _result + (isin != null ? isin.hashCode() : 0);
			_result = 31 * _result + (upi != null ? upi.hashCode() : 0);
			_result = 31 * _result + (productClassification != null ? productClassification.hashCode() : 0);
			_result = 31 * _result + (contractType != null ? contractType.hashCode() : 0);
			_result = 31 * _result + (assetClass != null ? assetClass.hashCode() : 0);
			_result = 31 * _result + (isCrypto != null ? isCrypto.hashCode() : 0);
			_result = 31 * _result + (underlyingIdentificationType != null ? underlyingIdentificationType.hashCode() : 0);
			_result = 31 * _result + (underlyingIdentification != null ? underlyingIdentification.hashCode() : 0);
			_result = 31 * _result + (nameOfTheUnderlyingIndex != null ? nameOfTheUnderlyingIndex.hashCode() : 0);
			_result = 31 * _result + (indicatorOfTheUnderlyingIndex != null ? indicatorOfTheUnderlyingIndex.hashCode() : 0);
			_result = 31 * _result + (basketConstituents != null ? basketConstituents.hashCode() : 0);
			_result = 31 * _result + (customBasketCode != null ? customBasketCode.hashCode() : 0);
			_result = 31 * _result + (settlementCurrency1 != null ? settlementCurrency1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (settlementCurrency2 != null ? settlementCurrency2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (valuationAmount != null ? valuationAmount.hashCode() : 0);
			_result = 31 * _result + (valuationCurrency != null ? valuationCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (valuationMethod != null ? valuationMethod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (delta != null ? delta.hashCode() : 0);
			_result = 31 * _result + (valuationTimestamp != null ? valuationTimestamp.hashCode() : 0);
			_result = 31 * _result + (confirmationTimestamp != null ? confirmationTimestamp.hashCode() : 0);
			_result = 31 * _result + (confirmed != null ? confirmed.hashCode() : 0);
			_result = 31 * _result + (clearingObligation != null ? clearingObligation.hashCode() : 0);
			_result = 31 * _result + (cleared != null ? cleared.hashCode() : 0);
			_result = 31 * _result + (clearingTimestamp != null ? clearingTimestamp.hashCode() : 0);
			_result = 31 * _result + (centralCounterparty != null ? centralCounterparty.hashCode() : 0);
			_result = 31 * _result + (masterAgreementType != null ? masterAgreementType.hashCode() : 0);
			_result = 31 * _result + (otherMasterAgreementType != null ? otherMasterAgreementType.hashCode() : 0);
			_result = 31 * _result + (masterAgreementVersion != null ? masterAgreementVersion.hashCode() : 0);
			_result = 31 * _result + (intragroup != null ? intragroup.hashCode() : 0);
			_result = 31 * _result + (ptrr != null ? ptrr.hashCode() : 0);
			_result = 31 * _result + (typeOfPTRRTechnique != null ? typeOfPTRRTechnique.hashCode() : 0);
			_result = 31 * _result + (ptrrServiceProvider != null ? ptrrServiceProvider.hashCode() : 0);
			_result = 31 * _result + (venueOfExecution != null ? venueOfExecution.hashCode() : 0);
			_result = 31 * _result + (executionTimestamp != null ? executionTimestamp.hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (expirationDate != null ? expirationDate.hashCode() : 0);
			_result = 31 * _result + (earlyTerminationDate != null ? earlyTerminationDate.hashCode() : 0);
			_result = 31 * _result + (finalContractualSettlementDate != null ? finalContractualSettlementDate.hashCode() : 0);
			_result = 31 * _result + (deliveryType != null ? deliveryType.hashCode() : 0);
			_result = 31 * _result + (price != null ? price.hashCode() : 0);
			_result = 31 * _result + (priceSchedule != null ? priceSchedule.hashCode() : 0);
			_result = 31 * _result + (packageTransactionPrice != null ? packageTransactionPrice.hashCode() : 0);
			_result = 31 * _result + (notionalLeg1 != null ? notionalLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalCurrency1 != null ? notionalCurrency1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalAmountScheduleLeg1 != null ? notionalAmountScheduleLeg1.hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityLeg1 != null ? totalNotionalQuantityLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalQuantityScheduleLeg1 != null ? notionalQuantityScheduleLeg1.hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityLeg2 != null ? totalNotionalQuantityLeg2.hashCode() : 0);
			_result = 31 * _result + (notionalLeg2 != null ? notionalLeg2.hashCode() : 0);
			_result = 31 * _result + (notionalCurrency2 != null ? notionalCurrency2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalAmountScheduleLeg2 != null ? notionalAmountScheduleLeg2.hashCode() : 0);
			_result = 31 * _result + (notionalQuantityScheduleLeg2 != null ? notionalQuantityScheduleLeg2.hashCode() : 0);
			_result = 31 * _result + (otherPayment != null ? otherPayment.hashCode() : 0);
			_result = 31 * _result + (fixedRateOfLeg1 != null ? fixedRateOfLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRateDayCountConventionLeg1 != null ? fixedRateDayCountConventionLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodLeg1 != null ? fixedRatePaymentFrequencyPeriodLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodMultiplierLeg1 != null ? fixedRatePaymentFrequencyPeriodMultiplierLeg1.hashCode() : 0);
			_result = 31 * _result + (identifierOfFloatingRateOfLeg1 != null ? identifierOfFloatingRateOfLeg1.hashCode() : 0);
			_result = 31 * _result + (indicatorOfTheFloatingRateOfLeg1 != null ? indicatorOfTheFloatingRateOfLeg1.hashCode() : 0);
			_result = 31 * _result + (nameOfTheFloatingRateOfLeg1 != null ? nameOfTheFloatingRateOfLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRateDayCountConventionOfLeg1 != null ? floatingRateDayCountConventionOfLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodOfLeg1 != null ? floatingRatePaymentFrequencyPeriodOfLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodMultiplierOfLeg1 != null ? floatingRatePaymentFrequencyPeriodMultiplierOfLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRateReferencePeriodOfLeg1TimePeriod != null ? floatingRateReferencePeriodOfLeg1TimePeriod.hashCode() : 0);
			_result = 31 * _result + (floatingRateReferencePeriodOfLeg1Multiplier != null ? floatingRateReferencePeriodOfLeg1Multiplier.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodOfLeg1 != null ? floatingRateResetFrequencyPeriodOfLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodMultiplierLeg1 != null ? floatingRateResetFrequencyPeriodMultiplierLeg1.hashCode() : 0);
			_result = 31 * _result + (spreadOfLeg1 != null ? spreadOfLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRateOfLeg2 != null ? fixedRateOfLeg2.hashCode() : 0);
			_result = 31 * _result + (fixedRateDayCountConventionLeg2 != null ? fixedRateDayCountConventionLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodLeg2 != null ? fixedRatePaymentFrequencyPeriodLeg2.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodMultiplierLeg2 != null ? fixedRatePaymentFrequencyPeriodMultiplierLeg2.hashCode() : 0);
			_result = 31 * _result + (identifierOfFloatingRateOfLeg2 != null ? identifierOfFloatingRateOfLeg2.hashCode() : 0);
			_result = 31 * _result + (indicatorOfTheFloatingRateOfLeg2 != null ? indicatorOfTheFloatingRateOfLeg2.hashCode() : 0);
			_result = 31 * _result + (nameOfTheFloatingRateOfLeg2 != null ? nameOfTheFloatingRateOfLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRateDayCountConventionOfLeg2 != null ? floatingRateDayCountConventionOfLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodOfLeg2 != null ? floatingRatePaymentFrequencyPeriodOfLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 != null ? floatingRatePaymentFrequencyPeriodMultiplierOfLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRateReferencePeriodOfLeg2TimePeriod != null ? floatingRateReferencePeriodOfLeg2TimePeriod.hashCode() : 0);
			_result = 31 * _result + (floatingRateReferencePeriodOfLeg2Multiplier != null ? floatingRateReferencePeriodOfLeg2Multiplier.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodOfLeg2 != null ? floatingRateResetFrequencyPeriodOfLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodMultiplierLeg2 != null ? floatingRateResetFrequencyPeriodMultiplierLeg2.hashCode() : 0);
			_result = 31 * _result + (spreadOfLeg2 != null ? spreadOfLeg2.hashCode() : 0);
			_result = 31 * _result + (packageTransactionSpread != null ? packageTransactionSpread.hashCode() : 0);
			_result = 31 * _result + (exchangeRate != null ? exchangeRate.hashCode() : 0);
			_result = 31 * _result + (forwardExchangeRate != null ? forwardExchangeRate.hashCode() : 0);
			_result = 31 * _result + (exchangeRateBasis != null ? exchangeRateBasis.hashCode() : 0);
			_result = 31 * _result + (baseProduct != null ? baseProduct.hashCode() : 0);
			_result = 31 * _result + (subProduct != null ? subProduct.hashCode() : 0);
			_result = 31 * _result + (furtherSubProduct != null ? furtherSubProduct.hashCode() : 0);
			_result = 31 * _result + (deliveryPoint != null ? deliveryPoint.hashCode() : 0);
			_result = 31 * _result + (interconnectionPoint != null ? interconnectionPoint.hashCode() : 0);
			_result = 31 * _result + (loadType != null ? loadType.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (deliveryReport != null ? deliveryReport.hashCode() : 0);
			_result = 31 * _result + (optionType != null ? optionType.hashCode() : 0);
			_result = 31 * _result + (optionStyle != null ? optionStyle.hashCode() : 0);
			_result = 31 * _result + (strikePrice != null ? strikePrice.hashCode() : 0);
			_result = 31 * _result + (strikePriceSchedule != null ? strikePriceSchedule.hashCode() : 0);
			_result = 31 * _result + (optionPremiumAmount != null ? optionPremiumAmount.hashCode() : 0);
			_result = 31 * _result + (optionPremiumCurrency != null ? optionPremiumCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (optionPremiumPaymentDate != null ? optionPremiumPaymentDate.hashCode() : 0);
			_result = 31 * _result + (maturityDateOfTheUnderlying != null ? maturityDateOfTheUnderlying.hashCode() : 0);
			_result = 31 * _result + (seniority != null ? seniority.hashCode() : 0);
			_result = 31 * _result + (referenceEntity != null ? referenceEntity.hashCode() : 0);
			_result = 31 * _result + (series != null ? series.hashCode() : 0);
			_result = 31 * _result + (version != null ? version.hashCode() : 0);
			_result = 31 * _result + (indexFactor != null ? indexFactor.hashCode() : 0);
			_result = 31 * _result + (tranche != null ? tranche.hashCode() : 0);
			_result = 31 * _result + (cdSIndexAttachmentPoint != null ? cdSIndexAttachmentPoint.hashCode() : 0);
			_result = 31 * _result + (cdSIndexDetachmentPoint != null ? cdSIndexDetachmentPoint.hashCode() : 0);
			_result = 31 * _result + (collateralPortfolioIndicator != null ? collateralPortfolioIndicator.hashCode() : 0);
			_result = 31 * _result + (collateralPortfolioCode != null ? collateralPortfolioCode.hashCode() : 0);
			_result = 31 * _result + (actionType != null ? actionType.hashCode() : 0);
			_result = 31 * _result + (eventType != null ? eventType.hashCode() : 0);
			_result = 31 * _result + (eventDate != null ? eventDate.hashCode() : 0);
			_result = 31 * _result + (level != null ? level.hashCode() : 0);
			_result = 31 * _result + (nonReportable != null ? nonReportable.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "ESMAEMIRTransactionReportBuilder {" +
				"reportingTimestamp=" + this.reportingTimestamp + ", " +
				"reportSubmittingEntityID=" + this.reportSubmittingEntityID + ", " +
				"entityResponsibleForReporting=" + this.entityResponsibleForReporting + ", " +
				"counterparty1=" + this.counterparty1 + ", " +
				"natureOfCounterparty1=" + this.natureOfCounterparty1 + ", " +
				"corporateSectorOfTheCounterparty1=" + this.corporateSectorOfTheCounterparty1 + ", " +
				"clearingThresholdOfCounterparty1=" + this.clearingThresholdOfCounterparty1 + ", " +
				"counterparty2=" + this.counterparty2 + ", " +
				"countryOfTheCounterParty2=" + this.countryOfTheCounterParty2 + ", " +
				"natureOfCounterparty2=" + this.natureOfCounterparty2 + ", " +
				"corporateSectorOfTheCounterparty2=" + this.corporateSectorOfTheCounterparty2 + ", " +
				"counterparty2IdentifierType=" + this.counterparty2IdentifierType + ", " +
				"clearingMember=" + this.clearingMember + ", " +
				"clearingThresholdOfCounterparty2=" + this.clearingThresholdOfCounterparty2 + ", " +
				"reportingObligationOfTheCounterparty2=" + this.reportingObligationOfTheCounterparty2 + ", " +
				"brokerID=" + this.brokerID + ", " +
				"direction=" + this.direction + ", " +
				"directionOfLeg1=" + this.directionOfLeg1 + ", " +
				"directionOfLeg2=" + this.directionOfLeg2 + ", " +
				"directlyLinkedToCommercialActivityOrTreasuryFinancing=" + this.directlyLinkedToCommercialActivityOrTreasuryFinancing + ", " +
				"uti=" + this.uti + ", " +
				"utiProprietary=" + this.utiProprietary + ", " +
				"reportTrackingNumber=" + this.reportTrackingNumber + ", " +
				"priorUti=" + this.priorUti + ", " +
				"priorUtiProprietary=" + this.priorUtiProprietary + ", " +
				"ptrrId=" + this.ptrrId + ", " +
				"packageIdentifier=" + this.packageIdentifier + ", " +
				"subsequentPositionUti=" + this.subsequentPositionUti + ", " +
				"isin=" + this.isin + ", " +
				"upi=" + this.upi + ", " +
				"productClassification=" + this.productClassification + ", " +
				"contractType=" + this.contractType + ", " +
				"assetClass=" + this.assetClass + ", " +
				"isCrypto=" + this.isCrypto + ", " +
				"underlyingIdentificationType=" + this.underlyingIdentificationType + ", " +
				"underlyingIdentification=" + this.underlyingIdentification + ", " +
				"nameOfTheUnderlyingIndex=" + this.nameOfTheUnderlyingIndex + ", " +
				"indicatorOfTheUnderlyingIndex=" + this.indicatorOfTheUnderlyingIndex + ", " +
				"basketConstituents=" + this.basketConstituents + ", " +
				"customBasketCode=" + this.customBasketCode + ", " +
				"settlementCurrency1=" + this.settlementCurrency1 + ", " +
				"settlementCurrency2=" + this.settlementCurrency2 + ", " +
				"valuationAmount=" + this.valuationAmount + ", " +
				"valuationCurrency=" + this.valuationCurrency + ", " +
				"valuationMethod=" + this.valuationMethod + ", " +
				"delta=" + this.delta + ", " +
				"valuationTimestamp=" + this.valuationTimestamp + ", " +
				"confirmationTimestamp=" + this.confirmationTimestamp + ", " +
				"confirmed=" + this.confirmed + ", " +
				"clearingObligation=" + this.clearingObligation + ", " +
				"cleared=" + this.cleared + ", " +
				"clearingTimestamp=" + this.clearingTimestamp + ", " +
				"centralCounterparty=" + this.centralCounterparty + ", " +
				"masterAgreementType=" + this.masterAgreementType + ", " +
				"otherMasterAgreementType=" + this.otherMasterAgreementType + ", " +
				"masterAgreementVersion=" + this.masterAgreementVersion + ", " +
				"intragroup=" + this.intragroup + ", " +
				"ptrr=" + this.ptrr + ", " +
				"typeOfPTRRTechnique=" + this.typeOfPTRRTechnique + ", " +
				"ptrrServiceProvider=" + this.ptrrServiceProvider + ", " +
				"venueOfExecution=" + this.venueOfExecution + ", " +
				"executionTimestamp=" + this.executionTimestamp + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"expirationDate=" + this.expirationDate + ", " +
				"earlyTerminationDate=" + this.earlyTerminationDate + ", " +
				"finalContractualSettlementDate=" + this.finalContractualSettlementDate + ", " +
				"deliveryType=" + this.deliveryType + ", " +
				"price=" + this.price + ", " +
				"priceSchedule=" + this.priceSchedule + ", " +
				"packageTransactionPrice=" + this.packageTransactionPrice + ", " +
				"notionalLeg1=" + this.notionalLeg1 + ", " +
				"notionalCurrency1=" + this.notionalCurrency1 + ", " +
				"notionalAmountScheduleLeg1=" + this.notionalAmountScheduleLeg1 + ", " +
				"totalNotionalQuantityLeg1=" + this.totalNotionalQuantityLeg1 + ", " +
				"notionalQuantityScheduleLeg1=" + this.notionalQuantityScheduleLeg1 + ", " +
				"totalNotionalQuantityLeg2=" + this.totalNotionalQuantityLeg2 + ", " +
				"notionalLeg2=" + this.notionalLeg2 + ", " +
				"notionalCurrency2=" + this.notionalCurrency2 + ", " +
				"notionalAmountScheduleLeg2=" + this.notionalAmountScheduleLeg2 + ", " +
				"notionalQuantityScheduleLeg2=" + this.notionalQuantityScheduleLeg2 + ", " +
				"otherPayment=" + this.otherPayment + ", " +
				"fixedRateOfLeg1=" + this.fixedRateOfLeg1 + ", " +
				"fixedRateDayCountConventionLeg1=" + this.fixedRateDayCountConventionLeg1 + ", " +
				"fixedRatePaymentFrequencyPeriodLeg1=" + this.fixedRatePaymentFrequencyPeriodLeg1 + ", " +
				"fixedRatePaymentFrequencyPeriodMultiplierLeg1=" + this.fixedRatePaymentFrequencyPeriodMultiplierLeg1 + ", " +
				"identifierOfFloatingRateOfLeg1=" + this.identifierOfFloatingRateOfLeg1 + ", " +
				"indicatorOfTheFloatingRateOfLeg1=" + this.indicatorOfTheFloatingRateOfLeg1 + ", " +
				"nameOfTheFloatingRateOfLeg1=" + this.nameOfTheFloatingRateOfLeg1 + ", " +
				"floatingRateDayCountConventionOfLeg1=" + this.floatingRateDayCountConventionOfLeg1 + ", " +
				"floatingRatePaymentFrequencyPeriodOfLeg1=" + this.floatingRatePaymentFrequencyPeriodOfLeg1 + ", " +
				"floatingRatePaymentFrequencyPeriodMultiplierOfLeg1=" + this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg1 + ", " +
				"floatingRateReferencePeriodOfLeg1TimePeriod=" + this.floatingRateReferencePeriodOfLeg1TimePeriod + ", " +
				"floatingRateReferencePeriodOfLeg1Multiplier=" + this.floatingRateReferencePeriodOfLeg1Multiplier + ", " +
				"floatingRateResetFrequencyPeriodOfLeg1=" + this.floatingRateResetFrequencyPeriodOfLeg1 + ", " +
				"floatingRateResetFrequencyPeriodMultiplierLeg1=" + this.floatingRateResetFrequencyPeriodMultiplierLeg1 + ", " +
				"spreadOfLeg1=" + this.spreadOfLeg1 + ", " +
				"fixedRateOfLeg2=" + this.fixedRateOfLeg2 + ", " +
				"fixedRateDayCountConventionLeg2=" + this.fixedRateDayCountConventionLeg2 + ", " +
				"fixedRatePaymentFrequencyPeriodLeg2=" + this.fixedRatePaymentFrequencyPeriodLeg2 + ", " +
				"fixedRatePaymentFrequencyPeriodMultiplierLeg2=" + this.fixedRatePaymentFrequencyPeriodMultiplierLeg2 + ", " +
				"identifierOfFloatingRateOfLeg2=" + this.identifierOfFloatingRateOfLeg2 + ", " +
				"indicatorOfTheFloatingRateOfLeg2=" + this.indicatorOfTheFloatingRateOfLeg2 + ", " +
				"nameOfTheFloatingRateOfLeg2=" + this.nameOfTheFloatingRateOfLeg2 + ", " +
				"floatingRateDayCountConventionOfLeg2=" + this.floatingRateDayCountConventionOfLeg2 + ", " +
				"floatingRatePaymentFrequencyPeriodOfLeg2=" + this.floatingRatePaymentFrequencyPeriodOfLeg2 + ", " +
				"floatingRatePaymentFrequencyPeriodMultiplierOfLeg2=" + this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 + ", " +
				"floatingRateReferencePeriodOfLeg2TimePeriod=" + this.floatingRateReferencePeriodOfLeg2TimePeriod + ", " +
				"floatingRateReferencePeriodOfLeg2Multiplier=" + this.floatingRateReferencePeriodOfLeg2Multiplier + ", " +
				"floatingRateResetFrequencyPeriodOfLeg2=" + this.floatingRateResetFrequencyPeriodOfLeg2 + ", " +
				"floatingRateResetFrequencyPeriodMultiplierLeg2=" + this.floatingRateResetFrequencyPeriodMultiplierLeg2 + ", " +
				"spreadOfLeg2=" + this.spreadOfLeg2 + ", " +
				"packageTransactionSpread=" + this.packageTransactionSpread + ", " +
				"exchangeRate=" + this.exchangeRate + ", " +
				"forwardExchangeRate=" + this.forwardExchangeRate + ", " +
				"exchangeRateBasis=" + this.exchangeRateBasis + ", " +
				"baseProduct=" + this.baseProduct + ", " +
				"subProduct=" + this.subProduct + ", " +
				"furtherSubProduct=" + this.furtherSubProduct + ", " +
				"deliveryPoint=" + this.deliveryPoint + ", " +
				"interconnectionPoint=" + this.interconnectionPoint + ", " +
				"loadType=" + this.loadType + ", " +
				"deliveryReport=" + this.deliveryReport + ", " +
				"optionType=" + this.optionType + ", " +
				"optionStyle=" + this.optionStyle + ", " +
				"strikePrice=" + this.strikePrice + ", " +
				"strikePriceSchedule=" + this.strikePriceSchedule + ", " +
				"optionPremiumAmount=" + this.optionPremiumAmount + ", " +
				"optionPremiumCurrency=" + this.optionPremiumCurrency + ", " +
				"optionPremiumPaymentDate=" + this.optionPremiumPaymentDate + ", " +
				"maturityDateOfTheUnderlying=" + this.maturityDateOfTheUnderlying + ", " +
				"seniority=" + this.seniority + ", " +
				"referenceEntity=" + this.referenceEntity + ", " +
				"series=" + this.series + ", " +
				"version=" + this.version + ", " +
				"indexFactor=" + this.indexFactor + ", " +
				"tranche=" + this.tranche + ", " +
				"cdSIndexAttachmentPoint=" + this.cdSIndexAttachmentPoint + ", " +
				"cdSIndexDetachmentPoint=" + this.cdSIndexDetachmentPoint + ", " +
				"collateralPortfolioIndicator=" + this.collateralPortfolioIndicator + ", " +
				"collateralPortfolioCode=" + this.collateralPortfolioCode + ", " +
				"actionType=" + this.actionType + ", " +
				"eventType=" + this.eventType + ", " +
				"eventDate=" + this.eventDate + ", " +
				"level=" + this.level + ", " +
				"nonReportable=" + this.nonReportable +
			'}';
		}
	}
}
