package drr.regulation.mas.rewrite.trade;

import cdm.base.staticdata.asset.common.ISOCurrencyCodeEnum;
import cdm.base.staticdata.asset.common.ProductIdTypeEnum;
import com.google.common.collect.ImmutableList;
import com.rosetta.model.lib.RosettaModelObject;
import com.rosetta.model.lib.RosettaModelObjectBuilder;
import com.rosetta.model.lib.annotations.RosettaAttribute;
import com.rosetta.model.lib.annotations.RosettaDataType;
import com.rosetta.model.lib.meta.RosettaMetaData;
import com.rosetta.model.lib.path.RosettaPath;
import com.rosetta.model.lib.process.BuilderMerger;
import com.rosetta.model.lib.process.BuilderProcessor;
import com.rosetta.model.lib.process.Processor;
import com.rosetta.model.lib.records.Date;
import com.rosetta.util.ListEquals;
import drr.regulation.mas.rewrite.trade.BasketConstituentsReport;
import drr.regulation.mas.rewrite.trade.MASTransactionReport;
import drr.regulation.mas.rewrite.trade.MASTransactionReport.MASTransactionReportBuilder;
import drr.regulation.mas.rewrite.trade.MASTransactionReport.MASTransactionReportBuilderImpl;
import drr.regulation.mas.rewrite.trade.MASTransactionReport.MASTransactionReportImpl;
import drr.regulation.mas.rewrite.trade.OtherPaymentReport;
import drr.regulation.mas.rewrite.trade.PriceScheduleReport;
import drr.regulation.mas.rewrite.trade.SpreadOfLeg1;
import drr.regulation.mas.rewrite.trade.SpreadOfLeg2;
import drr.regulation.mas.rewrite.trade.StrikePriceScheduleReport;
import drr.regulation.mas.rewrite.trade.meta.MASTransactionReportMeta;
import iso20022.auth030.esma.Frequency13Code__1;
import iso20022.auth030.esma.InterestComputationMethod4Code;
import iso20022.auth030.esma.ValuationType1Code;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static java.util.Optional.ofNullable;

/**
 * @version 4.2.1
 */
@RosettaDataType(value="MASTransactionReport", builder=MASTransactionReport.MASTransactionReportBuilderImpl.class, version="4.2.1")
public interface MASTransactionReport extends RosettaModelObject {

	MASTransactionReportMeta metaData = new MASTransactionReportMeta();

	/*********************** Getter Methods  ***********************/
	String getCounterparty1();
	String getCounterparty2();
	Boolean getCounterparty2IdentifierType();
	String getDirection();
	String getDirectionOfLeg1();
	String getDirectionOfLeg2();
	String getReportingSpecifiedPerson();
	String getDataSubmitter();
	String getUniqueTransactionIdentifier();
	String getUniqueProductIdentifier();
	String getPriorUTI();
	String getContractType();
	String getAssetClass();
	String getUnderlying();
	String getUnderlyingIdentificationType();
	String getUnderlyingIdOther();
	ProductIdTypeEnum getUnderlyingIdOtherSource();
	String getNameOfTheUnderlyingIndex();
	BigDecimal getFixedRateOfLeg1();
	String getIdentifierOfFloatingRateOfLeg1();
	String getNameOfTheFloatingRateOfLeg1();
	SpreadOfLeg1 getSpreadOfLeg1();
	BigDecimal getFixedRateOfLeg2();
	String getIdentifierOfFloatingRateOfLeg2();
	String getNameOfTheFloatingRateOfLeg2();
	SpreadOfLeg2 getSpreadOfLeg2();
	String getOptionType();
	String getOptionStyle();
	Date getFirstExerciseDate();
	Date getEffectiveDate();
	Date getExpirationDate();
	String getSwapLinkID();
	String getPackageIdentifier();
	String getDeliveryType();
	ISOCurrencyCodeEnum getSettlementCurrency1();
	ISOCurrencyCodeEnum getSettlementCurrency2();
	Date getFinalContractualSettlementDate();
	String getCleared();
	String getCentralCounterparty();
	String getClearingMember();
	ZonedDateTime getClearingTimestamp();
	String getPlatformIdentifier();
	String getReportingObligationOfSpecifiedPerson();
	String getBookingLocation();
	String getTraderLocation();
	String getConfirmed();
	ZonedDateTime getConfirmationTimestamp();
	ZonedDateTime getExecutionTimestamp();
	ZonedDateTime getReportingTimestamp();
	List<? extends BasketConstituentsReport> getBasketConstituents();
	BigDecimal getValuationAmount();
	ISOCurrencyCodeEnum getValuationCurrency();
	ValuationType1Code getValuationMethod();
	ZonedDateTime getValuationTimestamp();
	Boolean getCollateralPortfolioIndicator();
	String getInitialMarginCollateralPortfolioCode();
	String getVariationMarginCollateralPortfolioCode();
	BigDecimal getNotionalAmountOfTheContractOrLeg1();
	ISOCurrencyCodeEnum getNotionalCurrencyOfTheContractOrLeg1();
	BigDecimal getNotionalAmountOfLeg2();
	ISOCurrencyCodeEnum getNotionalCurrencyOfLeg2();
	BigDecimal getTotalNotionalQuantityOfTheContractOrLeg1();
	String getQuantityUnitOfMeasureForTheContractOrLeg1();
	BigDecimal getNotionalQuantityOfTheContractOrLeg1();
	String getQuantityFrequencyOfTheContractOrLeg1();
	Integer getQuantityFrequencyMultiplierOfTheContractOrLeg1();
	BigDecimal getTotalNotionalQuantityLeg2();
	String getQuantityUnitOfMeasureLeg2();
	BigDecimal getNotionalQuantityOfLeg2();
	String getQuantityFrequencyOfLeg2();
	Integer getQuantityFrequencyMultiplierOfLeg2();
	BigDecimal getCallAmount();
	ISOCurrencyCodeEnum getCallCurrency();
	BigDecimal getPutAmount();
	ISOCurrencyCodeEnum getPutCurrency();
	BigDecimal getPrice();
	ISOCurrencyCodeEnum getPriceCurrency();
	String getPriceNotation();
	String getPriceUnitOfMeasure();
	List<? extends PriceScheduleReport> getPriceSchedule();
	BigDecimal getStrikePrice();
	String getStrikePriceCurrency();
	String getStrikePriceNotation();
	List<? extends StrikePriceScheduleReport> getStrikePriceSchedule();
	BigDecimal getOptionPremiumAmount();
	ISOCurrencyCodeEnum getOptionPremiumCurrency();
	BigDecimal getExchangeRate();
	String getExchangeRateBasis();
	InterestComputationMethod4Code getFixedRateDayCountConventionOfLeg1();
	InterestComputationMethod4Code getFixedRateDayCountConventionOfLeg2();
	InterestComputationMethod4Code getFloatingRateDayCountConventionOfLeg1();
	InterestComputationMethod4Code getFloatingRateDayCountConventionOfLeg2();
	String getFloatingRateResetFrequencyPeriodOfLeg1();
	Integer getFloatingRateResetFrequencyPeriodMultiplierOfLeg1();
	String getFloatingRateResetFrequencyPeriodOfLeg2();
	Integer getFloatingRateResetFrequencyPeriodMultiplierOfLeg2();
	String getFixedRatePaymentFrequencyPeriodOfLeg1();
	Integer getFixedRatePaymentFrequencyPeriodMultiplierOfLeg1();
	String getFixedRatePaymentFrequencyPeriodOfLeg2();
	Integer getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2();
	Frequency13Code__1 getFloatingRatePaymentFrequencyPeriodOfLeg1();
	Integer getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1();
	Frequency13Code__1 getFloatingRatePaymentFrequencyPeriodOfLeg2();
	Integer getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2();
	List<? extends OtherPaymentReport> getOtherPayment();
	String getActionType();
	String getEventType();
	Date getEventDate();

	/*********************** Build Methods  ***********************/
	MASTransactionReport build();
	
	MASTransactionReport.MASTransactionReportBuilder toBuilder();
	
	static MASTransactionReport.MASTransactionReportBuilder builder() {
		return new MASTransactionReport.MASTransactionReportBuilderImpl();
	}

	/*********************** Utility Methods  ***********************/
	@Override
	default RosettaMetaData<? extends MASTransactionReport> metaData() {
		return metaData;
	}
	
	@Override
	default Class<? extends MASTransactionReport> getType() {
		return MASTransactionReport.class;
	}
	
	
	@Override
	default void process(RosettaPath path, Processor processor) {
		processor.processBasic(path.newSubPath("counterparty1"), String.class, getCounterparty1(), this);
		processor.processBasic(path.newSubPath("counterparty2"), String.class, getCounterparty2(), this);
		processor.processBasic(path.newSubPath("counterparty2IdentifierType"), Boolean.class, getCounterparty2IdentifierType(), this);
		processor.processBasic(path.newSubPath("direction"), String.class, getDirection(), this);
		processor.processBasic(path.newSubPath("directionOfLeg1"), String.class, getDirectionOfLeg1(), this);
		processor.processBasic(path.newSubPath("directionOfLeg2"), String.class, getDirectionOfLeg2(), this);
		processor.processBasic(path.newSubPath("reportingSpecifiedPerson"), String.class, getReportingSpecifiedPerson(), this);
		processor.processBasic(path.newSubPath("dataSubmitter"), String.class, getDataSubmitter(), this);
		processor.processBasic(path.newSubPath("uniqueTransactionIdentifier"), String.class, getUniqueTransactionIdentifier(), this);
		processor.processBasic(path.newSubPath("uniqueProductIdentifier"), String.class, getUniqueProductIdentifier(), this);
		processor.processBasic(path.newSubPath("priorUTI"), String.class, getPriorUTI(), this);
		processor.processBasic(path.newSubPath("contractType"), String.class, getContractType(), this);
		processor.processBasic(path.newSubPath("assetClass"), String.class, getAssetClass(), this);
		processor.processBasic(path.newSubPath("underlying"), String.class, getUnderlying(), this);
		processor.processBasic(path.newSubPath("underlyingIdentificationType"), String.class, getUnderlyingIdentificationType(), this);
		processor.processBasic(path.newSubPath("underlyingIdOther"), String.class, getUnderlyingIdOther(), this);
		processor.processBasic(path.newSubPath("underlyingIdOtherSource"), ProductIdTypeEnum.class, getUnderlyingIdOtherSource(), this);
		processor.processBasic(path.newSubPath("nameOfTheUnderlyingIndex"), String.class, getNameOfTheUnderlyingIndex(), this);
		processor.processBasic(path.newSubPath("fixedRateOfLeg1"), BigDecimal.class, getFixedRateOfLeg1(), this);
		processor.processBasic(path.newSubPath("identifierOfFloatingRateOfLeg1"), String.class, getIdentifierOfFloatingRateOfLeg1(), this);
		processor.processBasic(path.newSubPath("nameOfTheFloatingRateOfLeg1"), String.class, getNameOfTheFloatingRateOfLeg1(), this);
		processRosetta(path.newSubPath("spreadOfLeg1"), processor, SpreadOfLeg1.class, getSpreadOfLeg1());
		processor.processBasic(path.newSubPath("fixedRateOfLeg2"), BigDecimal.class, getFixedRateOfLeg2(), this);
		processor.processBasic(path.newSubPath("identifierOfFloatingRateOfLeg2"), String.class, getIdentifierOfFloatingRateOfLeg2(), this);
		processor.processBasic(path.newSubPath("nameOfTheFloatingRateOfLeg2"), String.class, getNameOfTheFloatingRateOfLeg2(), this);
		processRosetta(path.newSubPath("spreadOfLeg2"), processor, SpreadOfLeg2.class, getSpreadOfLeg2());
		processor.processBasic(path.newSubPath("optionType"), String.class, getOptionType(), this);
		processor.processBasic(path.newSubPath("optionStyle"), String.class, getOptionStyle(), this);
		processor.processBasic(path.newSubPath("firstExerciseDate"), Date.class, getFirstExerciseDate(), this);
		processor.processBasic(path.newSubPath("effectiveDate"), Date.class, getEffectiveDate(), this);
		processor.processBasic(path.newSubPath("expirationDate"), Date.class, getExpirationDate(), this);
		processor.processBasic(path.newSubPath("swapLinkID"), String.class, getSwapLinkID(), this);
		processor.processBasic(path.newSubPath("packageIdentifier"), String.class, getPackageIdentifier(), this);
		processor.processBasic(path.newSubPath("deliveryType"), String.class, getDeliveryType(), this);
		processor.processBasic(path.newSubPath("settlementCurrency1"), ISOCurrencyCodeEnum.class, getSettlementCurrency1(), this);
		processor.processBasic(path.newSubPath("settlementCurrency2"), ISOCurrencyCodeEnum.class, getSettlementCurrency2(), this);
		processor.processBasic(path.newSubPath("finalContractualSettlementDate"), Date.class, getFinalContractualSettlementDate(), this);
		processor.processBasic(path.newSubPath("cleared"), String.class, getCleared(), this);
		processor.processBasic(path.newSubPath("centralCounterparty"), String.class, getCentralCounterparty(), this);
		processor.processBasic(path.newSubPath("clearingMember"), String.class, getClearingMember(), this);
		processor.processBasic(path.newSubPath("clearingTimestamp"), ZonedDateTime.class, getClearingTimestamp(), this);
		processor.processBasic(path.newSubPath("platformIdentifier"), String.class, getPlatformIdentifier(), this);
		processor.processBasic(path.newSubPath("reportingObligationOfSpecifiedPerson"), String.class, getReportingObligationOfSpecifiedPerson(), this);
		processor.processBasic(path.newSubPath("bookingLocation"), String.class, getBookingLocation(), this);
		processor.processBasic(path.newSubPath("traderLocation"), String.class, getTraderLocation(), this);
		processor.processBasic(path.newSubPath("confirmed"), String.class, getConfirmed(), this);
		processor.processBasic(path.newSubPath("confirmationTimestamp"), ZonedDateTime.class, getConfirmationTimestamp(), this);
		processor.processBasic(path.newSubPath("executionTimestamp"), ZonedDateTime.class, getExecutionTimestamp(), this);
		processor.processBasic(path.newSubPath("reportingTimestamp"), ZonedDateTime.class, getReportingTimestamp(), this);
		processRosetta(path.newSubPath("basketConstituents"), processor, BasketConstituentsReport.class, getBasketConstituents());
		processor.processBasic(path.newSubPath("valuationAmount"), BigDecimal.class, getValuationAmount(), this);
		processor.processBasic(path.newSubPath("valuationCurrency"), ISOCurrencyCodeEnum.class, getValuationCurrency(), this);
		processor.processBasic(path.newSubPath("valuationMethod"), ValuationType1Code.class, getValuationMethod(), this);
		processor.processBasic(path.newSubPath("valuationTimestamp"), ZonedDateTime.class, getValuationTimestamp(), this);
		processor.processBasic(path.newSubPath("collateralPortfolioIndicator"), Boolean.class, getCollateralPortfolioIndicator(), this);
		processor.processBasic(path.newSubPath("initialMarginCollateralPortfolioCode"), String.class, getInitialMarginCollateralPortfolioCode(), this);
		processor.processBasic(path.newSubPath("variationMarginCollateralPortfolioCode"), String.class, getVariationMarginCollateralPortfolioCode(), this);
		processor.processBasic(path.newSubPath("notionalAmountOfTheContractOrLeg1"), BigDecimal.class, getNotionalAmountOfTheContractOrLeg1(), this);
		processor.processBasic(path.newSubPath("notionalCurrencyOfTheContractOrLeg1"), ISOCurrencyCodeEnum.class, getNotionalCurrencyOfTheContractOrLeg1(), this);
		processor.processBasic(path.newSubPath("notionalAmountOfLeg2"), BigDecimal.class, getNotionalAmountOfLeg2(), this);
		processor.processBasic(path.newSubPath("notionalCurrencyOfLeg2"), ISOCurrencyCodeEnum.class, getNotionalCurrencyOfLeg2(), this);
		processor.processBasic(path.newSubPath("totalNotionalQuantityOfTheContractOrLeg1"), BigDecimal.class, getTotalNotionalQuantityOfTheContractOrLeg1(), this);
		processor.processBasic(path.newSubPath("quantityUnitOfMeasureForTheContractOrLeg1"), String.class, getQuantityUnitOfMeasureForTheContractOrLeg1(), this);
		processor.processBasic(path.newSubPath("notionalQuantityOfTheContractOrLeg1"), BigDecimal.class, getNotionalQuantityOfTheContractOrLeg1(), this);
		processor.processBasic(path.newSubPath("quantityFrequencyOfTheContractOrLeg1"), String.class, getQuantityFrequencyOfTheContractOrLeg1(), this);
		processor.processBasic(path.newSubPath("quantityFrequencyMultiplierOfTheContractOrLeg1"), Integer.class, getQuantityFrequencyMultiplierOfTheContractOrLeg1(), this);
		processor.processBasic(path.newSubPath("totalNotionalQuantityLeg2"), BigDecimal.class, getTotalNotionalQuantityLeg2(), this);
		processor.processBasic(path.newSubPath("quantityUnitOfMeasureLeg2"), String.class, getQuantityUnitOfMeasureLeg2(), this);
		processor.processBasic(path.newSubPath("notionalQuantityOfLeg2"), BigDecimal.class, getNotionalQuantityOfLeg2(), this);
		processor.processBasic(path.newSubPath("quantityFrequencyOfLeg2"), String.class, getQuantityFrequencyOfLeg2(), this);
		processor.processBasic(path.newSubPath("quantityFrequencyMultiplierOfLeg2"), Integer.class, getQuantityFrequencyMultiplierOfLeg2(), this);
		processor.processBasic(path.newSubPath("callAmount"), BigDecimal.class, getCallAmount(), this);
		processor.processBasic(path.newSubPath("callCurrency"), ISOCurrencyCodeEnum.class, getCallCurrency(), this);
		processor.processBasic(path.newSubPath("putAmount"), BigDecimal.class, getPutAmount(), this);
		processor.processBasic(path.newSubPath("putCurrency"), ISOCurrencyCodeEnum.class, getPutCurrency(), this);
		processor.processBasic(path.newSubPath("price"), BigDecimal.class, getPrice(), this);
		processor.processBasic(path.newSubPath("priceCurrency"), ISOCurrencyCodeEnum.class, getPriceCurrency(), this);
		processor.processBasic(path.newSubPath("priceNotation"), String.class, getPriceNotation(), this);
		processor.processBasic(path.newSubPath("priceUnitOfMeasure"), String.class, getPriceUnitOfMeasure(), this);
		processRosetta(path.newSubPath("priceSchedule"), processor, PriceScheduleReport.class, getPriceSchedule());
		processor.processBasic(path.newSubPath("strikePrice"), BigDecimal.class, getStrikePrice(), this);
		processor.processBasic(path.newSubPath("strikePriceCurrency"), String.class, getStrikePriceCurrency(), this);
		processor.processBasic(path.newSubPath("strikePriceNotation"), String.class, getStrikePriceNotation(), this);
		processRosetta(path.newSubPath("strikePriceSchedule"), processor, StrikePriceScheduleReport.class, getStrikePriceSchedule());
		processor.processBasic(path.newSubPath("optionPremiumAmount"), BigDecimal.class, getOptionPremiumAmount(), this);
		processor.processBasic(path.newSubPath("optionPremiumCurrency"), ISOCurrencyCodeEnum.class, getOptionPremiumCurrency(), this);
		processor.processBasic(path.newSubPath("exchangeRate"), BigDecimal.class, getExchangeRate(), this);
		processor.processBasic(path.newSubPath("exchangeRateBasis"), String.class, getExchangeRateBasis(), this);
		processor.processBasic(path.newSubPath("fixedRateDayCountConventionOfLeg1"), InterestComputationMethod4Code.class, getFixedRateDayCountConventionOfLeg1(), this);
		processor.processBasic(path.newSubPath("fixedRateDayCountConventionOfLeg2"), InterestComputationMethod4Code.class, getFixedRateDayCountConventionOfLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRateDayCountConventionOfLeg1"), InterestComputationMethod4Code.class, getFloatingRateDayCountConventionOfLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRateDayCountConventionOfLeg2"), InterestComputationMethod4Code.class, getFloatingRateDayCountConventionOfLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodOfLeg1"), String.class, getFloatingRateResetFrequencyPeriodOfLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodMultiplierOfLeg1"), Integer.class, getFloatingRateResetFrequencyPeriodMultiplierOfLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodOfLeg2"), String.class, getFloatingRateResetFrequencyPeriodOfLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodMultiplierOfLeg2"), Integer.class, getFloatingRateResetFrequencyPeriodMultiplierOfLeg2(), this);
		processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodOfLeg1"), String.class, getFixedRatePaymentFrequencyPeriodOfLeg1(), this);
		processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodMultiplierOfLeg1"), Integer.class, getFixedRatePaymentFrequencyPeriodMultiplierOfLeg1(), this);
		processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodOfLeg2"), String.class, getFixedRatePaymentFrequencyPeriodOfLeg2(), this);
		processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodMultiplierOfLeg2"), Integer.class, getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodOfLeg1"), Frequency13Code__1.class, getFloatingRatePaymentFrequencyPeriodOfLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodMultiplierOfLeg1"), Integer.class, getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1(), this);
		processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodOfLeg2"), Frequency13Code__1.class, getFloatingRatePaymentFrequencyPeriodOfLeg2(), this);
		processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodMultiplierOfLeg2"), Integer.class, getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2(), this);
		processRosetta(path.newSubPath("otherPayment"), processor, OtherPaymentReport.class, getOtherPayment());
		processor.processBasic(path.newSubPath("actionType"), String.class, getActionType(), this);
		processor.processBasic(path.newSubPath("eventType"), String.class, getEventType(), this);
		processor.processBasic(path.newSubPath("eventDate"), Date.class, getEventDate(), this);
	}
	

	/*********************** Builder Interface  ***********************/
	interface MASTransactionReportBuilder extends MASTransactionReport, RosettaModelObjectBuilder {
		SpreadOfLeg1.SpreadOfLeg1Builder getOrCreateSpreadOfLeg1();
		SpreadOfLeg1.SpreadOfLeg1Builder getSpreadOfLeg1();
		SpreadOfLeg2.SpreadOfLeg2Builder getOrCreateSpreadOfLeg2();
		SpreadOfLeg2.SpreadOfLeg2Builder getSpreadOfLeg2();
		BasketConstituentsReport.BasketConstituentsReportBuilder getOrCreateBasketConstituents(int _index);
		List<? extends BasketConstituentsReport.BasketConstituentsReportBuilder> getBasketConstituents();
		PriceScheduleReport.PriceScheduleReportBuilder getOrCreatePriceSchedule(int _index);
		List<? extends PriceScheduleReport.PriceScheduleReportBuilder> getPriceSchedule();
		StrikePriceScheduleReport.StrikePriceScheduleReportBuilder getOrCreateStrikePriceSchedule(int _index);
		List<? extends StrikePriceScheduleReport.StrikePriceScheduleReportBuilder> getStrikePriceSchedule();
		OtherPaymentReport.OtherPaymentReportBuilder getOrCreateOtherPayment(int _index);
		List<? extends OtherPaymentReport.OtherPaymentReportBuilder> getOtherPayment();
		MASTransactionReport.MASTransactionReportBuilder setCounterparty1(String counterparty1);
		MASTransactionReport.MASTransactionReportBuilder setCounterparty2(String counterparty2);
		MASTransactionReport.MASTransactionReportBuilder setCounterparty2IdentifierType(Boolean counterparty2IdentifierType);
		MASTransactionReport.MASTransactionReportBuilder setDirection(String direction);
		MASTransactionReport.MASTransactionReportBuilder setDirectionOfLeg1(String directionOfLeg1);
		MASTransactionReport.MASTransactionReportBuilder setDirectionOfLeg2(String directionOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setReportingSpecifiedPerson(String reportingSpecifiedPerson);
		MASTransactionReport.MASTransactionReportBuilder setDataSubmitter(String dataSubmitter);
		MASTransactionReport.MASTransactionReportBuilder setUniqueTransactionIdentifier(String uniqueTransactionIdentifier);
		MASTransactionReport.MASTransactionReportBuilder setUniqueProductIdentifier(String uniqueProductIdentifier);
		MASTransactionReport.MASTransactionReportBuilder setPriorUTI(String priorUTI);
		MASTransactionReport.MASTransactionReportBuilder setContractType(String contractType);
		MASTransactionReport.MASTransactionReportBuilder setAssetClass(String assetClass);
		MASTransactionReport.MASTransactionReportBuilder setUnderlying(String underlying);
		MASTransactionReport.MASTransactionReportBuilder setUnderlyingIdentificationType(String underlyingIdentificationType);
		MASTransactionReport.MASTransactionReportBuilder setUnderlyingIdOther(String underlyingIdOther);
		MASTransactionReport.MASTransactionReportBuilder setUnderlyingIdOtherSource(ProductIdTypeEnum underlyingIdOtherSource);
		MASTransactionReport.MASTransactionReportBuilder setNameOfTheUnderlyingIndex(String nameOfTheUnderlyingIndex);
		MASTransactionReport.MASTransactionReportBuilder setFixedRateOfLeg1(BigDecimal fixedRateOfLeg1);
		MASTransactionReport.MASTransactionReportBuilder setIdentifierOfFloatingRateOfLeg1(String identifierOfFloatingRateOfLeg1);
		MASTransactionReport.MASTransactionReportBuilder setNameOfTheFloatingRateOfLeg1(String nameOfTheFloatingRateOfLeg1);
		MASTransactionReport.MASTransactionReportBuilder setSpreadOfLeg1(SpreadOfLeg1 spreadOfLeg1);
		MASTransactionReport.MASTransactionReportBuilder setFixedRateOfLeg2(BigDecimal fixedRateOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setIdentifierOfFloatingRateOfLeg2(String identifierOfFloatingRateOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setNameOfTheFloatingRateOfLeg2(String nameOfTheFloatingRateOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setSpreadOfLeg2(SpreadOfLeg2 spreadOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setOptionType(String optionType);
		MASTransactionReport.MASTransactionReportBuilder setOptionStyle(String optionStyle);
		MASTransactionReport.MASTransactionReportBuilder setFirstExerciseDate(Date firstExerciseDate);
		MASTransactionReport.MASTransactionReportBuilder setEffectiveDate(Date effectiveDate);
		MASTransactionReport.MASTransactionReportBuilder setExpirationDate(Date expirationDate);
		MASTransactionReport.MASTransactionReportBuilder setSwapLinkID(String swapLinkID);
		MASTransactionReport.MASTransactionReportBuilder setPackageIdentifier(String packageIdentifier);
		MASTransactionReport.MASTransactionReportBuilder setDeliveryType(String deliveryType);
		MASTransactionReport.MASTransactionReportBuilder setSettlementCurrency1(ISOCurrencyCodeEnum settlementCurrency1);
		MASTransactionReport.MASTransactionReportBuilder setSettlementCurrency2(ISOCurrencyCodeEnum settlementCurrency2);
		MASTransactionReport.MASTransactionReportBuilder setFinalContractualSettlementDate(Date finalContractualSettlementDate);
		MASTransactionReport.MASTransactionReportBuilder setCleared(String cleared);
		MASTransactionReport.MASTransactionReportBuilder setCentralCounterparty(String centralCounterparty);
		MASTransactionReport.MASTransactionReportBuilder setClearingMember(String clearingMember);
		MASTransactionReport.MASTransactionReportBuilder setClearingTimestamp(ZonedDateTime clearingTimestamp);
		MASTransactionReport.MASTransactionReportBuilder setPlatformIdentifier(String platformIdentifier);
		MASTransactionReport.MASTransactionReportBuilder setReportingObligationOfSpecifiedPerson(String reportingObligationOfSpecifiedPerson);
		MASTransactionReport.MASTransactionReportBuilder setBookingLocation(String bookingLocation);
		MASTransactionReport.MASTransactionReportBuilder setTraderLocation(String traderLocation);
		MASTransactionReport.MASTransactionReportBuilder setConfirmed(String confirmed);
		MASTransactionReport.MASTransactionReportBuilder setConfirmationTimestamp(ZonedDateTime confirmationTimestamp);
		MASTransactionReport.MASTransactionReportBuilder setExecutionTimestamp(ZonedDateTime executionTimestamp);
		MASTransactionReport.MASTransactionReportBuilder setReportingTimestamp(ZonedDateTime reportingTimestamp);
		MASTransactionReport.MASTransactionReportBuilder addBasketConstituents(BasketConstituentsReport basketConstituents0);
		MASTransactionReport.MASTransactionReportBuilder addBasketConstituents(BasketConstituentsReport basketConstituents1, int _idx);
		MASTransactionReport.MASTransactionReportBuilder addBasketConstituents(List<? extends BasketConstituentsReport> basketConstituents2);
		MASTransactionReport.MASTransactionReportBuilder setBasketConstituents(List<? extends BasketConstituentsReport> basketConstituents3);
		MASTransactionReport.MASTransactionReportBuilder setValuationAmount(BigDecimal valuationAmount);
		MASTransactionReport.MASTransactionReportBuilder setValuationCurrency(ISOCurrencyCodeEnum valuationCurrency);
		MASTransactionReport.MASTransactionReportBuilder setValuationMethod(ValuationType1Code valuationMethod);
		MASTransactionReport.MASTransactionReportBuilder setValuationTimestamp(ZonedDateTime valuationTimestamp);
		MASTransactionReport.MASTransactionReportBuilder setCollateralPortfolioIndicator(Boolean collateralPortfolioIndicator);
		MASTransactionReport.MASTransactionReportBuilder setInitialMarginCollateralPortfolioCode(String initialMarginCollateralPortfolioCode);
		MASTransactionReport.MASTransactionReportBuilder setVariationMarginCollateralPortfolioCode(String variationMarginCollateralPortfolioCode);
		MASTransactionReport.MASTransactionReportBuilder setNotionalAmountOfTheContractOrLeg1(BigDecimal notionalAmountOfTheContractOrLeg1);
		MASTransactionReport.MASTransactionReportBuilder setNotionalCurrencyOfTheContractOrLeg1(ISOCurrencyCodeEnum notionalCurrencyOfTheContractOrLeg1);
		MASTransactionReport.MASTransactionReportBuilder setNotionalAmountOfLeg2(BigDecimal notionalAmountOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setNotionalCurrencyOfLeg2(ISOCurrencyCodeEnum notionalCurrencyOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setTotalNotionalQuantityOfTheContractOrLeg1(BigDecimal totalNotionalQuantityOfTheContractOrLeg1);
		MASTransactionReport.MASTransactionReportBuilder setQuantityUnitOfMeasureForTheContractOrLeg1(String quantityUnitOfMeasureForTheContractOrLeg1);
		MASTransactionReport.MASTransactionReportBuilder setNotionalQuantityOfTheContractOrLeg1(BigDecimal notionalQuantityOfTheContractOrLeg1);
		MASTransactionReport.MASTransactionReportBuilder setQuantityFrequencyOfTheContractOrLeg1(String quantityFrequencyOfTheContractOrLeg1);
		MASTransactionReport.MASTransactionReportBuilder setQuantityFrequencyMultiplierOfTheContractOrLeg1(Integer quantityFrequencyMultiplierOfTheContractOrLeg1);
		MASTransactionReport.MASTransactionReportBuilder setTotalNotionalQuantityLeg2(BigDecimal totalNotionalQuantityLeg2);
		MASTransactionReport.MASTransactionReportBuilder setQuantityUnitOfMeasureLeg2(String quantityUnitOfMeasureLeg2);
		MASTransactionReport.MASTransactionReportBuilder setNotionalQuantityOfLeg2(BigDecimal notionalQuantityOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setQuantityFrequencyOfLeg2(String quantityFrequencyOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setQuantityFrequencyMultiplierOfLeg2(Integer quantityFrequencyMultiplierOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setCallAmount(BigDecimal callAmount);
		MASTransactionReport.MASTransactionReportBuilder setCallCurrency(ISOCurrencyCodeEnum callCurrency);
		MASTransactionReport.MASTransactionReportBuilder setPutAmount(BigDecimal putAmount);
		MASTransactionReport.MASTransactionReportBuilder setPutCurrency(ISOCurrencyCodeEnum putCurrency);
		MASTransactionReport.MASTransactionReportBuilder setPrice(BigDecimal price);
		MASTransactionReport.MASTransactionReportBuilder setPriceCurrency(ISOCurrencyCodeEnum priceCurrency);
		MASTransactionReport.MASTransactionReportBuilder setPriceNotation(String priceNotation);
		MASTransactionReport.MASTransactionReportBuilder setPriceUnitOfMeasure(String priceUnitOfMeasure);
		MASTransactionReport.MASTransactionReportBuilder addPriceSchedule(PriceScheduleReport priceSchedule0);
		MASTransactionReport.MASTransactionReportBuilder addPriceSchedule(PriceScheduleReport priceSchedule1, int _idx);
		MASTransactionReport.MASTransactionReportBuilder addPriceSchedule(List<? extends PriceScheduleReport> priceSchedule2);
		MASTransactionReport.MASTransactionReportBuilder setPriceSchedule(List<? extends PriceScheduleReport> priceSchedule3);
		MASTransactionReport.MASTransactionReportBuilder setStrikePrice(BigDecimal strikePrice);
		MASTransactionReport.MASTransactionReportBuilder setStrikePriceCurrency(String strikePriceCurrency);
		MASTransactionReport.MASTransactionReportBuilder setStrikePriceNotation(String strikePriceNotation);
		MASTransactionReport.MASTransactionReportBuilder addStrikePriceSchedule(StrikePriceScheduleReport strikePriceSchedule0);
		MASTransactionReport.MASTransactionReportBuilder addStrikePriceSchedule(StrikePriceScheduleReport strikePriceSchedule1, int _idx);
		MASTransactionReport.MASTransactionReportBuilder addStrikePriceSchedule(List<? extends StrikePriceScheduleReport> strikePriceSchedule2);
		MASTransactionReport.MASTransactionReportBuilder setStrikePriceSchedule(List<? extends StrikePriceScheduleReport> strikePriceSchedule3);
		MASTransactionReport.MASTransactionReportBuilder setOptionPremiumAmount(BigDecimal optionPremiumAmount);
		MASTransactionReport.MASTransactionReportBuilder setOptionPremiumCurrency(ISOCurrencyCodeEnum optionPremiumCurrency);
		MASTransactionReport.MASTransactionReportBuilder setExchangeRate(BigDecimal exchangeRate);
		MASTransactionReport.MASTransactionReportBuilder setExchangeRateBasis(String exchangeRateBasis);
		MASTransactionReport.MASTransactionReportBuilder setFixedRateDayCountConventionOfLeg1(InterestComputationMethod4Code fixedRateDayCountConventionOfLeg1);
		MASTransactionReport.MASTransactionReportBuilder setFixedRateDayCountConventionOfLeg2(InterestComputationMethod4Code fixedRateDayCountConventionOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setFloatingRateDayCountConventionOfLeg1(InterestComputationMethod4Code floatingRateDayCountConventionOfLeg1);
		MASTransactionReport.MASTransactionReportBuilder setFloatingRateDayCountConventionOfLeg2(InterestComputationMethod4Code floatingRateDayCountConventionOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setFloatingRateResetFrequencyPeriodOfLeg1(String floatingRateResetFrequencyPeriodOfLeg1);
		MASTransactionReport.MASTransactionReportBuilder setFloatingRateResetFrequencyPeriodMultiplierOfLeg1(Integer floatingRateResetFrequencyPeriodMultiplierOfLeg1);
		MASTransactionReport.MASTransactionReportBuilder setFloatingRateResetFrequencyPeriodOfLeg2(String floatingRateResetFrequencyPeriodOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setFloatingRateResetFrequencyPeriodMultiplierOfLeg2(Integer floatingRateResetFrequencyPeriodMultiplierOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setFixedRatePaymentFrequencyPeriodOfLeg1(String fixedRatePaymentFrequencyPeriodOfLeg1);
		MASTransactionReport.MASTransactionReportBuilder setFixedRatePaymentFrequencyPeriodMultiplierOfLeg1(Integer fixedRatePaymentFrequencyPeriodMultiplierOfLeg1);
		MASTransactionReport.MASTransactionReportBuilder setFixedRatePaymentFrequencyPeriodOfLeg2(String fixedRatePaymentFrequencyPeriodOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setFixedRatePaymentFrequencyPeriodMultiplierOfLeg2(Integer fixedRatePaymentFrequencyPeriodMultiplierOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodOfLeg1(Frequency13Code__1 floatingRatePaymentFrequencyPeriodOfLeg1);
		MASTransactionReport.MASTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1(Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg1);
		MASTransactionReport.MASTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodOfLeg2(Frequency13Code__1 floatingRatePaymentFrequencyPeriodOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2(Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg2);
		MASTransactionReport.MASTransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment0);
		MASTransactionReport.MASTransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment1, int _idx);
		MASTransactionReport.MASTransactionReportBuilder addOtherPayment(List<? extends OtherPaymentReport> otherPayment2);
		MASTransactionReport.MASTransactionReportBuilder setOtherPayment(List<? extends OtherPaymentReport> otherPayment3);
		MASTransactionReport.MASTransactionReportBuilder setActionType(String actionType);
		MASTransactionReport.MASTransactionReportBuilder setEventType(String eventType);
		MASTransactionReport.MASTransactionReportBuilder setEventDate(Date eventDate);

		@Override
		default void process(RosettaPath path, BuilderProcessor processor) {
			processor.processBasic(path.newSubPath("counterparty1"), String.class, getCounterparty1(), this);
			processor.processBasic(path.newSubPath("counterparty2"), String.class, getCounterparty2(), this);
			processor.processBasic(path.newSubPath("counterparty2IdentifierType"), Boolean.class, getCounterparty2IdentifierType(), this);
			processor.processBasic(path.newSubPath("direction"), String.class, getDirection(), this);
			processor.processBasic(path.newSubPath("directionOfLeg1"), String.class, getDirectionOfLeg1(), this);
			processor.processBasic(path.newSubPath("directionOfLeg2"), String.class, getDirectionOfLeg2(), this);
			processor.processBasic(path.newSubPath("reportingSpecifiedPerson"), String.class, getReportingSpecifiedPerson(), this);
			processor.processBasic(path.newSubPath("dataSubmitter"), String.class, getDataSubmitter(), this);
			processor.processBasic(path.newSubPath("uniqueTransactionIdentifier"), String.class, getUniqueTransactionIdentifier(), this);
			processor.processBasic(path.newSubPath("uniqueProductIdentifier"), String.class, getUniqueProductIdentifier(), this);
			processor.processBasic(path.newSubPath("priorUTI"), String.class, getPriorUTI(), this);
			processor.processBasic(path.newSubPath("contractType"), String.class, getContractType(), this);
			processor.processBasic(path.newSubPath("assetClass"), String.class, getAssetClass(), this);
			processor.processBasic(path.newSubPath("underlying"), String.class, getUnderlying(), this);
			processor.processBasic(path.newSubPath("underlyingIdentificationType"), String.class, getUnderlyingIdentificationType(), this);
			processor.processBasic(path.newSubPath("underlyingIdOther"), String.class, getUnderlyingIdOther(), this);
			processor.processBasic(path.newSubPath("underlyingIdOtherSource"), ProductIdTypeEnum.class, getUnderlyingIdOtherSource(), this);
			processor.processBasic(path.newSubPath("nameOfTheUnderlyingIndex"), String.class, getNameOfTheUnderlyingIndex(), this);
			processor.processBasic(path.newSubPath("fixedRateOfLeg1"), BigDecimal.class, getFixedRateOfLeg1(), this);
			processor.processBasic(path.newSubPath("identifierOfFloatingRateOfLeg1"), String.class, getIdentifierOfFloatingRateOfLeg1(), this);
			processor.processBasic(path.newSubPath("nameOfTheFloatingRateOfLeg1"), String.class, getNameOfTheFloatingRateOfLeg1(), this);
			processRosetta(path.newSubPath("spreadOfLeg1"), processor, SpreadOfLeg1.SpreadOfLeg1Builder.class, getSpreadOfLeg1());
			processor.processBasic(path.newSubPath("fixedRateOfLeg2"), BigDecimal.class, getFixedRateOfLeg2(), this);
			processor.processBasic(path.newSubPath("identifierOfFloatingRateOfLeg2"), String.class, getIdentifierOfFloatingRateOfLeg2(), this);
			processor.processBasic(path.newSubPath("nameOfTheFloatingRateOfLeg2"), String.class, getNameOfTheFloatingRateOfLeg2(), this);
			processRosetta(path.newSubPath("spreadOfLeg2"), processor, SpreadOfLeg2.SpreadOfLeg2Builder.class, getSpreadOfLeg2());
			processor.processBasic(path.newSubPath("optionType"), String.class, getOptionType(), this);
			processor.processBasic(path.newSubPath("optionStyle"), String.class, getOptionStyle(), this);
			processor.processBasic(path.newSubPath("firstExerciseDate"), Date.class, getFirstExerciseDate(), this);
			processor.processBasic(path.newSubPath("effectiveDate"), Date.class, getEffectiveDate(), this);
			processor.processBasic(path.newSubPath("expirationDate"), Date.class, getExpirationDate(), this);
			processor.processBasic(path.newSubPath("swapLinkID"), String.class, getSwapLinkID(), this);
			processor.processBasic(path.newSubPath("packageIdentifier"), String.class, getPackageIdentifier(), this);
			processor.processBasic(path.newSubPath("deliveryType"), String.class, getDeliveryType(), this);
			processor.processBasic(path.newSubPath("settlementCurrency1"), ISOCurrencyCodeEnum.class, getSettlementCurrency1(), this);
			processor.processBasic(path.newSubPath("settlementCurrency2"), ISOCurrencyCodeEnum.class, getSettlementCurrency2(), this);
			processor.processBasic(path.newSubPath("finalContractualSettlementDate"), Date.class, getFinalContractualSettlementDate(), this);
			processor.processBasic(path.newSubPath("cleared"), String.class, getCleared(), this);
			processor.processBasic(path.newSubPath("centralCounterparty"), String.class, getCentralCounterparty(), this);
			processor.processBasic(path.newSubPath("clearingMember"), String.class, getClearingMember(), this);
			processor.processBasic(path.newSubPath("clearingTimestamp"), ZonedDateTime.class, getClearingTimestamp(), this);
			processor.processBasic(path.newSubPath("platformIdentifier"), String.class, getPlatformIdentifier(), this);
			processor.processBasic(path.newSubPath("reportingObligationOfSpecifiedPerson"), String.class, getReportingObligationOfSpecifiedPerson(), this);
			processor.processBasic(path.newSubPath("bookingLocation"), String.class, getBookingLocation(), this);
			processor.processBasic(path.newSubPath("traderLocation"), String.class, getTraderLocation(), this);
			processor.processBasic(path.newSubPath("confirmed"), String.class, getConfirmed(), this);
			processor.processBasic(path.newSubPath("confirmationTimestamp"), ZonedDateTime.class, getConfirmationTimestamp(), this);
			processor.processBasic(path.newSubPath("executionTimestamp"), ZonedDateTime.class, getExecutionTimestamp(), this);
			processor.processBasic(path.newSubPath("reportingTimestamp"), ZonedDateTime.class, getReportingTimestamp(), this);
			processRosetta(path.newSubPath("basketConstituents"), processor, BasketConstituentsReport.BasketConstituentsReportBuilder.class, getBasketConstituents());
			processor.processBasic(path.newSubPath("valuationAmount"), BigDecimal.class, getValuationAmount(), this);
			processor.processBasic(path.newSubPath("valuationCurrency"), ISOCurrencyCodeEnum.class, getValuationCurrency(), this);
			processor.processBasic(path.newSubPath("valuationMethod"), ValuationType1Code.class, getValuationMethod(), this);
			processor.processBasic(path.newSubPath("valuationTimestamp"), ZonedDateTime.class, getValuationTimestamp(), this);
			processor.processBasic(path.newSubPath("collateralPortfolioIndicator"), Boolean.class, getCollateralPortfolioIndicator(), this);
			processor.processBasic(path.newSubPath("initialMarginCollateralPortfolioCode"), String.class, getInitialMarginCollateralPortfolioCode(), this);
			processor.processBasic(path.newSubPath("variationMarginCollateralPortfolioCode"), String.class, getVariationMarginCollateralPortfolioCode(), this);
			processor.processBasic(path.newSubPath("notionalAmountOfTheContractOrLeg1"), BigDecimal.class, getNotionalAmountOfTheContractOrLeg1(), this);
			processor.processBasic(path.newSubPath("notionalCurrencyOfTheContractOrLeg1"), ISOCurrencyCodeEnum.class, getNotionalCurrencyOfTheContractOrLeg1(), this);
			processor.processBasic(path.newSubPath("notionalAmountOfLeg2"), BigDecimal.class, getNotionalAmountOfLeg2(), this);
			processor.processBasic(path.newSubPath("notionalCurrencyOfLeg2"), ISOCurrencyCodeEnum.class, getNotionalCurrencyOfLeg2(), this);
			processor.processBasic(path.newSubPath("totalNotionalQuantityOfTheContractOrLeg1"), BigDecimal.class, getTotalNotionalQuantityOfTheContractOrLeg1(), this);
			processor.processBasic(path.newSubPath("quantityUnitOfMeasureForTheContractOrLeg1"), String.class, getQuantityUnitOfMeasureForTheContractOrLeg1(), this);
			processor.processBasic(path.newSubPath("notionalQuantityOfTheContractOrLeg1"), BigDecimal.class, getNotionalQuantityOfTheContractOrLeg1(), this);
			processor.processBasic(path.newSubPath("quantityFrequencyOfTheContractOrLeg1"), String.class, getQuantityFrequencyOfTheContractOrLeg1(), this);
			processor.processBasic(path.newSubPath("quantityFrequencyMultiplierOfTheContractOrLeg1"), Integer.class, getQuantityFrequencyMultiplierOfTheContractOrLeg1(), this);
			processor.processBasic(path.newSubPath("totalNotionalQuantityLeg2"), BigDecimal.class, getTotalNotionalQuantityLeg2(), this);
			processor.processBasic(path.newSubPath("quantityUnitOfMeasureLeg2"), String.class, getQuantityUnitOfMeasureLeg2(), this);
			processor.processBasic(path.newSubPath("notionalQuantityOfLeg2"), BigDecimal.class, getNotionalQuantityOfLeg2(), this);
			processor.processBasic(path.newSubPath("quantityFrequencyOfLeg2"), String.class, getQuantityFrequencyOfLeg2(), this);
			processor.processBasic(path.newSubPath("quantityFrequencyMultiplierOfLeg2"), Integer.class, getQuantityFrequencyMultiplierOfLeg2(), this);
			processor.processBasic(path.newSubPath("callAmount"), BigDecimal.class, getCallAmount(), this);
			processor.processBasic(path.newSubPath("callCurrency"), ISOCurrencyCodeEnum.class, getCallCurrency(), this);
			processor.processBasic(path.newSubPath("putAmount"), BigDecimal.class, getPutAmount(), this);
			processor.processBasic(path.newSubPath("putCurrency"), ISOCurrencyCodeEnum.class, getPutCurrency(), this);
			processor.processBasic(path.newSubPath("price"), BigDecimal.class, getPrice(), this);
			processor.processBasic(path.newSubPath("priceCurrency"), ISOCurrencyCodeEnum.class, getPriceCurrency(), this);
			processor.processBasic(path.newSubPath("priceNotation"), String.class, getPriceNotation(), this);
			processor.processBasic(path.newSubPath("priceUnitOfMeasure"), String.class, getPriceUnitOfMeasure(), this);
			processRosetta(path.newSubPath("priceSchedule"), processor, PriceScheduleReport.PriceScheduleReportBuilder.class, getPriceSchedule());
			processor.processBasic(path.newSubPath("strikePrice"), BigDecimal.class, getStrikePrice(), this);
			processor.processBasic(path.newSubPath("strikePriceCurrency"), String.class, getStrikePriceCurrency(), this);
			processor.processBasic(path.newSubPath("strikePriceNotation"), String.class, getStrikePriceNotation(), this);
			processRosetta(path.newSubPath("strikePriceSchedule"), processor, StrikePriceScheduleReport.StrikePriceScheduleReportBuilder.class, getStrikePriceSchedule());
			processor.processBasic(path.newSubPath("optionPremiumAmount"), BigDecimal.class, getOptionPremiumAmount(), this);
			processor.processBasic(path.newSubPath("optionPremiumCurrency"), ISOCurrencyCodeEnum.class, getOptionPremiumCurrency(), this);
			processor.processBasic(path.newSubPath("exchangeRate"), BigDecimal.class, getExchangeRate(), this);
			processor.processBasic(path.newSubPath("exchangeRateBasis"), String.class, getExchangeRateBasis(), this);
			processor.processBasic(path.newSubPath("fixedRateDayCountConventionOfLeg1"), InterestComputationMethod4Code.class, getFixedRateDayCountConventionOfLeg1(), this);
			processor.processBasic(path.newSubPath("fixedRateDayCountConventionOfLeg2"), InterestComputationMethod4Code.class, getFixedRateDayCountConventionOfLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRateDayCountConventionOfLeg1"), InterestComputationMethod4Code.class, getFloatingRateDayCountConventionOfLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRateDayCountConventionOfLeg2"), InterestComputationMethod4Code.class, getFloatingRateDayCountConventionOfLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodOfLeg1"), String.class, getFloatingRateResetFrequencyPeriodOfLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodMultiplierOfLeg1"), Integer.class, getFloatingRateResetFrequencyPeriodMultiplierOfLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodOfLeg2"), String.class, getFloatingRateResetFrequencyPeriodOfLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRateResetFrequencyPeriodMultiplierOfLeg2"), Integer.class, getFloatingRateResetFrequencyPeriodMultiplierOfLeg2(), this);
			processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodOfLeg1"), String.class, getFixedRatePaymentFrequencyPeriodOfLeg1(), this);
			processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodMultiplierOfLeg1"), Integer.class, getFixedRatePaymentFrequencyPeriodMultiplierOfLeg1(), this);
			processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodOfLeg2"), String.class, getFixedRatePaymentFrequencyPeriodOfLeg2(), this);
			processor.processBasic(path.newSubPath("fixedRatePaymentFrequencyPeriodMultiplierOfLeg2"), Integer.class, getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodOfLeg1"), Frequency13Code__1.class, getFloatingRatePaymentFrequencyPeriodOfLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodMultiplierOfLeg1"), Integer.class, getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1(), this);
			processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodOfLeg2"), Frequency13Code__1.class, getFloatingRatePaymentFrequencyPeriodOfLeg2(), this);
			processor.processBasic(path.newSubPath("floatingRatePaymentFrequencyPeriodMultiplierOfLeg2"), Integer.class, getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2(), this);
			processRosetta(path.newSubPath("otherPayment"), processor, OtherPaymentReport.OtherPaymentReportBuilder.class, getOtherPayment());
			processor.processBasic(path.newSubPath("actionType"), String.class, getActionType(), this);
			processor.processBasic(path.newSubPath("eventType"), String.class, getEventType(), this);
			processor.processBasic(path.newSubPath("eventDate"), Date.class, getEventDate(), this);
		}
		

		MASTransactionReport.MASTransactionReportBuilder prune();
	}

	/*********************** Immutable Implementation of MASTransactionReport  ***********************/
	class MASTransactionReportImpl implements MASTransactionReport {
		private final String counterparty1;
		private final String counterparty2;
		private final Boolean counterparty2IdentifierType;
		private final String direction;
		private final String directionOfLeg1;
		private final String directionOfLeg2;
		private final String reportingSpecifiedPerson;
		private final String dataSubmitter;
		private final String uniqueTransactionIdentifier;
		private final String uniqueProductIdentifier;
		private final String priorUTI;
		private final String contractType;
		private final String assetClass;
		private final String underlying;
		private final String underlyingIdentificationType;
		private final String underlyingIdOther;
		private final ProductIdTypeEnum underlyingIdOtherSource;
		private final String nameOfTheUnderlyingIndex;
		private final BigDecimal fixedRateOfLeg1;
		private final String identifierOfFloatingRateOfLeg1;
		private final String nameOfTheFloatingRateOfLeg1;
		private final SpreadOfLeg1 spreadOfLeg1;
		private final BigDecimal fixedRateOfLeg2;
		private final String identifierOfFloatingRateOfLeg2;
		private final String nameOfTheFloatingRateOfLeg2;
		private final SpreadOfLeg2 spreadOfLeg2;
		private final String optionType;
		private final String optionStyle;
		private final Date firstExerciseDate;
		private final Date effectiveDate;
		private final Date expirationDate;
		private final String swapLinkID;
		private final String packageIdentifier;
		private final String deliveryType;
		private final ISOCurrencyCodeEnum settlementCurrency1;
		private final ISOCurrencyCodeEnum settlementCurrency2;
		private final Date finalContractualSettlementDate;
		private final String cleared;
		private final String centralCounterparty;
		private final String clearingMember;
		private final ZonedDateTime clearingTimestamp;
		private final String platformIdentifier;
		private final String reportingObligationOfSpecifiedPerson;
		private final String bookingLocation;
		private final String traderLocation;
		private final String confirmed;
		private final ZonedDateTime confirmationTimestamp;
		private final ZonedDateTime executionTimestamp;
		private final ZonedDateTime reportingTimestamp;
		private final List<? extends BasketConstituentsReport> basketConstituents;
		private final BigDecimal valuationAmount;
		private final ISOCurrencyCodeEnum valuationCurrency;
		private final ValuationType1Code valuationMethod;
		private final ZonedDateTime valuationTimestamp;
		private final Boolean collateralPortfolioIndicator;
		private final String initialMarginCollateralPortfolioCode;
		private final String variationMarginCollateralPortfolioCode;
		private final BigDecimal notionalAmountOfTheContractOrLeg1;
		private final ISOCurrencyCodeEnum notionalCurrencyOfTheContractOrLeg1;
		private final BigDecimal notionalAmountOfLeg2;
		private final ISOCurrencyCodeEnum notionalCurrencyOfLeg2;
		private final BigDecimal totalNotionalQuantityOfTheContractOrLeg1;
		private final String quantityUnitOfMeasureForTheContractOrLeg1;
		private final BigDecimal notionalQuantityOfTheContractOrLeg1;
		private final String quantityFrequencyOfTheContractOrLeg1;
		private final Integer quantityFrequencyMultiplierOfTheContractOrLeg1;
		private final BigDecimal totalNotionalQuantityLeg2;
		private final String quantityUnitOfMeasureLeg2;
		private final BigDecimal notionalQuantityOfLeg2;
		private final String quantityFrequencyOfLeg2;
		private final Integer quantityFrequencyMultiplierOfLeg2;
		private final BigDecimal callAmount;
		private final ISOCurrencyCodeEnum callCurrency;
		private final BigDecimal putAmount;
		private final ISOCurrencyCodeEnum putCurrency;
		private final BigDecimal price;
		private final ISOCurrencyCodeEnum priceCurrency;
		private final String priceNotation;
		private final String priceUnitOfMeasure;
		private final List<? extends PriceScheduleReport> priceSchedule;
		private final BigDecimal strikePrice;
		private final String strikePriceCurrency;
		private final String strikePriceNotation;
		private final List<? extends StrikePriceScheduleReport> strikePriceSchedule;
		private final BigDecimal optionPremiumAmount;
		private final ISOCurrencyCodeEnum optionPremiumCurrency;
		private final BigDecimal exchangeRate;
		private final String exchangeRateBasis;
		private final InterestComputationMethod4Code fixedRateDayCountConventionOfLeg1;
		private final InterestComputationMethod4Code fixedRateDayCountConventionOfLeg2;
		private final InterestComputationMethod4Code floatingRateDayCountConventionOfLeg1;
		private final InterestComputationMethod4Code floatingRateDayCountConventionOfLeg2;
		private final String floatingRateResetFrequencyPeriodOfLeg1;
		private final Integer floatingRateResetFrequencyPeriodMultiplierOfLeg1;
		private final String floatingRateResetFrequencyPeriodOfLeg2;
		private final Integer floatingRateResetFrequencyPeriodMultiplierOfLeg2;
		private final String fixedRatePaymentFrequencyPeriodOfLeg1;
		private final Integer fixedRatePaymentFrequencyPeriodMultiplierOfLeg1;
		private final String fixedRatePaymentFrequencyPeriodOfLeg2;
		private final Integer fixedRatePaymentFrequencyPeriodMultiplierOfLeg2;
		private final Frequency13Code__1 floatingRatePaymentFrequencyPeriodOfLeg1;
		private final Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg1;
		private final Frequency13Code__1 floatingRatePaymentFrequencyPeriodOfLeg2;
		private final Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg2;
		private final List<? extends OtherPaymentReport> otherPayment;
		private final String actionType;
		private final String eventType;
		private final Date eventDate;
		
		protected MASTransactionReportImpl(MASTransactionReport.MASTransactionReportBuilder builder) {
			this.counterparty1 = builder.getCounterparty1();
			this.counterparty2 = builder.getCounterparty2();
			this.counterparty2IdentifierType = builder.getCounterparty2IdentifierType();
			this.direction = builder.getDirection();
			this.directionOfLeg1 = builder.getDirectionOfLeg1();
			this.directionOfLeg2 = builder.getDirectionOfLeg2();
			this.reportingSpecifiedPerson = builder.getReportingSpecifiedPerson();
			this.dataSubmitter = builder.getDataSubmitter();
			this.uniqueTransactionIdentifier = builder.getUniqueTransactionIdentifier();
			this.uniqueProductIdentifier = builder.getUniqueProductIdentifier();
			this.priorUTI = builder.getPriorUTI();
			this.contractType = builder.getContractType();
			this.assetClass = builder.getAssetClass();
			this.underlying = builder.getUnderlying();
			this.underlyingIdentificationType = builder.getUnderlyingIdentificationType();
			this.underlyingIdOther = builder.getUnderlyingIdOther();
			this.underlyingIdOtherSource = builder.getUnderlyingIdOtherSource();
			this.nameOfTheUnderlyingIndex = builder.getNameOfTheUnderlyingIndex();
			this.fixedRateOfLeg1 = builder.getFixedRateOfLeg1();
			this.identifierOfFloatingRateOfLeg1 = builder.getIdentifierOfFloatingRateOfLeg1();
			this.nameOfTheFloatingRateOfLeg1 = builder.getNameOfTheFloatingRateOfLeg1();
			this.spreadOfLeg1 = ofNullable(builder.getSpreadOfLeg1()).map(f->f.build()).orElse(null);
			this.fixedRateOfLeg2 = builder.getFixedRateOfLeg2();
			this.identifierOfFloatingRateOfLeg2 = builder.getIdentifierOfFloatingRateOfLeg2();
			this.nameOfTheFloatingRateOfLeg2 = builder.getNameOfTheFloatingRateOfLeg2();
			this.spreadOfLeg2 = ofNullable(builder.getSpreadOfLeg2()).map(f->f.build()).orElse(null);
			this.optionType = builder.getOptionType();
			this.optionStyle = builder.getOptionStyle();
			this.firstExerciseDate = builder.getFirstExerciseDate();
			this.effectiveDate = builder.getEffectiveDate();
			this.expirationDate = builder.getExpirationDate();
			this.swapLinkID = builder.getSwapLinkID();
			this.packageIdentifier = builder.getPackageIdentifier();
			this.deliveryType = builder.getDeliveryType();
			this.settlementCurrency1 = builder.getSettlementCurrency1();
			this.settlementCurrency2 = builder.getSettlementCurrency2();
			this.finalContractualSettlementDate = builder.getFinalContractualSettlementDate();
			this.cleared = builder.getCleared();
			this.centralCounterparty = builder.getCentralCounterparty();
			this.clearingMember = builder.getClearingMember();
			this.clearingTimestamp = builder.getClearingTimestamp();
			this.platformIdentifier = builder.getPlatformIdentifier();
			this.reportingObligationOfSpecifiedPerson = builder.getReportingObligationOfSpecifiedPerson();
			this.bookingLocation = builder.getBookingLocation();
			this.traderLocation = builder.getTraderLocation();
			this.confirmed = builder.getConfirmed();
			this.confirmationTimestamp = builder.getConfirmationTimestamp();
			this.executionTimestamp = builder.getExecutionTimestamp();
			this.reportingTimestamp = builder.getReportingTimestamp();
			this.basketConstituents = ofNullable(builder.getBasketConstituents()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.valuationAmount = builder.getValuationAmount();
			this.valuationCurrency = builder.getValuationCurrency();
			this.valuationMethod = builder.getValuationMethod();
			this.valuationTimestamp = builder.getValuationTimestamp();
			this.collateralPortfolioIndicator = builder.getCollateralPortfolioIndicator();
			this.initialMarginCollateralPortfolioCode = builder.getInitialMarginCollateralPortfolioCode();
			this.variationMarginCollateralPortfolioCode = builder.getVariationMarginCollateralPortfolioCode();
			this.notionalAmountOfTheContractOrLeg1 = builder.getNotionalAmountOfTheContractOrLeg1();
			this.notionalCurrencyOfTheContractOrLeg1 = builder.getNotionalCurrencyOfTheContractOrLeg1();
			this.notionalAmountOfLeg2 = builder.getNotionalAmountOfLeg2();
			this.notionalCurrencyOfLeg2 = builder.getNotionalCurrencyOfLeg2();
			this.totalNotionalQuantityOfTheContractOrLeg1 = builder.getTotalNotionalQuantityOfTheContractOrLeg1();
			this.quantityUnitOfMeasureForTheContractOrLeg1 = builder.getQuantityUnitOfMeasureForTheContractOrLeg1();
			this.notionalQuantityOfTheContractOrLeg1 = builder.getNotionalQuantityOfTheContractOrLeg1();
			this.quantityFrequencyOfTheContractOrLeg1 = builder.getQuantityFrequencyOfTheContractOrLeg1();
			this.quantityFrequencyMultiplierOfTheContractOrLeg1 = builder.getQuantityFrequencyMultiplierOfTheContractOrLeg1();
			this.totalNotionalQuantityLeg2 = builder.getTotalNotionalQuantityLeg2();
			this.quantityUnitOfMeasureLeg2 = builder.getQuantityUnitOfMeasureLeg2();
			this.notionalQuantityOfLeg2 = builder.getNotionalQuantityOfLeg2();
			this.quantityFrequencyOfLeg2 = builder.getQuantityFrequencyOfLeg2();
			this.quantityFrequencyMultiplierOfLeg2 = builder.getQuantityFrequencyMultiplierOfLeg2();
			this.callAmount = builder.getCallAmount();
			this.callCurrency = builder.getCallCurrency();
			this.putAmount = builder.getPutAmount();
			this.putCurrency = builder.getPutCurrency();
			this.price = builder.getPrice();
			this.priceCurrency = builder.getPriceCurrency();
			this.priceNotation = builder.getPriceNotation();
			this.priceUnitOfMeasure = builder.getPriceUnitOfMeasure();
			this.priceSchedule = ofNullable(builder.getPriceSchedule()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.strikePrice = builder.getStrikePrice();
			this.strikePriceCurrency = builder.getStrikePriceCurrency();
			this.strikePriceNotation = builder.getStrikePriceNotation();
			this.strikePriceSchedule = ofNullable(builder.getStrikePriceSchedule()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.optionPremiumAmount = builder.getOptionPremiumAmount();
			this.optionPremiumCurrency = builder.getOptionPremiumCurrency();
			this.exchangeRate = builder.getExchangeRate();
			this.exchangeRateBasis = builder.getExchangeRateBasis();
			this.fixedRateDayCountConventionOfLeg1 = builder.getFixedRateDayCountConventionOfLeg1();
			this.fixedRateDayCountConventionOfLeg2 = builder.getFixedRateDayCountConventionOfLeg2();
			this.floatingRateDayCountConventionOfLeg1 = builder.getFloatingRateDayCountConventionOfLeg1();
			this.floatingRateDayCountConventionOfLeg2 = builder.getFloatingRateDayCountConventionOfLeg2();
			this.floatingRateResetFrequencyPeriodOfLeg1 = builder.getFloatingRateResetFrequencyPeriodOfLeg1();
			this.floatingRateResetFrequencyPeriodMultiplierOfLeg1 = builder.getFloatingRateResetFrequencyPeriodMultiplierOfLeg1();
			this.floatingRateResetFrequencyPeriodOfLeg2 = builder.getFloatingRateResetFrequencyPeriodOfLeg2();
			this.floatingRateResetFrequencyPeriodMultiplierOfLeg2 = builder.getFloatingRateResetFrequencyPeriodMultiplierOfLeg2();
			this.fixedRatePaymentFrequencyPeriodOfLeg1 = builder.getFixedRatePaymentFrequencyPeriodOfLeg1();
			this.fixedRatePaymentFrequencyPeriodMultiplierOfLeg1 = builder.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg1();
			this.fixedRatePaymentFrequencyPeriodOfLeg2 = builder.getFixedRatePaymentFrequencyPeriodOfLeg2();
			this.fixedRatePaymentFrequencyPeriodMultiplierOfLeg2 = builder.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2();
			this.floatingRatePaymentFrequencyPeriodOfLeg1 = builder.getFloatingRatePaymentFrequencyPeriodOfLeg1();
			this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg1 = builder.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1();
			this.floatingRatePaymentFrequencyPeriodOfLeg2 = builder.getFloatingRatePaymentFrequencyPeriodOfLeg2();
			this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 = builder.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2();
			this.otherPayment = ofNullable(builder.getOtherPayment()).filter(_l->!_l.isEmpty()).map(list -> list.stream().filter(Objects::nonNull).map(f->f.build()).filter(Objects::nonNull).collect(ImmutableList.toImmutableList())).orElse(null);
			this.actionType = builder.getActionType();
			this.eventType = builder.getEventType();
			this.eventDate = builder.getEventDate();
		}
		
		@Override
		@RosettaAttribute("counterparty1")
		public String getCounterparty1() {
			return counterparty1;
		}
		
		@Override
		@RosettaAttribute("counterparty2")
		public String getCounterparty2() {
			return counterparty2;
		}
		
		@Override
		@RosettaAttribute("counterparty2IdentifierType")
		public Boolean getCounterparty2IdentifierType() {
			return counterparty2IdentifierType;
		}
		
		@Override
		@RosettaAttribute("direction")
		public String getDirection() {
			return direction;
		}
		
		@Override
		@RosettaAttribute("directionOfLeg1")
		public String getDirectionOfLeg1() {
			return directionOfLeg1;
		}
		
		@Override
		@RosettaAttribute("directionOfLeg2")
		public String getDirectionOfLeg2() {
			return directionOfLeg2;
		}
		
		@Override
		@RosettaAttribute("reportingSpecifiedPerson")
		public String getReportingSpecifiedPerson() {
			return reportingSpecifiedPerson;
		}
		
		@Override
		@RosettaAttribute("dataSubmitter")
		public String getDataSubmitter() {
			return dataSubmitter;
		}
		
		@Override
		@RosettaAttribute("uniqueTransactionIdentifier")
		public String getUniqueTransactionIdentifier() {
			return uniqueTransactionIdentifier;
		}
		
		@Override
		@RosettaAttribute("uniqueProductIdentifier")
		public String getUniqueProductIdentifier() {
			return uniqueProductIdentifier;
		}
		
		@Override
		@RosettaAttribute("priorUTI")
		public String getPriorUTI() {
			return priorUTI;
		}
		
		@Override
		@RosettaAttribute("contractType")
		public String getContractType() {
			return contractType;
		}
		
		@Override
		@RosettaAttribute("assetClass")
		public String getAssetClass() {
			return assetClass;
		}
		
		@Override
		@RosettaAttribute("underlying")
		public String getUnderlying() {
			return underlying;
		}
		
		@Override
		@RosettaAttribute("underlyingIdentificationType")
		public String getUnderlyingIdentificationType() {
			return underlyingIdentificationType;
		}
		
		@Override
		@RosettaAttribute("underlyingIdOther")
		public String getUnderlyingIdOther() {
			return underlyingIdOther;
		}
		
		@Override
		@RosettaAttribute("underlyingIdOtherSource")
		public ProductIdTypeEnum getUnderlyingIdOtherSource() {
			return underlyingIdOtherSource;
		}
		
		@Override
		@RosettaAttribute("nameOfTheUnderlyingIndex")
		public String getNameOfTheUnderlyingIndex() {
			return nameOfTheUnderlyingIndex;
		}
		
		@Override
		@RosettaAttribute("fixedRateOfLeg1")
		public BigDecimal getFixedRateOfLeg1() {
			return fixedRateOfLeg1;
		}
		
		@Override
		@RosettaAttribute("identifierOfFloatingRateOfLeg1")
		public String getIdentifierOfFloatingRateOfLeg1() {
			return identifierOfFloatingRateOfLeg1;
		}
		
		@Override
		@RosettaAttribute("nameOfTheFloatingRateOfLeg1")
		public String getNameOfTheFloatingRateOfLeg1() {
			return nameOfTheFloatingRateOfLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadOfLeg1")
		public SpreadOfLeg1 getSpreadOfLeg1() {
			return spreadOfLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRateOfLeg2")
		public BigDecimal getFixedRateOfLeg2() {
			return fixedRateOfLeg2;
		}
		
		@Override
		@RosettaAttribute("identifierOfFloatingRateOfLeg2")
		public String getIdentifierOfFloatingRateOfLeg2() {
			return identifierOfFloatingRateOfLeg2;
		}
		
		@Override
		@RosettaAttribute("nameOfTheFloatingRateOfLeg2")
		public String getNameOfTheFloatingRateOfLeg2() {
			return nameOfTheFloatingRateOfLeg2;
		}
		
		@Override
		@RosettaAttribute("spreadOfLeg2")
		public SpreadOfLeg2 getSpreadOfLeg2() {
			return spreadOfLeg2;
		}
		
		@Override
		@RosettaAttribute("optionType")
		public String getOptionType() {
			return optionType;
		}
		
		@Override
		@RosettaAttribute("optionStyle")
		public String getOptionStyle() {
			return optionStyle;
		}
		
		@Override
		@RosettaAttribute("firstExerciseDate")
		public Date getFirstExerciseDate() {
			return firstExerciseDate;
		}
		
		@Override
		@RosettaAttribute("effectiveDate")
		public Date getEffectiveDate() {
			return effectiveDate;
		}
		
		@Override
		@RosettaAttribute("expirationDate")
		public Date getExpirationDate() {
			return expirationDate;
		}
		
		@Override
		@RosettaAttribute("swapLinkID")
		public String getSwapLinkID() {
			return swapLinkID;
		}
		
		@Override
		@RosettaAttribute("packageIdentifier")
		public String getPackageIdentifier() {
			return packageIdentifier;
		}
		
		@Override
		@RosettaAttribute("deliveryType")
		public String getDeliveryType() {
			return deliveryType;
		}
		
		@Override
		@RosettaAttribute("settlementCurrency1")
		public ISOCurrencyCodeEnum getSettlementCurrency1() {
			return settlementCurrency1;
		}
		
		@Override
		@RosettaAttribute("settlementCurrency2")
		public ISOCurrencyCodeEnum getSettlementCurrency2() {
			return settlementCurrency2;
		}
		
		@Override
		@RosettaAttribute("finalContractualSettlementDate")
		public Date getFinalContractualSettlementDate() {
			return finalContractualSettlementDate;
		}
		
		@Override
		@RosettaAttribute("cleared")
		public String getCleared() {
			return cleared;
		}
		
		@Override
		@RosettaAttribute("centralCounterparty")
		public String getCentralCounterparty() {
			return centralCounterparty;
		}
		
		@Override
		@RosettaAttribute("clearingMember")
		public String getClearingMember() {
			return clearingMember;
		}
		
		@Override
		@RosettaAttribute("clearingTimestamp")
		public ZonedDateTime getClearingTimestamp() {
			return clearingTimestamp;
		}
		
		@Override
		@RosettaAttribute("platformIdentifier")
		public String getPlatformIdentifier() {
			return platformIdentifier;
		}
		
		@Override
		@RosettaAttribute("reportingObligationOfSpecifiedPerson")
		public String getReportingObligationOfSpecifiedPerson() {
			return reportingObligationOfSpecifiedPerson;
		}
		
		@Override
		@RosettaAttribute("bookingLocation")
		public String getBookingLocation() {
			return bookingLocation;
		}
		
		@Override
		@RosettaAttribute("traderLocation")
		public String getTraderLocation() {
			return traderLocation;
		}
		
		@Override
		@RosettaAttribute("confirmed")
		public String getConfirmed() {
			return confirmed;
		}
		
		@Override
		@RosettaAttribute("confirmationTimestamp")
		public ZonedDateTime getConfirmationTimestamp() {
			return confirmationTimestamp;
		}
		
		@Override
		@RosettaAttribute("executionTimestamp")
		public ZonedDateTime getExecutionTimestamp() {
			return executionTimestamp;
		}
		
		@Override
		@RosettaAttribute("reportingTimestamp")
		public ZonedDateTime getReportingTimestamp() {
			return reportingTimestamp;
		}
		
		@Override
		@RosettaAttribute("basketConstituents")
		public List<? extends BasketConstituentsReport> getBasketConstituents() {
			return basketConstituents;
		}
		
		@Override
		@RosettaAttribute("valuationAmount")
		public BigDecimal getValuationAmount() {
			return valuationAmount;
		}
		
		@Override
		@RosettaAttribute("valuationCurrency")
		public ISOCurrencyCodeEnum getValuationCurrency() {
			return valuationCurrency;
		}
		
		@Override
		@RosettaAttribute("valuationMethod")
		public ValuationType1Code getValuationMethod() {
			return valuationMethod;
		}
		
		@Override
		@RosettaAttribute("valuationTimestamp")
		public ZonedDateTime getValuationTimestamp() {
			return valuationTimestamp;
		}
		
		@Override
		@RosettaAttribute("collateralPortfolioIndicator")
		public Boolean getCollateralPortfolioIndicator() {
			return collateralPortfolioIndicator;
		}
		
		@Override
		@RosettaAttribute("initialMarginCollateralPortfolioCode")
		public String getInitialMarginCollateralPortfolioCode() {
			return initialMarginCollateralPortfolioCode;
		}
		
		@Override
		@RosettaAttribute("variationMarginCollateralPortfolioCode")
		public String getVariationMarginCollateralPortfolioCode() {
			return variationMarginCollateralPortfolioCode;
		}
		
		@Override
		@RosettaAttribute("notionalAmountOfTheContractOrLeg1")
		public BigDecimal getNotionalAmountOfTheContractOrLeg1() {
			return notionalAmountOfTheContractOrLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalCurrencyOfTheContractOrLeg1")
		public ISOCurrencyCodeEnum getNotionalCurrencyOfTheContractOrLeg1() {
			return notionalCurrencyOfTheContractOrLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalAmountOfLeg2")
		public BigDecimal getNotionalAmountOfLeg2() {
			return notionalAmountOfLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalCurrencyOfLeg2")
		public ISOCurrencyCodeEnum getNotionalCurrencyOfLeg2() {
			return notionalCurrencyOfLeg2;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityOfTheContractOrLeg1")
		public BigDecimal getTotalNotionalQuantityOfTheContractOrLeg1() {
			return totalNotionalQuantityOfTheContractOrLeg1;
		}
		
		@Override
		@RosettaAttribute("quantityUnitOfMeasureForTheContractOrLeg1")
		public String getQuantityUnitOfMeasureForTheContractOrLeg1() {
			return quantityUnitOfMeasureForTheContractOrLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityOfTheContractOrLeg1")
		public BigDecimal getNotionalQuantityOfTheContractOrLeg1() {
			return notionalQuantityOfTheContractOrLeg1;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyOfTheContractOrLeg1")
		public String getQuantityFrequencyOfTheContractOrLeg1() {
			return quantityFrequencyOfTheContractOrLeg1;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyMultiplierOfTheContractOrLeg1")
		public Integer getQuantityFrequencyMultiplierOfTheContractOrLeg1() {
			return quantityFrequencyMultiplierOfTheContractOrLeg1;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg2")
		public BigDecimal getTotalNotionalQuantityLeg2() {
			return totalNotionalQuantityLeg2;
		}
		
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg2")
		public String getQuantityUnitOfMeasureLeg2() {
			return quantityUnitOfMeasureLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityOfLeg2")
		public BigDecimal getNotionalQuantityOfLeg2() {
			return notionalQuantityOfLeg2;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyOfLeg2")
		public String getQuantityFrequencyOfLeg2() {
			return quantityFrequencyOfLeg2;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyMultiplierOfLeg2")
		public Integer getQuantityFrequencyMultiplierOfLeg2() {
			return quantityFrequencyMultiplierOfLeg2;
		}
		
		@Override
		@RosettaAttribute("callAmount")
		public BigDecimal getCallAmount() {
			return callAmount;
		}
		
		@Override
		@RosettaAttribute("callCurrency")
		public ISOCurrencyCodeEnum getCallCurrency() {
			return callCurrency;
		}
		
		@Override
		@RosettaAttribute("putAmount")
		public BigDecimal getPutAmount() {
			return putAmount;
		}
		
		@Override
		@RosettaAttribute("putCurrency")
		public ISOCurrencyCodeEnum getPutCurrency() {
			return putCurrency;
		}
		
		@Override
		@RosettaAttribute("price")
		public BigDecimal getPrice() {
			return price;
		}
		
		@Override
		@RosettaAttribute("priceCurrency")
		public ISOCurrencyCodeEnum getPriceCurrency() {
			return priceCurrency;
		}
		
		@Override
		@RosettaAttribute("priceNotation")
		public String getPriceNotation() {
			return priceNotation;
		}
		
		@Override
		@RosettaAttribute("priceUnitOfMeasure")
		public String getPriceUnitOfMeasure() {
			return priceUnitOfMeasure;
		}
		
		@Override
		@RosettaAttribute("priceSchedule")
		public List<? extends PriceScheduleReport> getPriceSchedule() {
			return priceSchedule;
		}
		
		@Override
		@RosettaAttribute("strikePrice")
		public BigDecimal getStrikePrice() {
			return strikePrice;
		}
		
		@Override
		@RosettaAttribute("strikePriceCurrency")
		public String getStrikePriceCurrency() {
			return strikePriceCurrency;
		}
		
		@Override
		@RosettaAttribute("strikePriceNotation")
		public String getStrikePriceNotation() {
			return strikePriceNotation;
		}
		
		@Override
		@RosettaAttribute("strikePriceSchedule")
		public List<? extends StrikePriceScheduleReport> getStrikePriceSchedule() {
			return strikePriceSchedule;
		}
		
		@Override
		@RosettaAttribute("optionPremiumAmount")
		public BigDecimal getOptionPremiumAmount() {
			return optionPremiumAmount;
		}
		
		@Override
		@RosettaAttribute("optionPremiumCurrency")
		public ISOCurrencyCodeEnum getOptionPremiumCurrency() {
			return optionPremiumCurrency;
		}
		
		@Override
		@RosettaAttribute("exchangeRate")
		public BigDecimal getExchangeRate() {
			return exchangeRate;
		}
		
		@Override
		@RosettaAttribute("exchangeRateBasis")
		public String getExchangeRateBasis() {
			return exchangeRateBasis;
		}
		
		@Override
		@RosettaAttribute("fixedRateDayCountConventionOfLeg1")
		public InterestComputationMethod4Code getFixedRateDayCountConventionOfLeg1() {
			return fixedRateDayCountConventionOfLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRateDayCountConventionOfLeg2")
		public InterestComputationMethod4Code getFixedRateDayCountConventionOfLeg2() {
			return fixedRateDayCountConventionOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateDayCountConventionOfLeg1")
		public InterestComputationMethod4Code getFloatingRateDayCountConventionOfLeg1() {
			return floatingRateDayCountConventionOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateDayCountConventionOfLeg2")
		public InterestComputationMethod4Code getFloatingRateDayCountConventionOfLeg2() {
			return floatingRateDayCountConventionOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodOfLeg1")
		public String getFloatingRateResetFrequencyPeriodOfLeg1() {
			return floatingRateResetFrequencyPeriodOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierOfLeg1")
		public Integer getFloatingRateResetFrequencyPeriodMultiplierOfLeg1() {
			return floatingRateResetFrequencyPeriodMultiplierOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodOfLeg2")
		public String getFloatingRateResetFrequencyPeriodOfLeg2() {
			return floatingRateResetFrequencyPeriodOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierOfLeg2")
		public Integer getFloatingRateResetFrequencyPeriodMultiplierOfLeg2() {
			return floatingRateResetFrequencyPeriodMultiplierOfLeg2;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodOfLeg1")
		public String getFixedRatePaymentFrequencyPeriodOfLeg1() {
			return fixedRatePaymentFrequencyPeriodOfLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierOfLeg1")
		public Integer getFixedRatePaymentFrequencyPeriodMultiplierOfLeg1() {
			return fixedRatePaymentFrequencyPeriodMultiplierOfLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodOfLeg2")
		public String getFixedRatePaymentFrequencyPeriodOfLeg2() {
			return fixedRatePaymentFrequencyPeriodOfLeg2;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierOfLeg2")
		public Integer getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2() {
			return fixedRatePaymentFrequencyPeriodMultiplierOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodOfLeg1")
		public Frequency13Code__1 getFloatingRatePaymentFrequencyPeriodOfLeg1() {
			return floatingRatePaymentFrequencyPeriodOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierOfLeg1")
		public Integer getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1() {
			return floatingRatePaymentFrequencyPeriodMultiplierOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodOfLeg2")
		public Frequency13Code__1 getFloatingRatePaymentFrequencyPeriodOfLeg2() {
			return floatingRatePaymentFrequencyPeriodOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierOfLeg2")
		public Integer getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2() {
			return floatingRatePaymentFrequencyPeriodMultiplierOfLeg2;
		}
		
		@Override
		@RosettaAttribute("otherPayment")
		public List<? extends OtherPaymentReport> getOtherPayment() {
			return otherPayment;
		}
		
		@Override
		@RosettaAttribute("actionType")
		public String getActionType() {
			return actionType;
		}
		
		@Override
		@RosettaAttribute("eventType")
		public String getEventType() {
			return eventType;
		}
		
		@Override
		@RosettaAttribute("eventDate")
		public Date getEventDate() {
			return eventDate;
		}
		
		@Override
		public MASTransactionReport build() {
			return this;
		}
		
		@Override
		public MASTransactionReport.MASTransactionReportBuilder toBuilder() {
			MASTransactionReport.MASTransactionReportBuilder builder = builder();
			setBuilderFields(builder);
			return builder;
		}
		
		protected void setBuilderFields(MASTransactionReport.MASTransactionReportBuilder builder) {
			ofNullable(getCounterparty1()).ifPresent(builder::setCounterparty1);
			ofNullable(getCounterparty2()).ifPresent(builder::setCounterparty2);
			ofNullable(getCounterparty2IdentifierType()).ifPresent(builder::setCounterparty2IdentifierType);
			ofNullable(getDirection()).ifPresent(builder::setDirection);
			ofNullable(getDirectionOfLeg1()).ifPresent(builder::setDirectionOfLeg1);
			ofNullable(getDirectionOfLeg2()).ifPresent(builder::setDirectionOfLeg2);
			ofNullable(getReportingSpecifiedPerson()).ifPresent(builder::setReportingSpecifiedPerson);
			ofNullable(getDataSubmitter()).ifPresent(builder::setDataSubmitter);
			ofNullable(getUniqueTransactionIdentifier()).ifPresent(builder::setUniqueTransactionIdentifier);
			ofNullable(getUniqueProductIdentifier()).ifPresent(builder::setUniqueProductIdentifier);
			ofNullable(getPriorUTI()).ifPresent(builder::setPriorUTI);
			ofNullable(getContractType()).ifPresent(builder::setContractType);
			ofNullable(getAssetClass()).ifPresent(builder::setAssetClass);
			ofNullable(getUnderlying()).ifPresent(builder::setUnderlying);
			ofNullable(getUnderlyingIdentificationType()).ifPresent(builder::setUnderlyingIdentificationType);
			ofNullable(getUnderlyingIdOther()).ifPresent(builder::setUnderlyingIdOther);
			ofNullable(getUnderlyingIdOtherSource()).ifPresent(builder::setUnderlyingIdOtherSource);
			ofNullable(getNameOfTheUnderlyingIndex()).ifPresent(builder::setNameOfTheUnderlyingIndex);
			ofNullable(getFixedRateOfLeg1()).ifPresent(builder::setFixedRateOfLeg1);
			ofNullable(getIdentifierOfFloatingRateOfLeg1()).ifPresent(builder::setIdentifierOfFloatingRateOfLeg1);
			ofNullable(getNameOfTheFloatingRateOfLeg1()).ifPresent(builder::setNameOfTheFloatingRateOfLeg1);
			ofNullable(getSpreadOfLeg1()).ifPresent(builder::setSpreadOfLeg1);
			ofNullable(getFixedRateOfLeg2()).ifPresent(builder::setFixedRateOfLeg2);
			ofNullable(getIdentifierOfFloatingRateOfLeg2()).ifPresent(builder::setIdentifierOfFloatingRateOfLeg2);
			ofNullable(getNameOfTheFloatingRateOfLeg2()).ifPresent(builder::setNameOfTheFloatingRateOfLeg2);
			ofNullable(getSpreadOfLeg2()).ifPresent(builder::setSpreadOfLeg2);
			ofNullable(getOptionType()).ifPresent(builder::setOptionType);
			ofNullable(getOptionStyle()).ifPresent(builder::setOptionStyle);
			ofNullable(getFirstExerciseDate()).ifPresent(builder::setFirstExerciseDate);
			ofNullable(getEffectiveDate()).ifPresent(builder::setEffectiveDate);
			ofNullable(getExpirationDate()).ifPresent(builder::setExpirationDate);
			ofNullable(getSwapLinkID()).ifPresent(builder::setSwapLinkID);
			ofNullable(getPackageIdentifier()).ifPresent(builder::setPackageIdentifier);
			ofNullable(getDeliveryType()).ifPresent(builder::setDeliveryType);
			ofNullable(getSettlementCurrency1()).ifPresent(builder::setSettlementCurrency1);
			ofNullable(getSettlementCurrency2()).ifPresent(builder::setSettlementCurrency2);
			ofNullable(getFinalContractualSettlementDate()).ifPresent(builder::setFinalContractualSettlementDate);
			ofNullable(getCleared()).ifPresent(builder::setCleared);
			ofNullable(getCentralCounterparty()).ifPresent(builder::setCentralCounterparty);
			ofNullable(getClearingMember()).ifPresent(builder::setClearingMember);
			ofNullable(getClearingTimestamp()).ifPresent(builder::setClearingTimestamp);
			ofNullable(getPlatformIdentifier()).ifPresent(builder::setPlatformIdentifier);
			ofNullable(getReportingObligationOfSpecifiedPerson()).ifPresent(builder::setReportingObligationOfSpecifiedPerson);
			ofNullable(getBookingLocation()).ifPresent(builder::setBookingLocation);
			ofNullable(getTraderLocation()).ifPresent(builder::setTraderLocation);
			ofNullable(getConfirmed()).ifPresent(builder::setConfirmed);
			ofNullable(getConfirmationTimestamp()).ifPresent(builder::setConfirmationTimestamp);
			ofNullable(getExecutionTimestamp()).ifPresent(builder::setExecutionTimestamp);
			ofNullable(getReportingTimestamp()).ifPresent(builder::setReportingTimestamp);
			ofNullable(getBasketConstituents()).ifPresent(builder::setBasketConstituents);
			ofNullable(getValuationAmount()).ifPresent(builder::setValuationAmount);
			ofNullable(getValuationCurrency()).ifPresent(builder::setValuationCurrency);
			ofNullable(getValuationMethod()).ifPresent(builder::setValuationMethod);
			ofNullable(getValuationTimestamp()).ifPresent(builder::setValuationTimestamp);
			ofNullable(getCollateralPortfolioIndicator()).ifPresent(builder::setCollateralPortfolioIndicator);
			ofNullable(getInitialMarginCollateralPortfolioCode()).ifPresent(builder::setInitialMarginCollateralPortfolioCode);
			ofNullable(getVariationMarginCollateralPortfolioCode()).ifPresent(builder::setVariationMarginCollateralPortfolioCode);
			ofNullable(getNotionalAmountOfTheContractOrLeg1()).ifPresent(builder::setNotionalAmountOfTheContractOrLeg1);
			ofNullable(getNotionalCurrencyOfTheContractOrLeg1()).ifPresent(builder::setNotionalCurrencyOfTheContractOrLeg1);
			ofNullable(getNotionalAmountOfLeg2()).ifPresent(builder::setNotionalAmountOfLeg2);
			ofNullable(getNotionalCurrencyOfLeg2()).ifPresent(builder::setNotionalCurrencyOfLeg2);
			ofNullable(getTotalNotionalQuantityOfTheContractOrLeg1()).ifPresent(builder::setTotalNotionalQuantityOfTheContractOrLeg1);
			ofNullable(getQuantityUnitOfMeasureForTheContractOrLeg1()).ifPresent(builder::setQuantityUnitOfMeasureForTheContractOrLeg1);
			ofNullable(getNotionalQuantityOfTheContractOrLeg1()).ifPresent(builder::setNotionalQuantityOfTheContractOrLeg1);
			ofNullable(getQuantityFrequencyOfTheContractOrLeg1()).ifPresent(builder::setQuantityFrequencyOfTheContractOrLeg1);
			ofNullable(getQuantityFrequencyMultiplierOfTheContractOrLeg1()).ifPresent(builder::setQuantityFrequencyMultiplierOfTheContractOrLeg1);
			ofNullable(getTotalNotionalQuantityLeg2()).ifPresent(builder::setTotalNotionalQuantityLeg2);
			ofNullable(getQuantityUnitOfMeasureLeg2()).ifPresent(builder::setQuantityUnitOfMeasureLeg2);
			ofNullable(getNotionalQuantityOfLeg2()).ifPresent(builder::setNotionalQuantityOfLeg2);
			ofNullable(getQuantityFrequencyOfLeg2()).ifPresent(builder::setQuantityFrequencyOfLeg2);
			ofNullable(getQuantityFrequencyMultiplierOfLeg2()).ifPresent(builder::setQuantityFrequencyMultiplierOfLeg2);
			ofNullable(getCallAmount()).ifPresent(builder::setCallAmount);
			ofNullable(getCallCurrency()).ifPresent(builder::setCallCurrency);
			ofNullable(getPutAmount()).ifPresent(builder::setPutAmount);
			ofNullable(getPutCurrency()).ifPresent(builder::setPutCurrency);
			ofNullable(getPrice()).ifPresent(builder::setPrice);
			ofNullable(getPriceCurrency()).ifPresent(builder::setPriceCurrency);
			ofNullable(getPriceNotation()).ifPresent(builder::setPriceNotation);
			ofNullable(getPriceUnitOfMeasure()).ifPresent(builder::setPriceUnitOfMeasure);
			ofNullable(getPriceSchedule()).ifPresent(builder::setPriceSchedule);
			ofNullable(getStrikePrice()).ifPresent(builder::setStrikePrice);
			ofNullable(getStrikePriceCurrency()).ifPresent(builder::setStrikePriceCurrency);
			ofNullable(getStrikePriceNotation()).ifPresent(builder::setStrikePriceNotation);
			ofNullable(getStrikePriceSchedule()).ifPresent(builder::setStrikePriceSchedule);
			ofNullable(getOptionPremiumAmount()).ifPresent(builder::setOptionPremiumAmount);
			ofNullable(getOptionPremiumCurrency()).ifPresent(builder::setOptionPremiumCurrency);
			ofNullable(getExchangeRate()).ifPresent(builder::setExchangeRate);
			ofNullable(getExchangeRateBasis()).ifPresent(builder::setExchangeRateBasis);
			ofNullable(getFixedRateDayCountConventionOfLeg1()).ifPresent(builder::setFixedRateDayCountConventionOfLeg1);
			ofNullable(getFixedRateDayCountConventionOfLeg2()).ifPresent(builder::setFixedRateDayCountConventionOfLeg2);
			ofNullable(getFloatingRateDayCountConventionOfLeg1()).ifPresent(builder::setFloatingRateDayCountConventionOfLeg1);
			ofNullable(getFloatingRateDayCountConventionOfLeg2()).ifPresent(builder::setFloatingRateDayCountConventionOfLeg2);
			ofNullable(getFloatingRateResetFrequencyPeriodOfLeg1()).ifPresent(builder::setFloatingRateResetFrequencyPeriodOfLeg1);
			ofNullable(getFloatingRateResetFrequencyPeriodMultiplierOfLeg1()).ifPresent(builder::setFloatingRateResetFrequencyPeriodMultiplierOfLeg1);
			ofNullable(getFloatingRateResetFrequencyPeriodOfLeg2()).ifPresent(builder::setFloatingRateResetFrequencyPeriodOfLeg2);
			ofNullable(getFloatingRateResetFrequencyPeriodMultiplierOfLeg2()).ifPresent(builder::setFloatingRateResetFrequencyPeriodMultiplierOfLeg2);
			ofNullable(getFixedRatePaymentFrequencyPeriodOfLeg1()).ifPresent(builder::setFixedRatePaymentFrequencyPeriodOfLeg1);
			ofNullable(getFixedRatePaymentFrequencyPeriodMultiplierOfLeg1()).ifPresent(builder::setFixedRatePaymentFrequencyPeriodMultiplierOfLeg1);
			ofNullable(getFixedRatePaymentFrequencyPeriodOfLeg2()).ifPresent(builder::setFixedRatePaymentFrequencyPeriodOfLeg2);
			ofNullable(getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2()).ifPresent(builder::setFixedRatePaymentFrequencyPeriodMultiplierOfLeg2);
			ofNullable(getFloatingRatePaymentFrequencyPeriodOfLeg1()).ifPresent(builder::setFloatingRatePaymentFrequencyPeriodOfLeg1);
			ofNullable(getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1()).ifPresent(builder::setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1);
			ofNullable(getFloatingRatePaymentFrequencyPeriodOfLeg2()).ifPresent(builder::setFloatingRatePaymentFrequencyPeriodOfLeg2);
			ofNullable(getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2()).ifPresent(builder::setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2);
			ofNullable(getOtherPayment()).ifPresent(builder::setOtherPayment);
			ofNullable(getActionType()).ifPresent(builder::setActionType);
			ofNullable(getEventType()).ifPresent(builder::setEventType);
			ofNullable(getEventDate()).ifPresent(builder::setEventDate);
		}

		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			MASTransactionReport _that = getType().cast(o);
		
			if (!Objects.equals(counterparty1, _that.getCounterparty1())) return false;
			if (!Objects.equals(counterparty2, _that.getCounterparty2())) return false;
			if (!Objects.equals(counterparty2IdentifierType, _that.getCounterparty2IdentifierType())) return false;
			if (!Objects.equals(direction, _that.getDirection())) return false;
			if (!Objects.equals(directionOfLeg1, _that.getDirectionOfLeg1())) return false;
			if (!Objects.equals(directionOfLeg2, _that.getDirectionOfLeg2())) return false;
			if (!Objects.equals(reportingSpecifiedPerson, _that.getReportingSpecifiedPerson())) return false;
			if (!Objects.equals(dataSubmitter, _that.getDataSubmitter())) return false;
			if (!Objects.equals(uniqueTransactionIdentifier, _that.getUniqueTransactionIdentifier())) return false;
			if (!Objects.equals(uniqueProductIdentifier, _that.getUniqueProductIdentifier())) return false;
			if (!Objects.equals(priorUTI, _that.getPriorUTI())) return false;
			if (!Objects.equals(contractType, _that.getContractType())) return false;
			if (!Objects.equals(assetClass, _that.getAssetClass())) return false;
			if (!Objects.equals(underlying, _that.getUnderlying())) return false;
			if (!Objects.equals(underlyingIdentificationType, _that.getUnderlyingIdentificationType())) return false;
			if (!Objects.equals(underlyingIdOther, _that.getUnderlyingIdOther())) return false;
			if (!Objects.equals(underlyingIdOtherSource, _that.getUnderlyingIdOtherSource())) return false;
			if (!Objects.equals(nameOfTheUnderlyingIndex, _that.getNameOfTheUnderlyingIndex())) return false;
			if (!Objects.equals(fixedRateOfLeg1, _that.getFixedRateOfLeg1())) return false;
			if (!Objects.equals(identifierOfFloatingRateOfLeg1, _that.getIdentifierOfFloatingRateOfLeg1())) return false;
			if (!Objects.equals(nameOfTheFloatingRateOfLeg1, _that.getNameOfTheFloatingRateOfLeg1())) return false;
			if (!Objects.equals(spreadOfLeg1, _that.getSpreadOfLeg1())) return false;
			if (!Objects.equals(fixedRateOfLeg2, _that.getFixedRateOfLeg2())) return false;
			if (!Objects.equals(identifierOfFloatingRateOfLeg2, _that.getIdentifierOfFloatingRateOfLeg2())) return false;
			if (!Objects.equals(nameOfTheFloatingRateOfLeg2, _that.getNameOfTheFloatingRateOfLeg2())) return false;
			if (!Objects.equals(spreadOfLeg2, _that.getSpreadOfLeg2())) return false;
			if (!Objects.equals(optionType, _that.getOptionType())) return false;
			if (!Objects.equals(optionStyle, _that.getOptionStyle())) return false;
			if (!Objects.equals(firstExerciseDate, _that.getFirstExerciseDate())) return false;
			if (!Objects.equals(effectiveDate, _that.getEffectiveDate())) return false;
			if (!Objects.equals(expirationDate, _that.getExpirationDate())) return false;
			if (!Objects.equals(swapLinkID, _that.getSwapLinkID())) return false;
			if (!Objects.equals(packageIdentifier, _that.getPackageIdentifier())) return false;
			if (!Objects.equals(deliveryType, _that.getDeliveryType())) return false;
			if (!Objects.equals(settlementCurrency1, _that.getSettlementCurrency1())) return false;
			if (!Objects.equals(settlementCurrency2, _that.getSettlementCurrency2())) return false;
			if (!Objects.equals(finalContractualSettlementDate, _that.getFinalContractualSettlementDate())) return false;
			if (!Objects.equals(cleared, _that.getCleared())) return false;
			if (!Objects.equals(centralCounterparty, _that.getCentralCounterparty())) return false;
			if (!Objects.equals(clearingMember, _that.getClearingMember())) return false;
			if (!Objects.equals(clearingTimestamp, _that.getClearingTimestamp())) return false;
			if (!Objects.equals(platformIdentifier, _that.getPlatformIdentifier())) return false;
			if (!Objects.equals(reportingObligationOfSpecifiedPerson, _that.getReportingObligationOfSpecifiedPerson())) return false;
			if (!Objects.equals(bookingLocation, _that.getBookingLocation())) return false;
			if (!Objects.equals(traderLocation, _that.getTraderLocation())) return false;
			if (!Objects.equals(confirmed, _that.getConfirmed())) return false;
			if (!Objects.equals(confirmationTimestamp, _that.getConfirmationTimestamp())) return false;
			if (!Objects.equals(executionTimestamp, _that.getExecutionTimestamp())) return false;
			if (!Objects.equals(reportingTimestamp, _that.getReportingTimestamp())) return false;
			if (!ListEquals.listEquals(basketConstituents, _that.getBasketConstituents())) return false;
			if (!Objects.equals(valuationAmount, _that.getValuationAmount())) return false;
			if (!Objects.equals(valuationCurrency, _that.getValuationCurrency())) return false;
			if (!Objects.equals(valuationMethod, _that.getValuationMethod())) return false;
			if (!Objects.equals(valuationTimestamp, _that.getValuationTimestamp())) return false;
			if (!Objects.equals(collateralPortfolioIndicator, _that.getCollateralPortfolioIndicator())) return false;
			if (!Objects.equals(initialMarginCollateralPortfolioCode, _that.getInitialMarginCollateralPortfolioCode())) return false;
			if (!Objects.equals(variationMarginCollateralPortfolioCode, _that.getVariationMarginCollateralPortfolioCode())) return false;
			if (!Objects.equals(notionalAmountOfTheContractOrLeg1, _that.getNotionalAmountOfTheContractOrLeg1())) return false;
			if (!Objects.equals(notionalCurrencyOfTheContractOrLeg1, _that.getNotionalCurrencyOfTheContractOrLeg1())) return false;
			if (!Objects.equals(notionalAmountOfLeg2, _that.getNotionalAmountOfLeg2())) return false;
			if (!Objects.equals(notionalCurrencyOfLeg2, _that.getNotionalCurrencyOfLeg2())) return false;
			if (!Objects.equals(totalNotionalQuantityOfTheContractOrLeg1, _that.getTotalNotionalQuantityOfTheContractOrLeg1())) return false;
			if (!Objects.equals(quantityUnitOfMeasureForTheContractOrLeg1, _that.getQuantityUnitOfMeasureForTheContractOrLeg1())) return false;
			if (!Objects.equals(notionalQuantityOfTheContractOrLeg1, _that.getNotionalQuantityOfTheContractOrLeg1())) return false;
			if (!Objects.equals(quantityFrequencyOfTheContractOrLeg1, _that.getQuantityFrequencyOfTheContractOrLeg1())) return false;
			if (!Objects.equals(quantityFrequencyMultiplierOfTheContractOrLeg1, _that.getQuantityFrequencyMultiplierOfTheContractOrLeg1())) return false;
			if (!Objects.equals(totalNotionalQuantityLeg2, _that.getTotalNotionalQuantityLeg2())) return false;
			if (!Objects.equals(quantityUnitOfMeasureLeg2, _that.getQuantityUnitOfMeasureLeg2())) return false;
			if (!Objects.equals(notionalQuantityOfLeg2, _that.getNotionalQuantityOfLeg2())) return false;
			if (!Objects.equals(quantityFrequencyOfLeg2, _that.getQuantityFrequencyOfLeg2())) return false;
			if (!Objects.equals(quantityFrequencyMultiplierOfLeg2, _that.getQuantityFrequencyMultiplierOfLeg2())) return false;
			if (!Objects.equals(callAmount, _that.getCallAmount())) return false;
			if (!Objects.equals(callCurrency, _that.getCallCurrency())) return false;
			if (!Objects.equals(putAmount, _that.getPutAmount())) return false;
			if (!Objects.equals(putCurrency, _that.getPutCurrency())) return false;
			if (!Objects.equals(price, _that.getPrice())) return false;
			if (!Objects.equals(priceCurrency, _that.getPriceCurrency())) return false;
			if (!Objects.equals(priceNotation, _that.getPriceNotation())) return false;
			if (!Objects.equals(priceUnitOfMeasure, _that.getPriceUnitOfMeasure())) return false;
			if (!ListEquals.listEquals(priceSchedule, _that.getPriceSchedule())) return false;
			if (!Objects.equals(strikePrice, _that.getStrikePrice())) return false;
			if (!Objects.equals(strikePriceCurrency, _that.getStrikePriceCurrency())) return false;
			if (!Objects.equals(strikePriceNotation, _that.getStrikePriceNotation())) return false;
			if (!ListEquals.listEquals(strikePriceSchedule, _that.getStrikePriceSchedule())) return false;
			if (!Objects.equals(optionPremiumAmount, _that.getOptionPremiumAmount())) return false;
			if (!Objects.equals(optionPremiumCurrency, _that.getOptionPremiumCurrency())) return false;
			if (!Objects.equals(exchangeRate, _that.getExchangeRate())) return false;
			if (!Objects.equals(exchangeRateBasis, _that.getExchangeRateBasis())) return false;
			if (!Objects.equals(fixedRateDayCountConventionOfLeg1, _that.getFixedRateDayCountConventionOfLeg1())) return false;
			if (!Objects.equals(fixedRateDayCountConventionOfLeg2, _that.getFixedRateDayCountConventionOfLeg2())) return false;
			if (!Objects.equals(floatingRateDayCountConventionOfLeg1, _that.getFloatingRateDayCountConventionOfLeg1())) return false;
			if (!Objects.equals(floatingRateDayCountConventionOfLeg2, _that.getFloatingRateDayCountConventionOfLeg2())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodOfLeg1, _that.getFloatingRateResetFrequencyPeriodOfLeg1())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodMultiplierOfLeg1, _that.getFloatingRateResetFrequencyPeriodMultiplierOfLeg1())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodOfLeg2, _that.getFloatingRateResetFrequencyPeriodOfLeg2())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodMultiplierOfLeg2, _that.getFloatingRateResetFrequencyPeriodMultiplierOfLeg2())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodOfLeg1, _that.getFixedRatePaymentFrequencyPeriodOfLeg1())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodMultiplierOfLeg1, _that.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg1())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodOfLeg2, _that.getFixedRatePaymentFrequencyPeriodOfLeg2())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodMultiplierOfLeg2, _that.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodOfLeg1, _that.getFloatingRatePaymentFrequencyPeriodOfLeg1())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodMultiplierOfLeg1, _that.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodOfLeg2, _that.getFloatingRatePaymentFrequencyPeriodOfLeg2())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodMultiplierOfLeg2, _that.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2())) return false;
			if (!ListEquals.listEquals(otherPayment, _that.getOtherPayment())) return false;
			if (!Objects.equals(actionType, _that.getActionType())) return false;
			if (!Objects.equals(eventType, _that.getEventType())) return false;
			if (!Objects.equals(eventDate, _that.getEventDate())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (counterparty1 != null ? counterparty1.hashCode() : 0);
			_result = 31 * _result + (counterparty2 != null ? counterparty2.hashCode() : 0);
			_result = 31 * _result + (counterparty2IdentifierType != null ? counterparty2IdentifierType.hashCode() : 0);
			_result = 31 * _result + (direction != null ? direction.hashCode() : 0);
			_result = 31 * _result + (directionOfLeg1 != null ? directionOfLeg1.hashCode() : 0);
			_result = 31 * _result + (directionOfLeg2 != null ? directionOfLeg2.hashCode() : 0);
			_result = 31 * _result + (reportingSpecifiedPerson != null ? reportingSpecifiedPerson.hashCode() : 0);
			_result = 31 * _result + (dataSubmitter != null ? dataSubmitter.hashCode() : 0);
			_result = 31 * _result + (uniqueTransactionIdentifier != null ? uniqueTransactionIdentifier.hashCode() : 0);
			_result = 31 * _result + (uniqueProductIdentifier != null ? uniqueProductIdentifier.hashCode() : 0);
			_result = 31 * _result + (priorUTI != null ? priorUTI.hashCode() : 0);
			_result = 31 * _result + (contractType != null ? contractType.hashCode() : 0);
			_result = 31 * _result + (assetClass != null ? assetClass.hashCode() : 0);
			_result = 31 * _result + (underlying != null ? underlying.hashCode() : 0);
			_result = 31 * _result + (underlyingIdentificationType != null ? underlyingIdentificationType.hashCode() : 0);
			_result = 31 * _result + (underlyingIdOther != null ? underlyingIdOther.hashCode() : 0);
			_result = 31 * _result + (underlyingIdOtherSource != null ? underlyingIdOtherSource.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (nameOfTheUnderlyingIndex != null ? nameOfTheUnderlyingIndex.hashCode() : 0);
			_result = 31 * _result + (fixedRateOfLeg1 != null ? fixedRateOfLeg1.hashCode() : 0);
			_result = 31 * _result + (identifierOfFloatingRateOfLeg1 != null ? identifierOfFloatingRateOfLeg1.hashCode() : 0);
			_result = 31 * _result + (nameOfTheFloatingRateOfLeg1 != null ? nameOfTheFloatingRateOfLeg1.hashCode() : 0);
			_result = 31 * _result + (spreadOfLeg1 != null ? spreadOfLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRateOfLeg2 != null ? fixedRateOfLeg2.hashCode() : 0);
			_result = 31 * _result + (identifierOfFloatingRateOfLeg2 != null ? identifierOfFloatingRateOfLeg2.hashCode() : 0);
			_result = 31 * _result + (nameOfTheFloatingRateOfLeg2 != null ? nameOfTheFloatingRateOfLeg2.hashCode() : 0);
			_result = 31 * _result + (spreadOfLeg2 != null ? spreadOfLeg2.hashCode() : 0);
			_result = 31 * _result + (optionType != null ? optionType.hashCode() : 0);
			_result = 31 * _result + (optionStyle != null ? optionStyle.hashCode() : 0);
			_result = 31 * _result + (firstExerciseDate != null ? firstExerciseDate.hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (expirationDate != null ? expirationDate.hashCode() : 0);
			_result = 31 * _result + (swapLinkID != null ? swapLinkID.hashCode() : 0);
			_result = 31 * _result + (packageIdentifier != null ? packageIdentifier.hashCode() : 0);
			_result = 31 * _result + (deliveryType != null ? deliveryType.hashCode() : 0);
			_result = 31 * _result + (settlementCurrency1 != null ? settlementCurrency1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (settlementCurrency2 != null ? settlementCurrency2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (finalContractualSettlementDate != null ? finalContractualSettlementDate.hashCode() : 0);
			_result = 31 * _result + (cleared != null ? cleared.hashCode() : 0);
			_result = 31 * _result + (centralCounterparty != null ? centralCounterparty.hashCode() : 0);
			_result = 31 * _result + (clearingMember != null ? clearingMember.hashCode() : 0);
			_result = 31 * _result + (clearingTimestamp != null ? clearingTimestamp.hashCode() : 0);
			_result = 31 * _result + (platformIdentifier != null ? platformIdentifier.hashCode() : 0);
			_result = 31 * _result + (reportingObligationOfSpecifiedPerson != null ? reportingObligationOfSpecifiedPerson.hashCode() : 0);
			_result = 31 * _result + (bookingLocation != null ? bookingLocation.hashCode() : 0);
			_result = 31 * _result + (traderLocation != null ? traderLocation.hashCode() : 0);
			_result = 31 * _result + (confirmed != null ? confirmed.hashCode() : 0);
			_result = 31 * _result + (confirmationTimestamp != null ? confirmationTimestamp.hashCode() : 0);
			_result = 31 * _result + (executionTimestamp != null ? executionTimestamp.hashCode() : 0);
			_result = 31 * _result + (reportingTimestamp != null ? reportingTimestamp.hashCode() : 0);
			_result = 31 * _result + (basketConstituents != null ? basketConstituents.hashCode() : 0);
			_result = 31 * _result + (valuationAmount != null ? valuationAmount.hashCode() : 0);
			_result = 31 * _result + (valuationCurrency != null ? valuationCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (valuationMethod != null ? valuationMethod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (valuationTimestamp != null ? valuationTimestamp.hashCode() : 0);
			_result = 31 * _result + (collateralPortfolioIndicator != null ? collateralPortfolioIndicator.hashCode() : 0);
			_result = 31 * _result + (initialMarginCollateralPortfolioCode != null ? initialMarginCollateralPortfolioCode.hashCode() : 0);
			_result = 31 * _result + (variationMarginCollateralPortfolioCode != null ? variationMarginCollateralPortfolioCode.hashCode() : 0);
			_result = 31 * _result + (notionalAmountOfTheContractOrLeg1 != null ? notionalAmountOfTheContractOrLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalCurrencyOfTheContractOrLeg1 != null ? notionalCurrencyOfTheContractOrLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalAmountOfLeg2 != null ? notionalAmountOfLeg2.hashCode() : 0);
			_result = 31 * _result + (notionalCurrencyOfLeg2 != null ? notionalCurrencyOfLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityOfTheContractOrLeg1 != null ? totalNotionalQuantityOfTheContractOrLeg1.hashCode() : 0);
			_result = 31 * _result + (quantityUnitOfMeasureForTheContractOrLeg1 != null ? quantityUnitOfMeasureForTheContractOrLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalQuantityOfTheContractOrLeg1 != null ? notionalQuantityOfTheContractOrLeg1.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyOfTheContractOrLeg1 != null ? quantityFrequencyOfTheContractOrLeg1.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyMultiplierOfTheContractOrLeg1 != null ? quantityFrequencyMultiplierOfTheContractOrLeg1.hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityLeg2 != null ? totalNotionalQuantityLeg2.hashCode() : 0);
			_result = 31 * _result + (quantityUnitOfMeasureLeg2 != null ? quantityUnitOfMeasureLeg2.hashCode() : 0);
			_result = 31 * _result + (notionalQuantityOfLeg2 != null ? notionalQuantityOfLeg2.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyOfLeg2 != null ? quantityFrequencyOfLeg2.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyMultiplierOfLeg2 != null ? quantityFrequencyMultiplierOfLeg2.hashCode() : 0);
			_result = 31 * _result + (callAmount != null ? callAmount.hashCode() : 0);
			_result = 31 * _result + (callCurrency != null ? callCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (putAmount != null ? putAmount.hashCode() : 0);
			_result = 31 * _result + (putCurrency != null ? putCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (price != null ? price.hashCode() : 0);
			_result = 31 * _result + (priceCurrency != null ? priceCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (priceNotation != null ? priceNotation.hashCode() : 0);
			_result = 31 * _result + (priceUnitOfMeasure != null ? priceUnitOfMeasure.hashCode() : 0);
			_result = 31 * _result + (priceSchedule != null ? priceSchedule.hashCode() : 0);
			_result = 31 * _result + (strikePrice != null ? strikePrice.hashCode() : 0);
			_result = 31 * _result + (strikePriceCurrency != null ? strikePriceCurrency.hashCode() : 0);
			_result = 31 * _result + (strikePriceNotation != null ? strikePriceNotation.hashCode() : 0);
			_result = 31 * _result + (strikePriceSchedule != null ? strikePriceSchedule.hashCode() : 0);
			_result = 31 * _result + (optionPremiumAmount != null ? optionPremiumAmount.hashCode() : 0);
			_result = 31 * _result + (optionPremiumCurrency != null ? optionPremiumCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (exchangeRate != null ? exchangeRate.hashCode() : 0);
			_result = 31 * _result + (exchangeRateBasis != null ? exchangeRateBasis.hashCode() : 0);
			_result = 31 * _result + (fixedRateDayCountConventionOfLeg1 != null ? fixedRateDayCountConventionOfLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (fixedRateDayCountConventionOfLeg2 != null ? fixedRateDayCountConventionOfLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRateDayCountConventionOfLeg1 != null ? floatingRateDayCountConventionOfLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRateDayCountConventionOfLeg2 != null ? floatingRateDayCountConventionOfLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodOfLeg1 != null ? floatingRateResetFrequencyPeriodOfLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodMultiplierOfLeg1 != null ? floatingRateResetFrequencyPeriodMultiplierOfLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodOfLeg2 != null ? floatingRateResetFrequencyPeriodOfLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodMultiplierOfLeg2 != null ? floatingRateResetFrequencyPeriodMultiplierOfLeg2.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodOfLeg1 != null ? fixedRatePaymentFrequencyPeriodOfLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodMultiplierOfLeg1 != null ? fixedRatePaymentFrequencyPeriodMultiplierOfLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodOfLeg2 != null ? fixedRatePaymentFrequencyPeriodOfLeg2.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodMultiplierOfLeg2 != null ? fixedRatePaymentFrequencyPeriodMultiplierOfLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodOfLeg1 != null ? floatingRatePaymentFrequencyPeriodOfLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodMultiplierOfLeg1 != null ? floatingRatePaymentFrequencyPeriodMultiplierOfLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodOfLeg2 != null ? floatingRatePaymentFrequencyPeriodOfLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 != null ? floatingRatePaymentFrequencyPeriodMultiplierOfLeg2.hashCode() : 0);
			_result = 31 * _result + (otherPayment != null ? otherPayment.hashCode() : 0);
			_result = 31 * _result + (actionType != null ? actionType.hashCode() : 0);
			_result = 31 * _result + (eventType != null ? eventType.hashCode() : 0);
			_result = 31 * _result + (eventDate != null ? eventDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MASTransactionReport {" +
				"counterparty1=" + this.counterparty1 + ", " +
				"counterparty2=" + this.counterparty2 + ", " +
				"counterparty2IdentifierType=" + this.counterparty2IdentifierType + ", " +
				"direction=" + this.direction + ", " +
				"directionOfLeg1=" + this.directionOfLeg1 + ", " +
				"directionOfLeg2=" + this.directionOfLeg2 + ", " +
				"reportingSpecifiedPerson=" + this.reportingSpecifiedPerson + ", " +
				"dataSubmitter=" + this.dataSubmitter + ", " +
				"uniqueTransactionIdentifier=" + this.uniqueTransactionIdentifier + ", " +
				"uniqueProductIdentifier=" + this.uniqueProductIdentifier + ", " +
				"priorUTI=" + this.priorUTI + ", " +
				"contractType=" + this.contractType + ", " +
				"assetClass=" + this.assetClass + ", " +
				"underlying=" + this.underlying + ", " +
				"underlyingIdentificationType=" + this.underlyingIdentificationType + ", " +
				"underlyingIdOther=" + this.underlyingIdOther + ", " +
				"underlyingIdOtherSource=" + this.underlyingIdOtherSource + ", " +
				"nameOfTheUnderlyingIndex=" + this.nameOfTheUnderlyingIndex + ", " +
				"fixedRateOfLeg1=" + this.fixedRateOfLeg1 + ", " +
				"identifierOfFloatingRateOfLeg1=" + this.identifierOfFloatingRateOfLeg1 + ", " +
				"nameOfTheFloatingRateOfLeg1=" + this.nameOfTheFloatingRateOfLeg1 + ", " +
				"spreadOfLeg1=" + this.spreadOfLeg1 + ", " +
				"fixedRateOfLeg2=" + this.fixedRateOfLeg2 + ", " +
				"identifierOfFloatingRateOfLeg2=" + this.identifierOfFloatingRateOfLeg2 + ", " +
				"nameOfTheFloatingRateOfLeg2=" + this.nameOfTheFloatingRateOfLeg2 + ", " +
				"spreadOfLeg2=" + this.spreadOfLeg2 + ", " +
				"optionType=" + this.optionType + ", " +
				"optionStyle=" + this.optionStyle + ", " +
				"firstExerciseDate=" + this.firstExerciseDate + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"expirationDate=" + this.expirationDate + ", " +
				"swapLinkID=" + this.swapLinkID + ", " +
				"packageIdentifier=" + this.packageIdentifier + ", " +
				"deliveryType=" + this.deliveryType + ", " +
				"settlementCurrency1=" + this.settlementCurrency1 + ", " +
				"settlementCurrency2=" + this.settlementCurrency2 + ", " +
				"finalContractualSettlementDate=" + this.finalContractualSettlementDate + ", " +
				"cleared=" + this.cleared + ", " +
				"centralCounterparty=" + this.centralCounterparty + ", " +
				"clearingMember=" + this.clearingMember + ", " +
				"clearingTimestamp=" + this.clearingTimestamp + ", " +
				"platformIdentifier=" + this.platformIdentifier + ", " +
				"reportingObligationOfSpecifiedPerson=" + this.reportingObligationOfSpecifiedPerson + ", " +
				"bookingLocation=" + this.bookingLocation + ", " +
				"traderLocation=" + this.traderLocation + ", " +
				"confirmed=" + this.confirmed + ", " +
				"confirmationTimestamp=" + this.confirmationTimestamp + ", " +
				"executionTimestamp=" + this.executionTimestamp + ", " +
				"reportingTimestamp=" + this.reportingTimestamp + ", " +
				"basketConstituents=" + this.basketConstituents + ", " +
				"valuationAmount=" + this.valuationAmount + ", " +
				"valuationCurrency=" + this.valuationCurrency + ", " +
				"valuationMethod=" + this.valuationMethod + ", " +
				"valuationTimestamp=" + this.valuationTimestamp + ", " +
				"collateralPortfolioIndicator=" + this.collateralPortfolioIndicator + ", " +
				"initialMarginCollateralPortfolioCode=" + this.initialMarginCollateralPortfolioCode + ", " +
				"variationMarginCollateralPortfolioCode=" + this.variationMarginCollateralPortfolioCode + ", " +
				"notionalAmountOfTheContractOrLeg1=" + this.notionalAmountOfTheContractOrLeg1 + ", " +
				"notionalCurrencyOfTheContractOrLeg1=" + this.notionalCurrencyOfTheContractOrLeg1 + ", " +
				"notionalAmountOfLeg2=" + this.notionalAmountOfLeg2 + ", " +
				"notionalCurrencyOfLeg2=" + this.notionalCurrencyOfLeg2 + ", " +
				"totalNotionalQuantityOfTheContractOrLeg1=" + this.totalNotionalQuantityOfTheContractOrLeg1 + ", " +
				"quantityUnitOfMeasureForTheContractOrLeg1=" + this.quantityUnitOfMeasureForTheContractOrLeg1 + ", " +
				"notionalQuantityOfTheContractOrLeg1=" + this.notionalQuantityOfTheContractOrLeg1 + ", " +
				"quantityFrequencyOfTheContractOrLeg1=" + this.quantityFrequencyOfTheContractOrLeg1 + ", " +
				"quantityFrequencyMultiplierOfTheContractOrLeg1=" + this.quantityFrequencyMultiplierOfTheContractOrLeg1 + ", " +
				"totalNotionalQuantityLeg2=" + this.totalNotionalQuantityLeg2 + ", " +
				"quantityUnitOfMeasureLeg2=" + this.quantityUnitOfMeasureLeg2 + ", " +
				"notionalQuantityOfLeg2=" + this.notionalQuantityOfLeg2 + ", " +
				"quantityFrequencyOfLeg2=" + this.quantityFrequencyOfLeg2 + ", " +
				"quantityFrequencyMultiplierOfLeg2=" + this.quantityFrequencyMultiplierOfLeg2 + ", " +
				"callAmount=" + this.callAmount + ", " +
				"callCurrency=" + this.callCurrency + ", " +
				"putAmount=" + this.putAmount + ", " +
				"putCurrency=" + this.putCurrency + ", " +
				"price=" + this.price + ", " +
				"priceCurrency=" + this.priceCurrency + ", " +
				"priceNotation=" + this.priceNotation + ", " +
				"priceUnitOfMeasure=" + this.priceUnitOfMeasure + ", " +
				"priceSchedule=" + this.priceSchedule + ", " +
				"strikePrice=" + this.strikePrice + ", " +
				"strikePriceCurrency=" + this.strikePriceCurrency + ", " +
				"strikePriceNotation=" + this.strikePriceNotation + ", " +
				"strikePriceSchedule=" + this.strikePriceSchedule + ", " +
				"optionPremiumAmount=" + this.optionPremiumAmount + ", " +
				"optionPremiumCurrency=" + this.optionPremiumCurrency + ", " +
				"exchangeRate=" + this.exchangeRate + ", " +
				"exchangeRateBasis=" + this.exchangeRateBasis + ", " +
				"fixedRateDayCountConventionOfLeg1=" + this.fixedRateDayCountConventionOfLeg1 + ", " +
				"fixedRateDayCountConventionOfLeg2=" + this.fixedRateDayCountConventionOfLeg2 + ", " +
				"floatingRateDayCountConventionOfLeg1=" + this.floatingRateDayCountConventionOfLeg1 + ", " +
				"floatingRateDayCountConventionOfLeg2=" + this.floatingRateDayCountConventionOfLeg2 + ", " +
				"floatingRateResetFrequencyPeriodOfLeg1=" + this.floatingRateResetFrequencyPeriodOfLeg1 + ", " +
				"floatingRateResetFrequencyPeriodMultiplierOfLeg1=" + this.floatingRateResetFrequencyPeriodMultiplierOfLeg1 + ", " +
				"floatingRateResetFrequencyPeriodOfLeg2=" + this.floatingRateResetFrequencyPeriodOfLeg2 + ", " +
				"floatingRateResetFrequencyPeriodMultiplierOfLeg2=" + this.floatingRateResetFrequencyPeriodMultiplierOfLeg2 + ", " +
				"fixedRatePaymentFrequencyPeriodOfLeg1=" + this.fixedRatePaymentFrequencyPeriodOfLeg1 + ", " +
				"fixedRatePaymentFrequencyPeriodMultiplierOfLeg1=" + this.fixedRatePaymentFrequencyPeriodMultiplierOfLeg1 + ", " +
				"fixedRatePaymentFrequencyPeriodOfLeg2=" + this.fixedRatePaymentFrequencyPeriodOfLeg2 + ", " +
				"fixedRatePaymentFrequencyPeriodMultiplierOfLeg2=" + this.fixedRatePaymentFrequencyPeriodMultiplierOfLeg2 + ", " +
				"floatingRatePaymentFrequencyPeriodOfLeg1=" + this.floatingRatePaymentFrequencyPeriodOfLeg1 + ", " +
				"floatingRatePaymentFrequencyPeriodMultiplierOfLeg1=" + this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg1 + ", " +
				"floatingRatePaymentFrequencyPeriodOfLeg2=" + this.floatingRatePaymentFrequencyPeriodOfLeg2 + ", " +
				"floatingRatePaymentFrequencyPeriodMultiplierOfLeg2=" + this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 + ", " +
				"otherPayment=" + this.otherPayment + ", " +
				"actionType=" + this.actionType + ", " +
				"eventType=" + this.eventType + ", " +
				"eventDate=" + this.eventDate +
			'}';
		}
	}

	/*********************** Builder Implementation of MASTransactionReport  ***********************/
	class MASTransactionReportBuilderImpl implements MASTransactionReport.MASTransactionReportBuilder {
	
		protected String counterparty1;
		protected String counterparty2;
		protected Boolean counterparty2IdentifierType;
		protected String direction;
		protected String directionOfLeg1;
		protected String directionOfLeg2;
		protected String reportingSpecifiedPerson;
		protected String dataSubmitter;
		protected String uniqueTransactionIdentifier;
		protected String uniqueProductIdentifier;
		protected String priorUTI;
		protected String contractType;
		protected String assetClass;
		protected String underlying;
		protected String underlyingIdentificationType;
		protected String underlyingIdOther;
		protected ProductIdTypeEnum underlyingIdOtherSource;
		protected String nameOfTheUnderlyingIndex;
		protected BigDecimal fixedRateOfLeg1;
		protected String identifierOfFloatingRateOfLeg1;
		protected String nameOfTheFloatingRateOfLeg1;
		protected SpreadOfLeg1.SpreadOfLeg1Builder spreadOfLeg1;
		protected BigDecimal fixedRateOfLeg2;
		protected String identifierOfFloatingRateOfLeg2;
		protected String nameOfTheFloatingRateOfLeg2;
		protected SpreadOfLeg2.SpreadOfLeg2Builder spreadOfLeg2;
		protected String optionType;
		protected String optionStyle;
		protected Date firstExerciseDate;
		protected Date effectiveDate;
		protected Date expirationDate;
		protected String swapLinkID;
		protected String packageIdentifier;
		protected String deliveryType;
		protected ISOCurrencyCodeEnum settlementCurrency1;
		protected ISOCurrencyCodeEnum settlementCurrency2;
		protected Date finalContractualSettlementDate;
		protected String cleared;
		protected String centralCounterparty;
		protected String clearingMember;
		protected ZonedDateTime clearingTimestamp;
		protected String platformIdentifier;
		protected String reportingObligationOfSpecifiedPerson;
		protected String bookingLocation;
		protected String traderLocation;
		protected String confirmed;
		protected ZonedDateTime confirmationTimestamp;
		protected ZonedDateTime executionTimestamp;
		protected ZonedDateTime reportingTimestamp;
		protected List<BasketConstituentsReport.BasketConstituentsReportBuilder> basketConstituents = new ArrayList<>();
		protected BigDecimal valuationAmount;
		protected ISOCurrencyCodeEnum valuationCurrency;
		protected ValuationType1Code valuationMethod;
		protected ZonedDateTime valuationTimestamp;
		protected Boolean collateralPortfolioIndicator;
		protected String initialMarginCollateralPortfolioCode;
		protected String variationMarginCollateralPortfolioCode;
		protected BigDecimal notionalAmountOfTheContractOrLeg1;
		protected ISOCurrencyCodeEnum notionalCurrencyOfTheContractOrLeg1;
		protected BigDecimal notionalAmountOfLeg2;
		protected ISOCurrencyCodeEnum notionalCurrencyOfLeg2;
		protected BigDecimal totalNotionalQuantityOfTheContractOrLeg1;
		protected String quantityUnitOfMeasureForTheContractOrLeg1;
		protected BigDecimal notionalQuantityOfTheContractOrLeg1;
		protected String quantityFrequencyOfTheContractOrLeg1;
		protected Integer quantityFrequencyMultiplierOfTheContractOrLeg1;
		protected BigDecimal totalNotionalQuantityLeg2;
		protected String quantityUnitOfMeasureLeg2;
		protected BigDecimal notionalQuantityOfLeg2;
		protected String quantityFrequencyOfLeg2;
		protected Integer quantityFrequencyMultiplierOfLeg2;
		protected BigDecimal callAmount;
		protected ISOCurrencyCodeEnum callCurrency;
		protected BigDecimal putAmount;
		protected ISOCurrencyCodeEnum putCurrency;
		protected BigDecimal price;
		protected ISOCurrencyCodeEnum priceCurrency;
		protected String priceNotation;
		protected String priceUnitOfMeasure;
		protected List<PriceScheduleReport.PriceScheduleReportBuilder> priceSchedule = new ArrayList<>();
		protected BigDecimal strikePrice;
		protected String strikePriceCurrency;
		protected String strikePriceNotation;
		protected List<StrikePriceScheduleReport.StrikePriceScheduleReportBuilder> strikePriceSchedule = new ArrayList<>();
		protected BigDecimal optionPremiumAmount;
		protected ISOCurrencyCodeEnum optionPremiumCurrency;
		protected BigDecimal exchangeRate;
		protected String exchangeRateBasis;
		protected InterestComputationMethod4Code fixedRateDayCountConventionOfLeg1;
		protected InterestComputationMethod4Code fixedRateDayCountConventionOfLeg2;
		protected InterestComputationMethod4Code floatingRateDayCountConventionOfLeg1;
		protected InterestComputationMethod4Code floatingRateDayCountConventionOfLeg2;
		protected String floatingRateResetFrequencyPeriodOfLeg1;
		protected Integer floatingRateResetFrequencyPeriodMultiplierOfLeg1;
		protected String floatingRateResetFrequencyPeriodOfLeg2;
		protected Integer floatingRateResetFrequencyPeriodMultiplierOfLeg2;
		protected String fixedRatePaymentFrequencyPeriodOfLeg1;
		protected Integer fixedRatePaymentFrequencyPeriodMultiplierOfLeg1;
		protected String fixedRatePaymentFrequencyPeriodOfLeg2;
		protected Integer fixedRatePaymentFrequencyPeriodMultiplierOfLeg2;
		protected Frequency13Code__1 floatingRatePaymentFrequencyPeriodOfLeg1;
		protected Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg1;
		protected Frequency13Code__1 floatingRatePaymentFrequencyPeriodOfLeg2;
		protected Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg2;
		protected List<OtherPaymentReport.OtherPaymentReportBuilder> otherPayment = new ArrayList<>();
		protected String actionType;
		protected String eventType;
		protected Date eventDate;
	
		public MASTransactionReportBuilderImpl() {
		}
	
		@Override
		@RosettaAttribute("counterparty1")
		public String getCounterparty1() {
			return counterparty1;
		}
		
		@Override
		@RosettaAttribute("counterparty2")
		public String getCounterparty2() {
			return counterparty2;
		}
		
		@Override
		@RosettaAttribute("counterparty2IdentifierType")
		public Boolean getCounterparty2IdentifierType() {
			return counterparty2IdentifierType;
		}
		
		@Override
		@RosettaAttribute("direction")
		public String getDirection() {
			return direction;
		}
		
		@Override
		@RosettaAttribute("directionOfLeg1")
		public String getDirectionOfLeg1() {
			return directionOfLeg1;
		}
		
		@Override
		@RosettaAttribute("directionOfLeg2")
		public String getDirectionOfLeg2() {
			return directionOfLeg2;
		}
		
		@Override
		@RosettaAttribute("reportingSpecifiedPerson")
		public String getReportingSpecifiedPerson() {
			return reportingSpecifiedPerson;
		}
		
		@Override
		@RosettaAttribute("dataSubmitter")
		public String getDataSubmitter() {
			return dataSubmitter;
		}
		
		@Override
		@RosettaAttribute("uniqueTransactionIdentifier")
		public String getUniqueTransactionIdentifier() {
			return uniqueTransactionIdentifier;
		}
		
		@Override
		@RosettaAttribute("uniqueProductIdentifier")
		public String getUniqueProductIdentifier() {
			return uniqueProductIdentifier;
		}
		
		@Override
		@RosettaAttribute("priorUTI")
		public String getPriorUTI() {
			return priorUTI;
		}
		
		@Override
		@RosettaAttribute("contractType")
		public String getContractType() {
			return contractType;
		}
		
		@Override
		@RosettaAttribute("assetClass")
		public String getAssetClass() {
			return assetClass;
		}
		
		@Override
		@RosettaAttribute("underlying")
		public String getUnderlying() {
			return underlying;
		}
		
		@Override
		@RosettaAttribute("underlyingIdentificationType")
		public String getUnderlyingIdentificationType() {
			return underlyingIdentificationType;
		}
		
		@Override
		@RosettaAttribute("underlyingIdOther")
		public String getUnderlyingIdOther() {
			return underlyingIdOther;
		}
		
		@Override
		@RosettaAttribute("underlyingIdOtherSource")
		public ProductIdTypeEnum getUnderlyingIdOtherSource() {
			return underlyingIdOtherSource;
		}
		
		@Override
		@RosettaAttribute("nameOfTheUnderlyingIndex")
		public String getNameOfTheUnderlyingIndex() {
			return nameOfTheUnderlyingIndex;
		}
		
		@Override
		@RosettaAttribute("fixedRateOfLeg1")
		public BigDecimal getFixedRateOfLeg1() {
			return fixedRateOfLeg1;
		}
		
		@Override
		@RosettaAttribute("identifierOfFloatingRateOfLeg1")
		public String getIdentifierOfFloatingRateOfLeg1() {
			return identifierOfFloatingRateOfLeg1;
		}
		
		@Override
		@RosettaAttribute("nameOfTheFloatingRateOfLeg1")
		public String getNameOfTheFloatingRateOfLeg1() {
			return nameOfTheFloatingRateOfLeg1;
		}
		
		@Override
		@RosettaAttribute("spreadOfLeg1")
		public SpreadOfLeg1.SpreadOfLeg1Builder getSpreadOfLeg1() {
			return spreadOfLeg1;
		}
		
		@Override
		public SpreadOfLeg1.SpreadOfLeg1Builder getOrCreateSpreadOfLeg1() {
			SpreadOfLeg1.SpreadOfLeg1Builder result;
			if (spreadOfLeg1!=null) {
				result = spreadOfLeg1;
			}
			else {
				result = spreadOfLeg1 = SpreadOfLeg1.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("fixedRateOfLeg2")
		public BigDecimal getFixedRateOfLeg2() {
			return fixedRateOfLeg2;
		}
		
		@Override
		@RosettaAttribute("identifierOfFloatingRateOfLeg2")
		public String getIdentifierOfFloatingRateOfLeg2() {
			return identifierOfFloatingRateOfLeg2;
		}
		
		@Override
		@RosettaAttribute("nameOfTheFloatingRateOfLeg2")
		public String getNameOfTheFloatingRateOfLeg2() {
			return nameOfTheFloatingRateOfLeg2;
		}
		
		@Override
		@RosettaAttribute("spreadOfLeg2")
		public SpreadOfLeg2.SpreadOfLeg2Builder getSpreadOfLeg2() {
			return spreadOfLeg2;
		}
		
		@Override
		public SpreadOfLeg2.SpreadOfLeg2Builder getOrCreateSpreadOfLeg2() {
			SpreadOfLeg2.SpreadOfLeg2Builder result;
			if (spreadOfLeg2!=null) {
				result = spreadOfLeg2;
			}
			else {
				result = spreadOfLeg2 = SpreadOfLeg2.builder();
			}
			
			return result;
		}
		@Override
		@RosettaAttribute("optionType")
		public String getOptionType() {
			return optionType;
		}
		
		@Override
		@RosettaAttribute("optionStyle")
		public String getOptionStyle() {
			return optionStyle;
		}
		
		@Override
		@RosettaAttribute("firstExerciseDate")
		public Date getFirstExerciseDate() {
			return firstExerciseDate;
		}
		
		@Override
		@RosettaAttribute("effectiveDate")
		public Date getEffectiveDate() {
			return effectiveDate;
		}
		
		@Override
		@RosettaAttribute("expirationDate")
		public Date getExpirationDate() {
			return expirationDate;
		}
		
		@Override
		@RosettaAttribute("swapLinkID")
		public String getSwapLinkID() {
			return swapLinkID;
		}
		
		@Override
		@RosettaAttribute("packageIdentifier")
		public String getPackageIdentifier() {
			return packageIdentifier;
		}
		
		@Override
		@RosettaAttribute("deliveryType")
		public String getDeliveryType() {
			return deliveryType;
		}
		
		@Override
		@RosettaAttribute("settlementCurrency1")
		public ISOCurrencyCodeEnum getSettlementCurrency1() {
			return settlementCurrency1;
		}
		
		@Override
		@RosettaAttribute("settlementCurrency2")
		public ISOCurrencyCodeEnum getSettlementCurrency2() {
			return settlementCurrency2;
		}
		
		@Override
		@RosettaAttribute("finalContractualSettlementDate")
		public Date getFinalContractualSettlementDate() {
			return finalContractualSettlementDate;
		}
		
		@Override
		@RosettaAttribute("cleared")
		public String getCleared() {
			return cleared;
		}
		
		@Override
		@RosettaAttribute("centralCounterparty")
		public String getCentralCounterparty() {
			return centralCounterparty;
		}
		
		@Override
		@RosettaAttribute("clearingMember")
		public String getClearingMember() {
			return clearingMember;
		}
		
		@Override
		@RosettaAttribute("clearingTimestamp")
		public ZonedDateTime getClearingTimestamp() {
			return clearingTimestamp;
		}
		
		@Override
		@RosettaAttribute("platformIdentifier")
		public String getPlatformIdentifier() {
			return platformIdentifier;
		}
		
		@Override
		@RosettaAttribute("reportingObligationOfSpecifiedPerson")
		public String getReportingObligationOfSpecifiedPerson() {
			return reportingObligationOfSpecifiedPerson;
		}
		
		@Override
		@RosettaAttribute("bookingLocation")
		public String getBookingLocation() {
			return bookingLocation;
		}
		
		@Override
		@RosettaAttribute("traderLocation")
		public String getTraderLocation() {
			return traderLocation;
		}
		
		@Override
		@RosettaAttribute("confirmed")
		public String getConfirmed() {
			return confirmed;
		}
		
		@Override
		@RosettaAttribute("confirmationTimestamp")
		public ZonedDateTime getConfirmationTimestamp() {
			return confirmationTimestamp;
		}
		
		@Override
		@RosettaAttribute("executionTimestamp")
		public ZonedDateTime getExecutionTimestamp() {
			return executionTimestamp;
		}
		
		@Override
		@RosettaAttribute("reportingTimestamp")
		public ZonedDateTime getReportingTimestamp() {
			return reportingTimestamp;
		}
		
		@Override
		@RosettaAttribute("basketConstituents")
		public List<? extends BasketConstituentsReport.BasketConstituentsReportBuilder> getBasketConstituents() {
			return basketConstituents;
		}
		
		public BasketConstituentsReport.BasketConstituentsReportBuilder getOrCreateBasketConstituents(int _index) {
		
			if (basketConstituents==null) {
				this.basketConstituents = new ArrayList<>();
			}
			BasketConstituentsReport.BasketConstituentsReportBuilder result;
			return getIndex(basketConstituents, _index, () -> {
						BasketConstituentsReport.BasketConstituentsReportBuilder newBasketConstituents = BasketConstituentsReport.builder();
						return newBasketConstituents;
					});
		}
		
		@Override
		@RosettaAttribute("valuationAmount")
		public BigDecimal getValuationAmount() {
			return valuationAmount;
		}
		
		@Override
		@RosettaAttribute("valuationCurrency")
		public ISOCurrencyCodeEnum getValuationCurrency() {
			return valuationCurrency;
		}
		
		@Override
		@RosettaAttribute("valuationMethod")
		public ValuationType1Code getValuationMethod() {
			return valuationMethod;
		}
		
		@Override
		@RosettaAttribute("valuationTimestamp")
		public ZonedDateTime getValuationTimestamp() {
			return valuationTimestamp;
		}
		
		@Override
		@RosettaAttribute("collateralPortfolioIndicator")
		public Boolean getCollateralPortfolioIndicator() {
			return collateralPortfolioIndicator;
		}
		
		@Override
		@RosettaAttribute("initialMarginCollateralPortfolioCode")
		public String getInitialMarginCollateralPortfolioCode() {
			return initialMarginCollateralPortfolioCode;
		}
		
		@Override
		@RosettaAttribute("variationMarginCollateralPortfolioCode")
		public String getVariationMarginCollateralPortfolioCode() {
			return variationMarginCollateralPortfolioCode;
		}
		
		@Override
		@RosettaAttribute("notionalAmountOfTheContractOrLeg1")
		public BigDecimal getNotionalAmountOfTheContractOrLeg1() {
			return notionalAmountOfTheContractOrLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalCurrencyOfTheContractOrLeg1")
		public ISOCurrencyCodeEnum getNotionalCurrencyOfTheContractOrLeg1() {
			return notionalCurrencyOfTheContractOrLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalAmountOfLeg2")
		public BigDecimal getNotionalAmountOfLeg2() {
			return notionalAmountOfLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalCurrencyOfLeg2")
		public ISOCurrencyCodeEnum getNotionalCurrencyOfLeg2() {
			return notionalCurrencyOfLeg2;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityOfTheContractOrLeg1")
		public BigDecimal getTotalNotionalQuantityOfTheContractOrLeg1() {
			return totalNotionalQuantityOfTheContractOrLeg1;
		}
		
		@Override
		@RosettaAttribute("quantityUnitOfMeasureForTheContractOrLeg1")
		public String getQuantityUnitOfMeasureForTheContractOrLeg1() {
			return quantityUnitOfMeasureForTheContractOrLeg1;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityOfTheContractOrLeg1")
		public BigDecimal getNotionalQuantityOfTheContractOrLeg1() {
			return notionalQuantityOfTheContractOrLeg1;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyOfTheContractOrLeg1")
		public String getQuantityFrequencyOfTheContractOrLeg1() {
			return quantityFrequencyOfTheContractOrLeg1;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyMultiplierOfTheContractOrLeg1")
		public Integer getQuantityFrequencyMultiplierOfTheContractOrLeg1() {
			return quantityFrequencyMultiplierOfTheContractOrLeg1;
		}
		
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg2")
		public BigDecimal getTotalNotionalQuantityLeg2() {
			return totalNotionalQuantityLeg2;
		}
		
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg2")
		public String getQuantityUnitOfMeasureLeg2() {
			return quantityUnitOfMeasureLeg2;
		}
		
		@Override
		@RosettaAttribute("notionalQuantityOfLeg2")
		public BigDecimal getNotionalQuantityOfLeg2() {
			return notionalQuantityOfLeg2;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyOfLeg2")
		public String getQuantityFrequencyOfLeg2() {
			return quantityFrequencyOfLeg2;
		}
		
		@Override
		@RosettaAttribute("quantityFrequencyMultiplierOfLeg2")
		public Integer getQuantityFrequencyMultiplierOfLeg2() {
			return quantityFrequencyMultiplierOfLeg2;
		}
		
		@Override
		@RosettaAttribute("callAmount")
		public BigDecimal getCallAmount() {
			return callAmount;
		}
		
		@Override
		@RosettaAttribute("callCurrency")
		public ISOCurrencyCodeEnum getCallCurrency() {
			return callCurrency;
		}
		
		@Override
		@RosettaAttribute("putAmount")
		public BigDecimal getPutAmount() {
			return putAmount;
		}
		
		@Override
		@RosettaAttribute("putCurrency")
		public ISOCurrencyCodeEnum getPutCurrency() {
			return putCurrency;
		}
		
		@Override
		@RosettaAttribute("price")
		public BigDecimal getPrice() {
			return price;
		}
		
		@Override
		@RosettaAttribute("priceCurrency")
		public ISOCurrencyCodeEnum getPriceCurrency() {
			return priceCurrency;
		}
		
		@Override
		@RosettaAttribute("priceNotation")
		public String getPriceNotation() {
			return priceNotation;
		}
		
		@Override
		@RosettaAttribute("priceUnitOfMeasure")
		public String getPriceUnitOfMeasure() {
			return priceUnitOfMeasure;
		}
		
		@Override
		@RosettaAttribute("priceSchedule")
		public List<? extends PriceScheduleReport.PriceScheduleReportBuilder> getPriceSchedule() {
			return priceSchedule;
		}
		
		public PriceScheduleReport.PriceScheduleReportBuilder getOrCreatePriceSchedule(int _index) {
		
			if (priceSchedule==null) {
				this.priceSchedule = new ArrayList<>();
			}
			PriceScheduleReport.PriceScheduleReportBuilder result;
			return getIndex(priceSchedule, _index, () -> {
						PriceScheduleReport.PriceScheduleReportBuilder newPriceSchedule = PriceScheduleReport.builder();
						return newPriceSchedule;
					});
		}
		
		@Override
		@RosettaAttribute("strikePrice")
		public BigDecimal getStrikePrice() {
			return strikePrice;
		}
		
		@Override
		@RosettaAttribute("strikePriceCurrency")
		public String getStrikePriceCurrency() {
			return strikePriceCurrency;
		}
		
		@Override
		@RosettaAttribute("strikePriceNotation")
		public String getStrikePriceNotation() {
			return strikePriceNotation;
		}
		
		@Override
		@RosettaAttribute("strikePriceSchedule")
		public List<? extends StrikePriceScheduleReport.StrikePriceScheduleReportBuilder> getStrikePriceSchedule() {
			return strikePriceSchedule;
		}
		
		public StrikePriceScheduleReport.StrikePriceScheduleReportBuilder getOrCreateStrikePriceSchedule(int _index) {
		
			if (strikePriceSchedule==null) {
				this.strikePriceSchedule = new ArrayList<>();
			}
			StrikePriceScheduleReport.StrikePriceScheduleReportBuilder result;
			return getIndex(strikePriceSchedule, _index, () -> {
						StrikePriceScheduleReport.StrikePriceScheduleReportBuilder newStrikePriceSchedule = StrikePriceScheduleReport.builder();
						return newStrikePriceSchedule;
					});
		}
		
		@Override
		@RosettaAttribute("optionPremiumAmount")
		public BigDecimal getOptionPremiumAmount() {
			return optionPremiumAmount;
		}
		
		@Override
		@RosettaAttribute("optionPremiumCurrency")
		public ISOCurrencyCodeEnum getOptionPremiumCurrency() {
			return optionPremiumCurrency;
		}
		
		@Override
		@RosettaAttribute("exchangeRate")
		public BigDecimal getExchangeRate() {
			return exchangeRate;
		}
		
		@Override
		@RosettaAttribute("exchangeRateBasis")
		public String getExchangeRateBasis() {
			return exchangeRateBasis;
		}
		
		@Override
		@RosettaAttribute("fixedRateDayCountConventionOfLeg1")
		public InterestComputationMethod4Code getFixedRateDayCountConventionOfLeg1() {
			return fixedRateDayCountConventionOfLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRateDayCountConventionOfLeg2")
		public InterestComputationMethod4Code getFixedRateDayCountConventionOfLeg2() {
			return fixedRateDayCountConventionOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateDayCountConventionOfLeg1")
		public InterestComputationMethod4Code getFloatingRateDayCountConventionOfLeg1() {
			return floatingRateDayCountConventionOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateDayCountConventionOfLeg2")
		public InterestComputationMethod4Code getFloatingRateDayCountConventionOfLeg2() {
			return floatingRateDayCountConventionOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodOfLeg1")
		public String getFloatingRateResetFrequencyPeriodOfLeg1() {
			return floatingRateResetFrequencyPeriodOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierOfLeg1")
		public Integer getFloatingRateResetFrequencyPeriodMultiplierOfLeg1() {
			return floatingRateResetFrequencyPeriodMultiplierOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodOfLeg2")
		public String getFloatingRateResetFrequencyPeriodOfLeg2() {
			return floatingRateResetFrequencyPeriodOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierOfLeg2")
		public Integer getFloatingRateResetFrequencyPeriodMultiplierOfLeg2() {
			return floatingRateResetFrequencyPeriodMultiplierOfLeg2;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodOfLeg1")
		public String getFixedRatePaymentFrequencyPeriodOfLeg1() {
			return fixedRatePaymentFrequencyPeriodOfLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierOfLeg1")
		public Integer getFixedRatePaymentFrequencyPeriodMultiplierOfLeg1() {
			return fixedRatePaymentFrequencyPeriodMultiplierOfLeg1;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodOfLeg2")
		public String getFixedRatePaymentFrequencyPeriodOfLeg2() {
			return fixedRatePaymentFrequencyPeriodOfLeg2;
		}
		
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierOfLeg2")
		public Integer getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2() {
			return fixedRatePaymentFrequencyPeriodMultiplierOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodOfLeg1")
		public Frequency13Code__1 getFloatingRatePaymentFrequencyPeriodOfLeg1() {
			return floatingRatePaymentFrequencyPeriodOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierOfLeg1")
		public Integer getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1() {
			return floatingRatePaymentFrequencyPeriodMultiplierOfLeg1;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodOfLeg2")
		public Frequency13Code__1 getFloatingRatePaymentFrequencyPeriodOfLeg2() {
			return floatingRatePaymentFrequencyPeriodOfLeg2;
		}
		
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierOfLeg2")
		public Integer getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2() {
			return floatingRatePaymentFrequencyPeriodMultiplierOfLeg2;
		}
		
		@Override
		@RosettaAttribute("otherPayment")
		public List<? extends OtherPaymentReport.OtherPaymentReportBuilder> getOtherPayment() {
			return otherPayment;
		}
		
		public OtherPaymentReport.OtherPaymentReportBuilder getOrCreateOtherPayment(int _index) {
		
			if (otherPayment==null) {
				this.otherPayment = new ArrayList<>();
			}
			OtherPaymentReport.OtherPaymentReportBuilder result;
			return getIndex(otherPayment, _index, () -> {
						OtherPaymentReport.OtherPaymentReportBuilder newOtherPayment = OtherPaymentReport.builder();
						return newOtherPayment;
					});
		}
		
		@Override
		@RosettaAttribute("actionType")
		public String getActionType() {
			return actionType;
		}
		
		@Override
		@RosettaAttribute("eventType")
		public String getEventType() {
			return eventType;
		}
		
		@Override
		@RosettaAttribute("eventDate")
		public Date getEventDate() {
			return eventDate;
		}
		
	
		@Override
		@RosettaAttribute("counterparty1")
		public MASTransactionReport.MASTransactionReportBuilder setCounterparty1(String counterparty1) {
			this.counterparty1 = counterparty1==null?null:counterparty1;
			return this;
		}
		@Override
		@RosettaAttribute("counterparty2")
		public MASTransactionReport.MASTransactionReportBuilder setCounterparty2(String counterparty2) {
			this.counterparty2 = counterparty2==null?null:counterparty2;
			return this;
		}
		@Override
		@RosettaAttribute("counterparty2IdentifierType")
		public MASTransactionReport.MASTransactionReportBuilder setCounterparty2IdentifierType(Boolean counterparty2IdentifierType) {
			this.counterparty2IdentifierType = counterparty2IdentifierType==null?null:counterparty2IdentifierType;
			return this;
		}
		@Override
		@RosettaAttribute("direction")
		public MASTransactionReport.MASTransactionReportBuilder setDirection(String direction) {
			this.direction = direction==null?null:direction;
			return this;
		}
		@Override
		@RosettaAttribute("directionOfLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setDirectionOfLeg1(String directionOfLeg1) {
			this.directionOfLeg1 = directionOfLeg1==null?null:directionOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("directionOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setDirectionOfLeg2(String directionOfLeg2) {
			this.directionOfLeg2 = directionOfLeg2==null?null:directionOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("reportingSpecifiedPerson")
		public MASTransactionReport.MASTransactionReportBuilder setReportingSpecifiedPerson(String reportingSpecifiedPerson) {
			this.reportingSpecifiedPerson = reportingSpecifiedPerson==null?null:reportingSpecifiedPerson;
			return this;
		}
		@Override
		@RosettaAttribute("dataSubmitter")
		public MASTransactionReport.MASTransactionReportBuilder setDataSubmitter(String dataSubmitter) {
			this.dataSubmitter = dataSubmitter==null?null:dataSubmitter;
			return this;
		}
		@Override
		@RosettaAttribute("uniqueTransactionIdentifier")
		public MASTransactionReport.MASTransactionReportBuilder setUniqueTransactionIdentifier(String uniqueTransactionIdentifier) {
			this.uniqueTransactionIdentifier = uniqueTransactionIdentifier==null?null:uniqueTransactionIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("uniqueProductIdentifier")
		public MASTransactionReport.MASTransactionReportBuilder setUniqueProductIdentifier(String uniqueProductIdentifier) {
			this.uniqueProductIdentifier = uniqueProductIdentifier==null?null:uniqueProductIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("priorUTI")
		public MASTransactionReport.MASTransactionReportBuilder setPriorUTI(String priorUTI) {
			this.priorUTI = priorUTI==null?null:priorUTI;
			return this;
		}
		@Override
		@RosettaAttribute("contractType")
		public MASTransactionReport.MASTransactionReportBuilder setContractType(String contractType) {
			this.contractType = contractType==null?null:contractType;
			return this;
		}
		@Override
		@RosettaAttribute("assetClass")
		public MASTransactionReport.MASTransactionReportBuilder setAssetClass(String assetClass) {
			this.assetClass = assetClass==null?null:assetClass;
			return this;
		}
		@Override
		@RosettaAttribute("underlying")
		public MASTransactionReport.MASTransactionReportBuilder setUnderlying(String underlying) {
			this.underlying = underlying==null?null:underlying;
			return this;
		}
		@Override
		@RosettaAttribute("underlyingIdentificationType")
		public MASTransactionReport.MASTransactionReportBuilder setUnderlyingIdentificationType(String underlyingIdentificationType) {
			this.underlyingIdentificationType = underlyingIdentificationType==null?null:underlyingIdentificationType;
			return this;
		}
		@Override
		@RosettaAttribute("underlyingIdOther")
		public MASTransactionReport.MASTransactionReportBuilder setUnderlyingIdOther(String underlyingIdOther) {
			this.underlyingIdOther = underlyingIdOther==null?null:underlyingIdOther;
			return this;
		}
		@Override
		@RosettaAttribute("underlyingIdOtherSource")
		public MASTransactionReport.MASTransactionReportBuilder setUnderlyingIdOtherSource(ProductIdTypeEnum underlyingIdOtherSource) {
			this.underlyingIdOtherSource = underlyingIdOtherSource==null?null:underlyingIdOtherSource;
			return this;
		}
		@Override
		@RosettaAttribute("nameOfTheUnderlyingIndex")
		public MASTransactionReport.MASTransactionReportBuilder setNameOfTheUnderlyingIndex(String nameOfTheUnderlyingIndex) {
			this.nameOfTheUnderlyingIndex = nameOfTheUnderlyingIndex==null?null:nameOfTheUnderlyingIndex;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRateOfLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setFixedRateOfLeg1(BigDecimal fixedRateOfLeg1) {
			this.fixedRateOfLeg1 = fixedRateOfLeg1==null?null:fixedRateOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("identifierOfFloatingRateOfLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setIdentifierOfFloatingRateOfLeg1(String identifierOfFloatingRateOfLeg1) {
			this.identifierOfFloatingRateOfLeg1 = identifierOfFloatingRateOfLeg1==null?null:identifierOfFloatingRateOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("nameOfTheFloatingRateOfLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setNameOfTheFloatingRateOfLeg1(String nameOfTheFloatingRateOfLeg1) {
			this.nameOfTheFloatingRateOfLeg1 = nameOfTheFloatingRateOfLeg1==null?null:nameOfTheFloatingRateOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("spreadOfLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setSpreadOfLeg1(SpreadOfLeg1 spreadOfLeg1) {
			this.spreadOfLeg1 = spreadOfLeg1==null?null:spreadOfLeg1.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("fixedRateOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setFixedRateOfLeg2(BigDecimal fixedRateOfLeg2) {
			this.fixedRateOfLeg2 = fixedRateOfLeg2==null?null:fixedRateOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("identifierOfFloatingRateOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setIdentifierOfFloatingRateOfLeg2(String identifierOfFloatingRateOfLeg2) {
			this.identifierOfFloatingRateOfLeg2 = identifierOfFloatingRateOfLeg2==null?null:identifierOfFloatingRateOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("nameOfTheFloatingRateOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setNameOfTheFloatingRateOfLeg2(String nameOfTheFloatingRateOfLeg2) {
			this.nameOfTheFloatingRateOfLeg2 = nameOfTheFloatingRateOfLeg2==null?null:nameOfTheFloatingRateOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("spreadOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setSpreadOfLeg2(SpreadOfLeg2 spreadOfLeg2) {
			this.spreadOfLeg2 = spreadOfLeg2==null?null:spreadOfLeg2.toBuilder();
			return this;
		}
		@Override
		@RosettaAttribute("optionType")
		public MASTransactionReport.MASTransactionReportBuilder setOptionType(String optionType) {
			this.optionType = optionType==null?null:optionType;
			return this;
		}
		@Override
		@RosettaAttribute("optionStyle")
		public MASTransactionReport.MASTransactionReportBuilder setOptionStyle(String optionStyle) {
			this.optionStyle = optionStyle==null?null:optionStyle;
			return this;
		}
		@Override
		@RosettaAttribute("firstExerciseDate")
		public MASTransactionReport.MASTransactionReportBuilder setFirstExerciseDate(Date firstExerciseDate) {
			this.firstExerciseDate = firstExerciseDate==null?null:firstExerciseDate;
			return this;
		}
		@Override
		@RosettaAttribute("effectiveDate")
		public MASTransactionReport.MASTransactionReportBuilder setEffectiveDate(Date effectiveDate) {
			this.effectiveDate = effectiveDate==null?null:effectiveDate;
			return this;
		}
		@Override
		@RosettaAttribute("expirationDate")
		public MASTransactionReport.MASTransactionReportBuilder setExpirationDate(Date expirationDate) {
			this.expirationDate = expirationDate==null?null:expirationDate;
			return this;
		}
		@Override
		@RosettaAttribute("swapLinkID")
		public MASTransactionReport.MASTransactionReportBuilder setSwapLinkID(String swapLinkID) {
			this.swapLinkID = swapLinkID==null?null:swapLinkID;
			return this;
		}
		@Override
		@RosettaAttribute("packageIdentifier")
		public MASTransactionReport.MASTransactionReportBuilder setPackageIdentifier(String packageIdentifier) {
			this.packageIdentifier = packageIdentifier==null?null:packageIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("deliveryType")
		public MASTransactionReport.MASTransactionReportBuilder setDeliveryType(String deliveryType) {
			this.deliveryType = deliveryType==null?null:deliveryType;
			return this;
		}
		@Override
		@RosettaAttribute("settlementCurrency1")
		public MASTransactionReport.MASTransactionReportBuilder setSettlementCurrency1(ISOCurrencyCodeEnum settlementCurrency1) {
			this.settlementCurrency1 = settlementCurrency1==null?null:settlementCurrency1;
			return this;
		}
		@Override
		@RosettaAttribute("settlementCurrency2")
		public MASTransactionReport.MASTransactionReportBuilder setSettlementCurrency2(ISOCurrencyCodeEnum settlementCurrency2) {
			this.settlementCurrency2 = settlementCurrency2==null?null:settlementCurrency2;
			return this;
		}
		@Override
		@RosettaAttribute("finalContractualSettlementDate")
		public MASTransactionReport.MASTransactionReportBuilder setFinalContractualSettlementDate(Date finalContractualSettlementDate) {
			this.finalContractualSettlementDate = finalContractualSettlementDate==null?null:finalContractualSettlementDate;
			return this;
		}
		@Override
		@RosettaAttribute("cleared")
		public MASTransactionReport.MASTransactionReportBuilder setCleared(String cleared) {
			this.cleared = cleared==null?null:cleared;
			return this;
		}
		@Override
		@RosettaAttribute("centralCounterparty")
		public MASTransactionReport.MASTransactionReportBuilder setCentralCounterparty(String centralCounterparty) {
			this.centralCounterparty = centralCounterparty==null?null:centralCounterparty;
			return this;
		}
		@Override
		@RosettaAttribute("clearingMember")
		public MASTransactionReport.MASTransactionReportBuilder setClearingMember(String clearingMember) {
			this.clearingMember = clearingMember==null?null:clearingMember;
			return this;
		}
		@Override
		@RosettaAttribute("clearingTimestamp")
		public MASTransactionReport.MASTransactionReportBuilder setClearingTimestamp(ZonedDateTime clearingTimestamp) {
			this.clearingTimestamp = clearingTimestamp==null?null:clearingTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("platformIdentifier")
		public MASTransactionReport.MASTransactionReportBuilder setPlatformIdentifier(String platformIdentifier) {
			this.platformIdentifier = platformIdentifier==null?null:platformIdentifier;
			return this;
		}
		@Override
		@RosettaAttribute("reportingObligationOfSpecifiedPerson")
		public MASTransactionReport.MASTransactionReportBuilder setReportingObligationOfSpecifiedPerson(String reportingObligationOfSpecifiedPerson) {
			this.reportingObligationOfSpecifiedPerson = reportingObligationOfSpecifiedPerson==null?null:reportingObligationOfSpecifiedPerson;
			return this;
		}
		@Override
		@RosettaAttribute("bookingLocation")
		public MASTransactionReport.MASTransactionReportBuilder setBookingLocation(String bookingLocation) {
			this.bookingLocation = bookingLocation==null?null:bookingLocation;
			return this;
		}
		@Override
		@RosettaAttribute("traderLocation")
		public MASTransactionReport.MASTransactionReportBuilder setTraderLocation(String traderLocation) {
			this.traderLocation = traderLocation==null?null:traderLocation;
			return this;
		}
		@Override
		@RosettaAttribute("confirmed")
		public MASTransactionReport.MASTransactionReportBuilder setConfirmed(String confirmed) {
			this.confirmed = confirmed==null?null:confirmed;
			return this;
		}
		@Override
		@RosettaAttribute("confirmationTimestamp")
		public MASTransactionReport.MASTransactionReportBuilder setConfirmationTimestamp(ZonedDateTime confirmationTimestamp) {
			this.confirmationTimestamp = confirmationTimestamp==null?null:confirmationTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("executionTimestamp")
		public MASTransactionReport.MASTransactionReportBuilder setExecutionTimestamp(ZonedDateTime executionTimestamp) {
			this.executionTimestamp = executionTimestamp==null?null:executionTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("reportingTimestamp")
		public MASTransactionReport.MASTransactionReportBuilder setReportingTimestamp(ZonedDateTime reportingTimestamp) {
			this.reportingTimestamp = reportingTimestamp==null?null:reportingTimestamp;
			return this;
		}
		@Override
		public MASTransactionReport.MASTransactionReportBuilder addBasketConstituents(BasketConstituentsReport basketConstituents) {
			if (basketConstituents!=null) this.basketConstituents.add(basketConstituents.toBuilder());
			return this;
		}
		
		@Override
		public MASTransactionReport.MASTransactionReportBuilder addBasketConstituents(BasketConstituentsReport basketConstituents, int _idx) {
			getIndex(this.basketConstituents, _idx, () -> basketConstituents.toBuilder());
			return this;
		}
		@Override 
		public MASTransactionReport.MASTransactionReportBuilder addBasketConstituents(List<? extends BasketConstituentsReport> basketConstituentss) {
			if (basketConstituentss != null) {
				for (BasketConstituentsReport toAdd : basketConstituentss) {
					this.basketConstituents.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("basketConstituents")
		public MASTransactionReport.MASTransactionReportBuilder setBasketConstituents(List<? extends BasketConstituentsReport> basketConstituentss) {
			if (basketConstituentss == null)  {
				this.basketConstituents = new ArrayList<>();
			}
			else {
				this.basketConstituents = basketConstituentss.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("valuationAmount")
		public MASTransactionReport.MASTransactionReportBuilder setValuationAmount(BigDecimal valuationAmount) {
			this.valuationAmount = valuationAmount==null?null:valuationAmount;
			return this;
		}
		@Override
		@RosettaAttribute("valuationCurrency")
		public MASTransactionReport.MASTransactionReportBuilder setValuationCurrency(ISOCurrencyCodeEnum valuationCurrency) {
			this.valuationCurrency = valuationCurrency==null?null:valuationCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("valuationMethod")
		public MASTransactionReport.MASTransactionReportBuilder setValuationMethod(ValuationType1Code valuationMethod) {
			this.valuationMethod = valuationMethod==null?null:valuationMethod;
			return this;
		}
		@Override
		@RosettaAttribute("valuationTimestamp")
		public MASTransactionReport.MASTransactionReportBuilder setValuationTimestamp(ZonedDateTime valuationTimestamp) {
			this.valuationTimestamp = valuationTimestamp==null?null:valuationTimestamp;
			return this;
		}
		@Override
		@RosettaAttribute("collateralPortfolioIndicator")
		public MASTransactionReport.MASTransactionReportBuilder setCollateralPortfolioIndicator(Boolean collateralPortfolioIndicator) {
			this.collateralPortfolioIndicator = collateralPortfolioIndicator==null?null:collateralPortfolioIndicator;
			return this;
		}
		@Override
		@RosettaAttribute("initialMarginCollateralPortfolioCode")
		public MASTransactionReport.MASTransactionReportBuilder setInitialMarginCollateralPortfolioCode(String initialMarginCollateralPortfolioCode) {
			this.initialMarginCollateralPortfolioCode = initialMarginCollateralPortfolioCode==null?null:initialMarginCollateralPortfolioCode;
			return this;
		}
		@Override
		@RosettaAttribute("variationMarginCollateralPortfolioCode")
		public MASTransactionReport.MASTransactionReportBuilder setVariationMarginCollateralPortfolioCode(String variationMarginCollateralPortfolioCode) {
			this.variationMarginCollateralPortfolioCode = variationMarginCollateralPortfolioCode==null?null:variationMarginCollateralPortfolioCode;
			return this;
		}
		@Override
		@RosettaAttribute("notionalAmountOfTheContractOrLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setNotionalAmountOfTheContractOrLeg1(BigDecimal notionalAmountOfTheContractOrLeg1) {
			this.notionalAmountOfTheContractOrLeg1 = notionalAmountOfTheContractOrLeg1==null?null:notionalAmountOfTheContractOrLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("notionalCurrencyOfTheContractOrLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setNotionalCurrencyOfTheContractOrLeg1(ISOCurrencyCodeEnum notionalCurrencyOfTheContractOrLeg1) {
			this.notionalCurrencyOfTheContractOrLeg1 = notionalCurrencyOfTheContractOrLeg1==null?null:notionalCurrencyOfTheContractOrLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("notionalAmountOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setNotionalAmountOfLeg2(BigDecimal notionalAmountOfLeg2) {
			this.notionalAmountOfLeg2 = notionalAmountOfLeg2==null?null:notionalAmountOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("notionalCurrencyOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setNotionalCurrencyOfLeg2(ISOCurrencyCodeEnum notionalCurrencyOfLeg2) {
			this.notionalCurrencyOfLeg2 = notionalCurrencyOfLeg2==null?null:notionalCurrencyOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("totalNotionalQuantityOfTheContractOrLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setTotalNotionalQuantityOfTheContractOrLeg1(BigDecimal totalNotionalQuantityOfTheContractOrLeg1) {
			this.totalNotionalQuantityOfTheContractOrLeg1 = totalNotionalQuantityOfTheContractOrLeg1==null?null:totalNotionalQuantityOfTheContractOrLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("quantityUnitOfMeasureForTheContractOrLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setQuantityUnitOfMeasureForTheContractOrLeg1(String quantityUnitOfMeasureForTheContractOrLeg1) {
			this.quantityUnitOfMeasureForTheContractOrLeg1 = quantityUnitOfMeasureForTheContractOrLeg1==null?null:quantityUnitOfMeasureForTheContractOrLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("notionalQuantityOfTheContractOrLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setNotionalQuantityOfTheContractOrLeg1(BigDecimal notionalQuantityOfTheContractOrLeg1) {
			this.notionalQuantityOfTheContractOrLeg1 = notionalQuantityOfTheContractOrLeg1==null?null:notionalQuantityOfTheContractOrLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("quantityFrequencyOfTheContractOrLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setQuantityFrequencyOfTheContractOrLeg1(String quantityFrequencyOfTheContractOrLeg1) {
			this.quantityFrequencyOfTheContractOrLeg1 = quantityFrequencyOfTheContractOrLeg1==null?null:quantityFrequencyOfTheContractOrLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("quantityFrequencyMultiplierOfTheContractOrLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setQuantityFrequencyMultiplierOfTheContractOrLeg1(Integer quantityFrequencyMultiplierOfTheContractOrLeg1) {
			this.quantityFrequencyMultiplierOfTheContractOrLeg1 = quantityFrequencyMultiplierOfTheContractOrLeg1==null?null:quantityFrequencyMultiplierOfTheContractOrLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("totalNotionalQuantityLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setTotalNotionalQuantityLeg2(BigDecimal totalNotionalQuantityLeg2) {
			this.totalNotionalQuantityLeg2 = totalNotionalQuantityLeg2==null?null:totalNotionalQuantityLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("quantityUnitOfMeasureLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setQuantityUnitOfMeasureLeg2(String quantityUnitOfMeasureLeg2) {
			this.quantityUnitOfMeasureLeg2 = quantityUnitOfMeasureLeg2==null?null:quantityUnitOfMeasureLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("notionalQuantityOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setNotionalQuantityOfLeg2(BigDecimal notionalQuantityOfLeg2) {
			this.notionalQuantityOfLeg2 = notionalQuantityOfLeg2==null?null:notionalQuantityOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("quantityFrequencyOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setQuantityFrequencyOfLeg2(String quantityFrequencyOfLeg2) {
			this.quantityFrequencyOfLeg2 = quantityFrequencyOfLeg2==null?null:quantityFrequencyOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("quantityFrequencyMultiplierOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setQuantityFrequencyMultiplierOfLeg2(Integer quantityFrequencyMultiplierOfLeg2) {
			this.quantityFrequencyMultiplierOfLeg2 = quantityFrequencyMultiplierOfLeg2==null?null:quantityFrequencyMultiplierOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("callAmount")
		public MASTransactionReport.MASTransactionReportBuilder setCallAmount(BigDecimal callAmount) {
			this.callAmount = callAmount==null?null:callAmount;
			return this;
		}
		@Override
		@RosettaAttribute("callCurrency")
		public MASTransactionReport.MASTransactionReportBuilder setCallCurrency(ISOCurrencyCodeEnum callCurrency) {
			this.callCurrency = callCurrency==null?null:callCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("putAmount")
		public MASTransactionReport.MASTransactionReportBuilder setPutAmount(BigDecimal putAmount) {
			this.putAmount = putAmount==null?null:putAmount;
			return this;
		}
		@Override
		@RosettaAttribute("putCurrency")
		public MASTransactionReport.MASTransactionReportBuilder setPutCurrency(ISOCurrencyCodeEnum putCurrency) {
			this.putCurrency = putCurrency==null?null:putCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("price")
		public MASTransactionReport.MASTransactionReportBuilder setPrice(BigDecimal price) {
			this.price = price==null?null:price;
			return this;
		}
		@Override
		@RosettaAttribute("priceCurrency")
		public MASTransactionReport.MASTransactionReportBuilder setPriceCurrency(ISOCurrencyCodeEnum priceCurrency) {
			this.priceCurrency = priceCurrency==null?null:priceCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("priceNotation")
		public MASTransactionReport.MASTransactionReportBuilder setPriceNotation(String priceNotation) {
			this.priceNotation = priceNotation==null?null:priceNotation;
			return this;
		}
		@Override
		@RosettaAttribute("priceUnitOfMeasure")
		public MASTransactionReport.MASTransactionReportBuilder setPriceUnitOfMeasure(String priceUnitOfMeasure) {
			this.priceUnitOfMeasure = priceUnitOfMeasure==null?null:priceUnitOfMeasure;
			return this;
		}
		@Override
		public MASTransactionReport.MASTransactionReportBuilder addPriceSchedule(PriceScheduleReport priceSchedule) {
			if (priceSchedule!=null) this.priceSchedule.add(priceSchedule.toBuilder());
			return this;
		}
		
		@Override
		public MASTransactionReport.MASTransactionReportBuilder addPriceSchedule(PriceScheduleReport priceSchedule, int _idx) {
			getIndex(this.priceSchedule, _idx, () -> priceSchedule.toBuilder());
			return this;
		}
		@Override 
		public MASTransactionReport.MASTransactionReportBuilder addPriceSchedule(List<? extends PriceScheduleReport> priceSchedules) {
			if (priceSchedules != null) {
				for (PriceScheduleReport toAdd : priceSchedules) {
					this.priceSchedule.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("priceSchedule")
		public MASTransactionReport.MASTransactionReportBuilder setPriceSchedule(List<? extends PriceScheduleReport> priceSchedules) {
			if (priceSchedules == null)  {
				this.priceSchedule = new ArrayList<>();
			}
			else {
				this.priceSchedule = priceSchedules.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("strikePrice")
		public MASTransactionReport.MASTransactionReportBuilder setStrikePrice(BigDecimal strikePrice) {
			this.strikePrice = strikePrice==null?null:strikePrice;
			return this;
		}
		@Override
		@RosettaAttribute("strikePriceCurrency")
		public MASTransactionReport.MASTransactionReportBuilder setStrikePriceCurrency(String strikePriceCurrency) {
			this.strikePriceCurrency = strikePriceCurrency==null?null:strikePriceCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("strikePriceNotation")
		public MASTransactionReport.MASTransactionReportBuilder setStrikePriceNotation(String strikePriceNotation) {
			this.strikePriceNotation = strikePriceNotation==null?null:strikePriceNotation;
			return this;
		}
		@Override
		public MASTransactionReport.MASTransactionReportBuilder addStrikePriceSchedule(StrikePriceScheduleReport strikePriceSchedule) {
			if (strikePriceSchedule!=null) this.strikePriceSchedule.add(strikePriceSchedule.toBuilder());
			return this;
		}
		
		@Override
		public MASTransactionReport.MASTransactionReportBuilder addStrikePriceSchedule(StrikePriceScheduleReport strikePriceSchedule, int _idx) {
			getIndex(this.strikePriceSchedule, _idx, () -> strikePriceSchedule.toBuilder());
			return this;
		}
		@Override 
		public MASTransactionReport.MASTransactionReportBuilder addStrikePriceSchedule(List<? extends StrikePriceScheduleReport> strikePriceSchedules) {
			if (strikePriceSchedules != null) {
				for (StrikePriceScheduleReport toAdd : strikePriceSchedules) {
					this.strikePriceSchedule.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("strikePriceSchedule")
		public MASTransactionReport.MASTransactionReportBuilder setStrikePriceSchedule(List<? extends StrikePriceScheduleReport> strikePriceSchedules) {
			if (strikePriceSchedules == null)  {
				this.strikePriceSchedule = new ArrayList<>();
			}
			else {
				this.strikePriceSchedule = strikePriceSchedules.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("optionPremiumAmount")
		public MASTransactionReport.MASTransactionReportBuilder setOptionPremiumAmount(BigDecimal optionPremiumAmount) {
			this.optionPremiumAmount = optionPremiumAmount==null?null:optionPremiumAmount;
			return this;
		}
		@Override
		@RosettaAttribute("optionPremiumCurrency")
		public MASTransactionReport.MASTransactionReportBuilder setOptionPremiumCurrency(ISOCurrencyCodeEnum optionPremiumCurrency) {
			this.optionPremiumCurrency = optionPremiumCurrency==null?null:optionPremiumCurrency;
			return this;
		}
		@Override
		@RosettaAttribute("exchangeRate")
		public MASTransactionReport.MASTransactionReportBuilder setExchangeRate(BigDecimal exchangeRate) {
			this.exchangeRate = exchangeRate==null?null:exchangeRate;
			return this;
		}
		@Override
		@RosettaAttribute("exchangeRateBasis")
		public MASTransactionReport.MASTransactionReportBuilder setExchangeRateBasis(String exchangeRateBasis) {
			this.exchangeRateBasis = exchangeRateBasis==null?null:exchangeRateBasis;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRateDayCountConventionOfLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setFixedRateDayCountConventionOfLeg1(InterestComputationMethod4Code fixedRateDayCountConventionOfLeg1) {
			this.fixedRateDayCountConventionOfLeg1 = fixedRateDayCountConventionOfLeg1==null?null:fixedRateDayCountConventionOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRateDayCountConventionOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setFixedRateDayCountConventionOfLeg2(InterestComputationMethod4Code fixedRateDayCountConventionOfLeg2) {
			this.fixedRateDayCountConventionOfLeg2 = fixedRateDayCountConventionOfLeg2==null?null:fixedRateDayCountConventionOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateDayCountConventionOfLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setFloatingRateDayCountConventionOfLeg1(InterestComputationMethod4Code floatingRateDayCountConventionOfLeg1) {
			this.floatingRateDayCountConventionOfLeg1 = floatingRateDayCountConventionOfLeg1==null?null:floatingRateDayCountConventionOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateDayCountConventionOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setFloatingRateDayCountConventionOfLeg2(InterestComputationMethod4Code floatingRateDayCountConventionOfLeg2) {
			this.floatingRateDayCountConventionOfLeg2 = floatingRateDayCountConventionOfLeg2==null?null:floatingRateDayCountConventionOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodOfLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setFloatingRateResetFrequencyPeriodOfLeg1(String floatingRateResetFrequencyPeriodOfLeg1) {
			this.floatingRateResetFrequencyPeriodOfLeg1 = floatingRateResetFrequencyPeriodOfLeg1==null?null:floatingRateResetFrequencyPeriodOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierOfLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setFloatingRateResetFrequencyPeriodMultiplierOfLeg1(Integer floatingRateResetFrequencyPeriodMultiplierOfLeg1) {
			this.floatingRateResetFrequencyPeriodMultiplierOfLeg1 = floatingRateResetFrequencyPeriodMultiplierOfLeg1==null?null:floatingRateResetFrequencyPeriodMultiplierOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setFloatingRateResetFrequencyPeriodOfLeg2(String floatingRateResetFrequencyPeriodOfLeg2) {
			this.floatingRateResetFrequencyPeriodOfLeg2 = floatingRateResetFrequencyPeriodOfLeg2==null?null:floatingRateResetFrequencyPeriodOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRateResetFrequencyPeriodMultiplierOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setFloatingRateResetFrequencyPeriodMultiplierOfLeg2(Integer floatingRateResetFrequencyPeriodMultiplierOfLeg2) {
			this.floatingRateResetFrequencyPeriodMultiplierOfLeg2 = floatingRateResetFrequencyPeriodMultiplierOfLeg2==null?null:floatingRateResetFrequencyPeriodMultiplierOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodOfLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setFixedRatePaymentFrequencyPeriodOfLeg1(String fixedRatePaymentFrequencyPeriodOfLeg1) {
			this.fixedRatePaymentFrequencyPeriodOfLeg1 = fixedRatePaymentFrequencyPeriodOfLeg1==null?null:fixedRatePaymentFrequencyPeriodOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierOfLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setFixedRatePaymentFrequencyPeriodMultiplierOfLeg1(Integer fixedRatePaymentFrequencyPeriodMultiplierOfLeg1) {
			this.fixedRatePaymentFrequencyPeriodMultiplierOfLeg1 = fixedRatePaymentFrequencyPeriodMultiplierOfLeg1==null?null:fixedRatePaymentFrequencyPeriodMultiplierOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setFixedRatePaymentFrequencyPeriodOfLeg2(String fixedRatePaymentFrequencyPeriodOfLeg2) {
			this.fixedRatePaymentFrequencyPeriodOfLeg2 = fixedRatePaymentFrequencyPeriodOfLeg2==null?null:fixedRatePaymentFrequencyPeriodOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("fixedRatePaymentFrequencyPeriodMultiplierOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setFixedRatePaymentFrequencyPeriodMultiplierOfLeg2(Integer fixedRatePaymentFrequencyPeriodMultiplierOfLeg2) {
			this.fixedRatePaymentFrequencyPeriodMultiplierOfLeg2 = fixedRatePaymentFrequencyPeriodMultiplierOfLeg2==null?null:fixedRatePaymentFrequencyPeriodMultiplierOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodOfLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodOfLeg1(Frequency13Code__1 floatingRatePaymentFrequencyPeriodOfLeg1) {
			this.floatingRatePaymentFrequencyPeriodOfLeg1 = floatingRatePaymentFrequencyPeriodOfLeg1==null?null:floatingRatePaymentFrequencyPeriodOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierOfLeg1")
		public MASTransactionReport.MASTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1(Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg1) {
			this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg1 = floatingRatePaymentFrequencyPeriodMultiplierOfLeg1==null?null:floatingRatePaymentFrequencyPeriodMultiplierOfLeg1;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodOfLeg2(Frequency13Code__1 floatingRatePaymentFrequencyPeriodOfLeg2) {
			this.floatingRatePaymentFrequencyPeriodOfLeg2 = floatingRatePaymentFrequencyPeriodOfLeg2==null?null:floatingRatePaymentFrequencyPeriodOfLeg2;
			return this;
		}
		@Override
		@RosettaAttribute("floatingRatePaymentFrequencyPeriodMultiplierOfLeg2")
		public MASTransactionReport.MASTransactionReportBuilder setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2(Integer floatingRatePaymentFrequencyPeriodMultiplierOfLeg2) {
			this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 = floatingRatePaymentFrequencyPeriodMultiplierOfLeg2==null?null:floatingRatePaymentFrequencyPeriodMultiplierOfLeg2;
			return this;
		}
		@Override
		public MASTransactionReport.MASTransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment) {
			if (otherPayment!=null) this.otherPayment.add(otherPayment.toBuilder());
			return this;
		}
		
		@Override
		public MASTransactionReport.MASTransactionReportBuilder addOtherPayment(OtherPaymentReport otherPayment, int _idx) {
			getIndex(this.otherPayment, _idx, () -> otherPayment.toBuilder());
			return this;
		}
		@Override 
		public MASTransactionReport.MASTransactionReportBuilder addOtherPayment(List<? extends OtherPaymentReport> otherPayments) {
			if (otherPayments != null) {
				for (OtherPaymentReport toAdd : otherPayments) {
					this.otherPayment.add(toAdd.toBuilder());
				}
			}
			return this;
		}
		
		@Override 
		@RosettaAttribute("otherPayment")
		public MASTransactionReport.MASTransactionReportBuilder setOtherPayment(List<? extends OtherPaymentReport> otherPayments) {
			if (otherPayments == null)  {
				this.otherPayment = new ArrayList<>();
			}
			else {
				this.otherPayment = otherPayments.stream()
					.map(_a->_a.toBuilder())
					.collect(Collectors.toCollection(()->new ArrayList<>()));
			}
			return this;
		}
		
		@Override
		@RosettaAttribute("actionType")
		public MASTransactionReport.MASTransactionReportBuilder setActionType(String actionType) {
			this.actionType = actionType==null?null:actionType;
			return this;
		}
		@Override
		@RosettaAttribute("eventType")
		public MASTransactionReport.MASTransactionReportBuilder setEventType(String eventType) {
			this.eventType = eventType==null?null:eventType;
			return this;
		}
		@Override
		@RosettaAttribute("eventDate")
		public MASTransactionReport.MASTransactionReportBuilder setEventDate(Date eventDate) {
			this.eventDate = eventDate==null?null:eventDate;
			return this;
		}
		
		@Override
		public MASTransactionReport build() {
			return new MASTransactionReport.MASTransactionReportImpl(this);
		}
		
		@Override
		public MASTransactionReport.MASTransactionReportBuilder toBuilder() {
			return this;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public MASTransactionReport.MASTransactionReportBuilder prune() {
			if (spreadOfLeg1!=null && !spreadOfLeg1.prune().hasData()) spreadOfLeg1 = null;
			if (spreadOfLeg2!=null && !spreadOfLeg2.prune().hasData()) spreadOfLeg2 = null;
			basketConstituents = basketConstituents.stream().filter(b->b!=null).<BasketConstituentsReport.BasketConstituentsReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			priceSchedule = priceSchedule.stream().filter(b->b!=null).<PriceScheduleReport.PriceScheduleReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			strikePriceSchedule = strikePriceSchedule.stream().filter(b->b!=null).<StrikePriceScheduleReport.StrikePriceScheduleReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			otherPayment = otherPayment.stream().filter(b->b!=null).<OtherPaymentReport.OtherPaymentReportBuilder>map(b->b.prune()).filter(b->b.hasData()).collect(Collectors.toList());
			return this;
		}
		
		@Override
		public boolean hasData() {
			if (getCounterparty1()!=null) return true;
			if (getCounterparty2()!=null) return true;
			if (getCounterparty2IdentifierType()!=null) return true;
			if (getDirection()!=null) return true;
			if (getDirectionOfLeg1()!=null) return true;
			if (getDirectionOfLeg2()!=null) return true;
			if (getReportingSpecifiedPerson()!=null) return true;
			if (getDataSubmitter()!=null) return true;
			if (getUniqueTransactionIdentifier()!=null) return true;
			if (getUniqueProductIdentifier()!=null) return true;
			if (getPriorUTI()!=null) return true;
			if (getContractType()!=null) return true;
			if (getAssetClass()!=null) return true;
			if (getUnderlying()!=null) return true;
			if (getUnderlyingIdentificationType()!=null) return true;
			if (getUnderlyingIdOther()!=null) return true;
			if (getUnderlyingIdOtherSource()!=null) return true;
			if (getNameOfTheUnderlyingIndex()!=null) return true;
			if (getFixedRateOfLeg1()!=null) return true;
			if (getIdentifierOfFloatingRateOfLeg1()!=null) return true;
			if (getNameOfTheFloatingRateOfLeg1()!=null) return true;
			if (getSpreadOfLeg1()!=null && getSpreadOfLeg1().hasData()) return true;
			if (getFixedRateOfLeg2()!=null) return true;
			if (getIdentifierOfFloatingRateOfLeg2()!=null) return true;
			if (getNameOfTheFloatingRateOfLeg2()!=null) return true;
			if (getSpreadOfLeg2()!=null && getSpreadOfLeg2().hasData()) return true;
			if (getOptionType()!=null) return true;
			if (getOptionStyle()!=null) return true;
			if (getFirstExerciseDate()!=null) return true;
			if (getEffectiveDate()!=null) return true;
			if (getExpirationDate()!=null) return true;
			if (getSwapLinkID()!=null) return true;
			if (getPackageIdentifier()!=null) return true;
			if (getDeliveryType()!=null) return true;
			if (getSettlementCurrency1()!=null) return true;
			if (getSettlementCurrency2()!=null) return true;
			if (getFinalContractualSettlementDate()!=null) return true;
			if (getCleared()!=null) return true;
			if (getCentralCounterparty()!=null) return true;
			if (getClearingMember()!=null) return true;
			if (getClearingTimestamp()!=null) return true;
			if (getPlatformIdentifier()!=null) return true;
			if (getReportingObligationOfSpecifiedPerson()!=null) return true;
			if (getBookingLocation()!=null) return true;
			if (getTraderLocation()!=null) return true;
			if (getConfirmed()!=null) return true;
			if (getConfirmationTimestamp()!=null) return true;
			if (getExecutionTimestamp()!=null) return true;
			if (getReportingTimestamp()!=null) return true;
			if (getBasketConstituents()!=null && getBasketConstituents().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getValuationAmount()!=null) return true;
			if (getValuationCurrency()!=null) return true;
			if (getValuationMethod()!=null) return true;
			if (getValuationTimestamp()!=null) return true;
			if (getCollateralPortfolioIndicator()!=null) return true;
			if (getInitialMarginCollateralPortfolioCode()!=null) return true;
			if (getVariationMarginCollateralPortfolioCode()!=null) return true;
			if (getNotionalAmountOfTheContractOrLeg1()!=null) return true;
			if (getNotionalCurrencyOfTheContractOrLeg1()!=null) return true;
			if (getNotionalAmountOfLeg2()!=null) return true;
			if (getNotionalCurrencyOfLeg2()!=null) return true;
			if (getTotalNotionalQuantityOfTheContractOrLeg1()!=null) return true;
			if (getQuantityUnitOfMeasureForTheContractOrLeg1()!=null) return true;
			if (getNotionalQuantityOfTheContractOrLeg1()!=null) return true;
			if (getQuantityFrequencyOfTheContractOrLeg1()!=null) return true;
			if (getQuantityFrequencyMultiplierOfTheContractOrLeg1()!=null) return true;
			if (getTotalNotionalQuantityLeg2()!=null) return true;
			if (getQuantityUnitOfMeasureLeg2()!=null) return true;
			if (getNotionalQuantityOfLeg2()!=null) return true;
			if (getQuantityFrequencyOfLeg2()!=null) return true;
			if (getQuantityFrequencyMultiplierOfLeg2()!=null) return true;
			if (getCallAmount()!=null) return true;
			if (getCallCurrency()!=null) return true;
			if (getPutAmount()!=null) return true;
			if (getPutCurrency()!=null) return true;
			if (getPrice()!=null) return true;
			if (getPriceCurrency()!=null) return true;
			if (getPriceNotation()!=null) return true;
			if (getPriceUnitOfMeasure()!=null) return true;
			if (getPriceSchedule()!=null && getPriceSchedule().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getStrikePrice()!=null) return true;
			if (getStrikePriceCurrency()!=null) return true;
			if (getStrikePriceNotation()!=null) return true;
			if (getStrikePriceSchedule()!=null && getStrikePriceSchedule().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getOptionPremiumAmount()!=null) return true;
			if (getOptionPremiumCurrency()!=null) return true;
			if (getExchangeRate()!=null) return true;
			if (getExchangeRateBasis()!=null) return true;
			if (getFixedRateDayCountConventionOfLeg1()!=null) return true;
			if (getFixedRateDayCountConventionOfLeg2()!=null) return true;
			if (getFloatingRateDayCountConventionOfLeg1()!=null) return true;
			if (getFloatingRateDayCountConventionOfLeg2()!=null) return true;
			if (getFloatingRateResetFrequencyPeriodOfLeg1()!=null) return true;
			if (getFloatingRateResetFrequencyPeriodMultiplierOfLeg1()!=null) return true;
			if (getFloatingRateResetFrequencyPeriodOfLeg2()!=null) return true;
			if (getFloatingRateResetFrequencyPeriodMultiplierOfLeg2()!=null) return true;
			if (getFixedRatePaymentFrequencyPeriodOfLeg1()!=null) return true;
			if (getFixedRatePaymentFrequencyPeriodMultiplierOfLeg1()!=null) return true;
			if (getFixedRatePaymentFrequencyPeriodOfLeg2()!=null) return true;
			if (getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2()!=null) return true;
			if (getFloatingRatePaymentFrequencyPeriodOfLeg1()!=null) return true;
			if (getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1()!=null) return true;
			if (getFloatingRatePaymentFrequencyPeriodOfLeg2()!=null) return true;
			if (getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2()!=null) return true;
			if (getOtherPayment()!=null && getOtherPayment().stream().filter(Objects::nonNull).anyMatch(a->a.hasData())) return true;
			if (getActionType()!=null) return true;
			if (getEventType()!=null) return true;
			if (getEventDate()!=null) return true;
			return false;
		}
	
		@SuppressWarnings("unchecked")
		@Override
		public MASTransactionReport.MASTransactionReportBuilder merge(RosettaModelObjectBuilder other, BuilderMerger merger) {
			MASTransactionReport.MASTransactionReportBuilder o = (MASTransactionReport.MASTransactionReportBuilder) other;
			
			merger.mergeRosetta(getSpreadOfLeg1(), o.getSpreadOfLeg1(), this::setSpreadOfLeg1);
			merger.mergeRosetta(getSpreadOfLeg2(), o.getSpreadOfLeg2(), this::setSpreadOfLeg2);
			merger.mergeRosetta(getBasketConstituents(), o.getBasketConstituents(), this::getOrCreateBasketConstituents);
			merger.mergeRosetta(getPriceSchedule(), o.getPriceSchedule(), this::getOrCreatePriceSchedule);
			merger.mergeRosetta(getStrikePriceSchedule(), o.getStrikePriceSchedule(), this::getOrCreateStrikePriceSchedule);
			merger.mergeRosetta(getOtherPayment(), o.getOtherPayment(), this::getOrCreateOtherPayment);
			
			merger.mergeBasic(getCounterparty1(), o.getCounterparty1(), this::setCounterparty1);
			merger.mergeBasic(getCounterparty2(), o.getCounterparty2(), this::setCounterparty2);
			merger.mergeBasic(getCounterparty2IdentifierType(), o.getCounterparty2IdentifierType(), this::setCounterparty2IdentifierType);
			merger.mergeBasic(getDirection(), o.getDirection(), this::setDirection);
			merger.mergeBasic(getDirectionOfLeg1(), o.getDirectionOfLeg1(), this::setDirectionOfLeg1);
			merger.mergeBasic(getDirectionOfLeg2(), o.getDirectionOfLeg2(), this::setDirectionOfLeg2);
			merger.mergeBasic(getReportingSpecifiedPerson(), o.getReportingSpecifiedPerson(), this::setReportingSpecifiedPerson);
			merger.mergeBasic(getDataSubmitter(), o.getDataSubmitter(), this::setDataSubmitter);
			merger.mergeBasic(getUniqueTransactionIdentifier(), o.getUniqueTransactionIdentifier(), this::setUniqueTransactionIdentifier);
			merger.mergeBasic(getUniqueProductIdentifier(), o.getUniqueProductIdentifier(), this::setUniqueProductIdentifier);
			merger.mergeBasic(getPriorUTI(), o.getPriorUTI(), this::setPriorUTI);
			merger.mergeBasic(getContractType(), o.getContractType(), this::setContractType);
			merger.mergeBasic(getAssetClass(), o.getAssetClass(), this::setAssetClass);
			merger.mergeBasic(getUnderlying(), o.getUnderlying(), this::setUnderlying);
			merger.mergeBasic(getUnderlyingIdentificationType(), o.getUnderlyingIdentificationType(), this::setUnderlyingIdentificationType);
			merger.mergeBasic(getUnderlyingIdOther(), o.getUnderlyingIdOther(), this::setUnderlyingIdOther);
			merger.mergeBasic(getUnderlyingIdOtherSource(), o.getUnderlyingIdOtherSource(), this::setUnderlyingIdOtherSource);
			merger.mergeBasic(getNameOfTheUnderlyingIndex(), o.getNameOfTheUnderlyingIndex(), this::setNameOfTheUnderlyingIndex);
			merger.mergeBasic(getFixedRateOfLeg1(), o.getFixedRateOfLeg1(), this::setFixedRateOfLeg1);
			merger.mergeBasic(getIdentifierOfFloatingRateOfLeg1(), o.getIdentifierOfFloatingRateOfLeg1(), this::setIdentifierOfFloatingRateOfLeg1);
			merger.mergeBasic(getNameOfTheFloatingRateOfLeg1(), o.getNameOfTheFloatingRateOfLeg1(), this::setNameOfTheFloatingRateOfLeg1);
			merger.mergeBasic(getFixedRateOfLeg2(), o.getFixedRateOfLeg2(), this::setFixedRateOfLeg2);
			merger.mergeBasic(getIdentifierOfFloatingRateOfLeg2(), o.getIdentifierOfFloatingRateOfLeg2(), this::setIdentifierOfFloatingRateOfLeg2);
			merger.mergeBasic(getNameOfTheFloatingRateOfLeg2(), o.getNameOfTheFloatingRateOfLeg2(), this::setNameOfTheFloatingRateOfLeg2);
			merger.mergeBasic(getOptionType(), o.getOptionType(), this::setOptionType);
			merger.mergeBasic(getOptionStyle(), o.getOptionStyle(), this::setOptionStyle);
			merger.mergeBasic(getFirstExerciseDate(), o.getFirstExerciseDate(), this::setFirstExerciseDate);
			merger.mergeBasic(getEffectiveDate(), o.getEffectiveDate(), this::setEffectiveDate);
			merger.mergeBasic(getExpirationDate(), o.getExpirationDate(), this::setExpirationDate);
			merger.mergeBasic(getSwapLinkID(), o.getSwapLinkID(), this::setSwapLinkID);
			merger.mergeBasic(getPackageIdentifier(), o.getPackageIdentifier(), this::setPackageIdentifier);
			merger.mergeBasic(getDeliveryType(), o.getDeliveryType(), this::setDeliveryType);
			merger.mergeBasic(getSettlementCurrency1(), o.getSettlementCurrency1(), this::setSettlementCurrency1);
			merger.mergeBasic(getSettlementCurrency2(), o.getSettlementCurrency2(), this::setSettlementCurrency2);
			merger.mergeBasic(getFinalContractualSettlementDate(), o.getFinalContractualSettlementDate(), this::setFinalContractualSettlementDate);
			merger.mergeBasic(getCleared(), o.getCleared(), this::setCleared);
			merger.mergeBasic(getCentralCounterparty(), o.getCentralCounterparty(), this::setCentralCounterparty);
			merger.mergeBasic(getClearingMember(), o.getClearingMember(), this::setClearingMember);
			merger.mergeBasic(getClearingTimestamp(), o.getClearingTimestamp(), this::setClearingTimestamp);
			merger.mergeBasic(getPlatformIdentifier(), o.getPlatformIdentifier(), this::setPlatformIdentifier);
			merger.mergeBasic(getReportingObligationOfSpecifiedPerson(), o.getReportingObligationOfSpecifiedPerson(), this::setReportingObligationOfSpecifiedPerson);
			merger.mergeBasic(getBookingLocation(), o.getBookingLocation(), this::setBookingLocation);
			merger.mergeBasic(getTraderLocation(), o.getTraderLocation(), this::setTraderLocation);
			merger.mergeBasic(getConfirmed(), o.getConfirmed(), this::setConfirmed);
			merger.mergeBasic(getConfirmationTimestamp(), o.getConfirmationTimestamp(), this::setConfirmationTimestamp);
			merger.mergeBasic(getExecutionTimestamp(), o.getExecutionTimestamp(), this::setExecutionTimestamp);
			merger.mergeBasic(getReportingTimestamp(), o.getReportingTimestamp(), this::setReportingTimestamp);
			merger.mergeBasic(getValuationAmount(), o.getValuationAmount(), this::setValuationAmount);
			merger.mergeBasic(getValuationCurrency(), o.getValuationCurrency(), this::setValuationCurrency);
			merger.mergeBasic(getValuationMethod(), o.getValuationMethod(), this::setValuationMethod);
			merger.mergeBasic(getValuationTimestamp(), o.getValuationTimestamp(), this::setValuationTimestamp);
			merger.mergeBasic(getCollateralPortfolioIndicator(), o.getCollateralPortfolioIndicator(), this::setCollateralPortfolioIndicator);
			merger.mergeBasic(getInitialMarginCollateralPortfolioCode(), o.getInitialMarginCollateralPortfolioCode(), this::setInitialMarginCollateralPortfolioCode);
			merger.mergeBasic(getVariationMarginCollateralPortfolioCode(), o.getVariationMarginCollateralPortfolioCode(), this::setVariationMarginCollateralPortfolioCode);
			merger.mergeBasic(getNotionalAmountOfTheContractOrLeg1(), o.getNotionalAmountOfTheContractOrLeg1(), this::setNotionalAmountOfTheContractOrLeg1);
			merger.mergeBasic(getNotionalCurrencyOfTheContractOrLeg1(), o.getNotionalCurrencyOfTheContractOrLeg1(), this::setNotionalCurrencyOfTheContractOrLeg1);
			merger.mergeBasic(getNotionalAmountOfLeg2(), o.getNotionalAmountOfLeg2(), this::setNotionalAmountOfLeg2);
			merger.mergeBasic(getNotionalCurrencyOfLeg2(), o.getNotionalCurrencyOfLeg2(), this::setNotionalCurrencyOfLeg2);
			merger.mergeBasic(getTotalNotionalQuantityOfTheContractOrLeg1(), o.getTotalNotionalQuantityOfTheContractOrLeg1(), this::setTotalNotionalQuantityOfTheContractOrLeg1);
			merger.mergeBasic(getQuantityUnitOfMeasureForTheContractOrLeg1(), o.getQuantityUnitOfMeasureForTheContractOrLeg1(), this::setQuantityUnitOfMeasureForTheContractOrLeg1);
			merger.mergeBasic(getNotionalQuantityOfTheContractOrLeg1(), o.getNotionalQuantityOfTheContractOrLeg1(), this::setNotionalQuantityOfTheContractOrLeg1);
			merger.mergeBasic(getQuantityFrequencyOfTheContractOrLeg1(), o.getQuantityFrequencyOfTheContractOrLeg1(), this::setQuantityFrequencyOfTheContractOrLeg1);
			merger.mergeBasic(getQuantityFrequencyMultiplierOfTheContractOrLeg1(), o.getQuantityFrequencyMultiplierOfTheContractOrLeg1(), this::setQuantityFrequencyMultiplierOfTheContractOrLeg1);
			merger.mergeBasic(getTotalNotionalQuantityLeg2(), o.getTotalNotionalQuantityLeg2(), this::setTotalNotionalQuantityLeg2);
			merger.mergeBasic(getQuantityUnitOfMeasureLeg2(), o.getQuantityUnitOfMeasureLeg2(), this::setQuantityUnitOfMeasureLeg2);
			merger.mergeBasic(getNotionalQuantityOfLeg2(), o.getNotionalQuantityOfLeg2(), this::setNotionalQuantityOfLeg2);
			merger.mergeBasic(getQuantityFrequencyOfLeg2(), o.getQuantityFrequencyOfLeg2(), this::setQuantityFrequencyOfLeg2);
			merger.mergeBasic(getQuantityFrequencyMultiplierOfLeg2(), o.getQuantityFrequencyMultiplierOfLeg2(), this::setQuantityFrequencyMultiplierOfLeg2);
			merger.mergeBasic(getCallAmount(), o.getCallAmount(), this::setCallAmount);
			merger.mergeBasic(getCallCurrency(), o.getCallCurrency(), this::setCallCurrency);
			merger.mergeBasic(getPutAmount(), o.getPutAmount(), this::setPutAmount);
			merger.mergeBasic(getPutCurrency(), o.getPutCurrency(), this::setPutCurrency);
			merger.mergeBasic(getPrice(), o.getPrice(), this::setPrice);
			merger.mergeBasic(getPriceCurrency(), o.getPriceCurrency(), this::setPriceCurrency);
			merger.mergeBasic(getPriceNotation(), o.getPriceNotation(), this::setPriceNotation);
			merger.mergeBasic(getPriceUnitOfMeasure(), o.getPriceUnitOfMeasure(), this::setPriceUnitOfMeasure);
			merger.mergeBasic(getStrikePrice(), o.getStrikePrice(), this::setStrikePrice);
			merger.mergeBasic(getStrikePriceCurrency(), o.getStrikePriceCurrency(), this::setStrikePriceCurrency);
			merger.mergeBasic(getStrikePriceNotation(), o.getStrikePriceNotation(), this::setStrikePriceNotation);
			merger.mergeBasic(getOptionPremiumAmount(), o.getOptionPremiumAmount(), this::setOptionPremiumAmount);
			merger.mergeBasic(getOptionPremiumCurrency(), o.getOptionPremiumCurrency(), this::setOptionPremiumCurrency);
			merger.mergeBasic(getExchangeRate(), o.getExchangeRate(), this::setExchangeRate);
			merger.mergeBasic(getExchangeRateBasis(), o.getExchangeRateBasis(), this::setExchangeRateBasis);
			merger.mergeBasic(getFixedRateDayCountConventionOfLeg1(), o.getFixedRateDayCountConventionOfLeg1(), this::setFixedRateDayCountConventionOfLeg1);
			merger.mergeBasic(getFixedRateDayCountConventionOfLeg2(), o.getFixedRateDayCountConventionOfLeg2(), this::setFixedRateDayCountConventionOfLeg2);
			merger.mergeBasic(getFloatingRateDayCountConventionOfLeg1(), o.getFloatingRateDayCountConventionOfLeg1(), this::setFloatingRateDayCountConventionOfLeg1);
			merger.mergeBasic(getFloatingRateDayCountConventionOfLeg2(), o.getFloatingRateDayCountConventionOfLeg2(), this::setFloatingRateDayCountConventionOfLeg2);
			merger.mergeBasic(getFloatingRateResetFrequencyPeriodOfLeg1(), o.getFloatingRateResetFrequencyPeriodOfLeg1(), this::setFloatingRateResetFrequencyPeriodOfLeg1);
			merger.mergeBasic(getFloatingRateResetFrequencyPeriodMultiplierOfLeg1(), o.getFloatingRateResetFrequencyPeriodMultiplierOfLeg1(), this::setFloatingRateResetFrequencyPeriodMultiplierOfLeg1);
			merger.mergeBasic(getFloatingRateResetFrequencyPeriodOfLeg2(), o.getFloatingRateResetFrequencyPeriodOfLeg2(), this::setFloatingRateResetFrequencyPeriodOfLeg2);
			merger.mergeBasic(getFloatingRateResetFrequencyPeriodMultiplierOfLeg2(), o.getFloatingRateResetFrequencyPeriodMultiplierOfLeg2(), this::setFloatingRateResetFrequencyPeriodMultiplierOfLeg2);
			merger.mergeBasic(getFixedRatePaymentFrequencyPeriodOfLeg1(), o.getFixedRatePaymentFrequencyPeriodOfLeg1(), this::setFixedRatePaymentFrequencyPeriodOfLeg1);
			merger.mergeBasic(getFixedRatePaymentFrequencyPeriodMultiplierOfLeg1(), o.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg1(), this::setFixedRatePaymentFrequencyPeriodMultiplierOfLeg1);
			merger.mergeBasic(getFixedRatePaymentFrequencyPeriodOfLeg2(), o.getFixedRatePaymentFrequencyPeriodOfLeg2(), this::setFixedRatePaymentFrequencyPeriodOfLeg2);
			merger.mergeBasic(getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2(), o.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2(), this::setFixedRatePaymentFrequencyPeriodMultiplierOfLeg2);
			merger.mergeBasic(getFloatingRatePaymentFrequencyPeriodOfLeg1(), o.getFloatingRatePaymentFrequencyPeriodOfLeg1(), this::setFloatingRatePaymentFrequencyPeriodOfLeg1);
			merger.mergeBasic(getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1(), o.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1(), this::setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1);
			merger.mergeBasic(getFloatingRatePaymentFrequencyPeriodOfLeg2(), o.getFloatingRatePaymentFrequencyPeriodOfLeg2(), this::setFloatingRatePaymentFrequencyPeriodOfLeg2);
			merger.mergeBasic(getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2(), o.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2(), this::setFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2);
			merger.mergeBasic(getActionType(), o.getActionType(), this::setActionType);
			merger.mergeBasic(getEventType(), o.getEventType(), this::setEventType);
			merger.mergeBasic(getEventDate(), o.getEventDate(), this::setEventDate);
			return this;
		}
	
		@Override
		public boolean equals(Object o) {
			if (this == o) return true;
			if (o == null || !(o instanceof RosettaModelObject) || !getType().equals(((RosettaModelObject)o).getType())) return false;
		
			MASTransactionReport _that = getType().cast(o);
		
			if (!Objects.equals(counterparty1, _that.getCounterparty1())) return false;
			if (!Objects.equals(counterparty2, _that.getCounterparty2())) return false;
			if (!Objects.equals(counterparty2IdentifierType, _that.getCounterparty2IdentifierType())) return false;
			if (!Objects.equals(direction, _that.getDirection())) return false;
			if (!Objects.equals(directionOfLeg1, _that.getDirectionOfLeg1())) return false;
			if (!Objects.equals(directionOfLeg2, _that.getDirectionOfLeg2())) return false;
			if (!Objects.equals(reportingSpecifiedPerson, _that.getReportingSpecifiedPerson())) return false;
			if (!Objects.equals(dataSubmitter, _that.getDataSubmitter())) return false;
			if (!Objects.equals(uniqueTransactionIdentifier, _that.getUniqueTransactionIdentifier())) return false;
			if (!Objects.equals(uniqueProductIdentifier, _that.getUniqueProductIdentifier())) return false;
			if (!Objects.equals(priorUTI, _that.getPriorUTI())) return false;
			if (!Objects.equals(contractType, _that.getContractType())) return false;
			if (!Objects.equals(assetClass, _that.getAssetClass())) return false;
			if (!Objects.equals(underlying, _that.getUnderlying())) return false;
			if (!Objects.equals(underlyingIdentificationType, _that.getUnderlyingIdentificationType())) return false;
			if (!Objects.equals(underlyingIdOther, _that.getUnderlyingIdOther())) return false;
			if (!Objects.equals(underlyingIdOtherSource, _that.getUnderlyingIdOtherSource())) return false;
			if (!Objects.equals(nameOfTheUnderlyingIndex, _that.getNameOfTheUnderlyingIndex())) return false;
			if (!Objects.equals(fixedRateOfLeg1, _that.getFixedRateOfLeg1())) return false;
			if (!Objects.equals(identifierOfFloatingRateOfLeg1, _that.getIdentifierOfFloatingRateOfLeg1())) return false;
			if (!Objects.equals(nameOfTheFloatingRateOfLeg1, _that.getNameOfTheFloatingRateOfLeg1())) return false;
			if (!Objects.equals(spreadOfLeg1, _that.getSpreadOfLeg1())) return false;
			if (!Objects.equals(fixedRateOfLeg2, _that.getFixedRateOfLeg2())) return false;
			if (!Objects.equals(identifierOfFloatingRateOfLeg2, _that.getIdentifierOfFloatingRateOfLeg2())) return false;
			if (!Objects.equals(nameOfTheFloatingRateOfLeg2, _that.getNameOfTheFloatingRateOfLeg2())) return false;
			if (!Objects.equals(spreadOfLeg2, _that.getSpreadOfLeg2())) return false;
			if (!Objects.equals(optionType, _that.getOptionType())) return false;
			if (!Objects.equals(optionStyle, _that.getOptionStyle())) return false;
			if (!Objects.equals(firstExerciseDate, _that.getFirstExerciseDate())) return false;
			if (!Objects.equals(effectiveDate, _that.getEffectiveDate())) return false;
			if (!Objects.equals(expirationDate, _that.getExpirationDate())) return false;
			if (!Objects.equals(swapLinkID, _that.getSwapLinkID())) return false;
			if (!Objects.equals(packageIdentifier, _that.getPackageIdentifier())) return false;
			if (!Objects.equals(deliveryType, _that.getDeliveryType())) return false;
			if (!Objects.equals(settlementCurrency1, _that.getSettlementCurrency1())) return false;
			if (!Objects.equals(settlementCurrency2, _that.getSettlementCurrency2())) return false;
			if (!Objects.equals(finalContractualSettlementDate, _that.getFinalContractualSettlementDate())) return false;
			if (!Objects.equals(cleared, _that.getCleared())) return false;
			if (!Objects.equals(centralCounterparty, _that.getCentralCounterparty())) return false;
			if (!Objects.equals(clearingMember, _that.getClearingMember())) return false;
			if (!Objects.equals(clearingTimestamp, _that.getClearingTimestamp())) return false;
			if (!Objects.equals(platformIdentifier, _that.getPlatformIdentifier())) return false;
			if (!Objects.equals(reportingObligationOfSpecifiedPerson, _that.getReportingObligationOfSpecifiedPerson())) return false;
			if (!Objects.equals(bookingLocation, _that.getBookingLocation())) return false;
			if (!Objects.equals(traderLocation, _that.getTraderLocation())) return false;
			if (!Objects.equals(confirmed, _that.getConfirmed())) return false;
			if (!Objects.equals(confirmationTimestamp, _that.getConfirmationTimestamp())) return false;
			if (!Objects.equals(executionTimestamp, _that.getExecutionTimestamp())) return false;
			if (!Objects.equals(reportingTimestamp, _that.getReportingTimestamp())) return false;
			if (!ListEquals.listEquals(basketConstituents, _that.getBasketConstituents())) return false;
			if (!Objects.equals(valuationAmount, _that.getValuationAmount())) return false;
			if (!Objects.equals(valuationCurrency, _that.getValuationCurrency())) return false;
			if (!Objects.equals(valuationMethod, _that.getValuationMethod())) return false;
			if (!Objects.equals(valuationTimestamp, _that.getValuationTimestamp())) return false;
			if (!Objects.equals(collateralPortfolioIndicator, _that.getCollateralPortfolioIndicator())) return false;
			if (!Objects.equals(initialMarginCollateralPortfolioCode, _that.getInitialMarginCollateralPortfolioCode())) return false;
			if (!Objects.equals(variationMarginCollateralPortfolioCode, _that.getVariationMarginCollateralPortfolioCode())) return false;
			if (!Objects.equals(notionalAmountOfTheContractOrLeg1, _that.getNotionalAmountOfTheContractOrLeg1())) return false;
			if (!Objects.equals(notionalCurrencyOfTheContractOrLeg1, _that.getNotionalCurrencyOfTheContractOrLeg1())) return false;
			if (!Objects.equals(notionalAmountOfLeg2, _that.getNotionalAmountOfLeg2())) return false;
			if (!Objects.equals(notionalCurrencyOfLeg2, _that.getNotionalCurrencyOfLeg2())) return false;
			if (!Objects.equals(totalNotionalQuantityOfTheContractOrLeg1, _that.getTotalNotionalQuantityOfTheContractOrLeg1())) return false;
			if (!Objects.equals(quantityUnitOfMeasureForTheContractOrLeg1, _that.getQuantityUnitOfMeasureForTheContractOrLeg1())) return false;
			if (!Objects.equals(notionalQuantityOfTheContractOrLeg1, _that.getNotionalQuantityOfTheContractOrLeg1())) return false;
			if (!Objects.equals(quantityFrequencyOfTheContractOrLeg1, _that.getQuantityFrequencyOfTheContractOrLeg1())) return false;
			if (!Objects.equals(quantityFrequencyMultiplierOfTheContractOrLeg1, _that.getQuantityFrequencyMultiplierOfTheContractOrLeg1())) return false;
			if (!Objects.equals(totalNotionalQuantityLeg2, _that.getTotalNotionalQuantityLeg2())) return false;
			if (!Objects.equals(quantityUnitOfMeasureLeg2, _that.getQuantityUnitOfMeasureLeg2())) return false;
			if (!Objects.equals(notionalQuantityOfLeg2, _that.getNotionalQuantityOfLeg2())) return false;
			if (!Objects.equals(quantityFrequencyOfLeg2, _that.getQuantityFrequencyOfLeg2())) return false;
			if (!Objects.equals(quantityFrequencyMultiplierOfLeg2, _that.getQuantityFrequencyMultiplierOfLeg2())) return false;
			if (!Objects.equals(callAmount, _that.getCallAmount())) return false;
			if (!Objects.equals(callCurrency, _that.getCallCurrency())) return false;
			if (!Objects.equals(putAmount, _that.getPutAmount())) return false;
			if (!Objects.equals(putCurrency, _that.getPutCurrency())) return false;
			if (!Objects.equals(price, _that.getPrice())) return false;
			if (!Objects.equals(priceCurrency, _that.getPriceCurrency())) return false;
			if (!Objects.equals(priceNotation, _that.getPriceNotation())) return false;
			if (!Objects.equals(priceUnitOfMeasure, _that.getPriceUnitOfMeasure())) return false;
			if (!ListEquals.listEquals(priceSchedule, _that.getPriceSchedule())) return false;
			if (!Objects.equals(strikePrice, _that.getStrikePrice())) return false;
			if (!Objects.equals(strikePriceCurrency, _that.getStrikePriceCurrency())) return false;
			if (!Objects.equals(strikePriceNotation, _that.getStrikePriceNotation())) return false;
			if (!ListEquals.listEquals(strikePriceSchedule, _that.getStrikePriceSchedule())) return false;
			if (!Objects.equals(optionPremiumAmount, _that.getOptionPremiumAmount())) return false;
			if (!Objects.equals(optionPremiumCurrency, _that.getOptionPremiumCurrency())) return false;
			if (!Objects.equals(exchangeRate, _that.getExchangeRate())) return false;
			if (!Objects.equals(exchangeRateBasis, _that.getExchangeRateBasis())) return false;
			if (!Objects.equals(fixedRateDayCountConventionOfLeg1, _that.getFixedRateDayCountConventionOfLeg1())) return false;
			if (!Objects.equals(fixedRateDayCountConventionOfLeg2, _that.getFixedRateDayCountConventionOfLeg2())) return false;
			if (!Objects.equals(floatingRateDayCountConventionOfLeg1, _that.getFloatingRateDayCountConventionOfLeg1())) return false;
			if (!Objects.equals(floatingRateDayCountConventionOfLeg2, _that.getFloatingRateDayCountConventionOfLeg2())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodOfLeg1, _that.getFloatingRateResetFrequencyPeriodOfLeg1())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodMultiplierOfLeg1, _that.getFloatingRateResetFrequencyPeriodMultiplierOfLeg1())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodOfLeg2, _that.getFloatingRateResetFrequencyPeriodOfLeg2())) return false;
			if (!Objects.equals(floatingRateResetFrequencyPeriodMultiplierOfLeg2, _that.getFloatingRateResetFrequencyPeriodMultiplierOfLeg2())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodOfLeg1, _that.getFixedRatePaymentFrequencyPeriodOfLeg1())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodMultiplierOfLeg1, _that.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg1())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodOfLeg2, _that.getFixedRatePaymentFrequencyPeriodOfLeg2())) return false;
			if (!Objects.equals(fixedRatePaymentFrequencyPeriodMultiplierOfLeg2, _that.getFixedRatePaymentFrequencyPeriodMultiplierOfLeg2())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodOfLeg1, _that.getFloatingRatePaymentFrequencyPeriodOfLeg1())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodMultiplierOfLeg1, _that.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg1())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodOfLeg2, _that.getFloatingRatePaymentFrequencyPeriodOfLeg2())) return false;
			if (!Objects.equals(floatingRatePaymentFrequencyPeriodMultiplierOfLeg2, _that.getFloatingRatePaymentFrequencyPeriodMultiplierOfLeg2())) return false;
			if (!ListEquals.listEquals(otherPayment, _that.getOtherPayment())) return false;
			if (!Objects.equals(actionType, _that.getActionType())) return false;
			if (!Objects.equals(eventType, _that.getEventType())) return false;
			if (!Objects.equals(eventDate, _that.getEventDate())) return false;
			return true;
		}
		
		@Override
		public int hashCode() {
			int _result = 0;
			_result = 31 * _result + (counterparty1 != null ? counterparty1.hashCode() : 0);
			_result = 31 * _result + (counterparty2 != null ? counterparty2.hashCode() : 0);
			_result = 31 * _result + (counterparty2IdentifierType != null ? counterparty2IdentifierType.hashCode() : 0);
			_result = 31 * _result + (direction != null ? direction.hashCode() : 0);
			_result = 31 * _result + (directionOfLeg1 != null ? directionOfLeg1.hashCode() : 0);
			_result = 31 * _result + (directionOfLeg2 != null ? directionOfLeg2.hashCode() : 0);
			_result = 31 * _result + (reportingSpecifiedPerson != null ? reportingSpecifiedPerson.hashCode() : 0);
			_result = 31 * _result + (dataSubmitter != null ? dataSubmitter.hashCode() : 0);
			_result = 31 * _result + (uniqueTransactionIdentifier != null ? uniqueTransactionIdentifier.hashCode() : 0);
			_result = 31 * _result + (uniqueProductIdentifier != null ? uniqueProductIdentifier.hashCode() : 0);
			_result = 31 * _result + (priorUTI != null ? priorUTI.hashCode() : 0);
			_result = 31 * _result + (contractType != null ? contractType.hashCode() : 0);
			_result = 31 * _result + (assetClass != null ? assetClass.hashCode() : 0);
			_result = 31 * _result + (underlying != null ? underlying.hashCode() : 0);
			_result = 31 * _result + (underlyingIdentificationType != null ? underlyingIdentificationType.hashCode() : 0);
			_result = 31 * _result + (underlyingIdOther != null ? underlyingIdOther.hashCode() : 0);
			_result = 31 * _result + (underlyingIdOtherSource != null ? underlyingIdOtherSource.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (nameOfTheUnderlyingIndex != null ? nameOfTheUnderlyingIndex.hashCode() : 0);
			_result = 31 * _result + (fixedRateOfLeg1 != null ? fixedRateOfLeg1.hashCode() : 0);
			_result = 31 * _result + (identifierOfFloatingRateOfLeg1 != null ? identifierOfFloatingRateOfLeg1.hashCode() : 0);
			_result = 31 * _result + (nameOfTheFloatingRateOfLeg1 != null ? nameOfTheFloatingRateOfLeg1.hashCode() : 0);
			_result = 31 * _result + (spreadOfLeg1 != null ? spreadOfLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRateOfLeg2 != null ? fixedRateOfLeg2.hashCode() : 0);
			_result = 31 * _result + (identifierOfFloatingRateOfLeg2 != null ? identifierOfFloatingRateOfLeg2.hashCode() : 0);
			_result = 31 * _result + (nameOfTheFloatingRateOfLeg2 != null ? nameOfTheFloatingRateOfLeg2.hashCode() : 0);
			_result = 31 * _result + (spreadOfLeg2 != null ? spreadOfLeg2.hashCode() : 0);
			_result = 31 * _result + (optionType != null ? optionType.hashCode() : 0);
			_result = 31 * _result + (optionStyle != null ? optionStyle.hashCode() : 0);
			_result = 31 * _result + (firstExerciseDate != null ? firstExerciseDate.hashCode() : 0);
			_result = 31 * _result + (effectiveDate != null ? effectiveDate.hashCode() : 0);
			_result = 31 * _result + (expirationDate != null ? expirationDate.hashCode() : 0);
			_result = 31 * _result + (swapLinkID != null ? swapLinkID.hashCode() : 0);
			_result = 31 * _result + (packageIdentifier != null ? packageIdentifier.hashCode() : 0);
			_result = 31 * _result + (deliveryType != null ? deliveryType.hashCode() : 0);
			_result = 31 * _result + (settlementCurrency1 != null ? settlementCurrency1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (settlementCurrency2 != null ? settlementCurrency2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (finalContractualSettlementDate != null ? finalContractualSettlementDate.hashCode() : 0);
			_result = 31 * _result + (cleared != null ? cleared.hashCode() : 0);
			_result = 31 * _result + (centralCounterparty != null ? centralCounterparty.hashCode() : 0);
			_result = 31 * _result + (clearingMember != null ? clearingMember.hashCode() : 0);
			_result = 31 * _result + (clearingTimestamp != null ? clearingTimestamp.hashCode() : 0);
			_result = 31 * _result + (platformIdentifier != null ? platformIdentifier.hashCode() : 0);
			_result = 31 * _result + (reportingObligationOfSpecifiedPerson != null ? reportingObligationOfSpecifiedPerson.hashCode() : 0);
			_result = 31 * _result + (bookingLocation != null ? bookingLocation.hashCode() : 0);
			_result = 31 * _result + (traderLocation != null ? traderLocation.hashCode() : 0);
			_result = 31 * _result + (confirmed != null ? confirmed.hashCode() : 0);
			_result = 31 * _result + (confirmationTimestamp != null ? confirmationTimestamp.hashCode() : 0);
			_result = 31 * _result + (executionTimestamp != null ? executionTimestamp.hashCode() : 0);
			_result = 31 * _result + (reportingTimestamp != null ? reportingTimestamp.hashCode() : 0);
			_result = 31 * _result + (basketConstituents != null ? basketConstituents.hashCode() : 0);
			_result = 31 * _result + (valuationAmount != null ? valuationAmount.hashCode() : 0);
			_result = 31 * _result + (valuationCurrency != null ? valuationCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (valuationMethod != null ? valuationMethod.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (valuationTimestamp != null ? valuationTimestamp.hashCode() : 0);
			_result = 31 * _result + (collateralPortfolioIndicator != null ? collateralPortfolioIndicator.hashCode() : 0);
			_result = 31 * _result + (initialMarginCollateralPortfolioCode != null ? initialMarginCollateralPortfolioCode.hashCode() : 0);
			_result = 31 * _result + (variationMarginCollateralPortfolioCode != null ? variationMarginCollateralPortfolioCode.hashCode() : 0);
			_result = 31 * _result + (notionalAmountOfTheContractOrLeg1 != null ? notionalAmountOfTheContractOrLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalCurrencyOfTheContractOrLeg1 != null ? notionalCurrencyOfTheContractOrLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (notionalAmountOfLeg2 != null ? notionalAmountOfLeg2.hashCode() : 0);
			_result = 31 * _result + (notionalCurrencyOfLeg2 != null ? notionalCurrencyOfLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityOfTheContractOrLeg1 != null ? totalNotionalQuantityOfTheContractOrLeg1.hashCode() : 0);
			_result = 31 * _result + (quantityUnitOfMeasureForTheContractOrLeg1 != null ? quantityUnitOfMeasureForTheContractOrLeg1.hashCode() : 0);
			_result = 31 * _result + (notionalQuantityOfTheContractOrLeg1 != null ? notionalQuantityOfTheContractOrLeg1.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyOfTheContractOrLeg1 != null ? quantityFrequencyOfTheContractOrLeg1.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyMultiplierOfTheContractOrLeg1 != null ? quantityFrequencyMultiplierOfTheContractOrLeg1.hashCode() : 0);
			_result = 31 * _result + (totalNotionalQuantityLeg2 != null ? totalNotionalQuantityLeg2.hashCode() : 0);
			_result = 31 * _result + (quantityUnitOfMeasureLeg2 != null ? quantityUnitOfMeasureLeg2.hashCode() : 0);
			_result = 31 * _result + (notionalQuantityOfLeg2 != null ? notionalQuantityOfLeg2.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyOfLeg2 != null ? quantityFrequencyOfLeg2.hashCode() : 0);
			_result = 31 * _result + (quantityFrequencyMultiplierOfLeg2 != null ? quantityFrequencyMultiplierOfLeg2.hashCode() : 0);
			_result = 31 * _result + (callAmount != null ? callAmount.hashCode() : 0);
			_result = 31 * _result + (callCurrency != null ? callCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (putAmount != null ? putAmount.hashCode() : 0);
			_result = 31 * _result + (putCurrency != null ? putCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (price != null ? price.hashCode() : 0);
			_result = 31 * _result + (priceCurrency != null ? priceCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (priceNotation != null ? priceNotation.hashCode() : 0);
			_result = 31 * _result + (priceUnitOfMeasure != null ? priceUnitOfMeasure.hashCode() : 0);
			_result = 31 * _result + (priceSchedule != null ? priceSchedule.hashCode() : 0);
			_result = 31 * _result + (strikePrice != null ? strikePrice.hashCode() : 0);
			_result = 31 * _result + (strikePriceCurrency != null ? strikePriceCurrency.hashCode() : 0);
			_result = 31 * _result + (strikePriceNotation != null ? strikePriceNotation.hashCode() : 0);
			_result = 31 * _result + (strikePriceSchedule != null ? strikePriceSchedule.hashCode() : 0);
			_result = 31 * _result + (optionPremiumAmount != null ? optionPremiumAmount.hashCode() : 0);
			_result = 31 * _result + (optionPremiumCurrency != null ? optionPremiumCurrency.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (exchangeRate != null ? exchangeRate.hashCode() : 0);
			_result = 31 * _result + (exchangeRateBasis != null ? exchangeRateBasis.hashCode() : 0);
			_result = 31 * _result + (fixedRateDayCountConventionOfLeg1 != null ? fixedRateDayCountConventionOfLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (fixedRateDayCountConventionOfLeg2 != null ? fixedRateDayCountConventionOfLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRateDayCountConventionOfLeg1 != null ? floatingRateDayCountConventionOfLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRateDayCountConventionOfLeg2 != null ? floatingRateDayCountConventionOfLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodOfLeg1 != null ? floatingRateResetFrequencyPeriodOfLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodMultiplierOfLeg1 != null ? floatingRateResetFrequencyPeriodMultiplierOfLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodOfLeg2 != null ? floatingRateResetFrequencyPeriodOfLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRateResetFrequencyPeriodMultiplierOfLeg2 != null ? floatingRateResetFrequencyPeriodMultiplierOfLeg2.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodOfLeg1 != null ? fixedRatePaymentFrequencyPeriodOfLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodMultiplierOfLeg1 != null ? fixedRatePaymentFrequencyPeriodMultiplierOfLeg1.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodOfLeg2 != null ? fixedRatePaymentFrequencyPeriodOfLeg2.hashCode() : 0);
			_result = 31 * _result + (fixedRatePaymentFrequencyPeriodMultiplierOfLeg2 != null ? fixedRatePaymentFrequencyPeriodMultiplierOfLeg2.hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodOfLeg1 != null ? floatingRatePaymentFrequencyPeriodOfLeg1.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodMultiplierOfLeg1 != null ? floatingRatePaymentFrequencyPeriodMultiplierOfLeg1.hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodOfLeg2 != null ? floatingRatePaymentFrequencyPeriodOfLeg2.getClass().getName().hashCode() : 0);
			_result = 31 * _result + (floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 != null ? floatingRatePaymentFrequencyPeriodMultiplierOfLeg2.hashCode() : 0);
			_result = 31 * _result + (otherPayment != null ? otherPayment.hashCode() : 0);
			_result = 31 * _result + (actionType != null ? actionType.hashCode() : 0);
			_result = 31 * _result + (eventType != null ? eventType.hashCode() : 0);
			_result = 31 * _result + (eventDate != null ? eventDate.hashCode() : 0);
			return _result;
		}
		
		@Override
		public String toString() {
			return "MASTransactionReportBuilder {" +
				"counterparty1=" + this.counterparty1 + ", " +
				"counterparty2=" + this.counterparty2 + ", " +
				"counterparty2IdentifierType=" + this.counterparty2IdentifierType + ", " +
				"direction=" + this.direction + ", " +
				"directionOfLeg1=" + this.directionOfLeg1 + ", " +
				"directionOfLeg2=" + this.directionOfLeg2 + ", " +
				"reportingSpecifiedPerson=" + this.reportingSpecifiedPerson + ", " +
				"dataSubmitter=" + this.dataSubmitter + ", " +
				"uniqueTransactionIdentifier=" + this.uniqueTransactionIdentifier + ", " +
				"uniqueProductIdentifier=" + this.uniqueProductIdentifier + ", " +
				"priorUTI=" + this.priorUTI + ", " +
				"contractType=" + this.contractType + ", " +
				"assetClass=" + this.assetClass + ", " +
				"underlying=" + this.underlying + ", " +
				"underlyingIdentificationType=" + this.underlyingIdentificationType + ", " +
				"underlyingIdOther=" + this.underlyingIdOther + ", " +
				"underlyingIdOtherSource=" + this.underlyingIdOtherSource + ", " +
				"nameOfTheUnderlyingIndex=" + this.nameOfTheUnderlyingIndex + ", " +
				"fixedRateOfLeg1=" + this.fixedRateOfLeg1 + ", " +
				"identifierOfFloatingRateOfLeg1=" + this.identifierOfFloatingRateOfLeg1 + ", " +
				"nameOfTheFloatingRateOfLeg1=" + this.nameOfTheFloatingRateOfLeg1 + ", " +
				"spreadOfLeg1=" + this.spreadOfLeg1 + ", " +
				"fixedRateOfLeg2=" + this.fixedRateOfLeg2 + ", " +
				"identifierOfFloatingRateOfLeg2=" + this.identifierOfFloatingRateOfLeg2 + ", " +
				"nameOfTheFloatingRateOfLeg2=" + this.nameOfTheFloatingRateOfLeg2 + ", " +
				"spreadOfLeg2=" + this.spreadOfLeg2 + ", " +
				"optionType=" + this.optionType + ", " +
				"optionStyle=" + this.optionStyle + ", " +
				"firstExerciseDate=" + this.firstExerciseDate + ", " +
				"effectiveDate=" + this.effectiveDate + ", " +
				"expirationDate=" + this.expirationDate + ", " +
				"swapLinkID=" + this.swapLinkID + ", " +
				"packageIdentifier=" + this.packageIdentifier + ", " +
				"deliveryType=" + this.deliveryType + ", " +
				"settlementCurrency1=" + this.settlementCurrency1 + ", " +
				"settlementCurrency2=" + this.settlementCurrency2 + ", " +
				"finalContractualSettlementDate=" + this.finalContractualSettlementDate + ", " +
				"cleared=" + this.cleared + ", " +
				"centralCounterparty=" + this.centralCounterparty + ", " +
				"clearingMember=" + this.clearingMember + ", " +
				"clearingTimestamp=" + this.clearingTimestamp + ", " +
				"platformIdentifier=" + this.platformIdentifier + ", " +
				"reportingObligationOfSpecifiedPerson=" + this.reportingObligationOfSpecifiedPerson + ", " +
				"bookingLocation=" + this.bookingLocation + ", " +
				"traderLocation=" + this.traderLocation + ", " +
				"confirmed=" + this.confirmed + ", " +
				"confirmationTimestamp=" + this.confirmationTimestamp + ", " +
				"executionTimestamp=" + this.executionTimestamp + ", " +
				"reportingTimestamp=" + this.reportingTimestamp + ", " +
				"basketConstituents=" + this.basketConstituents + ", " +
				"valuationAmount=" + this.valuationAmount + ", " +
				"valuationCurrency=" + this.valuationCurrency + ", " +
				"valuationMethod=" + this.valuationMethod + ", " +
				"valuationTimestamp=" + this.valuationTimestamp + ", " +
				"collateralPortfolioIndicator=" + this.collateralPortfolioIndicator + ", " +
				"initialMarginCollateralPortfolioCode=" + this.initialMarginCollateralPortfolioCode + ", " +
				"variationMarginCollateralPortfolioCode=" + this.variationMarginCollateralPortfolioCode + ", " +
				"notionalAmountOfTheContractOrLeg1=" + this.notionalAmountOfTheContractOrLeg1 + ", " +
				"notionalCurrencyOfTheContractOrLeg1=" + this.notionalCurrencyOfTheContractOrLeg1 + ", " +
				"notionalAmountOfLeg2=" + this.notionalAmountOfLeg2 + ", " +
				"notionalCurrencyOfLeg2=" + this.notionalCurrencyOfLeg2 + ", " +
				"totalNotionalQuantityOfTheContractOrLeg1=" + this.totalNotionalQuantityOfTheContractOrLeg1 + ", " +
				"quantityUnitOfMeasureForTheContractOrLeg1=" + this.quantityUnitOfMeasureForTheContractOrLeg1 + ", " +
				"notionalQuantityOfTheContractOrLeg1=" + this.notionalQuantityOfTheContractOrLeg1 + ", " +
				"quantityFrequencyOfTheContractOrLeg1=" + this.quantityFrequencyOfTheContractOrLeg1 + ", " +
				"quantityFrequencyMultiplierOfTheContractOrLeg1=" + this.quantityFrequencyMultiplierOfTheContractOrLeg1 + ", " +
				"totalNotionalQuantityLeg2=" + this.totalNotionalQuantityLeg2 + ", " +
				"quantityUnitOfMeasureLeg2=" + this.quantityUnitOfMeasureLeg2 + ", " +
				"notionalQuantityOfLeg2=" + this.notionalQuantityOfLeg2 + ", " +
				"quantityFrequencyOfLeg2=" + this.quantityFrequencyOfLeg2 + ", " +
				"quantityFrequencyMultiplierOfLeg2=" + this.quantityFrequencyMultiplierOfLeg2 + ", " +
				"callAmount=" + this.callAmount + ", " +
				"callCurrency=" + this.callCurrency + ", " +
				"putAmount=" + this.putAmount + ", " +
				"putCurrency=" + this.putCurrency + ", " +
				"price=" + this.price + ", " +
				"priceCurrency=" + this.priceCurrency + ", " +
				"priceNotation=" + this.priceNotation + ", " +
				"priceUnitOfMeasure=" + this.priceUnitOfMeasure + ", " +
				"priceSchedule=" + this.priceSchedule + ", " +
				"strikePrice=" + this.strikePrice + ", " +
				"strikePriceCurrency=" + this.strikePriceCurrency + ", " +
				"strikePriceNotation=" + this.strikePriceNotation + ", " +
				"strikePriceSchedule=" + this.strikePriceSchedule + ", " +
				"optionPremiumAmount=" + this.optionPremiumAmount + ", " +
				"optionPremiumCurrency=" + this.optionPremiumCurrency + ", " +
				"exchangeRate=" + this.exchangeRate + ", " +
				"exchangeRateBasis=" + this.exchangeRateBasis + ", " +
				"fixedRateDayCountConventionOfLeg1=" + this.fixedRateDayCountConventionOfLeg1 + ", " +
				"fixedRateDayCountConventionOfLeg2=" + this.fixedRateDayCountConventionOfLeg2 + ", " +
				"floatingRateDayCountConventionOfLeg1=" + this.floatingRateDayCountConventionOfLeg1 + ", " +
				"floatingRateDayCountConventionOfLeg2=" + this.floatingRateDayCountConventionOfLeg2 + ", " +
				"floatingRateResetFrequencyPeriodOfLeg1=" + this.floatingRateResetFrequencyPeriodOfLeg1 + ", " +
				"floatingRateResetFrequencyPeriodMultiplierOfLeg1=" + this.floatingRateResetFrequencyPeriodMultiplierOfLeg1 + ", " +
				"floatingRateResetFrequencyPeriodOfLeg2=" + this.floatingRateResetFrequencyPeriodOfLeg2 + ", " +
				"floatingRateResetFrequencyPeriodMultiplierOfLeg2=" + this.floatingRateResetFrequencyPeriodMultiplierOfLeg2 + ", " +
				"fixedRatePaymentFrequencyPeriodOfLeg1=" + this.fixedRatePaymentFrequencyPeriodOfLeg1 + ", " +
				"fixedRatePaymentFrequencyPeriodMultiplierOfLeg1=" + this.fixedRatePaymentFrequencyPeriodMultiplierOfLeg1 + ", " +
				"fixedRatePaymentFrequencyPeriodOfLeg2=" + this.fixedRatePaymentFrequencyPeriodOfLeg2 + ", " +
				"fixedRatePaymentFrequencyPeriodMultiplierOfLeg2=" + this.fixedRatePaymentFrequencyPeriodMultiplierOfLeg2 + ", " +
				"floatingRatePaymentFrequencyPeriodOfLeg1=" + this.floatingRatePaymentFrequencyPeriodOfLeg1 + ", " +
				"floatingRatePaymentFrequencyPeriodMultiplierOfLeg1=" + this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg1 + ", " +
				"floatingRatePaymentFrequencyPeriodOfLeg2=" + this.floatingRatePaymentFrequencyPeriodOfLeg2 + ", " +
				"floatingRatePaymentFrequencyPeriodMultiplierOfLeg2=" + this.floatingRatePaymentFrequencyPeriodMultiplierOfLeg2 + ", " +
				"otherPayment=" + this.otherPayment + ", " +
				"actionType=" + this.actionType + ", " +
				"eventType=" + this.eventType + ", " +
				"eventDate=" + this.eventDate +
			'}';
		}
	}
}
